/**
* Extension of the WebViewer3DAPI
*
* @module WebViewer3D
* @author DOGO, PHST
* @copyright    2015 Esko Stonecube Ltd. All rights reserved.
* @version  0.7.1
*/

define([
	'WV3D/WebViewer3DAPI',
    "WV3Dmod/LogMessage"
    ], function (
    	WebViewer3DAPI,
        LogMessage
        ) {

    'use strict';

	/**
     * The WebViewer3DAPIEXT object.
     *
     *  @function WebViewer3DAPIEXT
     *  @alias module:WebViewer3D
     *  @class WebViewer3DAPIEXT
     *  @param {HTMLObject} context - a reference to the DOM element in which the viewer should be inserted
     */
    var WebViewer3DAPIEXT = function(args)
    {
        WebViewer3DAPI.apply(this,arguments);
        LogMessage.Log("Creating WebViewer3DAPIEXT Object");
    }
    WebViewer3DAPIEXT.prototype = Object.create(WebViewer3DAPI.prototype);
    WebViewer3DAPIEXT.prototype.constructor = WebViewer3DAPIEXT;

    /**
     * Reload all textures
     *
     * @function ReLoad all textures (only material textures) that have been loaded by the json file.
     * @param {function} __callback - callback funtion used when rendering has been updated
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
     */
    WebViewer3DAPI.prototype.ReloadAllTextures = function(__callback, __redraw)
    {
        LogMessage.Log("API Reload all textures");

        this._controller.ReloadAllTextures(__callback, __redraw);
    }

    /**
    * Reload a texture
    *
    * @function ReloadTexture
    * @param {String} __textureURL - URL identifying the initial texture relative path from json to texture
    * @param {String} __newURL - URL identifying the new texture relative path from json to texture
    * @param {function} __callback - callback funtion used when rendering has been updated
    * @param {Boolean} [__keep_old_path = true] - indicates if the old VS new path has to be used now on to reference the texture
    * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
    */
    WebViewer3DAPI.prototype.ReloadTexture = function(__textureURL, __newURL, __callback, __keep_old_path, __redraw)
    {
        LogMessage.Log("API Reload texture");

        this._controller.ReloadTexture(__textureURL, __newURL, __callback, __keep_old_path, __redraw);
    }

    /**
     * Get the queried data info of an object using its URL
     *
     * @function Get
     * @param {string} __nodeURL -  the type of information we want to retrieve from object
     * @param {string} __datatype -  the type of information we want to retrieve from object
     * to choose among the following list:
     * - NODEOBJECTVIS returns (Boolean)
     * @returns {object|boolean|string} depending on the type of info
     */
    WebViewer3DAPIEXT.prototype.Get = function(__nodeURL, __datatype)
    {
        return this._controller.Get(__nodeURL, __datatype);
    }

    /**
     * Prints a memory report
     * @function LogMemoryReport
     */
    WebViewer3DAPIEXT.prototype.LogMemoryReport = function(__object)
    {
        return this._controller.LogMemoryReport();
    }

    /**
     * Print list of opaque scene vertices in world coordinates
     */
    WebViewer3DAPIEXT.prototype.PrintOpaqueSceneVertices = function()
    {
        return this._controller.PrintOpaqueSceneVertices();
    }

    /**
     * Test function
     */
    WebViewer3DAPIEXT.prototype.APIEXTTestFunction = function(__enableAnimations)
    {
        // //If needed to call super implem.
        // WebViewer3DAPI.prototype.toImplement.call(this, "myparam");
    }

    return WebViewer3DAPIEXT;

});
