/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2014 Esko Stonecube Ltd. All rights reserved.
 */

 define([
    "underscore",
    "backbone",
    "WV3Dmod/HttpRequestHandler",
    "WV3Dmod/LogMessage"
    ], function (
        _,
        bb,
        WV3DHttpRequestHandler,
        LogMessage
        ) {

    'use strict';


    /**
     * @alias module:WebViewer3D
     * @class WV3DHttpRequestManager
     */
    var WV3DHttpRequestManager = bb.Model.extend({

        initialize: function(__callback)
        {
            LogMessage.Log("Creating WV3DHttpRequestManager Object");
            this.ClearURLCache();
        },

        /**
         * Return true if the given string is a URL, else false
         *
         * @param {String} sceneURL URL to the scene data
         */
        IsURL: function(__string)
        {
            var regexprelative = /.\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/
            return (regexprelative.test(__string) || this.IsAbsoluteURL(__string));
        },

        /**
         * Return true if the given string is a URL, else false
         *
         *
         * @param {String} sceneURL URL to the scene data
         */
        IsRelativeURL: function(__string)
        {
            return this.IsURL(__string) && !this.IsAbsoluteURL(__string);
        },

        /**
         * Return true if the given string is a URL, else false
         *
         * @param {String} sceneURL URL to the scene data
         */
        IsAbsoluteURL: function(__string)
        {
            var regexpabsolute = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;
            return regexpabsolute.test(__string);
        },

        /**
         * Return the file name + extension from the URL
         *
         * @param {String} __string URL to the file
         */
        GetFileName: function(__string)
        {
            var idx_hash = __string.lastIndexOf("#");
            var __realstring;
            if(idx_hash == -1 )
            {
                __realstring = __string;
            }else{
                __realstring = __string.substring(0,__string.indexOf("#"));
            }
            var filename = __realstring.substring(__realstring.lastIndexOf("/")+1);
            return filename;
        },

        /**
         * Return the asset name + extension from the URL
         *
         * @param {String} __string URL to the asset
         */
        GetAssetName: function(__string)
        {
            var hash_pos = __string.lastIndexOf("#");
            if(hash_pos == -1)
            {
                return "";
            }
            else
            {
                return __string.substring(hash_pos+1);
            }
        },

        /**
         * Retrieve the real URL of a file, removing the json file prefix
         *
         * @param {String} sceneURL URL to the scene data
         */
        GetRealURL: function(__sceneURL, __parentSceneURL, __idx_hash)
        {
            //Remove everything that might be located after an ventual # symbol
            //since it is useless for html requests (unless the server itself can understand it)
            var idx_hash;
            if(__idx_hash === undefined)
            {
                idx_hash = __sceneURL.lastIndexOf("#");
            }else
            {
                idx_hash = __idx_hash;
            }
            var realSceneURL;
            if(idx_hash == -1 )
            {
                realSceneURL = __sceneURL;
            }else{
                realSceneURL = __sceneURL.substring(0,__sceneURL.indexOf("#"));
            }

            //Get the real URL by removing the base path
            if(this.IsRelativeURL(realSceneURL) && __parentSceneURL)
            {
                var __parentScenePath = __parentSceneURL.substring(0, __parentSceneURL.lastIndexOf("/")+1);
                realSceneURL = __parentScenePath.concat(realSceneURL);
            }else if (this.IsAbsoluteURL(__sceneURL))
            {
                realSceneURL = realSceneURL;
            }else
            {
                throw "Not a URL";
            }
            return realSceneURL;
        },

        /**
         * Retrieve the real URL of a file, removing the json file prefix
         *
         * @param {String} sceneURL URL to the scene data
         */
        GetRealURLWithAppendedRef: function(__sceneURL, __parentSceneURL)
        {
            //Remove everything that might be located after an ventual # symbol
            //since it is useless for html requests (unless the server itself can understand it)
            var idx_hash = __sceneURL.lastIndexOf("#");
            var realSceneURL;
            var appendedRef = "";
            if(idx_hash != -1 )
            {
                appendedRef = __sceneURL.substring(__sceneURL.indexOf("#"));
            }
            return this.GetRealURL(__sceneURL,__parentSceneURL, idx_hash) + appendedRef;
        },

        /**
         * Updates the query counter
         */
        IncrementQueryCounter: function()
        {
            this.set('_queriedassetnumber', this.get('_queriedassetnumber')+1);
        },

        /**
         * Load a scene from a URL resource
         *
         * @param {String} sceneURL URL to the scene data
         */
        LoadURL: function (__sceneURL, __parentSceneURL)
        {
            var _real_URL = this.GetRealURL(__sceneURL,__parentSceneURL);
            if(this.get('_urlCache')[_real_URL] === undefined)
            {
                this.IncrementQueryCounter();
                //Create a proper http request handler that can have its own attributes
                //Give to that new request the callback parameters inherited
                var httprequest = new WV3DHttpRequestHandler(_real_URL, this.get('_caller'), this.get('_callback'));
                //Keeps track of previously cached URLs during that session so that there
                //is no duplicate and overheads in AssetManager cache construction
                this.get('_urlCache')[_real_URL] = __parentSceneURL;
            }
        },

        /**
         * Clears the http request manager url cache
         */
        ClearURLCache: function(__string)
        {
            this.set('_urlCache',{});
            this.set('_queriedassetnumber', 0);
        }


    });

return WV3DHttpRequestManager;

});