/**
 * WV3DTextSpriteMaker provides functionality to create and maintain 2D text sprites within 3JS
 *
 * @module WV3DTextSpriteMaker
 */


define([
        "backbone",
        "WV3Dlib/three",
        "WV3Dmod/LogMessage"
        ],
        function(
                bb,
                THREE,
                LogMessage
                )
        {
            'use strict';

            var WV3DTextSpriteMaker = bb.Model.extend({

                initialize: function()
                {
                    LogMessage.Log("WV3DTextSpriteMaker initialization");
                },

                MakeTextSprite: function(__text, __parameters, __position, __offsetHorizontally, __revOffsetHorizontally, __offsetVertically, __revOffsetVertically, __offsetDepth, __revOffsetDepth, __orientation)
                {
                    if(__parameters == undefined || !__parameters)
                        __parameters = {};

                    if(__offsetHorizontally == undefined)
                        __offsetHorizontally = false;

                    if(__offsetVertically == undefined)
                        __offsetVertically = false;

                    var fontFace = __parameters.hasOwnProperty("fontface") ? __parameters["fontface"] : "Arial";
                    var fontSize = __parameters.hasOwnProperty("fontsize") ? __parameters["fontsize"] : 18;
                    var borderTh = __parameters.hasOwnProperty("borderThickness") ? __parameters["borderThickness"] : 2;
                    var borderCol = __parameters.hasOwnProperty("borderColor") ? __parameters["borderColor"] : { r:0, g:0, b:0, a:0.0 };
                    var bgCol = __parameters.hasOwnProperty("backgroundColor") ? __parameters["backgroundColor"] : { r:0, g:0, b:0, a:0.0 };
                    var txtCol = __parameters.hasOwnProperty("color") ? __parameters["color"] : { r:0, g:0, b:0, a:1.0 };

                    var docCanvas = document.createElement('canvas');
                    var canvCtx2D = docCanvas.getContext('2d');
                    canvCtx2D.font = "normal " + fontSize + "px " + fontFace;   //To get correct text width font needs to be set
                    var textWidth = canvCtx2D.measureText( __text ).width;

                    var spriteWidth = textWidth + 3 * borderTh;
                    var spriteHeight = fontSize + (4* borderTh);    //4 extra pixels to leave small gap between top/bottom boundary rect and text
                    docCanvas.width = spriteWidth;
                    docCanvas.height = spriteHeight;
                    canvCtx2D.font = "normal " + fontSize + "px " + fontFace;   //canvas2d context font needs to be reset after canvas sizes are reset

                    //Draw border and background
                    canvCtx2D.strokeStyle = "rgba(" + borderCol.r + ", " + borderCol.g + ", " + borderCol.b + ", " + borderCol.a + ")";
                    canvCtx2D.fillStyle = "rgba(" + bgCol.r + ", " + bgCol.g + ", " + bgCol.b + ", " + bgCol.a + ")";
                    canvCtx2D.lineWidth = borderTh;

                    this.CreateRoundRect(canvCtx2D, borderTh/2, borderTh/2, spriteWidth - borderTh, spriteHeight - borderTh, fontSize/5);

                    //Now draw the text
                    canvCtx2D.fillStyle = "rgba(" + txtCol.r + ", " + txtCol.g + ", " + txtCol.b + ", " + txtCol.a + ")";
                    canvCtx2D.fillText(__text, borderTh, spriteHeight - borderTh - (fontSize * 0.3)); //a factorial of fontsize to counter for the below baseline characters

                    //And create a texture from canvas context for sprite
                    var spriteTex = new THREE.Texture(docCanvas);
                    spriteTex.needsUpdate = true;
                    var spriteMat = new THREE.SpriteMaterial( {map: spriteTex} );

                    if(__orientation && __orientation == "vertical")
                        spriteMat.rotation = 1.5708;    //in radians. Rotate 90 deg CCW

                    var theSprite = new THREE.Sprite( spriteMat );
                    theSprite.scale.set(spriteWidth, spriteHeight, 1.0);
                    theSprite.__initial_scale = theSprite.scale.clone();

                    //Slightly offset (as per choice) sprite so that it appears positioned relative to top left corner when offsetted horizontally and vertically
                    //By default position is referred to sprite's centroid.
                    var halfSpriteW = spriteWidth/2 + 10;
                    var halfSpriteH = spriteHeight/2 + 10;

                    theSprite.__initial_att_position = __position.clone();
                    __position.x = __position.x + (__offsetDepth ? halfSpriteW : 0);
                    __position.x = __position.x - (__revOffsetDepth ? halfSpriteW : 0);
                    __position.y = __position.y + (__offsetHorizontally ? halfSpriteW : 0);
                    __position.y = __position.y - (__revOffsetHorizontally ? halfSpriteW : 0);
                    __position.z = __position.z - (__offsetVertically ? halfSpriteH : 0);
                    __position.z = __position.z + (__revOffsetVertically ? halfSpriteH : 0);
                    theSprite.position.set(__position.x, __position.y, __position.z);
                    theSprite.__initial_position = theSprite.position.clone();

                    return theSprite;
                },

                CreateRoundRect: function(__ctx, __x, __y, __w, __h, __radius)
                {
                    __ctx.beginPath();
                    __ctx.moveTo(__x + __radius, __y);
                    __ctx.lineTo(__x + __w - __radius, __y);
                    __ctx.quadraticCurveTo(__x + __w, __y, __x + __w, __y + __radius);
                    __ctx.lineTo(__x + __w, __y + __h - __radius);
                    __ctx.quadraticCurveTo(__x + __w, __y + __h, __x + __w - __radius, __y + __h);
                    __ctx.lineTo(__x + __radius, __y + __h);
                    __ctx.quadraticCurveTo(__x, __y + __h, __x, __y + __h - __radius);
                    __ctx.lineTo(__x, __y + __radius);
                    __ctx.quadraticCurveTo(__x, __y, __x + __radius, __y);
                    __ctx.closePath();
                    __ctx.fill();
                    __ctx.stroke();
                }
            });

            return WV3DTextSpriteMaker;
        });
