/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define([
    'backbone',
    "WV3Dmod/LogMessage",
], function(    bb,
                LogMessage ) {


    /**
     * @alias module:WebViewer3D
     * @class Utility gathering timer that can be accessed from any other class
     * Warning : this class is a singleton, any modification to one of its component will change for each reference to it.
     */

    var Timers = bb.Model.extend({

        initialize: function()
        {
            this.set("_timers",{});
        },

        reset: function(_name)
        {
            this.get("_timers")[_name]["accumulated"] = 0;
            this.start(_name);
        },

        start: function(_name)
        {
            var start_time = (window.performance!==undefined&&window.performance.now!==undefined)?window.performance.now():Date.now();
            if(this.get("_timers").hasOwnProperty(_name))
            {
                if(! this.get("_timers")[_name]["started"])
                {
                    this.get("_timers")[_name]["start"] = start_time;
                    LogMessage.Log("Timer "+_name+" started");
                    this.get("_timers")[_name]["started"] = true;
                }
                else
                {
                    LogMessage.Log("Timer "+_name+" is already started");
                    this.now(_name);
                }
            }
            else
            {
                this.get("_timers")[_name] = { start: start_time, accumulated:0, started:true };
                LogMessage.Log("Timer "+_name+" started");
            }
        },

        now: function(_name)
        {
            var now_time = (window.performance!==undefined&&window.performance.now!==undefined)?window.performance.now():Date.now();
            var elapsed = this.get("_timers")[_name]["accumulated"] + now_time - this.get("_timers")[_name]["start"];
            LogMessage.Log("Timer "+_name+" is now: "+elapsed);
        },

        stop: function(_name)
        {
            var stop_time = (window.performance!==undefined&&window.performance.now!==undefined)?window.performance.now():Date.now();
            var elapsed = this.get("_timers")[_name]["accumulated"] + stop_time - this.get("_timers")[_name]["start"];
            this.get("_timers")[_name]["accumulated"] = elapsed;
            this.get("_timers")[_name]["started"] = false;
            LogMessage.Log("Timer "+_name+" is stopped at: "+elapsed);
        }



    });

    return new Timers;//New is what makes it a singleton, see usage in ViewerView.js
});