/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define([
    'backbone'
], function( bb ) {


    /**
     * @alias module:WebViewer3D
     * @class Utils gather utility functions that can be reused in other classes
     * Warning : this class is a singleton, any modification to one of its component will change for each reference to it.
     */

    var Utils = bb.Model.extend({
        defaults:
        {
            /* Converts any object to a boolean */
            /* return true if __var == "TRUE", "true", "TRue",..., and true */
            /* otherwise returns false*/
            toBool: function(__var)
            {
                if (typeof (__var.toLowerCase) == 'function')
                {
                    return (__var.toLowerCase() === "true");
                }
                else
                {
                    return __var ? true : false;//return explicitely a boolean in case __var is null or undefined
                }
            },

            /* Returns size (in Bytes) of an object: This process might take a long time to end but should not have
             * any crash issue */
            SizeOf: function( __object )
            {
                var objectList = [];
                var stack = [__object];
                var bytes = 0;
                while(stack.length)
                {
                    var value = stack.pop();

                    if ( typeof value === 'boolean' ) {
                        bytes += 4;
                    }
                    else if ( typeof value === 'string' ) {
                        bytes += value.length * 2;
                    }
                    else if ( typeof value === 'number' ) {
                        bytes += 8;
                    }
                    else if ( typeof value === 'object' && value !== null && value !== undefined && objectList.indexOf( value ) === -1)
                    {
                        objectList.push( value );
                        for( i in value )
                        {
                            if(value.hasOwnProperty(i))
                            {
                                bytes += i.length * 2;
                                bytes += 8; // an assumed existence overhead
                                stack.push( value[ i ] );
                            }
                        }
                    }
                }
                return bytes;
            }

        }
    });

    return new Utils;//New is what makes it a singleton, see usage in ViewerView.js
});