/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2014 Esko Stonecube Ltd. All rights reserved.
 */

define([
    "underscore",
    "backbone",
    "WV3Dmod/AssetManager",
    "WV3Dmod/LogMessage"
    ], function (
        _,
        bb,
        WV3DAssetManager,
        LogMessage
        ) {
    
    'use strict';

    /**
     * @alias module:WebViewer3D
     * @class WV3DViewerModel
     */
	var WV3DViewerModel = bb.Model.extend({
		
	initialize: function(__httprequestmanager, __clientCallback)
	{
		LogMessage.Log("Creating WV3DViewerModel Object");

        __httprequestmanager.set('_caller',this);
        __httprequestmanager.set('_callback',this.LoadSceneJSON);
        this.set('_assetmanager', new WV3DAssetManager(__httprequestmanager));
        this.set('SceneId',null); 
        this.set('_clientCallback', __clientCallback);
	},


    /**
     * Query a scene from a url and gives LoadSceneJSON (next function) as a callback to load
     * this scene when the resource is available
     */
    LoadSceneURL : function(__sceneURL)
    {
        LogMessage.Log("Model LoadSceneURL function");

        this.get('_assetmanager').LoadURL(__sceneURL);
    },

    /**
     * Load a scene from a json file and trigger an event to tell the viewer that the model
     * has been updated. This function will be called once for each nested json file by the
     * httprequestmanager.
     * 
     * Also call the client's callback function to indicate state of scene loading. Argument
     * indicates loading state, 0 for success and successive values to indicate errors.
     * Desription of these values is provided in 3DAPI JS.
     * 
     * __sceneURL will replace the AssetID key to reference to that scene
     */
    LoadSceneJSON : function(__resource, __sceneURL) 
    {
        LogMessage.Log("Model LoadSceneJSON function");

        //TODO, change scene id handling
        var scene_id = this.get('_assetmanager').LoadAsset(__resource, __sceneURL);
        if(this.get('SceneId') == null)
        {
		  this.set('SceneId',scene_id);  
        }
        
        if(typeof this.get('_clientCallback') === "function")
        {
            this.get('_clientCallback')(0,0,0,
                                        this.get('_assetmanager').get('_assetloader').get('_httprequestmanager').get('_queriedassetnumber'),
                                        this.get('_assetmanager').get('_resolvedassetnumber'));
        }else
        {
            LogMessage.Log("Client callback not a function");
        }

        this.trigger('added_scene_resource');
    },

    /**
     * Delete the current scene
     */
    ResetAssetManager : function(__resource) 
    {
        LogMessage.Log("Reset Model function");

        this.get('_assetmanager').Clear();
        this.set('SceneId',null); 
    }
	});

    return WV3DViewerModel;

});