/**
 * @author SVN & DOGO
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
        "WV3Dmod/shaders/VertexShaderQuad",
        "WV3Dmod/shaders/ShaderUtils"
        ],
        function(THREE,
                 VertexShaderQuad,
                 ShaderUtils)
        {
        /**
         * @alias module:WebViewer3D
         * @class BackdropShader: draws a backdrop, with apropiate scaling so no black borders are visible
         */

        var BackdropShader = {

            uniforms:
            {
                "canvas_size": { type: "v2", value: null },

                "backdrop" : { type: "t", value: null },    //backdrop color
                "backdrop_enabled": {type: "i", value: 0.},
                "backdrop_scale": { type: "v2", value: new THREE.Vector2(1, 1) },
                "backdrop_offset": { type: "v2", value: new THREE.Vector2(0, 0) },

                "reflection": {type: "t", value: null},
                "reflection_enabled": {type: "i", value: 0.},
                "reflection_opacity": {type: "f", value: 0.},
                "reflection_fade": {type: "i", value: 0},
                "reflection_fade_size": {type: "f", value: 0.},
                "reflection_fade_position": {type: "f", value: 0.},

                "default_bg_color": {type:"v4", value: new THREE.Vector4(0.37,0.37,0.37,1.)}
            },

            vertexShader: VertexShaderQuad.vertexShader,

            fragmentShader:
            [
                "uniform vec2 canvas_size;",

                "uniform sampler2D backdrop;",
                "uniform bool backdrop_enabled;",
                "uniform vec2 backdrop_scale;",
                "uniform vec2 backdrop_offset;",

                "uniform sampler2D reflection;",
                "uniform bool reflection_enabled;",
                "uniform float reflection_opacity;",
                "uniform bool reflection_fade;",
                "uniform float reflection_fade_size;",
                "uniform float reflection_fade_position;",

                "uniform vec4 default_bg_color;",

                "void main()",
                "{",
                    "vec2 loc = vec2(gl_FragCoord.x/canvas_size.x, gl_FragCoord.y/canvas_size.y);",
                    "vec3 tex_reflect_color = texture2D(reflection, loc).rgb;",
                    "vec3 reflect_color = (reflection_enabled) ? tex_reflect_color : vec3(0.);",
                    "float refopacity = reflection_opacity;",
                    ShaderUtils.get('ReflectionFadeChunck'),
                    "vec3 tex_bg_color = texture2D(backdrop, loc * backdrop_scale + backdrop_offset ).rgb;",
                    "vec3 bgcol = (backdrop_enabled)? tex_bg_color : default_bg_color.rgb * default_bg_color.a;",
                    "gl_FragColor.rgb = (reflect_color == vec3(0.))? bgcol : (bgcol * (1. - refopacity) + reflect_color * refopacity);",
                    "gl_FragColor.a = (reflect_color == vec3(0.))? default_bg_color.a : refopacity + default_bg_color.a - refopacity * default_bg_color.a;",
                "}"
            ].join("\n")

        }

        return BackdropShader;

});