/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
		"WV3Dmod/shaders/VertexShaderQuad",
        "WV3Dmod/shaders/ShaderUtils"
		],
		function(THREE,
				 VertexShaderQuad,
                 ShaderUtils)
		{
        /**
         * @alias module:WebViewer3D
         * @class CopyShaderMaterial: copy a texture to another render texture
         */

        var D2Shader = {

            uniforms:
            {
                "canvas_size": { type: "v2", value: null },
                "CXRT" : { type: "t", value: null },//Color (next peel added, last one will be the opaque peel)
                "OpaqueRT" : { type: "t", value: null },//Mask color (next peel added, last one will be the opaque peel)
                "EnableMask" : { type: "i", value: 0 },
                "EnableRefOpacity": { type: "i", value: 0 },
                "EnableInvRefOpacity": { type: "i", value: 0 },
                "reflection_opacity": {type: "f", value: 0.},
                "reflection_fade": {type: "i", value: 0},
                "reflection_fade_size": {type: "f", value: 0.},
                "reflection_fade_position": {type: "f", value: 0.},
            },

			vertexShader: VertexShaderQuad.vertexShader,

            fragmentShader:
            [
                "uniform vec2 canvas_size;",
                "uniform sampler2D CXRT;",
                "uniform sampler2D OpaqueRT;",
                "uniform bool EnableMask;",
                "uniform bool EnableRefOpacity;",
                "uniform bool EnableInvRefOpacity;",
                "uniform float reflection_opacity;",
                "uniform bool reflection_fade;",
                "uniform float reflection_fade_size;",
                "uniform float reflection_fade_position;",

                "void main()",
                "{",
                    "vec2 loc = vec2(gl_FragCoord.x/canvas_size.x, gl_FragCoord.y/canvas_size.y);",
                    "float tex_opaquert = texture2D(OpaqueRT, loc).r;",
                    "if (EnableMask && (tex_opaquert == 0.)) discard;",//discards reflected transparent fragments situated behing opaque non reflected
                    "float refopacity = reflection_opacity;",
                    "if(EnableRefOpacity)",
                    "{",
                        ShaderUtils.get('ReflectionFadeChunck'),
                    "}",
                    "else",
                    "{",
                        "refopacity = 1.;",
                    "}",
                    "gl_FragColor = texture2D(CXRT, loc) * refopacity + (EnableInvRefOpacity? vec4(1.) * (1. - refopacity) : vec4(0.));",
                    // "if(EnableRefOpacity)
                    // "gl_FragColor.a = 1.;",
                "}"
            ].join("\n")

        }

        return D2Shader;

});