/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
        "WV3Dmod/shaders/VertexShaderNPM",
        "WV3Dmod/shaders/ShaderUtils"
        ],
        function(THREE,
                 VertexShaderNPM,
                 ShaderUtils)   
        {
        /**
         * @alias module:WebViewer3D
         * @class D1Shader: Used for first peel of transparent objects in combination
         * with a GL_LEQUAL test on depth. Is supposed to let through only the fragments 
         * that are located between the nearest opaque depth and the camera
         */

        var D1Shader = {

            uniforms: 
            {
                "D0RT" : { type: "t", value: null },
                "canvas_size": { type: "v2", value: null }
            },

            vertexShader: VertexShaderNPM.vertexShader,

            fragmentShader: 
            [
                ShaderUtils.get('DeclareDepthSamplerChunck'),
                ShaderUtils.get('EncodeDepthChunck'),
                ShaderUtils.get('DecodeDepthChunck'),

                "void main()",
                "{",
                    "vec2 loc = vec2(gl_FragCoord.x/canvas_size.x, gl_FragCoord.y/canvas_size.y);",
                    "float zmin = DecodeFloatRGBA(texture2D(D0RT, loc));",
                    "float z = DecodeFloatRGBA(EncodeFloatRGBA(1.0 - gl_FragCoord.z));",
                    "if ( zmin < z ) ",
                    "{",
                      "vec4 depth = EncodeFloatRGBA(1.0 - gl_FragCoord.z);",
                      ShaderUtils.get('SimulateEncodingChunckD1'),
                      "gl_FragColor = depth;",
                    "} else ",
                    "{",
                        "discard;",
                    "}",
                "}"
            ].join("\n")

        }

        return D1Shader;

});