/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
        "WV3Dmod/shaders/VertexShaderNPM",
        "WV3Dmod/shaders/ShaderUtils"
        ],
        function(THREE,
                 VertexShaderNPM,
                 ShaderUtils)   
        {
        /**
         * @alias module:WebViewer3D
         * @class D2Shader
         */

        var D2Shader = {

            uniforms: 
            {
                "DX1RT" : { type: "t", value: null },
                "DX2RT" : { type: "t", value: null },
                "canvas_size": { type: "v2", value: null }
            },

            vertexShader: VertexShaderNPM.vertexShader,

            fragmentShader: 
            [
                ShaderUtils.get('DeclareDepthSamplersChunck'),
                ShaderUtils.get('EncodeDepthChunck'),
                ShaderUtils.get('DecodeDepthChunck'),

                "void main()",
                "{",
                    "vec2 loc = vec2(gl_FragCoord.x/canvas_size.x, gl_FragCoord.y/canvas_size.y);",
                    "float zmin = DecodeFloatRGBA(texture2D(DX1RT, loc));",
                    "float zmax = DecodeFloatRGBA(texture2D(DX2RT, loc));",
                    "float z = DecodeFloatRGBA(EncodeFloatRGBA(1.0 - gl_FragCoord.z));",

                    //The following epsilon removes minor artifacts in depth-peeling algorithm that caused white spots to appear randomly
                    //it ensures that each peel will have a minimum "thickness" of 0.0000001
                    "if (zmin < z && z < zmax - 0.0000001)",
                    "{",
                        "vec4 depth = EncodeFloatRGBA(1.0 - gl_FragCoord.z);",
                        ShaderUtils.get('SimulateEncodingChunckD2'),
                        "gl_FragColor = depth;",
                    "} else ",
                    "{",
                        "discard;",
                    "}",
                "}"
            ].join("\n")

        }

        return D2Shader;

});