/**
 * @author SVN
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
        "WV3Dmod/shaders/VertexShaderNPM",
        "WV3Dmod/shaders/ShaderUtils"
        ],
        function(THREE,
                 VertexShaderNPM,
                 ShaderUtils)
        {
        /**
         * @alias module:WebViewer3D
         * @class DepthOnlyShaderMaterial: Writes out the (normal, unencoded) depth of the current peel in the depth buffer, and clears the color.
         *        This allows later passes to use blending to combine coincident surfaces, without fragments from the wrong peel
         *        interfering since these are eliminated by SelectPeelChunck. Can be used as OverrideMaterial
         */
        var DepthOnlyShader = {

            uniforms:
            {
                "canvas_size": { type: "v2", value: null },
                "DX1RT" : { type: "t", value: null },   // we use the same depth compares as the D2 depth shader
                "DX2RT" : { type: "t", value: null },
                "NBRTSRC" : { type: "i", value: 0 }
            },

            vertexShader: VertexShaderNPM.vertexShader,

            fragmentShader: 
            [
                ShaderUtils.get('DeclareDepthSamplersChunck'),
                ShaderUtils.get('EncodeDepthChunck'),
                ShaderUtils.get('DecodeDepthChunck'),
                   
                "void main()",
                "{",
                     ShaderUtils.get('SelectPeelChunck'),
                    "gl_FragColor = vec4(0., 0., 0., 0.);",
                "}"
            ].join("\n")
        }

        return DepthOnlyShader;
});