/**
 * @authors DOGO, zz85 / https://github.com/zz85 | https://www.lab4games.net/zz85/blog
 *
 * Edge Detection Shader using Sobel filter
 * Based on http://rastergrid.com/blog/2011/01/frei-chen-edge-detector
 *
 * @module WebViewer3D
 * @copyright   2016 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
		"WV3Dmod/shaders/ShaderUtils",
        "WV3Dmod/shaders/VertexShaderQuad"],
	function(THREE,
			ShaderUtils,
            VertexShaderQuad)
	{
	    /**
	     * @alias module:WebViewer3D
	     * @class EdgeeShader2
	     */

		var EdgeShader2 =
		{

			uniforms:
			{
				"depth_texture": { type: "t", value: null },
				"canvas_size": { type: "v2", value: new THREE.Vector2( 512, 512 ) },
				"color":    { type: "v3", value: new THREE.Vector3( 0., 0., 0. ) },

	            "offsetRepeat" : { type: "v4", value: new THREE.Vector4( 0, 0, 1, 1 ) },
	            "backTexture" : { type: "i", value: 0 },
	            "flipNormals" : { type: "i", value: 0 }
			},

			vertexShader: VertexShaderQuad.vertexShader,

			fragmentShader:
			[

				"uniform sampler2D depth_texture;",
	            "uniform vec3 color;",
				"uniform vec2 canvas_size;",

				"vec2 texel;",
				"vec2 vUv;",

				"mat3 G[2];",

				"const mat3 g0 = mat3( 1.0, 2.0, 1.0, 0.0, 0.0, 0.0, -1.0, -2.0, -1.0 );",//y filter (top left to bottom right)
				"const mat3 g1 = mat3( 1.0, 0.0, -1.0, 2.0, 0.0, -2.0, 1.0, 0.0, -1.0 );",//x filter (bottom right to top left)

                ShaderUtils.get('EncodeDepthChunck'),
                ShaderUtils.get('DecodeDepthChunck'),

				"void main(void)",
				"{",
					"mat3 I;",
					"float cnv[2];",
					"float depth;",
					"vec3 sample;",
					"vec3 value;",
					"vUv = gl_FragCoord.xy;",
					"texel = vec2(1.0 / canvas_size.x, 1.0 / canvas_size.y);",

					"G[0] = g0;",
					"G[1] = g1;",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2(-1.0,-1.0)) ));",
					"sample = vec3(depth);",
					"I[0][0] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 0.0,-1.0)) ));",
					"sample = vec3(depth);",
					"I[1][0] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 1.0,-1.0)) ));",
					"sample = vec3(depth);",
					"I[2][0] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2(-1.0, 0.0)) ));",
					"sample = vec3(depth);",
					"I[0][1] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 0.0, 0.0)) ));",
					"sample = vec3(depth);",
					"value = sample;",
					"I[1][1] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 1.0, 0.0)) ));",
					"sample = vec3(depth);",
					"I[2][1] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2(-1.0, 1.0)) ));",
					"sample = vec3(depth);",
					"I[0][2] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 0.0, 1.0)) ));",
					"sample = vec3(depth);",
					"I[1][2] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 1.0, 1.0)) ));",
					"sample = vec3(depth);",
					"I[2][2] = length(sample);",

					/* calculate the convolution values for all the masks */
					"for (int i=0; i<2; i++) {",
						"float dp3 = dot(G[i][0], I[0]) + dot(G[i][1], I[1]) + dot(G[i][2], I[2]);",
						"cnv[i] = dp3 * dp3; ",
					"}",

					"float val = 0.5 * sqrt(cnv[0]*cnv[0]+cnv[1]*cnv[1]);",
					"if (val > 0.02) gl_FragColor = vec4(color, 1.);",
					"else discard;",
				"} ",

			].join("\n")

		}

		return EdgeShader2;
	}
);