/**
 * @author DOGO
 *
 * Trivial peak detection by summing signed gradient values from "horizontal"
 * neighbors, from "vertical" neighbors, and from diagonals
 *
 * @module WebViewer3D
 * @copyright   2016 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
		"WV3Dmod/shaders/ShaderUtils",
        "WV3Dmod/shaders/VertexShaderQuad"],
	function(THREE,
			ShaderUtils,
            VertexShaderQuad)
	{
	    /**
	     * @alias module:WebViewer3D
	     * @class EdgeeShader3
	     */

		var EdgeShader3 =
		{

			uniforms:
			{
				"depth_texture": { type: "t", value: null },
				"canvas_size": { type: "v2", value: new THREE.Vector2( 512, 512 ) },
				"color":    { type: "v3", value: new THREE.Vector3( 0., 0., 0. ) },

	            "offsetRepeat" : { type: "v4", value: new THREE.Vector4( 0, 0, 1, 1 ) },
	            "backTexture" : { type: "i", value: 0 },
	            "flipNormals" : { type: "i", value: 0 }
			},

			vertexShader: VertexShaderQuad.vertexShader,

			fragmentShader:
			[

				"uniform sampler2D depth_texture;",
	            "uniform vec3 color;",
				"uniform vec2 canvas_size;",

				"vec2 texel;",
				"vec2 vUv;",

                ShaderUtils.get('EncodeDepthChunck'),
                ShaderUtils.get('DecodeDepthChunck'),

				"void main(void)",
				"{",
					"mat3 I;",
					"vUv = gl_FragCoord.xy;",
					"texel = vec2(1.0 / canvas_size.x, 1.0 / canvas_size.y);",

					"I[0][0] = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2(-1.0,-1.0))));",
					"I[1][0] = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 0.0,-1.0))));",
					"I[2][0] = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 1.0,-1.0))));",
					"I[0][1] = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2(-1.0, 0.0))));",
					"I[1][1] = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 0.0, 0.0))));",
					"I[2][1] = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 1.0, 0.0))));",
					"I[0][2] = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2(-1.0, 1.0))));",
					"I[1][2] = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 0.0, 1.0))));",
					"I[2][2] = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 1.0, 1.0))));",

					"float h_val = I[0][0] + 2.*I[1][0] + I[2][0] + I[0][2] + 2.*I[1][2] + I[2][2] - 8.*I[1][1];",
					"float v_val = I[0][0] + 2.*I[0][1] + I[0][2] + I[2][0] + 2.*I[2][1] + I[2][2] - 8.*I[1][1];",
					"float d1_val = 2.*I[0][0] + I[1][0] + I[0][1] + 2.*I[2][2] + I[1][2] + I[2][1] - 8.*I[1][1];",
					"float d2_val = I[1][0] + 2.*I[2][0] + I[2][1] + I[0][1] + 2.*I[0][2] + I[1][2] - 8.*I[1][1];",
					"if (abs(h_val) > 0.003 || abs(v_val) > 0.003 || abs(d1_val) > 0.003 ||abs(d2_val) > 0.003 ) gl_FragColor = vec4(color, 1.);",
					"else discard;",

				"} ",

			].join("\n")

		}

		return EdgeShader3;
	}
);