
/**
 * @author SVN & DOGO (Original by RAVO)
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
    "WV3Dmod/shaders/VertexShaderPM",
    "WV3Dmod/shaders/ShaderUtils"
    ],
    function(THREE,
         VertexShaderPM,
         ShaderUtils)
    {
      /**
       * @alias module:WebViewer3D
       * @class Filter Shader for Printable Materials
       */

    var FilterShaderPM = {

          uniforms: THREE.UniformsUtils.merge( [

             VertexShaderPM.uniforms,

            {
              "diffuseColor" : { type: "c", value: new THREE.Color( 0xffffff ) },
              "opacity" : { type: "f", value: null },
              "diffuseTexture" : { type: "t", value: null },

              "canvas_size": { type: "v2", value: new THREE.Vector2(1, 1) },
              "DX1RT" : { type: "t", value: null },
              "DX2RT" : { type: "t", value: null },
              "NBRTSRC" : { type: "i", value: 0 }
            }

          ] ),

          vertexShader: VertexShaderPM.vertexShader,

          fragmentShader: [

            "varying vec2 texcoordVarying;",
            "varying vec2 texcoordVaryingScaled;",

            "uniform vec3 diffuseColor;",
            "uniform float opacity;",
            "uniform sampler2D diffuseTexture;",

            ShaderUtils.get('DeclareDepthSamplersChunck'),
            ShaderUtils.get('EncodeDepthChunck'),
            ShaderUtils.get('DecodeDepthChunck'),

            "void main()",
            "{",
                ShaderUtils.get('SelectPeelChunck'),

               "vec4 diffColour;",
               "#ifdef USE_MAP",
                  "diffColour = texture2D(diffuseTexture, texcoordVaryingScaled);",
                  "#ifdef REMAPPED_DIFFUSE_ALPHA_CHANNEL",
                   //remapping due to WebGL IE11 desktop, Safari & Chrome (mobile devices) 
                   //premultiplying and dividing rgb by alpha when uploading rgba texture to GPU
                    "diffColour.a = clamp((diffColour.a - 127./255.) * 255. / 127., 0.,1.);",
                  "#endif",
               "#else ",
                 "diffColour = vec4(diffuseColor,opacity);",
               "#endif",

               "gl_FragColor = diffColour;",
               "gl_FragColor.a = 1.0;",
               // "gl_FragColor.a = diffColour.a ;",
            "}"

          ].join("\n")


    }

    return FilterShaderPM;

});