/**
 * @author SVN & DOGO (Original by RAVO)
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
   "WV3Dmod/shaders/VertexShaderNPM",
    "WV3Dmod/shaders/ShaderUtils"
    ],
    function(THREE,
         VertexShaderNPM,
         ShaderUtils)
    {
      /**
       * @alias module:WebViewer3D
       * @class Irradiance Shader for Printable Materials
       */

    var IrradianceShaderNPM = {

          uniforms: THREE.UniformsUtils.merge( [

        VertexShaderNPM.uniforms,

        {
          "diffuseColor" : { type: "c", value: new THREE.Color( 0xffffff ) },
          "opacity" : { type: "f", value: null },
          "diffuseTexture" : { type: "t", value: null },
          "isFloorReflection": { type: "i", value: 0 },
          "irradianceCubeMap" : { type: "t", value: null },
          "envMapRotQuat": { type: "v4", value: new THREE.Vector4(0, 0, 0, 1) },
          "shadowsShadingContrast" : { type: "f", value: 0.3},

          "canvas_size": { type: "v2", value: new THREE.Vector2(1, 1) },
          "DX1RT" : { type: "t", value: null },
          "DX2RT" : { type: "t", value: null },
          "NBRTSRC" : { type: "i", value: 0 }
        }

      ] ),

      vertexShader: VertexShaderNPM.vertexShader,

      fragmentShader: [

        "varying vec2 texcoordVaryingScaled;",
        "varying vec3 vNormal;",
        "varying vec4 vPosition;",
        "varying vec4 vWorldPosition;",

        "uniform vec3 diffuseColor;",
        "uniform float opacity;",
        "uniform sampler2D diffuseTexture;",
        "uniform bool isFloorReflection;",
        "uniform samplerCube irradianceCubeMap;",
        "uniform vec4 envMapRotQuat;",

        "uniform float shadowsShadingContrast;",

        ShaderUtils.get('DeclareDepthSamplersChunck'),
        ShaderUtils.get('EncodeDepthChunck'),
        ShaderUtils.get('DecodeDepthChunck'),
        ShaderUtils.get('EnvironmentRotationChunck'),

        "void main()",
        "{",
            ShaderUtils.get('SelectPeelChunck'),

           "#ifdef USE_MAP",
             "vec4 diffColour = texture2D(diffuseTexture, texcoordVaryingScaled) * vec4(diffuseColor,1.0);",
             "diffColour.a = opacity;",
           "#else",
             "vec4 diffColour = vec4(diffuseColor,opacity);",
           "#endif",

            "vec3 pN = normalize(vNormal);",

           // do the irradiance lookup using the normal
           "#ifdef USE_IRRMAP",
              "if(isFloorReflection) pN.y = -pN.y;",
              "pN = rotate_vec_from_quat(envMapRotQuat, pN);",
              "vec4 irradiance = textureCube(irradianceCubeMap, pN);",
           "#else",
              "vec4 irradiance = diffColour;",
           "#endif",

           "float shadowFactor;",
           "if(shadowsShadingContrast > 0.3)",
             "shadowFactor = 1.0 + ((shadowsShadingContrast - 0.3) / 0.7);", // this gives a shadowFactor of 2.0 at the 100% setting
           "else ",
             "shadowFactor = shadowsShadingContrast / 0.3;", // this is to match traditional Visualizer behaviour at shadowsShadingContrast = 0.3, which is the default;

           "gl_FragColor = clamp(shadowFactor * irradiance + (1.0 - shadowFactor), 0.0, 1.0) * diffColour.a;",
           "gl_FragColor.a = diffColour.a ;",

        "}"

   ].join("\n")
    }

    return IrradianceShaderNPM;
});