/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
		"WV3Dmod/shaders/ShaderUtils"
		],
		function(THREE,
				 ShaderUtils)	
		{
	    /**
	     * @alias module:WebViewer3D
	     * @class OutlineShader
	     */

		var OutlineShader = {

			uniforms: 
			{
				"offset": {type: "f", value: 1},
				"color":    { type: "v3", value: new THREE.Vector3( 0., 0., 0. ) },
				"D0RT" : { type: "t", value: null },		// only used if USE_DEPTH_TEXTURE is defined
                "canvas_size": { type: "v2", value: new THREE.Vector2(0, 0) } 	// only used if USE_DEPTH_TEXTURE is defined
			},

			vertexShader: 
			[
	            "uniform float offset;",
	            "void main()",
				"{",

					"vec4 pos = modelViewMatrix * vec4( position + normal * offset, 1.0 );",
					"gl_Position = projectionMatrix * pos;",
	            "}"
	        ].join("\n"),

	        fragmentShader: 
	        [
	        	"#ifdef USE_DEPTH_TEXTURE",
	        	ShaderUtils.get('DeclareDepthSamplerChunck'),
	        	ShaderUtils.get('DecodeDepthChunck'),
	        	"#endif",

	            "uniform vec3 color;",
	            "void main()",
				"{",
					// for scenes with transparency, we have to check the "manual" depth texture instead of the hardware depth buffer
					"#ifdef USE_DEPTH_TEXTURE",
						"vec2 loc = vec2(gl_FragCoord.x/canvas_size.x, gl_FragCoord.y/canvas_size.y);",
                    	"float scenez = DecodeFloatRGBA(texture2D(D0RT, loc));",
                    	"float z = 1.0 - gl_FragCoord.z;",
                    	"if (z < scenez )",
                    		"discard;",
	        		"#endif",

	                "gl_FragColor = vec4( color, 1.0 );",
	            "}"
	        ].join("\n")

		}

		return OutlineShader;

});