/**
 * @author SVN
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
        "WV3Dmod/shaders/VertexShaderNPM",
        "WV3Dmod/shaders/ShaderUtils"
        ],
        function(THREE,
                 VertexShaderNPM,
                 ShaderUtils)   
        {
        /**
         * @alias module:WebViewer3D
         * @class TransparencyBlendShaderMaterial: blends the current peel transparency with the combined previous peels, to get per-channel transparency
         *          Blending is not done in the shader but with the fixed-function blending, so the previous transprency stack should already be copied
         *          into the rendertarget.
         */

        var TransBlendShader = {

            uniforms: THREE.UniformsUtils.merge( [
                VertexShaderNPM.uniforms,
                {
                    "diffuseColor" : { type: "c", value: new THREE.Color( 0xffffff ) },
                    "opacity" : { type: "f", value: null },
                    "diffuseTexture" : { type: "t", value: null },

                    "canvas_size": { type: "v2", value: new THREE.Vector2(0, 0) },
                    "DX1RT" : { type: "t", value: null },   // we use the same depth compares as the D2 depth shader
                    "DX2RT" : { type: "t", value: null },
                    "NBRTSRC" : { type: "i", value: 0 },
                },
            ] ),

            vertexShader: VertexShaderNPM.vertexShader,

            fragmentShader: 
            [
                "varying vec2 texcoordVaryingScaled;",

                "uniform vec3 diffuseColor;",
                "uniform float opacity;",
                "uniform sampler2D diffuseTexture;",

                ShaderUtils.get('DeclareDepthSamplersChunck'),
                ShaderUtils.get('EncodeDepthChunck'),
                ShaderUtils.get('DecodeDepthChunck'),
                   
                "void main()",
                "{",
                    ShaderUtils.get('SelectPeelChunck'),
                    
                    "vec4 diffColour;",
                    "#ifdef USE_MAP",
                        "diffColour = texture2D(diffuseTexture, texcoordVaryingScaled);",                        
                        "#ifdef IS_NPM",
                            "diffColour.a = opacity;", 
                        "#else",
                           "#ifdef REMAPPED_DIFFUSE_ALPHA_CHANNEL",
                            //remapping due to WebGL IE11 desktop, Safari & Chrome (mobile devices) 
                            //premultiplying and dividing rgb by alpha when uploading rgba texture to GPU
                            "diffColour.a = clamp((diffColour.a - 127./255.) * 255. / 127., 0.,1.);",
                           "#endif",
                        "#endif",
                    "#else ",
                        "diffColour = vec4(diffuseColor,opacity);",
                    "#endif",
                    
                    "gl_FragColor.rgb = diffColour.rgb * ( 1.0 - diffColour.a );",
                    "gl_FragColor.a = 1.0;",
                "}"
            ].join("\n")

        }

        return TransBlendShader;

});