
/**
 * @author SVN & DOGO (Original by RAVO)
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three"],
    function(THREE)
    {
      /**
       * @alias module:WebViewer3D
       * @class Vertex Shader for Printable Materials
       */

    var VertexShaderPM =

        {

          uniforms: {

            "offsetRepeat" : { type: "v4", value: new THREE.Vector4( 0, 0, 1, 1 ) },
            "backTexture" : { type: "i", value: 0 },
            "flipNormals" : { type: "i", value: 0 },

            "ambientLightColor" : { type: "fv", value: [] },

            "directionalLightDirection" : { type: "fv", value: [] },
            "directionalLightColor" : { type: "fv", value: [] },

            "hemisphereLightDirection" : { type: "fv", value: [] },
            "hemisphereLightSkyColor" : { type: "fv", value: [] },
            "hemisphereLightGroundColor" : { type: "fv", value: [] },

            "pointLightColor" : { type: "fv", value: [] },
            "pointLightPosition" : { type: "fv", value: [] },
            "pointLightDistance" : { type: "fv1", value: [] },
            "pointLightDecay" : { type: "fv1", value: [] },

            "spotLightColor" : { type: "fv", value: [] },
            "spotLightPosition" : { type: "fv", value: [] },
            "spotLightDirection" : { type: "fv", value: [] },
            "spotLightDistance" : { type: "fv1", value: [] },
            "spotLightAngleCos" : { type: "fv1", value: [] },
            "spotLightExponent" : { type: "fv1", value: [] },
            "spotLightDecay" : { type: "fv1", value: [] }
          },

          vertexShader: [

            "attribute vec3 inst_translation;",
            "attribute vec4 inst_rotation;",
            "attribute vec3 inst_scale;",

            "varying vec3 vNormal;",
            "varying vec4 vPosition;",
            "varying vec4 vWorldPosition;",
            "varying vec2 texcoordVarying;",
            "varying vec2 texcoordVaryingScaled;",
            "uniform vec4 offsetRepeat;",
            "uniform int backTexture;",
            "uniform int flipNormals;",

            "void main()",
            "{",
                "vec3 temp = cross(inst_rotation.xyz, normal.xyz / inst_scale);",
                "vec3 rotNormal = temp * (2.0 * inst_rotation.w) + (cross(inst_rotation.xyz, temp) * 2.0 + normal.xyz);",

                "vNormal = mat3(modelMatrix) * ((flipNormals == 1) ? -rotNormal : rotNormal);",
                "vec4 newPosition = vec4(position,1.0);",
                "vPosition = newPosition;",

                "vec4 worldPosition = newPosition * vec4(inst_scale,1.0);",
                "temp = cross(inst_rotation.xyz, worldPosition.xyz);",
                "vec3 rotPosition = temp * (2.0 * inst_rotation.w) + (cross(inst_rotation.xyz, temp) * 2.0 + worldPosition.xyz);",
                "worldPosition = modelMatrix * vec4(rotPosition + inst_translation,1.0);",
                "vWorldPosition = worldPosition;",

                "vec2 texcoord = (backTexture == 1) ? vec2(1. - uv.x, uv.y) : uv;",
                "texcoordVarying = texcoord;",//used for bump map
                "texcoordVaryingScaled = texcoord * offsetRepeat.zw + offsetRepeat.xy;",//used for standard textures

                "gl_Position = projectionMatrix * viewMatrix * worldPosition;",
            "}"

          ].join("\n")

        }

    return VertexShaderPM;

});


