

/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2017 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three"],
    function(THREE)
    {
      /**
       * @alias module:WebViewer3D
       * @class Vertex Shader for Screen Quad
       */

    var VertexShaderQuad =

        {

          uniforms :  {

            "surfaceSide" : { type: "i", value: 0 },

            "offsetRepeat" : { type: "v4", value: new THREE.Vector4( 0, 0, 1, 1 ) },
            "backTexture" : { type: "i", value: 0 },
            "flipNormals" : { type: "i", value: 0 },

            "ambientLightColor" : { type: "fv", value: [] },

            "directionalLightDirection" : { type: "fv", value: [] },
            "directionalLightColor" : { type: "fv", value: [] },

            "hemisphereLightDirection" : { type: "fv", value: [] },
            "hemisphereLightSkyColor" : { type: "fv", value: [] },
            "hemisphereLightGroundColor" : { type: "fv", value: [] },

            "pointLightColor" : { type: "fv", value: [] },
            "pointLightPosition" : { type: "fv", value: [] },
            "pointLightDistance" : { type: "fv1", value: [] },
            "pointLightDecay" : { type: "fv1", value: [] },

            "spotLightColor" : { type: "fv", value: [] },
            "spotLightPosition" : { type: "fv", value: [] },
            "spotLightDirection" : { type: "fv", value: [] },
            "spotLightDistance" : { type: "fv1", value: [] },
            "spotLightAngleCos" : { type: "fv1", value: [] },
            "spotLightExponent" : { type: "fv1", value: [] },
            "spotLightDecay" : { type: "fv1", value: [] }

          },

          vertexShader : [

            "varying vec3 vNormal;",
            "varying vec4 vPosition;",
            "varying vec4 vWorldPosition;",
            "varying vec2 texcoordVaryingScaled;",
            "uniform vec4 offsetRepeat;",
            "uniform int backTexture;",
            "uniform int flipNormals;",

            "void main()",
            "{",
                "vNormal = mat3(modelMatrix) * ((flipNormals == 1) ? -normal : normal);",
                "vec4 newPosition = vec4(position,1.0);",
                "vPosition = newPosition;",

                "vec4 worldPosition = modelMatrix * newPosition;",
                "vWorldPosition = worldPosition;",

                "vec2 texcoord = uv;",
                "texcoordVaryingScaled = texcoord * offsetRepeat.zw + offsetRepeat.xy;",//used for standard textures

                "gl_Position = projectionMatrix * viewMatrix * worldPosition;",

            "}"

          ].join("\n")

        }

    return VertexShaderQuad;

});