/**
 * @author DOGO, parts taken from alteredq (http://alteredqualia.com/)
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

 define([
    "WV3Dlib/three",
    "backbone",
    "WV3Dmod/shaders/CopyShader"
    ], function (
        THREE,
        bb,
        CopyShader
        ) {

    'use strict';


    /**
     * @alias module:WV3DThreeEXT
     * @class WV3DThreeEXT
     */
    var WV3DThreeEXT = bb.Model.extend({

        /**
         * Extend 3JS capability by overwritting some existing functions and creating new ones
         */
        initialize: function()
        {

            //New functions for threejs

            /**
            * Set a callback traversal to the parents of the node
            * @function
            */
            THREE.Object3D.prototype.traverseParents = function ( callback )
            {
                callback( this );

                if ( this.parent ) this.parent.traverseParents( callback );
            };

            /**
            * Add "static" function to THREE.Vector3 to allow transforming a vertex using
            * translation rotation and scale
            */
            if(THREE.Vector3.transformVertex)
            {
                console.warn("Overriding transformVertex");
            }
            THREE.Vector3.transformVertex = function (__vertex, __transform)
            {
                var scaledPosition = new THREE.Vector3();
                scaledPosition.copy(__vertex).multiply(__transform["vScale"]);
                var tempVec = new THREE.Vector3();
                tempVec.crossVectors(__transform["vRotationXYZ"], scaledPosition);
                var vPosition = tempVec.clone().multiplyScalar(2.0 * __transform["vRotationW"]).add(__transform["vRotationXYZ"].clone().cross( tempVec ).multiplyScalar(2.0).add(__vertex));
                vPosition.add(__transform["vTranslation"]);
                return vPosition;
            }

            /**
            * Add "static" function to THREE to allow to build a transform using
            * translation rotation and scale buffers
            */
            if(THREE.GetInstanceTransform)
            {
                console.warn("Overriding GetInstanceTransform");
            }
            THREE.GetInstanceTransform = function (__translations, __rotations, __scales, __inst_nb)
            {
                var transform =
                {
                    "vTranslation": new THREE.Vector3(__translations.getX(__inst_nb),__translations.getY(__inst_nb), __translations.getZ(__inst_nb)),
                    "vRotationXYZ": new THREE.Vector3(__rotations.getX(__inst_nb), __rotations.getY(__inst_nb), __rotations.getZ(__inst_nb)),
                    "vRotationW": __rotations.getW(__inst_nb),
                    "vScale": new THREE.Vector3(__scales.getX(__inst_nb), __scales.getY(__inst_nb), __scales.getZ(__inst_nb))
                }
                return transform;
            }

            /**
            * New bbox functionalities which has, in our case, to be extended
            * to support instanced geometries (used for frustum culling)
            */
            if(THREE.Box3.prototype.ExpandFromInstances)
            {
                console.warning("Overriding ExpandFromInstances");
            }
            THREE.Box3.prototype.ExpandFromInstances = function(__localTempBox, __translations, __rotations, __scales, __nbInstances)
            {
                var p1, p2, p3, p4, p5, p6, p7, p8;
                p1 = new THREE.Vector3( __localTempBox.min.x, __localTempBox.min.y, __localTempBox.min.z ); // 000
                p2 = new THREE.Vector3( __localTempBox.min.x, __localTempBox.min.y, __localTempBox.max.z ); // 001
                p3 = new THREE.Vector3( __localTempBox.min.x, __localTempBox.max.y, __localTempBox.min.z ); // 010
                p4 = new THREE.Vector3( __localTempBox.min.x, __localTempBox.max.y, __localTempBox.max.z ); // 011
                p5 = new THREE.Vector3( __localTempBox.max.x, __localTempBox.min.y, __localTempBox.min.z ); // 100
                p6 = new THREE.Vector3( __localTempBox.max.x, __localTempBox.min.y, __localTempBox.max.z ); // 101
                p7 = new THREE.Vector3( __localTempBox.max.x, __localTempBox.max.y, __localTempBox.min.z ); // 110
                p8 = new THREE.Vector3( __localTempBox.max.x, __localTempBox.max.y, __localTempBox.max.z ); // 111

                if( __translations !== undefined && __rotations !== undefined && __scales !== undefined)
                {
                    for( var ins = 0; ins < __nbInstances; ++ins)
                    {
                        var transform = THREE.GetInstanceTransform(__translations, __rotations, __scales, ins);

                        this.expandByPoint(THREE.Vector3.transformVertex(p1, transform));
                        this.expandByPoint(THREE.Vector3.transformVertex(p2, transform));
                        this.expandByPoint(THREE.Vector3.transformVertex(p3, transform));
                        this.expandByPoint(THREE.Vector3.transformVertex(p4, transform));
                        this.expandByPoint(THREE.Vector3.transformVertex(p5, transform));
                        this.expandByPoint(THREE.Vector3.transformVertex(p6, transform));
                        this.expandByPoint(THREE.Vector3.transformVertex(p7, transform));
                        this.expandByPoint(THREE.Vector3.transformVertex(p8, transform));
                    }
                }
                else
                {
                    if(!warned)
                    {
                        console.warn("Attribute for InstancedBufferGeometry missing.");
                        warned = true;
                    }

                    this.expandByPoint(p1);
                    this.expandByPoint(p2);
                    this.expandByPoint(p3);
                    this.expandByPoint(p4);
                    this.expandByPoint(p5);
                    this.expandByPoint(p6);
                    this.expandByPoint(p7);
                    this.expandByPoint(p8);
                }
            }

            //Override 'basic' material to support instancing
            //only 'project_vertex' chunck and the three first lines were modified
            THREE.ShaderLib['basic'].vertexShader = "\n\
                attribute vec3 inst_translation;//ESKO specific\n\
                attribute vec4 inst_rotation;//ESKO specific\n\
                attribute vec3 inst_scale;//ESKO specific\n\
                #include <common>\n\
                #include <uv_pars_vertex>\n\
                #include <uv2_pars_vertex>\n\
                #include <envmap_pars_vertex>\n\
                #include <color_pars_vertex>\n\
                #include <morphtarget_pars_vertex>\n\
                #include <skinning_pars_vertex>\n\
                #include <logdepthbuf_pars_vertex>\n\
                #include <clipping_planes_pars_vertex>\n\
                void main() {\n\
                    #include <uv_vertex>\n\
                    #include <uv2_vertex>\n\
                    #include <color_vertex>\n\
                    #include <skinbase_vertex>\n\
                    #ifdef USE_ENVMAP\n\
                        #include <beginnormal_vertex>\n\
                        #include <morphnormal_vertex>\n\
                        #include <skinnormal_vertex>\n\
                        #include <defaultnormal_vertex>\n\
                    #endif\n\
                    #include <begin_vertex>\n\
                    #include <morphtarget_vertex>\n\
                    #include <skinning_vertex>\n\
                    vec3 worldposition = transformed.xyz * inst_scale;//ESKO Specific\n\
                    vec3 vcV = cross(inst_rotation.xyz, worldposition);//ESKO Specific\n\
                    vec3 vPosition = vcV * (2.0 * inst_rotation.w) + (cross(inst_rotation.xyz, vcV) * 2.0 + worldposition);//ESKO Specific\n\
                    vec4 mvPosition = modelViewMatrix * vec4( vPosition + inst_translation, 1.0 );\n\
                    gl_Position = projectionMatrix * mvPosition;\n\
                    #include <logdepthbuf_vertex>\n\
                    #include <worldpos_vertex>\n\
                    #include <clipping_planes_vertex>\n\
                    #include <envmap_vertex>\n\
                }\n";

            //Override setFromObject functionality which has, in our case, to be extended
            //to support instanced geometries (used for frustum culling)
            THREE.Box3.prototype.setFromObject = function ()
            {
                // Computes the world-axis-aligned bounding box of an object (including its children),
                // accounting for both the object's, and children's, world transforms

                var v1 = new THREE.Vector3();
                var warned = false;

                return function ( object ) {

                    var scope = this;

                    object.updateMatrixWorld( true );

                    this.makeEmpty();

                    object.traverse( function ( node ) {

                        var geometry = node.geometry;

                        if ( geometry !== undefined ) {

                            if ( geometry instanceof THREE.Geometry ) {

                                var vertices = geometry.vertices;

                                for ( var i = 0, il = vertices.length; i < il; i ++ ) {

                                    v1.copy( vertices[ i ] );
                                    v1.applyMatrix4( node.matrixWorld );

                                    scope.expandByPoint( v1 );

                                }

                            } else if ( geometry instanceof THREE.BufferGeometry && geometry.attributes[ 'position' ] !== undefined)
                            {
                                var positions = geometry.attributes[ 'position' ].array;
                                var localTempBox;
                                if(geometry instanceof THREE.InstancedBufferGeometry)
                                {
                                    localTempBox = new THREE.Box3();
                                }

                                for ( var i = 0, il = positions.length; i < il; i += 3 ) {

                                    v1.fromArray( positions, i );
                                    v1.applyMatrix4( node.matrixWorld );

                                    if(geometry instanceof THREE.InstancedBufferGeometry)
                                    {
                                        localTempBox.expandByPoint( v1 );
                                    }
                                    else
                                    {
                                        scope.expandByPoint( v1 );
                                    }

                                }

                                if(geometry instanceof THREE.InstancedBufferGeometry)
                                {
                                    scope.ExpandFromInstances(localTempBox, geometry.attributes[ 'inst_translation' ], geometry.attributes[ 'inst_rotation' ], geometry.attributes[ 'inst_scale' ], geometry.maxInstancedCount);
                                }

                            }

                        }

                    } );

                    return this;

                };

            }();

            //Override Compute sphere in object space which has, in our case, to be extended
            //to support instanced geometries (used for frustum culling)
            THREE.InstancedBufferGeometry.prototype.computeBoundingSphere = function () {

                var box = new THREE.Box3();
                var localTempBox = new THREE.Box3();
                var vector = new THREE.Vector3();
                var vector2 = new THREE.Vector3();
                var warned = false;

                return function computeBoundingSphere() {

                    if ( this.boundingSphere === null ) {

                        this.boundingSphere = new THREE.Sphere();

                    }

                    var positions = this.attributes.position.array;

                    if ( positions ) {

                        var center = this.boundingSphere.center;

                        for ( var i = 0, il = positions.length; i < il; i += 3 )
                        {
                            vector.fromArray( positions, i );
                            localTempBox.expandByPoint( vector );
                        }

                        box.ExpandFromInstances(localTempBox, this.attributes[ 'inst_translation' ], this.attributes[ 'inst_rotation' ], this.attributes[ 'inst_scale' ], this.maxInstancedCount);

                        box.center( center );

                        // hoping to find a boundingSphere with a radius smaller than the
                        // boundingSphere of the boundingBox: sqrt(3) smaller in the best case

                        var maxRadiusSq = 0;

                        for( var ins = 0, nbins = this.maxInstancedCount; ins < nbins; ins += 1)
                        {
                            if( this.attributes[ 'inst_translation' ] !== undefined
                             && this.attributes[ 'inst_rotation' ] !== undefined
                             && this.attributes[ 'inst_scale' ] !== undefined)
                            {
                                var transform = THREE.GetInstanceTransform(this.attributes[ 'inst_translation' ], this.attributes[ 'inst_rotation' ], this.attributes[ 'inst_scale' ], ins);

                                for ( var i = 0, il = positions.length; i < il; i += 3 ) {

                                    vector.fromArray( positions, i );
                                    var vPosition = THREE.Vector3.transformVertex(vector, transform);
                                    maxRadiusSq = Math.max( maxRadiusSq, center.distanceToSquared( vPosition ) );
                                }
                            }
                            else
                            {
                                for ( var i = 0, il = positions.length; i < il; i += 3 ) {

                                    vector.fromArray( positions, i );
                                    maxRadiusSq = Math.max( maxRadiusSq, center.distanceToSquared( vector ) );
                                }
                            }
                        }

                        this.boundingSphere.radius = Math.sqrt( maxRadiusSq );

                        if ( isNaN( this.boundingSphere.radius ) ) {

                            console.error( 'THREE.BufferGeometry.computeBoundingSphere(): Computed radius is NaN. The "position" attribute is likely to have NaN values.', this );

                        }

                    }

                };

            }()

            /**
             * @author alteredq / http://alteredqualia.com/
             */

            THREE.ShaderPass = function ( shader, textureID )
            {

                this.textureID = ( textureID !== undefined ) ? textureID : "tDiffuse";

                this.uniforms = THREE.UniformsUtils.clone( shader.uniforms );

                this.material = new THREE.ShaderMaterial( {

                    uniforms: this.uniforms,
                    vertexShader: shader.vertexShader,
                    fragmentShader: shader.fragmentShader

                } );

                this.renderToScreen = false;

                this.enabled = true;
                this.needsSwap = true;
                this.clear = false;

            };

            THREE.ShaderPass.prototype =
            {

                render: function ( renderer, writeBuffer, readBuffer, delta )
                {

                    if ( this.uniforms[ this.textureID ] )
                    {

                        this.uniforms[ this.textureID ].value = readBuffer;

                    }

                    THREE.EffectComposer.quad.material = this.material;

                    if ( this.renderToScreen )
                    {

                        renderer.render( THREE.EffectComposer.scene, THREE.EffectComposer.camera );

                    } else {

                        renderer.render( THREE.EffectComposer.scene, THREE.EffectComposer.camera, writeBuffer, this.clear );

                    }

                }

            };

            /**
             * @author alteredq / http://alteredqualia.com/
             */

            THREE.EffectComposer = function ( renderer, renderTarget ) {

                this.renderer = renderer;

                if ( renderTarget === undefined ) {

                    var width = window.innerWidth || 1;
                    var height = window.innerHeight || 1;
                    var parameters = { minFilter: THREE.LinearFilter, magFilter: THREE.LinearFilter, format: THREE.RGBFormat, stencilBuffer: false };

                    renderTarget = new THREE.WebGLRenderTarget( width, height, parameters );

                }

                this.renderTarget1 = renderTarget;
                this.renderTarget2 = renderTarget.clone();

                this.writeBuffer = this.renderTarget1;
                this.readBuffer = this.renderTarget2;

                this.passes = [];

                if ( CopyShader === undefined )
                    console.error( "THREE.EffectComposer relies on CopyShader" );

                this.copyPass = new THREE.ShaderPass( CopyShader );

            };

            THREE.EffectComposer.prototype = {

                swapBuffers: function() {

                    var tmp = this.readBuffer;
                    this.readBuffer = this.writeBuffer;
                    this.writeBuffer = tmp;

                },

                addPass: function ( pass ) {

                    this.passes.push( pass );

                },

                insertPass: function ( pass, index ) {

                    this.passes.splice( index, 0, pass );

                },

                render: function ( delta , scene, camera) {

                    this.writeBuffer = this.renderTarget1;
                    this.readBuffer = this.renderTarget2;

                    var maskActive = false;

                    var pass, i, il = this.passes.length;

                    for ( i = 0; i < il; i ++ ) {

                        pass = this.passes[ i ];

                        if(scene) pass.scene = scene;//Update the current scene reference if needed

                        if(camera) pass.camera = camera;//Update the current camera reference if needed

                        if ( !pass.enabled ) continue;

                        pass.render( this.renderer, this.writeBuffer, this.readBuffer, delta, maskActive );

                        this.renderer.autoClear = false;

                        if ( pass.needsSwap ) {

                            if ( maskActive ) {

                                var context = this.renderer.context;

                                context.stencilFunc( context.NOTEQUAL, 1, 0xffffffff );

                                this.copyPass.render( this.renderer, this.writeBuffer, this.readBuffer, delta );

                                context.stencilFunc( context.EQUAL, 1, 0xffffffff );

                            }

                            this.swapBuffers();

                        }

                        if ( pass instanceof THREE.MaskPass ) {

                            maskActive = true;

                        } else if ( pass instanceof THREE.ClearMaskPass ) {

                            maskActive = false;

                        }

                    }

                },

                reset: function ( renderTarget ) {

                    if ( renderTarget === undefined ) {

                        renderTarget = this.renderTarget1.clone();

                        renderTarget.width = window.innerWidth;
                        renderTarget.height = window.innerHeight;

                    }

                    this.renderTarget1 = renderTarget;
                    this.renderTarget2 = renderTarget.clone();

                    this.writeBuffer = this.renderTarget1;
                    this.readBuffer = this.renderTarget2;

                },

                setSize: function ( width, height ) {

                    var renderTarget = this.renderTarget1.clone();

                    renderTarget.width = width;
                    renderTarget.height = height;

                    this.reset( renderTarget );

                }

            };

            // shared ortho camera

            THREE.EffectComposer.camera = new THREE.OrthographicCamera( -1, 1, 1, -1, 0, 1 );

            THREE.EffectComposer.quad = new THREE.Mesh( new THREE.PlaneGeometry( 2, 2 ), null );

            THREE.EffectComposer.scene = new THREE.Scene();
            THREE.EffectComposer.scene.add( THREE.EffectComposer.quad );


            /**
             * @author alteredq / http://alteredqualia.com/
             */

            THREE.RenderPass = function ( scene, camera, overrideMaterial, clearColor, clearAlpha ) {

                this.scene = scene;
                this.camera = camera;

                this.overrideMaterial = overrideMaterial;

                this.clearColor = clearColor;
                this.clearAlpha = ( clearAlpha !== undefined ) ? clearAlpha : 1;

                this.oldClearColor = new THREE.Color();
                this.oldClearAlpha = 1;

                this.enabled = true;
                this.clear = true;
                this.needsSwap = false;

            };

            THREE.RenderPass.prototype = {

                render: function ( renderer, writeBuffer, readBuffer, delta ) {

                    this.scene.overrideMaterial = this.overrideMaterial;

                    if ( this.clearColor ) {

                        this.oldClearColor.copy( renderer.getClearColor() );
                        this.oldClearAlpha = renderer.getClearAlpha();

                        renderer.setClearColor( this.clearColor, this.clearAlpha );

                    }

                    renderer.render( this.scene, this.camera, readBuffer, this.clear );

                    if ( this.clearColor ) {

                        renderer.setClearColor( this.oldClearColor, this.oldClearAlpha );

                    }

                    this.scene.overrideMaterial = null;

                }

            };

            /**
             * @author alteredq / http://alteredqualia.com/
             */

            THREE.MaskPass = function ( scene, camera ) {

                this.scene = scene;
                this.camera = camera;

                this.enabled = true;
                this.clear = true;
                this.needsSwap = false;

                this.inverse = false;

            };

            THREE.MaskPass.prototype = {

                render: function ( renderer, writeBuffer, readBuffer, delta ) {

                    var context = renderer.context;

                    // don't update color or depth

                    context.colorMask( false, false, false, false );
                    context.depthMask( false );

                    // set up stencil

                    var writeValue, clearValue;

                    if ( this.inverse ) {

                        writeValue = 0;
                        clearValue = 1;

                    } else {

                        writeValue = 1;
                        clearValue = 0;

                    }

                    context.enable( context.STENCIL_TEST );
                    context.stencilOp( context.REPLACE, context.REPLACE, context.REPLACE );
                    context.stencilFunc( context.ALWAYS, writeValue, 0xffffffff );
                    context.clearStencil( clearValue );

                    // draw into the stencil buffer

                    renderer.render( this.scene, this.camera, readBuffer, this.clear );
                    renderer.render( this.scene, this.camera, writeBuffer, this.clear );

                    // re-enable update of color and depth

                    context.colorMask( true, true, true, true );
                    context.depthMask( true );

                    // only render where stencil is set to 1

                    context.stencilFunc( context.EQUAL, 1, 0xffffffff );  // draw if == 1
                    context.stencilOp( context.KEEP, context.KEEP, context.KEEP );

                }

            };


            THREE.ClearMaskPass = function () {

                this.enabled = true;

            };

            THREE.ClearMaskPass.prototype = {

                render: function ( renderer, writeBuffer, readBuffer, delta ) {

                    var context = renderer.context;

                    context.disable( context.STENCIL_TEST );

                }

            };

        }



    });

return WV3DThreeEXT;

});
