define(["wcr/webcenter_html",
		"utils/webi18n!Approval",
		"utils/datetools"
	],
	function (WCHTML,
		i18n,
		datetools) {
		"use strict";

		var ApprovalUtils = {

			/**
			 * Finds in this.projectMembers the member with this userId or groupId and returns the corresponding member object
			 * This is a recursive function!
			 * @param memberType "User" or "Group"
			 * @param id userId or GroupId depending on the type
			 * @param memberArr array containing members of the project or a group
			 */
			findMemberObj: function (memberType, id, memberArr) {
				try {
					var memberObj = null;
					if (!memberArr) {
						return null;
					}
					for (var member = 0; member < memberArr.length && !memberObj; member++) {
						if (memberArr[member].memberType == "User") {
							if (memberType == "User" && memberArr[member].userId == id) {
								memberObj = memberArr[member];
							}
						} else {
							// First search whether it's this group/role
							if (memberType == "Group" && memberArr[member].groupId == id) {
								memberObj = memberArr[member];
							}

							// Then search IN the group if you didn't find a match yet and if the group has members
							// This is a recursive call!
							// Remark: during code review, JANH asked whether there is no risk of an endless loop
							// when a group contains "itself" directly or indirectly
							// some research shows that you can't get into this situation because the XML
							// representing this situation is endlessly long (because each group must show its group members,
							// which again have to show their group members etc).
							// Obviously groups containing themselves are not a correct use case.
							// At the time of writing, group in group is not supported anyway. 
							if (!memberObj && memberArr[member].members) {
								memberObj = this.findMemberObj(memberType, id, memberArr[member].members);
							}
						}
					}
					return memberObj; // can be null!
				} catch (err) {
					window.alert("Problem in findMemberObj: " + err);
				}
			},

			/**
			 * Converts a date objecto a string representation
			 * @param   {Date} date
			 * @returns {string} formatted result
			 */
			getDateHourString: function (date) {
				try {
					// adapt summer/winter time
					var showDate = datetools.fixDateSummerWinterTime(date, false);
					var dateString = WCHTML.getDateString(showDate);
					var hourString = showDate.getHours(); // forcing it to be a string
					if (hourString < 10) {
						hourString = "0" + hourString;
					}
					var minuteString = showDate.getMinutes();
					if (minuteString < 10) {
						minuteString = "0" + minuteString;
					}
					//hourString = hourString.substring(0,hourString.length-3);  // parse off seconds
					dateString += '  ' + hourString + ":" + minuteString;
					return dateString;
				} catch (err) {
					return date;
				}

			},

			/**
			 * get approval type language tag
			 * @param approverObj
			 */
			getApprovalTypeLangTag: function (approverObj) {
				var approvalTypeLangTag = 'CApp' + approverObj.approvalType;
				if (approverObj.memberObj && approverObj.memberObj.isRole === true) {
					if (approverObj.approvalType == "OneOfGroup") {
						approvalTypeLangTag = "CAppAllOfRoleOneOfGroup";
					}
					if (approverObj.approvalType == "AllOfGroup") {
						approvalTypeLangTag = "CAppAllOfGroup";
					}
				}
				return approvalTypeLangTag;
			},

			/**
			 * returns the current time rounded off to the next hour but not going to the next day
			 * when after 23:00
			 */
			getNewCurrentTime: function () {
				var newDate = new Date();
				if (newDate.getHours() < 23) {
					newDate.setHours(newDate.getHours() + 1);
				}
				newDate.setMinutes(0);
				newDate.setSeconds(0);
				return newDate;
			},

			/**
			 * Get the ful name of a memberobj
			 * @param memberObj
			 * @returns
			 */
			getUserRepresentation: function (memberObj) {
				var user = {};
				user.name = "";
				if (!memberObj) {
					return user;
				}
				user.isGroup = memberObj.groupName !== undefined;
				user.isRole = memberObj.isRole;
				user.isUser = memberObj.lastName !== undefined;

				if (user.isUser) {
					// normal user
					user.name = WCHTML.getUserNameStringActiveHTML(null, memberObj.firstName, memberObj.lastName, memberObj.userName, memberObj.isActive, i18n.t("CAppUserDeactivated"), true);
				}

				if (user.isGroup) {
					user.name = memberObj.groupName;
				}
				return user;
			}

		};

		return ApprovalUtils;
	});
