define(["jquery", "jqueryUI", "jqueryUITouchPunch"], function ($, jQueryUI, jQueryUITouchPunch) {

	"use strict";

	/**
	 * This class initiates the sortable function on the approval view
	 */
	var SortableController = function (approvalView, approvalModel) {
		var sortableStartIndex = null;
		var sortableStopIndex = null;

		this._approvalView = approvalView;

		approvalView.getStagesContainer().sortable({
				axis: "y",
				items: ".approval-stage-container-sortable",
				cancel: "input,select,a,label",
				scroll: "false",
				cursor: "grabbing",
				containment: "body",
				placeholder: "approval-stage-container-sortable-placeholder",
				forcePlaceholderSize: true,
				handle: ".sortable-handler",
				tolerance: "intersect",
				start: function (event, ui) {
					ui.item.addClass("being-sorted");
					sortableStartIndex = ui.item.index();
				},
				stop: function (event, ui) {
					sortableStopIndex = ui.item.index();
					ui.item.removeClass("being-sorted");
					approvalModel.moveStage(sortableStartIndex, sortableStopIndex);
				}.bind(this)
			}

		);

	};

	/** 
	 * Disable the sortable functionality
	 */
	SortableController.prototype.disable = function () {
		this._approvalView.getStagesContainer().sortable("disable");
	};

	/** 
	 * enable the sortable functionality
	 */
	SortableController.prototype.enable = function () {
		this._approvalView.getStagesContainer().sortable("enable");
	};

	return SortableController;
});
