define(["approval/ApprovalConstants",
        "utils/Observable",
        "wcr/webcenter_xml",
        "jquery",
        "underscore"], 
function(ApprovalConstants,
		Observable,
		WCXML,
		$,
		_) {	
	
	"use strict";
	
	function ApprovalMember() { 
	}
	
	/**
	* Reads project members from xml element containing <member> elements
	* Called recursively for groups containing members (and groups which again can contain members...)
	* @param xml xml element (<member>) containing the members.  
	* @param isMember: true/false if we are reading users/groups who are still members of the project or not
	*/
	ApprovalMember.readMembers = function(xml, isMember) {
		var targetArr = [];
	    var userObj;
	    if (!xml) {return;}
	    var memberArr = WCXML.GetFirstLevelChilds(xml,"member");
	    if (memberArr) {
	      for (var member=0; member< memberArr.length; member++) {
	        var userId = WCXML.GetElementText(memberArr[member],"userId");
	        if (userId) { // this member is a simple user
	          userObj = {};
	          userObj.userId = userId;
	          userObj.userName = WCXML.GetElementText(memberArr[member],"userName");
	          userObj.firstName = WCXML.GetElementText(memberArr[member],"firstName");
	          userObj.lastName = WCXML.GetElementText(memberArr[member],"lastName");
	          userObj.eMail = WCXML.GetElementText(memberArr[member],"eMail");
	          userObj.canBeApprovalMaster = WCXML.GetElementBoolean(memberArr[member],"canBeApprovalMaster");
	          userObj.isActive = WCXML.GetElementBoolean(memberArr[member],"is_active");
	          userObj.memberType = "User";
	          userObj.isMember = isMember;
	          targetArr.push(userObj);
	        } else {
	          var groupId = WCXML.GetElementText(memberArr[member],"groupId");
	          if (groupId) { // this member is a group
	            userObj = {};
	            userObj.groupId = groupId;
	            userObj.groupName = WCXML.GetElementText(memberArr[member],"groupName");
	            userObj.canBeApprovalMaster = WCXML.GetElementBoolean(memberArr[member],"canBeApprovalMaster"); 
	            userObj.memberType = "Group";
	            var isARole = WCXML.GetElementText(memberArr[member],"isRole");
	            userObj.isRole = (isARole == "1");
	            userObj.isMember = isMember;
	            // read group members (could have been done also with GetFirstFirstLevelChild)
	            var elements = WCXML.GetFirstLevelChilds(memberArr[member],"groupMembers");
	            if (elements && elements.length>0) {
	              userObj.members = this.readMembers(elements[0], isMember);  // call yourself...
	            }
	            targetArr.push(userObj);          
	          }
	        }  // end if userId
	      } // end for member
	    }
	    
	    return targetArr;
	};

	return ApprovalMember;
});