define(["approval/model/ApprovalStage",
		"approval/ApprovalConstants",
	"underscore",
	"utils/webi18n!approval",
	"utils/dateShift"
], function (ApprovalStage, ApprovalConstants, _, i18n, dateShift) {

	"use strict";

	/**
	 * Stage Factory that will create stages for complex approval
	 * @param {ApprovalModel}
	 */
	var ComplexApprovalStageFactory = function (model) {
		this._model = model;
	};

	/**
	 * Create an approval stage
	 * @param {ApprovalStage} [previousStage] - the stage after which this stage will be added.
	 */
	ComplexApprovalStageFactory.prototype.createApprovalStage = function (previousStage) {

		var stepObj = new ApprovalStage(this._model); // no default due date
		stepObj.condApp = true;

		if (this._model.dueDateType === ApprovalConstants.DueDateType.FIXED) {
			var baseDate = new Date();
			baseDate.setMinutes(0);
			baseDate.setSeconds(0);
			if (previousStage && previousStage.dueDate) {
				baseDate = previousStage.dueDate;
			}
			stepObj.dueDate = dateShift.addDaysHours(baseDate, 1, 0, null, true);

		}

		var existingStages = this._model.getStages();
		if (existingStages && existingStages.length > 0) {
			var currentStageNames = _.pluck(existingStages, "name");
		var numberOfPreviousStage = this._model.getStageNumber(previousStage);
		var stageName = i18n.t("CAppStage") + " " + (numberOfPreviousStage + 1);

		var stageNameAlreadyExists = function (step) {
			return step === stageName;
		};

		while (_.find(currentStageNames, stageNameAlreadyExists)) {
			stageName = stageName + "_2";
		}		

		stepObj.name = stageName;
		}
		return stepObj;

	};
	return ComplexApprovalStageFactory;
});
