define(["jquery",
		"approval/model/ApprovalStage",
		"utils/webi18n!Approval",
		"utils/Observer"
	],
	function (
		$,
		ApprovalStage,
		i18n,
		Observer) {
		"use strict";

		/**
		 * 
		 * @param {ApprovalModel} approvalModel
		 * @param {ApprovalStage} addAfterStage the stage after which the new stage should be added.
		 */
		var AddApprovalStageView = function (approvalModel, addAfterStage) {
			Observer.mixin(this);

			this._model = approvalModel;
			this._stage = addAfterStage;

			this._observe(this._model, "changed.stages.*", this._updateAlwaysVisible.bind(this));
			this._observe(this._stage, "deleted", function () {
				this._stopObserveAll();
				this._container.remove();
			}, this);


			this._alwaysVisible = null;
			this._element = $("<div>", {
				"class": "add-approval-stage"
			});
			this._icon = $("<span>", {

			}).text(" " + i18n.t("CAppInsertStep")).prepend($("<i>", {
				"class": "icon icon-WCR_Actions-Add"
			})).appendTo(this._element);
			this._container = $("<div>", {
				"class": "add-approval-stage-container"
			});

			this._element.appendTo(this._container);
			this._element.attr("data-sahi-id", "add_stage_"+(this._stage.number - 1));
			
			this._element.on("click", function (e) {
				if (e) {
					// The click method on mobile also initiates a mouseover/mouseenter. 
					// This will cause the 'add-approval-stage_expand' class to be set
					// We don't want this as the mouseleave event isn't fired.
					if (this._timeout) {
						window.clearInterval(this._timeout);
						this._timeout = null;
					}
					e.stopPropagation();
					e.preventDefault();
					this._container.removeClass("add-approval-stage_expanded");
				}
				this._addNewStage();
			}.bind(this));

			this._element.on("touchstart", function (e) {
				e.stopPropagation();
				e.preventDefault();
				this._container.removeClass("add-approval-stage_expanded");
				this._addNewStage();
			}.bind(this));
			this._container.on("touchstart", function (e) {
				e.preventDefault();
				this._container.addClass("add-approval-stage_expanded");
				window.setTimeout(function () {
					this._container.removeClass("add-approval-stage_expanded");
				}.bind(this), 1750);
			}.bind(this));
			this._container.on("mouseenter", function () {
				this._timeout = window.setTimeout(function () {
					this._timeout = null;
					this._container.addClass("add-approval-stage_expanded");
				}.bind(this), 100);
			}.bind(this));
			this._container.on("mouseleave", function () {
				this._collapsePlaceholder();
			}.bind(this));
			this._updateAlwaysVisible();

		};

		/**
		 * Get the container that holds this view
		 * @returns {jQuery}
		 */
		AddApprovalStageView.prototype.getElement = function () {
			return this._container;
		};

		/**
		 * Calling this will insert a new stage in the view
		 * @private
		 */
		AddApprovalStageView.prototype._addNewStage = function () {
			this._model.addStageAfter(this._stage);
		};

		/**
		 * Collapse the placeholder
		 * @private
		 */
		AddApprovalStageView.prototype._collapsePlaceholder = function () {
			if (this._timeout) {
				window.clearInterval(this._timeout);
				this._timeout = null;
			}
			this._container.removeClass("add-approval-stage_expanded");
		};

		/**
		 * Check and update the visibility of this component. This view should only be visible when the stage it is attached to is the last one in the list.
		 * @private
		 */
		AddApprovalStageView.prototype._updateAlwaysVisible = function () {
			var isAlwaysVisible = (this._model.getStageNumber(this._stage) === this._model.getStageCount());
			if (isAlwaysVisible === this._alwaysVisible) {
				// no update
				return;
			}
			this._alwaysVisible = isAlwaysVisible;
			if (isAlwaysVisible) {
				this._container.addClass("add-approval-stage_always-visible");
				this._icon.text(" " + i18n.t("CAppAddStep")).prepend($("<i>", {
					"class": "icon icon-WCR_Actions-Add"
				}));
			} else {
				this._container.removeClass("add-approval-stage_always-visible");
				this._icon.text(" " + i18n.t("CAppInsertStep")).prepend($("<i>", {
					"class": "icon icon-WCR_Actions-Add"
				}));
			}
		};

		return AddApprovalStageView;

	}
);
