define(
	[
		"jquery",
		"approval/view/ApprovalStageView",
		"approval/view/AddApprovalStageView",
		"approval/ApprovalConstants",
		"utils/Observer",
		"utils/AnimationFrameRenderer"

	],
	function (
		$,
		ApprovalStageView,
		AddApprovalStageView,
		ApprovalConstants,
		Observer,
		AnimationFrameRenderer
	) {

		"use strict";

		/**
		 * @class
		 * a container class that contains both the ApprovalStageView and the AddApprovalStageView for inserting new approvals
		 */
		var ApprovalStageContainer = function (approvalModel, approvalStage) {
			Observer.mixin(this);
			AnimationFrameRenderer.mixin(this);
			// This encompassing element
			this.element = $("<div>", {
				"class": "approval-stage-container"
			});
			this._approvalModel = approvalModel;
			this.stageModel = approvalStage;
			this._approvalStageView = new ApprovalStageView(approvalStage);

			this._observe(approvalStage, "deleted", function () {
				this.element.remove();
				this._stopObserveAll();
			}.bind(this));
		};


		/**
		 * Render the ApprovalStageContainer
		 */
		ApprovalStageContainer.prototype.render = function(){
			this._render(); // animationframerenderer call
		};

		/**
		 * Render the ApprovalStageContainer. Do not call this directly, call .render() instead
		 * @private
		 */
		ApprovalStageContainer.prototype._doRender = function () {
			if (!this._approvalModel.readOnly && this._approvalModel.isStagedApprovalAllowed() &&
				(this._approvalModel.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED ||
					this._approvalModel.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_DEFINED) && (this.stageModel.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED ||
					this.stageModel.approvalStatus === ApprovalConstants.ApprovalStatus.PENDING)) {
				if (!this._addApprovalStageView) {
					this._addApprovalStageView = new AddApprovalStageView(this._approvalModel, this.stageModel);
					if (this.stageModel.approvalStatus !== ApprovalConstants.ApprovalStatus.PENDING) {
						this.element.addClass("approval-stage-container-sortable");
					} else {
						this.element.removeClass("approval-stage-container-sortable");
					}
					this.element.addClass("approval-stage-container-insertable");
				}
			} else {
				this._addApprovalStageView = null;
				this.element.removeClass("approval-stage-container-sortable");
			}

			this._approvalStageView.elem.detach();
			if (this._addApprovalStageView) {
				this._addApprovalStageView.getElement().detach();
			}
			this.element.empty();
			this.element.append(this._approvalStageView.elem);

			if (this._addApprovalStageView) {
				this.element.append(this._addApprovalStageView.getElement());
			}
			this._approvalStageView.render();
			return this.element;
		};

		return ApprovalStageContainer;

	}
);
