define(["Template",
		"approval/ApprovalConstants",
		"tpl!approval/view/template/ApproveRejectDialog",
        "utils/webi18n!Approval",
		"wcr/hover",
		"wcr/webcenter_xml",
        "jquery",
		"underscore"
    ],
	function (
		Template,
		ApprovalConstants,
		ApproveRejectDialog,
        i18n,
		hover,
	 	WCXML,
        $,
        _) {
		"use strict";

        /**
         * This view shows the popup to perform an action (Approve, Reject, Comment,...) to an approval
         * 
         * @class
         * @param {ApprovalModel} approvalModel
         * @param {ApproveRejectViewModel} viewModel
         */
        function ApproveRejectView(approvalModel, viewModel) {
			this.approvalModel = approvalModel;
			this.viewModel = viewModel;
			
			//UI elements
			this._dropDownMenu = null;
			this._cancelBtn = null;
			this._submitBtn = null;
			this._conditionField = null;
			this._pswdField = null;
			this._checkbox = null;			
		}
	
		/**
		 * Opens the popover to approve/reject/comment
		 * @param {object}   eventObj       - the key event
		 * @param {string} approvalStatus - the chosen approval status
		 * @param {integer} step           - the step number
		 */
		ApproveRejectView.prototype.open = function(eventObj, approvalStatus, step) {
			// True when viewerApprovalCommentType is set to "user annotation" or "any annotation" for viewfiles
			// In this case the user should use annotations instead of comments.
			var blockComment = (this.viewModel.isViewFile && this.approvalModel.viewerApprovalCommentType !== "Prompt");
			
			
			var tmpl = new Template(i18n, ApproveRejectDialog);
			
			if(this.approvalModel.forcedStep !== undefined){
				step = this.approvalModel.forcedStep;
			}
			
			tmpl.d.approvalStatus = approvalStatus;
			tmpl.d.conditionApproval = this.approvalModel.steps[step].condApp;
			tmpl.d.passwordValidation = this.approvalModel.passwordValidation;
			tmpl.d.blockComment = blockComment;
			tmpl.d.isViewFile = (this.viewModel.isViewFile === true);
			tmpl.d.userString = this.approvalModel.getCurrentUserString();
			
			
			if(approvalStatus === ApprovalConstants.ApprovalStatus.APPROVED 
			   || approvalStatus === ApprovalConstants.ApprovalStatus.FORCE_APPROVED){
				tmpl.d.buttonColor = "btn-success";
				tmpl.d.buttonText = i18n.t('CAppCommitApproval');
			}
			else if(approvalStatus == ApprovalConstants.ApprovalStatus.REJECTED 
					|| approvalStatus == ApprovalConstants.ApprovalStatus.FORCE_REJECTED){
				tmpl.d.buttonColor = "btn-danger";
				tmpl.d.buttonText = i18n.t('CAppCommitRejection');
			}
			else {
				tmpl.d.buttonColor = "btn-info";
				tmpl.d.buttonText = i18n.t('CAppCommitComment');
			}
			
			dropdownmenu(eventObj.target, eventObj, [tmpl.getRenderedHtml()], 400, null, null, true, null, true);
			
			this._dropDownMenu = $("#dropmenudiv");
			
			this._conditionField = $("#ApproveRejectComments", this._dropDownMenu);
			this._pswdField = $("[name=passwordEntry]", this._dropDownMenu);
			this._checkbox = $("[name=approvalConditionChkBx]", this._dropDownMenu);
			this._submitBtn = $("[name=submitBtn]", this._dropDownMenu);
			this._cancelBtn = $("[name=cancelBtn]", this._dropDownMenu);
			
			this._cancelBtn.on("click", function () {
				hidemenu();
			}.bind(this));
			
			this._submitBtn.on("click", function () {
				this._approveReject(approvalStatus, step);
			}.bind(this));
			
			if(this._checkbox.length > 0){
				this._checkbox.on("change", _.bind(function () {
					if (this._conditionField && this._checkbox) {
						if (true === this._checkbox.is(':checked') ) {
							if (blockComment) {
								this._conditionField.val(i18n.t('CAppDocumentAnnotated'));
								this._conditionField.hide();
								$("#UseAnnotationsMsg").show();
							} else {
								this._conditionField.prop('disabled', false);
								this._conditionField.show();
							}
						} else {
							this._conditionField.prop('disabled', true);
							this._conditionField.val("");
							this._conditionField.hide();
							$("#UseAnnotationsMsg").hide();
						}
						
						if(this.viewModel.adjustPopupHeight){
							this._adjustApproveRejectHeight();
							if(this.viewModel.toggleAction){
								this.viewModel.toggleAction();
							}
						}
					}
				},this));
			}
			
			if (this._pswdField.length > 0){
				this._pswdField.on("keydown", _.bind(function (e) {
					this._submitOnEnter(e, _.bind(function() {
						this._submitBtn.click();
					}, this));
				},this));
			}
			
			if(this.viewModel.adjustPopupHeight){
				if(this.viewModel.toggleAction){
					this.viewModel.toggleAction();
				}
			}
			
			return this._dropDownMenu;
		};
	
		/**
		 * Approve, reject or comment by current user
		 * @private
		 * @param {string}    approvalStatus         - the chosen approval status
		 * @param {integer}   step                   - the stepnumber
		 * @param {JQueryObj} popUpConditionField    - the textarea with the comments
		 * @param {JQueryObj} approvalConditionChkBx - the 'approve with condition' checkbox
		 * @param {JQueryObj} pwEntryField           - the inputfield for the password
		 */
		ApproveRejectView.prototype._approveReject = function (approvalStatus, step,  submitBtn, cancelBtn, drop) {
			// Execute custom functionality before submitting the approval (if it's defined) 
			if (this.viewModel.beforesubmit) {
				try {
					this.viewModel.beforesubmit();
				} catch (err) {
					return;
				}
			}
			var data = {
				comment: "",
				action: "",
				password: ""
			};

			// handle comment/condition
			if (this._conditionField.length > 0 && (this._checkbox.length === 0 || this._checkbox.is(":checked"))) {
				if (this._conditionField.val() === "" || this._conditionField.val() === null) {
					// empty comment. only allowed for rejection
					if (approvalStatus === ApprovalConstants.ApprovalStatus.COMMENT) {
						window.alert(i18n.t('CAppYouCannotGiveEmptyComment'));
						return;
					} else if (approvalStatus === ApprovalConstants.ApprovalStatus.APPROVED 
							   || approvalStatus === ApprovalConstants.ApprovalStatus.FORCE_APPROVED) {
						window.alert(i18n.t('CAppYouCannotGiveEmptyCondition'));
						return;
					}
				}

				data.comment = this._conditionField.val();
				if (data.comment.length > 512) {
					window.alert(i18n.t('CApp_SUBMIT_ERR__COMMENTFIELD_TOO_LONG'));
					return;
				}
			}

			// convert the approvalStatus to the name of the action
			// the name is not identical because of the different camel case standards used (actions are lower camel case 
			// while statuses are upper camel case.
			var actionName = {
				'Approved': 'approved',
				'Rejected': 'rejected',
				'Comment': 'comment',
				'ForceApproved': 'forceApproved',
				'ForceRejected': 'forceRejected'
			};

			var aName = approvalStatus;
			if (actionName[approvalStatus]) {
				aName = actionName[approvalStatus];
			}

			// check if comments were given
			if (aName === 'approved') {
				var stepObj = this.approvalModel.steps[step];
				for (var approver = 0; approver < stepObj.approvers.length; approver++) {
					if (this.approvalModel.currentUserApprovalStatus(step, approver) === 'MustApprove') {
						if (stepObj.approvers[approver].commentArr.length > 0) {

							var confmsg = ''; 
							if (data.comment.length > 0) { // user has entered a comment on this approval also
								confmsg = i18n.t('CAppCommentsWillBeOmitted') + "\n";
							} else {
								confmsg = i18n.t('CAppCommentsWillBeOmitted_NoCurrentComment') + "\n";
							}
							for (var comment = 0; comment < stepObj.approvers[approver].commentArr.length; comment++) {
								confmsg += "\n" + stepObj.approvers[approver].commentArr[comment].commentText;
							}

							var conf = window.confirm(confmsg);
							if (!conf) {
								return;
							}
						}
					}
				}
			}
		
			data.action = aName;

			// password entry
			if (this.approvalModel.passwordValidation === true) {
				if (this._pswdField.length > 0) { //check if the element exists
					data.password = WCXML.encode64(this._pswdField.val());
				} else {
					throw ("pwField or pwEntryField does not exist");
				}
			}

			// If page range is defined (page lists), add it to the call parameters
			if (this.viewModel.pageRange) {
				data.pageindex = this.viewModel.pageRange;
			}

			if (!this.approvalModel.submitDisabled) {
				// disable approval buttons	
				this.approvalModel.submitDisabled = true;
				if (this.approvalModel.submitAllowed) {
					this.approvalModel.confirmNeeded = false;

					if (this.viewModel.isEncapsulated) {
						// the complex approval is encapsulated, so submit with AJAX call (We don't want to trigger a page reload)
						data.encapsulated = 1;
						
						this._submitBtn.attr('disabled', 'disabled');
						this._cancelBtn.attr('disabled', 'disabled');
					
						$.ajax(this.viewModel.actionURL, {
							'data': data,
							type: "post",
							success: _.bind(function (response) {
								//enable buttons again	
								this.approvalModel.submitDisabled = false;

								// !DIRTY HACK! check if error occured in the docdetailsapprovers html page that comes back
								// Reason is that we don't want to duplicate all error and translation handling (strings not sitting in CApp etc)
								var isDocDetailsApprovers = response.match(/<title>Document Details - Approval Rule<\/title>/gim);
								var error = null;
								if (!(isDocDetailsApprovers && isDocDetailsApprovers.length > 0)) {
									// if we are not redirected to the docDetailsApprovers page, we are probably not logged in
									error = i18n.t('CAppSessionExpired');
								} else {
									// Another error occured
									var errorOccured = response.match(/errorOccured ?= ?true ?;/g);
									if (errorOccured && errorOccured.length > 0) {
										//an error occured, get the error text in the page
										error = $(response).find("#errorFeedBack").text().trim();
									}
								}

								// close the dialog
								hidemenu();

								if (error) {
									window.alert(error);
									if (this.viewModel.approveRejectCallback) {
										this.viewModel.approveRejectCallback(false, approvalStatus, error);
									}

									if(this.viewModel.onUpdateCallback){
										this.viewModel.onUpdateCallback(this.approvalModel);
									}
								} else {
									//approval was successful

									// reload info
									this.approvalModel.readFromXML(this.approvalModel.readURL);

									if (this.viewModel.approveRejectCallback) {
										this.approvalModel.once("initialized", _.bind(this.viewModel.approveRejectCallback, this, true, approvalStatus));
									}
								}
							},this),
							error: _.bind(function (response) {
								// this should actually never happen, the ajax request has failed
								this.approvalModel.submitDisabled = false;
								window.alert("Error while submitting approval, please try again"); // no translation, this should normally never happen
							},this)
						});
					} else {
						// we are not encapsulated, and can just submit the form (triggering a page reload)
						this.viewModel.submit(data);
					}
				} else {
					// close the dialog
					hidemenu();
					this.approvalModel.submitDisabled = false;
				}
			} else {
				window.alert(i18n.t('CAppApprovalActionOngoing'));
			}
		};
	
		/**
		 * Executes the callback when enter is pressed
		 * @private
		 * @param {object}   event    
		 * @param {function} callback
		 */
		ApproveRejectView.prototype._submitOnEnter = function(event, callback){
			var key = event.keyCode || event.which;
			if (key === 13) {//ENTER
				callback();
			}
		};
	
		/**
		 * Changes the position of the popup based on the height of the comment input field
		 * @private
		 */
		ApproveRejectView.prototype._adjustApproveRejectHeight = function(){
			var dropmenuTop = this._dropDownMenu.css("top");
			if (true === this._checkbox.is(':checked')) {
			  this._dropDownMenu.css("top",(dropmenuTop.substr(0, dropmenuTop.length-2) - $(this._conditionField).outerHeight(true) -4)  + "px");
			} else {
			  this._dropDownMenu.css("top",(parseInt(dropmenuTop.substr(0, dropmenuTop.length-2)) + $(this._conditionField).outerHeight(true) + 4)  + "px");
			}
		};	
	
		return ApproveRejectView;
		
});