define([
		"approval/ApprovalUtils",
		"tpl!approval/view/template/ApproversTableCommentHover",
		"utils/webi18n!Approval",
		"Template",
		"underscore",
		"wcr/webcenter_html"
	],
	function (
		ApprovalUtils,
		ApproversTableCommentHoverTempl,
		i18n,
		Template,
		_, WCHTML) {
		"use strict";


		/**
		 * This view is responsible for showing the popup when hovering over an approval comment
		 * @param {ApprovalStage} model
		 */
		function CommentHoverView(model) {
			this.model = model;
		}


		/**
		 * Displays the popup
		 * @param {integer} stepIndex
		 * @param {integer} approverIndex
		 * @param {integer} event
		 */
		CommentHoverView.prototype.open = function (approverIndex, event) {
			if (_.isEmpty(this.model.approvers)) {
				return;
			}
			var approverObj = this.model.approvers[approverIndex];
			if (!approverObj || _.isEmpty(approverObj.commentArr)) {
				return;
			}
			var tmpl = new Template(i18n, ApproversTableCommentHoverTempl);
			var table = $(tmpl.getRenderedHtml());

			_.each(approverObj.commentArr, function (commentObj) {
				var userString = "";
				var memberObj = ApprovalUtils.findMemberObj("User", commentObj.commentByUserId, this.model.approvalModel.projectMembers);
				if (memberObj) {
					userString = WCHTML.getUserNameStringActiveText(memberObj.firstName, memberObj.lastName, memberObj.userName, memberObj.isActive, i18n.t("CAppUserDeactivated"), false);
				}
				var timeString = "";
				if (commentObj.commentTime) {
					timeString = ApprovalUtils.getDateHourString(commentObj.commentTime);
				}
				table.append(
					$("<tr>")
					.append($('<td valign="top">').text(userString))
					.append($('<td valign="top" style="text-align:center">').text(timeString))
					.append($('<td valign="top" style="text-align:center">').text(commentObj.commentVersion))
					.append($('<td valign="top">').text(commentObj.commentText))
				);
			}.bind(this));

			var html = table.wrap('<div/>').parent().html();
			dropdownmenu(event.target, event, [html], 600, null, null, true, "#ffffcc", false);
		};

		return CommentHoverView;
	});