/* https://github.com/jrburke/r.js/blob/master/README.md */
/* http://requirejs.org/docs/optimization.html */
({
	appDir: "./",
	baseUrl: "./",
	mainConfigFile: "./config.js",
	dir: "../appworkdir-postbuild/",

	/**
	 * PATH Configuration
	 **/
	paths: {
		"jquery": "empty:",
		"jqueryUI": "empty:",
		"jqueryUITouchPunch": "empty:",
		"bootstrap": "empty:",
		"modernizr": "empty:",
		"Template": "empty:",
		"colorpicker": "empty:",
		"HTMLTruncator": "empty:",
		"tinyMCE": "empty:",
		"Mousetrap": "empty:",
		"WV3Dlib/three": "empty:",
		"codemirror": "empty:",
		stickykit: "empty:",
		"wcr/AddDocument": "empty:",
		"wcr/AttributeCategory": "empty:",
		"wcr/dateShift": "empty:",
		"wcr/datetools": "empty:",
		"wcr/docSearchOperations": "empty:",
		"wcr/documentSourceHandler": "empty:",
		"wcr/DocumentsTable": "empty:",
		"wcr/dynamic_homepage_utils": "empty:",
		"wcr/element": "empty:",
		"wcr/ManageRoles": "empty:",
		"wcr/modalpopup": "empty:",
		"wcr/mootools": "empty:",
		"wcr/MyTodo": "empty:",
		"wcr/ObjectArrayCloneUtil": "empty:",
		"wcr/projdetailsbom": "empty:",
		"wcr/ProjectCreation": "empty:",
		"wcr/sdrop": "empty:",
		"wcr/sdrop_common": "empty:",
		"wcr/search": "empty:",
		"wcr/validate": "empty:",
		"wcr/vector": "empty:",
		"wcr/webcenter_ajax": "empty:",
		"wcr/webcenter_html": "empty:",
		"wcr/webcenter_xml": "empty:",
		"wcr/popup": "empty:",
		"wcr/extpopup": "empty:",
		"wcrBase/custom/javascript/CustomFormExtension": "empty:",


		"debugger": "web-viewer/core/AbstractDebug"
	},
	/* for legal reasons, we cannot optimize tinymce
	 javascript_highlight_rules crashes build when not excluded */
	fileExclusionRegExp: /tinymce|javascript_highlight_rules/,



	/**
	 * MODULE Configuration
	 **/
	modules: [
		/**
		 * All Libraries
		 */
		{
			name: "lib/all",
		},
		/**
		 * All Utilities
		 */
		{
			name: "utils/all"
		},
		{
			name: "component/all",
			exclude: ["lib/all", "utils/all"]
		},
		{
			name: "exclude_lib_and_utils"
		},
		/**
		 * All build excludes
		 */
		{
			name: "excludes"
		},

		/**
		 * In the build, load two files of config for require
		 * The second file contains information about the optimization
		 *  (which files can be found in which bundle)
		 */
		{
			name: "config",
			include: ["config_build_bundles"]
		},


		/**
		 * 3D Web Viewer
		 */
		{
			name: "3d-web-viewer/WebViewer3DAPIEXT",
			exclude: ["excludes"]
		},




		/**
		 * Attribute Category
		 */
		{
			name: "attributecategory/AttributeCategory",
			exclude: ["excludes",
				"doctablex/DocTableX",
				"doctablex/DocTableXRenderConfig",
				"doctablex/TextContentTableXView",
				"doctablex/TextContentTableX",
			    "doctablex/TextContentRowView",
				"doctablex/TextContentRow"
			]
		},

		/**
		 * Ace
		 * 
		 * Only the necessary files for the JavaScript mode are included in ace/mode, adding new modes may result
		 * in an infinite loop while building
		 */
		{
			name: "ace/ace"
		},


/* COMPONENTS */

		/**
		 * component/3DViewerLauncher
		 */
		{
			name: "component/3DViewerLauncher/3DViewerLauncher",
			exclude: ["exclude_lib_and_utils",
				"3d-web-viewer/WebViewer3DAPIEXT",
				"WV3Dmod/Constants"
			]
		},

		/**
		 * component/actionbuttons/AbstractButton
		 */
		{
			name: "component/actionbuttons/AbstractButton",
			exclude: ["exclude_lib_and_utils",
					 "doctablex/DocTableXUtils",
					  "doctablex/AJAXDataCache"
			]
		},

		/**
		 * component/actionbuttons/DownloadButton
		 */
		{
			name: "component/actionbuttons/DownloadButton",
			exclude: ["exclude_lib_and_utils",
					 "component/actionbuttons/AbstractButton",
					 "doctablex/DocTableXUtils",
					 "doctablex/AJAXDataCache"
			]
		},

		/**
		 * component/actionbuttons/Open3DButton
		 */
		{
			name: "component/actionbuttons/Open3DButton",
			exclude: ["exclude_lib_and_utils",
					 "component/actionbuttons/AbstractButton",
					 "doctablex/DocTableXUtils",
					 "doctablex/AJAXDataCache"
			]
		},

		/**
		 * component/actionbuttons/OpenACADButton
		 */
		{
			name: "component/actionbuttons/OpenACADButton",
			exclude: ["exclude_lib_and_utils",
					 "component/actionbuttons/AbstractButton",
					 "doctablex/DocTableXUtils",
					 "doctablex/AJAXDataCache"
			]
		},

		/**
		 * component/actionbuttons/UploadNewVersionButton
		 */
		{
			name: "component/actionbuttons/UploadNewVersionButton",
			exclude: ["exclude_lib_and_utils",
					 "component/actionbuttons/AbstractButton",
					 "component/styledfileinput/StyledFileInput",
					 "doctablex/DocTableXUtils",
					 "doctablex/AJAXDataCache"
			]
		},

		/**
		 * component/actionbuttons/UploadNewVersionButton
		 */
		{
			name: "component/actionbuttons/ViewButton",
			exclude: ["exclude_lib_and_utils",
					 "component/actionbuttons/AbstractButton",
					 "doctablex/DocTableXUtils",
					 "doctablex/AJAXDataCache"
			]
		},


		/**
		 * component/actionmenu/ActionMenuBase
		 */
		{
			name: "component/actionmenu/ActionMenuBase",
			exclude: ["exclude_lib_and_utils"]
		},
		/**
		 * component/actionmenu/ActionMenu
		 */
		{
			name: "component/actionmenu/ActionMenu",
			exclude: ["exclude_lib_and_utils",
					 "component/actionmenu/ActionMenuBase"]
		},
		/**
		 * component/actionmenu/HeaderActionMenu
		 */
		{
			name: "component/actionmenu/HeaderActionMenu",
			exclude: ["exclude_lib_and_utils",
					 "component/actionmenu/ActionMenu"]
		},

		/**
		 * component/actionmenu/RowActionMenu
		 */
		{
			name: "component/actionmenu/RowActionMenu",
			exclude: ["exclude_lib_and_utils",
					 "component/actionmenu/ActionMenuBase"]
		},
		/**
		 * component/actionmenu/VerticalActionMenu
		 */
		{
			name: "component/actionmenu/VerticalActionMenu",
			exclude: ["exclude_lib_and_utils",
					 "component/actionmenu/ActionMenu"]
		},


		/**
		 * component/actionmenu/ImportPopup
		 */
		{
			name: "component/actionpopup/ImportPopup",
			exclude: ["exclude_lib_and_utils",
					 "component/modal/Modal",
					 "component/button/Button"]
		},

		/**
		 * component/badge/IconBadge
		 */
		{
			name: "component/badge/IconBadge",
			exclude: ["exclude_lib_and_utils"]
		},

		/**
		 * component/button/Button
		 */
		{
			name: "component/button/Button",
			exclude: [
				"exclude_lib_and_utils",
				"component/tooltip/Tooltip"
			]
		},

		/**
		 * component/cards/CardGridControl
		 */
		{
			name: "component/cards/CardGridControl",
			exclude: [
				"exclude_lib_and_utils"
			]
		},

		/**
		 * component/contentoverflow/OverflowContentContainer
		 */
		{
			name: "component/contentoverflow/OverflowContentContainer",
			exclude: [
				"exclude_lib_and_utils"
			]
		},


		/**
		 * component/documentselector/DocumentSelector
		 */
		{
			name: "component/documentselector/DocumentSelector",
			exclude: ["exclude_lib_and_utils",
					  "component/sdrop/SDrop"]
		},

		/**
		 * component/dropdownmenu
		 */
		{
			name: "component/dropdownmenu/DropdownMenu",
			exclude: [
				"exclude_lib_and_utils",
				"component/popover/Popover",
				"component/button/Button"
			]
		},


		/**
		 * component/errorhandler
		 */
		{
			name: "component/errorhandler/ErrorHandler",
			exclude: [
				"exclude_lib_and_utils"
			]
		},

		/**
		 * component/fileinput
		 */
		{
			name: "component/fileinput/MultiFileInput",
			exclude: [
				"exclude_lib_and_utils"
			]
		},

		/**
		 * component/filterList
		 */
		{
			name: "component/filterList/FilterList",
			exclude: [
				"exclude_lib_and_utils"
			]
		},


		/**
		 * component/helpIcon
		 */
		{
			name: "component/helpIcon/HelpIcon",
			exclude: [
				"exclude_lib_and_utils",
				"component/tooltip/Tooltip"
			]
		},


		/**
		 * component/inputDropdown
		 */
		{
			name: "component/inputDropdown/InputDropdown",
			exclude: [
				"exclude_lib_and_utils"
			]
		},

		/**
		 * component/javascripteditor
		 * BUNDLED TOGETHER WITH WFEDITOR
		 */

		/**
		 * component/keywordsearch
		 */
		{
			name: "component/keywordsearch/keywordsearch",
			exclude: [
				"exclude_lib_and_utils",
				"component/sdrop/SDrop"
			]
		},


		/**
		 * component/loadingindicator
		 */
		{
			name: "component/loadingindicator/LoadingIndicator",
			exclude: [
				"exclude_lib_and_utils"
			]
		},

		/**
		 * component/message
		 */
		{
			name: "component/message/MessageComponent",
			exclude: [
				"exclude_lib_and_utils"
			]
		},

		/**
		 * component/modal
		 */
		{
			name: "component/modal/Modal",
			exclude: [
				"exclude_lib_and_utils"
			]
		},


		/**
		 * component/multiSelect/MultiSelectList
		 */
		{
			name: "component/multiSelect/view/MultiSelectList",
			exclude: [
				"exclude_lib_and_utils",
				"component/filterList/FilterList",
				"component/tooltip/Tooltip"
			]
		},

		/**
		 * component/multiSelect/MultipleMultiSelectList
		 */
		{
			name: "component/multiSelect/view/MultipleMultiSelectList",
			exclude: [
				"exclude_lib_and_utils",
				"component/multiSelect/view/MultiSelectList"
			]
		},

		/**
		 * component/overlaymessage
		 */
		{
			name: "component/overlaymessage/OverlayMessage",
			exclude: [
				"exclude_lib_and_utils"
			]
		},

		/**
		 * component/pagemessagecomponent
		 */
		{
			name: "component/pagemessagecomponent/PageMessageComponent",
			exclude: [
				"exclude_lib_and_utils",
				"component/message/MessageComponent"
			]
		},


		/**
		 * component/pagemessagecomponent
		 */
		{
			name: "component/passwordpopup/PasswordValidationPopup",
			exclude: [
				"exclude_lib_and_utils",
				 "component/modal/Modal",
				 "services/passwordValidationService",
			     "web-viewer/layout/component/ModalPopup",
				 "web-viewer/layout/component/ViewerToolbarButton"
			]
		},


		/**
		 * component/popover
		 */
		{
			name: "component/popover/Popover",
			exclude: [
				"exclude_lib_and_utils"
			]
		},


		/**
		 * component/progressbar
		 */
		{
			name: "component/progressbar/ProgressBar",
			exclude: [
				"exclude_lib_and_utils"
			]
		},


		/**
		 * component/savedsearchdialog
		 */
		{
			name: "component/savedsearchdialog/ProjectSavedSearchDialog",
			exclude: [
				"exclude_lib_and_utils",
				"component/smartnames/savedsearchcontext/SavedSearchContextInfo",
				"component/modal/Modal",
				"component/savedsearchdialog/SavedSearchDialog",
				"component/pagemessagecomponent/PageMessageComponent"
			]
		},

		/**
		 * component/scrollbar
		 * BUNDLED TOGETHER WITH WFEDITOR
		 */

		/**
		 * component/sdrop
		 */
		{
			name: "component/sdrop/SDrop",
			exclude: [
				"exclude_lib_and_utils"
			]
		},

		/**
		 * Search Results JavaScript
		 */
		{
			name: "component/searchresults/internal/SearchResultsControl",
			exclude: [
				"exclude_lib_and_utils",
				"component/smartnames/input/smartnameinput",
				"component/smartnames/smartfield/smartfield",
				"component/smartnames/smartfield/smartnamedropdown",
				"component/documentselector/DocumentSelector",
				"component/actionmenu/ActionMenu",
    			"component/actionmenu/HeaderActionMenu",
				"component/loadingindicator/LoadingIndicator",
				"component/pagemessagecomponent/PageMessageComponent",
				"component/contentoverflow/OverflowContentContainer",
				"component/overlaymessage/OverlayMessage",
				"component/message/MessageComponent",
				"component/table/DropdownOverflowUtil",
				"component/table/Table",
				"dashboard/Dashboard"
			]
		},

		/**
		 * Search Setup JavaScript
		 */
		{
			name: "component/searchresults/internal/SearchSetupControl",
			include: [
			          "component/searchresults/implementations/searchsetuplayout/SearchSetupLayout_defaultxsl",
			          "component/searchresults/implementations/searchsetuplayout/SearchSetupLayout_attributes",
			          "component/searchresults/implementations/searchsetuplayout/SearchSetupLayout_displaypreferences"
			         ],
			exclude: [
				"exclude_lib_and_utils",
				"component/smartnames/input/smartnameinput",
				"component/smartnames/smartfield/smartfield",
				"component/smartnames/smartfield/smartnamedropdown",
				"component/documentselector/DocumentSelector",
				"component/actionmenu/ActionMenu",
    			"component/actionmenu/HeaderActionMenu",
				"component/loadingindicator/LoadingIndicator",
				"component/pagemessagecomponent/PageMessageComponent",
				"component/contentoverflow/OverflowContentContainer",
				"component/overlaymessage/OverlayMessage",
				"component/message/MessageComponent",
				"component/table/DropdownOverflowUtil",
				"component/table/Table",
				"dashboard/Dashboard"
			]
		},


		/**
		 * component/smartnames/input/smartnameinput
		 **/
		{
			name: "component/smartnames/input/smartnameinput",
			exclude: ["exclude_lib_and_utils"]
		},

		/**
		 * omponent/smartnames/smartfield/smartfield
		 **/
		{
			name: "component/smartnames/smartfield/smartfield",
			exclude: ["component/smartnames/input/smartnameinput",
				"exclude_lib_and_utils"
			]
		},

		/**
		 * component/styledfileinput
		 */
		{
			name: "component/styledfileinput/StyledFileInput",
			exclude: [
				"exclude_lib_and_utils"
			]
		},

		/**
		 * component/table
		 */
		{
			name: "component/table/Table",
			exclude: [
				"exclude_lib_and_utils"
			]
		},


		/**
		 * component/thirdlevelmenu
		 */
		{
			name: "component/thirdlevelmenu/ThirdLevelMenu",
			exclude: [
				"exclude_lib_and_utils"
			]
		},



		/**
		 * component/toolbar/ButtonGroup
		 */
		{
			name: "component/toolbar/ButtonGroup",
			exclude: [
				"exclude_lib_and_utils"
			]
		},


		/**
		 * component/toolbar/ToolbarButton
		 */
		{
			name: "component/toolbar/ToolbarButton",
			exclude: [
				"exclude_lib_and_utils",
				"component/button/Button",
				"component/popover/Popover"
			]
		},

		/**
		 * component/thirdlevelmenu
		 */
		{
			name: "component/toolbar/ToolbarComponent",
			exclude: [
				"exclude_lib_and_utils",
				"component/toolbar/ButtonGroup"
			]
		},


		/**
		 * component/tooltip
		 */
		{
			name: "component/tooltip/Tooltip",
			exclude: [
				"exclude_lib_and_utils",
				"component/popover/Popover"
			]
		},


		/**
		 * component/tree
		 */
		{
			name: "component/tree/treeitem",
			exclude: [
				"exclude_lib_and_utils"
			]
		},



		/**
		 * component/usercreation
		 */
		{
			name: "component/usercreation/CreateNewUserPopup",
			exclude: [
				"exclude_lib_and_utils",
				"component/modal/Modal",
				"component/overlaymessage/OverlayMessage",
				"component/sdrop/SDrop"
			]
		},
		{
			name: "component/usercreation/InviteMemberPopup",
			exclude: [
				"exclude_lib_and_utils",
				"component/modal/Modal",
				"component/overlaymessage/OverlayMessage"
			]
		},


		/**
		 * XDrop
		 */
		{
			name: "component/xdrop/XDrop",
			exclude: [
				"exclude_lib_and_utils",
				"component/modal/Modal"
			],
			include: [
				"component/xdrop/dataproviders/XDropDataProvider",
				"component/xdrop/dataproviders/CompositeDataProvider",
				"component/xdrop/dataproviders/CategoryDataProvider",
				"component/xdrop/dataproviders/ResultEntry",
				"component/xdrop/dataproviders/ResultSet",
				"component/xdrop/panel/XDropPanel",
				"component/xdrop/renderers/ButtonRenderer",
				"component/xdrop/renderers/LabelRenderer",
				"component/xdrop/renderers/CategoryTitleRenderer"
			]
		},



		/**
		 * DocTableX
		 */
		{
			name: "doctablex/DocTableX",
			exclude: ["excludes",
				"attributecategory/AttributeCategory"
			]
		},


		/**
		 * PCM
		 */
		{
			name: "packaging-content-management/text-content-services/ContentService",
			exclude: ["excludes"]
		},
		{
			name: "packaging-content-management/text-content-element-widget/TextContentElementWidget",
			exclude: ["excludes",
			          "packaging-content-management/text-content-services/languageService",
			          "packaging-content-management/text-content-services/ContentService",
			          "packaging-content-management/text-content-services/elementTypeService",
			          "packaging-content-management/text-content-services/tableElementTypeService"]
		},
		{
			name: "packaging-content-management/ContentBasicPage",
			exclude: ["excludes",
			          "packaging-content-management/text-content-element-widget/TextContentElementWidget",
			          "packaging-content-management/text-content-services/languageService",
			          "packaging-content-management/text-content-services/ContentService",
			          "packaging-content-management/text-content-services/elementTypeService",
			          "packaging-content-management/text-content-services/tableElementTypeService"]
		},

		/**
		 * Pages
		 */
		{
			name: "pages/tunnelmonitor/tunnelmonitor",
			exclude: ["component/tree/treeitem",
				"excludes"
			]
		},
		{
			name: "pages/workflow_connection/aesetupjmswizard",
			exclude: ["component/tree/treeitem",
				"excludes"
			]
		},
		{
			name: "pages/wctaskexecution/WcTaskExecutionPage",
			exclude: ["attributecategory/AttributeCategory",
				"excludes"
			]
		},
		{
			name: "pages/cartbadge/CartBadge",
			exclude: ["excludes"]
		},
		{
			name: "pages/projectcreation/CopyProjectOptionsTable",
			exclude: ["excludes"]
		},
		{
			name: "pages/rules/all",
			exclude: ["excludes"]
		},

		/**
		 * HTML5 Viewer
		 */

		{
			name: "web-viewer/App",
			include: ["web-viewer/annotation/AnnotationConstants",
				"web-viewer/approval/ApprovalConstants",
				"web-viewer/cad/CadConstants",
				"web-viewer/compare/CompareConstants",
				"web-viewer/core/CoreConstants",
				"web-viewer/data/DataConstants",
				"web-viewer/data/DataModule",
				"web-viewer/graphics/GraphicsConstants",
				"web-viewer/layout/LayoutModule",
				"web-viewer/measure/MeasureConstants",
				"web-viewer/navigation/NavigationModule",
				"web-viewer/pagelist/PagelistConstants",
				"web-viewer/print/PrintConstants",
				"web-viewer/utils/LineWorkWrapper",
				"web-viewer/webcenter/WebCenterModule"
			],
			exclude: ["excludes"]

		},
		{
			name: "web-viewer/attributeCategory/AttributeCategoryModule",
			exclude: ["web-viewer/App",
				"attributecategory/AttributeCategory",
				"doctablex/DocTableX",
				"excludes"
			]
		},
		{
			name: "web-viewer/annotation/AnnotationModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/approval/ApprovalModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/cad/CadModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/documentInfo/DocumentInfoModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/graphics/GraphicsModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/help/HelpModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/measure/MeasureModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/compare/CompareModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/print/PrintModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/pagelist/PagelistModule",
			exclude: ["web-viewer/App",
				"web-viewer/graphics/GraphicsModule",
				"excludes"
			]
		},
		{
			name: "web-viewer/textContent/TextContentModule",
			exclude: ["web-viewer/App",
                "attributecategory/AttributeCategory",
                "packaging-content-management/ContentConstants",
                "packaging-content-management/content/SingleTextContent",
                "packaging-content-management/content/TextContentElement",
                "excludes"
            ]
		},
		{
			name: "web-viewer/viewControls/ViewControlsModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/view3D/View3DModule",
			exclude: ["web-viewer/App",
				"excludes",
				"3d-web-viewer/WebViewer3DAPIEXT",
				"component/3DViewerLauncher/3DViewerLauncher"
			]
		},
		{
			name: "web-viewer/svg/SvgModule",
			exclude: ["web-viewer/App",
				"excludes"
			]
		},
		{
			name: "web-viewer/layout/desktop/DesktopLayout",
			exclude: ["web-viewer/App",
				"excludes"]
		},
		{
			name: "web-viewer/layout/flex/FlexLayout",
			exclude: ["web-viewer/App",
				"excludes"]
		},
		{
			name: "web-viewer/layout/mobile/MobileLayout",
			exclude: ["web-viewer/App",
				"excludes"]
		},

		/**
		 * WFEditor Framework
		 */
		{
			name: "WFEditor/core/WFEditor",
			exclude: ["excludes"],
			include: ["WFEditor/scene/AbstractViewFactory",
				"WFEditor/workflow/AbstractWorkflowFactory",
				"WFEditor/workflow/Workflow",
				"WFEditor/workflow/WorkflowItem",
				"WFEditor/workflow/node/Node",
				"WFEditor/scene/AbstractView",
				"WFEditor/workflow/node/NodeView",
				"WFEditor/utils/IconDataStorage"
			],
		},
		
		/**
		 * Approval
		 */
		{
			name:"approval/Approval",
			exclude : ["excludes"]
		},

		/**
		 * WFEditor WebCenter
		 */
		{
			name: "WFEditorWCR/core/WFEditorWCR",
			exclude: ["excludes",
				"WFEditor/core/WFEditor"
			]
		},

		/**
		 * C3 With Styling
		 */
		{
			name: "c3WithStyling",
			exclude: ["lib/less-rhino-1.7.5"]
		},

		/**
		 * dashboard
		 */
		{
			name: "dashboard/Dashboard",
			exclude: ["exclude_lib_and_utils"],
			include: [
				"dashboard/core/BlockDefinition",
				"dashboard/core/BlockDefinitionDataSource",
				"dashboard/core/Factory",
				"dashboard/core/parameters/ParameterSet",
				"dashboard/core/IDMap",
				"dashboard/blocks/ErrorBlock",
				"dashboard/core/Context",
				"dashboard/core/parameters/Parameter",
				"dashboard/core/parameters/LinkedParameter",
				"dashboard/core/parameters/StringParameter",
				"dashboard/core/parameters/BooleanParameter",
				"dashboard/core/Block",
				"dashboard/layout/BlockWrapper",
				"dashboard/core/Layout",
				"dashboard/Dashboard",
				"dashboard/core/parameters/CssSizeParameter",
				"dashboard/layout/ListLayout",
				"dashboard/layout/flex/FlexItem",
				"dashboard/layout/flex/FlexLayoutBlockWrapper",
				"dashboard/layout/flex/FlexColumnParameter",
				"dashboard/layout/flex/FlexLayout",
				"dashboard/i18n/LanguageBundle",
				"dashboard/i18n/LanguageProvider",
				"component/dashboard/i18n/WebCenterLanguageBundle",
				"component/dashboard/i18n/WebCenterLanguageProvider"
			]
		},

		/**
		 * All Basic utils => bundled together
		 */
		{
			name: "wcrbasicutils",
			create: true,
			/* Note: we also need this list in config_build_bundles.js */
			include: [
					"utils/AttributeEventBus",
					"utils/ColorUtils",
					"utils/Command",
					"utils/ConvertUtils",
					"utils/ExecutionQueue",
					"utils/extendedstring",
					"utils/FilterUtils",
					"utils/HTTPUtils",
					"utils/I18n",
					"utils/idUtils",
					"utils/init",
					"utils/Map",
					"utils/Observable",
					"utils/Observer",
					"utils/OrderedObject",
					"utils/Popup",
					"utils/Promisify",
					"utils/ResponsiveFit",
					"utils/ShapeUtils",
					"utils/SynchronizedUndoRedoController",
					"utils/UndoRedoController",
					"utils/WCEvent",
					"utils/wcrerror",
					"utils/WCRI18n",
					"utils/webcss-text",
					"utils/webcss",
					"utils/webi18n",
					"utils/XMLUtils"
			         ],
			exclude: ["lib/all"]
		}

		],


	/**
	 * OPTIMIZATION CONFIGURATION
	 */
	optimize: "closure",
	closure: {
		CompilerOptions: {
			languageIn: Packages.com.google.javascript.jscomp.CompilerOptions.LanguageMode.ECMASCRIPT5
		},
		CompilationLevel: 'SIMPLE_OPTIMIZATIONS',
		loggingLevel: 'WARNING'
	},



	/**
	 * Exclude pragmas on after build / at production
	 **/
	pragmas: {
		productionExclude: true
	},
	pragmasOnSave: {
		afterBuildExclude: true
	}
});