		/**
		 * Method to read the less/css file... (when optimizing)
		 * 
		 * CODE COPIED...
		 * See also layout/tpl (method is copied from there...)
		 * Or text! where the original code is located...
		 * 
		 * NOTE JEPE: If we were executing from command line, we could have used the build in Rhino method "readFile(path, encoding)"...
		 * (It is only available when run from command line)
		 */
		function readFile(url) {
			var stringBuffer, line,
				encoding = "utf-8",
				file = new java.io.File(url),
				lineSeparator = java.lang.System.getProperty("line.separator"),
				input = new java.io.BufferedReader(new java.io.InputStreamReader(new java.io.FileInputStream(file), encoding)),
				content = '';
			try {
				stringBuffer = new java.lang.StringBuffer();
				line = input.readLine();

				// Byte Order Mark (BOM) - The Unicode Standard, version 3.0, page 324
				// http://www.unicode.org/faq/utf_bom.html

				// Note that when we use utf-8, the BOM should appear as "EF BB BF", but it doesn't due to this bug in the JDK:
				// http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4508058
				if (line && line.length() && line.charAt(0) === 0xfeff) {
					// Eat the BOM, since we've already found the encoding on this file,
					// and we plan to concatenating this buffer with others; the BOM should
					// only appear at the top of a file.
					line = line.substring(1);
				}

				if (line !== null) {
					stringBuffer.append(line);
				}

				while ((line = input.readLine()) !== null) {
					stringBuffer.append(lineSeparator);
					stringBuffer.append(line);
				}
				//Make sure we return a JavaScript string and not a Java string.
				content = String(stringBuffer.toString()); //String   
			} finally {
				input.close();
			}
			return content;
		}

		/**
		 * COPIED AND MODIFIED FROM WEBCSS-BUILD, WHICH IS COPIED ON ITS TURN FROM LESSC-RHINO
		 * Needed to make less import statements work correctly
		 */
		less.Parser.fileLoader = function (file, currentFileInfo, callback, env) {

			var href = file;
			if (currentFileInfo && currentFileInfo.currentDirectory && !/^\//.test(file)) {
				href = less.modules.path.join(currentFileInfo.currentDirectory, file);
			}

			var path = less.modules.path.dirname(href);

			var newFileInfo = {
				currentDirectory: path + '/',
				filename: href
			};

			if (currentFileInfo) {
				newFileInfo.entryPath = currentFileInfo.entryPath;
				newFileInfo.rootpath = currentFileInfo.rootpath;
				newFileInfo.rootFilename = currentFileInfo.rootFilename;
				newFileInfo.relativeUrls = currentFileInfo.relativeUrls;
			} else {
				newFileInfo.entryPath = path;
				newFileInfo.rootpath = less.rootpath || path;
				newFileInfo.rootFilename = href;
				newFileInfo.relativeUrls = env.relativeUrls;
			}

			var j = file.lastIndexOf('/');
			if (newFileInfo.relativeUrls && !/^(?:[a-z-]+:|\/)/.test(file) && j != -1) {
				var relativeSubDirectory = file.slice(0, j + 1);
				newFileInfo.rootpath = newFileInfo.rootpath + relativeSubDirectory; // append (sub|sup) directory path of imported file
			}
			newFileInfo.currentDirectory = path;
			newFileInfo.filename = href;

			var data = null;
			try {
				data = readFile(href);
			} catch (e) {
				callback({
					type: 'File',
					message: "'" + less.modules.path.basename(href) + "' wasn't found"
				});
				return;
			}

			try {
				callback(null, data, href, newFileInfo, {
					lastModified: 0
				});
			} catch (e) {
				callback(e, null, href);
			}
		};

		function compileLessString(content, baseUrl, fileUrl, outBaseUrl, outFileUrl, cfg) {
			if (!cfg){
				cfg = {};
			}
			
			cfg.paths = [baseUrl];
			
			if (fileUrl) {
				cfg.filename = baseUrl + java.io.File.separator + fileUrl;
			} else {
				cfg.filename = baseUrl
			}
		
			(new less.Parser(cfg)).parse(content, function (err, root) {
				if (err) {
					// There is an error in the less file...
					throw (err + ' at ' + err.filename + ', line ' + err.line + " - " + err.column);
					return;

				}
				
				try {
					var css = root.toCSS(cfg);
				} catch (e) {
					throw (e.message);
					return;
				}

				var cssFile = new java.io.File(outBaseUrl + java.io.File.separator + outFileUrl);
				var cssFileWriter = new java.io.FileWriter(cssFile);

				logger.info("writing to file " + outBaseUrl + java.io.File.separator + outFileUrl);
				cssFileWriter.write(css);
				cssFileWriter.flush();
				cssFileWriter.close();

			});
		}
		// These variables are defined in java
		// staticDir: the directory that contains all the less files that should be compiled, 
		// customDir: the directy where the custom css files will be outputted
		// relativeCustomPath: relative path staticDir to customDir/wcr to the custom styles/wcr folder


		// create the wcr.css files, by importing the customvariables. these aren't namespaced yet
		var wcrLess = "@import 'variables.less';" +
			"@import '.." +  java.io.File.separator + relativeCustomPath +  java.io.File.separator + "customVariables.less';" +
			readFile(staticDir + java.io.File.separator + "wcr" + java.io.File.separator + "wcrModules.less");

		logger.info("Compiling wcr.less");
		compileLessString(wcrLess, staticDir + java.io.File.separator + "wcr", "wcrModules.less", customDir + java.io.File.separator + "wcr", "wcr.css");


		logger.info("Compiling style.less");
		// Namespace the custom css
		var nameSpacedLess = "@import 'wcr" + java.io.File.separator + "variables.less'; @import '" +
			relativeCustomPath +  java.io.File.separator + "customVariables.less';@import 'wcr" +
			java.io.File.separator + "namespacingBootstrap.less'; .w { @import (less) '" +
			relativeCustomPath +  java.io.File.separator + "wcr.css';}";
		
		compileLessString(nameSpacedLess, staticDir, "style.less", customDir, "style.css", {strictMath: true, compress: true});
