define(["component/actionbuttons/AbstractButton",
        "jquery",
		"underscore",
		"utils/webi18n!ActionBtnStrings",
		"doctablex/AJAXDataCache",
		"doctablex/DocTableXUtils"
	],
	function (	AbstractButton,
				$,
				_,
				ActionBtnStrings,
				AJAXDataCache,
				DocTableXUtils) {
		"use strict";

		/**
		 * This will generate a button to download a document
		 * 
		 * @exports DownloadButton 
		 * @constructor
		 * @param {String} docVerId
		 * @param {String} (optional) ctx - context to render button in; can be skipped now to be added later
		 */
		var DownloadButton = function(docVerId, ctx) {
			// default download type = none
			this.downloadType = 'none';
			
			AbstractButton.call(this, docVerId, ctx);
		};
		
		// extend from the AbstractButton
		DownloadButton.prototype = Object.create(AbstractButton.prototype);
		DownloadButton.prototype.constructor = DownloadButton;
		
		/**
		 * Render the button in a given context. This means the button can be on several locations.
		 * @override
		 * @param {JQuery object} ctx - context
		 */
		DownloadButton.prototype._renderBtnImpl = function(ctx) {
			var btn = $('<a/>')
					.addClass('action actionBtn')
					.attr('title', ActionBtnStrings.t('DOWNLOAD'));
			btn.click(_.bind(function(){
				if(this.downloadType == 'pespective'){
					this.showDownloadPerspectives();
				} else if(this.downloadType == 'locked'){
					// show warning before downloading
					if(confirm(ActionBtnStrings.t('CHECKED_OUT_DOWNLOAD_WARNING'))){
						this.downloadDocument();
					}
				} else if(this.downloadType = 'download') {
					// just download
					this.downloadDocument();
				}
				return false; // to prevent browsing to #
			},this));

			var icon = $('<i/>').addClass('icon icon-normal icon-WCR_Actions-Download12');
			btn.append(icon);
			ctx.replaceWith(btn);
		};
		
		/**
		 * Download the document
		 */
		DownloadButton.prototype.downloadDocument = function() {
			var sourcePage = DocTableXUtils.getSourcePageURLEncoded();
			window.location = this.baseUrl + 'doprojectdocsactions.jsp?actionVal=DownloadDoc&actionDocVerID=' + this.docVerId + '&projectID=' + this.projectId + '&sourcePage=' + sourcePage;
		};
		
		/**
		 * Fetch and display download formats
		 */
		DownloadButton.prototype.showDownloadPerspectives = function() {
			var perspectiveLoader =  AJAXDataCache.getInstance().getDocDownloadPerspectives(this.docVerId, this.docTypeId);
			$.when(perspectiveLoader).done(_.bind(function(perspectives){
				if(perspectives == null){
					alert(ActionBtnStrings.t("UNABLETOGETFORMATSFORDOWNLOAD"));
				} else if (perspectives.length == 0){
					alert(ActionBtnStrings.t("NOFORMATSFORFILETYPE"));
				} else {
					this.popupContent = $("<div />").addClass("selectformatdialog");
					for(var i=0; i<perspectives.length; i++) {
						var name = perspectives[i].name;
						var longName = perspectives[i].longName;
						if (name === 'Native') {
							name = ActionBtnStrings.t("NATIVE_PERSPECTIVE_NAME");
							longName = "";
						}
						
						var radio = $('<input type="radio" name="selectedDownloadFormat"/>');
						radio.val(longName);
						
						if(i === 0) {
							radio.prop("checked", true);
						}
						
						var radioDiv = $('<div>').addClass("radio").append($("<label>").append(radio, $(document.createTextNode(name))));
						
						this.popupContent.append(radioDiv);
					}
					this.showDialog();
				}
			}, this));
		};
		
		/**
		 * Show perspectives dialog
		 */
		DownloadButton.prototype.showDialog = function() {
			if(this.popupContent){
				// make buttons
				var buttonObj = new Object();
			    buttonObj[ActionBtnStrings.t("CANCEL")]= _.bind(function(){
			    	this.popupContent.dialog("close");
			    },this);
			    buttonObj[ActionBtnStrings.t("DOWNLOAD")]= _.bind(function(){
			        if (this.downloadPerspective()) {
			        	this.popupContent.dialog("close");
			        }
			    },this);

			    // show dialog
			    this.popupContent.dialog({
			        autoOpen: false,
			        dialogClass: "no-close",        
			        modal: true,
			        title : ActionBtnStrings.t("SELFORMAT_NOTREDUNDANT"),
			        position: 'center',
			        buttons: buttonObj
	    		}).dialog('open');
			}
		};
		
		/**
		 * Download document in certain perspectives
		 * @return {boolean}
		 */
		DownloadButton.prototype.downloadPerspective = function() {
			var selection = this.popupContent.find("input:checked");
	    	//check, if some icon is chosen
		   if (selection.length != 1) {
		       alert(ActionBtnStrings.t("ONEFORMATHASTOBESELECTEDTOPERFORMDOWNLOAD"));
		       return false;
		   }

		   window.location = this.baseUrl + 'dodocdetailsactions.jsp?actionVal=DoDownloadPerspective&docVerID=' + this.docVerId + '&perspectiveName=' + selection.val();
		   return true;
		};
		
		
		/**
		 * Update Document info based on server response
		 * @override
		 */
		DownloadButton.prototype.updateDocumentInfo = function(docInfo) {
			this.docTypeId = docInfo.docTypeID;
			if(docInfo.projList.length>0){
				this.projectId = docInfo.projList[0].id;
			}
			
			if(docInfo.userPerms.download && !docInfo.docIsUrl){
				if((docInfo.docTypeID == '00001_0000000001') || (docInfo.docTypeID == '00001_0000000002') || (docInfo.docTypeID == '00001_0000000018')){
					// document with perspectives
					this.downloadType = 'pespective';
				} else if(docInfo.docCheckedOutOwner){
					// locked document
					this.downloadType = 'locked';
				} else {
					// normal download
					this.downloadType = 'download';
				}
			}
			
			this.isVisible = (this.downloadType != 'none');
        };
		
        /*
    	 * This component is createable using data-init
    	 */
        DownloadButton.wcrinit = function(div) {
        	var docVerId = $(div).attr("data-docverid");
    		new DownloadButton(docVerId, $(div));
    	};
		
		return DownloadButton;
	});