define(["component/actionbuttons/AbstractButton",
        "jquery",
		"underscore",
		"utils/webi18n!ActionBtnStrings",
		"doctablex/AJAXDataCache",
		"doctablex/DocTableXUtils"
	],
	function (	AbstractButton,
				$,
				_,
				ActionBtnStrings,
				AJAXDataCache,
				DocTableXUtils) {
		"use strict";

		/**
		 * This will generate a button to Open the document in the 3D viewer
		 * This will only be shown when the document has a 3D view
		 * 
		 * @exports Open3DButton 
		 * @constructor
		 * @param {String} docVerId
		 * @param {String} (optional) ctx - context to render button in; can be skipped now to be added later
		 */
		var Open3DButton = function(docVerId, ctx) {
			AbstractButton.call(this, docVerId, ctx);
		};
		
		// extend from the AbstractButton
		Open3DButton.prototype = Object.create(AbstractButton.prototype);
		Open3DButton.prototype.constructor = Open3DButton;
		
		Open3DButton.prototype._renderBtnImpl = function(ctx) {
			var btn = $('<a/>')
					.addClass('action actionBtn')
					.attr('title', ActionBtnStrings.t('CLICK_HERE_TO_VIEW_3D'));
			btn.click(_.bind(function(){
				this.openDocument3D();
				return false; // to prevent browsing to #
			},this));
			
			var icon = $('<i/>').addClass('icon icon-normal icon-WCR_Actions-3D12');
			btn.append(icon);
			ctx.replaceWith(btn);
		};
		
		/**
		 * Open document in 3D viewer in new tab
		 */
		Open3DButton.prototype.openDocument3D = function() {
			var menuParam = DocTableXUtils.getMenuFileParamValue();
			var sourcePage = DocTableXUtils.getSourcePageURLEncoded();
			var url = this.baseUrl + 'doctypecolladaview.jsp?docVerID='+this.docVerId+'&menu_file='+menuParam+'&sourcePage='+sourcePage;

			this.openInNewTab(url);
		};
		
		/**
		 * Update Document info based on server response
		 */
		Open3DButton.prototype.updateDocumentInfo = function(docInfo) {
			this.isVisible = (docInfo.has3dView == '1');
        };
        
        /*
    	 * This component is createable using data-init
    	 */
        Open3DButton.wcrinit = function(div) {
        	var docVerId = $(div).attr("data-docverid");
    		new Open3DButton(docVerId, $(div));
    	};
		
		return Open3DButton;
	});