define(["component/actionbuttons/AbstractButton",
        "jquery",
		"underscore",
		"utils/webi18n!ActionBtnStrings",
		"doctablex/DocTableXUtils",
		"doctablex/AJAXDataCache"
	],
	function (	AbstractButton,
				$,
				_,
				ActionBtnStrings,
				DocTableXUtils,
				AJAXDataCache) {
		"use strict";

		/**
		 * This will generate a button to open the viewer, given only a document version Id
		 * 
		 * @exports ViewButton 
		 * @constructor
		 * @param {String} docVerId
		 * @param {String} (optional) ctx - context to render button in; can be skipped now to be added later
		 */
		var ViewButton = function(docVerId, ctx) {
			AbstractButton.call(this, docVerId, ctx);
		};
		
		// extend from the AbstractButton
		ViewButton.prototype = Object.create(AbstractButton.prototype);
		ViewButton.prototype.constructor = ViewButton;
		
		/**
		 * Render button which will open viewer in new tab
		 * @override
		 * @param ctx
		 */
		ViewButton.prototype._renderBtnImpl = function(ctx) {	
			var btn = $('<a/>')
					.addClass('action actionBtn')
					.attr('title', ActionBtnStrings.t('CLICK_HERE_TO_VIEW'));
			btn.click(_.bind(function(){
				this.openViewer();
				return false; // to prevent browsing to #
			},this));

			var icon = $('<i/>').addClass('icon icon-normal icon-WCR_Actions-View12');
			btn.append(icon);
			ctx.replaceWith(btn);
		};
		
		/**
		 * Open document in new tab
		 */
		ViewButton.prototype.openViewer = function() {
			var menuParam = DocTableXUtils.getMenuFileParamValue();
			var sourcePage = DocTableXUtils.getSourcePageURLEncoded();
			var url = this.baseUrl + 'viewdocument.jsp?docVerID='+this.docVerId+'&menu_file='+menuParam+'&pageIndex=0&sourcePage='+sourcePage;

			this.openInNewTab(url);
		};
		
		/**
		 * Update document info to determine visibility.
		 * @override
		 * @param {Object} docInfo
		 */
		ViewButton.prototype.updateDocumentInfo = function(docInfo) {
			this.isVisible = docInfo.userPerms.view && !(docInfo.docIsUrl);
		};
		
		/*
    	 * This component is createable using data-init
    	 */
		ViewButton.wcrinit = function(div) {
        	var docVerId = $(div).attr("data-docverid");
    		new ViewButton(docVerId, $(div));
    	};
		
		return ViewButton;
	});