define(["underscore"], function(_) {
	
	/**
	 * Will be passed to the action handler when an action get's executed.
	 * 
	 * Contains information about the action to execute
	 */
	var ActionExecuteEvent = function () {
		
		// {jQuery} - The action button that the user clicked to execute the action.
		// Note: will never be null
		this.actionbutton = null;
		
		// {Event} - The event that caused the action
		this.event = null;
		
		// {String} - The name of the action to execute. (See "data-name" on action)
		// Note: defined by "data-action" attribute on actionlink. can be null
		this.action = null;
		
		// The items... Use one of the methods below to access them.
		this._items = [];
		
		
		//
		// Information about the table & action menu
		//
		
		// {ActionMenu, RowActionMenu} - The action menu which contains the clicked action
		this.actionmenu = null;
		
		// {Table} - The table component which contains this action menu (if any) (can be null)
		this.table = null;
	};
	
	//
	// Methods for easier handling of actions
	//
	
	/**
	 * Prevent the default action of the original event
	 */
	ActionExecuteEvent.prototype.preventDefault = function() {
		if(this.event) {
			this.event.preventDefault();
		}
	};
	
	/**
	 * When the action is executed by the RowActionMenu we always have only one item.
	 * 
	 * @returns the first item.
	 */
	ActionExecuteEvent.prototype.getItem = function() {
		return this._items[0];
	};
	
	/**
	 * Get the array of checkboxes.
	 * 
	 * @returns 
	 */
	ActionExecuteEvent.prototype.getActionCheckboxes = function() {
		return _.map(this._items, _.property("checkbox"));
	};
	
	/**
	 * Get an attribute of every item and return it as an array
	 * 
	 * @returns {Array}
	 */
	ActionExecuteEvent.prototype.getActionItemsAttribute = function(attribute) {
		return _.filter(_.map(this._items, function(e) {return e.checkbox.attr(attribute);}), function(v){return !!v;});
	};
	
	
	return ActionExecuteEvent;
});