define(["utils/webi18n!GlobalActionHandler", "underscore", "jquery"], function(i18n, _, $){
	"use strict";
	
	/**
	 * Global action handler
	 * 
	 * @param {ActionExecuteEvent} event - information about the action to execute
	 */
	return {executeAction:function(event) {
		var actionbutton = event.actionbutton;
		var actionName = event.action;
		var itemIdSel = actionbutton.attr("data-itemidsel");
		if (typeof(itemIdSel) == 'undefined')
			itemIdSel = "docversionid";
		var itemIdName = actionbutton.attr("data-itemidname");
		if (typeof(itemIdName) == 'undefined')
			itemIdName = "docversionid";
		
		switch(actionName){
				
			case "GLOBAL_ACTION_TEST": 
				event.preventDefault();
				require(["component/pagemessagecomponent/PageMessageComponent"], function(PageMessageComponent){
					PageMessageComponent.sendSuccessMessage("Starting Call", null, false, "ACTION_CALLED");
				var selectedIDs = event.getActionItemsAttribute(itemIdSel);
				setTimeout(function(){
						PageMessageComponent.markObsolete("ACTION_CALLED");
					
						PageMessageComponent.sendSuccessMessage("Executed GLOBAL ACTION TEST on: ["+selectedIDs.join(", ")+"]", null, false, null);
					
					}, 1000);	
				});
				break;
				
			case "SCRIPT": 
				event.preventDefault();
				
				// make parameters available for script
				// depending on context, some of these will be "undefined"
				var scriptexecutioncontext = {};
				scriptexecutioncontext.selectedIDs = {};
				if (itemIdSel.length > 0) {
					scriptexecutioncontext.selectedIDs = event.getActionItemsAttribute(itemIdSel);
					if (scriptexecutioncontext.selectedIDs && scriptexecutioncontext.selectedIDs.length > 0) {
						for (var i = 0; i < scriptexecutioncontext.selectedIDs.length; i++) {
							var n = scriptexecutioncontext.selectedIDs[i].indexOf(",");
							// multiple ids - take first
							if (n != -1)
								scriptexecutioncontext.selectedIDs[i]=scriptexecutioncontext.selectedIDs[i].substring(0, n);
						}
					}
				}					
				scriptexecutioncontext.projectID = actionbutton.attr("data-projectid");
				scriptexecutioncontext.docVersionID = actionbutton.attr("data-docversionid");
				scriptexecutioncontext.menuFile = actionbutton.attr("data-menufile");
				
				// Note that the JavaScript minifier will rename all local variables!
				// However, we want the variables to be available in a local variable when running the script!
				var variablelist = [];
				var variablevaluelist = [];
				_.each(scriptexecutioncontext, function(val, key) {
					variablelist.push(key);
					variablevaluelist.push(val);
				});
				// Put a function around it which takes all the arguments we provide.
				// Note: newline at the end to handle // single line comments
				// Note: ( ) are required for eval to see it as a function expression (not a function statement)
				var funct = eval("(function("+variablelist.join(",")+"){\n"+actionbutton.attr("data-script")+"\n})");
				if(funct) {
					// Call the function with all the arguments.
					funct.apply(window, variablevaluelist);
				}
				break;
				
			case "REDIRECT": 
				event.preventDefault();
				
				var url = actionbutton.attr("data-url");

				require(["component/pagemessagecomponent/PageMessageComponent"], 
						function(PageMessageComponent){
					PageMessageComponent.markObsolete("ACTION_COMPLETED");

					var started = i18n.t("GLOBALACTIONHANDLER_TASKCREATIONSSTARTED");
					PageMessageComponent.sendSuccessMessage(started, null, true, "ACTION_COMPLETED");
					
					if (itemIdSel.length > 0) {
						var selectedIDs = event.getActionItemsAttribute(itemIdSel);
						if (selectedIDs && selectedIDs.length > 0) {
							for (var i = 0; i < selectedIDs.length; i++) {
								var n = selectedIDs[i].indexOf(",");
								// multiple ids - take first
								if (n != -1)
									selectedIDs[i]=selectedIDs[i].substring(0, n);
							}
							
							var sdkData = itemIdName + "=" + selectedIDs[0];
							for (var i = 1; i < selectedIDs.length; i++) {
								sdkData += "&" + itemIdName + "=" + selectedIDs[i];
							}
							var glue = '';
							var n1 = url.indexOf('?');
							var n2 = url.indexOf('&');
							if (!((n1 == (url.length - 1)) || (n2 == (url.length - 1)))) {
								if (n1 >= 0)
									glue = '&';
								else
									glue = '?';
							}
							url += glue + sdkData;
						}
					}
					
					//window.location.replace(url); // this will replace the source page in the browser page History
					window.location.href = url; // this will redirect, but leave the original page in the browser history
				});
				break;

			case "START_WORKFLOW": 
				event.preventDefault();
				
				var url = actionbutton.attr("data-url");

				require(["component/pagemessagecomponent/PageMessageComponent"], 
						function(PageMessageComponent){
					PageMessageComponent.markObsolete("ACTION_COMPLETED");

					var started = i18n.t("GLOBALACTIONHANDLER_SUBMITTED");
					PageMessageComponent.sendSuccessMessage(started, null, false, "ACTION_CALLED");
					
					var sdkData = '';					
					if (itemIdSel.length > 0) {
						var selectedIDs = event.getActionItemsAttribute(itemIdSel);
						if (selectedIDs && selectedIDs.length > 0) {
							for (var i = 0; i < selectedIDs.length; i++) {
								var n = selectedIDs[i].indexOf(",");
								// multiple ids - take first
								if (n != -1)
									selectedIDs[i]=selectedIDs[i].substring(0, n);
							}
							
							sdkData = itemIdName + "=" + selectedIDs[0];
							for (var i = 1; i < selectedIDs.length; i++) {
								sdkData += "&" + itemIdName + "=" + selectedIDs[i];
							}
						}
					}
					
					$.ajax({
						url: url,
						type: "POST",
						data: sdkData,
						dataType: 'xml',
						timeout: 120000,
						success: function(data){
							PageMessageComponent.markObsolete("ACTION_CALLED");
							
			                //check for error state first
			                var errCodeNode = data.getElementsByTagName("error");
			                if (errCodeNode[0]) {
				                var errCode = errCodeNode[0].getElementsByTagName("code");
				                var errText = errCodeNode[0].getElementsByTagName("message");
			                	var errorCodeVal = $(errCode[0]).text();
			                	var errorTextVal = $(errText[0]).text();
				                var message = i18n.t("GLOBALACTIONHANDLER_TASKCREATIONFAILEDSDK", '[' + errorCodeVal + '] ' + errorTextVal);
				                PageMessageComponent.sendErrorMessage(message, null, null);
			                	return;
			                }          
							
			                //process the returned XML
			                var resultXMLRootNode = data.getElementsByTagName("result");
			                if (resultXMLRootNode[0]) {
				                var tasksXML = resultXMLRootNode[0].getElementsByTagName("tasks");
				                if (tasksXML[0]) {
					                var taskNodesXML = tasksXML[0].getElementsByTagName("task");
					                if (taskNodesXML[0]) {
					                	var multiple = (taskNodesXML.length > 1);

					                	var feedback;
					                	if (multiple)
					                		feedback = $('<div>').text(i18n.t("GLOBALACTIONHANDLER_TASKSCREATED", taskNodesXML.length));
					                	else
					                		feedback = $('<div>').text(i18n.t("GLOBALACTIONHANDLER_TASKCREATED"));

					                	for (var task = 0; task < taskNodesXML.length; task++) {
					                		var taskNameElem = taskNodesXML[task].getElementsByTagName("taskName");
					                		var taskIDElem = taskNodesXML[task].getElementsByTagName("taskID");
					                		var projectIDElem = taskNodesXML[task].getElementsByTagName("projectID");
					                		if (multiple)
					                			feedback.append('<br/>');
					                		feedback.append('&nbsp;');
					                		var urlParams = {
				                				menu_file:"myallprojects",
				                				projectID:$(projectIDElem).text(),
				                				curProjTaskID:$(taskIDElem).text(),
					                		};						                		
					                		feedback.append($("<a>").attr("href", "projdetailswctaskdetailsworkflow.jsp?"+$.param(urlParams)).text($(taskNameElem).text()));
					                	}
					                		
					                	PageMessageComponent.sendSuccessMessage(feedback, null, false, "ACTION_COMPLETED");
					                	return;
					                }
				                }
			                }
		                	var message = i18n.t("GLOBALACTIONHANDLER_TASKCREATIONFAILEDXML");
		                	PageMessageComponent.sendErrorMessage(message, null, null);
			                
			                return;
			            },
						error: function(jqXHR, textStatus, errorThrown){
							PageMessageComponent.markObsolete("ACTION_CALLED");

							var message = i18n.t("GLOBALACTIONHANDLER_TASKCREATIONFAILEDAJAX", errorThrown);
		                	PageMessageComponent.sendErrorMessage(message, null, null);
					  	}
					});	      		
					
				});
				break;
				
			case "toggle-selection-table-all":
				if(event.table) {
					event.table.toggleSelectAll();
				}
				break;
			}
	}};
});