define(["./ActionMenu", "./ActionMenuBase", "jquery"], function (ActionMenu, ActionMenuBase, $) {
	/**
	 * ActionMenu that calculates correct sizing for header actions
	 * @class
	 * @constructor
	 * @exports HeaderActionMenu
	 * @augments ActionMenu
	 * @param {JQueryObj} context - div in which the action menu is defined
	 * @param {JQueryObj} [checkboxContainer] - container for which action menu is defined (this is used for data-require-selection)
	 * @param {String}	[checkboxSelector] - Selector string to select checkboxes in the checkboxContainer (this is used for data-require-selection)
	 */
	var HeaderActionMenu = function (context, checkboxContainer, checkboxSelector) {
		ActionMenu.call(this, context, checkboxContainer, checkboxSelector);
	};

	HeaderActionMenu.prototype = Object.create(ActionMenu.prototype);
	HeaderActionMenu.prototype.constructor = HeaderActionMenu;


	/**
	 * Get the maximum with of the main container
	 * @Override
	 * @returns {Number}
	 */
	HeaderActionMenu.prototype._getMaximumWidth = function () {
		var paddingLeft = 0,
			paddingRight = 0;
		
		var parentContainer = this._container.parent();
		
		try {
			//since we use the border-box model, we need to subtract padding from the width
			var computedStyle = getComputedStyle(parentContainer[0]);
			paddingLeft = parseFloat(computedStyle.getPropertyValue("padding-left"));
			paddingRight = parseFloat(computedStyle.getPropertyValue("padding-right"));
		} catch (e) {
			// ignore
		}

		var availableWidth = parentContainer.innerWidth() - paddingLeft - paddingRight;
		parentContainer.children().not(this._container).each(function(index, element){
			availableWidth-= $(element).outerWidth();
		});
		
		return availableWidth;
	};


	HeaderActionMenu.wcrinit = function(element){
		if (!(element instanceof $)) {
			element = $(element);
		}

		var data = ActionMenuBase._wcrinitCollectSettings(element);

		// Initialize
		var actionMenu = new HeaderActionMenu(element, data.checkboxContainer, data.checkboxSelector);

		// Find the parent Table component (if any)
		if(data.wcrtable.length>0) {
			init.whenInitialized(data.wcrtable, function() {
				var tableComponent = data.wcrtable.get(0).Table;
				actionMenu.setTable(tableComponent);
			});
		}

		// Load & attach handler
		ActionMenuBase._wcrinitRegisterActionHandler(element, data, actionMenu);	
	};

	return HeaderActionMenu;

});