define(["component/modal/Modal",
		"component/button/Button",
		"underscore",
		"jquery",

	],
	function (
		Modal,
		Button,
		_,
		$) {

		"use strict";

		/**
		 * A popup to import data to WebCenter
		 * @param {Object} languageKeys The languageStrings that have to be used by this popup 
		 *                  The complete structure of this object should be: 
		 *                  {
		 *                  	ImportConfirmText : '', 
		 *               		ImportConfirmTooltip: '',
		 *                 		ImportConfirmCancel: '',
		 *                      ImportConfirmOk : '',
		 *                      ImportTooltip: '',
		 *                      ImportCancel: '',
		 *                      ImportOk: ''
		 *					}
		 * @param {Template} template the template of the form that has to be submitted                              
		 * @class
		 */
		var ImportPopup = function (languageStrings, template) {
			
			// Confirm popup
			this._confirmContent = $("<div>").text(languageStrings.ImportConfirmText);
			this._importConfirm = new Modal(this._confirmContent);
			this._importConfirm.setTitle(languageStrings.ImportConfirmTooltip);

			this._importConfirmCancel = new Button(null, null, false, "Cancel", languageStrings.ImportConfirmCancel, true, _.bind(function () {
				this._doNotDestroyPopup = true;
				this._importConfirm.close();
				this._importPopup.open();
				this._importButton.setEnabled(true);
			}, this));
			this._importConfirmCancel.addClass("btn btn-default btn-md");

			this._importConfirm.onClose(_.bind(function () {
				if (!this._doNotDestroyPopup) {
					this._importConfirm.destroy();
					this._importPopup.destroy();
				}
				this._doNotDestroyPopup = false;
			}, this));

			this._importConfirmOK = new Button(null, null, false, "Cancel", languageStrings.ImportConfirmOk, true, _.bind(function (popup) {
				this.importConfirm();
				this._importConfirm.close();
			}, this));
			this._importConfirmOK.addClass("btn btn-primary btn-md");
			this._importConfirmButtonGroup = $("<div>").addClass("btn-group pull-right");
			this._importConfirmButtonGroup.append(this._importConfirmCancel.element).append(this._importConfirmOK.element);
			this._importConfirm.setFooter(this._importConfirmButtonGroup);

			// Popup
			this._content = $("<div>");

			// Template for popup content
			this.template = template;
			
			//set the context for the template, this is the content of the popup
			this.template.setContext(this._content);
			//register change event
			this.template.bind("change", ".datafile", _.partial(this._dataFileChanged, this));
			this._importPopup = new Modal(this._content);
			this._importPopup.setTitle(languageStrings.ImportTooltip);

			this._importPopup.onClose(_.bind(function () {
				if (!this._doNotDestroyPopup) {
					this._importConfirm.destroy();
					this._importPopup.destroy();
				}
				this._doNotDestroyPopup = false;
			}, this));

			this._cancelButton = new Button(null, null, false, "Cancel", languageStrings.ImportCancel, true, _.bind(function () {
				this._importPopup.close();
			}, this));
			this._cancelButton.addClass("btn btn-default btn-md");
			this._importButton = new Button(null, null, false, "OK", languageStrings.ImportOk, true, _.partial(function (popup) {
				this.setEnabled(false);
				popup._doNotDestroyPopup = true;
				popup._importPopup.close();
				popup._importConfirm.open();
			}, this));
			this._importButton.addClass("btn btn-primary btn-md");
			this._importButton.setEnabled(false);

			this._buttonGroup = $("<div>").addClass("btn-group pull-right");
			this._buttonGroup.append(this._cancelButton.element).append(this._importButton.element);
			this._importPopup.setFooter(this._buttonGroup);
		};

		/**
		 * Executed when data file is changed
		 */
		ImportPopup.prototype._dataFileChanged = function (popup, elem) {
			var fileName = $(elem).val();
			popup._importButton.setEnabled(fileName);
		};
	
		/**
		 * Executed when the import of a datafile is confirmed
		 */
		ImportPopup.prototype.importConfirm = function(){
			//should be overridden 
		};

		/**
		 * Open the popup
		 * Might have to be extended
		 */
		ImportPopup.prototype.open = function () {
			this.template.render();
			this._importPopup.open();
			this._importButton.setEnabled(false);
		};

		return ImportPopup;
	});
