/**
 * <p>A badge to show a number of items above an icon or something else</p>
 * 
 * <p>Attach this to the element surrounding the icon/thing...</p>
 * <p>An additional span will be inserted into the element which shows the badge.</p>
 * 
 * @author JEPE
 */
define(["lib/dom", "lib/text!./iconbadge-style.css"], function(dom, css) {
	// Import the css
	dom.importCssString(css);
	
	/**
	 * Constructor
	 */
	var IconBadge = function(div){
		// save the jQuery element
		this.$div = $(div);
		
		// make this object accessible
		this.$div.get(0).IconBadge = this;
		
		// make & save the badge
		this.$badge = $("<span />").addClass("wcriconbadge").hide();
		
		// add an additional class... Note: "big" will make the badge big.
		var badgeclass = this.$div.attr("data-badgeclass");
		if(badgeclass) {
			this.$badge.addClass(badgeclass);
		}
		
		// add the badge to the element
		this.$div.addClass("wcriconbadgecontainer");
		this.$div.prepend(this.$badge);
		
		// initialize the count
		var badgecount = this.$div.attr("data-badgecount");
		if(badgecount) {
			this.setCount(badgecount);
		}
	};
	
	/**
	 * Used to set the count
	 * 
	 * If content is null or the empty string or "0", the badge will be hidden.
	 * 
	 * @param content the count to display in the badge (could be text too in theory). 
	 */
	IconBadge.prototype.setCount = function(content) {
		if(content && content+""!=="0") {
			this.$badge.text(content);
			
			// check if it is already shown... 
			// before trying to show it again.
			if(!this.shown) {
				// fade in (otherwise, see it flash on page load...)
				this.$badge.stop().fadeIn(600);
			}
			this.shown = true;
		}else{
			this.$badge.text("");
			
			// hide it
			this.$badge.stop().fadeOut(600);
			this.shown = false;
		}
	};
	
	/**
	 * Initialize as a component
	 */
	IconBadge.wcrinit = function(div) {
		new IconBadge(div);
	};
	
	return IconBadge;
});
