define(["underscore", "jquery"], function(_, $) {
	
	// Constants... (used by JS layout)
	var SIZE_THUMBNAIL = 200;
	var BORDER = 1;
	var GRID_SPACING = 10;
	var GRID_MARGIN = 10;
	
	/**
	 * @constructor
	 * @param {DOMElement} div
	 */
	var CardGridControl = function(div, options){
		this.div = $(div);
		this.options = options;
		this.options["thumbnailselectscard"] = !!this.options["thumbnailselectscard"];
		
		// Initialize
		this._initialize();
		
		// Layout
		this._layout();
		
		$(window).resize(_.throttle(_.bind(function () {
			this._layout();
		}, this), 250));
	};
	
	/**
	 * Initialize all the listeners
	 */
	CardGridControl.prototype._initialize = function(){
		// Flip a card
		this.div.on("click", "[data-toggle='flip']", function(e){
			$(this).parents(".wrapper").first().toggleClass("flip");
			e.preventDefault();
		});
		
		// Select a card
		this.div.on("change", ".card-heading input[type='checkbox']", function(e){
			$(this).parents(".wrapper").first().toggleClass("selected", $(this).is(":checked"));
		});
		
		// If we already selected something when loading the page...
		$(".card-heading input[type='checkbox']", this.div).each(function(){
			$(this).parents(".wrapper").first().toggleClass("selected", $(this).is(":checked"));
		});
		
		// In document sources, clicking the thumbnail will select the card.
		if(this.options.thumbnailselectscard) {
			$(".card").each(function() {
				var card = $(this);
				var cardcheckbox = $(".card-heading input[type='checkbox']", card);
				$(".card-thumb", card).click(function() {
					cardcheckbox.prop("checked", !cardcheckbox.prop("checked"));
					cardcheckbox.trigger("change");
				});
			});
		}
	};
	
	/**
	 * Layout the cards using javascript
	 */
	CardGridControl.prototype._layout = function(){
		var availableSpace = this.div.width()-GRID_MARGIN*2;
		
		// measure the size of the header & footer... 
		//  = calculate the maximum of the height of the header + footer of all shown cards
		var additionalCardHeight = 
			_.max($(".card", this.div).map(function(){
					return 	$(".card-heading", $(this)).outerHeight(true);
							// don't count footer as part of the size... It is shown on top of the thumbnail
							//+$(".card-footer", $(this)).outerHeight(true);
				}).get());
		
		// How many cards can we fit on one row?
		var cardsPerRow = Math.floor((availableSpace+GRID_SPACING)/(BORDER*2+SIZE_THUMBNAIL+GRID_SPACING));
		
		// What is the spacing in such case?
		var cardsSpacing = availableSpace/cardsPerRow-(BORDER*2+SIZE_THUMBNAIL);
		
		// Sizes of the cards & card rows/columns
		var cardHeight = additionalCardHeight+BORDER*2+SIZE_THUMBNAIL;
		var cardWidth = BORDER*2+SIZE_THUMBNAIL;
		var cardHeightSpace = cardHeight+GRID_SPACING;
		var cardWidthSpace = cardWidth+cardsSpacing;
		
		// Set the height & position of each card
		var cards = $(".card", this.div);
		$(".card", this.div).each(function(i, e){
			var row = Math.floor(i/cardsPerRow);
			var col = i%cardsPerRow;
			
			$(e).css({
				top:cardHeightSpace*row+GRID_MARGIN+"px",
				left:cardWidthSpace*col+cardsSpacing/2+GRID_MARGIN+"px",
				height:cardHeight+"px",
				width:cardWidth+"px"
			});
		});
		
		// Set the height of the container div
		this.div.css({
			height:(Math.ceil(cards.length/cardsPerRow)*cardHeightSpace)+GRID_MARGIN*2+"px"
		});
	};
	
	
	
	/**
	 * For use with data-init
	 * 
	 * @param div
	 */
	CardGridControl.wcrinit = function(div){
		var thumbnailselectscard = $(div).attr("data-selectbythumbnail")==="1";
		
		new CardGridControl(div, 
				{thumbnailselectscard:thumbnailselectscard}
		);
	};
	
	return CardGridControl;
});