define(["../../dashboard/core/Context",
		"../../dashboard/core/BlockDefinition",
		"./DashboardExampleBlockDefinitions",
		"component/dashboard/i18n/WebCenterLanguageProvider",
		"underscore",
		"jquery",
		"dashboard/Dashboard"
	],
	function (Context,
		BlockDefinition,
		blockDefinitions,
		WebCenterLanguageProvider,
		_,
		$) {
		"use strict";



		function loadConfiguration(id, defaultConfig) {
			if (!id) {
				return defaultConfig;
			} else {
				try {
					var storageValue = localStorage.getItem("WCRDashboardExample_" + id);

					if (storageValue) {
						return JSON.parse(storageValue);
					}

				} catch (e) {
					console.warn("Error reading config from localStorage");
				}

				return defaultConfig;
			}

		}

		function storeConfiguration(id, config) {
			localStorage.setItem("WCRDashboardExample_" + id, JSON.stringify(config));
		}

		function initUndoRedo(context) {
			var undoredo = context.getUndoRedoController();

			var configBar = $("<div>", {
				'class': "btn-group pull-right"
			});
			var commandOngoing = false;
			var undo = $("<a>", {
					'class': "btn btn-default",
					title: "Undo"
				}).append($("<i>", {
					'class': "icon icon-WCR_Actions-Undo12"
				}))
				.click(function () {
					commandOngoing = true;
					onUndoRedoChanged();
					Promise.resolve(undoredo.undo()).then(function () {
						commandOngoing = false;
						onUndoRedoChanged();
					});
				});

			var redo = $("<a>", {
					'class': "btn btn-default",
					title: "Redo"
				}).append($("<i>", {
					'class': "icon icon-WCR_Actions-Redo12"
				}))
				.click(function () {
					commandOngoing = true;
					onUndoRedoChanged();
					Promise.resolve(undoredo.redo()).then(function () {
						commandOngoing = false;
						onUndoRedoChanged();
					});
				});


			var onUndoRedoChanged = _.debounce(function () {
				undo.toggleClass("disabled", commandOngoing || undoredo.undoCount() <= 0);
				undo.attr("title", "Undo " + (undoredo.peekUndo() ? undoredo.peekUndo().getDescription() : ""));
				redo.toggleClass("disabled", commandOngoing || undoredo.redoCount() <= 0);
				redo.attr("title", "Redo " + (undoredo.peekRedo() ? undoredo.peekRedo().getDescription() : ""));
			}, 10);
			undoredo.on("changed", onUndoRedoChanged);
			onUndoRedoChanged();


			configBar.append(undo).append(redo);

			return configBar;

		}


		function initDashboard(element, configurationID, defaultConfiguration, isConfigurable, openInEditMode) {

			var configuration = loadConfiguration(configurationID, defaultConfiguration);

			var context = new Context();
			context.setLanguageProvider(new WebCenterLanguageProvider());

			if (openInEditMode) {
				context.toggleEditMode(true);
			}
			
			var isDashboardLoaded = false;

			// show loading indicator only after 750ms
			setTimeout(function(){
				if (!isDashboardLoaded){					
					element.text("loading...");
				}
			}, 750);
			element[0].__CONTEXT = context;

			//Configure the datasource based on information set in the config
		
			context.getBlockDefinitionDataSource().then(function (datasource) {
				datasource.setBlockDefinitions(_.map(blockDefinitions, function (item) {
					return new BlockDefinition("", item.type, item);
				}));
				
				
				if (defaultConfiguration.blockDefinitions){
					_.each(defaultConfiguration.blockDefinitions, function(blockDef) {
						datasource.addBlockDefinition(new BlockDefinition("", blockDef.type, blockDef));
					});	
				}
			});

		

			var configBar = $("<div>", {
					'class': "btn-group pull-right"
				})
				.append($("<a>", {
						'class': "btn btn-default",
						title: "Revert"
					}).append($("<i>", {
						'class': "icon icon-WCR_WF-Clear"
					}))
					.click(function () {
						storeConfiguration(configurationID, defaultConfiguration);
						initDashboard(element, configurationID, defaultConfiguration, isConfigurable, openInEditMode);
					}))
				.append($("<a>", {
						'class': "btn btn-default",
						title: "Toggle Builder Mode"
					}).append($("<i>", {
						'class': "icon icon-WCR_Actions-AE12"
					}))
					.click(function () {
						context.toggleEditMode(!context.isEditModeActive());
					}));


			var undoRedoBar = initUndoRedo(context).addClass("add-spacing-right");


			context.registerAndLoadBlock(configuration).then(function (dashboard) {
				element[0].__DASHBOARD = dashboard;
				element.html("");
				isDashboardLoaded = true;

				if (isConfigurable) {
					// add buttons for save/revert
					var configBarSaveBtn = $("<a>", {
							'class': "btn btn-primary",
							title: "Save"
						}).append($("<i>", {
							'class': "icon icon-WCR_Actions-Save12"
						}))
						.click(function () {
							dashboard.getConfiguration(true).then(function (config) {
								storeConfiguration(configurationID, config);
								context.getUndoRedoController().clear();
							});
						});
					var updateSaveButton = function () {
						configBarSaveBtn.toggleClass("disabled", context.getUndoRedoController().undoCount() <= 0);
					};
					context.getUndoRedoController().on("changed", updateSaveButton);
					updateSaveButton();

					configBar
						.prepend($("<a>", {
								'class': "btn btn-default",
								title: "JSON dialog"
							}).append($("<i>", {
								'class': "icon icon-WCR_WF-Paste12"
							}))
							.click(function () {
								dashboard.getConfiguration(true).then(function (configJSON) {
									require(["component/modal/Modal"], function (Modal) {
										var modal = new Modal();

										modal.open();
										var textarea = $("<textarea>", {
											style: "min-height: 80vh;",
											'class': "form-control"
										}).val(JSON.stringify(configJSON, null, 2));
										modal.setContent(textarea);
										var footer = $("<div>");


										$("<a>", {
												'class': "btn btn-default",
												text: "   Revert",
												title: "Revert"
											}).prepend($("<i>", {
												'class': "icon icon-WCR_Revert12"
											})).appendTo(footer)
											.click(function () {
												textarea.val(JSON.stringify(configJSON, null, 2)).trigger("keyup");
											});
										$("<a>", {
												'class': "btn btn-default",
												text: "Cancel",
												title: "Cancel"
											}).appendTo(footer)
											.click(function () {
												modal.close();
											});


										$("<a>", {
												'class': "btn btn-primary",
												text: "  Save",
												title: "Save"
											}).prepend($("<i>", {
												'class': "icon icon-WCR_Actions-Save12"
											})).appendTo(footer)
											.click(function () {
												try {
													var newConfig = JSON.parse(textarea.val().trim());
													if (newConfig && newConfig.type && newConfig.config) {
														storeConfiguration(configurationID, newConfig);
														initDashboard(element, configurationID, defaultConfiguration, isConfigurable, openInEditMode);
														modal.close();
														return;
													} else {
														alert("Config should be valid JSON and should have at least a type and config property");
													}
												} catch (e) {
													console.error(e);
													alert("Invalid JSON: " + e.message);
												}
											});


										var JSONLabel = $("<span>", {
											'class': "label pull-left"
										});

										textarea.on("keyup", _.debounce(function () {
											var errorMessage = "";
											try {
												var newConfig = JSON.parse(textarea.val().trim());
												if (newConfig && newConfig.type && newConfig.config) {
													JSONLabel.toggleClass("label-success", true);
													JSONLabel.toggleClass("label-danger", false);
													JSONLabel.text("valid JSON");

													return;
												} else {
													errorMessage = "need type and config properties";
												}

											} catch (e) {
												// ignore	
												errorMessage = e.message;
											}

											JSONLabel.toggleClass("label-danger", true);
											JSONLabel.toggleClass("label-success", false);
											JSONLabel.text("invalid JSON: ");
											JSONLabel.append($("<small>").text(errorMessage));
											return;
										}, 200));

										textarea.trigger("keyup");

										footer.append(JSONLabel);
										modal.setFooter(footer);
									});
								});
							}))
						.prepend(configBarSaveBtn);


					element.append($("<div>", {
						'class': "clearfix add-spacing-bottom"
					}).append(configBar).append(undoRedoBar));
				}

				element.append(dashboard.getElement());


			}).catch(function (error) {
				element.empty();

				if (isConfigurable) {
					// add buttons for save/revert				
					element.append($("<div>", {
						'class': "clearfix add-spacing-bottom"
					}).append(configBar));
				}


				var errorMessage = $("<p>", {
					'class': "alert alert-danger",
					text: "Error Initializing Block."
				}).appendTo(element);

				if (error.message) {
					var details = $("<small>", {
						text: "(Details: " + error.message + ")"
					});
					errorMessage.append($("<br>")).append(details);
				}
				throw error;
			});

		}


		return {
			wcrinit: function (element) {

				var isConfigurable = $(element).attr("data-configurable") == "1";
				var openInEditMode = $(element).attr("data-open-in-edit-mode") == "1";
				var configurationID = $(element).attr("data-configuration-id");

				var blockDefinitions = {};
				var configElement = element.querySelector("script[data-config]");
				var defaultConfig;
				if (configElement) {
					try {
						var parsedConfig = JSON.parse(configElement.textContent);
						defaultConfig = parsedConfig;
						blockDefinitions = parsedConfig.blockDefinitionDataSource;
						delete defaultConfig.blockDefinitionDataSource;
					} catch (e) {
						$(element).addClass("alert alert-danger")
							.text("Error reading config \r\n" +
								e.message + "\r\n\r\n" +
								e.stack);

						return;
					}
				} else {
					$(element).addClass("alert alert-danger")
						.text("Error reading config \r\n" +
							"Please define a <script data-config=''></script> tag");
				}


				initDashboard($(element), configurationID, defaultConfig, isConfigurable, openInEditMode);

			}
		};
	});