define(["jquery",
        "utils/Popup",
        "component/documentselector/DocumentSelector-view",     
        "utils/webi18n!DocumentSelector"], function ($, Popup, DocumentSelectorView, webI18n) {
    
    var DocumentSelector = function (projectId, folderId, documentId, documentVersionID, documentRevisionCode) {
    	
    	//callbacks
    	this._onDocumentSelectedCallbacks = [];
    	this._onCancelClickedCallbacks = [];
    	        
        //UI handler
        this._DocumentSelectorView = new DocumentSelectorView(projectId, folderId, documentId, documentVersionID, documentRevisionCode);
        
        //JQuery popup handler 
        this._popUp = new Popup(this._DocumentSelectorView.getJQElement(),{dialogClass: "DocumentSelectorView-popup"});
        
        // Start loading the language strings for this component and define what to do when they are loaded.
        this._popUp.setTitle(webI18n.t("SELECT_DOCUMENT"));
        
        var self = this;
        //register "Select" button callback
        this._DocumentSelectorView.onDocumentSelected(function (projectID, folderID, documentID, documentVersions, documentRevision, targetDocVersionID, documentName, docVerRevName) {
        	//close the popup
        	self._popUp.close();
        	
        	//and trigger the registered callbacks
    		for (var i=0; i < self._onDocumentSelectedCallbacks.length; i++) {
    			self._onDocumentSelectedCallbacks[i](projectID, folderID, documentID, documentVersions, documentRevision, targetDocVersionID, documentName, docVerRevName);
    		}    	
        });
        
        //register "Cancel" button callback
        this._DocumentSelectorView.onCancelClicked(function () {
        	//just close the popup
        	self._popUp.close();
        	
        	//and trigger the registered callbacks
    		for (var i=0; i < self._onCancelClickedCallbacks.length; i++) {
    			self._onCancelClickedCallbacks[i]();
    		} 
        });
        
    };
   
    /**
     * Register call back functions for "Document Selected - submit form" event
     * 
     *  @param callBack
     */
    DocumentSelector.prototype.onDocumentSelected = function(callBack) {
    	this._onDocumentSelectedCallbacks.push(callBack);
    };
    
    /**
     * Register call back functions for "Cancel button clicked" event
     * 
     * @param callBack
     */
    DocumentSelector.prototype.onCancelClicked = function(callBack) {
    	this._onCancelClickedCallbacks.push(callBack);
    };
    
    /**
     * Initialization of the UI when the Selector Popup gets opened 
     */
    DocumentSelector.prototype.openDocumentSelector = function() {
    	this._popUp.open();
    };
    
    return DocumentSelector;
});