define(["underscore", "component/dropzone/Dropzone"], function (_, Dropzone) {

	"use strict";

	/**
	 * A dropzone that accepts files and emits the "filesDropped" event if files are dropped
	 * @class
	 * @export FileDropzone
	 * @augments Dropzone
	 * @param {Object} configuration - {[element]: <JQueryObject>}
	 */
	var FileDropzone = function (configuration) {
		Dropzone.call(this, configuration);
	};

	FileDropzone.prototype = Object.create(Dropzone.prototype);
	FileDropzone.prototype.constructor = FileDropzone;

	/** 
	 * @override
	 */
	FileDropzone.prototype._validateDataTransfer = function (event) {
		return _.contains(event.dataTransfer.types, "Files");
	};

	/** 
	 * @override
	 */
	FileDropzone.prototype._handleDrop = function (event) {
		if(this._validateDataTransfer(event)) {
			this._emit("filesDropped", event.dataTransfer.files);
		}
		event.preventDefault();
		event.stopPropagation();
	};


	return FileDropzone;
});
