define(["utils/Observable",
        "jquery",
        "underscore",
        "utils/webcss!./fileinput.css"],
	function (Observable,
			  $,
			  _) {
		"use strict";

		/**
		 * This contains a multi-select input.
		 * It will send an event when files are added and the input gets cleared.
		 * Make sure the files are stored where needed, as the input gets cleared after each change. 
		 * 
		 * @exports MultiFileInput
		 * @mixes Observable 
		 * @constructor
		 */
		var MultiFileInput = function(configuration, i18n) {
			Observable.mixin(this, "MultiFileInput");
			
			configuration = configuration || {};
			this._ctx =  (configuration.element || $("<div/>")).addClass("MultiFileInput");

			this._refreshInput();
			
			this._button = $("<button />").addClass("btn btn-primary")
				.append($("<i />").addClass("icon icon-normal icon-WCR_Actions-Upload12"))
				.append(" " + i18n.t("SELECT_FILES"));
			this._button.click(_.bind(function(){
				if(this._input){
					this._input.click();
				}
			}, this));
			this._ctx.append(this._button);
		};
		
		MultiFileInput.prototype.getElement = function(){
			return this._ctx;
		};
		
		MultiFileInput.prototype._refreshInput = function(){
			if(this._input){
				this._input.remove();
			}
			this._input = $('<input type="file" multiple />');
			this._ctx.append(this._input);
			
			this._input.on("change", _.bind(function(){
				this._emit("change", this._input[0].files);
				this._refreshInput();
			}, this));
		};

		return MultiFileInput;
	});