/**
 * KeyWordSearch 
 */

define(["utils/webi18n!KeyWordSearch", 
		"jquery",
		"./keywordsearch-view"],
function(i18n, 
		  $,
		  KeyWordSearchView) {
	
	"use strict";

	
	/**
	 * KeyWordSearch component for the menu bar
	 * @class
	 * @constructor
	 * @param {JQueryObj} element - container of the keyword search
	 * @param {Object} options - options for the KeyWordSearch: 
	 */
	var KeyWordSearch = function(element, options) {
		
		// Create the view on the element.
		this._view = new KeyWordSearchView(element);
		
		// Make a self reference for later use in callbacks.
		var self = this;
		
		// Static variables of this keyWordSearch Elements
		this._searchTypes = {"SEARCHTYPE_PROJECTS": {
			id: "SEARCHTYPE_PROJECTS",
			icon: "icon icon-WCR_Menu-Search-Projects12",
			action:  "doprojsearchactions.jsp",
			searchtypelistValue: "project",
			name: i18n.t("SEARCHTYPE_PROJECTS"),
			placeholder: i18n.t("SEARCHTYPE_PROJECTS_PLACEHOLDER")
		}, "SEARCHTYPE_DOCUMENTS": {
			id: "SEARCHTYPE_DOCUMENTS", 
			icon: "icon icon-WCR_Menu-Search-Documents12",
			action: "doalldocsearchactions.jsp",
			searchtypelistValue: "document",
			name: i18n.t("SEARCHTYPE_DOCUMENTS"),
			placeholder: i18n.t("SEARCHTYPE_DOCUMENTS_PLACEHOLDER")
		}, "SEARCHTYPE_SAVED_SEARCH": {
			id: "SEARCHTYPE_SAVED_SEARCH",
			icon: "icon icon-WCR_Menu-Search-SavedSearch12",
			action: "domysavedsearchesactions.jsp",
			searchtypelistValue: "savedsearch",
			name: i18n.t("SEARCHTYPE_SAVED_SEARCH"),
			placeholder: i18n.t("SEARCHTYPE_SAVED_SEARCH_PLACEHOLDER")
		}};
		
		
		this._defaultSearchType = "SEARCHTYPE_PROJECTS";
		
		// Check if a defaultSearchType was specified.
		if (options.defaultSearchType) {
			this._defaultSearchType = options.defaultSearchType;
		}
		
		this._savedSearches = [];
		if (options.savedSearches){
			this._savedSearches = options.savedSearches;
		}
		
		if (options.selectedSavedSearch){
			this._selectedSavedSearch = options.selectedSavedSearch;
		}
		
		
		// Add all the supported searchTypes
		this._view.setSearchTypes(this._searchTypes);
		
		// Set the default search type
		self.setSearchType(this._defaultSearchType);
						
		this._view.on("clicked.search", function(){
			self._view.submitSearch();	
		});
		
		// When a search type is changed ...
		this._view.on("changed.searchType", function (searchType) {			
			self.setSearchType(searchType, true);
		});
		
		this._view.on("changed.savedSearchSelection", function(savedSearchID){
			if (savedSearchID != ""){
				self._view.submitSearch();
			}
		});
	};
		
		
	/**
	 * Change the current Search Type
	 * @param {searchType} searchType - the new search type
	 * @param {Boolean} byUser - was the search type changed by the user or not?
	 */
	KeyWordSearch.prototype.setSearchType = function(searchType, byUser){
		var newSearchType = this._searchTypes[searchType];
		if (newSearchType){
			this._activeSearchType = newSearchType;
			this._view.setActiveSearchType(this._activeSearchType);
			
			if (searchType === "SEARCHTYPE_SAVED_SEARCH"){
				this._view.toggleDropdown(true, this._savedSearches, byUser, this._selectedSavedSearch); 	
			} else {
				this._view.toggleDropdown(false, null, byUser);
			}
		}		
		
	};
	
    /**
     * Get a jQuery wrapped reference to the encapsulating div of this component.
     * 
     * @returns {jQuery} The jQuery wrapped reference to the encapsulating div of this component.
     */
	KeyWordSearch.prototype.getJQElement = function () {
        return this._view.getJQElement();
    };
   
    /**
     * Method to automatically apply this module to an input or select element ...
     */
    KeyWordSearch.wcrinit = function(element){
        new KeyWordSearch(element);
    };
    
    /**
     * Method to automatically destroy this component once initialized for a div...
     * 
     * @param div {DomElement} the DomElement to destroy again
     */
    KeyWordSearch.wcrdestroy = function(element){
		// not supported yet
    };
	
	return KeyWordSearch;

});