define(["jquery", "underscore", "utils/webcss!./OverlayMessage.css", "utils/webi18n!overlayMessage"], function($, _, overlayMessageCSS, i18n){
	"use strict";
	
	/**
	 * Component to show a overlay message over a certain html element (target).
	 * The component gets blurred and faded out. The message id displayed in the center
	 * 
	 * Note: The component will temporarily add position:relative to the target
	 * @constructor
	 * @class
	 * @param {JQueryObj} target - element on which the overlay should be shown 
	 * @param {JQueryObj|String} [message] - the message (or overlay HTML) that should be shown. If true, the overlay is added immediataly
	 **/
	var OverlayMesage = function OverlayMessage(target, message){
		this._target = target;
		this._messageWrapper = null;
		
		if (message){
			this.show(message);	
		}		
	};
	
	/**
	 * Show the overlay message
	 * @param {JQueryObj|String} message - the message (or overlay HTML) that should be shown.
	 */
	OverlayMesage.prototype.show = function(message){
		this._target.addClass("overlay-message-target");
		this._blurred = this._target.children().addClass("overlay-message-blur");
		this._messageWrapper = $("<div>", {'class': "overlay-message"});
		
		if (_.isString(message)){
			message = $("<div>", {text: message});	
		} 
		message.addClass("message");
		this._messageWrapper.append(message);
		this._target.append(this._messageWrapper);	
	};
	
	
	/**
	 * Show a loading message
	 */
	OverlayMesage.prototype.showLoading = function(){
		var loader = $("<div>", {text: " "+i18n.t("LOADING")});
		loader.prepend($("<img src='/wcr-static/style/images/loaderTransparent.gif'/>", {alt: i18n.t("LOADING")}));	
				
		this.show(loader);		
	};
	
	/**
	 * Hide the overlay message (this cleans the DOM completely)
	 */
	OverlayMesage.prototype.hide = function(){
		if (this._messageWrapper){
			this._messageWrapper.detach();
			this._messageWrapper = null;
		}
		
		if (this._blurred){
			this._blurred.removeClass("overlay-message-blur");
			this._blurred = null;
		}
		this._target.removeClass("overlay-message-target");
	};
	
	return OverlayMesage;
});