define(
		[
			"component/smartnames/savedsearchcontext/SavedSearchContextInfo",
			"component/savedsearchdialog/SavedSearchDialog",
			"component/pagemessagecomponent/PageMessageComponent",
			"utils/webi18n!SavedSearchDialog",
			"jquery"
        ],function(SavedSearchContextInfo,SavedSearchDialog,PageMessageComponent,i18n,$){
	
			
	"use strict";	
	
	
			
	/**
	 * Shows a project saved search dialog 
	 * @param {JQueryObj} context 
	 *                            must contain projectID and savedSearchName attributes
	 *                            optional attribute: LinkInExisting, if this attribute is provided, 
	 *                     								the component will add the 'Link' button 
	 */
	var ProjectSavedSearchDialog = function(context){
		SavedSearchDialog.call(this,context);
		
		/**
		* @member {SavedSearchContextInfo} _savedSearchContext
		* @private    
		*/
		this._savedSearchContext = new SavedSearchContextInfo();
		this._savedSearchContext.setProjectId(this._currentProjectID);
		
		/**
		* @member {String} _dialogTitle
		* @private    
		*/
		this._dialogTitle = i18n.t("SEARCH_PROJECT");
		
		/**
		*the message that should be shown when selecting a project results in a change (e.g. adding a new child)
		* @member {String} _newProjectMessage
		* @private    
		*/
		this._newProjectMessage= "";
		/**
		*The message that should be shown when none change occurs when selecting a project (e.g. adding an already existing child)
		* @member {String} _childAlreadyExisted
		* @private    
		*/
		this._childAlreadyExisted="";
		
		
		//Provide the attribute LinkInExisting to make sure that the Link option is visible 
		if(context.getAttribute("LinkInExisting")){
			var self = this;
    		this._buttonObj[i18n.t("LINK")] = function(ctx){
				var selection = $("[name='selectproject']:checked");

				var url = window.webcenterConfig.jspBaseUrl + 'UpdateProjectRelation.jsp?projectid='+self._currentProjectID+'&action=add';


				if(selection.size() === 0){
					window.alert(self._noProjectsSelected);
				}else{
					for(var i = 0; i < selection.size(); i++){

						var projectID = selection[i].getAttribute("projectid");    		
						url+= "&childprojectid="+projectID;

					}

					$.ajax({
						url: url,				
						type: "POST",
						data: "text",
						contentType: "application/x-www-form-urlencoded",
						synchronous: false,
						timeout: 30000,
						success: function(data,textStatus,jqXHR){
							self.onProjectLinkSuccess(data);	
						},

						error: function(xhr,ajaxOptions,thrownError){
							//should never be reached
							window.console.error("An error occurred when trying to link a child project");
						},					
							objHdl: this
						});
				}

			};
		}
		
	};
			
	ProjectSavedSearchDialog.prototype.constructor = ProjectSavedSearchDialog;
	ProjectSavedSearchDialog.prototype = Object.create(SavedSearchDialog.prototype);	
	
	/**
	* Method to automatically apply this module, for info about context see constructor
    */
	ProjectSavedSearchDialog.wcrinit = function(context){
		context.ProjectSavedSearchDialog = new ProjectSavedSearchDialog(context);
	};
			
	/**
	 * Processes the results of a successfull ajax call
	 * @param {XML} xml The xml returned by the server (the structure can be found in the SDK call updateProjectRelation)
	 */
	ProjectSavedSearchDialog.prototype.onProjectLinkSuccess = function(xml){
		xml = $(xml);
		
		if(xml.find("error").length > 0 ){
			PageMessageComponent.markObsolete("programManagementError");
			PageMessageComponent.sendErrorMessage(xml.find("message").text(),undefined, "programManagementError");
		}else{
			
			var childrenNames = xml.find("project_new_relations").find("child_project_list").find("name");
		
			var message; 
			if(childrenNames.length !== 0){
				message = this._newChildMessage;// "New child(ren) added ";
				for(var i = 0 ; i < childrenNames.length;i++){
					message += ' "' + $(childrenNames[i]).text() + '"';
				}
			}else{
				message = this._childAlreadyExisted; //"All selected projects were already child projects of the current project";
			}

			
			PageMessageComponent.markObsolete("programManagementError");
			PageMessageComponent.sendSuccessMessage(message,undefined, true);

			
		}
		
		// Close the saved search modal dialog
		this.popup.close();
	};
	/**
	 * Set the message that has to be shown when new Child Project(s) are added
	 * @param {String} message The message to show
	 */
	ProjectSavedSearchDialog.prototype.setNewChildrenAddedMessage = function(message) {
		this._newChildMessage = message;
	};
	/**
	 * Get the message that has to be shown when new Child Project(s) are added
	 * @returns {String}
	 */
	ProjectSavedSearchDialog.prototype.getNewChildrenMessage = function(){
		return this._newChildMessage;
	};
	
	/**
	 * Set the message that has to be shown when all Projects were already Child Project(s)
	 * @param {String} message The message to show
	 */
	ProjectSavedSearchDialog.prototype.setChildrenAlreadyExistedMessage = function(message) {
		this._childAlreadyExisted = message;
	};
	/**
	 * Get the message that has to be shown when all Projects were already Child Project(s)
	 * @returns {String} 
	 */
	ProjectSavedSearchDialog.prototype.getChildrenAlreadyExistedMessage = function() {
		return this._childAlreadyExisted;
	};
	/**
	 * Sets the message that will be shown in an alert popup when no projects are selected
	 * @param {String} message The message to show
	 */
	ProjectSavedSearchDialog.prototype.setNoProjectsSelectedMessage = function(message){
		this._noProjectsSelected = message;
	};
	/**
	 * Get the message that has to be shown in an alert popup when no projects are selected
	 * @returns {String} 
	 */
	ProjectSavedSearchDialog.prototype.getNoProjectsSelectedMessage = function(message){
		return this._noProjectsSelected;
	};
	
	
	return ProjectSavedSearchDialog;
	
});