define(["./AbstractFacetBlock",
	"./SavedSearchCounterBlockStyles",
	"dashboard/core/parameters/StringParameter",
	"underscore",
	"jquery",
	"utils/webi18n!SavedSearchCounterBlock"
], function (
	AbstractFacetBlock,
	SavedSearchCounterBlockStyles,
	StringParameter,
	_,
	$,
	i18n) {
	"use strict";

	/**
	 * Block for displaying a facet graph
	 * 
	 * @constructor
	 * @augments AbstractSearchBlock
	 * @param {String} id
	 * @param {Context} context
	 */
	var FacetCounterBarBlock = function FacetCounterBarBlock(id, context) {
		AbstractFacetBlock.call(this, id, context, "facet_counter_bar");
		
		this._showTransparentFrame = false;
	};

	FacetCounterBarBlock.prototype = Object.create(AbstractFacetBlock.prototype);
	FacetCounterBarBlock.prototype.constructor = FacetCounterBarBlock;




	/**
	 * Get the type of the block
	 * @returns {string} [[Description]]
	 */
	FacetCounterBarBlock.prototype.getType = function () {
		return "component/searchresults/dashboard/blocks/FacetCounterBarBlock";
	};

	FacetCounterBarBlock.prototype._initializeDefaultParameters = function (config) {
		return AbstractFacetBlock.prototype._initializeDefaultParameters.call(this, config).then(function () {
			var facetParameter = this._parameters.getParameter("FacetParameter");
			// style
			var styleParameter = new StringParameter("counterStyle", this._context, i18n.t("STYLE"));
			styleParameter.setRestrictedSet(SavedSearchCounterBlockStyles);
			styleParameter.setValue("counter-default"); // default style
			facetParameter.addParameter(styleParameter);
		}.bind(this));
	};


	/**
	 * @override
	 */
	FacetCounterBarBlock.prototype._doSearchRender = function (searchParameter) {
		if (searchParameter.getFieldLayoutAPI()) {
			AbstractFacetBlock.prototype._doSearchRender.call(this, searchParameter);
		} else {
			// get the parameter value
			var label = this._i18n.t("FACET_COUNTER_BAR");

			var value = this._parameters.getParameter("FacetParameter").getParameterValue("facetids");
			if (value && value[0]) {
				var facetedfield = value[0].getValue("label", "");
				if (facetedfield) {
					label = label + " " + "\"" + facetedfield + "\"";
				}
			}

			this._showPlaceholder("icon-WCR_Chart-Counter", label);
		}
	};

	return FacetCounterBarBlock;

});