define(["./AbstractFacetBlock",
	"c3WithStyling",
	"underscore",
	"jquery",
	"dashboard/core/parameters/StringParameter",
	"dashboard/core/parameters/CssSizeParameter"
], function (
	AbstractFacetBlock,
	c3,
	_,
	$,
	StringParameter,
	CssSizeParameter) {
	"use strict";

	/**
	 * Block for displaying a facet graph
	 * 
	 * @constructor
	 * @augments AbstractSearchBlock
	 * @param {String} id
	 * @param {Context} context
	 */
	var FacetGraphBlock = function FacetGraphBlock(id, context) {
		AbstractFacetBlock.call(this, id, context, "graph");
	};

	FacetGraphBlock.prototype = Object.create(AbstractFacetBlock.prototype);
	FacetGraphBlock.prototype.constructor = FacetGraphBlock;
	


	/**
	 * Get the type of the block
	 * @returns {string} [[Description]]
	 */
	FacetGraphBlock.prototype.getType = function () {
		return "component/searchresults/dashboard/blocks/FacetGraphBlock";
	};
	
	/**
	 * Define the chart types. Also call _initializeDefaultParameters from AbstractFacetBlock.
	 * @private
	 * @override
	 */
	FacetGraphBlock.prototype._initializeDefaultParameters = function(config){
		return AbstractFacetBlock.prototype._initializeDefaultParameters.call(this, config).then(function () {
            var facetParameter = this._parameters.getParameter("FacetParameter");

            this._context.getLanguageBundle("SearchDashboard").then(function(bundle){
                this._i18n = bundle;
                // chart
                var chartParameter = new StringParameter("chartType", this._context, this._i18n.t("CHART_TYPE"));
                chartParameter.setRestrictedSet({
                    "pie": this._i18n.t("PIE_CHART"),
                    "donut": this._i18n.t("DONUT_CHART"),
                    "bar": this._i18n.t("BAR_CHART"),
                    "hbar": this._i18n.t("HBAR_CHART")
                });
                chartParameter.setValue("pie"); // default chart type
                facetParameter.addParameter(chartParameter);
                
                // Maximum width option
                var graphMaxWidth = new CssSizeParameter("graphWidth", this._context, this._i18n.t("GRAPH_WIDTH"), ["px"]);
                facetParameter.addParameter(graphMaxWidth);
                
                // Maximum height option
                var graphMaxHeight = new CssSizeParameter("graphHeight", this._context, this._i18n.t("GRAPH_HEIGHT"),["px"]);
                facetParameter.addParameter(graphMaxHeight);
                
            }.bind(this));
        }.bind(this));
	};

	/**
	 * if faceted field is of date type, switch to bar chart :)
	 * 
	 * @override
	 * @private
	 */
	FacetGraphBlock.prototype._onFacetIdsChanged = function(){
		AbstractFacetBlock.prototype._onFacetIdsChanged.apply(this, arguments);
		
		var facetParameter = this._parameters.getParameter("FacetParameter");
		var facetids =facetParameter.getParameter("facetids");
		
		var chartType = facetParameter.getParameter("chartType");
		var isDateTimeType = (facetids.getFacetedFieldType() === "DATE");
		
		if (isDateTimeType){			
			this._chartTypeBeforeDateTime = chartType.getValue();
			chartType.setValue("bar");
		} else if (this._chartTypeBeforeDateTime) {
			chartType.setValue(this._chartTypeBeforeDateTime);
			this._chartTypeBeforeDateTime = null;
		}
			
		
		chartType.setVisible(!isDateTimeType);
	};

	/**
	 * @override
	 */
	FacetGraphBlock.prototype._doSearchRender = function (searchParameter) {
		if (searchParameter.getFieldLayoutAPI()) {
			AbstractFacetBlock.prototype._doSearchRender.call(this, searchParameter);
		} else {
			// get the parameter value
			var label = this._i18n.t("FACET_GRAPH");
			
			var value = this._parameters.getParameter("FacetParameter").getParameterValue("facetids");
			if(value && value[0]) {
				var facetedfield = value[0].getValue("label", "");
				if (facetedfield){
					label = label + " " + "\"" + facetedfield + "\"";
				}
			}
						
			var icon;
			switch(this._parameters.getParameter("FacetParameter").getParameter("chartType").getValue()){
				
				case 'donut':
					icon ="icon-WCR_Chart-Donut";
					break;
				case 'bar':
					icon ="icon-WCR_Chart-Bar";
					break;
				case 'hbar':
					icon = "icon-WCR_Chart-BarH";
					break;
				case 'pie': 
				default:
					icon = "icon-WCR_Chart-Pie";
					
			}
			this._showPlaceholder(icon, label);
		}
	};

	return FacetGraphBlock;

});