define(["./AbstractSearchBlock",
	"underscore",
	"jquery"
], function (
	AbstractSearchBlock,
	_,
	$) {
	"use strict";

	/**
	 * Block for display of the complete Search Design Panel
	 * 
	 * @constructor
	 * @augments AbstractSearchBlock
	 * @param {String} id
	 * @param {Context} context
	 */
	var SearchParameterPanelBlock = function SearchParameterPanelBlock(id, context) {
		AbstractSearchBlock.call(this, id, context);
	};

	SearchParameterPanelBlock.prototype = Object.create(AbstractSearchBlock.prototype);
	SearchParameterPanelBlock.prototype.constructor = SearchParameterPanelBlock;

	/**
	 * Get the type of the block
	 * @returns {string} [[Description]]
	 */
	SearchParameterPanelBlock.prototype.getType = function () {
		return "component/searchresults/dashboard/blocks/SearchParameterPanelBlock";
	};


	/**
	 * Display the Search Design Panel (if available)
	 * @private
	 * @param {SearchContextParameter} 
	 */
	SearchParameterPanelBlock.prototype._doSearchRender = function (searchParameter) {
		if (searchParameter.getSearchDesignPanel()) {
			this._element.html("").append( searchParameter.getSearchDesignPanel().getElement());
		} else {
			// FUTURE in builder mode we probably want to show a nice preview
			this._showPlaceholder("icon-WCR_Search-Filter", this._i18n.t("DESIGN_PANEL_BLOCK"));
		}
	};

	return SearchParameterPanelBlock;

});