define(["webcenterapi/data/WebCenterSavedSearch",
		"dashboard/core/parameters/Parameter",
		"utils/Promisify"
	],
	function (WebCenterSavedSearch,
		Parameter,
		Promisify) {
		"use strict";

		/**
		 * SavedSearch Parameter
		 * @augments Parameter
		 * @param {String} name             - name of the parameter
		 * @param {String} userFriendlyName - user friendly representation of the parameter
		 */
		var WCRSavedSearchParameter = function (name, context, userFriendlyName) {
			Parameter.call(this, name, context, "component/searchresults/dashboard/parameters/WCRSavedSearchParameter", userFriendlyName);


		};

		WCRSavedSearchParameter.prototype = Object.create(Parameter.prototype);
		WCRSavedSearchParameter.prototype.constructor = WCRSavedSearchParameter;

		/**
		 * Create the SavedSearch Object
		 * @param {object} config 
		 */
		WCRSavedSearchParameter.prototype.configure = function (config) {
			if (config.value) {
				this.setValue(WebCenterSavedSearch.fromJSONObject(config.value));
			}else{
                this.setValue(null);
            }
		};

		/**
		 * @override
		 * serialize the WebCenterSavedSearch object
		 * @private
		 * @returns {object} [[Description]]
		 */
		WCRSavedSearchParameter.prototype._getParameterSpecificConfig = function () {
			var value = null;
			if (this.getValue() && this.getValue().toJSONObject) {
				value = this.getValue().toJSONObject();
			}

			return {
				value: value
			};
		};


		/**
		 * Get the configuration element for the parameter
		 * @returns {Promise.<ConfigurationPanel>} 
		 */
		WCRSavedSearchParameter.prototype.getConfigurationPanel = function () {
			return Promisify.require("component/searchresults/dashboard/parameters/WCRSavedSearchParameterConfiguration").then(function (ParameterConfigurationPanel) {
				return new ParameterConfigurationPanel(this);
			}.bind(this));
		};

		return WCRSavedSearchParameter;
	});