define(["dashboard/builder/configuration/ParameterConfigurationPanel",
		"webcenterapi/SavedSearchAPI",
		"component/sdrop/dataproviders/MapPromiseDataProvider",
		"component/sdrop/SDrop",
		"jquery",
		"underscore",
		"utils/Map",
		"utils/webi18n!SavedSearchCounterBlock"
	],
	function (ParameterConfigurationPanel,
		SavedSearchAPI,
		MapPromiseDataProvider,
		SDrop,
		$,
		_,
		Map,
		counterBlockI18n) {

		"use strict";

		// we want a shared saved search api
		var savedSearchAPI = new SavedSearchAPI(window.webcenterConfig);

		/**
		 * Configuration panel for a HTML parameter
		 * @augments ParameterConfigurationPanel
		 * @param {SavedSearchParameter} parameter 
		 */
		var WCRSavedSearchParameterConfiguration = function (parameter) {
			ParameterConfigurationPanel.call(this, parameter);
		};

		WCRSavedSearchParameterConfiguration.prototype = Object.create(ParameterConfigurationPanel.prototype);
		WCRSavedSearchParameterConfiguration.prototype.constructor = WCRSavedSearchParameterConfiguration;


		/**
		 * Initialize an sdrop for displaying the searches
		 * @private
		 */
		WCRSavedSearchParameterConfiguration.prototype._init = function () {
			this._element.addClass("form-group");
			this._$label = $("<label>", {
				text: this._parameter.getUserFriendlyName()
			});

			this._dropdown = $("<select>").attr("data-param-name", this._parameter.getName());

			var promise = savedSearchAPI.getSavedSearches().then(function (searchmap) {
				var filteredMap = new Map();
				_.each(searchmap.values(), function (search) {
					if (search.isGlobal()) {
						filteredMap.add(search.getName(), search);
					}
				});

				this._searches = filteredMap;
				return filteredMap;
			}.bind(this));

			this._dataProvider = new MapPromiseDataProvider({
				promise: promise,
				entityLabel: counterBlockI18n.t("SAVED_SEARCH")
			});

			this._element.append(this._$label).append($("<div>", {
				'class': "form-entry"
			}).append(this._dropdown));

			this._initSDrop();

			this._element.append(this._dropdown);
		};
    
        /**
         * Helper method to initialize the SDrop with the current value of the parameter
         * @private
         */
        WCRSavedSearchParameterConfiguration.prototype._initSDrop = function(){
            this._stopObserveAll();
            
            var sdropOptions = {
				manageStyleAndWidth: false
			};

			if (this._parameter.getValue()) {
				sdropOptions.initialValue = this._parameter.getValue().getName();
			}else{
                sdropOptions.initialValue = "";
            }
            
            var _sdrop = new SDrop(this._dropdown, "", this._dataProvider, sdropOptions);
            
            _sdrop.addEvent("changedByUser", function(){
                this._parameter.undoableSetValue(this._searches.get(this._dropdown.val()));
            }.bind(this));
            
            this._observeParam(function(){
                _sdrop.destroy();
                this._initSDrop();
            }.bind(this));
        }

		/**
		 * Get the configured search
		 * @override
		 * @returns {Object} 
		 */
		WCRSavedSearchParameterConfiguration.prototype.getConfiguration = function () {
			if (this._searches) {
				return ParameterConfigurationPanel.prototype.getConfiguration.call(this);
			} else {
				return this._parameter.getConfiguration();
			}
		};

		/**
		 * @override
		 * @private
		 * @returns {object} 
		 */
		WCRSavedSearchParameterConfiguration.prototype._getCustomConfiguration = function () {
			var search = this._searches.get(this._dropdown.val());
			var value = null;

			if (search) {
				value = search.toJSONObject();
			}

			return {
				value: value
			};
		};


		return WCRSavedSearchParameterConfiguration;
	});