/**
 * The DataProvider allows a Field in the Design Panel to access data 
 * for the field which is passed from the back-end.
 * 
 * See (serverside) SearchInformationProvider.getData & SearchInformationProvider.getFieldGlobalDataID & SearchInformationProvider.getFields.
 */
define([], function() {
	
	/**
	 * @constructor
	 * 
	 * @param {Object} localToGlobalIdMap map of local to global data id's for this field
	 * @param {Object} sharedData a reference to the sharedData
	 */
	var DataProvider = function(localToGlobalIdMap, sharedData){
		this._localToGlobalIdMap = localToGlobalIdMap;
		this._sharedData = sharedData;
	};
	
	/**
	 * Get some data for this field...
	 * 
	 * @param {String} id the field specific id of the data (not unique over all fields)
	 * @returns {ValueData} the information (or null if there is none for that id)
	 */
	DataProvider.prototype.getFieldSpecificData = function(id) {
		var globalid = this._localToGlobalIdMap[id];
		if(!globalid) {
			return null;
		}else{
			return this.getGlobalData(globalid);
		}
	};
	
	/**
	 * Get some global data...
	 * 
	 * E.g. the list of Project Statuses or ... ?
	 * 
	 * @param {String} globalid the global id of the data we want
	 * @returns {ValueData} a ValueData object containing this information
	 */
	DataProvider.prototype.getGlobalData = function(globalid) {
		var data = this._sharedData[globalid];
		if(data === null || data === undefined) {
			return null;
		}else if(data instanceof ValueData){
			return data;
		}
		// Should not happen
		console.log("DataProvider.getGlobalData: This should never happen...");
		return null;
	};
	
	return DataProvider;
});