/**
 * Data holder for external/global WCR settings + search related preferences
 */
define([], function(){
	
	/**
	 * @constructor
	 */
	var ExternalSearchSettings = function(preferences){
		/**
		 * JSON data that holds all the preferences supplied
		 */
		this.preferences = preferences;		
	};
	
	//GETTERS for the supported preference types
	/**
	 * Check whether the user is an admin
	 * @returns boolean
	 */
	ExternalSearchSettings.prototype.getIsAdmin = function() {
		return this.preferences["admin"]==="1";
	};
	
	/**
	 * Returns the User's current Units preference (metric VS imperial)
	 * @returns String; if not found will use metric as a fallback
	 */
	ExternalSearchSettings.prototype.getUserUnitsPreference = function() {
		return this.preferences["units"]?this.preferences["units"]:"metric";
	};
	
	/**
	 * Check whether the search results should be shown in "Simple GUI" modus
	 * @returns boolean
	 */
	ExternalSearchSettings.prototype.getIsSimpleGUI = function() {
		return !(this.preferences["simpleGUI"]==="0");
	};
	
	/**
	 * Get value of "Menu File URL" parameter
	 * Used to insert into URLs in order to select the appropriate Menu after following the link
	 * 
	 * @returns String or empty string (if missing)
	 */
	ExternalSearchSettings.prototype.getMenuFileURL = function() {
		//TODO: have to handle "no menu file url property" situation more gracefully - 
		//AND even moreso make sure we get the value in the first place
		return (this.preferences["menu_file_url"]?this.preferences["menu_file_url"]:"myallprojects");
	};
	
	/**
	 * Get value of "Date Attribute Render type" parameter
	 * Used to determine how to render a Date-type Attribute values in the Search Results
	 * 
	 * @returns String
	 */
	ExternalSearchSettings.prototype.getDateAttrRenderType = function() {
		return this.preferences["date_render_type"] === "1"?"date_time":"date"; //fallback set to DATE only format
	};
	
	/**
	 * Check whether "Force Approve" status submission is not disabled (Admin > Security)
	 * @returns boolean
	 */
	ExternalSearchSettings.prototype.getIsForceApprovalAllowed = function() {
		return !(this.preferences["pharma_can_force_approve"]==="0");
	};
	
	/**
	 * Check whether "Force Reject" status submission is not disabled (Admin > Security)
	 * @returns boolean
	 */
	ExternalSearchSettings.prototype.getIsForceRejectionAllowed = function() {
		return !(this.preferences["pharma_can_force_reject"]==="0");
	};
	
	/**
	 * Check whether to invite a group to a project when the group get's assigned to a role. (Admin > General > Roles)
	 * @returns {Boolean}
	 */
	ExternalSearchSettings.prototype.getRoleAssignInviteGroupDefault = function() {
		return !(this.preferences["invite_group_default"]==="0");
	};
	
	/**
	 * Whether this system has a task management license.
	 * @returns {Boolean}
	 */
	ExternalSearchSettings.prototype.getHasTaskManagementLicense = function() {
		return this.preferences["has_task_management_license"]==="1";
	};
	
	/**
	 * Whether BOM is enabled on this system.
	 * @returns {Boolean}
	 */
	ExternalSearchSettings.prototype.getBOMEnabled = function() {
		return this.preferences["bom_enabled"]==="1";
	};
	
	return ExternalSearchSettings;
});
