/**
 * An object containing display settings which apply to the result part only which will NOT change while on the page...
 * 
 * WILL BE PASSED TO A RENDERER
 * 
 * @author JEPE
 */
define([], function(){
	
	/**
	 * @constructor
	 */
	var TableDisplaySettings = function(){
		//
		// Settings
		//
		
		// Are thubmnails shown?
		this.showThumbnails = false;
		
		// Show as Grid?
		this.resultsStyle = "list";
		// Which type of grid? ("narrow" OR "wide")
		this.resultsStyleSubType = "";
		
		// Hyperlinks allowed?
		this.hyperLinksAllowed = true;
		
		//
		// Derived information
		//
		
		// What is the preferred height of the cell?
		this.cellheight = -1;
		
		// Whether to show the units next to the value or not
		this.showUnitsInValue = false;
		
		// Whether clicking on one overflow container will expand the other ones too...
		this.overflowContainersLinked = false;
	};
	
	/**
	 * Update the derived information in this object...
	 * 
	 * Should be called before the object is passed around.
	 */
	TableDisplaySettings.prototype.updateDerivedInfo = function() {
		//
		// How high do we make the cell?
		//
		
		// default height (row with thumbnails)
		this.cellheight = 50;
		if(this.resultsStyle==="list" && !this.showThumbnails) {
			this.cellheight = 30;
		}
		
		//
		// Whether to show the units next to the value or not
		//
		this.showUnitsInValue = (this.resultsStyle==="grid" || this.resultsStyle==="card");
		
		//
		// Whether the overflow containers are linked and will expand/collapse all together
		//
		this.overflowContainersLinked = this.resultsStyle==="list";
	};
	
	// return the constructor
	return TableDisplaySettings;
});