/**
 * "Cape" Display Method
 * 
 * Search for each field for the fields with _1 _2 and _3 and display all of them.
 * 
 * @author JEPE
 */
define(["jquery", "../../Utils"], function($, utils){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			// Get the display function (Used to display text & numbers. For numbers, formats them correctly + adds units)
			var displayFunc = utils.getDisplayFunction(tableDisplaySettings, fieldProperties, searchSettings);
			
			// Note that there should never be multiple items for fields with _1, _2, _3...
			// If there is... just ignore it.
			
			/** @type {ValueData} */
			var data1 = (data.getSubList(fieldname+"_1")||[])[0];
			/** @type {ValueData} */
			var data2 = (data.getSubList(fieldname+"_2")||[])[0];
			/** @type {ValueData} */
			var data3 = (data.getSubList(fieldname+"_3")||[])[0];
			
			//
			// We have two ways to display
			//
			// If there is just data for _1, then just display that data...
			// If there is data for _2 and _3 too, then show as a numbered list...
			// 1. data1
			// 2. data2
			// 3. data3
			if(data2) {
				// Show numbered list...
				//JAMI: add the left margin to prevent cutoff from the left side (on Chrome > see WCR-9892 JIRA log)
				var root = $("<ol>").css("margin-left", "3px");
				if(data1) {
					root.append($("<li>", {"text":displayFunc(data1.getValue("value"))}));
					if(data2) {
						root.append($("<li>", {"text":displayFunc(data2.getValue("value"))}));
						if(data3) {
							root.append($("<li>", {"text":displayFunc(data3.getValue("value"))}));
						}
					}
				}
				$div.append(root);
			}else{
				// Show just the value...
				if(data1) {
					var root = $("<span>", {"text":displayFunc(data1.getValue("value"))});
					$div.append(root);
				}
			}
		}
	};
});