/**
 * "Cut-off" Display Method 
 * 
 * As (for example) a Description (relevant for both Project and Document) value may be too long we support a "smart truncation" mechanism
 * 
 * @author JAMI
 */
define(["jquery", "underscore", "../../Utils"], function($, _, Utils){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			// Get the display function (Used to display text & numbers. For numbers, formats them correctly + adds units)
			var displayFunc = utils.getDisplayFunction(tableDisplaySettings, fieldProperties, searchSettings);
			
			
			var values = data.getSubList(fieldname);
			if(values){
				// We have data
				for(var i=0;i<values.length;i++) {
					var descriptionContainer = $("<div>").appendTo($div);
					//check if we need to truncate the value (if it is too long)
					Utils.truncateTextVal(displayFunc(values[i].getValue("value")), descriptionContainer);
				}	
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
		}
	};
});