/**
 * "Default" Display Method
 * 
 * Simply shows the value in the field...
 * 
 * @author JEPE, JAMI
 */
define(["jquery", "../../Utils"], function($, Utils){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties.
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			var returnopts = {};
			
			//prevent stretching of potentially oversized fields (and let the 'overflow container' mechanism do its magic)
			if ("list" == tableDisplaySettings.resultsStyle) {
				// Manage width of the parent cell.
				returnopts.cellcss = {"max-width": "300px"};
			}
			
			// Get the display function (Used to display text & numbers. For numbers, formats them correctly + adds units)
			var displayFunc = Utils.getDisplayFunction(tableDisplaySettings, fieldProperties, searchSettings);
			
			
			var values = data.getSubList(fieldname);
			if(values){
				// We have data
				for(var i=0;i<values.length;i++) {
					// Add multiple rows to the cell...
					$div.append($("<div>", {
						"text":displayFunc(values[i].getValue("value"))
					}));
				}
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
			
			return returnopts;
		}
	};
});