/**
 * "Documents List" Display Method
 * 
 * Shows a user friendly label of the referenced Document with a link that opens "details popup" (if able)
 * 
 * Used to render Task Documents List and Document Reference Attribute value (which references Documents)
 * 
 * @author JAMI
 */
define(["jquery"], function($){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			var values = data.getSubList(fieldname);
			if(values){
				// We have data
				for(var i=0;i<values.length;i++) {
					
					var linkedDocCellContainer = $("<div>");
					
					//retrieve the required parameters
					var docVersionID = values[i].getValue("docversionid");
					var docName = values[i].getValue("value");
					var isShowSimpleGUI = searchSettings.getIsSimpleGUI();
					var menuFileURL = searchSettings.getMenuFileURL();
					
					if (isShowSimpleGUI) {
						//just write the Document Name as simple text value
						linkedDocCellContainer.text(docName);
					} else {
						//render as a link to the referenced Document Version
						$("<a>", {
							"href" : "docdetails.jsp?docVerID="+docVersionID+"&menu_file="+menuFileURL,
							"text" : docName
						}).appendTo(linkedDocCellContainer);
					}

						$div.append(linkedDocCellContainer);
				}
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
		}
	};
});