/**
 * "Company/Location" Display Method
 * 
 * Shows a user friendly label of the referenced Company/Location with a link that opens "details popup" (if able)
 * 
 * @author JAMI
 */
define(["jquery"], function($){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			var values = data.getSubList(fieldname);
			if(values){
				// We have data
				for(var i=0;i<values.length;i++) {
					
					var locationCellContainer = $("<div>");
					
					//retrieve the required parameters
					var locationID = values[i].getValue("location_id");
					var locationName = values[i].getValue("location_name");
					var companyName = values[i].getValue("company_name");
					
					if (locationID && locationID.length > 0) {
						//render along with the popup link
						var locPopupLink = $("<a>", {"href":"#"});
						locPopupLink.on( "click", _.partial(ShowLocationDetails, locationID));
						locPopupLink.text(companyName+"/"+locationName);
						locationCellContainer.append(locPopupLink);
					} else if (companyName && companyName.length > 0) {
						//just write the company name as simple text
						locationCellContainer.text(companyName);
					}
					
					$div.append(locationCellContainer);
				}
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
		}
	};
});