/**
 * "Print Item" Display Method (MFG Document type)
 * 
 * Should display the Print Item name and only refer to the design in a tooltip
 * 
 * @author JAMI
 */
define(["jquery", "../../Utils"], function($, utils){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - [NOT USED] using harcoded field names > 'print_items' as the display value and 'design_name' as the tooltip value
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			// Get the display function (Used to display text & numbers. For numbers, formats them correctly + adds units)
			var displayFunc = utils.getDisplayFunction(tableDisplaySettings, fieldProperties, searchSettings);
			
			//NOTE: we gather the both the lists of display and tooltip values as arrays of ValueData objects
			//Both these lists have to match the no. of elements in the respective arrays - if we don't have a array size match > no tooltips will be added to the output
			/** @type {ValueData} */
			var displayValues = data.getSubList("print_items")||[];
			/** @type {ValueData} */
			var tooltipValues = data.getSubList("design_name")||[];
			
			
			if (displayValues.length > 0) {
				//check the array sizes match > only use tooltips if they are both of the same size (display and tooltip values on the same index in the arrays go together)
				var showTooltips = displayValues.length ===  tooltipValues.length;
				
				for(var i=0;i<displayValues.length;i++) {
					// Display each print item and add a tooltip if required
					var curPrintItemContent = $("<div>", {"text":displayFunc(displayValues[i].getValue("value"))});
					if (showTooltips) {
						curPrintItemContent.attr("title", tooltipValues[i].getValue("value"));
					}
					
					$div.append(curPrintItemContent);
				}				
			} 
		}
	};
});