/**
 * "Project Name(s)" Display Method
 * 
 * Shows a user friendly label of the referenced Project with a link that opens "details popup" (if able)
 * 
 * Used to render Project Name in "extended criteria" group
 * 
 * @author JAMI
 */
define(["jquery", "underscore"], function($, _){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			var values = data.getSubList(fieldname);
			
			// Optional - document version id - in case we want to link to the folder in which the document resides.
			var docVersionID = null;
			
			// configuration of this field...
			var linktooption = fieldProperties.displayConfig.getValue("linkto");
			if(linktooption==="linkto_decidefolder") {
				// Link it to the document in the table. 
				// (Not used for parent and child projects, since the document (for document search) is not in those projects...)
				if (data.getSubList("docversioninfo")) {
					docVersionID = data.getSubList("docversioninfo")[0].getValue("id");
				}
			}
				
			var isShowSimpleGUI = searchSettings.getIsSimpleGUI();
			var menuFileURL = searchSettings.getMenuFileURL();
			
			if(values){
				// We have data
				for(var i=0;i<values.length;i++) {
					
					var projectCellContainer = $("<div>");
					
					//retrieve the required parameters for the processed Project ref. value
					var projectID = values[i].getValue("id");
					var projectName = values[i].getValue("value");
					
					var projPopupLinkElem = $("<a>");
					if (isShowSimpleGUI) {
						//show a small info popup for the target Project
						if (projectID && projectID.length > 0 && projectName && projectName.length > 0) {
							projPopupLinkElem.attr("href", "#").on("click", _.partial(ShowProjectDetails, projectID));
						} 
					} else {
						//standard Project Details (Document Details page, if in Doc Search context)
						var projPopupUrl = "projdetails.jsp?projectID="+projectID+"&menu_file="+menuFileURL;
						if (docVersionID !== null && docVersionID.length > 0) {
							projPopupUrl = projPopupUrl.concat("&docverid="+docVersionID);
						}
						projPopupLinkElem.attr("href", projPopupUrl);
					}
					
					projPopupLinkElem.text(projectName);
					projectCellContainer.append(projPopupLinkElem);
					$div.append(projectCellContainer);
				}
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
		}
	};
});