/**
 * "Rich Text" Display Method
 * 
 * Unlike a simple (text) value - Rich Text (relevant for an Attribute) has HTML structure to be rendered by the browser
 * 
 * @author JAMI
 */
define(["jquery"], function($){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties.
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
					
			var values = data.getSubList(fieldname);
			if(values){
				// We have data
				for(var i=0;i<values.length;i++) {
					// Add multiple rows to the cell...
					$div.append($("<div>").addClass("ReadOnlyRichText").html(values[i].getValue("value")));
				};
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
		}
	};
});