/**
 * "Task Due Date" Display Method
 * 
 * Task Due Date rendering changes based on its "Completed" state
 * 
 * @author JAMI
 */
define(["jquery", "../../Utils", "wcr/datetools"], function($, Utils, datetools){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			// Get the display function (Used to display text & numbers. For numbers, formats them correctly + adds units)
			var displayFunc = Utils.getDisplayFunction(tableDisplaySettings, fieldProperties, searchSettings);
			
			var values = data.getSubList(fieldname);
			if(values){
				// We have data
				for(var i=0;i<values.length;i++) {
					
					var taskDueDateCellContainer = $("<div>");
					
					//retrieve the required parameters
					var taskNotCompleted = values[i].getValue("completed") && values[i].getValue("completed") === "0";
					var dueDateValue = values[i].getValue("value");
					
					var dueDateDisplayValue = displayFunc(values[i].getValue("value"));
					taskDueDateCellContainer.text(dueDateDisplayValue);
					
					//if the status is "Complete" (ID: 00001_0000000003) we don't do any extra styling (warn about/danger of overdue state)
					var className = "";
					if (taskNotCompleted && dueDateValue !== "0") {
						var curTime = Date.now();
						var cutOffTime = dateAddWorkDays(new Date(curTime), 3);
						
						if (curTime > dueDateValue) {
							//Task is overdue
							className = "danger";
						} else if (cutOffTime > dueDateValue) {
							//Task is 3-days from being overdue
							className = "warning";
						}
						
						//apply the final class
						taskDueDateCellContainer.addClass(className);
					}
					
					$div.append(taskDueDateCellContainer);
				}
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
		}
	};
});