/**
 * "Task Status" Display Method
 * 
 * Shows readout of a Task Status > icon (if found) - only the name otherwise
 * 
 * @author JAMI
 */
define(["jquery"], function($){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);	
			var values = data.getSubList(fieldname);
			
			if(values){
				// We have data
				for(var i=0;i<values.length;i++) {
					
					var taskStatusCellContainer = $("<div>");
					
					//retrieve the required parameters
					var statusName = values[i].getValue("value");
					var statusIconFilename = values[i].getValue("icon_name");
					var statusIconAvailable = values[i].getValue("icon_available") && values[i].getValue("icon_available") === "1";
					
					
					if (statusIconFilename && statusIconFilename.length > 0 && statusIconAvailable) {
						//add icon with the label
						$("<img>", {
						  "src" : "images/statuses/"+statusIconFilename,
						  "alt" : statusName,
						  "title" : statusName,
						  "border" : 0})
						  .appendTo(taskStatusCellContainer);
					} else {
						//no icon, just add the Status Name
						taskStatusCellContainer.text(statusName);
					}
					
					$div.append(taskStatusCellContainer);
				}
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
            
            return {overflowsnever: true};
		}
	};
});