/**
 * "Text Content" Display Method
 * 
 * Simply shows the text content value in the field...
 * 
 * @author PASH
 */
define(["jquery", 
        "../../Utils",
        "packaging-content-management/ContentConstants",
        "utils/webi18n!TextContentSearchStrings"], 
function($,
		utils,
        ContentConstants,
        TextContentSearchStrings){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			// Creating container to hold status icon and text in same row.
			var resultContainer = $('<div>');
			var statusIcon = $('<i>').addClass('statement-status');

			// Get value of element type for this row.
			var elementType = data.getSubList("ElementType")[0].getValue("value");
			if ("NUTRITIONFACTS" == elementType) {
				// If element type is Nutrition Facts,
				// Show information that field is not supported for this element type.
				resultContainer.addClass('results-container-NA');
				statusIcon.addClass('icon icon-WCR_Text-Status-NA');
				
				resultContainer.append($('<span>').append(statusIcon));
				resultContainer.append($('<span>').text(TextContentSearchStrings.t("NF_NO_SUPPORT_STATEMENT")));
				
				$div.append(resultContainer);
				return;
			}
			
			var values = data.getSubList(fieldname);
			if(values){
				resultContainer.addClass('results-container');
				var statusValues = data.getSubList(fieldname + "_status");
				if (statusValues) {
					var status = statusValues[0].getValue("value");
					if (ContentConstants.SearchState['DRAFT'] == status) {
						statusIcon.addClass("icon icon-WCR_Text-Status-Draft draft");
						statusIcon.attr('title', TextContentSearchStrings.t('DRAFT'));
					} else if (ContentConstants.SearchState['SIGNED_OFF'] == status) {
						statusIcon.addClass("icon icon-WCR_Text-Approve approved");
						statusIcon.attr('title', TextContentSearchStrings.t('SIGNED_OFF'));
					} else if (ContentConstants.SearchState['REJECT'] == status) {
						statusIcon.addClass("icon icon-WCR_Text-Reject rejected");
						statusIcon.attr('title', TextContentSearchStrings.t('REJECT'));
					} else if (ContentConstants.SearchState['READY_FOR_APPROVAL'] == status) {
						statusIcon.addClass("icon icon-WCR_Text-Status-Pending pending");
						statusIcon.attr('title', TextContentSearchStrings.t('READY_FOR_APPROVAL'));
					}
					resultContainer.append($('<span>').append(statusIcon));
				}
				resultContainer.append($('<span>').html(values[0].getValue("value")));
				
				$div.append(resultContainer);
				if ("list" == tableDisplaySettings.resultsStyle) {
					// Manage width of the column.
					var parentElem = $div.parents("td").first();
					parentElem.addClass("text-content-col");
				}
			}else {
				// There is no data for this field...
				// Show information that document doesn't contain this language.
				resultContainer.addClass('results-container-NA');
				statusIcon.addClass('icon icon-WCR_Text-Status-NA');
				
				resultContainer.append($('<span>').append(statusIcon));
				resultContainer.append($('<span>').text(TextContentSearchStrings.t("PCM_CONTENT_STATUS_EMPTY")));
				
				$div.append(resultContainer);
			}
		}
	};
});