/**
 * "Text Content Document Status" Display Method
 * 
 * 
 * @author RAHM
 */
define(["jquery", 
        "../../Utils",
        "utils/webi18n!TextContentSearchStrings",
        "packaging-content-management/ContentConstants"], 
function($,
		utils,
		TextContentSearchStrings,
        ContentConstants){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			var statusValues = data.getSubList(fieldname);
			if(statusValues){
				var status = statusValues[0].getValue("value");
				var statusIcon = $('<i>').addClass('statement-status');
				if (ContentConstants.SearchState['DRAFT'] == status) {
					statusIcon.addClass("icon icon-WCR_Text-Status-Draft draft");
					statusIcon.attr('title', TextContentSearchStrings.t('DRAFT'));
				} else if (ContentConstants.SearchState['SIGNED_OFF'] == status) {
					statusIcon.addClass("icon icon-WCR_Text-Approve approved");
					statusIcon.attr('title', TextContentSearchStrings.t('SIGNED_OFF'));
				} else if (ContentConstants.SearchState['REJECT'] == status) {
					statusIcon.addClass("icon icon-WCR_Text-Reject rejected");
					statusIcon.attr('title', TextContentSearchStrings.t('REJECT'));
				} else if (ContentConstants.SearchState['READY_FOR_APPROVAL'] == status) {
					statusIcon.addClass("icon icon-WCR_Text-Status-Pending pending");
					statusIcon.attr('title', TextContentSearchStrings.t('READY_FOR_APPROVAL'));
				}
				
				$div.append($('<span>').append(statusIcon));
			}else {
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
		}
	};
});