/**
 * "Text Content" Display Method
 * 
 * Simply shows the text content value in the field...
 * 
 * @author PASH
 */
define(["jquery", 
        "../../Utils",
        "packaging-content-management/ContentConstants",
        "packaging-content-management/text-content-services/elementTypeService",
        "utils/webi18n!TextContentStrings"], 
function($,
		utils,
        ContentConstants,
        elementTypeService,
        TextContentStrings){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			var values = data.getSubList(fieldname);
			if(values){
				var elementTypeCode = values[0].getValue("value");
				
				// Get the readable format of element type code and then display on results page.
				// This is a temporary solution to display readable string. Saving search results will save only element type codes.
				elementTypeService.getElementTypes(function (elementTypes) {
					elementTypeService.getReadableElementType(elementTypeCode, true, function (readableElementType) {
						$div.append(readableElementType);
					});
				});
			}else {
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
		}
	};
});