/**
 * "Thumbnail" Display Method
 * 
 * @author JEPE
 */
define(["jquery", "../../Utils"], function($, Utils){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties.
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			// Read some options
			// These are display options of the thumbnail field => can be different for documents/projects/...
			var thumbkey = fieldProperties.displayConfig.getValue("resultscolumnkey");
			var thumbzoomurl = fieldProperties.displayConfig.getValue("zoomurl");
			var thumbzoomsize = fieldProperties.displayConfig.getValue("zoomsize");
			var haszoomurl = !!thumbzoomurl;
			
			// Depending on the type of layout, the thumbnail will look different (size and such)
			var cssclass = "";
			var height = "100";
			var width = "";
			var imgalign = "";
			if(tableDisplaySettings.resultsStyle === "card") {
				// CARD
				cssclass = "card-thumb";
				imgalign = "middle";
				height = "";
			} else if (tableDisplaySettings.resultsStyle === "list") {
				// LIST
				height = "50";
				width = "50";
				imgalign = "middle";
			} else {
				// GRID
				height = "100";
				imgalign = "middle";
				if (tableDisplaySettings.resultsStyleSubType === "wide") {
					// grid with 6 or more columns => bigger thumbnail
					height = "200";
				}
			}
			
			// hrefValue = currently generated in the xsl still...
			// FUTURE: Move to Java or JS (but probably keep passing it the same way?)
			var hrefValue = data.getValue("thumbhref");
			
			// If hyperlinks are not allowed, don't use the href value
			if(!tableDisplaySettings.hyperLinksAllowed) {
				hrefValue = null;
			}
			
			// Read the correct entry (document info/project info/ ...)
			var items = data.getSubList(thumbkey);
			if(items && items.length>0){
				// We have data for this document/project/...
				var itemdata = items[0];
				
				// Get some information about this item
				// These are part of the Search Results Table (filled in on Java side)
				var thumbnailurl = itemdata.getValue("thumbnail_url");
				var itemname = itemdata.getValue("name");
				var itemid = itemdata.getValue("id");
				
				// Start building the thumbnail
				var $thumb = $("<img>", {"alt":"", "src":thumbnailurl, "border":"0"});
				$thumb.attr("align", imgalign);
				if(width) {
					$thumb.attr("width", width);
				}
				if(height) {
					$thumb.attr("height", height);
				}
				// Note: events call global functions "delayhidethumb" and "showthumb"
				$thumb.on("mouseout", delayhidethumb);
				if(haszoomurl) {
					// Calculate the zoom url  (get some preference and replace {#id} in it with the actual id of the item)
					var thumbzoomurlfilled = thumbzoomurl;
					thumbzoomurlfilled = thumbzoomurlfilled.replace(/\{#id\}/gi, itemid);
					
					$thumb.on("mouseover", _.partial(showthumb, _, thumbnailurl, itemname, "202", null, thumbzoomurlfilled, parseInt(thumbzoomsize, 10)));
				} else {
					$thumb.on("mouseover", _.partial(showthumb, _, thumbnailurl, itemname, "202"));
				}
				
				// Start building the thumbnail link
				var $link = $("<a>");
				if(hrefValue) {
					$link.attr("href", hrefValue);
				}
				if(cssclass) {
					$link.addClass(cssclass);
				}
				
				$div.append($link.append($thumb));
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
			
			return {overflowsnever:true};
		}
	};
});