/**
 * Implements a simple Text Field for the Design Panel.
 */
define([
        "jquery", 
        "underscore",
        "./ParameterField_date",
        "../../dataobjects/FieldProperties",
        "../../dataobjects/ValueData",
        "../../Utils",
        "utils/webi18n!SearchInputs"
        ], function(
        		$, _,
        		DateParameterField,
        		FieldProperties,
        		ValueData,
        		utils,
        		SearchInputs) {
		
		"use strict";

    /**
     * @constructor
     */
    var ApprovalStatusParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.changeCallbacks = [];
    	
    	this._settings = settings;
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    ApprovalStatusParameterField.prototype._initialize = function(){
    	
    	this._dateFieldId = "db_approval_date";
    	this._dateFieldDiv = $('<div>').addClass("expand-to-fill-input-group");
    	this._dateField = null;
    	
    	// Make the object with all the information which is exposed to the components.
		var fieldProperties = new FieldProperties();
		fieldProperties.displayUnit = null;
		fieldProperties.unitConversionFactor = null;
		fieldProperties.dataType = "date";
		fieldProperties.displayConfig = new ValueData();
		fieldProperties.displayConfig.setValue("date_render_type", "date_time");
		fieldProperties.displayConfig.setValue("labels", "show_due_date_labels");
		fieldProperties.displayConfig.setValue("mode", "past");
		fieldProperties.expandMode = null;
		
		this._dateField = DateParameterField.make(this._dateFieldDiv, this._dateFieldId, this._settings, null, fieldProperties);
    	this._dateField.setEnabled(false);
    	
    	this._dateField.onChange(_.bind(this._triggerChange, this));
    	
    	
    	// Make the approval status dropdown...
    	this.select = $("<select>", {
    		"data-sahi-id":this.fieldId+"_status",// For automated testing
    		"class":"form-control attachleft search-input-md"
    	});
    	
    	this._dateInputDiv = $("<div>").addClass("input-group").attr({"title": SearchInputs.t("ADVSEARCH_APPROVALDUEDATE")}).append(
    			$("<label>").addClass("input-group-addon").append($("<i>").addClass("icon icon-WCR_Calendar")),
    			this._dateFieldDiv
    	);
    	
    	this._dateInputOverlayDiv = $("<div>", {"class": "dateoverlay"}).show();
    	
    	this._dateInputDiv.append(this._dateInputOverlayDiv);
    	
    	// Add tooltip above date input field
    	require(["jqueryUI"], _.bind(function(){
    		this._dateInputDiv.tooltip({position:{my: "center bottom-5", at: "center top"}});
    		this._dateInputOverlayDiv.tooltip({position:{my: "center bottom-5", at: "center top"}});
		}, this));
    	
    	// Append the approval_status and approval_date to the wrapper
    	this.div.append(
    			$("<table>").addClass("table-full-width").append(
    					$("<tr>").append(
    							$("<td>").addClass("add-padding-right").append(this.select), 
    							$("<td>").append(this._dateInputDiv)
    					)
    			)
    	);
    	
    	this._changeApprovalStatus = _.bind(function() {
        	// Set the option to current option
        	if(this._value) {
				this.select.val(this._value);
				this._triggerChange(true);
				this._changeApprovalDate(this._value);
			}
    	}, this);
    	
    	
    	/*
    	 * Add options to select dropdown
    	 */
    	// Add a empty option
    	this.select.append($('<option>').attr({"value": ""})).prop("selected", true);
    	
    	// Ajax call to get approval type list and create options
    	$.when($.ajax({url: "GetSearchApprovalStatusTypesList.jsp", method: "POST", dataType: "xml"}))
    		.then(_.bind(function( data, textStatus, jqXHR ) {
    			
    			var approvalTypeListNodes = $(data).find("approval_type_list");
    			
    			if(approvalTypeListNodes.length > 0) {
    				// Iterate through all approval_type_list nodes
    				for(var i=0; i<approvalTypeListNodes.length; i++ ) {
    					var approvalTypeNodes = $(approvalTypeListNodes[i]).find("approval_type");
    					
    					if(approvalTypeNodes.length > 0) {
    						// Iterate through all approval_type nodes
    						for(var j=0; j<approvalTypeNodes.length ; j++) {
    							// Get the option text
    							var ApprovalTag = $("label_tag", approvalTypeNodes[j]).text();
    							// Get the option value
    							var ApprovalStatusId = $(approvalTypeNodes[j]).attr("id");
    							
    							if(ApprovalStatusId == "00001_0000000003 OR 00001_0000000005 OR 00001_0000000006 OR 00001_0000000007") {
    								ApprovalStatusId = "all_approved";
    							} else if(ApprovalStatusId == "00001_0000000001 OR 00001_0000000004") {
    								ApprovalStatusId = "all_rejected";
    							} else if(ApprovalStatusId == "00001_0000000003 OR 00001_0000000005 OR 00001_0000000006 OR 00001_0000000007 OR 00001_0000000001 OR 00001_0000000004") {
    								ApprovalStatusId = "all_completed";
    							}
    							
    							// Create the option html
    							var option = $("<option>").attr({"value": ApprovalStatusId}).text(SearchInputs.t(ApprovalTag));
    							
    							// Append the option to the dropdown
    							this.select.append(option);
    						}
    					}
    				}
    				
    				// Update the approval status dropdown
    				this._changeApprovalStatus();
    			}
    		}, this));
    	
    	
    	/**
    	 * Changes the approval date's value and disabled state
    	 */
    	this._changeApprovalDate = _.bind(function(approvalStateValue){
    		if(this._dateField) {
    			if (approvalStateValue == '00001_0000000002' || approvalStateValue === '') {
    				this._dateField.clear();
    				// for pending status and empty option, disable due date field
    				this._dateField.setEnabled(false);
    				this._dateInputOverlayDiv.show();
    				
    			} else {
    				// enable date field
    				this._dateField.setEnabled(true);
    				this._dateInputOverlayDiv.hide();
    			}
    		}
    	}, this);
    	
    	this.select.on("change", _.bind(function(event) {
    		this._onIntermediateUpdate();
    		this._changeApprovalDate(event.target.value);
    	}, this));
    };
    
    //
    // Internal event handling
    //
    ApprovalStatusParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    ApprovalStatusParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    ApprovalStatusParameterField.prototype.getValueData = function (value, feedback) {
    	var optionValue = $("option:selected", this.select).val();
    	
    	value.setValue("value", optionValue);
    	
    	if(this._dateField) {
    		var dateData = value.addNewItemToList("date_search_criterion");
    		
    		dateData.setValue("key", "db_approval_date");
    		
    		this._dateField.getValueData(dateData);
    	}
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    ApprovalStatusParameterField.prototype.setValueData = function (data) {
    	
    	var value = data.getValue("value");
    	
    	if(data.getSubList("date_search_criterion")) {
    		var dateData = data.getSubList("date_search_criterion")[0];
    		
    		this._dateField.setValueData(dateData);
    	}
    	
    	// Set the select dropdown to the current option
    	this._value = value;
    	this._changeApprovalStatus();
    	
    	return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    ApprovalStatusParameterField.prototype.clear = function() {
    	this.select.val("");
    	this._changeApprovalDate("");
    	this._triggerChange(false);
    };
    
    /**
     * Focus this field
     */
    ApprovalStatusParameterField.prototype.focus = function() {
    	this.select.focus();
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    ApprovalStatusParameterField.prototype.valueContainsSmartName = function() {
    	return false;
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    ApprovalStatusParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	return false;
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    ApprovalStatusParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    ApprovalStatusParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    ApprovalStatusParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new ApprovalStatusParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});