/**
 * Implements Document Reference Attribute type field
 * 
 * @auhtor JAMI
 */
define([
        "jquery", 
        "underscore", 
        "../../Utils",
        "../../../documentselector/DocumentSelector",
        "utils/webi18n!SearchInputs",
        "../../dataobjects/ExternalSearchSettings",
        
        // Smartname related stuff
        "component/smartnames/smartfield/smartfield",
        "component/smartnames/encoding/smartnameencoding"
        ], function(
        		$, _, utils,
        		DocumentSelector,
        		JSP_Strings,
        		ExternalSearchSettings,
        		SmartField,
        		SmartNameEncoding) {

    /**
     * @constructor
     */
    var DocumentReferenceParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.settings = settings;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    DocumentReferenceParameterField.prototype._initialize = function(){
    	var self = this;
    	
    	this.isInitializing = true;
    	
    	this.selectModeParms = {};
    	
    	// Create the HTML fields for DocRef field
    	this._constructField();
    	
    	if(!this.settings.isSmartNamesEnabled()) {
    		// We have a normal input
    		
	    	// listen to certain events on the field...
    		
	    	//keyup event >>> check whether Enter key was pressed (final update then) > otherwise intermediate
	    	this.matchesInput.on("keyup", function(event){
	    		self._onIntermediateUpdate();
	    		
	    		//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
	    		if (event.keyCode == 13) {
	    			$(this).blur();
	    		}
	    	});
	    	
	    	//blur=get out of the field >>> trigger final update event
	    	this.matchesInput.on("blur", _.bind(this._onFinalUpdate, this));
	    	
    	} else {
    		// We have a SmartField input
    		
    		// Convert the input into a SmartField
    		this.smartField = new SmartField(this.matchesInput);
    		
    		// listen to certain events on the field...
    		this.smartField.onValueChange(_.bind(this._onIntermediateUpdate, this));
    	}
    	
    	// On change event when user select revision option value
    	this.select.on("change", _.bind(this._onModeSelectChange, this));
    	
    	this.isInitializing = false;
    };
    
    /**
     * Make the Document Reference Field Parameter input
     * 
     * This method will create fields [this.input] , [this.select] - This fields are public and it will accessible all methods
     */
    DocumentReferenceParameterField.prototype._constructField = function() {  	
    	var table = $("<table>").addClass("parameterFieldTable");
    	var row = $("<tr>").appendTo(table);
    	
    	//--------------------
    	//Source mode selector
    	var tdSourceSelect = $("<td>").appendTo(row);
    	var selectDiv = $("<div>").appendTo(tdSourceSelect);
    	
    	this.select = $("<select>", {
			"data-sahi-id":this.fieldId+"_select",// For automated testing
			"class" : "form-control attachleft search-input-md",
		}).appendTo(selectDiv);
    	
    	//Add the source options to the select
    	this.emptyOption = $("<option>", {
    		"value" : "empty"}).text(JSP_Strings.t('DOCREF_EMPTY')).appendTo(this.select);
		
    	this.matchesOption = $("<option>", {
    		"value" : "matches"}).text(JSP_Strings.t('DOCREF_MATCHES')).appendTo(this.select);
    	
    	this.selectOption = $("<option>", {
    		"value" : "select"}).text(JSP_Strings.t('DOCREF_SELECT')).appendTo(this.select);
    	
    	//make "Matches" mode the initially selected one
    	this.select.val("matches");
    	
    	//temporary indicator for the currently selected mode > used as a fallback mechanism for "Cancel" button click-event of Document Selector
    	this.curSelectMode = this.select.val();

    	//------------------------------------------------------------------
    	//Matches source mode input field / Select source mode Doc Reference
    	var tdInputfield = $("<td>").appendTo(row);
    	this.inputDiv = $("<div>").appendTo(tdInputfield);
    	
    	this.matchesInput = $("<input>", {
    		"data-sahi-id":this.fieldId+"_matches",// For automated testing
			"type" : "text",
			"class" : "form-control search-input-sm"
		}).appendTo(this.inputDiv);
    	
    	this.selectReadoutDiv = $("<div>", {"class": "add-padding-left"}).hide().appendTo(this.inputDiv);
    	
    	this.div.append(table);
    };
    
    /** 
    * Callback for the "Search Mode" selector (this.select) onChange event.
    * 
    * Used to switch between "Empty", "Matches" and "Select" modes.
    */
    DocumentReferenceParameterField.prototype._onModeSelectChange = function() {

    	var selectVal = this.select.val();
    	
    	if ("empty" === selectVal) {
    		this.curSelectMode = this.select.val();
    		this.selectReadoutDiv.hide();
    		
    		if(this.settings.isSmartNamesEnabled()) {
    			this.smartField.getJQElement().show();
	    	}else{
	    		this.matchesInput.show();
	    	}	
    		this.inputDiv.css('visibility','hidden');
    		
    		if (!this.isInitializing) {
    			this._onFinalUpdate();
    		}
    	} else if ("matches" === selectVal) {
    		this.curSelectMode = this.select.val();
    		if (this.settings.isSmartNamesEnabled()) {
    			this.smartField.getJQElement().show();
    		} else {
    			this.matchesInput.show();
    		}
    		
    		this.inputDiv.css('visibility','visible');
    		this.selectReadoutDiv.hide();	
    		
    		if (!this.isInitializing) {
    			this._onIntermediateUpdate();
    		}
    		
    	} else if ("select" === selectVal) {
    		this.inputDiv.css('visibility','visible');
    		this.selectReadoutDiv.show();
    		if(this.settings.isSmartNamesEnabled()) {
    			this.smartField.getJQElement().hide();
	    	}else{
	    		this.matchesInput.hide();
	    	}
    		
    		//need to open the DocumentSelector modal popup
    		// Initialize the component first
    		this._initializeSelectPopup();
    			
    		//open the popup
    		this._docSelectPopup.openDocumentSelector();
    		
    		if (!this.isInitializing) {
    			this._onIntermediateUpdate();
    		}	
    	}
    };
    
    DocumentReferenceParameterField.prototype._updateFieldControlsState = function() {
    	var selectVal = this.select.val();
    	
    	if ("empty" === selectVal) {
    		this.curSelectMode = this.select.val();
    		this.selectReadoutDiv.hide();
    		if(this.settings.isSmartNamesEnabled()) {
    			this.smartField.getJQElement().show();
	    	}else{
	    		this.matchesInput.show();
	    	}	
    		this.inputDiv.css('visibility','hidden');
    	} else if ("matches" === selectVal) {
    		this.curSelectMode = this.select.val();
    		this.inputDiv.css('visibility','visible');
    		this.selectReadoutDiv.hide();
    		if(this.settings.isSmartNamesEnabled()) {
    			this.smartField.getJQElement().show();
	    	}else{
	    		this.matchesInput.show();
	    	}	
    	} else if ("select" === selectVal) {
    		this.inputDiv.css('visibility','visible');
    		this.selectReadoutDiv.show();
    		if(this.settings.isSmartNamesEnabled()) {
    			this.smartField.getJQElement().hide();
	    	}else{
	    		this.matchesInput.hide();
	    	}
    		
    		//also check if we have a selection > display the Document reference
    		this._renderSelectDocumentReference();
    	}
    };
    
    //
    // Internal event handling
    //
    DocumentReferenceParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    DocumentReferenceParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    DocumentReferenceParameterField.prototype.getValueData = function (value, feedback) {
            value.clear();
            
            var curMode = this.select.val();
            value.setValue("mode", curMode);
            
            if (curMode === "select" && this.selectModeParms != 'undefined') {
            	value.setValue("doc_id", this.selectModeParms.docID || "");
            	value.setValue("doc_name", this.selectModeParms.docName || "");
            	value.setValue("doc_ver_id", this.selectModeParms.docVerID || "");
            	value.setValue("doc_ver_rev_name", this.selectModeParms.docVerRevName || "");
            	value.setValue("folder_id", this.selectModeParms.folderID || "");
            	value.setValue("project_id", this.selectModeParms.projectID || "");
            	value.setValue("target_doc_ver_id", this.selectModeParms.targetDocVerID || "");
            } else if (curMode === "matches") {
            	if(this.settings.isSmartNamesEnabled()) {
    	    		value.setValue("valueSN", this.smartField.getValue());
    	    	}else{
    	    		value.setValue("value", this.matchesInput.val());
    	    	}	
            }	
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    DocumentReferenceParameterField.prototype.setValueData = function (data) {
    	this.isInitializing = true;
    	
    	this.selectModeParms.docID          = data.getValue("doc_id") || "";
    	this.selectModeParms.docName        = data.getValue("doc_name") || "";
    	this.selectModeParms.docVerID       = data.getValue("doc_ver_id") || "";
    	this.selectModeParms.docVerRevName  = data.getValue("doc_ver_rev_name") || "";
    	this.selectModeParms.folderID       = data.getValue("folder_id") || "";
    	this.selectModeParms.projectID      = data.getValue("project_id") || "";
    	this.selectModeParms.targetDocVerID = data.getValue("target_doc_ver_id") || "";
    	
    	//initialize the Select mode popup if needed
    	this._initializeSelectPopup();
    	
    	this.selectModeParms.mode           = data.getValue("mode");
    	
    	if (this.selectModeParms.mode) {
        	//if a valid Select-Document Reference is currently set > mark it 
        	if (this.selectModeParms.mode === 'select' && this.selectModeParms.targetDocVerID && this.selectModeParms.docName) {
        		this.selectModeParms.persistSelectVal = true;
        	}
    		
    		this.select.val(this.selectModeParms.mode);
    		//trigger render update
    		this._updateFieldControlsState();
        	
        	if (this.selectModeParms.mode == 'matches') {     		
        		if(this.settings.isSmartNamesEnabled()) {
        			this.smartField.enable();
            		// SmartName mode...
        			var valueSN = data.getValue("valueSN");
            		this.smartField.setValue(valueSN || "");
            	} else {
            		// Normal mode...
            		var value = data.getValue("value");
            		this.matchesInput.val(value || "");
            	}
        	}
    	}

    	this.isInitializing = false;
    	return true;
    };
    
    /**
     * Update values about a specific Document when coming back from "Document Selector" popup
     * 
     * @param projectId
     * @param folderId
     * @param documentId
     * @param documentVersions
     * @param documentRevision
     * @param targetDocVersionId
     * @param documentName
     * @param docVerRevName
     */
    DocumentReferenceParameterField.prototype._setDocSelectValues = function(projectId, folderId, documentId, documentVersions, documentRevision, targetDocVersionId, documentName, docVerRevName) {
    	//mark the new selected reference as persistent (cancel button click on the popup will go back to this selection)
    	this.selectModeParms.persistSelectVal = true;
    	
    	this.selectModeParms.docID          = documentId;
    	this.selectModeParms.docName        = documentName;
    	this.selectModeParms.docVerRevName  = docVerRevName;
    	this.selectModeParms.docRev         = documentRevision;
    	this.selectModeParms.folderID       = folderId;
    	this.selectModeParms.projectID      = projectId;
    	this.selectModeParms.targetDocVerID = targetDocVersionId;
    	
    	//as we could refer to multiple Doc Versions > process potential multi-value
    	var docVerStr = "";
    	if (documentVersions !== undefined) {
    		
    		if ($.isArray(documentVersions)) {
    			for(var i=0 ; i<documentVersions.length ; i++){
    				if (i===0) {
    				  docVerStr = documentVersions[0];	
    				} else {
    				  docVerStr += " OR " + documentVersions[i];					
    				}
    			}
    		} else {
    			docVerStr = documentVersions;
    		}
    	}
    	this.selectModeParms.docVerID = docVerStr;
    	
    	//using the updated values render the Document Reference select readout
    	this._renderSelectDocumentReference();
    	
    	this._onFinalUpdate();
    };
    
    /**
     * Generates the Document Reference readout and controls for Select mode
     */
    DocumentReferenceParameterField.prototype._renderSelectDocumentReference = function () {
    	if (this.selectModeParms && this.selectModeParms.targetDocVerID && this.selectModeParms.docName) {
    		
    		this.selectReadoutDiv.empty();
    		
    		//write the "edit Doc Ref selection" icon
    		var editIcon = $("<a>").addClass("docref_editbtn add-spacing-right")
    		.attr("href", "#")
    		.on("click", _.bind(function (event) {
    			event.preventDefault();
    			this.openDocumentSelector();
    		}, this._docSelectPopup))
    		.appendTo(this.selectReadoutDiv);
    		
    		$("<i>", {
    			"title" : JSP_Strings.t('EDIT_TAG'),
    			"class" : "icon icon-WCR_Actions-Edit12",
    			"id" : "select_pop_up_" + this.selectModeParms.targetDocVerID
    		}).appendTo(editIcon);
    		
    		//write the Document reference (as a link)
    		$("<a>", {"id" : "select_doc_name_href_"+ this.selectModeParms.targetDocVerID})
    		.addClass("docref_name")
    		.attr("href", "docdetails.jsp?docVerID="+this.selectModeParms.targetDocVerID+"&menu_file="+this.settings.getExternalSettings().getMenuFileURL())
    		.text(this.selectModeParms.docName)
    		.appendTo(this.selectReadoutDiv);
    		
    		if (this.selectModeParms.docVerRevName) {
    			$("<div>", {"id" : "select_doc_ver_rev_name_div_"+this.selectModeParms.targetDocVerID})
    			.addClass("docref_version add-spacing-left")
    			.css("display", "inline-block")
    			.text(this.selectModeParms.docVerRevName)
    			.appendTo(this.selectReadoutDiv);
    		}
    	} else { 
    		//fall back to the previous mode if no relevant Document selected
    		this.select.val(this.curSelectMode);
    		this._updateFieldControlsState();
    	}
    };
    
    /**
     * Prepares this._docSelectPopup (DocumentSelector) member variable, if not done yet
     */
    DocumentReferenceParameterField.prototype._initializeSelectPopup = function() {
    	if(!this._docSelectPopup) {
			// Create the component
			this._docSelectPopup = new DocumentSelector(this.selectModeParms.projectID, this.selectModeParms.folderID, 
					this.selectModeParms.docID, this.selectModeParms.docVerID, this.selectModeParms.docRev);
			
			this._docSelectPopup.onDocumentSelected(_.bind(this._setDocSelectValues, this));
			this._docSelectPopup.onCancelClicked(_.bind(this._selectPopupCancel, this));
		}
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    DocumentReferenceParameterField.prototype.clear = function() {
    	if(this.settings.isSmartNamesEnabled()) {
    		this.smartField.setValue("");
    	} else{
	    	this.matchesInput.val(""); 	
    	}
    	
    	this.select.val("matches");
    	this.curSelectMode = this.select.val();
    	this.selectModeParms = {};
    	this._docSelectPopup = null;
    	this._updateFieldControlsState();
    	this.selectModeParms.persistSelectVal = false;
    	this._triggerChange(false);
    };
    
    /**
     * Callback handler for when "Cancel" button is clicked on the Document Selector popup
     */
    DocumentReferenceParameterField.prototype._selectPopupCancel = function() {
    	//reset out the popup if there is no Document reference to go back to > it will get reinitialized on next use
    	if (!this.selectModeParms.persistSelectVal) {
    		this._docSelectPopup = null;
    	}
    	
    	this._renderSelectDocumentReference();
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {boolean} true when this field contains a SmartName
     */
    DocumentReferenceParameterField.prototype.valueContainsSmartName = function() {
    	return this.settings.isSmartNamesEnabled() && utils.hasSmartNameInSmartField(this.smartField);
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    DocumentReferenceParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	if(this.settings.isSmartNamesEnabled()) {
    		// Don't check for special characters in a SmartField...
    		// The hint would otherwise also show up if you use the autocomplete to type a SmartName...
    		// (For the autocomplete you start typing a SmartName with "[" , which is a special character)
    		//return utils.hasCharsInSmartField(this.smartfield, specialCharacters);
    		return false;
    	}else{
    		return utils.hasCharsInStr(this.matchesInput.val(), specialCharacters);
    	}
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    DocumentReferenceParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    DocumentReferenceParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    DocumentReferenceParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new DocumentReferenceParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});