/**
 * Implements a simple Text Field for the Design Panel.
 */
define([
        "jquery", 
        "underscore", 
        "../../Utils", 
        
        // Smartname related stuff
        "component/smartnames/smartfield/smartfield",
        "component/smartnames/encoding/smartnameencoding",
        
        "./ParameterField_text"
        ], function(
        		$, _, utils,
        		SmartField,
        		SmartNameEncoding, TextParameterFieldFactory) {

    /**
     * @constructor
     */
    var LocationParameterField = function(div, fieldId, settings, dataProvider, fieldProperties) {
    	// Call super constructor
    	TextParameterFieldFactory.constructor.call(this, div, fieldId, settings, dataProvider);
    	
    	// Get the "user/group/me" field to watch for
    	var userFieldId = fieldProperties.displayConfig.getValue("field_to_watch");
    	if(!userFieldId) {
    		throw new Error("field_to_watch setting for the LocationParameterField is not correctly configured.");
    	}

    	// Start watching the "user/group/me" or "user/me" field
    	this._locationenabled = true;
    	settings.watchCriterion(userFieldId, _.bind(this._onUserUpdate, this));
    };
    
    // Inherit from text parameter field
    LocationParameterField.prototype = Object.create(TextParameterFieldFactory.constructor.prototype);
    LocationParameterField.prototype.constructor = LocationParameterField;
    
    /**
     * Called whenever the other field containing the user updates...
     */
    LocationParameterField.prototype._onUserUpdate = function(otherValueData) {
    	if(otherValueData === null || otherValueData.getValue("user_type") === "USER") {
    		// enable the field
    		if(!this._locationenabled) {
    			this._locationenabled = true;
	    		if(this.settings.isSmartNamesEnabled()) {    			
	    			this.smartfield.enable();
	    		} else {
	    			this.input.prop("disabled", false);
	    		}
    		}
    	} else {
    		// disable the field
    		if(this._locationenabled) {
    			this._locationenabled = false;
	    		if(this.settings.isSmartNamesEnabled()) {
	    			this.smartfield.clear();
	    			this.smartfield.disable();
	    		} else {
	    			this.input.val("");
	    			this.input.prop("disabled", true);
	    		}
    		}
    	}
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new LocationParameterField(div, fieldName, settings, dataProvider, fieldProperties);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});