define(["underscore",
		"../../../dataobjects/ValueData",
		"../../../dataobjects/ValueDataUtils",
		"./FacetSelection"
	],
	function (_,
		ValueData,
		ValueDataUtils,
		FacetSelection) {


		"use strict";



		return {
			_normalizeFacetIds: function (ids) {
				if (!_.isArray(ids)){
					ids = [ids];
				}
				return ids.slice(0).sort();
			},

			/**
			 * Get the valueData for a certain the facetid combo of this util instance
			 * @private
			 * @param {Array.<String>} facetids
			 * @param   {ValueData} data 
			 * @returns {ValueData} [[Description]]
			 */
			_findFacetSelection: function (facetids, data) {
				facetids = this._normalizeFacetIds(facetids);

				return _.find(data.getSubList("selection"), function (facetSelectionData) {
					var selectionfacetids = ValueDataUtils.getStringList(facetSelectionData, "facetids");
					return selectionfacetids && _.isEqual(facetids, this._normalizeFacetIds(selectionfacetids));
				}, this);

			},

			/**
			 * Returns a list of selected items
			 * 
			 * @param {Array.<String>} facetids
			 * @param {ValueData} FacetSelectionCriterion data
			 *                                            
			 * @returns FacetSelection
			 */
			getSelection: function (facetids, data) {
				facetids = this._normalizeFacetIds(facetids);

				var facetSelectionData = this._findFacetSelection(facetids, data);

				if (facetSelectionData) {
					return FacetSelection.fromValueData(facetSelectionData);
				} else {
					return new FacetSelection(facetids);
				}
			},


			/**
			 * Set the selection of the facet
			 * @param {ValueData} data      data in which the selection should be stored
			 * @param {object}    selection (see getSelection)
			 */
			setSelection: function (facetids, data, selection) {
				var facetSelectionData = this._findFacetSelection(facetids, data);

				if (!facetSelectionData) {
					// not yet in data
					facetSelectionData = data.addNewItemToList("selection");
				}
				selection.toValueData(facetSelectionData);
			},

			/**
			 * Clear all data associated with a facetid combo
			 * @param   {Array.<String>} facetids 
			 * @param   {ValueData}      data     
			 */
			clearSelectionData: function (facetids, data) {
				facetids = this._normalizeFacetIds(facetids);

				var filteredList = _.filter(data.getSubList("selection"), function (facetSelectionData) {
					var selectionfacetids = ValueDataUtils.getStringList(facetSelectionData, "facetids");
					return selectionfacetids && !_.isEqual(facetids, this._normalizeFacetIds(selectionfacetids));
				}, this);

				data.setSubList("selection", filteredList);

			}
		};
	});