define(["jquery",
        "underscore",
        "utils/webi18n!SearchInputs"], 
	function($,
			_,
			SearchInputs) {
		
		"use strict";
		
		/**
		 * Create a single row of input fields for ink name, ink group, ink type
		 * 
		 * @param options [Object] - options 
		 */
		var InkSearchRow = function() {
			
			// delete buttons
			this._deleteButton = null;
			
			this._isDeleteDetached = false;
			
			// callbacks for delete
			this._onDeleteCallbacks = [];
			
			// Handler to execute respective callbacks when delete button is clicked
			this._deleteClickHandler = _.bind(function(event) {
				_.each(this._onDeleteCallbacks, _.bind(function(callback) {
					callback(event, this);
				}, this));
			}, this);
			
			
			this._constructInkRow();
		};
		
		/**
		 * Construct the html structure of the ink row
		 */
		InkSearchRow.prototype._constructInkRow = function() {
			
			// Input field to enter the ink name
			this._inkNameInput = $("<input>", {"size": "25", "class": "form-control customoptionfields", "name": "inkname"});
			
			// Select field to enter the ink group 
			this._inkGroupSelect = $("<select>", {"class": "form-control customoptionfields", "name": "inkgroup"});
			this._inkGroupSelect.append($("<option>", {"value": "Any"}).text("---"));
			this._inkGroupSelect.append($("<option>", {"value": "Process"}).text(SearchInputs.t("GRAPHICS_INKGROUP_PROCESS")));
			this._inkGroupSelect.append($("<option>", {"value": "Pantone"}).text(SearchInputs.t("GRAPHICS_INKGROUP_PANTONE")));
			this._inkGroupSelect.append($("<option>", {"value": "Designer"}).text(SearchInputs.t("GRAPHICS_INKGROUP_DESIGNER")));
			
			// Select field to enter the ink type
			this._inkTypeSelect = $("<select>", {"class": "form-control customoptionfields", "name": "inktype"});
			this._inkTypeSelect.append($("<option>", {"value": "Any"}).text("---"));
			this._inkTypeSelect.append($("<option>", {"value": "Regular"}).text(SearchInputs.t("GRAPHICS_INKCRITERIA_REGULARTYPE")));
			this._inkTypeSelect.append($("<option>", {"value": "Opaque"}).text(SearchInputs.t("GRAPHICS_INKCRITERIA_OPAQUETYPE")));
			this._inkTypeSelect.append($("<option>", {"value": "Technical"}).text(SearchInputs.t("GRAPHICS_INKCRITERIA_TECHNICALTYPE")));
			this._inkTypeSelect.append($("<option>", {"value": "Varnish"}).text(SearchInputs.t("GRAPHICS_INKCRITERIA_VARNISHTYPE")));
			
			// Delete button/icon to delete the row
			this._deleteButton = $("<i>", {"name": "customoptionbuttons", "class": "icon icon-WCR_Actions-Delete12", "title": SearchInputs.t("GRAPHICS_INKCRITERIA_REMOVEINK")}).css({"cursor": "pointer"}).hide();
			this._deleteButton.click(_.bind(function(event) {
				this._deleteClickHandler(event);
			}, this));
			
			
			this._inkRow = $("<tr>").append(
					$("<td>").append(this._inkNameInput),
					$("<td>").append(this._inkGroupSelect),
					$("<td>").append(this._inkTypeSelect),
					$("<td>", {"name": "deletebuttoncell"}).css({"width": "30px"}).append(this._deleteButton)
			);
			
			/*
			 * Mouse hover event to show/hide delete button
			 */
			this._inkRow.hover(_.bind(function(event) { 
					this._deleteButton.show(); 
				}, this), _.bind(function(event) { 
					this._deleteButton.hide(); 
				}, this));
			
		};
		
		/**
		 * Method to get the html jquery element for the row
		 * @returns
		 */
		InkSearchRow.prototype.getInkRowJQElement = function() {
			return this._inkRow;
		};
		
		/**
		 * Method to attach the delete button to html DOM, if detached before
		 */
		InkSearchRow.prototype.attachDeleteButton = function() {
			if(this._isDeleteDetached) {
				$("td[name='deletebuttoncell']", this._inkRow).append(this._deleteButton);
				this._isDeleteDetached = false;
			}
		};
		
		/**
		 * Method to detach the delete button from html DOM
		 */
		InkSearchRow.prototype.detachDeleteButton = function() {
			if(!this._isDeleteDetached) {
				this._deleteButton.detach();
				this._isDeleteDetached = true;
			}
		};
		
		/**
		 * Method to get the ink row information
		 * @returns
		 */
		InkSearchRow.prototype.getInkRowData = function() {
			var inkRowData = null;
			
			// if ink name/group/type entered
			if(this._inkNameInput.val() || (this._inkGroupSelect.val() !== "Any") || (this._inkTypeSelect.val()  !== "Any")) {
				inkRowData = {};
				inkRowData.ink_name = this._inkNameInput.val();
				inkRowData.ink_group = this._inkGroupSelect.val();
				inkRowData.ink_type = this._inkTypeSelect.val();
			}
			
			return inkRowData;
		};
		
		/**
		 * Method to set the ink row information
		 * @param inkname
		 * @param inkgroup
		 * @param inktype
		 */
		InkSearchRow.prototype.setInkRowData = function(inkname, inkgroup, inktype) {
			this._inkNameInput.val(inkname);
			this._inkGroupSelect.val(inkgroup);
			this._inkTypeSelect.val(inktype);
		};
		
		/**
		 * To add callback for cancel button
		 * @param callback
		 */
		InkSearchRow.prototype.onDelete = function(callback) {
			this._onDeleteCallbacks.push(callback);
		};
		
		return InkSearchRow;
	}
);

