define(["component/searchresults/internal/SearchResultsRerunSectionControl", "underscore"], function (SearchResultsRerunSectionControl, _) {
	"use strict";

	
	/**
	 * @class
	 * @augments SearchResultsRerunContainer
	 * @param {Element} searchresultstable - The DOM element contain the results table html.
	 * @param {Object} mainsearchresultscomponent - Needs to have ONE function: doRerunAfterBatchActions
	 * @param {SearchSetupInformation} setupInfo - information about the search setup (columns to show, ...)
	 * @param {SearchModelFrontEndData} searchData - Data which is passed from the server...
	 * @param {ChildSeachRenderer} childSearchTableRenderer - renderer used to display the results
	 */
	var ChildSearchResultsRerunSectionControl = function (searchresultstable, mainsearchresultscomponent, setupInfo, searchData, childSearchTableRenderer) {
		this._tablerenderer = childSearchTableRenderer;
		SearchResultsRerunSectionControl.call(this, searchresultstable, mainsearchresultscomponent, setupInfo, searchData);

	};

	ChildSearchResultsRerunSectionControl.prototype = Object.create(SearchResultsRerunSectionControl.prototype);
	ChildSearchResultsRerunSectionControl.prototype.constructor = ChildSearchResultsRerunSectionControl;

	/**
	 * @override
	 */
	ChildSearchResultsRerunSectionControl.prototype._initSearchTableRenderer = function () {
		// make sure the table renderer has the correct setup info and searchdata
		this._tablerenderer.setSearchData(this._searchData);
		this._tablerenderer.setSetupInfo(this._setupInfo); 
		return this._tablerenderer;
	};
	

	/**
	 * @override
	 */
	ChildSearchResultsRerunSectionControl.prototype._initialize = function (parameters) {
		this._tablerenderer.setSearchHTML(this.$div);
		SearchResultsRerunSectionControl.prototype._initialize.call(this, parameters);
	};
	
	/**
	 * @override
	 */
	ChildSearchResultsRerunSectionControl.prototype._render = function (div, parameters) {
		// Look for the place to put the actual table with results... 
		var tables = $(".searchresultstable", div);
		if (tables.length === 0) {
			this._tablerenderer.addNoChildrenRow();
		} else {
			$.each(tables, _.bind(function (ti, t) {
				// Display the results...
				this._tablerenderer.render($(t), parameters);
			}, this));
		}
	};

	return ChildSearchResultsRerunSectionControl;
});
