/*
 * author Jeroen Penninck, 2015
 * 
 * Search Actions on Projects.
 */

define(["utils/webi18n!SearchActions"], function(langBundle){
	
	/**
	 * Constructor
	 */
	var SearchResultsActionsOnProjects = function($div){
		this.div = $div;
	};
	
	/**
	 * (Public) Execute a search action button action
	 * 
	 * @param actionname - the action to execute. The executer should check if it can execute this action. 
	 * @param elem - The element which was clicked on
	 * @param context - An object which represents the context. 
	 *                Contains methods: getActionOptions, getDocumentsOnPage, getProjectsOnPage. More info, see calling class...
     * @param event - The event which caused this action to be executed
	 */
	SearchResultsActionsOnProjects.prototype.executeAction = function (actionname, elem, context, event) {
		var projects = context.getProjectsOnPage();
		var selectedproj = _.filter(projects, function(proj){return proj.isselected;});
		var nr_selected_proj = selectedproj.length;
		
		//
		// Check number of documents selected
		//
		
		switch(actionname) {
		case "delete":
		case "deleteviewfiles":
		case "deleteandregenerateviewfiles":
			// At least one document selected
			{
				if (nr_selected_proj == 0) {
					alert(langBundle.t("SELECTPROJ"));
					return;
				}
			}
			break;
		}
		
		if(actionname==="delete") {
			if (!confirm(langBundle.t('CONFIRM_DELETE'))) {
				return;
			}
			
			// redirect to the page to delete the projects
			this.submitSimpleAction("MultiDeleteProject", event);
		}else if(actionname==="deleteviewfiles"){
			if (!confirm(langBundle.t('CONFIRM_DELETING_VIEW_FILES'))) {
				return;
			}
			// redirect to the page to delete the projects
			this.submitSimpleAction("DeleteViewFiles", event);
		}else if(actionname==="deleteandregenerateviewfiles"){
			if (!confirm(langBundle.t('CONFIRM_REGENERATE_VIEW_FILES'))) {
				return;
			}
			// redirect to the page to delete the projects
			this.submitSimpleAction("DeleteAndRegenerateViewFiles", event);
		}
		// other actions... => see SearchResultsBatchActions... (actionname="searchaction")
	};
	
	/**
	 * ACTION: Submit a simple action
	 * This is done by submitting a form, but could become an ajax call in the future...
	 * 
	 * @param action
	 */
	SearchResultsActionsOnProjects.prototype.submitSimpleAction = function(action, event) {
		// Note: we could get rid of this form quite easily...
		// Just use ajax calls... We have all the required information...
		var actionform = $(".searchresultsactionform", this.div);
		var actionvalueinput = $("[name=actionVal]", actionform);
		actionvalueinput.val(action);
		actionform.get(0).submit();
		if(event) {
			event.stopPropagation();
		}
	};
	
	return SearchResultsActionsOnProjects;
});