define([
	"jquery",
	"underscore",
	"wcr/webcenter_xml",
	"./AssetBrowserSelectionDisplay"
], function (
	$,
	_,
	WCXML,
	AssetBrowserSelectionDisplay
) {
	"use strict";

	/**
	 * @param {DOMElement} div - the DOM element to put this component in
	 * @param {AssetBrowserModel} model - the model for this Asset Browser
	 */
	var AssetBrowserImagePane = function (div, model) {
		this._div = div; // .searchassetbrowserImageArea
		this._contentdiv = $("<div>").appendTo($(this._div));
		this._m = model;

		// Because of old code... Need some way of accessing this object using a global.
		if (!window.assetbrowserimagepane) {
			window.assetbrowserimagepane = {};
		}
		window.assetbrowserimagepane[this._m.id] = this;
		this._functionPrefix = "window.assetbrowserimagepane['" + this._m.id + "']";
		// End of old code workaround

		// Make the selection component (only once, in constructor)
		this._selectioncomponentdiv = $('<div style="width:150px;overflow-x:auto" id="' + this._m.id + 'CurrentSelectionDiv" class="searchassetbrowserCurrentSelectionDiv">');
		this._selectiondisplay = new AssetBrowserSelectionDisplay(this._selectioncomponentdiv.get(0), this._m);
		
		//
		// Mirror the view with the current and future states of the model =>
		//

		// When the focus changes, update what is shown
		this._m.on("focuschange", _.bind(this._update, this));

		// When the selection changes in the model, update the checkboxes
		this._m.on("selectionchange", _.bind(this._changeSelectionView, this));
		
		// When the visibility changes in the model, update the visibility of the view
		this._m.on("visibility", _.bind(this._updateVisibility, this));
		
		// And simulate the events we might have missed (before this object was constructed)
		// Note: if the panel is visible => this will call the _update also !!
		this._updateVisibility();
		
		// Resize the image panel when the browser is resizing
		$(window).on("resize", _.debounce(_.bind(this._updateSize, this), 200));
	};
	
	/**
	 * Update the size of the asset browser image panel
	 */
	AssetBrowserImagePane.prototype._updateSize = function() {
		// Measure the available space (make sure we don't influence the size ourself!)
		$(this._div).css({'overflow':'hidden', 'width':'100%'});
		this._contentdiv.css({'display':'none'});
		var curWidth = $(this._div).width();
		this._contentdiv.css({'width':curWidth+'px', 'display':''});
		$(this._div).css({'overflow':'', 'width':''});
	};

	/**
	 * @param elemId the id of the currently selected element in the tree
	 */
	AssetBrowserImagePane.prototype._update = function () {

		// If not shown, don't bother: 
		if (!this._m.isImagePanelVisible()) {
			return;
		}

		// What is the current "focus" in the list => need to show that in the image area!
		var elemId = this._m.getFocusTreeId();
		var idarr = this._m.convertIdToArray(elemId);

		// Get the focused node in the tree
		var treeNode = this._m.getTreeNodeById(elemId);

		// Some functions to call when the image is loaded...
		var onloadFunction = 'WCHTML.scaleImageProportionally(this,100,100);';
		var onerrorFunction = 'WCHTML.alternativeImage(this,null);';

		// Update the minimal size
		$(this._div).css({'min-width':(this._m.isTreePanelVisible()?'350px':'500px')});
		this._updateSize();
		
		// Start building the html
		var htmlcode = '<div style="display:flex">';

		// Make some space for the current selection - if this is the only panel visible
		if (!this._m.isTreePanelVisible()) {
			htmlcode += '<div style="flex: 0 0 auto;width:auto;" id="' + this._m.id + 'CurrentSelectionImageArea">';
			// -> Here we will put the current selection table component
			htmlcode += '</div>'
		}

		// Show the current selected node...
		if (elemId != 'allLists') {
			// Alternative thumbnail = thumbnail of the list
			var listName = this._m.lists[idarr[0]].name;
			onerrorFunction = 'WCHTML.alternativeImage(this,\'' + window.webcenterConfig.jspBaseUrl + 'custom/AttributeValueImages/' + listName + '.jpg\');';

			// Get the id of one level up
			var parentId = this._m.convertArrayToId(idarr.slice(0, idarr.length - 1));

			// go back icon
			htmlcode += '<div style="flex: 0 0 auto;width:auto;"><table>';
			var clickMethod = this._functionPrefix + '._expand(\'' + parentId + '\'); ';
			htmlcode += '<tr height="100px" valign="top"><td style="height:100px;vertical-align:middle;"><i class="icon icon-WCR_Search-Left searchassetbrowserback" onclick="' + clickMethod + '"/></td></tr>';
			htmlcode += '</table></div>';

			// the currently selected element
			var attributeValue = this._m.getLabel(elemId);
			htmlcode += '<div style="flex: 0 0 auto;width:auto;"><table>';
			htmlcode += '<tr><td style="height:100px;vertical-align:middle;"><img src="' + window.webcenterConfig.jspBaseUrl + 'custom/AttributeValueImages/' + attributeValue + '.jpg" height="100" onload="' + onloadFunction + '" onerror="' + onerrorFunction + '"></td></tr>';
			htmlcode += '<tr align="center" valign="top"><td class="darktext_greenlink_11" valign="top">' + attributeValue + '</td></tr>';
			htmlcode += '</table></div><div style="flex: 0 0 auto;width:70px;"></div>';
		}

		// scrolling area with thumbnails
		htmlcode += '<div style="flex: 1 1 auto;width:100%;overflow-x:auto;" id="' + this._m.id + 'CTree_MainImageDiv"><table id="' + this._m.id + 'CTree_MainImageTable"><tr>';

		// Look at the items of this node
		_.each(treeNode.items, function (item) {
			// what is the id of this node?
			var myIdArr = idarr.concat([item.idx]);
			var myId = this._m.convertArrayToId(myIdArr);

			// information about this node
			var label = this._m.getLabel(myId);
			var canExpand = item.items.length > 0;
			var canSelect = (elemId !== 'allLists');

			// Actions you can do on this element
			var actionNoAction = '';
			var actionExpandRow = this._functionPrefix + '._expand(\'' + myId + '\');';
			var actionSelectRow = this._functionPrefix + '._select(\'' + myId + '\');';
			var actionDefault = (canExpand ? actionExpandRow : canSelect ? actionSelectRow : actionNoAction);


			// Build the xml for this node
			var attributeValue = WCXML.xmlencode(label);

			htmlcode += '<td valign="top"><table>';
			htmlcode += '<tr align="center" valign="top"><td style="height:100px;vertical-align:middle;"><img src="' + window.webcenterConfig.jspBaseUrl + 'custom/AttributeValueImages/' + attributeValue + '.jpg" height="100" onclick="' + actionDefault + '" onload="' + onloadFunction + '" onerror="' + onerrorFunction + '"></td></tr>';

			if (canSelect) {
				htmlcode += '<tr align="center"><td>'+this._m.utilCheckboxMakeHtml(this._m.id + 'ImgCB' + myId, actionSelectRow)+'</td></tr>';
			}

			htmlcode += '<tr align="center" valign="top"><td class="darktext_greenlink_11"  valign="top">' + attributeValue + '</td></tr>';
			htmlcode += '</table></td>';

			htmlcode += '<td><img src="' + window.webcenterConfig.imageBaseUrl + 'clearpixel.gif" width="30px"></td>';
		}, this);
		htmlcode += '</tr></table></div>';
		// end scrolling area with thumbnails

		// close the main table
		htmlcode += '</div></div>';

		// Remove the selection component first
		this._selectioncomponentdiv.detach();

		// Put the new content of the image panel
		this._contentdiv.html(htmlcode);
		
		// Loop through all items again to put the correct selection
		_.each(treeNode.items, function (item) {
			// what is the id of this node?
			var myIdArr = idarr.concat([item.idx]);
			var myId = this._m.convertArrayToId(myIdArr);
			
			var canSelect = (elemId !== 'allLists');
			if(canSelect) {
				var checked = false;
				var indeterminate = false;
				
				if(this._m.isSelectedOrParent(myId)) {
					checked = true;
				}else {
					if(this._m.hasSelectedChildren(myId)) {
						indeterminate = true;
					}
				}
				
				var checkbox = $("#"+ this._m.id + 'ImgCB' + myId);
				this._m.utilCheckboxSetState(checkbox, checked, indeterminate);
			}
		}, this);

		// Put the selection display component in the right place again
		if (!this._m.isTreePanelVisible()) {
			$("#" + this._m.id + "CurrentSelectionImageArea").append(this._selectioncomponentdiv);
		}
	};

	//
	// ---
	// Listeners on the view
	// ---
	//

	/**
	 * -- Listener on the view --
	 * 
	 * Function that is called when you want expand a certain row (by clicking on an image)
	 * 
	 * @param id - the id representing an index in the tree (see also AssetBrowserModel)
	 */
	AssetBrowserImagePane.prototype._expand = function (id) {
		// Toggle the expansion of a row in the model (events will update the views)
		this._m.doExpand(id, true);

		// Focus this element!
		this._m.setFocusTreeId(id);
	};

	/**
	 * -- Listener on the view --
	 * 
	 * Function that is called when you want expand a certain row (by clicking on an image)
	 * 
	 * @param id - the id representing an index in the tree (see also AssetBrowserModel)
	 */
	AssetBrowserImagePane.prototype._select = function (id) {
		// Toggle the selection of a checkbox in the model (events will update the views)
		this._m.toggleSelection(id);
	};

	//
	// ---
	// Handle Showing the current state of the selection
	// ---
	//

	/**
	 * Called when the selection changes in the model
	 */
	AssetBrowserImagePane.prototype._changeSelectionView = function (id, selected) {
		// 1) Go through all parents
		//    => Show as some selected or not selected
		var idarr = this._m.convertIdToArray(id);
		var parentid = idarr[0];
		var parentsHaveSelectedChildren = selected;
		for(var i=1;i<idarr.length-1;i++) {
			parentid+="_"+idarr[i];
			// Note: only execute hasSelectedChildren if we are not sure whether we have selected children
			parentsHaveSelectedChildren = parentsHaveSelectedChildren || this._m.hasSelectedChildren(parentid);
			var chkbxElem = $("#" + this._m.id + "ImgCB" + parentid);
			this._m.utilCheckboxSetState(chkbxElem, false, parentsHaveSelectedChildren);
		}
		
		// 2) Find myself and show as selected
		this._m.utilCheckboxSetState($("#" + this._m.id + "ImgCB" + id), selected, false);
		
		// 3) Find all children and show them as selected
		this._m.utilCheckboxSetState($("[id^=" + this._m.id + "ImgCB" + id+"_]"), selected, false);
	};
	
	//
	// ---
	// Handle Visibility Changes
	// ---
	//
	
	/**
	 * -- Listener on model --
	 * 
	 * When the visibility changes in the model, hide or display this component
	 */
	AssetBrowserImagePane.prototype._updateVisibility = function() {
		if(this._m.isImagePanelVisible()) {
			$(this._div).css("display","");
			// Might need an update... => The selection could have been moved outside of the image area!
			// Also, we don't do updates when we are not visible...
			this._update();
		} else {
			$(this._div).css("display","none");
		}
	};


	return AssetBrowserImagePane;
});