define([
	"underscore",
	"jquery",
	"utils/webi18n!ACatStrings"
], function (
	_,
	$,
	ACatStrings
) {
	"use strict";

	/**
	 * A component showing the overview of current selection
	 * 
	 * @param {DOMElement} div - the DOM element to put this component in
	 * @param {AssetBrowserModel} model - the model for this Asset Browser
	 */
	var AssetBrowserSelectionDisplay = function (div, model) {
		this._div = div;
		this._m = model;

		this._m.on("selectionchangefinal", _.bind(this._update, this));

		this._update();
	};

	/**
	 * Update the content of the selection display
	 */
	AssetBrowserSelectionDisplay.prototype._update = function () {
		var table = [];

		// Make empty tables for each list
		_.each(this._m.lists, function () {
			table.push($("<table>"));
		});

		// Loop through all the selections
		_.each(this._m.getSelection(), function (selectId) {
			// Which list is this selection in?
			var idarr = this._m.convertIdToArray(selectId);
			var listIndex = idarr[0];

			// Add some row to the display for this list
			var labels = this._m.getLabels(selectId);
			var labelfull = labels[0];
			for (var i = 1; i < labels.length; i++) {
				labelfull += ' ~ ' + labels[i];
			}

			// Make the element
			var deleteIcon = $('<i class="icon icon-WCR_Remove assetbrowserremoveselection"/>');

			// What to do when you click that icon?
			deleteIcon.click(_.bind(function () {
				// Delete this id from the model:
				this._m.doSelect(selectId, false);
			}, this));

			// Build the complete row
			table[listIndex].append(
				$('<tr>').append(
					$('<td>').append(deleteIcon),
					$('<td class="current-selection-td-details">').text(labelfull)
				)
			);
		}, this);

		// Make the root table
		var root = $('<table>').attr("id", this._m.id + 'CurrentSelectionTable').append(
			$('<tr>').append(
				$('<td class="selection-header">').text(ACatStrings.t('ACatCurrentSelection'))
			)
		);

		// Make a row for each cascading list
		for (var list = 0; list < this._m.cascadingLists.length; list++) {
			root.append(
				$('<tr>').append(
					$('<td>').append(table[this._m.cascadingLists[list].listIdx])
				)
			);
		}

		// Update the contents of the parent div
		$(this._div).empty().append(root);
	};

	return AssetBrowserSelectionDisplay;
});