define([
	"jquery",
	"underscore",
	"utils/webi18n!ACatStrings"
], function (
	$,
	_,
	ACatStrings
) {
	"use strict";

	/**
	 * @param {DOMElement} div - the DOM element to put this component in
	 * @param {AssetBrowserModel} model - the model for this Asset Browser
	 */
	var AssetBrowserVisibilityDropdown = function (div, model) {
		this._div = div;
		this._m = model;

		this._initialize();
	};

	/**
	 * Make the dropdown
	 */
	AssetBrowserVisibilityDropdown.prototype._initialize = function () {
		// Make the Asset Browser View Dropdown
		this._viewdropdown = $("<select>", {
			"class": "assetviewdropdown form-control",
			"id": this._id + "treeDisplaySelect"
		});
		this._viewdropdown.append($("<option>", {
			"value": "LeftPanelOnly",
			"text": ACatStrings.t('ACatSearchLeftPanelOnly')
		}).prop("selected", true));
		this._viewdropdown.append($("<option>", {
			"value": "ImagePanelOnly",
			"text": ACatStrings.t('ACatSearchImagePanelOnly')
		}));
		this._viewdropdown.append($("<option>", {
			"value": "LeftAndImagePanel",
			"text": ACatStrings.t('ACatSearchLeftAndImagePanel')
		}));

		// Add to the correct place
		$(this._div).append(this._viewdropdown);

		// Attach listener to the dropdown
		this._viewdropdown.on("change", _.bind(this._onDropdownChange, this));

		// Attach listener to the model
		this._m.on("visibility", _.bind(this._onModelChange, this));
		
		// And simulate the events we might have missed (before this object was constructed)
		this._onModelChange();
	};

	/**
	 * Reset the dropdown to the value in the object model
	 */
	AssetBrowserVisibilityDropdown.prototype._onModelChange = function () {
		this._viewdropdown.val(this._m.getDisplayVisibilityOption());
	};

	/**
	 * If a dropdown changes, update the model
	 */
	AssetBrowserVisibilityDropdown.prototype._onDropdownChange = function () {
		if (this._viewdropdown.val() !== this._m.getDisplayVisibilityOption()) {
			this._m.changeDisplayVisibilityOption(this._viewdropdown.val());
		}
	};

	return AssetBrowserVisibilityDropdown;
});