define(["underscore"], function(_) {
	
	/**
	 * Manages the browser back button for search
	 */
	var SearchBrowserHistoryManager = function (searchid) {
		this._enabled = !!(window["history"]);
		this._searchid = "search"+searchid;
	};
	
	/**
	 * Get the history of ANY searches from the history.
	 * If it does not exist, it will create an object which can be placed in the history if needed.
	 * 
	 * @returns
	 */
	SearchBrowserHistoryManager.prototype._getBaseSearchBrowserHistoryState = function() {
		if(this._enabled) {
			if(window.history.state) {
				// So, we have state!
				if(!window.history.state["search"]) {
					// If we don't have a search section, add it.
					// Note: keep the other information that might be in the state.
					// We have state for SOME searches on this page...
					window.history.state.search = [];
				}
				return window.history.state;
			}
		}
		// Oh, no state or no history support
		return {search:{}};
	};
	
	/**
	 * When using the browser back button => this method will return the data that was on the page before.
	 * 
	 * @returns {any} the data which was stored in the history of the current page
	 */
	SearchBrowserHistoryManager.prototype.getRestoreData = function() {
		var searchhistorydata = this._getBaseSearchBrowserHistoryState();
		if(_.has(searchhistorydata.search, this._searchid)) {
			return searchhistorydata.search[this._searchid];
		}
	};
	
	/**
	 * When doing some changes to the setup => this method will store the changes
	 * 
	 * @param {any} searchdata - the data to store in the history of the current page
	 */
	SearchBrowserHistoryManager.prototype.storeState = function(searchdata) {
		var searchhistorydata = this._getBaseSearchBrowserHistoryState();
		searchhistorydata.search[this._searchid] = searchdata;
		history.replaceState(searchhistorydata, "");
	};
	
	// return the constructor
	return SearchBrowserHistoryManager;
});