define(["underscore", "utils/webi18n!SearchInputs"], function(_, bundle) {
	
	/**
	 * @constructor
	 * 
	 * @param {jQuery} div - the container for the search setup page
	 * @param {SearchSetupInformation} setupInfo - the Search Setup Object Model
	 */
	var SpecialCharactersHintController = function(div, setupInfo) {
		this.div = div;
		this._alreadyAsked = false;
		
		this._setupInfo = setupInfo;
	};
	
	/**
	 * Whether we should check whether the user starts typing special characters...
	 * 
	 * We don't need to check if we already asked or if he is already using them.
	 */
	SpecialCharactersHintController.prototype.needToCheck = function() {
		return !this._setupInfo.getFeatures().getUseAdvancedChars() && !this._alreadyAsked; 
	};
	
	
	/**
	 * Trigger the hint.
	 * 
	 * In the new architecture, the SearchParameterFieldControl will trigger the hint when a field starts using special characters.
	 */
	SpecialCharactersHintController.prototype.trigger = function() {
		if(this.needToCheck()) {
			// Where to put the warning?
			var warningcontainer = $(".searchwarningcontainer", this.div);
			
			// Make the warning message
			var warningdiv = $("<div>", {'class':'alert alert-warning alert-dismissible', 'style':'margin-bottom:0px'});
			var closeicon = $('<button type="button" class="close" data-dismiss="alert">&times;</button>');
			var messagediv = $("<div>", {'style':'padding-bottom:10px', 'text':bundle.t('ADV_CHAR_WARNING').replace(/\n/g, "_newline_")});
			var buttonok = $("<button>", {'type': 'button', 'class':'btn btn-warning', 'text': bundle.t('ADV_CHAR_WARNING_OK')});
			var buttonclose = $("<button>", {'type': 'button', 'class':'btn btn-default', 'text': bundle.t('ADV_CHAR_WARNING_CLOSE'), 'data-dismiss':'alert'});
			
			// fix newlines
			messagediv.html(messagediv.html().replace(/_newline_/g, "<br/>"));
			
			warningdiv.append(closeicon).append(messagediv).append(buttonok).append(" ").append(buttonclose);
			
			warningcontainer.append(warningdiv);
			
			// When you click the ok button
			buttonok.click(_.bind(function(){
				this._setupInfo.getFeatures().setUseAdvancedChars(true);
				warningdiv.remove();
			}, this));
			
			// When you change the "special characters checkbox" while the alert is being shown
			this._setupInfo.getFeatures().onUseAdvancedCharsChange(_.bind(function(){
				warningdiv.remove();
			}, this));
			
			// Don't ask this again
			this.setDontAskAgain();
		}
	};
	
	/**
	 * Don't show the hint anymore
	 */
	SpecialCharactersHintController.prototype.setDontAskAgain = function() {
		this._alreadyAsked = true;
	};
	
	return SpecialCharactersHintController;
});