/**
 * Information about the design panel items
 */
define([
        "./SearchItemsBase",
        "./SearchDesignPanelItem"
        ], function(
        		SearchItemsBase,
        		SearchDesignPanelItem
        		){
	/**
	 * @constructor
	 */
	var SearchDesignPanelItems = function(){
		SearchItemsBase.call(this, SearchDesignPanelItem, true);
	};
	
	// Extend from SearchItemsBase to get listener functionality and such
	SearchDesignPanelItems.prototype = Object.create(SearchItemsBase.prototype);
	SearchDesignPanelItems.prototype.constructor = SearchDesignPanelItems;
	
	//
	// Functions available on this object:
	//
	// + each = loop through the design panel items
	// + add = add a design panel item to the list
	// + findByFieldId = Find a design panel item by its fieldid
	// + reorderItemsByIds = Reorder the list (or part of it) by a given list of ids. Does not have any effect on the values.
	// + tryToRemoveItem = Remove a design panel item. 
	//				Note, will fail if there is something which is editing the design panel item settings... 
	//				(see take() & release() in SearchDesignPanelItem)
	// + onChange = Register a new change listener
	// + commitChanges = We don't want to invoke change listeners while the change we are doing is not complete.
	// 				So, in general, we invoke the change listeners the frame after changes occured.
	// 
	// 				However, sometimes you want to do it sooner. That is when you call this method.
	// 				E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	// 
	// 				Note: you do not need to call this method! (but you probably should)
	// + createItem =  Creates a new design panel item object. (Just a method for ease of use.)
	// 				Note: this method DOES NOT add the design panel item to this list
	//
	
	//
	// Other functions specific to this type of list:
	//
	
	/**
	 * Handy method to get a correctly translated label.
	 * 
	 * @param {SearchField} the field to get the label for.
	 */
	SearchDesignPanelItems.prototype.getCustomizedLabel = function (searchField) {
		var designpanelitem = this.findByFieldId(searchField.fieldId);
		if(designpanelitem && designpanelitem.getLabel()) {
			return designpanelitem.getLabel();
		} else {
			return searchField.getDefaultLabel();
		}
	};
	
	return SearchDesignPanelItems;
});