/**
 * Information about the display columns
 */
define([
        "./SearchItemsBase",
        "./SearchDisplayColumn"
        ], function(
        		SearchItemsBase,
        		SearchDisplayColumn
        		){
	/**
	 * @constructor
	 */
	var SearchDisplayColumns = function(){
		SearchItemsBase.call(this, SearchDisplayColumn, true);
	};
	
	// Extend from SearchItemsBase to get listener functionality and such
	SearchDisplayColumns.prototype = Object.create(SearchItemsBase.prototype);
	SearchDisplayColumns.prototype.constructor = SearchDisplayColumns;
	
	//
	// Functions available on this object:
	//
	// + each = Loop through all the display columns
	// + add = add a display column to the list
	// + findByFieldId = Find a display column by its fieldid
	// + reorderItemsByIds = Reorder the list (or part of it) by a given list of ids. Does not have any effect on the values.
	// + tryToRemoveItem = Remove a display column. 
	//				Note, will fail if there is something which is editing the display column settings... 
	//				(see take() & release() in SearchDisplayColumn)
	// + onChange = Register a new change listener
	// + commitChanges = We don't want to invoke change listeners while the change we are doing is not complete.
	// 				So, in general, we invoke the change listeners the frame after changes occured.
	// 
	// 				However, sometimes you want to do it sooner. That is when you call this method.
	// 				E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	// 
	// 				Note: you do not need to call this method! (but you probably should)
	// + createItem =  Creates a new display column object. (Just a method for ease of use.)
	// 				Note: this method DOES NOT add the display column item to this list
	//
	
	return SearchDisplayColumns;
});