define([
        "utils/Observable"
        ], function(Observable){
	
	var SearchFieldFeaturesInfo = function() {
		this._useSmartNames = false;
		this._useSpecialChars = false;
		
		// This object is observable!
		Observable.mixin(this, "model");
	};

	/**
	 * Are SmartNames enabled?
	 * 
	 * @return {Boolean}
	 */
	SearchFieldFeaturesInfo.prototype.getUseSmartNames = function() {
		return this._useSmartNames;
	};

	/**
	 * Set the information: are SmartNames enabled?
	 * 
	 * @param {Boolean} useSmartNames - the new setting
	 */
	SearchFieldFeaturesInfo.prototype.setUseSmartNames = function(useSmartNames) {
		this._useSmartNames = useSmartNames;
	};
	
	/**
	 * Are special characters enabled?
	 * 
	 * @returns {Boolean}
	 */
	SearchFieldFeaturesInfo.prototype.getUseAdvancedChars = function() {
		return this._useSpecialChars;
	};
	
	/**
	 * Set the information: are special characters enabled?
	 * 
	 * @param useSpecialChars
	 */
	SearchFieldFeaturesInfo.prototype.setUseAdvancedChars = function(useSpecialChars) {
		this._useSpecialChars = useSpecialChars;
		this._emit("advancedcharschange");
	};
	
	/**
	 * Listen to changes in the advanced search characters checkbox
	 * 
	 * @param {Function} callback - the callback to execute when the advanced characters checkbox changes.
	 */
	SearchFieldFeaturesInfo.prototype.onUseAdvancedCharsChange = function(callback) {
		this.on("advancedcharschange", callback);
	};
	
	return SearchFieldFeaturesInfo;
});