/**
 * Information about the sorting criterions
 */
define([
        "./SearchItemsBase",
        "./SearchSortingCriterion"
        ], function(
        		SearchItemsBase,
        		SearchSortingCriterion
        		){
	/**
	 * @constructor
	 */
	var SearchSortingCriteriaList = function(){
		SearchItemsBase.call(this, SearchSortingCriterion, true);
	};
	
	// Extend from SearchItemsBase to get listener functionality and such
	SearchSortingCriteriaList.prototype = Object.create(SearchItemsBase.prototype);
	SearchSortingCriteriaList.prototype.constructor = SearchSortingCriteriaList;
	
	//
	// Functions available on this object:
	//
	// + each = loop through the sorting criteria
	// + add = add a sorting criterion to the list
	// + findByFieldId = Find a sorting criterion by its fieldid
	// + reorderItemsByIds = Reorder the list (or part of it) by a given list of ids. Does not have any effect on the values.
	// + tryToRemoveItem = Remove a sorting criterion. 
	//				Note, will fail if there is something which is editing the sorting criterion settings... 
	//				(see take() & release() in SearchSortingCriterion)
	// + onChange = Register a new change listener
	// + commitChanges = We don't want to invoke change listeners while the change we are doing is not complete.
	// 				So, in general, we invoke the change listeners the frame after changes occured.
	// 
	// 				However, sometimes you want to do it sooner. That is when you call this method.
	// 				E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	// 
	// 				Note: you do not need to call this method! (but you probably should)
	// + createItem =  Creates a new sorting criterion object. (Just a method for ease of use.)
	// 				Note: this method DOES NOT add the sorting criterion to this list
	//
	
	return SearchSortingCriteriaList;
});