/**
 * A part of a string with SmartNames.
 * 
 * The string with SmartNames is split up in objects like this. See SmartNameEncoding.
 */

/*
 *
 * !!! THE IMPLEMENTATION IN THIS FILE SHOULD MATCH THE IMPLEMENTATION IN THE JAVA FILE "SmartNameStringPart.java".
 *
 */

define(function(require, exports, module) {
"use strict";
    
    /**
	 * Constructor is private
	 */
    var SmartNameStringPart = function(){};
    
    (function(){
        
        /**
         * Is this part of the string a SmartName?
         * 
         * @return true if this part is a SmartName
         */
        this.isSmartNamePart = function() {
            return this._smartName !== null;
        };

        /**
         * Is this part of the string just text?
         * 
         * @return true if this part is just text.
         */
        this.isTextPart = function() {
            return this._text !== null;
        };

        /**
         * Get the SmartName this part represents.
         * 
         * @return the SmartName this part represents or null if this is not a SmartName (and thus normal text)
         */
        this.getAsSmartName = function() {
            return this._smartName;
        };

        /**
         * Get the normal text this part represents.
         * 
         * @return the normal text this part represents or null if this is not plain text (and thus a SmartName)
         */
        this.getAsText = function() {
            return this._text;
        };
        
    }).call(SmartNameStringPart.prototype);
    
    /**
	 * Make a part of a SmartName string which is a SmartName
	 * 
	 * @param smartName {SmartName}
	 *            The SmartName to add to the string.
	 * @return
	 */
    exports.makeSmartNamePart = function(smartName) {
		if (smartName == null) {
			throw "A \"SmartName\" part of a SmartName string can not be null.";
		}
		var part = new SmartNameStringPart();
		part._smartName = smartName;
        part._text = null;
		return part;
	};
    
    /**
	 * Make a part of a SmartName string which is plain text
	 * 
	 * @param text
	 *            The text to add to the string.
	 * @return
	 */
    exports.makePlainTextPart = function(text) {
		if (text == null) {
			throw "A \"Text\" part of a SmartName string can not be null.";
		}
		var part = new SmartNameStringPart();
        part._smartName = null;
		part._text = text;
		return part;
	};
    
});