define(["jquery",
        "./smartnameeditpanel-view",
        "component/smartnames/encoding/smartname"
        ],
	function ($, SmartNameEditPanelView, SmartName) {
		"use strict";

		/**
		 * Panel to edit a smartname.
		 * @param {SmartName} smartName            - The smartname to edit
		 * @param {Object}    smartNameEditOptions - Options for the smartname to edit. Example:
		 *                                         {labels: ["label1", "label2"], // translated labels for the editable parameters
		 *                                         hasStrictParams: true, // true for strict set of parameters, false for dynamic amount of parameters
		 *                                         smartnameInputOptions: {Object} // see options of SmartNameInput. This is used to construct the correct/same smartfields for the parameters as its parent.
		 *                                         }
		 */
		var SmartNameEditPanel = function (smartName, smartNameEditOptions) {

			// See what arguments are already set in the smartname ...
			this._name = smartName.getName();
			this._arguments = smartName.getArguments();

			// Create the view ...
			this._view = new SmartNameEditPanelView(this._name, this._arguments, smartNameEditOptions);
		};

		/**
		 * The name of the smartname
		 * @returns {String}
		 */
		SmartNameEditPanel.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Get the arguments of the SmartName
		 * 
		 * @return {Array.<String>} the arguments of the SmartName
		 */
		SmartNameEditPanel.prototype.getArguments = function () {
			return this._view.getArguments();
		};

		/**
		 * Get the smartname of this panel (copy)
		 * @returns {SmartName}
		 */
		SmartNameEditPanel.prototype.getSmartName = function () {
			return new SmartName(this.getName(), this.getArguments());
		};

		/**
		 * Add callback to execute on save
		 * @param {Function} callback
		 */
		SmartNameEditPanel.prototype.onSave = function (callback) {
			this._view.onSave(callback);
		};

		/**
		 * Add callback to execute on cancel
		 * @param {Function} callback
		 */
		SmartNameEditPanel.prototype.onCancel = function (callback) {
			this._view.onCancel(callback);
		};

		/**
		 * Get the view element of the panel
		 * @returns {JQueryObj}
		 */
		SmartNameEditPanel.prototype.getJQElement = function () {
			return this._view.getJQElement();
		};

		/**
		 * Disable the elements of the smartname edit panel
		 */
		SmartNameEditPanel.prototype.disable = function () {
			this._view.disable();
		};

		/**
		 * Enable the elements of the smartname edit panel
		 */
		SmartNameEditPanel.prototype.enable = function () {
			this._view.enable();
		};

		/**
		 * Called when the panel is resized/moved
		 */
		SmartNameEditPanel.prototype.resize = function () {
			this._view.resize();
		};

		/**
		 * Called when the edit panel has to be destroyed
		 */
		SmartNameEditPanel.prototype.destroy = function () {
			this._view.destroy();
		};

		return SmartNameEditPanel;

	});