/**
 * In a Search with SmartNames, you want to be able to use the values of the attributes that are currently on that page.
 * 
 * @author JEPE
 */
define(["lib/oop", "./DynamicSmartNameCollector", "./AttributeSmartNamesUtil", "component/smartnames/encoding/smartnameargumentencoding"], function(oop, DynamicSmartNameCollector, AttributeSmartNamesUtil, SmartNameArgumentEncoding){
	"use strict";
	
	/**
	 * Constructor for AttributeCategorySmartNameCollector.
	 * 
	 * @param attributeCategory {AttributeCategory} a reference to the attributeCategory object.
	 */
	var AttributeCategoryDynamicSmartNameCollector = function(attributeCategory) {
		this._attributeCategory = attributeCategory;
	};
	
	// inherit functions from DynamicSmartNameCollector
	oop.inherits(AttributeCategoryDynamicSmartNameCollector, DynamicSmartNameCollector);
	
	AttributeCategoryDynamicSmartNameCollector.prototype.smartNamePrefix = {
			"attrcatsetup": ["ProjAttr", "DocAttr"],
			"project": ["ProjAttr"],
			"document": ["DocAttr"]
	};
	
	// override the collect method
	/**
	 * Run this collector and the ones after it. Save all the current values of the SmartNames in the given dynamicsmartnamecollection.
	 * 
	 * @param dynamicsmartnamecollection {DynamicSmartNameCollection} the collection to add the SmartNames into.
	 */
	AttributeCategoryDynamicSmartNameCollector.prototype.collect = function(dynamicsmartnamecollection, onDone) {
		var self = this;
		var type = self._attributeCategory.type;
		var attributePrefix = self.smartNamePrefix[type || "attrcatsetup"] || ["ProjAttr", "DocAttr"];
		var attributes = self._attributeCategory.attributes;
		
		var attrUtil = new AttributeSmartNamesUtil();
		
		// loop through all attributes in the attribute category and add them as a SmartName.
		for(var i=0;i<attributes.length;i++) {
			var attrObj = attributes[i];
			
		    var name = attrObj.name;
		    var values = attrUtil.attributeValueToSmartNameValue(attrObj);
			
		    for(var a=0;a<attributePrefix.length;a++) {
		    	dynamicsmartnamecollection.put(SmartNameArgumentEncoding.DEFAULT.encodeSmartName(attributePrefix[a],[name]), values);
		    }
		}

		// continue...
		self._runNext(dynamicsmartnamecollection, onDone);
	};
	
	return AttributeCategoryDynamicSmartNameCollector;
});