/*
 *
 * !!! THE IMPLEMENTATION IN THIS FILE SHOULD MATCH THE IMPLEMENTATION IN THE JAVA FILE "AttributeSmartNamesUtil.java".
 *
 */
define(["wcr/date"], function(date_module){
	"use strict";

	/**
	 * Create a {@link AttributeSmartNamesUtil}.
	 * 
	 * @param timeZoneOffsetMillies
	 *            The timezone offset to use when resolving date related smart names.
	 */
	var AttributeSmartNamesUtil = function(/*timeZoneOffsetMillies*/){
		//this.timeZoneOffsetMillies = timeZoneOffsetMillies;
	};
	
	/**
	 * Format the date values of a smart name in the requested encoding and the user's timezoneoffset.
	 * 
	 * @param datearr
	 *            Array with the value of the smart name.
	 * @param formatString
	 *            The string indicating how the dates should be formatted.
	 * @return The list of formatted date strings.
	 */
	AttributeSmartNamesUtil.prototype.formatDateSmartNameValue = function(datearr, formatString) {
	
		// Construct the dateformat
		if (formatString === undefined) {
			// Default encoding
			formatString = "dd.MM.yyyy HH:mm a";
		}
	
		// Format the date values.
		var values = [];
		for (var i=0;i<datearr.length;i++) {
			var date = datearr[i];
			values.push(formatDate(date, formatString));
		}
		return values;
	};
	
	/**
	 * Resolve this attribute value to a Collection of Strings. Most attributes will just be resolved to their String
	 * representation but must
	 * 
	 * @param attributeUnitType
	 * @param attrValue
	 * @return
	 */
	AttributeSmartNamesUtil.prototype.attributeValueToSmartNameValue = function(attribute) {
	
		var attributeValue = [];
		
		// If it's a text value, it can be
		if(attribute.isText() || attribute.isDocReference()) {
			if (attribute.value.indexOf(" | ")!==-1) {
				// Looks like this is a multiple value attribute ...
				// Note: I really hate how this is done but I have no choice here ... for now ...
				attributeValue = attribute.value.split(/\s\|\s/);
			} else {
				// Single value attribute, just return the string.
				attributeValue.push(attribute.value);
			}
			// Note: the value of a document reference attribute (on the front-end) is the document name (or if multiple, separated by | )
		} else if(attribute.type == "DateTime") {
			// If it's a date value ...
			try {
				// Try to parse and format the date
				attributeValue = attributeValue.concat(this.formatDateSmartNameValue([attribute.value]));
			} catch (e) {
				// Something went wrong, just insert the valueString like that.
				attributeValue.push(attribute.value);
			}
		}else{
			// Default case
			attributeValue.push(attribute.value);
		}
	
		return attributeValue;
	};
	
	return AttributeSmartNamesUtil;
});