/**
 * In a Search with SmartNames, you want to be able to use the values of the attributes that are currently on that page.
 * 
 * This class defines the base class for Dynamic SmartName collectors.
 * 
 * @author JEPE
 */
define([], function(){
	"use strict";
	
	var DynamicSmartNameCollector = function() {};
	
	(function(){
		/**
		 * Set the collector to run after this one has finished...
		 * 
		 * @param nextCollector {DynamicSmartNameCollector} the next collector to run.
		 */
		this.next = function(nextCollector){
			this._next = nextCollector;
		};
		
		/**
		 * Run this collector and the ones after it. Save all the current values of the SmartNames in the given dynamicsmartnamecollection.
		 * 
		 * @param dynamicsmartnamecollection {DynamicSmartNameCollection} the collection to add the SmartNames into.
		 */
		this.collect = function(dynamicsmartnamecollection, onDone) {
			// continue... (override this method!)
			this._runNext(dynamicsmartnamecollection, onDone);
		};
		
		/**
		 * (private)
		 * Run the next collector (in case there are multiple)
		 * 
		 * @param dynamicsmartnamecollection {DynamicSmartNameCollection} the collection to add the SmartNames into.
		 */
		this._runNext = function(dynamicsmartnamecollection, onDone){
			if(this._next!==undefined && this._next.collect!=undefined) {
				this._next.collect(dynamicsmartnamecollection, onDone);
			}else{
				onDone();
			}
		};
	}).call(DynamicSmartNameCollector.prototype);
	
	return DynamicSmartNameCollector;
});