/**
 * In a Search with SmartNames, you want to be able to use the values of the attributes that are currently on that page.
 * 
 * @author JEPE
 */
define(["lib/oop", "./DynamicSmartNameCollector", "jquery"], function(oop, DynamicSmartNameCollector, $){
	"use strict";
	
	/**
	 * Constructor for ProjectCreationDetailsSmartNameCollector.
	 */
	var ProjectCreationDetailsDynamicSmartNameCollector = function() {};
	
	// inherit functions from DynamicSmartNameCollector
	oop.inherits(ProjectCreationDetailsDynamicSmartNameCollector, DynamicSmartNameCollector);
	
	//
	// Supported SmartNames
	//
	/**
	 * The list of supported "dynamic" SmartNames from the Project Creation Details Section
	 * 
	 * When no value can be found for any of these, they are still send to the server as an empty value 
	 * to avoid being overridden by the template.
	 * 
	 * The information will be searched for in $(field) when $(displaygroup) is visible by using the (use) of the field. (value, label, other...)
	 */
	ProjectCreationDetailsDynamicSmartNameCollector.prototype.smartnames = {
			"Project Name": 
					{field: "#projectNameId", displaygroup: "#projectNameRowId", use: "value"},
			"Project Template Name": 
					{field: "#selectTemplateId", displaygroup: "#selectTemplateRowId", use: "label*"},
			"Parent Project ID": 
					{field: "#sourceProjectIDId",  use: "value"},
			"Project Manager Name": 
					{field: "#selectProjectManagerId", displaygroup: "#projectManagerRowId", use: "label:name"},
			"Project Manager Last Name": 
					{},
			"Project Manager Location": 
					{},
			"Project Manager Location Code": 
					{},
			"Customer Name": 
					{field: "#selectProjectCustomerId", displaygroup: "#projectCustomerRowId", use: "label*"},
			"Customer Location": 
					{field: "#selectProjectLocationId", displaygroup: "#projectCustomerRowId", use: "label*"},
			"Customer Location Code": 
					{},
			
			// NOT in dictionary yet, but already supported on project creation page...
			// (cannot be selected from the popup though)
			"Project Requestor":
					{field: "#selectRequestorId", use: "label:name"},
			"Project Status":
					{field: "#selectProjectStatusId", displaygroup: "#projectManagerRowId", use: "label*"},
			"Project Description":
					{field: "#projectDescriptionId", displaygroup: "#projectDecriptionRowId", use: "value"}
		};
	
	/**
	 * How to get a certain type of value from a field...
	 */
	ProjectCreationDetailsDynamicSmartNameCollector.prototype.valuegetter = {
			// The value for the SmartName is simply the value of the field.
			"value": function($e) {
				return {val:[$e.val()]};
				},
			// The value for the SmartName is simply the label of the field (if the value is not empty for that label)
			"label*": function($e) {
				return {val:[($e.val()!==undefined && $e.val()!==""?$('option:selected', $e).text():"")]};
				},
			// The value for the SmartName is ... (in case of "me", use the username...)
			"label:name": function($e) {
				var label = $('option:selected', $e).text();
				var value = $e.val();
				if(value==="N/A") {
					return {val:[""]};
				}else if(value==="me"){
					return {val:[""], valonserver:"!username"};
				}else{
					return label;
				}
			}
	};
	
	
	// override the collect method
	/**
	 * Run this collector and the ones after it. Save all the current values of the SmartNames in the given dynamicsmartnamecollection.
	 * 
	 * @param dynamicsmartnamecollection {DynamicSmartNameCollection} the collection to add the SmartNames into.
	 */
	ProjectCreationDetailsDynamicSmartNameCollector.prototype.collect = function(dynamicsmartnamecollection, onDone) {
		// loop through all SmartNames defined in this class
		for(var smartname in this.smartnames) {
			var collectinfo = this.smartnames[smartname];
			var values = [""];
			var valonserver = undefined;
			// check if they are visible...
			if(collectinfo.displaygroup === undefined || $(collectinfo.displaygroup).is(':visible')) {
				if(collectinfo.field!==undefined && collectinfo.use!==undefined) {
					// get the field & the value...
					var $field = $(collectinfo.field);
					var valueinfo = this.valuegetter[collectinfo.use]($field);
					values = valueinfo.val;
					valonserver = valueinfo.valonserver;
				}
			}
			
			// put them in the collection
			dynamicsmartnamecollection.put(smartname, values, valonserver);
		}

		// continue...
		this._runNext(dynamicsmartnamecollection, onDone);
	};
	
	return ProjectCreationDetailsDynamicSmartNameCollector;
});