/**
 * In a Search with SmartNames, you want to be able to use the values of the attributes that are currently on that page.
 * 
 * How to use?
 * 
 * Pass along an object like this, and push collectors on this object for the different kind of dynamic SmartNames that need to be collected from the page...
 * 
 * @author JEPE
 */
define(["./DynamicSmartNameCollection", "./DynamicSmartNameCollector"], function(DynamicSmartNameCollection, DynamicSmartNameCollector){
	"use strict";
	
	var SavedSearchContextInfo = function() {
		this._collector = new DynamicSmartNameCollector();
		this._lastcollector = this._collector; 
	};
	
	(function(){
		/**
		 * Set the collector that is used to collect the SmartNames...
		 * 
		 * @param collector {DynamicSmartNameCollector} the next collector to run.
		 */
		this.pushCollector = function(collector){
			this._lastcollector.next(collector);
			this._lastcollector = collector;
		};
		
		/**
		 * Set the projectID to use for Saved Search with context.
		 */
		this.setProjectId = function(projectID){
			this._projectID = projectID;
		};
		
		
		/**
		 * Get the projectID to use for Saved Search with context.
		 */
		this.getProjectId = function(){
			return this._projectID;
		};
		
		/**
		 * Set the documentID to use for Saved Search with context.
		 */
		this.setDocumentId = function(documentID){
			this._documentID = documentID;
		};
		
		/**
		 * Get the documentID to use for Saved Search with context.
		 */
		this.getDocumentId = function(){
			return this._documentID;
		};
		
		/**
		 * Set the docVerID to use for Saved Search with context.
		 */
		this.setDocumentVersionId = function(docVerID){
			this._docVerID = docVerID;
		};
		
		/**
		 * Get the docVerID to use for Saved Search with context.
		 */
		this.getDocumentVersionId = function(){
			return this._docVerID;
		};
		
		/**
		 * Get the url part with the projectID or docVerID...
		 * @returns {String}
		 */
		this.getContextUrlPart = function(){
			if(this.getDocumentVersionId()) {
				return "&docVerID="+this.getDocumentVersionId();
			}else if(this.getDocumentId()) {
				return "&docID="+this.getDocumentId();
			}else if(this.getProjectId()) {
				return "&projectID="+this.getProjectId();
			}else{
				return "";
			} 
		};
		
		/**
		 * Get the xml containing the names & values of the "dynamic" SmartNames...
		 * 
		 * @param resultcallback callback which results the xml with dynamic smartnames
		 */
		this.getStaticSmartNamesXml = function(resultcallback) {
			var dynamicsmartnamecollection = new DynamicSmartNameCollection();
			
			var onDone = function(){
				resultcallback(dynamicsmartnamecollection.exportToXml());
			};
			
			if(this._collector!==undefined && this._collector.collect!=undefined) {
				this._collector.collect(dynamicsmartnamecollection, onDone);
			}
		};
	}).call(SavedSearchContextInfo.prototype);
	
	return SavedSearchContextInfo;
});