/**
 * In a Search with SmartNames, you want to be able to use the values of the Task Specifications that are currently on that page (TEP). 
 * 
 * @author JAMI, JEPE
 */
define(["lib/oop", "./DynamicSmartNameCollector"], function(oop, DynamicSmartNameCollector){
	"use strict";

	/**
	 * Constructor for TaskSpecificationsDynamicSmartNameCollector.
	 * 
	 * @param aCatObjDOMElemName an ID reference to the DOM object that holds the aCatObj reference for the .
	 */
	var TaskSpecificationsDynamicSmartNameCollector = function(aCatObjDOMElemName) {
		this._aCatObjDOMElemName = aCatObjDOMElemName;
	};
	
	// inherit functions from DynamicSmartNameCollector
	oop.inherits(TaskSpecificationsDynamicSmartNameCollector, DynamicSmartNameCollector);
	
	// override the collect method
	/**
	 * Run this collector and the ones after it. Save all the current values of the SmartNames in the given dynamicsmartnamecollection.
	 * 
	 * @param dynamicsmartnamecollection {DynamicSmartNameCollection} the collection to add the SmartNames into.
	 */
	TaskSpecificationsDynamicSmartNameCollector.prototype.collect = function(dynamicsmartnamecollection) {
		//check if the specified aCatObjDOM element is initialized with an aCatObj properly already
		var aCatObj = $("#"+_aCatObjDOMElemName).get(0).AttributeCategory;
		
		var doneSub = function(){
			// continue...
			this._runNext(dynamicsmartnamecollection, onDone);
		};
		
		if (aCatObj !== undefined) {
			//execute the AttributeCategoryDynamicSmartNameCollector if the aCatObj is ready
			var aCatCollector = new AttributeCategoryDynamicSmartNameCollector(aCatObj);
			aCatCollector.collect(dynamicsmartnamecollection, doneSub);
		}else{
			doneSub();
		}
		
	};
	
	return TaskSpecificationsDynamicSmartNameCollector;
});