/**
 * SmartField View
 * - takes care of the user interface for Smart Name selection>insertion by the User.
 * 
 * @author JAMI, KADA
 * @since 19.1.2015
 */
define(["jquery",
        "./smartfield-style"
        ], function ($) {

	"use strict";

	/**
	 * A SmartFieldView is constructed from three jQeury wrapped elements:
	 * 
	 * - The first element is the wrapping element of the NormalInput.
	 * - The second element is the one of the SmartNameInput.
	 * - The third element is the one of the SmartNameSelector.
	 */
	var SmartFieldView = function (normalInputViewElement, smartNameInputViewElement, smartNameSelectorViewElement, smartNameDropdown) {

		// Remember the other view elements.
		this._normalInputViewElement = normalInputViewElement;
		this._smartNameInputViewElement = smartNameInputViewElement;
		this._smartNameSelectorViewElement = smartNameSelectorViewElement;
		this._smartNameDropdown = smartNameDropdown;

		// Create the hidden input that will always hold the correct value of the SmartField.
		// This hidden input will simulate the normal input and therefore take over it's name.
		var normalInputName = normalInputViewElement.attr("name");
		if (normalInputName !== undefined && normalInputName.length > 0) {
			normalInputViewElement.attr("name", "");
			this._hiddenValueInput = $("<input name=\"" + normalInputName + "\" type=\"hidden\"></input>");
		} else {
			this._hiddenValueInput = $("<input type=\"hidden\"></input>");
		}
		
		if(normalInputViewElement) {
			normalInputViewElement.addClass("form-control");
		}

		// Start constructing the HTML of this component
		this._wrapper = normalInputViewElement.wrap("<div class='SmartNameInput main-content'></div>").parent();
		this._wrapper.append(smartNameInputViewElement);
		this._wrapper.append(this._hiddenValueInput);
		var smartNameInput = this._wrapper;
		this._wrapper = this._wrapper.wrap("<div class='SmartField'></div>").parent();
		this._wrapper.wrapInner($('<div class="input-group">'));
		$(".input-group", this._wrapper).append(smartNameSelectorViewElement);
		if(smartNameDropdown){
			smartNameSelectorViewElement.before(smartNameDropdown);
			smartNameInput.css("right", smartNameSelectorViewElement.outerWidth(true) + smartNameDropdown.outerWidth(true));
			smartNameInput.css("border-right", 0);
		}

		// Set inline by default
		var smartFieldAsBlock = normalInputViewElement.attr("data-smartfieldblock");
		this.setIsInline(smartFieldAsBlock !== "true");

		// Check if we have a prefered width for the smartfield
		var smartNameWidth = normalInputViewElement.attr("data-smartfieldwidth");
		if (smartNameWidth !== undefined) {
			this._wrapper.css("width", smartNameWidth);
		}

	};

	/**
	 * Transform this SmartField back to the original input.
	 */
	SmartFieldView.prototype.toNormalInput = function () {

		// Assign the original name back that was "stolen" by the hidden value input.
		var normalInputName = this._hiddenValueInput.attr("name");
		if (normalInputName !== undefined && normalInputName.length > 0) {
			this._normalInputViewElement.attr("name", normalInputName);
		}

		// Move it out of the wrapping element of this component.
		this._normalInputViewElement.insertBefore(this._wrapper);

		// Make sure it's shown
		this._normalInputViewElement.show();

		// Clear it's value
		this._normalInputViewElement.val("");

		// Dereference the SmartField property on the normal input.
		this._normalInputViewElement[0].SmartField = undefined;

		// Delete the wrapping element of this component.
		this._wrapper.remove();
	};

	/**
	 * Display the view as disabled.
	 */
	SmartFieldView.prototype.disable = function () {
		this._wrapper.addClass("SmartField-disabled");
		$(".form-control", this._wrapper).attr("disabled", "disabled");
	};

	/**
	 * Display the view as enabled.
	 */
	SmartFieldView.prototype.enable = function () {
		this._wrapper.removeClass("SmartField-disabled");
		$(".form-control", this._wrapper).removeAttr("disabled");
	};

	/**
	 * Switch between inline-block or block.
	 * Default is inline-block, which behaves the same as the transformed input.
	 * When using a percentage in the width field, the component should be set to block
	 */
	SmartFieldView.prototype.setIsInline = function (inline) {
		if (inline) {
			this._wrapper.addClass("SmartField-inline");
		} else {
			this._wrapper.removeClass("SmartField-inline");
		}
	};

	/**
	 * Set the width the view.
	 * 
	 * @param value [String] CSS width property value.
	 */
	SmartFieldView.prototype.setWidth = function (smartNameWidth) {
		this._wrapper.css("width", smartNameWidth);
	};

	/**
	 * Set the value of the hidden input of the view.
	 * 
	 * @param value [Boolean] Boolean indicating wether the view should be hidden/shown.
	 */
	SmartFieldView.prototype.setHiddenValue = function (value) {
		this._hiddenValueInput.val(value);
	};

	/**
	 * Return the jQElement of the wrapping div of this view.
	 * 
	 * @returns [JQuery] The jQuery wrapped element of the wrapping div.
	 */
	SmartFieldView.prototype.getJQElement = function () {
		return this._wrapper;
	};

	return SmartFieldView;

});