define([
	"underscore",
	"jquery"
], function (
	_,
	$
) {
	"use strict";

	/**
	 * A dropdown element to be added when the smartname is attached to a select
	 * 
	 * @param {Array.<Object>} options -  the options
	 * @param {SmartNameField} smartnameField - samrtnamefield in which it is placed
	 * @param {String} delimiter - delimiter which will allow multivalue
	 */
	var SmartNameDropdown = function (options, smartnameField, delimiter) {
		/**
		 * @member {Array.<Object>} options
		 */
		this._options = options;

		/**
		 * @member {SmartNameField} smartnameField
		 */
		this._smartnameField = smartnameField;

		/**
		 * @member {String} delimiter
		 */
		this._delimiter = delimiter;

		this._elem = $("<div/>").addClass("SmartNameDropdown input-group-addon").append('<i class="icon icon-WCR_Down-S"/>');

		var ul = $("<ul/>");
		_.each(this._options, _.bind(function (option) {
			var li = $('<li />');
			li.html(option.text);
			li.on("mousedown", _.bind(function () {
				var initText = this._smartnameField.getValue();
				if (this._delimiter && initText.indexOf(this._delimiter) !== -1) {
					this._smartnameField.setValue(initText + option.value);
				} else {
					this._smartnameField.setValue(option.value);
				}
				this._smartnameField.focus();
				this._popup.hide();
			}, this));
			ul.append(li);
		}, this));

		this._popup = ul.hide().addClass('SmartNameDropdownPopup sdropResults darktext_darklink_11');

		$("body").append(this._popup);

		var f = _.bind(function () {
			this._popup.hide();
			$("body").off("mousedown", f);
		}, this);

		this._elem.click(_.bind(function (e) {
			var inputDiv = this._elem.parents(".input-group").first();
			var smartNameDiv = this._elem.parents(".SmartField");
			$("body").on("mousedown", f);
			this._popup.show();
			this._popup.css({
				top: (inputDiv.outerHeight() + inputDiv.offset().top) + 'px',
				left: (inputDiv.offset().left) + 'px',
				// The width is the width of the smartnameinput minus the smartnameselector minus the thickness of the borders
				width: (smartNameDiv.width() - smartNameDiv.find(".SmartNameSelector").outerWidth(true) - (this._popup.outerWidth() - this._popup.innerWidth())) + 'px'
			});
			e.preventDefault();
			e.stopPropagation();
		}, this));
	};

	/**
	 * get JQuery Element
	 * @returns {JQueryObject}
	 */
	SmartNameDropdown.prototype.getJQElement = function () {
		return this._elem;
	};
	
	/**
	 * disable the smartname dropdown
	 */
	SmartNameDropdown.prototype.disable = function () {
		this._elem.off("click");
		this._elem.addClass("disabled");
	}

	/**
	 * Destroy element
	 */
	SmartNameDropdown.prototype.destroy = function () {
		this._popup.remove();
	};

	return SmartNameDropdown;
});
