define(["jquery"], function($) {
	
	var api = {};
	
	/**
	 * Changes the dropdowns in context to use absolute positioning when opened.
	 * This is needed to circumvent issues with overflow:auto on a container to add scrollbars.
	 * 
	 * Sideeffects: css issues! Since the dropdown is moved outside of the container.
	 * Sideeffects: click events: they no longer bubble to the container...
	 * 
	 * @param {jQuery} context - the context to look in
	 * @param {string} [namespace] - optional, the class to add to the dropdown when it is moved out of the container
	 */
	api.repositionDropdowns = function(context, namespace){
		context.on("show.bs.dropdown", function(e){
			var dropdownMenu = $(e.target).find(".dropdown-menu");
			
			$.data(e.target, "dropdownMenuReference", dropdownMenu);
			$.data(dropdownMenu[0], "dropdownButtonReference", e.target);
			
			// detach it and append it to the body
			$('body').append(dropdownMenu.detach());
			
			// Indicate that the dropdown was repositioned.
			dropdownMenu.addClass("RepositionedDropdown");
			
			// grab the new offset position
			var eOffset = $(e.target).offset();
			
			var rightAligned = dropdownMenu.hasClass("dropdown-menu-right");

			// make sure to place it where it would normally go (this could be improved)
			if(!rightAligned) {
				dropdownMenu.css({
					'display': 'block',
					'top': eOffset.top + $(e.target).outerHeight(),
					'left': eOffset.left
				});
			} else {
				dropdownMenu.css({
					'display': 'block',
					'top': eOffset.top + $(e.target).outerHeight(),
					'right': $(window).width() - eOffset.left-$(e.target).outerWidth()
				});
			}
			if(namespace) {
				dropdownMenu.addClass(namespace);
			}
			e.stopPropagation();
		
		}).on("hide.bs.dropdown", function(e){
			var dropdownMenu = $.data(e.target, "dropdownMenuReference");
			if (dropdownMenu.length > 0){				
				$(e.target).append(dropdownMenu.detach().css({"display":"", "top": "", "left": "", "right": ""}));
				
				// Indicate that the dropdown is no longer repositioned.
				dropdownMenu.removeClass("RepositionedDropdown");
				
				// Also remove the namespace
				if(namespace) {
					$(e.target).removeClass(namespace);
				}
			}
			
			e.stopPropagation();
		});
	};
	
	/**
	 * Check that all repositioned dropdowns on the page reference still an existing button in the DOM, otherwise clean them up.
	 */
	api.cleanupDropdowns = function(){
		// Select all the dropdowns in the DOM who are repositioned at the moment.
		$(".RepositionedDropdown").each(function () {
			// Get a reference to the button that references this dropdown.
			var buttonRef = $.data(this, "dropdownButtonReference");
			// Check the state of that button.
			if (buttonRef == undefined) {
				// No such button exists.
				// Then also remove this dropdown.
				$(this).remove();
			} else if (!$.contains(document, buttonRef)) {
			    // Button is detached from the DOM.
				// Append this dropdown back after it..
				$(this).removeClass("RepositionedDropdown");
				$(buttonRef).after(this);
			}
		});
		
	};
	
	api.wcrinit = function(div){
		var namespace = $(div).attr("data-overflownamespace");
		
		api.repositionDropdowns($(div), namespace);
	};
	
	return api;
});