define(["utils/ResponsiveFit",
		"utils/webi18n!ActionMenu",
		"jquery"],
	function (ResponsiveFit,
		i18n,
		$) {
	
	
		/**
		 * Component for displaying a Third Level Menu
		 * @augments ResponsiveFit
		 * @constructor
		 * @class
		 * @param {JQueryObj} context - menu div
		 */
		var ThirdLevelMenu = function (context) {
			this._context = context;


			this._initMoreContainer();

			// initialize the responsive fit
			ResponsiveFit.call(this, this._context, this._moreElement, this._moreContainer);

			this.setMaintainSequence(true);

			$(window).resize(_.throttle(_.bind(function () {
				this.resize();
			}, this), 40));

			this.resize();

			this._context.addClass("initialized");


			this.resize();

		};

		ThirdLevelMenu.prototype = Object.create(ResponsiveFit.prototype);
		ThirdLevelMenu.prototype.constructor = ThirdLevelMenu;

		/**
		 * Can an element be moved to the more container
		 * @Override
		 * @param {JQueryObj} child
		 * @returns {Boolean}
		 */
		ThirdLevelMenu.prototype._canMoveToMore = function (child) {
			return !child.hasClass("active");
		};


		/**
		 * Get the maximum with of the main container
		 * @Override
		 * @returns {Number}
		 */
		ThirdLevelMenu.prototype._getMaximumWidth = function () {
			var paddingLeft = 0,
				paddingRight = 0;
			try {
				//since we use the border-box model, we need to subtract padding from the width
				var computedStyle = getComputedStyle(this._container[0]);
				paddingLeft = parseFloat(computedStyle.getPropertyValue("padding-left"));
				paddingRight = parseFloat(computedStyle.getPropertyValue("padding-right"));
			} catch (e) {
				// ignore
			}

			return this._container.innerWidth() - paddingLeft - paddingRight;
		};

		/**
		 * Initialize the more container
		 */
		ThirdLevelMenu.prototype._initMoreContainer = function () {
			this._moreText = $("<span>", {
				text: i18n.tFallback("ACTIONMENU_MORE_ACTIONS", "More...")
			});

			this._moreButton = $("<a>", {
				'class': "dropdown-toggle",
				'href': "#",
				'data-toggle': "dropdown"

			}).append(this._moreText, $("<span>", {
				'class': "caret"
			}));

			this._moreContainer = $("<ul>", {
				'class': "dropdown-menu dropdown-menu-right"
			});
			this._moreElement = $("<li>", {
				'class': "dropdown more"
			}).append(this._moreButton, this._moreContainer);

			this._context.append(this._moreElement);

		};


		ThirdLevelMenu.wcrinit = function (element) {
			if (!(element instanceof $)) {
				element = $(element);
			}
			new ThirdLevelMenu(element);
		};

		return ThirdLevelMenu;
	});