/**
 * TreeItemView
 *
 * @author KADA
 * @since 9.2.2015
 */
define(["jquery","./treeitem-style"], function ($) {
	
	"use strict";
	
	var TreeItemView = function () {
		
		// This boolean will indicate whether the children are expanded or not.
        this._hideChildViews = true;
        
        // Create the wrapping element;
        this._wrapper = $("<li></li>");
        
        // Create the wrapping div for the title of the item.
        var itemTitleWrapper = $('<div></div>');
        
        // Create the The bindings expand button.
        this._expandIcon = $("<i class='Tree-List-Item-Expand-Icon " + this._childExpandImageUrl + "'></i>");
        
        // Create and fill the title table
        this._titleTable = $("<table><tr><td/><td style='width: 100%'/></tr></table>");
        $(this._titleTable[0].rows[0].cells[0]).append(this._expandIcon);
        
        // Create the element that will contain the content of the item. 
        this._itemContentElement = $("<div class='Tree-List-Item-Content-Box'></div>");
		
        // Create wrapping divs for the item (not the expandable content)
        this._itemWrapper = $("<div class='Tree-List-Item'></div>");
        var itemOuterWrapper = $("<div class='Tree-List-Item-Outer'></div>");
        
        // Create wrapping div for the expandable content.
        this._expandableWrapper = $("<div class='Tree-List-Expandable'></div>");
        
        // Create the underlying list for the children.
        this._childrenList = $("<ul></ul>");
        
        // Create the underlying list + item for messages.
        this._msgBox = $("<li></li>");
        this._msgBox.wrap("<ul></ul>");
        this._msgBox.hide(); // hide by default.
        
        // Combine all the elements in the right order
        itemTitleWrapper.append(this._titleTable);
        this._itemWrapper.append(itemTitleWrapper);
        this._itemWrapper.append(this._itemContentElement);
        itemOuterWrapper.append(this._itemWrapper);
        this._expandableWrapper.append(this._childrenList);
        this._expandableWrapper.append(this._msgBox.parent());
        this._wrapper.append(itemOuterWrapper);
        this._wrapper.append(this._expandableWrapper);
        
        // When the title is clicked, toggle whether the children should be expanded.
        var self = this;
        this._expandIcon.on("click", function () {
        	if (self._hideChildViews) {
        		self.expand();
        	} else {
        		self.collapse();
        	}
        });
        
        // Collapse this by default.
        this.collapse();
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype._childExpandImageUrl = "icon icon-normal icon-WCR_Plus-square12";
	
	/**
	 * 
	 */
	TreeItemView.prototype._childCollapseImageUrl = "icon icon-normal icon-WCR_Minus-square12";
	
	/**
	 * 
	 */
	TreeItemView.prototype.setTitleElement = function (jQElement) {
		$(this._titleTable[0].rows[0].cells[1]).empty();
		$(this._titleTable[0].rows[0].cells[1]).append(jQElement);
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.setContentElement = function (jQElement) {
		this._itemContentElement.empty();
		this._itemContentElement.append(jQElement);
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.addChildView = function (jQElement) {
		
		this._childrenList.append(jQElement);
		
		// Remark the last one (for CSS)
		this._childrenList.children().removeClass("lastChild");
		var childElements = this._childrenList.children();
		$(childElements.get(childElements.length-1)).addClass("lastChild");

	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.removeChildView = function (jQElement) {

		if(this._childrenList[0] == jQElement.parent()[0]) {
			jQElement.remove();
		}
		
		// Remark the last one (for CSS)
		this._childrenList.children().removeClass("lastChild");
		var childElements = this._childrenList.children();
		if (childElements.length > 0) {
			$(childElements.get(childElements.length-1)).addClass("lastChild");
		}
	};
	

	/**
	 * 
	 */
	TreeItemView.prototype.expand = function () {
		this._expandableWrapper.show();
		this._expandIcon.attr("class",this._childCollapseImageUrl);
		this._hideChildViews = false;
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.collapse = function () {
		this._expandableWrapper.hide();
		this._expandIcon.attr("class",this._childExpandImageUrl);
		this._hideChildViews = true;
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.switchToDisplayingChildren = function () {
		this._childrenList.show();
		this._msgBox.hide();
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.switchToDisplayingMsg = function () {
		this._childrenList.hide();
		this._msgBox.show();
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.showMessage = function (msg) {
		this.switchToDisplayingMsg();
		var msgSpan = $("<span class='Tree-Item-Msg'></span>");
		msgSpan.text(msg);
		this._msgBox.empty();
		this._msgBox.append(msgSpan);
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.showErrorMsg = function (errorMsg) {
		this.switchToDisplayingMsg();
		var errorSpan = $("<span class='Tree-Item-ErrorMsg'></span>");
		errorSpan.text(errorMsg);
		this._msgBox.empty();
		this._msgBox.append(errorSpan);
		this.expand();
	};
	
	
	/**
	 * 
	 */
	TreeItemView.prototype.clearChildViews = function () {
		this._childrenList.find('li').remove();
	};
	
	
	/**
	 * 
	 */
	TreeItemView.prototype.setRoot = function() {
		this._msgBox.parent().addClass('Tree-List');
		this._childrenList.addClass('Tree-List');
		var children = this._wrapper.children();
		this._wrapper = $("<div class='Tree-List-RootWrapper'/>");
		this._wrapper.append(children);
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.setLeaf = function() {
		this._wrapper.addClass('Tree-List-Item-Leaf');
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.setBackgroundColor = function(color) {
		var childElements = this._childrenList.children();
		if (childElements.length > 0) {
			$(childElements.get(childElements.length-1)).css("background-color",color);
		}
	};
	
	
	/**
	 * 
	 */
	TreeItemView.prototype.setWrapperBackgroundColor = function(color) {
		this._wrapper.css("background-color",color);
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.setItemBoxColor = function(color) {
		this._itemWrapper.css("background-color",color);
	};
	
	
	/**
	 * 
	 */
	TreeItemView.prototype.destroy = function() {
		this._wrapper.remove();
	};
	
	/**
	 * 
	 */
	TreeItemView.prototype.getJQElement = function() {
		return this._wrapper;
	};
	
	return TreeItemView;
});