define(["jquery",
        "underscore",
        "webcenterapi/data/WebCenterUser"
        ], 
    	function($,
    			_,
    			WebCenterUser
    			) {
    		
    		"use strict";
    		
    		 /**
    	      *  Warning : this class is a singleton, any modification to one of its component will change for each reference to it.
    	      */
    		var CreateUserAndInviteInProject = function () {
    	        
    		};
    		
    		/**
    		 *  Ajax call to create new user
    		 *  
    		 * @param {Object} action - Specify whether a new user should be created (action = “create”) or an existing user should be modified (action = “modify”).
    		 * @param {String} folderId - folder id, if it is document/folder approval setup page should be specified to restrict permission
    		 * @param {Object} params - Object indicating User details to be created in the system
    		 * @param {Function} successCallback - Will be executed on successful return from the back end 
    		 * @param {Function} errorCallback - Will be executed on error from back end.
    		 * @param {Boolean} doNotInviteMember - should we refrain from inviting the user?
    		 */
    		
    		CreateUserAndInviteInProject.prototype.createOrModifyAndInvite = function (action, folderId, params, successCallback, errorCallback, doNotInviteMember) {
    			
    			/**
    			 * 
    			 * Add required details to the request data --> action = "create" to create new user
    			 * getnewuserinfo = "1" to get newly create full information , if this property is 0  then response will get only userId (not full uesr information)
    			 * 
    			 */
				
				var defaultParams = {action: action, getnewuserinfo: "1"};
				
				_.extend(defaultParams, params);
    			
    			// ajax request to create new user
				var createUserPromise = $.ajax({url: "CreateUser.jsp", method: "POST", dataType: "xml", data: defaultParams});
				
    			$.when(createUserPromise).then(function(data, textStatus, jqXHR) {
    				
    				var xml = $(data).children().get(0);
    				
    				if(xml && xml.tagName) {
    					
    					// handle the response and get newly created user information
    					if(xml.tagName === "new_user_info") {
    						
    						var userID = $("> userID", xml).text();
    						
    						var userName = $("> userName", xml).text();
    						var firstName = $("> firstName", xml).text();
    						var lastName = $("> lastName", xml).text();
    						var email = $("> email", xml).text();
    						var isActive = $("> isActive", xml).text();
    						
    						var newUser = new WebCenterUser(userID, userName, firstName, lastName, email, (isActive === "true"));
        					
    						// create request data to invite new user to project
        					var inviteUserInfo = {};
        					inviteUserInfo.usermemberid = userID;
        					
        					if(defaultParams.roleid !== "") {
        						inviteUserInfo.roleid = defaultParams.roleid;
        					}
        					
        					inviteUserInfo.projectid = defaultParams.projectid;
        					if(!doNotInviteMember) {
        						// Invite member to project 
        						this.inviteMemberToProject(inviteUserInfo, newUser, folderId, successCallback, errorCallback);
							} else {
								successCallback(newUser);
							}
    					} 
    					
    					// Handle the error
    					if(xml.tagName === "error") {
    						errorCallback(xml);
        				}
    				}
    					
    			}.bind(this), function(data, textStatus, jqXHR) {
    				errorCallback("UNKNOWNEXCEPTION_TAG", true);
    			}.bind(this));
    		};
    		
    		/**
    		 * Ajax call to Invite user as Project member    		
             *
    		 * @param inviteUserInfo -  Object indicating UserId, roleId and projectId details to invite member into the project
    		 * @param newUser - Newly created user information
    		 * @param {String} folderId - folder id, if it is document/folder approval setup page should be specified to restrict permission
    		 * @param successCallback - Will be executed on successful return from the back end
    		 * @param errorCallback - Will be executed on error from back end.
    		 */
    		CreateUserAndInviteInProject.prototype.inviteMemberToProject = function (inviteUserInfo, newUser, folderId, successCallback, errorCallback) {
    			
    			// Make sure pass folder id, only in-case role is not available - because when role is available permission's are set to based on role.
    			if(!inviteUserInfo.roleid) {
    				// If folder is undefined or null it means root folder, so set root folder id as zero (folder id zero identified as root folder in WebCenter)
        			inviteUserInfo.folderid = folderId || "0";
    			}
    			
    			// ajax request to invite member to project
    			var inviteMemberToProjectPromise = $.ajax({url: "InviteMemberToProject.jsp", method: "POST", dataType: "xml", data: inviteUserInfo});
    			
    			$.when(inviteMemberToProjectPromise).then(function(data, textStatus, jqXHR) {
    				
    				var xml = $(data).children().get(0);
    				
    				if(xml && xml.tagName) {
    					
    					// if success send the Newly created user
    					if(xml.tagName === "result") {
    						successCallback(newUser);
    					}
        				
    					// Handle the error
        				if(xml.tagName === "error") {
        					errorCallback(xml);
        				}
    				}
    				
    			}.bind(this), function(data, textStatus, jqXHR) {
    				errorCallback("UNKNOWNEXCEPTION_TAG", true);
    			}.bind(this));
    		};
    		
    		return new CreateUserAndInviteInProject();	
});
