define(["./XDropDataProvider",
        "./ResultEntry",
		"webcenterapi/data/WebCenterUser",
        "utils/webi18n!UserCreationStrings",
	    "require"], 
        function (XDropDataProvider, 
        		ResultEntry,
				WebCenterUser,
        		i18n, 
				require) {
	"use strict";
	
	/**
	 * Create new user data provider
	 */
	var CreateUserDataProvider = function (projectId, folderId) {
		XDropDataProvider.call(this);
		this._projectId = projectId;
		this._folderId = folderId;
		this._init();
	};

	CreateUserDataProvider.prototype = Object.create(XDropDataProvider.prototype);
	CreateUserDataProvider.prototype.constructor = CreateUserDataProvider;

	CreateUserDataProvider.prototype._init = function () {
		this._resultSet.append(this._generateEntry());
		
		// preload the user creation popup
		setTimeout(function(){
			require(["component/usercreation/CreateNewUserPopup"], function(){});
		}, 1500);
	};
	
	/**
	 * @override
	 */
	CreateUserDataProvider.prototype.setQuery = function(query){
		this._query = query;
	};

	
	/**
	 * Generate the Result Entry that will show a button
	 * When clicking on the button the user creation popup is opened
	 * @private
	 * @returns {ResultEntry}
	 */
	CreateUserDataProvider.prototype._generateEntry = function () {
		var self = this;
		
		var resultEntry = new ResultEntry("button", {
			getLabel: function () {
				return i18n.t("CREATE_AND_INVITE_NEW_USER");
			}, 
			doAction: function(event, api){
				// once the user clicks the button, we want to open the new user popup
				require(["component/usercreation/CreateNewUserPopup"], function(CreateNewUserPopup){				
					var popup = new CreateNewUserPopup(self._projectId, self._folderId);
				
					// Add callback to execute when a user is successfully created
					popup.onSubmit(function(userDetails) {
						var resultEntry = new ResultEntry("user", userDetails);
						userDetails.INCONTEXTCREATED = true;
						api.selectResultEntry(resultEntry);
						api.closeXDrop();
					});
					// open the popup
					popup.show(self._query);
				});
			},
			getIcon: function(){
				return "icon icon-WCR_Actions-User-Add12";
			}
		});
		
		resultEntry.setDefaultSelectionIndex(80); // this has a lower default selection index than other results (if other things are matched they will be selected first)
		return resultEntry;

	};
	return CreateUserDataProvider;
});