define(["./Renderer", 
        "jquery"], 
        function(Renderer, 
        		$){
	"use strict";
	
	/**
	 * A renderer for displaying a button which executes "action" on the resultentry
	 * @class
	 * @augments Renderer 
	 * @param {JQueryObj}   div         
	 * @param {ResultEntry} resultEntry 
	 * @param {Object}      api        -  render API
	 */
	var ButtonRenderer = function(div, resultEntry, api){
		Renderer.call(this, div, resultEntry, api);	
		
		
		
		var button = $("<a>", {"class": "btn btn-primary action-button", text: this._resultEntry.getData().getLabel()});
		
		if (resultEntry.getData().getClassNames){
			button.removeClass();
			button.addClass(resultEntry.getData().getClassNames());
			button.addClass("action-button");
		}
		
		if (resultEntry.getData().getIcon){
			button.prepend("&nbsp;");
			button.prepend($("<i>", {'class': resultEntry.getData().getIcon(), title: this._resultEntry.getData().getLabel()}));
		}
		this._element.append(button);
	};
	
	ButtonRenderer.prototype = Object.create(Renderer.prototype);
	ButtonRenderer.prototype.constructor = ButtonRenderer;
	
	/**
	 * On activation we execute the action of the result entry
	 * @override
	 */
	ButtonRenderer.prototype.activate = function(event){		
		this._focusResultEntry(this._resultEntry);
		this._resultEntry.getData().doAction(event, this._api);
	};
	
	return {		
		make: function(div, resultEntry, api){
			return new ButtonRenderer(div, resultEntry, api);	
		}
	};


});