define(["./Renderer", 
        "jquery"], 
        function(Renderer, 
        		$){
	"use strict";
	
	/**
	 * A renderer for displaying a category title
	 * @class
	 * @augments Renderer 
	 * @param {JQueryObj}   div         
	 * @param {ResultEntry} resultEntry 
	 * @param {Object}      api        -  render API
	 */
	var CategoryTitleRenderer = function(div, resultEntry, api){
		Renderer.call(this, div, resultEntry, api);	
		
		this._element.addClass(resultEntry.getData().getClassName());
		this._icon = $();
		
		if (resultEntry.getData().allowCollapse()){
			this._icon = $("<i>", {"class": "icon"});	
		}
		
		this._title = $("<span>", {"class": "title", text: resultEntry.getData().getTitle()});
		
		this._element.attr("data-sahi-id", "XDROPCATEGORY "+resultEntry.getData().getTitle());
		
		this._element.append(this._icon).append(this._title);
		
		this.render();
	};
	
	CategoryTitleRenderer.prototype = Object.create(Renderer.prototype);
	CategoryTitleRenderer.prototype.constructor = CategoryTitleRenderer;
	
	/**
	 * On activation we execute the action of the result entry
	 * @override
	 */
	CategoryTitleRenderer.prototype.activate = function(event){		
		this._resultEntry.getData().toggle();
	};
	
	/**
	 * @override
	 */
	CategoryTitleRenderer.prototype.setQuery = function(){
		
		// do nothing;
	};
	
	/**
	 * @override
	 */
	CategoryTitleRenderer.prototype.render = function(){
		var isCollapsed = this._resultEntry.getData().isCollapsed();		
		this._icon.toggleClass("icon-WCR_Right-S", isCollapsed).toggleClass("icon-WCR_Down-S", !isCollapsed);
	};
	
	
	return {		
		make: function(div, resultEntry, api){
			return new CategoryTitleRenderer(div, resultEntry, api);	
		}
	};


});