define(["./Renderer", "underscore", "jquery"], function (Renderer, _, $) {

	"use strict";

	/**
	 * Renderer that renders @class WebCenterUser
	 * @class
	 * @augments Renderer
	 * @param {JQueryObj}   div         
	 * @param {ResultEntry} resultEntry 
	 * @param {Object}      api        -  render API
	 */
	var UserRenderer = function (div, resultEntry, api) {
		Renderer.call(this, div, resultEntry, api);
		this._init();
	};

	UserRenderer.prototype = Object.create(Renderer.prototype);
	UserRenderer.prototype.constructor = UserRenderer;

	UserRenderer.prototype._init = function() {
		var user = this._resultEntry.getData();		
		var email = user.getEmail();
		var icon = user.getDisplayIcon();
		this._element.append($("<i>", {'class': "icon icon-normal "+ icon +" add-spacing-right"})).append(user.getDisplayElement(false, false));
		
		this._emailElem = $("<div class='more-details'>").append(email);
		this._element.append(this._emailElem);
				
		this._element.attr("data-sahi-id",  user.getSAHIMarker());		
	};
	
	/**
	 * We want to hide/show the e-mail address if it is matched or not by the query
	 * @param {String} query 
	 */
	UserRenderer.prototype.setQuery = function(query){
		if (this._previousQuery !== query){
			if (query.length > 0 && this._resultEntry.getData().getEmail().indexOf(query) >= 0){
				this._element.append(this._emailElem);
			} else {
				this._emailElem.detach();
			}
		}
		Renderer.prototype.setQuery.apply(this, arguments);
	};
	
	return {
		/**
		 * @returns UserRenderer
		 */
		make: function (div, resultEntry, api) {
			return new UserRenderer(div, resultEntry, api);
		}
	};
});
