/**
 * config.js Contains the configuration for requirejs.
 */
requirejs.config({
	shim: {
		modernizr: {
			exports: "Modernizr"
		},
		tinyMCE: {
			exports: "tinyMCE",
			init: function () {
				this.tinyMCE.DOM.events.domLoaded = true;
				return this.tinyMCE;
			}
		},
		"lib/jquery-migrate-src-1.2.1": {
			deps: ["lib/jquery-src-1.11.1"],
			exports: "jQuery"
		},
		"lib/jqueryUI-src-1.10.4": {
			deps: ["jquery"],
			exports: "jQuery"
		},
		"lib/jquery.floatingscroll": {
			deps: ["jquery"],
			exports: "jQuery"
		},
		"lib/stickykit": {
			deps: ["jquery"],
			exports: "jQuery"
		},
		"style/js/bootstrap/bootstrap": {
			deps: ["jquery"], 
			exports: "jQuery"
		},
		"lib/PromisePolyfill": {
			exports: "Promise"
		},
		/** WebCenter SHIM configuration **/
		"wcr/webcenter_html": {
			deps: ["wcr/datetools",
				"wcr/dateShift",
				"wcr/webcenter_xml",
                   "jquery"],
			exports: "WCHTML"
		},
		"wcr/datetools": {
			deps: ["wcr/element"]
		},
		"wcr/webcenter_xml": {
			exports: "WCXML"
		},
		"wcr/webcenter_ajax": {
			deps: ["wcr/webcenter_xml"],
			exports: "ajax"
		},
		"wcr/documentSourceHandler": {
			deps: ["wcr/webcenter_xml",
				"wcr/webcenter_ajax",
                   "wcr/docSearchOperations"], 
			exports: "DocumentSourceHandler"
		},
		"wcr/projdetailsbom": {
			deps: ["wcr/DocumentsTable",
				"wcr/documentSourceHandler",
				"wcr/AddDocument",
				"wcr/webcenter_html",
				"wcr/ObjectArrayCloneUtil",
				"wcr/webcenter_xml"
			]
		},
		"wcr/DocumentsTable": {
			deps: ["wcr/sdrop",
				"wcr/webcenter_xml",
				"wcr/AddDocument",
				"wcr/documentSourceHandler",
				"wcr/webcenter_ajax",
				"wcr/ObjectArrayCloneUtil",
                   "wcr/validate"]

		},
		"wcr/AddDocument": {
			deps: ["wcr/sdrop",
				"wcr/webcenter_xml",
				"wcr/webcenter_ajax",
				"wcr/ObjectArrayCloneUtil",
				"wcr/documentSourceHandler",
				"wcr/validate"
			]
		},
		"wcr/ObjectArrayCloneUtil": {
			exports: "ObjectHandler"
		},
		"wcr/ProjectCreation": {
			deps: ["wcr/sdrop",
				"wcr/AddDocument",
				"wcr/webcenter_ajax",
				"wcr/webcenter_xml",
				"wcr/ManageRoles",
				"wcr/DocumentsTable",
				"wcr/sdrop_common",
				"wcr/webcenter_html",
				"wcr/documentSourceHandler",
				"wcr/ObjectArrayCloneUtil",
				"wcr/projdetailsbom",
                   "wcr/validate"]
		},
		"wcr/sdrop": {
			deps: ["wcr/webcenter_ajax",
				"wcr/webcenter_xml",
				"wcr/mootools",			
				"wcr/FilterUtils",
				"jquery"
			]
		},
		"wcr/sso": {
			deps: ["wcr/sso-private"]
		},
		"wcr/ManageRoles": {
			deps: ["wcr/sdrop",
                   "wcr/webcenter_xml"]
		},
		"wcr/search": {
			deps: ["wcr/sdrop",
				"wcr/validate",
				"wcr/webcenter_xml",
				"wcr/documentSourceHandler",
				"wcr/dynamic_homepage_utils",
				"wcr/webcenter_html",
                   "wcr/webcenter_ajax"]
		},
		"wcr/sdrop_common": {
			deps: ["wcr/sdrop",
                   "wcr/webcenter_xml"] 
		},
		"wcr/dynamic_homepage_utils": {
			deps: ["wcr/vector",
                   "wcr/webcenter_xml"]
		},
		"wcr/MyTodo": {
			deps: ["wcr/dynamic_homepage_utils",
				"wcr/documentSourceHandler",
				"wcr/webcenter_ajax",
				"wcr/search",
                   "wcr/webcenter_xml"]
		},
		"wcr/Vector": {
			exports: "Vector"
		},
		"wcr/docSearchOperations": {
			deps: ["wcr/webcenter_ajax",
                   "wcr/webcenter_xml"],
			exports: "DocSearchOperations"
		},
		"wcr/dateShift": {
			deps: []
		},
		"wcr/modalpopup": {
			deps: []
		},
		"wcr/mootools": {
			deps: []
		},
		"wcr/validate": {
			deps: []
		},
		"wcr/date": {
			deps: []
		},
		"wcrBase/custom/javascript/CustomFormExtension": {
			exports: "gCustomFormExtension"
		},
		"wcr/popup": {
			deps: []
		},
		"wcr/extpopup": {
			deps: ["wcr/popup"]
		},
		"wcr/approvalInfoPopup": {
			exports: "showapprovaldetails",
			deps: ["wcr/hover"]
		},
		"wcr/FilterUtils": {
			exports: "FilterUtils"
		},
		"wcr/projectwctaskschedule": {
			deps: ["wcr/datetools", "wcr/dateShift", "wcr/webcenter_html", "jquery"]
		},
		"wcr/ProjectCalendar": {
			deps: ["wcr/datetools", "wcr/dateShift", "wcr/webcenter_xml", "wcr/projectwctaskschedule", "jquery"]
		},
		"WV3Dlib/three": {
			exports: 'THREE'
		},
		"underscore": {
			exports: '_'
        },
        "lib/medium/medium": {
			exports: 'Medium'
        },
        "lib/medium/undo": {
			exports: 'Undo'
        },
        // Note: only used in build...
        "lib/less-rhino-1.7.5" : {
        	exports: 'less'
		}
	},
	map: {
		// Do not use the text! util included in ace 
		// because it is made for a different build mechanism...
		// Removing this would break the build.
		// Note however that for normal operation (not building), 
		// it does not matter which text is used.
		// But just for the sake of not using two different ones...
		'ace': {
			"ace/requirejs/text": "lib/text",
			"ace/requirejs/text_build": "lib/text"
		}
	},
	paths: {
		"underscore": "lib/underscore",
		"domReady": "lib/domReady",
		"jquery": "lib/jquery",
		"jqueryUI": "lib/jqueryUI",
		"jqueryUITouchPunch": "lib/jqueryUITouchPunch",
		"bootstrap": "style/js/bootstrap/bootstrap",
		"modernizr": "lib/modernizr",
		"tpl": "layout/tpl",
		"Template": "layout/Template",
		"colorpicker": "lib/colorpicker",
		"HTMLTruncator": "lib/HTMLTruncator",
		"debugger": "core/DevDebug",
		"tinyMCE": "lib/tinymce/tinymce.min",
		"Mousetrap": "lib/Mousetrap",
		"wcr": "empty:",
		"backbone": "lib/backbone",
		"WV3Dlib": "lib",
		"WV3Dmod": "3d-web-viewer/modules",
		"WV3D": "3d-web-viewer",
		"rbush": "lib/rbush",
		"rbushknn": "lib/rbushknn",
		"d3": "lib/d3",
		"c3": "lib/c3/c3",
		"c3WithStyling": "lib/c3/c3WithStyling",
		
		"stickykit": "lib/stickykit",

        "typeahead": "lib/bootstrap3-typeahead",
        "gridster" : "lib/jquery.gridster",
        "gridster-extra" : "lib/jquery.gridster.with-extras"
	},
	waitSeconds: 90
});

// IE9 fix for inheritance mechanism
if (!Object.create) {
	Object.create = function (o) {
		function F() {
		}
		F.prototype = o;
		return new F();
	};
}