define(["jquery", "underscore", "lib/dom", "utils/Observer"],
	function ($, _, dom, Observer) {

		"use strict";

	
		/**
		 * controls the compilation of less for a certain customizationsection 
		 * @class
		 * @mixes Observer
		 * @exports PreviewController
		 * @param {CustomizationSection} customization
		 * @param {Array.<String>} lessFiles 
		 * @param {Boolean} doNotNamespace
		 */
		var PreviewController = function (customization, lessFiles, doNotNamespace) {
			Observer.mixin(this);
			
			this._customization = customization;


			this._boundBuildFn = _.bind(this._buildCustomization, this);

			// The less files that should be imported
			this._lessFilesRequires = _.map(lessFiles, function (file) {
				return "@import '../wcr-static/" + file+"';";
			});

			this._observe(customization, "changed", this._boundBuildFn);

			this._doNotNamespace = doNotNamespace;

		};

		
		/** 
		 * builds the css files for the  customization
		 */
		PreviewController.prototype._buildCustomization = function () {
			
			// The variables that can be customized
			var variables = this._customization.getVariables();

			// Import the default variables
			var lessString = '@import "../wcr-static/style/less/wcr/variables.less";\n';

			// Append the customized variables
			_.each(variables, function (lessVar) {
				if (lessVar.getValue()) {
					lessString += '@' + lessVar.getVariableName() + ':' + lessVar.getValue() + ";\n";
				}
			});

			// Import the script that calculates the other variables based on the customized variables
			lessString += '@import "../wcr-static/style/less/wcr/calculateVariables.less";\n';
			lessString += "@import (optional) 'custom/styles/wcr/calculateVariables.less';";
			

			// Import the less files that need to be built
			_.each(this._lessFilesRequires, function (importStatement) {
				lessString += importStatement;
			}, this);
			buildStylesheet.call(this, lessString);
		};

		/**
		 * render the less content into css
		 */
		function buildStylesheet(content) {
			// LESS - Build the less...
			requirejs(["lib/less"], _.bind(function (less) {
				less.render(content, _.bind(function (err, output) {
					if(this._doNotNamespace) {
						dom.importCssString(output.css);
					} else {
						less.render(".w {"+output.css+"}", function(err, output) {
							dom.importCssString(output.css);
					});
					}
				}, this));
			}, this));
		}


		return PreviewController;

	});
