define(["jquery", "lib/colorpicker", "component/tooltip/Tooltip", "utils/webi18n!customization"],
	function ($, colorpicker, Tooltip, i18n) {

		"use strict";

		/**
		 * This class respresents a variable that can be changed.
		 * @param {JQueryElement} context - the context of this variable. The context should contain a input/select that has the .variable-select class and an attribute
		 *									data-lessvar whose value is the less variable (without the @) that should be customized
		 * @param {CustomizationSection} customization - the component this variable belongs to
		 */
		var VariableSelector = function (context, customization) {

			/** 
			 * @member {JQueryElement}
			 * @private
			 */
			this._input = context.find(".variable-select");

			var resetBtn = context.find(".reset-var-btn");
			if (resetBtn.length > 0) {
				this._defaultValue = resetBtn.attr("data-reset-to");
				new Tooltip(undefined, resetBtn, $("body"), $("body"), false, i18n.t("resetDefaultTT"));
				resetBtn.on('click', _.bind(function () {
					this.setValue(this._defaultValue);
					customization.fireVariableChanged();
				}, this));
			}

			this._colorPatch = context.find(".color-patch");

			this._input.on("change", _.bind(function () {
				this.setValue(this._input.val(), true);
				this._isDirty = false;
				customization.fireVariableChanged();
			}, this));
			if (this._input.is(".less-color-variable")) {
				this._input.ColorPicker({
					onChange: _.bind(function (hsb, hex, rgb) {
						this.setValue("#" + hex.toUpperCase());
					}, this),
					onHide: _.bind(function () {
						if (this._isDirty) {
							customization.fireVariableChanged();
							this._isDirty = false;
						}
					}, this),
					color: this._input.val().substr(1)
				});
				this._colorPatch.ColorPicker({
					onChange: _.bind(function (hsb, hex, rgb) {
						this.setValue("#" + hex.toUpperCase());
					}, this),
					onHide: _.bind(function () {
						if (this._isDirty) {
							customization.fireVariableChanged();
							this._isDirty = false;
						}
					}, this),
					color: this._input.val().substr(1)
				});

			}

			this._isDirty = false;

			this._variableName = this._input.attr("data-lessvar");

		};

		/** 
		 * @return {String} value of this variable
		 */
		VariableSelector.prototype.getValue = function () {
			return this._input.val();
		};

		/**
		 * @return {String} name of this variable
		 */
		VariableSelector.prototype.getVariableName = function () {
			return this._variableName;
		};

		/**
		 * @param {String} value - the value to be set
		 */
		VariableSelector.prototype.setValue = function (value, force) {
			if (this._input.val() !== value || force) {
				this._input.val(value);
				this._input.ColorPickerSetColor(value);
				this._colorPatch.ColorPickerSetColor(value);
				this._colorPatch.css({
					"background-color": value
				});
				this._isDirty = true;
			}
		};

		return VariableSelector;

	});
