define(["underscore", "utils/Map"], function (_, Map) {
    "use strict";

    /**
     * Interface for DataMappers for damconnections
     * 
     * @class
     * @exports DataMapper
     *              
     * @param {JQueryObj} container - Container in which the datamapper will reside
     * @param {DataMapper.types} type - determines which type of rows should be created
     */
    var DataMapper = function (container, type) {
        /**
         * @member {node} the html element corresponding with the dialog
         */
        this.container = container;
        /**
         * @member {node} the html element where the rows have to be added to
         */
        this.body = $(".mapperbody", this.container);

        /**
         * @member {Integer} to indicate which type of row this mapper is managing 
         */
        this.type = type;
        /**
         * @member {Integer} counter to give an Id to the rows 
         */
        this.rowId = 0;
        /**
         * @member {Map} mapping between the rowIds and the rows for the attributemapping
         *                       id -> {row}
         */
        this.rows = new Map();
        /**
         * @member {Map} mapping between the rowId and the elements of that row
         *                       id -> {key, value}
         */
        this.elementsMap = new Map();
        /**
         * @member {Button} Button that will add rows to the container
         */
        this.addRowBtn = null;
    }

    /**
     * @member {Object} Enum to indicate the possible rows
     */
    DataMapper.types = {
        SEARCH: 1, //to define a search request
        FETCH: 2, //to get metadata from the other system
        PUSH: 3, //to push metadata back to the other system
    };

    /**
     * Adds the new metadata to the popup
     * @param {object} data - metadata
     */
    DataMapper.prototype.setMetadata = function (data) {
        //clear the popup
        this.clearMetadata();
        //fill popup with new data
        if (data && !$.isEmptyObject(data) && data.length > 0) {
            if (data) {
                for (var i = 0; i < data.length; i++) {
                    var item = data[i];
                    this._addRow(item);
                }
            }
        }
    };

    /**
     * Clears the popup
     */
    DataMapper.prototype.clearMetadata = function () {
        var rowsValues = this.rows.values();
        for (var i = 0; i < rowsValues.length; i++) {
            rowsValues[i].remove();
        }
        this.rows.clear();
        this.elementsMap.clear();
    };

    /**
     * Deletes the current row from the modal
     * @private
     * @param {integer} id - the id of the calling element
     */
    DataMapper.prototype._deleteRow = function (id) {
        this.rows.get(id).remove();
        this.rows.remove(id);
        this.elementsMap.remove(id);
    };

    return DataMapper;
});
