define(["underscore", "utils/extendedstring"], function(_){
	
	"use strict";
	
	var DocSourceIframeHandler = function () {};
	
	    /**
     * Fetch the external files from the system
     * @param modal
     * @param dam 
     * @param callback - needs to be called when you have the resource information
     */
    DocSourceIframeHandler.prototype.getExternalFiles = function (modal, dam, data, callback) {
    	var namespaces = "";
    	if(data && data.attributeMapping && data.attributeMapping.length > 0){
    		// get comma separated list of namespaces
    		namespaces = _.unique(_.pluck(data.attributeMapping, "namespace")).join(",");
    	}
    	
		var url = dam.url + "/wf/WCR/getSelectionForWCR";
		if(namespaces){
			url += "?namespace=" + encodeURIComponent(namespaces);
		}

    	$.ajax({
            url: url,
            type: "GET",
            dataType: 'jsonp',
            cache: false,
            timeout: 120000
        }).done(function(data){
        	callback(data.options);
        });
    };
    
    /**
     * Get mapping for matching attributes.
     * @param externalAtributes
     * @param DAMMetadata
     * @returns Object that contains Attribute ID or name and value: {id: xx, name:xx, value: xx}
     */
    DocSourceIframeHandler.prototype.getAttributeValueMapping = function(externalAttributes, DAMMetadata){
    	var attrMap = [];
    	if(externalAttributes && DAMMetadata.attributeMapping){
    		_.each(DAMMetadata.attributeMapping, function(mapping){
    			var fullAssetName = mapping.namespace + " " + mapping.field;
    			if(externalAttributes[fullAssetName]){
    				attrMap.push({id: mapping.value, value: externalAttributes[fullAssetName]});
    			}
    		});
    	}
    	return attrMap;
    };
    
    /**
     * returns the extra parameters that have to be passed through the URL based on the defined metadata.
     * @private
     * @param {String} metadata
     */
    DocSourceIframeHandler.prototype.getUrlExtension = function (metadata) {
    	if (!metadata) {
            metadata = {};
        }

     	// hide menu
        var url = "mod=hideMenu;;"; 
        //specify workspace
        if (metadata.workspace && metadata.acl) {
        	// selected workspace
        	url += "acl=" + encodeURIComponent(metadata.acl) + ";";
            url += "w=" + metadata.workspace + ";;";
        } else if (metadata.workspace) {
        	// selected workspace
            url += "p=" + metadata.workspace + ";;";
        } else {
        	// automatically select
        	url += "act=spv;;";
        }
        var addedSearchValues = false;
        if (metadata.searchitems) {
            for (var i = 0; i < metadata.searchitems.length; i++) {
                var item = metadata.searchitems[i];

                if (item.value !== "" && item.namespace !== undefined) {
                    if (!addedSearchValues) {
                        url += "loc=";
                        addedSearchValues = true;
                    }
                    url += "f=A+";
                    //search element
                    url += item.namespace + "%20" + item.field;
                    url += "*";
                    url += item.namespace.replaceAll("/", "%5C/").replaceAll(":", "%5C:") + "%5C+%5C+" + item.field;
                    url += (item.is_container) ? "_cont" : "_s";
                    url += ":";
                    //value
                    url += encodeURIComponent(item.value.toLowerCase());
                    url += ";";
                }
            }
        }

        return "/gwtmain/GwtMain.html?dest=!MP;;" + url + "#!MP;;" + url;
    };
    
    /**
     * Get iframe sandbox mode
     * @returns {String} sandbox mode
     */
    DocSourceIframeHandler.prototype.getSandboxMode = function(){
    	return null;
    };
    
    /**
     * Extra behavior on page load
     */
    DocSourceIframeHandler.prototype.onPageLoad = function(dam, data, dlsObj){
    	// check page parameter for mbdata
    	var params = this.getQueryParams();
    	var mbdata = params.mbdata;
    	
    	if(mbdata){
    		var dataObj = JSON.parse(mbdata);
    		if(dataObj.system === arguments[0].id || dataObj.system === arguments[0].name){
    			this.getExternalFiles(null, dam, data, function(documents){
                    $.each(documents, function (i,selectedDocument) {
                        selectedDocument.DAM = dam;
                        selectedDocument.DAMJS = this;
                        dlsObj.docTableObj.addExternalFileRow(dlsObj, selectedDocument);
                    });
    			});
    		}
    	}
    };
	
    /**
     * Get the parameters from the url
     * @returns {object}
     */
    DocSourceIframeHandler.prototype.getQueryParams = function(){
    	var qs = window.location.search;
        qs = qs.split('+').join(' ');

        var params = {},
            tokens,
            re = /[?&]?([^=]+)=([^&]*)/g;

        while (tokens = re.exec(qs)) {
            params[decodeURIComponent(tokens[1])] = decodeURIComponent(tokens[2]);
        }

        return params;
    };
	
	return DocSourceIframeHandler;
});