define(["WFEditor/workflow/panel/CompositePanelComponent",
	"WFEditorWCR/workflow/panel/SmartnameInputField",
	"WFEditorWCR/workflow/panel/BooleanPanel",
	"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
	"utils/webi18n!MediaBeaconDAMPushDocumentNode",
	"utils/webi18n!WFEditorWCR",
	"underscore",
	"WFEditor/workflow/panel/Parameter",
	"damconnections/MediaBeacon/MBDataMapper"
], function (CompositePanelComponent,
	SmartNameInputField,
	BooleanPanel,
	WebCenterPanelConstants,
	i18n,
	WFEditorWCRStrings,
	_,
	Parameter,
	MBDataMapper) {

	"use strict";

	/**
	 * @class
	 * @augments CompositePanelComponent
	 * Handler of the push to dam Workflow Node for the MediaBeacon Plugin
	 * @param {Object} api - used to get the workflow dictionary service
	 */
	var WorkflowConfig = function (api) {
		CompositePanelComponent.call(this, []);
        
        /**
         * @member {MBDataMapper} MBDataMapper that manages the rows
         */
        this.mapper = null;
        /**
         * @member {Object} api
         */
        this.api = api;

		// Create all the panels
		var pathPanel = new SmartNameInputField("path", i18n.t("path"), api.getWorkflowDictionaryService());
		var addAsNewVersionPanel = new BooleanPanel("addasnewversion", WFEditorWCRStrings.t("copymove.newversionifexists"));

		this.add(pathPanel);
		this.add(addAsNewVersionPanel);

		this._element.addClass("parameter-panel form-horizontal form-compact");

	};

	WorkflowConfig.prototype = Object.create(CompositePanelComponent.prototype);
	WorkflowConfig.prototype.constructor = WorkflowConfig;

	/**
	 * Init the container
	 */
	WorkflowConfig.prototype.init = function (container, data) {
		var defaultParams = this._getDefaultParameters();
		_.each(data, function (val, key) {
			var param = _.find(defaultParams, function (param) {
				return param.getName() === key;
			});
			if (param) {
				param.setValue(val);
			}
		});
		this._element.prependTo(container);
		
		// init metadata fields
        var fieldJSON = JSON.parse($("input[name='fieldMap']", container).val());
		this.mapper = new MBDataMapper(container, MBDataMapper.types.PUSH, fieldJSON, this.api.getWorkflowDictionaryService());
		this.mapper.setMetadata(data.mapping);
		
		CompositePanelComponent.prototype.init.call(this);
		this.setParameters(defaultParams);

	};

	WorkflowConfig.prototype._getDefaultParameters = function () {
		return [new Parameter("path", "", ""),
			new Parameter("addasnewasset", "", "false"),
			new Parameter("addasnewversion", "", "false"),
			new Parameter("addasnewdocornot", "", "addasnewversion"),
		];
	};

	/**
	 * should retrieve the data from the input
	 */
	WorkflowConfig.prototype.getData = function () {
		var object = {};
		var parameters = this.getParameters();
		_.each(parameters, function (param) {
			object[param.getName()] = param.getValue();
		});
		object.mapping = this.mapper.getMetaData();
		return object;


	};
    
    /**
     * Disables all elements
     * @override
     */
    WorkflowConfig.prototype.disable = function (){
        CompositePanelComponent.prototype.disable.call(this);
        this.mapper.disable();
    }
    
    /**
     * Disables all elements
     * @override
     */
    WorkflowConfig.prototype.enable = function (){
        CompositePanelComponent.prototype.enable.call(this);
        this.mapper.enable();
    }
	
	/**
	 * Get a width that can show the metadata mapping properly
	 * @returns {Number}
	 */
	WorkflowConfig.prototype.getPreferredWidth = function(){
		return 700;
	};
	

	return WorkflowConfig;

});
