define(["underscore"], function(_){
	
	"use strict";
	
	var DocSourceIframeHandler = function() {};	
	
	/**
     * Returns the extra parameters that have to be passed through the URL based on the defined metadata.
     * @param   {object} metadata
     * @returns {string} string that have to be added to the DAM url
     */
    DocSourceIframeHandler.prototype.getUrlExtension = function (metadata) {
        if (!metadata) {
            metadata = {};
        }
        var params = metadata.parameters || [];

        var staticSmartNames = "<staticsmartnames>";
        // create static smartnames for other server
        for (var i = 0; i < params.length; i++) {
            var item = params[i];
            if (item.value !== "") {
                staticSmartNames += "<sn>";
                staticSmartNames += "<n>" + item.key + "</n>";
                staticSmartNames += "<v>" + item.value + "</v>";
                staticSmartNames += "</sn>";
            }
        }
        staticSmartNames += "</staticsmartnames>";

        return "/dosavedsearch.jsp?searchName=" + encodeURIComponent(metadata.savedSearch) + "&sandBox=1" + (staticSmartNames ? "&staticSmartNames=" + encodeURIComponent(staticSmartNames) : "");
    };

    /**
     * Actions that need to be triggered when Asset Popup opens
     * @param modal
     */
    DocSourceIframeHandler.prototype.onOpenPopup = function (modal) {
    	var self = this;
    	var confirmButton = modal.getModalFooter().find(".btn-primary");
    	confirmButton.prop('disabled', true);
    	this.windowMessageListener = function (e) {
            if (e.data.command === "notify_parent") {
                confirmButton.prop('disabled', false);
            } else if (e.data.command === "return_selected_files") {
                if(self.callback){
                	self.callback(e.data.options);
                }
            }
        };
    	
        window.addEventListener("message", this.windowMessageListener, false);
    };
    
    /**
     * Actions that need to be triggered when Asset Popup closes
     * @param modal
     */
    DocSourceIframeHandler.prototype.onClosePopup = function (modal) {
    	window.removeEventListener("message", this.windowMessageListener);
    };
    
    /**
     * Fetch the external files from te system
     * @param iframe
     * @param callback - needs to be called when you have the resource information
     */
    DocSourceIframeHandler.prototype.getExternalFiles = function (modal, dam, data, callback) {
    	var iframe = modal.getModalContent().find("iframe");
    	if(iframe && iframe[0]){
    		iframe[0].contentWindow.postMessage({
                command: "get_selected_files"
            }, "*");
        	this.callback = callback;
    	}
    };
    
    /**
     * Get mapping for matching attributes.
     * @param externalAttributes
     * @param DAMMetadata
     * @returns Object that contains Attribute ID or name and value: {id: xx, name:xx, value: xx}
     */
    DocSourceIframeHandler.prototype.getAttributeValueMapping = function(externalAttributes, DAMMetadata){
    	var attrMap = [];
    	if (externalAttributes && externalAttributes.atts && externalAttributes.atts.att) {
            // Adds the attributes to the map based on matching names
            if (DAMMetadata.matchAttributeByName) {
                _.each(externalAttributes.atts.att, function (externalAttr) {
                	attrMap.push({name: externalAttr.an, value: externalAttr.av});
                });
            }

            // Adds the attributes to the map based on configured mapping
            _.each(DAMMetadata.attributeMapping, function (mapping) {
                var externalAttr = _.find(externalAttributes.atts.att, function (attr) {
                    return attr.id === mapping.externalAttribute;
                });
                if(externalAttr){
                	attrMap.push({id: mapping.internalAttribute, value: externalAttr.av});
                }
            }.bind(this));
        }
    	return attrMap;
    };
    
    /**
     * Get iframe sandbox mode
     * @returns {String} sandbox mode
     */
    DocSourceIframeHandler.prototype.getSandboxMode = function(){
    	return "allow-same-origin allow-scripts allow-popups allow-forms";
    };
    
    /**
     * Extra behavior on page load
     */
    DocSourceIframeHandler.prototype.onPageLoad = function(){
    	// do nothing
    };
	
	return DocSourceIframeHandler;
	
	
	
	
});