define(["WFEditor/workflow/panel/CompositePanelComponent",
    "WFEditorWCR/workflow/panel/SmartnameInputField",
    "WFEditorWCR/workflow/panel/RadioGroupPanel",
    "WFEditorWCR/workflow/panel/RadioPanel",
    "WFEditorWCR/workflow/panel/WebCenterPanelConstants",
    "utils/webi18n!WebCenterDAMPushDocumentNode",
    "underscore",
    "WFEditor/workflow/panel/Parameter",
    "damconnections/Webcenter/WCRDataMapper"
], function (CompositePanelComponent,
    SmartNameInputField,
    RadioGroupPanel,
    RadioPanel,
    WebCenterPanelConstants,
    i18n,
    _,
    Parameter,
    WCRDataMapper) {

    "use strict";

    /**
     * @class
     * @augments CompositePanelComponent
     * Handler of the push to dam Workflow Node for the MediaBeacon Plugin
     * @param {Object} api - used to get the workflow dictionary service
     */
    var WorkflowConfig = function (api) {
        CompositePanelComponent.call(this, []);

        /**
         * @member {node} element (checkbox) that defines if autonaming will be enable/disabled
         */
        this.autonamingInput = null;
        /**
         * @member {WCRDataMapper} WCRDataMapper that manages the rows for the request
         */
        this.mapper = null;
        /**
         * @member {Object} api
         */
        this.api = api;


        // Create all the panels
        var projectNamePanel = new SmartNameInputField("projectname", i18n.t("projectname"), api.getWorkflowDictionaryService());
        var addAsNewDocumentPanel = new RadioPanel("addasnewdoc", i18n.t("addasnewdoc"));
        var projectFolderPanel = new SmartNameInputField("folder", i18n.t("folder"), api.getWorkflowDictionaryService());
        var descriptionPanel = new SmartNameInputField("docdescription", i18n.t("docdescription"), api.getWorkflowDictionaryService());
        var addAsNewVersionPanel = new RadioPanel("addasnewversion", i18n.t("addasnewversion"));
        var newVersionCommentPanel = new SmartNameInputField("comment", i18n.t("comment"), api.getWorkflowDictionaryService());
        var addAsNewDocOrNotPanel = new RadioGroupPanel("addasnewdocornot", WebCenterPanelConstants.SUBPARAMETERSTATE.INVISIBLE, i18n.t("uploaddocto", api.getDAMName()));

        // Add them to each other
        addAsNewDocumentPanel.add(projectNamePanel);
        addAsNewDocumentPanel.add(projectFolderPanel);
        addAsNewDocumentPanel.add(descriptionPanel);
        addAsNewVersionPanel.add(newVersionCommentPanel);

        addAsNewDocOrNotPanel.add(addAsNewVersionPanel);
        addAsNewDocOrNotPanel.add(addAsNewDocumentPanel);

        this.add(addAsNewDocOrNotPanel);

        this._element.addClass("parameter-panel form-horizontal form-compact");

    };

    WorkflowConfig.prototype = Object.create(CompositePanelComponent.prototype);
    WorkflowConfig.prototype.constructor = WorkflowConfig;

    /**
     * Init the container
     */
    WorkflowConfig.prototype.init = function (container, data) {
        var defaultParams = this._getDefaultParameters();
        _.each(data, function (val, key) {
            var param = _.find(defaultParams, function (param) {
                return param.getName() === key;
            });
            if (param) {
                param.setValue(val);
            }
        });
        this._element.prependTo(container);

        // init metadata fields
        this.autonamingInput = $("input[name='matchByName']", container);
        if (data.matchAttributeByName) {
            this.autonamingInput.prop('checked', data.matchAttributeByName);
        }
        var attributesXML = $("input[name='attributesXML']", container).val();
        this.mapper = new WCRDataMapper(container, WCRDataMapper.types.PUSH, attributesXML, this.api.getWorkflowDictionaryService());
        this.mapper.setMetadata(data.mapping);

        CompositePanelComponent.prototype.init.call(this);
        this.setParameters(defaultParams);

    };

    WorkflowConfig.prototype._getDefaultParameters = function () {
        return [new Parameter("projectname", "", ""),
            new Parameter("addasnewdoc", "", "false"),
            new Parameter("addasnewversion", "", "false"),
            new Parameter("docdescription", "", ""),
            new Parameter("addasnewdocornot", "", "addasnewversion"),
            new Parameter("comment", "", ""),
            new Parameter("folder", "", "")
        ];
    };

    /**
     * should retrieve the data from the input
     */
    WorkflowConfig.prototype.getData = function () {
        var object = {};
        var parameters = this.getParameters();
        _.each(parameters, function (param) {
            object[param.getName()] = param.getValue();
        });
        object.mapping = this.mapper.getMetaData();
        object.matchAttributeByName = this.autonamingInput.prop('checked');
        return object;
    };
    
    /**
     * Disables all elements
     * @override
     */
    WorkflowConfig.prototype.disable = function (){
        CompositePanelComponent.prototype.disable.call(this);
        this.mapper.disable();
        this.autonamingInput.prop('disabled', true);
    }
    
    /**
     * Disables all elements
     * @override
     */
    WorkflowConfig.prototype.enable = function (){
        CompositePanelComponent.prototype.enable.call(this);
        this.mapper.enable();
        this.autonamingInput.prop('disabled', false);
    }

    /**
     * Get a width that can show the metadata mapping properly
     * @returns {Number}
     */
    WorkflowConfig.prototype.getPreferredWidth = function () {
        return 600;
    };

    return WorkflowConfig;

});
