define(["../core/Block", "underscore","lib/PromisePolyfill"], function(Block, _, PromisePolyfill){
	"use strict";
	
	/** 
	 * Configurator for a Dashboard
	 * @augments Block
	 */
	var DashboardBuilder = function(id, context){
		Block.call(this, id, context);
		
		this._layout = null;
		this._isActive = false;
	};
	
	DashboardBuilder.prototype = Object.create(Block.prototype);
	DashboardBuilder.prototype.constructor = DashboardBuilder;

	/**
	 * @Override
	 */
	DashboardBuilder.prototype.getType = function(){
		return "dashboard/builder/DashboardBuilder";
	};
	 
	
	/**
	 * @Override
	 */
	DashboardBuilder.prototype.isPersistent = function(){
		return false;
	};
	
	/**
	 * @Override
	 * @protected
	 */
	DashboardBuilder.prototype._doConfigure = function(config){
		this._dashboard = config.dashboard;
		this._dashboardLayout = config.layout;
	};
	
	
	/**
	 * @Override
	 */
	DashboardBuilder.prototype._doInit = function(){
		var self = this; 
		
		this._galleryID = this._context.registerBlock({
			type: "dashboard/builder/gallery/Gallery",
			config: {}
		});
		
		this._layout = this._context.registerAndLoadBlock({
			type: "dashboard/layout/flex/FlexLayout",
            config: {
                items: [],
				parameters: [{
					name: "FlexColumnParameter",
					type: "dashboard/layout/flex/FlexColumnParameter",
                    config:{
                        value: {
                            columns: [
                                {grow: 1, shrink:1, lockedColumn: true}, 
                                {grow: 0, shrink: 0, width: '15%', maxWidth: "300px", minWidth: "150px", lockedColumn: true}
                            ],
                            flexDirection: "row"
                        }
                    }
				}]
			}
		});
		
		return this._layout.then(function(layout){			
			self._registerChild(layout);			
			layout.replaceColumnWithBlock(1, self._galleryID);
			
			
			self._element.append(layout.getElement());					
			return layout;
		});
	};
	
	
	/**
	 * Activate the builder
	 * Move in the layout element if needed
	 * @param {Boolean} active 
	 */
	DashboardBuilder.prototype.activate = function(active){
		if (this._isActive != active){
			if (active){
				Promise.all([this._layout, this._dashboardLayout]).then(function(values){
					var builderLayout = values[0];
					var dashboardLayout = values[1];
				
					builderLayout.replaceColumnWithBlock(0, dashboardLayout.getID());
				});				
				
			}
			
			this._isActive = active;
		}
	};
	
	
	return DashboardBuilder;
});