define(["../ParameterConfigurationPanel",
        "jquery",
        "underscore",
        "colorpicker",
        "utils/webcss!web-viewer/layout/general/css/colorpicker.css"
    ],
    function (ParameterConfigurationPanel,
        $,
        _,
        colorpicker) {

        "use strict";

        /**
         * Configuration panel for a HTML parameter
         * @augments ParameterConfigurationPanel
         * @param {[[Type]]} parameter [[Description]]
         */
        var ColorPickerConfigurationPanel = function (parameter) {
            ParameterConfigurationPanel.call(this, parameter);
        };

        ColorPickerConfigurationPanel.prototype = Object.create(ParameterConfigurationPanel.prototype);
        ColorPickerConfigurationPanel.prototype.constructor = ColorPickerConfigurationPanel;

        /**
         * By default a parameter is shown as a form-group with a label and an input field
         * @private
         */
        ColorPickerConfigurationPanel.prototype._init = function () {
            // we don't display a label currently :)
            this._element.addClass("form-group");

            this._$label = $("<label>", {
                text: this._parameter.getUserFriendlyName()
            });
            this._$input = $("<input>", {
                type: "text",
                'class': "form-control",
                value: this._parameter.getValue()
            });


            this._$colorpatch = $("<div>", {
                'class': "colorPickerColorPatch"
            });
            this._$colorpatch.ColorPicker({
                'onChange': function (hsb, hex, rgb) {
                    this._setColor(hex);
                }.bind(this),
                'onHide': function () {
                    this._parameter.undoableSetValue(this._color);
                }.bind(this),
                'onBeforeShow': function () {
                    $(this).ColorPickerSetColor(this._color);

                }.bind(this),
                'onSubmit': function (hsb, hex, rgb, el) {
                    this._setColor(hex);
                    this._parameter.undoableSetValue(this._color);
                    $(el).ColorPickerHide();
                }.bind(this)
            });

            this._$input.change(function () {
                var newColor = this._$input.val();
                if (this._validateHex(newColor)) {
                    this._setColor(newColor);
                    this._parameter.undoableSetValue(newColor);
                } else {
                    this._$input.val(this._color);
                }
            }.bind(this));
            this._observeParam(function () {
                this._$input.val(this._parameter.getValue());
                this._setColor(this._parameter.getValue());
            }.bind(this));

            this._element.append(this._$label).append($("<div>", {
                'class': "form-entry"
            }).append($("<div>", {
                'class': "input-group"
            }).append(this._$input).append(this._$colorpatch)));


            this._setColor(this._parameter.getValue());
        };


        /**
         * Change the color (this updates both input and color patch)
         * @private
         * @param {string} hex [[Description]]
         */
        ColorPickerConfigurationPanel.prototype._setColor = function (hex) {
            this._color = hex;
            if (hex.indexOf('#') < 0) {
                this._color = "#" + this._color;
            }

            this._$input.val(this._color);
            this._$colorpatch.css("background-color", this._color);
            this._$colorpatch.ColorPickerSetColor(hex.replace("#", ""));

        };


        /**
         * @override
         * @private
         * @returns {object} [[Description]]
         */
        ColorPickerConfigurationPanel.prototype._getCustomConfiguration = function () {
            return {
                value: this._color
            };
        };

        /**
         * Checks if the given hex string is valid (eg. #fff or #ffffff)
         * @private
         * @param   {string} hex
         * @returns {boolean} True if valid, otherwise false
         */
        ColorPickerConfigurationPanel.prototype._validateHex = function (hex) {
            return /^#([0-9a-f]{3}){1,2}$/i.test(hex);
        };


        return ColorPickerConfigurationPanel;
    });
