define(["../ParameterConfigurationPanel",
        "dashboard/layout/flex/UnitInputField",
        "utils/webi18n!Dashboard",
		"jquery",
		"underscore",
		],
	function (ParameterConfigurationPanel,
			  UnitInputField,
			  i18n,
			  $,
			  _) 
	{

		"use strict";

		/**
		 * Configuration panel for a CSS Size parameter
		 * @augments ParameterConfigurationPanel
		 * @param {[[Type]]} parameter [[Description]]
		 */
		var CssSizeParameterConfigurationPanel = function (parameter) {
			this._unitTypes = ["px", "%", "em"];
			if (parameter.getUnitTypes()) {
				this._unitTypes = parameter.getUnitTypes();
			}
			ParameterConfigurationPanel.call(this, parameter);
		};

		CssSizeParameterConfigurationPanel.prototype = Object.create(ParameterConfigurationPanel.prototype);
		CssSizeParameterConfigurationPanel.prototype.constructor = CssSizeParameterConfigurationPanel;

		/**
		 * By default a parameter is shown as a form-group with a label and an input field
		 * @private
		 */
		CssSizeParameterConfigurationPanel.prototype._init = function () {
			var self = this;
            
			this._input = new UnitInputField(this._parameter.getUserFriendlyName(), "0px", this._unitTypes, true, {max: 999, min: 0});
            this._input.on('changed', function(){
                self._parameter.undoableSetValue(self._input.getValue());
            });
            this._input.setValue(this._parameter.getValue());
            
            this._observeParam(function(){
                this._input.setValue(this._parameter.getValue());
            }.bind(this));
            
			this._element.append(this._input.element);
		};

		/**
		 * @override
		 * @private
		 * @returns {object} [[Description]]
		 */
		CssSizeParameterConfigurationPanel.prototype._getCustomConfiguration = function () {
			return {
				value: this._input.getValue()
			};
		};


		return CssSizeParameterConfigurationPanel;
	});