define(["../ParameterConfigurationPanel",
		"jquery",
		"underscore",
		"lib/summernote/summernote",
		"utils/webcss!lib/summernote/summernote.css"
	],
	function (ParameterConfigurationPanel,
		$,
		_,
		summernote,
		summernotecss
		) {

		"use strict";

		/**
		 * Configuration panel for a number parameter
		 * @augments ParameterConfigurationPanel
		 * @param {Parameter} parameter 
		 */
		var NumberParameterConfigurationPanel = function (parameter) {
			ParameterConfigurationPanel.call(this, parameter);
			
			this._validNumber = this._parameter.getDefaultNumber();
		};

		NumberParameterConfigurationPanel.prototype = Object.create(ParameterConfigurationPanel.prototype);
		NumberParameterConfigurationPanel.prototype.constructor = NumberParameterConfigurationPanel;

		/**
		 * By default a parameter is shown as a form-group with a label and an input field
		 * @private
		 */
		NumberParameterConfigurationPanel.prototype._init = function () {
			this._element.addClass("form-group");

			this._$label = $("<label>", {
				text: this._parameter.getUserFriendlyName()
			});
			
			var constraints = this._parameter.getConstraints();
			this._$input = $("<input>", {
				type: "number",
				min: constraints.min,
				max: constraints.max,				
				'class': "form-control",
				value: this._parameter.getValue()
			});

			this._$input.attr("data-param-name", this._parameter.getName());

			this._$help = $("<div>", {
				'class': "help-block",
			});
			this._$help.append($("<div>",{
				text: this._parameter.getHelpText(),
				'class': "pull-left"
			})).append($("<div>",{
				'class': "icon icon-WCR_Info-round12 icon-grey pull-left",
				title: this._parameter.getToolTipText(),
			}));
			
			this._$input.on('change', function(event){
				if(this._$input.val() === "" || this._$input.val() === "-"){
					this._$input.val(this._parameter.getDefaultNumber());
				}
            	this._parameter.undoableSetValue(this._validNumber.toString(10));
        	}.bind(this));
			
			// Every change from the user is instantly checked and corrected if needed.
			this._$input.on('input', function(event){
				var defaultNumber = this._parameter.getDefaultNumber();
				var value = this._$input.val();
				
				var number = value;
				var valid = true;
				if(value){
					if(isNaN(Number(number))){
						valid = false;
					} else if(constraints){
						if(constraints.type === "int"){
							number = parseInt(value, 10);
						} else if (constraints.type === "float"){
							number = parseFloat(value, 10);
						} else {
							//Wrong constraint type
							valid = false;
						}
						if("min" in constraints){
							if(number < constraints.min){
								valid = false;
							}						
						} 
						if("max" in constraints){
							if(number > constraints.max){
								valid = false;
							}						
						} 
					}
				} else {
					valid = false;
				}
				if(valid){
					this._validNumber = number;
				} else {
					this._validNumber = defaultNumber;
					if(value !== "" && value !== "-"){ //allow input of the "-" for negative numbers
						this._$input.val(defaultNumber);						
					}
				}
			}.bind(this));

			this._observeParam(function(){
				this._$input.val(this._parameter.getValue());
			}.bind(this));

			this._element.append(this._$label).append($("<div>", {
				'class': "form-entry"
			}).append(this._$input).append(this._$help));

		};

		return NumberParameterConfigurationPanel;
	});