define(["utils/Promisify"], function (Promisify) {
	"use strict";

	/**
	 * A Factory is responsible for initializing a block of a certain type
	 * The default factory uses Require to load the definition of a block
	 * Custom factories could be defined to use other loading mechanisms
	 * @constructor
	 * @class
	 */
	var Factory = function () {

	};

	/**
	 * Initialize a block of a certain type
	 * @param {String} id - ID of the block
	 * @param {Context} context - context in which the block is being initialized
	 * @param {String} type - type definition of the block
	 * @param {Object} config - configuration object of the block 
	 */
	Factory.prototype.initBlock = function (id, context, type, config) {
		// load the block definition from require
		return Promisify.require(type).then(function(Block){
			//initialize the block
			return new Block(id, context);
		}).catch(function(error){
			error.message = "Could not load definition for Block ("+id+", type: "+type+") \r\nRequireJS error: " + error.message;
			throw error;
		});
	};


	return Factory;
});