define(["./Parameter", "utils/Promisify"], function (Parameter, Promisify) {
	"use strict";

	/**
	 * Css Size Parameter
	 * @augments Parameter
	 * @param {String} name             - name of the parameter
	 * @param {String} userFriendlyName - user friendly representation of the parameter
	 */
	var CssSizeParameter = function (name, context, userFriendlyName, unitTypes) {
		this._unitTypes = unitTypes;
		Parameter.call(this, name, context, "dashboard/core/parameters/CssSizeParameter", userFriendlyName);
	};

	CssSizeParameter.prototype = Object.create(Parameter.prototype);
	CssSizeParameter.prototype.constructor = CssSizeParameter;

	
	/**
	 * Get the configuration element for the parameter
	 * @returns {Promise.<ConfigurationPanel>} 
	 */
	CssSizeParameter.prototype.getConfigurationPanel = function(){
		return Promisify.require("dashboard/builder/configuration/parameters/CssSizeParameterConfigurationPanel").then(function(ParameterConfigurationPanel){
			return new ParameterConfigurationPanel(this);
		}.bind(this));
	};
	
	CssSizeParameter.prototype.getUnitTypes = function(){
		return this._unitTypes;
	};
	
	return CssSizeParameter;
});