define(["./Parameter", "utils/Promisify"], function (Parameter, Promisify) {
	"use strict";

	/**
	 * Number Parameter
	 * @augments Parameter
	 * @param {String} name             - name of the parameter
	 * @param {String} userFriendlyName - user friendly representation of the parameter
	 * @param {number} defaultNumber	- default value for when the constraints are not met 
	 * @param {object} constraints      - { type: "int"/"float", [min: <number>, max: <number>] }
	 */
	var NumberParameter = function (name, context, userFriendlyName, defaultNumber, constraints, helpText, extraInfoText) {
		Parameter.call(this, name, context, "dashboard/core/parameters/NumberParameter", userFriendlyName, helpText, extraInfoText);
		
		this._defaultNumber = defaultNumber;
		this._constraints = constraints;
	};

	NumberParameter.prototype = Object.create(Parameter.prototype);
	NumberParameter.prototype.constructor = NumberParameter;

	/**
	 * Get the default number
	 * @returns {number} 
	 */
	NumberParameter.prototype.getDefaultNumber = function(){
		return this._defaultNumber;
	};
	
	/**
	 * Get the constraints
	 * @returns {object} - { type: "int"/"float", [min: <number>, max: <number>] }
	 */
	NumberParameter.prototype.getConstraints = function(){
		return this._constraints;
	};
	
	/**
	 * Get the configuration element for the parameter
	 * @returns {Promise.<ConfigurationPanel>} 
	 */
	NumberParameter.prototype.getConfigurationPanel = function(){
		return Promisify.require("dashboard/builder/configuration/parameters/NumberParameterConfigurationPanel").then(function(ParameterConfigurationPanel){
			return new ParameterConfigurationPanel(this);
		}.bind(this));
	};
	
	return NumberParameter;
});