define(["doctablex/DocTableXUtils"],
	function (DocTableXUtils) {

		/**
		 * AddedTextContentRow.js
		 *
		 * Object representing a single text statement just added to given documents table (before it's saved to backend).
		 * It holds required data and implements controller logic of the represented text statement (row).
		 *
		 * @author JANH, January 2016
		 */

		"use strict";

		var AddedTextContentRow = function () {

			this.__isMixin = true; //JANH debug then DEL!!!


			/**
			 * This method overrides the TextContentRow.updateOtherInfoFromServer() to do nothing
			 * for all created (new) document rows. They handle the initialization of the text content
			 * themselves at the right time...
			 * 
			 * @override mixin
			 * 
			 * @returns Always false as it never updates text content.
			 */
			if (this.updateOtherInfoFromServer) {
				AddedTextContentRow.__overridden.updateOtherInfoFromServer = this.updateOtherInfoFromServer;
			}
			this.updateOtherInfoFromServer = function (repaint, forceUpdate) {
				// Call super
				AddedTextContentRow.__overridden.updateOtherInfoFromServer.apply(this, arguments);

				// text content specific document row classes representing document sources have their own
				// initialization of the text content!
				return false; // nothing changed in this method
			};

			/**
			 * Method generating XML to save changes in this text statement row.
			 * This method is dedicated for text content document reference attributes only!
			 * Text content updates are inside <text_content> tag as JSON String
			 * 
			 * @override mixin
			 * @returns {String} - XML complying to JavaScript -> JSP interface to save changes done in document table.
			 */
			if (this.convertToXML) {
				AddedTextContentRow.__overridden.convertToXML = this.convertToXML;
			}
			this.convertToXML = function (dsXML) {
				var xmlValue = '';
				try {
					// add text content data changes if any
					if (dsXML && DocTableXUtils.hasTextContentItem(this.renderOptions.columns)) {
						var tcXML = this.convertTextContentToXML();
						if (tcXML) {
							// insert text content XML into <ds> tag (at the end)
							// this way we avoid adding text content XML in all the inherited row classes
							dsXML = dsXML.replace('</ds>', tcXML + '</ds>');
						}
					}

					// call the original DocumentRow.convertToXML() method (overwritten by this mixin method)
					// containing the common logic
					xmlValue += AddedTextContentRow.__overridden.convertToXML.call(this, dsXML);

				} catch (err) {
					DocTableXUtils.log('Problem in AddedTextContentRow.convertToXML:\n' + err);
					xmlValue = '';
				}
				return xmlValue;
			};

		};


		/**
		 * Mixin constructor/initializer.
		 * (class method)
		 */
		AddedTextContentRow.__initMixin = function (options) {

			// this data model mixin class does not have view mixin class counterpart (yet)
			//    	AddedTextContentRowView.__initMixin.call(this.view, this.renderOptions);   // the view mixin class is not needed so far...

			//TODO: Move the text content related loading methods from StaticDocumentRowView to this class
			//when have factory method to create row view classes dynamically

			return this;
		};


		/**
		 * List of overridden methods with their original code.
		 */
		AddedTextContentRow.__overridden = {};


		return AddedTextContentRow;
	});