/*
 * TODO:
 * 
 * - Refactor localization strings - introduce new "application" for this class
 * - Validation is missing! (not regression)
 * - Reference attribute category "on load" callback doesn't work!
 * 
 * 
 */


define(["wcr/webcenter_xml",
		"utils/webi18n!ACatStrings",
		"utils/webi18n!ACLayoutStrings",
        "utils/Promisify",
        "jquery",
		"underscore",
		"jqueryUI",
		"lib/PromisePolyfill",
		"utils/webcss!./DocTableXRenderConfig.css"],
	function (WCXML,
		ACatStrings,
		ACLayoutStrings,
		Promisify,
		$,
		_) {

		/**
		 * DocTableXRenderConfig.js
		 * 
		 * This class implements a class handling rendering options and configuration of a documents table (DocTableX class).
		 * 
		 * @author JANH - September 2015
		 */

		"use strict";

		var DocTableXRenderConfig = function (options) {

			this.defaultName = options.defaultName || '';

			this.section = options.section || '';

			this.phElemID = options.phElemID || 'DocRenderConfigDivID';
			this.functionPrefix = '$(\'#' + this.phElemID + '\')[0].wrapperObj';

			this.defaultAttrCatView = options.defaultAttrCatView;
			if (!this.defaultAttrCatView && (this.section != 'TaskDocuments')) {
				// in task documents, it's not an option to set 'Default' attr. cat. view by default
				this.defaultAttrCatView = 'Default';
			}


			this.dltProperties = ['type', 'typeDetail', 'label', 'visibility', 'thumbnailSize', 'useCreationView', 'uploadNewVersion', 'projectStatusLayout', 'attributeName', 'width', 'displayLanguages', 'langShowMode', 'retainTextContentStatus', 'hideDescription', 'hideSource', 'hideFolder', 'hideDocTemplate', 'hide'];
			this.documentActions = ['View', 'OpenAcad', 'Open3d', 'Download', 'UploadNewVersion'];
			this.dltTypes = ['DocumentName', 'DocumentActions', 'DocumentProperty', 'ReferenceProperty', 'DocumentAttribute', 'AttributeFlapout', 'ProjectProperty', 'ProjectAttribute', 'LocalRemote', 'SequenceNumber', 'RowSelector', 'UpDownArrows', 'ReplaceButton', 'CloneButton', 'DeleteButton'];

			// When the PCM license is active, add options to show the text content cols.
			if (window.webcenterConfig.isPCMEnabled) {
				this.dltTypes = ['DocumentName', 'DocumentActions', 'DocumentProperty', 'ReferenceProperty', 'DocumentAttribute', 'AttributeFlapout', 'ProjectProperty', 'ProjectAttribute', 'MasterText', 'TranslationText', 'TextLanguages', 'LocalRemote', 'SequenceNumber', 'RowSelector', 'UpDownArrows', 'ReplaceButton', 'CloneButton', 'DeleteButton'];
			}

			if (this.section == 'TaskDocuments') {
				this._removePredefinedOption(this.dltTypes, 'ReferenceProperty');
				this._removePredefinedOption(this.dltTypes, 'DocumentAttribute');
			}

			this.dltDocPropertyDetails = ['Description', 'VersionRevision', 'Project', 'ApprovalStatus', 'ApprovalStage', 'VersionAuthor', 'ModifiedDate', 'Thumbnail'];
			this.dltDocThumbSizes = ['25', '50', '75', '100', '150', '200'];

			this.dltProjectPropertyDetails = ['ProjectName', 'ProjectStatus', 'ProjectDueDate'];
			this.dltProjectStatusOptions = ['LabelAndIcon', 'OnlyLabel', 'ColorLabel', 'OnlyIcon'];

			if (options.configObj) {
				this.configObj = options.configObj;
				// fix potential incomplete configuration object
				if (!this.configObj.name) {
					this.configObj.name = this.defaultName;
				}
				if (this.configObj.hideheader !== true && this.configObj.hideheader !== false) {
					// note hideHeader should be a real boolean
					this.configObj.hideHeader = false;
				}
				if (this.configObj.useCreationView !== false) {
					// by default the useCreationView is enabled
					this.configObj.useCreationView = true;
				}
				if (!this.configObj.columns) {
					this.configObj.columns = [];
				}
			} else {
				this.configObj = this.getDefaultConfiguration();
			}

			this.attributes = options.attributes || [];
			this.properties = options.properties || [];

			// flag defining whether to correct attributes (and properties) IDs based on names (== true), 
			// or not (== false)
			this.isImported = options.isImported || false;

			this.changeCallback = options.changeCallback;

			return this;
		};


		/**
		 * This private method is used to remove an option from a specified list of options e.g. this.dltTypes,
		 * if it is not supported in a given context.
		 * 
		 * IMPORTANT: Be aware that no checking whether given option is already "in use" by a configuration
		 * is done! The behaviour is not well defined when a removed option is already defined in an existing
		 * configuration and this configuration is loaded!
		 * 
		 * @param {Array.<String>} optionsList - List of options from which the specified option should be removed.
		 * @param {String} option - Option (its name) to be removed from the list. If it doesn't exist in the list,
		 * 		nothing happens. If it's there more than once, only the first occurrence is removed.
		 */
		DocTableXRenderConfig.prototype._removePredefinedOption = function (optionsList, option) {
			var index = optionsList.indexOf(option);
			if (index > -1) {
				optionsList.splice(index, 1);
			}
		};


		DocTableXRenderConfig.prototype.getConfigObj = function () {
			return this.configObj;
		};

		DocTableXRenderConfig.prototype.getName = function () {
			return this.configObj.name;
		};

		DocTableXRenderConfig.prototype.setName = function (newName) {
			this.configObj.name = newName;
		};

		DocTableXRenderConfig.prototype.getHideHeader = function () {
			return this.configObj.hideHeader;
		};

		DocTableXRenderConfig.prototype.setHideHeader = function (value) {
			this.configObj.hideHeader = (value === true);
		};

		DocTableXRenderConfig.prototype.setAttributes = function (attrs) {
			this.attributes = attrs || [];
		};

		DocTableXRenderConfig.prototype.setProperties = function (attrs) {
			// note an attribute and a property is in fact the same object (Attribute)
			this.properties = attrs || [];
		};


		/**
		 * Loads the documents table rendering configuration from specified XML string (xmlString).
		 */
		DocTableXRenderConfig.prototype.readFromXMLString = function (xmlString, callback) {
			if (xmlString) {
				// convert the input XML string to XML object supporting the XML search/manipulation functions
				var xmlObj = WCXML.xmlTextToObj(xmlString);
				var docRenderElem = WCXML.GetFirstFirstLevelChild(xmlObj, "docRender");
				if (docRenderElem) {
					this.readFromXML(docRenderElem, callback);
				}
			}
		};

		/**
		 * Loads the documents table rendering configuration from specified XML object (xmlRootElem).
		 * Note just the configObj is stored to and loaded from XML (the rest is contextual info).
		 */
		DocTableXRenderConfig.prototype.readFromXML = function (xmlRootElem, callback) {
			// create empty configuration object to be filled by loaded data
			this.configObj = this.getEmptyConfiguration();

			if (!xmlRootElem) {
				return;
			}

			// start loading data into the prepared empty configuration object
			this.configObj.name = WCXML.GetElementText(xmlRootElem, "name");
			this.configObj.hideHeader = WCXML.GetElementBoolean(xmlRootElem, "hideHeader");
			var dltArr = WCXML.GetFirstLevelChilds(xmlRootElem, "dlt");
			if (dltArr && (dltArr.length > 0)) {
				for (var dlt = 0; dlt < dltArr.length; dlt++) {
					var dltElem = dltArr[dlt];
					var dltObj = {};
					for (var idx = 0; idx < this.dltProperties.length; idx++) {
						var dltPropName = this.dltProperties[idx];
						dltObj[dltPropName] = WCXML.GetElementText(dltElem, dltPropName);
					}
					if (dltObj.visibility == 'Hidden') {
						// skip the hidden columns! Currently there is no way to specify a hidden column
						// and it was possible only for reference properties.
						// Now all reference properties not in rendering configuration 
						// (with non-empty value) are implicitly hidden.
						// this is needed to be compatible with configurations already containing hidden columns.
						continue;
					}

					if ((dltObj.type == 'DocumentProperty') && (dltObj.typeDetail == 'Thumbnail') && !dltObj.thumbnailSize) {
						// set default thumbnail size to 50 (only does something on old configurations)
						dltObj.thumbnailSize = '50';
					}
					if ((dltObj.type == 'DocumentProperty') && (dltObj.typeDetail == 'VersionRevision')) {
						if (dltObj.uploadNewVersion === 'false') {
							dltObj.uploadNewVersion = false;
						} else {
							// either true or undefined (default value)
							dltObj.uploadNewVersion = true;
						}
					}
					if ((dltObj.type === 'ProjectProperty') && (dltObj.typeDetail === 'ProjectStatus') && !dltObj.projectStatusLayout) {
						// set default
						dltObj.projectStatusLayout = 'LabelAndIcon';
					}
					if (this.isImported && ((dltObj.type == 'DocumentAttribute') || (dltObj.type == 'ReferenceProperty') || (dltObj.type == 'ProjectAttribute'))) {
						// look for attribute by name
						var attObj = this.findItemByNameOrID(this.attributes, dltObj.attributeName);

						if (attObj) {
							// adjust typedetail, so attribute ID is correct
							dltObj.typeDetail = attObj.id;
						} else if (dltObj.attributeName) {
							// did not find attribute, go to the server
							var currentDlt = dltObj;
							currentDlt.typeDetail = "";
							var attrRequest = Promisify.require("doctablex/AJAXDataCache").then(function (AJAXDataCache) {
								return Promise.resolve(AJAXDataCache.getInstance().getAttribute(dltObj.attributeName));
							});
							attrRequest.then(_.partial(_.bind(function (attr, dltObj) {
								if (attr) {
									// push attributes in attribute list, to be used further in this session
									attr.hidden = true; // don't display in available attributes
									this.attributes.push(attr);

									// adjust typedetail, so attribute ID is correct
									dltObj.typeDetail = attr.id;
								}
								// execute the callback (if any)
								if (callback) {
									callback();
								}
							}, this), _, currentDlt));
							attrRequest.catch(_.bind(function () {
								// attribute not found, should be displayed in error state - execute the callback (if any)
								if (callback) {
									callback();
								}
							}, this));
						} else {
							// there is no attribute name specified - should be displayed in error state, 
							// but still execute the callback (if any)
							if (callback) {
								callback();
							}
						}
					}
					if (dltObj.type == 'AttributeFlapout') {
						if (dltObj.useCreationView === 'false') {
							dltObj.useCreationView = false;
						} else {
							// either true or undefined (default value)
							dltObj.useCreationView = true;
						}
					}
					this.configObj.columns.push(dltObj);
				}
			}
		};



		//JANH TODO!!! 'attList' parameter should be deleted!!!
		/**
		 * Saves the documents table rendering configuration to XML
		 * Note just the configObj is stored to and loaded from XML (the rest is contextual info).
		 */
		DocTableXRenderConfig.prototype.convertToXML = function () {
			// start the XML generation immediately
			var xmlcode = '<docRender>';
			xmlcode += WCXML.ElementEncode('name', this.getName());
			xmlcode += WCXML.ElementEncodeBoolean('hideHeader', this.getHideHeader());
			for (var dlt = 0; dlt < this.configObj.columns.length; dlt++) {
				var dltObj = this.configObj.columns[dlt];
				if (dltObj) {
					if (dltObj.type == 'DocumentAttribute' || dltObj.type == 'ReferenceProperty' || dltObj.type == 'ProjectAttribute') {
						// always update the attribute name/id when saving - why?
						// name: could be empty (never added) or the attribute was renamed
						// id: different after import
						var attObj = this.findItemByNameOrID(this.attributes, dltObj.attributeName, dltObj.typeDetail);
						if (attObj) {
							dltObj.attributeName = attObj.name;
							dltObj.typeDetail = attObj.id;
						}
					}

					xmlcode += '<dlt>';
					for (var prop in dltObj) {
						if (!dltObj.hasOwnProperty(prop)) {
							continue;
						}
						if ((dltObj[prop] !== undefined) && (dltObj[prop] !== '') && (dltObj[prop] !== null)) {
							// (note the condition is like this, because boolean value 'false' should be saved)
							xmlcode += WCXML.ElementEncode(prop, dltObj[prop]);
						}
					}
					xmlcode += '</dlt>';
				}
			}
			xmlcode += '</docRender>';

			return xmlcode;
		};


		/**
		 * Displays the whole documents table render configuration options setup
		 */
		DocTableXRenderConfig.prototype.render = function () {

			try {
				var phElem = $('#' + this.phElemID);
				if (phElem.length === 0) {
					return;
				}

				var htmlcode = '';
				// checkbox to display the header (note the first parameter of changeDocumentTableRow() method
				// is ignored in case of 'clickHeaderChk')
				var clickMethod = this.functionPrefix + '.changeDocumentTableRow(0,\'clickHeaderChk\');';

				htmlcode += '<div class="add-spacing-left add-spacing-bottom"><div class="checkbox"><label>';
				htmlcode += '<input type="checkbox" id="docRefDialog_HeaderChk" onclick="' + clickMethod + '"';
				if (this.configObj.hideHeader) {
					htmlcode += ' checked="checked" ';
				}
				htmlcode += '/>' + ACLayoutStrings.t('ACLayoutHideHeader');
				htmlcode += '</label></div></div>';

				// the table with columns placeholder
				htmlcode += '<div id="DocRenderConfigTableID"></div>';
				phElem.html(htmlcode);
				if (phElem.length == 1) {
					// if rendering of the documents table rendering configuration succeeded store the current 
					// DocTableXRenderConfig instance (the configuration wrapper object) itself in DOM 
					// to be accessible by action handlers in generated HTML
					phElem[0].wrapperObj = this;
				}

				$('#DocRenderConfigTableID').html(this.displayDocRefTableRenderOptions());
				$('#DocRenderConfigTableID tbody').sortable({
					items: "tr.DocXColRow",
					placeholder: "PlaceHolderRow",
					axis: "y",
					forcePlaceholderSize: true,
					start: function (event, ui) {
						// When dragging, set the height of the helper to better indicate where the row will be placed.
						var height = (ui.item.height() + 4) + "px";
						ui.placeholder.height(height);
					}.bind(this),
					stop: function (event, ui) {
						var newCols = [];
						$('#DocRenderConfigTableID tbody > tr.DocXColRow').each(function (i, rowElem) {
							var row = $(rowElem);
							var index = parseInt(row.attr("data-col-index"));
							if (index >= 0) {
								newCols.push(this.configObj.columns[index]);
							}
						}.bind(this));
						this.configObj.columns = newCols;
						this.repaint();
					}.bind(this),
					helper : function(e, ui) {  
						ui.children().each(function() {  
							$(this).width($(this).width());  
						});  
						return ui;  
					}
				});

			} catch (err) {
				window.alert('Problem in DocTableXRenderConfig.render: ' + err.message);
				throw err;
			}
		};

		/**
		 * Method to repaint the whole doc. table rendering configuration table
		 */
		DocTableXRenderConfig.prototype.repaint = function () {
			this.isInRepaint = true; //JANH TODO!!! (not used for now)
			this.render();
			this.isInRepaint = false;
		};

		/**
		 * Displays the table with columns for document links rendered in a table
		 */
		DocTableXRenderConfig.prototype.displayDocRefTableRenderOptions = function () {
			var htmlcode = '';
			try {
				var clickMethod = '';

				htmlcode += '<table class="table">';
				// (don't move out the table to the caller since then you need to update the content of the table instead of the content of the td, which IE can't do)
				// labels
				htmlcode += '<thead>';
				htmlcode += '<tr>';
				htmlcode += '<th class="EmptyOffsetCol"/>';
				htmlcode += '<th class="SortableCol"/>';
				htmlcode += '<th class="TypeCol">' + ACatStrings.t('ACatType') + '</th>';
				htmlcode += '<th class="DetailsCol">' + ACLayoutStrings.t('ACLayoutDetails') + '</th>';
				htmlcode += '<th class="LabelCol">' + ACLayoutStrings.t('ACLayoutLabel') + '</th>';
				htmlcode += '<th class="VisibilityCol">' + ACLayoutStrings.t('ACLayoutVisibility') + '</th>';
				htmlcode += '<th class="WidthCol">' + ACLayoutStrings.t('ACLayoutWidth') + '</th>';
				htmlcode += '<th/>';
				htmlcode += '</tr>';
				htmlcode += '</thead>';

				htmlcode += '<tbody>';

				// Display the rows
				for (var row = 0; row < this.configObj.columns.length; row++) {
					// object pointing to 1 row in the doc. table rendering configuration table
					var dltObj = this.configObj.columns[row];
					if (dltObj) {
						htmlcode += this.displayDocumentTableRow(dltObj, row, (row == this.configObj.columns.length - 1));
					}
				}

				// Link to add a row
				clickMethod = this.functionPrefix + '.changeDocumentTableRow(' + row + ',\'Add\'); return false;';
				htmlcode += '<tr class="DocRenderDialogLinks">';
				htmlcode += '<td colspan="8" id="docRenderDialogLinks">';
				htmlcode += '<a class="btn btn-default" href="#" onclick="' + clickMethod + '">' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutAddDocRefAttr')) + '</a>';

				// Link to revert the configuration to the WebCenter default
				clickMethod = this.functionPrefix + '.resetToDefault(true); return false;';
				htmlcode += '<a class="btn btn-default" href="#" onclick="' + clickMethod + '">' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutRevertToWebCenterDefault')) + '</a>';
				htmlcode += '</td>';
				htmlcode += '</tr>';

				htmlcode += '</tbody>';
				htmlcode += '</table>';
				htmlcode += '</td></tr>';

			} catch (err) {
				window.alert('Problem in DocTableXRenderConfig.displayDocRefTableRenderOptions: ' + err.message);
				throw err;
			}
			return htmlcode;

		};


		/**
		 * Reverts a doc reference table layout configuration to the WebCenter default layout
		 */
		DocTableXRenderConfig.prototype.resetToDefault = function (repaint) {
			this.configObj = this.getDefaultConfiguration();

			if (this.changeCallback) {
				// notify about the configuration being changed
				this.changeCallback();
			}
			if (repaint) {
				this.repaint();
			}
		};

		/** 
		 * Adds a line to the list of document sources
		 * 
		 * @param dltObj: the object having the data to add
		 * @param row: index in the table
		 * @param last: indicates that it's the last row in the table : then don't put the arrow down icon
		 */
		DocTableXRenderConfig.prototype.displayDocumentTableRow = function (dltObj, row, last) {
			var htmlcode = '';
			try {
				var options;

				// Check if this is a currently supported collumn type
				if ($.inArray(dltObj.type, this.dltTypes) < 0) {
					// This type is not supported, don't generate a row for it.
					return htmlcode;
				}

				// construct a row
				htmlcode += '<tr class="DocXColRow" data-col-index="' + row + '">';

				// Empty offset col
				htmlcode += '<td class="EmptyOffsetCol"/>';
				
				// SortableCol
				htmlcode += '<td class="SortableCol">';
				htmlcode += '</td>';

				var changeMethod = this.functionPrefix + '.changeDocumentTableDetail(this)';

				// type
				htmlcode += '<td class="TypeCol">';
				htmlcode += '<select id="docLinkTableType_' + row + '" onchange="' + changeMethod + '" class="form-control">';
				options = this.dltTypes;
				for (var opt = 0; opt < options.length; opt++) {
					var option = options[opt];
					if ((this.properties.length === 0) && (option == 'ReferenceProperty')) {
						// skip 'ReferenceProperty' option if there is no referenceProperty attr. category defined
						continue;
					} else if (option == 'Count') {
						// skip 'Count' as it is not supported for this version
						continue;
					}

					// Add seperator before certain options in order to group them  
					if (option == 'LocalRemote' || option == 'SequenceNumber' || option == 'MasterText' || option == 'ReplaceButton') {
						htmlcode += this.generateSeperatorToTableRow();
					}

					htmlcode += '<option value="' + option + '" ';
					if (dltObj.type == option) {
						htmlcode += ' selected ';
					}
					htmlcode += '>' + ACLayoutStrings.t('ACLayout' + option) + '</option>';


				}
				htmlcode += '</select>';

				htmlcode += '</td>';

				// Details
				htmlcode += '<td class="DetailsCol">';

				switch (dltObj.type) {
				case 'DocumentName':

					var wrapper = $('<div>');

					var showSourceInput = $('<input id="docLinkTableDetail_' + row + '_hideSource" type="checkbox" onchange="' + changeMethod + '" name="hideSource">');
					var showSourceLabel = $('<span>').text(ACLayoutStrings.t('ACLayoutNameShowDocSource'));
					wrapper.append($('<div>').addClass('checkbox').append($('<label>').append(showSourceInput, showSourceLabel)));

					var showDescriptionInput = $('<input id="docLinkTableDetail_' + row + '_hideDescription" type="checkbox" onchange="' + changeMethod + '" name="hideDescription">');
					var showDescriptionLabel = $('<span>').text(ACLayoutStrings.t('ACLayoutNameShowDocDescription'));
					wrapper.append($('<div>').addClass('checkbox').append($('<label>').append(showDescriptionInput, showDescriptionLabel)));

					var showFolderInput = $('<input id="docLinkTableDetail_' + row + '_hideFolder" type="checkbox" onchange="' + changeMethod + '" name="hideFolder">');
					var showFolderLabel = $('<span>').text(ACLayoutStrings.t('ACLayoutNameShowFolder'));
					wrapper.append($('<div>').addClass('checkbox').append($('<label>').append(showFolderInput, showFolderLabel)));

					var showDocTemplateInput = $('<input id="docLinkTableDetail_' + row + '_hideDocTemplate" type="checkbox" onchange="' + changeMethod + '" name="hideDocTemplate">');
					var showDocTemplateLabel = $('<span>').text(ACLayoutStrings.t('ACLayoutNameShowDocTemplate'));
					wrapper.append($('<div>').addClass('checkbox').append($('<label>').append(showDocTemplateInput, showDocTemplateLabel)));

					// Check the correct inputs.
					if (dltObj.hideSource != "true") {
						showSourceInput.attr('checked', 'checked');
					}
					if (dltObj.hideDescription != "true") {
						showDescriptionInput.attr('checked', 'checked');
					}
					if (dltObj.hideFolder != "true") {
						showFolderInput.attr('checked', 'checked');
					}
					if (dltObj.hideDocTemplate != "true") {
						showDocTemplateInput.attr('checked', 'checked');
					}

					htmlcode += wrapper.prop('outerHTML');
					break;

				case 'DocumentActions':
					var addMethod = this.functionPrefix + '.addDocRefDocAction(' + row + '); return false;';

					var values = dltObj.typeDetail.split("|");

					htmlcode += '<div class="documentactions">';
					for (var i = 0; i < values.length; i++) {
						var removeMethod = this.functionPrefix + '.removeDocRefDocAction(' + row + ', ' + i + '); return false;';
						var changeValueMethod = this.functionPrefix + '.changeDocRefDocAction(this, ' + row + ', ' + i + ');';

						htmlcode += '<div class="input-group">';

						htmlcode += '<select onchange="' + changeValueMethod + '" class="form-control">';
						htmlcode += '<option value="">---</option>';
						for (var j = 0; j < this.documentActions.length; j++) {
							var selectedStr = (this.documentActions[j] == values[i]) ? 'selected="selected" ' : '';
							htmlcode += '<option ' + selectedStr + 'value="' + this.documentActions[j] + '">' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutDocAction' + this.documentActions[j])) + '</option>';
						}
						htmlcode += '</select>';
						if (values.length > 1) {
							htmlcode += '<a class="input-group-btn add-padding-left" href="#" onclick="' + removeMethod + '"><i class="icon icon-normal icon-WCR_Remove12"></i></a>';
						}

						htmlcode += '</div>';
					}

					htmlcode += '</div>';


					htmlcode += '<a class="btn btn-default add-spacing-top" href="#" onclick="' + addMethod + '">' + ACLayoutStrings.t('ACLayoutAddAction') + '</a>';
					break;

				case 'DocumentProperty':
					options = this.dltDocPropertyDetails;
					htmlcode += '<div class="input-group">';
					htmlcode += '<select id="docLinkTableDetail_' + row + '" onchange="' + changeMethod + '" class="form-control">';
					for (var opt = 0; opt < options.length; opt++) {
						htmlcode += '<option value="' + options[opt] + '" ';
						if (dltObj.typeDetail == options[opt]) {
							htmlcode += ' selected ';
						}
						htmlcode += '>' + ACLayoutStrings.t('DocRefColLabel_' + options[opt]) + '</option>';
					}
					htmlcode += '</select>';

					// thumbnail size
					if (dltObj.typeDetail == 'Thumbnail') {
						options = this.dltDocThumbSizes;
						htmlcode += '<div class="input-group-addon-select"><select id="docLinkTableThumbnail_' + row + '"  onchange="' + changeMethod + '">';
						for (var opt = 0; opt < options.length; opt++) {
							htmlcode += '<option value="' + options[opt] + '" ';
							if (dltObj.thumbnailSize == options[opt]) {
								htmlcode += ' selected ';
							}
							htmlcode += '>' + options[opt] + '</option>';
						}
						htmlcode += '</select></div><label class="input-group-addon" for="docLinkTableThumbnail_' + row + '">px</label>';

						htmlcode += '</div>';

					} else if (dltObj.typeDetail == 'VersionRevision') {
						// checkbox for upload new version

						htmlcode += '</div>';

						htmlcode += '<div class="checkbox"><label>';
						htmlcode += '<input type="checkbox" id="docLinkTableUploadNewVersion_' + row + '" ' + (dltObj.uploadNewVersion === true ? 'checked="checked"' : '') + 'onchange="' + changeMethod + '"/>';
						htmlcode += ACLayoutStrings.t('ACLayoutAllowUploadNewVersion');
						htmlcode += '</label></div>';
						break;
					}

					break;

				case 'AttributeFlapout':
					if (!dltObj.typeDetail) {
						dltObj.typeDetail = this.defaultAttrCatView;
					}
					htmlcode += '<div class="input-group">';
					htmlcode += '<label class="input-group-addon" for="docLinkTableDetail_' + row + '">' + ACLayoutStrings.t('ACLayoutView') + '</label>';
					htmlcode += '<input type="text" id="docLinkTableDetail_' + row + '" value="' + dltObj.typeDetail + '" onchange="' + changeMethod + '" class="form-control"/>';
					htmlcode += '</div>';

					// checkbox for creation view
					htmlcode += '<div class="checkbox"><label>';
					htmlcode += '<input type="checkbox" id="docLinkTableUseCreationView_' + row + '" ' + (dltObj.useCreationView === true ? 'checked="checked"' : '') + 'onchange="' + changeMethod + '"/>';
					htmlcode += ACLayoutStrings.t('ACLayoutUseCreationView');
					htmlcode += '</label></div>';
					break;

				case 'ReferenceProperty':
					htmlcode += '<select id="docLinkTableDetail_' + row + '" onchange="' + changeMethod + '" class="form-control">';
					if (this.properties.length > 0) {
						var currentAttr = this.findItemByNameOrID(this.properties, dltObj.attributeName, dltObj.typeDetail);
						if (!currentAttr) {
							currentAttr = {}; // empty object for comparison
						}

						options = this.properties; // linked properties (== attributes of reference properties attribute category)
						for (var opt = 0; opt < options.length; opt++) {
							var aProp = options[opt];
							if (aProp && !aProp.isDocReference() && !aProp.isRichText()) {
								htmlcode += '<option value="' + aProp.id + '" ';
								if (currentAttr.id == aProp.id) {
									htmlcode += ' selected ';
								}
								htmlcode += '>' + aProp.name + '</option>';
							}
						}
					}
					htmlcode += '</select>';
					break;

				case 'DocumentAttribute':
					htmlcode += '<select id="docLinkTableDetail_' + row + '"  onchange="' + changeMethod + '" class="form-control">';

					if (this.attributes.length > 0) {
						var currentAttr = this.findItemByNameOrID(this.attributes, dltObj.attributeName, dltObj.typeDetail);
						if (!currentAttr) {
							currentAttr = {}; // empty object for comparison
						}

						for (var opt = 0; opt < this.attributes.length; opt++) {
							var anAttr = this.attributes[opt];
							// rich text attributes are not supported here - filter them out
							if (anAttr && anAttr.isRichText()) {
								continue;
							}

							htmlcode += '<option value="' + anAttr.id + '" ';
							if (currentAttr.id == anAttr.id) {
								htmlcode += ' selected ';
							}
							htmlcode += '>' + WCXML.xmlencode(anAttr.name) + '</option>';
						}
					}
					htmlcode += '</select>';
					break;

				case 'ProjectProperty':
					options = this.dltProjectPropertyDetails;
					options = _.sortBy(options, function (option) {
						return ACLayoutStrings.t('DocRefColLabel_' + option);
					});

					htmlcode += '<div class="form-inline">';

					htmlcode += '<div class="form-group"><select id="docLinkTableDetail_' + row + '" onchange="' + changeMethod + '" class="form-control">';
					for (var opt = 0; opt < options.length; opt++) {
						htmlcode += '<option value="' + options[opt] + '" ';
						if (dltObj.typeDetail == options[opt]) {
							htmlcode += ' selected ';
						}
						htmlcode += '>' + ACLayoutStrings.t('DocRefColLabel_' + options[opt]) + '</option>';
					}
					htmlcode += '</select></div>';

					// Project Status
					if (dltObj.typeDetail == 'ProjectStatus') {
						options = this.dltProjectStatusOptions;
						htmlcode += '<div class="form-group"><select class="form-control projectStatusLayout" id="docLinkTableProjectStatus_' + row + '"  onchange="' + changeMethod + '">';
						for (var opt = 0; opt < options.length; opt++) {
							htmlcode += '<option value="' + options[opt] + '" ';
							if (dltObj.projectStatusLayout == options[opt]) {
								htmlcode += ' selected ';
							}
							htmlcode += '>' + ACLayoutStrings.t('ACLayoutPSLayout' + options[opt]) + '</option>';
						}
						htmlcode += '</select></div>';
					}
					htmlcode += '</div>';
					break;

				case 'ProjectAttribute':
					htmlcode += '<select id="docLinkTableDetail_' + row + '"  onchange="' + changeMethod + '" class="form-control">';

					if (this.attributes.length > 0) {
						var currentAttr = this.findItemByNameOrID(this.attributes, dltObj.attributeName, dltObj.typeDetail);
						if (!currentAttr) {
							currentAttr = {}; // empty object for comparison
						}

						for (var opt = 0; opt < this.attributes.length; opt++) {
							var anAttr = this.attributes[opt];
							// rich text attributes are not supported here - filter them out
							if (anAttr && anAttr.isRichText()) {
								continue;
							}

							htmlcode += '<option value="' + anAttr.id + '" ';
							if (currentAttr.id == anAttr.id) {
								htmlcode += ' selected ';
							}
							htmlcode += '>' + WCXML.xmlencode(anAttr.name) + '</option>';
						}
					}
					htmlcode += '</select>';
					break;

				case 'MasterText':
					var defLangCode = dltObj.typeDetail ? dltObj.typeDetail : "";
					htmlcode += '<input placeholder="Default language code" id="docLinkTableDetail_' + row + '" type="text" onchange="' + changeMethod + '" value="' + defLangCode + '" class="form-control">';
					break;

				case 'TranslationText':
					var defLangCode = dltObj.typeDetail ? dltObj.typeDetail : "";
					htmlcode += '<input placeholder="Default language code" id="docLinkTableDetail_' + row + '" type="text" onchange="' + changeMethod + '" value="' + defLangCode + '" class="form-control">';
					break;

				case 'TextLanguages':
					htmlcode += '<div class="input-group" style="padding-bottom: 2px;">';
					htmlcode += '<label class="input-group-addon" for="docLinkTableDetail_' + row + '_show_for">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesShowFor")) + '</label>';
					htmlcode += '<select id="docLinkTableDetail_' + row + '_show_for" name="displayLanguages" onchange="' + changeMethod + '" class="form-control">';
					if (dltObj.displayLanguages == "master") {
						htmlcode += '<option value="master" selected="selected">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesForMaster")) + '</option>';
					} else {
						htmlcode += '<option value="master">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesForMaster")) + '</option>';
					}
					if (dltObj.displayLanguages == "translation") {
						htmlcode += '<option value="translation" selected="selected">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesForTranslation")) + '</option>';
					} else {
						htmlcode += '<option value="translation">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesForTranslation")) + '</option>';
					}
					htmlcode += '</select>';
					htmlcode += '</div>';

					htmlcode += '<div class="input-group" style="padding-top: 2px;">';
					htmlcode += '<label class="input-group-addon" for="docLinkTableDetail_' + row + '_mode">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesMode")) + '</label>';
					htmlcode += '<select id="docLinkTableDetail_' + row + '_mode" name="langShowMode" onchange="' + changeMethod + '" class="form-control">';
					if (dltObj.langShowMode == "compact") {
						htmlcode += '<option value="compact" selected="selected">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesCompact")) + '</option>';
					} else {
						htmlcode += '<option value="compact">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesCompact")) + '</option>';
					}
					if (dltObj.langShowMode == "normal") {
						htmlcode += '<option value="normal" selected="selected">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesNormal")) + '</option>';
					} else {
						htmlcode += '<option value="normal">' + WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesNormal")) + '</option>';
					}
					htmlcode += '</select>';
					htmlcode += '</div>';
					break;

				case 'LocalRemote':

					if (window.webcenterConfig.isPCMEnabled) {
						htmlcode += '<div class="checkbox">';
						htmlcode += '<label>';
						if (dltObj.retainTextContentStatus === "true") {
							htmlcode += '<input type="checkbox" name="retainTextContentStatus" id="docLinkTableDetail_' + row + '" onchange="' + changeMethod + '" checked="checked" />';
						} else {
							htmlcode += '<input type="checkbox" name="retainTextContentStatus" id="docLinkTableDetail_' + row + '" onchange="' + changeMethod + '" />';
						}
						htmlcode += 'Retain text document status';
						htmlcode += '</label>';
						htmlcode += '</div>';
					}
					break;

				default:
					// do nothing
				}


				htmlcode += '</td>';

				// Label
				htmlcode += '<td class="LabelCol">';
				if (dltObj.type == 'AttributeFlapout') {
					// not editable '+' symbol
					htmlcode += '+';
				} else if (dltObj.type == 'UpDownArrows' || dltObj.type == 'DeleteButton' || dltObj.type == 'ReplaceButton' || dltObj.type == 'CloneButton' || dltObj.type == 'LocalRemote' || dltObj.type == 'RowSelector') {
					// put nothing
				} else {
					// by default the label contains language tag, but customized labels contains the value itself
					var labelStr = '';
					if (dltObj.label.indexOf('DocRefColLabel_') === 0) {
						// 'label' contains a translation tag - use it
						labelStr = ACLayoutStrings.t(dltObj.label);
					} else if (ACLayoutStrings.tFallback('DocRefColLabel_' + dltObj.label, '')) { // fall back on '' if it doesn't exist, or we will see the label with 'DocRefColLabel_'
						// for back-compatibility: try to prepend the translation tag prefix and see if it exists
						labelStr = ACLayoutStrings.t('DocRefColLabel_' + dltObj.label);
					}
					if (!labelStr) {
						// not supported translation tag or a custom label
						labelStr = dltObj.label;
					}
					// the label is editable
					htmlcode += '<input type="text" id="docLinkTableLabel_' + row + '" value="' + WCXML.xmlencode(labelStr) + '" onchange="' + changeMethod + '" class="form-control">';
				}
				htmlcode += '</td>';

				// Visibility	
				htmlcode += '<td class="VisibilityCol">';

				if (dltObj.type == 'ReferenceProperty' || dltObj.type == 'DocumentAttribute' || dltObj.type == 'Count' || dltObj.type == 'AttributeFlapout' || dltObj.type == 'LocalRemote' || dltObj.type == 'MasterText' || dltObj.type == 'TranslationText' || dltObj.type == 'TextLanguages') {
					htmlcode += '<select id="docLinkTableVisibility_' + row + '" onchange="' + changeMethod + '" class="form-control">';
					if (dltObj.type == 'AttributeFlapout' || dltObj.type == 'LocalRemote' || dltObj.type == 'MasterText' || dltObj.type == 'TranslationText' || dltObj.type == 'TextLanguages') {
						options = ['Editable', 'ReadOnly'];
					} else {
						options = ['Editable', 'Required', 'ReadOnly'];
					}

					for (var opt = 0; opt < options.length; opt++) {
						htmlcode += '<option value="' + options[opt] + '" ';
						if (dltObj.visibility == options[opt]) {
							htmlcode += ' selected ';
						}
						htmlcode += '>' + ACLayoutStrings.t('ACLayout' + options[opt]) + '</option>';
					}
					htmlcode += '</select>';
				} else {
					dltObj.visibility = 'ReadOnly'; // probably should not be here, mixture of controller with view!
					htmlcode += ACLayoutStrings.t('ACLayoutReadOnly');
				}
				htmlcode += '</td>';


				// Width of DocTableX columns settings
				htmlcode += '<td class="WidthCol">';

				var widthMode = 'none';
				switch (dltObj.type) {
					case 'DocumentName':
					case 'ReferenceProperty':
					case 'DocumentAttribute':
					case 'ProjectAttribute':
					case 'MasterText':
					case 'TranslationText':
					case 'TextLanguages':
						if (dltObj.width) {
							widthMode = 'editableChanged';
						} else {
							widthMode = 'editableDefault';
						}
						break;
	
					case 'DocumentProperty':
						switch (dltObj.typeDetail) {
						case 'Description':
						case 'Project':
						case 'ApprovalStage':
						case 'VersionAuthor':
						case 'ModifiedDate':
							if (dltObj.width) {
								widthMode = 'editableChanged';
							} else {
								widthMode = 'editableDefault';
							}
							break;
						}
						break;
					case 'ProjectProperty':
						switch (dltObj.typeDetail) {
						case 'ProjectName':
						case 'ProjectStatus':
						case 'ProjectDueDate':
							if (dltObj.width) {
								widthMode = 'editableChanged';
							} else {
								widthMode = 'editableDefault';
							}
							break;
						}
						break;
				}
				
				// render HTML based on the calculated "mode" of the width edit field
				if (widthMode == 'editableChanged') {
					htmlcode += '<input type="text" id="docLinkTableWidth_' + row + '" onchange="' + changeMethod + '" value="' + dltObj.width + '" class="form-control"/>';
				} else if (widthMode == 'editableDefault') {
					htmlcode += '<input type="text" id="docLinkTableWidth_' + row + '" onchange="' + changeMethod + '" value="auto" class="form-control"/>';
				} else {
					htmlcode += 'auto';
				}
				htmlcode += '</td>';
				
				// remove icon, but only if there are at least 2 rows. Also cannot remove document name column
				htmlcode += '<td class="RemoveCol">';
				var clickMethod;
				if (!(row === 0 && last) && !(dltObj.type == 'DocumentName')) {
					clickMethod = this.functionPrefix + '.changeDocumentTableRow(' + row + ',\'Remove\'); return false;';
					htmlcode += '<i class="icon icon-normal icon-WCR_Remove12" onclick="' + clickMethod + '" title="' + WCXML.xmlencode(ACLayoutStrings.t('REMOVE')) + '"/>';
				}
				htmlcode += '</td>';

				//Close it up
				htmlcode += '</tr>';


				return htmlcode;

			} catch (err) {
				window.alert('Problem in DocTableXRenderConfig.displayDocumentTableRow: ' + err.message);
				throw err;
			}

		};


		/**
		 * Add a seperator to the list of options
		 */
		DocTableXRenderConfig.prototype.generateSeperatorToTableRow = function () {
			return "<option disabled>-------------</option>";
		};


		/**
		 * Handles row operations on the document sources table 
		 * @param idx: index of the row on which the operation is done (null for operation Add)
		 * @param operation: one of 'Add','Remove','Up','Down'
		 */
		DocTableXRenderConfig.prototype.changeDocumentTableRow = function (idx, operation) {
			var configObj = this.configObj;
			if (!configObj || !configObj.columns) {
				configObj = this.getEmptyConfiguration();
			}
			switch (operation) {
			case 'Add':
				var dltObj = {};
				dltObj.type = 'DocumentProperty';
				dltObj.typeDetail = 'Description';
				dltObj.label = 'DocRefColLabel_Description';
				dltObj.visibility = 'ReadOnly';
				configObj.columns.push(dltObj);
				break;

			case 'Remove':
				if (configObj.columns.length > idx) {
					configObj.columns.splice(idx, 1);
				}
				break;

			case 'Up':
				if (idx > 0 && configObj.columns.length > idx) {
					var dltObj = configObj.columns[idx]; // remember the object
					configObj.columns.splice(idx, 1); // take it out of the Array
					configObj.columns.splice(idx - 1, 0, dltObj); // put it back 1 up
				}
				break;

			case 'Down':
				if (configObj.columns.length > idx + 1) {
					var dltObj = configObj.columns[idx]; // remember the object
					configObj.columns.splice(idx, 1); // take it out of the Array
					configObj.columns.splice(idx + 1, 0, dltObj); // put it back 1 down
				}
				break;

			case 'clickHeaderChk':
				configObj.hideHeader = ($('#docRefDialog_HeaderChk').attr('checked'));
				break;
			default:
				throw new Error('Unknown operation : ' + operation);
			}

			try {
				if (operation != 'clickHeaderChk') {
					// repaint the table of the dialog
					var phElem = $('#' + this.phElemID);
					if (phElem.length > 0) {
						this.repaint();

						// handle dialogs with too many rows to fit on a screen
						var dialogElem = $('#docRenderDialog');
						if (dialogElem.length > 0) {
							// get the height of the browser's window and adapt the max. dialog height according to it.
							var viewportHeight = $(window).height();
							var maxHeight = viewportHeight - viewportHeight / 10;
							if (maxHeight < 400) {
								maxHeight = 400;
							}
							if (dialogElem.height() > maxHeight - 100) {
								// resize the dialog to maximal height that still fits on the given screen
								dialogElem.dialog('option', 'height', maxHeight);
							}
							if (operation == 'Add') {
								// scroll to the very bottom of the dialog content in order to still see the "Add" 
								// button, but only if Add button was clicked!
								dialogElem.scrollTop(dialogElem[0].scrollHeight);
							}
							if (!dialogElem.attr('moved')) {
								dialogElem.dialog("option", "position", "center");
							}
						}
					}
				}

				// repaint the preview is not needed since it is repainted when the dialog closes.

				if (this.changeCallback) {
					// notify about the configuration being changed
					this.changeCallback();
				}

			} catch (err) {
				window.alert('Problem in DocTableXRenderConfig.changeDocumentTableRow: ' + err.message);
				throw err;
			}

		};


		/**
		 * Handles all changes to the document reference table pop-up
		 * @param {DOMObj} elem - the DOM element which was changed. It has an id looking like property_row
		 */
		DocTableXRenderConfig.prototype.changeDocumentTableDetail = function (elem) {
			try {
				var repaintNeeded = false;
				if (!elem) {
					throw new Error('elem is not defined');
				}

				var configObj = this.configObj;

				// find row and property from the id which looks like 'property_row'
				var id = elem.id;
				var idArr = id.split('_');
				var property = idArr[0];
				var row = idArr[1];

				var dltObj = configObj.columns[row];
				if (!dltObj) {
					throw new Error('No dltObj for row ' + row);
				}

				switch (property) {
				case 'docLinkTableType':
					dltObj.type = elem.value;
					dltObj.typeDetail = ''; // reset type detail
					switch (dltObj.type) {
					case 'DocumentName':
						dltObj.label = 'DocRefColLabel_DocumentSpec';
						dltObj.width = 'auto';
						break;
					case 'DocumentActions':
						dltObj.label = 'DocRefColLabel_DocumentActions';
						dltObj.typeDetail = 'Open3d|View|OpenAcad|Download';
						break;
					case 'AttributeFlapout':
						dltObj.typeDetail = this.defaultAttrCatView;
						dltObj.label = '';
						dltObj.visibility = 'ReadOnly';
						dltObj.useCreationView = true;
						break;
					case 'DocumentProperty':
						dltObj.typeDetail = 'Description';
						dltObj.label = 'DocRefColLabel_Description';
						break;
					case 'ProjectProperty':
						dltObj.typeDetail = 'ProjectName';
						dltObj.label = 'DocRefColLabel_ProjectName';
						break;
					case 'DocumentAttribute':
					case 'ProjectAttribute':
						// find first non-richtext, non-reference attribute
						var attInd = 0;
						for (attInd = 0; attInd < this.attributes.length; attInd++) {
							var anAttr = this.attributes[attInd];
							if (anAttr && !anAttr.isRichText() && !anAttr.isDocReference()) {
								break;
							}
						}
						// fill out typedetail and label of first attribute
						if (attInd < this.attributes.length) {
							dltObj.typeDetail = this.attributes[attInd].id;
							dltObj.label = this.attributes[attInd].name;
							dltObj.attributeName = this.attributes[attInd].name;
						}
						break;
					case 'ReferenceProperty':
						if (this.properties.length > 0) {
							var i = 0;
							// fill out first element from the list, which will be the first non-richtext and non-docref attribute
							while (i < this.properties.length) {
								if (this.properties[i].isDocReference() || this.properties[i].isRichText()) {
									i++;
								} else {
									break;
								}
							}
							if (i < this.properties.length) {
								var firstProp = this.properties[i];
								dltObj.typeDetail = firstProp.id;
								dltObj.label = firstProp.name;
								dltObj.attributeName = firstProp.name;
							}
						}
						dltObj.visibility = 'Editable';
						break;
					case 'Count':
						dltObj.label = 'DocRefColLabel_Count';
						break;
					case 'SequenceNumber':
						dltObj.label = '#';
						break;
					case 'MasterText':
						dltObj.label = 'DocRefColLabel_MasterText';
						dltObj.visibility = 'Editable';
						dltObj.width = 'auto';
						break;
					case 'TranslationText':
						dltObj.label = 'DocRefColLabel_TranslationText';
						dltObj.visibility = 'Editable';
						dltObj.width = 'auto';
						break;
					case 'TextLanguages':
						dltObj.label = 'DocRefColLabel_TextLanguages';
						dltObj.visibility = 'Editable';
						break;
					default:
						dltObj.label = '';
						break;
					}
					repaintNeeded = true;
					break;
				case 'docLinkTableDetail':
					dltObj.typeDetail = elem.value;
					// set the default label
					switch (dltObj.type) {

					case 'DocumentName':
						dltObj[elem.name] = !elem.checked + "";
						break;
					case 'DocumentProperty':
					case 'ProjectProperty':
						dltObj.label = 'DocRefColLabel_' + dltObj.typeDetail;
						break;
					case 'DocumentAttribute':
					case 'ProjectAttribute':
						if (this.attributes.length > 0) {
							var docAttObj = this.findItemByNameOrID(this.attributes, null, dltObj.typeDetail); // typeDetail contains the attribute ID
							dltObj.attributeName = docAttObj.name;
							dltObj.label = dltObj.attributeName;
						}
						break;
					case 'ReferenceProperty':
						if (this.properties.length > 0) {
							var refAttObj = this.findItemByNameOrID(this.properties, null, elem.value);
							dltObj.attributeName = refAttObj.name;
							dltObj.label = refAttObj.name;
						}
						break;
					case 'MasterText':
						break;
					case 'TranslationText':
						break;
					case 'TextLanguages':
						dltObj[elem.name] = elem.value;
						break;
					case 'LocalRemote':
						dltObj[elem.name] = elem.checked + "";
						break;
					default:
						dltObj.label = '';
					}
					// set or reset thumbnail size
					if (dltObj.type == 'DocumentProperty' && dltObj.typeDetail == 'Thumbnail') {
						dltObj.thumbnailSize = '50';
					} else {
						dltObj.thumbnailSize = '';
					}
					repaintNeeded = true;
					break;
				case 'docLinkTableProjectStatus':
					dltObj.projectStatusLayout = elem.value;
					break;
				case 'docLinkTableThumbnail':
					dltObj.thumbnailSize = elem.value;
					break;
				case 'docLinkTableUploadNewVersion':
					dltObj.uploadNewVersion = ($(elem).attr('checked') === "checked");
					break;
				case 'docLinkTableLabel':
					dltObj.label = elem.value;
					break;
				case 'docLinkTableVisibility':
					dltObj.visibility = elem.value;
					break;
				case 'docLinkTableUseCreationView':
					dltObj.useCreationView = ($(elem).attr('checked') === "checked");
					break;
				case 'docLinkTableWidth':
					// Check if the value is of format '60'
					if (/^([0-9]+)$/.exec(elem.value)) {
						elem.value = elem.value + "px";
						dltObj.width = elem.value;
						break;
					}
					// Check if the value is of format '60px'
					if (/^([0-9]+)px$/.exec(elem.value)) {
						dltObj.width = elem.value;
						break;
					}
					// Check if the value is of format '60%'
					if (/^([0-9]+)%$/.exec(elem.value)) {
						dltObj.width = elem.value;
						break;
					}
					// Check if the value is 'auto'.
					if (elem.value.toUpperCase().trim() === 'AUTO') {
						dltObj.width = elem.value;
						break;
					}
					// Not a valid format, set to auto.
					elem.value = 'auto';
					dltObj.width = elem.value;
					break;
				default:
					throw new Error('unknown property : ' + property);
				}

				if (this.changeCallback) {
					// notify about the configuration being changed
					this.changeCallback();
				}

				// repaint the document source table in the cell details			
				if (repaintNeeded) {
					this.repaint();
				}
			} catch (err) {
				window.alert('Problem in DocTableXRenderConfig.changeDocumentTableDetail: ' + err.message);
				throw err;
			}

		};

		/**
		 * Add option to document actions configuration
		 * The actions are pipe-separated, so we're just adding a pipe in the end
		 */
		DocTableXRenderConfig.prototype.addDocRefDocAction = function (row) {
			var dltObj = this.configObj.columns[row];
			if (!dltObj) {
				throw new Error('No dltObj for row ' + row);
			}

			dltObj.typeDetail += "|";

			if (this.changeCallback) {
				// notify about the configuration being changed
				this.changeCallback();
			}
			this.repaint();
		};

		/**
		 * change option in document actions configuration
		 */
		DocTableXRenderConfig.prototype.changeDocRefDocAction = function (elem, row, actionIndex) {
			var dltObj = this.configObj.columns[row];
			if (!dltObj) {
				throw new Error('No dltObj for row ' + row);
			}

			var actions = dltObj.typeDetail.split("|");
			if (actions.length > actionIndex) {
				actions[actionIndex] = $(elem).val();
			}
			dltObj.typeDetail = actions.join("|");

			if (this.changeCallback) {
				// notify about the configuration being changed
				this.changeCallback();
			}
			this.repaint();
		};

		/**
		 * Remove an action form the document actions configuration for doc references
		 */
		DocTableXRenderConfig.prototype.removeDocRefDocAction = function (row, actionIndex) {
			var dltObj = this.configObj.columns[row];
			if (!dltObj) {
				throw new Error('No dltObj for row ' + row);
			}

			var actions = dltObj.typeDetail.split("|");
			if (actions.length > actionIndex) {
				actions.splice(actionIndex, 1);
			}
			dltObj.typeDetail = actions.join("|");

			if (this.changeCallback) {
				// notify about the configuration being changed
				this.changeCallback();
			}
			this.repaint();
		};


		/**
		 * Creates an empty documents table configuration.
		 */
		DocTableXRenderConfig.prototype.getEmptyConfiguration = function () {
			var emptyConfig = {};
			emptyConfig.name = this.defaultName || '';
			emptyConfig.columns = [];
			emptyConfig.hideHeader = false;
			emptyConfig.useCreationView = true;
			return emptyConfig;
		};

		/**
		 * Creates a default documents table configuration.
		 * To be called each time a new document reference is set to table rendering
		 */
		DocTableXRenderConfig.prototype.getDefaultConfiguration = function () {
			var defaultConfig = this.getEmptyConfiguration();
			var dltObj = {};

			if (this.section == 'TaskDocuments') {
				// selector row - specific for TEP page
				dltObj = {};
				dltObj.type = 'RowSelector';
				dltObj.label = '';
				dltObj.visibility = 'ReadOnly';
				defaultConfig.columns.push(dltObj);
			}

			if ((this.section == 'AttributeCategory') || (this.section == 'TaskDocuments') || (this.section == 'Upload') || (this.section == 'ProjectCreateDocument')) {
				//  Attribute +
				dltObj = {};
				dltObj.type = 'AttributeFlapout';
				dltObj.label = ''; // label is rendered explicitely as a read-only '+' symbol
				dltObj.typeDetail = this.defaultAttrCatView;
				if (this.section == 'TaskDocuments' || this.section == 'Upload' || (this.section == 'ProjectCreateDocument')) {
					// documents linked to a task can be by default unlinked etc. (if logged in user has corresponding rights)
					dltObj.visibility = 'Editable';
				} else {
					dltObj.visibility = 'ReadOnly';
				}
				dltObj.useCreationView = true;
				defaultConfig.columns.push(dltObj);

				if (this.section != 'TaskDocuments') {
					// Up/down arrows
					dltObj = {};
					dltObj.type = 'UpDownArrows';
					dltObj.label = '';
					dltObj.visibility = 'ReadOnly';
					defaultConfig.columns.push(dltObj);
				}

				// Thumbnail
				dltObj = {};
				dltObj.type = 'DocumentProperty';
				dltObj.typeDetail = 'Thumbnail';
				dltObj.thumbnailSize = '50'; // always a string
				dltObj.label = 'DocRefColLabel_Thumbnail';
				dltObj.visibility = 'ReadOnly';
				defaultConfig.columns.push(dltObj);

				//Document Name
				dltObj = {};
				dltObj.type = 'DocumentName';
				dltObj.label = 'DocRefColLabel_DocumentSpec';
				dltObj.visibility = 'ReadOnly';
				defaultConfig.columns.push(dltObj);

				//Document Actions
				dltObj = {};
				dltObj.type = 'DocumentActions';
				dltObj.typeDetail = 'Open3d|View|OpenAcad|Download';
				dltObj.label = 'DocRefColLabel_DocumentActions';
				defaultConfig.columns.push(dltObj);

				// Version
				dltObj = {};
				dltObj.type = 'DocumentProperty';
				dltObj.typeDetail = 'VersionRevision';
				dltObj.label = 'DocRefColLabel_VersionRevision';
				dltObj.visibility = 'ReadOnly';
				dltObj.uploadNewVersion = true;
				defaultConfig.columns.push(dltObj);

				// Approval status
				if (this.section == 'TaskDocuments') {
					dltObj = {};
					dltObj.type = 'DocumentProperty';
					dltObj.typeDetail = 'ApprovalStatus';
					dltObj.label = 'DocRefColLabel_ApprovalStatus';
					dltObj.visibility = 'ReadOnly';
					defaultConfig.columns.push(dltObj);
				}

				// Delete button
				dltObj = {};
				dltObj.type = 'DeleteButton';
				dltObj.label = '';
				dltObj.visibility = 'ReadOnly';
				defaultConfig.columns.push(dltObj);
			}

			if (this.section == 'UploadTool') {
				//Document Name
				dltObj = {};
				dltObj.type = 'DocumentName';
				dltObj.label = 'DocRefColLabel_DocumentSpec';
				dltObj.visibility = 'ReadOnly';
				defaultConfig.columns.push(dltObj);

				// Delete button
				dltObj = {};
				dltObj.type = 'DeleteButton';
				dltObj.label = '';
				dltObj.visibility = 'ReadOnly';
				defaultConfig.columns.push(dltObj);
			}

			return defaultConfig;
		};


		/**
		 * A general method to find an item in an array of items by name or ID. 
		 * Note name has priority.
		 */
		DocTableXRenderConfig.prototype.findItemByNameOrID = function (items, name, id) {
			var foundItem = null;
			if (name) {
				var lowerCaseName = name.toLowerCase();
				for (var i = 0; i < items.length; i++) {
					if (items[i].name.toLowerCase() == lowerCaseName) {
						foundItem = items[i];
						break; // jump out of the for loop
					}
				}
			}
			if (id && !foundItem) { // not found, try with the ID
				for (var i = 0; i < items.length; i++) {
					if (items[i].id == id) {
						foundItem = items[i];
						break; // jump out of the for loop
					}
				}
			}
			return foundItem;
		};


		return DocTableXRenderConfig;
	});