define(["doctablex/DocumentRow",
        "doctablex/DocumentRowBrowse",
        "doctablex/DocumentRowFileView",
        "doctablex/DocumentTypeUtils",
        "jquery"], 
function(DocumentRow,
		 DocumentRowBrowse,
         DocumentRowFileView,
         DocRefStrings,
         $){
	"use strict";
	
    /**
     * This is a subclass of DocumentRow designed to handle adding of documents via "Browse" document 
     * source (upload operation).
     * 
     * @exports DocumentRowFile 
     * @class
	 * @augments DocumentRowBrowse
	 * @constructor
	 * 
	 * @param {Object} options
     */
	var DocumentRowFile = function(options) {
        if (!options) {
            // when there is nothing to initialize from, return just itself (not initialized)
            return this;
        }
        // set the view class of this data model class before calling superclass constructor to be created in the base class
        if (!options.viewClass) options.viewClass = DocumentRowFileView;
        DocumentRowBrowse.call(this, options);   // calling superclass constructor!
        
        // need to call this here, because the call in the parent is hacked away
        this._initializeFromServer();
        
        /**
         * @member {File} _file - file that should be uploaded for this row
         */
        this._file = options.file;
        this.docName = this._file.name;
        
        // you always start with a file selected, so thrigger a file changed
        this.fileChanged();
        
        return this;
    };

    // define DocumentRowFile as a subclass of DocumentRowBrowse
    DocumentRowFile.prototype = new DocumentRowBrowse();
    DocumentRowFile.prototype.constructor = DocumentRowFile;
    DocumentRowFile.prototype.parent = DocumentRowBrowse.prototype;

    /**
     * Get file for this row
     * @override
     * @returns {File} file
     */
    DocumentRowFile.prototype.getFile = function() {
    	return this._file;
    };
    
    /**
     * Setter for this._file
     * @param {File} file
     */
    DocumentRowFile.prototype.setFile = function(file) {
        var changeName = (this.docName == this._file.name);
    	this._file = file;
    	if(changeName){
    		this.docName = this._file.name;
    	}
    };
    
    /**
	 * @override
     */
    DocumentRowFile.prototype.getCurrentDocName = function(){
        return this._file.name;
    };
    
	/**
	 * @override
	 */
    DocumentRowFile.prototype.isValidFileSelected = function () {
		return true;
	};

	/**
	 * @override
	 */
    DocumentRowFile.prototype.getEffectiveFileInputObj = function () {
		return $("<input />").attr("name", this.fileInputName);
	};
	
	/**
	 * @override
	 */
	DocumentRowFile.prototype.validateDocRow = function (validObj, depth) {
		var isOK = true;
		// call the super validation implementation that takes care of required properties and documents
		// (both in documents table column and associated attr. category).
		if (!DocumentRow.prototype.validateDocRow.call(this, validObj, depth)) {
			isOK = false;
		}
		return isOK;
	};
    
    return DocumentRowFile;
});