define(["doctablex/DocumentRowBrowseView",
        "doctablex/DocTableXUtils",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml"], 
function(DocumentRowBrowseView, 
          DocTableXUtils, 
          DocRefStrings,
          WCXML){
    "use strict";
	 
    /**
	 * This class implements rendering methods for an upload document row with an injected file instead of file input.
	 * 
	 * @exports DocumentRowFileView 
	 * @class
	 * @augments DocumentRowBrowseView
	 * @constructor
	 */
    var DocumentRowFileView = function(dataModel, phElementID, options) {
		if (!dataModel && !phElementID && !options) {
			// when there is nothing to initialize from, return just itself (not initialized)
			return this;
		}
		DocumentRowBrowseView.call(this, dataModel, phElementID, options);   // calling superclass constructor!
	
		return this;
    };

    // define DocumentRowFileView as a subclass of DocumentRowBrowseView
    DocumentRowFileView.prototype = new DocumentRowBrowseView();
    DocumentRowFileView.prototype.constructor = DocumentRowFileView;
    DocumentRowFileView.prototype.parent = DocumentRowBrowseView.prototype;

    /**
	 * The render method generating to show the selected document and it's properties
	 * 
	 * @override
     * @param {jQueryObject} phElem - HTML DOM element, where to render the content generated by this method.
     * @param {Object} config
	 */
    DocumentRowFileView.prototype.renderDocumentSpec = function(phElem, config) {
		try {
			// Unregister the old resizer for this collumn.
			if (this._nameColResizer) {
				$(window).off("resize", this._nameColResizer);
			}
			
			// Create the collumn element.
        	var colElem = $('<' + this.itemRootElemStr +'>');
        	
        	// Set a custom width if this was specified.
        	if (config.width && config.width != "auto") {
        		colElem.width(config.width);
        	}
        	
        	// Make sure that the contents will obey this width. They may choose to ignore the width on the cell in order to fit so force them!
            var container = $(this.addStartOfCustomWidthDiv(config, '150px'));
        	
        	// add label, hidden input and change file link
            var labelDiv = $('<div>').addClass("FileRowLabel");
			var label = $('<span>').addClass("docname-label").html(this.dm.dlsName + ": " + this.dm._file.name);
			labelDiv.append(label);
			var input = $('<input type="file" />').addClass("hidden-file-input").on("change", _.bind(function(){
				if(input[0].files && input[0].files.length==1){
					this.dm.setFile(input[0].files.item(0));
					label.html(this.dm.dlsName + ": " + this.dm.getFile().name);
					if(!this.dm.isAutonamingEnabled()){
						docNameInput.val(this.dm.docName);
					}
					this.dm.fileChanged();
				}
			}, this));
			var link = $('<a href="#" />').html(DocRefStrings.t("ChangeFile")).on("click", function(){
				input.click();
			});
			labelDiv.append(label).append(input).append(' ('+this.userFriendlySize(this.dm.getFile().size)+') ');
			if(this.dm.isEditable){
				labelDiv.append(link);
			}
			
						
			// document name, depending on autonaming settings
			var docNameInput = $('<input>', {"type" : "text"}).addClass('docname-input '+this.getSAHIHelperStr('docname'));
			if (this.dm.isAutonamingEnabled()) {
				if (this.dm.hasAutonameSuffix()) {
					docNameInput.attr('placeholder', DocRefStrings.t('DocRowDocumentSuffix'));
					docNameInput.val(this.dm.docNameSuffix);
					docNameInput.on("change", _.bind(function(){
						this.dm.docNameSuffixChanged(docNameInput.val());
					},this));
				} else {
					docNameInput.attr('placeholder', this.getAutoNamingPreview());
					docNameInput.attr('disabled', 'disabled');
				}
			} else {
				docNameInput.attr('placeholder', DocRefStrings.t('DocRowDocumentName'));
				docNameInput.val(this.dm.docName);
				docNameInput.on("change", _.bind(function(){
					this.dm.docNameChanged(docNameInput.val());
				},this));
			}

			// document description
			var docDescOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docDescChanged(this.value);';
			var docDesc = (this.dm.docDesc ? this.dm.docDesc : '');
			var docDescInput = $("<input>", {"type" : "text", 
				"class" : this.getSAHIHelperStr('docdescription')+" docname-input", 
				"placeholder" : DocRefStrings.t('DocRowDocumentDescription'),
				"value" : docDesc,
				"onchange" : docDescOnChangeMethod});

			// template selector
			var templateSelector = this.renderDocTemplateSelector(config.hideDocTemplate != "true");
			
			// folder selector
			var folderSelector = this.renderDocFolderSelector(config.hideFolder != "true");

			// Disable fields when not editable
            if(!this.dm.isEditable){
            	docNameInput.attr("disabled", "disabled");
            	docDescInput.attr("disabled", "disabled");
            }
            
            // Responsive doc description/name
            var responsiveDocNameDescWrapper = $("<div>", {"style" : "width: 100%"});
			var responsiveDocNameWrapper = $("<div>", {"style" : "float: left; width: 100%;", "class" : "DocumentNameRow"});
			var responsiveDocDescWrapper = $("<div>", {"style" : "float: left; width: 100%", "class" : "DocumentNameRow"});
			responsiveDocNameWrapper.append(docNameInput);
			responsiveDocDescWrapper.append(docDescInput);
			responsiveDocNameDescWrapper.append(responsiveDocNameWrapper, responsiveDocDescWrapper);
			
			// Responsive template/ folder selectors
			var responsiveTemplateFolderWrapper = $("<div>", {"style" : "width: 100%", "id" : this.phElemID + "_docTempFolderSelPHID"});
			var responsiveTemplateWrapper = $("<div>", {"style" : "float: left; width: 100%;"});
			var responsiveFolderWrapper = $("<div>", {"style" : "float: left; width: 100%"});
			if (templateSelector) {
				responsiveTemplateWrapper.append(templateSelector);
			} else {
				responsiveTemplateWrapper.hide();
			}
			responsiveFolderWrapper.append(folderSelector);
			responsiveTemplateFolderWrapper.append(responsiveTemplateWrapper, responsiveFolderWrapper);
			
			// Apply visibility.
			if (config.hideSource == "true") {
				sourceFile.hide();
			}
			if (config.hideDescription  == "true") {
				responsiveDocDescWrapper.hide();
			}
			if (config.hideDocTemplate == "true") {
				responsiveTemplateWrapper.hide();
			}
			if (config.hideFolder == "true") {
				responsiveFolderWrapper.hide();
			}
			
			// Create a resizer for this collumn.
			this._nameColResizer = function () {
				var breakPoint = 450;
				if (responsiveDocNameDescWrapper && responsiveDocNameWrapper.is(':visible') && responsiveDocDescWrapper.is(':visible')) {
					var width = responsiveDocNameDescWrapper.width();
					if (width < breakPoint) {
						responsiveDocNameWrapper.css("width","100%");
						responsiveDocNameWrapper.css("padding-right", "0px");
						responsiveDocDescWrapper.css("width", "100%");
					} else {
						responsiveDocNameWrapper.css("width","50%");
						responsiveDocNameWrapper.css("padding-right", "2px");
						responsiveDocDescWrapper.css("width", "50%");
					}
				}
				if (responsiveTemplateFolderWrapper && responsiveTemplateWrapper.is(':visible') && responsiveFolderWrapper.is(':visible')) {
					var width = responsiveTemplateFolderWrapper.width();
					if (width < breakPoint) {
						responsiveTemplateWrapper.css("width","100%");
						responsiveTemplateWrapper.css("padding-right", "0px");
						responsiveFolderWrapper.css("width", "100%");
					} else {
						responsiveTemplateWrapper.css("width","50%");
						responsiveTemplateWrapper.css("padding-right", "2px");
						responsiveFolderWrapper.css("width", "50%");
					}
				}
			};
			$(window).resize(this._nameColResizer);
			// Also try to execute the resizer after the table has rendered, so defer the execution of the resizer.
			window.setTimeout(function () {
				this._nameColResizer();
			}.bind(this), 1);
			
			// Add the elements.
			container.append(labelDiv, responsiveDocNameDescWrapper, responsiveTemplateFolderWrapper);
            colElem.append(container);
            
            // Add the cell to the row.
            phElem.append(colElem);
            
		} catch (err) {
			DocTableXUtils.log('Problem in DocumentRowFileView.renderDocumentSpec:\n' + err);
		}
    };
    
    DocumentRowFileView.prototype.userFriendlySize = function(inBytes){
    	if(inBytes < 1024){
    		return inBytes + ' B';
    	} else {
    		inBytes = Math.round(inBytes*100/1024)/100; 
    		if(inBytes < 1024){
        		return inBytes + ' kB';
        	} else {
        		inBytes = Math.round(inBytes*100/1024)/100; 
        		if(inBytes < 1024){
            		return inBytes + ' MB';
            	} else {
            		inBytes = Math.round(inBytes*100/1024)/100; 
            		return inBytes + ' GB';
            	}
        	}
    	}
    };
    
    

    return DocumentRowFileView;
});