define(["doctablex/DocumentRowView",
        "doctablex/DocTableXUtils",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml"], 
function(DocumentRowView, 
          DocTableXUtils, 
          DocRefStrings,
          WCXML){
    /**
     * DocumentRowLinkView.js
     * 
     * This class implements rendering methods for document row of the "Link" document source.
     * 
     * @author GRBO - March 2015
     */

    "use strict";

    var DocumentRowLinkView = function(dataModel, phElementID, options) {
        if (!dataModel && !phElementID && !options) {
            // when there is nothing to initialize from, return just itself (not initialized)
            return this;
        }
        DocumentRowView.call(this, dataModel, phElementID, options);   // calling superclass constructor!

        // note this.dataModel and phElemID are initialized in the super class...)


        // private initialization ========================================

        if (this.constructor == DocumentRowLinkView.prototype.constructor) {
            // this part of initialization is private for the instances of this class only!
            // it won't be executed for any subclass

        }

        return this;
    };

    // define DocumentRowLinkView as a subclass of DocumentRowView
    DocumentRowLinkView.prototype = new DocumentRowView();
    DocumentRowLinkView.prototype.constructor = DocumentRowLinkView;
    DocumentRowLinkView.prototype.parent = DocumentRowView.prototype;


    // the main render method is inherited from the base class



    /**
     * The render method generating HTML for "Current Project" (Reference) document source to be able to 
     * select a document from the current project and add it to this documents table (value of actual
     * document reference attribute)
     * 
     * @override
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
     * @param {Object} config - Holds the documents table rendering configuration for this column/item.
     * @returns {String} - The generated HTML.
     */
    DocumentRowLinkView.prototype.renderDocumentSpec = function(phElem, config) {
    	try {
			// Create the collumn element.
        	var colElem = $('<' + this.itemRootElemStr +'>');
        	
        	// Set a custom width if this was specified.
        	if (config.width && config.width != "auto") {
        		colElem.width(config.width);
        	}
        	
        	// Make sure that the contents will obey this width. They may choose to ignore the width on the cell in order to fit so force them!
            var container = $(this.addStartOfCustomWidthDiv(config, '150px'));
            
            if (this.dm.isInErrorStatus()) {
            	if (this.dm.isInConfigPreview()) {
                    var message = $('<span>').text(DocRefStrings.t('DocSrcHdlNoProjectInConfig'));
                    container.append(message);
            	} else {
            		var message = $('<span>').text(DocRefStrings.t('DocRowLoadingDocsError'));
            		container.append(message);
            	}
            } else {
	            
            	var docRow = $("<div>", {"class" : "DocumentNameRow"});
	        	if (!this.dm.availableDocuments) {
	                // documents from this project not loaded yet
	            	var message = $('<span>').text(DocRefStrings.t('DocRowLoadingDocs'));
	            	docRow.append(message);
	        	} else if (this.dm.availableDocuments.length == 0) {
	                // loading of documents finished, but there are no documents in the project
	        		// (or they are not visible to the logged in user)
	            	var message = $('<span>').text(DocRefStrings.t('DocRowNoDocsAvailable'));
	            	docRow.append(message);
	            } else if (this.dm.isModifiedElsewhere()) {
	            	var selDocName = $('<span>').text(this.dm.docName);
	            	docRow.append(selDocName);
	            } else {
	                // there is one or more documents available to be referenced (linked) - render
	            	// the selector (dropdown) of available documents
	                var docVersionSelectedMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docVersionSelected(this.value);';
	                var docSelector = $("<select>", {"class" : "docname-input "+this.getSAHIHelperStr('docname'), "onChange" : docVersionSelectedMethod});
	                // render all the options of the document template selector
	                var helpOption = $("<option>", {"value" : ""}).text(DocRefStrings.t('DocRowPleaseSelect'));
	                docSelector.append(helpOption);
	                for (var idx = 0; idx < this.dm.availableDocuments.length; idx++) {
	                	var id = this.dm.availableDocuments[idx].id;
	                	var name = this.dm.availableDocuments[idx].name ;
	                	var option = $("<option>", {"value" : id}).text(name);
	                	if (this.dm.docVerID  == id) {
	                    	option.attr("selected", "selected");
	                    }
	                	docSelector.append(option);
	            	}
	                docRow.append(this.renderDocDetails());
	                docRow.append(docSelector);
	            }
	        	container.append(docRow);
            }
            
            // Add the html to the collumn element.
            colElem.append(container);
            
            // Add the cell to the row.
            phElem.append(colElem);
            
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowLinkView.renderDocumentSpec:\n' + err);
        }
    };
    
    /**
     * Return the details about the linked document.
     */
    DocumentRowLinkView.prototype.renderDocDetails = function() {
    	return $();
    };
    
    /**
	 * Method rendering the thumbnail or its placeholder (if it's not loaded yet).
	 * This override variation of the method always draws the thumbnail representing a new, unknown document.
	 * 
	 * @override
	 * @returns {String} - The generated HTML.
	 */
    DocumentRowLinkView.prototype.renderThumbnail = function(phElem, config) {
        try {
            if (this.dm.docVerID) {
            	DocumentRowLinkView.prototype.parent.renderThumbnail.call(this, phElem, config);
            } else {
                var htmlcode = '<' + this.itemRootElemStr + '>';
    			htmlcode += this.customThumbnailHTML('thumbnails/other.gif', config.thumbnailSize);
            	htmlcode += '</' + this.itemRootElemStr + '>';
                phElem.append(htmlcode);
            }
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowLinkView.renderThumbnail:\n' + err);
        }
    };
    
    return DocumentRowLinkView;
});