define(["doctablex/DocumentRowView", 
        "doctablex/DocTableXUtils", 
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml", 
        "jquery",
        "component/modal/Modal"],   //JANH is this really needed? Wouldn't jQuery dialog do the same trick?
function(DocumentRowView,
         DocTableXUtils,
         DocRefStrings,
         WCXML,
         $,
         Modal){
    /**
     * DocumentRowNewView.js
     * 
     * This class implements rendering methods for DocumentRowNew class.
     * 
     * @author JANH - February 2015
     */

    "use strict";

    var DocumentRowNewView = function(dataModel, phElementID, options) {
        if (!dataModel && !phElementID && !options) {
            // when there is nothing to initialize from, return just itself (not initialized)
            return this;
        }
        DocumentRowView.call(this, dataModel, phElementID, options);   // calling superclass constructor!

        // note this.dataModel and phElemID are initialized in the super class...)

        return this;
    };

    // define DocumentRowNewView as a subclass of DocumentRowView
    DocumentRowNewView.prototype = new DocumentRowView();
    DocumentRowNewView.prototype.constructor = DocumentRowNewView;
    DocumentRowNewView.prototype.parent = DocumentRowView.prototype;


    // the main render and repaint methods are inherited from the base class



    /**
     * Method to create a document template selection dialog place holder element (a <div>) with a basic 
     * jQuery dialog configuration, if it doesn't exist yet.
     * 
     * @returns {jQuery object} - Returns a jQuery object holding the found or created place holder for the dialog.
     */
    DocumentRowNewView.prototype.getDocTemplateDialogPlaceholder = function() {
    	
    	if (!this._docTemplateDialog) {
    		this._docTemplateDialog = new Modal($("<div>"), {"customClass" : "DocTemplateDialog"});
    		this._docTemplateDialog.setTitle(DocRefStrings.t('SELECTDOCTEMPLATE'));
    		
    		var cancelButton = $("<button>",{"class":"btn btn-default"}).text(DocRefStrings.t('CANCEL'));
    		var addButton = $("<button>",{"class":"btn btn-primary"}).text(DocRefStrings.t('OK'));
    		var footer = $("<div>");
    		footer.append(cancelButton);
    		footer.append(addButton);
    		this._docTemplateDialog.setFooter(footer);
    		
    		// callback to be executed whenever the _docTemplateDialog dialog is closed
    		this._docTemplateDialog.onClose(function () {
    			if (!this._docTemplateDialogSelectionConfirmed) {
	    			// in case the document template selection was cancelled make sure the partly initialized
	    			// document row is completely gone (e.g. that the "document name validator" won't complain 
	    			// about missing document name, etc.). 
	    			// (note deleteItself() method can handle that the row was not added to documents table yet...)
	    			this.dm.deleteItself(false);
    			}
   			}.bind(this));
    		
    		addButton.on("click", function () {
    			if (this._selectedDocTempIdx !== undefined) {
        			this._docTemplateDialogSelectionConfirmed = true;
    				this._docTemplateDialog.close();
    				this.dm.addDocTemplate(this._selectedDocTempIdx);
    			}
    		}.bind(this));
    		
    		cancelButton.on("click", function () {
    			this._docTemplateDialogSelectionConfirmed = false;
    			this._docTemplateDialog.close();
    		}.bind(this));
    	}
        return this._docTemplateDialog;
    };


    DocumentRowNewView.prototype.renderDocTemplateDialog = function() {
        var modal = this.getDocTemplateDialogPlaceholder();
        
        if (!this.dm.docTemplates || (this.dm.docTemplates.length == 0) || !modal) {
            return null;
        }

        var content = $("<div>", {"style": "overflow: auto; height: 350px;"});
        
        var wcrtable = $("<div>", {"class": "wcr-table"});
       
        var filterInput = $("<input>",{"id":"docTemplateDialogFilter", "name" : "docTemplateDialogFilter", "class":"form-control"});
        var filterInputDiv = $("<div>", {"class": "input-group full-width add-padding-bottom"}).append(
        		$("<label>", {"class": "input-group-addon","for" : "docTemplateDialogFilter"}).text(DocRefStrings.t('FILTER')),
        		filterInput
        		);
        
        var filterTotalDiv = $("<label>", {"style": "margin-top: 5px;"}).append(
        		$("<span>", {"class": "add-spacing-right"}).text(DocRefStrings.t('PROJDETAILSTOTALDOCS_TAG') + ':'),
        		$("<span>").text(this.dm.docTemplates.length));
        
        var filterwcrtable = $("<div>", {"class": "wcr-table-filter"}).append(
        		$("<div>", {"class": "wcr-table-actions-row no-actions"}).append(
        				$("<div>", {"class": "col-xs-12 col-sm-9"}).append(filterInputDiv),
        				$("<div>", {"class": "col-xs-12 col-sm-3"}).append(filterTotalDiv)
        			)
        		);
        
        
        var tablesection = $("<div>", {"class": "wcr-table-section"});
        
        
        var table = $("<table>", {"class":"ordtable DocTemplateDialogTable"}).attr({"id": this.dm.parentDocTable.prefix + "_doctemplatedialogtable"});
                
        var tbody = $("<tbody>");
        var addCallbackToRow = function (row, docTempIdx) {
        	row.on("click", function (){
        		// Set the selected docTempIdx
        		this._selectedDocTempIdx = docTempIdx;
        		// Clear the previous selected rows.
        		$(".docTemplateRow.activeRow").find("input").prop("checked", false);
        		tbody.find("> tr").removeClass("activeRow");
        		// Select this one.
        		row.addClass("activeRow");
        		$("input", row).prop("checked", true);
        	}.bind(this));
        	
        	row.dblclick(function () {
        		// Set the selected docTempIdx
        		this._selectedDocTempIdx = docTempIdx;
        		this._docTemplateDialogSelectionConfirmed = true;
        		// Close the dialog an add the template
        		this._docTemplateDialog.close();
				this.dm.addDocTemplate(this._selectedDocTempIdx);
        	}.bind(this));
        	
        	
        }.bind(this);
        for (var dti = 0; dti < this.dm.docTemplates.length; dti++) {
            var docTemp = this.dm.docTemplates[dti];
            var row = $("<tr>", {"class" : "docTemplateRow", "style" : "cursor: pointer"});
            
            
            var radioCell =  $('<td>').addClass("td-narrow").append($('<input>').attr({"type":"radio","value":dti,"name":"docTemplateSelectOption"}));
            
            if (dti == 0)  $('input', radioCell).prop("checked",true);   // by default check the first doc. template in the list
            
            var thumbCell = $("<td>",{"style":"text-align: center;"}).addClass("td-narrow");
            var thumbNail = $("<img>", {"src" : docTemp.thumbnailURL, "width" : 50, "height" : 50, "align": "middle"});
            
            var nameCell = $("<td>");
            var name = $("<span>", {"class":"docTemplateName"}).text(docTemp.name);
            
            row.append(radioCell,thumbCell);
            thumbCell.append(thumbNail);
            row.append(nameCell);
            nameCell.append(name);
            
            // Add a callback
            addCallbackToRow(row, dti);
            
            // Select the first row by default
            if (dti == 0) {
                this._selectedDocTempIdx = 0;
                row.addClass("activeRow");
            }
            
            // Add the row to the body.
            tbody.append(row);
        }
        table.append(tbody);

        filterInput.on('keyup', function() {
            var filter = filterInput.val().toLowerCase();   // the filter is case insensitive
            if (filter.length > 0) {
                $("tr", tbody).each(function(i, context) {
                    var name = $("td > span.docTemplateName", context).text().toLowerCase();
                	var row = $(context);
                    // show what complies to the filter and hide the rest
                    if (name.indexOf(filter) != -1) {
                    	row.show();
                    } else {
                    	row.hide();
                    }
                });	
            }
            else {
                // show all doc. templates if there is no filter
            	$("tr", tbody).show();
            }
        }.bind(this));

        tablesection.append(table);
        wcrtable.append(filterwcrtable);
        wcrtable.append(tablesection);
        content.append(wcrtable);
        
        modal.setContent(content);

        return modal;
    };




    /**
     * The render method generating HTML for "Document template" (New) document source to be able to create 
     * a new document based on existing document template including name and description definition fields
     * (if applicable).
     * 
     * @override
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
     * @returns {String} - The generated HTML.
     */
    DocumentRowNewView.prototype.renderDocumentSpec = function(phElem, config) {
    	try {
			// Unregister the old resizer for this collumn.
			if (this._nameColResizer) {
				$(window).off("resize", this._nameColResizer);
			}
			
			// Create the collumn element.
        	var colElem = $('<' + this.itemRootElemStr +'>');
        	
        	// Set a custom width if this was specified.
        	if (config.width && config.width != "auto") {
        		colElem.width(config.width);
        	}
        	
        	// Make sure that the contents will obey this width. They may choose to ignore the width on the cell in order to fit so force them!
            var container = $(this.addStartOfCustomWidthDiv(config, '150px'));   
        
            // Check wether a document template was defined ...
            if (this.dm.selDocTemplate) {
            	
                // document row "label" with selected doc. template name
            	// ... but first make sure if the config allows to render this:
            	var sourceFile = $("<div>");
            	var sourceLabel = $('<span>', {"class" :"docname-label"}).text(this.dm.dlsName);
                var sourceName = this.renderDocDetails();
                sourceFile.append(sourceLabel, sourceName);
                
            	// document name
                var docNameInput;
    			if (this.dm.isAutonamingEnabled()) {
    				// when autonaming is on the behavior of "document name" field differs
    				if (this.dm.hasAutonameSuffix()) {
    					var docSuffixOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameSuffixChanged(this.value);';
    					var docSuffix = (this.dm.docNameSuffix ? this.dm.docNameSuffix : '');
    					docNameInput = $("<input>", {"type" : "text", 
    						"class" : this.getSAHIHelperStr('docnamesuffix')+" docname-input", 
    						"placeholder" : DocRefStrings.t('DocRowDocumentSuffix'),
    						"value" : docSuffix,
    						"onchange" : docSuffixOnChangeMethod});
    				} else {
    					docNameInput = $("<input>", {"type" : "text", 
    						"class" : this.getSAHIHelperStr('docname')+" docname-input", 
    						"placeholder" : this.getAutoNamingPreview(),
    						"disabled": "disabled"});
    				}
    			} else {
    				var docNameOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameChanged(this.value);';
    				var docName = (this.dm.docName ? this.dm.docName : '');
    				docNameInput = $("<input>", {"type" : "text", 
    					"class" : this.getSAHIHelperStr('docname')+" docname-input", 
    					"placeholder" : DocRefStrings.t('DocRowDocumentName'),
    					"value" : docName,
    					"onchange" : docNameOnChangeMethod});
    			}
    			
    			// document description
    			var docDescOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docDescChanged(this.value);';
    			var docDesc = (this.dm.docDesc ? this.dm.docDesc : '');
    			var docDescInput = $("<input>", {"type" : "text", 
    				"class" : this.getSAHIHelperStr('docdescription')+" docname-input", 
    				"placeholder" : DocRefStrings.t('DocRowDocumentDescription'),
    				"value" : docDesc,
    				"onchange" : docDescOnChangeMethod});

    			// folder selector
    			var folderSelector = this.renderDocFolderSelector(config.hideFolder != "true");
    			
    			// Disable fields when not editable
                if(!this.dm.isEditable){
                	docNameInput.attr("disabled", "disabled");
                	docDescInput.attr("disabled", "disabled");
                }
                
                // Responsive doc description/name
                var responsiveDocNameDescWrapper = $("<div>", {"style" : "width: 100%"});
    			var responsiveDocNameWrapper = $("<div>", {"style" : "float: left; width: 100%;", "class" : "DocumentNameRow"});
    			var responsiveDocDescWrapper = $("<div>", {"style" : "float: left; width: 100%;", "class" : "DocumentNameRow"});
    			responsiveDocNameWrapper.append(docNameInput);
    			responsiveDocDescWrapper.append(docDescInput);
    			responsiveDocNameDescWrapper.append(responsiveDocNameWrapper, responsiveDocDescWrapper);
    			
    			// Non responsive folder wrapper
    			var folderWrapper = $("<div>", {"style" : "width: 100%", "id" : this.phElemID + "_folderSelPHID"});
    			folderWrapper.append(folderSelector);
    			
    			
    			// Apply visibility.
    			if (config.hideSource == "true") {
    				sourceFile.hide();
    			}
    			if (config.hideDescription  == "true") {
    				responsiveDocDescWrapper.hide();
    			}
    			if (config.hideFolder == "true") {
    				folderWrapper.hide();
    			}
    			
    			// Create a resizer for this collumn.
    			this._nameColResizer = function () {
    				var breakPoint = 360;
    				if (responsiveDocNameDescWrapper && responsiveDocNameWrapper.is(':visible') && responsiveDocDescWrapper.is(':visible')) {
    					var width = responsiveDocNameDescWrapper.width();
    					if (width < breakPoint) {
    						responsiveDocNameWrapper.css("width","100%");
    						responsiveDocNameWrapper.css("padding-right", "0px");
    						responsiveDocDescWrapper.css("width", "100%");
    					} else {
    						responsiveDocNameWrapper.css("width","50%");
    						responsiveDocNameWrapper.css("padding-right", "2px");
    						responsiveDocDescWrapper.css("width", "50%");
    					}
    				}
    			};
    			$(window).resize(this._nameColResizer);
    			// Also try to execute the resizer after the table has rendered, so defer the execution of the resizer.
    			window.setTimeout(function () {
    				this._nameColResizer();
    			}.bind(this), 1);
    			
    			// Add everything to the container.
    			container.append(sourceFile);
    			container.append(responsiveDocNameDescWrapper);
    			container.append(folderWrapper);
                
            } else {
            	
            	// No document template was defined!
            	
            	// Allow the user to select one again.
            	var button = $("<button>",{"type":"button", "class" : "btn btn-default"}).text(WCXML.xmlencode(DocRefStrings.t('DocRowSelectTemplate')));
            	button.on("click", function () {
            		this.dm.showDocTemplateDialog();
            	}.bind(this));
            	
            	var hiddenDocNameInput = $("<input>",{"type":"hidden", "class" : "docname-input "+this.getSAHIHelperStr('docname')});
            	
            	container.append(button);
            	container.append(hiddenDocNameInput);          	
            }
            
            // Add the html to the collumn element.
            colElem.append(container);
            
            // Add the cell to the row.
            phElem.append(colElem);
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowNewView.renderDocumentSpec:\n' + err);
        }
    };
   
    /**
     * Method to render the name of the sourced document.
     * 
     * @returns {jQuery} - The element containing the name of the sourced document.
     */
    DocumentRowNewView.prototype.renderDocDetails = function() {
    	return $('<span>', {"class" :"docname-text"}).text(this.dm.selDocTemplate.name);
    };
    
    /**
     * Method to render Clone button in the row based on render configuration in Attribute Category 
     * 
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
     */
    DocumentRowNewView.prototype.renderCloneButton = function(phElem) {
        try {
        	// call the StaticDocumentRow.renderCloneButton() directly skipping the default implementation
        	// for all new document rows from DocumentRow class
        	DocumentRowNewView.prototype.parent.parent.renderCloneButton.call(this, phElem);
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowSearchView.renderCloneButton:\n' + err);
        }
    };
    
    return DocumentRowNewView;
});
