define(["jquery",
		"doctablex/DocTableXUtils",
		"wcr/webcenter_xml"],
function($,
		DocTableXUtils,
		WCXML) {
	
	/**
	 * TextContentRowLink.js
	 *
	 * Object representing a single text statement added to documents table (DocTableX instance)
	 * via SavedSearch document source. It holds required data and implements controller logic of the 
	 * represented text statement (row).
	 *
	 * @author JANH, January 2016
	 */

	"use strict";

	var TextContentRowLink = function() {
		
		this.__isMixin = true;   //JANH debug then DEL!!!


	    /**
	     * Action handler when the selected document changes.
	     * We need to ask the server information about the selected document
	     * 
	     * @override mixin
	     * @param {String} docVersionID - Version ID of the newly selected document.
	     * @returns 
	     * 		- null if nothing has to be done, 
	     * 		- a Promise object telling when the update operation finishes, when the update has to be done.
	     */
	    if (this.docVersionSelected) { TextContentRowLink.__overridden.docVersionSelected = this.docVersionSelected; }
	    this.docVersionSelected = function(docVersionID) {
	    	// first execute the overridden functionality of the base class
	    	var updateState = TextContentRowLink.__overridden.docVersionSelected.call(this, docVersionID);
	    	
	    	if (updateState != null) {
	    		// if the an update needs to be done, perform the text content part as well
		    	updateState.done(function() {
			        // if table has any text content column, fetch the data required for rendering text content view 
			    	if (DocTableXUtils.hasTextContentItem(this.renderOptions.columns) && this.docTypeID == '00001_0000000019') {
			    		this.setStatus('loading'); 
			    		var tcInfoReq = this.updateTextContentDataFromServer(this.docVerID , false);
						$.when(tcInfoReq).done(function(textContentList) {
							this.setStatus('ready');
		    				// repainting the document row with the updated values/info
		    				this.view.repaintRow();
						}.bind(this))
						.fail(function(errorMsg) {   // on error
							this.setStatus('error');
							// no need to alert the error message as the document row rendering is altered based on the error status!
							DocTableXUtils.log('Failed to load text content information from the server:\n' +errorMsg);
						}.bind(this));
			    	}
		    	}.bind(this));   // end of: updateState.done(...
	    	}
	    };
	
	};
	
	
	/**
	 * Mixin constructor/initializer.
	 * (class method)
	 */ 
	TextContentRowLink.__initMixin = function(options) {

//    	TextContentRowLinkView.__initMixin.call(this.view, this.renderOptions);   // the view mixin class is not needed so far...
	
    	//TODO: Move the text content related loading methods from StaticDocumentRowView to this class
    	
		return this;
	};

	
	/**
	 * List of overridden methods with their original code.
	 */ 
	TextContentRowLink.__overridden = {
	};


	return TextContentRowLink;
});
