define(["jquery",
		"doctablex/DocTableXUtils",
		"wcr/webcenter_xml"],
function($,
		DocTableXUtils,
		WCXML) {
	
	/**
	 * TextContentRowNew.js
	 *
	 * Object representing a single text statement added to documents table (DocTableX instance)
	 * via new document source. It holds required data and implements controller logic of the 
	 * represented text statement (row).
	 *
	 * @author JANH, January 2016
	 */

	"use strict";

	var TextContentRowNew = function() {
		
		this.__isMixin = true;   //JANH debug then DEL!!!

		/**
		 * List of overridden methods with their original code.
		 */ 
		TextContentRowNew.__overridden = {
		};

	    /**
	     * Method overriding the DocumentRowNew.updateDocTemplateSelection() method. 
	     * This version does in addition copy document name, thumbnail and document type ID from the selected 
	     * doc. template to this doc. row as default values for the newly created document.
	     * 
	     * @override mixin
	     * @param {int} selIdx - Index of the selected document template. It corresponds to the index of the doc.
	     * 		template in this.docTemplates array.
	     */
	    if (this.updateDocTemplateSelection) { TextContentRowNew.__overridden.updateDocTemplateSelection = this.updateDocTemplateSelection; }
	    this.updateDocTemplateSelection = function(selIdx) {
	    	// first execute the overridden functionality of the base class
	    	var updateState = TextContentRowNew.__overridden.updateDocTemplateSelection.call(this, selIdx);
	    	
	    	updateState.done(function() {
		        // if table has any text content column, fetch the data required for rendering text content view 
		    	if (DocTableXUtils.hasTextContentItem(this.renderOptions.columns) && this.docTypeID == '00001_0000000019') {
		    		this.setStatus('loading');
		    		var tcInfoReq = this.updateTextContentDataFromServer(this.selDocTemplate.docVerID, false, true);
		    		tcInfoReq.done(function(textContentList) {
						this.setStatus('ready');
	    				// repainting the document row with the updated values/info
	    				this.view.repaintRow();
		    		}.bind(this))
					.fail(function(errorMsg) {   // on error
						this.setStatus('error');
						// no need to alert the error message as the document row rendering is altered based on the error status!
						DocTableXUtils.log('Failed to load text content information from the server:\n' +errorMsg);
					}.bind(this));
		    	}
	    	}.bind(this));
	    };
	
	};
	
	/**
	 * Mixin constructor/initializer.
	 * (class method)
	 */ 
	TextContentRowNew.__initMixin = function(options) {
		return this;
	};


	return TextContentRowNew;
});
