define(["jquery",
		"packaging-content-management/text-content-services/elementTypeService"
],
function($,
		elementTypeService) {
	
	/**
	 * TextContentRowNewView.js
	 *
	 * Object representing the view of a single text statement added to documents table (DocTableX instance)
	 * via New document source. 
	 *
	 * @author KADA, August 2016
	 */

	"use strict";

	var TextContentRowNewView = function() {
		
		this.__isMixin = true;   //JANH debug then DEL!!!

		/**
		 * List of overridden methods with their original code.
		 */ 
		TextContentRowNewView.__overridden = {};

	    /**
	     * Method overriding the renderDocSourceName() method. 
	     */
	    if (this.renderDocDetails) { TextContentRowNewView.__overridden.renderDocDetails = this.renderDocDetails; }
	    this.renderDocDetails = function() {
	    	
	    	if (this.dm.textContent) {
	    		
				var container = $("<span>",{"class" :"docname-text"}).text(this.dm.selDocTemplate.name);

	        	// In order to render the name collumn we'll need some more information on the element type.
	        	elementTypeService.getElementTypes(function (elementTypes) {
	    		
		    		// Get the elementType details for this element type.
	        		var elementType = elementTypes[this.dm.textContent.elementType];
	        		
	        		// Create a div that will contain the information about the element type.
	        		var elementTypeName = $("<div>", {"style" : "display: inline-block"});
	        		
	        		// Show the color of the element type of this text content document.
	        		elementTypeName.append($("<div>", {"class": "textcontent elementypecolor"}).css({
	        			"margin-left" : "4px",
	        			"background-color" : elementType.color
	        		}));
	        		
	        		// Show the human readable string for the element type.
	        		elementTypeName.append($("<b>").text(elementType.elementHumanReadable));
	        		
	        		// Add it to the container.
	        		container.empty();
	        		container.append(elementTypeName);
	        		
	        	}.bind(this));
    		
	    		// Add it to the container.
	        	return container;
        	
	    	} else {
				// Document is not a text content document, render the normal document specification.
				var parentRenderMethod = TextContentRowNewView.__overridden.renderDocDetails.bind(this);
				return parentRenderMethod();
			}
	    };
	};
	
	/**
	 * Mixin constructor/initializer.
	 * (class method)
	 */ 
	TextContentRowNewView.__initMixin = function(options) {
		return this;
	};

	return TextContentRowNewView;
});
