define(["jquery",
		"doctablex/DocTableXUtils",
		"wcr/webcenter_xml"],
function($,
		DocTableXUtils,
		WCXML) {
	
	/**
	 * TextContentRowSearch.js
	 *
	 * Object representing a single text statement added to documents table (DocTableX instance)
	 * via SavedSearch document source. It holds required data and implements controller logic of the 
	 * represented text statement (row).
	 *
	 * @author JANH, January 2016
	 */

	"use strict";

	var TextContentRowSearch = function() {
		
		this.__isMixin = true;   //JANH debug then DEL!!!

		
		/**
		 * Private initialization method using AJAX to load asynchronously required info that is not passed by
		 * 'options' (constructor parameter) from the server.
		 */
		this._initializeMixinFromServer = function() {
			this.setStatus('loading');
			
			// make sure the textcontent initialization starts only after the base object is fully initialized!
			this._initFromServerState.done(function() {
	            // if the documents table has any text content column, fetch the data required for
				// the rendering of the text content view
	            var textContentReq = null;
	        	if (DocTableXUtils.hasTextContentItem(this.renderOptions.columns) && (this.docTypeID == '00001_0000000019')) {
	        		textContentReq = this.updateTextContentDataFromServer(this.docVerID, false, true);
	        		textContentReq.done(function() {   // on success
		        		// text content data ready - set the status of the row to "ready"
		        		this.setStatus('ready');
		        	}.bind(this))
		        	.fail(function(errorMsg) {   // on error
		                this.setStatus('error');
		        		// no need to alert the error message as the document row rendering is altered based on the error status!
		        		DocTableXUtils.log("Failed to update text content data from the server.\n" + errorMsg);
		            }.bind(this));	
	        	}
			}.bind(this));
		};
		
	};

	
	/**
	 * Mixin constructor/initializer. 
	 * (class method)
	 */ 
	TextContentRowSearch.__initMixin = function(options) {

//    	TextContentRowSearchView.__initMixin.call(this.view, this.renderOptions);   // the view mixin class is not needed so far...
	
		// initialization of this mixin additional functionality requiring server calls 
    	this._initializeMixinFromServer();
    	
    	//TODO: Move the text content related loading methods from StaticDocumentRowView to this class
    	
		return this;
	};

	
	/**
	 * List of overridden methods with their original code.
	 */ 
	TextContentRowSearch.__overridden = {
	};


	return TextContentRowSearch;
});
