define(["jquery",
        "utils/webi18n!DocRefStrings",
		"packaging-content-management/text-content-services/elementTypeService"
],
function($,
		DocRefStrings,
		elementTypeService) {
	
	/**
	 * TextContentRowSearchView.js
	 *
	 * Object representing the view of a single text statement added to documents table (DocTableX instance)
	 * via a search document source. 
	 *
	 * @author KADA, August 2016
	 */

	"use strict";

	var TextContentRowSearchView = function() {
		
		this.__isMixin = true;   //JANH debug then DEL!!!

		/**
		 * List of overridden methods with their original code.
		 */ 
		TextContentRowSearchView.__overridden = {};

	    /**
	     * Method overriding the renderDocSourceName() method. 
	     */
	    if (this.renderDocDetails) { TextContentRowSearchView.__overridden.renderDocDetails = this.renderDocDetails; }
	    this.renderDocDetails = function(isClone) {
	    	
	    	if (this.dm.textContent) {
	    		
	    		
	    		// document row "label" with selected doc. template name
	        	// ... but first make sure if the config allows to render this:
	            var sourceFile = $("<div>").hide();
	    		if ((this.dm.origDocRow) && !(this.dm.dlsName.indexOf('Replace') != -1)) {
	    			var label = $("<span>", {"class" : "docname-label"}).text(DocRefStrings.t('DocRowLocalRemoteMakeLocal'));
	    			sourceFile.append(label);
	    		} else {
	    			if(isClone){
	    				var label = $("<span>", {"class" : "docname-label"}).text(this.dm.dlsName);
	    				sourceFile.append(label);
	        		}else{
	        			var label = $("<span>", {"class" : "docname-label"}).text(this.dm.docOperation);
	        			sourceFile.append(label);
	        		}
	    		}
	    		var text = $("<span>", {"class" : "docname-text"}).text(this.dm.selDocName);
	    		sourceFile.append(text);
	    		
	        	// In order to render the name collumn we'll need some more information on the element type.
	        	elementTypeService.getElementTypes(function (elementTypes) {
	    		
		    		// Get the elementType details for this element type.
	        		var elementType = elementTypes[this.dm.textContent.elementType];
	        		
	        		// Create a div that will contain the information about the element type.
	        		var elementTypeName = $("<div>", {"style" : "display: inline-block"});
	        		
	        		// Show the color of the element type of this text content document.
	        		elementTypeName.append($("<div>", {"class": "textcontent elementypecolor"}).css({
	        			"background-color" : elementType.color,
	        			"margin-left": "4px"
	        		}));
	        		
	        		// Show the human readable string for the element type.
	        		elementTypeName.append($("<b>").text(elementType.elementHumanReadable));
	        		
	        		// Add it to the source name label.
	        		sourceFile.append(elementTypeName);
	        		
	        	}.bind(this));
    		
	    		// Add it to the container.
	        	sourceFile.show();
	        	return sourceFile;
        	
	    	} else {
				// Document is not a text content document, render the normal document specification.
				var parentRenderMethod = TextContentRowSearchView.__overridden.renderDocDetails.bind(this);
				return parentRenderMethod();
			}
	    };
	};
	
	/**
	 * Mixin constructor/initializer.
	 * (class method)
	 */ 
	TextContentRowSearchView.__initMixin = function(options) {
		return this;
	};

	return TextContentRowSearchView;
});
