define(["doctablex/DocTableXUtils",
        "jquery", 
		"utils/webi18n!DocRefStrings",
        "utils/webi18n!TextContentStrings",
		"packaging-content-management/text-content-element-widget/TextContentElementWidget",
		"packaging-content-management/language-indicator/LanguageIndicator",
		"packaging-content-management/text-content-services/elementTypeService"
		],
function(DocTableXUtils,
		$,
		DocRefStrings,
		webi18n,
        TextContentElementWidget,
        LanguageIndicator,
        elementTypeService) {
	
	/**
	 * TextContentRowView.js
	 *
	 * This class implements reusable rendering methods for all document rows.
	 *
	 * @author JANH, HUBU - February 2015
	 */

	"use strict";
	
    var TextContentRowView = function() {

		this.__isMixin = true;   //JANH debug then DEL!!!

		/**
		 * List of overridden methods with their original code.
		 */ 
		TextContentRowView.__overridden = {};
		
		
		/**
		 * Method overriding the renderDocNameLink() method.
		 */
	    if (this.renderDocNameLink) { TextContentRowView.__overridden.renderDocNameLink = this.renderDocNameLink; }
		this.renderDocNameLink = function() {
			
			if (this.dm.textContent) {
				
				var container = $("<div>");

	        	// In order to render the name collumn we'll need some more information on the element type.
	        	elementTypeService.getElementTypes(function (elementTypes) {
	        		
	        		// Get the elementType details for this element type.
	        		var elementType = elementTypes[this.dm.textContent.elementType];
	        		
	        		// If an element type was found ..
	        		if (elementType) {
	        			
	        			// Create a div that will contain the information about the element type.
		        		var elementTypeName = $("<div>");
		        		
		        		// Show the color of the element type of this text content document.
		        		elementTypeName.append($("<div>", {"class": "textcontent elementypecolor"}).css({
		        			"background-color" : elementType.color
		        		}));
		        		
		        		// Show the human readable string for the element type.
		        		elementTypeName.append($("<b>").text(elementType.elementHumanReadable));
		        		
		        		// Add it to the collumn.
		        		container.prepend(elementTypeName);
	        		}
	        	}.bind(this));
	        	
	        	// Create a div that will contain the information about the document name.
        		var documentName = $("<div>");
        		
        		// Create a link to the document.
        		var documentUrl = window.webcenterConfig.jspBaseUrl + 'docdetails.jsp?docVerID=' + this.dm.docVerID + '&amp;menu_file=' + DocTableXUtils.getMenuFileParamValue();
        		var documentLink = $("<a>", {
        			"target" : "_blank",
        			"class" : "docrefdoclink", 
        			"href" : documentUrl
        		}).text(this.dm.docName);
        		documentName.append(documentLink);
        		
        		// Add it to the container.
        		container.append(documentLink);
	        	return container;
	        	
			} else {
				// Document is not a text content document, render the normal document specification.
				var parentRenderMethod = TextContentRowView.__overridden.renderDocNameLink.bind(this);
				return parentRenderMethod();
			}
		};
		
		/**
		 * Render method for generating HTML for Master text column
		 * TODO: Move this to subclass implementaion once we have factory method to create class dynamically
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		this.renderMasterText = function(phElem, config) {
			//TODO: Move this to subclass implementation once we have factory method to create row classes dynamically
	   		this.renderTextContentData(phElem, true, config);
		};
		
		/**
		 * Render method for generating HTML for Translation text column
		 * TODO: Move this to subclass implementaion once we have factory method to create class dynamically
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		this.renderTranslationText = function(phElem, config) {
			//TODO: Move this to subclass implementation once we have factory method to create row classes dynamically
			this.renderTextContentData(phElem, false, config);
		};
		
		
		/**
		 * Method that renders the text content data for a language
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Boolean} isMaster
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
	    this.renderTextContentData = function (phElem, isMaster, config) {
	    	//TODO: Move this to subclass implementation once we have factory method to create row classes dynamically
	    	try {
	    		
	    		// Get the index of this collumn
	    		var columElem = $('<'+this.itemRootElemStr+'/>');
	    		phElem.append(columElem);
	    		var index = $(columElem).index();

	    		// Set a custom width if this was specified.
	        	if (config.width && config.width != "auto") {
	        		columElem.width(config.width);
	        	}
	    			    		
	    		// Based on the index of the collumn, get the language dropdown of the header.
	    		var selectElem = null;
	    		if (isMaster) {
	    			selectElem = this.dm.parentDocTable.view.languageMasterDropdownMap[index];
	    		} else {
	    			selectElem = this.dm.parentDocTable.view.languageTranslatedDropdownMap[index];
	    		}

	    		// Find out which language is selected in that dropdown.
	    		var selectedLanguage = null;
	    		if(selectElem){
	    			// If the selectElem has at least one option, take the selected one.
	    			if ($("option", selectElem).length > 0) {
	    				selectedLanguage = selectElem.val();
	    			} else {
	    				// No options in the selector (maybe header was hidden), choose the default one to display.
	    				selectedLanguage= config.typeDetail;
	    			}
	    		} else if (config.typeDetail && config.typeDetail.length > 0){
	    			// No selector available, if there's a default code to display, use that one.
	    			selectedLanguage= config.typeDetail;
	    		}

				//render exclamation mark if column cannot be rendered
	    		//Check doc type is loaded from server
	    		if( this.dm.docTypeID != '00001_0000000019'){
	    			this.addExclamationMessage(columElem, WCXML.xmlencode(DocRefStrings.t('DocRowApplicableForTextContentTypeOnly')));
	    			return;
	    		}
	    		// When no language is selected.
	    		if(!selectedLanguage){
	    			this.addExclamationMessage(columElem, WCXML.xmlencode(DocRefStrings.t('DocRowTextContentLangugeNotSelected')));
	    			return;
	    		}
	            
	    		// When no language is selected or the text content is not selected.
	    		if(selectedLanguage && this.dm.textContent){
	    			
	    			// When the text content has no element.
	    			if(!this.dm.textContent.element){
		    			this.addExclamationMessage(columElem, WCXML.xmlencode(DocRefStrings.t('DocRowApplicableForTextContentTypeOnly')));
		    			return;
		    		}
	    			
	    			// Find out the Document Version ID of the template or document this row represents.
	    			var rowDocVerID = this.dm.docVerID;
	    			if(!rowDocVerID && this.dm.selDocTemplate){
	    				rowDocVerID = this.dm.selDocTemplate.docVerID;
	    			}

	    			// Find the TextContentElement of this ContentStatement and language.
	    			var elemToRender = this.dm.textContent.provideElement(selectedLanguage, isMaster);

	    			
	    			var readOnly = false;

	    			// For Move, Copy and New doc. operations, check the target folder permissions. 
	    			// If we do not have these, mention that this operation is illegal and will fail.
	    			if (this.dm.docOperation === 'Move' || this.dm.docOperation === 'Copy' || this.dm.docOperation === 'New') {
	    				if(this.dm.selTargetFolder) {
	    					//For root folder id is '0'
	    					if( this.dm.selTargetFolder.id != '0' && !this.dm.selTargetFolder.checkoutPerm){
	    						readOnly = true;
	    					}
	    				}
	    			}

	    			// Get the correct Text Content widget for this collumn.
	    			var tcWidget = this.textContentWidgets[selectedLanguage + "_index" + index];
	    			if (!tcWidget) {
	    				
	    				// If no such text content widget could be found, create one for this column ...
	    				
	    				// Should we be able to edit this ContentStamentElement based on the configuration of this cell?
	        			readOnly = readOnly || config.visibility == 'ReadOnly' || !this.dm.isEditable;
	    				
	        			// Create the TextContentElementWidget for this column ...
	        			var options = {
	        					viewOptions: {
	        						editable: !readOnly,
	        						reviewable: !readOnly,
	        						widgetview: 'doctablex',
	        						maxChars : this.dm.textContent.getMaxChars()
	        					}	
	        			};
	    				tcWidget = new TextContentElementWidget(elemToRender, options);
	    				
	    				// When the text content element widget switches between single line and normal mode ...
	    				tcWidget.onSingleLineModeToggle(this.textContWidgetsToggleSingleLineMode.bind(this));
	    				
	    				// Add it to the map of text content widgets for this widget, this way we'll find it back later ...
	    				this.textContentWidgets[selectedLanguage + "_index" + index] = tcWidget;
	    				
	    			} else {
	    				// Update the TextContentWidget.
	    				tcWidget.update(elemToRender);
	    			}

	    			//Create text content section
	    			columElem.append(tcWidget.getJQElement());
	    			columElem.css("min-width","300px");
	    			
	    			//Notify the model whenever there is change in the widget
    				tcWidget.onChange(_.bind(function() {
    					this.dm.setIsChanged();
    				}, this));
    				
    				// Attempt to show the text content widget on a single line.
    				tcWidget.shownInPage();
    				var isSingleLineMode = tcWidget.isSingleLineMode();
    				
    				// Toggle single line mode for the other text content widgets based on wether it worked for this one. 
    				this.textContWidgetsToggleSingleLineMode(isSingleLineMode);
	    		}

	    	} catch (err) {
	    		DocTableXUtils.log('Problem in TextContentRowView.renderTextContentData()\n' + err);
	    	}
	    };
	    

	    /**
	     * Try to toggle wether all text content widgets in this row should be shown in single line mode or not.
	     */
		this.textContWidgetsToggleSingleLineMode = function(isSingleLineMode) {

			if (isSingleLineMode) {
				// We will try to toggle SingleLine mode for all widgets ...

				// Variable will indicate wether we succeeded.
				var success = true;

				// Iterate over all other widgets in this row ...
				for ( var key in this.textContentWidgets) {
					if (this.textContentWidgets.hasOwnProperty(key)) {
						var widget = this.textContentWidgets[key];
						// Try to display them in single line mode (if they're not already).
						if (!widget.trySingleLine() && !widget.isSingleLineMode()) {
							// Could not display them in single line mode ...
							success = false;
							break;
						} else {
							// Widget successfully toggled to single line ... continue with next.
						}

					}
				}
				// If we could not toggle single line mode for all, all show them again in normal mode
				if (!success) {
					// If the widget no longer is in single line mode ...
					// Iterate over all other widgets in this row ...
					for ( var key in this.textContentWidgets) {
						if (this.textContentWidgets.hasOwnProperty(key)) {
							var widget = this.textContentWidgets[key];
							// Display them also in normal mode.
							widget.setSingleLineMode(false);
						}
					}
				}
			} else {
				
				// We will try to toggle normal mode for all widgets ...
				
				// If the widget no longer is in single line mode ...
				// Iterate over all other widgets in this row ...
				for ( var key in this.textContentWidgets) {
					if (this.textContentWidgets.hasOwnProperty(key)) {
						var widget = this.textContentWidgets[key];
						// Display them also in normal mode.
						widget.setSingleLineMode(false);
					}
				}
				
			}
		};
		
		/**
		 * Method that adds an exclamation message
		 * 
		 * @param {jQuery
		 *            object} phElem - HTML DOM element, where to render the
		 *            content generated by this method.
		 * @param {String}
		 *            translMsg - Message to to shown
		 */
	    this.addExclamationMessage = function (phElem, translMsg) {
	    	var msgCodeSpan = $('<span style="color: #cccccc"/>');
	    	msgCodeSpan.text(translMsg);
	    	phElem.append(msgCodeSpan);
	    };
	    
	    
		/**
		 * Render method for generating HTML for text language column
		 * TODO: Move this to subclass implementaion once we have factory method to create class dynamically
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		this.renderTextLanguages = function(phElem, config) {

			// Get the index of this collumn
			var columElem = $('<'+this.itemRootElemStr+'/>');
			phElem.append(columElem);
			
			// Get the index of this collumn
			var index = $(columElem).index();
			
			// Set a custom width if this was specified.
        	if (config.width && config.width != "auto") {
        		columElem.width(config.width);
        	}
        	
        	// Create a div for the contents of this cell to make sure that it will obey this width. They may choose to ignore the width on the cell in order to fit so force them!
            var container = $(this.addStartOfCustomWidthDiv(config, '300px'));;            
            columElem.append(container);
			
			//Check if doc type is text content document 
			if(this.dm.docTypeID && this.dm.docTypeID != '00001_0000000019'){
				//render exclamation mark if column cannot be rendered
				this.addExclamationMessage(columElem, WCXML.xmlencode(DocRefStrings.t('DocRowApplicableForTextContentTypeOnly')));
				return;
			}
			
			// Fetch the text language indicator for this collumn.
			var textLangIndicator = this.textLangIndicatorMap[index];
			
			// Only continue if this row has a "contentStatement".
			if (!this.dm.textContent) {
				return;
			}
			
			// Check if the content has elements ...
			if(!this.dm.textContent.element){
    			this.addExclamationMessage(columElem, WCXML.xmlencode(DocRefStrings.t('DocRowApplicableForTextContentTypeOnly')));
    			return;
    		}
			
			if (!textLangIndicator) {
				// If no such language indicator does exist yet for this collumn, create one.
				
				// Construct the options for the language indicator based on the config:
				var options = {};
				if (config.displayLanguages == "translation") {
					options.showMaster = false;
					options.showTranslation = true;
				} else {
					options.showMaster = true;
					options.showTranslation = false;
				}
				
				// If this ContentStatement is invariant, do not show the Lang Indicator.
				// Parse whether this statement is invariant or not.
				var isInvariant = this.dm.textContent.invariant;
				isInvariant = (typeof isInvariant === "string") ? isInvariant === "true" : isInvariant;
				// If this statement is invariant and we need to display languages for the translations, do ...
				if (!options.showMaster && isInvariant) {
					// Show a message indicating that this statement is invariant and may not hold translations.
					var msg = $("<div>",{}).text("This statement is invariant.");
					columElem.append(msg);
					return;
				}
				
				// The language indicator should follow the languages of the table.
				if (config.displayLanguages == "translation") {
					options.getLanguages = function () {
						return this.dm.parentDocTable.getTranslationLanguages();
					}.bind(this);
				} else {
					options.getLanguages = function () {
						return this.dm.parentDocTable.getMasterLanguages();
					}.bind(this);
				}
				
				// Set the width.
				if (config.width && config.width != "auto") {
					options.width = config.width;
				}
				// Check if we should show the languages as compact.
				options.showCompact = config.langShowMode !== 'normal';
				// Check if we have the permissions to add/remove languages and whether they're configured to be editable.
				options.readOnly = config.visibility !== 'Editable' || !this.dm.textContent.canManage().allowed ;
				// Never show the label in the DocTableX
				options.showLabels = false;
				
				// The projectId of the current project.
				options.projectId = this.dm.parentDocTable.projectID;
				
				// Create the callback that should be executed when something changed (e.g. langadded or removed).
				options.changeCallback = _.bind(function() {
					this.dm.setIsChanged();
				}, this);
				
				// Create the language indicuator.
				textLangIndicator = new LanguageIndicator(this.dm.textContent, options.showMaster, options);
				
				// Save it in the map.
				this.textLangIndicatorMap[index] = textLangIndicator;
	    
			} else {
				// Language indicator does already exist, update it.
				
				// Before adding, update the languages in the indicator.
				textLangIndicator.setContentStatement(this.dm.textContent);
				textLangIndicator.update();
			}

			// add it to the collumn.
			container.append(textLangIndicator.getJQElement());
		};
		
	    /**
	     * Method to render column/item with a button to make the document in the corresponding row local to the current project.
	     * 
	     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
	     * @param {Object} config - Holds the documents table rendering configuration for this column/item.
	     */
	    if (this.renderLocalRemoteButton) { TextContentRowView.__overridden.renderLocalRemoteButton = this.renderLocalRemoteButton; }
	    this.renderLocalRemoteButton = function(phElem, config) {
	        try {
	    		if (!this.dm.parentDocTable.isEditable) {
	        		// don't render this column at all if the documents table is in read-only or static mode
	    			return;
	    		}

	            var htmlcode = '';
	        	htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow">';
	        	if (!this.dm.isInErrorStatus() && this.dm.getParentDocTable().hasLocalRemoteButton() && this.dm.isRemoteDocument()) {
	                if (this.dm.isEditable && this.dm.canUserAddDocuments() 
	                		&& (config.visibility != 'ReadOnly') && !this.dm.dependsOnItself) {
	                	if (!config.retainTextContentStatus) {
	                		config.retainTextContentStatus = "false";
	                	}
	                	// the local/remote button has an action associated only when the row/table is 
	                	// editable, the user has "Add" permission (can add documents in this project),
	                	// the Local/Remote button is configured to be "editable" (not just an info) and 
	                	// this is not a cyclic dependent document
	                	var clickMethod = 'javascript: ' + this.dm.getParentInDOM() + '.localRemoteButtonClicked(' + this.rowIndex + ',' + '\'LocalRemote\'' +  ', null, null, null,' + config.retainTextContentStatus + ');';
		                var translMsg = WCXML.xmlencode(DocRefStrings.t('DocRowLocalRemoteButtonTooltip'));
		                
		                htmlcode += '<button type="button" class="btn btn-xs btn-default actionBtn" title="'+translMsg +'" onclick="'+clickMethod+'">';
		                if (config.retainTextContentStatus === "true") {
		                	htmlcode += '<i class="icon icon-WCR_Actions-MakeLocal12" style="color: green;" />';
		                } else {
		                	htmlcode += '<i class="icon icon-WCR_Actions-MakeLocal12" />';
		                }
		                htmlcode += '</button>';
	                } else {
	                	var translMsg = WCXML.xmlencode(DocRefStrings.t('DocRowLocalRemoteInfoTooltip'));
		            	htmlcode += '<i class="icon icon-WCR_Actions-MakeLocal12" title="'+translMsg +'" />';
	                }  
	            }
	            htmlcode += '</' + this.itemRootElemStr + '>';
	            // render the prepared HTML code
	            phElem.append(htmlcode);
	        } catch (err) {
	            DocTableXUtils.log('Problem in StaticDocumentRowView.renderLocalRemoteButton:\n' + err);
	        }
	    };
	    
	    
	    /**
		 * Method called before the DocTableRow elements are thrown out of the DOM.
		 * 
		 * @param {boolean} destroyACatObj - 'true' if the attributeCategory object should be removed from 
		 * 		the memory (all changes user might have already done will be lost);
		 * 		'false'/<no value> when the attributeCategory object should be kept
		 */
	    if (this.destroy) { TextContentRowView.__overridden.destroy = this.destroy; }
	    this.destroy = function (destroyACatObj) {

	        // Call the super
	        TextContentRowView.__overridden.destroy(destroyACatObj);

	        if (!destroyACatObj) {
	            // Detach the text content widgets and language indicators from the DOM.  
	            if (this.textLangIndicatorMap) {
	                $.each(this.textLangIndicatorMap, function (id, textLangIndicator) {
	                    textLangIndicator.getJQElement().detach();
	                });
	            }

	            // Detach the text content widgets and language indicators from the DOM. 
	            if (this.textContentWidgets) {
	                $.each(this.textContentWidgets, function (id, textContentWidget) {
	                    textContentWidget.getJQElement().detach();
	                });
	            }

	        }
	    };
    };
    

    
	/**
	 * Mixin constructor/initializer.
	 * (class method)
	 */ 
    TextContentRowView.__initMixin = function(options) {
		if (!options) {
			// nothing to initialize from
			return this;
		}

		// extensions of renderMethodMap member
        this.renderMethodMap['MasterText'] = this.renderMasterText;
        this.renderMethodMap['TranslationText'] = this.renderTranslationText;
        this.renderMethodMap['TextLanguages'] = this.renderTextLanguages;

        /** {Object} textLangIndicatorMap - Stores the Text Language Indicators to make it available to 
         * 		this row. They're mapped in this object on the index of their containing column. */
		this.textLangIndicatorMap = {};
        
		/** {Object} textContentWidgets - Stores the TextContentElementWidget widgets that display 
		 * 		master/translations texts of this row. */
		this.textContentWidgets = {};
		
		return this;
	};


	return TextContentRowView;
});