define(["jquery",
        "underscore",
        "doctablex/DocTableXUtils",
        "doctablex/AJAXDataCache",
        "utils/webi18n!TextContentStrings",
		"packaging-content-management/ContentConstants",
        "wcr/webcenter_xml",
        "packaging-content-management/PCMEventBus"],
function ($,
    _,
    DocTableXUtils,
    AJAXDataCache,
    webi18n,
    ContentConstants,
    WCXML,
    pcmEventBus) {

    /**
     * TextContentTableXView.js
     *
     * This class implements rendering methods for documents table when in Text Content render mode.
     *
     * @author JANH - February 2015
     */

    "use strict";


    var TextContentTableXView = function() {

		this.__isMixin = true;   //JANH debug then DEL!!!

		
	    this.sortOption = function ($select) {
	    	$select.append($select.find("option").remove().sort(function(a, b) {
	    	    var at = $(a).text(), bt = $(b).text();
	    	    return (at > bt)?1:((at < bt)?-1:0);
	    	}));
	    };
	    
	
	    /**
	     * Populates master and translation language drop down in the table header
	     */
	    this.updateLanguageDropDowns = function (masterLangToSelect, transLangToSelect) {
	    	if (this.dm._masterLangs) {
	    		//Fill Master language dropdown
		    	this._addLangsToDropDowns(this.dm._masterLangs, this.languageMasterDropdownMap, masterLangToSelect);
	    	}
	    	if (this.dm._transLangs) {
				//Fill translation language dropdown
		    	this._addLangsToDropDowns(this.dm._transLangs, this.languageTranslatedDropdownMap, transLangToSelect);
	    	}
	    };
	
	    
	    /**
	     * 
	     * @param langs
	     * @param languageDropdownMap
	     * @param langToSelect - language code
	     */
	    this._addLangsToDropDowns = function (langs, languageDropdownMap, langToSelect) {
	    	
			for ( var colIndex in languageDropdownMap) {
				if (languageDropdownMap.hasOwnProperty(colIndex)) {
	
					var select = languageDropdownMap[colIndex];
					
					// Get the current selected language before removing the options
					var currentSelectedLang = select.val();
					
					// Select empty.
					select.empty();
					
	    			// Create a function that nicely sorts the languages ...
					var sortLangsByDesc = function(lang1, lang2) {
						var lang1Desc = lang1.langDesc.toLowerCase();
						var lang2Desc = lang2.langDesc.toLowerCase();
						return ((lang1Desc < lang2Desc) ? -1 : ((lang1Desc > lang2Desc) ? 1	: 0));
					};
					langs.sort(sortLangsByDesc);
	    			
	    			//Populate the language dropdown
					for (var i=0; i < langs.length; i++) {
	    				select.append(new Option(langs[i].langDesc, langs[i].langCode));
	    			}	
	    			
	    			if(langToSelect) {
	    				select.attr("data-default-langcode", langToSelect);
	    			} else if(currentSelectedLang) {
	    				select.attr("data-default-langcode", currentSelectedLang);
	    			}
	    			
	    			// Select the default if available.
	    			var defLangCode = select.attr("data-default-langcode");
	    			select.find('option[value="'+defLangCode+'"]').attr('selected', 'selected');
	    			select.change();
	    			
	    			// Show the dropdown
	    			select.show();
				}
			}
	    	
	    };
	    
		/**
		 * Table header for Master Language
		 */
		this.renderMasterTextHeader = function(phElem, config) {
			this._renderTextHeader(phElem, config, this.languageMasterDropdownMap, this.dm._masterLangs);
		};
		
		/**
		 * Table header for Translation Language
		 */
		this.renderTranslationTextHeader = function(phElem, config) {
			this._renderTextHeader(phElem, config, this.languageTranslatedDropdownMap, this.dm._transLangs);
		};
		
		/**
		 * Render text languages header, render the header for a text language collumn.
		 * 
		 * @param phElem The element in which the header should be rendered.
		 * @param config The configuration of this collumn.
		 * @param dropdownMap The map that keeps track of possible dropdowns to show in this header.
		 */
		this._renderTextHeader = function(phElem, config, dropdownMap, langs) {
			// Create the header element
			var thElem = $('<th/>').addClass('docrefdoccol');
			phElem.append(thElem);
			// Add the correct label.
			thElem.append('<span>' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</span>&nbsp;&nbsp;');
			// Get the appropiate dropdown for this collumn.
			var dropdown = dropdownMap[thElem.index()];
			// If the dropdown does not yet exist ...
			if (!dropdown) {
				//Set margin to reduce the Doctablex header row height for text content table
				// Create it ...
				dropdown = $('<select/>',{"class" : "text-lang-selector"}).css("margin", "0px 5px").attr("data-default-langcode",config.typeDetail).hide();
				dropdownMap[thElem.index()] = dropdown;
				// Fill it with the available language options.
				this._addLangsToDropDowns(langs, dropdownMap);
			}
			
			//Fix for IE browser:: When select element is moved in DOM, options are not available
			//Hence re-add options in the select element
			if ($("option", dropdown).length < langs.length) {
				// Fill it with the available language options.
				this._addLangsToDropDowns(langs, dropdownMap);
			}
			
			// Add the dropdown
			thElem.append(dropdown);
			dropdown.change(_.bind(function(e){
				this.repaintDocRows();
			}, this));
						
		};
		
		/**
		 * Table header for Text Languages
		 */
		this.renderTextLanguagesHeader = function(phElem, config) {
			
			var thElem = $('<th/>').addClass('docrefdoccol');
			phElem.append(thElem);
			thElem.append('<span>' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</span>');
		};
		
		
/*//JANH these are not necessary as master and translation texts are not renedered at all when there is no PCM license		
		// UTGO:: Here only create placeholder element. 
		DocTableXView.prototype.renderMasterTextHeader = function(phElem, config) {
			// Actual rendering with language selector should happen in the subclass TextContentTableXView
			var thElem = $('<th/>').addClass('docrefdoccol');
			phElem.append(thElem);
			thElem.append('<span>' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</span>&nbsp;&nbsp;');
		};

		// UTGO:: Here only create placeholder element
		DocTableXView.prototype.renderTranslationTextHeader = function(phElem, config) {
			// Actual rendering with language selector should happen in the subclass TextContentTableXView
			var thElem = $('<th/>').addClass('docrefdoccol');
			phElem.append(thElem);
			thElem.append('<span>' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</span>&nbsp;&nbsp;');
		};
*/

	};

	
	/**
	 * Mixin constructor/initializer.
	 * (class method)
	 */ 
	TextContentTableXView.__initMixin = function(options) {
    	if (!options) {
    		// nothing to initialize from
    		return this;
    	}

    	// extend the mapping of rendering methods (by text content specific methods)
    	this.renderMethodMap['MasterText'] = this.renderMasterTextHeader;
    	this.renderMethodMap['TranslationText'] = this.renderTranslationTextHeader;
    	this.renderMethodMap['TextLanguages'] = this.renderTextLanguagesHeader;

    	
    	// Stores the Language dropdown element to make it available to rows.
    	this.languageMasterDropdownMap = {};
    	this.languageTranslatedDropdownMap = {};

    	// by default make green links appear at top
    	this.renderOptions.showGreenLinksOnTop = true;
    	this.renderOptions.showGreenLinksAtBottom = false;

    	//The languageBundle necessary for this component.
    	this.i18n = webi18n;
	};

	
	/**
	 * List of overridden methods with their original code.
	 */ 
	TextContentTableXView.__overridden = {
	};

	
    return TextContentTableXView;
});
