/**
 *
 * For JavaScript API and SmartName support in ace editor
 * 
 * Modification to the ace framework.
 *
 * Modifies AutoCompletion for inserting Object Methods.
 *
 * @author KWZA
 */
define(["ace/autocomplete", "ace/autocomplete/util"], function(autocomplete, util){
    
	"use strict";
	/**
     * New retrievePrecedingIdentifier function in case of SmartNames...
     * Tells AutoCompletion to not stop autocompleting when a '.' is found
     */
    var retrievePrecedingIdentifierModification = function(text, pos, regex, isSmartNamesEnabled) {
		if(!isSmartNamesEnabled) {
			regex = /[a-zA-Z_0-9$.[]/;  //CODE MOD
			var buf = [];
			for (var i = pos-1; i >= 0; i--) {
				if (regex.test(text[i])){
					buf.push(text[i]);
				}
				else{
					break;
				}

			}
			return buf.reverse().join("");
		}else{
			var lastIndex = text.lastIndexOf('[',pos);
			if(lastIndex === -1){
				return "";
			}
			return text.substring(lastIndex,pos);
		}
		return result;
    };
    
    /**
     * Override the way to match prefixes in AutoComplete... (used by both autocomplete and language_tools)
     */
    util.retrievePrecedingIdentifier = retrievePrecedingIdentifierModification;
	
	/**
	 * This function has to be updated to only start autocompletion in smartname input fields 
	 * when the user types a '['
	 * @param   {Editor}   editor 
	 * @returns {string} the prefix
	 */
	var getCompletionPrefixModification = function (editor) {
		var pos = editor.getCursorPosition();
		var line = editor.session.getLine(pos.row);
		return this.retrievePrecedingIdentifier(line, pos.column,undefined, editor.session.isSmartNamesEnabled()); // CODE MOD
	};
	
	util.getCompletionPrefix = getCompletionPrefixModification;
	
	(function(){
        /**
         * Override the function insertMatch in AutoCompletion
         *
         * Change 1: When there is no option selected in the insert menu [enter] will insert the first match
         *  -> we might want to make this optional in ace because it might be a problem when using it with multiple lines...
         *  -> with multiple lines you don't always want to implement a match when pressing [enter]
         *     [TODO] MAKE OPTIONAL, only do this when this editor has SmartNames editing enabled
         * 
         * Change 2: 
         *  -> When the match we want to insert has a smartname variable, we insert the SmartName.
         */
        this.insertMatch = function(data) {
            if (!data)
                data = this.popup.getData(this.popup.getRow());
            if (!data) { // CODE MOD
                data = this.popup.getData(0);// CODE MOD
                if(!data)
                    return false;
            }

            if (data.completer && data.completer.insertMatch) {
                data.completer.insertMatch(this.editor, data);
            } else {
                if (this.completions.filterText) {
                    var ranges = this.editor.selection.getAllRanges();
                    for (var i = 0, range; range = ranges[i]; i++) {
                        range.start.column -= this.completions.filterText.length;
                        this.editor.session.remove(range);
                    }
                }
                if (data.snippet)
                    snippetManager.insertSnippet(this.editor, data.snippet);
                else if(data.smartname)
					this.editor.insertSmartName(data.smartname);// CODE MOD
                else
                    this.editor.execCommand("insertstring", data.value || data);
            }
            this.detach();
        };
        
        /**
         * Removes the action on "Space" in AutoCompletion
         * 
         * When pressing space, do not close the popup. (SmartNames can contain names)
         *
         * [TODO] MAKE OPTIONAL, only do this when this editor has SmartNames editing enabled
         */
        delete this.commands["Space"];
    }).call(autocomplete.Autocomplete.prototype);
    
});