/**
 *
 * For SmartName support in ace editor
 * 
 * Modification to the ace framework.
 *
 * Modifies Document for undo/redo support for SmartNames.
 *
 * @author JEPE
 */
define(["ace/document", "ace/range", "ace/apply_delta", "./smartnameanchorcontrol", "./smartnameextensionutils"], function(document, range, applyDelta,smartnameanchorcontrol, smartnameextensionutils) {
    var Range = range.Range;

    
    // function to call to do undo/redo on SmartNameAnchors.
    var applySmartNameDelta = smartnameanchorcontrol.undoRedo;
    
    // function to call to notify SmartNameAnchors of changes in the document
    var pushSmartNamesChange = smartnameanchorcontrol.applyDelta;
    
    /*
     * Override/extend functionality in document... for undo/redo support
     */
    (function(){
		
		
		/**
         * Are SmartNames enabled in this input field?
         *
         * @returns {Boolean} are SmartNames enabled in this document?
         */
        this.isSmartNamesEnabled = function() {
            if(this._smartNamesEnabled===undefined){
                return false;
            }else{
                return this._smartNamesEnabled;
            }
        };

        /**
         * Enable the SmartNames in this document will be set by the session
         * 
         * @param {SmartNameEncoding} the encoding to use
         */
        this.enableSmartNames = function (){
			this._smartNamesEnabled = true;
        };
		
		
		/**
		 * Applies `delta` to the document. Extended this method to work with smartnames
		 * 
		 **/
		this.applyDelta = function(delta, doNotValidate) {
			var isInsert = delta.action == "insert";
			var isRemove = delta.action == "remove"; 	// CODE MOD
			
			// An empty range is a NOOP.
			if (isInsert ? delta.lines.length <= 1 && !delta.lines[0]
				: !Range.comparePoints(delta.start, delta.end)) {
				return;
			}

			if (isInsert && delta.lines.length > 20000)
				this.$splitAndapplyLargeDelta(delta, 20000);
			if(!isInsert && !isRemove){ //CODE MOD
				var undo = typeof delta.undo !== "undefined" && delta.undo; //CODE MOD
				applySmartNameDelta(this,delta,undo); //CODE MOD
				return;
			}else{
				if(this.isSmartNamesEnabled()){
					smartnameextensionutils.startSmartNameChangeGroup(this);	//CODE MOD	
					// Apply.
					applyDelta.applyDelta(this.$lines, delta, doNotValidate);	//CODE MOD
					pushSmartNamesChange(this, {action: delta.action, start: delta.start, end: delta.end});	//CODE MOD
					smartnameextensionutils.stopSmartNameChangeGroup(this, { start : delta.start, end: delta.end});	//CODE MOD
				}else{
					applyDelta.applyDelta(this.$lines, delta, doNotValidate);
				}
				
				
			}

			
			this._signal("change", delta);
		};

		/**
		 * Reverts `delta` from the document.
		 * @param {Object} delta A delta object (can include "insert" and "remove" actions)
		 **/
		this.revertDelta = function(delta) {
			
			var action = delta.action;     		//CODE MOD
			if(delta.action === "insert"){		//CODE MOD
				action = "remove";				//CODE MOD
			}else if(delta.action === "remove"){		//CODE MOD
				action = "insert";				//CODE MOD
			}									//CODE MOD								
			
			var resultDelta = {start: this.clonePos(delta.start), //CODE MOD
				end: this.clonePos(delta.end),  //CODE MOD
				action: action,					//CODE MOD
				undo: true						//CODE MOD
			};
			//Check if the current delta contains a smartname if it does => append it to the delta
			if(typeof delta.smartname !== "undefined"){		//CODE MOD
				resultDelta.smartname = delta.smartname;	//CODE MOD
			}
			if(action === "insert" || action == "remove"){	//CODE MOD
				resultDelta.lines =  delta.lines.slice();	//CODE MOD
			}
			
			this.applyDelta(resultDelta);
		};
    }).call(document.Document.prototype);

});