/**
 *
 * For SmartName support in ace editor
 * 
 * Modification to the ace framework.
 *
 * Modifies the Text Layer (rendering) to render the SmartName character (yes, it is a single character) as multiple characters...
 *
 * @author JEPE
 */
define(["ace/layer/text", "ace/lib/lang", "./smartnameanchorcontrol", "./smartnameextensionutils", "utils/ConvertUtils"], function(text, lang, smartnameanchorcontrol, smartnameextensionutils, ConvertUtils) {
    (function(){
        /**
         * Overrides the method $renderToken in ace/layer/text to render SmartNames correctly
         * (They are processed in the back-end as one character)
         */
        this.$renderToken = function(stringBuilder, screenColumn, token, value) {
            var self = this;
            // CODE MOD NOTE: with \x10 is SMARTNAMECHAR
            var replaceReg = /\x10|\t|&|<|( +)|([\x00-\x1f\x80-\xa0\xad\u1680\u180E\u2000-\u200f\u2028\u2029\u202F\u205F\u3000\uFEFF])|[\u1100-\u115F\u11A3-\u11A7\u11FA-\u11FF\u2329-\u232A\u2E80-\u2E99\u2E9B-\u2EF3\u2F00-\u2FD5\u2FF0-\u2FFB\u3000-\u303E\u3041-\u3096\u3099-\u30FF\u3105-\u312D\u3131-\u318E\u3190-\u31BA\u31C0-\u31E3\u31F0-\u321E\u3220-\u3247\u3250-\u32FE\u3300-\u4DBF\u4E00-\uA48C\uA490-\uA4C6\uA960-\uA97C\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFAFF\uFE10-\uFE19\uFE30-\uFE52\uFE54-\uFE66\uFE68-\uFE6B\uFF01-\uFF60\uFFE0-\uFFE6]/g;
            var replaceFunc = function(c, a, b, tabIdx, idx4) {
                //console.log("("+c+","+a+","+b+","+tabIdx+","+idx4+")");
                if (a) {
                    return self.showInvisibles ?
                        "<span class='ace_invisible ace_invisible_space'>" + lang.stringRepeat(self.SPACE_CHAR, c.length) + "</span>" :
                        lang.stringRepeat("\xa0", c.length);
                } else if (c == "&") {
                    return "&#38;";
                } else if (c == "<") {
                    return "&#60;";
                } else if (c == "\t") {
                    var tabSize = self.session.getScreenTabSize(screenColumn + tabIdx);
                    screenColumn += tabSize - 1;
                    return self.$tabStrings[tabSize];
                } else if (c == "\u3000") {
                    var classToUse = self.showInvisibles ? "ace_cjk ace_invisible ace_invisible_space" : "ace_cjk";
                    var space = self.showInvisibles ? self.SPACE_CHAR : "";
                    screenColumn += 1;
                    return "<span class='" + classToUse + "' style='width:" +
                        (self.config.characterWidth * 2) +
                        "px'>" + space + "</span>";
                } else if (b) {
                    return "<span class='ace_invisible ace_invisible_space ace_invalid'>" + self.SPACE_CHAR + "</span>";
                } else {
                    screenColumn += 1;
                    return "<span class='ace_cjk' style='width:" +
                        (self.config.characterWidth * 2) +
                        "px'>" + c + "</span>";
                }
            };
            
            // CODE MOD SmartName character
            var replaceFuncWithSmartNames = function(c, a, b, tabIdx, idx4) {
            	if (c == smartnameextensionutils.SMARTNAMECHAR && self.session.isSmartNamesEnabled()) {
                    // Get information about the SmartName:

                    // NOTE: not correct in case we allow multiple lines in a SmartField edit field or if wrapping is enabled... 
                    // In that case: row!=0 OR column != column in document.
                    // (!) Will probably need to modify ace framework if we want to support that (three or more small changes to pass the row to this method)
                    var pos = self.session.screenToDocumentPosition(0, screenColumn + tabIdx);
                    
                    var info = smartnameanchorcontrol.getSmartNameInfoAt(self.session.getDocument(), pos.row, pos.column);
                    var activePosition = smartnameanchorcontrol.getActiveSmartNamePosition(self.session.getDocument());
                    var activeSN = (activePosition!=null && pos.row === activePosition.row && pos.column === activePosition.column);
                    var display = smartnameextensionutils.getSmartNameDisplayString(self.session.getDocument(), info);

                    screenColumn += display.length-1;
                    
                    var displayoutput = display.replace(replaceReg, replaceFunc);
                    
                    return "<span class='ace_smartname"+(activeSN?" ace_smartname_active":"")+"'>"+displayoutput+"</span>";
                }else{
                	return replaceFunc(c, a, b, tabIdx, idx4);
                }
            };

            var output = value.replace(replaceReg, replaceFuncWithSmartNames);

            if (!this.$textToken[token.type]) {
                var classes = "ace_" + token.type.replace(/\./g, " ace_");
                var style = "";
                if (token.type == "fold")
                    style = " style='width:" + (token.value.length * this.config.characterWidth) + "px;' ";
                stringBuilder.push("<span class='", classes, "'", style, ">", output, "</span>");
            }
            else {
                stringBuilder.push(output);
            }
            return screenColumn + value.length;
        };
    }).call(text.Text.prototype);

});