/**
 *
 * For SmartName support in ace editor
 *
 * Changes were needed to correctly copy the smartname in the deltas
 *
 * @author KWZA
 */
define([
	"ace/undomanager", 
	], 
	function(undomanager) {
	
	 /*
     * Override/extend functionality in undomanager... for undo/redo smartname support
     */
	(function(){
		
		// Serializes deltaSets to reduce memory usage.
		this.$serializeDeltas = function(deltaSets) {
			return cloneDeltaSetsObj(deltaSets, $serializeDelta);
		};

		// Deserializes deltaSets to allow application to the document.
		this.$deserializeDeltas = function(deltaSets) {
			return cloneDeltaSetsObj(deltaSets, $deserializeDelta);
		};

		/**  
		 * Updated this function to also copy the smartname property
		 * @param   {AceDelta} delta 
		 * @returns {AceDelta} 
		 */
		function $serializeDelta(delta){
			return {
				action: delta.action,
				start: delta.start,
				end: delta.end,
				lines: delta.lines.length == 1 ? null : delta.lines,
				text: delta.lines.length == 1 ? delta.lines[0] : null,
				smartname: delta.smartname				//CODE MOD
			};
		}
		
		/**  
		 * Updated this function to also copy the smartname property
		 * @param   {AceDelta} delta 
		 * @returns {AceDelta} 
		 */
		function $deserializeDelta(delta) {
			return {
				action: delta.action,
				start: delta.start,
				end: delta.end,
				lines: delta.lines || [delta.text],
				smartname: delta.smartname 			//CODE MOD
			};
		}
		
		/**
		 * This method is an exact copy of cloneDeltaSetsObj (found in ace/undomanager)
		 * the copy was needed because this is a private method
		 */
		function cloneDeltaSetsObj(deltaSets_old, fnGetModifiedDelta) {
			var deltaSets_new = new Array(deltaSets_old.length);
			for (var i = 0; i < deltaSets_old.length; i++) {
				var deltaSet_old = deltaSets_old[i];
				var deltaSet_new = { group: deltaSet_old.group, deltas: new Array(deltaSet_old.length)};

				for (var j = 0; j < deltaSet_old.deltas.length; j++) {
					var delta_old = deltaSet_old.deltas[j];
					deltaSet_new.deltas[j] = fnGetModifiedDelta(delta_old);
				}

				deltaSets_new[i] = deltaSet_new;
			}
			return deltaSets_new;
		}
		
	} ).call(undomanager.UndoManager.prototype);
});