define(["jquery",
		"packaging-content-management/content/SingleTextContent",
		"packaging-content-management/dyn-content/TextDynContent",
		"packaging-content-management/dyn-content/TextDynContentLocale"
	],
	function ($,
		SingleTextContent,
		TextDynContent,
		TextDynContentLocale) {



		/**
		 * 
		 */
		var DynContentService = function () {

			this._isFetchingDynContentFor = {};

			this._dynContentsByDocVerID = [];
			
			this._dynContentLocaleByDocVerID = [];
		};

		/**
		 * 
		 * @param docVersionId
		 */
		DynContentService.prototype.getDynContentFor = function (docVersionId, forceUpdate) {

			// force the update of the cache
			if (forceUpdate) this._dynContentsByDocVerID[docVersionId] = null;

			var cacheItem = this._dynContentsByDocVerID[docVersionId];

			if (!cacheItem) {
				// create a new cache item for the data to be loaded
				cacheItem = {};
				this._dynContentsByDocVerID[docVersionId] = cacheItem;
			}

			if (!cacheItem.request) {
				// create a new Deferred object representing the result of this method (not only the AJAX call!)
				cacheItem.request = $.Deferred();
			} else {
				// return the Promise object in the cache (representing the request), which already contains 
				// the result or is waiting for the executed AJAX call to finish
				return cacheItem.request.promise();
			}

			// The parameters of the call
			var params = {
				"ajax": "1",
				"docverid": docVersionId
			};
			// The baseUrl of the service.
			var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
			var url = jspBaseUrl + "GetDynContent.jsp?json=1&" + $.param(params, true);

			// Get the ajax JSON.
			$.getJSON(url, function (json) {

				// json = {
				//			textDynContents: [
				//				{
				//					contentStatement: <json>,
				//					locations: {
				//						fr-FR: [...],
				//						en-US: [...],
				//						...
				//					}
				//				},
				//				{
				//					...
				//				}, ...
				//			];
				// }

				var dynContents = [];
				if (json) {
					// Parse and generate the text dynamic content objects. 
					for (var i = 0; i < json.length; i++) {
						var contentStatement = new SingleTextContent(json[i]);
						for (var j = 0; j < contentStatement.element.length; j++) {
							var textElement = contentStatement.element[j];
							var locations = [];
							if (textElement.locationList) {
								locations = textElement.locationList;
							}
							var textDynContent = new TextDynContent(textElement, locations);
							dynContents.push(textDynContent);
						}
					}
				}
				// return the parsed value for the request.done() method.
				cacheItem.request.resolve(dynContents);


			}.bind(this)).fail(function (jqXHR, textStatus, errorThrown) {
				var errorXml = $.parseXML(jqXHR.responseText);
				$xmlErr = $(errorXml);
				$code = $xmlErr.find( "code" );
				// check for error - if attribute category is not set, or AC doesnt contain any text content documents, just return 
				if($code.text() == "CONTENT_NO_ATTR_EXIST"){
					return;
				}
				
				//Return with generic message, caller can show customized message
				cacheItem.request.reject("Unable to get Text Contents Documents");
			});

			// value not yet prepared in the cache (the loading is in progress) - return a Promise object
			// to be able to register callbacks in the caller(s)
			return cacheItem.request.promise();
		};
		
		/**
		 * Get locale information for the artwork
		 * @param docVersionId - artwork docversion id
		 * @returns
		 */
		DynContentService.prototype.getDynContentLocale = function (docVersionId) {
			var cacheItem = this._dynContentLocaleByDocVerID[docVersionId];

			if (!cacheItem) {
				// create a new cache item for the data to be loaded
				cacheItem = {};
				this._dynContentLocaleByDocVerID[docVersionId] = cacheItem;
			}

			if (!cacheItem.request) {
				// create a new Deferred object representing the result of this method (not only the AJAX call!)
				cacheItem.request = $.Deferred();
			} else {
				// return the Promise object in the cache (representing the request), which already contains 
				// the result or is waiting for the executed AJAX call to finish
				return cacheItem.request.promise();
			}

			// The parameters of the call
			var params = {
				"ajax": "1",
				"artworkdocVersionId": docVersionId
			};
			// The baseUrl of the service.
			var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
			var url = jspBaseUrl + "GetDynContentLocale.jsp?json=1&" + $.param(params, true);

			// Get the ajax JSON.
			$.getJSON(url, function (json) {

			//			json =	{
			//					"WEBSITE_1": {
			//						"localeLocationList": {
			//							"3": [{
			//								"orgx": ..,
			//								"orgy": ..,
			//								"sizex": ..,
			//								"sizey": ..,
			//								"rotation": ..,
			//								"orientation": ..
			//							}],
			//							"2": [{
			//								"orgx": ..,
			//								"orgy": ..,
			//								"sizex": ..,
			//								"sizey": ..,
			//								"rotation": ..,
			//								"orientation": ..
			//							}]
			//						},
			//						"elementType": "WEBSITE",
			//						"instanceId": "1"
			//					}
			//			}

				var dynContentLocales = [];
				if (json) {
					// Parse and generate the text dynamic content objects. 
					$.each(json, _.bind(function(key, element){
						$.each(element.localeLocationList, _.bind(function(locale, locationInfo){							
							var dynContentLocale = new TextDynContentLocale(key, element.elementType, element.instanceId, locationInfo,
									locale);
							dynContentLocales.push(dynContentLocale);							
						}, this));
					}, this));					
					
				}
				// return the parsed value for the request.done() method.
				cacheItem.request.resolve(dynContentLocales);


			}.bind(this)).fail(function (jqXHR, textStatus, errorThrown) {
				//Return with generic message, caller can show customized message
				cacheItem.request.reject("Unable to get Locale Information");
			});

			// value not yet prepared in the cache (the loading is in progress) - return a Promise object
			// to be able to register callbacks in the caller(s)
			return cacheItem.request.promise();
		};

		return new DynContentService();
	});
