define(["underscore",
        "jquery",
        "../view/AddTableElementWidgetView",
        "packaging-content-management/ContentConstants",
        "packaging-content-management/text-content-services/tableElementTypeService"],
	function (_,
			$,
			AddTableElementWidgetView,
			ContentConstants,
			tableElementTypeService) {
	
		"use strict";
		
		var AddTableElementWidget = function (isMaster, completeElementsList, elementsAlreadyAdded, masterElementsList, tableContentPage) {
			
			this._isMaster = isMaster;
			this._completeElementsList = completeElementsList;
			this._elementsAlreadyAdded = elementsAlreadyAdded;
			this._masterElementsList = masterElementsList;
			this._tableContentPage = tableContentPage;
			this._selectedPreviewName = null;
			this._previewElementsList = [];
			
			if(this._isMaster){
				this._tableContentPage.getPreviewElements().done(_.bind(function(previewElementEntries){
					this._previewElementsList = previewElementEntries;
					if(this._previewElementsList.length > 0){
						this._selectedPreviewName = this._tableContentPage.getPreviewName();
					}
				}, this));
			}
			
			this._selectedFilterList = [];			// Filter list on right hand side - in 1st and 2nd option
			this._previewFilterList = [];			// Filter list on left hand side - in 1st option
			this._libraryFilterList = [];			// Filter list on left hand side - in 2nd option
			this._masterFilterList = [];
			
			this._addRemoveElementsList = [];		// List of newly added elements
			
			this._initializeFilterLists();
			
			this._view = new AddTableElementWidgetView(this._selectedFilterList, this._previewFilterList, this._libraryFilterList, this._masterFilterList, this._selectedPreviewName, this._isMaster);	
			
			this._view.onAddCustomField(_.bind(function (customElementType, instanceId) {
				if (this._canAddNewElementType(customElementType , instanceId)) {
					this._addRemoveElementsList.push({
						id : customElementType +"_"+instanceId,
						elementType : customElementType,
						instanceId : instanceId,
						action : ContentConstants.LanguageAction.ADD
					});
					return true;
				}else{
					return false;
				}
			}, this));
			
			// What should happen when an entry is selected in 1st and 2nd options
			this._view.onEntrySelected(_.bind(function (selectedId) {				
				return this._addSelectedElement(selectedId);
			}, this));
			
			// What should happend when we deselect an entry, this._addRemoveElementList should be updated to remove
			// that element
			this._view.onEntryDeselected(_.bind(function (id) {
				var elementType = id.substring(0, id.lastIndexOf("_"));
				var instanceId = id.substring(id.lastIndexOf("_")+1);
				this._removeElementOnDeselect(elementType, instanceId);
			}, this));
			
			this._view.onCancel(_.bind(function(){
				// When the modal is cancelled, doesnt want his changes to be applied, so lets create the updated list before closing the modal
				this.clearAddRemoveElementsList();
			}, this));			
		};	
		
		/**
		 * Add the entry selected to this._addRemoveElementsList
		 */
		AddTableElementWidget.prototype._addSelectedElement = function(selectedId){
			// Add the entry to this._addRemoveElementsList array, which will be used after we close the popup
			var elementType = selectedId.substring(0, selectedId.lastIndexOf("_"));
			var instanceId = selectedId.substring(selectedId.lastIndexOf("_")+1);
			
			// Calculate new instance Id
			var newInstanceId = this._calculateNewId(elementType, instanceId);
			var newId = elementType+"_"+newInstanceId;
			var humanReadableElementType =  undefined;
			tableElementTypeService.getReadableElementType(elementType, function (humanReadableFormElementType) {
				humanReadableElementType = humanReadableFormElementType.hasOwnProperty("tableElementHumanReadable") ? humanReadableFormElementType.tableElementHumanReadable :
					humanReadableFormElementType;
			});
			var elementDetails = {};
			elementDetails.id = newId;
			elementDetails.elementType = elementType;
			elementDetails.instanceId = newInstanceId;
			elementDetails.action = ContentConstants.LanguageAction.ADD;
			elementDetails.humanReadableElementType = humanReadableElementType;
			
			var labelField = (newInstanceId == "1")  ?  humanReadableElementType : humanReadableElementType + " (" + newInstanceId + ")";
			elementDetails.label = labelField;
			
			// Check the element is marked for adding was marked for deleted. If yes, we will only update the _addRemoveElementsList to remove that element
			var indexOfDeleteElement = this._isElementMarkedForDeleted(elementType, newInstanceId);
			if(indexOfDeleteElement == -1){
				this._addRemoveElementsList.push(elementDetails);
			}else{
				newId = selectedId;
				newInstanceId = instanceId;
				this._addRemoveElementsList.splice(indexOfDeleteElement, 1);					
			}
			
			// Update selected flag for element in selected elements list, if already available.
			var addInSelectedElement = true;
			$.each(this._selectedFilterList, _.bind(function (index, selectedElement) {
				if (selectedElement.id == newId) {
					selectedElement.selected = true;
					addInSelectedElement = false;
					return false;
				}
			}, this));
			
			// In case element is not available, then add element in selected elements list.
			if (addInSelectedElement) {
				this._selectedFilterList.push({
	                id: newId,
	                label: labelField,
	                selected: true,
	                oldOrNew: "new"
	            });
			}
			
			return elementDetails;
		};
		
		/**
		 * Initialize all the filter Lists required to prepare the view
		 */
		AddTableElementWidget.prototype._initializeFilterLists = function(){
			// Prepare sorted list of preview element types that will act as gallery for multi-select list - 1st option	
			this._processPreviewListElements();
			
			// Prepare sorted list of complete element types that will used for multi-select list - Second option
			$.each(this._completeElementsList, _.bind(function(key, elementType){
				var id = elementType.instanceId ? key : key+"_1";
				this._libraryFilterList.push({
		                id: id,
		                label: elementType.tableElementHumanReadable,
		                selected: false
		            });
			}, this));
			
			this._libraryFilterList = _.sortBy(this._libraryFilterList, 'label');	// Sort the list based on label field
			
			// Prepare sorted list of selected element types that will be used for multi-select list - Second option
			$.each(this._elementsAlreadyAdded, _.bind(function(key, element){
				
				if(!element.isDeleted()){
					var instanceId = element.parentStatement.instanceId ? element.parentStatement.instanceId : "1";
					var humanReadableElementType =  undefined;
					tableElementTypeService.getReadableElementType(element.parentStatement.elementType, _.bind(function(humanReadableFormElementType){
						humanReadableElementType = humanReadableFormElementType.hasOwnProperty("tableElementHumanReadable") ? humanReadableFormElementType.tableElementHumanReadable :
							humanReadableFormElementType;
					}), this);					
					var labelField = (instanceId == "1")  ?  humanReadableElementType : humanReadableElementType + " (" + element.parentStatement.instanceId + ")";
						
					this._selectedFilterList.push({
		                id: element.parentStatement.elementType +"_"+ instanceId,
		                label: labelField,
		                selected: true,
		                oldOrNew: "old"
		            });
				}
							
			}, this));
							
			this._selectedFilterList = _.sortBy(this._selectedFilterList, 'label');	// Sort the list based on label field
			
			// This is applicable for translation languages. Prepares the list of master elements
			var masterElementsListCopy = this._processMasterListElements(this._masterElementsList, this._elementsAlreadyAdded);
			
			$.each(masterElementsListCopy, _.bind(function(key, element){				
				this._masterFilterList.push({
	                id: key,
	                label: element.tableElementHumanReadable,
	                selected: false
	            });
			}, this));
			
			this._masterFilterList = _.sortBy(this._masterFilterList, 'label');
		};
		
		/**
		 * Given gallery list and alreadyAddedList, this method will remove the elements that are already added and returns only the elements that are not still added
		 * This is also used in translation languages, where in user should not be able to add elements other than master. If the element is already added remove it from
		 * gallery.
		 * 
		 * @param galleryList
		 * @param alreadyAddedList
		 */
		AddTableElementWidget.prototype._processMasterListElements = function(galleryList, alreadyAddedList){
			var updatedList = {};
			$.each(galleryList, _.bind(function(elementCode, element){
				var elementType = elementCode.substring(0, elementCode.lastIndexOf("_"));
				var elementFound = false;
				$.each(alreadyAddedList, _.bind(function(key, csElement){					
					if(csElement.parentStatement.elementType == elementType && !csElement.deleted){
						elementFound = true;
						return false;
					}					
				}, this));
				if(!elementFound){
					updatedList[elementCode] = {};
					updatedList[elementCode] = element;
				}
			}, this));
			return updatedList;
		};
		
		/**
		 * Process Preview Element List - <br>
		 * Create id and label for each element. <br>
		 * Update selected flag for each element based on availability in already added elements list.
		 */		
		AddTableElementWidget.prototype._processPreviewListElements = function() {
			// Reinitialize preview filter list.
			this._previewFilterList = [];
			
			// Iterate over all the elements available in preview.
			$.each(this._previewElementsList, _.bind(function (index, previewElement) {
				var elementFound = false;
				
				// Check whether this element is already available in selected list.
				$.each(this._elementsAlreadyAdded, _.bind(function (index, addedElement) {
					// If element is marked for deletion.
					if (addedElement.deleted) {
						return true;
					}
					
					// Continue when instance id is not same.
					if (previewElement.instance != addedElement.parentStatement.getInstance()) {
						return true;
					}
					
					// Continue is element types are not same.
					if(previewElement.type != addedElement.parentStatement.elementType) {
						return true;
					}
					
					// Element is not deleted and id of the element is same, so element is already added.
					elementFound = true;
					return false;
				}, this));
				
				// Get readable form of element type to create element label.
				tableElementTypeService.getReadableElementType(previewElement.type, _.bind(function(readableTableElementType) {
					var readableElementType = readableTableElementType.hasOwnProperty("tableElementHumanReadable") ? readableTableElementType.tableElementHumanReadable :
						readableTableElementType;

					// Create label and id for the element with element type and instance id.
					var labelField = (previewElement.instance == 1) ?  readableElementType : readableElementType + " (" + previewElement.instance + ")";
					var idField = previewElement.type + "_" + previewElement.instance;
					
					// Update preview filter list with id, label and selected flag.
					this._previewFilterList.push({
						id : idField,
						label : labelField,
						selected : elementFound
					});
				}, this));
			}, this));
			
			// Sort the list based on label field
			this._previewFilterList = _.sortBy(this._previewFilterList, 'label');
		};
		
		/**
		 * This method will check if the elementType+instanceId in selected and new elements list and marks it for delete
		 * @param elementType
		 * @param instanceId
		 */
		AddTableElementWidget.prototype._removeElementOnDeselect = function(elementType, instanceId){
			var searchId = elementType + "_" + instanceId;
			
			// check if element with same elementtype && instanceid is available in selected filter 
			for(var i=0;i<this._selectedFilterList.length;i++){
				if((this._selectedFilterList[i].id).toLowerCase() == searchId.toLowerCase()){

					if(this._selectedFilterList[i].oldOrNew == "old") {
						var elementDetails = {};
						elementDetails.id = searchId;
						elementDetails.elementType = elementType;
						elementDetails.instanceId = instanceId;
						elementDetails.action = ContentConstants.LanguageAction.REMOVE;
						this._addRemoveElementsList.push(elementDetails);
					}

					// Update selected flag as false for element.
					this._selectedFilterList[i].selected = false;
					
					break;
				}
			}
			
			// If element is not found in selectedfilter list, check in new elements list			
			for(var i=0;i<this._addRemoveElementsList.length;i++){
				if((this._addRemoveElementsList[i].id).toLowerCase() == searchId.toLowerCase() && (this._addRemoveElementsList[i].action == ContentConstants.LanguageAction.ADD)){
					this._addRemoveElementsList.splice(i, 1);
					break;
				}
			}
		};
		
		
		/**
		 * Return true if we can add the element type and instance Id, false otherwise
		 * 
		 * @param elementType
		 * @param instanceId
		 * @returns {Boolean}
		 */
		AddTableElementWidget.prototype._canAddNewElementType = function(elementType, instanceId){
			var searchId = elementType +"_"+instanceId;
			// check if element with same elementtype && instanceid is available in selected filter 
			var elementTypeAlreadyAdded =  $.grep(this._selectedFilterList, _.bind(function(element, index){
					return (element.id).toLowerCase() == searchId.toLowerCase() && element.selected;
				}, this));
			
			// If the element is not available in selectedFilter then search in _addRemoveElementsList
			if(elementTypeAlreadyAdded.length <= 0){
				elementTypeAlreadyAdded =  $.grep(this._addRemoveElementsList, _.bind(function(element, index){
					// Currently added element was marked for delete
					return (element.id).toLowerCase() == searchId.toLowerCase() && element.action == ContentConstants.LanguageAction.ADD;
				}, this));
			}
			
			// If length of elementTypeAlreadyAdded > 0, there is already an element with same element type
			return elementTypeAlreadyAdded.length <= 0;
		};
		
		/**
		 * Checks if passed elementType and InstanceId is already available, if yes, then it will recursively call
		 * itself and returns the correct elementType_InstanceId
		 * @param elementType
		 * @param instanceId
		 * @returns
		 */
		AddTableElementWidget.prototype._calculateNewId = function(elementType, instanceId){					
			// We found that the element is already added
			if(!this._canAddNewElementType(elementType, instanceId)){
				instanceId = parseInt(instanceId)+1;
				return this._calculateNewId(elementType, instanceId);
			}else{
				return instanceId;
			}
		};
		
		/**
		 * Checks if element is marked for delete and returns the index of element 
		 * @param elementType
		 * @param instanceId
		 * @returns
		 */
		AddTableElementWidget.prototype._isElementMarkedForDeleted = function(elementType, instanceId){	
			var searchId = elementType +"_"+instanceId;
			var index = -1;
			for(var i=0; i<this._addRemoveElementsList.length;i++){
				// Currently added element was marked for delete
				if((this._addRemoveElementsList[i].id).toLowerCase() == searchId.toLowerCase() && this._addRemoveElementsList[i].action == ContentConstants.LanguageAction.REMOVE){
					index = i;
					break;
				}
			};
			
			return index;
		};
		
		/**
		 * Returns JQuery element for AddTableElementWidgetView
		 * @returns
		 */
		AddTableElementWidget.prototype.getJQElement = function(){
			return this._view.getJQElement();
		};
		
		/**
		 * Returns addRemoveElementsList 
		 * @returns {Array}
		 */
		AddTableElementWidget.prototype.getAddRemoveElementsList = function(){			
			return this._addRemoveElementsList;
		};
		
		/**
		 * Clears addRemoveElementsList, resets to blank array
		 */
		AddTableElementWidget.prototype.clearAddRemoveElementsList = function(){
			this._addRemoveElementsList = [];
		};
		
		/**
		 * This method will return the JQelement for modal popup
		 * @param modal
		 * @returns
		 */
		AddTableElementWidget.prototype.getModalFooterJQElement = function(modal){
			return this._view.getModalFooterJQElement(modal);
		};
		
		return AddTableElementWidget;
});