define(["underscore",
        "jquery",
        "packaging-content-management/ContentConstants",
        "packaging-content-management/text-content-utils/ContentUtils",
        "packaging-content-management/table-content-page/view/TableContentLanguageWidgetView",
        "packaging-content-management/text-content-element-widget/TextContentElementWidget",
        "packaging-content-management/text-content-utils/LanguageUtils",
        "./AddTableElementWidget",
        "component/modal/Modal",
        "packaging-content-management/content/TextContentElement",
        "packaging-content-management/content/TableTextContent",
        "packaging-content-management/PCMEventBus",
        "utils/webi18n!TextContentStrings"],
	function (_, 
			$, 
			ContentConstants,
			ContentUtils,
			TableContentLanguageWidgetView,
			TextContentElementWidget, 
			LanguageUtils,
			AddTableElementWidget,
			Modal,
			TextContentElement,
			TableTextContent,
			pcmEventBus,
			TextContentStrings) {
		
	/**
	 * TableContentLanguageWidget.js
	 * 
	 * This class implements controller for Table Content Language
	 *  
	 * @author RAHM 
	 */
	
		"use strict";

		var TableContentLanguageWidget = function (canManage, elementsArray, languageCode, langWidgetParam, tableContentPage) {
			
			this._elementsArray = elementsArray;
			this._languageCode = languageCode;
			this._tableContentPage = tableContentPage;
			this._isAdminModeOn = false;
			this._webi18n = TextContentStrings;

			//Language and elements parameters
			this._language = langWidgetParam.language;
			this._isMaster = langWidgetParam.isMaster;
			this._isProjectLang = langWidgetParam.isProjectLang;
			this.projMastLangs = langWidgetParam.projMastLangs;
			this.projTransLangs = langWidgetParam.projTransLangs;
			this._fullTableElementTypesList = langWidgetParam.fullTableElementTypesList;
			this._elementTypesGroups = [];
			
			// The callbacks that get executed when the content of the language.
			this._onChange = [];
			this._onMouseIn = [];
			this._onMouseOut = [];
			this._onFocus = [];
			this._onBlur = [];
			
			this._currentState = undefined;
			this._nextStates = [];
			
			this._masterElementTypes = {};

			this._isContentManagerView = canManage;
			
			// The text content element widgets.
			this._textContentElementWidgets = [];
			
			// Group the element types in "this._elementTypesGroups".
			var groupedPromise = this._groupTableContentElements();	

			// Create the view.
			this._view = new TableContentLanguageWidgetView(this._currentState, this._nextStates, this._languageCode,  this._isContentManagerView);
			
			// Just construct the header div. If this._language is not evaluated, the constructHeaderDiv, it will use language code to display the language
			this._view.constructHeaderDiv(!this._language ? this._languageCode : this._language, this._isMaster);
					
			// Calculate the current and next states.
			this.calculateCurrentAndNextStatesForLanguageWidget();
			
			// When the element types are grouped, render the language widgets.
			$.when(groupedPromise).then(function () {
				// Construct table content language widget.
				this._constructTableContentLanguageWidget();
			}.bind(this));	
				
			// What needs to happen when the add element button is clicked - Open Modal
			this._view.onAddElement(_.bind(function () {		
				
				// Since user is adding the elements he should be admin!! This is required, when user is adding new language and elements as well
				this._isAdminModeOn = true;
				
				// Now that we have the element types, lets prepare the Add Table Element widget ready for use
				if(this._isMaster) {
					
					// For master language call widget with standard list of element types
					this._addTableElementWidget = new AddTableElementWidget(this._isMaster, this._fullTableElementTypesList, this._elementsArray, [], this._tableContentPage);
					
				} else {
					
					// For translation languages call add element widget with only the elements that are added for master language
					var masterElementTypesArray = this._tableContentPage.getTableContent().getMasterElementTypes();
											
					for(var i=0; i< masterElementTypesArray.length; i++){
						var elemTypeCode = masterElementTypesArray[i].parentStatement.getElementType();
						var elementInstanceid = masterElementTypesArray[i].parentStatement.instanceId;
						var elemId = elemTypeCode + "_" + elementInstanceid;
						if(this._fullTableElementTypesList[elemTypeCode]){
							var elemObj = jQuery.extend(true, {}, this._fullTableElementTypesList[elemTypeCode]);
							var id = elementInstanceid == "1" ? 
									elemObj.tableElementHumanReadable :	elemObj.tableElementHumanReadable + " ("+elementInstanceid+")";
							this._masterElementTypes[elemId] = elemObj;
							this._masterElementTypes[elemId].tableElementHumanReadable = id;									
						}
						else{
							this._masterElementTypes[elemId] = {};
							this._masterElementTypes[elemId].tableElementCat = "Others";
							this._masterElementTypes[elemId].tableElementHumanReadable = elementInstanceid == "1" ? elemTypeCode :
								elemTypeCode + " ("+elementInstanceid+")";
						}
						this._masterElementTypes[elemId].instanceId = elementInstanceid;
				
					}
					this._addTableElementWidget = new AddTableElementWidget(this._isMaster, [], this._elementsArray, this._masterElementTypes, this._tableContentPage);
				}
				
				this._addTableElementWidget.clearAddRemoveElementsList();
				
				var langDesc = !this._language ? this._languageCode : this._language.langDesc;
						
				this._modal = new Modal(this._addTableElementWidget.getJQElement(),{
	                width: 'modal-lg', allowClose: false
	            });
				this._modal.setTitle(this._webi18n.t("ADD_ELEMENT_TITLE", langDesc));
				this._modal.setFooter(this._addTableElementWidget.getModalFooterJQElement(this._modal));
				this._modal.open();
									
				this._modal.onClose(_.bind(function(){
					this._addOrRemoveElement();
				}, this));
										
			}, this));
				
			this.registerBulkStateChangeHandler();
			this.registerBulkRevertHandler();
			
			
			var calcLangWidgetState = _.debounce(function(e) {
				// Calculate the new state of the of language widget (and only rerender if it changed).
				this.calculateCurrentAndNextStatesForLanguageWidget();
			}.bind(this), 150);
			
			// Start listening for changes in all the TextContentElementWidgets.
			pcmEventBus.on("PCM.TextContentElementWidget.update", function (textContent) {
				
				// Check if the change of this text content could affect this language widget.
				var affects = false;
				$.each(this._elementsArray, function (i, element) {
					if (element.parent() == textContent) {
						affects = true;
						return false; // And break.
					}
				});		
				
				// Recalculate status for state full widget based on its elements.
				if (affects) {
					calcLangWidgetState();
				}
			}.bind(this));
		};
		
		/**
		 * Register call back function on what should happend when popup  is closed
		 */
		TableContentLanguageWidget.prototype._addOrRemoveElement = function() {
			// Get New Elements that were added in popup
			this._newElementsAddedRemoved = this._addTableElementWidget.getAddRemoveElementsList();
			var operationCancelled = false;
			this._userInputOk = false;
						
			// 1. First check if the element is already added in the file - TableContent.prototype.checkAndReturnTableTextContent
			// 2. If its not added - create new tableContent that is wcc element 
			for(var i=0; i< this._newElementsAddedRemoved.length; i++) {
				
				var newElementType =  this._newElementsAddedRemoved[i];
				
				if(!newElementType.elementType || newElementType.elementType == "")
					continue;
				
				var updatedElement = undefined;
				if(newElementType.action == ContentConstants.LanguageAction.ADD){
					this._processAddNewElement(newElementType);
				}else if(newElementType.action == ContentConstants.LanguageAction.REMOVE){
					updatedElement = this._processRemoveElement(newElementType);
					if(!updatedElement){
						operationCancelled = true;
						break;
					}
				}
			}
			
			// Group the table content elements
			this._groupTableContentElements();
			
			if(!operationCancelled && this._newElementsAddedRemoved.length > 0){
				this._view.empty();
				this._textContentElementWidgets.length = 0;
				this._constructTableContentLanguageWidget();
				
				// Adding or removing elements may affect status of the language widget.
				this.calculateCurrentAndNextStatesForLanguageWidget();
			}
		};
		
		
		/**
		 * When delete operation is triggered what should happen?
		 * This is invoked when user clicks on delete button on every single textcontentelementwidget 
		 * 
		 * @param tableContentElement
		 * @param textContentElementWidget
		 */
		TableContentLanguageWidget.prototype._onDeleteAction = function (tableContentElement, textContentElementWidget) {
			var elementTypeTobeRemoved = {};
			elementTypeTobeRemoved.instanceId = tableContentElement.parentStatement.instanceId;
			elementTypeTobeRemoved.elementType = tableContentElement.parentStatement.elementType;
			var deletedElement = this._processRemoveElement(elementTypeTobeRemoved);
			
			// reset the flag to false, so that again when user clicks on delete button for another element, 
			// error message should be shown again
			this._userInputOk = false;
			
			// Update the widget
			if(deletedElement){
				this.manageDeleteAction(tableContentElement, textContentElementWidget);
			}
		};
		
		/**
		 * This method is to be called when an element is deleted. 
		 * When last master element is removed at that time language widget should be updated for the list already added elements.
		 * This method also updated widgets for those elements which affected by delete operation.
		 *  
		 * @param tableContentElement
		 * @param textContentElementWidget
		 */
		TableContentLanguageWidget.prototype.manageDeleteAction = function (tableContentElement, textContentElementWidget) {
						
			// Update the widget
			textContentElementWidget.update(tableContentElement);
			textContentElementWidget.removeCustomActionButton();
			
			// Removing elements may affect status of the language widget.
			this.calculateCurrentAndNextStatesForLanguageWidget();
		};
		
		/**
		 * Remove the specified element type from given language
		 * @param elementTypeToBeRemoved
		 */
		TableContentLanguageWidget.prototype._processRemoveElement = function(elementTypeToBeRemoved){
			var instanceId = elementTypeToBeRemoved.instanceId;
			var elementType = elementTypeToBeRemoved.elementType;
			
			var tableContent = this._tableContentPage.getTableContent();
			var tableContentElement = undefined;
			
			// Find the element to be deleted from the elementsArray and mark it as deleted
			$.each(this._elementsArray, _.bind(function (index, element) {
				if (element.parentStatement.instanceId == instanceId && element.parentStatement.elementType ==  elementType) {
					// Verify if this is the last master and there are translations also that has to be deleted
					var removeOtherElements = element.isMaster() && element.parentStatement.isLastMasterToBeRemoved(element.getLangCode(), 
							this._textContentElementWidgets[0]._projMasterLanguages) ;
					
					if (removeOtherElements) {
						var hasMoreElementsToBeDeleted = $.grep(element.parentStatement.getElementsList(), function(element, index){
							return (!element.isDeleted() && element.getCurrentState() != ContentConstants.State.EMPTY);
						}).length > 1;							
						
						
						// Show the confirm message only once. If there are 10 master elements that will be removed, then show the confirm message only once
						if(!this._userInputOk && hasMoreElementsToBeDeleted){
								this._userInputOk = confirm(this._webi18n.t("REMOVE_ELEMENT_ALERT"));
						}
						
						if(this._userInputOk || !hasMoreElementsToBeDeleted){
							// Iterate over all the elements of this text content and update widget.
							tableContent.removeTableTextContent(element, this._textContentElementWidgets[0]._projMasterLanguages);
							
							$.each(element.parentStatement.getElementsList(), _.bind(function (index, contentElement) {
								if (contentElement.getState() != ContentConstants.State.EMPTY) {
									$.each(this._onChange, _.bind(function (index, callback) {
										callback(contentElement, ContentConstants.OnChangeCallbackType.DELETED);
									}, this));
								}
							}, this));
						}else{
							return false;
						}
						
					} else {
						tableContent.removeTableTextContent(element, this._textContentElementWidgets[0]._projMasterLanguages);
						
						$.each(this._onChange, _.bind(function (index, callback) {
							callback(element, ContentConstants.OnChangeCallbackType.DELETED);
						}, this));
					}
					
					tableContentElement = element;
					return false;
				}
			}, this));

			return tableContentElement;
		};
		
		/**
		 * Method to add new element type to language
		 * @param newElementType
		 */
		TableContentLanguageWidget.prototype._processAddNewElement = function (newElementType) {
			// Check if this element type and instance Id is already available. In that case, we just need to create a element for language and add it
			var existingTableTextContent = this._tableContentPage.getTableContent().checkAndReturnTableTextContent(
					newElementType.elementType, newElementType.instanceId);
			if (!existingTableTextContent) {
				// Create text content element and saveactions for new tabletextcontent
				var textContentElement = this._tableContentPage.getTableContent().addTableTextContent(newElementType.elementType, newElementType.instanceId,
						this._languageCode, this._isMaster);
				
				this._addElementTo(textContentElement);
			}else{
				existingTableTextContent.addLanguage(this._languageCode, this._isMaster, _.bind(function(csElem){
					existingTableTextContent.element.push(csElem);
					this._addElementTo(csElem);
				}, this));
			}
		};
		
		/**
		 * Method to add element to the elementsArray, that elementTypesGroup
		 * @param csElem
		 */
		TableContentLanguageWidget.prototype._addElementTo = function(csElem){
			this._elementsArray.push(csElem);

			for (var i=0; i < this._onChange.length; i++) {
				this._onChange[i](csElem, ContentConstants.OnChangeCallbackType.ADDED);
			}

			

		};
		
		/**
		 * Registering call back for change for new state.
		 */
		TableContentLanguageWidget.prototype.registerBulkStateChangeHandler = function() {
			// When click happens to do bulk approval.
			this._view.registerBulkStateChangeHandler(_.bind(function (newState) {
				// Iterate over all the element widgets of this language and change their element status.
				_.each(this._textContentElementWidgets, function (textContentElementWidget) {
					var textContentElement = textContentElementWidget.getTextContentElement();
					
					var canManage = textContentElement.parent().canManage().allowed;
					var canReview = textContentElement.canReview(this.projMastLangs).allowed;
					var canEdit = textContentElement.canEdit(this.projMastLangs).allowed;
					var hasEdit = textContentElement.hasEdit().allowed;
					
					var elementCurrentState = textContentElement.getCurrentState();
					// Check for the current status and new status, update the widget if not same.
					if (newState == elementCurrentState) {
						// continue with this loop for next widget.
						return true;
					}
					
					// Update widget only when the current state of the element and the state of the language widget is same.
					if (this._state != elementCurrentState) {
						// continue with this loop for next widget.
						return true;
					}
					
					// For detailed documentation of these condition, refer "TextContentElementApprovalSaveAction.java class".
					switch (elementCurrentState) {
					case ContentConstants.State.REJECTED :
						if (ContentConstants.State.DRAFT == newState && hasEdit) {
							textContentElementWidget.revertToDraft();
						}
						break;
					case ContentConstants.State.DRAFT :
						if (ContentConstants.State.READY_FOR_APPROVAL == newState && canEdit) {
							textContentElementWidget.startApproval();
						} else if (ContentConstants.State.APPROVED == newState && canEdit && canReview) {
							textContentElementWidget.approve();
						}
						break;
					case ContentConstants.State.READY_FOR_APPROVAL :
						if (ContentConstants.State.DRAFT == newState && hasEdit) {
							textContentElementWidget.revertToDraft();
						} else if (ContentConstants.State.APPROVED == newState && canReview) {
							textContentElementWidget.approve();
						} else if (ContentConstants.State.REJECTED == newState && canReview) {
							textContentElementWidget.reject();
						}
						break;
					case ContentConstants.State.APPROVED :
						if (ContentConstants.State.DRAFT == newState && canManage) {
							textContentElementWidget.revertToDraft();
						}
						break;
					default:
						break;
					}
					
					// Update widget with latest changes.
					textContentElementWidget.update(textContentElement);
					
					this._currentState = newState;
					// Execute all the call backs.
					for (var i=0; i < this._onChange.length; i++) {
						this._onChange[i](textContentElement);
					}
					
				}.bind(this));
				
			}, this));
		};
		
		/**
		 * Registering call back for change for reverting approval action.
		 */
		TableContentLanguageWidget.prototype.registerBulkRevertHandler = function() {
			this._view.registerBulkRevertHandler(_.bind(function (wasNewState) {
				// Iterate over all the element widgets of this language and change their element status.
				_.each(this._textContentElementWidgets, _.bind(function (textContentElementWidget) {
					
					// Undo the last approval.
					var textContentElement = textContentElementWidget.undoApproval();
					
					// Execute all the call backs.
					for (var i=0; i < this._onChange.length; i++) {
						this._onChange[i](textContentElement, ContentConstants.OnChangeCallbackType.APPROVAL_REVERTED);
					}
				}, this));
			}, this));
		};
		
		/**
		 * Return whether widget is for master language or for translation language.
		 */
		TableContentLanguageWidget.prototype.isMaster = function() {
			return this._isMaster;
		};
		
		/**
		 * This method will group Content elements based on their Element Types.
		 * This will also determine the current and next state of the TableContentLanguageWidget
		 * 
		 * @returns A promise that will be resolved when they're grouped.
		 */
		TableContentLanguageWidget.prototype._groupTableContentElements = function(){
			
			// Create a deferred object.
			var dfd = jQuery.Deferred();
			
			// Sort the elements by their order in the preview.
			this._sortByPreviewOrder(this._elementsArray, _.bind(function(elementsArr){
				
				// All the elements
				this._elementsArray = elementsArr;
				this._elementTypesGroups = [];
				
				// Loop over the elements.
				_.each(this._elementsArray, _.bind(function (tableContentElement) {
					
					// Group them by category.
					var elementType = tableContentElement.parentStatement.elementType;
					var humanReadableFormElementType = this._fullTableElementTypesList[elementType] || elementType;
					var tableContElemAndElemType = {};
					var tableElementType = humanReadableFormElementType.hasOwnProperty("tableElementHumanReadable") ? humanReadableFormElementType.tableElementHumanReadable : humanReadableFormElementType;
					var tableElementCat = humanReadableFormElementType.hasOwnProperty("tableElementCat") ? humanReadableFormElementType.tableElementCat : "NO_CATEGORY";
					tableContElemAndElemType.tableElementType = tableElementType;
					tableContElemAndElemType.tableContentElement = tableContentElement;

					// Add the table content element to the correct table elements group.
					var foundElementTypesGroup = false;
					for (var i=0; i < this._elementTypesGroups.length; i++) {
						var category = this._elementTypesGroups[i].category;
						if (category == tableElementCat) {
							this._elementTypesGroups[i].elements.push(tableContElemAndElemType);
							foundElementTypesGroup = true;
							break;
						}
					}
					// If we didn't found a group to put the table element in ... create such a group.
					if (!foundElementTypesGroup) {
						this._elementTypesGroups.push({
							category: tableElementCat,
							elements: [tableContElemAndElemType]
						});
					}
				}, this));
				
				// Resolve the deferred.
				dfd.resolve(this._elementTypesGroups);
				
			}, this));
			
			// Return the promise
			return dfd.promise();
		};
		
		/**
		 * Sort the elements based on the order in which they appear in the preview.
		 * @param elements
		 * @returns
		 */
		TableContentLanguageWidget.prototype._sortByPreviewOrder = function(elements, callback) {
			
			this._tableContentPage.getPreviewElements().done(_.bind(function(previewElementEntries){
				
				// If preview elements are found ...
				if (previewElementEntries && previewElementEntries.length > 0) {
					var sortedElements = elements.sort(function (element1, element2) {
						
						// These will be the indexes of the elements in the preview.
						var index1 = -1;
						var index2 = -1;
						
						for (var i =0; i < previewElementEntries.length; i++) {
							var previewElementEntry = previewElementEntries[i];
							if (element1.getParent().elementType == previewElementEntry.type) {
								index1 = i;
							}
							if (element2.getParent().elementType == previewElementEntry.type) {
								index2 = i;
							}
							if (index1 >= 0 && index2 >= 0) {
								break;
							}
						}
						
						// Now that you calculated the index for each of these elements in the preview ... compare them.
						if (index1 >= 0 && index2 >= 0) {
							if (index1 < index2) {
								return -1;
							} else if (index1 == index2) {
								return 0;
							} else {
								return 1;
							}
						} else {
							if (index1 >= 0 && index2 < 0) {
								return -1;
							} else if (index1 < 0 && index2 < 0) {
								return 0;
							} else {
								return 1;
							}
						}
					});
					
					callback(sortedElements);
				} else {
					callback(elements);
				}
			}, this));
		};
			
		/**
		 * Return whether widget language is part of project languages
		 */
		TableContentLanguageWidget.prototype.isProjectLanguage = function() {
			return this._isProjectLang;
		};
		
		/**
		 * This method will create TextContentWidget for every value in the table content. It prepares the elements in groups. 
		 *
		 */
		TableContentLanguageWidget.prototype._constructTableContentLanguageWidget = function (){
			
			var options = {};
			options.viewOptions = {widgetview: 'table'};
			
			_.each(this._elementTypesGroups, _.bind(function(elementTypeGroup){
				//Lets check the number of elements in the group
				var groupElementsCount = elementTypeGroup.elements.length;
				
				// This variable will tell, if in the group can we display two elements in one row.
				var canDisplayTwoElementsPerRow = true;
				
				// If we have even number of elements then we can display two elements in one row, or else in one of the rows, we will have to display
				// one element (that will be unlucky one, without its friend ;) )
				if(groupElementsCount % 2 != 0){
					canDisplayTwoElementsPerRow = false;
				}
				
				var rowCounter = 0;
				var referenceToRow = null;
				var elementDiv = this._view.constructElementGroupDiv();
				
				_.each(elementTypeGroup.elements, _.bind(function(elementTypeAndElementsObj){
					var tableElementType = elementTypeAndElementsObj.tableElementType;
					var tableContentElement = elementTypeAndElementsObj.tableContentElement;
					var instanceId = tableContentElement.parentStatement.instanceId;
					

					var textContentElementWidget = new TextContentElementWidget(tableContentElement, options);
					
					if(rowCounter == 0 && !canDisplayTwoElementsPerRow){ 
						this._view.createRow(textContentElementWidget, tableElementType, instanceId, true, elementDiv);
						this._rowAlreadyCreated = false;
						rowCounter++;
					}
					else if(this._rowAlreadyCreated){
						this._view.appendToRow(textContentElementWidget, tableElementType, instanceId, referenceToRow);
						this._rowAlreadyCreated = false;
					}
					else{
						referenceToRow = this._view.createRow(textContentElementWidget, tableElementType, instanceId, false, elementDiv);
						this._rowAlreadyCreated = true;
						rowCounter++;
					}
					
					// When the content of the widget changes ...
					textContentElementWidget.onChange(function (changetype, textElement) {
						// Execute all the call backs.
						for (var i=0; i < this._onChange.length; i++) {
							this._onChange[i](tableContentElement);
						}
					}.bind(this));
					
					// When the user hovers over the widget ...
					textContentElementWidget.getJQElement().hover(function () {
						// Execute all the callbacks.
						for (var i=0; i < this._onMouseIn.length; i++) {
							this._onMouseIn[i](tableContentElement);
						}
					}.bind(this), function () {
						// Execute all the callbacks.
						for (var i=0; i < this._onMouseOut.length; i++) {
							this._onMouseOut[i](tableContentElement);
						}
					}.bind(this));
					
					// When the widget is focused.
					textContentElementWidget.onFocus(function () {
						// Unselect all other widgets.
						_.each(this._textContentElementWidgets, function (otherWidget) {
							if (otherWidget != textContentElementWidget) {
								otherWidget.select(false);
							}
						}.bind(this));
						
						// Execute all the callbacks.
						for (var i=0; i < this._onFocus.length; i++) {
							this._onFocus[i](tableContentElement);
						}
					}.bind(this));
					
					// When the widget is blurred.
					textContentElementWidget.onBlur(function () {
						// Execute all the callbacks.
						for (var i=0; i < this._onBlur.length; i++) {
							this._onBlur[i](tableContentElement);
						}
					}.bind(this));
					
					// Add delete icons for element widgets, if admin mode is on. CM added new element, and after update of widget also, delete icons should be visible 
					if(this._isAdminModeOn && !tableContentElement.deleted){
						// If the element is already marked for deleted, then we shouldnt show delete button on widget
						this._onAdminButton(tableContentElement, textContentElementWidget);						
					}
										
					this._view.onAdminButton(_.bind(function () {
						_.each(this._textContentElementWidgets, function (otherWidget) {
							this._isAdminModeOn = true;
							if (otherWidget == textContentElementWidget) {
								// If the element is already marked for deleted, then we shouldnt show delete button on widget
								if(!tableContentElement.isDeleted()){
									this._onAdminButton(tableContentElement, textContentElementWidget);
								}
							}							
						}.bind(this));
					}, this));
					
					this._view.onExitAdmin(_.bind(function () {
						this._isAdminModeOn = false;
						// Once user gets out of admin mode, remove all delete buttons from all widgets
						_.each(this._textContentElementWidgets, function (elementWidget) {		
							textContentElementWidget.removeCustomActionButton();
						}.bind(this));
					}, this));
					
					// Try to display the widget in a single line.
					textContentElementWidget.shownInPage();
					
					// Add it to the list of widgets.
					this._textContentElementWidgets.push(textContentElementWidget);
					
				}, this));				
				
			}, this));			
		};
		
		/**
		 * What should happen when Admin button is clicked, Delete buttons should appear in all text widgets. Delete icon is shown on all elements
		 * irrespective of its status, master language, last master language
		 * 
		 * @param tableContentElement
		 * @param textContentElementWidget
		 */
		TableContentLanguageWidget.prototype._onAdminButton = function(tableContentElement, textContentElementWidget){			
						
			// Delete button
			var tableContentElement_var = tableContentElement;
			var textContentElementWidget_var = textContentElementWidget;
			var button = this._view.constructDeleteButtonForElementWidget();
			button.click(function (event) {
				this._onDeleteAction(tableContentElement_var, textContentElementWidget_var);
	        }.bind(this));
			
			textContentElementWidget.addCustomActionButton(button);
				
		};
		
		/**
		 * Set new state of statefullcontentwidget.
		 */
		TableContentLanguageWidget.prototype.setCurrentState = function(newState){
			this._view.setCurrentState(newState);
		};	
		
		/**
		 * Register a callback that should be executed when the user focusses on a text content element.
		 * @param callback {function} - The callback that will be executed with the targeted text content element as argument.
		 */
		TableContentLanguageWidget.prototype.onFocus = function(callback){
			this._onFocus.push(callback);
		};
		
		/**
		 * Register a callback that should be executed when the user blurs on a text content element.
		 * @param callback {function} - The callback that will be executed with the targeted text content element as argument.
		 */
		TableContentLanguageWidget.prototype.onBlur = function(callback){
			this._onBlur.push(callback);
		};
		
		/**
		 * Register callbacks that should be executed when the user hovers over a text content element.
		 * @param callback {function} - The callback that will be executed with the targeted text content element as argument.
		 */
		TableContentLanguageWidget.prototype.onHover = function(mouseInCallback, mouseOutCallback){
			this._onMouseIn.push(mouseInCallback);
			this._onMouseOut.push(mouseOutCallback);
		};
		
		/**
		 * Register a callback that should be executed when the content of one the elements changes.
		 * @param callback {function} - The callback that will be executed with the targeted text content element as argument.
		 */
		TableContentLanguageWidget.prototype.onChange = function (callback) {
			this._onChange.push(callback);
		};
		
		/**
		 * Get current status of the language widget.
		 */
		TableContentLanguageWidget.prototype.getState = function () {
			return this._currentState;
		};
		
		/**
		 * Get language code of the language widget.
		 */
		TableContentLanguageWidget.prototype.getLanguageCode = function () {
			return this._languageCode;
		};
		
		/**
		 * Get array of all elements of the language widget.
		 */
		TableContentLanguageWidget.prototype.getLanguageElements = function () {
			return this._elementsArray;
		};
		
		/**
		 * Get array of all element widgets of the language widget.
		 */
		TableContentLanguageWidget.prototype.getElementWidgets = function () {
			return this._textContentElementWidgets;
		};
		
		/**
		 * Calculates what should be the current and next states for the language widget
		 * CurrentState = lowest state of any element. This means that, if there is any one element that is in DRAFT state,
		 * then the language widget state will be DRAFT.
		 */
		TableContentLanguageWidget.prototype.calculateCurrentAndNextStatesForLanguageWidget = function(){
			
			// Check if the state of the language widget was already calculated.
			if (!this._state) {
				
				// No state was calculated, do it.
				this._state = ContentUtils.calculateCurrentState(this._elementsArray);
				this._currentState = this._state;
				
				// Calculate new next states for translation language only when at-least 1 master language is approved.
				if (this.isMaster() || (!this.isMaster() && this._tableContentPage.isMasterLanguageApproved())) {
					this._nextStates = ContentUtils.getNextStatesForLanguage(this._isContentManagerView, this._state, this._elementsArray, this.projMastLangs);
				} else {
					this._nextStates = [];
				}
				
				// Set the states in the view.
				this._view.setNextStates(this._state, this._nextStates);
				
			} else {
				// State was already calculated in the past, we need to calculate the new state.
				
				var newCurrentState = ContentUtils.calculateCurrentState(this._elementsArray);
				
				// Calculate new next states for translation language only when at-least 1 master language is approved. 
				var newNextStates = [];
				if (this.isMaster() || (!this.isMaster() && this._tableContentPage.isMasterLanguageApproved())) {
					newNextStates = ContentUtils.getNextStatesForLanguage(this._isContentManagerView, newCurrentState, this._elementsArray, this.projMastLangs);
				}
				
				if (_.isEqual(this._nextStates, newNextStates)) {
					if (newCurrentState == this._currentState) {
						return;
					} else {
						this._currentState = newCurrentState;
						this._view.setCurrentState(newCurrentState);
					}
				} else {
					this._currentState = newCurrentState;
					this._nextStates = newNextStates;
					// We've got new next states so we need to call this method.
					this._view.setNextStates(this._state, this._nextStates);
					this._view.setCurrentState(this._currentState);
				}
			}
		};

		/**
		 * Register a callback that should be executed when the content of one the elements changes.
		 * @param callback {function} - The callback that will be executed with the targeted text content element as argument.
		 */
		TableContentLanguageWidget.prototype.focus = function(contentTextElementId){
			
			// Iterate over all the widgets.
			var targetedWidget = undefined;
			_.each(this._textContentElementWidgets, function (elementWidget) {
				// Get the element.
				var element = elementWidget.getTextContentElement();
				// Check if the ID of the element of the widget matches the passed one ...
				if (element.getContentId() == contentTextElementId) {
					targetedWidget = elementWidget;
				} else {
					elementWidget.select(false);
				}
			}.bind(this));
			
			// Focus on the widget
			if (targetedWidget) {
				
				// Expand the view.
				this._view.expand();
				
				// Scroll to the widget.
				$('html, body').stop();
				$('html, body').animate({
					duration: 200,
			        scrollTop: targetedWidget.getJQElement().offset().top - 150
			    }, 1000);
				
				// Focus on the widget.
				targetedWidget.select(true);
				targetedWidget.focus(true);
			}
			
		};
		
		/**
		 * Disable widgets based on provided message.
		 */
		TableContentLanguageWidget.prototype.disable = function(msgTag) {
			this._view.disable(msgTag);
			
			_.each(this._textContentElementWidgets, function (textContentElementWidget) {
				textContentElementWidget.disable(msgTag);
			}.bind(this));
		};
		
		/**
		 * Enable widgets based on content type.
		 */
		TableContentLanguageWidget.prototype.enable = function() {
			this._view.enable();
			
			_.each(this._textContentElementWidgets, function (textContentElementWidget) {		
				textContentElementWidget.enable();
			}.bind(this));
		};
		
		/**
		 * Gets the JQ of StatefullContentWidget
		 * @returns
		 */
		TableContentLanguageWidget.prototype.getJQElement = function(){
			return this._view.getJQElement();
		};
			
		// Return the constructor.
		return TableContentLanguageWidget;
		
});
