define(["underscore",
        "jquery",
        "utils/webi18n!TextContentStrings",
        "component/multiSelect/view/MultipleMultiSelectList",
        "utils/Observer",
        "utils/FilterUtils",
        "utils/webcss!./AddTableElementWidget.css"],
	function (_,
			$,
			TextContentStrings,
			MultipleMultiSelectList,
			Observer,
			FilterUtils
			) {
	
		"use strict";
		
		var AddTableElementWidgetView = function (selectedElementsFilter, previewFilterGallery, libraryFilter, masterElementsFilter, previewName, isMaster) {
			this.webi18n = TextContentStrings;
			this._selectedElementsFilter = selectedElementsFilter;
			this._previewFilterGallery = previewFilterGallery;
			this._masterElementsFilter = masterElementsFilter;
			this._libraryFilter = libraryFilter;
			this._previewName = previewName;
			this._isMaster = isMaster;
			
			this._onEntrySelected = [];
			this._onEntryDeselected = [];
			this._onCancel = [];
			this._onAddCustomField = [];
			
			this._previewOptionDiv;
			this._galleryOptionDiv;
			this._customOptionDiv;
        				
			this._maxRows = 100;
			
			// Register Observer
			Observer.mixin(this);
			this._wrapper = $("<div>").addClass("addTableElementWidget");
			this._wrapper.empty(); // Empty the widget
			
			if(this._isMaster){
				this._prepareMasterLists();
				this._constructRadioButtonOptions();	
			}else{
				this._prepareTranslationList();
			}
		};
		
		/**
		 * Prepare widget for translation language 
		 */
		AddTableElementWidgetView.prototype._prepareTranslationList = function() {
			// Creating functions for MultiSelect List - Master Gallery
			this._getMasterElementsGallery = _.bind(function (pattern, page) {
				var dfr = $.Deferred();
				var selected = 0;
				var regex = FilterUtils.generateRegex(pattern, true);
				
				dfr.resolve(_.filter(this._masterElementsFilter, _.bind(function (object) {
					if (regex.test(object.label) && !object.selected) {
						selected++;
						// Only get the not selected objects that are on the right page
						return selected >= (this._maxRows * (page - 1)) && selected <= this._maxRows * page;
					}
				}, this)));
				
	            return dfr.promise();
			}, this);
			
			// Creating function for MultiSelect List - Selected List
			this._getSelectedList = _.bind(function (pattern, page) {
				var dfr = $.Deferred();
				var selected = 0;
				var regex = FilterUtils.generateRegex(pattern, true);
				
				dfr.resolve(_.filter(this._selectedElementsFilter, _.bind(function (object) {
					if (regex.test(object.label) && object.selected) {
						selected++;
						// Only get the not selected objects that are on the right page
						return selected >= (this._maxRows * (page - 1)) && selected <= this._maxRows * page;
					}
				}, this)));
				
				return dfr.promise();
			}, this);
			
			// MultiSelect List used in Translation option - 
			this._multiSelectListTranslationOptionDiv = $("<div>").addClass("multiSelectDiv");
			this._multiSelectTranslationOption = new MultipleMultiSelectList(this._getMasterElementsGallery, this._getSelectedList, this._multiSelectListTranslationOptionDiv, {
				notSelectedLabel: this.webi18n.t("GALLERY_LIST"),
				selectedLabel: this.webi18n.t("SELECTED_LIST"),
				selectAllTooltip: this.webi18n.t("SELECT_ALL"),
				deselectAllTooltip: this.webi18n.t("DESELECT_ALL"),
				cssClass : "addElementToolTip"
			});
			
			// When an entry is selected set the selected property of each attribute to true
			this._observe(this._multiSelectTranslationOption, "entrySelected", function (ids) {
				_.each(ids, _.bind(function(id){
					// Find master element with the help of id.
					var masterElements = $.grep(this._masterElementsFilter, _.bind(function (masterElement) {
						return masterElement.id == id;
					}, this));
					
					// Here this condition should always satisfy.
					if (masterElements.length > 0) {
						this._executeOnSelect(masterElements[0], true);
					}
				}, this));

				// Sort the list based on label field
				this._selectedElementsFilter = _.sortBy(this._selectedElementsFilter, 'label');
				
				// Render selected list.
				this._multiSelectTranslationOption.render();
			});

			// When an entry is deselected set the selected property of each attribute to false
			this._observe(this._multiSelectTranslationOption, "entryDeselected", function (ids) {
				_.each(ids, function (id) {
					// Find selected element with the help of id.
					var selectedElements = $.grep(this._selectedElementsFilter, _.bind(function (selectedElement) {
						return selectedElement.id == id;
					}, this));
					
					this._executeOnDeselect(selectedElements[0], this._masterElementsFilter);
				}, this);
				
				// Render selected list.
				this._multiSelectTranslationOption.render();
			});
			
			// select all entries that have a label that starts with 'pattern'
			this._observe(this._multiSelectTranslationOption, "selectAll", function (pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				var filteredMasterElements = _.filter(this._masterElementsFilter, function (element) {
					return regex.test(element.label) && !element.selected;
				});
				
				// Iterate over all the filtered elements and add to selected elements list
				$.each(filteredMasterElements, _.bind(function(index, masterElement) {
					this._executeOnSelect(masterElement, true);
				}, this));
				
				// Sort the list based on label field
				this._selectedElementsFilter = _.sortBy(this._selectedElementsFilter, 'label');
				
				// Render selected list.
				this._multiSelectTranslationOption.render();
			});

			// deselect all entries that have a label that starts with 'pattern'
			this._observe(this._multiSelectTranslationOption, "deselectAll", function (pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				var filteredSelectedElements = _.filter(this._selectedElementsFilter, function (element) {
					return regex.test(element.label) && element.selected;
				});
				
				// Iterate over all the filtered elements and add to selected elements list
				$.each(filteredSelectedElements, _.bind(function(index, selectedElement) {
					this._executeOnDeselect(selectedElement, this._masterElementsFilter);
				}, this));
				
				// Render selected list.
				this._multiSelectTranslationOption.render();
			});
						
			this._translationForm = $("<div>").addClass("translationForm");
			this._wrapper.append(this._translationForm);
			
			// Preview Fields Radio Button - Preview Fields
			var translationPanelDiv = $("<div>").addClass("panel panel-default translationGroup");
			this._translationForm.append(translationPanelDiv);
			var translationpanelBody = $("<div>").addClass("panel-body");
			translationPanelDiv.append(translationpanelBody);
           
			// Add MultiSelect List
			translationpanelBody.append(this._multiSelectListTranslationOptionDiv);
			
			this._multiSelectTranslationOption.render();
		};
			
		/**
		 * Prepares the list of element types - preview, gallery and custom elements
		 */
		AddTableElementWidgetView.prototype._prepareMasterLists = function() {
			// Creating functions for MultiSelect List - Preview Gallery
			this._getPreviewListGallery = _.bind(function (pattern, page) {
				var dfr = $.Deferred();
				var selected = 0;
				var regex = FilterUtils.generateRegex(pattern, true);
				
				dfr.resolve(_.filter(this._previewFilterGallery, _.bind(function (object) {
					if (regex.test(object.label) && !object.selected) {
						selected++;
						// Only get the not selected objects that are on the right page
						return selected >= (this._maxRows * (page - 1)) && selected <= this._maxRows * page;
					}
				}, this)));
				
	            return dfr.promise();
			}, this);
			
			
			// Creating function for MultiSelect List - Selected List
			this._getSelectedList = _.bind(function (pattern, page) {				 
				var dfr = $.Deferred();
				var selected = 0;
				var regex = FilterUtils.generateRegex(pattern, true);

				dfr.resolve(_.filter(this._selectedElementsFilter, _.bind(function (object) {
					if (regex.test(object.label) && object.selected) {
						selected++;
						// Only get the not selected objects that are on the right page
						return selected >= (this._maxRows * (page - 1)) && selected <= this._maxRows * page;
					}
				}, this)));
				 
				return dfr.promise();
			}, this);
			
			// Creating function for MultiSelect List - Library (complete elements list)
			this._getLibraryList = _.bind(function (pattern, page) {				
				var dfr = $.Deferred();		
				var selected = 0;
				var regex = FilterUtils.generateRegex(pattern, true);

				dfr.resolve(_.filter(this._libraryFilter, _.bind(function (object) {
					if (regex.test(object.label)) {
						selected++;
						// Only get the not selected objects that are on the right page
						return selected >= (this._maxRows * (page - 1)) && selected <= this._maxRows * page;
					}
				}, this)));
				          
	            return dfr.promise();		
			}, this);

			// MultiSelect List used in preview option - 
			this._multiSelectListPreviewOptionDiv = $("<div>").addClass("multiSelectDiv");
			this._multiSelectPreviewOption = new MultipleMultiSelectList(this._getPreviewListGallery, this._getSelectedList, this._multiSelectListPreviewOptionDiv, {
				notSelectedLabel: this.webi18n.t("PREVIEW_LIST"),
				selectedLabel: this.webi18n.t("SELECTED_LIST"),
				selectAllTooltip: this.webi18n.t("SELECT_ALL"),
				deselectAllTooltip: this.webi18n.t("DESELECT_ALL"),
				cssClass: "addElementToolTip"
			});
			
			// When an entry is selected set the selected property of each attribute to true
			this._observe(this._multiSelectPreviewOption, "entrySelected", function (ids) {
				_.each(ids, _.bind(function(id){
					// Find preview element with the help of id.
					var previewElements = $.grep(this._previewFilterGallery, _.bind(function (previewElement) {
						return previewElement.id == id;
					}, this));
					
					// Here this condition should always satisfy.
					if (previewElements.length > 0) {
						this._executeOnSelect(previewElements[0], true);
					}
				}, this));

				// Sort the list based on label field
				this._selectedElementsFilter = _.sortBy(this._selectedElementsFilter, 'label');
				
				// Render selected list for preview with sorted selected elements.
				this._multiSelectPreviewOption.render();
				
				// Update selected elements list for gallery option.
				this._multiSelectGalleryOption.render();
				
				// Update selected elements list for custom option
				this._updateSelectedForCustom(false);
			});

			// When an entry is deselected set the selected property of each attribute to false
			this._observe(this._multiSelectPreviewOption, "entryDeselected", function (ids) {
				_.each(ids, function (id) {
					// Find selected element with the help of id.
					var selectedElements = $.grep(this._selectedElementsFilter, _.bind(function (selectedElement) {
						return selectedElement.id == id;
					}, this));
					
					this._executeOnDeselect(selectedElements[0], this._previewFilterGallery);
				}, this);
				
				// Render selected list for preview with sorted selected elements.
				this._multiSelectPreviewOption.render();
				
				// Update selected elements list for gallery option.
				this._multiSelectGalleryOption.render();
				
				// Update selected elements list for custom option
				this._updateSelectedForCustom(false);
			});
			
			// select all entries that have a label that starts with 'pattern'
			this._observe(this._multiSelectPreviewOption, "selectAll", function (pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				var filteredPreviewElements = _.filter(this._previewFilterGallery, function (element) {
					return regex.test(element.label) && !element.selected;
				});
				
				// Iterate over all the filtered elements and add to selected elements list
				$.each(filteredPreviewElements, _.bind(function(index, previewElement) {
					this._executeOnSelect(previewElement, true);
				}, this));
				
				// Sort the list based on label field
				this._selectedElementsFilter = _.sortBy(this._selectedElementsFilter, 'label');
				
				// Update selected elements list for gallery option.
				this._multiSelectGalleryOption.render();
				
				// Update selected elements list for custom option
				this._updateSelectedForCustom(false);
			});

			// deselect all entries that have a label that starts with 'pattern'
			this._observe(this._multiSelectPreviewOption, "deselectAll", function (pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				var filteredSelectedElements = _.filter(this._selectedElementsFilter, function (element) {
					return regex.test(element.label) && element.selected;
				});
				
				// Iterate over all the filtered elements and add to selected elements list
				$.each(filteredSelectedElements, _.bind(function(index, selectedElement) {
					this._executeOnDeselect(selectedElement, this._previewFilterGallery);
				}, this));
				
				// Update selected elements list for gallery option.
				this._multiSelectGalleryOption.render();
				
				// Update selected elements list for custom option
				this._updateSelectedForCustom(false);
			});
			
			// MultiSelect List used in gallery option -
			this._mulitSelectListGalleryOptionDiv = $("<div>").addClass("multiSelectDiv");
			this._multiSelectGalleryOption = new MultipleMultiSelectList(this._getLibraryList, this._getSelectedList, this._mulitSelectListGalleryOptionDiv, {
				"notSelectedLabel": this.webi18n.t("GALLERY_LIST"),
				"selectedLabel": this.webi18n.t("SELECTED_LIST"),
				"selectAllTooltip": this.webi18n.t("SELECT_ALL"),
				"deselectAllTooltip": this.webi18n.t("DESELECT_ALL"),
				"cssClass" : "addElementToolTip"
			});
			
			// When an entry is selected set the selected property of each attribute to true
			this._observe(this._multiSelectGalleryOption, "entrySelected", function (ids) {
				_.each(ids, _.bind(function(id){
					// Find preview element with the help of id.
					var previewElements = $.grep(this._previewFilterGallery, _.bind(function (previewElement) {
						return previewElement.id == id;
					}, this));
					
					if (previewElements.length > 0) {
						this._executeOnSelect(previewElements[0], true);
					} else {
						// Find preview element with the help of id.
						var libraryElements = $.grep(this._libraryFilter, _.bind(function (libraryElement) {
							return libraryElement.id == id;
						}, this));
						
						this._executeOnSelect(libraryElements[0], false);
					}
				}, this));

				// Sort the list based on label field
				this._selectedElementsFilter = _.sortBy(this._selectedElementsFilter, 'label');
				
				// Update selected elements list for gallery option.
				this._multiSelectGalleryOption.render();
				
				// Render selected list for preview with sorted selected elements.
				this._multiSelectPreviewOption.render();
				
				// Update selected elements list for custom option
				this._updateSelectedForCustom(false);
			});

			// When an entry is deselected set the selected property of each attribute to false
			this._observe(this._multiSelectGalleryOption, "entryDeselected", function (ids) {
				_.each(ids, function (id) {
					// Find preview element with the help of id.
					var selectedElements = $.grep(this._selectedElementsFilter, _.bind(function (selectedElement) {
						return selectedElement.id == id;
					}, this));
					
					this._executeOnDeselect(selectedElements[0], this._previewFilterGallery);
				}, this);
				
				// Update selected elements list for gallery option.
				this._multiSelectGalleryOption.render();
				
				// Render selected list for preview with sorted selected elements.
				this._multiSelectPreviewOption.render();
				
				// Update selected elements list for custom option
				this._updateSelectedForCustom(false);
			});
			
			// select all entries that have a label that starts with 'pattern'
			this._observe(this._multiSelectGalleryOption, "selectAll", function (pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				var filteredGalleryElements = $.grep(this._libraryFilter, function (element) {
					return regex.test(element.label);
				});
				
				// Iterate over all the filtered elements and add to selected elements list
				$.each(filteredGalleryElements, _.bind(function(index, galleryElement) {
					// Find preview element with the help of id.
					var previewElements = $.grep(this._previewFilterGallery, _.bind(function (previewElement) {
						return previewElement.id == galleryElement.id;
					}, this));
					
					if (previewElements.length > 0) {
						this._executeOnSelect(previewElements[0], true);
					} else {
						this._executeOnSelect(galleryElement, false);
					}
				}, this));
				
				// Sort the list based on label field
				this._selectedElementsFilter = _.sortBy(this._selectedElementsFilter, 'label');
				
				// Update selected elements list for gallery option.
				this._multiSelectPreviewOption.render();
				
				// Update selected elements list for custom option
				this._updateSelectedForCustom(false);
			});

			// deselect all entries that have a label that starts with 'pattern'
			this._observe(this._multiSelectGalleryOption, "deselectAll", function (pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				var filteredSelectedElements = _.filter(this._selectedElementsFilter, function (element) {
					return regex.test(element.label) && element.selected;
				});
				
				// Iterate over all the filtered elements and add to selected elements list
				$.each(filteredSelectedElements, _.bind(function(index, selectedElement) {
					this._executeOnDeselect(selectedElement, this._previewFilterGallery);
				}, this));
				
				// Update selected elements list for gallery option.
				this._multiSelectPreviewOption.render();
				
				// Update selected elements list for custom option
				this._updateSelectedForCustom(false);
			});
			
		};
		
		/**
		 * Execute the on entry select callback when element is selected.
		 * Updates elements list for selected and non selected element to update view.
		 *   
		 * @param element {Object} - Element which is selected
		 * @param removeFromGallery {Boolean} - whether selected element should be removed from gallery or not.
		 */
		AddTableElementWidgetView.prototype._executeOnSelect = function (element, removeFromGallery) {
			//Execute all call backs
			_.each(this._onEntrySelected, _.bind(function(callback){
				var elementDetails = callback(element.id);
	
				if (removeFromGallery) {
					// Update selected flag as true for this gallery element. 
					element.selected = true;
				}
				
				// Update selected flag for element in selected elements list, if already available.
				var addInSelectedElement = true;
				$.each(this._selectedElementsFilter, _.bind(function (index, selectedElement) {
					if (selectedElement.id == elementDetails.id) {
						selectedElement.selected = true;
						addInSelectedElement = false;
						return false;
					}
				}, this));
				
				// In case element is not available, then add element in selected elements list.
				if (addInSelectedElement) {
					this._selectedElementsFilter.push({
		                id: elementDetails.id,
		                label: elementDetails.label,
		                selected: true,
		                oldOrNew: "new"
		            });
				}
			}, this));
		};
		
		/**
		 * Execute the on entry deselect callback when element is unselected.
		 * Updates elements list for selected and non selected element to update view.
		 *   
		 * @param element {Object} - Element which is selected to be removed
		 * @param galleryList {List} - gallery list where unselected element should appear.
		 */
		AddTableElementWidgetView.prototype._executeOnDeselect = function (selectedElement, galleryList) {
			//Execute all call backs
			_.each(this._onEntryDeselected, function(callback){
				callback(selectedElement.id);
			});
	
			// Update selected flag as false for this selected element. 
			selectedElement.selected = false;
			
			// Update selected flag for element in preview elements list, if already available.
			$.each(galleryList, _.bind(function (index, galleryElement) {
				if (selectedElement.id == galleryElement.id) {
					galleryElement.selected = false;
					return false;
				}
			}, this));
		};
		
		/** 
		 * Update selected list view for custom option
		 * 
		 * @param idToBeSelected {String} - Id of the element which suppose to be shown as selected.
		 */
		AddTableElementWidgetView.prototype._updateSelectedForCustom = function (idToBeSelected) {
			// Remove entries from custom option also
			this._selectElement.empty().show();

			var hideSelectedList = true;
			
			// Iterate over all the selected element list.
			$.each(this._selectedElementsFilter, _.bind(function(index, selectedElement) {
				if (selectedElement.selected) {
					// Update custom option
					var option = $("<option>");
					option.attr({
						"value" : selectedElement.id,
						"class" : "galleryOptions"
					});
					
					// Show as selected when selected element id is same as requested.
					if (selectedElement.id == idToBeSelected) {
						option.attr("selected", "selected");
					}
					
					// Append option in select. 
					this._selectElement.append(option);
					option.append(selectedElement.label);
					
					// We found at-least 1 element to show.
					hideSelectedList = false;
				}
			}, this));
			
			// If there is not element selected then hide selected list in custom option.
			if (hideSelectedList) {
				this._selectElement.hide();
			} else {
				this._selectElement.show();
			}
			
		};
		
		/**
		 * Returns JQuery element of the popup AddTableElement
		 * @returns
		 */
		AddTableElementWidgetView.prototype.getJQElement = function(){
			return this._wrapper;
		};
				
		/**
		 * Constructs three Radio buttons 
		 */
		AddTableElementWidgetView.prototype._constructRadioButtonOptions = function(){
			this._radioFormGroup = $("<div>").addClass("radioOptionForm");
			this._wrapper.append(this._radioFormGroup);
			
			// Preview Fields Radio Button - Preview Fields
			var previewRadioDiv = $("<div>").attr({"class":"radioButton", "selected":"selected"});
			this._radioFormGroup.append(previewRadioDiv);
            var previewRadioInput = $("<input>").attr({"name":"selectRadio", "value":"previewElementsOption", "type":"radio"});
            var previewRadioLabel = $("<label>").addClass("leftPadding").append(previewRadioInput).
            	append(this.webi18n.t("ADD_PREVIEW_FIELDS", this._previewName));
            previewRadioDiv.append(previewRadioLabel);
            
            // Register on click event for preview radio button
            previewRadioInput.on("click", function(){
            	this._constructPreviewOptionDetails(previewRadioDiv);
            }.bind(this));
            
            if(this._previewName == null)
            	previewRadioDiv.hide();
			                       
            this._constructPreviewOptionDetails(previewRadioDiv);
            
            this._radioFormGroup.append($("<div>").addClass("emptyDiv"));
                        
			// Radio Button for gallery options.
			var galleryRadioDiv = $("<div>").addClass("radioButton");
			this._radioFormGroup.append(galleryRadioDiv);
            var galleryRadioInput = $("<input>").attr({"name":"selectRadio", "value":"galleryElementsOption", "type":"radio"});
            var galleryRadioLabel = $("<label>").addClass("leftPadding").append(galleryRadioInput).append(this.webi18n.t("ADD_LISTED_FIELD"));
            galleryRadioDiv.append(galleryRadioLabel);
            
            // Register on click event for gallery radio button
            galleryRadioInput.on("click", function(){
            	this._constructGalleryOptionDetails(galleryRadioDiv);            	            	
            }.bind(this));
            this._constructGalleryOptionDetails(galleryRadioDiv);
            
            this._radioFormGroup.append($("<div>").addClass("emptyDiv"));
            
            // Radio Button for custom options
            var customRadioDiv = $("<div>").addClass("radioButton");
            this._radioFormGroup.append(customRadioDiv);
            var customRadioInput = $("<input>").attr({"name":"selectRadio", "value":"customElementOption", "type":"radio"}); // by default set it to true, so this is selected
            var customRadioLabel = $("<label>").addClass("leftPadding").append(customRadioInput).append(this.webi18n.t("ADD_NEW_CUSTOM_FIELD"));
            customRadioDiv.append(customRadioLabel);
            
            // Register onclick event for custom radio button
            customRadioInput.on("click", function(){
            	this._constructCustomOptionDetails(customRadioDiv);            	
            }.bind(this));

            this._constructCustomOptionDetails(customRadioDiv); 
            
			if(this._previewName){	
				this._previewOptionDiv.show();
				previewRadioInput.attr({"checked":"checked"});
			}else{
				this._galleryOptionDiv.show();
				galleryRadioInput.attr({"checked":"checked"});
			}
			this._customOptionDiv.hide();
           
		};
		
		
		/**
		 * Cunstruct details for the preview options and manage visibility for all options.
		 * 
		 * @param radioDiv
		 */
		AddTableElementWidgetView.prototype._constructPreviewOptionDetails = function(previewRadioDiv){
			if(this._customOptionDiv ){
				this._customOptionDiv.hide();
			}
			
			if(this._galleryOptionDiv){
				this._galleryOptionDiv.hide();
			}
			
			if(this._previewOptionDiv){
				this._previewOptionDiv.show();
				return;
			}
			
			this._previewOptionDiv = $("<div>").addClass("radioOptions");
			previewRadioDiv.append(this._previewOptionDiv);
						
			// Add MultiSelect List
			this._previewOptionDiv.append(this._multiSelectListPreviewOptionDiv);
			
			this._multiSelectPreviewOption.render();
		};
		
		/**
		 * Cunstruct details for the gallery options and manage visibility for all options.
		 * 
		 * @param radioDiv
		 */
		AddTableElementWidgetView.prototype._constructGalleryOptionDetails = function(radioDiv){
			if(this._customOptionDiv){
				this._customOptionDiv.hide();
			}
			
			if(this._previewOptionDiv){
				this._previewOptionDiv.hide();
			}
			
			if(this._galleryOptionDiv){
				this._galleryOptionDiv.show();
				return;
			}
			
			this._galleryOptionDiv = $("<div>").addClass("radioOptions");
			radioDiv.append(this._galleryOptionDiv);
						
			// Add MultiSelect List
			this._galleryOptionDiv.append(this._mulitSelectListGalleryOptionDiv);

			this._multiSelectGalleryOption.render();
			
			// Description of the element type selected in library
			this._descriptionDiv = $("<div>").addClass("elementDescription");
			var ballonIcon = $("<i>").attr({"class":"icon icon-WCR_Info-Balloon icon-normal", "title":"help"});
			this._descriptionDiv.append(ballonIcon);
			this._galleryOptionDiv.append(this._descriptionDiv);
			this._descriptionDiv.hide();
		};
		
		/**
		 * Cunstruct details for the custom options and manage visibility for all options.
		 * @param radioDiv
		 */
		AddTableElementWidgetView.prototype._constructCustomOptionDetails = function(radioDiv){
			if(this._galleryOptionDiv){
				this._galleryOptionDiv.hide();
			}
			
			if(this._previewOptionDiv){
				this._previewOptionDiv.hide();
			}
			
			if(this._customOptionDiv){
        		this._customOptionDiv.show();
        		return;
        	}
			
			this._customOptionDiv = $("<div>").addClass("radioOptions");
			radioDiv.append(this._customOptionDiv);
			
			var leftDiv = $("<div>").addClass("leftDiv");
			var rightDiv = $("<div>").addClass("rightDiv");
			
			this._customOptionDiv.append(leftDiv);
			this._customOptionDiv.append(rightDiv);
			this._customOptionDiv.append($("<div>").css({"clear":"both"}));
			
			var codeDiv = $("<div>").addClass("inputDivs");
			leftDiv.append(codeDiv);
			var codeInput = $("<input>", {"type":"text", "class":"inputFieldSize", "name":"customElementType" });
			var codeLabel = $("<label>", {"class":"secondOptionsLabel"});
			codeLabel.append(this.webi18n.t("ELEMENT_TYPE_CODE"));
			codeDiv.append(codeLabel);
			codeDiv.append(codeInput);

			var insSeqDiv = $("<div>").addClass("inputDivs");
			leftDiv.append(insSeqDiv);
			var insSeqInput = $("<input>", {"type":"text", "class":"inputFieldSize", "name":"instanceSeq"});
			var insSeqLabel = $("<label>", {"class":"secondOptionsLabel"});
			insSeqLabel.append(this.webi18n.t("INSTANCE_SEQUENCE"));
			insSeqDiv.append(insSeqLabel);
			insSeqDiv.append(insSeqInput);
			
			this._errorMessageDiv = $("<div>").addClass("inputDivs");
			this._errorMessageDiv.hide();
			leftDiv.append(this._errorMessageDiv);
			
			//Create Select options on right div
			var selectDiv = $("<div>").addClass("customLabel");
			rightDiv.append(selectDiv);
			
			this._selectElement = $("<select>").attr({"name" : "addedElementList", "size" : "10"});
			selectDiv.append(this._selectElement);
			
			// Update all selected option in select.
			this._updateSelectedForCustom(false);
			
			//Create Add Field Button
			var buttonDiv = $("<div>").addClass("inputDivs");
			leftDiv.append(buttonDiv);
			var button = $("<button>", {'type':'button', 'class':'btn btn-primary secondOptionsFloatLeft'});
			button.append(this.webi18n.t("ADD_FIELD"));
			buttonDiv.append(button);
			
			button.click(function() {
				// make sure to hide error message if there was one showing
				this._errorMessageDiv.hide();
				var elementType = codeInput.val();
				var instanceId = insSeqInput.val() == "" ? 1 : insSeqInput.val();
				var canAdd = true;
				
				//Check if the instance id is numeric number
				var reg = new RegExp(/^\d+$/);
				canAdd = reg.test(instanceId);
				
				// If instance Id is not a numeric number, dont add the element and throw error
				if (canAdd) {
					_.each(this._onAddCustomField, _.bind(function(callback) {
						if(elementType.trim() != "") {
							canAdd = callback(elementType, instanceId);
						} else {
							canAdd = false; // Element Type was not provided
						}
					}, this));
				} else {
					this._showErrorMessage(this.webi18n.t("ADD_ELEMENT_ERROR1"));
					return;
				}
				
				if (canAdd) {
					var id = elementType + "_" + instanceId;
					var label = instanceId == '1' ? elementType : elementType + " (" + instanceId + ")";
					
					this._selectedElementsFilter.push({
						id: id,
						label: label,
						selected: true
					});
					
					// Sort the list based on label field
					this._selectedElementsFilter = _.sortBy(this._selectedElementsFilter, 'label');
					
					// Render selected list for preview with sorted selected elements.
					this._multiSelectPreviewOption.render();
					
					// Update selected elements list for gallery option.
					this._multiSelectGalleryOption.render();
					
					// Update selected elements list for custom option
					this._updateSelectedForCustom(id);
				}else{
					this._showErrorMessage(this.webi18n.t("ADD_ELEMENT_ERROR2"));
					return;
				}
			}.bind(this));
		};
		
		
		AddTableElementWidgetView.prototype._showErrorMessage = function(errorMessage){
			// Empty the error div before add contents to it
			this._errorMessageDiv.empty();
			var iconClass = $("<i>").addClass("icon icon-WCR_Msg-Error");
			iconClass.css({"padding-right":"5px"});
			this._errorMessageDiv.append(iconClass, errorMessage);
			this._errorMessageDiv.show();
		};
		
		/**
		 * Add footer OK and Cancel Buttons
		 */
		AddTableElementWidgetView.prototype.getModalFooterJQElement = function(modal){
			var okButton = $("<button>", {'class': 'btn btn-primary'}).text(this.webi18n.t("PCM_OK"));
			var cancelButton = $("<button>", {'class': 'btn btn-default'}).text(this.webi18n.t("PCM_CANCEL"));
			
			var buttonsDiv = $("<div>", {"id":"buttonsDiv"});
			buttonsDiv.append(okButton, cancelButton);
			
			cancelButton.on("click", _.bind(function(){
				_.each(this._onCancel, _.bind(function(callback) {
					callback();
				}, this));
				
				modal.close();
			}, this));
			
			okButton.on("click", _.bind(function(){
				// Just close the modal, and the rest of the processing is done on "Onclose" event that is added in TableContentLanguageWidget
				modal.close();
			}));
			
			return buttonsDiv;
		};
		
		/**
		 * Register callbacks that should be executed when an element is selected 
		 * @param callback
		 * @returns
		 */
		AddTableElementWidgetView.prototype.onEntrySelected = function (callback) {
			return this._onEntrySelected.push(callback);
		};
		
		/**
		 * Register callbacks that should be executed when an element is deselected 
		 * @param callback
		 * @returns
		 */
		AddTableElementWidgetView.prototype.onEntryDeselected = function (callback) {
			return this._onEntryDeselected.push(callback);
		};

		/**
		 * Register callbacks that should be executed when an element added - Custom Option (Add Custom Element Type) 
		 * @param callback
		 * @returns
		 */
		AddTableElementWidgetView.prototype.onAddCustomField = function (callback) {
			return this._onAddCustomField.push(callback);
		};
		
		/**
		 * Register callbacks for cancel
		 * @param callback
		 */
		AddTableElementWidgetView.prototype.onCancel = function (callback) {
			return this._onCancel.push(callback);
		};
		
		return AddTableElementWidgetView;
});