define(["underscore",
        "jquery",
        "./StateFullContentWidget",
        "component/tooltip/Tooltip",
        "../../ContentConstants",
        "../controller/AddTableElementWidget",
        "utils/webi18n!TextContentStrings",
        "utils/webcss!./TableContentLanguageWidget.css",
        "utils/webcss!../../text-content-element-widget/TextContentElementWidgetView.css"],
	function (_, 
			$,
			StateFullContentWidget,
			Tooltip,
			ContentConstants,
			AddTableElementWidget,
			TextContentStrings) {
		
	"use strict";
	
	var TableContentLanguageWidgetView = function (currentState, nextStates, languageCode, isContentManagerView) {
		this._currentState = currentState;
		this._nextStates = nextStates;
		this._languageCode = languageCode;
		this._isContentManagerView = isContentManagerView;
		this._onAddElementCallBack = [];
		this._onAdminCallBack = [];
		this._onExitAdminCallBack = [];
		this._textContentElementWidgets = [];
				
		// Create the wrapping div of this widget.
		this._wrapper = $("<div>").addClass("TableContentLanguageWidget");

		this._containerFluid = $("<div>").addClass("container-fluid");
		
		this._wrapper.append(this._containerFluid);
		
		this._stateFullContentWidget = new StateFullContentWidget(this._currentState, this._nextStates);
		
		// Prepare gear icon's only if this is content manager view
		if(this._isContentManagerView){
			this._constructStateFullContentWidgetActions();
		}
		
		this._stateFullContentWidget.setMainContent(this._wrapper);
		
		// Keep the widget collapsed by default
		this._stateFullContentWidget.collapse();
		
	};
	
	/**
	 * Add calls backs when addelement is done
	 * @param callback
	 */
	TableContentLanguageWidgetView.prototype.onAddElement = function(callback) {
		this._onAddElementCallBack.push(callback);
	};
	
	/**
	 * Create Gear icon for settings. This will be added to statefullcontentwidget
	 */
	TableContentLanguageWidgetView.prototype._constructStateFullContentWidgetActions = function(){
		// Add gear icons 
		var adminIcon = $("<i>", {"class" : "icon icon-WCR_Menu-Admin"});
		this._adminButton = $("<div>", {"class": "CustomButton"}).append(adminIcon);
		
		// Create the tooltip for the admin button.
		
		new Tooltip(undefined, 
				this._adminButton, 
				this._stateFullContentWidget.getJQElement(), 
				$("body"), 
				false, 
				TextContentStrings.t("CONFIGURE_TT"), 
				undefined, undefined, {});

		var addElementIcon = $("<i>", {"class" : "icon icon-WCR_Text-AddField"});
		this._addElementButton = $("<div>", {"class": "CustomButton"}).append(addElementIcon);
		this._addElementButton.hide();
		
		// Create the tooltip for the Add/Remove button.
		new Tooltip(undefined, 
				this._addElementButton, 
				this._stateFullContentWidget.getJQElement(), 
				$("body"), 
				false, 
				TextContentStrings.t("CONFIGURE_ADD_REMOVE"), 
				undefined, undefined, {});
		
		this._adminButton.click(function (event) {
			if (adminIcon.hasClass("icon-WCR_Menu-Admin")) {
				// Remove all classes and add class to show exit admin symbol.
				adminIcon.removeClass();
				adminIcon.addClass("icon icon-WCR_Menu-AdminExit2");
				
				// Show add elements button.
				this._addElementButton.show();

				// Execute all the call backs. 
				_.each(this._onAdminCallBack, _.bind(function(callback) {
					callback();
				}, this));
				
				// Expand the widget.
				this._stateFullContentWidget.expand();
				
			} else {
				// Remove all classes and add class to show admin mode symbol.
				adminIcon.removeClass();
				adminIcon.addClass("icon icon-WCR_Menu-Admin");
				adminIcon.attr("title", TextContentStrings.t("ADMIN_MODE"));
				
				// Hide add elements button.
				this._addElementButton.hide();
				
				// Execute all the call backs. 
				_.each(this._onExitAdminCallBack, _.bind(function(callback) {
					callback();
				}, this));
				
				// Collapse the widget.
				this._stateFullContentWidget.collapse();
			}
			
			event.stopPropagation();
		}.bind(this));
		
		this._addElementButton.click(function(event){
			for (var i=0; i < this._onAddElementCallBack.length; i++) {
				this._onAddElementCallBack[i]();
			}
			
			event.stopPropagation();
		}.bind(this));
		
		// Add admin and add element buttons to header of the widgets.
		this._stateFullContentWidget.addCustomButton(this._adminButton);
		this._stateFullContentWidget.addCustomButton(this._addElementButton);
	};
	
	/**
	 * Register callback for changing state of the language widget.
	 */
	TableContentLanguageWidgetView.prototype.registerBulkStateChangeHandler = function (callback) {
		this._stateFullContentWidget.onNewState(callback);
	};
	
	/**
	 * Register call back for reverting approval action.
	 */
	TableContentLanguageWidgetView.prototype.registerBulkRevertHandler = function(callback) {
		this._stateFullContentWidget.onRevertState(callback);
	};
	
	/**
	 * Prepare the div for Header which contains caret, flag and language description. Add this header to statefullContentWidget
	 * @param language
	 * @param isMaster
	 */
	TableContentLanguageWidgetView.prototype.constructHeaderDiv = function(language, isMaster) {
		
		this._language = language;
		this._isMaster = isMaster;
		
		var languageLabel = null;
		var languageCode = null;
		
		// create language description div to header
		var languageHeaderDiv = $("<div>", {"class": "languageDescriptionHeader floatLeft", "id" : "languageDescDiv"});
		
		// Update the language. Evaluate the language description or language code to show in header. Add flag and language details to language div
		if (this._language) {
			if (this._language.langDesc) {
				languageLabel = this._language.langDesc;
				languageCode = this._language.langCode;
			} else if (this._language.langCode) {
				languageLabel = this._language.langCode;
				languageCode = this._language.langCode;
			} else {
				languageLabel = this._languageCode;
				languageCode = this._languageCode;
			}
			
			languageHeaderDiv.empty();
			if(this._isMaster) {
				languageHeaderDiv.append($('<span>', {"class" : "LangName Master"}).text(languageLabel));
			} else {
				languageHeaderDiv.append($('<span>', {"class" : "LangName"}).text(languageLabel));
			}
			
			if(this._language.langFlag) {
				languageHeaderDiv.prepend($('<img>', {"class" : "Flag"}).attr({"src": "images/country_flags/" + this._language.langFlag}));
			}
		}
		
		// Create wrapper header div that holds, caret, flag and language
		this._headerDiv = $("<div>").addClass("tableContentHeaderDiv").attr({"id":"tableContentHeaderDiv"+languageCode});
		
		// Create caret icon for expand and collapse
		var caretRightIcon = $("<i>", {"class" : "icon icon-WCR_Menu-Caret-Right"});
		this._caretRightIconDiv = $("<div>").addClass("floatLeft caretIcon");
		this._caretRightIconDiv.append(caretRightIcon);
		
		var caretDownIcon = $("<i>", {"class" : "icon icon-WCR_Menu-Caret-Down"});
		this._caretDownIconDiv = $("<div>").addClass("floatLeft caretIcon");
		this._caretDownIconDiv.append(caretDownIcon);
		
		// Add both caret icons to wrapper header div
		this._headerDiv.append(this._caretRightIconDiv).append(this._caretDownIconDiv);
		// Hide the down caret, as it is show only when the state full widget is expanded
		this._caretDownIconDiv.hide();
		
		// Now that the caret is added, add the language div that contains flag and language
		this._headerDiv.append(languageHeaderDiv);
		
		// Add wrapper header div to statefullwidget
		this._stateFullContentWidget.setHeaderContent(this._headerDiv);
		
		//Add onclick for tableContentHeaderDiv parent headerdiv 
		var parentDiv = $("#tableContentHeaderDiv"+languageCode).parent();
		parentDiv.css({"cursor":"pointer"});
		parentDiv.on("click", _.bind(function (event) {
			this._toggleVisibility();	
			event.stopPropagation();
		}, this));
		
		this.onHeaderClick();
	};
	
	/**
	 * Method is to manage visibility of state full widget.
	 */
	TableContentLanguageWidgetView.prototype._toggleVisibility = function () {
		if (this._stateFullContentWidget.isExpanded()) {
			this.collapse();
		} else {
			this.expand();
		}
	};
	
	TableContentLanguageWidgetView.prototype.onHeaderClick = function () {
		this._stateFullContentWidget.onHeaderClick(_.bind(function () {
			this._toggleVisibility();		 
		}, this));
	};
	
	/**
	 * Creates one new row and adds one textContent Widget to new row
	 * @param textContentElementWidget
	 * @param elementType
	 * @param isSingleRowElement
	 */
	TableContentLanguageWidgetView.prototype.createRow = function (textContentElementWidget, elementType, instanceId, isSingleRowElement, elementGroupDiv) {
		
		// Remember the text content widget.
		this._textContentElementWidgets.push(textContentElementWidget);
		
		var row = $("<div>").addClass("row rowContent");
		row.append(divRowContent);
		var insId = instanceId == "1" ? "" : " ("+instanceId+")";
		var elementTypeDiv = $("<div>").addClass("elementType").html(elementType + "<b>"+insId+"</b>");
		
		var divRowContent = isSingleRowElement ? $("<div>").addClass("col-xs-12 col-md-12") : $("<div>").addClass("col-xs-12 col-md-6");
		divRowContent.append(elementTypeDiv);
		divRowContent.append(textContentElementWidget.getJQElement());
		
		row.append(divRowContent);
		elementGroupDiv.append(row);
		
		return row;
		
	};
	
	/**
	 * Appends textContent Widget to existing row
	 * @param textContentElementWidget
	 * @param elementType - Element type that will be displayed on top of textcontentelementwidget
	 * @param row - Row reference where the new div element should be appended
	 */
	TableContentLanguageWidgetView.prototype.appendToRow = function (textContentElementWidget, elementType, instanceId, row) {
		
		// Remember the text content widget.
		this._textContentElementWidgets.push(textContentElementWidget);
		
		var insId = instanceId == "1" ? "" : " ("+instanceId+")";
		var divRowContent = $("<div>").addClass("col-xs-12 col-md-6");
		var elementTypeDiv = $("<div>").addClass("elementType").html(elementType + "<b>"+insId+"</b>");
		divRowContent.append(elementTypeDiv);
		divRowContent.append(textContentElementWidget.getJQElement());
		row.append(divRowContent);
	};
	
	/**
	 * Create Elements group where Table Text Contents of same category are grouped
	 * @param groupName
	 * @returns
	 */
	TableContentLanguageWidgetView.prototype.constructElementGroupDiv = function (groupName) {
		var divGroup = $("<div>").addClass("elementGroup");
		this._containerFluid.append(divGroup);		
		return divGroup;
	};
	
	/**
	 * Set the state of statefull content widget
	 * @param currentState
	 * @param nextStates
	 */
	TableContentLanguageWidgetView.prototype.setNextStates = function (currentState, nextStates) {
		this._currentState = currentState;
		this._nextStates = nextStates;
		this._stateFullContentWidget.update(currentState, nextStates);
	};
	
	/**
	 * Expand the widget.
	 */
	TableContentLanguageWidgetView.prototype.expand = function () {
		this._stateFullContentWidget.expand();
		this._caretRightIconDiv.hide();
		this._caretDownIconDiv.show();
		// Try to show the text content widgets as a single line (now that they're visible).
		$.each(this._textContentElementWidgets, function (index, elementWidget) {
			window.setTimeout(function () {
				elementWidget.trySingleLine();
			}, 50);
		}.bind(this));
	};
	
	/**
	 * Collapse the widget.
	 */
	TableContentLanguageWidgetView.prototype.collapse = function () {
		this._stateFullContentWidget.collapse();
		this._caretRightIconDiv.show();
		this._caretDownIconDiv.hide();
	};
	
	/**
	 * Prepare the Delete button for every content widget, and returns the button
	 * @returns button
	 */
	TableContentLanguageWidgetView.prototype.constructDeleteButtonForElementWidget = function () {
		var button = $("<div>", {"class" : "TextContent-ActionTable"});
        button.append($("<i>", {"class" : "icon icon-WCR_Trash"}));
        
        return button;
	};
	
	/**
	 * Register callbacks that should be executed when the Admin button is clicked
	 * @param callback
	 * @returns
	 */
	TableContentLanguageWidgetView.prototype.onAdminButton = function (callback) {
		return this._onAdminCallBack.push(callback);
	};
	
	/**
	 * Register callbacks that should be executed when the Exit Admin button is clicked
	 * @param callback
	 * @returns
	 */
	TableContentLanguageWidgetView.prototype.onExitAdmin = function (callback) {
		return this._onExitAdminCallBack.push(callback);
	};
	
	TableContentLanguageWidgetView.prototype.empty = function () {
		this._containerFluid.empty();
	};
	
	/**
	 * Enable widgets.
	 */
	TableContentLanguageWidgetView.prototype.enable = function() {
		this._stateFullContentWidget.update(this._currentState, this._nextStates);
	};
	
	/**
	 * Disable widgets.
	 */
	TableContentLanguageWidgetView.prototype.disable = function(msgTag) {
		this._stateFullContentWidget.disable(msgTag);
	};
	
	/**
	 * Set new status for the state full widget.
	 * @param newState
	 */
	TableContentLanguageWidgetView.prototype.setCurrentState = function(currentState) {
		this._stateFullContentWidget.setCurrentState(currentState);
	};
	
	/**
	 * Get the wrapping jQuery element of the table page.
	 */
	TableContentLanguageWidgetView.prototype.getJQElement = function () {
		return this._stateFullContentWidget.getJQElement();
	};
		
	return TableContentLanguageWidgetView;
});