define(["underscore",
        "jquery",
        "../../ContentConstants",
        "utils/webi18n!TextContentStrings",
        "stickykit",
        "utils/webcss!./TableContentPageView.css"],
	function (_, 
			$,
			ContentConstants,
			TextContentStrings) {
		
		
	var TableContentPageView = function (tableSettingsWidget, previewSelector, previewWidget, parent) {
				
		// Construct the skeleton.
		this._container = $("<div>", {"class" : "container-fluid"});
		this._wrapper = $("<div>", {"class" : "TableContentPage row"});
		
		this._leftCol = $("<div>", {"class" : "col-xs-12 col-sm-6 col-md-7 col-lg-8"});
		this._rightCol = $("<div>", {"class" : "col-xs-12 col-sm-6 col-md-5 col-lg-4"});
		this._wrapper.append(this._leftCol, this._rightCol, this._clearFix);
		this._container.append(this._wrapper);
		
		// Add to the left column.
		// Table content setting widget
		this._leftCol.append(tableSettingsWidget.getJQElement());
		
		// Add to the right column.
		this._rightCol.append(previewSelector.getJQElement());
		this._rightCol.append(previewWidget.getJQElement());
		
		// Set the parent.
		previewWidget.setParent(this._rightCol);
		
		// Make the widget sticky.
		window.setTimeout(function () {
			previewWidget.getJQElement().stick_in_parent({
				parent: parent
			});
			previewWidget.getJQElement().on('sticky_kit:bottom', function(e) {
			    $(this).parent().css('position', 'static');
			});
			previewWidget.getJQElement().on('sticky_kit:unbottom', function(e) {
			    $(this).parent().css('position', 'relative');
			});
		}, 1000);
		
	};
	
	/**
	 * Add the language widgets to the page
	 * 
	 * @param languageWidgets
	 */
	TableContentPageView.prototype.addAllLanguageWidgets = function (languageWidgets) {
		
		var allWidgets = {};
		allWidgets.master = [];
		allWidgets.nonprojmaster = [];
		allWidgets.translation = [];
		allWidgets.nonprojtrans = [];
		
		// GroupBy master or translation widget(s)
		$.each(languageWidgets, function(index, widget) {
			if(widget.isMaster()) {
				if(widget.isProjectLanguage()) {
					allWidgets.master.push(widget);
				} else {
					allWidgets.nonprojmaster.push(widget);
				}
			} else {
				if(widget.isProjectLanguage()) {
					allWidgets.translation.push(widget);
				} else {
					allWidgets.nonprojtrans.push(widget);
				}
			}
		}.bind(this));
		
		// Create separate sections for master and translations
		this._masterLangSection = $("<div>", {"class": "masterlang_section"});
		this._transLangSection = $("<div>", {"class": "translang_section"});
		
		// Create separate sections for non-project master and translation languages
		this._nonProjLangSection = $("<div>", {"class": "nonproj_translang_section"});
		
		// Add the sections to the leftcol
		this._leftCol.append(
				this._masterLangSection, 
				this._transLangSection,
				this._nonProjLangSection
				);
		
		// Master Language widgets
		$.each(allWidgets.master, function(index, widget) {
			this._masterLangSection.append(widget.getJQElement());
		}.bind(this));
		
		// Translation Language widgets
		$.each(allWidgets.translation, function(index, widget) {
			this._transLangSection.append(widget.getJQElement());
		}.bind(this));
		
		var nonProjLangWidgets = allWidgets.nonprojmaster.concat(allWidgets.nonprojtrans);
		this._addWidgetsToNonProjSection(this._nonProjLangSection, nonProjLangWidgets);
		
		
	};
	
	TableContentPageView.prototype._addWidgetsToNonProjSection = function (section, widgets) {
		if(widgets.length > 0) {
			// Add all language statement which language does not exists in project languages.
			var expandNonProjLink = $("<div>", {"class": "ShowHideLink"});
			var expandNonProjSymbol = $("<i>", {"class" : "icon icon-WCR_Menu-Caret-Right"});
			var expandNonProjLabel = $("<span>").text(TextContentStrings.t("PCM_SHOW_NON_PROJ_LANG"));
			expandNonProjLink.append(expandNonProjSymbol, expandNonProjLabel);
			
			var widgetsWrapper = $("<div>").hide();
			
			expandNonProjLink.on("click", _.bind(function () {
				if (widgetsWrapper.is(":hidden")) {
					widgetsWrapper.show();
					expandNonProjSymbol.removeClass();
					expandNonProjSymbol.addClass("icon icon-WCR_Menu-Caret-Down");
					expandNonProjLabel.text(TextContentStrings.t("PCM_HIDE_NON_PROJ_LANG"));
				} else {
					widgetsWrapper.hide();
					expandNonProjSymbol.removeClass();
					expandNonProjSymbol.addClass("icon icon-WCR_Menu-Caret-Right");
					expandNonProjLabel.text(TextContentStrings.t("PCM_SHOW_NON_PROJ_LANG"));
				}
			}, this));
			
			section.append(expandNonProjLink);
			section.append(widgetsWrapper);
			
			// Language widgets
			$.each(widgets, function(index, widget) {
				widgetsWrapper.append(widget.getJQElement());
			}.bind(this));
		}
		
	};
	
	TableContentPageView.prototype.addLanguageWidget = function (widget) {
		if(widget.isMaster()) {
			this._masterLangSection.append(widget.getJQElement());
		} else {
			this._transLangSection.append(widget.getJQElement());
		}
	};
	
	/**
	 * Get the wrapping jQuery element of the table page.
	 */
	TableContentPageView.prototype.getJQElement = function () {
		return this._container;
	};
	
	return TableContentPageView;
});