define(["jquery",
        "Template",
        "tpl!packaging-content-management/table-content-page/view/TableContentPreviewSelectorView",
        "utils/webi18n!TextContentStrings",
        "component/modal/Modal",
        "component/tooltip/Tooltip",
        "../../language-indicator/LanguageSelector",
        "utils/webi18n!TextContentStrings",
        "utils/webcss!./TableContentPreviewSelectorView.css"],
	function ($, Template, tpl, webi18n, Modal, Tooltip, LanguageSelector, TextContentStrings) {
	
	/**
	 * The view of the TableContentPreview.
	 */
	var TableContentPreviewSelectorView = function (previewEntryMap, selectedPreview, projectId) {
		
		// Remember the projectId
		this._projectId = projectId;
		this._previewEntryMap = previewEntryMap;
		this._selectedPreview = selectedPreview;
		this._selectedPreviewSettings;
		if (selectedPreview) {
			this.selectedPreviewName = selectedPreview.name;
		} else {
			this.selectedPreviewName = undefined;
		}
		
		// The callbacks to be executed when a preview is selected.
		this._onNewPreviewSettingsCallbacks = [];
		this._onNewPreviewTypeSelectedCallbacks = [];
		this._onNewPreviewNameSelectedCallbacks = [];
		
		
		// Create the view.
		this._wrapper = $("<div>", {"class" : "TableContentPreviewSelector"});
		this._selectorButton = $("<div>", {"class" : "PreviewButton"});
		this._selectorButtonIcon = $("<i>", {"class" : "icon icon-WCR_Menu-Admin"});
		this._selectorButton.append(this._selectorButtonIcon);
		
		// Create a tooltip for the button.
		new Tooltip(undefined, 
				this._selectorButton, 
				$("body"), 
				$("body"), 
				false, 
				TextContentStrings.t("CONFIGURE_PREVIEW_TT"));

		// Open the modal
		this._selectorButton.on("click", function () {
			this.openSettingsModal();
		}.bind(this));
		
		// Add the prefix
		this._previewPrefix = $("<div>", {"class" : "PreviewPrefix"});
		this._previewPrefix.append($("<span>").text(webi18n.t("PCM_PREVIEW_PREFIX")));
		
		// Add the info link
		var infoLink = $('<a/>', {
			'class':'inline_help', 
			'href': 'help.jsp?topic=common/wc/concept/co_wc_nutrition_facts_preview&lang=en',
			'onclick' : "var pop = window.open('help.jsp?topic=common/wc/concept/co_wc_nutrition_facts_preview&lang=en', 'help', 'scrollbars=1, width=800 ,height=500, toolbar=yes, location=yes, menu=yes'); pop.focus(); return false;"});
		var infoSymbol = $("<i>", {
			'class' : 'icon icon-WCR_Info-Balloon icon-normal',
			'title': 'Help'
		});
		this._previewPrefix.append(infoLink.append(infoSymbol));
		
		// The name of the preview
		this._previewName = $("<div>", {"class" : "PreviewName"});
		this._previewName.append($("<div>").text(""));
		
		// Add it all to the wrapper.
		this._wrapper.append(this._previewPrefix);
		this._wrapper.append(this._previewName);
		this._wrapper.append(this._selectorButton);
	};
	
	/**
	 * Create the modal.
	 */
	TableContentPreviewSelectorView.prototype.createSettingsModal = function () {
		if (!this._modal) {		
			// Create the content of the modal.
			this._content = $("<div>");
			// Create the template
			this._t = new Template(webi18n, tpl);
			// Set the data.
			this._t.d.projectId = this._projectId;
			this._t.d.selectedPreview = this._selectedPreview;
			this._t.d.previewEntryMap = this._previewEntryMap;
			
			if(this._selectedPreview.name=="Default"){
				this._t.d.settings = "Default";
			}else{
				this._t.d.settings = this._selectedPreviewSettings;
			}
			
			// Add the template to the element.
			this._t.setContext(this._content);
			// Render the template
			this._t.render();
			// Init the language selectors.
			$.each(this._content.find(".LanguageSelector"), function (index, select) {
					LanguageSelector.wcrinit(select);
			}.bind(this));
			
			// Bind the inputs.
			this._t.bind("change", "select[name='PreviewType']", function(source, event){
				var type = event.target.value;
				for (var i=0; i < this._onNewPreviewTypeSelectedCallbacks.length; i++) {
					this._onNewPreviewTypeSelectedCallbacks[i](type);
				}
			}.bind(this));
			
			// Bind the inputs for preview name/standard selected.
			this._t.bind("change", "select[name='PreviewName']", function(source, event){
				var name = event.target.value;
				var type = this._content.find("select[name='PreviewType']").val();
				for (var i=0; i < this._onNewPreviewNameSelectedCallbacks.length; i++) {
					this._onNewPreviewNameSelectedCallbacks[i](type, name);
				}
			}.bind(this));
			
			// Create the modal
			this._modal = new Modal(this._content, {});
			// The header.
			this._modal.setTitle(webi18n.t("PCM_PREVIEW_SETTINGS"));
			
			var applyButton = $("<button>", {"type" : "button", "class" : "btn btn-primary"}).text(webi18n.t("PCM_SAVE"));
			var cancelButton = $("<button>", {"type" : "button", "class" : "btn btn-default"}).text(webi18n.t("PCM_CANCEL"));
			
			applyButton.on("click", function () {
				// Update the settings.
				var localeMapping = {};
				var localeSelectors = this._content.find("select.LanguageSelector");
				var foundMapping = false;
				$.each(localeSelectors, function (index, localeSelector) {
					var locale = $(localeSelector).attr("data-locale");
					var langCode = $(localeSelector).val();
					if (langCode) {
						localeMapping[locale] = langCode;
						foundMapping = true;
					}
				}.bind(this));
				
				// If a mapping was found.
				if (foundMapping) {
					this._selectedPreviewSettings.localeMapping = localeMapping;
				}
				
				// Execute all the callbacks.
				$.each(this._onNewPreviewSettingsCallbacks, function (index, callback) {
					callback(this._selectedPreviewSettings);
				}.bind(this));
				
				// Close the modal.
				this._modal.close();
				
			}.bind(this));
			cancelButton.on("click", function () {
				this._modal.close();
			}.bind(this));
			// Set the footer
			this._modal.setFooter($("<div>").append(cancelButton, applyButton));
		}
	};
	
	/**
	 * Open the modal.
	 */
	TableContentPreviewSelectorView.prototype.openSettingsModal = function () {
		// If it doesn't exist, create it first.
		if (!this._modal) {
			this.createSettingsModal();
		}
		this._modal.open();
	};
	
	/**
	 * Update the modal.
	 */
	TableContentPreviewSelectorView.prototype.updateSettingsModal = function () {
		// If it doesn't exist, create it first.
		if (!this._modal) { 
			this.createSettingsModal();
		} else {
			this._t.d.previewEntryMap = this._previewEntryMap;
			this._t.d.selectedPreview = this._selectedPreview;
			if(this._selectedPreview.name=="Default"){
				this._t.d.settings = "Default";
			}else{
				this._t.d.settings = this._selectedPreviewSettings;
			}
			
			this._t.d.entryName = this._selectedPreviewEntryName;
			this._t.render();
			
			// Init the language selectors.
			$.each(this._content.find(".LanguageSelector"), function (index, select) {
				LanguageSelector.wcrinit(select);
			}.bind(this));
		}
	};
	
	
	/**
	 * Register the callbacks that should be called when the preview settings changed.
	 */
	TableContentPreviewSelectorView.prototype.onNewPreviewSettings = function (callback) {
		this._onNewPreviewSettingsCallbacks.push(callback);
	};
	
	
	/**
	 * Register a callback to be executed when a preview is selected.
	 */
	TableContentPreviewSelectorView.prototype.setPreviewName = function (name) {
		this._previewName.text(name);
	};
	
	/**
	 * Register a callback to be executed when a preview is selected.
	 */
	TableContentPreviewSelectorView.prototype.onNewPreviewTypeSelected = function (callback) {
		this._onNewPreviewTypeSelectedCallbacks.push(callback);
	};
	
	
	/**
	 * Register a callback to be executed when a preview name is selected.
	 */
	TableContentPreviewSelectorView.prototype.onNewPreviewNameSelected = function (callback) {
		this._onNewPreviewNameSelectedCallbacks.push(callback);
	};
	/**
	 * Register a callback to be executed when a preview name is selected.
	 */
	TableContentPreviewSelectorView.prototype.onNewPreviewNameSelected = function (callback) {
		this._onNewPreviewNameSelectedCallbacks.push(callback);
	};
	
	/**
	 * Set the selected preview settings.
	 */
	TableContentPreviewSelectorView.prototype.setSelectedPreviewSettings = function (selectedPreviewEntry, selectedPreviewSettings, selectedPreviewEntryName) {
		this._selectedPreviewSettings = selectedPreviewSettings;
		this._selectedPreview = selectedPreviewEntry;
		
		if(selectedPreviewEntryName){
			this._selectedPreviewEntryName = selectedPreviewEntryName;
		}else{
			this._selectedPreviewEntryName = selectedPreviewEntry.name;
		}
		
		this.updateSettingsModal();
	};
	
	/**
	 * Set the default locale mapping in the preview widget.
	 */
	TableContentPreviewSelectorView.prototype.setDefaultPreviewLocaleMapping = function (defaultLocaleMapping) {
		this._selectedPreviewSettings.localeMapping = defaultLocaleMapping;
		this.updateSettingsModal();
	};
	
	/**
	 * Get the wrapping jQuery element.
	 */ 
	TableContentPreviewSelectorView.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return TableContentPreviewSelectorView;
});
