define([ "jquery"],
		function($) {
	
	var PreviewLoader = function () {
		
	};
	
	/**
	 * Load all the preview entries.
	 * 
	 * @param callback
	 */
	PreviewLoader.prototype.loadPreviews = function (callback) {
		
		// The path to the list.xml with built in previews:
		var builtinPath = window.webcenterConfig.requireBaseUrl+"packaging-content-management/table-content-preview/previews/";
		
		// The path to the list.xml with custom in previews:
		var customPath = window.webcenterConfig.jspBaseUrl+"custom/pcm/table_layouts/";
		
		var d1 = $.Deferred();
		var d2 = $.Deferred();
		
		this.loadFrom(builtinPath, d1);
		this.loadFrom(customPath, d2);
		
		// Wait for both load requests to be resolved
		$.when( d1, d2 ).done(function ( previews1, previews2 ) {
			// Combine the two previews in the same list.
		    var previews = previews1.concat(previews2);
		    // Execute the callback.
		    callback(previews);
		});
	};
	
	/**
	 * Load the preview entries from a "list.xml" given at the specified path.
	 * 
	 * The entries will be resolved against the passed defferred.
	 * 
	 * @param path
	 * @param defferred
	 */
	PreviewLoader.prototype.loadFrom = function (rootListPath, defferred) {
		$.ajax({
			type: "GET",
			url: rootListPath+"list.xml",
			cache: false,
			dataType: "xml",
			success: function (xml) {
				var entries = [];
				
				// Iterate over all the layouts in the XML.
				var layouts = $(xml).find("layout");
				for (var i=0; i < layouts.length; i++) {
					
					// Extract the necessary information from the layout.
					var name = $(layouts[i]).find("name").text();
					var type = $(layouts[i]).find("type").text();
					var path = $(layouts[i]).find("path").text();
					
					// Make sure the necessary information is there in the layout.
					if (name && name.length > 0 && path && path.length > 0) {
						entries.push({
							name: name,
							type: type,
							path: rootListPath+path
						});
					}
				}
				
				// Resolve the parsed entries.
				defferred.resolve(entries);
				
			}.bind(this),
			error: function () {
				// The list.xml could not be found on that location, resolve an empty array.
				defferred.resolve([]);
			}
		});
	};
	
	return PreviewLoader;
});
