define([ "jquery", 
         "./HTMLParser"],
		function($, HTMLParser) {

			/**
			 * Create a preview processor.
			 */
			var PreviewProcessor = function(previewEntry, previewPath) {
				this._previewEntry = previewEntry;
				if (previewPath){
					this.previewPath = previewPath;
				}else{
					this.previewPath = this._previewEntry.path;
				}
				this._parsedHTML;
				this._settings;
			};

			/**
			 * 
			 * @param callback
			 */
			PreviewProcessor.prototype._process = function(callback) {
				// AJAX the HTML of the preview.
				var path = this.previewPath;
				$.ajax({
				    url:path,
				        type:'GET',
				        success: function(html){
				        	// Parse the incoming HTML
				        	var parser = new HTMLParser();
				        	this._parsedHTML = html;
				        	this._settings = this._processSettings(this._parsedHTML);
				        	// Execute the callback
				        	callback();
				        }.bind(this),
				        cache: false
				});
			};
			
			/**
			 * Extract the settings of the preview out of the HTML of the preview.
			 */
			PreviewProcessor.prototype._processSettings = function(html) {
				
				// This object will contain the settings of the preview.
				var settings = {};
				
				// The found locales in the HTML.
				var locales = [];
								
				// Find all the placeholders in the HTML.
				var all =  $(html).find(".Placeholder");
				
				// If there are placeholders without a "data-locale" attribute, we should let them fall back on locale "1".
				if (all.filter(":not([data-locale])")) {
					locales.push(1);
				}
				
				// Add the locale of the placeholder to the list of locales (if it's not already in there).
				var withLocale = all.filter("[data-locale]");
				$.each(withLocale, function(index, placeholder) {
					var locale = parseInt($(placeholder).attr("data-locale"));
					if (locale != NaN && $.inArray(locale, locales) < 0) {
						locales.push(locale);
					}
				});
				
				// Set the locales in the settings.
				settings.locales = locales;
				
				// If a preview doesn't have a type, assign it the custom type.
				if (!this._previewEntry.type || this._previewEntry.type.length == 0) {
					this._previewEntry.type = "Custom";
				}
				
				// Return the settings.
				return settings;
			};
			
			/**
			 * Get the settings and HTML of the preview in a callback.
			 */
			PreviewProcessor.prototype.getPreview = function(callback) {
				if (this._settings && this._parsedHTML) {
					callback(this._parsedHTML, this._settings);
				} else {
					this._process(function () {
						callback(this._parsedHTML, this._settings);
					}.bind(this));
				}
			};

			
			/**
			 * Get the settings of the preview in a callback.
			 */
			PreviewProcessor.prototype.getSettings = function(callback) {
				if (this._settings) {
					callback(this._settings);
				} else {
					this._process(function () {
						callback(this._settings);
					}.bind(this));
				}
			};

			/**
			 * Get the HTML of the preview in a callback.
			 */
			PreviewProcessor.prototype.getHTML = function(callback) {
				if (this._parsedHTML) {
					callback(this._parsedHTML);
				} else {
					this._process(function () {
						callback(this._parsedHTML);
					}.bind(this));
				}
			};

			return PreviewProcessor;
		});