define(["jquery", 
        "utils/webi18n!TextContentStrings",
        "component/tooltip/Tooltip",
        "wcr/webcenter_xml"], 
        function ($,
        		TextContentStrings,
        		Tooltip,
        		WCXML) {
	
	var SimpleEditor = function (content) {
		
		// The wrapping div of the editor.
		this._wrapper = $("<div>");
		
		this._editorWrapper = $('<div>', {"class" : "SimpleEditor"});
		this._input = $('<textarea>', {"class" : "SimpleTextArea"}).text(content.content);
		this._editorWrapper.append(this._input);
		this._wrapper.append(this._editorWrapper);
	};
	
	SimpleEditor.prototype.focus = function () {
		this._input.focus();
	};
	
	SimpleEditor.prototype.blur = function () {
		this._input.blur();
	};
	
	SimpleEditor.prototype.onFocus = function (callback) {
		this._input.on("focus", callback);
	};
	
	SimpleEditor.prototype.onBlur= function (callback) {
		this._input.on("blur", callback);
	};
	
	SimpleEditor.prototype.onChange = function (callback) {
		this._input.on("change keyup", callback);
	};
	
	SimpleEditor.prototype.enable = function () {
		this._input.removeProp("readonly");
		this._editorWrapper.removeClass("Disabled");
	};
	
	SimpleEditor.prototype.disable= function () {
		this._input.prop("readonly", "readonly");
		this._editorWrapper.addClass("Disabled");
	};
	
	SimpleEditor.prototype.setHeight = function (height) {
		// Don't do anything.
	};
	
	SimpleEditor.prototype.setContent = function (content) {
		this._input.text(content.content);
	};
	
	/**
	 * This is going to be called when the editor is shown in the page.
	 */
	SimpleEditor.prototype.shownInPage = function () {
		if (this._setMaxChar) {
			this.setMaxChar(this._setMaxChar);
		}
	};
	
	/**
	 * Set the maximum number of chars
	 * 
	 * @param {Number} - maxChar
	 * @param {Boolean} - isSingleLineMode
	 */
	SimpleEditor.prototype.setMaxChar = function (maxChar) {
		this._setMaxChar = maxChar;
		
		var encodedContent = WCXML.xmlencode(this.getContent().content);
		var actualText = $('<div>').html(encodedContent).text();
		var maxCharInt = parseInt(this._setMaxChar);

		if(! this._maxCharDiv){
			this._maxCharDiv = $('<div>', {"class" : "MaxCharIndicator"});
			this._wrapper.append(this._maxCharDiv);
		}

		if (parseInt(this._setMaxChar) < actualText.length) {

			if(!this._maxCharWarningIcon){
				this._maxCharWarningIcon = $("<i>", {"class": "icon icon-normal icon-WCR_Msg-Warning icon-orange", "name": "maxcharwarning"});
				this._maxCharDiv.append(this._maxCharWarningIcon);		
			}
			
			var msg = TextContentStrings.t("PCM_MAX_CHARS")+": " + actualText.length+"/"+maxCharInt;
			if (!this._maxCharWarningTT) {
				this._maxCharWarningTT = new Tooltip(undefined, this._maxCharWarningIcon, $("body"), $("body"), false, msg, undefined, undefined, {color: "#EBEBE4", cssClass : "TextContent-Restricted-Popover", restrictBorder : 8});
			} else {
				this._maxCharWarningTT.setTooltipContent(msg);
			}

			//Avoid overlap if scroll bar is displayed
			if(this.hasVerticalScroll()){
				this._maxCharDiv.css({"right": "25px"});
			} else {
				this._maxCharDiv.css({"right": "5px"});
			}

			this._maxCharWarningIcon.show();
		} else {
			//Hide the max warning icon if is shown already
			if(this._maxCharWarningIcon)
				this._maxCharWarningIcon.hide();
		}
	};
	
	SimpleEditor.prototype.getContent = function () {
		return {
			content: this._input.val()
		};
	};
	
	SimpleEditor.prototype.hasVerticalScroll = function () {
		return this._input[0].scrollHeight > this._input[0].clientHeight;
	};
	
	SimpleEditor.prototype.getJQElement = function () {
		return this._wrapper;
	};

	return SimpleEditor;
	
});