define(["underscore",
        "jquery",
        "./TextContentElementWidgetView",
        "../ContentConstants",
        "utils/webi18n!TextContentStrings"],
	function (_,
			$,
			TextContentElementWidgetView,
			ContentConstants,
			TextContentStrings) {
	
		"use strict";

		/**
		 * Create the TextContentElementWidgetViewerView.
		 * 
		 * @param {Object} text - The content object from the TextContentElementWidget represented by this widget.
		 * @param {Array} comments - The array of Comment objects from the TextContentElementWidget represented by this widget.
		 * @param {String} state - The state of the TextContentElementWidget represented by this widget.
		 * @param {String} language - The language object of the TextContentElementWidget represented by this widget.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} markedApprovalState - Approval action taken if any, Current status otherwise
		 * @param {String} textType - text type as "richtext" or "simple".
		 */
		var TextContentElementWidgetViewerView = function (text, isMaster, comments, state, newState, languageCode, 
				canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
				viewOptions, textType) {
			
			// This view always supports single line mode.
			viewOptions.supportSingleLineMode = false;
			
			// By default, the height of the widget is 76px.
			viewOptions.defaultContentHeight  = 76;
			
			// call the parent prototype object to inherit properties
			TextContentElementWidgetView.call(this, text, isMaster, comments, state, newState, languageCode, 
					canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
					viewOptions, textType);
			
			// Do not show tooltips in the viewer.
			this._showTooltips = false;
			
			// call the overridden function to render the doctablex view of the widget
			this._constructView();
		};
		
		TextContentElementWidgetViewerView.prototype = Object.create(TextContentElementWidgetView.prototype);
		TextContentElementWidgetViewerView.prototype.constructor = TextContentElementWidgetViewerView;
		TextContentElementWidgetViewerView.prototype.parent = TextContentElementWidgetView.prototype;
		
		/**
		 * First override the content height before executing the initializeDOMElements method.
		 */
		TextContentElementWidgetViewerView.prototype._initializeDOMElements = function () {
			this._contentHeight = 76;
			TextContentElementWidgetView.prototype._initializeDOMElements.call(this);
		};
		
		/**
		 * Construct the HTML elements of this view.
		 */
		TextContentElementWidgetViewerView.prototype._constructView = function () {
			this._wrapper.empty();
			this._wrapper.addClass("Viewer");
			this._wrapper.addClass("enableTextSelection");
			
			// Init the elements.
			this._initializeDOMElements();
			
			// create widget elements and append to the wrapper
			this._wrapper.append(
					this._constructApprovalDiv(),
					$('<div>').addClass("TextContent-ContentCol").append(
							$('<div>').addClass("TextContent-InnerContentCol").append(this._constructTextDiv())
					)
			);
		};
		
		return TextContentElementWidgetViewerView;
		
});