define(["jquery"], function ($) {

	
	
	var TableElementTypeService = function () {
		
		// These private variables will cache the table element types.
        this._cachedTableElementTypes = undefined;
        
        // These private variables take care of preventing duplicate requests.
        this._tableElementTypesAreBeingRetrieved = false;
        
        this._onTableElementTypesRetrieved = [];
	};
	
	/**
	 * Method is to get readable format of the element type code.
	 * 
	 * @param {String} tableElementTypeCode : Element type code for which readable format is required.
	 */
	TableElementTypeService.prototype.getReadableElementType = function (tableElementTypeCode, callback) {
		if (tableElementTypeCode) {
			
			if (this._cachedTableElementTypes) {
				if (this._cachedTableElementTypes[tableElementTypeCode]) {
					callback(this._cachedTableElementTypes[tableElementTypeCode]);
				} else {
					callback(tableElementTypeCode);
				}
			} else {
				this.getTableElementTypes(function (tableElementTypes) {
					var tableElementTypeReadable = tableElementTypes[tableElementTypeCode];
					if (tableElementTypeReadable) {
						callback(tableElementTypeReadable);
					} else {
						callback(tableElementTypeCode);
					}
				});
			}
		}
	};
	
	/**
	 * Gets all element type for table content.
	 */
	TableElementTypeService.prototype.getTableElementTypes = function (callback) {
		
		// Check if we have cached element types.
		if (this._cachedTableElementTypes) {
			
			callback(this._cachedTableElementTypes);
			
		} else {
			
			this._onTableElementTypesRetrieved.push(callback);
			
			if (!this._tableElementTypesAreBeingRetrieved) {
				
				this._tableElementTypesAreBeingRetrieved = true;
				
				// The baseUrl of the service.
		        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
				
				$.ajax({
					url: jspBaseUrl+"GetRowsFromList.jsp?listType=WCR_DB_List&indexed=1",
					type: "GET",
					dataType : "text",
					data : {
						outputType:"JSON",
						list : "TABLE_ELEMENT_TYPES",
						maxrows : 200,
						maxcolrows : 200
					},
					success: function(jsonContent) {
						
						this._cachedTableElementTypes = {};
						try {
							var jsonObj;
							try {
								jsonObj = jsonParse(jsonContent);
							} catch(err) {			// if jsonParse fails, fall back on eval (which is slower)
								jsonObj = eval('(' + jsonContent + ')');
							}
							if (!jsonObj.success || jsonObj.success == "0") {
								callback('Cannot get values ' + jsonObj.error.code + ' : '+ jsonObj.error.message);
							} else {
								
								var tableElementTypesValues = [];
								var tableElementTypesColIndex = -1;
								var humanReadableValues = [];
								var humanReadableColIndex = -1;
								var tableElementCategoryValues = [];
								var tableElementCategoryColIndex = -1;
								
								for (var i=0; i < jsonObj.columns.length; i++) {
									if (jsonObj.columns[i].name == "STRUCT_ELEMENT_TYPES") {
										tableElementTypesValues = jsonObj.columns[i].values;
										tableElementTypesColIndex = i;
									} else if (jsonObj.columns[i].name == "STRUCT_HUMAN_READABLE_FORM") {
										humanReadableValues = jsonObj.columns[i].values;
										humanReadableColIndex = i;
									} else if (jsonObj.columns[i].name == "STRUCT_ELEMENT_CATEGORY") {
										tableElementCategoryValues = jsonObj.columns[i].values;
										tableElementCategoryColIndex = i;
									} 
								}
								var combinations = jsonObj.combinations;
								
								$.each(combinations, function(index, value) {
									
									var tableElementTypes = {};
									var combinationsArr = value.split('~');

									var indexElemType = combinationsArr[tableElementTypesColIndex];
									this._cachedTableElementTypes[tableElementTypesValues[indexElemType]] = tableElementTypes;
									
									if (humanReadableColIndex >= 0) {
										tableElementTypes.tableElementHumanReadable = humanReadableValues[combinationsArr[humanReadableColIndex]];
									}
									
									if (tableElementCategoryColIndex >= 0) {
										tableElementTypes.tableElementCat = tableElementCategoryValues[combinationsArr[tableElementCategoryColIndex]];
									}
									
								}.bind(this));
							}
						} catch (err) {
							alert(err);
						}
						
						// Execute the onLangsRetrieved call backs.
		    			for (var i=0; i < this._onTableElementTypesRetrieved.length; i++) {
		    				this._onTableElementTypesRetrieved[i](this._cachedTableElementTypes);
		    			}
		    			
					}.bind(this)
				});
			}
		}
	};
	
	return new TableElementTypeService();
	
});