define(["jquery",
        "underscore",
        "utils/ConvertUtils",
        "packaging-content-management/text-content-utils/LanguageUtils",
        "packaging-content-management/ContentConstants",
        "packaging-content-management/text-content-element-widget/TextContentElementWidget",
        "packaging-content-management/language-indicator/LanguageIndicator",
        "packaging-content-management/text-content-services/languageService",
        "packaging-content-management/text-content-services/elementTypeService",
        "component/passwordpopup/PasswordValidationPopup",
        "utils/webcss!./InitTextInformationView.css"],
		function($,
				_,
				ConvertUtils,
				LanguageUtils,
				ContentConstants,
				TextContentElementWidget,
				LanguageIndicator,
				languageService,
				elementTypeService,
				PasswordValidationPopup) {
			
			"use strict";
			
			var InitTextInformationView = function(webi18n) {
				
				this.webi18n = webi18n;
				this.languageMasterIndicator = null;
				this.languageTranslationIndicator = null;
				
				this._onSaveClicked = [];
				this._onInvariantChange = [];
				this._onElementTypeChange = [];
				this._onApprovalTypeChange = [];
				this._onTextTypeChange = [];
				this._onMaxCharsChange = [];
				this._onLangChange = [];
				this._saveWarningDiv; 
				this._contentObj = null;
				
				this._masterElementWidgets = [];
				this._translationElementWidgets = [];
				this._saveInProgress = false;
				
				// The wrapping element.
				this._wrapper = $("<div>", {"id" : "text-information", "class" : "text-information-container"});
				
				// Create a Container to hold the top buttons of the page.
				this._topWarningContainer = $("<div/>", {'class':'WarningContainer'});
				this._wrapper.append(this._topWarningContainer);
				
				// Create a Container to hold the properties of the text content file.
				this._propertiesContainer = $("<div/>", {'class':'PropertiesContainer' });
				this._propertiesTable = $("<table>", {'class' : 'PropertiesTable'});
				var propertiesPanelBody = $("<div>", {'class':'panel-body'}).append(this._propertiesTable);
				var propertiesPanel = $("<div>", {'class':'panel panel-default'}).append(propertiesPanelBody);
				this._lastUpdatedContainer = $("<div/>", {'class' :'LastUpdatedContainer'});
				this._propertiesContainer.append(this._lastUpdatedContainer, propertiesPanel);
				this._wrapper.append(this._propertiesContainer);
				
				// Create a Container to hold the master elements.
				this._masterElementsContainer = $("<div/>", {'class': 'MasterTextContainer'});
				this._wrapper.append(this._masterElementsContainer);
				
				// Create a Container to hold the translation widgets.
				this._translationElementsContainer = $("<div/>", {'class':'TranslationsContainer'});
				this._wrapper.append(this._translationElementsContainer);
				
				// Create a Container to hold the bottom buttons of the page.
				this._bottomWarningContainer = $("<div/>", {'class':'WarningContainer'});
				this._wrapper.append(this._bottomWarningContainer);
			};
			
			/**
			 * Register the callbacks for this view.
			 */
			InitTextInformationView.prototype.onSaveClicked = function(callback) {
				this._onSaveClicked.push(callback);
			};
			InitTextInformationView.prototype.onInvariantChange = function(callback) {
				this._onInvariantChange.push(callback);
			};
			InitTextInformationView.prototype.onElementTypeChange = function(callback) {
				this._onElementTypeChange.push(callback);
			};
			InitTextInformationView.prototype.onApprovalTypeChange = function(callback) {
				this._onApprovalTypeChange.push(callback);
			};
			InitTextInformationView.prototype.onTextTypeChange = function(callback) {
				this._onTextTypeChange.push(callback);
			};
			InitTextInformationView.prototype.onMaxCharsChange = function(callback) {
				this._onMaxCharsChange.push(callback);
			};
			InitTextInformationView.prototype.onLangChange = function(callback) {
				this._onLangChange.push(callback);
			};
			
			/**
			 * Displays last updated date of text content document
			 */
			InitTextInformationView.prototype.getTextUpdatedDate = function() {
				var updated = this._contentObj.updated;
				if(updated) {
					this._lastUpdatedContainer.append(
							$('<span>').addClass("text-updatedDate").text(this.webi18n.t("PCM_LAST_UPDATED") + ': ' 
													+ ConvertUtils.getLangFormattedDateAndTime(this.webi18n, new Date(updated)))
					);
				}
			};
			
			/**
			 * Shows unsaved warning popup upon leaving the page (note that it is browser dependent)
			 */
			InitTextInformationView.prototype.onBeforeUnload = function (){
				window.onbeforeunload = _.bind(function(){
					if(this._contentObj.isChanged() && !this._saveInProgress){
						return this.webi18n.t("PCM_UNSAVED_CHANGES_CONFIRM");
					}
				}, this);
			};
			
			/**
			 * Display a dismissable warning message.
			 * 
			 * @param message
			 */
			InitTextInformationView.prototype.displaySaveWarning = function(message, isDismissable) {
								
				if(this._saveWarningDiv){
					return;
				}
				
				this._saveWarningDiv = $("<div>", {'class':'alert alert-warning', 'role' : 'alert', 'id' : 'saveDiv1'});
				if (isDismissable) {
					var warningDismiss = $("<button>", {'type':'button', 'class':'close', 'data-dismiss' : 'alert'});
					warningDismiss.append($("<span>").html("&times;"));
					this._saveWarningDiv.append(warningDismiss);
					this._saveWarningDiv.addClass("alert-dismissible");
				}
				var warningText = $("<span>").text(message);
				var saveButton = $("<button>", {'class':'btn btn-primary btn-sm pull-right SaveButton'}).text(this.webi18n.t("PCM_SAVE"));
				var saveCell = $("<td>",{'class' : 'SaveCell'}).append(saveButton);
				var warningInnerRow = $("<tr>").append($("<td>").append(warningText), saveCell);
				var warningInnerTable = $("<table>", {'class' : 'WarningTable'}).append(warningInnerRow);
				this._saveWarningDiv.append(warningInnerTable);
				
				var bottomWarning = this._saveWarningDiv.clone();
				var warnings = $().add(this._saveWarningDiv).add(bottomWarning);
				
				var saveButtonHandler = _.bind(function() {
					for(var i=0; i < this._onSaveClicked.length; i++) {
						this._onSaveClicked[i]();
					}
					$(".SaveButton", warnings).hide();
					$(".SaveCell", warnings).append($("<img/>",{'src':'./images/loaderTransparent.gif'}));
					this._saveInProgress = true;
				}, this);
				
				$(".SaveButton", warnings).on("click", function () {
					
					if(this._contentObj.isPasswordRequiredForApproval()) {
						this._passwordPopup = new PasswordValidationPopup();
						this._passwordPopup.onSubmit(saveButtonHandler);
						this._passwordPopup.open();
					} else {
						saveButtonHandler();
					}
				}.bind(this));
				
				this._topWarningContainer.empty();
				this._bottomWarningContainer.empty();
				this._topWarningContainer.append(this._saveWarningDiv);
				this._bottomWarningContainer.append(bottomWarning);
				
			};
			
			/**
			 * Display a dismissable error message when viewing text content.
			 * 
			 * @param message
			 */
			InitTextInformationView.prototype.displayLoadError = function(message, isDismissable) {
				
				var error = $("<div>", {'class':'alert alert-danger', 'role' : 'alert'});
				
				if (isDismissable) {
					var errorDismiss = $("<button>", {'type':'button', 'class':'close', 'data-dismiss' : 'alert'});
					errorDismiss.append($("<span>").html("&times;"));
					error.append(errorDismiss);
					error.addClass("alert-dismissible");
				}
				
				var errorText = $("<span>").text(message);
				var errorInnerRow = $("<tr>").append($("<td>").append(errorText));
				var errorInnerTable = $("<table>", {'class' : 'ErrorTable'}).append(errorInnerRow);
				error.append(errorInnerTable);
				
				this._propertiesContainer.hide();
				this._topLoadingErrorContainer.empty();
				this._topLoadingErrorContainer.append(error);
				
			};
			
			
			/**
			 * Displays Project Attribute Languages selected
			 */
			InitTextInformationView.prototype.getProjectLanguagesRow = function(){
				
				var contentManagerView = this._contentObj.canManage().allowed;
				
				var options = {showLabels:false};
				if (!contentManagerView) {
					options.readOnly = true;
				}
				
				this.languageMasterIndicator = new LanguageIndicator(this._contentObj, true, options);
				var jqMasterElement = this.languageMasterIndicator.getJQElement();
				
				this.languageTranslationIndicator = new LanguageIndicator(this._contentObj, false, options);
				var jqTranslationElement = this.languageTranslationIndicator.getJQElement();				
				
				var masterLangRow = $("<tr>", {'class':''});
				var masterLangLabelCell = $("<td>", {'class' : 'labelCell'});
				masterLangLabelCell.text(this.webi18n.t("PCM_MASTER_LANGUAGE"));
				var masterLangCell = $("<td>", {'class' : ''});
				masterLangCell.append(jqMasterElement);
				masterLangRow.append(masterLangLabelCell, masterLangCell);
				
				var transLangRow = $("<tr>", {'class':'TranslationLangRow'});
				var transLangLabelCell = $("<td>", {'class' : 'labelCell'});
				transLangLabelCell.text(this.webi18n.t("PCM_TRANSLATION_LANGUAGE"));
				var transLangCell = $("<td>", {'class' : ''});
				transLangCell.append(jqTranslationElement);
				transLangRow.append(transLangLabelCell, transLangCell);
				
				if(this._contentObj.invariant) {
					transLangRow.hide();
				}
				
				languageService.getLanguages(this._contentObj.projectId, function (projMastLangs, projTransLangs) {
					if (projMastLangs.length > 0 && projTransLangs.length > 0) {
						//If both source and target languages are defined show both source and target language list
						this._propertiesTable.append(masterLangRow);
						this._propertiesTable.append(transLangRow);
					} else if (projMastLangs.length > 0) {
						this._propertiesTable.append(masterLangRow);
					} else if (projTransLangs.length > 0) {
						this._propertiesTable.append(transLangRow);
					}
				}.bind(this));

				pcmEventBus.on("PCM.TextContent.langAdded", function(contentStatement, textElement){
					this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
					
					for (var i=0; i < this._onLangChange.length; i++) {
						this._onLangChange[i]();
					}
					
					this.languageMasterIndicator.update();
					this.languageTranslationIndicator.update();
				}.bind(this));
				
				// When this is a new (e.g. copy/new) document statement
				pcmEventBus.on("PCM.TextContent.langRemoved", function(contentStatement, textElement){
					this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
					
					for (var i=0; i < this._onLangChange.length; i++) {
						this._onLangChange[i]();
					}
					
					this.languageMasterIndicator.update();
					this.languageTranslationIndicator.update();
				}.bind(this));
				
				
			};
		
			/**
			 * Displays element type field and Mark as invariant
			 */
			InitTextInformationView.prototype.getMetaDataRow = function() {

				var contentManagerView = this._contentObj.canManage().allowed;
				var elementType = this._contentObj.elementType;
				var invariant = this._contentObj.invariant;
				
				/*
				 * Create a property entry that controls whether the text content file is invariant or not.
				 */ 
				
				var invariantRow = $("<tr>", {'class':''});
				var invariantLabel = $("<label>");
				var invariantInput = $("<input>", {'type':'checkbox'});
				if (invariant) {
					invariantInput.attr('checked','checked');
				}
				var invariantLabelText = $("<span>", {'class': 'checkBoxLabel'}).text(this.webi18n.t("PCM_MAKE_INVARIANT"));
				invariantLabel.append(invariantInput, invariantLabelText);
				var invariantCell = $("<td>", {'colspan': '2'});
				invariantCell.append(invariantLabel);
				invariantRow.append(invariantCell);
				
				// If we do not have the permissions to change this, do not show it.
				if (contentManagerView) {
					this._propertiesTable.append(invariantRow);
				}
				
				// When somebody changes the value of the input.
				invariantInput.on("change", function() {
					
					// Execute the necessary callbacks.
					for (var i=0; i < this._onInvariantChange.length; i++) {
						
						this._onInvariantChange[i](invariantInput.is(":checked"));
					}
					
					// Display the save warning.
					this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
					
				}.bind(this));
				
				
				/*
				 * Create a property entry that controls the element type of the text content file.
				 */ 
				
				var elementTypeRow = $("<tr>", {'class':''});
				var elementTypeLabelCell = $("<td>", {'class' : 'labelCell'});
				elementTypeLabelCell.text(this.webi18n.t("PCM_ELEMENT_TYPE"));
				var elementTypeInput = $("<select>", {'class':'form-control', 'name': 'elementType'});
				var elementTypeInputCell = $("<td>", {'class' : ''});
				elementTypeService.getElementTypes(function (elementTypes) {
					//If element type is custom and not available in universal ElementType list,
					//add custom element type to drop down. Once user changes the element type, it wont be available again
					if(typeof elementTypes[elementType] === 'undefined'){
						var option = new Option(elementType, elementType);							
						$(option).attr("selected", "selected");
						elementTypeInput.append(option);
					}	
					
					// Populate the drop down.
					for (var key in elementTypes) {
						if (elementTypes.hasOwnProperty(key)){
							var option = new Option(elementTypes[key].elementHumanReadable, key);
							if (key === elementType) {
								$(option).attr("selected", "selected");
							}
							elementTypeInput.append(option);
						}
					}
					// Depending on our permissions, add the input or the readonly value.
					if (contentManagerView) {
						elementTypeInputCell.append(elementTypeInput);
					} else {
						var text;
						if (elementTypes[elementType]) {
							text = elementTypes[elementType].elementHumanReadable;
						} else {
							text = elementType;
						}
						elementTypeInputCell.text(text);
					}
				});

				elementTypeRow.append(elementTypeLabelCell, elementTypeInputCell);
				this._propertiesTable.append(elementTypeRow);
				
				// When somebody changes the value of the input.
				elementTypeInput.on("change", function() {
					
					// Execute the necessary callbacks.
					for (var i=0; i < this._onElementTypeChange.length; i++) {
						this._onElementTypeChange[i](elementTypeInput.val());
					}
					
					// Display the save warning.
					this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
					
				}.bind(this));
				
			};
			
			
			/**
			 * gets the approval setup row
			 */
			InitTextInformationView.prototype.getApprovalSetupRow = function() {
				
				var contentManagerView = this._contentObj.canManage().allowed;
				var approvalType = this._contentObj.approvalType;
				
				var approvalTypeString=null;
				var approvalTypeCode=null;
				
				if(approvalType == ContentConstants.ApprovalSetup.DEFAULT || approvalType == null || approvalType == "" || typeof approvalType ==='undefined') {
					approvalTypeString = this.webi18n.t("PCM_SETUP_DEFAULT");
					approvalTypeCode = ContentConstants.ApprovalSetup.DEFAULT;
					
				}else if(approvalType == ContentConstants.ApprovalSetup.ONE_OF_GROUP) {
					approvalTypeString = this.webi18n.t("PCM_ALLOFROLE_ONEPERGROUP");
					approvalTypeCode   = ContentConstants.ApprovalSetup.ONE_OF_GROUP;
					
				}else if(approvalType == ContentConstants.ApprovalSetup.ONE_OF_ROLE) {
					approvalTypeString = this.webi18n.t("PCM_ONE_OF_ROLE");
					approvalTypeCode   = ContentConstants.ApprovalSetup.ONE_OF_ROLE;
					
				}

				var approvalTypeRow = $("<tr>", {'class':''});
				var approvalTypeLabelCell = $("<td>", {'class' : 'labelCell'});
				approvalTypeLabelCell.text(this.webi18n.t("PCM_APPROVAL_SETUP"));
				var approvalTypeInput = $("<select>", {'class':'form-control', 'name': 'approvalType'});
				approvalTypeInput.append(new Option(this.webi18n.t("PCM_SETUP_DEFAULT"), ContentConstants.ApprovalSetup.DEFAULT));
				approvalTypeInput.append(new Option(this.webi18n.t("PCM_ALLOFROLE_ONEPERGROUP"), ContentConstants.ApprovalSetup.ONE_OF_GROUP));
				approvalTypeInput.append(new Option(this.webi18n.t("PCM_ONE_OF_ROLE"), ContentConstants.ApprovalSetup.ONE_OF_ROLE));
				approvalTypeInput.val(approvalTypeCode);
				var approvalTypeInputCell = $("<td>", {'class' : ''});
				approvalTypeInputCell.append(approvalTypeInput);
				// Depending on our permissions, add the input or the read only value.
				if (contentManagerView) {
					approvalTypeInputCell.append(approvalTypeInput);
				} else {
					approvalTypeInputCell.text(approvalTypeString);
				}
				approvalTypeRow.append(approvalTypeLabelCell, approvalTypeInputCell);
				this._propertiesTable.append(approvalTypeRow);
				
				// When somebody changes the value of the input.
				approvalTypeInput.on("change", function() {
					
					// Execute the necessary callbacks.
					for (var i=0; i < this._onApprovalTypeChange.length; i++) {
						this._onApprovalTypeChange[i](approvalTypeInput.val());
					}
					
					// Display the save warning.
					this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
					
				}.bind(this));
					
			};
			
			/**
			 * gets the text type row
			 */
			InitTextInformationView.prototype.getTextTypeRow = function() {
				
				var contentManagerView = this._contentObj.canManage().allowed;
				var textType = this._contentObj.textType;
				
				var textTypeRow = $("<tr>");
				this._propertiesTable.append(textTypeRow);

				var textTypeLabelCell = $("<td>", {'class' : 'labelCell'});
				textTypeLabelCell.text(this.webi18n.t("PCM_TEXT_TYPE"));
				
				var textTypeInputCell = $("<td>");
				textTypeRow.append(textTypeLabelCell, textTypeInputCell);
				
				// If type of the document is not defined to rich or simple text, then it considard as simple text.
				if (ContentConstants.TextType.RICHTEXT !== textType && ContentConstants.TextType.SIMPLE !== textType) {
					textType = ContentConstants.TextType.SIMPLE;
				}
				
				// Depending on our permissions, add the input or the readonly value.
				if (contentManagerView) {
					this._textTypeInput = $("<select>", {'class': 'form-control', 'name': 'textType'});
					this._textTypeInput.append(new Option(this.webi18n.t("PCM_SIMPLE_TEXT"), ContentConstants.TextType.SIMPLE));
					this._textTypeInput.append(new Option(this.webi18n.t("PCM_RICH_TEXT"), ContentConstants.TextType.RICHTEXT));
					this._textTypeInput.val(textType);
					
					textTypeInputCell.append(this._textTypeInput);
					
					// When somebody changes the value of the input.
					this._textTypeInput.on("change", _.bind(function() {
						var textTypeUpdated = false;
						
						// Execute the necessary callbacks.
						for (var i=0; i < this._onTextTypeChange.length; i++) {
							textTypeUpdated = this._onTextTypeChange[i](this._textTypeInput.val());
						}
						
						if (textTypeUpdated) {
							// Display the save warning.
							this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
						}
						
					}, this));
				} else {
					textTypeInputCell.text(textType);
				}
			};
			
			/**
			 * gets the max chars row
			 */
			InitTextInformationView.prototype.getMaxCharsRow = function() {
				
				var contentManagerView = this._contentObj.canManage().allowed;
				var maxChars = this._contentObj.getMaxChars();
				
				var maxCharsRow = $("<tr>");
				this._propertiesTable.append(maxCharsRow);

				var maxCharsLabelCell = $("<td>", {'class' : 'labelCell'});
				maxCharsLabelCell.text(this.webi18n.t("PCM_MAX_CHARS"));
				
				var maxCharsInputCell = $("<td>");
				maxCharsRow.append(maxCharsLabelCell, maxCharsInputCell);
				
				// Depending on our permissions, add the input or the readonly value.
				if (contentManagerView) {
					this._maxCharsInput = $("<input>", 
							{'name': 'maxChars', 'type':'number', 'min':'1', 'step':'1', 'class':'form-control', 'placeholder':this.webi18n.t('PCM_MAX_CHARS_DEFAULT')});
					this._maxCharsInput.val(maxChars);
					
					maxCharsInputCell.append(this._maxCharsInput);
					
					// When somebody changes the value of the input.
					this._maxCharsInput.on("change keyup", _.bind(function() {
						
						var newVal = this._maxCharsInput.val();
						
						//Remove non-digit chars
						newVal = newVal.replace(/\D/g, '');
						
						var pattern = new RegExp(/^[1-9]+[0-9]*$/);
						var isValidNumber = pattern.test(newVal);

						if(!isValidNumber){
							newVal = '';
						}
						
						this._maxCharsInput.val(newVal);
						
						var maxCharsUpdated = false;

						// Execute the necessary callbacks.
						for (var i=0; i < this._onMaxCharsChange.length; i++) {
							maxCharsUpdated = this._onMaxCharsChange[i](newVal);
						}

						if (maxCharsUpdated) {
							// Display the save warning.
							this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
							this.setMaxChar(newVal);
						}

					}, this));
				} else {
					maxCharsInputCell.text(maxChars);
				}
			};
			
			
			/**
			 * This method is to set selection value of textType drop down as rich text.
			 * Which will help user to roll back if user is not confirmed for textType change as simple.
			 * 
			 * @param textType {String} - Text content documents text type as simple or rich.
			 */
			InitTextInformationView.prototype.setTextType = function(textType) {
				this._textTypeInput.val(textType);
			};
			
			/**
			 * Renders the text content element rows
			 * @param elements {Array} - All the elements available in the text content document.
			 * @param invariant {String} - true as string if document is marked as invariant.
			 */
			InitTextInformationView.prototype.getTextContentRows = function() {
				
				languageService.getLanguages(this._contentObj.projectId, function (projMastLangs, projTransLangs) {
				
					var options = {};
					options.viewOptions = {widgetview: 'default', maxChars:this._contentObj.getMaxChars()};
		
					/*
					 * Master Text
					 */
					// Extract the master elements
					var masterElements = this._contentObj.getMasterElements();
					
					var masterWrapper = this._createTextContentElementWrapper(masterElements, projMastLangs, projMastLangs, this.webi18n.t('PCM_MASTER_TEXT'), options);
					
					this._masterElementsContainer.empty();
					this._masterElementsContainer.append(masterWrapper);
					
					/*
					 * Translations
					 */
					// Extract the master elements
					var translationElements = this._contentObj.getTranslationElements();
					
					var transWrapper = this._createTextContentElementWrapper(translationElements, projMastLangs, projTransLangs, this.webi18n.t('PCM_TRANSLATED'), options);
					
					this._translationElementsContainer.empty();
					this._translationElementsContainer.append(transWrapper);
					
					
					// Indicate to the widgets that they were shown in the page.
					var widgets = this._masterElementWidgets.concat(this._translationElementWidgets);
					$.each(widgets, _.bind(function (index, widget) {
						widget.shownInPage();
					}, this));
					
					
					// Hide translation's section if document is marked as invariant or does not contain any translation.
					if(this._contentObj.isInvariant() || translationElements.length == 0) {
						this._translationElementsContainer.hide();
					} else {
						this._translationElementsContainer.show();
					}
				
				}.bind(this));
			};
			
			InitTextInformationView.prototype.setContentObj = function (contentObj){
				this._contentObj = contentObj;
			};
			
			InitTextInformationView.prototype._createTextContentElementWrapper = function(elements, projMasterLanguages, projLanguages, wrapperTitle, options) {
				var wrapper = $("<div>");

				// Check for availability of elements, if not element then do not create wrapper.
				if (elements.length < 1) {
					return wrapper; 
				}
				
				var projLanguageWrapper = $("<div>");
				var nonProjLanguageWrapper = $("<div>").hide();
				
				// Add a header to indacte the master statements.
				wrapper.append($("<h2>").text(wrapperTitle));
				
				// Add the widgets
				for (var i=0; i < elements.length; i++) {
					var widget = new TextContentElementWidget(elements[i], options);
					
					if (this._contentObj.canManage().allowed) {
						// Create and add a delete button to the widget.
						var button = this._createDeleteButtonFor(elements[i], widget, projMasterLanguages);
						if (button) {
							widget.addCustomActionButton(button);
						}
					}
					
					widget.onChange(_.bind(function() {
						this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"), false);
					}, this));
					
					// Add it to the correct wrapper.
					if (LanguageUtils.isAProjectAttrLang(projLanguages, elements[i].language)) {
						projLanguageWrapper.append(widget.getJQElement());
					} else {
						nonProjLanguageWrapper.append(widget.getJQElement());
					}
					
					if (elements[i].isMaster()) {
						this._masterElementWidgets.push(widget);
					} else {
						this._translationElementWidgets.push(widget);
					}
				}
				
				// Add all language statement which language is exists in project languages.
				if (projLanguageWrapper.children().length > 0) {
					wrapper.append(projLanguageWrapper);
				}
				
				// Add all language statement which language does not exists in project languages.
				if (nonProjLanguageWrapper.children().length > 0) {
					var expandNonProjLink = $("<div>", {"class": "ShowHideLink"});
					var expandNonProjSymbol = $("<i>", {"class" : "icon icon-WCR_Menu-Caret-Right"});
					var expandNonProjLabel = $("<span>").text(this.webi18n.t("PCM_SHOW_NON_PROJ_LANG"));
					expandNonProjLink.append(expandNonProjSymbol, expandNonProjLabel);
					
					expandNonProjLink.on("click", _.bind(function () {
						if (nonProjLanguageWrapper.is(":hidden")) {
							nonProjLanguageWrapper.show();
							expandNonProjSymbol.removeClass();
							expandNonProjSymbol.addClass("icon icon-WCR_Menu-Caret-Down");
							expandNonProjLabel.text(this.webi18n.t("PCM_HIDE_NON_PROJ_LANG"));
						} else {
							nonProjLanguageWrapper.hide();
							expandNonProjSymbol.removeClass();
							expandNonProjSymbol.addClass("icon icon-WCR_Menu-Caret-Right");
							expandNonProjLabel.text(this.webi18n.t("PCM_SHOW_NON_PROJ_LANG"));
						}
						
					}, this));
					
					wrapper.append(expandNonProjLink);
					wrapper.append(nonProjLanguageWrapper);
				}
				
				return wrapper;
			};
			
			/**
			 * Create a delete button for a TextContentElementWidget
			 */
			InitTextInformationView.prototype._createDeleteButtonFor = function(element, widget, projMasterLanguages) {
				// Get list of master element for this text content document.
				var masterElements = element.parentStatement.getMasterElements();

				// Make sure you're not removing the only master.
				if (element.isMaster() && masterElements.length == 1) {
					return;
				}
				
				// Get list of elements which are part of project languages.
				var partOfProjLanguages = $.grep(masterElements, function(element, index) {
					return LanguageUtils.isAProjectAttrLang(projMasterLanguages, element.language);
				});
				
				// Master element can not be removed if it's only element which is part of project languages.
				if (element.isMaster() && partOfProjLanguages.length == 1 && partOfProjLanguages[0].language == element.language) {
					return;
				}
				
				// Make sure that you're not removing the only signedoff master, which is part of project languages.
				var signedOffMasterLangs = element.parentStatement.getSignedOffMasterLangs(projMasterLanguages);
				if (element.isMaster() && signedOffMasterLangs.length == 1 && element.language == signedOffMasterLangs[0].langCode) {
					return;
				}
				
				// Create delete button for this element.
				var button = $("<div>", {"class": "TextContent-Action"});
				var icon = $("<i>", {"class" : "icon icon-WCR_Trash"});
				button.append(icon);
				
				// Register click evet for this button.
				button.on("click", function () {
					var deletedElement = element.parentStatement.removeLanguage(element.language, element.isMaster());
					if (deletedElement) {
						pcmEventBus.emit("PCM.TextContent.langRemoved", element.parentStatement, element);
					}
					button.hide();
				});
				
				return button;
			};
			
			/**
			 * Disable widgets based on content type.
			 */
			InitTextInformationView.prototype.disableWidgets = function(master, translation) {
				var widgets = [];
				
				// if requested for master, then keep widgets for master elements
				if (master) {
					widgets = widgets.concat(this._masterElementWidgets);
				}
				
				// if requested for translation, then keep widgets for translations
				if (translation) {
					widgets = widgets.concat(this._translationElementWidgets);
				}
				
				// if no information if given then consider for both type of elements.
				if (widgets.length == 0) {
					widgets = this._masterElementWidgets.concat(this._translationElementWidgets);
				}
				
				// Disable all the requested widgets.
				$.each(widgets, _.bind(function (index, widget) {
					widget.disable(this.webi18n.t("CONTENT_CANNOT_EDIT_SAVE_FIRST"));
				}, this));
			};
			
			/**
			 * Enable widgets based on content type.
			 */
			InitTextInformationView.prototype.enableWidgets = function(master, translation) {
				var widgets = [];
				
				// if requested for master, then keep widgets for master elements
				if (master) {
					widgets = widgets.concat(this._masterElementWidgets);
				}
				
				// if requested for translation, then keep widgets for translations
				if (translation) {
					widgets = widgets.concat(this._translationElementWidgets);
				}
				
				// if no information if given then consider for both type of elements.
				if (widgets.length == 0) {
					widgets = this._masterElementWidgets.concat(this._translationElementWidgets);
				}
				
				// Enable all the requested widgets.
				$.each(widgets, _.bind(function (index, widget) {
					widget.enable();
				}, this));
			};
			
			/**
			 * Set maximum number of chars for all the widgets
			 */
			InitTextInformationView.prototype.setMaxChar = function(maxChar) {
				var widgets = this._masterElementWidgets.concat(this._translationElementWidgets);
				$.each(widgets, _.bind(function (index, widget) {
					widget.setMaxChar(maxChar);
				}, this));
			};
			
			/**
			 * Get the wrapping jQuery element for this page.
			 */
			InitTextInformationView.prototype.getJQElement = function () {
				return this._wrapper;
			};
			
			return InitTextInformationView;
		}
);

