define(["require", "wcr/ObjectArrayCloneUtil"], function (require, ObjectHandler) {
	"use strict";

	/**
	 * Constructor
	 * This component initializes the div that shows the project header attributes with wcr-init
	 * @param {JQueryObj} - div to show the project header attributes in
	 */
	var ProjectHeaderAttributes = function (div) {
		div = $(div);

		var projectCategoryID = div.attr("data-categoryid");
		var projectID = div.attr("data-projectid");
		var lastModified = div.attr("data-lastmodified");

		try {
			if (projectID && projectCategoryID) {
				require(["attributecategory/AttributeCategory"], function (AttributeCategory) {
					var readURL = "GetAttributeCategoryXML.jsp?categoryid=" + projectCategoryID;

					// ensure to only append the category's mod date when it is known
					if (lastModified) {
						readURL += '&lastmodified=' + lastModified;
					}
					var valuesURL = "GetAttributes.jsp?projectid=" + projectID + "&editable=0"; // header view is not editable
					var currentUser = ObjectHandler.getCloneOfObject(window.gCurrentUser);
					currentUser.canEditAttributes = false;
					var aCatOptions = {};
					aCatOptions.ACatObj = "projectHeaderAttributesId";
					aCatOptions.ACatPanel = "projectHeaderAttributesId";
					aCatOptions.readURL = readURL;
					aCatOptions.currentUser = currentUser;
					aCatOptions.valuesURL = valuesURL;
					aCatOptions.view = "Header";
					aCatOptions.prefix = "header_"; // not to interfere with attributes in project details
					aCatOptions.type = "project";
					aCatOptions.projectID = projectID;
					window.myACat = new AttributeCategory(aCatOptions);
					div.show();
				});
			} else {
				div.empty();
				div.hide();
			}
		} catch (err) {
			//just does not display attribute category header - the most probable is that it's not suppose to be displayed, so do nothing
		}
	};

	/**
	 * Initialize as a component
	 */
	ProjectHeaderAttributes.wcrinit = function (div) {
		new ProjectHeaderAttributes(div);
	};

	return ProjectHeaderAttributes;
});