define(["utils/webi18n!ContextAwareLandingPage",
		"underscore",
	   	"jquery"],
	function (
		i18n,
		_,
		$) {
		"use strict";

		/**
		 * This represents a row in the LandingPageTable
		 * 
		 * @exports LandingPageRowView
		 * @class
		 * 
		 * @param {JSONObject} row
		 */
		var LandingPageRowView = function (row, isBOMEnabled, areTasksEnabled) {

			this._row = row;

			/**
			 * @member {Boolean}
			 * @private 
			 */
			this._enabled = true;

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._element = $("<tr>", {'data-row-id': row.id});

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._checkboxInput = $("<input>", {
				"type": "checkbox",
				"name": "selectitem",
				"value": this._row.id
			});
			this._checkbox = $("<td>").addClass("td-checkbox").append(this._checkboxInput);

			if (!this._row.roles || this._row.roles.length === 0) {
				this._row.roles = [''];
			}
			var role = this._row.roles[0];
			for (var i = 1; i < this._row.roles.length; i++) {
				role = role + ', ' + this._row.roles[i];
			}

			var roleEditField = $("<input />")
				.attr("type", 'text')
				.attr("name", "roles" + (this._row.id))
				.attr("id", "roles" + (this._row.id))
				.attr("placeholder", i18n.t("HINT_ROLES"))
				.addClass("form-control")
				.val(role);

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._role = $("<td>").append(roleEditField);

			var landing = "projdetailsdocs.jsp";
			var details = "";
			var url = this._row.url;
			if (url && url.length > 0) {
				url = url.replace("searchName=", "");
				url = url.replace("folderName=", "");
				url = url.replace("attrcatview=", "");
				url = url.split('?');
				landing = url[0];
				details = !url[1] ? "" : decodeURIComponent(url[1]).replace("+", " ");
			}
			var detailsEditField = $("<input>")
				.attr("type", 'text')
				.attr("name", "details" + (this._row.id))
				.attr("id", "details" + (this._row.id))
				.addClass("form-control")
				.val(details);

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._details = $("<td>").append(detailsEditField);

			var options = [
				{
					"url": "projdetailsdocs.jsp",
					"text": i18n.t("DOC_PAGE"),
					"hint": i18n.t("HINT_FOLDER_NAME_OR_EMPTY"),
					},
				{
					"url": "projdetailsattrs.jsp",
					"text": i18n.t("ATTR_PAGE"),
					"hint": i18n.t("HINT_VIEW_NAME_OR_EMPTY"),
					},
				{
					"url": "dosavedsearch.jsp",
					"text": i18n.t("SAV_SEARCH"),
					"hint": i18n.t("HINT_SAVED_SERACH_NAME"),
					},
				{
					"url": "projdetailsapprovers.jsp",
					"text": i18n.t("APPROVAL_PAGE")
					}
			];
			if (areTasksEnabled === "1" ? true : false) {
				options.push({
					"url": "projdetailswctasks.jsp",
					"text": i18n.t("TASKS_PAGE")
				});
			}
			if (isBOMEnabled === "1" ? true : false) {
				options.push({
					"url": "projdetailsbom.jsp",
					"text": i18n.t("BOM_PAGE")
				});
			}

			var optionsHtml = "";
			for (var a = 0; a < options.length; a++) {
				optionsHtml += '<option value="' + options[a].url;
				if (options[a].url === landing) {
					optionsHtml += '" selected="';
					if (options[a].hint) {
						detailsEditField.attr("placeholder", options[a].hint);
					}
					else {
						detailsEditField.attr("disabled", "disabled");
						detailsEditField.val("");
					}
				}
				optionsHtml += '">' + options[a].text + '</option>';
			}

			var landingPageSelect = $("<select>")
				.attr("name", "landingpage" + (this._row.id))
				.attr("id", "landingpage" + (this._row.id))
				.addClass("form-control")
				.append(optionsHtml);

			landingPageSelect.on("change", function (event) {
				detailsEditField.val("");
				var newValue = event.target.options[event.target.selectedIndex].value;
				this._row.url = newValue;
				for (var a = 0; a < options.length; a++) {
					if (options[a].url === newValue) {
						if (options[a].hint) {
							detailsEditField.removeAttr("disabled");
							detailsEditField.attr("placeholder", options[a].hint);
						}
						else {
							detailsEditField.attr("disabled", "disabled");
							detailsEditField.removeAttr("placeholder");
						}
					}
				}
			}.bind(this));

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._page = $("<td>").append(landingPageSelect);

			this._element.append(this._checkbox).append(this._role).append(this._page).append(this._details);
		};

		/**
		 * Set the row view enabled for input or not
		 * @param {Boolean} enabled - True if enabled, false if disabled
		 */
		LandingPageRowView.prototype.setEnabled = function (enabled) {
			this._enabled = enabled;
			this._updateEnabled();
		};

		/**
		 * Update the fields based on the enabled flag
		 * @private
		 */
		LandingPageRowView.prototype._updateEnabled = function () {
			this._checkboxInput.prop('disabled', !this._enabled);
		};


		/**
		 * Get the element
		 * @returns {JQueryObj}
		 */
		LandingPageRowView.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Called when the view needs to be destroyed
		 */
		LandingPageRowView.prototype.destroy = function () {
			// Nothing to do for now
		};

		return LandingPageRowView;
	});
