define(["utils/HTTPUtils"], function (HTTPUtils) {
	"use strict";

	/**
	 * SearchResultsCountBadge represents the badge next to a menu item for a saved search.
	 *
	 * @exports SearchResultsCountBadge
	 * @class
	 * @param {JQueryObj} div - The div to paint the badge in                      
	 */
	var SearchResultsCountBadge = function (div) {
		this._element = $(div);
		this._count = "-";
		this._update();
	};


	/**
	 * Do an update of the count
	 * @private
	 */
	SearchResultsCountBadge.prototype._update = function () {
		var href = this._element.parent().attr("href");
		if (href) { // for active menus the href is not set (what we want)
			var from = href.indexOf("?");
			if (from >= 0) {
				// we just copy all parameters from the parent anchor href
				var params = "?action=count&" + href.substring(from + 1);
				var url = HTTPUtils.constructURL("dosearch.wcr" + params, {}, window.webcenterConfig.jspBaseUrl);
				$.ajax({
					'url': url,
					'dataType': "JSON",
					'timeout': 120000,
					'success': function (json) {
						if (json.count !== undefined) {
							this._count = json.count;
							this._render();
						}
					}.bind(this),
					'error': function (jqXHR, textStatus, errorThrown) {
						if (window.console && window.console.warn) {
							window.console.warn("Error occured while loading SearchResultsCountBadge: " + errorThrown);
						}
					}.bind(this)
				});
			}
		} else {
			window.console.warn(href);
		}
	};

	/**
	 * Render the badge based on this._count
	 * @private
	 */
	SearchResultsCountBadge.prototype._render = function () {
		this._element.text(this._count);
	};

	/**
	 * Initialize as a component
	 */
	SearchResultsCountBadge.wcrinit = function (div) {
		new SearchResultsCountBadge(div);
	};

	return SearchResultsCountBadge;
});