define(["./RuleAction",
		"./RuleConstants",
		"underscore"],
	function (RuleAction, RuleConstants, _) {
		"use strict";

		/**
		 * This represents a rule action for starting a task
		 *
		 * @exports RuleActionStartTask
		 * @class
		 * @augments RuleAction
		 */
		var RuleActionStartTask = function () {
			RuleAction.call(this, RuleConstants.ActionType.STARTTASK);

			/**
			 * @member {String} The id of the task type
			 * @private
			 */
			this.taskTypeID = null;
		};

		RuleActionStartTask.prototype = Object.create(RuleAction.prototype);
		RuleActionStartTask.prototype.constructor = RuleActionStartTask;

		/**
		 * Get the task type id of the task to start
		 * @returns {String}
		 */
		RuleActionStartTask.prototype.getTaskTypeID = function () {
			return this.taskTypeID;
		};

		/**
		 * Set the task type id of the task to start
		 * @param {String} newTaskTypeID
		 */
		RuleActionStartTask.prototype.setTaskTypeID = function (newTaskTypeID) {
			this.taskTypeID = newTaskTypeID;
		};

		/**
		 * @override
		 */
		RuleActionStartTask.prototype.copy = function () {
			var copy = new RuleActionStartTask();
			copy.setTaskTypeID(this.getTaskTypeID());
			return copy;
		};

		return RuleActionStartTask;
	}
);