define(["./RuleConstants",
		"./RuleTriggerProjectAttribute",
		"./RuleTrigger",
		"./RuleActionStartTask",
		"underscore"],
	function (RuleConstants,
		RuleTriggerProjectAttribute,
		RuleTrigger,
		RuleActionStartTask,
		_) {

		"use strict";

		/**
		 * This reprents a Rule factory for creating triggers and actions.
		 *
		 * @exports RuleFactory
		 * @class
		 */
		var RuleFactory = function () {};

		/**
		 * Create a new trigger object
		 * @param {RuleConstants.TriggerType} type - The type of the trigger                                                                                             
		 * @returns {RuleTrigger} - A trigger of the appropriate type
		 */
		RuleFactory.prototype.createTrigger = function (type) {
			switch (type) {
			case RuleConstants.TriggerType.PROJECTATTRIBUTE:
				return new RuleTriggerProjectAttribute();
			case RuleConstants.TriggerType.PROJECTDUEDATE:
			case RuleConstants.TriggerType.PROJECTSTATUS:
				return new RuleTrigger(type);
			default:
				return undefined;
			}
		};

		/**
		 * Create a new action object
		 * @param {RuleConstants.ActionType} type - The type of the action
		 * @returns {RuleAction} - An action of the appropriate type
		 */
		RuleFactory.prototype.createAction = function (type) {
			switch (type) {
			case RuleConstants.ActionType.STARTTASK:
				return new RuleActionStartTask();
			default:
				return undefined;
			}
		};

		return RuleFactory;
	}
);