define(["utils/webi18n!RulesTable",
		"underscore",
	   	"jquery"],
	function (
		i18n,
		_,
		$) {
		"use strict";

		/**
		 * This represents a row in the RulesTable
		 * 
		 * @exports RuleRowView
		 * @class
		 * 
		 * @param {Rule} rule
		 * @param {Object} options - Options for the rule row view. Currently available: {{Boolean=true} enabled}
		 */
		var RuleRowView = function (rule, options) {
			/**
			 * @member {Rule}
			 * @private 
			 */
			this._rule = rule;

			/**
			 * @member {Object}
			 * @private 
			 */
			this._defaultOptions = {
				enabled: true
			};
			/**
			 * @member {Object}
			 * @private 
			 */
			this._options = _.extend(this._defaultOptions, options);

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._element = $("<tr>");

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			if (this._options.enabled) {
				// disabled row views don't need a checkbox
				this._checkboxInput = $("<input>", {
					"type": "checkbox",
					"name": "selectitem",
					"value": this._rule.getID()
				});
				this._checkbox = $("<td>").addClass("td-checkbox").append(this._checkboxInput);
				this._element.append(this._checkbox);
			}

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._order = $("<td>").addClass("td-narrow").append(this._rule.getOrder());

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._nameAnchor = $("<a>", {
				"class": "tableaction",
				"title": i18n.t("RULEEDIT"),
				"data-action": "edit_rule"
			}).text(this._rule.getName());
			this._nameAnchor[0].rule = this._rule;
			this._name = $("<td>").append(this._nameAnchor);

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._description = $("<td>").text(this._rule.getDescription());


			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._enabledCheckboxInput = $("<input>", {
				"class": "tableaction",
				"type": "checkbox",
				"name": "enablerule",
				"data-action": "enable_rule",
				"value": this._rule.getID()
			}).prop("checked", this._rule.isEnabled()).prop('disabled', !this._options.enabled).on("click", _.bind(function (e) {
				this._rule.setEnabled($(e.target).prop("checked"));
			}, this));
			this._enabledCheckboxInput[0].rule = this._rule;
			this._enabledCheckbox = $("<td>").addClass("td-checkbox").append(this._enabledCheckboxInput);

			this._element.append(this._order).append(this._name).append(this._description).append(this._enabledCheckbox);
		};

		/**
		 * Get the rule for which this view was created
		 * @returns {Rule}
		 */
		RuleRowView.prototype.getRule = function () {
			return this._rule;
		};

		/**
		 * Get the element
		 * @returns {JQueryObj}
		 */
		RuleRowView.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Called when the view needs to be destroyed
		 */
		RuleRowView.prototype.destroy = function () {
			// Nothing to do for now
		};

		/**
		 * Set the row view enabled or not
		 * @param {Boolean} enabled - True for enabled, false for disabled
		 */
		RuleRowView.prototype.setEnabled = function (enabled) {
			if (this._checkboxInput) {
				this._checkboxInput.prop("disabled", !enabled);

			}
			this._nameAnchor.prop("disabled", !enabled);
			this._enabledCheckboxInput.prop("disabled", !enabled);
		};

		return RuleRowView;
	});