define(["./RuleTrigger",
		"./RuleConstants",
		"underscore"],
	function (RuleTrigger, RuleConstants, _) {
		"use strict";

		/**
		 * This represents a rule trigger for project attribute changes
		 *
		 * @exports RuleTriggerProjectAttribute
		 * @class
		 * @augments RuleTrigger
		 */
		var RuleTriggerProjectAttribute = function () {
			RuleTrigger.call(this, RuleConstants.TriggerType.PROJECTATTRIBUTE);

			/**
			 * @member {String} The id of the attribute
			 * @private
			 */
			this.attributeID = null;
		};

		RuleTriggerProjectAttribute.prototype = Object.create(RuleTrigger.prototype);
		RuleTriggerProjectAttribute.prototype.constructor = RuleTriggerProjectAttribute;

		/**
		 * Get the attribute id
		 * @returns {String}
		 */
		RuleTriggerProjectAttribute.prototype.getAttributeID = function () {
			return this.attributeID;
		};

		/**
		 * Set the attribute id
		 * @param {String} newAttributeID
		 */
		RuleTriggerProjectAttribute.prototype.setAttributeID = function (newAttributeID) {
			this.attributeID = newAttributeID;
		};

		/**
		 * @override
		 */
		RuleTriggerProjectAttribute.prototype.copy = function () {
			var copy = new RuleTriggerProjectAttribute();
			copy.setAttributeID(this.getAttributeID());
			return copy;
		};

		return RuleTriggerProjectAttribute;
	}
);