define(["./RuleConstants",
		"./RuleTriggerView",
		"./RuleTriggerProjectAttributeView",
	   	"./RuleActionStartTaskView"],
	function (
		RuleConstants,
		RuleTriggerView,
		RuleTriggerProjectAttributeView,
		RuleActionStartTaskView) {

		"use strict";

		/**
		 * This reprents a Rule view factory for creating views for triggers and actions.
		 *
		 * @exports RuleFactory
		 * @class
		 */
		var RuleViewFactory = function () {};

		/**
		 * Create a new trigger view object for a given trigger
		 * @param {RuleTrigger} trigger - The trigger to create a view for
		 * @param {Rule} rule - The rule where this trigger is part of
		 * @returns {RuleTriggerView} - A trigger view of the appropriate type
		 */
		RuleViewFactory.prototype.createTriggerViewFor = function (trigger, rule) {
			switch (trigger.getType()) {
			case RuleConstants.TriggerType.PROJECTATTRIBUTE:
				return new RuleTriggerProjectAttributeView(trigger, rule);
			case RuleConstants.TriggerType.PROJECTDUEDATE:
			case RuleConstants.TriggerType.PROJECTSTATUS:
				return new RuleTriggerView(trigger, rule);
			default:
				return undefined;
			}
		};

		/**
		 * Create a new action view object for a given action
		 * @param {RuleAction} action - The action to create a view for
		 * @param {Rule} rule - The rule where this action is part of
		 * @returns {RuleActionView} - A trigger view of the appropriate type
		 */
		RuleViewFactory.prototype.createActionViewFor = function (action, rule) {
			switch (action.getType()) {
			case RuleConstants.ActionType.STARTTASK:
				return new RuleActionStartTaskView(action, rule);
			default:
				return undefined;
			}
		};

		return RuleViewFactory;
	}
);