/**
 * TunnelMonitor
 *
 * @author KADA
 * @since 9.2.2015
 */
define(["jquery",
        "./view/tunnelmonitor-view",
        "./tunnelmonitor-service"
        ], function ($, TunnelMonitorView, TunnelMonitorService) {
	
	"use strict";
	
	/**
	 * Create a TunnelMonitor from a div HTML element.
	 */
	var TunnelMonitor = function (element) {
		
		// Wrap the element with jQuery if this is not the case ...
		if (!(element instanceof jQuery)) {
        	element = $(element);
        }
		
        // Extract the tunnel name from the element.
		this._clientName = element.attr("data-tunnel-client");
		
		// Create the necessary components.
		this._tunnelMonitorService = TunnelMonitorService.getInstance();
        this._tunnelMonitorView = new TunnelMonitorView(element, this._clientName);
        
        
        // What to do when someone toggles pause/resume monitoring
        var self = this;
        this._tunnelMonitorView.onMonitoringToggled(function (isPaused) {
        	if (!isPaused) {
        		self.startMonitoring();
        	} else {
        		self.pauseMonitoring();
        	}
        });
        
        // Indicates whether we're currently monitoring.
        this._isMonitoring = false;
        
		// Start monitoring.
		this.startMonitoring();
	};
	
	
	/**
	 * Start monitoring the tunnels for the passed client.
	 */
	TunnelMonitor.prototype.startMonitoring = function () {
		
		// We should no longer pause monitoring (used for terminating).
		this._pauseMonitoring = false;
		
		// If we are already monitoring ...
		if (this._isMonitoring) {
			return;
		}
		
		var self = this;
		function monitor () {
			// Only do this when not paused.
			if (!self._pauseMonitoring) {
				
				self._isMonitoring = true;
				
				// Do this every 500 sec.
				setTimeout(function(){
					// Request the current tunnelInfo from the TunnelMonitorService
					self._tunnelMonitorService.getTunnelMonitorInfoFor(self._clientName, function (tunnelInfos, timeStamp) {
							// OnSuccess, update the Tunnel Info in the TunnelMonitorView (if not paused).
							if (!self._pauseMonitoring) {
								self._tunnelMonitorView.updateTunnelInfos(tunnelInfos, timeStamp);
							}
							// Request new tunnelMonitorService update.
							monitor();
						}, function (errorMsg) {
							// Something went wrong requesting the tunnelInfo, show the error (if not paused).
							if (!self._pauseMonitoring) {
								self._tunnelMonitorView.showError(errorMsg);
							}
							// Request new tunnelMonitorService update.
							monitor();
					});
				}, 500);
			} else {
				self._isMonitoring = false;
			}
		}
		monitor();
	};
	
	/**
	 * 
	 */
	TunnelMonitor.prototype.pauseMonitoring = function () {
		// Pause the monitoring.
		this._pauseMonitoring = true;
	};
	
	
    /**
     * Method to automatically apply this module to a div element ...
     */
	TunnelMonitor.wcrinit = function(element){
        new TunnelMonitor(element);
    };
	
	return TunnelMonitor;
});