define(["wcr/webcenter_html",
        "./PageSection",
        "utils/EventBasedValidator",
        "jquery",
        "underscore",
        "component/passwordpopup/PasswordValidationPopup",
        "component/sdrop/SDrop",
        "wcr/AddDocument",
        "wcr/DocumentsTable",
        "wcr/validate"], 
function (WCHTML, 
		PageSection, 
		EventBasedValidator, 
		$,
		_,
		PasswordValidationPopup,
		SDrop) {
	
	
	function WcTaskExecutionPage(data, langStrings)	{
		
		try {
			// ID of currently displayed project task
			if (!data.taskID || (data.taskID.length == 0)) {
				alert("No task to process.");
				return;
			}
			this.taskID = data.taskID;
			this.projectID = data.projectID;
			this.loggedInUserID = data.loggedInUserID;
			this.loggedInUserName = data.loggedInUserName;
			this.loggedInUserFirstName = data.loggedInUserFirstName;
			this.loggedInUserLastName = data.loggedInUserLastName;
		
			this.isProjectActive = data.isProjectActive;
			this.isTaskEditable = data.isTaskEditable;
			this.isChangeable = data.isChangeable;
			this.isStatusEditable = data.isStatusEditable;
			this.canChangeAssignee = data.canChangeAssignee;
			this.isTaskStarted = data.isTaskStarted;
			this.isTaskCompleted = data.isTaskCompleted;
			// variable for debug purposes - fast switch between task execution and editation modes
			this.taskExecution = data.taskExecution;
			
			// an object to store all inner page sections of task execution page
			this.pageSections = new Object();
			
			// array of language strings read from input XML
			this.langArr = new Array();
			this.setLanguageStrings(langStrings);
			
			// by default, attempts to leave the page will throw a confirm box if the page is modified
			this.exitConfirmNeeded = true;

			// need to omit "leave page" message in the case od document download
			this.downloaDoc = false;
			// when no changes are made, change btn is disabled and Cancel/Return button behaves as cancel. This is to notify the state of button, thus avoid changing it(enabling change btn and change return->cancel btn) if it is already changed
			this.changeBtnEnabled = false;
			// to remember whether all the validations went OK and the page/action form was submitted (true) or not (false)
			this.actionFormSubmitted = false;
			
			// events for various inputs - to detect when to switch page to "changed mode"-> display "change" button and change "return" to "cancel" button
			this.eventNames = "change.changeDetect select.changeDetect keyup.changeDetect";
			// elements needed to catch the previous events for
			this.eventSourceSelector = "textarea, select, input, div";
			
			// special event for image element - don't want to handle click event for any other element, like input - no need, that just consumes resources
			this.eventName_Img = "click.changeDetect";
			//the img element :)
			this.eventSourceSelector_Img = "img";

			/* @param mainForm {jQuery object} - jQuery object representing the page form to be fixed for FireFox. */
			var applyFFSubmitWorkaround = function(mainForm) {
				if (mainForm.length > 0) {
					if (mainForm.find(".forceSubmit").length === 0) {
						mainForm.append('<input type="submit" class="forceSubmit displayNone" />');
					}	
					var originalSubmit = mainForm[0].submit;
					// replace the original submit() method with a custom one that simulates a click on submit button
	    			mainForm[0].submit = function() {
	    				mainForm[0].submit = originalSubmit;
	    				mainForm.find(".forceSubmit").click();
	        		};
				}
			};

	    	// Firefox has problems with submitting forms without clicking a submit button.
	    	// It only seems to go wrong when using document references as it uses removing of DOM elements heavily and it seems to trigger the problem.
	    	// Because forms are consistently being used wrongly in WebCenter, we cannot fix it everywhere, so we add a hack on affected places.
			applyFFSubmitWorkaround($("#actionformId"));

			// confirmExit is a method called whenever user tries to get away from the page
			var thisObj = this;
			window.onbeforeunload = function() {
				var retValue = thisObj.confirmExit();
				if (retValue) {
					// if the submit is cancelled re-apply the FireFox submit workaround for the next submit...
					applyFFSubmitWorkaround($("#actionformId"));
				}
				return retValue;
			};
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage constructor: " + err);
		}
	}

	
	
	/**
	* Method to set translation strings
	*/
	WcTaskExecutionPage.prototype.setLanguageStrings = function(langStrings) {
		try {
			if (!langStrings) {
				return;
			}
			
			for (var key in langStrings) {
				if (langStrings.hasOwnProperty(key)){
					this.langArr[key] = langStrings[key];
				}
			}
		} catch (err) {
			alert("Problem in WcTaskExecutionPage.setLanguageStrings() method: " + err);
		}
	};

	/**
	* Method to set the SDrop for the Task Assignee
	*/
	WcTaskExecutionPage.prototype.setSDrop = function(sdrop)
	{
		this.SDrop = sdrop; 		
	};

	/**
	*	Method to add a new page section to task execution page under a given name (overwriting is not checked).
	*/
	WcTaskExecutionPage.prototype.addSection = function(section, name)
	{
	/*	
				genTaskInfo : genInfoSection,
				documents : documentsSection,
				checklist : checklistSection,
				attributes : attributesSection,
				discussion : discussionSection
	*/
		try {
			// perform basic initialization of the newly added page section
			section.parent = this;
			section.initialize(name);
			
			// store it in pageSections object under given name
			this.pageSections[name] = section;
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.addSection() method: " + err);
		}
	};

	/**
	*	Method to remove a page section from task execution page with given name.
	*/
	WcTaskExecutionPage.prototype.removeSection = function(name)
	{
		try {
			// if exists remove the section property from pageSections object completely
			if (this.pageSections[name]) delete this.pageSections[name];
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.removeSection() method: " + err);
		}
	};


	WcTaskExecutionPage.prototype.show = function() {
		try {
			var numberOfVisibleSections = 0;
			for (var item in this.pageSections) {
				if (this.pageSections.hasOwnProperty(item)){
					var pItem = this.pageSections[item];
					if (pItem) {
						pItem.show();
						if (pItem.state != "HIDE") {
							numberOfVisibleSections++;
						}
					}
				}
			}
			// non of "hidable" sections is visible (general info section is visible always) => just a simple 
			// task without docs, checklist, etc. is shown...
			// in this case hide the bottom row of buttons
			if (numberOfVisibleSections > 1) {
				$("#lowerButtonRow").show();
			} else {
				$("#lowerButtonRow").hide();
			}
		} catch (err) {
			alert("Problem in WcTaskExecutionPage.show() method: " + err);
		}
	};

	WcTaskExecutionPage.prototype.reset = function() {
		try {
			for (var item in this.pageSections) {
				if (this.pageSections.hasOwnProperty(item)){
					var pItem = this.pageSections[item];
					if (pItem) {
						pItem.reset();
					}
				}
			}
		} catch (err) {
			alert("Problem in WcTaskExecutionPage.reset() method: " + err);
		}
	};


	WcTaskExecutionPage.prototype.isModified = function() {
		try {
			for (var item in this.pageSections) {
				if (this.pageSections.hasOwnProperty(item)){
					if (this.pageSections[item].isModified()) {
						return true;
					}
				}
			}
		} catch (err) {
			alert("Problem in WcTaskExecutionPage.isModified() method: " + err);
		}
		return false;
	};

	/**
	 * Checks various events(cliciing image, changing select, type to an input, ...) and checks, if the page changed. If so, then unregister the events and change the page to "changed mode"-> display "change" button and change "return" to "cancel" button
	 * @param a - specifically for input element that is created a outside of this page - so to be noticed, when the content of the input changed right after some letter was typed
	 */
	WcTaskExecutionPage.prototype.checkIfModifiedAndUpdateButtons = function(a) 
	{
	/*	the text value is used to test if some text was typed in some ajax inserted input(or input created in some template, where is no desire to modify its event firing) - that is right after pressing some letter. 
		Don't want to wait until "onchanged" event is fired, because that is fired after loosing focus from that input and that is too late */
		var somethingModified = false;
		if (typeof a != 'undefined' && 'target' in a && 'defaultValue' in a.target && 'value' in a.target && a.target.defaultValue !== a.target.value) {
			somethingModified = true;
		}
			
		if ((this.isModified() || somethingModified)&& !this.changeBtnEnabled ) {
			this.updateButtons();
		} 
	};

	WcTaskExecutionPage.prototype.updateButtons = function() 
	{
		$(this.eventSourceSelector).off(this.eventNames);
		$("[name='cancelButton']").val(this.langArr.cancel);
		var changeBtns = $("[name='createWCTask']");
		changeBtns.prop("disabled", false);
		changeBtns.show();
		this.changeBtnEnabled = true;
	};

	/**
	 * This will register all kinds of events - so It can be tested, whether the page has changed and so to change the page to "changed mode" -> display "change" button and change "return" to "cancel" button
	 */
	WcTaskExecutionPage.prototype.registerEventToDetectChanges = function() 
	{
		$(this.eventSourceSelector).on(this.eventNames, this.checkIfModifiedAndUpdateButtons.bind(this));
		$(this.eventSourceSelector_Img).on(this.eventName_Img, this.checkIfModifiedAndUpdateButtons.bind(this));
		
		if(this.pageSections.attributes.myAttrCat){
			this.pageSections.attributes.myAttrCat.valueChangedListeners.push(function() {
				this.updateButtons();
			}.bind(this));
		}
		
		require(["utils/WCEvent"],function(WCEvent){
			WCEvent.on("btnClicked.UploadNewVersion.*", function(){
				this.updateButtons();
			}.bind(this));
		}.bind(this));
	};

	/** 
	* Callback method that is called whenever user navigates away from the page
	* (this == window in this method due to it's a callback!)
	*/
	WcTaskExecutionPage.prototype.confirmExit = function() 
	{
		try {
			if (this.exitConfirmNeeded && this.isModified() && !this.downloaDoc) {		  	
				return this.langArr.PAGE_EXIT_CONFIRM;
			} 
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.confirmExit() callback method: " + err);
		}
		//reset the flag, that announces document download => avoiding leave page message
		this.downloaDoc = false;
	};

	WcTaskExecutionPage.prototype.changeButtonClicked = function(force, completeTask)
	{
		// disable all submit buttons to prevent multiple form submits
    	// (it needs to be here as well because of the postponed validation below)
		this.enableSubmitButtons(false);
		
		// The delay of submit is needed here to move the validation and submit (including generation of document reference "saveXML")
		// in the asynchronous calls queue after the potentially pending debounced attribute category re-paint(s), which can be triggered
		// by rules defined in attribute category.
		setTimeout(function() {
			try {
				var isPasswordRequired = false;
				
				for(var item in this.pageSections) {
					var pItem = this.pageSections[item];
					
					// skip empty and not modified sections
					if (pItem && pItem.isModified() && pItem.docTableObj && pItem.docTableObj.checkIfPasswordRequired) {
						// (note checkIfPasswordRequired() is defined only when PCM license is active!)
						isPasswordRequired = pItem.docTableObj.checkIfPasswordRequired();
						
						if(isPasswordRequired) {
							break;
						}
					}
				}
				
				// If password is required - open popup to get password from user , save otherwise
				if(isPasswordRequired) {
					if(this.passwordValidationPopup){
						//Remove the old callbacks
						this.passwordValidationPopup.clearCallbacks();
					} else {
						this.passwordValidationPopup = new PasswordValidationPopup();
					}
					
					this.passwordValidationPopup.onSubmit(_.bind(function() {
						this.validateAndSaveChanges(force, completeTask);
						if (!this.actionFormSubmitted) {
							// the form not submitted (probably due to validation issue detected) - enable all submit buttons again
							this.enableSubmitButtons(true);
						}
					}, this, force));
					this.passwordValidationPopup.open();
				} else {
					this.validateAndSaveChanges(force, completeTask);
					if (!this.actionFormSubmitted) {
						// the form not submitted (probably due to validation issue detected) - enable all submit buttons again
						this.enableSubmitButtons(true);
					}
				}
			} catch (err) {
				alert("Problem in WcTaskExecutionPage.changeButtonClicked() method: " + err);
			}
		}.bind(this), 500);
	};
	
	WcTaskExecutionPage.prototype.validateAndSaveChanges = function(force, completeTask) {
		try {
			// 'modSections' string is used to store space-delimited list of names of modified sections 
			var modSections = "";
			var paramsValid = true;
			
			for (var item in this.pageSections) {
				if (this.pageSections.hasOwnProperty(item)){
					var pItem = this.pageSections[item];
					// skip empty and not modified sections unless the force parameter is set (true) - that
					// is in case of completing the task only!
					if (pItem && (pItem.isModified() || force)) {
						// remember actual section as modified
						modSections += pItem.name + " ";
						// check whether all required fields are filled and values are acceptable
						if (!pItem.validateFormParams(force)) {
							paramsValid = false;
							break;
						}
					}
				}
			}
			
			// when nothing was changed...
			if (modSections.length == 0) {
				alert(this.langArr.NO_CHANGES_MADE);
				paramsValid = false;
			}
			
			// return back without submitting the form because one or more parameters filled by user is/are invalid
			// the appropriate warning/error message is displayed within corresponding 
			// PageSection.validateFormParams() method
			if (!paramsValid) {
				return;
			}
			
			// check unique document names
			var evInst = EventBasedValidator.getInstance();
        	var uniqueNamesObj = evInst.createUniqueNamesObj();
        	evInst.validateNamesGlobally(uniqueNamesObj);
			if(!uniqueNamesObj.isOk()){
				return false;
			}
			
			// store info about what sections are modified to designated input field to be passed to TaskActionsHelper.java
			$("#modifiedSections").val(modSections);
			// finally set the action value and submit the whole page
			$("#actionValId").val("ChangeTaskOnExecutionPage");
			
			if(completeTask){
				$("#taskStatusInput").val("00001_0000000003");
			}
			
			// do not prompt user to confirm leaving of the page
			this.exitConfirmNeeded = false;
			this.actionFormSubmitted = true;
			
			$("#actionformId").submit();
			
			this.enableSubmitButtons(false);
			return true;
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.changeButtonClicked() method: " + err);
		}
	};
	
	/**
	 * Function to enable/disable all submit buttons on the page.
	 */
	WcTaskExecutionPage.prototype.enableSubmitButtons = function(enable) {
		// select all input buttons, which are of type button or submit with exception of buttons with "forceSubmit" class
		// (that are used only internally == never visible to the user)
		Array.prototype.filter.call($("#actionformId").find("input"), 
		(function(el){
			return (el.type === "submit" || el.type === "button") && el.className.indexOf("forceSubmit") == -1;
		})).forEach(function(el){
			el.disabled = !enable;
		});
	};

	/**
	 * Assign To Me action:
	 * initialize the SDrop to 'Me' and recreate the SDROP
	 */
	WcTaskExecutionPage.prototype.assignToMeButtonClicked = function(){
		
		// assign to me option
		var taskAssigneeDP =  this.SDrop.dataProvider;
		var sdropOptions = this.SDrop.options;
		
		taskExPage.pageSections.genTaskInfo.modified = true;
		
		// Destroy the current SDrop and initialize the drop down with the 'Me' value
		this.SDrop.destroy(); 
		$("#taskAssigneeSDrop").empty().append($("<option>", {selected: true, value: "current_user", text: this.langArr['ME']})).val("current_user");
		
		// recreate the new SDrop
		this.SDrop = new SDrop($("#taskAssigneeSDrop"), null, taskAssigneeDP, sdropOptions);
	};
	
	
	WcTaskExecutionPage.prototype.cancelButtonClicked = function()
	{
		try {
			$("#actionValId").val("GoBackFromTaskExecutionPage");

			// do not prompt user to confirm leaving of the page
			this.exitConfirmNeeded = false;
			
			//on this depends, if the page is gonna be redirected back to source page(if no changes are made) or stay on TEP and revert changes(if changes are made)
			$("#pageModifiedId").val(this.isModified()?"1":"0");

			$("#actionformId").submit();
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.cancelButtonClicked() method: " + err);
		}
	};

	WcTaskExecutionPage.prototype.startButtonClicked = function()
	{
		try {
			if (confirm(this.langArr.SAVE_AND_START_WCTASK_CONFIRM)) {
				var statusOptions = $("#taskStatusInput option");
				// (the first status of started task is always second in the task status select box - first is "Created")
				statusOptions[1].selected = true;
				// mark the General Info section as modified, because the changed task status is part of it
				this.pageSections.genTaskInfo.modified = true;
				// can't easily submit here because other wctask params could be changed too
				// (changeButtonClicked() method incorporates all validation etc. including submit...)
				this.changeButtonClicked();
			}
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.startButtonClicked() method: " + err);
		}
	};

	WcTaskExecutionPage.prototype.completeButtonClicked = function()
	{
		try {
			if (confirm(this.langArr.SAVE_AND_COMPLETE_WCTASK_CONFIRM)) {
				
				// mark the General Info section as modified, because the changed task status is part of it
				this.pageSections.genTaskInfo.modified = true;
				
				// can't easily submit here because other wctask params could be changed too
				// (changeButtonClicked() method incorporates all validation etc. including submit...)
				this.changeButtonClicked(true, true);
			}
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.completeButtonClicked() method: " + err);
		}
	};


	WcTaskExecutionPage.prototype.editTaskDetailsClicked = function()
	{
		try {
			// set the hidden inputs controlling the outcome of submitting the page to redirect to task details page
			$("input[name=projectTaskID]").val(this.taskID);   //JANH is this necessary???
			$("#actionValId").val("ViewTaskDetails");

			if ($("#actionformId").length > 0) {
				try {				
					$("#actionformId").submit();
				}
				catch (IEerr) {
					// When a page loading action initiated by JavaScript is cancelled by the onbeforeunload event
					// IE will throw an exception. Too bad there's no way to identify this specific exception
					// so ignore exceptions on form submission
				}
			}
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.editTaskDetailsClicked() method: " + err);
		}
	};


	WcTaskExecutionPage.prototype.showTaskWorkflowClicked = function()
	{
		try {
			// set the hidden inputs controlling the outcome of submitting the page to redirect to task details page
			$("input[name=projectTaskID]").val(this.taskID);   //JANH is this necessary???
			$("#actionValId").val("ViewTaskWorkflow");

//			// do not prompt user to confirm leaving of the page
//			this.exitConfirmNeeded = false;

			$("#actionformId").submit();
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.showTaskWorkflowClicked() method: " + err);
		}
	};

	return WcTaskExecutionPage;
});

