/**
 * AEEngineSetup
 *
 * @author KADA
 * @since 9.2.2015
 */
define(["jquery",
        "layout/Template",
        "tpl!pages/workflow_connection/aesetupjmswizard",
        "utils/webi18n!AESetupJMSWizard",
        "./tunnel-service",
        "component/helpIcon/HelpIcon",
        "./aesetupjmswizard-style"
        ], function ($, Template, tpl, webI18n, TunnelService, HelpIcon) {
	
	"use strict";

	/**
	 * 
	 */
	var AESetupJMSWizard = function (element) {
		
		// Wrap the element with jQuery if this is not the case ...
		if (!(element instanceof jQuery)) {
        	this._element = $(element);
        }
		
		// Create the template
		this._t = new Template(webI18n, tpl);
		
		// Add the template to the element.
		this._t.setContext(this._element);
		
		// Setup initial data
		this._t.d.isLAN = false;
		this._t.d.hostName = "";
		this._t.d.tunnelName = "";
		this._t.d.bindingSmartNames = [];
		this._t.d.help = HelpIcon.getInlineHelpTopicHTML;
		
		// What needs to happen when the "AEinLAN" dropdrown is changed.
		this._t.bind("change", "input[name='AEinLAN']", function(source, event){
			
			// Find out if we're now in a LAN 
			this._t.d.isLAN = !this._t.d.isLAN ;
			
			// We need te rerender the template (to show different steps).
			this._t.render();
			
		}, this);
		
		// When the "Setup URL" button is clicked.
		this._t.bind("click", "button[name='SetupURLButton']", function(source, event){
			// Setup the url in the final URL input.
			this.setupURL($(this._finalURLInputSelector));
		}, this);
		
		// Render
		this._t.render();
		
		// Fetch all the active HTTP tunnel bindings
		this._tunnelService = TunnelService.getInstance();
		
		// Start fetching the binding SmartNames
		this.startMonitoringBindingSmartNames(3000);
		
	};
	
	/**
	 * The jQuery selector to get the final URL input.
	 */
	AESetupJMSWizard.prototype._finalURLInputSelector = "#JMFService";
	
	/**
	 * The jQuery selector to get the button to test the final URL input.
	 */
	AESetupJMSWizard.prototype._finalURLCheckButtonSelector = "#urlcheckJMFService";
	
	
	/**
	 * Start monitoring the Binding SmartNames.
	 */
	AESetupJMSWizard.prototype.startMonitoringBindingSmartNames = function (interval) {
		
		var self = this;
		function monitor () {
			setTimeout(function(){
				// Request the current Binding SmartNames
				self._tunnelService.getBindingSmartNames(function (bindingSmartNames) {
					// OnSuccess, update the Binding Smartnames
					self.updateBindingSmartNames(bindingSmartNames);
					// Request new Binding SmartNames (after 1 sec).
					monitor();
				}, function (errorMsg) {
					// Something went wrong requesting the tunnelInfo, show the error (if not paused).
					self.showErrorFetchingBindingSmartNames(errorMsg);
					// Request new tunnelMonitorService update.
					monitor();
				});
			}, interval);
		}
		monitor();
	};
	
	
	/**
	 * Update the bindings.
	 * 
	 * @param bindingSmartNames
	 */
	AESetupJMSWizard.prototype.updateBindingSmartNames = function(bindingSmartNames) {
		
		// Save the smartnames in the template ...
		this._t.d.bindingSmartNames = bindingSmartNames;
		
		// Hide potential error message.
		this.hideErrorFetchingBindingSmartNames();
		
		// Get the dropdown (if it exists)
		var dropDown = this._element.find("select[name='TunnelBindings']");
		
		// Check if the dropdown does exist.
		if (dropDown.length > 0) {
			
			// Remove the loading icon:
			$(".BindingSmartNameLoadingIndicator").hide();
			
			// Remember the selected option value
			var selectedOptionVal = dropDown.val();
			
			// Clear all options.
			dropDown.find('option').remove();
			
			// Always add an "empty" option.
			dropDown.append($("<option></option>").attr("value","").text(""));
			
			// Add all these smartnames to the dropdown.
			for (var i=0; i < bindingSmartNames.length; i++) {
				var smartNameStr = "["+bindingSmartNames[i].getFullName()+"]";
				var smartNameTxt = bindingSmartNames[i].getArguments()[0]+" ("+bindingSmartNames[i].getArguments()[1]+")";
				dropDown.append($("<option></option>").attr("value",smartNameStr).text(smartNameTxt)); 
			}
			
			// Try to select the previous selected value.
			dropDown.find("option[value='"+selectedOptionVal+"']").attr("selected","selected");
		}
	};
	
	
	/**
	 * Set and show the error msg.
	 * 
	 * @param errorMsg
	 */
	AESetupJMSWizard.prototype.showErrorFetchingBindingSmartNames = function(errorMsg) {
		var errorBox = $("div.BindingSmartNameLoadingErrorBox");
		if (errorBox.length > 0) {
			errorBox.empty();
			var errorSpan = $("<span/>");
			errorSpan.text(webI18n.t("WORKFLOWCONNECTION_AE_FETCHING_ERROR_BINDING_SMARTNAMES")+errorMsg);
			errorBox.append("<img class='BindingSmartNameLoadingErrorIcon' src='images/rejected.gif'/>");
			errorBox.append(errorSpan);
			errorBox.show();
		}
		
	};
	
	/**
	 * Hide the error msg.
	 * 
	 * @param errorMsg
	 */
	AESetupJMSWizard.prototype.hideErrorFetchingBindingSmartNames = function() {
		var errorBox = $("div.BindingSmartNameLoadingErrorBox");
		if (errorBox.length > 0) {
			errorBox.hide();
		}
		
	};
	
	/**
	 * 
	 */
	AESetupJMSWizard.prototype.setupURL = function(urlInput) {
		
		var protocol = "http://";
		var port = ":4411";
		var servicePath = "/JDFP/JMF";
		
		// Is AE in a LAN?
		if (!this._t.d.isLAN) {
			// Retrieve the dropdown that contains the Tunnel Binding SmartNames
			var tunnelBindingSmartNameDropDown = this._element.find("select[name='TunnelBindings']");
			// Check if the dropdown exists
			if (tunnelBindingSmartNameDropDown.length > 0) {
				// Get the selected Tunnel Binding SmartName
				var tunnelBindingSmartName = tunnelBindingSmartNameDropDown.val();
				// Check if one is selected.
				if (tunnelBindingSmartName != "") {
					// Create the URL
					var URL = protocol+tunnelBindingSmartName+servicePath;
					// Insert it in the URL Input
					urlInput.val(URL);
					// Test the url
					$( this._finalURLCheckButtonSelector ).trigger("click");
				} else {
					alert(webI18n.t("WORKFLOWCONNECTION_AE_SETUP_NEED_TUNNEL"));
				}
			} else {
				alert(webI18n.t("WORKFLOWCONNECTION_AE_SETUP_NEED_TUNNEL"));
			}
		} else {
			// Retrieve the domainName that contains the Tunnel Binding SmartNames
			var domainNameInput = this._element.find("input[name='AEDomainName']");
			// Check if the input exists
			if (domainNameInput.length > 0) {
				// Get the selected Tunnel Binding SmartName
				var domainName = domainNameInput.val();
				// Check if a host is specified.
				if (domainName != "") {
					// Create the URL
					var URL = protocol+domainName+port+servicePath;
					// Insert it in the URL Input
					urlInput.val(URL);
					// Test the url
					$( this._finalURLCheckButtonSelector ).trigger("click");
				} else {
					alert(webI18n.t("WORKFLOWCONNECTION_AE_SETUP_NEED_HOST"));
				}
			} else {
				alert(webI18n.t("WORKFLOWCONNECTION_AE_SETUP_NEED_HOST"));
			}
		}
	};
	
	
    /**
     * Method to automatically apply this module to a div element ...
     */
	AESetupJMSWizard.wcrinit = function(element){
        new AESetupJMSWizard(element);
    };
	
	return AESetupJMSWizard;
	
});
	