/**
 * A service to resolve strings containing smartnames.
 */
define(["jquery",
        "utils/webi18n!TextContentStrings"], 
        function ($,
        		TextContentStrings) {
	
	var SmartNameResolverService = function () {
        
	};
    
	/**
	 * Resolve a JSON containing strings with smartnames.
	 * 
	 * @param {String} projectId - The Id of the current project (give undefined if non existing).       
	 * @param {String} docVersionId - The id of the current document version (give undefined if non existing).
	 * @param {String} staticSmartNames - A XML containing static smartnames (give undefined if non existing).
	 * @param {Object} smartNameStrings - A JSON containing strings with smartnames in them that need resolving.
	 * @param {Boolean} ignoreEmptyOrNonResolvedSmartNames - What should happen when a smartname could not be resolved or is empty? 
	 *                                                     Should the string be resolved to an empty value (true)? 
	 *                                                     Or should it be set as null in the returned JSON (false)?
	 * @param {Function} callback - Will be executed on successful return from the back end (and be passed the content).
	 * @param {Function} errorCallback - Will be executed on error from back end.
	 */
	SmartNameResolverService.prototype.resolve = function (projectId, docVersionId, staticSmartNames, smartNameStrings, ignoreEmptyOrNonResolvedSmartNames, callback, errorCallback) {
		
		var params = {
			"json" : "1",
			"ajax": "1",
			"neverFail" : ignoreEmptyOrNonResolvedSmartNames
		};
        
        var data = {
            toBeResolved: JSON.stringify(smartNameStrings)
        };
        
        if(projectId) data.projectId = projectId;
        if(docVersionId) data.docVersionId = docVersionId;
        if(staticSmartNames) data.staticSmartNames = staticSmartNames;
		
		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"ResolveSmartName.jsp?" + $.param(params, true);
		$.ajax(url, {
			dataType: "json",
			cache: false,
			noAjaxParam: false,
			contentType: "application/x-www-form-urlencoded; charset=UTF-8",
			type: "POST",
			data: data,
			success: function (jsonData) {
				callback(jsonData);
			},
			error: function (jqXHR, textStatus, errorThrown) {
				if(errorCallback) {
					var msg = $(jqXHR.responseText).find('message').text();
					if(msg) {
						errorCallback(msg);
	            	} else {
	            		errorCallback(TextContentStrings.t("SMARTNAME_RESOLVE_ERROR"));
	            	}
	        	}
			}
		});
	};
	
	return new SmartNameResolverService();
	
});