define(["jquery",
        "underscore"], 
function($, 
		 _){
    "use strict";
    
    /**
     * Simple util class which caches (jQUery) Ajax calls.
     * Just use it as if you would use $.ajax
     */
    var AjaxCache = (function(){
    	var cachedFunction = _.memoize($.ajax, function(options){
    		if(options.type === "GET" && options.url && options.url.length > 2048){
    			var data = parseUrl(options.url);
    			options.url = data.url;
    			options.type = "POST";
        		options.enctype = 'multipart/form-data';
        		options.processData = false;
        		options.contentType = false;
        		options.data = data.fd;
    		}
    		
    		// if there's formData, add extra identifier, as all formData objects will generate the same JSON
    		if(options.data && options.data instanceof FormData){ 
    			if(options.data.keys){
    				var keys = options.data.keys(options.data.keys);
        			var map = {};
        			
        			var next = keys.next().value;
        			while(next){
        				map[next] = options.data.getAll(next);
        				next = keys.next().value;
        			}

        			options.identifier = map;
    			} else {
    				// NOTE: when data.keys is not available (e.g. in IE11), caching will be less powerful for post requests
    				options.identifier = _.uniqueId("AjaxCache");
    			}
    			
    		} 
    		
    		return JSON.stringify(options);
    	});
    	
    	var parseUrl = function(url){
    		var newurl;
    		var parameters;
    		var fd = new FormData();
    		var dataMap = {};
    		
    		var questionmark = url.indexOf("?");
    		if(questionmark === -1){
    			newurl = url;
    		} else {
    			newurl = url.substring(0, questionmark);
    			parameters = url.substring(questionmark+1);

    			_.each(parameters.split("&"), function(param){
    				var currentName = decodeURIComponent(param.substring(0, param.indexOf("=")));
    				var currentValue = decodeURIComponent(param.substring(param.indexOf("=")+1));
    				 (dataMap[currentName] = dataMap[currentName] || []).push(currentValue);
    			});
    		}
    		
    		for(var prop in dataMap){
    			if(dataMap.hasOwnProperty(prop)){
    				if(dataMap[prop].length < 2){
    					fd.append(prop, dataMap[prop][0]);
    				} else {
    					_.each(dataMap[prop], function(inst){
    						fd.append(prop, inst);
    					});
    				}
    			}
    		}
    		
    		return {url: newurl, fd: fd};
    	};
        
        return {
        	ajax: cachedFunction
        };
    })();
    
    return AjaxCache;
});
