define(["jquery",
        "underscore",
        "utils/WCEvent"], 
function($,
         _,
         WCEvent){
	"use strict";

	var AttributeEventBus = (function() {

		var isBusEnabled = true;
		
		var loadedCategories = [];

		function init() {
			return {
				onChangeAttrValueListener : function(callback) {
					WCEvent.on("attribute.valueChanged", callback);
				},
				onCategoryLoadedListener : function(callback) {
					WCEvent.on("attribute.categoryLoaded", callback);
				},
				getLoadedCategories : function () {
					return loadedCategories;
				},
				removeOnChangeAttrValueListener : function(callback) {
					WCEvent.off("attribute.valueChanged", callback);
				},
				removeOnCategoryLoadedListener : function(callback) {
					WCEvent.off("attribute.categoryLoaded", callback);
				},
				
				emitAttrValueChanged : function(srcID, srcAttObj) {
					if (isBusEnabled && !srcAttObj.processingOnChangeEvent) {
						WCEvent.emit("attribute.valueChanged", srcID, srcAttObj);
					}
				},
				emitCategoryLoaded : function(srcACatObj) {
					if (isBusEnabled) {
						loadedCategories.push(srcACatObj);
						WCEvent.emit("attribute.categoryLoaded", srcACatObj);
					}
				}
			};
		};

		return {
			// Get the Singleton instance if one exists
			// or create one if it doesn't
			getInstance : function() {
				if (!window.AttributeEventBus) {
					window.AttributeEventBus = init();
				}
				return window.AttributeEventBus;
			},
			
			isEnabled : function() {
				return isBusEnabled;
			},
			
			setEnabled : function(enabled) {
				isBusEnabled = enabled;
			}
		};

	})();
	
	return AttributeEventBus;
});