define(["jquery",
		"underscore",
		"utils/ConvertUtils"
	],
	function ($,
		_,
		ConvertUtils) {
		"use strict";

		var HTTPUtils = {
			/**
			 * Construct an url (with urlencoded arguments)
			 * @param servlet {String} the servlet to call
			 * @param args {Object} parameters for url (key-value pairs)
			 * @param base {String} url (optional)
			 * @returns {String}
			 */
			constructURL: function (servlet, args, base) {
				var url = "";
				if (base) {
					url += base;
				}
				url += servlet;

				var pre = "?";
				for (var key in args) {
					if (args.hasOwnProperty(key)) {
						if (_.isArray(args[key])) {
							for (var i = 0; i < args[key].length; i++) {
								url += pre + encodeURIComponent(key) + "=" + encodeURIComponent(ConvertUtils.decodeHtmlEntities(args[key][i]));
								pre = "&";
							}
						} else {
							url += pre + encodeURIComponent(key) + "=" + encodeURIComponent(ConvertUtils.decodeHtmlEntities(args[key]));
						}
					}
					pre = "&";
				}


				return url;
			},
			/**
			 * Get a map of all query string parameters
			 * (note: it doesn't support multiple entries with the same name (but it will not crash)
			 * @returns {object} 
			 */
			getQueryStringParams: function () {
				return (function (a) {
					var b = {};
					for (var i = 0; i < a.length; ++i) {
						if (a[i] !== ""){
							var p = a[i].split('=', 2);
							if (p.length == 1) {
								b[p[0]] = "";
							} else {
								b[p[0]] = decodeURIComponent(p[1].replace(/\+/g, " "));
							}
						}
					}
					return b;
				})(window.location.search.substr(1).split('&'));
			}
		};

		return HTTPUtils;
	});