/**
 * @namespace HelpModule
 */
define(["web-viewer/core/AbstractModule",
		"web-viewer/help/HelpWidget",
		"web-viewer/help/ShortcutsPopup",
		"Mousetrap",
		"underscore"
	],
	function (AbstractModule,
		HelpWidget,
		ShortcutsPopup,
		Mousetrap,
		_) {
		"use strict";

		/**
		 * This module is responsible for displaying a help widget
		 *
		 * @exports HelpModule
		 * @constructor
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var HelpModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "help");

			// publish the help widget
			this._publishWidget(HelpWidget.widgetName, function (layoutType) {
				return new HelpWidget(f);
			});

			if ("onhelp" in window) { // IE doesn't allow to block f1 (opens windows help)
				window.onhelp = _.bind(function (e) {
					this.openHelpPages();

					e.preventDefault();
					e.stopPropagation();

					return false;
				}, this);
			} else {
				Mousetrap.bind("f1", _.bind(function (e) {
					this.openHelpPages();

					e.preventDefault();
					e.stopPropagation();
				}, this), "keydown");
			}

			Mousetrap.bind("?", _.bind(function (e) {
				this.openShortcutsPopup();

				e.preventDefault();
				e.stopPropagation();
			}, this));
		};

		// inherit
		HelpModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		HelpModule.prototype.constructor = HelpModule;

		HelpModule.prototype.openHelpPages = function () {
			// note we rely on the fact that the WebCenter data adapter is used, the help system is a WebCenter Specifc tool, so this is not a problem here!
			// in this way the HTML5Viewer help can also be customized by the customer
			var helpURL = this.f.getDataAdapter().WebCenter("help.jsp", {
				lang: this.f.i18n.getLocale(),
				topic: "common/wc/concept/co_wc_TheWebCenterViewer"
			});
			// this is the same popup as the one that is being used in WebCenter :)
			var popup = window.open(helpURL, "help", "scrollbars=1, width=800 ,height=400, toolbar=yes, location=yes, menu=yes");
			popup.focus();
		};

		/**
		 * Open the popup that shows all the shortcuts
		 */
		HelpModule.prototype.openShortcutsPopup = function () {
			if (!this._shortcutsPopup) {
				this._shortcutsPopup = new ShortcutsPopup(this.f);
			}

			this._shortcutsPopup.show();
		};

		return HelpModule;

	});