<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE xsl:stylesheet [
<!ENTITY copy "&#169;">
<!ENTITY nbsp "&#160;">
  <!ENTITY lk "/webcenter_page/user_lang_key">
]>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:java="http://xml.apache.org/xslt/java" xmlns:XSLUtils="com.esko.webcenter.jsp.XSLUtils" xmlns:Lang="com.esko.webcenter.jsp.LanguageMgr" exclude-result-prefixes="XSLUtils java Lang">
	<xsl:import href="templatedate.xsl"/>
	<xsl:import href="approvaltemplate.xsl"/>
	<xsl:import href="wctaskstatustemplates.xsl"/>
	<xsl:import href="projmgmnttemplate.xsl"/>
	<xsl:import href="template.xsl"/>


  <xsl:key name="projectStatusKey" match="/webcenter_page/project_status_list/custom_project_status" use="@id"/>
  <!-- Key path that leads to a Assignment Data (User and Group Members assigned) for a specified Project Role (by its ID) -->
  <xsl:key name="roleAssignmentDataKey" match="/webcenter_page/project_role_assignment_map_list/project_role_assignment_map" use="role_group/@id"/>
  <!-- Key path that leads to a User Member Role Assignment Data (contains Role Names of the assigned Roles)-->
  <xsl:key name="projUserMemberAssignmentDataKey" match="/webcenter_page/project_role_assignment_map_list/project_user_members_role_assignments_list/user" use="@id"/>
  <!-- Key path that leads to a Group Member Role Assignment Data (contains Role Names of the assigned Roles)-->
  <xsl:key name="projGroupMemberAssignmentDataKey" match="/webcenter_page/project_role_assignment_map_list/project_group_members_role_assignments_list/group" use="@id"/>
  <!-- Key path that leads to a Project Group Member > User Members data -->
  <xsl:key name="projGroupMemberUserMembersDataKey" match="/webcenter_page/proj_group_members_user_members/group_list/group" use="@id"/>

	<!-- global variables -->
 	<xsl:variable name="isProjectActive" select="key('projectStatusKey',/webcenter_page/project/status_id)/is_active"/>

	<!--
	
		Template is used for showing header for project task details pages
	
  -->
	<xsl:template name="DisplayTaskPageHeader">
		<xsl:param name="headerText"/>
		<xsl:param name="taskName"/>
		<xsl:param name="displayAction" select="false()"/>
		<xsl:param name="actionLinkName" select="''"/>

		<xsl:choose>
			<xsl:when test="$displayAction">
				<xsl:call-template name="DisplayHeaderWithAction">
					<xsl:with-param name="Name">
						<xsl:value-of select="Lang:getStr(&lk;,$headerText)"/>&nbsp;
						<xsl:value-of select="$taskName"/>
					</xsl:with-param>
					<xsl:with-param name="Action"><xsl:value-of select="$actionLinkName"/></xsl:with-param>
					<xsl:with-param name="Url" select="'javascript:PerformAction(1);'"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="DisplayHeaderNoAction">
					<xsl:with-param name="Name">
						<xsl:value-of select="Lang:getStr(&lk;,$headerText)"/>&nbsp;
						<xsl:value-of select="$taskName"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- template to generate action buttons for task details pages -->
	<xsl:template name="generateTaskDetailsHeaderActions">
		<xsl:param name="displayCopyAction" select="false()" />
		
	  	<div class="header-action-buttons">
	  		<xsl:if test="$displayCopyAction">
	  			<a class="btn btn-default" href="javascript:PerformAction(3)">
	  				<xsl:value-of select="Lang:getStr(&lk;,'COPYPROJECTTASK')"/>
	  			</a>
	  		</xsl:if>
	  		<a class="btn btn-primary" href="javascript:openTaskExecutionPage()">
	  			<xsl:value-of select="Lang:getStr(&lk;, 'TASK_EXECUTION_PAGE_TAG')"/>
	  		</a>
	  	</div>
	</xsl:template>
	<!--
	
	Display a list of documents 
	
-->
	<xsl:template name="DisplayTaskDocs">
		<xsl:param name="docRoot"/>
		<xsl:param name="showDropdownActions" select="false()"/>
		<xsl:param name="showApproval" select="false()"/>
		<xsl:param name="readOnly" select="false()"/>
		<xsl:param name="showNumDocs" select="false()"/>
		<xsl:param name="badgeNum" select="'0'"/>

					<xsl:choose>
						<xsl:when test="count($docRoot) = 0">
							<xsl:call-template name="EmptyTableSection"/>
						</xsl:when>
						<xsl:otherwise>
							<!-- data table -->
							<table class="ordtable">
								<!-- Table header -->
								<thead>
									<tr>
										<!-- Adding a select all toggle -->
										<th class="td-checkbox">
											<xsl:choose>
												<xsl:when test="$readOnly = true()">#</xsl:when>
												<xsl:otherwise>
													<input type="checkbox" name="selectedAllDocs" data-toggle-all="input[name='selectedDocs']"/>
												</xsl:otherwise>
											</xsl:choose>
										</th>
	
										<th>
											<xsl:value-of select="Lang:getStr(&lk;,'PROJDETAILSDOCS_TAG')"/>
											<xsl:if test="$showNumDocs">
												&nbsp;<span class="badge">
													<xsl:value-of select="$badgeNum"/>
												</span>
											</xsl:if>
											<xsl:if test="$showDropdownActions">
												<xsl:call-template name="ShowActionMenu"/>
											</xsl:if>
										</th>
										
										<th class="td-nowrap">
											<xsl:value-of select="Lang:getStr(&lk;,'DESCRIPTION')"/>
										</th>
	
										<th class="td-number">
											<xsl:value-of select="Lang:getStr(&lk;,'VERSION')"/>
										</th>
	
										<xsl:if test="$showApproval">
											<th class="td-status">
												<xsl:value-of select="Lang:getStr(&lk;,'HEADER_APPROVAL')"/>
											</th>
										</xsl:if>
									</tr>
								</thead>
								<tbody>
									<xsl:for-each select="$docRoot">
										<xsl:call-template name="DocumentRow">
											<xsl:with-param name="showApproval" select="$showApproval"/>
											<xsl:with-param name="readOnly" select="$readOnly"/>
										</xsl:call-template>
									</xsl:for-each>
									<xsl:if test="count($docRoot) = 0">
										<xsl:call-template name="EmptyRow">
											<xsl:with-param name="Cols">
												<xsl:choose>
													<xsl:when test="$showApproval">
														<xsl:value-of select="5"/>
													</xsl:when>
													<xsl:otherwise>
														<xsl:value-of select="4"/>
													</xsl:otherwise>
												</xsl:choose>
											</xsl:with-param>
										</xsl:call-template>
									</xsl:if>
								</tbody>
							</table>
						</xsl:otherwise>
					</xsl:choose>
							
	</xsl:template>

	<xsl:template name="DocumentRow">
		<xsl:param name="showApproval" select="true()"/>
		<xsl:param name="readOnly" select="false()"/>

		<xsl:variable name="showThumbnails" select="number(/webcenter_page/show_thumbnails)"/>

		<tr>

			<!-- Check box -->
			<td class="td-checkbox">
				<xsl:choose>
					<xsl:when test="$readOnly = true()">
						<xsl:value-of select="position()"/>
					</xsl:when>
					<xsl:otherwise>
						<input type="checkbox" name="selectedDocs" onclick="Toggle(this)">
							<xsl:attribute name="value">
								<xsl:value-of select="@id"/>
							</xsl:attribute>
						</input>
					</xsl:otherwise>
				</xsl:choose>
			</td>

			<xsl:variable name="description" select="document/description"/>

			<!-- Thumbnail & name -->
			<td>
				<xsl:choose>
					<xsl:when test="$readOnly = true()">
						<img src="{thumbnail_url}" alt="{$description}" title="{$description}" width="50" height="50" border="0" style="padding-right: 5px;" align="middle"/>
						<xsl:call-template name="CutDescription">
							<xsl:with-param name="Description" select="document/name"/>
							<xsl:with-param name="Width" select="'40'"/>
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<a>
							<xsl:call-template name="docDetailsHref">
								<xsl:with-param name="docVerID" select="@id"/>
							</xsl:call-template>
							<xsl:if test="$showThumbnails = '1'">
								<img src="{thumbnail_url}" alt="{$description}" title="{$description}" width="50" height="50" border="0" style="padding-right: 5px;" align="middle"/>
							</xsl:if>
							<xsl:call-template name="CutDescription">
								<xsl:with-param name="Description" select="document/name"/>
								<xsl:with-param name="Width" select="'40'"/>
							</xsl:call-template>
						</a>
					</xsl:otherwise>
				</xsl:choose>
			</td>

			<!-- description  -->
			<td>
				<xsl:call-template name="CutDescription">
					<xsl:with-param name="Description" select="$description"/>
					<xsl:with-param name="Width" select="'50'"/>
				</xsl:call-template>
			</td>

			<!-- version number -->
			<td class="td-number">
				<xsl:call-template name="DocVersionRevisionLabel">
      				<xsl:with-param name="version" select="version_number"/>
      				<xsl:with-param name="revisionLetter" select="revision_letter"/>
      			</xsl:call-template>
			</td>

			<!-- approval info. -->
			<xsl:if test="$showApproval">
				<td class="td-status">
					<xsl:variable name="approval_cycle_state" select="approval_cycle_state"/>
					<xsl:variable name="approval_state_id">
						<xsl:choose>
							<xsl:when test="string-length(approval_state)=0">
								<xsl:value-of select="/webcenter_page/approval_types/approval_type_list/approval_type[type_name='PENDING']/@id"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="approval_state"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>

					<!-- Show the approval icon and the tooltip -->
					<xsl:if test="($approval_cycle_state='cycle_started') or ($approval_cycle_state='cycle_finished') or ($approval_cycle_state='cycle_stopped')">
						<xsl:call-template name="approvalStatus">
				      		<xsl:with-param name="approvalStatusID" select="$approval_state_id"/>
				      		<xsl:with-param name="approvalCycle" select="$approval_cycle_state"/>
				      		<xsl:with-param name="includeFormatting" select="false()"/>      		      		
				      	</xsl:call-template>
					</xsl:if>
				</td>
			</xsl:if>
		</tr>
	</xsl:template>



	<!-- 
	
    template to display task table header
	
  -->
	<xsl:template name="TaskTableHeader">
	  <xsl:param name="isEditable"/>
	  <xsl:param name="showAllDateFields" select="false()"/>

		<xsl:variable name="TaskCount" select="count(project_wc_task_list/project_wctask)"/>
	
	  <thead>
		<tr>
			<th class="td-number">#</th>

			<xsl:if test="$isEditable">
				<th class="td-checkbox">
					<input type="checkbox" name="selectall" value="" data-toggle-all="input[name='selectmember']"/>
				</th>
			</xsl:if>

			<th colspan="2">
				<xsl:value-of select="Lang:getStr(&lk;,'SORT_PROJ_TASK_BY_NAME')"/>
				&nbsp;<span class="badge"><xsl:value-of select="$TaskCount"/></span>
			</th>
			<th>
				<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_OWNER')"/>
			</th>

			<th>
				<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_DEPENDS_ON')"/>
			</th>

			<th class="td-status">
				<xsl:value-of select="Lang:getStr(&lk;, 'SORT_PROJ_TASK_BY_STATUS')"/>
			</th>

			<xsl:if test="$showAllDateFields">
				<th class="td-date">
					<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_CREATED_DATE')"/>
				</th>
			</xsl:if>

			<th class="td-date">
				<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_STARTED_DATE')"/>
			</th>

			<xsl:if test="$showAllDateFields">
				<th class="td-date">
					<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_EXTIMATED_DATE')"/>
				</th>
			</xsl:if>

			<th class="td-date">
				<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_DUE_DATE')"/>
			</th>

			<th class="td-date">
				<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_COMPLETED_DATE')"/>
			</th>
			
			<th>
				<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_DOCS')"/>
			</th>
		</tr>
	  </thead>
	</xsl:template>


	<!--
		Total tasks based on the results of query
	-->
	<xsl:template name="DisplayTotals">
		<xsl:param name="isEditable"/>

		<xsl:variable name="TaskCount" select="count(project_wc_task_list/project_wctask)"/>

		<table border="0" width="100%" cellspacing="0" cellpadding="0">
			<tr>
				<td align="left" class="large_text" nowrap="nowrap" style="width: 40%;">
					<img src="images/clearpixel.gif" width="10" height="1"/>
					<xsl:value-of select="Lang:getStr(&lk;,'SEARCHCOUNT_PART1')"/>&nbsp;
					<xsl:value-of select="$TaskCount"/>
				</td>

				<xsl:if test="$isEditable">
					<td align="left" class="large_text_action_link">
						<a href="javascript:PerformAction(1);">
							<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_NEW')"/>
						</a>
					</td>
				</xsl:if>
			</tr>
		</table>
	</xsl:template>




	<!-- 
	
    template to display task details
	
  -->
	<xsl:template name="ProjectWcTaskDetailsRow">
		<xsl:param name="isEditable"/>
		<xsl:param name="showAllDateFields" select="false()"/>
		<xsl:param name="fullEditLinkOn" select="false()"/>

		<xsl:variable name="curTaskID" select="@id"/>
		
		<xsl:variable name="isOwnerRole">
			<xsl:choose>
				<xsl:when test="(task_owner_type='group') and (group/is_role = '1')"><xsl:value-of select="true()"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="false()"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:variable name="taskOwnerRoleNames">
			<xsl:choose>
				<xsl:when test="(task_owner_type='group') and (group/is_role = '0')"><xsl:value-of select="key('projGroupMemberAssignmentDataKey',task_owner_id)"/></xsl:when>
				<xsl:when test="(task_owner_type='user')"><xsl:value-of select="key('projUserMemberAssignmentDataKey',task_owner_id)"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="''"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<tr id="node-{position()}">
			<td class="td-number">
				<xsl:value-of select="position()"/>
			</td>

			<xsl:if test="$isEditable ">
				<td class="td-checkbox">
					<input type="checkbox" name="selectmember" onclick="Toggle(this)" value="{@id}"/>
				</td>
			</xsl:if>
   
			<!-- task name column -->
			<xsl:call-template name="WcTaskDetailsName">
				<xsl:with-param name="taskName" select="name"/>
				<xsl:with-param name="projectTaskID" select="@id"/>
				<xsl:with-param name="icon" select="task_type_icon"/>
				<xsl:with-param name="classNames" select="'treeColumn'"/>
			</xsl:call-template>

			<!-- task "action" column -->
			<td>
				<div class="row-actions">
 				<xsl:if test="has_workflow = '1' and $isEditable">
						<a class="action">
							<xsl:attribute name="href">javascript:ShowTaskWorkflow('<xsl:value-of select="$curTaskID"/>');</xsl:attribute>
							<xsl:attribute name="title"><xsl:value-of select="Lang:getStr(&lk;,'TASKTYPEDETAILSWORKFLOW_TAG')"/></xsl:attribute>
							<i class="icon icon-WCR_Actions-Workflow12"></i>
						</a>
 				</xsl:if>

				<xsl:if test="$fullEditLinkOn">
						<a class="action">
							<xsl:attribute name="href">javascript:ShowFullTaskDetails('<xsl:value-of select="$curTaskID"/>');</xsl:attribute>
							<xsl:attribute name="title"><xsl:value-of select="Lang:getStr(&lk;,'EDIT_TAG')"/></xsl:attribute>
							<i class="icon icon-WCR_Actions-Edit12"></i>
						</a>
				</xsl:if>
				</div>
			</td>

			<!-- task owner -->
			<td>
				<xsl:choose>
					<!--  User type Task Owner/Assignee -->
					<xsl:when test="task_owner_type='user'">
						<xsl:call-template name="DisplayUserName">
							<xsl:with-param name="userID" select="task_owner_id"/>
							<xsl:with-param name="lastName" select="user/last_name"/>
							<xsl:with-param name="firstName" select="user/first_name"/>
							<xsl:with-param name="loginName" select="user/name" />
							<xsl:with-param name="isActive" select="user/is_active" />
							<xsl:with-param name="displayIcon" select="1"/>
							<xsl:with-param name="roleGroupNameList" select="$taskOwnerRoleNames"/>
						</xsl:call-template>
					</xsl:when>
					<!--  Group type Task Owner/Assignee -->
					<xsl:when test="(task_owner_type='group') and ($isOwnerRole = 'false')">
						<xsl:call-template name="DisplayGroupName">
							<xsl:with-param name="groupID" select="task_owner_id"/>
							<xsl:with-param name="groupName" select="group/name"/>
							<xsl:with-param name="isRole" select="group/is_role"/>
							<xsl:with-param name="displayIcon" select="1"/>
							<xsl:with-param name="roleGroupNameList" select="$taskOwnerRoleNames"/>
						</xsl:call-template>
					</xsl:when>
					<!--  Role type Task Owner/Assignee -->
					<xsl:when test="$isOwnerRole">
						<xsl:choose>
							<!-- If the Role is actually assigned -->
							<xsl:when test="key('roleAssignmentDataKey',task_owner_id)">
								<xsl:call-template name="DisplayRoleNameWithAssignments">
									<xsl:with-param name="roleAssignmentData" select="key('roleAssignmentDataKey',task_owner_id)"/>
									<xsl:with-param name="projectID" select="project_id"/>
								</xsl:call-template>
							</xsl:when>
							<!-- The Role has no assignments yet : just write the Role Name/Link -->
							<xsl:otherwise>
								<xsl:call-template name="DisplayGroupName">
									<xsl:with-param name="groupID" select="task_owner_id"/>
									<xsl:with-param name="projectID" select="project_id"/>
									<xsl:with-param name="groupName" select="group/name"/>
									<xsl:with-param name="isRole" select="group/is_role"/>
									<xsl:with-param name="displayIcon" select="1"/>
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
				</xsl:choose>
			</td>

			<!-- comma-separated list of row numbers of current task predecessors -->
			<td id="PredecessorList{$curTaskID}">
				<!-- the content of this table cell is generated by generateDependsOnList() function from wctasks.js -->
				<!-- (see script at the end of this template...) -->
			</td>
			
			<!-- store if task is completed inside of the hidden field -->
			<xsl:variable name="statusID" select="status/@id"/>
			
			<xsl:variable name="statusFieldName" select="concat('isCompletedStatus_', @id)"/>
			<input type="hidden" id="{$statusFieldName}">
				<xsl:attribute name="value">
					<xsl:choose>
						<xsl:when test="$statusID = '00001_0000000003'">1</xsl:when>
						<xsl:otherwise>0</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
			</input>

			<!-- task status -->
			<xsl:call-template name="DisplayProjectWcTaskStatus">
				<xsl:with-param name="projectWcTask" select="."/>
			</xsl:call-template>


			<!-- create date -->
			<xsl:if test="$showAllDateFields">
				<td class="td-date">
					<xsl:call-template name="HumanReadableLongDate">
						<xsl:with-param name="LongDateString" select="created_date"/>
					</xsl:call-template>
				</td>
			</xsl:if>

			<!-- start date -->
			<td class="td-date">
				<xsl:call-template name="HumanReadableLongDate">
					<xsl:with-param name="LongDateString" select="started_date"/>
				</xsl:call-template>
			</td>

			<!-- estimated due date -->
			<xsl:if test="$showAllDateFields">
				<td class="td-date">
					<xsl:attribute name="class">
						<xsl:choose>
							<!-- for completed tasks use the regular font -->
							<xsl:when test="$statusID = '00001_0000000003'">darktext_darklink_11</xsl:when>
							<xsl:otherwise>
								<xsl:call-template name="GetDateStyle">
									<xsl:with-param name="inputDate" select="estimated_date"/>
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>

					<img border="0" src="images/clearpixel.gif" width="5" height="1"/>
					<xsl:call-template name="HumanReadableLongDate">
						<xsl:with-param name="LongDateString" select="estimated_date"/>
					</xsl:call-template>
				</td>
			</xsl:if>

			<!-- due date -->
			<td>
				<xsl:attribute name="class">
					<xsl:choose>	
					<xsl:when test="status/@id = '00001_0000000003'">
						<!-- task is COMPLETED ... just display the due date without emphasizing -->
						<xsl:text>td-date</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<!-- process class attribute based on the value of task due date -->
		
						<xsl:call-template name="GetDateStyle">
							<xsl:with-param name="inputDate" select="due_date"/>
						</xsl:call-template>
					</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
				
						<xsl:call-template name="HumanReadableLongDate">
							<xsl:with-param name="LongDateString" select="due_date"/>
						</xsl:call-template>
			</td>


			<!-- actual completed date -->
			<td class="td-date">
				<xsl:call-template name="HumanReadableLongDate">
					<xsl:with-param name="LongDateString" select="actual_completed_date"/>
				</xsl:call-template>
			</td>

			<!-- list of docs -->
			<td>
				<table border="0" cellspacing="0" cellpadding="0" here="1">
				<xsl:variable name="nDocuments" select="count(doc_version_list/doc_version_pair)" />
				<xsl:variable name="nShown" select="2" />
				
				<xsl:choose>
					<xsl:when test="$nDocuments &lt;= ($nShown+1)">
						<xsl:for-each select="doc_version_list/doc_version_pair">
							<tr><td>
								<xsl:call-template name="ProjectWcTaskDocumentRow">
									<xsl:with-param name="version_id" select="@id"/>
									<xsl:with-param name="doc_name" select="doc_name"/>
								</xsl:call-template>
							</td></tr>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>
					
						<!-- Only show the first two documents. -->
						<xsl:for-each select="doc_version_list/doc_version_pair[position() &lt;= $nShown]">
							<tr><td>
								<xsl:call-template name="ProjectWcTaskDocumentRow">
									<xsl:with-param name="version_id" select="@id"/>
									<xsl:with-param name="doc_name" select="doc_name"/>
								</xsl:call-template>
							</td></tr>
						</xsl:for-each>
					
						<!-- If there are more than two documents, show the ... and make a popup menu.  -->
					
						<tr><td>
						<a>
						<xsl:attribute name="href"></xsl:attribute>
						<xsl:attribute name="onclick">return clickreturnvalue(event)</xsl:attribute>
						<xsl:attribute name="onmouseover">dropdown(this, event, docmenu<xsl:value-of select="position()" />,"400","400", false,"",false,-8,-3)</xsl:attribute>
						<xsl:attribute name="onmouseout">delayhidemenu()</xsl:attribute>…</a></td></tr>
						
					
						<!-- Create the menu with links -->
						<script type="text/javascript">
							<xsl:text disable-output-escaping="yes">&lt;!--</xsl:text>&#xA;
							
								var menu = new Array();
								<xsl:for-each select="doc_version_list/doc_version_pair[position() &gt; $nShown]">
								
									menu.push('<a class="hoverOverflow">
											<xsl:call-template name="docDetailsHref">
												<xsl:with-param name="docVerID" select="@id"/>
											</xsl:call-template>
											<xsl:call-template name="encodesinglequotes"><xsl:with-param name="string" select="doc_name"/></xsl:call-template>
											</a>&lt;br&gt;');
								
								</xsl:for-each>
							
								var docmenu<xsl:value-of select="position()"></xsl:value-of> = menu;
							<xsl:text disable-output-escaping="yes">//--&gt;</xsl:text>&#xA;
						</script>
					</xsl:otherwise>
				</xsl:choose>
				</table>
			</td>
		</tr>
		
		<xsl:if test="count(predecessor_list/predecessor_id) &gt; 0">
		<!-- Script generating "Depends On" column (predecessor list) content -->
	  <script type="text/javascript">
	    <xsl:text disable-output-escaping="yes">&lt;!--</xsl:text>&#xA;
				var dependsOnIDs = new Array(
				  <xsl:for-each select="predecessor_list/predecessor_id">
							  "<xsl:value-of select="."/>"<xsl:if test="position()!=last()">,</xsl:if>
				  </xsl:for-each>
				);
				generateDependsOnList("PredecessorList"+"<xsl:value-of select="$curTaskID"/>", dependsOnIDs, predecessorIdxs);
	    <xsl:text disable-output-escaping="yes">//--&gt;</xsl:text>&#xA;
	  </script>
		</xsl:if>
	</xsl:template>

	<!-- 

    template to display task name as a link to task details page

  -->
	<xsl:template name="WcTaskDetailsName">
		<xsl:param name="taskName"/>
		<xsl:param name="projectTaskID"/>
		<xsl:param name="isNameLink" select="true()"/>
		<xsl:param name="classNames" select="''"/>

		<td class="{$classNames} td-breakword">
			<xsl:choose>
				<xsl:when test="$isNameLink">
					<a>
						<xsl:attribute name="href">javascript:ShowTaskDetails('<xsl:value-of select="$projectTaskID"/>');</xsl:attribute>
						<xsl:value-of select="$taskName"/>
					</a>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$taskName"/>
				</xsl:otherwise>
			</xsl:choose>
		</td>
	</xsl:template>

	<!-- 
    template to display a doc in the task 
  -->
	<xsl:template name="ProjectWcTaskDocumentRow">
		<xsl:param name="version_id"/>
		<xsl:param name="doc_name"/>

			<a class="task-doc-link">
				<xsl:call-template name="docDetailsHref">
					<xsl:with-param name="docVerID" select="$version_id"/>
				</xsl:call-template>
				<xsl:value-of select="$doc_name"/>
			</a>
	</xsl:template>


	<!--
	
	Template for display of main action buttons on Task General Info page.
	
  -->
	<xsl:template name="ProjectWcTaskGenInfoMainButtons">
		<xsl:param name="isChangeable"/>
		<xsl:param name="newTaskComposing"/>
		<xsl:param name="isTaskComplete"/>
	
		<!--  Change button -->
    <input type="button" name="createWCTask" class="btn btn-primary">
     	<xsl:attribute name="onclick">changeButtonClicked(this, <xsl:value-of select="$isChangeable"/>);</xsl:attribute>
    	<xsl:choose>
    		<xsl:when test="$newTaskComposing">
    			<xsl:attribute name="value"><xsl:value-of select="Lang:getStr(&lk;,'CREATE')"/></xsl:attribute>
    		</xsl:when>
    		<xsl:otherwise>
    			<xsl:attribute name="value"><xsl:value-of select="Lang:getStr(&lk;,'CHANGE')"/></xsl:attribute>
    		</xsl:otherwise>
    	</xsl:choose>
    </input>
  
		<!--  Cancel button -->
    <input type="button" name="cancelButton" class="btn btn-default"
  		onclick="document.actionform.actionVal.value = 'GoBackToTaskList'; document.actionform.submit();">
			<xsl:choose>
 			<xsl:when test="$isTaskComplete">
 				<xsl:attribute name="value"><xsl:value-of select="Lang:getStr(&lk;,'RETURN')"/></xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
 				<xsl:attribute name="value"><xsl:value-of select="Lang:getStr(&lk;,'CANCEL')"/></xsl:attribute>
				</xsl:otherwise>
				</xsl:choose>
    </input>
	</xsl:template>
	
	
	<!--
	
		Check if all sections are hidden
	
	-->
	<xsl:template name="allHidden">
		<xsl:param name="hide" select="''"/>
		
		<xsl:choose>
			<xsl:when test="contains($hide, 'B1') and contains($hide, 'T') and contains($hide, 'S') and contains($hide, 'Y') and 
							contains($hide, 'M') and contains($hide, 'A') and contains($hide, 'N') and contains($hide, 'D') and 
							contains($hide, 'P') and contains($hide, 'L') and contains($hide, 'U') and contains($hide, 'E') and 
							contains($hide, 'R') and contains($hide, 'C') and contains($hide, 'B2')">
				<xsl:value-of select="1"/> 
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="0"/> 
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	

	<!--
	
	Template for display of task general info.
	
  -->
	<xsl:template name="ProjectWcTaskGenInfo">
		<xsl:param name="displayAction" select="true()"/>
		<xsl:param name="origStatus" select="''"/>
		
		<!-- 	introduced because of workflow error status - but might be merged with other options later on -->
		<xsl:variable name="isTaskPageReadonly" select="/webcenter_page/is_task_page_readonly = '1'"/>
		
    	<xsl:variable name="isProjectEditable" select="is_project_editable = '1'" />
		<xsl:variable name="isTemplate" select="project/is_template = '1'"/>
		<xsl:variable name="isEditableTemplate" select="$isTemplate and $isProjectEditable"/>
		<xsl:variable name="isProjectClosed" select="($isProjectActive != '1') and not($isEditableTemplate)"/>

		<xsl:variable name="newTaskComposing" select="not(project_wctask/status)"/>
		<xsl:variable name="isTaskComplete" select="project_wctask/is_complete = '1'" />
		<!-- task is started if not just under creation process and if its status is not "created" -->
		<xsl:variable name="isTaskStarted" select="not($newTaskComposing) and (project_wctask/status/@id != '00001_0000000001')"/>
		<xsl:variable name="changeTaskRightGranted" select="project_user_permissions/permission/change_task = '1'"/>
		<xsl:variable name="isAssigneeLoggedIn" select="is_current_user_assignee = '1'"/>

		<xsl:variable name="isChangeable" select="not($isTaskComplete) and ($newTaskComposing or $changeTaskRightGranted or ($isEditableTemplate and not($isTaskComplete) and not($isTaskPageReadonly)))"/>
		<xsl:variable name="isStatusEditable" select="(((is_task_status_editable = '1') and not($isProjectClosed)) or $isEditableTemplate) and not($isTaskPageReadonly)"/>
		<!-- if user is direct assignee or member of group assignee and task is not complete -->
		<!-- and project (in which the task is) is active set 'canChangeAssignee' to true    -->
		<xsl:variable name="canChangeAssignee" select="($isAssigneeLoggedIn and not($isTaskComplete) and (($isProjectActive = '1') or $isProjectEditable)) and not($isTaskPageReadonly)"/>
		<xsl:variable name="currentUserAssignee" select="(project_wctask/task_owner_id = /webcenter_page/loginuserid) and (project_wctask/task_owner_type = 'user')"/>
		
		<xsl:variable name="hide" select="hiddenElements"/>
		<xsl:variable name="start" select="startTaskOption"/>
		<xsl:variable name="execute" select="executeOnCreate"/>
		<xsl:variable name="sourcePage" select="Source_URL"/>
		

<!-- JANH for debug use 
===================================<br/>
newTaskComposing: <xsl:value-of select="$newTaskComposing"/><br/>
isTemplate: <xsl:value-of select="$isTemplate"/><br/>
isEditableTemplate: <xsl:value-of select="$isEditableTemplate"/><br/>
isProjectActive: <xsl:value-of select="$isProjectActive = '1'"/><br/>
isChangeable: <xsl:value-of select="$isChangeable"/><br/>
isProjectClosed: <xsl:value-of select="$isProjectClosed"/><br/>
isStatusEditable: <xsl:value-of select="$isStatusEditable"/><br/>
canChangeAssignee: <xsl:value-of select="$canChangeAssignee"/><br/>
isTaskStarted: <xsl:value-of select="$isTaskStarted"/><br/>
===================================<br/>
isTaskComplete: <xsl:value-of select="$isTaskComplete"/>|<xsl:value-of select="not($isTaskComplete)"/><br/>
===================================
-->

		<input type="hidden" name="completeForm" value="{$isChangeable}"/>
		<input type="hidden" name="executeOnCreate" value="{$execute}"/>
		<xsl:for-each select="docversionids/docversionid">
			<input type="hidden" name="docversionid" value="{.}"/>
		</xsl:for-each>
		<input type="hidden" name="sourcePageURL" value="{$sourcePage}"/>
		
		<div class="form-section">
			
			<!-- upper main action buttons ("Change" and "Cancel") -->
			<xsl:if test="$isChangeable or $isStatusEditable or $canChangeAssignee">
				<div class="form-group">
					<xsl:if test="contains($hide, 'B1')">
						<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
					</xsl:if>
					<label class="offset-label"></label>
					<div class="form-entry">
						<xsl:call-template name="ProjectWcTaskGenInfoMainButtons">
							<xsl:with-param name="isChangeable" select="$isChangeable"/>
							<xsl:with-param name="newTaskComposing" select="$newTaskComposing"/>
							<xsl:with-param name="isTaskComplete" select="$isTaskComplete"/>
						</xsl:call-template>
					</div>
				</div>
			</xsl:if>
			
			<!-- task type -->
			<div class="form-group">
				<xsl:if test="contains($hide, 'T')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<label for="taskType">
					<xsl:if test="$isChangeable">
						<xsl:attribute name="class">form-required</xsl:attribute>
					</xsl:if>
					<xsl:value-of select="Lang:getStr(&lk;,'TASKTYPE')"/>
				</label>
				<div class="form-entry">
					<xsl:choose>
						<xsl:when test="$newTaskComposing">
							<xsl:choose>
								<xsl:when test="count(/webcenter_page/wctask_type_list/wctask_type_view)=1">
									<!-- single option, no need to show choice -->
									<p><xsl:value-of select="/webcenter_page/wctask_type_list/wctask_type_view/wctask_type/task_type_name"/></p>
									<input type="hidden" id="taskType" name="taskType" value="{/webcenter_page/wctask_type_list/wctask_type_view/wctask_type/@id}"/>
								</xsl:when>
								<xsl:otherwise>
									<select name="taskType" id="taskType" onchange="selectTaskType(this)" class="form-control">
										<option value="" >---</option>
										<xsl:for-each select="/webcenter_page/wctask_type_list/wctask_type_view">
											<xsl:sort select="wctask_type/wctask_type_name" data-type="text" order="ascending"/>
											<option value="{wctask_type/@id}">
												<xsl:value-of select="wctask_type/task_type_name"/>
											</option>
									  </xsl:for-each>
									</select>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise>
							<!-- task type cannot be modified after a task is created! -->
							<p><strong><xsl:value-of select="project_wctask/task_type_name"/></strong></p>
							<input type="hidden" id="taskType" name="taskType" value="{project_wctask/task_type_id}"/>
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>
			
			<!-- status -->
			<div class="form-group">
				<xsl:if test="contains($hide, 'S')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<input type="hidden" id="taskStatusID" name="taskStatusID" value=""/>
				<label for="taskStatusInput"><xsl:value-of select="Lang:getStr(&lk;,'TASKTYPEDETAILSSTATUS_TAG')"/></label>
				<div class="form-entry" id="taskStatusTD">
					<!-- content completely generated by Javascript (wctasks.js, method generateTaskStatusContent()) -->
				</div>
			</div>
			
			<!-- created by -->
			<!-- TODO: should we not hide this section when creating a new task ($newTaskComposing)? - check if FS needs to be updated for this -->
			<div class="form-group" id="taskCreatedByRow">
				<xsl:if test="contains($hide, 'Y')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<label><xsl:value-of select="Lang:getStr(&lk;,'TASK_TYPE_CREATED_BY')"/></label>
				<div class="form-entry">
					<p>
					<xsl:choose>
						<xsl:when test="$newTaskComposing">
							<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_ASSIGNTOMYSELF')"/>
							<xsl:if test="not($isTemplate) or $isEditableTemplate">
								&nbsp;<xsl:value-of select="Lang:getStr(&lk;,'AT')"/>&nbsp;<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_THIS_MOMENT')"/>
							</xsl:if>
						</xsl:when>
						<xsl:otherwise>
								<xsl:call-template name="DisplayUserName">
									<xsl:with-param name="userID" select="project_wctask/creator/user/@id"/>
									<xsl:with-param name="lastName" select="project_wctask/creator/user/last_name"/>
									<xsl:with-param name="firstName" select="project_wctask/creator/user/first_name"/>
									<xsl:with-param name="loginName" select="project_wctask/creator/user/name"/>
									<xsl:with-param name="isActive" select="project_wctask/creator/user/is_active" />
									<xsl:with-param name="displayIcon" select="1"/>
								</xsl:call-template>
								
							<xsl:if test="not($isTemplate) or $isEditableTemplate">
								&nbsp;<xsl:value-of select="Lang:getStr(&lk;,'AT')"/>&nbsp;
								<xsl:call-template name="HumanReadableLongDateAndTime">
									<xsl:with-param name="LongDateString" select="created_date/long_date"/>
								</xsl:call-template>
							</xsl:if>
						</xsl:otherwise>
					</xsl:choose>
					</p>
				</div>
			</div>
			
			
			<!-- Allowed assignee selector :-  
				
				 This row is only shown when working (creating a new one or editing an existing one) with a restricted task.
				 
				 If it is shown a dropdown that shows the Allowed Assignee selection is displayed.
				
				 It is only editable when an Admin user is editing a 'restricted task' 
				 which does not have 'preselected Allowed Assignee' on the parent Task Type level.
				
				 When the Task Type has the Allowed Assignee (aka Main Assignee) pre-selected, 
				 no-one can overrule this setting and the dropdown is disabled.
			
				 If a non-Admin User is accesing "edit task" page of a restricted Task he will only see the current Main Assignee in
				 a disabled dropdown selector. It will also be disabled when any user edits a restricted Task which is Completed.
			-->
			<xsl:variable name="editTaskMainAssigneeID" select="project_wctask/main_assignee_id"/>	
			<xsl:variable name="taskTypeMainAssigneeID"><xsl:value-of select="project_wctask/wctask_type/main_assignee_id"/></xsl:variable>
			<xsl:variable name="hideMARow" select="$newTaskComposing or string-length($editTaskMainAssigneeID) = 0 or contains($hide, 'M')"/>
			<xsl:variable name="MAselectionROMode" select="not(/webcenter_page/is_admin) or string-length($taskTypeMainAssigneeID) > 0 or $isTaskComplete"/>
			
			<div class="form-group" id="taskMainAssigneeRow">
				<xsl:if test="$hideMARow">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				
				<label for="taskMainAssignee"><xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_ALLOWED_ASSIGNEE')"/></label>
				
				<div class="form-entry" id="taskMainAssigneeSelectorCell">
					<select name="taskMainAssignee" id="taskMainAssignee" class="form-control">
						<xsl:if test="$MAselectionROMode">
							<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
						</xsl:if>
						
						<!-- Show all Role members  -->
						<xsl:for-each select="project_group_member_view_list/project_group_member_view[group/is_role ='1']">
							<xsl:sort select="group/name" data-type="text" order="ascending"/>	
					
							<!-- write the main ROLE option-->
							<option value="{group/@id}"> 
								<xsl:if test="group/@id = $editTaskMainAssigneeID">
									<xsl:attribute name="selected">selected</xsl:attribute>
								</xsl:if>
					
								<xsl:call-template name="DisplayGroupName">
									<xsl:with-param name="groupName" select="group/name"/>
									<xsl:with-param name="isRole" select="group/is_role"/>
								</xsl:call-template>
							</option>
						</xsl:for-each>
					
						<!--  show all group project members -->
						<xsl:for-each select="project_group_member_view_list/project_group_member_view[group/is_role ='0']">
							<xsl:sort select="group/name" data-type="text" order="ascending"/>
						
							<option value="{group/@id}">
								<xsl:if test="group/@id = $editTaskMainAssigneeID">
									<xsl:attribute name="selected"/>
								</xsl:if>
					
								<xsl:call-template name="DisplayGroupName">
									<xsl:with-param name="groupName" select="group/name"/>
									<xsl:with-param name="isRole" select="group/is_role"/>
								</xsl:call-template>
							</option>
						</xsl:for-each>
					</select>
					
					<!-- hidden input to still be able to pass over Allowed Assignee reference, even if we disable the select element above -->
					<input type="hidden" id="taskMainAssigneePreselected" name="taskMainAssigneePreselected" value="" />
					
					<!-- RO readout mode -->
					<div id="taskMainAssigneeROModeDiv">
						<xsl:if test="not($MAselectionROMode)">
							<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
						</xsl:if>
					
						<xsl:variable name="groupName" select="key('projGroupMemberUserMembersDataKey', $editTaskMainAssigneeID)/name"/>
						<xsl:variable name="isRole" select="key('projGroupMemberUserMembersDataKey', $editTaskMainAssigneeID)/is_role"/>
						<p>
						<xsl:call-template name="DisplayGroupName">
							<xsl:with-param name="groupID" select="$editTaskMainAssigneeID"/>
							<xsl:with-param name="projectID" select="project_wctask/project_id"/>
							<xsl:with-param name="groupName" select="$groupName"/>
							<xsl:with-param name="isRole" select="$isRole"/>
							<xsl:with-param name="displayIcon" select="1"/>
							<xsl:with-param name="roleGroupNameList" select="key('projGroupMemberAssignmentDataKey', $editTaskMainAssigneeID)"/>
						</xsl:call-template>
						</p>
					</div>
				</div>
			</div>
			
			<!-- Task Assignee - Assigned to -->
			<div class="form-group">
				<xsl:if test="contains($hide, 'A')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
						<label for="taskOwner"><xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_ASSIGNE_TO')"/></label>
				<div class="form-entry">
						<xsl:choose>
							<xsl:when test="not($canChangeAssignee or $isChangeable)">
								<!-- if task completed or just looking into read-only template - task assignee can't be changed -->
								<xsl:choose>
									<xsl:when test="project_wctask/user/@id">
										<!-- assignee is a user -->
										<xsl:call-template name="DisplayUserName">
											<xsl:with-param name="userID" select="project_wctask/user/@id"/>
											<xsl:with-param name="lastName" select="project_wctask/user/last_name"/>
											<xsl:with-param name="firstName" select="project_wctask/user/first_name"/>
											<xsl:with-param name="loginName" select="project_wctask/user/name"/>
											<xsl:with-param name="isActive" select="project_wctask/user/is_active"/>
											<xsl:with-param name="displayIcon" select="1"/>
									<xsl:with-param name="roleGroupNameList" select="key('projUserMemberAssignmentDataKey', project_wctask/user/@id)"/>
										</xsl:call-template>
									</xsl:when>
									<xsl:when test="project_wctask/group/@id">
										<!-- assignee is a group -->
										<xsl:call-template name="DisplayGroupName">
											<xsl:with-param name="groupID" select="project_wctask/group/@id"/>
											<xsl:with-param name="projectID" select="project_wctask/project_id"/>
											<xsl:with-param name="groupName" select="project_wctask/group/name"/>
											<xsl:with-param name="isRole" select="project_wctask/group/is_role"/>
											<xsl:with-param name="displayIcon" select="1"/>
											<xsl:with-param name="roleGroupNameList" select="key('projGroupMemberAssignmentDataKey', project_wctask/group/@id)"/>
										</xsl:call-template>
									</xsl:when>
								</xsl:choose>
							</xsl:when>
							<xsl:otherwise>
								<!-- flag whether currently logged in user is selected as assignee of the task -->
								<xsl:variable name="currentUserAssignee" select="(project_wctask/task_owner_id = /webcenter_page/loginuserid) and (project_wctask/task_owner_type = 'user')"/>
						
								<!-- write the SELECT element itself -->
								<xsl:call-template name="ProjectWCTaskAssigneeDropdownSelector">
									<xsl:with-param name="project_wctask" select="project_wctask"/>
									<xsl:with-param name="canChangeAssignee" select="$canChangeAssignee"/>
									<xsl:with-param name="isAssigneeLoggedIn" select="$isAssigneeLoggedIn"/>
									<xsl:with-param name="isChangeable" select="$isChangeable"/>
									<xsl:with-param name="currentUserAssignee" select="$currentUserAssignee"/>
									<xsl:with-param name="newTaskComposing" select="$newTaskComposing"/>
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
				</div>
			</div>
			
			<!-- 
				Allowed Assignee determined by initial Task Assignment row 
				Initially hidden - will only be shown when selecting an appropriate restricted Task Type
				in order to help the Task Creator in chosing a valid Task Assignee.
			-->
			<div class="form-group" id="freeMainAssigneeTipRow" style="display:none">
				<!-- Note: Check for hide parameter (A section) is unneeded as this section is hidden by default (enabled from JS only) -->				
				<label class="offset-label"></label>
				<div class="form-entry">
					<p class="help-block">
						<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_ALLOWED_ASSIGNEE_TIP')"/>
					</p>
				</div>
			</div>
			
			<!-- previous assignee (owner) -->
			<xsl:if test="not($newTaskComposing)">
				<div class="form-group" id="taskPreviousAssigneeRow">
					<label><xsl:value-of select="Lang:getStr(&lk;,'SEARCH_PREV_PROJ_TASK_OWNER')"/></label>
					<div class="form-entry">
						<p>
						<xsl:choose>
							<xsl:when test="project_wctask/previous_owner/user/@id">
								<!-- assignee is a user -->
								<xsl:call-template name="DisplayUserName">
									<xsl:with-param name="userID" select="project_wctask/previous_owner/user/@id"/>
									<xsl:with-param name="lastName" select="project_wctask/previous_owner/user/last_name"/>
									<xsl:with-param name="firstName" select="project_wctask/previous_owner/user/first_name"/>
									<xsl:with-param name="loginName" select="project_wctask/previous_owner/user/name"/>
									<xsl:with-param name="isActive" select="project_wctask/previous_owner/user/is_active" />
									<xsl:with-param name="roleGroupNameList" select="key('projUserMemberAssignmentDataKey', project_wctask/previous_owner/user/@id)"/>
									<xsl:with-param name="displayIcon" select="1"/>
								</xsl:call-template>
							</xsl:when>
							<xsl:when test="project_wctask/previous_owner/group/@id">
								<!-- assignee is a group -->
								<xsl:call-template name="DisplayGroupName">
									<xsl:with-param name="groupID" select="project_wctask/previous_owner/group/@id"/>
									<xsl:with-param name="projectID" select="project_wctask/project_id"/>
									<xsl:with-param name="groupName" select="project_wctask/previous_owner/group/name"/>
									<xsl:with-param name="isRole" select="project_wctask/previous_owner/group/is_role"/>
									<xsl:with-param name="roleGroupNameList" select="key('projGroupMemberAssignmentDataKey', project_wctask/task_owner_id)"/>
									<xsl:with-param name="displayIcon" select="1"/>
								</xsl:call-template>
							</xsl:when>
						</xsl:choose>
						</p>
					</div>
				</div>
			</xsl:if>
			
			<!-- task name -->
			<div class="form-group">
				<xsl:if test="contains($hide, 'N')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<label for="taskName">
					<xsl:if test="$isChangeable">
						<xsl:attribute name="class">form-required</xsl:attribute>
					</xsl:if>
					<xsl:value-of select="Lang:getStr(&lk;,'NAME')"/>
				</label>
				<div class="form-entry">
					<xsl:choose>
						<xsl:when test="$isChangeable">
							<input type="text" id="taskName" name="taskName" value="{project_wctask/name}" onchange="taskNameChangedManually=true;"
									size="60" maxlength="{field_sizes/field_size [@key='projwctask.name']/@value}" class="form-control"/>
						</xsl:when>
						<xsl:otherwise>
							<p><xsl:value-of select="project_wctask/name"/></p>
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>
			
			<!-- description -->
			<div class="form-group">
				<xsl:if test="contains($hide, 'D')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<label for="taskDesc"><xsl:value-of select="Lang:getStr(&lk;,'DESCRIPTION')"/></label>
				<div class="form-entry">
					<xsl:choose>
						<xsl:when test="$isChangeable">
							<textarea id="taskDesc" name="taskDesc" rows="3" cols="60" class="form-control">
								<xsl:attribute name="onkeyup">return imposeMaxLength(this, <xsl:value-of select="field_sizes/field_size [@key='projwctask.description']/@value"/>);</xsl:attribute>
								<xsl:attribute name="onchange">return imposeMaxLength(this, <xsl:value-of select="field_sizes/field_size [@key='projwctask.description']/@value"/>);</xsl:attribute>
								<xsl:value-of select="project_wctask/description"/>
							</textarea>
						</xsl:when>
						<xsl:otherwise>
							<p><xsl:value-of select="project_wctask/description"/></p>
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>
			
			<!-- adding of predecessors -->
			<div class="form-group">
				<xsl:if test="contains($hide, 'P')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<label for="addPredecessors"><xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_START_AFTER')"/></label>
				<div class="form-entry">
							<xsl:if test="$isChangeable and not($isTaskStarted)">
								<div class="dropdown">
									<button class="btn btn-default dropdown-toggle" type="button" id="addPredecessors" data-toggle="dropdown" aria-haspopup="true" aria-expanded="true" 
										onMouseover="dropdownmenu(this, event, tasksMenu, taskMenuWidth+'px', null, null, null, null, null, null, 200)" 
										onMouseout="delayhidemenu()" 
										onclick="dropdownmenu(this, event, tasksMenu, taskMenuWidth+'px', null, null, null, null, null, null, 200)">
										<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_ADD_PREDECESSOR')"/>&nbsp;
										<span class="caret"></span>
									</button>
								</div>
							</xsl:if>
					
					<input type="hidden" id="taskPredecessorIDs" name="taskPredecessorIDs" value=""/>
					
					<!-- table of predecesors -->
						<div id="predecessorsTableTD">
							<!-- content of this table cell is entirely generated by Javascript (wctasks.js) -->
						</div>
					
					<!-- checkbox "Automatically link documents from predecessor tasks when task is started" -->
							<div class="checkbox">
								<label>
									<input id="linkPredecsDocsWhenStarted" name="linkPredecsDocsWhenStarted" type="checkbox" onclick="selectLinkPredecsDocs(this)" value="">
										<xsl:if test="project_wctask/link_predecessors_docs = '1'">
											<xsl:attribute name="checked">checked</xsl:attribute>
										</xsl:if>
										<xsl:if test="not($isChangeable)">
											<xsl:attribute name="disabled">disabled</xsl:attribute>
										</xsl:if>
									</input>
									<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_AUTO_LINK_DOCS_WHEN_STARTED')"/>
								</label>
							</div>
				</div>
			</div>
			
			<!-- start date -->
			<xsl:choose>
				<xsl:when test="/webcenter_page/started_date/long_date">
					<xsl:call-template name="DisplayTaskDate">
						<xsl:with-param name="langString" select="'PROJ_TASK_STARTED_DATE'"/>
						<xsl:with-param name="taskDateRO" select="/webcenter_page/started_date/long_date"/>
						<xsl:with-param name="isEditable" select="false()"/>
						<xsl:with-param name="className" select="'darktext_darklink_bold_12'"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise> 
					<xsl:if test="$isTaskComplete">
						<xsl:call-template name="DisplayTaskDate">
							<xsl:with-param name="langString" select="'PROJ_TASK_STARTED_DATE'"/>
							<xsl:with-param name="taskDateRO" select="/webcenter_page/actual_completed_date/long_date"/>
							<xsl:with-param name="isEditable" select="false()"/>
							<xsl:with-param name="className" select="'darktext_darklink_bold_12'"/>
						</xsl:call-template>
					</xsl:if>
				</xsl:otherwise>
			</xsl:choose>
			
			<!-- lead time (estimated duration) -->
			<div class="form-group">
				<xsl:if test="contains($hide, 'L')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<label for="lengthInput"><xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_LEAD_TIME')"/></label>
				<div class="form-entry">
					<xsl:call-template name="DaysHoursLengthInput">
						<xsl:with-param name="isEditable" select="$isChangeable"/>
						<xsl:with-param name="daysInputName" select="'lengthInput'"/>
						<xsl:with-param name="daysInputValue" select="project_wctask/approximate_length"/>
						<xsl:with-param name="hoursInputName" select="'lengthTime'"/>
						<xsl:with-param name="hoursInputValue" select="project_wctask/approximate_time"/>
					</xsl:call-template>
				</div>
			</div>
			
			<!-- due date -->
			<div id="wctaskDueDateField">
				<xsl:if test="contains($hide, 'U')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
						<xsl:call-template name="DisplayTaskDate">
							<xsl:with-param name="isEditable" select="$isChangeable"/>
							<xsl:with-param name="langString" select="'PROJ_TASK_DUE_DATE'"/>
							<xsl:with-param name="taskDate" select="/webcenter_page/due_date"/>
							<xsl:with-param name="taskDateRO" select="/webcenter_page/due_date/long_date"/>
							<xsl:with-param name="fieldName" select="'due_date'"/>
							<xsl:with-param name="PeriodOfDayFieldName" select="'due_periodOfDay'" />
							<xsl:with-param name="className" select="'darktext_darklink_bold_12'"/>
					<xsl:with-param name="onChangeMethod" select="'onDateChange'"/>
						</xsl:call-template>
									</div>
									
			<!-- estimated date -->
			<div id="wctaskEstimatedDateField">
				<xsl:if test="contains($hide, 'E')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<xsl:call-template name="DisplayTaskDate">
					<xsl:with-param name="isEditable" select="$isChangeable"/>
					<xsl:with-param name="langString" select="'PROJ_TASK_EXTIMATED_DATE'"/>
					<xsl:with-param name="taskDate" select="/webcenter_page/estimated_date"/>
					<xsl:with-param name="taskDateRO" select="/webcenter_page/estimated_date/long_date"/>
					<xsl:with-param name="fieldName" select="'estimated_date'"/>
					<xsl:with-param name="PeriodOfDayFieldName" select="'estimated_periodOfDay'" />
					<xsl:with-param name="className" select="'darktext_darklink_bold_12'"/>
				</xsl:call-template>
			</div>

			<!-- complete date -->
			<xsl:if test="$isTaskComplete">
				<xsl:call-template name="DisplayTaskDate">
					<xsl:with-param name="langString" select="'PROJ_TASK_COMPLETED_DATE'"/>
					<xsl:with-param name="taskDateRO" select="/webcenter_page/actual_completed_date/long_date"/>
					<xsl:with-param name="isEditable" select="false()"/>
					<xsl:with-param name="className" select="'darktext_darklink_bold_12'"/>
				</xsl:call-template>
			</xsl:if>
			
			
			<!-- reset start date button -->
			<div class="form-group hidden" id="resetStartDateFormGroup">
				<label class="offset-label"></label>
				<div class="form-entry" id="resetStartDateTD">
					<!-- content completely generated by Javascript (wctasks.js, method updateStartDateGUI()) -->
				</div>
			</div>
			
			<!-- start task types -->
			<xsl:if test="not($isTaskComplete)">
				<div class="form-group">
					<xsl:if test="contains($hide, 'R')">
						<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
					</xsl:if>
					<label><xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_START_TASK')"/></label>
					<div class="form-entry">
						<xsl:variable name="start_task_option">
							<xsl:choose>
								<xsl:when test="not($newTaskComposing)">
									<xsl:value-of select="project_wctask/start_task_option"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$start"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<div class="radio">
							<label>
								<input name="startTaskOption" type="radio" onclick="selectStartTaskOption(this)" value="0">
									<xsl:if test="$start_task_option='0'">
										<xsl:attribute name="checked">checked</xsl:attribute>
									</xsl:if>
									<xsl:if test="not($isChangeable)">
										<xsl:attribute name="disabled">disabled</xsl:attribute>
									</xsl:if>
								</input>
								<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_MANUALLY')"/>
							</label>
						</div>
						
						<div class="radio">
							<label>
								<input name="startTaskOption" type="radio" onclick="selectStartTaskOption(this)" value="1">
									<xsl:if test="$start_task_option='1'">
										<xsl:attribute name="checked">checked</xsl:attribute>
									</xsl:if>
									<xsl:if test="not($isChangeable)">
										<xsl:attribute name="disabled">disabled</xsl:attribute>
									</xsl:if>
								</input>
								<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_AUTO_AT_DATE')"/>&nbsp;
								<span id="taskStartDate1" style="font-weight: bold;">
									<!-- the content (date) is generated by Javasript -->
								</span>
								<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_AUTO_AT_DATE_ANNOTATION')"/>
							</label>
						</div>
						
						<div class="radio">
							<label>
								<input name="startTaskOption" type="radio" onclick="selectStartTaskOption(this)" value="2">
									<xsl:if test="$start_task_option='2'">
										<xsl:attribute name="checked">checked</xsl:attribute>
									</xsl:if>
									<xsl:if test="not($isChangeable)">
										<xsl:attribute name="disabled">disabled</xsl:attribute>
									</xsl:if>
								</input>
								<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_AUTO_AFTER_PREDECESSORS')"/>
							</label>
						</div>
						
						<div class="radio">
							<label>
								<input name="startTaskOption" type="radio" onclick="selectStartTaskOption(this)" value="3">
									<xsl:if test="$start_task_option='3'">
										<xsl:attribute name="checked">checked</xsl:attribute>
									</xsl:if>
									<xsl:if test="not($isChangeable)">
										<xsl:attribute name="disabled">disabled</xsl:attribute>
									</xsl:if>
								</input>
								<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_AUTO_AFTER_PRED_AND_DATE')"/>&nbsp;
								<span id="taskStartDate2" style="font-weight: bold;">
									<!-- the content (date) is generated by Javasript -->
								</span>
								<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_AUTO_AT_DATE_ANNOTATION')"/>
							</label>
						</div>
						
						<div class="radio">
							<label>
								<input name="startTaskOption" type="radio" onclick="selectStartTaskOption(this)" value="4">
									<xsl:if test="$start_task_option='4'">
										<xsl:attribute name="checked">checked</xsl:attribute>
									</xsl:if>
									<xsl:if test="not($isChangeable)">
										<xsl:attribute name="disabled">disabled</xsl:attribute>
									</xsl:if>
								</input>
								<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_AUTO_AFTER_PROJECTCREATION')"/>
							</label>
						</div>
					</div>
				</div>
			</xsl:if>
			
			<!-- checkbox "assignee can start task manually" -->
			<div class="form-group">
				<xsl:if test="contains($hide, 'R')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<label class="offset-label" for="canStartTaskManually"></label>
				<div class="form-entry">
					<div class="checkbox">
						<label>
							<input id="canStartTaskManually" name="canStartTaskManually" type="checkbox" onclick="selectAllowManualStart(this);" value="AllowManualStart">
								<xsl:if test="(project_wctask/allow_manual_start = '1') or $newTaskComposing">
									<xsl:attribute name="checked">checked</xsl:attribute>
								</xsl:if>
								<xsl:if test="not($isChangeable)">
									<xsl:attribute name="disabled">disabled</xsl:attribute>
								</xsl:if>
							</input>
							<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_CAN_START_TASK_MANUALLY')"/>
						</label>
					</div>
				</div>
			</div>
			
			
			<!-- checkbox "completion of this task changes project status to..." -->
			<div class="form-group">
				<xsl:if test="contains($hide, 'C')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<label class="offset-label"></label>
				<div class="form-entry">
					<div class="checkbox with-form-control">
						<label>
							<input id="projectStatusCheckbox" name="projectStatusCheckbox" type="checkbox" onclick="selectProjectStatusCheckbox(this)" value="">
								<xsl:if test="string-length(project_wctask/project_status_after_completion) &gt; 0">
									<xsl:attribute name="checked">checked</xsl:attribute>
								</xsl:if>
								<xsl:if test="not($isChangeable)">
									<xsl:attribute name="disabled">disabled</xsl:attribute>
								</xsl:if>
							</input>
							<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_COMPLETION_ACTION')"/>
						</label>
						
						<xsl:choose>
						<xsl:when test="$isChangeable">
							<select id="projectStatusSelect" name="projectStatusSelect" class="filter form-control">
								<xsl:call-template name="CreateSelectWithProjectStatusesAllowedToSet">
									<xsl:with-param name="selectedProjectStatusId" select="project/status_id" />
									<xsl:with-param name="displaySelectedIgnoreDeleted" select="true()"/>
								</xsl:call-template>									
							</select> 
						</xsl:when>
						<xsl:otherwise>
							<p>
							<span>
								<xsl:choose>
						 		<xsl:when test="string-length(project_wctask/project_status_after_completion) &gt; 0">
									<xsl:variable name="langTag" select="project_status_list/custom_project_status[@id=/webcenter_page/project_wctask/project_status_after_completion]/translation_tag"/>
									<xsl:variable name="projStatusName" select="project_status_list/custom_project_status[@id=/webcenter_page/project_wctask/project_status_after_completion]/status_name"/>
									<!-- "<xsl:value-of select="Lang:getStr(&lk;, $langTag)"/>"-->
									"<xsl:call-template name="GetLocalizedLabelOrAlternative">
										<xsl:with-param name="langKey" select="$langTag"/>
										<xsl:with-param name="alternativeLabel" select="$projStatusName"/>
									</xsl:call-template>"

								</xsl:when>
								<xsl:otherwise>
									&lt;<xsl:value-of select="Lang:getStr(&lk;,'NOTHING_TAG')"/>&gt;
								</xsl:otherwise>
								</xsl:choose>
							</span>
							</p>
						</xsl:otherwise>
						</xsl:choose>
					</div>
					
				</div>
			</div>
			
		</div>
		
		
		<!-- lower main action buttons ("Change" and "Cancel") -->
		<xsl:if test="$isChangeable or $isStatusEditable or $canChangeAssignee">
			<div class="form-section">
				<xsl:if test="contains($hide, 'B2')">
					<xsl:attribute name="style"><xsl:value-of select="'display:none'"/></xsl:attribute>
				</xsl:if>
				<div class="form-group">
					<label class="offset-label"></label>
					<div class="form-entry">
						<xsl:call-template name="ProjectWcTaskGenInfoMainButtons">
							<xsl:with-param name="isChangeable" select="$isChangeable"/>
							<xsl:with-param name="newTaskComposing" select="$newTaskComposing"/>
							<xsl:with-param name="isTaskComplete" select="$isTaskComplete"/>
						</xsl:call-template>
					</div>
				</div>
			</div>
		</xsl:if>

		<xsl:variable name="executeOnLoad">
			<xsl:call-template name="allHidden">
				<xsl:with-param name="hide" select="$hide"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:if test="$executeOnLoad = '1'">
			<div class="form-section">
				<div class="form-group">
					<label class="offset-label"></label>
					<div class="form-entry">
						<xsl:value-of select="Lang:getStr(&lk;,'CREATING_TASK')"/>
					</div>
				</div>
			</div>
		</xsl:if>
		

		<!-- script generating and handling task predecessors table and related functionality -->
	  <script type="text/javascript">
	    <xsl:text disable-output-escaping="yes">&lt;!--</xsl:text>&#xA;		

		    initWCTasksJS('predecessorsTableTD', tasks, taskTypes, <xsl:value-of select="$isChangeable and not($isTaskPageReadonly)"/>, <xsl:value-of select="$isStatusEditable"/>);
		    
		    generateTaskMenu();
		    generatePredecessorsTable();
	
	    <xsl:text disable-output-escaping="yes">//--&gt;</xsl:text>&#xA;
	  </script>
	</xsl:template>


	<!-- 
  
            template that controls table header
            
  -->

	<xsl:template name="DetailsChecklistTableHeader">
		<xsl:param name="readOnly" select="false()"/>

		<thead>
			<tr>
				<th class="td-checkbox">
					<xsl:choose>
						<xsl:when test="$readOnly = 'true'">#</xsl:when>
						<xsl:otherwise>
							<input type="checkbox" name="selectall"  value="">
								<xsl:attribute name="onclick">SubmitAll();</xsl:attribute>
							</input>
						</xsl:otherwise>
					</xsl:choose>
				</th>
				<th>
					<xsl:value-of select="Lang:getStr(&lk;, 'NAME')"/>
				</th>
				<th>
					<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_COMPLETED_BY')"/>
				</th>
				<th class="td-date">
					<xsl:value-of select="Lang:getStr(&lk;, 'PROJ_TASK_COMPLETED_ON')"/>
				</th>
			</tr>
		</thead>
	</xsl:template>

	<!--
	
	Template for showing checklist item row
	
-->
	<xsl:template name="ChecklistItemRow">
		<xsl:param name="readOnly" select="false()"/>
		<tr>
			<td class="td-checkbox">
				<xsl:choose>
					<xsl:when test="$readOnly = 'true'">
						<xsl:value-of select="position()"/>
					</xsl:when>
					<xsl:otherwise>
						<input type="checkbox" name="selectItem" value="{item_id}">
							<xsl:attribute name="onchange">PerformAction(this);</xsl:attribute>											
							<xsl:if test="is_checked = 1">
								<xsl:attribute name="checked">checked</xsl:attribute>

								<!-- allow to project managers to undo checked items -->
<!-- JANH not supported in WCR 10.0 (it's not in line with Task Execution page logic!
								<xsl:variable name="allowUndo" select="//is_project_editable &gt; 0"/>
								<xsl:if test="not($allowUndo)">
									<xsl:attribute name="disabled">disabled</xsl:attribute>
								</xsl:if>
-->								
							</xsl:if>
						</input>
					</xsl:otherwise>
				</xsl:choose>
			</td>

			<td>
				<xsl:value-of select="item_name"/>
			</td>

			<td>
				<xsl:call-template name="DisplayUserName">
					<xsl:with-param name="userID" select="user_operator_id"/>
					<xsl:with-param name="lastName" select="user/last_name"/>
					<xsl:with-param name="firstName" select="user/first_name"/>
					<xsl:with-param name="loginName" select="user/name"/>
					<xsl:with-param name="isActive" select="user/is_active" />
				</xsl:call-template>
			</td>

			<td class="td-date">
				<xsl:call-template name="HumanReadableLongDateAndTime">
					<xsl:with-param name="LongDateString" select="check_date"/>
				</xsl:call-template>
			</td>
		</tr>
	</xsl:template>


	<!-- 
	
    template to display task details from mytodo list and delete dependency list
	without showing any actions
	
  -->   
	<xsl:template name="ProjectWcTaskDetailsNoActionRow">
		<xsl:param name="posOffset">0</xsl:param>
		<xsl:param name="showPrevOwner" select="false()"/>
		<xsl:param name="showCurOwner" select="false()"/>
		<xsl:param name="showRadioButton" select="false()"/>
		<xsl:param name="icon" select="''"/>
		<xsl:param name="showSimpleGUI" select="'0'"/>
		<xsl:param name="sourcePage" select="'projdetailswctasks.jsp'"/>		
		
		<!-- determines whether the 'task owner group type' is a regular group, or a role in fact -->
		<xsl:variable name="isOwnerRole">
			<xsl:choose>
				<xsl:when test="(task_owner_type='group') and (group/is_role = '1')"><xsl:value-of select="true()"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="false()"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="isPrevOwnerRole">
			<xsl:choose>
				<xsl:when test="(previous_task_owner_type='group') and (previous_owner/group/is_role = '1')"><xsl:value-of select="true()"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="false()"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<tr>
			<xsl:if test="position() mod 2 = 0">
				<xsl:attribute name="class"><xsl:value-of select="'alt'"/></xsl:attribute>
			</xsl:if>

			<td class="darktext_darklink_11" align="center">
				<xsl:choose>
					<xsl:when test="$showRadioButton = '1'">
						<input type="radio" name="selectTemplate" onclick="SelectRadio(this)" value="{@id}"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="position() + number($posOffset)"/>
					</xsl:otherwise>
				</xsl:choose>
			</td>

			<!-- task name (with a link to the TEP page)-->
			<xsl:call-template name="ProjectWcTaskDetailsName">
				<xsl:with-param name="taskName" select="name"/>
				<xsl:with-param name="projectTaskID" select="@id"/>
				<xsl:with-param name="projectID" select="project_id"/>
				<xsl:with-param name="page" select="'wctaskexecutionpage.jsp'"/>
				<xsl:with-param name="icon" select="$icon"/>	
				<xsl:with-param name="sourcePage" select="$sourcePage"/>						
			</xsl:call-template>

			<!-- current task owner -->
			<xsl:if test="$showCurOwner">
				<td class="darktext_darklink_11" align="left">
					<xsl:choose>
						<xsl:when test="task_owner_type = 'user'">
							<!-- User-->
							<xsl:call-template name="DisplayUserName">
								<xsl:with-param name="userID" select="task_owner_id"/>
								<xsl:with-param name="lastName" select="user/last_name"/>
								<xsl:with-param name="firstName" select="user/first_name"/>
								<xsl:with-param name="loginName" select="user/name"/>
								<xsl:with-param name="isActive" select="user/is_active" />
								<xsl:with-param name="displayIcon" select="1"/>
							</xsl:call-template>
						</xsl:when>
						<!-- Group -->
						<xsl:when test="(task_owner_type='group') and ($isOwnerRole = 'false')">
							<xsl:call-template name="DisplayGroupName">
								<xsl:with-param name="groupID" select="task_owner_id"/>
								<xsl:with-param name="groupName" select="group/name"/>
								<xsl:with-param name="displayIcon" select="1"/>
								<xsl:with-param name="isRole" select="group/is_role"/>
							</xsl:call-template>
						</xsl:when>
						<!--  Role type Task Owner/Assignee -->
						<xsl:when test="$isOwnerRole">
							<xsl:choose>
								<!-- If the Role is actually assigned -->
								<xsl:when test="role_assignment_data">
									<xsl:call-template name="DisplayRoleNameWithAssignmentsInTable">
										<xsl:with-param name="roleAssignmentData" select="role_assignment_data"/>
										<xsl:with-param name="projectID" select="project_id"/>
									</xsl:call-template>
								</xsl:when>
								<!-- The Role has no assignments : just write the Role Name/Link -->
								<xsl:otherwise>
									<xsl:call-template name="DisplayGroupName">
										<xsl:with-param name="groupID" select="task_owner_id"/>
										<xsl:with-param name="groupName" select="group/name"/>
										<xsl:with-param name="displayIcon" select="1"/>
										<xsl:with-param name="isRole" select="group/is_role"/>
									</xsl:call-template>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
					</xsl:choose>
				</td>
			</xsl:if>

			<!-- previous task owner -->
			<xsl:if test="$showPrevOwner">
				<td class="darktext_darklink_11" align="left">
					<xsl:choose>
						<xsl:when test="previous_task_owner_type = 'user'">
							<!--  previous task owner is User-->
							<xsl:call-template name="DisplayUserName">
								<xsl:with-param name="userID" select="previous_task_owner_id"/>
								<xsl:with-param name="lastName" select="previous_owner/user/last_name"/>
								<xsl:with-param name="firstName" select="previous_owner/user/first_name"/>
								<xsl:with-param name="loginName" select="previous_owner/user/name"/>
								<xsl:with-param name="isActive" select="previous_owner/user/is_active" />
								<xsl:with-param name="displayIcon" select="1"/>
							</xsl:call-template>
						</xsl:when>
						<xsl:when test="(previous_task_owner_type='group') and ($isPrevOwnerRole = 'false')">
							<!--  previous task owner is Group-->
							<xsl:call-template name="DisplayGroupName">
								<xsl:with-param name="groupID" select="previous_task_owner_id"/>
								<xsl:with-param name="groupName" select="previous_owner/group/name"/>
								<xsl:with-param name="displayIcon" select="1"/>
								<xsl:with-param name="isRole" select="previous_owner/group/is_role"/>
							</xsl:call-template>
						</xsl:when>
						<!--  Role type Task Owner/Assignee -->
						<xsl:when test="$isPrevOwnerRole">
							<xsl:choose>
								<!-- If the Role is actually assigned -->
								<xsl:when test="previous_owner/role_assignment_data">
									<xsl:call-template name="DisplayRoleNameWithAssignments">
										<xsl:with-param name="roleAssignmentData" select="previous_owner/role_assignment_data"/>
										<xsl:with-param name="projectID" select="project_id"/>
									</xsl:call-template>
								</xsl:when>
								<!-- The Role has no assignments : just write the Role Name/Link -->
								<xsl:otherwise>
									<xsl:call-template name="DisplayGroupName">
										<xsl:with-param name="groupID" select="previous_task_owner_id"/>
										<xsl:with-param name="groupName" select="previous_owner/group/name"/>
										<xsl:with-param name="displayIcon" select="1"/>
										<xsl:with-param name="isRole" select="previous_owner/group/is_role"/>
									</xsl:call-template>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
					</xsl:choose>
				</td>
			</xsl:if>

			<!-- task status -->
			<xsl:call-template name="DisplayProjectWcTaskStatus">
				<xsl:with-param name="projectWcTask" select="."/>
			</xsl:call-template>

			<!-- due date -->
			<td align="left">
				<xsl:attribute name="class">
					<xsl:choose>	
						<xsl:when test="status/@id = '00001_0000000003'">
							<!-- task is COMPLETED ... just display the due date without emphasizing -->
							td-date
						</xsl:when>
						<xsl:otherwise>
							<!-- process class attribute based on the value of task due date -->
							<xsl:call-template name="GetDateStyle">
								<xsl:with-param name="inputDate" select="due_date"/>
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>

				<xsl:call-template name="HumanReadableLongDateAndTime">
					<xsl:with-param name="LongDateString" select="due_date"/>
				</xsl:call-template>
			</td>

			<!-- start date -->
			<td class="td-date" align="left">
				<xsl:call-template name="HumanReadableLongDateAndTime">
					<xsl:with-param name="LongDateString" select="started_date"/>
				</xsl:call-template>
			</td>

		</tr>
	</xsl:template>
	
	<!-- 
  
            template that controls task table header
			without any actions
            
  -->
	<xsl:template name="TaskTableNoActionHeader">
		<xsl:param name="showPrevOwner" select="false()"/>
		<xsl:param name="showCurOwner" select="false()"/>
		<xsl:param name="showRadioButton" select="false()"/>
		<xsl:param name="showNotStartedTasks" select="false()"/>

		<tr>
			<th width="20" align="center">
				<xsl:choose>
					<xsl:when test="$showRadioButton = '1'">
						<img src="images/clearpixel.gif" width="5" height="" border="0"/>
					</xsl:when>
					<xsl:otherwise>#</xsl:otherwise>
				</xsl:choose>
			</th>


			<th align="left">
				<!-- in case we are already ordering by doc name, reverse the order -->
				<xsl:choose>
					<xsl:when test="/webcenter_page/taskorder = 1">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(2)">
							<img src="images/ascending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(2);">
							<xsl:value-of select="Lang:getStr(&lk;,'TASK')"/>
						</a>
					</xsl:when>
					<xsl:when test="/webcenter_page/taskorder = 2">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(1)">
							<img src="images/descending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(1)">
							<xsl:value-of select="Lang:getStr(&lk;,'TASK')"/>
						</a>
					</xsl:when>
					<xsl:otherwise>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(1)">
							<xsl:value-of select="Lang:getStr(&lk;,'TASK')"/>
						</a>
					</xsl:otherwise>
				</xsl:choose>
			</th>

			<!--  Current Proj Task Owner  -->
			<xsl:if test="($showCurOwner)">
			  <th align="left" nowrap="nowrap">
				<!-- in case we are already ordering by doc name, reverse the order -->
				<xsl:choose>
					<xsl:when test="/webcenter_page/taskorder = 9">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(10)">
							<img src="images/ascending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(10);">
							<xsl:value-of select="Lang:getStr(&lk;,'SORT_PROJ_TASK_BY_OWNER')"/>
						</a>
					</xsl:when>
					<xsl:when test="/webcenter_page/taskorder = 10">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(9)">
							<img src="images/descending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(9)">
							<xsl:value-of select="Lang:getStr(&lk;,'SORT_PROJ_TASK_BY_OWNER')"/>
						</a>
					</xsl:when>
					<xsl:otherwise>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(9)">
							<xsl:value-of select="Lang:getStr(&lk;,'SORT_PROJ_TASK_BY_OWNER')"/>
						</a>
					</xsl:otherwise>
				</xsl:choose>
			  </th>
			</xsl:if>

			<!--  Previous Proj Task Owner  -->
			<xsl:if test="$showPrevOwner">
				<th align="left">
					<!-- in case we are already ordering by doc name, reverse the order -->
					<xsl:choose>
						<xsl:when test="/webcenter_page/taskorder = 13">
							<a class="small_text_bold" href="javascript:SortOnTaskItem(14)">
								<img src="images/ascending.png" border="0"/>
							</a>
							<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
							<a class="small_text_bold" href="javascript:SortOnTaskItem(14);">
								<xsl:value-of select="Lang:getStr(&lk;,'SEARCH_PREV_PROJ_TASK_OWNER')"/>
							</a>
						</xsl:when>
						<xsl:when test="/webcenter_page/taskorder = 14">
							<a class="small_text_bold" href="javascript:SortOnTaskItem(13)">
								<img src="images/descending.png" border="0"/>
							</a>
							<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
							<a class="small_text_bold" href="javascript:SortOnTaskItem(13)">
								<xsl:value-of select="Lang:getStr(&lk;,'SEARCH_PREV_PROJ_TASK_OWNER')"/>
							</a>
						</xsl:when>
						<xsl:otherwise>
							<a class="small_text_bold" href="javascript:SortOnTaskItem(13)">
								<xsl:value-of select="Lang:getStr(&lk;,'SEARCH_PREV_PROJ_TASK_OWNER')"/>
							</a>
						</xsl:otherwise>
					</xsl:choose>
				</th>
			</xsl:if>
			
			
			<!--  Status  -->
			  <th align="center">
				<!-- in case we are already ordering by doc name, reverse the order -->
				<xsl:choose>
					<xsl:when test="/webcenter_page/taskorder = 7">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(8)">
							<img src="images/ascending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(8);">
							<xsl:value-of select="Lang:getStr(&lk;,'SORT_PROJ_TASK_BY_STATUS')"/>
						</a>
					</xsl:when>
					<xsl:when test="/webcenter_page/taskorder = 8">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(7)">
							<img src="images/descending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(7)">
							<xsl:value-of select="Lang:getStr(&lk;,'SORT_PROJ_TASK_BY_STATUS')"/>
						</a>
					</xsl:when>
					<xsl:otherwise>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(7)">
							<xsl:value-of select="Lang:getStr(&lk;,'SORT_PROJ_TASK_BY_STATUS')"/>
						</a>
					</xsl:otherwise>
				</xsl:choose>
			  </th>

			<!-- due date -->
			  <th class="td-date" nowrap="nowrap">
				<xsl:choose>
					<xsl:when test="/webcenter_page/taskorder = 5">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(6)">
							<img src="images/ascending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(6);">
							<xsl:value-of select="Lang:getStr(&lk;,'TODO_DUEDATE')"/>
						</a>
					</xsl:when>
					<xsl:when test="/webcenter_page/taskorder = 6">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(5)">
							<img src="images/descending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(5)">
							<xsl:value-of select="Lang:getStr(&lk;,'TODO_DUEDATE')"/>
						</a>
					</xsl:when>
					<xsl:otherwise>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(5)">
							<xsl:value-of select="Lang:getStr(&lk;,'TODO_DUEDATE')"/>
						</a>
					</xsl:otherwise>
				</xsl:choose>
			  </th>

			<!-- started date -->
			  <th class="td-date" nowrap="nowrap">
				<xsl:choose>
					<xsl:when test="/webcenter_page/taskorder = 11">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(12)">
							<img src="images/ascending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(12);">
							<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_STARTED_DATE')"/>
						</a>
					</xsl:when>
					<xsl:when test="/webcenter_page/taskorder = 12">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(11)">
							<img src="images/descending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(11)">
							<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_STARTED_DATE')"/>
						</a>
					</xsl:when>
					<xsl:otherwise>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(11)">
							<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_STARTED_DATE')"/>
						</a>
					</xsl:otherwise>
				</xsl:choose>
			  </th>
			  
		      <th align="left">
				<xsl:choose>
					<xsl:when test="/webcenter_page/taskorder = 3">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(4)">
							<img src="images/ascending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(4);">
							<xsl:value-of select="Lang:getStr(&lk;,'TODO_PROJECT')"/>
						</a>
					</xsl:when>
					<xsl:when test="/webcenter_page/taskorder = 4">
						<a class="small_text_bold" href="javascript:SortOnTaskItem(3)">
							<img src="images/descending.png" border="0"/>
						</a>
						<img src="images/clearpixel.gif" width="3" height="0" border="0"/>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(3)">
							<xsl:value-of select="Lang:getStr(&lk;,'TODO_PROJECT')"/>
						</a>
					</xsl:when>
					<xsl:otherwise>
						<a class="small_text_bold" href="javascript:SortOnTaskItem(3)">
							<xsl:value-of select="Lang:getStr(&lk;,'TODO_PROJECT')"/>
						</a>
					</xsl:otherwise>
				</xsl:choose>
			  </th>
		</tr>
	</xsl:template>


	<!-- 

    template to display task name as a link to task details page

  -->
	<xsl:template name="ProjectWcTaskDetailsName">
		<xsl:param name="taskName"/>
		<xsl:param name="projectTaskID"/>
		<xsl:param name="projectID"/>
		<xsl:param name="page" select="'projdetailswctaskdetailsgeninfo.jsp'"/>
		<xsl:param name="icon" select="''"/>
		<xsl:param name="sourcePage" select="'projdetailswctasks.jsp'"/>

		<td>
		
		    <xsl:if test="not($icon = '')">
			  <a>	
				<xsl:attribute name="href">
				  <xsl:value-of select="$page"/>?projectID=<xsl:value-of select="$projectID"/>&amp;curProjTaskID=<xsl:value-of select="$projectTaskID"/>&amp;<xsl:call-template name="menuFileUrlParameter"/>&amp;sourcePage=<xsl:value-of select="$sourcePage"/>
				</xsl:attribute>
				<img align="absmiddle" class="add-spacing-right">
					<xsl:attribute name="src"><xsl:value-of select="$icon"/></xsl:attribute>
				</img>
			  </a>
			</xsl:if>	
		
			<a>
				<xsl:call-template name="taskDetailsHref">
					<xsl:with-param name="page" select="$page"/>
					<xsl:with-param name="projectID" select="$projectID"/>
					<xsl:with-param name="wctaskID" select="$projectTaskID"/>
					<xsl:with-param name="sourcePage" select="$sourcePage"/>
				</xsl:call-template>
				<xsl:value-of select="$taskName"/>
			</a>

		</td>
	</xsl:template>

	<!--
	
	  This template is called from my to-do list
	  and shows task details in general
	  
	  Note: only used by (dead code) projwctaskpickcopyfromtasknew.xsl
  -->
	<xsl:template name="DisplayToDoListDetails">
		<xsl:param name="taskTag"/>
		<xsl:param name="sort"/>
		<xsl:param name="posOffset">0</xsl:param>
		<xsl:param name="showPrevOwner" select="false()"/>
		<xsl:param name="showCurOwner" select="false()"/>
		<xsl:param name="showRadioButton" select="false()"/>
		<xsl:param name="showSimpleGUI" select="'0'"/>
		<xsl:param name="sourcePage" select="'projdetailswctasks.jsp'"/>

		<tr>
			<td>
				<table border="0" cellpadding="0" cellspacing="0" align="left" width="100%">
					<tr>
						<xsl:call-template name="AddWhiteWidth"/>
						<td>
							<table class="ordtable" width="100%">
								<!-- Write the header -->
								<thead> 
									<xsl:call-template name="TaskTableNoActionHeader">
										<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
										<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
										<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
									</xsl:call-template>
								</thead>
								<!-- Use different sorting there -->
								<xsl:choose>
									<!-- sorting by task name - ascending -->
									<xsl:when test="number($sort)=number(1) ">
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="name" data-type="text" order="ascending"/>
											<xsl:sort select="project_name"/>
											<xsl:sort select="task_owner_type"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by task name - descending -->
									<xsl:when test="number($sort)=number(2) ">
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="name" data-type="text" order="descending"/>
											<xsl:sort select="project_name"/>
											<xsl:sort select="task_owner_type"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by project name - ascending -->
									<xsl:when test="number($sort)=number(3) ">
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="project_name" data-type="text" order="ascending"/>
											<xsl:sort select="name" data-type="text" order="ascending"/>
											<xsl:sort select="task_owner_type"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by project - descending -->
									<xsl:when test="number($sort)=number(4) ">
										<!-- sort == abs(1) : sort on name -->
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="project_name" data-type="text" order="descending"/>
											<xsl:sort select="name" data-type="text" order="ascending"/>
											<xsl:sort select="task_owner_type"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by due date  - ascending -->
									<xsl:when test="number($sort)=number(5) ">
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="due_date" data-type="number" order="ascending"/>
											<xsl:sort select="project_name" data-type="text" order="ascending"/>
											<xsl:sort select="name" data-type="text" order="ascending"/>
											<xsl:sort select="task_owner_type"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by due date - descending -->
									<xsl:when test="number($sort)=number(6) ">
										<!-- sort == abs(1) : sort on name -->
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="due_date" data-type="number" order="descending"/>
											<xsl:sort select="project_name" data-type="text" order="ascending"/>
											<xsl:sort select="name" data-type="text" order="ascending"/>
											<xsl:sort select="task_owner_type"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by status - ascending -->
									<xsl:when test="number($sort)=number(7) ">
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="status/status_name" data-type="text" order="ascending"/>
											<xsl:sort select="due_date" data-type="number" order="descending"/>
											<xsl:sort select="project_name" data-type="text" order="ascending"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by status- descending -->
									<xsl:when test="number($sort)=number(8) ">
										<!-- sort == abs(1) : sort on name -->
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="status/status_name" data-type="text" order="descending"/>
											<xsl:sort select="due_date" data-type="number" order="descending"/>
											<xsl:sort select="project_name" data-type="text" order="ascending"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by owner - group owner first - ascending-->
									<xsl:when test="number($sort)=number(9) ">
										<!-- sort == abs(1) : sort on name -->
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="task_owner_type" data-type="text" order="ascending"/>
											<xsl:sort select="group/is_role" data-type="number" order="descending"/>
											<xsl:sort select="group/name" data-type="text" order="ascending"/>
											<xsl:sort select="user/name" data-type="text" order="ascending"/>
											<xsl:sort select="due_date" data-type="number" order="descending"/>
											<xsl:sort select="project_name" data-type="text" order="ascending"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by owner - group owner first  - ascending -->
									<xsl:when test="number($sort)=number(10) ">
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="task_owner_type" data-type="text" order="descending"/>
											<xsl:sort select="group/is_role" data-type="number" order="descending"/>
											<xsl:sort select="group/name" data-type="text" order="descending"/>
											<xsl:sort select="user/name" data-type="text" order="descending"/>
											<xsl:sort select="due_date" data-type="number" order="descending"/>
											<xsl:sort select="project_name" data-type="text" order="ascending"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by started date  - ascending -->
									<xsl:when test="number($sort)=number(11) ">
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="started_date" data-type="number" order="ascending"/>
											
											<xsl:sort select="due_date" data-type="number" order="descending"/>
											<xsl:sort select="project_name" data-type="text" order="ascending"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by started date - descending -->
									<xsl:when test="number($sort)=number(12) ">
										<!-- sort == abs(1) : sort on name -->
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="started_date" data-type="number" order="descending"/>
											<xsl:sort select="due_date" data-type="number" order="descending"/>
											<xsl:sort select="project_name" data-type="text" order="ascending"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by previous owner - group owner first - ascending-->
									<xsl:when test="number($sort)=number(13) ">
										<!-- for the previous owner, we first show the non empty lines  -->
										<xsl:for-each select="$taskTag/project_wctask">
											<!-- make sure to always list the assigned previous owners first -->
											<xsl:sort select="has_previous_task_owner" data-type="text" order="descending"/>
											<xsl:sort select="previous_task_owner_type" data-type="text" order="descending"/>
											<xsl:sort select="previous_owner/user/last_name" data-type="text" order="ascending"/>
											<xsl:sort select="previous_owner/user/first_name" data-type="text" order="ascending"/>
											<xsl:sort select="previous_owner/group/is_role" data-type="number" order="descending"/>
											<xsl:sort select="previous_owner/group/name" data-type="text" order="ascending"/>
											<xsl:sort select="due_date" data-type="number" order="descending"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>

									<!-- sorting by previous owner - group owner first  - ascending -->
									<xsl:when test="number($sort)=number(14) ">
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="has_previous_task_owner" data-type="text" order="descending"/>
											<xsl:sort select="previous_task_owner_type" data-type="text" order="descending"/>
											<xsl:sort select="previous_owner/user/last_name" data-type="text" order="descending"/>
											<xsl:sort select="previous_owner/user/first_name" data-type="text" order="descending"/>
											<xsl:sort select="previous_owner/group/is_role" data-type="number" order="descending"/>
											<xsl:sort select="previous_owner/group/name" data-type="text" order="descending"/>
											<xsl:sort select="due_date" data-type="number" order="descending"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:when>
									<!-- default behavior - sort on due date   -->
									<xsl:otherwise>
										<xsl:for-each select="$taskTag/project_wctask">
											<xsl:sort select="due_date" data-type="number" order="ascending"/>
											<xsl:sort select="task_owner_type" data-type="text" order="descending"/>
											<xsl:sort select="group/is_role" data-type="number" order="descending"/>
											<xsl:sort select="group/name" data-type="text" order="descending"/>
											<xsl:sort select="user/name" data-type="text" order="descending"/>
											<xsl:value-of select="Test"/>
											<xsl:call-template name="ProjectWcTaskDetailsNoActionRow">
												<xsl:with-param name="posOffset" select="$posOffset"/>
												<xsl:with-param name="showPrevOwner" select="$showPrevOwner"/>
												<xsl:with-param name="showCurOwner" select="$showCurOwner"/>
												<xsl:with-param name="showRadioButton" select="$showRadioButton"/>
												<xsl:with-param name="showSimpleGUI" select="$showSimpleGUI"/>
												<xsl:with-param name="sourcePage" select="$sourcePage"/>
											</xsl:call-template>
										</xsl:for-each>
									</xsl:otherwise>
								</xsl:choose>
							</table>
						</td>
						<xsl:call-template name="AddWhiteWidth"/>
					</tr>
				</table>
			</td>
		</tr>
	</xsl:template>


	<!-- 
  
  	Template of "days: ___  hours: ___" input

  -->
	<xsl:template name="DaysHoursLengthInput">
		<xsl:param name="isEditable" select="true()"/>
		<xsl:param name="daysInputName" select="'lengthInput'"/>
		<xsl:param name="daysInputValue" select="''"/>
		<xsl:param name="hoursInputName" select="'lengthTime'"/>
		<xsl:param name="hoursInputValue" select="''"/>
		
		<xsl:choose>
			<xsl:when test="$isEditable = true()">
				<div class="row">
					<div class="col-xs-6">
						<div class="input-group">
							<input id="{$daysInputName}" name="{$daysInputName}" type="text" size="10" maxlength="4" 
								value="{$daysInputValue}" onchange="if (typeof daysHoursLengthChanged == 'function') daysHoursLengthChanged({$daysInputName}, {$hoursInputName});" class="form-control"/>
							<label class="input-group-addon" for="{$daysInputName}"><xsl:value-of select="Lang:getStr(&lk;,'DAYS')"/></label>
						</div>
					</div>
					
					<div class="col-xs-6">
						<div class="input-group">
							<input id="{$hoursInputName}" name="{$hoursInputName}" type="text" size="10" maxlength="4" 
								value="{$hoursInputValue}" onchange="if (typeof daysHoursLengthChanged == 'function') daysHoursLengthChanged({$daysInputName}, {$hoursInputName});" class="form-control"/>
							<label class="input-group-addon" for="{$hoursInputName}"><xsl:value-of select="Lang:getStr(&lk;, 'HOURS')"/></label>
						</div>
					</div>
				</div>
			</xsl:when>
			<xsl:otherwise>
				<p>
					<span id="{$daysInputName}">
						<xsl:value-of select="$daysInputValue"/>
					</span>&nbsp;
					<xsl:value-of select="Lang:getStr(&lk;,'DAYS')"/>&nbsp;&nbsp;&nbsp;
				
					<span id="{$hoursInputName}">
						<xsl:value-of select="$hoursInputValue"/>
					</span>&nbsp;
					<xsl:value-of select="Lang:getStr(&lk;, 'HOURS')"/>
				</p>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- 
  
        template for due date options used on project task screen and group task manager screen

	-->
	<xsl:template name="taskDueDateOptions">
		<div class="wcr-filter">
		<label><xsl:value-of select="Lang:getStr(&lk;,'SORT_PROJ_TASK_BY_DUEDATE')"/></label>		
        <select class="form-control filter" name="modDays" >
		  <xsl:attribute name="onchange">document.projtaskform.filterDueDate.value = this.value;</xsl:attribute>            

          <xsl:variable name="modDueDateValue" select="project_wctask_filter/due_date"/>
          <xsl:variable name="modDueDateType" select="project_wctask_filter/due_date/@typeDD"/>

		  <xsl:for-each select="groupTasksMgmnt_mod_duedate_options/mod_duedate_option">
		  	<xsl:call-template name="duedateFilterOption" >
				<xsl:with-param name="modDueDateValue" select="$modDueDateValue" />
				<xsl:with-param name="modDueDateType" select="$modDueDateType" />					
				<xsl:with-param name="typeDD" select="@typeDD" />
				<xsl:with-param name="label" select="Lang:getStr(&lk;,'DAY')" />
				<xsl:with-param name="labelPl" select="Lang:getStr(&lk;,'DAYS')" />
		  	</xsl:call-template>
		  </xsl:for-each>
        </select>
        </div>
	</xsl:template>	

  	<!-- 
  	  	Print out options for duedate filter. Handles various options, which are days, weeks, months, to future(next, when negative sign is present), to past(overdue) and special cases (like "Overdue" and "All")
	 -->
    <xsl:template name="duedateFilterOption">
			    <xsl:param name="modDueDateValue" />
			    <xsl:param name="modDueDateType" />
				<xsl:param name="typeDD" />
                <!-- Test for minus value==keyword next - also used for computing new duedate - positive values goes to past, negative to future -->
				<xsl:param name="labelDay" />
				<xsl:param name="labelDays" />
				<xsl:param name="labelWeek" />
				<xsl:param name="labelWeeks" />
				<xsl:param name="labelMonth" />
				<xsl:param name="labelMonths" />
				
                <xsl:variable name="isNegative" select="starts-with(., '-')"/>
				<xsl:variable name="default" select="@selected"/>
				
				<!-- removes sign - the variable "numberOfDays" is used for display purposes only, so sign needs to be removed. In "value" attribute needs to stay, but that's handled elsewhere with "." -->
<!-- 		number of days represents value. So in the case of days it is days, in the case of months it is number of months... in the case of special type its 'all' or 'overdue' -->
				<xsl:variable name="numberOfDays">
					<xsl:choose>				
					<xsl:when test="$isNegative">
	                    <xsl:value-of select="substring(.,2)"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="."/>
					</xsl:otherwise>						
					</xsl:choose>
				</xsl:variable>			

				
				<!-- evaluate label for duedate filter option -->
				<xsl:variable name="duedateFilterOptionLabel">
					<xsl:call-template name="determineDuedateFilterOptionLabel">
						<xsl:with-param name="typeDD" select="$typeDD" />
						<xsl:with-param name="numberOfDays" select="$numberOfDays" />
						<xsl:with-param name="isNegative" select="$isNegative" />								
					</xsl:call-template>
				</xsl:variable>

                <option>
	                <!-- create value by concating type(dayes, weeks...) and value(how many) -->
				  	<xsl:attribute name="value"><xsl:value-of select="concat($typeDD,'_', .)"/></xsl:attribute>
	
	                <!-- determine, which one is selected -->
		            <xsl:if test="(($modDueDateType = $typeDD) and ($modDueDateValue=.))">
						<xsl:attribute name="selected"/>
					</xsl:if>
					<xsl:value-of select="$duedateFilterOptionLabel" />
                </option>               
			
  </xsl:template>	

<!-- 
	Create label for option in duedate filter. Many parameters for decision - future(with negative sign)/overdue, single/plural, special cases(Overdue(from this moment), All) 
	
	First try(the commented out) was to go through all possibilities - no it's build up
-->
  	<xsl:template name="determineDuedateFilterOptionLabel">
		<xsl:param name="typeDD" />
<!-- 		number of days represents value. So in the case of days it is days, in the case of months it is number of months... in the case of special type its 'all' or 'overdue' -->
		<xsl:param name="numberOfDays" />
		<xsl:param name="isNegative" />
		
			<xsl:choose>
				<xsl:when test="$typeDD = 'special'">
                 	<xsl:if test="$numberOfDays='all'"><xsl:value-of select="Lang:getStr(&lk;,'ALL')"/>&nbsp;</xsl:if>
                 	<xsl:if test="$numberOfDays='overdue'"><xsl:value-of select="Lang:getStr(&lk;,'OVERDUE_FC')"/>&nbsp;</xsl:if>
				</xsl:when>
				<xsl:otherwise>
							
					<xsl:if test="$isNegative">
						<xsl:value-of select="concat(Lang:getStr(&lk;,'NEXT'), ' ')" />
					</xsl:if>
					
					<xsl:choose>
						<xsl:when test="$numberOfDays > 1">
							<xsl:value-of select="concat($numberOfDays, ' ')" />
							<xsl:choose>
								<xsl:when test="$typeDD = 'days'">
									<xsl:value-of select="concat(Lang:getStr(&lk;,'DAYS'), ' ')" />
								</xsl:when>
								<xsl:when test="$typeDD = 'weeks'">
									<xsl:value-of select="concat(Lang:getStr(&lk;,'WEEKS'), ' ')" />
								</xsl:when>
								<xsl:when test="$typeDD = 'months'">
									<xsl:value-of select="concat(Lang:getStr(&lk;,'MONTHS'), ' ')" />									
								</xsl:when>																		
							</xsl:choose>															
						</xsl:when>
						<xsl:when test="$numberOfDays = 1">
							<xsl:choose>
								<xsl:when test="$typeDD = 'days'">
									<xsl:value-of select="concat(Lang:getStr(&lk;,'DAY'), ' ')" />
								</xsl:when>
								<xsl:when test="$typeDD = 'weeks'">
									<xsl:value-of select="concat(Lang:getStr(&lk;,'WEEK_FC'), ' ')" />
								</xsl:when>
								<xsl:when test="$typeDD = 'months'">
									<xsl:value-of select="concat(Lang:getStr(&lk;,'MONTH'), ' ')" />									
								</xsl:when>																		
							</xsl:choose>							
						</xsl:when>
						<xsl:when test="$numberOfDays = 0">
							<xsl:choose>
								<xsl:when test="$typeDD = 'days'">
									<xsl:value-of select="concat(Lang:getStr(&lk;,'TODAY_FC'), ' ')" />
								</xsl:when>
								<xsl:when test="$typeDD = 'weeks'">
								</xsl:when>
								<xsl:when test="$typeDD = 'months'">
								</xsl:when>																		
							</xsl:choose>							
						</xsl:when>
					</xsl:choose>
					
					<xsl:if test="not($isNegative) and $numberOfDays > 0">
						<xsl:value-of select="concat(Lang:getStr(&lk;,'OVERDUE'), ' ')" />
					</xsl:if>				
				</xsl:otherwise>
			</xsl:choose>

	</xsl:template>
	
	<!--
		Writes HTML output of a Task Assignee dropdown list selector with potential assignees 
		listed along with role assignments 
	 -->
	<xsl:template name="ProjectWCTaskAssigneeDropdownSelector">
		<xsl:param name="project_wctask"/>
		<xsl:param name="isTEP" select="false()"/>
		<xsl:param name="newTaskComposing" select="false()"/>
		<xsl:param name="canChangeAssignee" select="true()"/>
		<xsl:param name="isAssigneeLoggedIn" select="true()"/>
		<xsl:param name="isChangeable" select="true()"/>
		<xsl:param name="currentUserAssignee" select="false()"/>
		
		<xsl:variable name="curTaskAssigneeID" select="/webcenter_page/project_wctask/task_owner_id"/>
		<xsl:variable name="curTaskAssigneeType" select="/webcenter_page/project_wctask/task_owner_type"/>
		
		<!-- Determine whether the curernt user is a valid assignee = should show 'Me' option in the dropdown list -->
		<xsl:variable name="showMeOption">
			<xsl:choose>
				<!-- check if the user is among the valid User members -->
				<xsl:when test="project_user_member_view_list/project_user_member_view/user_view/user[@id = /webcenter_page/loginuserid]">
					<xsl:value-of select="true()"/>
				</xsl:when>
				<!-- check if the user is among the valid Group members' User list-->
				<xsl:when test="proj_group_members_user_members/group_list/group/user_group_list/user_group[user_id = /webcenter_page/loginuserid]">
					<xsl:value-of select="true()"/>
				</xsl:when>
				<!-- Not found among the Project members = do not show the 'Me' option -->
				<xsl:otherwise><xsl:value-of select="false()"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		
		<select name="taskOwner" id="taskOwner" class="form-control">
			<xsl:if test="$isTEP"><xsl:attribute name="onchange">taskExPage.pageSections.genTaskInfo.modified = true;</xsl:attribute></xsl:if>
		
			<!-- writing ASSIGNEE selector dropdown list options -->
			
			<!-- 'nobody' can be set as task assignee only while task is in 'created' status or its just being created -->
			<!-- NOTE: 'Nobody' option is not avaiable when working with 'restricted' tasks  -->
			<xsl:if test="($newTaskComposing and not($project_wctask/wctask_type/restrict_assignees = '1')) 
			or ($project_wctask/status/@id = '00001_0000000001' and string-length($project_wctask/main_assignee_id) = 0)">
				<option value="nobody">&lt;<xsl:value-of select="Lang:getStr(&lk;,'NOBODY_TAG')"/>&gt;</option>
			</xsl:if>
			
			<!-- show "me" option in assignee dropdown box only if logged in user is among project members -->
			<xsl:if test="$showMeOption = 'true'">
			 	<option value="current_user">
			 		<xsl:if test="$currentUserAssignee and (/webcenter_page/loginuserid = $curTaskAssigneeID) and ($curTaskAssigneeType = 'user')">
						<xsl:attribute name="selected">selected</xsl:attribute>
					</xsl:if>
				 	<xsl:value-of select="Lang:getStr(&lk;,'PROJ_TASK_ASSIGNTOMYSELF')"/>
			 	</option>
		 	</xsl:if>
								  
			<xsl:choose>
				<!-- Display all the valid potential assignees if the current user can change the task's assignees -->
				<xsl:when test="($canChangeAssignee and $isAssigneeLoggedIn) or $isChangeable">
					
					<!-- When the user is de-activated, he is not part of the project user/group member list.
						Add this user seperately to the drop down list -->
					<xsl:if test="project_wctask/user/is_active = '0' ">
						<option value="U{project_wctask/user/@id}">
							<xsl:attribute name="selected">selected</xsl:attribute>
							
							<xsl:call-template name="DisplayUserNameTextOnly">
								<xsl:with-param name="lastName" select="project_wctask/user/last_name"/>
								<xsl:with-param name="firstName" select="project_wctask/user/first_name"/>
								<xsl:with-param name="loginName" select="project_wctask/user/name"/>
								<xsl:with-param name="isActive" select="project_wctask/user/is_active" />
							</xsl:call-template>
						</option>
					</xsl:if>
					
					<!-- Show all Role members with their assignments first -->
					<xsl:for-each select="project_group_member_view_list/project_group_member_view[group/is_role ='1']">
						<xsl:sort select="group/name" data-type="text" order="ascending"/>	
						
						<!-- write the main ROLE option-->
						<option value="G{group/@id}">
							<xsl:if test="not($currentUserAssignee) and (group/@id = $curTaskAssigneeID) and ($curTaskAssigneeType = 'group')">
								<xsl:attribute name="selected">selected</xsl:attribute>
							</xsl:if>
							<xsl:call-template name="DisplayGroupName">
								<xsl:with-param name="groupName" select="group/name"/>
								<xsl:with-param name="isRole" select="group/is_role"/>
							</xsl:call-template>
						</option>
							
						<!-- also write any 'role assignment' options related to the current Role-->
						<xsl:if test="key('roleAssignmentDataKey', group/@id)">
							<xsl:call-template name="IndividualRoleAssignmentAssigneeOptions">
								<xsl:with-param name="role_assignment_data_root" select="key('roleAssignmentDataKey', group/@id)"/>
							</xsl:call-template>
						</xsl:if>
					</xsl:for-each>
							  
					<!--  show all user project members -->
					<xsl:for-each select="project_user_member_view_list/project_user_member_view">
						<xsl:sort select="user_view/user/last_name" data-type="text" order="ascending"/>
						<xsl:sort select="user_view/first_name" data-type="text" order="ascending"/>
									
						<option value="U{user_view/user/@id}">
							<xsl:if test="not($currentUserAssignee) and (user_view/user/@id = $curTaskAssigneeID) and ($curTaskAssigneeType = 'user')">
								<xsl:attribute name="selected">selected</xsl:attribute>
							</xsl:if>
							
							<xsl:call-template name="DisplayUserNameTextOnly">
								<xsl:with-param name="lastName" select="user_view/user/last_name"/>
								<xsl:with-param name="firstName" select="user_view/user/first_name"/>
								<xsl:with-param name="loginName" select="user_view/user/name"/>
								<xsl:with-param name="isActive" select="user_view/user/is_active" />
							</xsl:call-template>
						</option>
					</xsl:for-each>
					
					<!--  show all group project members -->
					<xsl:for-each select="project_group_member_view_list/project_group_member_view[group/is_role ='0']">
						<xsl:sort select="group/name" data-type="text" order="ascending"/>
									
						<option value="G{group/@id}">
							<xsl:if test="not($currentUserAssignee) and (group/@id = $curTaskAssigneeID) and ($curTaskAssigneeType = 'group')">
								<xsl:attribute name="selected">selected</xsl:attribute>
							</xsl:if>
							<xsl:call-template name="DisplayGroupName">
								<xsl:with-param name="groupName" select="group/name"/>
								<xsl:with-param name="isRole" select="group/is_role"/>
							</xsl:call-template>
						</option>
						
						<!-- Write options for all the Group's User members as well -->
						<xsl:if test="key('projGroupMemberUserMembersDataKey', group/@id)">
							<xsl:for-each select="key('projGroupMemberUserMembersDataKey', group/@id)/user_group_list/user_group">
								<option value="U{user_id}">
									<xsl:if test="not($currentUserAssignee) and (user_id = $curTaskAssigneeID) and ($curTaskAssigneeType = 'user')">
										<xsl:attribute name="selected">selected</xsl:attribute>
									</xsl:if>
									&nbsp;&nbsp;&nbsp;&nbsp;
									<xsl:call-template name="DisplayUserNameTextOnly">
										<xsl:with-param name="lastName" select="last_name"/>
										<xsl:with-param name="firstName" select="first_name"/>
										<xsl:with-param name="loginName" select="user_name"/>
										<xsl:with-param name="isActive" select="is_active" />
									</xsl:call-template>
								</option>
							</xsl:for-each>
						</xsl:if>
					</xsl:for-each>
				</xsl:when>
				<!-- 
					Handle the situation when logged in user has no "change task" permission and
				  	is not currently assignee of edited wctask. 
					In this case the original assignee has to be added to list of permitted assignees manually... 
				-->
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="(string-length($curTaskAssigneeID) &gt; 0) and ($curTaskAssigneeType = 'user') and (/webcenter_page/is_current_user_assignee != '1')">
							<option value="U{project_wctask/task_owner_id}" selected="selected">
								<xsl:call-template name="DisplayUserNameTextOnly">
									<xsl:with-param name="lastName" select="project_wctask/user/last_name"/>
									<xsl:with-param name="firstName" select="project_wctask/user/first_name"/>
									<xsl:with-param name="loginName" select="project_wctask/user/name"/>
									<xsl:with-param name="isActive" select="project_wctask/user/is_active" />
								</xsl:call-template>
							</option>
						</xsl:when>
						<xsl:when test="(string-length($curTaskAssigneeID) &gt; 0) and ($curTaskAssigneeType = 'group')">
							<option value="G{project_wctask/task_owner_id}" selected="selected">
								<xsl:call-template name="DisplayGroupName">
									<xsl:with-param name="groupName" select="project_wctask/group/name"/>
									<xsl:with-param name="isRole" select="project_wctask/group/is_role"/>
								</xsl:call-template>
							</option>
						</xsl:when>
						<xsl:otherwise>
							<option value="nobody" selected="selected">&lt;<xsl:value-of select="Lang:getStr(&lk;,'NOBODY_TAG')"/>&gt;</option>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		</select>
	</xsl:template>

	<!--
		Writes SELECT options that represent specified Role's assignments 
	 -->
	<xsl:template name="IndividualRoleAssignmentAssigneeOptions">
		<xsl:param name="role_assignment_data_root"/>
		
		<!-- write User assignees first -->
		<xsl:for-each select="$role_assignment_data_root/assigned_users_list/assigned_user">
				<option value="U{user/@id}">&nbsp;&nbsp;&nbsp;&nbsp;<xsl:call-template name="encodesinglequotes"><xsl:with-param name="string"><xsl:call-template name="DisplayUserNameTextOnly">
								<xsl:with-param name="lastName" select="user/last_name"/>
								<xsl:with-param name="firstName" select="user/first_name"/>
								<xsl:with-param name="loginName" select="user/name"/>
								<xsl:with-param name="isActive" select="user/is_active"/>
								</xsl:call-template></xsl:with-param></xsl:call-template></option>
		</xsl:for-each>
	
		<!-- and then also write all Group assignees -->
		<xsl:for-each select="$role_assignment_data_root/assigned_groups_list/assigned_group">
			<xsl:if test="not(assigned_group_assignee) or (assigned_group_assignee/user/is_active = '1')">
				<xsl:variable name="optionValue">
					<xsl:choose>
						<xsl:when test="assigned_group_assignee">U<xsl:value-of select="assigned_group_assignee/user/@id"/></xsl:when>
						<xsl:otherwise>G<xsl:value-of select="@id"/></xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:variable name="optionText">
					<xsl:if test="assigned_group_assignee"><xsl:call-template name="encodesinglequotes"><xsl:with-param name="string"><xsl:call-template name="DisplayUserNameTextOnly">
						<xsl:with-param name="lastName" select="assigned_group_assignee/user/last_name"/>
						<xsl:with-param name="firstName" select="assigned_group_assignee/user/first_name"/>
						<xsl:with-param name="loginName" select="assigned_group_assignee/user/name"/>
						<xsl:with-param name="isActive" select="assigned_group_assignee/user/is_active"/>
						</xsl:call-template></xsl:with-param></xsl:call-template>&nbsp;@&nbsp;</xsl:if>
					<xsl:call-template name="DisplayGroupName">
						<xsl:with-param name="groupName" select="assigned_group_name"/>
						<xsl:with-param name="isRole" select="0"/>
					</xsl:call-template>
				</xsl:variable>
				<option value="{$optionValue}">&nbsp;&nbsp;&nbsp;&nbsp;<xsl:value-of select="$optionText"/></option>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
		
	<!-- 
  
            template which writes the javascript needed for the project details

  -->
	<xsl:template name="ProjWcTaskDetailsJavaScript">
    <script src="javascript/date.js"/>&#xA;	
    <script src="javascript/datetools.js"/>&#xA;	

		<script src="javascript/element.js" type="text/javascript"></script>
		<script src="javascript/hover.js" type="text/javascript"></script>
		<script src="javascript/webcenter_html.js" type="text/javascript"></script>
		<script src="javascript/wctasks.js" type="text/javascript"></script>
   

		<script type="text/javascript">
			<xsl:text disable-output-escaping="yes">&lt;!--</xsl:text>&#xA;
			
	/* start of getting data for wctasks.js functionality */

		// process all the possible predecessor tasks
		tasks = new Array(
			<xsl:for-each select="/webcenter_page/project_wctask_list/project_wctask">
 				<xsl:sort select="due_date" data-type="number" order="ascending"/>
				{predecessor:false, id:'<xsl:value-of select="@id"/>',
					name:'<xsl:value-of select="XSLUtils:encodeForJScript(name)"/>', 
					assignedTo:'<xsl:value-of select="XSLUtils:encodeForJScript(task_owner_name)"/>',
					dueDate:'<xsl:value-of select="due_date"/>', 
					taskTypeID:'<xsl:value-of select="task_type_id"/>',
					successors:[
					<xsl:for-each select="successor_list/successor_id">
						{id:'<xsl:value-of select="."/>', idx:null}
						<xsl:if test="position()!=last()">,</xsl:if>
					</xsl:for-each>
					]
				},
			</xsl:for-each>
			null
		);
		tasks.length -= 1; // remove the last auxiliary element (== null)

		// store task predecessors IDs to auxiliary array
		predecessorIDs = new Array(
			<xsl:for-each select="/webcenter_page/project_wctask/predecessor_list/predecessor_id">
				'<xsl:value-of select="."/>'
				<xsl:if test="position()!=last()">,</xsl:if>
			</xsl:for-each>
		);
		
		// process all defined task types and their statuses
		taskTypes = new Array(
			<xsl:for-each select="/webcenter_page/wctask_type_list/wctask_type_view">
				<xsl:sort select="wctask_type/wctask_type_name" data-type="text" order="ascending"/>
				{id:'<xsl:value-of select="wctask_type/@id"/>',
					name:'<xsl:value-of select="XSLUtils:encodeForJScript(wctask_type/task_type_name)"/>', 
					creatorName:'<xsl:value-of select="XSLUtils:encodeForJScript(user/name)"/>',
					days:parseInt('<xsl:value-of select="wctask_type/approximate_length"/>'), 
					hours:parseInt('<xsl:value-of select="wctask_type/approximate_time"/>'),
					restrictAssignees:'<xsl:value-of select="wctask_type/restrict_assignees"/>', 
					preselectedMainAssigneeID:'<xsl:value-of select="wctask_type/main_assignee_id"/>',
					preselectedMainAssigneeName:'<xsl:value-of select="XSLUtils:encodeForJScript(wctask_type/main_assignee_name)"/>',
					preselectedMainAssigneeIsRole: '<xsl:value-of select="wctask_type/main_assignee_isrole"/>',
					statuses:[
					<xsl:for-each select="status_list/task_type_status_view">
						<xsl:sort select="wc_status/display_order" data-type="number" order="ascending"/>
						{id:'<xsl:value-of select="wc_status/@id"/>',
						 name:'<xsl:value-of select="XSLUtils:encodeForJScript(wc_status/status_name)"/>',
						 taskTypeId:'<xsl:value-of select="wctask_type/@id"/>'}
						<xsl:if test="position()!=last()">,</xsl:if>
					</xsl:for-each>
					] 
				}
				<xsl:if test="position()!=last()">,</xsl:if>
			</xsl:for-each>
		);
		
		loggedInUserID = '<xsl:value-of select="/webcenter_page/loginuserid"/>';
		newTaskComposing = <xsl:value-of select="not(/webcenter_page/project_wctask/status)"/>;
		isTemplate = <xsl:value-of select="/webcenter_page/project/is_template = '1'"/>;
		
		currentTaskID = '<xsl:value-of select="/webcenter_page/project_wctask/@id"/>';
		taskStatusID = '<xsl:value-of select="/webcenter_page/project_wctask/status/@id"/>';
		origStatusName = '<xsl:value-of select="XSLUtils:encodeForJScript(/webcenter_page/project_wctask/status/status_name)"/>';

		newProjectStatusID = '<xsl:value-of select="/webcenter_page/project_wctask/project_status_after_completion"/>';
		<xsl:choose>
			<xsl:when test="/webcenter_page/project_wctask/task_owner_type = 'user'">
				taskOwnerID = 'U<xsl:value-of select="/webcenter_page/project_wctask/task_owner_id"/>';
			</xsl:when>
			<xsl:otherwise>
				taskOwnerID = 'G<xsl:value-of select="/webcenter_page/project_wctask/task_owner_id"/>';
			</xsl:otherwise>
		</xsl:choose>

		userTimezoneOffset = parseInt('<xsl:value-of select="/webcenter_page/timezone_offset"/>');

		dueDate = parseInt('<xsl:value-of select="/webcenter_page/project_wctask/due_date"/>');
		leadDays = parseInt('<xsl:value-of select="/webcenter_page/project_wctask/approximate_length"/>');
		leadHours = parseInt('<xsl:value-of select="/webcenter_page/project_wctask/approximate_time"/>');

		// language strings used in Javascript
		// -----------------------------------
		langStrTaskName = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'TASKNAME')"/></xsl:call-template>";
		langStrAssignedTo = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'PROJ_TASK_OWNER')"/></xsl:call-template>";
		langStrDueDate = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'TODO_DUEDATE')"/></xsl:call-template>";
		langStrNoTaskToAdd = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'PROJ_TASK_NO_TASK_TO_ADD')"/></xsl:call-template>";
		langStrNoPredecessor = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'PROJ_TASK_NO_PREDECESSOR_DEFINED')"/></xsl:call-template>";
		langStrStartDateReset = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'PROJ_TASK_RESET_START_DATE')"/></xsl:call-template>";
		langStrNobodyOption = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'NOBODY_TAG')"/></xsl:call-template>";

		// error messages		
		langStrInvalidLeadTime = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'PROJ_TASK_INVALID_LEADTIME')"/></xsl:call-template>";
		langStrNoDueDateErr = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'PROJ_TASK_DUEDATE_REQUIRED')"/></xsl:call-template>";
		langStrDueDateInPastErr = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'PROJ_TASK_DUEDATE_IN_PAST')"/></xsl:call-template>";
		langStrStartDateInPastErr = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'PROJ_TASK_STARTDATE_IN_PAST')"/></xsl:call-template>";
		langStrTaskSequenceErr = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'PROJ_TASK_TIME_SEQUENCE_ERR')"/></xsl:call-template>";
		langStrNonnegativeValueErr = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'NEGATIVENUMERICRESTRICTIONERROR')"/></xsl:call-template>";
		langStrDeletePredecessor = "<xsl:call-template name="escapequotationmark"><xsl:with-param name="string" select="Lang:getStr(&lk;,'NEWTASK_REMOVE_PREDECESSOR')"/></xsl:call-template>";

	/* end of getting data for wctasks.js functionality */


	// onDateChange(elem, timeElemID) handles date field changes and its counterpart
	// onDateChangeTime(elem, dateElemeID) handles corresponding time field changes.
		
		function onDateChange(elem, timeElemID)
		{
			var d = Date.parse(elem.value);
			var date = new Date(d);
			
			if (timeElemID &amp;&amp; (timeElemID.length > 0))
			{
				var time = $("#" + timeElemID).val();
				var timeInt = parseInt(time);
				if (!isNaN(timeInt) &amp;&amp; (timeInt > -1)) 
				{
					date.setHours(Math.floor(timeInt / 2));
					date.setMinutes((timeInt % 2) * 30);
				}
			}
			
			setDueDate(date);			
		}
		
		function onDateChangeTime(elem, dateElemID)
		{
			var d = Date.parse($("#" + dateElemID).val());
			var date = new Date(d);

			var time = elem.value;
			var timeInt = parseInt(time);
			if (!isNaN(timeInt) &amp;&amp; (timeInt > -1)) 
			{
				date.setHours(Math.floor(timeInt / 2));
				date.setMinutes((timeInt % 2) * 30);
				}
			setDueDate(date);			
				}

				
		function MoveFocusFromStatus(elem)
    {
	    var isComplete = '<xsl:value-of select="/webcenter_page/project_wctask/is_complete"/>';
		if (isComplete == 0)
		{
			document.actionform.taskName.focus();
		}	 
	    return;
		}
		
			<xsl:text disable-output-escaping="yes">//--&gt;</xsl:text>&#xA;
		</script>&#xA;
	</xsl:template>
</xsl:stylesheet>
