<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE xsl:stylesheet [
  <!ENTITY copy "&#169;">
  <!ENTITY nbsp "&#160;">
  <!ENTITY lk "/webcenter_page/user_lang_key">
]>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:Lang="com.esko.webcenter.jsp.LanguageMgr" exclude-result-prefixes="Lang">
  <xsl:import href="template.xsl"/>
  <xsl:import href="templatedetailsreadonly.xsl"/>
  <xsl:import href="securitytemplates.xsl"/>  

  
  
  
  <xsl:key name="LocationViewKey" match="/webcenter_page/project_user_member_list/user_view/location_view" use="@id"/>
  <xsl:key name="FolderPermViewKey" match="/webcenter_page/folder_user_permission_list/folder_user_permission/folder" use="@id"/>
  
  <!-- 
  
            template that controls the generation of the body content
            
  -->
  <xsl:template name="bodycontent">
	<tr class="white_background"><td height="5" ><img src="images/clearpixel.gif" width="1" /></td></tr>
	
	<xsl:variable name="userListLength" select="count(project_user_member_view_list/project_user_member_view/user_view/user)"/>
	<xsl:choose>
		<xsl:when test="$userListLength = 0">
			<!-- Project contains no users -->
			<xsl:call-template name="NoMembers">
				<xsl:with-param name="Name" select="Lang:getStr(&lk;,'NOUSERSAVAIL')"/>
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise>
			<!-- User permissions -->
			<xsl:call-template name="ShowUserPermissions"/>
		</xsl:otherwise>
	</xsl:choose>
  	
  </xsl:template>
  
  
  <!--
	Display user permissions
  -->
  <xsl:template name="ShowUserPermissions">
		
	<!-- Pick user to show -->
    <form  id="filterform" name="filterform">
		<input type="hidden" name="actionVal" value=""/>
		<xsl:variable name="SelectedUserID" select="project_user_member_view/member_user_id"/>
		<xsl:variable name="SelectedProjUserMemberID" select="project_user_member_view/member_id"/>
		
        <input type="hidden" name="selectedUserId" value="{$SelectedUserID}"/>
        <xsl:call-template name="projDetailsHiddenInputs"/>

	<tr class="white_background"><td class="darktext_darklink_bold_12">
		<img src="images/clearpixel.gif" width="5" height="" border="0"/>
		<span class="darktext_darklink_bold_12">	
			<xsl:value-of select="Lang:getStr(&lk;,'PERMSFORUSER')"/>&nbsp;
		<i class="icon icon-normal icon-grey icon-WCR_Assignee-User"></i>&nbsp;
		</span>
			
		<select name="userID" onchange="GetUserPerm(this);">
			
			<xsl:for-each select="project_user_member_view_list/project_user_member_view">
			   <xsl:variable name="CurUserMemberID" select="member_id"/>
		
				<option value="{$CurUserMemberID}" >				
					<xsl:if test="$CurUserMemberID=$SelectedProjUserMemberID">
					  <xsl:attribute name="selected">selected</xsl:attribute>
					</xsl:if>
					<xsl:call-template name="DisplayUserNameTextOnly">
						<xsl:with-param name="userID" select="user_view/user/@id"/>
						<xsl:with-param name="lastName" select="user_view/user/last_name"/>
						<xsl:with-param name="firstName" select="user_view/user/first_name"/>
						<xsl:with-param name="isActive" select="user_view/user/is_active" />
						<xsl:with-param name="loginName" select="user_view/user/name"/>
						<xsl:with-param name="roleGroupNameList" select="role_assignment_data"/>
					</xsl:call-template>				
				</option>
			
			</xsl:for-each>
		</select>
	</td></tr>
		
    </form>
	
	<tr class="white_background"><td height="10" ><img src="images/clearpixel.gif" width="1" /></td></tr>	
	
	<!-- User's permissions -->
    <form id="actionform" name="actionform">
        <input type="hidden" name="actionVal" value=""/>
		
		<xsl:variable name="SelectedUserID" select="project_user_member_view/member_user_id"/>
		<xsl:variable name="SelectedProjUserMemberID" select="project_user_member_view/member_id"/>
		
        <input type="hidden" name="userID" value="{$SelectedUserID}"/>
        <xsl:call-template name="projDetailsHiddenInputs"/>
		<input type="hidden" name="selectedFolderID"      value=""/>
		<input type="hidden" name="selectedProjectID"      value=""/>
		<input type="hidden" name="selectedUserID"      value="{$SelectedUserID}"/>
		<input type="hidden" name="selectedProjectUserMemberId" value="{$SelectedProjUserMemberID}"/>
									
		<!-- Project/folder permissions -->
		<tr class="white_background"><td height="10" ><img src="images/clearpixel.gif" width="1" /></td></tr>	
		<xsl:call-template name="ProjectFolderPermissions" />			
			
    </form>	
	
  </xsl:template>
  
  <!--
	Template that controls project/folder permissions
	-->
  <xsl:template name="ProjectFolderPermissions">
		  
	  <tr class="white_background"><td height="10" ><img src="images/clearpixel.gif" width="1" /></td></tr>	
	  <tr>
		<td>
			
			<table width="100%" border="0" cellspacing="0" cellpadding="0">
				<tr>
					<xsl:call-template name="AddWhiteWidth"/> 
					<td>
			
						<table class="ordtable" width="100%">
							<!-- table header -->							  
							<thead>
							  <tr class="largerHeader">
								<xsl:call-template name="PermissionTableHeader"> 
									<xsl:with-param name="displayFullCheckbox" select="''"/>
								</xsl:call-template>
							  </tr>
							</thead>
									
							<!-- Get project permissions as default permissions -->
							<xsl:variable name="defaultView" select="permissions_overview/project_user_permissions/permission/view"/>
							<xsl:variable name="defaultAdd" select="permissions_overview/project_user_permissions/permission/add"/>
							<xsl:variable name="defaultUpdate" select="permissions_overview/project_user_permissions/permission/update"/>
							<xsl:variable name="defaultDelete" select="permissions_overview/project_user_permissions/permission/delete"/>
							<xsl:variable name="defaultLock" select="permissions_overview/project_user_permissions/permission/checkout"/>
							<xsl:variable name="defaultDownload" select="permissions_overview/project_user_permissions/permission/download"/>
							<xsl:variable name="defaultChangeTask" select="permissions_overview/project_user_permissions/permission/change_task"/>
							<xsl:variable name="defaultChangeBom" select="permissions_overview/project_user_permissions/permission/change_bom"/>
							<xsl:variable name="defaultChangeApproval" select="permissions_overview/project_user_permissions/permission/change_approval"/>
							<xsl:variable name="isAdmin" select="permissions_overview/project_user_permissions/is_admin"/>
							
							<tbody>
								<!-- Show project permissions -->
								<xsl:call-template name="PermissionRowRO">
									<xsl:with-param name="view" select="$defaultView"/>
									<xsl:with-param name="add" select="$defaultAdd"/>
									<xsl:with-param name="update" select="$defaultUpdate"/>
									<xsl:with-param name="delete" select="$defaultDelete"/>
									<xsl:with-param name="lock" select="$defaultLock"/>
									<xsl:with-param name="download" select="$defaultDownload"/>
									<xsl:with-param name="changeApproval" select="$defaultChangeApproval"/>
									<xsl:with-param name="changeTask" select="$defaultChangeTask"/>
									<xsl:with-param name="changeBom" select="$defaultChangeBom"/>
									<xsl:with-param name="isAdmin" select="$isAdmin"/>
									<xsl:with-param name="isProject" select="'1'"/>
									<xsl:with-param name="action" select="'SetUserPerm'"/>
									<xsl:with-param name="name" select="project/name"/>		
								</xsl:call-template>
							
						  
							  <!-- show folder permissions -->
							  <xsl:for-each select="project/folder_list/folder">	
								<xsl:variable name="folderID" select="@id"/>
								<xsl:variable name="folderName" select="name"/>
								<xsl:variable name="existFolderPerm" select="key('FolderPermViewKey', $folderID)"/>	
	
								<xsl:choose>
									<xsl:when test="$existFolderPerm">
										<!-- Get folder permissions then call permission row -->
										<xsl:for-each select="/webcenter_page/folder_user_permission_list/folder_user_permission">
											<xsl:if test="folder/@id = $folderID">
												<xsl:call-template name="PermissionRowRO">
													<xsl:with-param name="view" select="permission/view"/>
													<xsl:with-param name="add" select="permission/add"/>
													<xsl:with-param name="update" select="permission/update"/>
													<xsl:with-param name="delete" select="permission/delete"/>
													<xsl:with-param name="lock" select="permission/checkout"/>
													<xsl:with-param name="download" select="permission/download"/>
													<xsl:with-param name="changeApproval" select="permission/change_approval"/>
													<xsl:with-param name="changeTask" select="$defaultChangeTask"/>
													<xsl:with-param name="changeBom" select="$defaultChangeBom"/>
													<xsl:with-param name="isAdmin" select="$isAdmin"/>
													<xsl:with-param name="action" select="'SetUserPerm'"/>
													<xsl:with-param name="name" select="$folderName"/>	
													<xsl:with-param name="folderID" select="$folderID"/>	
												</xsl:call-template>	
											</xsl:if>
										</xsl:for-each>
									</xsl:when>
									<xsl:otherwise>
												<!-- call permission row with default permissions -->
												<xsl:call-template name="PermissionRowRO">
													<xsl:with-param name="view" select="$defaultView"/>
													<xsl:with-param name="add" select="$defaultAdd"/>
													<xsl:with-param name="update" select="$defaultUpdate"/>
													<xsl:with-param name="delete" select="$defaultDelete"/>
													<xsl:with-param name="lock" select="$defaultLock"/>
													<xsl:with-param name="download" select="$defaultDownload"/>
													<xsl:with-param name="changeApproval" select="$defaultChangeApproval"/>
													<xsl:with-param name="changeTask" select="$defaultChangeTask"/>
													<xsl:with-param name="changeBom" select="$defaultChangeBom"/>
													<xsl:with-param name="isAdmin" select="$isAdmin"/>
													<xsl:with-param name="action" select="'SetUserPerm'"/>
													<xsl:with-param name="name" select="$folderName"/>	
													<xsl:with-param name="folderID" select="$folderID"/>	
												</xsl:call-template>			
									</xsl:otherwise>
								</xsl:choose>																
							  </xsl:for-each>			  
							</tbody>
						</table>
					</td>
					<xsl:call-template name="AddWhiteWidth"/> 
				</tr>
			</table>

		</td>
	  </tr> 
	  
  </xsl:template>
  

  <xsl:template name="DisplayName" >
	<xsl:param name="name" />	
	<xsl:param name="isProject" />			
	<xsl:param name="folderID" />		
		
	<xsl:variable name="userID" select="cur_user_id"/>
			
	<table width="100%">
	  <tr>
		<td nowrap="nowrap" align="left" valign="middle" class="darktext_darklink_11">
		<img src="images/clearpixel.gif" width="5" height="" border="0"/>
		<img src="images/folder_closed.png" border="0"/>
		<img src="images/clearpixel.gif" width="5" height="" border="0"/>
		<xsl:value-of select="$name"/>
		</td>
	  </tr>
	</table>
  </xsl:template>

  
  <!-- 
  
            template that controls the inclusion of javascript functionality 
            
  -->
  <xsl:template name="scriptincs">
    <script src="javascript/validate.js"/>&#xA;
    <script src="javascript/TableSelection.js"/>&#xA;
    <script type="text/javascript">
      <xsl:text disable-output-escaping="yes">&lt;!--</xsl:text>&#xA;		
         
	  function GetUserPerm(obj)
	  {		
		var curProjectID = document.projactionform.projectID.value;
		var curProjuserMemberID = obj.value;
		
		window.location.href = "projdetailssecurityuser.jsp?curSecurityMemberID=" + curProjuserMemberID + "&amp;projectID="+curProjectID+"&amp;template_details=1" ;
		
		//document.filterform.submit();
	  }				
		
      <xsl:text disable-output-escaping="yes">//--&gt;</xsl:text>&#xA;
    </script>
    <xsl:call-template name="ProjDetailsJavaScript"/>  
  </xsl:template>    
</xsl:stylesheet>
