/**
 * @author DOGO, PHST
 * @module WebViewer3D
 * @copyright   2014 Esko Stonecube Ltd. All rights reserved.
 */

define([
    "backbone",
    "underscore",
    "WV3Dlib/three",
    "WV3Dmod/threeEXT",
    "WV3Dmod/Math",
    "WV3Dmod/SystemChecker",
    "WV3Dmod/HttpRequestHandler",
    "WV3Dmod/ThreeDObjectHandler",
    "WV3Dmod/TextSpriteMaker",
    "WV3Dmod/shaders/UniformColorShader",
    "WV3Dmod/shaders/EdgeShader2",//Choose between EdgeShader, EdgeShader2, EdgeShader3 (EdgeShader3 seems to be the less expensive with better results)
    "WV3Dmod/shaders/ShaderUtils",
    "WV3Dmod/shaders/D0Shader",
    "WV3Dmod/shaders/D1Shader",
    "WV3Dmod/shaders/D2Shader",
    "WV3Dmod/shaders/DepthOnlyShader",
    "WV3Dmod/shaders/BackdropShader",
    "WV3Dmod/shaders/ColorBlendShader",
    "WV3Dmod/shaders/TransparencyBlendShader",
    "WV3Dmod/shaders/CopyShader",
    "WV3Dmod/shaders/DesignerShader",
    "WV3Dmod/shaders/FilterShaderPM",
    "WV3Dmod/shaders/FilterShaderNPM",
    "WV3Dmod/shaders/ReflectanceShaderPM",
    "WV3Dmod/shaders/ReflectanceShaderNPM",
    "WV3Dmod/shaders/IrradianceShaderPM",
    "WV3Dmod/shaders/IrradianceShaderNPM",
    "WV3Dmod/shaders/DiffractionShaderPM",
    "WV3Dmod/Constants",
    "WV3Dmod/Utils",
    "WV3Dmod/FPSCounter",
    "WV3Dmod/Timers",
    "WV3Dmod/ExecutionQueue",
    "WV3Dmod/LogMessage"
    ], function (
        bb,
        _,
        THREE,
        WV3DThreeEXT,
        WV3DMath,
        WV3DSystemChecker,
        WV3DHttpRequestHandler,
        WV3DObjectHandler,
        WV3DTextSpriteMaker,
        UniformColorShader,
        EdgeShader,
        ShaderUtils,
        D0Shader,
        D1Shader,
        D2Shader,
        DepthOnlyShader,
        BackdropShader,
        ColorBlendShader,
        TransBlendShader,
        CopyShader,
        DesignerShader,
        FilterShaderPM,
        FilterShaderNPM,
        ReflectanceShaderPM,
        ReflectanceShaderNPM,
        IrradianceShaderPM,
        IrradianceShaderNPM,
        DiffractionShaderPM,
        WV3DConstants,
        WV3DUtils,
        WV3DFPSCounter,
        WV3DTimers,
        ExecutionQueue,
        LogMessage
        ) {

    'use strict';

    /**
     * @alias module:WebViewer3D
     * @class WV3DViewerView
     */
    var WV3DViewerView = bb.Model.extend({

    initialize: function(__context, __WebViewerModel, __clientUIAddElementCallback)
    {

        LogMessage.Log("Creating WV3DViewerView Object");

        /** @private */
        this.set('_context', __context);

        /** @private */
        this.set('_model', __WebViewerModel);

        /** @private */
        this.set('_clientUIAddElementCallback', __clientUIAddElementCallback);

        /** @private */
        var threejsproperties =
        {
            alpha:true,
            antialias: WV3DSystemChecker.get('_glantialias'),
            preserveDrawingBuffer: true //this allows to take screenshots using toDataURL("image/jpeg")
            // logarithmicDepthBuffer: true//Warning: this only works with a perspective camera, otherwise z-test may fail
        };
        /** Extend threejs before using it*/
        new WV3DThreeEXT();
        /** @private */
        this.set("_3JSrenderer", new THREE.WebGLRenderer(threejsproperties));
        this.get("_3JSrenderer").sortObjects = false;//We are responsible for sorting surfaces (especially for coincident surfaces)

        /*Initialize render targets for depth peeling*/
        this.InitializeRenderTargets();

        /** @private */
        this.set('_textSpriteMaker', new WV3DTextSpriteMaker());

        /** Default resource path */
        this.set('_resourcePath', '../Resources')

        this.set("_current_fps", 0);
        //We want at least an interactive framerate for scenes with transparency
        this.set("_target_fps_min", 25);
        this.set("_target_fps_max", 60);
        //We want at least transparency to be "fine" during interaction
        this.set("_nb_max_peels", 7);
        this.set("_nb_min_peels", 2);

        this.set("_reflection_config","JSON");
        this.set('_enableInteractionOptimization', false);
        this.set('_enableAnimations', false);

        this.set("_lighting_contrast",0.5);

        this.set("_optimised_view", false);
        this.set("_optimised_view_border", 0);

        this.set("_obj_in_frustum_prev", true);
        this.set("_obj_in_frustum", true);

        this.set("_polygon_offset_factor",0.5);
        this.set("_polygon_offset_units",1);    // For unknown reasons, the units factor is scaled down somewhere by a huge factor.
                                                // I had to test numbers in the 10000-range to see visual differences, unless zoomed in!
                                                // Does Three.JS use the OpenGl 1.0 polygon offset (with bias) instead of 1.1 (with units)?

        this.set("_build_scene_exec_queue",new ExecutionQueue(false, false, _.bind(this.NotifySceneBuildProgress,this)));

        ///////////
        //TODO: This dummy textures should set to null in future versions of threejs (rev 77 ?)
        //however fix at this url seems to break textures, be cautious when upgrading
        //see https://github.com/mrdoob/three.js/commit/ae81860411fb397e5905b8282eb497351a61121b#comments
        var img = new Image(2, 2);
        img.src = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAIAAAACCAIAAAD91JpzAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4AUMDiIqLjEjFgAAAAtJREFUCNdjYEAGAAAOAAEx6d0VAAAAAElFTkSuQmCC";
        var dummy_tex = new THREE.Texture(img);
        var dummy_cube_tex = new THREE.CubeTexture([img,img,img,img,img,img]);
        img.onload = function () {
            //For the moment this asynchronous callback is triggered after some
            //intermediate draw calls (ViewerView.Render() function) might occur
            //this might result in some WebGL warning message while file is still loading
            //Depending on the speed/priorities of the browser
            //However, when file is finished loading, no more warning should be output from WebGL
            dummy_tex.needsUpdate = true;
            dummy_cube_tex.needsUpdate = true;
            LogMessage.Log("Dummy texture loaded");
        };

        this.set('_DummyTexture', dummy_tex);
        this.set('_DummyTextureCube', dummy_cube_tex);
        ///////////

        /** create a new scene: this resets the cache, without setting the shaders */
        this.ResetScene();

        /**Set default camera*/
        this.CameraUpdate(false);

        /** subscribe to any change in the model */
        this.listenTo(this.get('_model'), 'scene_reset', this.ResetScene);
        this.listenTo(this.get('_model'), 'added_scene_resource', this.AddedResource);

        window.addEventListener( 'resize', _.bind(this.Resize, this), false );

        this.Resize();

        this.SetBackgroundColor(0.37,0.37,0.37,1., false);
        this.AppendRendererToDocument();
    },

    NotifySceneBuildProgress: function(__geometry_percent, __texture__percent, __error__code)
    {
        if(typeof this.get('_model').get('_clientCallback') === "function")
        {
            if(this.get('_model').get('_assetmanager').get('_textureCounter') == 0)//If no texture to load, then emit a default 100% conplete signal
            {
                __texture__percent = 100;
            }
            if (__texture__percent === undefined)
            {
                __texture__percent = this.get('_model').get('_assetmanager').get('_textureLoadedCounter') / this.get('_model').get('_assetmanager').get('_textureCounter') * 100;
            }
            if (__error__code === undefined)
            {
                __error__code = 0;
            }
            this.get('_model').get('_clientCallback')(__error__code,__geometry_percent,__texture__percent,
                this.get('_model').get('_assetmanager').get('_assetloader').get('_httprequestmanager').get('_queriedassetnumber'),
                this.get('_model').get('_assetmanager').get('_resolvedassetnumber'));
        }else
        {
            LogMessage.Log("ViewerView Warning: Client callback not a function");
        }
    },

    SetDefaultResourceRelativePath: function(__path)
    {
        this.set('_resourcePath', __path);
    },

    ReleaseRenderTarget: function(__renderTarget)
    {
        if(__renderTarget) __renderTarget.dispose();
    },

    ReleaseDummyTextures: function()
    {
        this.ReleaseRenderTarget(this.get('_DummyTexture'));
        this.ReleaseRenderTarget(this.get('_DummyTextureCube'));
    },

    ReleaseAllRenderTargets: function()
    {
        this.ReleaseRenderTarget(this.get('_OpaqueDepthRT'));
        this.ReleaseRenderTarget(this.get('_D1RT'));
        this.ReleaseRenderTarget(this.get('_D2RT'));
        this.ReleaseRenderTarget(this.get('_ColorStackRT'));
        this.ReleaseRenderTarget(this.get('_PeelColorRT'));
        this.ReleaseRenderTarget(this.get('_TransparencyStackRT'));
        this.ReleaseRenderTarget(this.get('_OpaqueBackgroundMaskRT'));
        this.ReleaseRenderTarget(this.get('_MirrorOpaqueDepthRT'));
        this.ReleaseRenderTarget(this.get('_MirrorColorStackRT'));
        this.ReleaseRenderTarget(this.get('_MirrorColorStackRT2'));
        this.ReleaseRenderTarget(this.get('_MirrorPeelColorRT'));
        this.ReleaseRenderTarget(this.get('_MirrorTransparencyStackRT'));
        this.ReleaseRenderTarget(this.get('_FadedMirrorTransparencyStackRT'));
        this.ReleaseRenderTarget(this.get('_MirrorOpaqueBackgroundMaskRT'));
    },

    InitializeRenderTargets: function(__overrideWidth, __overrideHeight)
    {
            //Initialize render targets for depth peeling
            var width = __overrideWidth?__overrideWidth:this.get('_context').clientWidth;
            var height = __overrideHeight?__overrideHeight:this.get('_context').clientHeight;
            // width = WV3DMath.isPowerOfTwo(width)?width:WV3DMath.nextHighestPowerOfTwo(width);
            // height = WV3DMath.isPowerOfTwo(height)?height:WV3DMath.nextHighestPowerOfTwo(height);
            var depthTargetsFormat, colorTargetsFormat;

            this.ReleaseAllRenderTargets();

            //Depth encoding in the depth shaders allows use of RGBA textures instead
            //of floating point textures to support many mobile devices (except Ipads
            //which are apparently not very good at anything... :/)
            if(WV3DSystemChecker.get('_fullSupportFloatingPointTextures'))
            {
                depthTargetsFormat =    {
                                            minFilter: THREE.LinearFilter,
                                            format: THREE.RGBAFormat,
                                            type: THREE.FloatType
                                        };
                colorTargetsFormat =    {
                                            minFilter: THREE.LinearFilter,
                                            format: THREE.RGBAFormat
                                        };

                this.set('_OpaqueDepthRT', new THREE.WebGLRenderTarget(width, height, depthTargetsFormat));
                this.set('_D1RT', new THREE.WebGLRenderTarget(width, height, depthTargetsFormat));
                this.set('_D2RT', new THREE.WebGLRenderTarget(width, height, depthTargetsFormat));
                this.set('_ColorStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_PeelColorRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_TransparencyStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_OpaqueBackgroundMaskRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorOpaqueDepthRT', new THREE.WebGLRenderTarget(width, height, depthTargetsFormat))
                this.set('_MirrorColorStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorColorStackRT2', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorPeelColorRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorTransparencyStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_FadedMirrorTransparencyStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorOpaqueBackgroundMaskRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
            }else if (WV3DSystemChecker.get('_fullSupportHFloatingPointTextures'))
            {
                depthTargetsFormat =    {
                                            minFilter: THREE.LinearFilter,
                                            magFilter: THREE.NearestFilter,
                                            format: THREE.RGBAFormat,
                                            type: THREE.HalfFloatType
                                        };
                colorTargetsFormat =    {
                                            minFilter: THREE.LinearFilter,
                                            magFilter: THREE.NearestFilter,
                                            format: THREE.RGBAFormat
                                        };

                this.set('_OpaqueDepthRT', new THREE.WebGLRenderTarget(width, height, depthTargetsFormat));
                this.set('_D1RT', new THREE.WebGLRenderTarget(width, height, depthTargetsFormat));
                this.set('_D2RT', new THREE.WebGLRenderTarget(width, height, depthTargetsFormat));
                this.set('_ColorStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_PeelColorRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_TransparencyStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_OpaqueBackgroundMaskRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorOpaqueDepthRT', new THREE.WebGLRenderTarget(width, height, depthTargetsFormat))
                this.set('_MirrorColorStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorColorStackRT2', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorPeelColorRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorTransparencyStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_FadedMirrorTransparencyStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_MirrorOpaqueBackgroundMaskRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
            }else{
                colorTargetsFormat =    {
                                            minFilter: THREE.LinearFilter,
                                            magFilter: THREE.NearestFilter,
                                            format: THREE.RGBAFormat
                                        };
                this.set('_ColorStackRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
                this.set('_PeelColorRT', new THREE.WebGLRenderTarget(width, height, colorTargetsFormat));
            }
    },

    ResetOutlineShaders: function()
    {
        //Set up outline shaders
        if(!this.get('_renderercache')["materials"]["colordepth"])
        {
            //- Uniform Color shader
            var colordepth_uniforms = THREE.UniformsUtils.clone(UniformColorShader.uniforms);
            var colordepth_material = new THREE.ShaderMaterial({
                "uniforms": colordepth_uniforms,
                "vertexShader": UniformColorShader.vertexShader,
                "fragmentShader": UniformColorShader.fragmentShader,
                "name": "ColorDepth",
                "side": THREE.DoubleSide
            });
            this.get('_renderercache')["materials"]["colordepth"] = colordepth_material;
        }

        if(!this.get('_renderercache')["materials"]["edge"])
        {
            //- Edge shader
            var color = this.get('_renderercache')['materials']['defaultcolor'];
            var edge_uniforms = THREE.UniformsUtils.clone(EdgeShader.uniforms);
            edge_uniforms["color"]["value"] = new THREE.Vector3( color[0] / 255, color[1] / 255, color[2] / 255);
            edge_uniforms["depth_texture"]["value"] = this.get("_DummyTexture");
            var edge_material = new THREE.ShaderMaterial({
                "uniforms": edge_uniforms,
                "vertexShader": EdgeShader.vertexShader,
                "fragmentShader": EdgeShader.fragmentShader,
                "name": "Edge",
                "wireframe":true
            });
            this.get('_renderercache')["materials"]["edge"] = edge_material;
        }
    },

    ResetShaders: function()
    {
        //Set up depth peeling shaders
        if(!this.get('_renderercache')["materials"]["D0ShaderMaterial"])
        {
            var D0_uniforms = THREE.UniformsUtils.clone(D0Shader.uniforms);
            var D0ShaderMaterial = new THREE.ShaderMaterial({
                "uniforms": D0_uniforms,
                "vertexShader": D0Shader.vertexShader,
                "fragmentShader": D0Shader.fragmentShader,
                "side":THREE.DoubleSide,
                "depthWrite": true,
                "transparent":false,
                "name": "D0ShaderMaterial"
            });
            this.get('_renderercache')["materials"]["D0ShaderMaterial"] = D0ShaderMaterial;
        }

        if(!this.get('_renderercache')["materials"]["D1ShaderMaterial"])
        {
            var D1_uniforms = THREE.UniformsUtils.clone(D1Shader.uniforms);
            D1_uniforms["D0RT"]["value"] = this.get("_DummyTexture");
            var D1ShaderMaterial = new THREE.ShaderMaterial({
                "uniforms": D1_uniforms,
                "vertexShader": D1Shader.vertexShader,
                "fragmentShader": D1Shader.fragmentShader,
                "side":THREE.DoubleSide,
                "depthWrite": true,
                "transparent":false,
                "name": "D1ShaderMaterial"
            });
            this.get('_renderercache')["materials"]["D1ShaderMaterial"] = D1ShaderMaterial;
        }

        if(!this.get('_renderercache')["materials"]["D2ShaderMaterial"])
        {
            var D2_uniforms = THREE.UniformsUtils.clone(D2Shader.uniforms);
            D2_uniforms["DX1RT"]["value"] = this.get("_DummyTexture");
            D2_uniforms["DX2RT"]["value"] = this.get("_DummyTexture");
            var D2ShaderMaterial = new THREE.ShaderMaterial({
                "uniforms": D2_uniforms,
                "vertexShader": D2Shader.vertexShader,
                "fragmentShader": D2Shader.fragmentShader,
                "side":THREE.DoubleSide,//starting from second peel, we want to draw all peels, even back facing ones
                "depthWrite": true,
                "transparent":false,
                "name": "D2ShaderMaterial"
            });
            this.get('_renderercache')["materials"]["D2ShaderMaterial"] = D2ShaderMaterial;
        }

        if(!this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"])
        {
            var depth_only_uniforms = THREE.UniformsUtils.clone( DepthOnlyShader.uniforms );
            depth_only_uniforms["DX1RT"]["value"] = this.get("_DummyTexture");
            depth_only_uniforms["DX2RT"]["value"] = this.get("_DummyTexture");
            var DepthOnlyShaderMaterial = new THREE.ShaderMaterial({
                "uniforms": depth_only_uniforms,
                "vertexShader": DepthOnlyShader.vertexShader,
                "fragmentShader": DepthOnlyShader.fragmentShader,
                "side": THREE.DoubleSide,
                "depthWrite": true,
                "colorWrite": false,
                "transparent":false,
                "name": "DepthOnlyShaderMaterial"
            });
            this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"] = DepthOnlyShaderMaterial;
        }

        if(!this.get('_renderercache')["materials"]["BackdropShaderMaterial"])
        {
            var Backdrop_uniforms = THREE.UniformsUtils.clone(BackdropShader.uniforms);
            Backdrop_uniforms["backdrop"]["value"] = this.get("_DummyTexture");
            Backdrop_uniforms["reflection"]["value"] = this.get("_DummyTexture");
            var BackdropShaderMaterial = new THREE.ShaderMaterial({
                "uniforms": Backdrop_uniforms,
                "vertexShader": BackdropShader.vertexShader,
                "fragmentShader": BackdropShader.fragmentShader,
                "side":THREE.FrontSide,
                "depthWrite": false,
                "transparent":false,
                "name": "BackdropShaderMaterial"
            });
            this.get('_renderercache')["materials"]["BackdropShaderMaterial"] = BackdropShaderMaterial;
        }

        if(!this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"])
        {
            var ColorBlend_uniforms = THREE.UniformsUtils.clone(ColorBlendShader.uniforms);
            ColorBlend_uniforms["CX1RT"]["value"] = this.get("_DummyTexture");
            ColorBlend_uniforms["TransparencyStack"]["value"] = this.get("_DummyTexture");
            ColorBlend_uniforms["OpaqueRT"]["value"] = this.get("_DummyTexture");
            ColorBlend_uniforms["ColorStackRT"]["value"] = this.get("_DummyTexture");
            var ColorBlendShaderMaterial = new THREE.ShaderMaterial({
                "uniforms": ColorBlend_uniforms,
                "vertexShader": ColorBlendShader.vertexShader,
                "fragmentShader": ColorBlendShader.fragmentShader,
                "side":THREE.FrontSide,
                "depthWrite": true,
                "transparent":true,
                "blending":THREE.AdditiveBlending,
                "name": "ColorBlendShaderMaterial"
            });
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"] = ColorBlendShaderMaterial;
        }

        if(!this.get('_renderercache')["materials"]["CopyShaderMaterial"])
        {
            var Copy_uniforms = THREE.UniformsUtils.clone(CopyShader.uniforms);
            Copy_uniforms["CXRT"]["value"] = this.get("_DummyTexture");
            Copy_uniforms["OpaqueRT"]["value"] = this.get("_DummyTexture");
            var CopyShaderMaterial = new THREE.ShaderMaterial({
                "uniforms": Copy_uniforms,
                "vertexShader": CopyShader.vertexShader,
                "fragmentShader": CopyShader.fragmentShader,
                "side":THREE.FrontSide,
                "depthWrite": true,
                "transparent":false,
                "blending":THREE.AdditiveBlending,
                "name": "CopyShaderMaterial"
             });
            this.get('_renderercache')["materials"]["CopyShaderMaterial"] = CopyShaderMaterial;
        }
        //Set up other shaders...
        this.ResetOutlineShaders();
    },

    ApplyFctToMaterialStruct: function(__mat_struct, __callback)
    {
        for (var n = 0, keys0 = Object.keys(__mat_struct["mat"]["untextured_mat"]), o = keys0.length; n < o; ++n)
        {
            var pass = __mat_struct["mat"]["untextured_mat"][keys0[n]];
            for (var m = 0, keys1 = Object.keys(pass), p = keys1.length; m < p; ++m)
            {
                var material = pass[keys1[m]];
                __callback(material);
                material.needsUpdate = true;
            }
        }
        for (var n = 0, keys0 = Object.keys(__mat_struct["mat"]["textured_mat"]), o = keys0.length; n < o; ++n)
        {
            var pass = __mat_struct["mat"]["textured_mat"][keys0[n]];
            for (var m = 0, keys1 = Object.keys(pass), p = keys1.length; m < p; ++m)
            {
                var meshID = pass[keys1[m]];
                for (var l = 0, keys2 = Object.keys(meshID), q = keys2.length; l < q; ++l)
                {
                    var material = meshID[keys2[l]];
                    __callback(material);
                    material.needsUpdate = true;
                }
            }
        }
    },

    ApplyFctToAllMaterials: function(__callback)
    {
        //Update and trigger rebuild of default material shader
        __callback(this.get('_renderercache')['materials']['default_material']);
        this.get('_renderercache')['materials']['default_material'].needsUpdate = true;

        // For the moment, we do not need to call any callback function on these materials
        // __callback(this.get('_renderercache')["materials"]["highlight"]);
        // __callback(this.get('_renderercache')["materials"]["D0ShaderMaterial"]);
        // __callback(this.get('_renderercache')["materials"]["D1ShaderMaterial"]);
        // __callback(this.get('_renderercache')["materials"]["D2ShaderMaterial"]);
        // __callback(this.get('_renderercache')["materials"]["BackdropShaderMaterial"]);
        // __callback(this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]);
        // __callback(this.get('_renderercache')["materials"]["CopyShaderMaterial"]);

        for (var n = 0, keys0 = Object.keys(this.get('_renderercache')['materials']['materiallist']), o = keys0.length; n < o; ++n)
        {
            var mat_structs = this.get('_renderercache')['materials']['materiallist'][keys0[n]];
            this.ApplyFctToMaterialStruct(mat_structs.front,__callback);
            this.ApplyFctToMaterialStruct(mat_structs.back,__callback);
        }
    },

    ReleaseMaterial: function(__material, __destroy_textures)
    {
        //Dispose related material and all related uniforms
        if(__material)
        {
            __material.dispose();
            if(__material.uniforms)
            {
                for (var mat_uni_idx = 0, keys = Object.keys(__material.uniforms), l = keys.length;
                   mat_uni_idx < l; ++mat_uni_idx)
                {
                    var uniform = __material.uniforms[keys[mat_uni_idx]];
                    if(uniform["value"])
                    {
                        //We want to dispose of any uniform that is not a texture
                        //and has a dispose function, in any case
                        //In addition, we want to release textures uniforms if
                        //__destroy_textures is true (or default)
                        if((typeof(uniform["value"]["dispose"]) === "function") &&
                            ((__destroy_textures === undefined) || __destroy_textures || !(uniform["value"] instanceof THREE.Texture)))
                        {
                            uniform["value"].dispose();
                        }
                    }
                }
            }
        }
    },

    ReleaseMaterialsAndTextures: function(__mat_struct)
    {
        if(__mat_struct)
        {
            var meshestoupdate = __mat_struct["boundmeshlist"];
            //Update material reference in meshes that are already bound to this material
            if(meshestoupdate)
            {
                if(Object.keys(meshestoupdate).length == 0) LogMessage.Log("ViewerView Warning: No meshes to update");
                for (var j = 0, keys1 = Object.keys(meshestoupdate), L = keys1.length; j < L; ++j)
                {
                    var renderpass = keys1[j];
                    if(Object.keys(meshestoupdate[renderpass]).length == 0) LogMessage.Log("ViewerView Warning: No render passes to update");
                    for (var i = 0, keys2 = Object.keys(meshestoupdate[renderpass]), l = keys2.length; i < l; ++i)
                    {
                        var side_mat = keys2[i];
                        for (var spass_cpt = 0, spass_keys = Object.keys(meshestoupdate[renderpass][side_mat]), l_spass = spass_keys.length;
                            spass_cpt < l_spass; spass_cpt++)
                        {
                            var spass_id = spass_keys[spass_cpt];
                            this.ReleaseMaterial(__mat_struct["mat"]["untextured_mat"][renderpass][spass_id]);
                            if(Object.keys(meshestoupdate[renderpass][side_mat][spass_id]).length == 0) LogMessage.Log("ViewerView Warning: No sided materials to update");
                            for (var k = 0, keys3 = Object.keys(meshestoupdate[renderpass][side_mat][spass_id]), m = keys3.length; k < m; ++k)
                        {
                            var surfID = keys3[k];
                            //Release material and textures
                                this.ReleaseMaterial(__mat_struct["mat"]["textured_mat"][renderpass][spass_id][surfID]);
                                delete __mat_struct["mat"]["textured_mat"][renderpass][spass_id][surfID];
                                delete meshestoupdate[renderpass][side_mat][spass_id][surfID];
                        }
                            delete meshestoupdate[renderpass][side_mat][spass_id];
                        }
                        delete meshestoupdate[renderpass][side_mat];
                        delete meshestoupdate[renderpass];
                    }
                }
                delete __mat_struct["boundmeshlist"];
            }else
            {
                LogMessage.Log("ViewerView Warning: No meshes to update");
            }


            if(__mat_struct["diff_texture"]) __mat_struct["diff_texture"].dispose();
            if(__mat_struct["spec_texture"]) __mat_struct["spec_texture"].dispose();
            if(__mat_struct["int_spec_texture"]) __mat_struct["int_spec_texture"].dispose();
            if(__mat_struct["normal_chan0_texture"]) __mat_struct["normal_chan0_texture"].dispose();
            if(__mat_struct["normal_chan1_texture"]) __mat_struct["normal_chan1_texture"].dispose();
            if(__mat_struct["diff_grating_texture"]) __mat_struct["diff_grating_texture"].dispose();
            if(__mat_struct["diff_grating_filter_texture"]) __mat_struct["diff_grating_filter_texture"].dispose();

        }
    },

    ReleaseAllMaterialsAndTextures: function()
    {
        if(this.get('_renderercache'))
        {
            this.ReleaseMaterial(this.get('_renderercache')['materials']['default_material']);
            this.ReleaseMaterial(this.get('_renderercache')["materials"]["highlight"]);
            this.ReleaseMaterial(this.get('_renderercache')["materials"]["edge"]);
            this.ReleaseMaterial(this.get('_renderercache')["materials"]["colordepth"]);
            this.ReleaseMaterial(this.get('_renderercache')["materials"]["D0ShaderMaterial"]);
            this.ReleaseMaterial(this.get('_renderercache')["materials"]["D1ShaderMaterial"]);
            this.ReleaseMaterial(this.get('_renderercache')["materials"]["D2ShaderMaterial"]);
            this.ReleaseMaterial(this.get('_renderercache')["materials"]["BackdropShaderMaterial"]);
            this.ReleaseMaterial(this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]);
            this.ReleaseMaterial(this.get('_renderercache')["materials"]["CopyShaderMaterial"]);

            for (var n = 0, keys0 = Object.keys(this.get('_renderercache')['materials']['materiallist']), o = keys0.length; n < o; ++n)
            {
                var mat_structs = this.get('_renderercache')['materials']['materiallist'][keys0[n]];
                this.ReleaseMaterialsAndTextures(mat_structs.front);
                this.ReleaseMaterialsAndTextures(mat_structs.back);
            }

            if(this.get('_renderercache')["textures"]["Designer"]["irradiancemap"]["data"])
                this.get('_renderercache')["textures"]["Designer"]["irradiancemap"]["data"].dispose();
            if(this.get('_renderercache')["textures"]["Designer"]["envmap"]["glossy"]["data"])
                this.get('_renderercache')["textures"]["Designer"]["envmap"]["glossy"]["data"].dispose();
            if(this.get('_renderercache')["textures"]["Designer"]["envmap"]["matte"]["data"])
                this.get('_renderercache')["textures"]["Designer"]["envmap"]["matte"]["data"].dispose();
            if(this.get('_renderercache')["textures"]["Visualizer"]["irradiancemap"]["data"])
                this.get('_renderercache')["textures"]["Visualizer"]["irradiancemap"]["data"].dispose();
            if(this.get('_renderercache')["textures"]["Visualizer"]["envmap"]["data"])
                for (var envmap_id = 0; envmap_id < this.get('_renderercache')["textures"]["Visualizer"]["envmap"]["data"].length; envmap_id ++)
                    if( this.get('_renderercache')["textures"]["Visualizer"]["envmap"]["data"][envmap_id] )
                        this.get('_renderercache')["textures"]["Visualizer"]["envmap"]["data"][envmap_id].dispose();
            if(this.get('_renderercache')["textures"]["Visualizer"]["spectrum"]["data"])
                this.get('_renderercache')["textures"]["Visualizer"]["spectrum"]["data"].dispose();

        }
    },

    ReleaseGeomMatTex: function(__scenegraph)
    {
        if(__scenegraph)
        {
            var scope = this;
            var releaseGeometryAndMaterialCallback = function(__obj)
            {
                if(__obj instanceof THREE.Mesh && __obj.geometry)
                {
                   __obj.geometry.dispose();
                   scope.ReleaseMaterial(__obj.material);
                }
            }
            __scenegraph.traverse(releaseGeometryAndMaterialCallback);
        }
    },

    ReleaseSGGeometriesAndMaterials: function()
    {
        this.ApplyFctToAllScenes(this.ReleaseGeomMatTex.bind(this));
    },

    ReleaseThreejsnodesReferences: function()
    {
        if(this.get('_renderercache') && this.get('_renderercache').hasOwnProperty('nodeviewproperties'))
        {
            var nodeviewprops = this.get('_renderercache').nodeviewproperties;
            for (var node_url_id = 0, keys = Object.keys(nodeviewprops), l = keys.length; node_url_id < l; node_url_id++)
            {
                var threejsnodes = nodeviewprops[keys[node_url_id]].threejsnodes;
                for (var node_id = 0, keys2 = Object.keys(threejsnodes), l2 = keys2.length; node_id < l2; node_id++)
                {
                    //Delete nodes in all coincident groups
                    for (var node_cg = 0, keys3 = threejsnodes[keys2[node_id]], l3 = keys3.length; node_cg < l3; node_cg++)
                    {
                        delete threejsnodes[keys2[node_id]][keys3[node_cg]];
                    }
                    delete threejsnodes[keys2[node_id]];
                }
                delete nodeviewprops[keys[node_url_id]].threejsnodes;
                delete nodeviewprops[keys[node_url_id]];
            }
        }
    },

    ClearInstanceArrays: function()
    {
        var clear_temp_instanciation_array = function ( obj )
        {
            if(obj instanceof THREE.Mesh)
            {
                if(obj.geometry instanceof THREE.InstancedBufferGeometry)
                {
                    if(obj.geometry.attributes["inst_translation"].jarray)
                        obj.geometry.attributes["inst_translation"].jarray = null;
                    if(obj.geometry.attributes["inst_rotation"].jarray)
                        obj.geometry.attributes["inst_rotation"].jarray = null;
                    if(obj.geometry.attributes["inst_scale"].jarray)
                        obj.geometry.attributes["inst_scale"].jarray = null;
                }
            }
        };

        this.TraverseAllScenes(clear_temp_instanciation_array);
    },

    ApplyFctToAllScenes: function(__callback)
    {
        __callback(this.get('_3JSsceneOp'));
        __callback(this.get('_3JSsceneOpDepth'));
        __callback(this.get('_3JSsceneOpaqueDiffuse'));
        if(this.get('_3JSsceneOpCoincidentGroupsPasses'))
        {
            for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
            {
                var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
                __callback(this.get('_3JSsceneOpCoincidentGroups')[cg_id]);
                __callback(this.get('_3JSsceneOpCoincidentGroupsDepth')[cg_id]);
                __callback(this.get('_3JSsceneOpCoincidentGroupsDiffuse')[cg_id]);
            }
        }
        __callback(this.get('_3JSsceneTransp'));
        __callback(this.get('_3JSsceneTranspDepth'));
        __callback(this.get('_3JSsceneTranspDiffuse'));
        if(this.get('_3JSsceneTranspCoincidentGroupsPasses'))
        {
            for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
            {
                var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
                __callback(this.get('_3JSsceneTranspCoincidentGroups')[cg_id]);
                __callback(this.get('_3JSsceneTranspCoincidentGroupsDepth')[cg_id]);
                __callback(this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id]);
            }
        }
        __callback(this.get('_3JSsceneHightlights'));
        __callback(this.get('_3JSsceneLabels'));
    },

    TraverseAllScenes: function(__callback)
    {
        this.get('_3JSsceneOp').traverse(__callback);
        this.get('_3JSsceneOpDepth').traverse(__callback);
        this.get('_3JSsceneOpaqueDiffuse').traverse(__callback);
        if(this.get('_3JSsceneOpCoincidentGroupsPasses'))
        {
            for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
            {
                var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
                this.get('_3JSsceneOpCoincidentGroups')[cg_id].traverse(__callback);
                this.get('_3JSsceneOpCoincidentGroupsDepth')[cg_id].traverse(__callback);
                this.get('_3JSsceneOpCoincidentGroupsDiffuse')[cg_id].traverse(__callback);
            }
        }
        this.get('_3JSsceneTransp').traverse(__callback);
        this.get('_3JSsceneTranspDepth').traverse(__callback);
        this.get('_3JSsceneTranspDiffuse').traverse(__callback);
        if(this.get('_3JSsceneTranspCoincidentGroupsPasses'))
        {
            for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
            {
                var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
                this.get('_3JSsceneTranspCoincidentGroups')[cg_id].traverse(__callback);
                this.get('_3JSsceneTranspCoincidentGroupsDepth')[cg_id].traverse(__callback);
                this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id].traverse(__callback);
            }
        }
        this.get('_3JSsceneHightlights').traverse(__callback);
        this.get('_3JSsceneLabels').traverse(__callback);
    },

    ReleaseScene: function()
    {
        // Texture cache cleared temporarily here since not allowing
        // texture to be properly linked to the material (see use of loadTexture
        // and linkedmaterial in ViewerView.js). Callback is being called before
        // texture can be linked to material when texture already present in cache
        // However, the browser is already caching images, then no overhead can be detected.
        this.ReleaseAllMaterialsAndTextures();
        this.ReleaseSGGeometriesAndMaterials();
        this.ReleaseThreejsnodesReferences();
        THREE.Cache.clear();

        //Reset all attributes
        this.set('_3JSsceneOp', null);
        this.set('_3JSsceneOpDepth', null);
        this.set('_3JSsceneOpaqueDiffuse', null);
        this.set('_3JSsceneTransp', null);
        this.set('_3JSsceneTranspDepth', null);
        this.set('_3JSsceneTranspDiffuse', null);
        this.set('_3JSsceneHightlights', null);
        this.set('_3JSsceneLabels', null);

        this.set('_3JSsceneBBox', null);
        this.set('_3JSsceneBBoxFocus', null);

        //Node and geomtry counters
        this.set('_nodecounter',0);
        this.set('_geometrycounter',0);
        this.set("_labelCreationCallbackList",[])

        this.set('_renderercache',null);

        this.set("_emptyTransparentScene", true);
        this.set("_emptyOpaqueScene", true);

        this.set('_3JSsceneOpCoincidentGroupsIds', {});
        this.set('_3JSsceneOpCoincidentGroupsPasses', []);
        this.set('_3JSsceneNbOpCoincidentGroups',0);

        this.set('_3JSsceneTranspCoincidentGroupsIds', {});
        this.set('_3JSsceneTranspCoincidentGroupsPasses', []);
        this.set('_3JSsceneNbTranspCoincidentGroups',0);
    },

    /**
     * TRIGGERED EVENT (model::'scene_reset'): Reset the scene and creates a new one
     */
    ResetScene: function(__coplanarityEpsilon, __boundaryhardedges)
    {
        this.ReleaseScene();

        //Create the scene objects using the asset manager
        this.set('_3JSsceneOp', new THREE.Scene());
        this.set('_3JSsceneOpCoincidentGroups', {});
        this.set('_3JSsceneOpDepth', new THREE.Scene());
        this.set('_3JSsceneOpCoincidentGroupsDepth', []);
        this.set('_3JSsceneOpaqueDiffuse', new THREE.Scene());
        this.set('_3JSsceneOpCoincidentGroupsDiffuse', []);
        this.set('_3JSsceneTransp', new THREE.Scene());
        this.set('_3JSsceneTranspCoincidentGroups', {});
        this.set('_3JSsceneTranspDepth', new THREE.Scene());
        this.set('_3JSsceneTranspCoincidentGroupsDepth', []);
        this.set('_3JSsceneTranspDiffuse', new THREE.Scene());
        this.set('_3JSsceneTranspCoincidentGroupsDiffuse', []);
        this.set('_3JSsceneHightlights', new THREE.Scene());
        this.set('_3JSsceneLabels', new THREE.Scene());

        this.set('_3JSsceneBBox', new THREE.Box3());
        this.set('_3JSsceneBBoxFocus', new THREE.Box3());
        // this.get('_3JSscene').add(scene_bbox);    //Display scene bbox for debug only

        // var axisHelper = new THREE.AxisHelper( 200 );
        // this.get('_3JSsceneOp').add( axisHelper );

        this.set('_renderercache',
        {
            "nodeviewproperties"://in use for 3JS nodes URL mapping
            {

            },
            "coplanarityepsilon":(__coplanarityEpsilon !== undefined)?__coplanarityEpsilon:0.2,//in use
            "boundaryhardedges":(__boundaryhardedges !== undefined)?__boundaryhardedges:true,
            "geometries": [],//in use for three JS geometries
            "materials"://in use for three JS material (colored faces)
            {
                "enablehighlights":false,
                "enablehorizonhighlights":true,
                "enablehardedgehighlights":true,
                "enableirradiancemap":true,
                "enableenvmap":true,
                "defaultcolor":[30, 30, 30],
                "default_material":{},
                "materiallist":{},//The map containing all materials for imported materials
                "shaders":{}
            },
            "labels"://label assets loaded from JSON
            {
                "visible": true,
                "list": {}
            },
            "textures"://in use for three JS textures
            {
                //Textures used in Designer Quality
                "Designer":
                {
                    "path": "lighting_environments/designer_quality/",
                    "irradiancemap":
                    {
                        "files": [  "irr_map_right.PNG",//+X
                                    "irr_map_left.PNG",//-X
                                    "irr_map_top.PNG",//+Y
                                    "irr_map_bottom.PNG",//-Y
                                    "irr_map_front.PNG",//+Z
                                    "irr_map_back.PNG"//-Z
                                    ],
                        "data": null,
                        "loaded": false
                    },
                    "envmap":
                    {
                        "glossy":
                        {
                            "files": [  "env_map_glossy_right.PNG",//+X
                                        "env_map_glossy_left.PNG",//-X
                                        "env_map_glossy_top.PNG",//+Y
                                        "env_map_glossy_bottom.PNG",//-Y
                                        "env_map_glossy_front.PNG",//+Z
                                        "env_map_glossy_back.PNG"//-Z
                                        ],
                            "data": null,
                            "loaded": false
                        },
                        "matte":
                        {
                            "files": [  "env_map_mate_right.PNG",//+X
                                        "env_map_mate_left.PNG",//-X
                                        "env_map_mate_top.PNG",//+Y
                                        "env_map_mate_bottom.PNG",//-Y
                                        "env_map_mate_front.PNG",//+Z
                                        "env_map_mate_back.PNG"//-Z
                                        ],
                            "data": null,
                            "loaded": false
                        }
                    },
                    "backdrop":
                    {
                        "loaded" : false,
                        "data": null,
                        "original_width": null,
                        "original_height": null
                    }
                },
                //Textures used in Visualizer Quality
                "Visualizer":
                {
                    "path": "lighting_environments/visualizer_quality/",
                    "irradiancemap":
                    {
                        "files": [  "cubic_irradiance_xpos.jpg",//+X
                                    "cubic_irradiance_xneg.jpg",//-X
                                    "cubic_irradiance_ypos.jpg",//+Y
                                    "cubic_irradiance_yneg.jpg",//-Y
                                    "cubic_irradiance_zpos.jpg",//+Z
                                    "cubic_irradiance_zneg.jpg"//-Z
                                    ],
                        "data": null,
                        "loaded": false
                    },
                    "envmap":
                    {
                        "files": [  "cubic_blur_00_xpos.jpg",//+X
                                    "cubic_blur_00_xneg.jpg",//-X
                                    "cubic_blur_00_ypos.jpg",//+Y
                                    "cubic_blur_00_yneg.jpg",//-Y
                                    "cubic_blur_00_zpos.jpg",//+Z
                                    "cubic_blur_00_zneg.jpg",//-Z
                                    "cubic_blur_02_xpos.jpg",//+X
                                    "cubic_blur_02_xneg.jpg",//-X
                                    "cubic_blur_02_ypos.jpg",//+Y
                                    "cubic_blur_02_yneg.jpg",//-Y
                                    "cubic_blur_02_zpos.jpg",//+Z
                                    "cubic_blur_02_zneg.jpg",//-Z
                                    "cubic_blur_04_xpos.jpg",//+X
                                    "cubic_blur_04_xneg.jpg",//-X
                                    "cubic_blur_04_ypos.jpg",//+Y
                                    "cubic_blur_04_yneg.jpg",//-Y
                                    "cubic_blur_04_zpos.jpg",//+Z
                                    "cubic_blur_04_zneg.jpg",//-Z
                                    "cubic_blur_06_xpos.jpg",//+X
                                    "cubic_blur_06_xneg.jpg",//-X
                                    "cubic_blur_06_ypos.jpg",//+Y
                                    "cubic_blur_06_yneg.jpg",//-Y
                                    "cubic_blur_06_zpos.jpg",//+Z
                                    "cubic_blur_06_zneg.jpg",//-Z
                                    "cubic_blur_08_xpos.jpg",//+X
                                    "cubic_blur_08_xneg.jpg",//-X
                                    "cubic_blur_08_ypos.jpg",//+Y
                                    "cubic_blur_08_yneg.jpg",//-Y
                                    "cubic_blur_08_zpos.jpg",//+Z
                                    "cubic_blur_08_zneg.jpg"//-Z
                                    ],
                        "data": null,
                        "loaded": false
                    },
                    "spectrum":
                    {
                        "file": "spectrum_00.png",
                        "data": null,
                        "loaded": false,
                        "queried": false
                    },
                    "backdrop":
                    {
                        "loaded" : false,
                        "data": null,
                        "original_width": null,
                        "original_height": null
                    }
                },
                //Texture loaded from Json
                "FromFile":{}
            },
            "cameras"://in use
            {
                "defaultcamera"://default camera in use
                {
                    "cam_type": "PERSPECTIVE",
                    "azimuthAngle": 0,
                    "elevationAngle": 0,
                    "fieldOfView": 30,
                    "scale": 1,
                    "panOffset": {"x":0,"y":0}
                }
            },
            "view_settings":
            {
                "mode":1,//0: floating mode (not implemented), 1: azim_elev mode (default)
                "model_xform": new THREE.Matrix4()
            },
            "camera_preset":
            {
                "eye_position": null,
            },
            "environment_settings":
            {
                "rotation": new THREE.Quaternion()
            },
            "lights":{},//in use for ambiant and directional lights
            "reflections":
            {
                "enabled": false,//reflects what is in json structure
                "opacity": 0.25,
                "fade":false,
                "fade_size": 1.2,  // Optional.
                "fade_position": 0.95  //Optional.
            },
            "hudelements":[],//not in use
            "diagrams":[],//not in use
            "viewports":[],//not in use
            "animationkeyframes":[]//not in use
        });

        var designer_uniforms = THREE.UniformsUtils.clone(DesignerShader.uniforms);
        designer_uniforms["diffuse"]["value"] = new THREE.Vector3(1.,1.,1.);
        designer_uniforms["specular"]["value"] = new THREE.Vector3(0.,0.,0.);

        //This is the single and only pass in designer quality. i.e. one scene construction
        var enableHardEdgeHighlights = this.get('_renderercache')['materials']['enablehardedgehighlights'];
        var enablePolygonOffset = this.get('_renderercache')['materials']['enablehighlights'] && __enableHardEdgeHighlights;
        this.get('_renderercache')['materials']['default_material'] = new THREE.ShaderMaterial({
            "name":"designer",
            "uniforms": designer_uniforms,
            "vertexShader": DesignerShader.vertexShader,
            "fragmentShader": DesignerShader.fragmentShader,
            "depthTest":true,
            "vertexColors": THREE.FaceColors,
            "side":THREE.FrontSide,
            "transparent":false,
            "defines":{},
            "lights":true,
            "polygonOffset":true,
            "polygonOffsetFactor": enablePolygonOffset ? this.get( '_polygon_offset_factor') : 0,
            "polygonOffsetUnits": enablePolygonOffset ? this.get( '_polygon_offset_units') : 0
        })

        //Highlights hard edges default material
        var default_color = this.get('_renderercache')['materials']['defaultcolor'];
        var highlightColor = (default_color[0] << 16) + (default_color[1] << 8) + default_color[2];
        var line_material = new THREE.LineBasicMaterial({
            "color": highlightColor,
            "linewidth": 1,
            "depthTest": true
        });
        this.get('_renderercache')["materials"]["highlight"] = line_material;
        var black_line_material = new THREE.LineBasicMaterial({
            "color": 0x000000,
            "linewidth": 1,
            "depthTest": true
        });
        this.get('_renderercache')["materials"]["blackhighlight"] = black_line_material;

        this.AddDirectionalAndAmbientLightsToScene(this.get('_3JSsceneOp'), "cg_0", false);
        this.AddDirectionalAndAmbientLightsToScene(this.get('_3JSsceneTransp'), "cg_0", true);

        this.set('_InitialisedSceneSettings', false);
        this.set('_QueriedGlobalTextures', false);
        this.set('_TextureSizeWarningMessageDisplayed', false);
        this.set('_TextureSizeByPass', false);
        this.set('_SceneLoaded', false);
        this.set('_parsedEntireSceneGraphOnce', false);
    },

    AddDirectionalAndAmbientLightsToScene: function(__scene, __coincident_group_id, __transparent)
    {
        // add a directional light
        if(!this.get('_renderercache')['lights']['dir_light']) this.get('_renderercache')['lights']['dir_light'] = {};
        var directionalLight = new THREE.DirectionalLight( 0xaaaaaa, 0.5 );//this light will be updated afterwards when updating lighting contrast in UpdateLightingContrast
        this.get('_renderercache')['lights']['dir_light'][(__transparent?"tr_":"")+__coincident_group_id] = directionalLight;
        __scene.add(directionalLight);

        // add subtle ambient lighting
        if(!this.get('_renderercache')['lights']['amb_light']) this.get('_renderercache')['lights']['amb_light'] = {};
        var ambientLight = new THREE.AmbientLight(0x888888);//this light will be updated afterwards when updating lighting contrast in UpdateLightingContrast
        this.get('_renderercache')['lights']['amb_light'][(__transparent?"tr_":"")+__coincident_group_id] = ambientLight;
        __scene.add(ambientLight);
    },

    InitOpaqueCoincidentGroupScene: function(__coincident_group_id)
    {
        if (!this.get('_3JSsceneOpCoincidentGroups').hasOwnProperty(__coincident_group_id))
        {
            this.get('_3JSsceneOpCoincidentGroups')[__coincident_group_id] = new THREE.Scene();
            this.AddDirectionalAndAmbientLightsToScene(this.get('_3JSsceneOpCoincidentGroups')[__coincident_group_id], __coincident_group_id, false)
            this.get('_3JSsceneOpCoincidentGroupsDepth')[__coincident_group_id] = new THREE.Scene();
            this.get('_3JSsceneOpCoincidentGroupsDiffuse')[__coincident_group_id] = new THREE.Scene();
        }
    },

    InitTransparentCoincidentGroupScene: function(__coincident_group_id)
    {
        if (!this.get('_3JSsceneTranspCoincidentGroups').hasOwnProperty(__coincident_group_id))
        {
            this.get('_3JSsceneTranspCoincidentGroups')[__coincident_group_id] = new THREE.Scene();
            this.AddDirectionalAndAmbientLightsToScene(this.get('_3JSsceneTranspCoincidentGroups')[__coincident_group_id], __coincident_group_id, true)
            this.get('_3JSsceneTranspCoincidentGroupsDepth')[__coincident_group_id] = new THREE.Scene();
            this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[__coincident_group_id] = new THREE.Scene();
        }
    },

    CreateLabels: function()
    {
        for (var i = 0; i < this.get("_labelCreationCallbackList").length; ++i)
        {
            this.get("_labelCreationCallbackList")[i]();
        }

        //Create ThreeJS sprites for Label assets from JSON
        this.CreateSprites();
    },

    PostSceneLoadActions: function()
    {
        this.set('_SceneLoaded', true);
        this.get('_3DObjectHandler').ClearBuildMaps();
        this.ClearInstanceArrays();
        this.CreateLabels();

        this.CameraUpdate(true);
    },

    CheckPostSceneLoadActions: function()
    {
        var nb_qd_asset = this.get('_model').get('_assetmanager').get('_assetloader').get('_httprequestmanager').get('_queriedassetnumber');
        var nb_res_asset = this.get('_model').get('_assetmanager').get('_resolvedassetnumber');
        var nb_qd_tex = this.get('_model').get('_assetmanager').get('_textureLoadedCounter');
        var nb_res_tex = this.get('_model').get('_assetmanager').get('_textureCounter');
        var percent_tex_loaded = nb_res_tex / nb_qd_tex * 100;
        if( (nb_qd_asset == nb_res_asset) && (nb_qd_tex == nb_res_tex))
        {
            this.PostSceneLoadActions();
        }
        return percent_tex_loaded;
    },

    /**
     * TRIGGERED EVENT (model::'added_scene_resource'): Handle changes to the model
     */
    AddedResource: function()
    {
        this.ResetShaders();
        this.SetUpScene();

        var scope = this;
        this.get("_build_scene_exec_queue").whenFinished(function()
        {
            scope.UpdateReflectionUniforms();

            if(scope.get('_model').get('_assetmanager').get('_textureCounter') == 0)//If no texture to load, then emit a default 100% conplete signal
            {
                if(scope.get('_model').get('_assetmanager').get('_assetloader').get('_httprequestmanager').get('_queriedassetnumber') == scope.get('_model').get('_assetmanager').get('_resolvedassetnumber'))
                {
                    this.PostSceneLoadActions();
                }
            }
            scope.Render();
            scope.NotifySceneBuildProgress(100,0,0);
        });
    },

    GetFileList: function(__path, __files)
    {
        var files = [];
        for (var i = 0, keys = Object.keys(__files), l = keys.length; i < l; ++i)
        {
            var file_idx = keys[i];
            var filename = __path+__files[file_idx];
            files.push(filename);
        }
        return files
    },

    CheckTextureSizeByPass: function(__texture)
    {
        var confirmed = true;
        if ( __texture["image"]["naturalHeight"] > WV3DConstants.textureSupport.MAX_NATURAL_HEIGHT
            || __texture["image"]["naturalWidth"] > WV3DConstants.textureSupport.MAX_NATURAL_WIDTH)
        {
            if(!this.get('_TextureSizeWarningMessageDisplayed'))
            {
                confirmed = window.confirm("WARNING ! Some textures have a size greater than "+WV3DConstants.textureSupport.MAX_NATURAL_WIDTH+"x"+WV3DConstants.textureSupport.MAX_NATURAL_HEIGHT+". This might not be supported by your system and you might experience browser or graphics card crashes.");
                this.set('_TextureSizeWarningMessageDisplayed', true);
                this.set('_TextureSizeByPass', confirmed);
            }else
            {
                confirmed = this.get('_TextureSizeByPass');//Keep user choice for large textures
            }
        }
        return confirmed;
    },

    /**
     * Start the asynchronous loading of a texture, if required
     */
    StartAsyncTextureLoad: function(__scope, __assets, __matprops, __prop_tex_name, __mat_tex_name, __material_struct, __path, __format, __unpack_alignment)
    {
        if(__matprops["front"].hasOwnProperty(__prop_tex_name) &&  __matprops["front"][__prop_tex_name] &&
             __assets.hasOwnProperty(__matprops["front"][__prop_tex_name] ))
        {
                var texture_node = __assets[__matprops["front"][__prop_tex_name]];
                var remappedAlphaChannel = texture_node["Properties"].hasOwnProperty("remapped_alpha_channel") ? WV3DUtils.get('toBool')(texture_node["Properties"]["remapped_alpha_channel"]) : false;
                //We try to load the textures, if there is a ref to it
                var onerror = function(__event)
                {
                    //Otherwise we let the geometries attached to the untextured material
                    LogMessage.Log("ViewerView Warning: Problem loading texture");
                }
                var onload_tex = function(__texture)
                {
                    __scope.get("_build_scene_exec_queue").whenFinished(function()
                    {
                        var confirmed = __scope.CheckTextureSizeByPass(__texture);
                        __scope.get('_model').get('_assetmanager').set('_textureLoadedCounter', __scope.get('_model').get('_assetmanager').get('_textureLoadedCounter') + 1);
                        var percent_tex_loaded = __scope.CheckPostSceneLoadActions();
                        //If the texture has been successfully loaded we attach the geometries to the textured material
                        if(confirmed && __material_struct)
                        {
                            var scale_x, scale_y; scale_x = scale_y = 1.;
                            if(__material_struct.hasOwnProperty("originalTexWidth"))
                            {
                                scale_x = (__material_struct["originalTexWidth"]-1) / __texture["image"]["naturalWidth"];
                            }
                            if(__material_struct.hasOwnProperty("originalTexHeight"))
                            {
                                scale_y = (__material_struct["originalTexHeight"]-1) / __texture["image"]["naturalHeight"];
                            }
                            __texture.repeat.set( scale_x, scale_y);
                            __texture["remappedAlphaChannel"] = remappedAlphaChannel;
                            __material_struct[__mat_tex_name] = __texture;
                            __scope.UpdateTexturedMaterial(__material_struct);
                            __scope.Render();
                        }
                        __scope.NotifySceneBuildProgress(100, percent_tex_loaded, confirmed?0:1);
                    });
                }
                var texpath;
                if(this.get("_model").get("_assetmanager").get("_assetloader").get('_httprequestmanager').IsAbsoluteURL(texture_node["Properties"]["filepath"]))
                {
                    texpath = texture_node["Properties"]["filepath"];
                }
                else
                {
                    texpath = __path+texture_node["Properties"]["filepath"];
                }

                var loader = new THREE.TextureLoader();
				//allow cross origin loading
				loader.crossOrigin = '';

                var texture = loader.load( texpath, function(__texture){
                    onload_tex(__texture);
                }, undefined, onerror);
                texture.name = texpath;
                texture.format = __format;
                texture.unpackAlignment = __unpack_alignment;
                texture.anisotropy = 16;
                // __texture.minFilter = THREE.NearestFilter;
                // __texture.generateMipmaps = true;
                __scope.get('_model').get('_assetmanager').set('_textureCounter', __scope.get('_model').get('_assetmanager').get('_textureCounter') + 1);
                __scope.SelectTextureWrappingMode(texture, texture_node["Properties"], __material_struct);
                //Add reference to this texture url in the cache
                if(!this.get('_renderercache')["textures"]["FromFile"].hasOwnProperty(texpath))
                {
                    this.get('_renderercache')["textures"]["FromFile"][texpath]=null;
                }

            return true;
        }else
        {
            return false;
        }
    },

    StartAsyncGlobalTextureLoad: function(__scope, __type, __path)
    {
        if(__scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type])
        {
                var onload_global_tex = function(__texture)
                {
                    __scope.get('_model').get('_assetmanager').set('_textureLoadedCounter', __scope.get('_model').get('_assetmanager').get('_textureLoadedCounter') + 1);
                    var percent_tex_loaded = __scope.CheckPostSceneLoadActions();
                    __scope.Render();
                    var confirmed = __scope.CheckTextureSizeByPass(__texture);
                    __scope.NotifySceneBuildProgress(100, percent_tex_loaded, confirmed?0:1);
                }
                var onload_standard_tex = function(__texture)
                {
                    __scope.get("_build_scene_exec_queue").whenFinished(function()
                    {
                        __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["loaded"]=true;
                        __scope.UpdateMaterialsGlobalLighting();
                        onload_global_tex(__texture);
                    });
                }
                var onload_env_map_glos_tex = function(__texture)
                {
                    __scope.get("_build_scene_exec_queue").whenFinished(function()
                    {
                        __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')]["envmap"]["glossy"]["loaded"] = true;
                        __scope.UpdateMaterialsGlobalLighting();
                        onload_global_tex(__texture);
                    });
                }
                var onload_env_map_matte_tex = function(__texture)
                {
                    __scope.get("_build_scene_exec_queue").whenFinished(function()
                    {
                        __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')]["envmap"]["matte"]["loaded"] = true;
                        __scope.UpdateMaterialsGlobalLighting();
                        onload_global_tex(__texture);
                    });
                }
                var onload_backdrop_tex = function(__texture)
                {
                    __scope.get("_build_scene_exec_queue").whenFinished(function()
                    {
                        __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["loaded"]=true;
                        onload_global_tex(__texture);
                    });
                }
                var onerror = function(__event)
                {
                    //Otherwise we let the geometries attached to the untextured material
                    LogMessage.Log("ViewerView Warning: Problem loading texture");
                }

                switch(__type)
                {
                    default:
                        LogMessage.Log("ViewerView Warning: Unhandled global texture type: default to irradiancemap");

                    case "irradiancemap":
                        var loader = new THREE.CubeTextureLoader();
                        var files = __scope.GetFileList(__path, __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["files"]);
                        var texture = loader.load( files, onload_standard_tex, undefined, onerror);
                        texture.format = THREE.RGBFormat;
                        __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["data"] = texture;
                        __scope.get('_model').get('_assetmanager').set('_textureCounter', __scope.get('_model').get('_assetmanager').get('_textureCounter') + 1);
                        break;

                    case "envmap":
                        var loader = new THREE.CubeTextureLoader();
                        switch(__scope.get('_RenderingQuality'))
                        {
                            case "Designer":
                                //Load glossy files and create cube texture
                                var files_glossy = __scope.GetFileList(__path, __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["glossy"]["files"]);
                                var glossy_texture = loader.load( files_glossy, onload_env_map_glos_tex, undefined, onerror);
                                glossy_texture.format = THREE.RGBFormat;
                                __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["glossy"]["data"] = glossy_texture;

                                //Load matte files and create cube texture
                                var files_matte = __scope.GetFileList(__path, __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["matte"]["files"]);
                                var matte_texture = loader.load( files_matte, onload_env_map_matte_tex, undefined, onerror);
                                matte_texture.format = THREE.RGBFormat;
                                __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["matte"]["data"] = matte_texture;

                                __scope.get('_model').get('_assetmanager').set('_textureCounter', __scope.get('_model').get('_assetmanager').get('_textureCounter') + 2);
                                break;

                            case "Visualizer":
                                //Here five cube textures haze levels are to be loaded
                                __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["data"] = {};
                                __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["loaded"] = {};

                                var loadIFile = function(__scope, __i)
                                {
                                    var onload_env_map_haze_i_tex = function(__texture)
                                    {
                                        __scope.get("_build_scene_exec_queue").whenFinished(function()
                                        {
                                            __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["loaded"][__i] = true;
                                            __scope.UpdateMaterialsGlobalLighting();
                                            onload_global_tex(__texture);
                                        });
                                    }
                                    var files = __scope.GetFileList(__path, __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["files"].slice(6*__i,6*(__i+1)));
                                    var texture = loader.load( files, onload_env_map_haze_i_tex, undefined, onerror);
                                    texture.format = THREE.RGBFormat;
                                    __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["data"][__i] = texture;
                                }

                                for (var i = 0; i < 5; ++i)
                                {
                                    loadIFile(__scope, i);
                                }

                                __scope.get('_model').get('_assetmanager').set('_textureCounter', __scope.get('_model').get('_assetmanager').get('_textureCounter') + 5);
                                break;
                        }
                        break;

                    case "spectrum":
                        var loader = new THREE.TextureLoader();
                        var texpath = __path + __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["file"];
                        var texture = loader.load( texpath, onload_standard_tex, undefined, onerror);
                        texture.format = THREE.RGBFormat;
                        texture.name = texpath;
                        __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["data"] = texture;
                        __scope.get('_model').get('_assetmanager').set('_textureCounter', __scope.get('_model').get('_assetmanager').get('_textureCounter') + 1);
                        break;

                    case "backdrop":
                        var loader = new THREE.TextureLoader();
                        //allow cross origin loading
                        loader.crossOrigin = '';
                        var texture = loader.load( __path, onload_backdrop_tex, undefined, onerror);
                        texture.format = THREE.RGBFormat;
                        texture.name = __path;
                        __scope.get('_renderercache')["textures"][__scope.get('_RenderingQuality')][__type]["data"] = texture;
                        __scope.get('_model').get('_assetmanager').set('_textureCounter', __scope.get('_model').get('_assetmanager').get('_textureCounter') + 1);
                        break;
                }
        }
    },

    InitializeSceneSettings: function(__root_node)
    {
        if(!this.get('_InitialisedSceneSettings'))
        {
            //Default values for rendering quality and environment
            this.set('_RenderingQuality',"Designer");
            this.set('_EnvironmentName',"Office");
            if(__root_node['Properties'])
            {
                //Set environment settings
                if(__root_node['Properties'].hasOwnProperty('rendering_quality') &&
                (WV3DSystemChecker.get('_fullSupportFloatingPointTextures') ||
                    WV3DSystemChecker.get('_fullSupportHFloatingPointTextures')))
                {
                    this.set('_RenderingQuality',__root_node['Properties']['rendering_quality']);
                    if(__root_node['Properties'].hasOwnProperty('environment_name'))
                    {
                        this.set('_EnvironmentName',__root_node['Properties']['environment_name']);
                        this.get('_renderercache')["textures"]["Visualizer"]["path"] = this.get('_renderercache')["textures"]["Visualizer"]["path"] + __root_node['Properties']['environment_name'] + "/";
                    }
                }
                // //Load Camera preset
                // if(__root_node['Properties'].hasOwnProperty('view_settings'))
                // {
                //     var view_settings = __root_node['Properties']["view_settings"];
                //     this.get("_renderercache")["view_settings"]["mode"] = parseInt(view_settings["view_mode"]);
                //     switch(this.get("_renderercache")["view_settings"]["mode"])
                //     {
                //         default:
                //             LogMessage.Log("View mode not implemented yet. Camera view settings will not be loaded.");
                //             break;

                //         // case 0:
                //         // ...
                //         // break;

                //         case 1:
                //             var camera_preset = __root_node["Assets"][view_settings["camera_asset_ref"]];
                //             if(camera_preset["AssetType"] !== "Camera")
                //             {
                //                 LogMessage.Log("Referenced camera asset not having proper type. Are you sure this camera asset id is uniquely named.");
                //             }
                //             else
                //             {

                //                 //ROTATION
                //                 //Rotation extraction inspired from http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToEuler/index.htm
                //                 /*  [m00 m01 m02]
                //                     [m10 m11 m12]
                //                     [m20 m21 m22]   */
                //                 var camera_props = camera_preset["Properties"];
                //                 var orientation_matrix = camera_props["azim_elev_cam_xform"]["orientation_matrix"];
                //                 var m = {
                //                             "m00" : parseFloat(orientation_matrix[0]),
                //                             "m01" : parseFloat(orientation_matrix[1]),
                //                             "m02" : parseFloat(orientation_matrix[2]),
                //                             "m10" : parseFloat(orientation_matrix[3]),
                //                             "m11" : parseFloat(orientation_matrix[4]),
                //                             "m12" : parseFloat(orientation_matrix[5]),
                //                             "m20" : parseFloat(orientation_matrix[6]),
                //                             "m21" : parseFloat(orientation_matrix[7]),
                //                             "m22" : parseFloat(orientation_matrix[8])
                //                         };

                //                 var camera_transform = new THREE.Matrix4();
                //                 camera_transform.set(
                //                                         parseFloat(orientation_matrix[0]),
                //                                         parseFloat(orientation_matrix[1]),
                //                                         parseFloat(orientation_matrix[2]),
                //                                         parseFloat(model_settings["x"]),
                //                                         parseFloat(orientation_matrix[3]),
                //                                         parseFloat(orientation_matrix[4]),
                //                                         parseFloat(orientation_matrix[5]),
                //                                         parseFloat(model_settings["y"]),
                //                                         parseFloat(orientation_matrix[6]),
                //                                         parseFloat(orientation_matrix[7]),
                //                                         parseFloat(orientation_matrix[8]),
                //                                         parseFloat(model_settings["z"]),
                //                                         0,0,0,1
                //                                     );

                //                 var heading, attitude;
                //                 if (m.m10 > 0.998) { // singularity at north pole
                //                     heading = Math.atan2(m.m02,m.m22);
                //                     attitude = Math.PI/2;
                //                     // bank = 0;//unused
                //                     return;
                //                 }
                //                 if (m.m10 < -0.998) { // singularity at south pole
                //                     heading = Math.atan2(m.m02,m.m22);
                //                     attitude = -Math.PI/2;
                //                     // bank = 0;//unused
                //                     return;
                //                 }
                //                 heading = Math.atan2(-m.m20,m.m00);
                //                 //bank = Math.atan2(-m.m12,m.m11);//unused
                //                 attitude = Math.asin(m.m10);
                //                 this.SetCameraAngles(WV3DMath.degrees(-heading - Math.PI/2.),WV3DMath.degrees(-attitude), false);

                //                 //PANNING
                //                 //Store the eye position to determine panning and scale when scene has finished loading
                //                 var eye_pos = new THREE.Vector3(
                //                                     parseFloat(camera_props["eye_pos"]["x"]),
                //                                     parseFloat(camera_props["eye_pos"]["y"]),
                //                                     parseFloat(camera_props["eye_pos"]["z"])
                //                                 );
                //                 //Use this one added to eye position ?
                //                 var azim_elev_pos = new THREE.Vector3(
                //                                     parseFloat(camera_props["azim_elev_cam_xform"]["x"]),
                //                                     parseFloat(camera_props["azim_elev_cam_xform"]["y"]),
                //                                     parseFloat(camera_props["azim_elev_cam_xform"]["z"])
                //                                 );

                //                 this.get("_renderercache")["camera_preset"]["eye_position"] = eye_pos;//.add(azim_elev_pos);

                //                 //FIELD OF VIEW
                //                 var vfov = parseFloat(camera_props["vfov"]);
                //                 this.SetPerspectiveCamera(vfov,false);

                //                 // var dir_of_sight = camera_props["dir_of_sight"];not needed ? if so, can be exported from collada
                //             }
                //         break;
                //     }
                // }
                // //Load Model Settings
                // if(__root_node['Properties'].hasOwnProperty('model_settings'))
                // {
                //     var model_settings;
                //     switch(this.get("_renderercache")["view_settings"]["mode"])
                //     {
                //         default:
                //             LogMessage.Log("View mode not implemented yet. Model view settings will not be loaded.");
                //         break;
                //         // case 0:
                //             // model_settings = __root_node['Properties']["model_settings"]["floating_model_xform"];
                //             // ...
                //         // break;
                //         case 1:
                //             model_settings = __root_node['Properties']["model_settings"]["azim_elev_model_xform"];
                //             var orientation_matrix = model_settings["orientation_matrix"];
                //             var model_transform = new THREE.Matrix4();
                //             model_transform.set(
                //                                     parseFloat(orientation_matrix[0]),
                //                                     parseFloat(orientation_matrix[1]),
                //                                     parseFloat(orientation_matrix[2]),
                //                                     parseFloat(model_settings["x"]),
                //                                     parseFloat(orientation_matrix[3]),
                //                                     parseFloat(orientation_matrix[4]),
                //                                     parseFloat(orientation_matrix[5]),
                //                                     parseFloat(model_settings["y"]),
                //                                     parseFloat(orientation_matrix[6]),
                //                                     parseFloat(orientation_matrix[7]),
                //                                     parseFloat(orientation_matrix[8]),
                //                                     parseFloat(model_settings["z"]),
                //                                     0,0,0,1
                //                                 );
                //             this.get("_renderercache")["view_settings"]["model_xform"] = model_transform;
                //         break;
                //     }
                // }

                //Set reflection settings
                if(__root_node['Properties'].hasOwnProperty('reflection_settings'))
                {
                    this.get('_renderercache')["reflections"]["enabled"] = __root_node['Properties']["reflection_settings"].hasOwnProperty("reflective_table_enabled")?  WV3DUtils.get('toBool')(__root_node['Properties']["reflection_settings"]["reflective_table_enabled"]) : false;
                    if(this.get('_renderercache')["reflections"]["enabled"])
                    {
                        this.get('_renderercache')["reflections"]["opacity"] = __root_node['Properties']["reflection_settings"].hasOwnProperty("reflection_opacity")?  parseFloat(__root_node['Properties']["reflection_settings"]["reflection_opacity"]) : 0.25;

                        if(__root_node['Properties']["reflection_settings"].hasOwnProperty("reflection_fade_size") &&
                           __root_node['Properties']["reflection_settings"].hasOwnProperty("reflection_fade_position"))
                        {
                            this.get('_renderercache')["reflections"]["fade"] = true;
                            this.get('_renderercache')["reflections"]["fade_size"] = parseFloat(__root_node['Properties']["reflection_settings"]["reflection_fade_size"]);  // Optional.
                            this.get('_renderercache')["reflections"]["fade_position"] = parseFloat(__root_node['Properties']["reflection_settings"]["reflection_fade_position"])  //Optional.
                        }
                    }
                }
            }
            // Check backdrop, if any, otherwise sets default background color
            this.LoadBackdrop( __root_node );

            this.set('_InitialisedSceneSettings', true);
        }
    },

    /*This function allows to update scene settings after scene has been loaded*/
    UpdateSceneSettings: function()
    {
        if(this.get("_renderercache")["camera_preset"]["eye_position"])
        {
            //Make sure that camera has already been set up according to the scene bounding box
            this.CameraUpdate(false);

            //Use camera preset position to calculate proper panning and scaling offset
            //this approach is inspired from http://stackoverflow.com/questions/5227373/minimal-perpendicular-vector-between-a-point-and-a-line

            //Get the bounding box center
            var scenecenter = this.get('_3JSsceneBBox').center();
            //Get the current view vector
            var current_campos = this.get('_3JScamera').position;
            var pres_campos = this.get("_renderercache")["camera_preset"]["eye_position"].clone();

            var viewvec = scenecenter.clone().sub(current_campos);
            var view_dir_offset = viewvec.clone().multiplyScalar(pres_campos.clone().sub(current_campos).dot(viewvec));

            var upvec = this.get('_3JScamera').up;
            var rightvec = viewvec.clone().normalize().cross(upvec);
            var right_dir_offset = rightvec.clone().multiplyScalar(pres_campos.clone().sub(current_campos).dot(rightvec));
            var up_dir_offset = upvec.clone().multiplyScalar(pres_campos.clone().sub(current_campos).dot(upvec));

            this.SetCameraPanOffset({"x":right_dir_offset.length(), "y":up_dir_offset.length()}, false);

            var prev_length = viewvec.length();
            var new_length = viewvec.clone().sub(view_dir_offset).length();
            this.SetCameraScale(new_length / prev_length);
        }
    },

    LoadGlobalLightingTextures: function()
    {
        if(!this.get('_QueriedGlobalTextures'))
        {
            var self = this;
            var path = this.get('_resourcePath')+"/"+this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["path"];

            if(this.get('_renderercache')["materials"]["enableirradiancemap"])
                this.StartAsyncGlobalTextureLoad(self, "irradiancemap", path);

            if(this.get('_renderercache')["materials"]["enableenvmap"])
                this.StartAsyncGlobalTextureLoad(self, "envmap", path);

            this.set('_QueriedGlobalTextures', true);
        }
    },

    /**
     * Check backdrop, if any, otherwise sets default background color
     */
    LoadBackdrop: function( __root_node)
    {
        if(__root_node['Properties'] && __root_node['Properties'].hasOwnProperty('backdrop'))
        {
            if (__root_node['Properties']['backdrop']['type'] === "COLOR")
            {
                var bdColor = __root_node['Properties']['backdrop']['color'];
                this.SetBackgroundColor( bdColor[0], bdColor[1], bdColor[2], 1., false);
            }
            else if (__root_node['Properties']['backdrop']['type'] === "IMAGE")
            {
                var self = this;
                var texIndex = __root_node['Properties']['backdrop']['image_tex'];
                var texProps = __root_node['Assets'][texIndex]["Properties"];

                var path;
                if(this.get("_model").get("_assetmanager").get("_assetloader").get('_httprequestmanager').IsAbsoluteURL(texProps["filepath"]))
                {
                    path = texProps["filepath"];
                }
                else
                {
                    var current_scene_id = this.get('_model').get('SceneId');
                    path = current_scene_id.substring(0, current_scene_id.lastIndexOf("/")+1)+texProps["filepath"];
                }
                this.StartAsyncGlobalTextureLoad(self, "backdrop", path);

                this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["backdrop"]["original_height"] = texProps["original_height"];
                this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["backdrop"]["original_width"] = texProps["original_width"];
                this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["backdrop"]["enabled"] = true;
            }
            else
            {
                LogMessage.Log("ViewerView Warning: Backdrop type not supported (default to grey).");
                this.SetBackgroundColor(0.37,0.37,0.37,1., false); // default background of plugin
            }
        }
        else
        {
             this.SetBackgroundColor(0.37,0.37,0.37,1., false); // default background of plugin
        }
    },

    /**
     * Set up the scene from the asset manager
     */
    SetUpScene: function()
    {
        //Create 3JS Node hierarchy
        WV3DTimers.start("Scene Hierarchy Creation");
        this.CreateTHREEJSObjectHierarchy();
        WV3DTimers.stop("Scene Hierarchy Creation");

        var scope = this;
        this.get("_build_scene_exec_queue").whenFinished(function()
        {
            var bboxop = new THREE.Box3();
            bboxop.setFromObject(scope.get('_3JSsceneOp'));
            for (var cg = 0; cg < scope.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
            {
                var bboxopcg = new THREE.Box3();
                var cg_id = scope.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
                bboxopcg.setFromObject(scope.get('_3JSsceneOpCoincidentGroups')[cg_id]);
                bboxop.union(bboxopcg);
            }
            var bboxtr = new THREE.Box3();
            bboxtr.setFromObject(scope.get('_3JSsceneTransp'));
            bboxop.union(bboxtr);
            for (var cg = 0; cg < scope.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
            {
                var bboxopcg = new THREE.Box3();
                var cg_id = scope.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
                bboxopcg.setFromObject(scope.get('_3JSsceneTranspCoincidentGroups')[cg_id]);
                bboxop.union(bboxopcg);
            }

            scope.set('_3JSsceneBBox', bboxop)
            scope.set('_3JSsceneBBoxFocus', bboxop);

            // var bboxhelper = new THREE.BoundingBoxHelper( scope.get('_3JSsceneOp'), 0xffffff);
            // bboxhelper.update();
            // scope.get('_3JSsceneOp').add(bboxhelper);

            //Prepare reflections (rely on bounding box)
            scope.SetupReflections();

            //Ajust camera to scene (rely on bounding box and reflections)
            scope.CameraUpdate(false);
            //The camera position needs to be set before computing the distance to scene center
            scope.set('_InitialDistanceToSceneCenter',scope.get('_3JSsceneBBox').center().clone().sub(scope.get('_3JScamera').position).length());
        });
    },

    CreateSprites: function()
    {
        for (var labelIndex = 0, labels_list = Object.keys(this.get('_renderercache')['labels']['list']), n_labels = labels_list.length; labelIndex < n_labels; ++labelIndex)
        {
            var label_asset = this.get('_renderercache')['labels']['list'][labels_list[labelIndex]];
            var sprite = this.get('_textSpriteMaker').MakeTextSprite(label_asset['text'], label_asset['display_style'],
                                                                    new THREE.Vector3(0, 0, 0), false, false, false, false, false, false, label_asset['orientation']);

            // add to renderlayer
            this.get('_3JSsceneLabels').add( sprite );
            // add backreference for easy position updates
            label_asset['threejs_sprite'] = sprite;
        }
    },

    SetupReflections: function()
    {
        //Epsilon is needed to offset the reflection, otherwise it can be coincident with the object's bottom
        var Y_axis_symmetry_transform_matrix = new THREE.Matrix4();
        Y_axis_symmetry_transform_matrix.set(1,0,0,0,
                                             0,-1,0,0,
                                             0,0,1,0,
                                             0,0,0,1);
        this.set("_reflection_Y_scale", Y_axis_symmetry_transform_matrix);
        this.set("_mirror_offset", 0.01);
        var reflection_Y_trans = this.get('_3JSsceneBBox').min.y * 2 - this.get('_mirror_offset');
        var Y_axis_translation_transform_matrix = new THREE.Matrix4();
        Y_axis_translation_transform_matrix.makeTranslation(0,reflection_Y_trans,0);
        this.set("_reflection_Y_translation", Y_axis_translation_transform_matrix);
        var Y_axis_opp_translation_transform_matrix = new THREE.Matrix4();
        Y_axis_opp_translation_transform_matrix.makeTranslation(0,-reflection_Y_trans,0);
        this.set("_reflection_Y_opp_translation", Y_axis_opp_translation_transform_matrix);
    },

    /**
     * Add a ui element using client callback and forwarding all parameters
     */
    AddUIElement: function(args)
    {
        if(typeof this.get('_clientUIAddElementCallback') === "function")
        {
            this.get('_clientUIAddElementCallback').apply(null, arguments);
        }else
        {
            LogMessage.Log("ViewerView Warning: Client callback not a function");
        }
    },

    /**
     * Creates the root node and start the recursive traversal
     * of the json structure to build the 3JS scene graph
     */
    CreateTHREEJSObjectHierarchy: function()
    {
        if(!this.get("_3DObjectHandler"))
        {
            this.set("_3DObjectHandler",new WV3DObjectHandler(this.get("_build_scene_exec_queue")));
        }
        this.get("_3DObjectHandler").set('_geometrycounter',0);

        //We add a default empty callback to 3DobjectHandlerExecutionQueue to avoid the whenFinished callbacks
        //to be triggered as soon as callback is added to the execution queue (LoadGlobalLightingTextures,
        //AsyncTextureLoad, SetUpScene, etc.)
        this.get("_build_scene_exec_queue").run(function(){},this);

        //Initialize scene quality, backdrop and reflection properties
        this.get("_build_scene_exec_queue").run(function()
        {
            var cache = this.get('_model').get('_assetmanager').get('_cache');
            var root_node_unique_id = this.get('_model').get('SceneId');
            var root_node = cache['rootnodes'][root_node_unique_id];
            this.InitializeSceneSettings(root_node);
        }, this);

        //Load environment and irradiance maps
        this.get("_build_scene_exec_queue").run(function()
        {
            this.LoadGlobalLightingTextures();
        }, this);

        this.get("_build_scene_exec_queue").run(function(){LogMessage.Log("Building Surfaces...")},this);
        this.get("_build_scene_exec_queue").whenFinished(function(){LogMessage.Log("Finished Building Surfaces.")},this);

        this.get("_build_scene_exec_queue").run(function()
        {
            var cache = this.get('_model').get('_assetmanager').get('_cache');
            var root_node_unique_id = this.get('_model').get('SceneId');
            var root_node = cache['rootnodes'][root_node_unique_id];
            //BuildOpaqueScenes is calling this.CreateNewCoincidentGroupPass every time a coincident group is found
            this.BuildOpaqueScenes(root_node, root_node_unique_id);
        }, this);

        //Build an opaque scene for each coincident group, with its own shading passes
        this.get("_build_scene_exec_queue").run(function()
        {
            var cache = this.get('_model').get('_assetmanager').get('_cache');
            var root_node_unique_id = this.get('_model').get('SceneId');
            var root_node = cache['rootnodes'][root_node_unique_id];
            for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
            {
                var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
                this.BuildOpaqueScenes(root_node, root_node_unique_id, cg_id);
            }
        }, this)

        this.get("_build_scene_exec_queue").run(function()
        {
            if(!this.get("_emptyTransparentScene"))
            {
                var cache = this.get('_model').get('_assetmanager').get('_cache');
                var root_node_unique_id = this.get('_model').get('SceneId');
                var root_node = cache['rootnodes'][root_node_unique_id];

                //Build transparent scene for each shading pass once
                this.BuildTransparentScenes(root_node, root_node_unique_id);

                //Build a transparent scene for each transparent coincident group, with its own shading passes
                for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
                {
                    var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
                    this.BuildTransparentScenes(root_node, root_node_unique_id, cg_id);
                }
            }
        }, this)

        this.get("_build_scene_exec_queue").run(function()
        {
            this.UpdateLightingContrast();
        }, this);

        this.get("_build_scene_exec_queue").run(function()
        {
            var cache = this.get('_model').get('_assetmanager').get('_cache');
            var root_node_unique_id = this.get('_model').get('SceneId');
            var root_node = cache['rootnodes'][root_node_unique_id];

            //Build scene for hard edge highlights rendering
            var highlightchild = this.GetTHREEJSObjectHierarchy(root_node, "highlightnode", root_node_unique_id, root_node_unique_id);
            if(highlightchild.newlycreated)
                this.get('_3JSsceneHightlights').add(highlightchild.node);

            this.set('_geometrycounter',this.get('_geometrycounter')+this.get('_3DObjectHandler').get('_geometrycounter'));
            LogMessage.Log(this.get('_nodecounter')+" nodes have been created for this scene");
            LogMessage.Log(this.get('_geometrycounter')+" geometries have been created for this scene");
        }, this)
    },

    /**
     * Private helper function (to reuse code) to copy a subset of label properties from one object to the other.
     * only copies text, orientation, preferred_loc_h, preferred_loc_v
     */
    CopyRestOfLabelProperties: function(__toAsset, __fromAsset)
    {
        if(__fromAsset.hasOwnProperty('text') && __fromAsset.text)
            __toAsset["text"] = __fromAsset["text"];

        if(__fromAsset.hasOwnProperty('orientation') && __fromAsset.orientation)
            __toAsset["orientation"] = __fromAsset["orientation"];

        if(__fromAsset.hasOwnProperty('preferred_loc_h') && __fromAsset.preferred_loc_h)
            __toAsset["preferred_loc_h"] = __fromAsset["preferred_loc_h"];

        if(__fromAsset.hasOwnProperty('preferred_loc_v') && __fromAsset.preferred_loc_v)
            __toAsset["preferred_loc_v"] = __fromAsset["preferred_loc_v"];

        if(__fromAsset.hasOwnProperty('pixel_offset') && __fromAsset.pixel_offset)
            __toAsset["pixel_offset"] = __fromAsset["pixel_offset"];
    },

    //! Private helper function (to reuse code)
    SetLabelProperties: function(__label_asset, __properties)
    {
        if(__properties.hasOwnProperty('display_style') && __properties.display_style)
        {
            var displayStyleProps = __properties["display_style"];

            if(displayStyleProps.hasOwnProperty('fontface') && displayStyleProps.fontface)
                __label_asset["display_style"].fontface = displayStyleProps.fontface;

            if(displayStyleProps.hasOwnProperty('fontsize') && displayStyleProps.fontsize)
                __label_asset["display_style"].fontsize = displayStyleProps.fontsize;

            if(displayStyleProps.hasOwnProperty('borderThickness') && displayStyleProps.borderThickness)
                __label_asset["display_style"].borderThickness = displayStyleProps.borderThickness;

            if(displayStyleProps.hasOwnProperty('borderColor') && displayStyleProps.borderColor)
                __label_asset["display_style"].borderColor = displayStyleProps.borderColor;

            if(displayStyleProps.hasOwnProperty('backgroundColor') && displayStyleProps.backgroundColor)
                __label_asset["display_style"].backgroundColor = displayStyleProps.backgroundColor;

            if(displayStyleProps.hasOwnProperty('color') && displayStyleProps.color)
                __label_asset["display_style"].color = displayStyleProps.color;
        }

        this.CopyRestOfLabelProperties(__label_asset, __properties);

        var prevVisibility = __label_asset['threejs_sprite'].visible;

        this.get('_3JSsceneLabels').remove( __label_asset['threejs_sprite'] );

        var sprite = this.get('_textSpriteMaker').MakeTextSprite(__label_asset['text'], __label_asset['display_style'],
                                                                new THREE.Vector3(0, 0, 0), false, false, false, false, false, false, __label_asset['orientation']);

        this.get('_3JSsceneLabels').add( sprite );

        __label_asset['threejs_sprite'] = sprite;

        if(__properties.hasOwnProperty('visible'))
            __label_asset['threejs_sprite'].visible = WV3DUtils.get('toBool')(__properties.visible);
        else
            __label_asset['threejs_sprite'].visible = prevVisibility;
    },

    UpdateLabelProperties: function(__nodeURL, __properties)
    {
        var shortNodeURL = this.URLToKeyString(__nodeURL);

        if(!(shortNodeURL in this.get('_renderercache')['labels']['list']))
            return false; //Invalid URL

        var label_asset = this.get('_renderercache')['labels']['list'][shortNodeURL];
        if(label_asset === undefined)
            throw "UpdateLabelProperties: Cannot find requested label";

        this.SetLabelProperties(label_asset, __properties);

        this.CameraUpdate(true);
        return true;
    },

    UpdateAllLabelsProperties: function(__properties)
    {
        var lbl = 0;
        for (var lbl_nodes = Object.keys(this.get('_renderercache')['labels']['list']), n_lbls = lbl_nodes.length; lbl < n_lbls; ++lbl)
        {
            var label_asset = this.get('_renderercache')['labels']['list'][lbl_nodes[lbl]];
            if(label_asset === undefined)
                throw "UpdateAllLabelsProperties: Cannot find intended label!";

            this.SetLabelProperties(label_asset, __properties);
        }

        if(lbl > 0)
            this.CameraUpdate(true);

        return lbl > 0;
    },

    //! Deduces edge_list type bounds for given bounding box type. Allowed values for bbox type: x, y, z, xyz
    DeduceLabelEdgesFromBoundingBox: function(__bounding_box_type, __label_asset, __bbox)
    {
        if(__bounding_box_type == "x_axis" || __bounding_box_type == "xyz_axes")
        {
            //Four edges for x axis
            __label_asset["edge_list"][0] = [__bbox.min.x, __bbox.min.y, __bbox.min.z, __bbox.max.x, __bbox.min.y, __bbox.min.z];
            __label_asset["edge_list"][1] = [__bbox.min.x, __bbox.min.y, __bbox.max.z, __bbox.max.x, __bbox.min.y, __bbox.max.z];
            __label_asset["edge_list"][2] = [__bbox.min.x, __bbox.max.y, __bbox.max.z, __bbox.max.x, __bbox.max.y, __bbox.max.z];
            __label_asset["edge_list"][3] = [__bbox.min.x, __bbox.max.y, __bbox.min.z, __bbox.max.x, __bbox.max.y, __bbox.min.z];
        }

        if(__bounding_box_type == "y_axis" || __bounding_box_type == "xyz_axes")
        {
            var idx = 0;
            if(__bounding_box_type == "xyz_axes")
                idx = 4
            //Four edges for y axis
            __label_asset["edge_list"][idx] = [__bbox.max.x, __bbox.min.y, __bbox.max.z, __bbox.max.x, __bbox.min.y, __bbox.min.z];
            __label_asset["edge_list"][idx+1] = [__bbox.max.x, __bbox.max.y, __bbox.max.z, __bbox.max.x, __bbox.max.y, __bbox.min.z];
            __label_asset["edge_list"][idx+2] = [__bbox.min.x, __bbox.max.y, __bbox.max.z, __bbox.min.x, __bbox.max.y, __bbox.min.z];
            __label_asset["edge_list"][idx+3] = [__bbox.min.x, __bbox.min.y, __bbox.max.z, __bbox.min.x, __bbox.min.y, __bbox.min.z];
        }

        if(__bounding_box_type == "z_axis" || __bounding_box_type == "xyz_axes")
        {
            var idx = 0;
            if(__bounding_box_type == "xyz_axes")
                idx = 8
            //Four edges for z axis
            __label_asset["edge_list"][idx] = [__bbox.min.x, __bbox.min.y, __bbox.max.z, __bbox.min.x, __bbox.max.y, __bbox.max.z];
            __label_asset["edge_list"][idx+1] = [__bbox.max.x, __bbox.min.y, __bbox.max.z, __bbox.max.x, __bbox.max.y, __bbox.max.z];
            __label_asset["edge_list"][idx+2] = [__bbox.max.x, __bbox.min.y, __bbox.min.z, __bbox.max.x, __bbox.max.y, __bbox.min.z];
            __label_asset["edge_list"][idx+3] = [__bbox.min.x, __bbox.min.y, __bbox.min.z, __bbox.min.x, __bbox.max.y, __bbox.min.z];
        }

    },

    /**
    * Load 'Label' assets from JSON
    */
    LoadLabel: function(__root_node, __node_path, __label_asset, __label_name, __label_settings_map, __transmat, __parent_threejsnode)
    {
        if(__label_asset.AssetType != "Label" || !__label_asset.hasOwnProperty('Properties') || __label_asset.Properties == null)
            return;

        var label_obj_props = __label_asset.Properties;

        var label_asset =
        {
            "text":label_obj_props["text"],
            "orientation":label_obj_props["orientation"],
            "preferred_loc_h":label_obj_props["preferred_loc_h"],
            "preferred_loc_v":label_obj_props["preferred_loc_v"],
            "edge_list":[],
            "point_list":[],
            "ellipse":
            {
                "center":null,
                "radius1_vec":null,
                "radius2_vec":null
            },
            "bounds_type":null,
            "threejs_sprite":null,    //This is set later in CreateSprites
            "display_style":null,
            "pixel_offset":null
        };

        if(label_obj_props.hasOwnProperty('display_style') && label_obj_props.display_style)
            label_asset["display_style"] = __root_node['Properties'].label_display_styles[label_obj_props["display_style"]];
        else if(__root_node['Properties'].hasOwnProperty('label_display_styles') && __root_node['Properties'].label_display_styles)
            label_asset["display_style"] = __root_node['Properties'].label_display_styles["default_style"];

        if(label_obj_props.hasOwnProperty("pixel_offset") && label_obj_props.pixel_offset)
            label_asset["pixel_offset"] = label_obj_props["pixel_offset"];
        else
            label_asset["pixel_offset"] = [0, 0];

        if(label_obj_props.hasOwnProperty('edge_list') && label_obj_props.edge_list)
        {
            label_asset["bounds_type"] = "edge_list";
            for (var ed = 0, ed_keys = Object.keys(label_obj_props.edge_list), n_eds = ed_keys.length; ed < n_eds; ++ed)
            {
                // Clone (deep copy) the coordinates array. Below is the simplest method of all when not using complex objects
                label_asset["edge_list"][ed] = JSON.parse(JSON.stringify(label_obj_props.edge_list[ed_keys[ed]]));
                __transmat.applyToVector3Array(label_asset["edge_list"][ed]);
            }
        }
        else if(label_obj_props.hasOwnProperty('point_list') && label_obj_props.point_list)
        {
            label_asset["bounds_type"] = "point_list";
            for (var pt = 0, pt_keys = Object.keys(label_obj_props.point_list), n_pts = pt_keys.length; pt < n_pts; ++pt)
            {
                // Clone (deep copy) the coordinates array. Below is the simplest method of all when not using complex objects
                label_asset["point_list"][pt] = JSON.parse(JSON.stringify(label_obj_props.point_list[pt_keys[pt]]));
                __transmat.applyToVector3Array(label_asset["point_list"][pt]);
            }
        }
        else if(label_obj_props.hasOwnProperty('ellipse') && label_obj_props.ellipse)
        {
            label_asset["bounds_type"] = "ellipse";
            // Clone (deep copy) the coordinates array. Below is the simplest method of all when not using complex objects
            label_asset["ellipse"]["center"] = JSON.parse(JSON.stringify(label_obj_props.ellipse.center));
            label_asset["ellipse"]["radius1_vec"] = JSON.parse(JSON.stringify(label_obj_props.ellipse.radius1_vec));
            label_asset["ellipse"]["radius2_vec"] = JSON.parse(JSON.stringify(label_obj_props.ellipse.radius2_vec));
            __transmat.applyToVector3Array(label_asset["ellipse"]["center"]);

            var transmat3 = new THREE.Matrix3();
            transmat3.setFromMatrix4(__transmat);

            transmat3.applyToVector3Array(label_asset["ellipse"]["radius1_vec"]);
            transmat3.applyToVector3Array(label_asset["ellipse"]["radius2_vec"]);
        }
        else if(label_obj_props.hasOwnProperty('bounding_box') && label_obj_props.bounding_box)
        {
            //! We convert bounding_box into a set of edges
            // This process is being defered due to lack of information from the scene
            // Bounding box is processed after the entire geometry has been parsed.
            label_asset["bounds_type"] = "edge_list";
            var bounding_box = new THREE.Box3();
            bounding_box.setFromObject(__parent_threejsnode.node);
            this.DeduceLabelEdgesFromBoundingBox(label_obj_props["bounding_box"], label_asset, bounding_box);
        }
        else
        {
            LogMessage.Log("Invalid bounds specification in Label asset: " + __label_name);
            return;
        }

        if(__label_settings_map instanceof Object && Object.keys(__label_settings_map).length != 0 && __label_settings_map.hasOwnProperty(__label_name))
        {
            var asset_override_props = __label_settings_map[__label_name];
            if(!asset_override_props)
                return; // Visibility turned off for this label at a particular node. Do not add this into [labels][list]

            if(asset_override_props.hasOwnProperty('display_style') && asset_override_props.display_style)
                label_asset["display_style"] = __root_node['Properties'].label_display_styles[asset_override_props["display_style"]];

            this.CopyRestOfLabelProperties(label_asset, asset_override_props);
        }

        var node_path = this.URLToKeyString(__node_path);
        this.get('_renderercache')['labels']['list'][node_path] = label_asset;
        this.AddUIElement(node_path, true); //Trigger application callback with label nodel url

        LogMessage.Log("Built Label asset: " + __label_name);
    },

    BuildOpaqueScenes: function(__root_node, __root_node_unique_id, __coincident_group_id)
    {
        // Build scene for each requested opaque pass
        var pass_scene = this.GetTHREEJSObjectHierarchy(__root_node, "opaque_pass", __root_node_unique_id, __root_node_unique_id, __coincident_group_id);
        //If there is at least one opaque pass create an opaque depth pass
        var depth_pass_scene = this.GetTHREEJSObjectHierarchy(__root_node, "depth_pass", __root_node_unique_id, __root_node_unique_id, __coincident_group_id);
        //If there is at least one opaque pass create a transparent depth pass
        var opaque_diffuse_pass_scene = this.GetTHREEJSObjectHierarchy(__root_node, "opaque_diffuse_pass", __root_node_unique_id, __root_node_unique_id, __coincident_group_id);
        if(__coincident_group_id === undefined)
        {
            if(pass_scene.newlycreated)
            {
                pass_scene.node.applyMatrix(this.get("_renderercache")["view_settings"]["model_xform"])
                this.get('_3JSsceneOp').add(pass_scene.node);
            }
            if(depth_pass_scene.newlycreated)
            {
                depth_pass_scene.node.applyMatrix(this.get("_renderercache")["view_settings"]["model_xform"])
                this.get('_3JSsceneOpDepth').add(depth_pass_scene.node);
            }
            if(opaque_diffuse_pass_scene.newlycreated)
            {
                opaque_diffuse_pass_scene.node.applyMatrix(this.get("_renderercache")["view_settings"]["model_xform"])
                this.get('_3JSsceneOpaqueDiffuse').add(opaque_diffuse_pass_scene.node);
            }
        }
        else //Create one scene hierarchy per coincident group
        {
            this.InitOpaqueCoincidentGroupScene(__coincident_group_id);
            if(pass_scene.newlycreated)
            {
                pass_scene.node.applyMatrix(this.get("_renderercache")["view_settings"]["model_xform"])
                this.get('_3JSsceneOpCoincidentGroups')[__coincident_group_id].add(pass_scene.node);
            }
            if(depth_pass_scene.newlycreated)
            {
                depth_pass_scene.node.applyMatrix(this.get("_renderercache")["view_settings"]["model_xform"])
                this.get('_3JSsceneOpCoincidentGroupsDepth')[__coincident_group_id].add(depth_pass_scene.node);
            }
            if(opaque_diffuse_pass_scene.newlycreated)
            {
                opaque_diffuse_pass_scene.node.applyMatrix(this.get("_renderercache")["view_settings"]["model_xform"])
                this.get('_3JSsceneOpCoincidentGroupsDiffuse')[__coincident_group_id].add(opaque_diffuse_pass_scene.node);
            }
        }
    },

    BuildTransparentScenes: function(__root_node, __root_node_unique_id, __coincident_group_id)
    {
        // Build scene for each requested transparent pass
        var pass_scene = this.GetTHREEJSObjectHierarchy(__root_node, "transparent_pass", __root_node_unique_id, __root_node_unique_id, __coincident_group_id);
        // Create an opaque depth pass
        var depth_pass_scene = this.GetTHREEJSObjectHierarchy(__root_node, "transp_depth_pass", __root_node_unique_id, __root_node_unique_id, __coincident_group_id);
        // Create a transparent depth pass
        var transp_diffuse_pass_scene = this.GetTHREEJSObjectHierarchy(__root_node, "transp_diffuse_pass", __root_node_unique_id, __root_node_unique_id, __coincident_group_id);
        if(__coincident_group_id === undefined)
        {
            if(pass_scene.newlycreated)
            {
                this.get('_3JSsceneTransp').add(pass_scene.node);
            }
            if(depth_pass_scene.newlycreated)
            {
                this.get('_3JSsceneTranspDepth').add(depth_pass_scene.node);
            }
            if(transp_diffuse_pass_scene.newlycreated)
            {
                this.get('_3JSsceneTranspDiffuse').add(transp_diffuse_pass_scene.node);
            }
        }
        else //Create one scene hierarchy per coincident group
        {
            this.InitTransparentCoincidentGroupScene(__coincident_group_id);
            if(pass_scene.newlycreated)
            {
                this.get('_3JSsceneTranspCoincidentGroups')[__coincident_group_id].add(pass_scene.node);
            }
            if(depth_pass_scene.newlycreated)
            {
                this.get('_3JSsceneTranspCoincidentGroupsDepth')[__coincident_group_id].add(depth_pass_scene.node);
            }
            if(transp_diffuse_pass_scene.newlycreated)
            {
                this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[__coincident_group_id].add(transp_diffuse_pass_scene.node);
            }
        }
    },

    SetNonEmptyScene: function(__transparent)
    {
        if(__transparent)
        {
            this.set("_emptyTransparentScene", false);
        }
        else
        {
            this.set("_emptyOpaqueScene", false);
        }
    },

    CreateNewCoincidentGroupPass: function(__i, __transparent)
    {
        if(!__transparent)
        {
            if ( !this.get('_3JSsceneOpCoincidentGroupsIds').hasOwnProperty(__i) )
            {
                this.get('_3JSsceneOpCoincidentGroupsIds')[__i] = {};
                this.get('_3JSsceneOpCoincidentGroupsPasses').push(__i);
            }
        }
        else
        {
            if ( !this.get('_3JSsceneTranspCoincidentGroupsIds').hasOwnProperty(__i) )
            {
                this.get('_3JSsceneTranspCoincidentGroupsIds')[__i] = {};
                this.get('_3JSsceneTranspCoincidentGroupsPasses').push(__i);
            }
        }
    },

    AsynchronousLightLoad: function(Lights)
    {
        var scope = this;
        scope.get("_build_scene_exec_queue").whenFinished(function()
        {
            scope.get('_renderercache')['lights']['holo_light'] = Lights;
            scope.UpdateMaterialsGlobalLighting();
            scope.get('_model').get('_assetmanager').set('_textureLoadedCounter', scope.get('_model').get('_assetmanager').get('_textureLoadedCounter') + 1);
            var percent_tex_loaded = scope.CheckPostSceneLoadActions();
            scope.Render();
            scope.NotifySceneBuildProgress(100, percent_tex_loaded, 0);
        });
    },

    /* Select texture wrapping mode*/
    SelectTextureWrappingMode: function(__texture, __properties, __mat_struct)
    {
        if(__properties.hasOwnProperty("original_width"))
        {
            __mat_struct["originalTexWidth"] = parseFloat(__properties["original_width"]);
        }
        if(__properties.hasOwnProperty("original_height"))
        {
            __mat_struct["originalTexHeight"] = parseFloat(__properties["original_height"]);
        }
        switch (__properties["swrap"])
        {
            default:
            case "GL_REPEAT":
                __texture.wrapS = THREE.RepeatWrapping;
                // THREE.MirroredRepeatWrapping;
                break;
            case "GL_CLAMP":
            case "GL_CLAMP_TO_BORDER":
            case "GL_CLAMP_TO_EDGE":
                __texture.wrapS = THREE.ClampToEdgeWrapping;
                break;
        }
        switch (__properties["twrap"])
        {
            default:
            case "GL_REPEAT":
                __texture.wrapT = THREE.RepeatWrapping;
                // THREE.MirroredRepeatWrapping;
                break;
            case "GL_CLAMP":
            case "GL_CLAMP_TO_BORDER":
            case "GL_CLAMP_TO_EDGE":
                __texture.wrapT = THREE.ClampToEdgeWrapping;
                break;
        }
    },

    LoadMaterial: function(__matnode, __assets, __current_scene_id, __backTexture, __flipNormals)
    {
        if(__matnode['AssetType'] == "Material")
        {
            if(__matnode.hasOwnProperty("Properties"))
            {
                var matprops = __matnode["Properties"];
                var rd = 1, gd = 1, bd = 1, rs = 1, gs = 1, bs = 1, ra = 1, ga = 1, ba = 1;

                if(matprops["front"].hasOwnProperty("diffuse"))
                {
                    rd =  matprops["front"]["diffuse"][0];
                    gd =  matprops["front"]["diffuse"][1];
                    bd =  matprops["front"]["diffuse"][2];
                }
                if(matprops["front"].hasOwnProperty("specular"))
                {
                    rs =  matprops["front"]["specular"][0];
                    gs =  matprops["front"]["specular"][1];
                    bs =  matprops["front"]["specular"][2];
                }
                if(matprops["front"].hasOwnProperty("ambient"))
                {
                    ra =  matprops["front"]["ambient"][0];
                    ga =  matprops["front"]["ambient"][1];
                    ba =  matprops["front"]["ambient"][2];
                }

                var dif_col = new THREE.Color(rd, gd, bd);
                var spec_col = new THREE.Color(rs, gs, bs);
                var amb_col = new THREE.Color(ra, ga, ba);

                var shininess = matprops["front"].hasOwnProperty("shininess") ? WV3DMath.clamp(parseFloat(matprops["front"]["shininess"]),0.,1.):.5;
                var double_sided = matprops.hasOwnProperty("double_sided") ? ( WV3DUtils.get('toBool')( matprops["double_sided"] ) ) : false;
                var opacity = matprops.hasOwnProperty("opacity") ? WV3DMath.clamp(parseFloat(matprops["opacity"]),0.0,1.):1.;
                var double_grating = matprops["front"].hasOwnProperty("double_grating") ? (  WV3DUtils.get('toBool')( matprops["front"]["double_grating"] ) ) : false;
                var grating_spacing_scale = matprops["front"].hasOwnProperty("grating_spacing_scale") ? parseFloat(matprops["front"]["grating_spacing_scale"]):null;
                var default_diff_haze = matprops["front"].hasOwnProperty("default_diff_haze") ? parseFloat(matprops["front"]["default_diff_haze"]):null;
                var default_diff_spacing = matprops["front"].hasOwnProperty("default_diff_spacing") ? parseFloat(matprops["front"]["default_diff_spacing"]):null;

                var holographic =   matprops["front"].hasOwnProperty("diff_grating_tex") ||
                                    matprops["front"].hasOwnProperty("diff_grating_filter_tex");

                var printable =     holographic || matprops["front"].hasOwnProperty("int_specular_tex") ||
                                    matprops["front"].hasOwnProperty("normal_chan0_tex") ||
                                    matprops["front"].hasOwnProperty("normal_chan1_tex");

                //Material opacity can be overriden if a transparent diffuse texture is attached,
                //a default opacity parameter (see below in material_struct initialization)
                //takes over the default one to ensure that material will be seen transparent
                //as long as we are waiting for the attached texture (which has the right opacity
                //value for each texel)
                var transparent_diffuse_texture = false;
                if(   matprops["front"].hasOwnProperty("diffuse_tex")
                   && __assets[matprops["front"]["diffuse_tex"]]["Properties"].hasOwnProperty("transparent"))
                {
                    //This only happens for printable parts (which have a diffuse alpha channel)
                    //Check if the alpha channel of the diffuse texture has any transparency
                    transparent_diffuse_texture = WV3DUtils.get('toBool')( __assets[matprops["front"]["diffuse_tex"]]["Properties"]["transparent"] );
                    //Conditionally replace the opacity of the untextured material
                    opacity = transparent_diffuse_texture?0.5:opacity;
                }

                //This specific structure gathers the material data, depending on
                //which textures are being loaded and allowed to be displayed (front is the list of
                //current materials bound to the meshes in boundmeshlist, the one to be displayed)
                var material_struct =
                {
                    //Here live the threejs materials,
                    "mat":{"untextured_mat":null, "textured_mat":{}},
                    //Here are the meshes that use these materials
                    "boundmeshlist":null,
                    "reflectance_model":matprops["reflectance_model"],
                    "diff_texture":null,
                    "spec_texture":null,
                    "int_spec_texture":null,
                    "normal_chan0_texture":null,
                    "normal_chan1_texture":null,
                    "diff_grating_texture":null,
                    "diff_grating_filter_texture":null,
                    "diff_col":dif_col,
                    "spec_col":spec_col,
                    "amb_col":amb_col,
                    "shininess":shininess,
                    "double_sided": double_sided,
                    "opacity":opacity,
                    "transp_texture":transparent_diffuse_texture,
                    "double_grating": double_grating,
                    "grating_spacing_scale": grating_spacing_scale,
                    "default_diff_haze": default_diff_haze,
                    "default_diff_spacing": default_diff_spacing,
                    "holographic":holographic,
                    "printable":printable,
                    "backTexture":__backTexture,
                    "flipNormals":__flipNormals
                };

                //By default, we display the untextured material, waiting for the textures to be loaded asynchronuously
                var untexturedmat = this.CreateMaterial(material_struct);
                material_struct["mat"]["untextured_mat"] = untexturedmat;

                var path;
                if(this.get("_model").get("_assetmanager").get("_assetloader").get('_httprequestmanager').IsAbsoluteURL(__current_scene_id))
                {
                    path = __current_scene_id.substring(0, __current_scene_id.lastIndexOf("/")+1);
                }
                else
                {
                    path = this.get('_resourcePath')+"/";//default resource location for samplescenedata
                }

                this.StartAsyncTextureLoad( this, __assets, matprops, "diffuse_tex", "diff_texture", material_struct, path, printable?THREE.RGBAFormat:THREE.RGBFormat, 4);
                this.StartAsyncTextureLoad( this, __assets, matprops, "specular_tex", "spec_texture", material_struct, path, THREE.RGBAFormat, 4);
                if( this.get('_RenderingQuality') == "Visualizer")
                {
                    this.StartAsyncTextureLoad( this, __assets, matprops, "int_specular_tex", "int_spec_texture", material_struct, path, THREE.RGBAFormat, 4);
                    this.StartAsyncTextureLoad( this, __assets, matprops, "normal_chan0_tex", "normal_chan0_texture", material_struct, path, THREE.RGBFormat, 1);
                    this.StartAsyncTextureLoad( this, __assets, matprops, "normal_chan1_tex", "normal_chan1_texture", material_struct, path, THREE.RGBFormat, 1);
                    var dif_grat = this.StartAsyncTextureLoad( this, __assets, matprops, "diff_grating_tex", "diff_grating_texture", material_struct, path, THREE.RGBAFormat, 4);
                    dif_grat = dif_grat && this.StartAsyncTextureLoad( this, __assets, matprops, "diff_grating_filter_tex", "diff_grating_filter_texture", material_struct, path, THREE.RGBAFormat, 4);
                }

                if( this.get('_RenderingQuality') == "Visualizer" && holographic &&
                    !this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["spectrum"]["queried"])
                {
                    //Load needed resources for holographics
                    var self = this;
                    var path = this.get('_resourcePath')+"/"+this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["path"];

                    //The two next asynchronuous calls are treated separately, however when and only when
                    //both are completed holographic can be enabled.
                    this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["spectrum"]["queried"] = true;

                    //Lights positions and other parameters
                    var request = new WV3DHttpRequestHandler(path + "PointLightSources.json", this, this.AsynchronousLightLoad);
                    //Here we use the texture counter (used for resources)
                    this.get('_model').get('_assetmanager').set('_textureCounter', this.get('_model').get('_assetmanager').get('_textureCounter') + 1);

                    //Spectrum texture
                    this.StartAsyncGlobalTextureLoad(self, "spectrum", path);
                }

                return material_struct;
            }
            else
            {
                LogMessage.Log("ViewerView Warning: Material properties not defined");
            }
        }
        else
        {
            LogMessage.Log("ViewerView Warning: Queried asset is not a material");
        }
    },

    /**
     * CreateMaterial
     * __renderpass is optional: if set, only create the required renderpass
     */
    CreateMaterial: function(__mat_struct, __renderpass)
    {
        var envmap = null; // for designer quality, this is a texture. For visualizer quality, this is an array of textures
        var irrmap = null;

        //TEXTURE OFFSET AND SCALE
        var uvScaleMap;
        var offsetRepeat;
        if ( __mat_struct["diff_texture"] ) { uvScaleMap = __mat_struct["diff_texture"]; }
        else if ( __mat_struct["spec_texture"] ) { uvScaleMap = __mat_struct["spec_texture"];}
        if ( uvScaleMap !== undefined )
        {
            var offset = uvScaleMap.offset;
            var repeat = uvScaleMap.repeat;
            offsetRepeat = new THREE.Vector4( offset.x, offset.y, repeat.x, repeat.y );
        }
        else
        {
            offsetRepeat = new THREE.Vector4( 0, 0, 1, 1 );
        }

        //Select custom material to be assigned
        var shader_properties = {};
        var support_transparency = WV3DSystemChecker.get('_fullSupportFloatingPointTextures') || WV3DSystemChecker.get('_fullSupportHFloatingPointTextures');
        var opaque = (__mat_struct["opacity"] >= 1 ) && !__mat_struct["transp_texture"] || !support_transparency;
        switch(this.get('_RenderingQuality'))
        {
            case "Designer":

                //Designer Quality
                //Attaching irradiance and environment maps if loaded
                if(__mat_struct["shininess"] >= 0.5)
                {
                    if(this.get('_renderercache')["textures"]["Designer"]["envmap"]["glossy"]["loaded"])
                    {
                        envmap = this.get('_renderercache')["textures"]["Designer"]["envmap"]["glossy"]["data"];
                        //Enable irradiancemap only when envmap is enabled
                        if (this.get('_renderercache')["textures"]["Designer"]["irradiancemap"]["loaded"])
                        {
                            irrmap = this.get('_renderercache')["textures"]["Designer"]["irradiancemap"]["data"];
                        }
                    }
                }
                else
                {
                    if(this.get('_renderercache')["textures"]["Designer"]["envmap"]["matte"]["loaded"])
                    {
                        envmap = this.get('_renderercache')["textures"]["Designer"]["envmap"]["matte"]["data"];
                        //Enable irradiancemap only when envmap is enabled
                        if (this.get('_renderercache')["textures"]["Designer"]["irradiancemap"]["loaded"])
                        {
                            irrmap = this.get('_renderercache')["textures"]["Designer"]["irradiancemap"]["data"];
                        }
                    }
                }

                var designer_uniforms = THREE.UniformsUtils.clone(DesignerShader.uniforms);
                designer_uniforms["diffuse"]["value"] = __mat_struct["diff_col"];
                designer_uniforms["opacity"]["value"] = opaque?1.0:__mat_struct["opacity"];
                designer_uniforms["specular"]["value"] =  __mat_struct["spec_col"];
                designer_uniforms["shininess"]["value"] = Math.max(0.001,__mat_struct["shininess"]);
                designer_uniforms["map"]["value"] = __mat_struct["diff_texture"];
                designer_uniforms["specularMap"]["value"] = __mat_struct["spec_texture"];
                designer_uniforms["irr_tex"]["value"] = irrmap;
                designer_uniforms["envMap"]["value"] = envmap;
                designer_uniforms["reflectivity"]["value"] = 0.5;
                designer_uniforms["offsetRepeat"]["value"] = offsetRepeat;

                //This is the single and only pass in designer quality. i.e. one scene construction
                shader_properties[1] = {
                    "name":"designer",
                    "uniforms": designer_uniforms,
                    "vertexShader": DesignerShader.vertexShader,
                    "fragmentShader": DesignerShader.fragmentShader,
                    "depthTest":true,
                    "vertexColors": THREE.FaceColors,
                    "side":__mat_struct["double_sided"]?THREE.DoubleSide:THREE.FrontSide,
                    "transparent":!opaque,
                    "blending":THREE.NormalBlending,
                    "defines":{"ENVMAP_TYPE_CUBE":"","ENVMAP_MODE_REFLECTION":"", "ENVMAP_BLENDING_ADD":""},
                    "lights":true
                }

                break;

            case "Visualizer":

                //Visualizer Quality
                //Attaching irradiance and environment maps if loaded
                var local_envmaps = {};
                var all_loaded = true;
                for (var i = 0; i < 5; ++i)
                {
                    if(this.get('_renderercache')["textures"]["Visualizer"]["envmap"]["loaded"][i])
                    {
                        local_envmaps[i] = this.get('_renderercache')["textures"]["Visualizer"]["envmap"]["data"][i];
                    }
                    else
                    {
                        all_loaded = false;
                        break;
                    }
                }

                if (all_loaded && this.get('_renderercache')["textures"]["Visualizer"]["irradiancemap"]["loaded"])
                {
                    irrmap = this.get('_renderercache')["textures"]["Visualizer"]["irradiancemap"]["data"];
                    envmap = local_envmaps;
                }

                //HERE, ACCORDING TO THE MATERIAL WE WANT TO CREATE, WE CAN ORDER OUR SHADERS PASSES
                if(__mat_struct["printable"])
                {
                    //// IRRADIANCE (+BUMP) PASS
                    var irradiance_shader_pm_uniforms_pass_0 = THREE.UniformsUtils.clone( IrradianceShaderPM.uniforms );
                    irradiance_shader_pm_uniforms_pass_0["offsetRepeat"]["value"] = offsetRepeat;
                    irradiance_shader_pm_uniforms_pass_0["diffuseColor"]["value"] = __mat_struct["diff_col"];
                    irradiance_shader_pm_uniforms_pass_0["opacity"]["value"] = opaque?1.0:__mat_struct["opacity"];
                    irradiance_shader_pm_uniforms_pass_0["diffuseTexture"]["value"] = __mat_struct["diff_texture"]?__mat_struct["diff_texture"]:this.get("_DummyTexture");
                    irradiance_shader_pm_uniforms_pass_0["normalMap_chan_0"]["value"] = __mat_struct["normal_chan0_texture"]?__mat_struct["normal_chan0_texture"]:this.get("_DummyTexture");
                    irradiance_shader_pm_uniforms_pass_0["normalMap_chan_1"]["value"] = __mat_struct["normal_chan1_texture"]?__mat_struct["normal_chan1_texture"]:this.get("_DummyTexture");
                    irradiance_shader_pm_uniforms_pass_0["DX1RT"]["value"] = this.get("_DummyTexture");
                    irradiance_shader_pm_uniforms_pass_0["DX2RT"]["value"] = this.get("_DummyTexture");
                    irradiance_shader_pm_uniforms_pass_0["irradianceCubeMap"]["value"] = irrmap?irrmap:this.get("_DummyTextureCube");
                    irradiance_shader_pm_uniforms_pass_0["backTexture"]["value"] = __mat_struct["backTexture"]?1:0;
                    irradiance_shader_pm_uniforms_pass_0["flipNormals"]["value"] = __mat_struct["flipNormals"]?1:0;

                    var irradiance_shader_pm_defines_pass_0 = {};
                    if(__mat_struct["normal_chan0_texture"]) irradiance_shader_pm_defines_pass_0["USE_CHAN0_TEXTURE"] = "";
                    if(__mat_struct["normal_chan1_texture"]) irradiance_shader_pm_defines_pass_0["USE_CHAN1_TEXTURE"] = "";

                    shader_properties [1] = {
                        "name":"irradiancepm",
                        "uniforms": irradiance_shader_pm_uniforms_pass_0,
                        "vertexShader": IrradianceShaderPM.vertexShader,
                        "fragmentShader": IrradianceShaderPM.fragmentShader,
                        "depthTest":true,
                        "vertexColors": THREE.FaceColors,
                        "side":(__mat_struct["double_sided"]?THREE.DoubleSide:THREE.FrontSide),
                        "defines":irradiance_shader_pm_defines_pass_0,
                        // "lights":true,
                        "blending":opaque?THREE.NormalBlending:THREE.CustomBlending, // Coincident transparent surfaces need special blending
                        "blendSrc":THREE.OneFactor,//For CustomBlending only
                        "blendDst":THREE.OneMinusSrcAlphaFactor,//For CustomBlending only
                        "transparent": true,//<-- transparency is necessary to enable blending
                        "extensions":{
                            derivatives: true, // set to use derivatives
                            fragDepth: false, // set to use fragment depth values
                            drawBuffers: false, // set to use draw buffers
                            shaderTextureLOD: false // set to use shader texture LOD
                        }
                     };

                    //// FILTER (DIFFUSE) PASS
                    var filter_shader_pm_uniforms_pass_1 = THREE.UniformsUtils.clone( FilterShaderPM.uniforms );
                    filter_shader_pm_uniforms_pass_1["offsetRepeat"]["value"] = offsetRepeat;
                    filter_shader_pm_uniforms_pass_1["diffuseColor"]["value"] = __mat_struct["diff_col"];
                    filter_shader_pm_uniforms_pass_1["opacity"]["value"] = opaque?1.0:__mat_struct["opacity"];
                    filter_shader_pm_uniforms_pass_1["diffuseTexture"]["value"] = __mat_struct["diff_texture"]?__mat_struct["diff_texture"]:this.get("_DummyTexture");
                    filter_shader_pm_uniforms_pass_1["DX1RT"]["value"] = this.get("_DummyTexture");
                    filter_shader_pm_uniforms_pass_1["DX2RT"]["value"] = this.get("_DummyTexture");
                    filter_shader_pm_uniforms_pass_1["backTexture"]["value"] = __mat_struct["backTexture"]?1:0;
                    filter_shader_pm_uniforms_pass_1["flipNormals"]["value"] = __mat_struct["flipNormals"]?1:0;

                    shader_properties [3] = {
                        "name":"filterpm",
                        "uniforms": filter_shader_pm_uniforms_pass_1,
                        "vertexShader": FilterShaderPM.vertexShader,
                        "fragmentShader": FilterShaderPM.fragmentShader,
                        "depthTest":true,
                        "vertexColors": THREE.FaceColors,
                        "side":(__mat_struct["double_sided"]?THREE.DoubleSide:THREE.FrontSide),
                        "defines":{},
                        // "lights":true,
                        "blending":THREE.CustomBlending,
                        "blendSrc":THREE.DstColorFactor,
                        "blendDst":THREE.ZeroFactor,
                        "transparent":true //<--this is necessary to enable blending
                    };


                    //// REFLECTANCE (ENVIRONMENT + BUMP) PASS
                    var reflectance_shader_pm_uniforms_pass_2;
                    if(WV3DSystemChecker.get("_glmaxtextureimageunits") >= 11)
                    {
                        reflectance_shader_pm_uniforms_pass_2 = THREE.UniformsUtils.clone( ReflectanceShaderPM.EXTuniforms );
                        reflectance_shader_pm_uniforms_pass_2["haze1CubeMap"]["value"] = envmap?envmap[0]:this.get("_DummyTextureCube");
                        reflectance_shader_pm_uniforms_pass_2["haze2CubeMap"]["value"] = envmap?envmap[1]:this.get("_DummyTextureCube");
                        reflectance_shader_pm_uniforms_pass_2["haze3CubeMap"]["value"] = envmap?envmap[2]:this.get("_DummyTextureCube");
                        reflectance_shader_pm_uniforms_pass_2["haze4CubeMap"]["value"] = envmap?envmap[3]:this.get("_DummyTextureCube");
                        reflectance_shader_pm_uniforms_pass_2["haze5CubeMap"]["value"] = envmap?envmap[4]:this.get("_DummyTextureCube");
                    }else
                    {
                        reflectance_shader_pm_uniforms_pass_2 = THREE.UniformsUtils.clone( ReflectanceShaderPM.uniforms );
                        reflectance_shader_pm_uniforms_pass_2["haze1CubeMap"]["value"] = envmap?envmap[0]:this.get("_DummyTextureCube");
                        reflectance_shader_pm_uniforms_pass_2["haze2CubeMap"]["value"] = envmap?envmap[4]:this.get("_DummyTextureCube");
                    }
                    reflectance_shader_pm_uniforms_pass_2["offsetRepeat"]["value"] = offsetRepeat;
                    reflectance_shader_pm_uniforms_pass_2["diffuseColor"]["value"] = __mat_struct["diff_col"];
                    reflectance_shader_pm_uniforms_pass_2["specularColor"]["value"] = __mat_struct["spec_col"];
                    reflectance_shader_pm_uniforms_pass_2["shininess"]["value"] = __mat_struct["shininess"];
                    reflectance_shader_pm_uniforms_pass_2["diffuseTexture"]["value"] = __mat_struct["diff_texture"]?__mat_struct["diff_texture"]:this.get("_DummyTexture");
                    reflectance_shader_pm_uniforms_pass_2["specularTexture"]["value"] = __mat_struct["spec_texture"]?__mat_struct["spec_texture"]:this.get("_DummyTexture");
                    reflectance_shader_pm_uniforms_pass_2["internalSpecTex"]["value"] = __mat_struct["int_spec_texture"]?__mat_struct["int_spec_texture"]:this.get("_DummyTexture");
                    reflectance_shader_pm_uniforms_pass_2["normalMap_chan_0"]["value"] = __mat_struct["normal_chan0_texture"]?__mat_struct["normal_chan0_texture"]:this.get("_DummyTexture");
                    reflectance_shader_pm_uniforms_pass_2["normalMap_chan_1"]["value"] = __mat_struct["normal_chan1_texture"]?__mat_struct["normal_chan1_texture"]:this.get("_DummyTexture");
                    reflectance_shader_pm_uniforms_pass_2["DX1RT"]["value"] = this.get("_DummyTexture");
                    reflectance_shader_pm_uniforms_pass_2["DX2RT"]["value"] = this.get("_DummyTexture");
                    reflectance_shader_pm_uniforms_pass_2["backTexture"]["value"] = __mat_struct["backTexture"]?1:0;
                    reflectance_shader_pm_uniforms_pass_2["flipNormals"]["value"] = __mat_struct["flipNormals"]?1:0;

                    var reflectance_shader_pm_defines_pass_2 = {};
                    if(__mat_struct["int_spec_texture"]) reflectance_shader_pm_defines_pass_2["USE_INT_SPEC_TEXTURE"] = "";
                    if(__mat_struct["normal_chan0_texture"]) reflectance_shader_pm_defines_pass_2["USE_CHAN0_TEXTURE"] = "";
                    if(__mat_struct["normal_chan1_texture"]) reflectance_shader_pm_defines_pass_2["USE_CHAN1_TEXTURE"] = "";

                    shader_properties [6] = {
                        "name":"reflectancepm",
                        "uniforms": reflectance_shader_pm_uniforms_pass_2,
                        "vertexShader": ReflectanceShaderPM.vertexShader,
                        "fragmentShader": (WV3DSystemChecker.get("_glmaxtextureimageunits") >= 11) ? ReflectanceShaderPM.EXTfragmentShader : ReflectanceShaderPM.fragmentShader,
                        "depthTest":true,
                        "vertexColors": THREE.FaceColors,
                        "side":(__mat_struct["double_sided"]?THREE.DoubleSide:THREE.FrontSide),
                        "defines":reflectance_shader_pm_defines_pass_2,
                        // "lights":true,
                        "blending":THREE.CustomBlending,
                        "blendSrc":THREE.OneFactor,
                        "blendDst":THREE.OneFactor,
                        "transparent":true, //<--this is necessary to enable blending
                        "extensions":{
                            derivatives: true, // set to use derivatives
                            fragDepth: false, // set to use fragment depth values
                            drawBuffers: false, // set to use draw buffers
                            shaderTextureLOD: false // set to use shader texture LOD
                        }
                    };

                    if(__mat_struct["holographic"])
                    {

                        //// DIFFRACTION PASS
                        var diffraction_shader_pm_uniforms_pass_3 = THREE.UniformsUtils.clone( DiffractionShaderPM.uniforms );
                        diffraction_shader_pm_uniforms_pass_3["offsetRepeat"]["value"] = offsetRepeat;
                        diffraction_shader_pm_uniforms_pass_3["uvdhTexture"]["value"] = __mat_struct["diff_grating_texture"]?__mat_struct["diff_grating_texture"]:this.get("_DummyTexture");
                        diffraction_shader_pm_uniforms_pass_3["diffFilterTexture"]["value"] = __mat_struct["diff_grating_filter_texture"]?__mat_struct["diff_grating_filter_texture"]:this.get("_DummyTexture");
                        diffraction_shader_pm_uniforms_pass_3["normalMap_chan_0"]["value"] = __mat_struct["normal_chan0_texture"]?__mat_struct["normal_chan0_texture"]:this.get("_DummyTexture");
                        diffraction_shader_pm_uniforms_pass_3["normalMap_chan_1"]["value"] = __mat_struct["normal_chan1_texture"]?__mat_struct["normal_chan1_texture"]:this.get("_DummyTexture");
                        diffraction_shader_pm_uniforms_pass_3["DX1RT"]["value"] = this.get("_DummyTexture");
                        diffraction_shader_pm_uniforms_pass_3["DX2RT"]["value"] = this.get("_DummyTexture");
                        diffraction_shader_pm_uniforms_pass_3["grating_spacing_scale"]["value"] = __mat_struct["grating_spacing_scale"];

                        diffraction_shader_pm_uniforms_pass_3["spectrumTexture"]["value"] = this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["spectrum"]["data"];
                        diffraction_shader_pm_uniforms_pass_3["defaultDiffSpacing"]["value"] = __mat_struct["default_diff_spacing"];
                        diffraction_shader_pm_uniforms_pass_3["defaultDiffHaze"]["value"] = __mat_struct["default_diff_haze"];
                        diffraction_shader_pm_uniforms_pass_3["backTexture"]["value"] = __mat_struct["backTexture"]?1:0;
                        diffraction_shader_pm_uniforms_pass_3["flipNormals"]["value"] = __mat_struct["flipNormals"]?1:0;

                        var diffraction_shader_pm_defines_pass_3 = {};

                        if( __mat_struct["double_grating"] ) diffraction_shader_pm_defines_pass_3["DOUBLE_GRATING"] = "";
                        if(__mat_struct["normal_chan0_texture"]) diffraction_shader_pm_defines_pass_3["USE_CHAN0_TEXTURE"] = "";
                        if(__mat_struct["normal_chan1_texture"]) diffraction_shader_pm_defines_pass_3["USE_CHAN1_TEXTURE"] = "";

                        if(this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["spectrum"]["loaded"] &&
                            this.get('_renderercache')['lights'].hasOwnProperty('holo_light'))
                        {
                            diffraction_shader_pm_defines_pass_3["USE_HOLOGRAPHICS"] = "";
                            diffraction_shader_pm_uniforms_pass_3["lightPos"]["value"] = [];
                            diffraction_shader_pm_uniforms_pass_3["lightIntensity"]["value"] = [];
                            diffraction_shader_pm_uniforms_pass_3["lightRadius"]["value"] = [];
                            diffraction_shader_pm_uniforms_pass_3["lightHardness"]["value"] = [];

                            // Light[] = [ x, y, z, intensity, radius, hardness, spectrumIndex ]
                            for (var i = 0; i < this.get('_renderercache')['lights']['holo_light']['Lights'].length && (i < 4); ++i)
                            {
                                var theLight = this.get('_renderercache')['lights']['holo_light']['Lights'][i];
                                diffraction_shader_pm_uniforms_pass_3["lightPos"]["value"].push( new THREE.Vector3( theLight[0], theLight[1], theLight[2] ));
                                diffraction_shader_pm_uniforms_pass_3["lightIntensity"]["value"].push( theLight[3]);
                                diffraction_shader_pm_uniforms_pass_3["lightRadius"]["value"].push( theLight[4]);
                                diffraction_shader_pm_uniforms_pass_3["lightHardness"]["value"].push( theLight[5]);
                            }
                        }

                        shader_properties [5] = {
                            "name":"holographicspm",
                            "uniforms": diffraction_shader_pm_uniforms_pass_3,
                            "vertexShader": DiffractionShaderPM.vertexShader,
                            "fragmentShader": DiffractionShaderPM.fragmentShader,
                            "depthTest":true,
                            "vertexColors": THREE.FaceColors,
                            "side":(__mat_struct["double_sided"]?THREE.DoubleSide:THREE.FrontSide),
                            "defines":diffraction_shader_pm_defines_pass_3,
                            // "lights":true,
                            "blending":THREE.CustomBlending,
                            "blendSrc":THREE.OneMinusDstColorFactor,
                            "blendDst":THREE.OneFactor,
                            "transparent":true, //<--this is necessary to enable blending
                            "extensions":{
                                derivatives: true, // set to use derivatives
                                fragDepth: false, // set to use fragment depth values
                                drawBuffers: false, // set to use draw buffers
                                shaderTextureLOD: false // set to use shader texture LOD
                            }
                        };
                    }
                }
                else
                {
                    //// IRRADIANCE (NO BUMP) PASS
                    var irradiance_shader_npm_uniforms_pass_4 = THREE.UniformsUtils.clone( IrradianceShaderNPM.uniforms );
                    irradiance_shader_npm_uniforms_pass_4["offsetRepeat"]["value"] = offsetRepeat;
                    irradiance_shader_npm_uniforms_pass_4["diffuseColor"]["value"] = __mat_struct["diff_col"];
                    irradiance_shader_npm_uniforms_pass_4["opacity"]["value"] = opaque?1.0:__mat_struct["opacity"];
                    irradiance_shader_npm_uniforms_pass_4["diffuseTexture"]["value"] = __mat_struct["diff_texture"]?__mat_struct["diff_texture"]:this.get("_DummyTexture");
                    irradiance_shader_npm_uniforms_pass_4["DX1RT"]["value"] = this.get("_DummyTexture");
                    irradiance_shader_npm_uniforms_pass_4["DX2RT"]["value"] = this.get("_DummyTexture");
                    irradiance_shader_npm_uniforms_pass_4["irradianceCubeMap"]["value"] = irrmap?irrmap:this.get("_DummyTextureCube");
                    irradiance_shader_npm_uniforms_pass_4["backTexture"]["value"] = __mat_struct["backTexture"]?1:0;
                    irradiance_shader_npm_uniforms_pass_4["flipNormals"]["value"] = __mat_struct["flipNormals"]?1:0;

                    shader_properties [2] = {
                        "name":"irradiancenpm",
                        "uniforms": irradiance_shader_npm_uniforms_pass_4,
                        "vertexShader": IrradianceShaderNPM.vertexShader,
                        "fragmentShader": IrradianceShaderNPM.fragmentShader,
                        "depthTest":true,
                        "vertexColors": THREE.FaceColors,
                        "side":(__mat_struct["double_sided"]?THREE.DoubleSide:THREE.FrontSide),
                        "defines":{},
                        // "lights":true,
                        "blending":opaque?THREE.NormalBlending:THREE.CustomBlending, // Coincident transparent surfaces need special blending
                        "blendSrc":THREE.OneFactor,//For CustomBlending only
                        "blendDst":THREE.OneMinusSrcAlphaFactor,//For CustomBlending only
                        "transparent": true//<-- transparency is necessary to enable blending
                    };

                    //// FILTER (DIFFUSE) PASS
                    var filter_shader_npm_uniforms_pass_5 = THREE.UniformsUtils.clone( FilterShaderNPM.uniforms );
                    filter_shader_npm_uniforms_pass_5["offsetRepeat"]["value"] = offsetRepeat;
                    filter_shader_npm_uniforms_pass_5["diffuseColor"]["value"] = __mat_struct["diff_col"];
                    filter_shader_npm_uniforms_pass_5["opacity"]["value"] = opaque?1.0:__mat_struct["opacity"];
                    filter_shader_npm_uniforms_pass_5["diffuseTexture"]["value"] = __mat_struct["diff_texture"]?__mat_struct["diff_texture"]:this.get("_DummyTexture");
                    filter_shader_npm_uniforms_pass_5["DX1RT"]["value"] = this.get("_DummyTexture");
                    filter_shader_npm_uniforms_pass_5["DX2RT"]["value"] = this.get("_DummyTexture");
                    filter_shader_npm_uniforms_pass_5["backTexture"]["value"] = __mat_struct["backTexture"]?1:0;
                    filter_shader_npm_uniforms_pass_5["flipNormals"]["value"] = __mat_struct["flipNormals"]?1:0;

                    shader_properties [4] = {
                        "name":"filternpm",
                        "uniforms": filter_shader_npm_uniforms_pass_5,
                        "vertexShader": FilterShaderNPM.vertexShader,
                        "fragmentShader": FilterShaderNPM.fragmentShader,
                        "depthTest":true,
                        "vertexColors": THREE.FaceColors,
                        "side":(__mat_struct["double_sided"]?THREE.DoubleSide:THREE.FrontSide),
                        "defines":{},
                        // "lights":true,
                        "blending":THREE.CustomBlending,
                        "blendSrc":THREE.DstColorFactor,
                        "blendDst":THREE.ZeroFactor,
                        "transparent":true //<--this is necessary to enable blending
                    };

                    //// REFLECTANCE (ENVIRONMENT + NO BUMP) PASS
                    var reflectance_shader_npm_uniforms_pass_6;
                    if(WV3DSystemChecker.get("_glmaxtextureimageunits") >= 11)
                    {
                        reflectance_shader_npm_uniforms_pass_6 = THREE.UniformsUtils.clone( ReflectanceShaderNPM.EXTuniforms );
                        reflectance_shader_npm_uniforms_pass_6["haze1CubeMap"]["value"] = envmap?envmap[0]:this.get("_DummyTextureCube");
                        reflectance_shader_npm_uniforms_pass_6["haze2CubeMap"]["value"] = envmap?envmap[1]:this.get("_DummyTextureCube");
                        reflectance_shader_npm_uniforms_pass_6["haze3CubeMap"]["value"] = envmap?envmap[2]:this.get("_DummyTextureCube");
                        reflectance_shader_npm_uniforms_pass_6["haze4CubeMap"]["value"] = envmap?envmap[3]:this.get("_DummyTextureCube");
                        reflectance_shader_npm_uniforms_pass_6["haze5CubeMap"]["value"] = envmap?envmap[4]:this.get("_DummyTextureCube");
                    }else
                    {
                        reflectance_shader_npm_uniforms_pass_6 = THREE.UniformsUtils.clone( ReflectanceShaderNPM.uniforms );
                        reflectance_shader_npm_uniforms_pass_6["haze1CubeMap"]["value"] = envmap?envmap[0]:this.get("_DummyTextureCube");
                        reflectance_shader_npm_uniforms_pass_6["haze2CubeMap"]["value"] = envmap?envmap[4]:this.get("_DummyTextureCube");
                    }
                    reflectance_shader_npm_uniforms_pass_6["DX1RT"]["value"] = this.get("_DummyTexture");
                    reflectance_shader_npm_uniforms_pass_6["DX2RT"]["value"] = this.get("_DummyTexture");
                    reflectance_shader_npm_uniforms_pass_6["offsetRepeat"]["value"] = offsetRepeat;
                    reflectance_shader_npm_uniforms_pass_6["specularCol"]["value"] = __mat_struct["spec_col"];
                    reflectance_shader_npm_uniforms_pass_6["shininess"]["value"] = __mat_struct["shininess"];
                    reflectance_shader_npm_uniforms_pass_6["backTexture"]["value"] = __mat_struct["backTexture"]?1:0;
                    reflectance_shader_npm_uniforms_pass_6["flipNormals"]["value"] = __mat_struct["flipNormals"]?1:0;

                    shader_properties [7] = {
                        "name":"reflectancenpm",
                        "uniforms": reflectance_shader_npm_uniforms_pass_6,
                        "vertexShader": ReflectanceShaderNPM.vertexShader,
                        "fragmentShader": (WV3DSystemChecker.get("_glmaxtextureimageunits") >= 11) ? ReflectanceShaderNPM.EXTfragmentShader : ReflectanceShaderNPM.fragmentShader,
                        "depthTest":true,
                        "vertexColors": THREE.FaceColors,
                        "side":(__mat_struct["double_sided"]?THREE.DoubleSide:THREE.FrontSide),
                        "defines":{},
                        // "lights":true,
                        "blending":THREE.CustomBlending,
                        "blendSrc":THREE.OneFactor,
                        "blendDst":THREE.OneFactor,
                        "transparent":true //<--this is necessary to enable blending
                    };

                }
                break;
        }

        //For each set of shader properties, we build the corresponding shader
        //Then we return the set of shaders that the Material requires
        var shader_passes = {};

        if(!__renderpass
           || (__renderpass == "depth_pass")
           || (__renderpass == "opaque_diffuse_pass")
           || (__renderpass == "transp_depth_pass")
           || (__renderpass == "transp_diffuse_pass"))
        {
            // These shaders are material-dependent, so they cannot be put in the global material rendercache,
            // but should NOT be added to the usual renderpasses.
            var D0ShaderMaterial = null;
            var TransBlendShaderMaterial = null;

            if(!__renderpass
               || (__renderpass == "depth_pass")
               || (__renderpass == "transp_depth_pass"))
            {
                var D0_uniforms = THREE.UniformsUtils.clone(D0Shader.uniforms);
                D0ShaderMaterial = new THREE.ShaderMaterial({
                    "uniforms": D0_uniforms,
                    "vertexShader": D0Shader.vertexShader,
                    "fragmentShader": D0Shader.fragmentShader,
                    "side":THREE.DoubleSide,
                    "depthWrite": true,
                    "transparent":false,
                    "name": "D0ShaderMaterial",
                    "lights":false
                });
            }

            if(!__renderpass
               || (__renderpass == "opaque_diffuse_pass")
               || (__renderpass == "transp_diffuse_pass"))
            {
                var trans_blend_uniforms = THREE.UniformsUtils.clone( TransBlendShader.uniforms );
                trans_blend_uniforms["diffuseColor"]["value"] = __mat_struct["diff_col"];
                trans_blend_uniforms["opacity"]["value"] = __mat_struct["opacity"];
                trans_blend_uniforms["diffuseTexture"]["value"] = __mat_struct["diff_texture"]?__mat_struct["diff_texture"]:this.get("_DummyTexture");
                trans_blend_uniforms["offsetRepeat"]["value"] = offsetRepeat;
                trans_blend_uniforms["DX1RT"]["value"] = this.get("_DummyTexture");
                trans_blend_uniforms["DX2RT"]["value"] = this.get("_DummyTexture");
                trans_blend_uniforms["backTexture"]["value"] = __mat_struct["backTexture"]?1:0;
                trans_blend_uniforms["flipNormals"]["value"] = __mat_struct["flipNormals"]?1:0;
                TransBlendShaderMaterial = new THREE.ShaderMaterial({
                    "uniforms": trans_blend_uniforms,
                    "vertexShader": TransBlendShader.vertexShader,
                    "fragmentShader": TransBlendShader.fragmentShader,
                    "side":(__mat_struct["double_sided"]?THREE.DoubleSide:THREE.FrontSide),
                    "depthWrite": true,
                    "transparent":true,
                    // We use multiplicative blending to ensure that if several concident surfaces are drawn on the same peel,
                    // the result is the same as if they were each drawn on their own peel
                    "blending":THREE.CustomBlending,
                    "blendSrc":THREE.DstColorFactor,
                    "blendDst":THREE.ZeroFactor,
                    "name": "TransBlendShader",
                    "lights":false
                });

                if(__mat_struct["diff_texture"] )
                    TransBlendShaderMaterial["defines"]["USE_MAP"] = "";

                if( ! __mat_struct["printable"] )
                    TransBlendShaderMaterial["defines"]["IS_NPM"] = "";

                if(__mat_struct["diff_texture"] && __mat_struct["diff_texture"]["remappedAlphaChannel"])
                    TransBlendShaderMaterial["defines"]["REMAPPED_DIFFUSE_ALPHA_CHANNEL"] = "";

                if(__mat_struct["spec_texture"] && __mat_struct["spec_texture"]["remappedAlphaChannel"])
                    TransBlendShaderMaterial["defines"]["REMAPPED_SPECULAR_ALPHA_CHANNEL"] = "";

                if(__mat_struct["int_spec_texture"] && __mat_struct["int_spec_texture"]["remappedAlphaChannel"])
                    TransBlendShaderMaterial["defines"]["REMAPPED_INT_SPECULAR_ALPHA_CHANNEL"] = "";

                if(__mat_struct["diff_grating_texture"] && __mat_struct["diff_grating_texture"]["remappedAlphaChannel"])
                    TransBlendShaderMaterial["defines"]["REMAPPED_DIFF_GRATING_ALPHA_CHANNEL"] = "";

            }

            if(opaque)
            {
                if(D0ShaderMaterial) shader_passes["depth_pass"] = [D0ShaderMaterial];//single shading pass material
                if(TransBlendShaderMaterial) shader_passes["opaque_diffuse_pass"] = [TransBlendShaderMaterial];//single shading pass material
            }
            else
            {
                if(D0ShaderMaterial) shader_passes["transp_depth_pass"] = [D0ShaderMaterial];//single shading pass material
                if(TransBlendShaderMaterial) shader_passes["transp_diffuse_pass"] = [TransBlendShaderMaterial];//single shading pass material
            }
        }

        if(!shader_passes.hasOwnProperty("opaque_pass")) shader_passes["opaque_pass"] = [];
        if(!shader_passes.hasOwnProperty("transparent_pass")) shader_passes["transparent_pass"] = [];
        for (var i = 0, keys = Object.keys(shader_properties), l = keys.length; i < l; ++i)
        {
            //On-the-fly computation of the maximum needed number of passes
            var id = keys[i];
            if(shader_properties[id])
            {
                //COMMON DEFINES for DESIGNER, VISUALISERPM and VISUALISERNPM
                //These defines names are the same that are used internally by three.js
                //so we keep them as they are for comparison/understanding purposes.
                //However, we could change them as long as we change the corresponding
                //defines names in DesignerShader.js, FilterShaderPM/NPM.js, IrradianceShaderPM/NPM.js,
                // ReflectanceShaderPM/NPM.js, and DiffractionShaderPM.js

                if(__mat_struct["double_sided"])
                    shader_properties[id]["defines"]["DOUBLE_SIDED"] = "";

                if(__mat_struct["diff_texture"] )
                    shader_properties[id]["defines"]["USE_MAP"] = "";

                if(__mat_struct["spec_texture"])
                    shader_properties[id]["defines"]["USE_SPECULARMAP"] = "";

                if(irrmap)
                    shader_properties[id]["defines"]["USE_IRRMAP"] = "";

                if(envmap)
                    shader_properties[id]["defines"]["USE_ENVMAP"] = "";

                if(__mat_struct["diff_texture"] && __mat_struct["diff_texture"]["remappedAlphaChannel"])
                    shader_properties[id]["defines"]["REMAPPED_DIFFUSE_ALPHA_CHANNEL"] = "";

                if(__mat_struct["spec_texture"] && __mat_struct["spec_texture"]["remappedAlphaChannel"])
                    shader_properties[id]["defines"]["REMAPPED_SPECULAR_ALPHA_CHANNEL"] = "";

                if(__mat_struct["int_spec_texture"] && __mat_struct["int_spec_texture"]["remappedAlphaChannel"])
                    shader_properties[id]["defines"]["REMAPPED_INT_SPECULAR_ALPHA_CHANNEL"] = "";

                if(__mat_struct["diff_grating_texture"] && __mat_struct["diff_grating_texture"]["remappedAlphaChannel"])
                    shader_properties[id]["defines"]["REMAPPED_DIFF_GRATING_ALPHA_CHANNEL"] = "";

                 // shader_properties[id]["lights"] = true;

                if(opaque && (!__renderpass || __renderpass == "opaque_pass"))
                {
                    var newShader = new THREE.ShaderMaterial(shader_properties[id]);
                    newShader["polygonOffset"] = true;
                    var enableHardEdgeHighlights = this.get('_renderercache')['materials']['enablehardedgehighlights'];
                    var enablePolygonOffset = this.get('_renderercache')['materials']['enablehighlights'] && __enableHardEdgeHighlights;
                    newShader["polygonOffsetFactor"] = enablePolygonOffset ? this.get( '_polygon_offset_factor') : 0;
                    newShader["polygonOffsetUnits"] = enablePolygonOffset ? this.get( '_polygon_offset_units') : 0;

                    shader_passes["opaque_pass"].push( newShader);//Multiple shading passes material
                }
                else if (!opaque && (!__renderpass || __renderpass == "transparent_pass"))
                {
                    var newShader = new THREE.ShaderMaterial(shader_properties[id]);
                    newShader["polygonOffset"] = false;
                    shader_passes["transparent_pass"].push( newShader );//Multiple shading passes material
                }
                else
                {
                    //Material not matching current pass begin built
                }
            }
        }
        return shader_passes;
    },

    /**
     * Update link from mesh to material
     */
    UpdateTexturedMaterial: function(__mat_struct)
    {
        if(__mat_struct)
        {
            var meshestoupdate = __mat_struct["boundmeshlist"];
            //Update material reference in meshes that are already bound to this material
            if(meshestoupdate)
            {
                if(Object.keys(meshestoupdate).length == 0) LogMessage.Log("ViewerView Warning: No meshes to update");
                for (var j = 0, keys1 = Object.keys(meshestoupdate), L = keys1.length; j < L; ++j)
                {
                    var renderpass = keys1[j];
                    if(Object.keys(meshestoupdate[renderpass]).length == 0) LogMessage.Log("ViewerView Warning: No render passes to update");
                    for (var i = 0, keys2 = Object.keys(meshestoupdate[renderpass]), l = keys2.length; i < l; ++i)
                    {
                        var side_mat = keys2[i];
                        if(Object.keys(meshestoupdate[renderpass][side_mat]).length == 0) LogMessage.Log("ViewerView Warning: No sided materials to update");
                        for (var k = 0, keys3 = Object.keys(meshestoupdate[renderpass][side_mat]), m = keys3.length; k < m; ++k)
                        {
                            var shadingpassID = keys3[k];
                            if(Object.keys(meshestoupdate[renderpass][side_mat][shadingpassID]).length == 0) LogMessage.Log("ViewerView Warning: No shading pass to update");
                            for (var q = 0, keys4 = Object.keys(meshestoupdate[renderpass][side_mat][shadingpassID]), p = keys4.length; q < p; ++q)
                            {
                                var surfID = keys4[q];
                                //Create replacement material
                                var material = this.CreateMaterial(__mat_struct, renderpass);
                                //Retrieve placeholder material to set polygon side visibility
                                material[renderpass][shadingpassID]["side"] = __mat_struct["mat"]["textured_mat"][renderpass][shadingpassID][surfID]["side"];
                                //Release before overwriting placeholder material
                                this.ReleaseMaterial(__mat_struct["mat"]["textured_mat"][renderpass][shadingpassID][surfID], false);
                                //Replace placeholder material and update bound meshes
                                __mat_struct["mat"]["textured_mat"][renderpass][shadingpassID][surfID] = material[renderpass][shadingpassID];

                                for (var n = 0, keys5 = Object.keys(meshestoupdate[renderpass][side_mat][shadingpassID][surfID]), o = keys5.length; n < o; ++n)
                                {
                                    //For each bound mesh, we create a new material since each of them needs a different
                                    //Create a new material for each surface and replace the place holder material
                                    var meshtoupdate = meshestoupdate[renderpass][side_mat][shadingpassID][surfID][keys5[n]];
                                    meshtoupdate.material = material[renderpass][shadingpassID];
                                    meshtoupdate.geometry.buffersNeedUpdate = true;
                                    meshtoupdate.geometry.uvsNeedUpdate = true;
                                }
                            }
                        }
                    }
                }
            }else
            {
                LogMessage.Log("ViewerView Warning: No meshes to update");
            }
        }
    },

    /**
     * Update link from all meshes to their material
     */
    UpdateMaterialsGlobalLighting: function()
    {
        var materials = this.get('_renderercache')['materials'];
        for (var k = 0, _mats = Object.keys(materials['materiallist']), l2 = _mats.length; k < l2; ++k)
        {
            var mat = _mats[k];
            var mat_struct_front = materials['materiallist'][mat]["front"];
            var mat_struct_back = materials['materiallist'][mat]["back"];
            this.UpdateTexturedMaterial(mat_struct_front);
            this.UpdateTexturedMaterial(mat_struct_back);
        }
    },

    /**
     * Use a new texture for the current material structure
     */
    UpdateMaterialTexture: function(__mat_struct, __textureURL, __newTexture)
    {
        if(__mat_struct)
        {
            var passestoupdate = __mat_struct["mat"]["textured_mat"];
            //Update material struct texture reference
            for (var idx = 0, keys = Object.keys(__mat_struct), l = keys.length; idx < l; ++idx)
            {
                var prop = keys[idx];
                if( __mat_struct[prop] && __mat_struct[prop] instanceof THREE.Texture &&
                    __mat_struct[prop].name != "" && __mat_struct[prop].name == __textureURL)
                {
                    __mat_struct[prop].dispose();
                    __mat_struct[prop] = __newTexture;
                }
            }

            //Update each material texture reference
            if(passestoupdate)
            {
                if(Object.keys(passestoupdate).length == 0) LogMessage.Log("ViewerView Warning: No render passes to update");
                for (var j = 0, keys1 = Object.keys(passestoupdate), L = keys1.length; j < L; ++j)
                {
                    var renderpass = keys1[j];
                    if(Object.keys(passestoupdate[renderpass]).length == 0) LogMessage.Log("ViewerView Warning: No shading passes to update");
                    for (var i = 0, keys2 = Object.keys(passestoupdate[renderpass]), l = keys2.length; i < l; ++i)
                    {
                        var shadingpassID = keys2[i];
                        if(Object.keys(passestoupdate[renderpass][shadingpassID]).length == 0) LogMessage.Log("ViewerView Warning: No surface materials to update");
                        for (var k = 0, keys3 = Object.keys(passestoupdate[renderpass][shadingpassID]), m = keys3.length; k < m; ++k)
                        {
                            var surfID = keys3[k];
                            var material = passestoupdate[renderpass][shadingpassID][surfID];

                            if(material && material.uniforms)
                            {
                                for (var mat_uni_idx = 0, keys = Object.keys(material.uniforms), lk = keys.length; mat_uni_idx < lk; ++mat_uni_idx)
                                {
                                    var uniform = material.uniforms[keys[mat_uni_idx]];
                                    if(uniform["value"])
                                    {
                                        if( typeof(uniform["value"]["dispose"]) === "function" && uniform["value"] instanceof THREE.Texture)
                                        {
                                            var previous_texture = uniform["value"];
                                            if(previous_texture.name != "" && previous_texture.name == __textureURL)
                                            {
                                                __newTexture.format = previous_texture.format;
                                                __newTexture.unpackAlignment = previous_texture.unpackAlignment;
                                                __newTexture.anisotropy = previous_texture.anisotropy;
                                                __newTexture.repeat = previous_texture.repeat;
                                                __newTexture.remappedAlphaChannel = previous_texture.remappedAlphaChannel;
                                                uniform["value"] = __newTexture;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }else
            {
                LogMessage.Log("ViewerView Warning: No textured material to update");
            }
        }
    },

    /**
     * Reload all textures
     */
    ReloadAllTextures: function(__callback, __redraw)
    {
        //Reload each texture
        var loadcounter = 0;
        var scope = this;
        for( var k = 0, _texts = Object.keys(this.get('_renderercache')["textures"]["FromFile"]), l = _texts.length; k < l; ++k)
        {
            var text_url = _texts[k];
            var cb = function()
            {
                ++loadcounter;
                if(loadcounter == l)
                {
                    if(__redraw === undefined || __redraw)
                        scope.Render();
                    if(typeof(__callback) == "function")
                    {
                        __callback();
                    }
                }
            }
            this.ReloadTexture(text_url, text_url, cb, true, false);
        }

    },

    /**
     * Reload a texture
     */
    ReloadTexture: function(__textureURL, __newURL, __callback, __keep_old_path, __redraw)
    {
        var scope = this;
        var onload_tex = function(__textureURL, __newTexture)
        {
            var materials = scope.get('_renderercache')['materials'];
            for (var k = 0, _mats = Object.keys(materials['materiallist']), l2 = _mats.length; k < l2; ++k)
            {
                var mat = _mats[k];
                var mat_struct_front = materials['materiallist'][mat]["front"];
                var mat_struct_back = materials['materiallist'][mat]["back"];
                scope.UpdateMaterialTexture(mat_struct_front, __textureURL, __newTexture);
                scope.UpdateMaterialTexture(mat_struct_back, __textureURL, __newTexture);
            }

            if(__redraw === undefined || __redraw)
                scope.Render();

            if(typeof(__callback) == "function")
            {
                __callback();
            }
        }
        var onerror = function(__event)
        {
            //Otherwise we let the geometries attached to the untextured material
            LogMessage.Log("ViewerView Warning: Problem loading texture");
        }

        var loader = new THREE.TextureLoader();
		//allow cross origin texture loading
		loader.crossOrigin = '';

        //Adding "?t="+((new Date()).getTime()) allows to bypass the browser
        //cache since it is very likely that the file has been updated on the disk
        var texture = loader.load( __newURL+"?t="+((new Date()).getTime()), function(__texture){
            onload_tex(__textureURL, __texture);
        }, undefined, onerror);
        if(__keep_old_path || __keep_old_path == undefined)
        {
            //Should we keep the old texture path as reference ?
            texture.name = __textureURL;
        }
        else
        {
            //Should the new name (path) replace the old one ?
            texture.name = __newURL;
        }
    },

    /**
     * Parse an asset reference and
     * Precondition: __assetRef is a well formed URL
     * @param {String} __assetRef - asset reference url
     * @param {Obj} __rootnodes - the scene root nodes used to fetch the scene from the url
     * @returns {new_scene_id, refnode} if parsed succesfully using the expected type
     */
    AssetRefParse: function(__assetRef, __rootnodes)
    {
        var refnode_n_new_scene_id = {};

        //Extract url to retrieve the scene
        var idx_hash = __assetRef.lastIndexOf("#");
        if(idx_hash == -1 )
        {
            refnode_n_new_scene_id.new_scene_id = __assetRef;
            refnode_n_new_scene_id.refnode = __rootnodes[refnode_n_new_scene_id.new_scene_id];
        }else
        {
            refnode_n_new_scene_id.new_scene_id = __assetRef.substring(0, idx_hash);
            var scene_object = __rootnodes[refnode_n_new_scene_id.new_scene_id];
            if(scene_object)
            {
                var object_local_ref = __assetRef.substring(idx_hash+1);
                var object_type;
                do{
                    var index = object_local_ref.indexOf("/");
                    if (index == -1)
                    {
                        object_type = object_local_ref;
                        object_local_ref = "";
                    }else
                    {
                        object_type = object_local_ref.substring(0,index);
                    }
                    scene_object = scene_object[object_type];
                    object_local_ref = object_local_ref.substring(Math.min(object_local_ref.indexOf("/")+1, object_local_ref.length));
                }
                while(object_local_ref.length>0 && scene_object !== undefined)
                if(!scene_object) scene_object == null;
            }

            if (scene_object === null)
            {
                LogMessage.Log(refnode_n_new_scene_id.new_scene_id + " is null");
            }else if (scene_object == undefined)
            {
                LogMessage.Log(refnode_n_new_scene_id.new_scene_id + " not yet in cache or not existing");
            }

            refnode_n_new_scene_id.refnode = scene_object;

        }
        return refnode_n_new_scene_id;
    },

    /**
     * Function to convert an URL to a string that can be used as a key in a map
     * Replaces # with / and chops off http, https and ftp
     */
    URLToKeyString: function(URL)
    {
        var keyStr = URL.replace("#", "/");
        keyStr = keyStr.replace(/(ftp|http|https):\/\/?/, "");
        return keyStr;
    },
    /**
     * Register a node in the renderer cache if not already created and return the corresponding node object
     */
    RegisterObjectNode:function(__asset_absolute_path, __mode, __isleaf, __coincident_group_id, __add_ui_elem)
    {
        var __absolute_path = this.URLToKeyString(__asset_absolute_path);

        var nodeviewproperties = this.get('_renderercache').nodeviewproperties;
        var created = false;
        if(!nodeviewproperties.hasOwnProperty(__absolute_path))
        {
            nodeviewproperties[__absolute_path] = {"threejsnodes":{}}
        }
        if(!(nodeviewproperties[__absolute_path].threejsnodes.hasOwnProperty(__mode)))
        {
            nodeviewproperties[__absolute_path].threejsnodes[__mode] = {};
        }
        if(__coincident_group_id === undefined)
        {
            if(!(nodeviewproperties[__absolute_path].threejsnodes[__mode].hasOwnProperty("default_cg")))
            {
                this.set('_nodecounter',this.get('_nodecounter')+1);
                nodeviewproperties[__absolute_path].threejsnodes[__mode]["default_cg"] = new THREE.Object3D();
                created = true;
            }
        }
        else
        {
            if(!(nodeviewproperties[__absolute_path].threejsnodes[__mode].hasOwnProperty(__coincident_group_id)))
            {
                this.set('_nodecounter',this.get('_nodecounter')+1);
                nodeviewproperties[__absolute_path].threejsnodes[__mode][__coincident_group_id] = new THREE.Object3D();
                created = true;
            }
        }
        if(__mode == 'highlightnode' && (__add_ui_elem === undefined || __add_ui_elem === true))
        {
            this.AddUIElement(__absolute_path, __isleaf);
        }

        var returnnode = (__coincident_group_id !== undefined) ? nodeviewproperties[__absolute_path].threejsnodes[__mode][__coincident_group_id] : nodeviewproperties[__absolute_path].threejsnodes[__mode]["default_cg"];
        return {'node': returnnode, 'newlycreated': created, "register": true};
    },

    /**
     * Add an intermediate instance node to the scene and apply its instance properties
     */
    GetTHREEJSObjectHierarchyRec: function(__asset, __mode, __current_scene_id, __current_asset_prefix, __coincident_group_id, __asset_ref_unique_name, __parent_local_materials_refs, __parent_label_settings_map, __transmat, __forceSWInstPath, __parent_threejsnode)
    {
        //Retrieve local material references and lazy query materials from asset manager if needed
        var properties = __asset.AssetReferenceProperties;
        var local_materials_refs = {};//output: used for meshes materials since references are unique per instance, not in the whole scene
        if(__parent_local_materials_refs)
        {
            for (var i = 0, keys = Object.keys(__parent_local_materials_refs), l = keys.length; i < l; ++i)
            {
                var mat = keys[i];
                local_materials_refs[mat] = __parent_local_materials_refs[mat];
            }
        }
        if(properties)
        {
            var local_mats_refs = properties["materialmap"];//input
            if(local_mats_refs)
            {
                for (var i = 0, keys = Object.keys(local_mats_refs), l = keys.length; i < l; ++i)
                {
                    var mat = keys[i];
                    var materiallist = this.get('_renderercache')['materials']["materiallist"];
                    //Fetch from asset manager
                    var rootnodes = this.get('_model').get('_assetmanager').get('_cache').rootnodes

                    var material_key = local_mats_refs[mat][0]+((local_mats_refs[mat][1])?("/"+local_mats_refs[mat][1]):"");

                    //The next lines have to be considered differently if a parent instance
                    //node has to have precedence on the material setup over its children
                    //in that case the caller should give its own local_mats_refs
                    //(named parent_local_materials_refs here), inherited from the parent
                    //and this part should check if mat is already present in it. If present,
                    //then put that reference in the current local_materials_refs
                    //i.e.
                    //if (__parent_local_materials_refs.hasOwnProperty(mat))
                    //  local_materials_refs[mat] = __parent_local_materials_refs[mat]
                    //else //create the ref as usual using next lines

                    //RETRIEVE THE FRONT AND BACK ASSETS IN THE CURRENT SCENE ASSET POOL
                    var assets = rootnodes[__current_scene_id]['Assets'];
                    var front_mat_struct = null;
                    var back_mat_struct = null;
                    var mat_structs;
                    if(!materiallist.hasOwnProperty(material_key))
                    {
                        var shared_material = false;
                        if (!assets.hasOwnProperty(local_mats_refs[mat][0]))
                        {
                            LogMessage.Log("ViewerView Warning: Front material "+local_mats_refs[mat][0]+" not found");
                        }
                        else
                        {
                            var mat_node_front = assets[local_mats_refs[mat][0]];
                            front_mat_struct = this.LoadMaterial(mat_node_front, assets, __current_scene_id, false, false);
                        }

                        var backfound = false;
                        if(local_mats_refs[mat][1])
                        {
                            if (!assets.hasOwnProperty(local_mats_refs[mat][1]))
                            {
                                LogMessage.Log("ViewerView Warning: Back material "+local_mats_refs[mat][1]+" not found. Trying to assign front material.");
                            }
                            else
                            {
                                var mat_node_back = assets[local_mats_refs[mat][1]];
                                back_mat_struct = this.LoadMaterial(mat_node_back, assets, __current_scene_id, true, true);
                                backfound = true;
                            }
                        }

                        if (!backfound && assets.hasOwnProperty(local_mats_refs[mat][0]))
                        {
                            var mat_node_back = assets[local_mats_refs[mat][0]];
                            back_mat_struct = this.LoadMaterial(mat_node_back, assets, __current_scene_id, false, true);
                            shared_material = true;
                        }

                        //Update local material list
                        mat_structs = {"material_name":material_key,"front": front_mat_struct, "back":back_mat_struct, "sharedmaterial":shared_material}; //Used here
                        materiallist[material_key] = mat_structs;
                    }
                    else
                    {
                        //Retrieve front mat from material list
                        mat_structs = materiallist[material_key];
                    }

                    local_materials_refs[mat] = mat_structs; //Used to link meshes in 3DObjectHandler
                }
            }
        }

        //Depth first traversal to enable getting the bounding box of the leaf node if needed later

        var transmat = new THREE.Matrix4;

        if(!properties)
        {
            transmat.set(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0);
        }else
        {
            //Retrieve local transformation

            var local_trans = new THREE.Vector3(parseFloat(properties.x), parseFloat(properties.y), parseFloat(properties.z));
            var offset = new THREE.Vector3(0,0,0);

            ////NODE ROTATION + TRANSLATION
            if(properties.hasOwnProperty("orientation_matrix"))
            {
               transmat.set(  properties["orientation_matrix"][0],properties["orientation_matrix"][1],properties["orientation_matrix"][2],local_trans.x,
                            properties["orientation_matrix"][3],properties["orientation_matrix"][4],properties["orientation_matrix"][5],local_trans.y,
                            properties["orientation_matrix"][6],properties["orientation_matrix"][7],properties["orientation_matrix"][8],local_trans.z,
                            0.0, 0.0, 0.0, 1.0);
            }
            else
            {
                // for a full explanation of the Cape orientation constants see: https://wiki.esko.com/x/fhU_BQ
                switch(properties.orientation)
                {
                    case  "2":  transmat.set( 0.0,-1.0, 0.0, local_trans.x, 1.0, 0.0, 0.0, local_trans.y, 0.0, 0.0, 1.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case  "3":  transmat.set(-1.0, 0.0, 0.0, local_trans.x, 0.0,-1.0, 0.0, local_trans.y, 0.0, 0.0, 1.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case  "4":  transmat.set( 0.0, 1.0, 0.0, local_trans.x,-1.0, 0.0, 0.0, local_trans.y, 0.0, 0.0, 1.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case  "5":  transmat.set( 1.0, 0.0, 0.0, local_trans.x, 0.0, 0.0, 1.0, local_trans.y, 0.0,-1.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case  "6":  transmat.set( 0.0, 0.0,-1.0, local_trans.x, 1.0, 0.0, 0.0, local_trans.y, 0.0,-1.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case  "7":  transmat.set(-1.0, 0.0, 0.0, local_trans.x, 0.0, 0.0,-1.0, local_trans.y, 0.0,-1.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case  "8":  transmat.set( 0.0, 0.0, 1.0, local_trans.x,-1.0, 0.0, 0.0, local_trans.y, 0.0,-1.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case  "9":  transmat.set( 0.0, 0.0,-1.0, local_trans.x, 0.0, 1.0, 0.0, local_trans.y, 1.0, 0.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "10":  transmat.set( 0.0,-1.0, 0.0, local_trans.x, 0.0, 0.0,-1.0, local_trans.y, 1.0, 0.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "11":  transmat.set( 0.0, 0.0, 1.0, local_trans.x, 0.0,-1.0, 0.0, local_trans.y, 1.0, 0.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "12":  transmat.set( 0.0, 1.0, 0.0, local_trans.x, 0.0, 0.0, 1.0, local_trans.y, 1.0, 0.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "13":  transmat.set(-1.0, 0.0, 0.0, local_trans.x, 0.0, 1.0, 0.0, local_trans.y, 0.0, 0.0,-1.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "14":  transmat.set( 0.0,-1.0, 0.0, local_trans.x,-1.0, 0.0, 0.0, local_trans.y, 0.0, 0.0,-1.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "15":  transmat.set( 1.0, 0.0, 0.0, local_trans.x, 0.0,-1.0, 0.0, local_trans.y, 0.0, 0.0,-1.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "16":  transmat.set( 0.0, 1.0, 0.0, local_trans.x, 1.0, 0.0, 0.0, local_trans.y, 0.0, 0.0,-1.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "17":  transmat.set(-1.0, 0.0, 0.0, local_trans.x, 0.0, 0.0, 1.0, local_trans.y, 0.0, 1.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "18":  transmat.set( 0.0, 0.0,-1.0, local_trans.x,-1.0, 0.0, 0.0, local_trans.y, 0.0, 1.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "19":  transmat.set( 1.0, 0.0, 0.0, local_trans.x, 0.0, 0.0,-1.0, local_trans.y, 0.0, 1.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "20":  transmat.set( 0.0, 0.0, 1.0, local_trans.x, 1.0, 0.0, 0.0, local_trans.y, 0.0, 1.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "21":  transmat.set( 0.0, 0.0, 1.0, local_trans.x, 0.0, 1.0, 0.0, local_trans.y,-1.0, 0.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "22":  transmat.set( 0.0,-1.0, 0.0, local_trans.x, 0.0, 0.0, 1.0, local_trans.y,-1.0, 0.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "23":  transmat.set( 0.0, 0.0,-1.0, local_trans.x, 0.0,-1.0, 0.0, local_trans.y,-1.0, 0.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    case "24":  transmat.set( 0.0, 1.0, 0.0, local_trans.x, 0.0, 0.0,-1.0, local_trans.y,-1.0, 0.0, 0.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                    //case  "1":
                    default:    transmat.set( 1.0, 0.0, 0.0, local_trans.x, 0.0, 1.0, 0.0, local_trans.y, 0.0, 0.0, 1.0, local_trans.z, 0.0, 0.0, 0.0, 1.0); break;
                }
            }
        }

        if(__transmat instanceof THREE.Matrix4)
        {
            transmat.premultiply(__transmat);
        }

        var label_settings_map_merged = {};
        if(__parent_label_settings_map && !this.get('_parsedEntireSceneGraphOnce'))
        {
            for (var plsm = 0, plsm_keys = Object.keys(__parent_label_settings_map), n_plsmkeys = plsm_keys.length; plsm < n_plsmkeys; ++plsm)
            {
                label_settings_map_merged[plsm_keys[plsm]] = __parent_label_settings_map[plsm_keys[plsm]];
            }
        }

        if(properties && !this.get('_parsedEntireSceneGraphOnce'))
        {
            var labelSettingsMap_local = properties["labelsettingsmap"];
            if(labelSettingsMap_local)
            {
                for (var lsm = 0, lsm_keys = Object.keys(labelSettingsMap_local), n_lsmkeys = lsm_keys.length; lsm < n_lsmkeys; ++lsm)
                {
                    label_settings_map_merged[lsm_keys[lsm]] = labelSettingsMap_local[lsm_keys[lsm]];
                }
            }
        }

        //Transformation matrix is not used directly on the current node, but instead forwarded
        //recursively to GetTHREEJSObjectHierarchy and to addassettonode functions, to feed the instance array.
        // node.node.applyMatrix(transmat);

        return this.GetTHREEJSObjectHierarchy(__asset, __mode, __current_scene_id, __current_asset_prefix, __coincident_group_id, __asset_ref_unique_name, local_materials_refs, label_settings_map_merged, transmat, __forceSWInstPath, __parent_threejsnode);
    },

    IsAssetBoundingBoxLabelAssetType: function(__assetRef, rootnodes, __current_scene_id)
    {
        var refnode;
        if (this.get('_model').get('_assetmanager').get('_assetloader').IsURL(__assetRef))
            refnode = this.AssetRefParse(__assetRef, rootnodes).refnode;
        else
            refnode = rootnodes[__current_scene_id]['Assets'][__assetRef];

        if(refnode)
            return (refnode.AssetType == "Label" && refnode.Properties.hasOwnProperty('bounding_box'));
        else
            return false;
    },

    /**
     * Add specific object to the scene
     */
    GetTHREEJSObjectHierarchy: function(__asset, __mode, __current_scene_id, __current_asset_prefix, __coincident_group_id, __asset_ref_unique_name, __local_materials_refs, __local_label_settings_map, __transmat, __forceSWInstPath, __parent_threejsnode)
    {

        var rootnodes = this.get('_model').get('_assetmanager').get('_cache').rootnodes;
        var threejsnode;

        if (__asset == undefined)
        {
            LogMessage.Log("ViewerView Warning: Reference to __asset '"+ __current_asset_prefix+"' not yet loaded in cache. This node should be updated when asynchronuous data is loaded.");
            return {'node': null, 'newlycreated': false, 'register': false};
        }

        //Create a new threejs abstract node
        var isleaf = !((__asset.hasOwnProperty('Children')&&__asset.Children)||(__asset.hasOwnProperty('AssetRef')))

        if(__asset.hasOwnProperty('Children')&&__asset.Children)
        {
            //Before submiting the current node for registration as a separate branch
            //we have to know first if the children queried for being registered as
            //separate instances.
            //If at least one of the children requests it, we register the parent node
            //(this node) as a distinct node. As this operation is expensive for big scenes
            //with thousands of instances, it is preferable to use hardware instancing and,
            //therefore avoid registering thousands of items.

            var forceSoftwareInstance = false;
            if(__asset.hasOwnProperty("FlattenSubTree"))
            {
                forceSoftwareInstance = ! WV3DUtils.get('toBool')(__asset["FlattenSubTree"]);
            }

            var nodes_for_scene_graph = [];
            for (var i = 0, keys = Object.keys(__asset.Children), l = keys.length; i < l; ++i)
            {
                var id = keys[i];
                //! Loop through all children except for Label type assets. We loop through Label assets after generating ThreeJSNode for current parent node
                if(this.IsAssetBoundingBoxLabelAssetType(__asset.Children[id].AssetRef, rootnodes, __current_scene_id))
                    continue;

                var child_asset_name = __current_asset_prefix+"/"+id;
                var child = this.GetTHREEJSObjectHierarchyRec(__asset.Children[id], __mode, __current_scene_id, child_asset_name, __coincident_group_id, __asset.AssetRef, __local_materials_refs, __local_label_settings_map, __transmat, forceSoftwareInstance?child_asset_name:__forceSWInstPath, null);
                // child{"node": node, "register": register, "trans_mat": transmat}
                if(child.register && child.newlycreated)
                {
                    nodes_for_scene_graph.push(child.node);
                }
            }

            if(nodes_for_scene_graph.length > 0)
            {
                threejsnode = this.RegisterObjectNode(__current_asset_prefix, __mode, isleaf, __coincident_group_id);
                for (var i = 0, keys = Object.keys(nodes_for_scene_graph), l = keys.length; i < l; ++i)
                {
                    var node_to_add = keys[i];
                    threejsnode.node.add(nodes_for_scene_graph[node_to_add]);
                }

                //! Loop again through children of node but this time only for Label type assets
                for (var i = 0, keys = Object.keys(__asset.Children), l = keys.length; i < l; ++i)
                {
                    var id = keys[i];
                    if(!this.IsAssetBoundingBoxLabelAssetType(__asset.Children[id].AssetRef, rootnodes, __current_scene_id))
                        continue;

                    var child_asset_name = __current_asset_prefix+"/"+id;
                    var child = this.GetTHREEJSObjectHierarchyRec(__asset.Children[id], __mode, __current_scene_id, child_asset_name, __coincident_group_id, __asset.AssetRef, __local_materials_refs, __local_label_settings_map, __transmat, forceSoftwareInstance?child_asset_name:__forceSWInstPath, threejsnode);
                }
            }
            else
            {
                threejsnode = {'node': null, 'newlycreated': false, 'register': false};
            }

        }
        else if(__asset.hasOwnProperty('AssetRef'))
        {
            var refnode, new_scene_id, child_asset_name;
            if (this.get('_model').get('_assetmanager').get('_assetloader').IsURL(__asset.AssetRef))
            {
                //RETRIEVE THE ASSET IN AN EXTERNAL SCENE ASSET POOL
                var refnode_n_new_scene_id = this.AssetRefParse(__asset.AssetRef, rootnodes);
                refnode = refnode_n_new_scene_id.refnode;
                new_scene_id = refnode_n_new_scene_id.new_scene_id;
                child_asset_name = __current_asset_prefix;

            }else
            {
                //RETRIEVE THE ASSET IN THE CURRENT SCENE ASSET POOL
                refnode = rootnodes[__current_scene_id]['Assets'][__asset.AssetRef];
                new_scene_id = __current_scene_id;
                child_asset_name = __current_asset_prefix;
            }
            threejsnode = this.GetTHREEJSObjectHierarchy(refnode, __mode, new_scene_id, child_asset_name, __coincident_group_id, __asset.AssetRef, __local_materials_refs, __local_label_settings_map, __transmat, __forceSWInstPath, __parent_threejsnode);
        }
        else
        {
            var register = false;
            if(__asset.hasOwnProperty('Properties') && __asset.Properties && (__asset.AssetType == "Cuboid" || __asset.AssetType == "Mesh"))
            {
               //A NODE CONTAINING THE GEOMETRY
                var geometry_type = __asset.AssetType;
                var properties = __asset.Properties;
                var geometries = this.get('_renderercache')["geometries"];
                var materials = this.get('_renderercache')['materials'];
                var coplanarity_epsilon = this.get('_renderercache')['coplanarityepsilon'];
                var enable_boundary_hard_edges = this.get('_renderercache')['boundaryhardedges'];

                var unique_geometry_id = __current_scene_id + __asset_ref_unique_name.substring(__asset_ref_unique_name.lastIndexOf("/"));

                var register_objects = this.get('_3DObjectHandler').InstanciateGeometry(geometry_type, unique_geometry_id, __mode, properties, geometries, materials, __local_materials_refs, coplanarity_epsilon, enable_boundary_hard_edges, __coincident_group_id, __transmat, __forceSWInstPath, this);

                if(register_objects.register)
                {
                    register = true;
                    threejsnode = this.RegisterObjectNode(__current_asset_prefix, __mode, isleaf, __coincident_group_id);
                    for (var i = 0, keys = Object.keys(register_objects.objects), l = keys.length; i < l; ++i)
                    {
                        var registered_object = keys[i];
                        threejsnode.node.add(register_objects.objects[registered_object]);
                        if(register_objects.objects[registered_object].geometry.hasOwnProperty("useSoftwareInstancing")
                            && register_objects.objects[registered_object].geometry.useSoftwareInstancing)
                        {
                            register_objects.objects[registered_object].applyMatrix(__transmat);
                        }
                    }
                }
            }
            else if(__asset.hasOwnProperty('Properties') && __asset.Properties && __asset.AssetType == "Label" && !this.get('_parsedEntireSceneGraphOnce'))
            {
                //A label asset, which creation is deferred to the end of the scene construction
                var labelCreationCallback = function()
                {
                    this.LoadLabel(rootnodes[__current_scene_id], __current_asset_prefix, __asset, __asset_ref_unique_name, __local_label_settings_map, __transmat, __parent_threejsnode);
                }
                this.get("_labelCreationCallbackList").push(labelCreationCallback.bind(this));
            }

            if(!register)
            {
                threejsnode = {'node': null, 'newlycreated': false, 'register': false};
            }
        }

        if(__asset.hasOwnProperty('AssetType') && __asset.AssetType == "Scene" )
        {
            //We consider scene has been entirely parse only if this asset is the root node
            if(this.get("_model").get("SceneId") == __asset.AssetId) this.set('_parsedEntireSceneGraphOnce', true);
            //Post process traversal is triggered at the end of the whole recursive traversal process
            if(threejsnode.node)
            {
                var setup_instanciation_array = function ( obj )
                {
                    if(obj instanceof THREE.Mesh)
                    {
                        if(obj.geometry instanceof THREE.InstancedBufferGeometry)
                        {
                            //Update instance count
                            if( obj.geometry.attributes["inst_translation"].jarray &&
                                obj.geometry.attributes["inst_rotation"].jarray &&
                                obj.geometry.attributes["inst_scale"].jarray &&
                                obj.geometry.attributes["inst_translation"].jarray.length/3 > 1 )
                            {
                                obj.geometry.maxInstancedCount = obj.geometry.attributes["inst_translation"].jarray.length/3;
                                obj.geometry.adjacentfacesedgeslines.maxInstancedCount = obj.geometry.maxInstancedCount;
                                //Creating new bigger array using ALL instances
                                var newoffsets = new Float32Array( obj.geometry.attributes["inst_translation"].jarray );
                                var neworientations = new Float32Array( obj.geometry.attributes["inst_rotation"].jarray );
                                var newscales = new Float32Array( obj.geometry.attributes["inst_scale"].jarray );
                                obj.geometry.attributes["inst_translation"].array = newoffsets;
                                obj.geometry.attributes["inst_rotation"].array = neworientations;
                                obj.geometry.attributes["inst_scale"].array = newscales;
                                obj.geometry.adjacentfacesedgeslines.attributes["inst_translation"].array = newoffsets;
                                obj.geometry.adjacentfacesedgeslines.attributes["inst_rotation"].array = neworientations;
                                obj.geometry.adjacentfacesedgeslines.attributes["inst_scale"].array = newscales;
                            }
                        }
                    }
                };
                threejsnode.node.traverse(setup_instanciation_array);
            }
        }
        //return the node to the scene
        return threejsnode;
    },

    RotateOutOfBoundsVisibleLabels: function(__camera, __globalBBox, __screenwidth, __screenheight, __marginscreenwidth, __marginscreenheight)
    {
        var success = true;
        for (var idx_key = 0, keys = Object.keys(this.get('_renderercache')['labels']['list']), l = keys.length; idx_key < l; ++idx_key)
        {
            var id = keys[idx_key];
            var label_asset = this.get('_renderercache')['labels']['list'][id];
            //Get the labels bounding box
            var label_bbox = this.GetLabelBoundingBox(label_asset, label_asset["orientation"]);
            var modified_layout = undefined;
            //Rotate label if out of bounds in one direction
            if(label_asset["orientation"] != "vertical")
            {
                if( label_bbox.max.x > __marginscreenwidth || label_bbox.min.x < -__marginscreenwidth)
                {
                    label_asset["threejs_sprite"].material.rotation = 1.5708;
                    modified_layout = "vertical";
                }
                else
                {
                    if( label_bbox.max.y > __marginscreenheight || label_bbox.min.y < -__marginscreenheight)
                    {
                        success = false;
                    }
                    label_asset["threejs_sprite"].material.rotation = 0;
                }
            }
            else
            {
                if( label_bbox.max.y > __marginscreenheight || label_bbox.min.y < -__marginscreenheight)
                {
                    label_asset["threejs_sprite"].material.rotation = 0;
                    modified_layout = "horizontal";
                }
                else
                {
                    if( label_bbox.max.x > __marginscreenwidth || label_bbox.min.x < -__marginscreenwidth)
                    {
                        success = false;
                    }
                    label_asset["threejs_sprite"].material.rotation = 1.5708;
                }
            }
            if(modified_layout)
            {
                //Update the position (label might end up in a different different place)
                this.UpdateLabelPosition(label_asset, __camera, __screenwidth/2, __screenheight/2, modified_layout);
                //Get its new bbox
                label_bbox = this.GetLabelBoundingBox(label_asset, modified_layout);
                //Test against it new bounding box
                if( label_bbox.max.x > __marginscreenwidth ||
                    label_bbox.min.x < -__marginscreenwidth ||
                    label_bbox.max.y > __marginscreenheight ||
                    label_bbox.min.y < -__marginscreenheight )
                {
                    success = false;
                }
            }
            //Now update the global bounding box
            __globalBBox.union(label_bbox);
        }
        return success;
    },

    GetLabelBoundingBox: function(__label_asset, __overrideLayout)
    {
        var sprite_object = __label_asset["threejs_sprite"];
        var label_bbox = new THREE.Box2();
        var spriteDimensions = {"spriteWidth":sprite_object.scale.x ,"spriteHeight":sprite_object.scale.y};
        var spriteHWidth = 0.5 * ((( __overrideLayout ? __overrideLayout : __label_asset["orientation"]) == "vertical") ? spriteDimensions.spriteHeight : spriteDimensions.spriteWidth);
        var spriteHHeight = 0.5 * ((( __overrideLayout ? __overrideLayout : __label_asset["orientation"]) == "vertical") ? spriteDimensions.spriteWidth : spriteDimensions.spriteHeight);

        label_bbox.expandByPoint(new THREE.Vector2(sprite_object.position.x - spriteHWidth, sprite_object.position.y - spriteHHeight));
        label_bbox.expandByPoint(new THREE.Vector2(sprite_object.position.x - spriteHWidth, sprite_object.position.y + spriteHHeight));
        label_bbox.expandByPoint(new THREE.Vector2(sprite_object.position.x + spriteHWidth, sprite_object.position.y - spriteHHeight));
        label_bbox.expandByPoint(new THREE.Vector2(sprite_object.position.x + spriteHWidth, sprite_object.position.y + spriteHHeight));
        return label_bbox;
    },

    UpdateLabelsPositions: function(__camera, __overrideWidth, __overrideHeight)
    {
        __camera.updateMatrixWorld();
        var screenwidth = __overrideWidth?__overrideWidth:this.get('_context').clientWidth;
        var screenheight = __overrideHeight?__overrideHeight:this.get('_context').clientHeight;

        for (var idx_key = 0, keys = Object.keys(this.get('_renderercache')['labels']['list']), l = keys.length; idx_key < l; ++idx_key)
        {
            var id = keys[idx_key];
            var label_asset = this.get('_renderercache')['labels']['list'][id];
            this.UpdateLabelPosition(label_asset, __camera, screenwidth/2, screenheight/2);
        }
    },

    UpdateLabelPosition: function(__label_asset, __camera, __screenwidth, __screenheight, __overrideLayout)
    {
        //Position each label contained in rendercache list
        var sprite_object = __label_asset["threejs_sprite"];

        //Get the list of attachment points
        var attachment_points = [];
        var attConvexHull = []; // convex hull of pointlist or edges endpoints
        var vcenter; // projected center, in case of ellipse label
        switch(__label_asset["bounds_type"])
        {
            case "edge_list":
                for (var ed = 0, ed_keys = Object.keys(__label_asset.edge_list), n_eds = ed_keys.length; ed < n_eds; ++ed)
                {
                    var edge = __label_asset.edge_list[ed_keys[ed]];
                    var v1 = new THREE.Vector3(edge[0],edge[1],edge[2]);
                    v1.project(__camera);
                    attConvexHull.push(new THREE.Vector2(v1.x * __screenwidth, v1.y * __screenheight));
                    var v2 = new THREE.Vector3(edge[3],edge[4],edge[5]);
                    v2.project(__camera);
                    attConvexHull.push(new THREE.Vector2(v2.x * __screenwidth, v2.y * __screenheight));
                    var att_pt = v1.add(v2).multiplyScalar(.5);

                    //Offset mid point of the edge in the direction of user preference
                    attachment_points.push(new THREE.Vector2(att_pt.x * __screenwidth, att_pt.y * __screenheight));
                }
            break;
            case "point_list":
                for (var ed = 0, ed_keys = Object.keys(__label_asset.point_list), n_eds = ed_keys.length; ed < n_eds; ++ed)
                {
                    var vertex = __label_asset.point_list[ed_keys[ed]];
                    var att_pt = new THREE.Vector3(vertex[0],vertex[1],vertex[2]);
                    att_pt.project(__camera);

                    attConvexHull.push(new THREE.Vector2(att_pt.x * __screenwidth, att_pt.y * __screenheight));

                    attachment_points.push(new THREE.Vector2(att_pt.x * __screenwidth, att_pt.y * __screenheight));
                }
            break;
            case "ellipse":
                //We use the equations mentioned at http://mathforum.org/library/drmath/view/63373.html
                var center = __label_asset["ellipse"]["center"];
                var vec_radius_1 = __label_asset["ellipse"]["radius1_vec"];
                var vec_radius_2 = __label_asset["ellipse"]["radius2_vec"];

                //Find the outer most points based on ellipse plane's direction w.r.t. camera
                //Project ellipse vertices into view space
                vcenter = new THREE.Vector3(center[0],center[1],center[2]);
                var vradius1 = new THREE.Vector3(vec_radius_1[0],vec_radius_1[1],vec_radius_1[2]);
                var vradius2 = new THREE.Vector3(vec_radius_2[0],vec_radius_2[1],vec_radius_2[2]);

                var ellipse_plane_normal = vradius1.clone().cross(vradius2).normalize();
                ellipse_plane_normal.x = Math.abs(ellipse_plane_normal.x);
                ellipse_plane_normal.y = Math.abs(ellipse_plane_normal.y);
                ellipse_plane_normal.z = Math.abs(ellipse_plane_normal.z);
                var horizontal_layout = ellipse_plane_normal.angleTo(new THREE.Vector3(0.,1.,0.)) < Math.PI * 0.25;

                //Radii are converted into points before projection
                vradius1.add(vcenter);
                vradius2.add(vcenter);

                vcenter.project(__camera);
                vradius1.project(__camera);
                vradius2.project(__camera);

                //Radii are converted back to vectors after projection
                vradius1.sub(vcenter);
                vradius2.sub(vcenter);

                //Ensure radius 1 is the longest of the two radii (swap them if necessary)
                if(vradius1.length() < vradius2.length())
                {
                    vradius1 = [vradius2, vradius2 = vradius1][0];
                }

                //At this stage vcenter, vradius1, and vradius2 contain the projected ellipse in view space
                var a = vradius1.length(), b = vradius2.length();
                var unit_vradius1 = vradius1.clone().normalize();
                var unit_vradius2 = vradius2.clone().normalize();
                var xc,yc,zc,x1,x2,x3,x4,y1,y2,y3,y4,t1,t2,t3,t4,att_pt1,att_pt2,att_pt3,att_pt4;
                xc = vcenter.x;
                yc = vcenter.y;
                zc = vcenter.z;

                //intersections with Cx axis
                t1 = Math.atan2(-a * unit_vradius1.y,b * unit_vradius2.y);
                //min_x
                x1 = a*Math.cos(t1)*unit_vradius1.x + b*Math.sin(t1)*unit_vradius2.x;
                y1 = a*Math.cos(t1)*unit_vradius1.y + b*Math.sin(t1)*unit_vradius2.y;

                //max_x
                t2 = t1 + Math.PI;
                x2 = a*Math.cos(t2)*unit_vradius1.x + b*Math.sin(t2)*unit_vradius2.x;
                y2 = a*Math.cos(t2)*unit_vradius1.y + b*Math.sin(t2)*unit_vradius2.y;

                //intersections with Cy axis
                t3 = Math.atan2(-a * unit_vradius1.x,b * unit_vradius2.x);
                t3 = t2 + Math.PI*0.5;
                //min_y
                x3 = a*Math.cos(t3)*unit_vradius1.x + b*Math.sin(t3)*unit_vradius2.x;
                y3 = a*Math.cos(t3)*unit_vradius1.y + b*Math.sin(t3)*unit_vradius2.y;

                //max_y
                t4 = t3 + Math.PI;
                x4 = a*Math.cos(t4)*unit_vradius1.x + b*Math.sin(t4)*unit_vradius2.x;
                y4 = a*Math.cos(t4)*unit_vradius1.y + b*Math.sin(t4)*unit_vradius2.y;

                x1 = x1 + xc;
                y1 = y1 + yc;
                x2 = x2 + xc;
                y2 = y2 + yc;
                x3 = x3 + xc;
                y3 = y3 + yc;
                x4 = x4 + xc;
                y4 = y4 + yc;
                att_pt1 = new THREE.Vector2(x1 * __screenwidth, y1 * __screenheight);
                att_pt2 = new THREE.Vector2(x2 * __screenwidth, y2 * __screenheight);
                att_pt3 = new THREE.Vector2(x3 * __screenwidth, y3 * __screenheight);
                att_pt4 = new THREE.Vector2(x4 * __screenwidth, y4 * __screenheight);

                //From these attachement points, we want to select the ones that are
                //located far from each other.

                var sel_att_pt1,sel_att_pt2;
                var lengtp1p2 = att_pt1.distanceToSquared(att_pt2);
                var lengtp1p3 = att_pt1.distanceToSquared(att_pt3);
                var lengtp1p4 = att_pt1.distanceToSquared(att_pt4);
                var lengtp2p3 = att_pt2.distanceToSquared(att_pt3);
                var lengtp2p4 = att_pt2.distanceToSquared(att_pt4);
                var lengtp3p4 = att_pt3.distanceToSquared(att_pt4);
                var max_length = 0;

                if(lengtp1p2 >= max_length)
                {
                    max_length = lengtp1p2;
                    sel_att_pt1 = att_pt1;
                    sel_att_pt2 = att_pt2;
                }
                if(lengtp1p3 >= max_length)
                {
                    max_length = lengtp1p3;
                    sel_att_pt1 = att_pt1;
                    sel_att_pt2 = att_pt3;
                }
                if(lengtp1p4 >= max_length)
                {
                    max_length = lengtp1p4;
                    sel_att_pt1 = att_pt1;
                    sel_att_pt2 = att_pt4;
                }
                if(lengtp2p3 >= max_length)
                {
                    max_length = lengtp2p3;
                    sel_att_pt1 = att_pt2;
                    sel_att_pt2 = att_pt3;
                }
                if(lengtp2p4 >= max_length)
                {
                    max_length = lengtp2p4;
                    sel_att_pt1 = att_pt2;
                    sel_att_pt2 = att_pt4;
                }
                if(lengtp3p4 >= max_length)
                {
                    max_length = lengtp3p4;
                    sel_att_pt1 = att_pt3;
                    sel_att_pt2 = att_pt4;
                }

                attachment_points.push(sel_att_pt1);
                attachment_points.push(sel_att_pt2);
                attConvexHull.push(new THREE.Vector2(vcenter.x, vcenter.y));
            break;
            default:
                LogMessage.Log("Warning! Unsupported label_asset type.");
                return;
        }

        var convexHullCtr = undefined, nbHullPts = undefined;
        var x = 0, y = 0, n_attps = attachment_points.length;
        switch( n_attps )
        {
            case 0:
                LogMessage.Log("Warning: no attachment point found.");
                break;
            case 1:
                x = attachment_points[0].x;
                y = attachment_points[0].y;
                break;
            default:
                //Get 2D convex hull of attachment points
                var attPtsProjBBox = new THREE.Box2();
                for (var attPtIdx = 0, n_attpt = attachment_points.length; attPtIdx < n_attpt; ++attPtIdx)
                {
                    var attPt = attachment_points[attPtIdx];
                    attPtsProjBBox.expandByPoint(new THREE.Vector2(attPt.x, attPt.y));
                }

                //Check which atachment points are valid relatively to itself
                var valid_att_points = [];
                for (var attPtIdx = 0, n_attpt = attachment_points.length; attPtIdx < n_attpt; ++attPtIdx)
                {
                    var attPt = attachment_points[attPtIdx];

                    if (    attPt.x == attPtsProjBBox.min.x || attPt.x == attPtsProjBBox.max.x ||
                            attPt.y == attPtsProjBBox.min.y || attPt.y == attPtsProjBBox.max.y )
                    {
                        valid_att_points.push(attPt);
                    }
                }

                //if no valid point (meaning there is probably an issue)
                switch(valid_att_points.length)
                {
                    case 0:
                        //calculate main direction for the entire point cloud and choose one anyway according to preferences
                        console.log("There is probably an issue here. at least for vertices and edges");
                        break;
                    case 1:
                        //if only one is valid choose it
                        x = valid_att_points[0].x,
                        y = valid_att_points[0].y;
                        break;
                    default:
                        //using user preferences we have to select the best attachment point
                        if(__label_asset.preferred_loc_h == "center" && __label_asset.preferred_loc_v == "center")
                        {
                            nbHullPts = WV3DMath.CreateConvexHullFromPointlist(attConvexHull);
                            // convexHullCtr = WV3DMath.ConvexHullCenter( attConvexHull, nbHullPts );
                            // We use all points contained in convex hull list otherwise the center is wrongly calculated
                            // when convex hull vertices are alined (easy to spot in orthographics view)
                            convexHullCtr = WV3DMath.ConvexHullCenter( attConvexHull, attConvexHull.length-1 );
                            x = convexHullCtr.x;
                            y = convexHullCtr.y;
                        }
                        else if(__label_asset.preferred_loc_h == "center" || __label_asset.preferred_loc_v == "center")
                        {
                            x = valid_att_points[0].x,
                            y = valid_att_points[0].y;
                            for (var attPtIdx = 1, n_attpt = valid_att_points.length; attPtIdx < n_attpt; ++attPtIdx)
                            {
                                var attPt = valid_att_points[attPtIdx];
                                if(__label_asset.preferred_loc_h == "left")
                                {
                                    if(attPt.x < x)
                                    {
                                        x = attPt.x,
                                        y = attPt.y;
                                    }
                                }
                                else if(__label_asset.preferred_loc_h == "right")
                                {
                                    if(attPt.x > x)
                                    {
                                        x = attPt.x,
                                        y = attPt.y;
                                    }
                                }
                                else if(__label_asset.preferred_loc_v == "bottom")
                                {
                                    if(attPt.y < y)
                                    {
                                        x = attPt.x,
                                        y = attPt.y;
                                    }
                                }
                                else if(__label_asset.preferred_loc_v == "top")
                                {
                                    if(attPt.y > y)
                                    {
                                        x = attPt.x,
                                        y = attPt.y;
                                    }
                                }
                                else
                                {
                                    LogMessage.Log("Unrecognised Label user preference !");
                                    x = 0;
                                    y = 0;
                                }
                            }
                        }
                        else
                        {
                            //calculate main direction for the point cloud and choose one according to preferences
                            var cumul_diff = new THREE.Vector2();
                            for (var attPtIdx = 0, n_attpt = valid_att_points.length; attPtIdx < n_attpt; ++attPtIdx)
                            {
                                var attPt = valid_att_points[attPtIdx];
                                for (var attPtIdx2 = attPtIdx + 1; attPtIdx2 < n_attpt; ++attPtIdx2)
                                {
                                    var attPt2 = valid_att_points[attPtIdx2];
                                    var delta = attPt.clone().sub(attPt2);
                                    cumul_diff.add(new THREE.Vector2(Math.abs(delta.x), Math.abs(delta.y)));
                                }
                            }
                            x = valid_att_points[0].x,
                            y = valid_att_points[0].y;
                            if(cumul_diff.x > cumul_diff.y)
                            {
                                // console.log("horizontal layout ! dx/dy: "+cumul_diff.x+"/"+cumul_diff.y);
                                //select right or left most vertex from valid_att_points according to user preferences
                                for (var attPtIdx = 1, n_attpt = valid_att_points.length; attPtIdx < n_attpt; ++attPtIdx)
                                {
                                    var attPt = valid_att_points[attPtIdx];
                                    if(__label_asset.preferred_loc_h == "left")
                                    {
                                        if(attPt.x < x)
                                        {
                                            x = attPt.x,
                                            y = attPt.y;
                                        }
                                    }
                                    else if(__label_asset.preferred_loc_h == "right")
                                    {
                                        if(attPt.x > x)
                                        {
                                            x = attPt.x,
                                            y = attPt.y;
                                        }
                                    }
                                    else
                                    {
                                        LogMessage.Log("Unrecognised horizontal layout: "+__label_asset.preferred_loc_h);
                                    }
                                }
                            }
                            else
                            {
                                // console.log("vertical layout ! dx/dy: "+cumul_diff.x+"/"+cumul_diff.y);
                                //select top or bottom most vertex from valid_att_points according to user preferences
                                for (var attPtIdx = 1, n_attpt = valid_att_points.length; attPtIdx < n_attpt; ++attPtIdx)
                                {
                                    var attPt = valid_att_points[attPtIdx];
                                    if(__label_asset.preferred_loc_v == "bottom")
                                    {
                                        if(attPt.y < y)
                                        {
                                            x = attPt.x,
                                            y = attPt.y;
                                        }
                                    }
                                    else if(__label_asset.preferred_loc_v == "top")
                                    {
                                        if(attPt.y > y)
                                        {
                                            x = attPt.x,
                                            y = attPt.y;
                                        }
                                    }
                                    else
                                    {
                                        LogMessage.Log("Unrecognised vertical layout: "+__label_asset.preferred_loc_v);
                                    }
                                }
                            }
                        }
                }
                break;
        }

        //Offset labels
        var px_offset_x = 0, px_offset_y = 0;

        // calculate length of string + scaling
        var spriteDimensions = {"spriteWidth":__label_asset["threejs_sprite"].scale.x ,"spriteHeight":__label_asset["threejs_sprite"].scale.y};
        var spriteHWidth = 0.5 * ((( __overrideLayout? __overrideLayout : __label_asset["orientation"]) == "vertical") ? spriteDimensions.spriteHeight : spriteDimensions.spriteWidth);
        var spriteHHeight = 0.5 * ((( __overrideLayout? __overrideLayout : __label_asset["orientation"]) == "vertical") ? spriteDimensions.spriteWidth : spriteDimensions.spriteHeight);

        var user_pixel_offset = new THREE.Vector2(__label_asset.pixel_offset[0], __label_asset.pixel_offset[1]);
        if ( n_attps == 1 && __label_asset["bounds_type"] == "point_list" || (__label_asset.preferred_loc_h == "center" && __label_asset.preferred_loc_v == "center"))
        {
            // don't do anything here as we can't derive a horizontal/vertical layout from a single point.
            // only add user offset if any
            var x_dir = 0, y_dir = 0;
            if(__label_asset.preferred_loc_h == "right")
            {
                x_dir = 1;
            }
            else if(__label_asset.preferred_loc_h == "left")
            {
                x_dir = -1;
            }
            if(__label_asset.preferred_loc_v == "top")
            {
                y_dir = 1;
            }
            else if(__label_asset.preferred_loc_v == "bottom")
            {
                y_dir = -1;
            }
            px_offset_x += user_pixel_offset.x*Math.sign(x_dir);
            px_offset_y += user_pixel_offset.y*Math.sign(y_dir);
        }
        else if(__label_asset["bounds_type"] == "ellipse") // for an ellipse, push away from the center of the ellipse.
        {
            // vCenter in camera space, but not in pixel coordinates
            var center = new THREE.Vector2(vcenter.x * __screenwidth, vcenter.y * __screenheight);
            px_offset_x = x - center.x + user_pixel_offset.x;
            px_offset_y = y - center.y + user_pixel_offset.y;
        }
        else // edge list or multiple points
        {
            // calculate convex hull and calculate direction in which to push the attachment point
            var n_hullPts = nbHullPts? nbHullPts : WV3DMath.CreateConvexHullFromPointlist(attConvexHull);
            // var center = convexHullCtr? convexHullCtr : WV3DMath.ConvexHullCenter( attConvexHull, n_hullPts );
            var center = convexHullCtr? convexHullCtr : WV3DMath.ConvexHullCenter( attConvexHull, attConvexHull.length-1 );

            var dir = new THREE.Vector2(x - center.x, y - center.y).normalize();

            // push label by moving according to layout and user preference
            var cum_diff_hull = new THREE.Vector2();
            for (var attPtIdx = 0, n_attpt = attConvexHull.length-1; attPtIdx < n_attpt; ++attPtIdx)
            {
                var attPt = attConvexHull[attPtIdx];
                for (var attPtIdx2 = attPtIdx + 1; attPtIdx2 < n_attpt; ++attPtIdx2)
                {
                    var attPt2 = attConvexHull[attPtIdx2];
                    var delta = attPt.clone().sub(attPt2);
                    cum_diff_hull.add(new THREE.Vector2(Math.abs(delta.x), Math.abs(delta.y)));
                }
            }
            var x_dir,y_dir;
            if(cum_diff_hull.x > cum_diff_hull.y)
            {
                if(__label_asset.preferred_loc_v == "top")
                {
                    x_dir = 0;
                    y_dir = 1;
                }
                else if(__label_asset.preferred_loc_v == "bottom")
                {
                    x_dir = 0;
                    y_dir = -1;
                }
                else if(__label_asset.preferred_loc_h == "right")
                {
                    x_dir = 1;
                    y_dir = 0;
                }
                else if(__label_asset.preferred_loc_h == "left")
                {
                    x_dir = -1;
                    y_dir = 0;
                }
            }
            else
            {
                if(__label_asset.preferred_loc_h == "right")
                {
                    x_dir = 1;
                    y_dir = 0;
                }
                else if(__label_asset.preferred_loc_h == "left")
                {
                    x_dir = -1;
                    y_dir = 0;
                }
                else if(__label_asset.preferred_loc_v == "top")
                {
                    x_dir = 0;
                    y_dir = 1;
                }
                else if(__label_asset.preferred_loc_v == "bottom")
                {
                    x_dir = 0;
                    y_dir = -1;
                }
            }
            var epsilon = 0.001;
            px_offset_x = (spriteHWidth + epsilon) * x_dir;
            px_offset_y = (spriteHHeight + epsilon) * y_dir;

            //Move iteratively the label to ensure that it is entirely (corners) outside of the convex hull
            var nb_max_iterations = 10000;//Safety for following loop
            var contains = true;
            while( nb_max_iterations && contains )
            {
                nb_max_iterations--;
                var label_corners = [   new THREE.Vector2(x+spriteHWidth+px_offset_x,y+spriteHHeight+px_offset_y),
                                        new THREE.Vector2(x-spriteHWidth+px_offset_x,y+spriteHHeight+px_offset_y),
                                        new THREE.Vector2(x+spriteHWidth+px_offset_x,y-spriteHHeight+px_offset_y),
                                        new THREE.Vector2(x-spriteHWidth+px_offset_x,y-spriteHHeight+px_offset_y)   ];

                //Test box corners against convex hull
                if( WV3DMath.ConvexHullContainsPoint( attConvexHull, n_hullPts, label_corners[0])||
                    WV3DMath.ConvexHullContainsPoint( attConvexHull, n_hullPts, label_corners[1])||
                    WV3DMath.ConvexHullContainsPoint( attConvexHull, n_hullPts, label_corners[2])||
                    WV3DMath.ConvexHullContainsPoint( attConvexHull, n_hullPts, label_corners[3]))
                {
                    px_offset_x+= dir.x;
                    px_offset_y+= dir.y;
                    continue;
                }

                //Test each convex hull vertex against label bbox
                var label_box = new THREE.Box2();
                label_box.setFromPoints(label_corners);
                for (var idx_cvx_hull = 0; idx_cvx_hull < attConvexHull; ++idx_cvx_hull)
                {
                    if(label_box.containsPoint(attConvexHull[idx_cvx_hull]))
                    {
                        px_offset_x+= dir.x;
                        px_offset_y+= dir.y;
                        continue;
                    }
                }

                contains = false;
            }
            if(nb_max_iterations == 0)
            {
                LogMessage.Log("ViewerView Warning: too many iterations when pushing label outside of its convex hull.");
            }
            px_offset_x += user_pixel_offset.x*Math.sign(x_dir);
            px_offset_y += user_pixel_offset.y*Math.sign(y_dir);
        }

        //Update labels positions
        sprite_object.position.set( x + px_offset_x, y + px_offset_y, 0);
    },

    /**
     * Set camera settings according to presets
     */
    CameraUpdate: function(__redraw, __overrideWidth, __overrideHeight)
    {
        //Update bounding boxes
        var sceneBBox = this.get('_3JSsceneBBox').clone();
        if(sceneBBox.isEmpty()) return;

        // Adjust camera to scene (using focus bounding box)
        this.SetCamera(this.get('_3JSsceneBBoxFocus'), __overrideWidth, __overrideHeight);

        // Setup the direction light to provide light from above right of camera
        var camPos = this.get('_3JScamera').position.clone();
        var camUp = this.get('_3JScamera').up.clone().normalize();
        var camPoint = sceneBBox.center().clone().sub(camPos);

        camPoint.normalize();
        var camRight = camPoint.clone().cross(camUp);

        var lightDir = camPoint.clone().sub(camUp.clone().multiplyScalar(0.5)).sub(camRight.clone().multiplyScalar(0.7)); //camPoint - 0.3 * camUp - 0.3 * camRight

        for (var i = 0, cg_keys = Object.keys(this.get('_renderercache')['lights']['dir_light']), l = cg_keys.length; i < l; ++i)
        {
            var dir_light = this.get('_renderercache')['lights']['dir_light'][cg_keys[i]];
            if(dir_light)
            {
                //for default and designer quality materials
                dir_light.position.set(-lightDir.x, -lightDir.y, -lightDir.z);
                dir_light.needsUpdate = true;
           }
        }

        //Also need to update near and far clipping planes of camera relatively to the reflected scene
        var reflectedpoint = this.get('_3JSsceneBBox').min.clone();
        var trans_reflection = new THREE.Vector3(0.,this.get('_3JSsceneBBox').size().y + this.get('_mirror_offset'),0.);
        reflectedpoint.sub(trans_reflection);
        sceneBBox.expandByPoint(reflectedpoint);

        // Adjust camera to scene
        this.UpdateCamNearAndFar(sceneBBox, __overrideWidth, __overrideHeight);

        //Check if scene is outside of frustum
        var intersect_frustum = false;
        // update camera matrices and frustum
        if ( this.get('_3JScamera').parent === null ) this.get('_3JScamera').updateMatrixWorld();

        var _projScreenMatrix = new THREE.Matrix4();
        var _frustum = new THREE.Frustum();
        this.get('_3JScamera').matrixWorldInverse.getInverse( this.get('_3JScamera').matrixWorld );
        _projScreenMatrix.multiplyMatrices( this.get('_3JScamera').projectionMatrix, this.get('_3JScamera').matrixWorldInverse );
        _frustum.setFromMatrix( _projScreenMatrix );

        var bboxgeom = new THREE.Geometry();
        bboxgeom.vertices.push(
            this.get('_3JSsceneBBox').min,
            this.get('_3JSsceneBBox').max
        );
        var bboxmesh = new THREE.Mesh(bboxgeom, null);

        this.set("_obj_in_frustum_prev", this.get("_obj_in_frustum"));
        var intersect_frustum = _frustum.intersectsObject(bboxmesh);
        this.set("_obj_in_frustum", intersect_frustum);

        if(__redraw === undefined || __redraw)
            this.Render();
    },

    /**
     * Set the angular position of the camera relative to the default front position of the scene
     *
     * @function
     * @param {Number} azimuthAngle - the walk-around angle measured counterclockwise from the default front view
     * @param {Number} elevationAngle - the elevantion angle - positive values lift the camera above the scene centre
     */
    SetCameraAngles: function(__azimuthAngle, __elevationAngle, __redraw)
    {
        var prev_az = this.get('_renderercache').cameras["defaultcamera"].azimuthAngle;
        var prev_el = this.get('_renderercache').cameras["defaultcamera"].elevationAngle;
        this.get('_renderercache').cameras["defaultcamera"].azimuthAngle = __azimuthAngle;
        this.get('_renderercache').cameras["defaultcamera"].elevationAngle = __elevationAngle;
        this.CameraUpdate(__redraw);

        if(!this.get("_obj_in_frustum") && this.get("_obj_in_frustum_prev"))
        {
            //Object was previously in bounds but went out of frustum because
            //of these new parameters: reverting to previous state
            this.get('_renderercache').cameras["defaultcamera"].azimuthAngle = prev_az;
            this.get('_renderercache').cameras["defaultcamera"].elevationAngle = prev_el;
            this.CameraUpdate(__redraw);
        }
    },

    GetCameraAngles: function()
    {
        return {azimuth: this.get('_renderercache').cameras["defaultcamera"].azimuthAngle,
                elevation: this.get('_renderercache').cameras["defaultcamera"].elevationAngle};
    },

    /**
     * Set the size of the scene within the camera view. This is achieved by automatically
     * adjusting the camera properties (e.g. changing the camera distance for a perspective camera)
     *
     * @function
     * @param {Number} scale - the size of the scene bounding box in the view. A value of 1 will result in the scene filling the view in at least one dimension
     */
    SetCameraScale: function(__scale, __redraw)
    {
        var prev_scale = this.get('_renderercache').cameras["defaultcamera"].scale;
        this.get('_renderercache').cameras["defaultcamera"].scale = __scale;
        this.CameraUpdate(__redraw);

        if(!this.get("_obj_in_frustum") && this.get("_obj_in_frustum_prev"))
        {
            //Object was previously in bounds but went out of frustum because
            //of these new parameters: reverting to previous state
            this.get('_renderercache').cameras["defaultcamera"].scale = prev_scale;
            this.CameraUpdate(__redraw);
        }
    },

    GetCameraScale: function()
    {
        return this.get('_renderercache').cameras["defaultcamera"].scale;
    },


    /**
     * Set/Get the panning offset of the camera.
     *
     * @function
     * @param {Object} {x:Number,y:Number} - An object containing two properties X, horizontal offset, and Y vertical offset
     */
    SetCameraPanOffset: function(__panOffset, __redraw)
    {
        var prev_pan = this.get('_renderercache').cameras["defaultcamera"].panOffset;
        this.get('_renderercache').cameras["defaultcamera"].panOffset = __panOffset;
        this.CameraUpdate(__redraw);

        if(!this.get("_obj_in_frustum") && this.get("_obj_in_frustum_prev"))
        {
            //Object was previously in bounds but went out of frustum because
            //of these new parameters: reverting to previous state
            this.get('_renderercache').cameras["defaultcamera"].panOffset = prev_pan;
            this.CameraUpdate(__redraw);
        }
    },

    GetCameraPanOffset: function()
    {
        var pan_offset = this.get('_renderercache').cameras["defaultcamera"].panOffset;
        return {"x":pan_offset.x, "y":pan_offset.y};
    },

    /**
     * Set up a perspective camera. The camera distance from the scene centre will be set automaticall based on the current scale setting.
     *
     * @function
     * @param {Number} [fieldOfView=45] - the field of view in degrees
     */
    SetPerspectiveCamera: function(__fieldOfView, __redraw)
    {
        this.get('_renderercache').cameras["defaultcamera"].cam_type = "PERSPECTIVE";
        this.get('_renderercache').cameras["defaultcamera"].fieldOfView = __fieldOfView;
        this.CameraUpdate(__redraw);
    },


    /**
     * Set up an orthographic camera. The viewport size will be set automaticall based on the current scale setting.
     *
     * @function
     */
    SetOrthographicCamera: function(__redraw)
    {
        this.get('_renderercache').cameras["defaultcamera"].cam_type = "ORTHOGRAPHIC";
        this.CameraUpdate(__redraw);
    },

    UpdateCamNearAndFar: function(__scene_bbox, __overrideWidth, __overrideHeight)
    {
        var camera = this.get('_3JScamera');

        var fov = this.get('_renderercache').cameras['defaultcamera'].fieldOfView;
        var max = __scene_bbox.max;
        var min = __scene_bbox.min;
        var box_size = new THREE.Vector3(max.x-min.x,max.y-min.y,max.z-min.z);//scene size
        var point = new THREE.Vector3((min.x+max.x)/2,(min.y+max.y)/2,(min.z+max.z)/2);//scene center

        var screenwidth = __overrideWidth?__overrideWidth:this.get('_context').clientWidth;
        var screenheight = __overrideHeight?__overrideHeight:this.get('_context').clientHeight;
        var aspectratio = screenwidth / screenheight;
        //Fit the frustum to englobe the scene bbox for perpective view
        //by computing the minimal distance to the scene according to the fov
        var min_fov = Math.min(fov,aspectratio*fov);
        //Get max length (which is the diagonal length of the scene bbox)
        var max_box_length = Math.sqrt(box_size.x * box_size.x + box_size.y * box_size.y + box_size.z * box_size.z);

        //Set camera position according to the view angles
        var cam_proj_position = this.get('_3JScamera').position.clone();
        var __asset_camera = this.get('_renderercache').cameras["defaultcamera"];
        cam_proj_position.addScaledVector(this.get("_cameraleftdirvec"), __asset_camera.panOffset.x);
        cam_proj_position.addScaledVector(this.get('_3JScamera').up, -__asset_camera.panOffset.y);
        var radius = point.sub(cam_proj_position).length();

        //Sometimes, especially when the camera focus point is not on the scene center
        //or if the camera zooms to much towards the scene, The computed near plane
        //position could be negative (radius - max_box_length/2). To avoid such case
        //that would result in an invalid camera projection matrix, we clamp the near
        //plane position to 0.1. A smaller value would result in z fighting with
        //very close surfaces such as highlights (see OutlineShader.js). A negative
        //value would result in the scene disappearing in some particualr camera angles,
        //zoom, scale, and focus points.
        camera.near = Math.max(radius - max_box_length/2,0.1);
        camera.far = Math.max(radius + max_box_length/2,0.2);
        camera.aspect = aspectratio;

        camera.updateProjectionMatrix();
    },

    GetOptimisedScaleForCurrentView: function(__overrideWidth, __overrideHeight)
    {
        var __asset_camera = this.get('_renderercache').cameras['defaultcamera'];

        var screenwidth = __overrideWidth?__overrideWidth:this.get('_context').clientWidth;
        var screenheight = __overrideHeight?__overrideHeight:this.get('_context').clientHeight;
        var max = this.get('_3JSsceneBBoxFocus').max;
        var min = this.get('_3JSsceneBBoxFocus').min;

        var fov = __asset_camera.fieldOfView;
        var aspectratio = screenwidth / screenheight;
        var radius;
        var max_box_length;
        var box_size = new THREE.Vector3(Math.abs(max.x-min.x),Math.abs(max.y-min.y),Math.abs(max.z-min.z));
        max_box_length = Math.sqrt(box_size.x * box_size.x + box_size.y * box_size.y + box_size.z * box_size.z);

        //Optimize scale factor to make the bbox fit the screen
        //exactly depending on the current view angles

        //Center bbox coordinates relatively to its centroid
        var min_x = min.x - (min.x + max.x)/2;
        var min_y = min.y - (min.y + max.y)/2;
        var min_z = min.z - (min.z + max.z)/2;
        var max_x = max.x - (min.x + max.x)/2;
        var max_y = max.y - (min.y + max.y)/2;
        var max_z = max.z - (min.z + max.z)/2;

        //Transform the 8 vertices of the bbox using opposed camera angles
        var az = WV3DMath.radians(__asset_camera.azimuthAngle);
        var el = WV3DMath.radians(-__asset_camera.elevationAngle);

        var rot_mat = new THREE.Matrix3();
        rot_mat.set(
            Math.cos(el)*Math.cos(az),  -Math.sin(el),  Math.cos(el)*-Math.sin(az),
            Math.sin(el)*Math.cos(az),   Math.cos(el),   Math.sin(el)*-Math.sin(az),
            Math.sin(az),                0,              Math.cos(az)
        )

        var p1 = new THREE.Vector3(min_x, min_y, min_z);
        var p2 = new THREE.Vector3(min_x, min_y, max_z);
        var p3 = new THREE.Vector3(min_x, max_y, min_z);
        var p4 = new THREE.Vector3(min_x, max_y, max_z);
        var p5 = new THREE.Vector3(max_x, min_y, min_z);
        var p6 = new THREE.Vector3(max_x, min_y, max_z);
        var p7 = new THREE.Vector3(max_x, max_y, min_z);
        var p8 = new THREE.Vector3(max_x, max_y, max_z);

        p1.applyMatrix3(rot_mat);
        p2.applyMatrix3(rot_mat);
        p3.applyMatrix3(rot_mat);
        p4.applyMatrix3(rot_mat);
        p5.applyMatrix3(rot_mat);
        p6.applyMatrix3(rot_mat);
        p7.applyMatrix3(rot_mat);
        p8.applyMatrix3(rot_mat);

        min_x = Math.min(p1.x,Math.min(p2.x,Math.min(p3.x,Math.min(p4.x,Math.min(p5.x,Math.min(p6.x,Math.min(p7.x,p8.x)))))));
        min_y = Math.min(p1.y,Math.min(p2.y,Math.min(p3.y,Math.min(p4.y,Math.min(p5.y,Math.min(p6.y,Math.min(p7.y,p8.y)))))));
        min_z = Math.min(p1.z,Math.min(p2.z,Math.min(p3.z,Math.min(p4.z,Math.min(p5.z,Math.min(p6.z,Math.min(p7.z,p8.z)))))));
        max_x = Math.max(p1.x,Math.max(p2.x,Math.max(p3.x,Math.max(p4.x,Math.max(p5.x,Math.max(p6.x,Math.max(p7.x,p8.x)))))));
        max_y = Math.max(p1.y,Math.max(p2.y,Math.max(p3.y,Math.max(p4.y,Math.max(p5.y,Math.max(p6.y,Math.max(p7.y,p8.y)))))));
        max_z = Math.max(p1.z,Math.max(p2.z,Math.max(p3.z,Math.max(p4.z,Math.max(p5.z,Math.max(p6.z,Math.max(p7.z,p8.z)))))));

        if(__asset_camera.cam_type == "PERSPECTIVE")
        {
            //Fit the frustum to englobe the scene bbox for perpective view
            //by computing the minimal distance to the scene according to the fov
            var fov_y = fov;
            var fov_z = aspectratio*fov;
            var box_size = new THREE.Vector3(Math.abs(max_x-min_x),Math.abs(max_y-min_y),Math.abs(max_z-min_z));
            var min_camera_distance_y = Math.abs(box_size.y/(2*Math.tan(WV3DMath.radians(fov_y)/2)));
            var min_camera_distance_z = Math.abs(box_size.z/(2*Math.tan(WV3DMath.radians(fov_z)/2)));
            var min_camera_distance = Math.max(min_camera_distance_z,min_camera_distance_y);
            //This is the radius we want to use
            radius = Math.abs(max_x-min_x) * 0.5 +  min_camera_distance;
            // radius = max_box_length * 0.5 +  min_camera_distance;

            //For this we have to calculate the matching scale in the current view configuration (perspective)
            var min_fov = Math.min(fov,aspectratio*fov);
            var min_camera_distance_normal_view = Math.abs(max_box_length/(2*Math.tan(WV3DMath.radians(min_fov)/2)));
            return min_camera_distance_normal_view / (radius - max_box_length * 0.5);
        }else
        {
            //Fit the frustum to englobe the scene bbox for perpective view
            //by computing the appropriate scaling factor
            var ratiowidth = Math.abs(max_z-min_z) / screenwidth;
            var ratioheight = Math.abs(max_y-min_y) / screenheight;
            var ratio = Math.max(ratiowidth, ratioheight);
            var width_scale = ratio * screenwidth;
            var height_scale = ratio * screenheight;

            var ratiowidth_normal_view = max_box_length / screenwidth;
            var ratioheight_normal_view = max_box_length / screenheight;
            var ratio_normal_view = Math.max(ratiowidth_normal_view, ratioheight_normal_view);
            var width_scale_normal_view = ratio_normal_view * screenwidth;
            var height_scale_normal_view = ratio_normal_view * screenheight;

            var scale_x = width_scale_normal_view / width_scale;
            var scale_y = height_scale_normal_view / height_scale;

            return Math.max(scale_x, scale_y);
        }
    },

    SetOptimisedViewRendering: function(__border, __enableOptimisation, __redraw)
    {
        this.set("_optimised_view", __enableOptimisation);
        this.set("_optimised_view_border", __border);
        this.CameraUpdate(__redraw);
    },

    ComputeNewCamera: function(__asset_camera, __point, __scale, __panX, __panY, __max_box_length, __min_camera_distance, __aspectratio, __screenwidth, __screenheight)
    {
        var camera;
        //Set camera position according to the view angles
        var radius = __max_box_length * 0.5 + __min_camera_distance / __scale;
        //Clamp minimum distance to scene
        radius = Math.max(radius,__max_box_length * 0.5);

        var position = new THREE.Vector3(__point.x, __point.y, __point.z);
        position.x += radius * Math.sin(Math.PI/2 - WV3DMath.radians(__asset_camera.elevationAngle)) * Math.cos(WV3DMath.radians(__asset_camera.azimuthAngle));
        position.z -= radius * Math.sin(Math.PI/2 - WV3DMath.radians(__asset_camera.elevationAngle)) * Math.sin(WV3DMath.radians(__asset_camera.azimuthAngle));
        position.y += radius * Math.cos(Math.PI/2 - WV3DMath.radians(__asset_camera.elevationAngle));
        var viewdir = new THREE.Vector3(__point.x, __point.y, __point.z);
        viewdir.sub(position);
        var leftcamvecdir;
        var el_angle;
        if(__asset_camera.elevationAngle >= 90)
        {
            el_angle = __asset_camera.elevationAngle = 89.99;
        }else
        {
            el_angle = __asset_camera.elevationAngle;
        }
        leftcamvecdir = new THREE.Vector3(0,0,0);
        leftcamvecdir.x += radius * Math.sin(Math.PI/2 - WV3DMath.radians(el_angle)) * Math.cos(WV3DMath.radians(__asset_camera.azimuthAngle) - Math.PI/2);
        leftcamvecdir.z -= radius * Math.sin(Math.PI/2 - WV3DMath.radians(el_angle)) * Math.sin(WV3DMath.radians(__asset_camera.azimuthAngle) - Math.PI/2);
        leftcamvecdir.normalize();
        var upvector = viewdir.cross(leftcamvecdir).normalize();
        var near = radius - __max_box_length/2;
        var far = radius + __max_box_length/2;

        //Fit the frustum to englobe the scene bbox for orthographic view
        var ratiowidth = __max_box_length / __screenwidth;
        var ratioheight = __max_box_length / __screenheight;
        var ratio = Math.max(ratiowidth, ratioheight);
        var width_scale = ratio * __screenwidth / __scale;
        var height_scale = ratio * __screenheight / __scale;

        if(__asset_camera.cam_type == "PERSPECTIVE")
        {
            camera = new THREE.PerspectiveCamera(__asset_camera.fieldOfView, __aspectratio , near, far);
        }else if(__asset_camera.cam_type == "ORTHOGRAPHIC")
        {
            camera = new THREE.OrthographicCamera( width_scale / - 2, width_scale / 2, height_scale / 2, height_scale / - 2, near, far);
        }

        camera.up = upvector;
        camera.position.set(position.x,position.y,position.z);
        camera.lookAt(__point);

        this.set("_cameraleftdirvec", leftcamvecdir);
        camera.position.set(position.x,position.y,position.z);

        __asset_camera.panOffset.x = __panX;//set the proper panning offset
        __asset_camera.panOffset.y = __panY;//set the proper panning offset

        camera.position.addScaledVector(leftcamvecdir, -__panX);
        camera.position.addScaledVector(upvector, __panY);

        return camera;
    },

    /** Set up the camera according to the current camera settings in the asset manager renderercache*/
    SetCamera: function(__scene_bbox, __overrideWidth, __overrideHeight)
    {
        var __asset_camera = this.get('_renderercache').cameras['defaultcamera'];

        var screenwidth = __overrideWidth?__overrideWidth:this.get('_context').clientWidth;
        var screenheight = __overrideHeight?__overrideHeight:this.get('_context').clientHeight;
        var max = __scene_bbox.max;
        var min = __scene_bbox.min;
        var point = new THREE.Vector3((min.x+max.x)/2,(min.y+max.y)/2,(min.z+max.z)/2);//scene center

        var fov = __asset_camera.fieldOfView;
        var aspectratio = screenwidth / screenheight;

        //This optimisation takes into account the projection of the bounding box onto the image plane
        //to make it fit perfectly inside it. This override the scale factor.
        var scale = __asset_camera.scale = this.get("_optimised_view") ? this.GetOptimisedScaleForCurrentView(__overrideWidth, __overrideHeight) : __asset_camera.scale;

        var radius;
        var max_box_length;
        var box_size = new THREE.Vector3(Math.abs(max.x-min.x),Math.abs(max.y-min.y),Math.abs(max.z-min.z));
        max_box_length = Math.sqrt(box_size.x * box_size.x + box_size.y * box_size.y + box_size.z * box_size.z);

        //Fit the frustum to englobe the scene bbox for perpective view
        //by computing the minimal distance to the scene according to the fov
        var min_fov = Math.min(fov,aspectratio*fov);
        var min_camera_distance = Math.abs(max_box_length/(2*Math.tan(WV3DMath.radians(min_fov)/2)));

        var camera;

        if(this.get("_optimised_view"))
        {
            camera = this.ComputeNewCamera(__asset_camera, point, scale, 0, 0, max_box_length, min_camera_distance, aspectratio, screenwidth, screenheight);
            //If we are currently using optimised view, scale has already been set such as the object
            //fits into screen. Optimized labels' position will alway be according to a default 0,0 panning
            //Update labels positions at least once before we start to optimize them
            this.UpdateLabelsPositions(camera, screenwidth, screenheight);
            //Update labels position and retrieve their projected bbox.
            var pt1 = __scene_bbox.min.clone();
            var pt2 = new THREE.Vector3(__scene_bbox.min.x, __scene_bbox.min.y, __scene_bbox.max.z);
            var pt3 = new THREE.Vector3(__scene_bbox.min.x, __scene_bbox.max.y, __scene_bbox.min.z);
            var pt4 = new THREE.Vector3(__scene_bbox.min.x, __scene_bbox.max.y, __scene_bbox.max.z);
            var pt5 = new THREE.Vector3(__scene_bbox.max.x, __scene_bbox.min.y, __scene_bbox.min.z);
            var pt6 = new THREE.Vector3(__scene_bbox.max.x, __scene_bbox.min.y, __scene_bbox.max.z);
            var pt7 = new THREE.Vector3(__scene_bbox.max.x, __scene_bbox.max.y, __scene_bbox.min.z);
            var pt8 = __scene_bbox.max.clone();

            var marginscreenwidth = (screenwidth/2)-this.get("_optimised_view_border").x;
            var marginscreenheight = (screenheight/2)-this.get("_optimised_view_border").y;

            var globalBBox = new THREE.Box2();
            globalBBox.expandByPoint(pt1.clone().project(camera));
            globalBBox.expandByPoint(pt2.clone().project(camera));
            globalBBox.expandByPoint(pt3.clone().project(camera));
            globalBBox.expandByPoint(pt4.clone().project(camera));
            globalBBox.expandByPoint(pt5.clone().project(camera));
            globalBBox.expandByPoint(pt6.clone().project(camera));
            globalBBox.expandByPoint(pt7.clone().project(camera));
            globalBBox.expandByPoint(pt8.clone().project(camera));
            globalBBox.min.x *= screenwidth/2;
            globalBBox.min.y *= screenheight/2;
            globalBBox.max.x *= screenwidth/2;
            globalBBox.max.y *= screenheight/2;

            //Add user defined border
            var sceneBBsuccess = (globalBBox.min.x >= -marginscreenwidth) && (globalBBox.max.x <= marginscreenwidth) &&
                                    (globalBBox.min.y >= -marginscreenheight) && (globalBBox.max.y <= marginscreenheight);
            var success = sceneBBsuccess && this.RotateOutOfBoundsVisibleLabels(camera, globalBBox, screenwidth, screenheight, marginscreenwidth, marginscreenheight);
            var currentPanOffset = new THREE.Vector2(0,0);
            var nb_max_iterations_opt = 100;
            while(!success && scale > 0.1)
            {
                var labelsize = new THREE.Vector2();
                globalBBox.size(labelsize);

                if(labelsize.x > marginscreenwidth || labelsize.y > marginscreenheight || !nb_max_iterations_opt)
                {
                    //panning would not even be enough: need rescale
                    __asset_camera.scale = scale -= 0.005;
                }
                else
                {
                    //try to pan towards the center of the projection
                    var boxcenter = globalBBox.center();
                    __asset_camera.panOffset.x = currentPanOffset.x += Math.sign(boxcenter.x)*1.5;
                    __asset_camera.panOffset.y = currentPanOffset.y += Math.sign(boxcenter.y)*1.5;
                    nb_max_iterations_opt--;
                }
                camera = this.ComputeNewCamera(__asset_camera, point, scale, currentPanOffset.x, currentPanOffset.y, max_box_length, min_camera_distance, aspectratio, screenwidth, screenheight);
                //Try with the current camera
                this.UpdateLabelsPositions(camera, screenwidth, screenheight);
                //Update labels position and retrieve their projected bbox.
                globalBBox.makeEmpty();
                globalBBox.expandByPoint(pt1.clone().project(camera));
                globalBBox.expandByPoint(pt2.clone().project(camera));
                globalBBox.expandByPoint(pt3.clone().project(camera));
                globalBBox.expandByPoint(pt4.clone().project(camera));
                globalBBox.expandByPoint(pt5.clone().project(camera));
                globalBBox.expandByPoint(pt6.clone().project(camera));
                globalBBox.expandByPoint(pt7.clone().project(camera));
                globalBBox.expandByPoint(pt8.clone().project(camera));
                globalBBox.min.x *= screenwidth/2;
                globalBBox.min.y *= screenheight/2;
                globalBBox.max.x *= screenwidth/2;
                globalBBox.max.y *= screenheight/2;
                var sceneBBsuccess = (globalBBox.min.x >= -marginscreenwidth) && (globalBBox.max.x <= marginscreenwidth) &&
                                        (globalBBox.min.y >= -marginscreenheight) && (globalBBox.max.y <= marginscreenheight);
                success = sceneBBsuccess && this.RotateOutOfBoundsVisibleLabels(camera, globalBBox, screenwidth, screenheight, marginscreenwidth, marginscreenheight);
            }
        }
        else
        {
            camera = this.ComputeNewCamera(__asset_camera, point, scale, __asset_camera.panOffset.x, __asset_camera.panOffset.y, max_box_length, min_camera_distance, aspectratio, screenwidth, screenheight);

            this.UpdateLabelsPositions(camera, screenwidth, screenheight);
        }
        this.set('_3JScamera', camera);
    },

    /**
    * Focus camera from a node's bbox using its URL in the scene
    * default focus of camera is on the root node of the scene
    *
    * @function SetCameraFocus
    * @param {String} __nodeURL - URL identifying the scene node
    * @param {Boolean} __redraw - If true, redraw the scene after box update - default is true
    */
    SetCameraFocus: function(__nodeURL, __redraw)
    {
        var shortNodeURL = this.URLToKeyString(__nodeURL);
        for (var i = 0, keys = Object.keys(this.get('_renderercache')['nodeviewproperties'][shortNodeURL]['threejsnodes']), l = keys.length; i < l; ++i)
        {
            //Set the focus based on the first non empty object bbox found in the first pass available (then redraw and exit)
            var key = keys[i];
            var objects = this.get('_renderercache')['nodeviewproperties'][shortNodeURL]['threejsnodes'][key];
            var box = new THREE.Box3();
            //Set camera focus based on the bbox of all objects in all coincident groups
            for (var node_cg = 0, keys2 = Object.keys(objects), l2 = keys2.length; node_cg < l2; node_cg++)
            {
                var boxob = new THREE.Box3();
                boxob.setFromObject(objects[keys2[node_cg]]);
                if (!boxob.empty()) box.union(boxob);
            }
            if (!box.empty())
            {
                this.set('_3JSsceneBBoxFocus', box);
                this.CameraUpdate(__redraw);
                break;
            }
        }
    },

    /**
    * Set environment rotation from quaternion
    *
    * @function SetEnvironmentRotation
    *
    */
    SetEnvironmentRotation: function(__quaternion, __redraw)
    {
        this.get('_renderercache')["environment_settings"]["rotation"] = __quaternion.clone();

        //Iterate through all materials to update environment rotation quaternion uniform
        this.ApplyFctToAllMaterials(this.UpdateMaterialEnvironmentRotation(this.get('_renderercache')["environment_settings"]["rotation"]));

        if(__redraw === undefined || __redraw)
            this.Render();
    },

    /**
    * Get environment rotation quaternion
    *
    * @function GetEnvironmentRotation
    *
    */
    GetEnvironmentRotation: function()
    {
        return this.get('_renderercache')["environment_settings"]["rotation"].clone();
    },

    UpdateMaterialEnvironmentRotation: function(__quaternion)
    {
        var rotation = new THREE.Vector4(__quaternion.x,__quaternion.y,__quaternion.z,__quaternion.w);
        var self = this;
        return function (__material)
        {
            if(__material)
            {
                if(__material.hasOwnProperty("uniforms"))
                {
                    if(__material["uniforms"].hasOwnProperty("envMapRotQuat"))
                    {
                        __material["uniforms"]["envMapRotQuat"]["value"] = rotation;
                    }
                    if(__material["uniforms"].hasOwnProperty("lightPos"))
                    {
                        for (var i = 0; i < self.get('_renderercache')['lights']['holo_light']['Lights'].length && (i < 4); ++i)
                        {
                            var light = self.get('_renderercache')['lights']['holo_light']['Lights'][i];
                            var lightPos = new THREE.Vector3( light[0], light[1], light[2]);
                            lightPos.applyQuaternion(__quaternion);
                            __material["uniforms"]["lightPos"]["value"][i] = lightPos;
                        }
                    }
                }
            }
        };
    },

    FlipMaterialSide: function(__flipped)
    {
        return function (__material)
        {
            //No need to flip if the material is double sided
            if(__material)
            {
                //This mecanism ensures that a material cannot be flipped more than once if it is shared between several surfaces
                if(__material["flipped"] != __flipped)
                {
                    if(__material["side"] != THREE.DoubleSide)
                    {
                        __material["side"] = (__material["side"] == THREE.FrontSide) ? THREE.BackSide : THREE.FrontSide;
                    }
                    if(__material.hasOwnProperty("uniforms") && __material["uniforms"].hasOwnProperty("isFloorReflection"))
                    {
                        __material["uniforms"]["isFloorReflection"]["value"] = (THREE.FrontSide == __flipped) ? 0 : 1;
                    }
                }
                __material["flipped"] = __flipped;
            }
        };
    },

    UpdateReflectionUniforms: function(__opacity, __fade, __position, __fadingSize)
    {
        this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["reflection_opacity"]["value"] = (__opacity !== undefined)? __opacity : this.get('_renderercache')["reflections"]["opacity"];
        this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["reflection_fade"]["value"] = (__fade !== undefined)? __fade : this.get('_renderercache')["reflections"]["fade"];
        this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["reflection_fade_position"]["value"] = (__position !== undefined)? __position : this.get('_renderercache')["reflections"]["fade_position"];
        this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["reflection_fade_size"]["value"] = (__fadingSize !== undefined)? __fadingSize : this.get('_renderercache')["reflections"]["fade_size"];
        this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["reflection_opacity"]["value"] = (__opacity !== undefined)? __opacity : this.get('_renderercache')["reflections"]["opacity"];
        this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["reflection_fade"]["value"] = (__fade !== undefined)? __fade : this.get('_renderercache')["reflections"]["fade"];
        this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["reflection_fade_position"]["value"] = (__position !== undefined)? __position : this.get('_renderercache')["reflections"]["fade_position"];
        this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["reflection_fade_size"]["value"] = (__fadingSize !== undefined)? __fadingSize : this.get('_renderercache')["reflections"]["fade_size"];
        this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["reflection_opacity"]["value"] = (__opacity !== undefined)? __opacity : this.get('_renderercache')["reflections"]["opacity"];
        this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["reflection_fade"]["value"] = (__fade !== undefined)? __fade : this.get('_renderercache')["reflections"]["fade"];
        this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["reflection_fade_position"]["value"] = (__position !== undefined)? __position : this.get('_renderercache')["reflections"]["fade_position"];
        this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["reflection_fade_size"]["value"] = (__fadingSize !== undefined)? __fadingSize : this.get('_renderercache')["reflections"]["fade_size"];
    },

    IsReflectionsRenderingEnabled: function()
    {
        //enable if scene loaded entirely
        //and
        //camera above ground
        //and
        //((config JSON and json refl enabled) or (config ON))
        return (this.get('_SceneLoaded') && this.get('_3JSsceneBBox').min.y < this.get('_3JScamera').position.y) &&
                (( (this.get("_reflection_config") == "JSON") && (this.get('_renderercache')["reflections"]["enabled"]) ) || (this.get("_reflection_config") == "ON"));
    },

    //The following flip functions are used to render the reflected scene
    //TODO: move these functions into a separate file.

    FlipScene: function(__scene)
    {
        //Apply scaling matrix * -1 on Y axis
        __scene.applyMatrix(this.get("_reflection_Y_scale"));
        //Translate the object so that the bottom of the object nearly (offset) touches the top of the reflection
        __scene.applyMatrix(this.get("_reflection_Y_translation"));
    },

    UnflipScene: function(__scene)
    {
        __scene.applyMatrix(this.get("_reflection_Y_opp_translation"));
        __scene.applyMatrix(this.get("_reflection_Y_scale"));
    },

    FlipAllScenes: function()
    {
        this.FlipScene(this.get('_3JSsceneOp'));
        this.FlipScene(this.get('_3JSsceneOpDepth'));
        this.FlipScene(this.get('_3JSsceneOpaqueDiffuse'));
        for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
            this.FlipScene(this.get('_3JSsceneOpCoincidentGroups')[cg_id]);
            this.FlipScene(this.get('_3JSsceneOpCoincidentGroupsDepth')[cg_id]);
            this.FlipScene(this.get('_3JSsceneOpCoincidentGroupsDiffuse')[cg_id]);
        }
        this.FlipScene(this.get('_3JSsceneTransp'));
        this.FlipScene(this.get('_3JSsceneTranspDepth'));
        this.FlipScene(this.get('_3JSsceneTranspDiffuse'));
        for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
            this.FlipScene(this.get('_3JSsceneTranspCoincidentGroups')[cg_id]);
            this.FlipScene(this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id]);
            this.FlipScene(this.get('_3JSsceneTranspCoincidentGroupsDepth')[cg_id]);
        }
    },

    UnFlipAllScenes: function()
    {
        this.UnflipScene(this.get('_3JSsceneOp'));
        this.UnflipScene(this.get('_3JSsceneOpDepth'));
        this.UnflipScene(this.get('_3JSsceneOpaqueDiffuse'));
        for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
            this.UnflipScene(this.get('_3JSsceneOpCoincidentGroups')[cg_id]);
            this.UnflipScene(this.get('_3JSsceneOpCoincidentGroupsDepth')[cg_id]);
            this.UnflipScene(this.get('_3JSsceneOpCoincidentGroupsDiffuse')[cg_id]);
        }
        this.UnflipScene(this.get('_3JSsceneTransp'));
        this.UnflipScene(this.get('_3JSsceneTranspDepth'));
        this.UnflipScene(this.get('_3JSsceneTranspDiffuse'));
        for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
            this.UnflipScene(this.get('_3JSsceneTranspCoincidentGroups')[cg_id]);
            this.UnflipScene(this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id]);
            this.UnflipScene(this.get('_3JSsceneTranspCoincidentGroupsDepth')[cg_id]);
        }
    },

    FlipOpaqueScene: function()
    {
        this.FlipScene(this.get('_3JSsceneOp'));
        for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
            this.FlipScene(this.get('_3JSsceneOpCoincidentGroups')[cg_id]);
        }
    },

    UnFlipOpaqueScene: function()
    {
        this.UnflipScene(this.get('_3JSsceneOp'));
        for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
            this.UnflipScene(this.get('_3JSsceneOpCoincidentGroups')[cg_id]);
        }
    },

    FlipTransparentScene: function()
    {
        this.FlipScene(this.get('_3JSsceneTransp'));
        for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
            this.FlipScene(this.get('_3JSsceneTranspCoincidentGroups')[cg_id]);
        }
    },

    UnFlipTransparentScene: function()
    {
        this.UnflipScene(this.get('_3JSsceneTransp'));
        for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
            this.UnflipScene(this.get('_3JSsceneTranspCoincidentGroups')[cg_id]);
        }
    },

    FlipLights: function()
    {
        for (var i = 0, cg_keys = Object.keys(this.get('_renderercache')['lights']['dir_light']), l = cg_keys.length; i < l; ++i)
        {
            var dir_light = this.get('_renderercache')['lights']['dir_light'][cg_keys[i]];
            if(dir_light)
            {
                dir_light.position.set(-dir_light.position.x, dir_light.position.y, -dir_light.position.z);
                dir_light.needsUpdate = true;
           }
        }
    },

    UnFlipLights: function()
    {
        this.FlipLights();
    },

    //Render functions
    //TODO: move these functions into another file

    RenderScene: function(__scene, __renderTarget, __reflection)
    {
        this.get('_3JSrenderer').render(__scene, this.get('_3JScamera'), __renderTarget);
    },

    RenderOpaqueScene: function(__renderTarget)
    {
        //  Render opaque non coincident groups (standard) + non ard coincident groups (bags)
        this.RenderScene(this.get('_3JSsceneOp'), __renderTarget);

        //  Render opaque ard coincident groups (ard)
        var setWriteDepth = function (__depthwrite){
            return function ( obj )
            {
                if(obj instanceof THREE.Mesh)
                {
                    if(obj["material"])
                    {
                        obj["material"]["depthWrite"] = __depthwrite;
                    }
                }
            };
        };

        this.get('_3JSrenderer').autoClear = false;
        for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
        {
            //      -Render once with zwrite disabled (update color only)
            var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
            this.get('_3JSsceneOpCoincidentGroups')[cg_id].traverse(setWriteDepth(false));
            this.RenderScene(this.get('_3JSsceneOpCoincidentGroups')[cg_id], __renderTarget);

            //      -Render once with colorwrite disabled (update depth only)
            this.get('_3JSrenderer').context.colorMask( false, false, false, false );
            this.get('_3JSsceneOpCoincidentGroups')[cg_id].traverse(setWriteDepth(true));
            this.RenderScene(this.get('_3JSsceneOpCoincidentGroups')[cg_id], __renderTarget);
            this.get('_3JSrenderer').context.colorMask( true, true, true, true );
        }
        this.get('_3JSrenderer').autoClear = true;
    },

    RenderOpaqueSceneDepth: function(__renderTarget, __overrideMaterial)
    {
        this.get('_3JSsceneOpDepth').overrideMaterial = __overrideMaterial ? __overrideMaterial : this.get('_renderercache')["materials"]["D0ShaderMaterial"];
        this.RenderScene(this.get('_3JSsceneOpDepth'), __renderTarget);

        this.get('_3JSrenderer').autoClear = false;
        for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
            this.get('_3JSsceneOpCoincidentGroupsDepth')[cg_id].overrideMaterial = __overrideMaterial ? __overrideMaterial : this.get('_renderercache')["materials"]["D0ShaderMaterial"];
            this.RenderScene(this.get('_3JSsceneOpCoincidentGroupsDepth')[cg_id], __renderTarget);
        }
        this.get('_3JSrenderer').autoClear = true;
    },

    RenderOpaqueSceneDiffuse: function(__renderTarget)
    {
        //  Render opaque non coincident groups (standard) + non ard coincident groups (bags)
        this.RenderScene(this.get('_3JSsceneOpaqueDiffuse'), __renderTarget);

        //  Render opaque ard coincident groups (ard)
        var setWriteDepth = function (__depthwrite){
            return function ( obj )
            {
                if(obj instanceof THREE.Mesh)
                {
                    if(obj["material"])
                    {
                        obj["material"]["depthWrite"] = __depthwrite;
                    }
                }
            };
        };

        this.get('_3JSrenderer').autoClear = false;
        for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
        {
            //      -Render once with zwrite disabled (update color only)
            var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
            this.get('_3JSsceneOpCoincidentGroupsDiffuse')[cg_id].traverse(setWriteDepth(false));
            this.RenderScene(this.get('_3JSsceneOpCoincidentGroupsDiffuse')[cg_id], __renderTarget);

            //      -Render once with colorwrite disabled (update depth only)
            this.get('_3JSrenderer').context.colorMask( false, false, false, false );
            this.get('_3JSsceneOpCoincidentGroupsDiffuse')[cg_id].traverse(setWriteDepth(true));
            this.RenderScene(this.get('_3JSsceneOpCoincidentGroupsDiffuse')[cg_id], __renderTarget);
            this.get('_3JSrenderer').context.colorMask( true, true, true, true );
        }
        this.get('_3JSrenderer').autoClear = true;
    },

    RenderTransparentScene: function(__renderTarget, __sceneGraphCallbackFct)
    {
        //  Render transparent non coincident groups (standard) + non ard coincident groups (bags)
        if(typeof __sceneGraphCallbackFct === "function")
        {
            this.get('_3JSsceneTransp').traverse(__sceneGraphCallbackFct);
        }
        this.RenderScene(this.get('_3JSsceneTransp'), __renderTarget);

        //  Render opaque ard coincident groups (ard)
        var setWriteDepth = function (__depthwrite){
            return function ( obj )
            {
                if(obj instanceof THREE.Mesh)
                {
                    if(obj["material"])
                    {
                        obj["material"]["depthWrite"] = __depthwrite;
                    }
                }
            };
        };

        this.get('_3JSrenderer').autoClear = false;
        for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
            this.get('_3JSsceneTranspCoincidentGroups')[cg_id].traverse(__sceneGraphCallbackFct);

            //      -Render once with zwrite disabled (update color only)
            this.get('_3JSsceneTranspCoincidentGroups')[cg_id].traverse(setWriteDepth(false));
            this.RenderScene(this.get('_3JSsceneTranspCoincidentGroups')[cg_id], __renderTarget);

            //      -Render once with colorwrite disabled (update depth only)
            this.get('_3JSrenderer').context.colorMask( false, false, false, false );
            this.get('_3JSsceneTranspCoincidentGroups')[cg_id].traverse(setWriteDepth(true));
            this.RenderScene(this.get('_3JSsceneTranspCoincidentGroups')[cg_id], __renderTarget);
            this.get('_3JSrenderer').context.colorMask( true, true, true, true );
        }
        this.get('_3JSrenderer').autoClear = true;
    },

    RenderTransparentSceneDepth: function(__renderTarget, __overrideMaterial)
    {
        this.get('_3JSsceneTranspDepth').overrideMaterial = __overrideMaterial;
        this.RenderScene(this.get('_3JSsceneTranspDepth'), __renderTarget);
        this.get('_3JSsceneTranspDepth').overrideMaterial = null;

        this.get('_3JSrenderer').autoClear = false;
        for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
            this.get('_3JSsceneTranspCoincidentGroupsDepth')[cg_id].overrideMaterial = __overrideMaterial;
            this.RenderScene(this.get('_3JSsceneTranspCoincidentGroupsDepth')[cg_id], __renderTarget);
            this.get('_3JSsceneTranspCoincidentGroupsDepth')[cg_id].overrideMaterial = null;
        }
        this.get('_3JSrenderer').autoClear = true;
    },

    RenderTransparentSceneDiffuse: function(__renderTarget, __sceneGraphCallbackFct)
    {
        //  Render opaque non coincident groups (standard) + non ard coincident groups (bags)
        this.get('_3JSsceneTranspDiffuse').traverse(__sceneGraphCallbackFct);
        this.RenderScene(this.get('_3JSsceneTranspDiffuse'), __renderTarget);

        //  Render opaque ard coincident groups (ard)
        var setWriteDepth = function (__depthwrite){
            return function ( obj )
            {
                if(obj instanceof THREE.Mesh)
                {
                    if(obj["material"])
                    {
                        obj["material"]["depthWrite"] = __depthwrite;
                    }
                }
            };
        };

        this.get('_3JSrenderer').autoClear = false;
        for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
            this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id].traverse(__sceneGraphCallbackFct);

            //      -Render once with zwrite disabled (update color only)
            this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id].traverse(setWriteDepth(false));
            this.RenderScene(this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id], __renderTarget);

            //      -Render once with colorwrite disabled (update depth only)
            this.get('_3JSrenderer').context.colorMask( false, false, false, false );
            this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id].traverse(setWriteDepth(true));
            this.RenderScene(this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id], __renderTarget);
            this.get('_3JSrenderer').context.colorMask( true, true, true, true );
        }
        this.get('_3JSrenderer').autoClear = true;
    },

    SetSizeOfRTSrc: function(__width, __height)
    {
        return function ( obj )
        {
            if(obj instanceof THREE.Mesh)
            {
                if(obj["material"].hasOwnProperty("uniforms"))
                {
                    if(obj["material"]["uniforms"].hasOwnProperty("canvas_size"))
                    {
                        obj["material"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(__width, __height);
                    }
                }
            }
        };
    },

    SetAsRTSrc: function (__width, __height, __rTSrc1, __rTSrc2)
    {
        var nb_arg = Math.max(arguments.length - 2,0);//Remove width and height from the count
        var self = this;
        return function ( obj )
        {
            if(obj instanceof THREE.Mesh)
            {
                if(obj["material"].hasOwnProperty("uniforms"))
                {
                    if(obj["material"]["uniforms"].hasOwnProperty("DX1RT")
                       &&obj["material"]["uniforms"].hasOwnProperty("DX2RT")
                       &&obj["material"]["uniforms"].hasOwnProperty("NBRTSRC")
                       &&obj["material"]["uniforms"].hasOwnProperty("canvas_size"))
                    {
                        obj["material"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(__width, __height);
                        obj["material"]["uniforms"]["DX1RT"]["value"] = __rTSrc1?__rTSrc1.texture:self.get('_DummyTexture');
                        obj["material"]["uniforms"]["DX2RT"]["value"] = __rTSrc2?__rTSrc2.texture:self.get('_DummyTexture');
                        obj["material"]["uniforms"]["NBRTSRC"]["value"] = nb_arg;
                    }
                    if(obj["material"]["uniforms"].hasOwnProperty("transparent"))
                    {
                        obj["material"]["uniforms"]["transparent"]["value"] = __rTSrc1?1:0;
                    }
                }
            }
        };
    },

    RenderDepthPeel: function(__drawHardEdgeHighlights, __cameraRTT, __screen_plane, __width, __height,
                              __colorStackRT, __transparencyStackRT, __opaqueDepthRT, __d1RT, __d2RT, __peelColorRT, __opaqueBackgroundMaskRT,
                              __blackClearDepthColor, __whiteClearDepthColor)
    {
        // 1) Render opaque objects with an override material that writes depth as grey/color (D0)
        this.get('_3JSrenderer').setClearColor( __blackClearDepthColor, 1 );
        this.RenderOpaqueSceneDepth(__opaqueDepthRT);

        // 2A) Render 1st peel depth in __colorStackRT, using OpaqueDepthRT. This enables a later pass to use blending, needed for coincident surfaces.
        this.get('_3JSrenderer').autoClearDepth = true;
        this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"]["uniforms"]["DX1RT"]["value"] = __opaqueDepthRT.texture;
        this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"]["uniforms"]["NBRTSRC"]["value"] = 1;

        this.RenderTransparentSceneDepth(__colorStackRT, this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"]);
        this.get('_3JSrenderer').autoClearDepth = false;

        // 2B: Render 1st peel color in __colorStackRT, using __opaqueDepthRT.
        this.get('_3JSrenderer').setClearColor( __blackClearDepthColor, 1 ); // color peel must have black BG and Alpha of background
        this.RenderTransparentScene(__colorStackRT, this.SetAsRTSrc(__width, __height, __opaqueDepthRT));

        // 2C) Render 1st peel depth in __transparencyStackRT, using __opaqueDepthRT.
        this.get('_3JSrenderer').autoClearDepth = true;
        this.RenderTransparentSceneDepth(__transparencyStackRT, this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"]);

        // 2D) Render 1st peel transparency in __transparencyStackRT, using __opaqueDepthRT.
        this.get('_3JSrenderer').autoClearDepth = false;
        this.get('_3JSrenderer').setClearColor( __whiteClearDepthColor, 1 ); // start 100% transparent, so background shows through unused pixels

        this.RenderTransparentSceneDiffuse(__transparencyStackRT, this.SetAsRTSrc(__width, __height, __opaqueDepthRT ));

        this.get('_3JSrenderer').autoClearDepth = true;


        //Force number of peels for debug
        // this.set("_current_nb_peels", 1 );//remove after debug
        if(this.get("_current_nb_peels") > 1 && this.get('_SceneLoaded'))
        {
            // 3) RENDER NEXT PEELS. For this we need the depth D1 of the first (previous) peel.
            // Render transp objects with an override material using GL_LEQUAL and D0 to render depth into D1
            // here we draw only the fragments that are in front of opaque surfaces
            this.get('_3JSrenderer').setClearColor( __blackClearDepthColor, 1 );
            this.get('_renderercache')["materials"]["D1ShaderMaterial"]["uniforms"]["D0RT"]["value"] = __opaqueDepthRT.texture;

            this.RenderTransparentSceneDepth(__d1RT, this.get('_renderercache')["materials"]["D1ShaderMaterial"]);

            this.get('_renderercache')["materials"]["D2ShaderMaterial"]["uniforms"]["DX1RT"]["value"] = __opaqueDepthRT.texture;

            this.get('_3JSrenderer').setClearColor( __blackClearDepthColor, 1 ); // color peel must have black BG
            for (var i=0; i < this.get("_current_nb_peels") - 1; ++i)//This should be removed when occlusion query is implemented (with a max_nb_peel for safety)
            {
                // 4A) Render the next peels depth in __peelColorRT. This enables a later pass to use blending, needed for coincident surfaces
                this.get('_3JSrenderer').autoClearDepth = true;
                this.get('_3JSrenderer').autoClearColor = true;
                this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"]["uniforms"]["DX2RT"]["value"] = (i%2)? __d2RT.texture : __d1RT.texture;
                this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"]["uniforms"]["NBRTSRC"]["value"] = 2;

                this.RenderTransparentSceneDepth(__peelColorRT, this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"]);
                this.get('_3JSrenderer').autoClearDepth = false;

                // 4B Render the next peels color in __peelColorRT, using __opaqueDepthRT.
                this.RenderTransparentScene(__peelColorRT, this.SetAsRTSrc(__width, __height, __opaqueDepthRT, (i%2)? __d2RT : __d1RT));
                this.get('_3JSrenderer').autoClearDepth = true;

                // 5) Blend the current peel color with the color stack, using the transparency stack
                this.get('_3JSrenderer').autoClearColor = false;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["CX1RT"]["value"] = __peelColorRT.texture;//current peel
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["TransparencyStack"]["value"] = __transparencyStackRT.texture;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableBGBlend"]["value"] = 0;

                var screen_quad_color_blend = new THREE.Mesh(__screen_plane, this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]);
                var screen_quad_scene_color_blend = new THREE.Scene();
                screen_quad_scene_color_blend.add(screen_quad_color_blend);
                this.get('_3JSrenderer').render(screen_quad_scene_color_blend, __cameraRTT, __colorStackRT);
                this.get('_3JSrenderer').autoClearColor = true;

                // 6A) Render the next peel depth in the transparency stack, without overwriting the color. Needed for coincident surfaces.
                this.get('_3JSrenderer').autoClearDepth = true;
                this.get('_3JSrenderer').autoClearColor = false;

                this.RenderTransparentSceneDepth(__transparencyStackRT, this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"]);
                this.get('_3JSrenderer').autoClearDepth = false;

                // 6B) Blend the next peel's transparency with the transparency stack. Blending is done via the material blending.
                this.get('_3JSrenderer').autoClearColor = false;

                this.RenderTransparentSceneDiffuse(__transparencyStackRT, this.SetAsRTSrc(__width, __height, __opaqueDepthRT, (i%2)? __d2RT : __d1RT));
                this.get('_3JSrenderer').autoClearColor = true;
                this.get('_3JSrenderer').autoClearDepth = true;

                // 7) Render transp objects with an override material using GL_LEQUAL and D0 to render depth into D1
                //here we draw only the fragments that have depth strictly between D0 and D1- with LEQUAL depth function
                this.get('_renderercache')["materials"]["D2ShaderMaterial"]["uniforms"]["DX2RT"]["value"] = (i%2)? __d2RT.texture : __d1RT.texture;

                this.RenderTransparentSceneDepth((i%2)? __d1RT : __d2RT, this.get('_renderercache')["materials"]["D2ShaderMaterial"]);
            }
        }

        ////Render opaque objects + hard edges highlights into PeelColor
        this.get('_3JSrenderer').setClearColor( __blackClearDepthColor, 1 );//Restore clear color
        this.RenderOpaqueScene(__peelColorRT);
        if(__drawHardEdgeHighlights)
        {
            this.get('_3JSrenderer').autoClear = false;
            this.get('_3JSrenderer').render(this.get('_3JSsceneHightlights'), this.get('_3JScamera'),  __peelColorRT );
            this.get('_3JSrenderer').autoClear = true;
        }

        // 9) RENDER ALPHA OF OPAQUE PEEL.
        this.get('_3JSrenderer').setClearColor( __whiteClearDepthColor, 1 ); // start 100% opaque, so background shows through unused pixels
        this.RenderOpaqueSceneDiffuse(__opaqueBackgroundMaskRT);
        if(__drawHardEdgeHighlights)
        {
            //Also draw highlight edges to update opaque mask
            this.get('_3JSrenderer').autoClear = false;
            this.get('_3JSsceneHightlights').overrideMaterial = this.get('_renderercache')["materials"]["blackhighlight"];
            this.get('_3JSrenderer').render(this.get('_3JSsceneHightlights'), this.get('_3JScamera'),  __opaqueBackgroundMaskRT );
            this.get('_3JSsceneHightlights').overrideMaterial = null;
            this.get('_3JSrenderer').autoClear = true;
        }
    },

    /** Render the scene */
    Render: function(__offscreen, __overrideWidth, __overrideHeight)
    {

        //If the scenes do not contain any object, rendering them is useless (return)
        if(this.get('_emptyOpaqueScene') && this.get('_emptyTransparentScene')) return;

        var width = __overrideWidth?__overrideWidth:this.get('_context').clientWidth;
        var height = __overrideHeight?__overrideHeight:this.get('_context').clientHeight;

        if(__offscreen || this.get("_resetRenderTarget"))
        {
            this.InitializeRenderTargets(width,height);
            this.CameraUpdate(false, width, height);
            this.set('_3JSHUDcamera', new THREE.OrthographicCamera( - width/2, width/2, height/2, - height/2, 0, 10));
            this.set("_resetRenderTarget", false);
        }

        var nodeViewProperties = this.get('_renderercache')['nodeviewproperties'];
        var enableHighlights = this.get('_renderercache')['materials']['enablehighlights'];
        var enableHardEdgeHighlights = this.get('_renderercache')['materials']['enablehardedgehighlights'];
        var enableHorizonHighlights = this.get('_renderercache')['materials']['enablehorizonhighlights'];
        var enableLabels = this.get('_renderercache')['labels']['visible'];

        //Prepare quad geometry to render it with any render target attached as texture
        var ColorStackRT = this.get('_ColorStackRT');
        var cameraRTT = new THREE.OrthographicCamera( - width/2, width/2, height/2, -height/2, -100000, 100000 );
        cameraRTT.position.z = 10000;
        var screen_plane = new THREE.PlaneGeometry( width, height );

        //Compute backdrop (background image) parameters if needed
        var backdropTex, bd_scaleX, bd_scaleY, bd_offsetX, bd_offsetY ;
        if ( this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["backdrop"]["loaded"]
            && this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["backdrop"]["enabled"])
        {
            backdropTex = this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["backdrop"]["data"];

            var bd_orig_width = this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["backdrop"]["original_width"];
            var bd_orig_height = this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["backdrop"]["original_height"];
            var bd_nat_width = backdropTex["image"]["naturalWidth"];
            var bd_nat_height = backdropTex["image"]["naturalHeight"];
            if ( ! bd_orig_width)
                bd_orig_width = bd_nat_width;
            if ( ! bd_orig_height)
                bd_orig_height = bd_nat_height;

            var canvasRatio = width / height;
            var backdropRatio = bd_nat_width / bd_orig_height;

            bd_scaleX = 1;
            bd_scaleY = 1;
            if ( canvasRatio < backdropRatio)
                bd_scaleX = canvasRatio / backdropRatio;
            else
                bd_scaleY = backdropRatio / canvasRatio;
            bd_scaleX *= bd_orig_width / bd_nat_width;
            bd_scaleY *= bd_orig_height / bd_nat_height;

            bd_offsetX = -.5 * bd_scaleX + .5 * bd_orig_width / bd_nat_width;
            bd_offsetY = -.5 * bd_scaleY + .5 * bd_orig_height / bd_nat_height;
        }

        WV3DFPSCounter.Update();
        var current_fps = WV3DFPSCounter.GetCountPerMinute();
        // Uncomment these three lines to check performances - comment at the end of the story
        if(current_fps != this.get("_current_fps"))
        {
            LogMessage.Log("Average framerate: "+current_fps);
            this.set("_current_fps", current_fps);
        }

        //Render scene, with depth peeling algorithm if containing some transparent elements and using Visualizer quality
        if(!this.get('_emptyTransparentScene') && this.get('_RenderingQuality') == "Visualizer")
        {
            if(this.get("_interactiveFrameRate") && this.get("_enableInteractionOptimization"))
            {
                //Calculate interactively the number of peels according to the performances
                //Current average window is set to 2 seconds in WV3DFPSCounter

                if (current_fps > this.get("_target_fps_max"))
                {
                    //We are loose on performance, let's increase the number of peels
                    this.set("_current_nb_peels", this.get("_current_nb_peels")+1);
                }else if (current_fps < this.get("_target_fps_min"))
                {
                    //We are losing performance, let's decrease the number of peels
                    this.set("_current_nb_peels", this.get("_current_nb_peels")-1);
                }
                //Clamp the number of peels
                this.set("_current_nb_peels", Math.max(Math.min(this.get("_current_nb_peels"), this.get("_nb_max_peels")), this.get("_nb_min_peels")));
            }else
            {
                this.set("_current_nb_peels", this.get("_nb_max_peels"));
            }

            //0) Initialize render targets
            var OpaqueDepthRT = this.get('_OpaqueDepthRT');
            var D1RT = this.get('_D1RT');
            var D2RT = this.get('_D2RT');
            var PeelColorRT = this.get('_PeelColorRT');
            var TransparencyStackRT = this.get('_TransparencyStackRT');
            var OpaqueBackgroundMaskRT = this.get('_OpaqueBackgroundMaskRT');

            var MirrorColorStackRT = this.get('_MirrorColorStackRT');
            var MirrorColorStackRT2 = this.get('_MirrorColorStackRT2');
            var MirrorOpaqueDepthRT = this.get('_MirrorOpaqueDepthRT');
            var MirrorPeelColorRT = this.get('_MirrorPeelColorRT');
            var MirrorTransparencyStackRT = this.get('_MirrorTransparencyStackRT');
            var FadedMirrorTransparencyStackRT = this.get('_FadedMirrorTransparencyStackRT');
            var MirrorOpaqueBackgroundMaskRT = this.get('_MirrorOpaqueBackgroundMaskRT');

            this.get('_3JSsceneOp').traverse(this.SetSizeOfRTSrc(width, height));
            this.get('_3JSsceneTransp').traverse(this.SetSizeOfRTSrc(width, height));
            this.get('_3JSsceneTranspDepth').traverse(this.SetSizeOfRTSrc(width, height));
            this.get('_3JSsceneTranspDiffuse').traverse(this.SetSizeOfRTSrc(width, height));
            this.get('_3JSsceneOpaqueDiffuse').traverse(this.SetSizeOfRTSrc(width, height));
            this.get('_3JSrenderer').autoClear = true;
            var whiteClearDepthColor = new THREE.Color(1.,1.,1.);
            var blackClearDepthColor = new THREE.Color(0.,0.,0.);
            var backgroundColor = this.get("_backgroundColor");
            var backgroundAlpha = this.get("_backgroundAlpha");

            this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableBGBlend"]["value"] = 0;//disable background blending for next uses
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableBackdrop"]["value"] = 0;
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["Backdrop"]["value"] = this.get('_DummyTexture');
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["CX1RT"]["value"] = this.get('_DummyTexture');
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["TransparencyStack"]["value"] = this.get('_DummyTexture');
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["OpaqueRT"]["value"] = this.get('_DummyTexture');
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["ColorStackRT"]["value"] = this.get("_DummyTexture");

            this.get('_renderercache')["materials"]["DepthOnlyShaderMaterial"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);
            this.get('_renderercache')["materials"]["D1ShaderMaterial"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);
            this.get('_renderercache')["materials"]["D2ShaderMaterial"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);

            if(this.IsReflectionsRenderingEnabled())
            {
                //Flip all scenes
                this.FlipAllScenes();

                //Flip GL_FRONT to GL_BACK and vice versa to compensate scaling *-1 on Y axis
                this.ApplyFctToAllMaterials(this.FlipMaterialSide(THREE.BackSide));

                //Flip lights positions (for designer quality), otherwise lighting is wrong in reflection (since normal hasn't been flipped)
                this.FlipLights();

                //Depth peel for reflected scenes
                this.RenderDepthPeel(false, cameraRTT, screen_plane, width, height,
                                     MirrorColorStackRT, MirrorTransparencyStackRT, MirrorOpaqueDepthRT, D1RT, D2RT, MirrorPeelColorRT, MirrorOpaqueBackgroundMaskRT,
                                     blackClearDepthColor, whiteClearDepthColor);

                //Unflip all scenes
                this.UnFlipAllScenes();

                this.ApplyFctToAllMaterials(this.FlipMaterialSide(THREE.FrontSide));

                this.UnFlipLights();
            }
            //Depth peel for normal scene
            this.RenderDepthPeel((enableHighlights&&enableHardEdgeHighlights), cameraRTT, screen_plane, width, height,
                                 ColorStackRT, TransparencyStackRT, OpaqueDepthRT, D1RT, D2RT, PeelColorRT, OpaqueBackgroundMaskRT,
                                 blackClearDepthColor, whiteClearDepthColor);

            //Blend with opaque color !
            //Clear alpha channel
            this.get('_3JSrenderer').getContext().colorMask( false, false, false, true );
            this.get('_3JSrenderer').setClearColor( blackClearDepthColor, 0 );
            this.get('_3JSrenderer').clearTarget(ColorStackRT, true, false, false);
            this.get('_3JSrenderer').getContext().colorMask( true, true, true, true );
            this.get('_3JSrenderer').setClearColor( blackClearDepthColor, 1 );

            this.get('_3JSrenderer').autoClearColor = false;

            var screen_quad_color_blend = new THREE.Mesh(screen_plane, this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]);
            var screen_quad_scene_color_blend = new THREE.Scene();
            screen_quad_scene_color_blend.add(screen_quad_color_blend);

            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableBGBlend"]["value"] = 1;
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["BGColor"]["value"] = backgroundColor;
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["BGAlpha"]["value"] = backgroundAlpha;
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["Backdrop"]["value"] = backdropTex;
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableBackdrop"]["value"] = backdropTex ? 1 : 0;
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableBackdropAlpha"]["value"] = backdropTex ? 0 : 1;
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["Backdrop_scale"]["value"] = new THREE.Vector2(bd_scaleX, bd_scaleY);
            this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["Backdrop_offset"]["value"] = new THREE.Vector2(bd_offsetX, bd_offsetY);

            if(this.IsReflectionsRenderingEnabled())
            {
                this.get('_3JSrenderer').autoClearColor = true;
                this.get('_3JSrenderer').setClearColor( blackClearDepthColor, 0 );
                var screen_quad_copy_shader = new THREE.Mesh(screen_plane, this.get('_renderercache')["materials"]["CopyShaderMaterial"]);
                var screen_quad_scene_copy_shader = new THREE.Scene();
                screen_quad_scene_copy_shader.add(screen_quad_copy_shader);

                //Render the reflection first onto standard background
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableMirror"]["value"] = 1 ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["CX1RT"]["value"] = MirrorPeelColorRT.texture ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["TransparencyStack"]["value"] = MirrorTransparencyStackRT.texture;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["OpaqueRT"]["value"] = MirrorOpaqueBackgroundMaskRT.texture ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["ColorStackRT"]["value"] = MirrorColorStackRT.texture;
                this.get('_3JSrenderer').render(screen_quad_scene_color_blend, cameraRTT, MirrorColorStackRT2);

                //Render normal scene using previous render target as background
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableMirror"]["value"] = 0 ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableBackdrop"]["value"] = 1;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableBackdropAlpha"]["value"] = 1;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["Backdrop_scale"]["value"] = new THREE.Vector2(1, 1);
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["Backdrop_offset"]["value"] = new THREE.Vector2(0, 0);
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["Backdrop"]["value"] = MirrorColorStackRT2.texture;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["CX1RT"]["value"] = PeelColorRT.texture ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["TransparencyStack"]["value"] = TransparencyStackRT.texture;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["OpaqueRT"]["value"] = OpaqueBackgroundMaskRT.texture ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["ColorStackRT"]["value"] = ColorStackRT.texture;
                this.get('_3JSrenderer').render(screen_quad_scene_color_blend, cameraRTT, MirrorColorStackRT);

                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableMirror"]["value"] = 0 ;
            }
            else
            {
                this.get('_3JSrenderer').autoClearColor = true;
                this.get('_3JSrenderer').setClearColor( blackClearDepthColor, 0 );
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["EnableMirror"]["value"] = 0 ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["CX1RT"]["value"] = PeelColorRT.texture ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["TransparencyStack"]["value"] = TransparencyStackRT.texture ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["OpaqueRT"]["value"] = OpaqueBackgroundMaskRT.texture ;
                this.get('_renderercache')["materials"]["ColorBlendShaderMaterial"]["uniforms"]["ColorStackRT"]["value"] = ColorStackRT.texture;
                this.get('_3JSrenderer').render(screen_quad_scene_color_blend, cameraRTT, MirrorColorStackRT);
            }

            this.get('_3JSrenderer').autoClearColor = true;

            if((enableHighlights && enableHorizonHighlights) || enableLabels)
            {
                this.get('_3JSrenderer').clear(false, true, false); // clear the hardware Z-buffer in last active render target

                if(enableHighlights && enableHorizonHighlights)
                {
                    // THIRD PASS: Render outline of objects (silhouette of smooth surfaces, opaque + transparent)
                    this.get('_3JSrenderer').autoClear = true;
                    this.get('_3JSrenderer').setClearColor( blackClearColor, 0 );
                    this.RenderOpaqueSceneDepth(OpaqueDepthRT, this.get('_renderercache')["materials"]["colordepth"]);
                    this.get('_3JSrenderer').autoClear = false;
                    this.RenderTransparentSceneDepth(OpaqueDepthRT, this.get('_renderercache')["materials"]["colordepth"]);
                    this.get('_3JSrenderer').autoClear = false;//previous transp scene draw call is setting autoclear back to true

                    this.get('_renderercache')["materials"]["edge"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);
                    this.get('_renderercache')["materials"]["edge"]["uniforms"]["depth_texture"]["value"] = OpaqueDepthRT.texture;
                    this.RenderOpaqueSceneDepth(MirrorColorStackRT, this.get('_renderercache')["materials"]["edge"]);
                    this.get('_3JSrenderer').autoClear = false;
                    this.RenderTransparentSceneDepth(MirrorColorStackRT, this.get('_renderercache')["materials"]["edge"]);
                    this.get('_3JSrenderer').autoClear = false;
                }

                // Render labels
                if(enableLabels)
                {
                    this.get('_3JSrenderer').autoClear = false;
                    this.get('_3JSrenderer').render(this.get('_3JSsceneLabels'), this.get('_3JSHUDcamera'), MirrorColorStackRT);
                }

                this.get('_3JSrenderer').autoClear = true;
            }

            if(!__offscreen)
            {
                // Copy colorstack to screen if needed.
                this.get('_3JSrenderer').setClearColor( blackClearDepthColor, 0 );//black ?
                this.get('_renderercache')["materials"]["CopyShaderMaterial"]["transparent"] = true;
                this.get('_renderercache')["materials"]["CopyShaderMaterial"]["blending"] = THREE.AdditiveBlending;
                this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);
                this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["CXRT"]["value"] = MirrorColorStackRT.texture;
                this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["OpaqueRT"]["value"] = this.get("_DummyTexture");
                this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["EnableMask"]["value"] = 0;
                this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["EnableRefOpacity"]["value"] = 0;
                this.get('_renderercache')["materials"]["CopyShaderMaterial"]["uniforms"]["EnableInvRefOpacity"]["value"] = 0;

                var screen_quad_copy_shader = new THREE.Mesh(screen_plane, this.get('_renderercache')["materials"]["CopyShaderMaterial"]);
                var screen_quad_scene_copy_shader = new THREE.Scene();
                screen_quad_scene_copy_shader.add(screen_quad_copy_shader);
                this.get('_3JSrenderer').render(screen_quad_scene_copy_shader, cameraRTT, null);
            }

            // this.get('_3JSrenderer').setClearColor( backgroundColor, 1 );//Restore clear color
            this.get('_3JSrenderer').autoClear = false;
        }
        else
        {
            //No transparency or basic designer quality transparency, render it using standard shading algorithm,
            //this version has no aliasing problems since we use only the screen as render target.

            //borrow render target to render reflections
            var ReflectionRT = null;
            if(this.IsReflectionsRenderingEnabled())
            {
                var ReflectionRT = this.get('_PeelColorRT');
                var blackClearColor = new THREE.Color(0.,0.,0.);
                this.get('_3JSrenderer').setClearColor( blackClearColor, 1. );

                this.FlipOpaqueScene();
                this.FlipTransparentScene();

                //Flip GL_FRONT to GL_BACK and vice versa to compensate scaling *-1 on Y axis
                this.ApplyFctToAllMaterials(this.FlipMaterialSide(THREE.BackSide));

                //Flip lights positions (for designer quality), otherwise lighting is wrong in reflection (since normal hasn't been flipped)
                this.FlipLights();

                this.RenderOpaqueScene(ReflectionRT, true);
                this.get('_3JSrenderer').autoClear = false;
                this.RenderTransparentScene(ReflectionRT, true);

                this.UnFlipOpaqueScene();
                this.UnFlipTransparentScene();

                this.ApplyFctToAllMaterials(this.FlipMaterialSide(THREE.FrontSide));
                this.UnFlipLights();
            }

            if ( backdropTex )
            {
                this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);
                this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["backdrop"]["value"] = backdropTex;
                this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["backdrop_enabled"]["value"] = 1;
                this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["backdrop_scale"]["value"] = new THREE.Vector2(bd_scaleX, bd_scaleY);
                this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["backdrop_offset"]["value"] = new THREE.Vector2(bd_offsetX, bd_offsetY);

                this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["reflection"]["value"] = ReflectionRT;
                this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["reflection_enabled"]["value"] = this.IsReflectionsRenderingEnabled()? 1:0;

                var backdrop_plane = new THREE.Mesh(screen_plane, this.get('_renderercache')["materials"]["BackdropShaderMaterial"]);
                var backdrop_shader_scene = new THREE.Scene();
                backdrop_shader_scene.add(backdrop_plane);
                this.get('_3JSrenderer').render(backdrop_shader_scene, cameraRTT, __offscreen?ColorStackRT:null);
                this.get('_3JSrenderer').autoClear = false;
            }
            else
            {
                if(this.IsReflectionsRenderingEnabled())
                {
                    this.get('_3JSrenderer').setClearColor( this.get("_backgroundColor"), this.get("_backgroundAlpha") );
                    this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);
                    this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["backdrop"]["value"] = this.get('_DummyTexture');
                    this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["backdrop_enabled"]["value"] = 0;
                    this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["reflection"]["value"] = ReflectionRT.texture;
                    this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["reflection_enabled"]["value"] = 1;
                    this.get('_renderercache')["materials"]["BackdropShaderMaterial"]["uniforms"]["default_bg_color"]["value"] = new THREE.Vector4(this.get("_backgroundColor").r, this.get("_backgroundColor").g, this.get("_backgroundColor").b, this.get("_backgroundAlpha"));

                    var reflection_plane = new THREE.Mesh(screen_plane, this.get('_renderercache')["materials"]["BackdropShaderMaterial"]);
                    var reflection_shader_scene = new THREE.Scene();
                    reflection_shader_scene.add(reflection_plane);
                    this.get('_3JSrenderer').render(reflection_shader_scene, cameraRTT, __offscreen?ColorStackRT:null);
                    this.get('_3JSrenderer').autoClear = false;
                }
                else
                {
                    this.get('_3JSrenderer').setClearColor( this.get("_backgroundColor"), this.get("_backgroundAlpha") );
                    this.get('_3JSrenderer').autoClear = true;
                }
            }

            // Render opaque objects
            this.RenderOpaqueScene(__offscreen?ColorStackRT:null);
            this.get('_3JSrenderer').autoClear = false;
            // Render transparent objects
            this.RenderTransparentScene(__offscreen?ColorStackRT:null);
            this.get('_3JSrenderer').autoClear = false;

            if(enableHighlights)
            {
                if(enableHorizonHighlights)
                {
                    // THIRD PASS: Render outline of objects (silhouette of smooth surfaces, opaque + transparent)
                    this.get('_3JSrenderer').autoClear = true;
                    var OpaqueDepthRT = this.get('_OpaqueDepthRT');
                    this.get('_3JSrenderer').setClearColor( blackClearDepthColor, 0 );
                    this.RenderOpaqueSceneDepth(OpaqueDepthRT, this.get('_renderercache')["materials"]["colordepth"]);
                    this.get('_3JSrenderer').autoClear = false;
                    this.RenderTransparentSceneDepth(OpaqueDepthRT, this.get('_renderercache')["materials"]["colordepth"]);
                    this.get('_3JSrenderer').autoClear = false;

                    this.get('_renderercache')["materials"]["edge"]["uniforms"]["canvas_size"]["value"] = new THREE.Vector2(width, height);
                    this.get('_renderercache')["materials"]["edge"]["uniforms"]["depth_texture"]["value"] = OpaqueDepthRT.texture;
                    this.RenderOpaqueSceneDepth(__offscreen?ColorStackRT:null, this.get('_renderercache')["materials"]["edge"]);
                    this.get('_3JSrenderer').autoClear = false;
                    this.RenderTransparentSceneDepth(__offscreen?ColorStackRT:null, this.get('_renderercache')["materials"]["edge"]);
                    this.get('_3JSrenderer').autoClear = false;
                }

                if(enableHardEdgeHighlights)
                {
                    // FOURTH PASS: Render highlights (hard edges)
                    this.get('_3JSrenderer').render(this.get('_3JSsceneHightlights'), this.get('_3JScamera'), __offscreen?ColorStackRT:null);
                }

            }

            // Render labels
            if(enableLabels)
                this.get('_3JSrenderer').render(this.get('_3JSsceneLabels'), this.get('_3JSHUDcamera'), __offscreen?ColorStackRT:null);
        }

        this.get('_3JSrenderer').autoClear = true;


        if(__offscreen)
        {
            //Reset render target sizes after offscreen render
            this.set("_resetRenderTarget", true);
        }
    },

    StartInteractiveFramerate: function()
    {
        this.set("_interactiveFrameRate", true);
        this.set("_current_nb_peels", this.get("_nb_min_peels"));
    },

    EndInteractiveFramerate: function(__redraw)
    {
        this.set("_interactiveFrameRate", false);
        if(__redraw === undefined || __redraw)
            this.Render();
    },

    /** Animate */
    Animate: function()
    {
        LogMessage.Log("View Animate function");

        var self = this;

        var render = function () {
            if (self.get('_enableAnimations'))
            {
                requestAnimationFrame(render);

                var __asset_camera = self.get('_renderercache').cameras["defaultcamera"];

                var increment = 0.1;
                var updated_az_angle = __asset_camera.azimuthAngle + increment;
                var moduled_angle = (((updated_az_angle+180) % 360) - 180);
                __asset_camera.azimuthAngle = moduled_angle;
                self.CameraUpdate();
                self.Render();
            }
        };

        render();
    },

    /** Resize */
    Resize: function(__event, __overrideWidth, __overrideHeight)
    {
        var scope = this;
        this.get("_build_scene_exec_queue").whenFinished(function()
        {
            var screenwidth = __overrideWidth?__overrideWidth:scope.get('_context').clientWidth;
            var screenheight = __overrideHeight?__overrideHeight:scope.get('_context').clientHeight;

            scope.get('_3JSrenderer').setSize(screenwidth, screenheight);
            scope.set('_3JSHUDcamera', new THREE.OrthographicCamera( - screenwidth/2, screenwidth/2, screenheight/2, - screenheight/2, 0, 10));
            scope.InitializeRenderTargets();
            scope.CameraUpdate( true, __overrideWidth, __overrideHeight);
        });
    },

    UpdatePolygonOffset: function(__enablePolygonOffset)
    {
        var scope = this;
        return function (__material)
        {
            if(__material && __material["polygonOffset"])
            {
                __material["polygonOffsetFactor"] = __enablePolygonOffset ? scope.get( '_polygon_offset_factor') : 0;
                __material["polygonOffsetUnits"] = __enablePolygonOffset ? scope.get( '_polygon_offset_units') : 0;
            }
        };
    },

    /** SetHighlight */
    SetHighlights: function(__enableHighlights, __redraw)
    {
        var materials = this.get('_renderercache')['materials'];
        materials['enablehighlights']= __enableHighlights;

        var enablePolygonOffset = __enableHighlights && this.get('_renderercache')['materials']['enablehardedgehighlights'];
        this.ApplyFctToAllMaterials(this.UpdatePolygonOffset(enablePolygonOffset));

        if(__redraw == undefined || __redraw) this.Render();
    },

    /** SetHighlightsConfiguration */
    SetHighlightsConfiguration: function(__enableHorizonHighlights, __enableHardEdgeHighlights, __redraw)
    {
        var materials = this.get('_renderercache')['materials'];
        materials['enablehardedgehighlights']= __enableHardEdgeHighlights;
        materials['enablehorizonhighlights']= __enableHorizonHighlights;

        var enablePolygonOffset = materials['enablehighlights'] && __enableHardEdgeHighlights;
        this.ApplyFctToAllMaterials(this.UpdatePolygonOffset(enablePolygonOffset));

        if(__redraw == undefined || __redraw) this.Render();
    },

    /** SetHighlightColor */
    SetHighlightsColor: function(__red, __green, __blue, __redraw)
    {
        var highlightColor = (__red << 16) + (__green << 8) + __blue;

        //Change hightlight color for hard edges
        this.get('_renderercache')["materials"]["highlight"]["color"].setHex(highlightColor);
        //Change highlight color for smoothe edges
        this.get('_renderercache')["materials"]["edge"]["uniforms"]["color"]["value"] = new THREE.Vector3(__red/255, __green/255, __blue/255 );

        if(this.get('_renderercache')['materials']['enablehighlights'] && (__redraw == undefined || __redraw)) this.Render();
    },

    /** SetBackgroundColor */
    SetBackgroundColor: function(__red, __green, __blue, __alpha, __redraw)
    {
        this.set("_backgroundColor", new THREE.Color(__red, __green, __blue));
        this.set("_backgroundAlpha", __alpha );

        if(this.get('_renderercache')["materials"].hasOwnProperty("ColorBlendShaderMaterial"))
        {
            this.get('_renderercache')["textures"][this.get('_RenderingQuality')]["backdrop"]["enabled"] = false;
        }

        if(__redraw == undefined || __redraw) this.Render();
    },

    /**
     * SetLabels visibility
     *
     * @param show true to show all labels and false to hide them all
     */
    SetLabelsVisible: function(__show, __redraw)
    {
        this.get('_renderercache')['labels']['visible'] = __show;

        if(__redraw == undefined || __redraw)
            this.CameraUpdate(true);
    },

    /**
    * Reset visiblity of the main scene to its default (in json structure??? to define)
    * default visiblity of the main scene is true at startup
    *
    * @function
    * @param {Boolean} __redraw - If true, redraw the scene after visibility change - default is true
    */
    ResetMainSceneVisibility: function(__redraw)
    {
        var resetvisibilitycallback = function ( obj )
        {
            if(!(obj instanceof THREE.Scene || obj instanceof THREE.Mesh))
            {
                obj.visible = true;
            }
        };
        this.get('_3JSsceneOp').traverse( resetvisibilitycallback );
        this.get('_3JSsceneTransp').traverse( resetvisibilitycallback );
        this.get('_3JSsceneHightlights').traverse( resetvisibilitycallback );
        if(__redraw == undefined || __redraw) this.Render();
    },

    /**
    * Toggle visiblity of a node using its URL in the scene
    * default visiblity of a node is true at startup
    *
    * @function
    * @param {String} __nodeURL - URL identifying a scene node
    * @param {Boolean} __recurseChildren - If true, traverse all children to toggle their visibility state
    * @param {Boolean} __recurseParents - If true, traverse all parents to toggle their visibility state
    * @param {Boolean} __redraw - If true, redraw the scene after visibility change - default is true
    */
    ToggleNodeVisibility: function(__nodeURL, __recurseChildren, __recurseParents, __redraw)
    {
        this.SetNodeVisibility(__nodeURL, null, __recurseChildren, __recurseParents, __redraw)
    },

    /**
    * Set visiblity state of a node using its URL in the scene
    * default visiblity of a node is true at startup
    *
    * @function
    * @param {String} __nodeURL - URL identifying a scene node
    * @param {Boolean} __visible - Set the object visibility to __visible
    * @param {Boolean} __recurseChildren - If true, traverse all children to set their visibility to __visible
    * @param {Boolean} __recurseParents - If true, traverse all parents to set their visibility to __visible
    * @param {Boolean} __redraw - If true, redraw the scene after visibility change - default is true
    */
    SetNodeVisibility: function(__nodeURL, __visible, __recurseChildren, __recurseParents, __redraw)
    {
        var objects;
        var shortNodeURL = this.URLToKeyString(__nodeURL);

        if(this.get('_renderercache')['nodeviewproperties'][shortNodeURL] === undefined)
            return;

        for (var i = 0, keys = Object.keys(this.get('_renderercache')['nodeviewproperties'][shortNodeURL]['threejsnodes']), l = keys.length; i < l; ++i)
        {
            var setvisibilitycallback = function ( obj )
            {
                if(!(obj instanceof THREE.Scene || obj instanceof THREE.Mesh))
                {
                    if(__visible === null)
                    {
                        obj.visible = !obj.visible ;//Simply toggle
                    }
                    else
                    {
                        obj.visible = __visible ;//Force visibility state
                    }
                }
            };

            //Get all objects found in the available passes/modes
            objects = this.get('_renderercache')['nodeviewproperties'][shortNodeURL]['threejsnodes'][keys[i]];

            for (var node_cg = 0, keys2 = Object.keys(objects), l2 = keys2.length; node_cg < l2; node_cg++)
            {
                var object = objects[keys2[node_cg]];

                setvisibilitycallback (object);
                if(__recurseChildren != undefined && __recurseChildren)
                {
                    for (var j = 0, keys3 = Object.keys(object.children), l3 = keys3.length; j < l3; ++j)
                    {
                        var id = keys3[j];
                        object.children[id].traverse(setvisibilitycallback);
                    }
                }
                if(__recurseParents != undefined && __recurseParents)
                {
                    if(object.parent)
                    {
                        object.parent.traverseParents(setvisibilitycallback);
                    }
                }
            }
        }
        if(__redraw == undefined || __redraw) this.Render();
    },

    /**
     * Get the queried data info of a node
     * @function
     * @param {string} __object_node - the type of information we want to retrieve from node
     * @param {string} __datatype -  the type of information we want to retrieve from node
     * to choose among the following list:
     * - NODEOBJECTVIS returns (Boolean)
     * @returns object, boolean, string, depending on the type of info
     */
    GetNodeInfo: function(__object_node, __datatype)
    {
        switch(__datatype)
        {
            case "NODEOBJECTVIS":
                //Get the first object found in the first pass available
                var objects = __object_node['threejsnodes'][Object.keys(__object_node['threejsnodes'])[0]];
                var object = objects[Object.keys(objects)[0]];
                return object.visible;
            break;
            default:
                LogMessage.Log("Unknown queried data type");
        }
    },


    /**
     * Get the queried data info of a node using its URL
     *
     * @function
     * @param {string} __nodeURL -  the type of information we want to retrieve from node
     * @param {string} __datatype -  the type of information we want to retrieve from node
     * to choose among the following list:
     * - NODEOBJECTVIS returns (Boolean)
     * @returns object, boolean, string, depending on the type of info
     */
    Get: function(__nodeURL, __datatype)
    {
        var shortNodeURL = this.URLToKeyString(__nodeURL);
        var object_ref =  this.get('_renderercache')['nodeviewproperties'][shortNodeURL];
        if (object_ref)
        {
            return this.GetNodeInfo(object_ref, __datatype);
        }else
        {
            LogMessage.Log("ViewerView Warning: No existing node for "+ shortNodeURL);
        }
    },

    /**
     * Prints a memory report
     * @function LogMemoryReport
     */
    LogMemoryReport: function()
    {
        LogMessage.Log("Node View Properties Size: "+WV3DUtils.get('SizeOf')(this.get('_renderercache').nodeviewproperties)/1000000 + "MB");
        LogMessage.Log("_3JSsceneOp Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneOp'))/1000000 + "MB");
        LogMessage.Log("_3JSsceneOpDepth Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneOpDepth'))/1000000 + "MB");
        LogMessage.Log("_3JSsceneOpaqueDiffuse Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneOpaqueDiffuse'))/1000000 + "MB");
        LogMessage.Log("_3JSsceneTransp Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneTransp'))/1000000 + "MB");
        LogMessage.Log("_3JSsceneTranspDepth Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneTranspDepth'))/1000000 + "MB");
        LogMessage.Log("_3JSsceneTranspDiffuse Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneTranspDiffuse'))/1000000 + "MB");

        //Get size of opaque scene for each coincident group, with its own shading passes
        for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
            LogMessage.Log("_3JSsceneOpCoincidentGroups["+cg_id+"] Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneOpCoincidentGroups')[cg_id])/1000000 + "MB");
            LogMessage.Log("_3JSsceneOpCoincidentGroupsDepth["+cg_id+"] Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneOpCoincidentGroupsDepth')[cg_id])/1000000 + "MB");
            LogMessage.Log("_3JSsceneOpCoincidentGroupsDiffuse["+cg_id+"] Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneOpCoincidentGroupsDiffuse')[cg_id])/1000000 + "MB");
        }

        //Get size of opaque scene for each transparent coincident group, with its own shading passes
        for (var cg = 0; cg < this.get('_3JSsceneTranspCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneTranspCoincidentGroupsPasses')[cg];
            LogMessage.Log("_3JSsceneTranspCoincidentGroups["+cg_id+"] Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneTranspCoincidentGroups')[cg_id])/1000000 + "MB");
            LogMessage.Log("_3JSsceneTranspCoincidentGroupsDepth["+cg_id+"] Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneTranspCoincidentGroupsDepth')[cg_id])/1000000 + "MB");
            LogMessage.Log("_3JSsceneTranspCoincidentGroupsDiffuse["+cg_id+"] Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneTranspCoincidentGroupsDiffuse')[cg_id])/1000000 + "MB");
        }

        LogMessage.Log("_3JSsceneHightlights Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneHightlights'))/1000000 + "MB");
        LogMessage.Log("_3JSsceneLabels Size: "+WV3DUtils.get('SizeOf')(this.get('_3JSsceneLabels'))/1000000 + "MB");
    },

    /**
     * Print list of opaque scene vertices in world coordinates
     */
    PrintOpaqueSceneVertices: function()
    {
        LogMessage.Log("List of vertices (opaques scene)");

        var v1 = new THREE.Vector3();

        var OuputVertex = function(v)
        {
            if(v instanceof THREE.Vector3 && !isNaN(v.x) && !isNaN(v.y) && !isNaN(v.z))
            {
                LogMessage.Log("("+v.x+","+v.y+","+v.z+")");
            }
        }

        var PrintNodeVertices = function ( node )
        {
            var geometry = node.geometry;
            if ( geometry !== undefined )
            {
                //We have to distinguish between standard threejs geometry type
                //and threejs buffered geometry since the access method is different
                //for the two types. (vector type copy VS index array access)
                //In addition to that we must transformation again instanced buffer
                //geometry (inherits buffered geometry) that contains additional transformation
                // in its 'inst_translation', 'inst_rotation', and 'inst_scale' (instantiation) attributes
                if ( geometry instanceof THREE.Geometry )
                {
                    var vertices = geometry.vertices;
                    for ( var i = 0, il = vertices.length; i < il; ++i )
                    {
                        v1.copy( vertices[ i ] );
                        v1.applyMatrix4( node.matrixWorld );
                        OuputVertex( v1 );
                    }
                } else if ( geometry instanceof THREE.BufferGeometry && geometry.attributes[ 'position' ] !== undefined)
                {
                    var positions = geometry.attributes[ 'position' ].array;
                    var localTempBox;
                    if(geometry instanceof THREE.InstancedBufferGeometry)
                    {
                        localTempBox = new THREE.Box3();
                    }
                    for ( var i = 0, il = positions.length; i < il; i += 3 ) {

                        v1.fromArray( positions, i );
                        v1.applyMatrix4( node.matrixWorld );

                        if(geometry instanceof THREE.InstancedBufferGeometry
                            && geometry.attributes[ "inst_translation" ] !== undefined
                            && geometry.attributes[ "inst_rotation" ] !== undefined
                            && geometry.attributes[ "inst_scale" ] !== undefined )
                        {
                            for( var ins = 0, nbins = geometry.maxInstancedCount; ins < nbins; ++ins)
                            {
                                var transform = THREE.GetInstanceTransform( geometry.attributes[ "inst_translation" ],
                                                                            geometry.attributes[ "inst_rotation" ],
                                                                            geometry.attributes[ "inst_scale" ], ins);

                                OuputVertex(THREE.Vector3.transformVertex(v1, transform));
                            }
                        }
                        else
                        {
                            OuputVertex( v1 );
                        }
                    }
                }
            }
        }

        this.get('_3JSsceneOp').updateMatrixWorld( true );
        this.get('_3JSsceneOp').traverse(PrintNodeVertices);
        for (var cg = 0; cg < this.get('_3JSsceneOpCoincidentGroupsPasses').length; ++cg)
        {
            var cg_id = this.get('_3JSsceneOpCoincidentGroupsPasses')[cg];
            this.get('_3JSsceneOpCoincidentGroups')[cg_id].updateMatrixWorld( true );
            this.get('_3JSsceneOpCoincidentGroups')[cg_id].traverse(PrintNodeVertices);
        }
    },

    /**
     * SetReflections
     *
     * @function SetReflections sets the reflection on, off, or relative to what is set in the Json file.
     * @param {string} - a string, "ON", "OFF", or "JSON" ("JSON" is already set by default)
     */
    SetReflections: function(__config, __opacity, __fade, __position, __fadingSize, __redraw)
    {
        this.set("_reflection_config", __config);
        if(__config == "ON")
        {
            this.UpdateReflectionUniforms(__opacity, __fade, __position, __fadingSize);
        }
        else if(__config == "JSON")
        {
            this.UpdateReflectionUniforms();
        }
        if(__redraw == undefined || __redraw) this.Render();
    },

    /** SetAnimations */
    SetAnimations: function(__enableAnimations)
    {
        this.set('_enableAnimations', __enableAnimations);
        this.Animate();
    },

    /** SetInteractionOptimization */
    SetInteractionOptimization: function(__enableInteractionOptimization)
    {
        this.set('_enableInteractionOptimization', __enableInteractionOptimization);
    },

    /**
     * SetQualityLevel
     *
     * @function SetQualityLevel sets maximum quality when no interaction with the model or when interaction optimization is disabled
     * by setting the number of transparency peels for transparent models. This quality level does not affect quality of opaque models.
     * @param {number} - an integer between 1 and 10 (included)
     */
    SetQualityLevel: function(__qualityLevel, __redraw)
    {
        this.set("_nb_max_peels", __qualityLevel);
        if(__redraw == undefined || __redraw) this.Render();
    },

    /**
     * SetLightingContrast
     *
     * @function SetLightingContrast sets the lighting contrast (in designer quality only)
     * by reducing the directional light and increasing the ambient light components.
     * @param {number} - a floating number between 0 and 1 (included)
     */
    SetLightingContrast: function(__lightingContrast, __redraw)
    {
        this.set("_lighting_contrast",__lightingContrast);
        this.UpdateLightingContrast();
        if(__redraw == undefined || __redraw) this.Render();
    },

    UpdateLightingContrast: function()
    {
        // set directional light intensity
        // default is THREE.DirectionalLight( 0xffffff, 0.5 ) for __lightingContrast == 0.5;
        for (var cg_idx = 0, cg_keys = Object.keys(this.get('_renderercache')['lights']['dir_light']), l = cg_keys.length;
             cg_idx < l; cg_idx++)
        {
            var __coincident_group_id = cg_keys[cg_idx];
            var directionalLight = this.get('_renderercache')['lights']['dir_light'][__coincident_group_id];
            directionalLight.intensity = this.get("_lighting_contrast");
        }

        // set ambient lighting intensity
        // default is THREE.AmbientLight(0x777777) for __lightingContrast == 0.5;
        for (var cg_idx = 0, cg_keys = Object.keys(this.get('_renderercache')['lights']['amb_light']), l = cg_keys.length;
             cg_idx < l; cg_idx++)
        {
            var __coincident_group_id = cg_keys[cg_idx];
            var ambientLight = this.get('_renderercache')['lights']['amb_light'][__coincident_group_id];
            ambientLight.intensity = 1 + (0.5 - this.get("_lighting_contrast"));
        }
    },

    /** Append the Renderer to the target DOM element */
    AppendRendererToDocument: function()
    {
        if(typeof (this.get('_context').appendChild) === 'function')
        {
            this.get('_context').appendChild(this.get('_3JSrenderer').domElement);
        }
    },


    /**
    * Get a snapshot of the current viewer state as specified in format using offscreen rendering
    *
    * @function
    * @param {string} [format="png"] - image format
    * @returns {Object} image data
    */
    GetSnapshot : function(__format, __width, __height)
    {
        var flipV = function(__imageData)
        {
            'use strict';
            var si, di, sp, dp, temp_swap;
            // loop through over half of row of pixels
            for (var row=0;row<__imageData.height/2;row++)
            {
            // loop over every column
                for (var col=0;col<__imageData.width;col++)
                {
                    // source pixel
                    sp=(__imageData.width*row)+col;

                    // destination pixel
                    dp=(__imageData.width*((__imageData.height-1)-row))+col;

                    // source and destination indexes, will always reference the red pixel
                    si=sp*4;
                    di=dp*4;

                    //swap red channel
                    temp_swap = __imageData.data[di];
                    __imageData.data[di]=__imageData.data[si];
                    __imageData.data[si]=temp_swap;
                    //swap green channel
                    temp_swap = __imageData.data[di+1];
                    __imageData.data[di+1]=__imageData.data[si+1];
                    __imageData.data[si+1]=temp_swap;
                    //swap blue channel
                    temp_swap = __imageData.data[di+2];
                    __imageData.data[di+2]=__imageData.data[si+2];
                    __imageData.data[si+2]=temp_swap;
                    //swap alpha channel
                    temp_swap = __imageData.data[di+3];
                    __imageData.data[di+3]=__imageData.data[si+3];
                    __imageData.data[si+3]=temp_swap;
                }
            }
        }

        this.Render(true, __width, __height);

        var gl = this.get('_3JSrenderer').getContext();
        var screenshotdata = new Uint8Array(__width * __height * 4);
        //If bug on safari check screenshotdata['data'] !!!
        gl.readPixels( 0, 0, __width, __height, gl.RGBA, gl.UNSIGNED_BYTE, screenshotdata );

        var canvas = document.createElement('canvas');
        canvas.width = __width;
        canvas.height = __height;
        var context = canvas.getContext('2d');

        // Copy the pixels to a 2D canvas
        var imageData = context.createImageData(__width, __height);
        imageData.data.set(screenshotdata);

        // Flip the image data since opengl buffer is mirrored compared to png
        flipV(imageData);
        context.putImageData(imageData, 0, 0);

        if(typeof (this.get('_context').appendChild) === 'function')
        {
            this.CameraUpdate(true, this.get('_context').clientWidth, this.get('_context').clientHeight);
        }

        return canvas.toDataURL('image/'+__format);
    }


    });

    return WV3DViewerView;

});
