define(["wcr/webcenter_html",
        "wcr/webcenter_xml",
        "wcr/documentSourceHandler",
        "component/passwordpopup/PasswordValidationPopup",
        "attributecategory/Attribute",
        "attributecategory/Property",
        "component/smartnames/savedsearchcontext/SavedSearchContextInfo",
        "component/smartnames/savedsearchcontext/AttributeCategoryDynamicSmartNameCollector",
        "component/button/Button",
        "doctablex/DocTableXUtils",
       	"attributecategory/CustomFormExtension!", // note the exclamation mark! we are executing a require plugin to load the CustomFormExtension File
       	"jquery",  
        "underscore",
        "tinyMCE",
        "utils/DomUtils",
        "utils/webi18n!DocSrcHdlrStrings",
        "utils/webi18n!ACatStrings",
        "utils/EventBasedValidator",
        "utils/AttributeEventBus",
        "utils/ConvertUtils",
        "utils/AjaxCache",
        "utils/ExecutionQueue",
        "lib/summernote/summernote",
        "utils/webcss!lib/summernote/summernote.css",
		"wcr/ObjectArrayCloneUtil",
        "utils/Promisify",
        "jqueryUI",
        "lib/PromisePolyfill",
        "wcr/json-minified",
        "wcr/hover",
        "wcr/modalpopup"], 
function(WCHTML,
          WCXML,
          DocumentSourceHandler,
          PasswordValidationPopup,
          Attribute,
          Property,
          SavedSearchContextInfo,
          AttributeCategoryDynamicSmartNameCollector,
          Button,
          DocTableXUtils,
		  customFormExtension,
          $,
          _,
          tinyMCE,
          DomUtils,
          DocSrcHdlrStrings,
          ACatStrings,
          EventBasedValidator,
          AttributeEventBus,
          ConvertUtils,
          AjaxCache,
          ExecutionQueue,
          summernote,
          summernotecss,
		  ObjectHandler,
          Promisify) {
	
    /** AttributeCategory.js
    * @author: HADW
    */
    /**
    * Structure of main object attributeCategory  {
    *   attributes: array of attObj elements
    *   cells: 2D array containing the layout and the attributes as cellObj objects of the current view
    *   nRows: number of rows in the table model
    *   nCols: number of columns in the table model
    *	layout: layout of the current view, one of OneColumn, Grid
    *	currentUser: instance of currentUser derived from global gCurrentUser object
    *   valueClass: Class to be used for the values
    *   labelClass: class to be used for the labels
    *   errorMsg:  error message which will be displayed after ajax loading failsla
    *   name: name of the attribute category
    *   type: to what WCR object this attribute category is attached (one of: project, document, task)
    *	projectID: ID of project this attr. cat. is attached to. Initialized for all types (if known)
    *	documentID: ID of document this attr. cat. is attached to. Used only if type = document 
    *	docVersionID: ID of document version this attr. cat. is attached to. Used only if type = document
    *	taskID: ID of task this attr. cat. is attached to. Used only if type = task
    *   views: array of viewObj. if views is null, no views are used (so only the default view)
    * 	rules: array of ruleObj, each one representing a (visibility) rule used in the attribute category
    *	styles: array of styleObj, each one representing a style used in the attribute category
    *	keepSessionAlive: if true, will send keepSessionAlive ajax calls to the web server to keep the session alive. default off
    * 	currentView: name of the current view. One of 'Default', 'Creation', 'Header' or a self-chosen string (literal, no tranlations)
    * 	currentViewIdx: index of the current view in this.views (o for Default, 1 for Creation, 2 for Header, then going further in order of creation, so lowest for a custom view is 3, even if Header does not exist
    * }
    * Structure of attObj {
    *   id: id of the attribute like in the database
    *   name: name of the attribute
    *   type: non-localized type of the attribute
    *   localizedType: localized type of the attribute
    *   value: value converted to the preference unit if applicable
    *   unit: non-localized unit of the attribute
    * 	localizedUnit: localized unite of the attribute
    * }
    * Structure of cellObj {
    *   type:  type of the cell, one of Attribute, Title, Table, Ruler, Spacer, HeaderBar, Button
    *   id: 	id of the attribute put on this position. Null if it's not an attribute
    *   colSpan: 	Number of columns which this element will span. These subsequent cells should be null in the data model.
    *   attObj: attribute object (an element of this.attributes) if type is "Attribute"
    *   label:	Label which will be written in the first column, or text for a title
    *   background: color value of the background for spacer cells
    * }
    * Structure of viewObj {
    *    nRows: number of rows in the view
    *    nCols: number of columns in the view
    *    layout: layout of the view
    *    name: name of the view
    * }
    * Structure of currentUser {
    * 	userID: ID of the currently logged in user
    * 	lang: language tag of the user (f.e. en, fr, cz, etc.)
    * 	units: units set for the user (metric or imperial)
    * 	tzOffset: time zone offset set for the user
    * 	canEditAttributes: flag whether the user can edit attributes or just see them read-only.
    * }
    */

    // NOTE: PJPR: these are kept as real globals, to make the attributecategorylayout work correctly... should be fixed in the future!


    //TO DO HADW: (these are minor problems)
    // Editing a JavaScript button: when you select another cell before defocusing the javaScript text area, you lose your changes


    window.aCatStoredObj= null;  // purely for debugging
    window.debugMsg = "";   // also for debugging
    window.sessionLostMsg = "Your session is lost. You will not be able to save your work. Please log on again";

    window.cMaxRepaints = 250;  // number of repaints caused by rules, protection against endless loops
    window.cMaxButtonClick = 200;   // maximum number of times you are allowed to click a button - to detect endless loops
    window.cMaxResultsDBValues = 2000;  // maximum number of values you ever get from the database for cascading dropdowns.  Beyond this point users must use filtering
    window.gACatCache = new Array();



    /**
    * attributeCategory constructor
    * @param aCatOptions: object with all the options. See the UDD for more details.
    */

    function attributeCategory(aCatOptions) {
        try {
            // put a debounce on the method _displayAttributeCategory to prevent a queue of rendering calls 
            // when there are a lot of lists or button dependencies (rendering happens on every change,
            // while it should only be done once for chains)
            this.displayAttributeCategory = WCHTML.debounce(this._displayAttributeCategory, 100);

            this.aCatOptions = aCatOptions;
            this.displayId = aCatOptions.ACatPanel;
            this.id = aCatOptions.ACatObj;
            this.saveFieldId = aCatOptions.saveFieldId;
            var storeElem = document.getElementById(this.id);
            if (storeElem) storeElem.ACatObj = this;
            this.functionPrefix = 'javascript:document.getElementById(\'' + this.id + '\').ACatObj';        
            this.readURL = aCatOptions.readURL;
            this.currentUser = aCatOptions.currentUser;
            this.labelClass = "darktext_darklink_bold_11";
            this.valueClass = "darktext_darklink_11";       
            this.alternateBackground = true;  // lightgrey/white alternating background by default
            this.cViewOptions = new Array("Default","Creation", "ProjectCreation", "Header", "TaskExecution");  // possible view names - contrary to its naming convention, it gets extended if there are more views defined...
            this.cOperators = new Array("=",">",">=","<","<=","!=","One Of");
            this.alignmentOptions = new Array("Left","Center","Right");
            this.fontOptions = new Array("Roboto", "Verdana","Arial","ArialNarrow","Comic Sans MS","Courier New","Georgia","Times New Roman","Trebuchet, Trebuchet MS");
            this.fontStyleOptions = new Array("Normal","Italic","Oblique");
            this.fontWeightOptions = new Array("Normal","Bold");
            this.widthHeightUnitOptions = new Array("px","%");
            this.documentLinkOperations = ['Link','Copy','CopyLink','Move'];  // possible operations when adding a new document link
            this.documentLinkSources =['Browse','ProjectFolder','DocumentTemplates','SavedSearch'];	  // possible methods to find a browse/find a document to link via a document link	
            this.dlsProperties = ['sourceType','label','documentType','operation','targetFolder','sourceFolder','sourceProject','savedSearch', 'DAMSystem', 'DAMMetadata'];
            this.buttonFuncOptions = ['LookUpAttributes','CheckUniqueValue','JavaScript'];
            this.delayedLoading = aCatOptions.delayedLoading; // with this flag, only the category XML is loaded and nothing is displayed. 
                                                              // Lists and values are obtained from the server when a view is loaded 
            this.docSavedSearchNameList = aCatOptions.docSavedSearchNameList;

            this.submitInProgress = false;   // to control "window.onbeforeunload logic" on all pages with attribute category
            
            this.components = new Array(1, 2); //1:textfield, 2:dropdown
            this.showTypeAsUnit = true;  // show type of attributes and dropdowns if not a unit

            // softwareVersion: Used for fixing incompatibilities in the XML format of the attribute category
            // version 3 is introduced in WCR 14.1. Changes:
            // Interpretation of invalidateAutomatically in the XML for database lists. Value when reading from version 2 is ignored and always set to true since that is the actual behavior (setting was ignored in WCR14).		
            this.softwareVersion = 4;

            // 'docRefAttrInReferenceCategory' is there to prevent endless loops when doc. ref. attribute is 
            // placed in an attribute category used as a Reference Category. It effectively prevent
            // the doc. ref. attribute value to be loaded.
            this.docRefAttrInReferenceCategory = aCatOptions.docRefAttrInReferenceCategory || false;

            // This flag switches the loading of doc. ref. attribute values to a "limited mode" that
            // prevents infinite loops. It's also used during unique attribute value identifier generation.
            this.isDocRefPropertiesACat = aCatOptions.isDocRefPropertiesACat || false;

            // This flag similar to above 'this.isDocRefPropertiesACat' switches the loading of 
            // doc. ref. attribute values to a "limited mode" that prevents infinite loops.
            this.isProjectInfoACat = aCatOptions.isProjectInfoACat || false;


            this.parentDocRow = aCatOptions.parentDocRow || null;
            
            // used to propagate 'isInactive' flag from parent attributes to descendants 
            // (used in document reference context only!)
            this.isInactive = aCatOptions.isInactive || false;
            
            // used to limit number of attempts to show the save button. Without this flag it will 
            // be "shown" for each editable attribute (during its rendering)
            this.isSaveButtonShown = false;
            
            // used in document references context only! it's used there to limit the depth of descendats
            // loading and initialization to a bare minimum to optimize speed and memory consumption
            // it is also a fundamental part of validation logic that skips descendants not seen by the user!
            this.aCatNotExpandedYet = aCatOptions.aCatNotExpandedYet || false;
            
            this.multiSep = ' | ';   // separator for multi-value attributes
            this.valueChangedListeners = new Array();
            this.loaded = false;
            this.BOMAllowed = aCatOptions.BOMAllowed || false;
            
            /*
             * Browser compatibility:
             * Alternative rendering will not work in Safari, so disable it
             * Don't disable it while editing though.
             */
            if (navigator.userAgent.indexOf('Safari') != -1 && navigator.userAgent.indexOf('Chrome') == -1) {
            	this.blockAlternativeRendering = true;
            }

            // Setup of list reading via servlet for cascading dropdowns based on Excel
            if (aCatOptions.readListsURL) {
                this.readListsURL = aCatOptions.readListsURL;
            } else {
                this.readListsURL = "ReadExcelServletCombined";
            }
            if (aCatOptions.allListsURL) {
                this.allListsURL = aCatOptions.allListsURL;
            } else {
                this.allListsURL = "ReadExcelServletCombined?action=getlists";
            }

            this.isTaskTypePage = aCatOptions.isTaskTypePage;
            if(this.isTaskTypePage){
                this.canEditTaskType = aCatOptions.canEditTaskType;
                this.assignRolesByTaskSpecsCheckbox = aCatOptions.assignRolesByTaskSpecsCheckbox;
                this.autoUpdateProjAttrsCheckbox = aCatOptions.autoUpdateProjAttrsCheckbox;
                this.cleanProjAttrsViaEmptySpecs = aCatOptions.cleanProjAttrsViaEmptySpecs;
            }
            this.projectAttributeValuesURL = aCatOptions.projectAttributeValuesURL;

            // adjust rendering for printing
            if (aCatOptions.printLayout) {
                this.printLayout = aCatOptions.printLayout;
            } else {
                this.printLayout = false;			
            }

            // set up for help windows
            var helpHeight = 500, helpWidth = 500; 
            var leftVal = (screen.width - helpWidth) / 2;
            var topVal = (screen.height - helpHeight) / 2;
            this.helpOptionsString = '&quot;scrollbars=1, width='+ helpWidth +' ,height='+ helpHeight +',top='+topVal+',left='+leftVal+' &quot';

            this.terms = this.addTerms();
            this.showEditTerminology = aCatOptions.useTerminology || false;
            if (this.isTaskTypePage){
            	this.showEditTerminology = false;  // new in 14 but behind easter egg
            }
            this.showInfoPage = true;	// moved to default true from WCR 12 onwards


            this.currentViewIdx = 0; // index in cViewOptions of the current view 
            this.currentViewFound = false;
            this.noCurrentViewDisplay = false;  // indicates that a task type view was asked but not found --> nothing will be displayed and lists will not be loaded

            // Initialize Styles Gallery
            this.styles = new Array(); // array containing standard and user defined styles
            // standard styles
            if(aCatOptions.defaultStyleObject){
            	// style was defined in a lever higher than this object and passed along; make it the default
            	var defaultStyle = aCatOptions.defaultStyleObject;
            	defaultStyle.name = "WebCenter Attribute";
            	this.styles.push(defaultStyle);
            } else {
            	this.styles.push({
                    name:"WebCenter Attribute",
                    classLabel:"attributeLabel",
                    classValue:"",
                    standardStyle:true
                });
            }
            this.styles.push({
                name:"WebCenter Title",
                classLabel:"label-title",
                classValue:"",  // value not used for titles
                standardStyle:true
            });

            // Initialize Rules Gallery
            this.rules = new Array(); // array containing standard and user defined rules
            this.rules.push({
                name:"Editable",
                standardRule:true
            });
            this.isRequiredRule = {
                name:"Required",
                standardRule:true
            };
            this.rules.push(this.isRequiredRule);
            this.rules.push({
                name:"ReadOnly",
                standardRule:true
            });
            this.rules.push({
                name:"Hidden",
                standardRule:true
            });
            this.rules.push({
                name:"HiddenKeepValues",
                standardRule:true
            });

            // Initialize Document Sources Gallery: list of all document sources used for document references
            this.docSourcesGallery = new Array();  // as long as no document references are used, this gallery is empty		
            this.dshConfigTablePHID = 'docSourceConfigTableID';		

			// Initialize Document Render Gallery: list of all document table rendering configuration used for document references
            // as long as no document references are used, this gallery is empty
			this.docRenderGallery = new Array();  // array of DocTableXRenderConfig objects
            this.layoutId = aCatOptions.layoutId;

            this.cells = new Array();  // array containing the cells of both onecolumn and grid layout. Always 2D even for OneColumn
            this.debug = 0;  // to debug ajax calls
            this.nRows = 0;
            this.nCols = 0;
            this.name = "";


            this.typeOptions = ['project','document','task'];   // possible options for attribute type
            if (aCatOptions.type) this.type = aCatOptions.type;   // even this might be empty (in searches)
            if (aCatOptions.projectID) this.projectID = aCatOptions.projectID; else this.projectID = "";
            if (aCatOptions.documentID) this.documentID = aCatOptions.documentID; else this.documentID = "";
            if (aCatOptions.docVersionID) this.docVersionID = aCatOptions.docVersionID; else this.docVersionID = "";
            if (aCatOptions.taskID) this.taskID = aCatOptions.taskID; else this.taskID = "";


            this.allAttributesURL = aCatOptions.allAttributesURL;

            if (aCatOptions.view){
        		this.currentView = aCatOptions.view;
        	} else{
        		this.currentView = "Default";
        	}            
            this.requestedView = this.currentView;  // remembers what was originally asked as view.  Needed for remembering that a creation view was asked to also submit hidden/readonly attributes

            // prefix to be put before names and id to make distinction between different panels on the same page
            if (aCatOptions.prefix) this.prefix = aCatOptions.prefix; else this.prefix="";
            if (aCatOptions.callbackOnLoad) this.callbackOnLoad = aCatOptions.callbackOnLoad;
            if (aCatOptions.callbackOnFail) this.callbackOnFail = aCatOptions.callbackOnFail;
            if (aCatOptions.callbackOnRender) this.callbackOnRender = aCatOptions.callbackOnRender;
            
		
			
            // in printlayout, all Rich Text Attribute iframes must shown completely (no scrollbars)
            if(this.printLayout){
            	var existingRenderCallback = this.callbackOnRender;
            	this.callbackOnRender = function(){
            		if(existingRenderCallback){
            			existingRenderCallback();
            		}					
					this.prepareForPrint();
            	};
            }

			// when the browser will print, we will also modify the styling of iframes of rich text attributes
			// IE/FF support beforeprint, Chrome supports mediaquery listeners
			window.addEventListener ("beforeprint",this.prepareForPrint.bind(this));	
			var mediaQueryList = window.matchMedia('print');
			mediaQueryList.addListener(function(mql) {
				if (mql.matches) {
					this.prepareForPrint();
				}
			}.bind(this));

            
            // default hour, given as a number between 0 and 47 (number of half hours)
            this.defaultHourAsHalfHours = 34;
            if (aCatOptions.defaultHourAsHalfHours) this.defaultHourAsHalfHours = aCatOptions.defaultHourAsHalfHours;

            // note for templates only document reference blocking issues are checked (example: missing
            // document name during upload), not the required fields.
            this.isTemplate = (aCatOptions.isTemplate && (aCatOptions.isTemplate == '1'));

            // check whether values of attributes need to be loaded
            this.copyValues = true;
            if (aCatOptions.copyValues != null) this.copyValues = aCatOptions.copyValues;

            // check whether grid layout is allowed (grid layout is a licensed option in WebCenter)
            this.allowGridLayout = true;
            if (aCatOptions.allowGridLayout != null) this.allowGridLayout = aCatOptions.allowGridLayout;

            this.checkIdenticalLists = false;  // remembers whether you already checked for identical lists

            this.valuesURL = aCatOptions.valuesURL;  // URL to load the values via Ajax
            this.valuesURLMerge = aCatOptions.valuesURLMerge;  // URL to load the values via Ajax to merge with first loaded values
            if (this.currentView != 'Header' && !window.aCatStoredObj) window.aCatStoredObj = this;  // easier to debug in firebug

            this.addOtherProperties(); 
            
            if (aCatOptions.readURL) {  // read the attribute category via ajax
                this.readACatFromXML(aCatOptions.readURL);
            };

            /** @member {boolean} attributesChanged - true if at least one attribute has been changed from the last save. */
            this.attributesChanged = false;

            if(this.autoUpdateProjAttrsCheckbox) tinyMceMgrObj.attCatObj = this; // in this case, the onchange of the rich text attributes need the cell details
                                                                                 // always adding it would lead to conflicts when there are multiple layouts on one page
            this.keepSessionAlive = false;
            this.useAlternativeRendering = false;

            // flag to see if this category is imported (ID's won't match in that case)
            this.imported = false;

            if (aCatOptions.projectID) {
                this.projectID = aCatOptions.projectID;			
            }
            
            // listener reacting on any attribute value change in this attribute category
            this._attrValueChangeListenerCallback = _.bind(this.onChangeAttrValueEventHandler, this);
            // listener reacting on any new attribute category instantiated and fully loaded (not necessarily displayed)
            this._categoryLoadedListenerCallback = _.bind(this.onCategoryLoadedEventHandler, this);

            // flag to limit number of this.onCategoryLoadedEventHandler() calls to exactly one for each attribute category
			this._onCategoryLoadedEventProcessed = false;

            var attrEventBus = AttributeEventBus.getInstance();
            attrEventBus.onChangeAttrValueListener(this._attrValueChangeListenerCallback);
            attrEventBus.onCategoryLoadedListener(this._categoryLoadedListenerCallback);

            // Stores the DOM element of a certain attribute by id of that attribute
            this._attributeElementMap = {};
            
            // Stores the array of callbacks that are fired when the cascading dropdowns are deselected/cleared
            this._listValuesDeselectedCallbacks = [];
            
            // If set to true than AttributeCategory will not attempt to copy the changes made to an attribute to other attributes on the same page
            this._disableSynchronizingAttributes = aCatOptions.disableSynchronizingAttributes || false;
			
			// Array of callbacks that listen to changes of cascading dropdowns with multiple values
			this._searchSelectMultipleCallbacks = [];
        } catch(err) {
            alert(ACatStrings.t('ACatConstructorProblem') + err.lineNumber + ': ' +  err.message );
        }
        return this;
    }
	
	/**
	 * Resize richtextfields so they are printed without scrollbars
	 */
	attributeCategory.prototype.prepareForPrint = function(){		
		$("iframe", $('#'+this.displayId)).each(function(){
			$(this).load(function () {
				$(this).height($(this).contents().height());
			});
			$(this)
				.height($(this).contents().height())
				.addClass("no-overflow-print")
				.css("max-height", "");
			
			
		});	
		
		// make sure there is no more heights/max-heights defined on summernote leemnets
		$(".note-editor, .note-dropzone, .note-editing-area, .note-editable", $('#'+this.displayId)).css({height: "", 'max-height': ""});

	}

    attributeCategory.prototype.addTerms = function() {
        try {
            var terms = new Array();

            terms.push({id:"PROJ_CREATION_PROJ_DETAILS",name:"Details",defaultValue:""});
            terms.push({id:"PROJ_CREATION_NAME",name:"Name",defaultValue:""});
            terms.push({id:"PROJ_CREATION_REQUESTOR",name:"Requestor",defaultValue:""});
            terms.push({id:"PROJ_CREATION_PROJMANAGER",name:"Project Manager",defaultValue:""});	
            terms.push({id:"PROJ_CREATION_STATUS",name:"Status",defaultValue:""});
            terms.push({id:"PROJ_CREATION_CUSTOMER",name:"Customer",defaultValue:""});
            terms.push({id:"PROJ_CREATION_LOCATION",name:"Location",defaultValue:""});
            terms.push({id:"PROJ_CREATION_DESCRIPTION",name:"Description",defaultValue:""});
            terms.push({id:"PROJ_CREATION_PROJECT_DUE_DATE",name:"Due Date",defaultValue:""});
            terms.push({id:"PROJDETAILSBOM_TAG",name:"Bill of Materials",defaultValue:""});
            terms.push({id:"PROJ_CREATION_DOCS",name:"Documents",defaultValue:""});
            terms.push({id:"PROJ_CREATION_DOCUMENT_TEMPLATE",name:"Document Template",defaultValue:""});
            terms.push({id:"PROJ_CREATION_MEMBERS",name:"Members",defaultValue:""});
            terms.push({id:"PROJECTROLE",name:"Role",defaultValue:""});  // Could be removed since no longer any need (sine we don't call it placeholder groups anymore) and it also does not work anymore (since the dialog is replaced)
            terms.push({id:"PROJ_CREATION_INVITE_MEMBERS",name:"Invite other members",defaultValue:""});
            terms.push({id:"PROJ_CREATION_ATTRIBUTES",name:"Attributes",defaultValue:""});
            terms.push({id:"PROJ_TASK_SPECS",name:"Specifications",defaultValue:"",TEP:true});
            terms.push({id:"PROJ_TASK_DISCUSSION",name:"Discussions",defaultValue:"",TEP:true});
            terms.push({id:"PROJ_TASK_DOCS",name:"Documents",defaultValue:"",TEP:true});
            terms.push({id:"PROJ_TASK_CHECKLIST",name:"Checklist",defaultValue:"",TEP:true});

            return terms;
        } catch (err) {
            return '';
        }
    };
	
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    // I/O FUNCTIONS
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
    * Calls a web service via Ajax to read the attribute category XML (if there is one). 
    * @param readURL URL of the attribute category, typically the ... sdk call
    * The Ajax call is cached in gACatCache. Multiple calls with the same readURL will only do 1 call.
    */
    attributeCategory.prototype.readACatFromXML = function(readURL) {
        try {
            if (!readURL) return;        
            var aCatCacheObj;
            // show that you are busy
            var displayElement = document.getElementById(this.displayId);
            if (displayElement && this.currentView != 'Header' && !this.delayedLoading) {
                var htmlcode = "<span class=\"darktext_darklink_bold_12\">" + ACatStrings.t('ACatLoadingACat') + "</span>";
                displayElement.innerHTML = htmlcode;
            }

            // check whether a call is already done for this attribute category
            var cacheIdx = 0;		
            for (cacheIdx=0; cacheIdx < gACatCache.length; cacheIdx++) {
                if (readURL == gACatCache[cacheIdx].readURL) break; 
            }			

            if (cacheIdx < gACatCache.length) {
                aCatCacheObj = gACatCache[cacheIdx];
                aCatCacheObj.aCatObjects.push(this);
                if (aCatCacheObj.status != 'Called') { // the ajax call is still underway, when it finishes, it will call readReturnXML for all attached copies
                    this.readReturnXML(aCatCacheObj.returnXML,aCatCacheObj, true);				
                }

            } else {
                // make a new entry

                var aCatCacheObj = new Object();
                gACatCache.push(aCatCacheObj);
                aCatCacheObj.readURL = readURL;
                aCatCacheObj.aCatId = readURL;
                aCatCacheObj.aCatObjects = new Array();
                aCatCacheObj.aCatObjects.push(this);
                aCatCacheObj.returnXML = null;

                var aCatObj = this;  // closure

                // go fetching it        
                AjaxCache.ajax({
                    url: readURL,				
        			type:'GET',
        			cache: false,
        			dataType: 'xml',
                    timeout: 120000
                }).done(function(returnXML) {					
                    aCatObj.readReturnXML(returnXML,aCatCacheObj,false);
                }).fail(_.bind(this.readErrorObj, this));
                aCatCacheObj.status = 'Called';
            }


        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " readACatFromXML: " + err);
        }
    };
    

    /**
     * Get the attributes of the current view and return them as a list of ids
     * @returns {String}
     */
    attributeCategory.prototype.getAttributesOfCurrentView = function() {
    	var currentView = this.currentView ;
    	
    	// look up the current view in the list of views 
    	if (this.views) {
            for (var view=0; view<this.views.length; view++) {
                var viewObj = this.views[view];
                if ( viewObj){
                	if (viewObj.name == currentView){
                		return viewObj.attrsInView;
                	}
              	}
            }
        } else {
        	// oncolumn layout: return all attribute ids
        	return _.pluck(this.attributes, 'id');
        }
        return new Array();
    };
    

    /**
    * Gets values for the attributes, typically via a call to getAttributes.jsp
    */
    attributeCategory.prototype.getAttributeValues = function(secondrun) {
        try {

            var url = null;
            if(secondrun==true){
                url = this.valuesURLMerge;
            }else{
                url = this.valuesURL;
            }

            if (!url) return;
            
            // raise the ajax flag 
            url +=  "&ajax=1";
            var requestType =  'POST';
            
            // Don't load restricted sets if not editable, unless we have a special way to render them
            if(!this.isAttrCatEditable()){
            	// check for rs rendering
            	var hasRs = false;
            	_.each(this.cells, _.bind(function(cellrow){
            		_.each(cellrow, _.bind(function(cell){
                		if(cell && cell.type === "Attribute" 
                			&& cell.attObj && cell.attObj.restrictType === "RestrictedSet" 
                			&& cell.renderOption && cell.renderOption != "Dropdown"){
                				hasRs = true;
                		}
                	}, this));
            	}, this));
            	
            	if(!hasRs){
            		url +=  "&editable=0";
            	}
            }
            
            
            // Only when working with non-default view: 
            // check whether we have the attributes of the current view, if so, pass them to the URL
            var fd ;
            
            try{
            	fd = new FormData();
	            var attrIDData = '';
	            if (this.currentView != 'Default'){
	                var attrList = this.getAttributesOfCurrentView();
	                if (attrList.length>0 ){
	                	for (var attrIndex = 0 ; attrIndex < attrList.length ; attrIndex++){
	                		if (attrIndex >0){
	                			attrIDData += '|';	
	                		} 
	                		attrIDData +=   attrList[attrIndex];
	                	}            	
	                }     
                	fd.append("attributeids",attrIDData) ;
	            }
            }
            catch(err){
            	// in this case we are most likely on IE9 which does not support FormData - we fall back on the non-optimized code.
            	requestType = 'GET';
            }
                    
            var displayElement = document.getElementById(this.displayId);
            if (displayElement && this.currentView == 'Default') {
                var htmlcode = "<span class=\"darktext_darklink_bold_12\">" + ACatStrings.t('ACatReadingValues') + "</span>";
                displayElement.innerHTML = htmlcode;
            }
            
            // make the ajax call
            $.ajax({
    			url: url,
    			type:requestType,
    			enctype: 'multipart/form-data',
    			processData: false,
                contentType: false,
    			data: fd,
    			debug: this.debug,
    			dataType: 'xml',
    			timeout: 120000,
    			async : true,
    		})
    		.done(_.bind(this.readValuesFromXML, this))
    		.fail(_.bind(this.readErrorObj, this));	
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " getAttributeValues: " + err);
        }
    };

    /**
    * Gets project attribute values for the specifications, typically via a call to getAttributes.jsp
    */
    attributeCategory.prototype.getProjectAttributeValues = function() {
        try {		
            var url = this.projectAttributeValuesURL;
            if (!url) return;
            var displayElement = document.getElementById(this.displayId);
            if (displayElement && this.currentView == 'Default') {
                var htmlcode = "<span class=\"darktext_darklink_bold_12\">" + ACatStrings.t('ACatReadingProjectAttributeValues') + "</span>";
                displayElement.innerHTML = htmlcode;
            }
            
            // indicate that the values are read-only, this prevents the rsets from being written to xml
            url += '&editable=0&ajax=1' ; 

            AjaxCache.ajax({
                url: url,
                type: "GET",
                dataType: "xml",
                cache: false,
                timeout: 120000
            })
            .done(_.bind(this.readProjectAttributeValuesFromXML, this))
            .fail(_.bind(this.readErrorObj, this));
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " getProjectAttributeValues: " + err);
        }
    };

    /**
    * Reads the xml returned by the call to readURL (typically GetAttributeCategoryXML.jsp)
    * and converts the xml information into javascript structures in the attribute category object
    */
    attributeCategory.prototype.readReturnXML = function(returnxml,aCatCacheObj, fromCache) {
	
        if(!fromCache){
            // aCacheObj contains all categories for which it XML must be loaded, including current categoryObj
            // If the xml is ready, call this for all categories that need it
            if (aCatCacheObj) {
                aCatCacheObj.returnXML = returnxml;
                aCatCacheObj.status = 'XMLReady';
                if (aCatCacheObj.aCatObjects) {
                    // call readReturnXML on all other copies
                    for (var copy = 1; copy < aCatCacheObj.aCatObjects.length; copy++) {
                        var attCatObj2 = aCatCacheObj.aCatObjects[copy];
                        if (attCatObj2) attCatObj2.readReturnXML(aCatCacheObj.returnXML,aCatCacheObj,true);
                    }
                }
            }
        }

        var attCatObj = null;
		var renderGalleryMsg = '';
        try {
            //attCatObj = this.objHdl;  // retrieve the attribute category object
            attCatObj = this;  // TODO: if this works, just use "this" instead (refactoring)

            var showLayout = attCatObj.layoutId != null;	
            if (attCatObj.layoutId == "") showLayout = false;

            var attCatElem = document.getElementById(attCatObj.id);
            if (attCatElem == null || attCatElem.ACatObj != attCatObj) {
                //logger +="\n" + new Date() + ": mismatch between id and object of attribute category. Overwritten --> Ajax call is outdated. Only the most recent one will be filled";		
                return;
            }

            if (!attCatObj) throw(ACatStrings.t('ACatAjaxHandleProblem'));

            var noDefaultViewException = false;  // flag raised if an xml is returned without a default view
            var xmlLoadFailed = false;  // flag raised if parsing/interpreting the XML throws an exception 

            var errorElem = WCXML.GetFirstFirstLevelChild(returnxml,"error");
            if (errorElem) {
                var errorMsg = WCXML.GetElementText(errorElem,"ErrorMessage");
                alert(ACatStrings.t('ACatErrorLoadingACat') + ":\n" + errorMsg);
                attCatObj.errorMsg = errorMsg;

                attCatObj._displayAttributeCategory();
                return;
            } else {
                attCatObj.errorMsg = null; // otherwise nothing will be displayed
            }


            var attributeCategoryElem=WCXML.GetFirstFirstLevelChild(returnxml,"attribute_category");
            if (!attributeCategoryElem) {
            	throw(ACatStrings.t('ACatServerXMLProblem3'));
            }

            // read name and category id
            attCatObj.name = WCXML.GetElementText(attributeCategoryElem,"name");

            // read attribute list if one is provided
            if (!attCatObj.importEaster) {  // if importing via easter Egg, don't read the attribute list since you already have the only good one
                var attListElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"attribute_list");
                if (attListElem) attCatObj.readAttributeList(attListElem);
            }

            // read layout if one is provided
            var layoutXMLElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"category_XML");
            var attributeCategoryElem = null;
            if (layoutXMLElem) attributeCategoryElem=WCXML.GetFirstFirstLevelChild(layoutXMLElem,"attributeCategory");

            this.docRenderGalleryPromise = Promise.resolve();
            
            if (attributeCategoryElem) {  // read the actual layout
                try {
                    attCatObj.version = WCXML.GetElementText(attributeCategoryElem,"version");
                    if (attCatObj.version && attCatObj.version > attCatObj.softwareVersion) alert(ACatStrings.t('ACatVersionProblem'));
                    attCatObj.modificationTimeStamp = WCXML.GetElementText(attributeCategoryElem,"modificationTimeStamp");
                    attCatObj.currentViewAtLastSave = WCXML.GetElementText(attributeCategoryElem,"currentViewAtLastSave");
                                    // set imported flag if needed
                    var importedElem = WCXML.GetElementText(attributeCategoryElem,"migrated");
                    if (importedElem && importedElem == "1"){
                        attCatObj.imported = true;
                    }

                    // read terminology if there is any
                    if (this.showEditTerminology || attCatObj.layoutId || attCatObj.isTaskTypePage) {  
                        // DT121899: edit terminology is not supposed to do anything for header or default view or document's task type view or BOM view. 
                        // Remark that currentView can be 'Default' also in project creation if there is no creation view, therefore the check is on requestedView					
                        // Also remark that you need to read terminology in the admin edit page, otherwise it's lost at next save
                        // Through EasterEgg, also taskTypePage can have terminology.
                        var terminologyElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"terminology");
                        if (terminologyElem) {
                            attCatObj.readTerminologyFromXML(terminologyElem);
                            attCatObj.showEditTerminology = true;
                        }
                    }


                    //read styles if there are any
                    var styleListElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"styles");
                    if (styleListElem) attCatObj.readStylesFromXML(styleListElem);

                    //read rules if there are any
                    var ruleListElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"rules");
                    if (ruleListElem) attCatObj.readRulesFromXML(ruleListElem);

                    //read document Sources Gallery if there are any
                    var docSourcesGalleryElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"docSourcesGallery");
                    if (docSourcesGalleryElem) {
                    	attCatObj.readDocSourcesGalleryFromXML(docSourcesGalleryElem);
                    }

					var docRenderGalleryElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"docRenderGallery");
					if (docRenderGalleryElem) {
						this.docRenderGalleryPromise = attCatObj.readDocRenderGalleryFromXML(docRenderGalleryElem);
					} 
					
					this.docRenderGalleryPromise.then(function () {
		                // continue only if the documents table rendering configuration is ready or not needed 
		                // (in both cases the 'this.docRenderGalleryPromise' is resolved)
						
	                    // read views
	                    //////////////////////////////////////
	                    var viewArr=WCXML.GetFirstLevelChilds(attributeCategoryElem,"view");
	                    if (!viewArr || viewArr.length < 1) throw(ACatStrings.t('ACatServerXMLProblem4'));
	
	                    attCatObj.views = new Array();
	
	                    var selectedView; // to store requested view
	                    for (var view = 0 ; view < viewArr.length; view++) {
	
	                        var viewElem = viewArr[view];
	                        if (!viewElem) continue; // very unlikely to happen
	
	                        viewObj = new Object();
	                        viewObj.layout = WCXML.GetElementText(viewElem,"layout");
	                        viewObj.name = WCXML.GetElementText(viewElem,"name");
	                        var viewIdx = 0;
	                        for (viewIdx=0; viewIdx<attCatObj.cViewOptions.length; viewIdx++) {
	                            if (viewObj.name == attCatObj.cViewOptions[viewIdx]) break;
	                        }
	                        if (viewIdx >= attCatObj.cViewOptions.length) attCatObj.cViewOptions.push(viewObj.name);
	                        // viewIdx is now the index in cViewOptions of the current view
	
	                        viewObj.showTypeAsUnit = WCXML.GetElementBoolean(viewElem,"showTypeAsUnit");
	                        if (viewObj.showTypeAsUnit.constructor != Boolean) {  // xml from alpha period
	                            if (viewObj.name == 'Header') viewObj.showTypeAsUnit=false; else viewObj.showTypeAsUnit=true;
	                        }
	                        viewObj.addToGeneralSection = WCXML.GetElementBoolean(viewElem,"addToGeneralSection");	// Experimental feature	
	                        viewObj.keepSessionAlive = WCXML.GetElementBoolean(viewElem,"keepSessionAlive");
	                        viewObj.useAlternativeRendering = WCXML.GetElementBoolean(viewElem,"useAlternativeRendering");
	
	                        viewObj.alternateBackground = WCXML.GetElementBoolean(viewElem,"alternateBackground");
	                        if (viewObj.alternateBackground.constructor != Boolean) {  // xml from alpha period
	                            if (viewObj.name == 'Header')  viewObj.alternateBackground=false; else viewObj.alternateBackground=true;
	                        }
	                        
	
	                        viewObj.nRows = parseInt(WCXML.GetElementText(viewElem,"nRows"));
	                        viewObj.nCols = parseInt(WCXML.GetElementText(viewElem,"nCols"));
	                        if (!viewObj.nRows || !viewObj.nCols) {
	
	                            alert(ACatStrings.t('ACatRowsColsProblem1') + view + ACatStrings.t('ACatRowsColsProblem2'));
	                            continue;
	                        }
	                        viewObj.cells = new Array(viewObj.nRows);
	                        
	                        for (var row=0; row < viewObj.nRows;row++) {
	                        	viewObj.cells[row] = new Array(viewObj.nCols);
	                        }
	                        cellArr = WCXML.GetFirstLevelChilds(viewElem,"cell");
	                        if (cellArr && cellArr.length>0) {
	                        	
	                        	// Keep track of the attributes in the view
	                            viewObj.attrsInView = new Array();
	                        	
	                            for (var cell = 0; cell < cellArr.length; cell++) {
	                                var cellElem = cellArr[cell];
	                                if (!cellElem) continue;
	                                var row =  WCXML.GetElementText(cellElem,"row");
	                                var col =  WCXML.GetElementText(cellElem,"col");
	                                if (row > viewObj.nRows) viewObj.nRows = row +1;  // safety, should not happen
	                                if (col > viewObj.nCols) viewObj.nCols = col +1;  // safety, should not happen
	                                if (row!="" && col!="") {
	                                    var cellObj = new Object();
	                                    cellObj.type = WCXML.GetElementText(cellElem,"type");
	                                    cellObj.label = WCXML.GetElementText(cellElem,"label", null, true); // extra parameters to distinguish null and ''
	                                    cellObj.labelImage = WCXML.GetElementText(cellElem,"labelImage");
	                                    cellObj.labelPosition = WCXML.GetElementText(cellElem,"labelPosition");
	                                    if (!cellObj.labelPosition) cellObj.labelPosition = 'Left';
	                                    cellObj.customUnitLabel = WCXML.GetElementText(cellElem,"customUnitLabel");
	
	                                    // AttValueImages
	                                    cellObj.renderOption = WCXML.GetElementText(cellElem,"renderOption");
	                                    cellObj.nImageColumns =  WCXML.GetElementText(cellElem,"nImageColumns");
	                                    cellObj.imageAlignment =  WCXML.GetElementText(cellElem,"imageAlignment");
	                                    if (cellObj.renderOption == 'Images' && !cellObj.imageAlignment) cellObj.imageAlignment = 'DistributeOverWidth'; 
	                                    cellObj.restrictedValueLabelPosition =  WCXML.GetElementText(cellElem,"restrictedValueLabelPosition");
	
	
	                                    cellObj.renderHeight =  WCXML.GetElementText(cellElem,"renderHeight");
	                                    cellObj.imageHeight =  WCXML.GetElementText(cellElem,"imageHeight");
	                                    cellObj.renderImageType =  WCXML.GetElementText(cellElem,"renderImageType");
	                                    if (viewObj.name == 'Default'){
	                                        cellObj.allowMultipleValues = WCXML.GetElementBoolean(cellElem,"allowMultipleValues");
	                                        cellObj.fractionalIfImperial = WCXML.GetElementBoolean(cellElem,"fractionalIfImperial");
	                                    }
	                                    var styleName = WCXML.GetElementText(cellElem,"style");
	                                    if (styleName) {
	                                        try {
	                                            // find the styleObj
	                                            for (var style=0; style<attCatObj.styles.length; style++) {
	                                                if (attCatObj.styles[style].name == styleName) {
	                                                    cellObj.styleObj = attCatObj.styles[style];
	                                                    break;
	                                                }
	                                            }
	                                        } catch(err) {
	                                            // do nothing -->  will get standard style
	                                        }
	                                    }
	                                    if (!cellObj.styleObj && viewObj.name != 'Header') { // !styleName or not found in the list --> assign standard style
	                                        if (cellObj.type == 'Attribute' || cellObj.type == 'Property') {
	                                            cellObj.styleObj = attCatObj.styles[0]; 
	                                        } else if (cellObj.type == "Title") {
	                                            cellObj.styleObj=attCatObj.styles[1];
	                                        }  // ruler, headerbar and spacer don't need a style object
	                                    }
	
	
	                                    var ruleName = WCXML.GetElementText(cellElem,"rule");
	                                    if (ruleName) {
	                                        // find the ruleObj
	                                        for (var rule=0; rule<attCatObj.rules.length; rule++) {
	                                            if (attCatObj.rules[rule].name == ruleName) {
	                                                cellObj.ruleObj = attCatObj.rules[rule];
	                                                break;
	                                            }
	                                        }
	                                    }
	                                    if (!cellObj.ruleObj) cellObj.ruleObj = attCatObj.rules[0]; // Editable
	
	                                    cellObj.docSourceName = WCXML.GetElementText(cellElem,"docSourceName");
	
	
	                                    cellObj.colSpan = WCXML.GetElementText(cellElem,"colSpan");
	                                    cellObj.rowSize = WCXML.GetElementText(cellElem,"rowSize");
	
	                                    cellObj.infoPage = WCXML.GetElementText(cellElem,"infoPage");
	                                    if (cellObj.infoPage) attCatObj.showInfoPage = true;
	
	                                    switch (cellObj.type) {
	                                        case 'Attribute':
	
	                                            cellObj.id = WCXML.GetElementText(cellElem,"id");
	                                            cellObj.name = WCXML.GetElementText(cellElem,"name");
	
	                                            // go searching for the attribute in the list
	                                            // first search on name (for export/import reasons)
	                                            var attObj = null; 
	                                            if (attCatObj.imported) {
	                                                attObj = attCatObj.findAttribute(null, cellObj.name);
	                                                // make sure id is stored correctly so it loads correctly once the imported flag is removed
	                                                if(attObj) cellObj.id = attObj.id;
	                                                else cellObj.id = '';
	                                            } else {
	                                                attObj = attCatObj.findAttribute(cellObj.id);
	                                                if (!attObj && cellObj.name) {
	                                                    attObj = attCatObj.findAttribute(null, cellObj.name);
	                                                    if (attObj) {
	                                                        cellObj.id = attObj.id;
	                                                    } else {
	                                                        cellObj.id = '';
	                                                    }
	                                                }
	                                            }
	                                            
	                                            // add the attribute to the list of attributes in the view
	                                            viewObj.attrsInView.push(cellObj.id); 
	
	                                            // if label was never filled out, fill it out now 
	                                            if(attCatObj.version < 2){
	                                                cellObj.label = WCXML.GetElementText(cellElem,"name");
	                                                // if name was empty, make the label empty string, this was done on purpose to show nothing
	                                                if(!cellObj.label) cellObj.label = '';
	                                            }
	
	                                            cellObj.attObj = attObj;
	                                            if (!attObj) {											
	                                                cellObj = null;  // skip references to attributes which are taken out of the system
	                                            } else {
	                                                cellObj.name = attObj.name; // if there is an attribute, always make sure the name of the cell is updated
	                                                if (!attObj.placedArr) attObj.placedArr = new Array(attCatObj.cViewOptions.length);
	                                                if (attObj.placedArr[viewIdx]) {
	                                                    attObj.placedArr[viewIdx]++;
	                                                } else {
	                                                    attObj.placedArr[viewIdx] = 1;
	                                                }
	                                                attObj.placedOriginally = true;
	                                                if (attObj.placed) attObj.placed++; else attObj.placed=1;
	                                                if (viewObj.name == 'Default') {
	                                                    attObj.allowMultipleValues = cellObj.allowMultipleValues;
	                                                    attObj.fractionalIfImperial = cellObj.fractionalIfImperial;
	                                                }
	
	                                                // pick up restrictions on this attribute (other than normal restricted sets and lists)
	                                                if (view == 0) {
	                                                    var restrictType = WCXML.GetElementText(cellElem,"restrictType");
	                                                    if (restrictType == 'Attribute') { 
	                                                        var restrictAttributeId =  WCXML.GetElementText(cellElem,"restrictAttributeId");
	                                                        var restrictAttributeName =  WCXML.GetElementText(cellElem,"restrictAttributeName");
	                                                        attObj.restrictToAttObj = attCatObj.findAttribute(restrictAttributeId,restrictAttributeName);
	                                                        if (attObj.restrictToAttObj) attObj.restrictToAttObj.isInRule = true;
	                                                    }
	                                                    attObj.restrictType = restrictType;
	                                                    if (!attObj.restrictType) attObj.restrictType = 'None';  // default restrictType
	                                                }
	                                                if (cellObj.restrictType != 'None' &&  cellObj.renderOption != 'Dropdown' && !cellObj.restrictedValueLabelPosition) cellObj.restrictedValueLabelPosition = 'LabelBelow'; 
	
	                                                // read document reference sources and create a document table with them (if exists)
	                                                if (attObj.isDocReference()) {
	                                                	this.initializeDocTableWithDocSources(cellObj, viewObj.name);
														var renderName = WCXML.GetElementText(cellElem,"docRenderName");
														if (renderName) renderGalleryMsg += this.linkCellToRenderGallery(cellObj, renderName, viewObj.name); 
	                                                }
	
	                                            }
	                                            break;
	
	                                        case 'Property':
	                                            cellObj.name = WCXML.GetElementText(cellElem,"name");
	                                            
	                                            // go searching for it in the extended list										
	                                            var attObj = null; 										
	
	                                            for (var prop=0; prop<this.otherProperties.length; prop++) {
	                                                if (cellObj.name == this.otherProperties[prop].name) break;
	                                            }
	                                            if (prop <this.otherProperties.length) {
	                                                attObj = cellObj.attObj = this.otherProperties[prop];
	
	                                                if (!attObj.placedArr) attObj.placedArr = new Array(attCatObj.cViewOptions.length);
	                                                if (attObj.placedArr[viewIdx]) {
	                                                    attObj.placedArr[viewIdx]++;
	                                                } else {
	                                                    attObj.placedArr[viewIdx] = 1;
	                                                }
	                                                attObj.placedOriginally = true;
	                                                if (attObj.placed) attObj.placed++; else attObj.placed=1;
	                                            } else {
	                                                cellObj = null;  // throw out what no longer exists - shoudl probably not happen
	                                            }
	
	
	                                            break;
	                                        case 'Ruler':
	                                            cellObj.rulerThickness = WCXML.GetElementText(cellElem,"rulerThickness");
	                                            cellObj.rulerColor = WCXML.GetElementText(cellElem,"rulerColor");									
	                                            break;
	                                        case 'Spacer':
	                                            cellObj.height = WCXML.GetElementText(cellElem,"height");
	                                            cellObj.background = WCXML.GetElementText(cellElem,"background");	
	                                            break;
	                                        case 'Title':															
	                                        case 'HeaderBar':
	                                        case 'Button':  
	                                            break;	
	
	
	                                        default:
	                                            alert(ACatStrings.t('ACatUnknownCellTypeProblem') + cellObj.type + ACatStrings.t('ACatId') + cellObj.id);
	                                            continue;
	                                            break;
	
	                                    }  // end switch cellObj.type
	
	                                    this.readButtonsFromXML(cellElem,cellObj, viewObj);  // read buttons (can be part of many cell types)
	
	                                    // from here on, cellObj can be null --> in the case an attribute has been taken out of the system
	
	                                    viewObj.cells[row][col]=cellObj;
	
	                                }
	                            } // end for cells
	                        } // end if cellArr
	                        // check if this is the current view
	
	                        if (viewObj.name && attCatObj.currentView && viewObj.name.toLowerCase() == attCatObj.currentView.toLowerCase()) {
	                        	selectedView = viewObj;
	                        }
	
	                        attCatObj.views[viewIdx] = viewObj;
	
	
	                    } // end for view
	                    
	                    if (selectedView) {
	                        this.selectView(selectedView);
	                    }
	                    
	                    // check if at least one view fits this.currentView
	                    if (!attCatObj.currentViewFound) {
	                        if (attCatObj.currentView ==  'Header') {
	                            attCatObj.noHeader = true;  // remember that there is no header view at all when asked for display in normal projects or documents
	                        } else if (attCatObj.currentView ==  'Creation' || attCatObj.currentView ==  'Default' || attCatObj.currentView ==  'BOM') {   // Default will normally never happen, but it is surely possible to ask a Creation view if there is none
	                            if (attCatObj.views[0]) {
	                            	// This is where currentView becomed different from requestedView.
									this.selectView(attCatObj.views[0]);
	                            } else {
	                                noDefaultViewException = true;
	                            }
	                        } else {
	                            // a task type view is asked and there is none --> display nothing and don't ask to read the cascading dropdowns
	                            attCatObj.noCurrentViewDisplay = true;
	
	                        }
	                    }
	                    if (viewArr.length <2) attCatObj.views=null;  // no views used
	
	                    // read lists if there are any
	                    if (attCatObj.currentView !=  'Header' && !attCatObj.noCurrentViewDisplay) {
	                        var listsElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"lists");
	                        if (listsElem){
	                            if(attCatObj.delayedLoading){
	                                attCatObj.delayedListLoading = attCatObj.readListsFromXML.bind(this,listsElem,attCatObj.readListsURL,aCatCacheObj,fromCache);
	                            } else {
	                                attCatObj.readListsFromXML(listsElem,attCatObj.readListsURL,aCatCacheObj,fromCache);
	                            }
	                        }
	                    }
					}.bind(this));   // end of this.docRenderGalleryPromise.then(...
					
                } catch (err) {
                    // problem reading the XML --> fall back on OneColumn while ignoring the XML
        //				alert("error reading XML on line " + err.lineNumber + ": " +  err.message );
                    xmlLoadFailed = true;
                }

            }   // end of reading attribute category layout (done only when grid layout is on!)

            this.docRenderGalleryPromise.then(function () {
                // continue only if the documents table rendering configuration is ready or not needed 
                // (in both cases the 'this.docRenderGalleryPromise' is resolved)
            	
	            if (!attributeCategoryElem || xmlLoadFailed || noDefaultViewException) { 
	                // no layout exists yet or it failed loading or no default view --> OneColumn layout for the default view
	                attCatObj.layout = 'OneColumn';
	                attCatObj.nCols = 1;
	                if (!noDefaultViewException) attCatObj.noHeader = true; // if there is no layoutXML, there cannot be views and also not a header view
	                if (attCatObj.attributes) {
	                    attCatObj.nRows = attCatObj.attributes.length;
	                    if (attCatObj.nRows > 0) {
	                        attCatObj.cells = new Array(attCatObj.nRows);
	                    } else {
	                        attCatObj.cells = new Array();
	                    }
	                    for (var row=0;row<attCatObj.nRows;row++) attCatObj.cells[row] = new Array(attCatObj.nCols);
	                    for (var row=0;row<attCatObj.attributes.length;row++) {
	                        var cellObj = new Object();
	                        attCatObj.cells[row][0] = cellObj;
	                        cellObj.attObj = attCatObj.attributes[row];
	                        cellObj.type = 'Attribute';
	                        // TO DO: decide whether you want to have id and name doubled in the data structure.
	                        cellObj.id = cellObj.attObj.id;
	                        cellObj.name = cellObj.attObj.name;
	                        if (cellObj.attObj.placed) cellObj.attObj.placed++; else cellObj.attObj.placed=1;
	                        cellObj.attObj.placedArr[0] = 1;
	                        // set required flag here if required task specification
	                        if(attCatObj.isTaskTypePage && cellObj.attObj.isRequired) {
	                            cellObj.ruleObj = attCatObj.isRequiredRule;
	                        }
	
	                        // initialize document table with default document reference sources (there is no definition in one column layout)
	                        this.initializeDocTableWithDocSources(cellObj, 'Default');
	
	                    }
	                } else {
	                    attCatObj.attributes = new Array();
	                    attCatObj.nRows = 0;
	                }
	                if (noDefaultViewException) {
	
	                    alert(ACatStrings.t('ACatLayoutViewProblem') ); // should not happen
	                    if (attCatObj.views) {  // put the default view also in views[0];
	                        attCatObj.views[0].cells = attCatObj.cells;
	                        attCatObj.views[0].layout = attCatObj.layout;
	                        attCatObj.views[0].nRows = attCatObj.nRows;
	                        attCatObj.views[0].nCols = attCatObj.nCols;
	                        // showTypeAsUnit and alternateBackground don't have to be copied for OneColumn layout
	                    }
	                }
	            }
	            attCatObj.recreateDocTableXForCurrentView();
	
	            // load attribute values if necessary	
	            var loadvalues = false; 
	            if (!attCatObj.delayedLoading && attCatObj.valuesURL && !attCatObj.noCurrentViewDisplay && !(attCatObj.currentView == 'Header' && attCatObj.noHeader)) {
	                attCatObj.getAttributeValues();	
	                loadvalues = true;
	            }
	
	            // current view
	
	            if (attCatObj.views) {
	                // set current view index
	                for (var view = 0; view< attCatObj.views.length; view++) {
	                    if (attCatObj.views[view] && attCatObj.views[view].name == attCatObj.currentView) attCatObj.currentViewIdx = view;
	                }
	            }
	
	            if (attCatObj) {
	                //var showLayout = ((attCatObj.layoutId != null) && (attCatObj.layoutId != ""));  // this line seems to be wrong but I don't see why
	
	
	                if (attCatObj.addToGeneralSection) {   // experimental feature
	                    var generalSectionId = attCatObj.displayId + '2';
	                    if (document.getElementById(generalSectionId)) {
	                        attCatObj.displayId=generalSectionId;
	                        var attributesSection = document.getElementById('mainAttributesRowId');
	                        if (attributesSection) attributesSection.style.display='none';					
	                    }
	                }
	                if (showLayout) {
	                    attCatObj.startLayout(attCatObj.layoutId);
	
	                    attCatObj.loadingDone = true;  // indicate that the category (not necessary values) is loaded
	                } else {
	                    if (!loadvalues) {
	                    	this.loaded = true;
	                    }
	                    attCatObj._displayAttributeCategory();
	
	                    attCatObj.loadingDone = true;  // indicate that the category (not necessary values) is loaded
	
	                    if (attCatObj.callbackOnLoad) attCatObj.callbackOnLoad();
	                    if (attCatObj.keepSessionAlive &&  !showLayout  && this.isAttrCatEditable()) {
	                        // to avoid that you lose your session after a number of minutes while editing
                            window.setInterval(function(){
                                if(keepSessionAlive){
                                    keepSessionAlive(attributeCategory.prototype.alertSessionLost);
                                }
                            }, 300000);
	                    }
	                }
	            }
            }.bind(this));
        } catch (err) {
            var errormsg = ACatStrings.t("ACatErrorLoadingACat");
            var displayElement = document.getElementById(attCatObj.displayId);
            if (displayElement) {
                displayElement.innerHTML = errormsg;
            } else {
                alert(errormsg);
            }
            if (attCatObj) attCatObj.errorStatus = errormsg;
            if (attCatObj.callbackOnFail){
                try{
                    attCatObj.callbackOnFail();
                    if (aCatCacheObj) {
                        if (aCatCacheObj &&  aCatCacheObj.aCatObjects && !fromCache) {
                            // call readReturnXML on all other copies
                            for (var copy = 1; copy < aCatCacheObj.aCatObjects.length; copy++) {
                                var attCatObj2 = aCatCacheObj.aCatObjects[copy];
                                if (attCatObj2) {
                                    var displayElement2 = document.getElementById(attCatObj2.displayId);
                                    if (displayElement2) {
                                        displayElement2.innerHTML = errormsg;
                                    } else {
                                        alert(errormsg);
                                    }
                                    attCatObj2.errorStatus = errormsg;
                                    if(attCatObj2.callbackOnFail){
                                        attCatObj2.callbackOnFail();        							
                                    }
                                }
                            }

                        }
                    }
                }catch(err2){}
            }
        } finally {
			if (renderGalleryMsg) alert(renderGalleryMsg);
            this.importEaster = false;
        }
    };

    /**
    * Recreate docTableX for all document reference cells in current view
    * To be called when changing view of after loading an attribute category
    * @author HADW
    */
    attributeCategory.prototype.recreateDocTableXForCurrentView = function () {

        "use strict";
        try {
            for (var row = 0; row < this.cells.length; row++) {
                if (!this.cells[row]) continue;
                for (var col = 0; col < this.cells[row].length; col++) {
                    var cellObj = this.cells[row][col];
                    if (cellObj && cellObj.dshObj) {
                        this.createDocTableX(cellObj);
                    }
                }
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " recreateDocTableXForCurrentView: " + err);
        }
    }
    /**
    * Constructor for buttonObj. All new creations of buttonObj should pass through this constructor
    * @param {String} [func] - function to be called when clicking the button. Allowed values are defined in aCatObj.buttonFuncOptions: LookUpAttributes, JSP, JavaScript, CheckUniqueValue, Print
    * @param {String} [label] - label to be shown on the button (or the text for the green link). Defaults to ''
    * @param {String] [style] - style of the button: allowed are buttonprimary, buttonsecondary, plain and greenlink
    * @param {String} [javaScript] - javascript code which will be executed if func == 'JavaScript'
    * @param {String} [viewName] - view in which the button will be added. one of 'Default', 'Creation', ...
    */
    attributeCategory.prototype.buttonObj = function (func, label, style,javaScript, viewName) {
        var buttonObj = {};	// CR: is this needed?
        this.func = func;	  // since buttonObj is called with new buttonObj, this refers to the just created object
        if (!this.func) this.func = 'LookUpAttributes';
        this.label = label;
        if (!this.label) this.label = '';
        this.style = style;
        if (!this.style) this.style = 'buttonprimary';
        this.inputAtts = [];
        this.outputAtts = [];
        this.changeAtts = [];
        this.javaScript = javaScript;
        this.listObj = null;
        this.execOnLoad = false;  // execute the button as soon as all data is loaded
        this.execOnSave = false;	// execute the button at validation time right before saving
        this.execOnChange = false;   // execute the button when another attribute changes
        this.clicked = 0;  // count how many times the button is clicked
        this.viewName =  viewName ? viewName : 'Default';  // view in which this button is placed

	
    }



    /**
    * reads button xml and constructs cellObj.buttons
    * xml looks like 
    * <buttons>
    * 	<button>
    * 		<label>Click</label>
    *		<func>LookUpAttributes</func>
    *		<inputAtt>
    *			<id>00002_0000001078</id>
    *			<name>Brand</name>
    *			<columnName>CountryCode</columnName>
    *			<exact>true</exact>
    *		</inputAtt>
    *		<outputAtt>
    *			<id>00002_0000001078</id>
    *			<name>Brand</name>
    *			<columnName>CountryCode</columnName>
    *		</outputAtt>
    *		<style>buttonprimary</style>
    *		<listName>Countries and Managers</listName>
    *	</button>
    *</buttons>
    */
    attributeCategory.prototype.readButtonsFromXML = function(cellElem,cellObj, viewObj) {
        "use strict";
        try {
            if (!cellElem) return;
            if (this.currentView == 'Header') return;   // never have buttons in Header view, so don't bother reading them, let alone fetch necessary information.
            var buttonsElem = WCXML.GetFirstFirstLevelChild(cellElem,"buttons");
            if (buttonsElem) {
                var buttonArr = WCXML.GetFirstLevelChilds(buttonsElem,"button");

                if (buttonArr && buttonArr.length>0) {
                    cellObj.buttons = [];
                    for (var button=0; button < buttonArr.length; button++) {
                        var buttonElem = buttonArr[button];
                        var buttonObj = new this.buttonObj();
                        cellObj.buttons.push(buttonObj);	

                        buttonObj.label = WCXML.GetElementText(buttonElem,'label');
                        buttonObj.func = WCXML.GetElementText(buttonElem,'func');
                        buttonObj.style = WCXML.GetElementText(buttonElem,'style');
                        buttonObj.execOnLoad = WCXML.GetElementBoolean(buttonElem,'execOnLoad');
                        buttonObj.execOnSave = WCXML.GetElementBoolean(buttonElem,'execOnSave');
                        buttonObj.execOnChange = WCXML.GetElementBoolean(buttonElem,'execOnChange');
                        buttonObj.viewName = viewObj? viewObj.name: 'Default';					

                        if (buttonObj.execOnChange) {
                            this.readReferencedAttributeList(buttonObj, buttonElem,'changeAtts','changeAtt');						
                        }

                        if (buttonObj.func == 'JavaScript') {
                            buttonObj.javaScript = WCXML.GetElementText(buttonElem,'javaScript');						
                        }

                        if (buttonObj.func == 'LookUpAttributes') {
                            // retrieve listName. listObj cannot be found yet because lists are not read at this moment
                            buttonObj.listName = WCXML.GetElementText(buttonElem,'listName');	
                            buttonObj.listCounter = WCXML.GetElementText(buttonElem,'listCounter');	
                            if (buttonObj.listCounter != '') buttonObj.listCounter = parseInt(buttonObj.listCounter, 10);

                            // input attributes
                            this.readReferencedAttributeList(buttonObj, buttonElem,'inputAtts','inputAtt');

                            // output attributes		
                            this.readReferencedAttributeList(buttonObj, buttonElem,'outputAtts','outputAtt');						

                        }


                    }
                }

            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " readStylesFromXML: " + err);
        }

    }

    /**
    * Reads inputAtts, outputAtts and changeAtts from XML for a button represented by buttonObj
    * @param {Obj} buttonObj - buttonObj for which to read the references attributes
    * @param {XMLDOMElem} buttonElem - element in the XML representing the button
    * @param {String} type - one of 'changeAtts', 'inputAtts', 'outputAtts'
    * @param {String} tagName - the XML tag containing the attributes ('changeAtt' etc).
    */
    attributeCategory.prototype.readReferencedAttributeList = function(buttonObj, buttonElem, type, tagName) {
        'use strict';
        try {
            var attsArr = [];
            if (!buttonObj) throw new Error('buttonObj does not exist'); 
            buttonObj[type] = attsArr;  // means f.e. buttonObj.changeAtts = [];

            var attArr =  WCXML.GetFirstLevelChilds(buttonElem,tagName);
            if (attArr && attArr.length>0) {
                for (var att=0; att < attArr.length; att++) {
                    var attElem = attArr[att];
                    // find attObj
                    var id = WCXML.GetElementText(attElem,'id');
                    var name = WCXML.GetElementText(attElem,'name');
                    var attObj;
                    if (!this.imported) {
                        attObj = this.findAttribute(id,name);  // find by id
                    } else {
                        attObj = this.findAttribute(null,name);  // find by name when imported from a foreign system
                    }
                    if (attObj) {

                        var attObj2 = {};
                        attsArr.push(attObj2);
                        attObj2.attObj = attObj;    

                        if (type != 'changeAtts') {
                            attObj2.columnName =  WCXML.GetElementText(attElem,'columnName');
                            // columnObj cannot be retrieved yet because lists are not yet read at this point
                        }
                        if (type == 'inputAtts') {
                            attObj2.exact = WCXML.GetElementBoolean(attElem,'exact');
                        }
                        if (type == 'changeAtts') {
                            if (!attObj.buttonDependency) attObj.buttonDependency = [];
                            attObj.buttonDependency.push(buttonObj);
                        }

                    } else {
                        //alert('Attribute ' + name + ' is referenced by button ' + buttonObj.label + ' but is no longer in the category' );
                    }
                }
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " readReferencedAttributeList: " + err);
        }
    }

    attributeCategory.prototype.readStylesFromXML = function(stylesElem) {
        try {
            if (!stylesElem) return;
            if (!this.styles) this.styles = new Array();
            var styleArr = WCXML.GetFirstLevelChilds(stylesElem,"style");
            for (var style=0; style<styleArr.length; style++) {
                var styleElem = styleArr[style];
                var styleObj = ObjectHandler.getCloneOfObject(this.styles[0]);
                styleObj.standardStyle = WCXML.GetElementBoolean(styleElem,"standardStyle");
                styleObj.name = WCXML.GetElementText(styleElem,"name");
                if (1) {
                    // look whether you already have it, typical for standard styles
                    var found = false;
                    for (var style2=0; style2<this.styles.length; style2++) {
                        if (styleObj.name == this.styles[style2].name) {
                            found=true; break;
                        };
                    }
                    if (found) {
                        continue;  // don't add an existing (standard) style
                    } else {
                        // declared as standard style but no longer a standard style --> degrade
                        styleObj.standardStyle = false;
                    }
                }
                this.styles.push(styleObj);  // if the style is not known yet, add to the list
                // read styles from XML; if not defined, use default style
                styleObj.classLabel = WCXML.GetElementText(styleElem,"classLabel") || styleObj.classLabel;
                styleObj.classValue = WCXML.GetElementText(styleElem,"classValue") || styleObj.classValue;
                styleObj.background = WCXML.GetElementText(styleElem,"background") || styleObj.background;
                styleObj.alignment = WCXML.GetElementText(styleElem,"alignment") || styleObj.alignment;
                styleObj.fontColor = WCXML.GetElementText(styleElem,"fontColor") || styleObj.fontColor;
                styleObj.fontStyle = WCXML.GetElementText(styleElem,"fontStyle") || styleObj.fontStyle;
                styleObj.fontWeight = WCXML.GetElementText(styleElem,"fontWeight") || styleObj.fontWeight;
                styleObj.fontFamily = WCXML.GetElementText(styleElem,"fontFamily") || styleObj.fontFamily;
                styleObj.fontSize = WCXML.GetElementText(styleElem,"fontSize") || styleObj.fontSize;
                styleObj.width = WCXML.GetElementText(styleElem,"width") || styleObj.width;
                styleObj.widthunit = WCXML.GetElementText(styleElem,"widthunit") || styleObj.widthunit;
                styleObj.height = WCXML.GetElementText(styleElem,"height") || styleObj.height;
                styleObj.columnWidth = WCXML.GetElementText(styleElem,"columnWidth") || styleObj.columnWidth;
                styleObj.columnWidthUnit = WCXML.GetElementText(styleElem,"columnWidthUnit") || styleObj.columnWidthUnit;

                var valueDifferent = WCXML.GetElementBoolean(styleElem,"valueDifferent");
                if (valueDifferent) {
                    styleObj.isDifferentStyles = true;
                    var valueStyleObj = styleObj.valueStyleObj || new Object();
                    styleObj.valueStyleObj = valueStyleObj;
                    // read styles from XML; if not defined, use default style
                    valueStyleObj.background = WCXML.GetElementText(styleElem,"valueBackground") || valueStyleObj.background || '';
                    valueStyleObj.alignment = WCXML.GetElementText(styleElem,"valueAlignment") || valueStyleObj.alignment || '';
                    valueStyleObj.fontColor = WCXML.GetElementText(styleElem,"valueFontColor") || valueStyleObj.fontColor || '';
                    valueStyleObj.fontStyle = WCXML.GetElementText(styleElem,"valueFontStyle") || valueStyleObj.fontStyle || '';
                    valueStyleObj.fontWeight = WCXML.GetElementText(styleElem,"valueFontWeight") || valueStyleObj.fontWeight || '';
                    valueStyleObj.fontFamily = WCXML.GetElementText(styleElem,"valueFontFamily") || valueStyleObj.fontFamily || '';
                    valueStyleObj.fontSize = WCXML.GetElementText(styleElem,"valueFontSize") || valueStyleObj.fontSize || '';
                    valueStyleObj.width = WCXML.GetElementText(styleElem,"valueWidth") || valueStyleObj.width || '';
                    valueStyleObj.widthunit = WCXML.GetElementText(styleElem,"valueWidthunit") || valueStyleObj.widthunit || '';
                    valueStyleObj.height = WCXML.GetElementText(styleElem,"valueHeight") || valueStyleObj.height || '';
                    valueStyleObj.columnWidth = WCXML.GetElementText(styleElem,"valueColumnWidth") || valueStyleObj.columnWidth || '';
                    valueStyleObj.columnWidthUnit = WCXML.GetElementText(styleElem,"valueColumnWidthUnit") || valueStyleObj.columnWidthUnit || '';

                } else {
                    styleObj.isDifferentStyles = false;
                }

            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " readStylesFromXML: " + err);
        }
    };

    /**
    * reads document source configuration gallery from XML and fills it in this.docSourcesGallery
    * @author HADW -  personal review 5-Feb-2015
    */
    attributeCategory.prototype.readDocSourcesGalleryFromXML = function(DocSourcesGalleryElem) {
        'use strict';
        try {
            if (!DocSourcesGalleryElem) return;

            this.docSourcesGallery = [];  // wipe whatever was there. 
            var docSourcesArr = WCXML.GetFirstLevelChilds(DocSourcesGalleryElem,"docSources");

            for (var ds=0; ds < docSourcesArr.length; ds++) {
                this.createDocumentSource(this.dshConfigTablePHID, null, docSourcesArr[ds]);
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " readDocSourcesGalleryFromXML: " + err);
        }
    };

    /**
	* reads document render configuration gallery from XML and fills it in this.docRenderGallery
	* @author HADW - personal review March 23, 2015
	* 
	* @return {Promise} Object that is resolved when the documents table rendering configuration is loaded or not needed.
	*/
	attributeCategory.prototype.readDocRenderGalleryFromXML = function(DocRenderGalleryElem) {
		'use strict';
		try {
			if (!DocRenderGalleryElem) return;
			
			this.docRenderGallery = [];  // wipe whatever was there. 
			
			var docLinkTableOptions = null;
		
			var docRenderArr = WCXML.GetFirstLevelChilds(DocRenderGalleryElem, "docRender");
			if (docRenderArr) {
				// wait for optional DocTableXRenderConfig to be loaded via required.js
				return Promisify.require("doctablex/DocTableXRenderConfig").then(function (DocTableXRenderConfig) {
					
					for (var index = 0; index < docRenderArr.length; index++) {
						var docRenderElem = docRenderArr[index];
						if (docRenderElem) {
							// create a new instance of documents table rendering configuration...
							docLinkTableOptions = new DocTableXRenderConfig({
								section : 'AttributeCategory',
								phElemID : 'DocRenderConfigDivID',
								configObj : null,
								attributes : this.attributes,
								isImported : this.imported,
								changeCallback : (this.setACatChanged) ? this.setACatChanged.bind(this) : null
							});
							//... and fill it with data from the input XML
							docLinkTableOptions.readFromXML(docRenderElem, this.displayAttributeCategory);
						}
						this.docRenderGallery.push(docLinkTableOptions);
					}
				}.bind(this));
			}
		
			return Promise.resolve();
			
		} catch (err) {
			alert(ACatStrings.t('ACatProblemIn') + " readDocRenderGalleryFromXML: " + err);
		}
	};

	/**
    * Reads terminology from xml
    * The terminology looks as follows
    * <terminology>
    * 	<term>
    *		<id>PROJ_CREATION_PROJECT_DUE_DATE</id>
    *		<default>Samples to be delivered by</default>
    *		<fr>echantillons a delivrer vers</fr>
    *	</term>
    *   ... more terms
    * </terminology>
	* The code goes looking on the page to fix-up for elements with the <id> provided and replaces it with what is found in <default> or in the language provided 
    * Language is however on other places not yet supported
    */
    attributeCategory.prototype.readTerminologyFromXML = function(terminologyElem) {
        try {
            if (!terminologyElem) return;

            var terminologyArr = WCXML.GetFirstLevelChilds(terminologyElem,"term");

            if (!terminologyArr) return;
            for (var term=0; term<terminologyArr.length; term++) {
                var termElem = terminologyArr[term]; 			
                var id = WCXML.GetElementText(termElem,"id");
                if (id ) {
                    for (var term2=0; term2<this.terms.length; term2++) {
                        if (this.terms[term2] && this.terms[term2].id == id) {
                            var termObj = this.terms[term2];
                            var elem = WCXML.GetFirstFirstLevelChild(termElem,"default");
                            if (elem) {
                                var value = WCXML.GetElementText(termElem,"default");

                                if (this.language) {  // only overrule if there is actually a language string defined for this langauge
                                    var elem= WCXML.GetFirstFirstLevelChild(termElem,this.language);
                                    if (elem) value = WCXML.GetElementText(termElem,this.language);					
                                }

                                termObj.defaultValue = value;

                                // do the actual change of the term in the project creation page
                                value = WCXML.xmlencode(value);
                                if (document.getElementById(id)) {
                                    // try rescuing the * but only if it's not yet in the new name (otherwise you will get 2 *s. Done for historical reasons: setups exist with the * in there already)
                                    // only add a * when there was a *-span in the old value and it was not hidden (meaning it was required)
                                    var oldvalue = document.getElementById(id).innerHTML;
                                    if (oldvalue && oldvalue.toLowerCase().indexOf('<span')!=-1 && oldvalue.toLowerCase().indexOf('display: none;') == -1 
                                            && value.indexOf('*') == -1) 
                                        value += '&nbsp;<span class="esko-green" id="projectNameRowReqId" style="display: inline;">*</span>';
                                    document.getElementById(id).innerHTML = value;							
                                }
                            }
                        }
                    }

                }

            }
            // now look whether to switch sequence of the actual boxes
            var sequenceString = WCXML.GetElementText(terminologyElem,"sequence");
            this.projectCreationSequence = sequenceString;

            if (sequenceString && sequenceString.length >0) {		
                var sectionNames = {G:'mainProjectDetailsRowId',B:'mainBomDetailsRowId',D:'mainDocumentDetailsRowId', M: 'mainMembersRowId', A: 'mainAttributesRowId'};
                var item;


                // Code changed for DT115720.  Moving the G section around causes its calendars (due date, last task date, first task date) to break as a consequence of jQuery miser

                // find where G is in the sequenceString
                for (var itemG=0; itemG < sequenceString.length; itemG++) {
                    if (sequenceString.substr(itemG,1) == 'G') break;
                }
                // until item G, put it before item G.  If itemG is 0, this does nothing
                for (var item=0; item<itemG; item++) {
                    $("#mainProjectDetailsRowId").before($("#"+sectionNames[sequenceString.substr(item,1)]));
                }
                // after item G, put it after the last item
                for (var item = itemG+1; item < sequenceString.length; item++) {
                    $("#"+sectionNames[sequenceString.substr(item-1,1)]).after($("#"+sectionNames[sequenceString.substr(item,1)]));
                }
                // CHECKLIST
                // Protect against sections which don't exist --> OK: sections are always placed but hidden as needed. So you can still shuffle them around
                // Protect against wrong strings --> The section name will not be found and jQuery will simply not do the statement. It will NOT go to the catch statement
                // Protect against G not appearing in the sequenceString: itemG will be equal to sequenceString.length. The first for statement will do all shuffling (so all will appear before G). 

            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " readTerminologyFromXML: " + err);
        }
    };

    attributeCategory.prototype.readRulesFromXML = function(rulesElem) {
        try {
            if (!rulesElem) return;
            if (!this.rules) this.rules = new Array();
            var ruleArr = WCXML.GetFirstLevelChilds(rulesElem,"rule");
            for (var rule=0; rule< ruleArr.length; rule++) {
                var ruleElem = ruleArr[rule];
                var ruleObj = new Object();

                ruleObj.standardRule = WCXML.GetElementBoolean(rulesElem,"standardRule");
                ruleObj.name = WCXML.GetElementText(ruleElem,"name");

                // look whether you already have it, typical for standard rules
                var found = false;
                for (var rule2=0; rule2<this.rules.length;rule2++) {
                    if (ruleObj.name == this.rules[rule2].name) {
                        found=true; break;
                    };
                }
                if (found) {
                    continue;  // don't add an existing (standard) rule
                } else {
                    // declared as standard rule but no longer a standard rule --> degrade --> unlikely to happen
                    ruleObj.standardRule = false;
                }

                this.rules.push(ruleObj);  // if the rule is not known yet, add to the list

                this.readConditionFromXML(ruleElem,ruleObj);
                ruleObj.depth = parseInt(WCXML.GetElementText(ruleElem,"depth"));
                ruleObj.visibility = WCXML.GetElementText(ruleElem,"visibility");
                ruleObj.visibilityRule = WCXML.GetElementText(ruleElem,"visibilityRule");            
                ruleObj.visibilityElse = WCXML.GetElementText(ruleElem,"visibilityElse");
                ruleObj.visibilityElseRule = WCXML.GetElementText(ruleElem,"visibilityElseRule");



            }
            // replace rule names with rule objects
            // This code looks more advanced than what we actually use.
            for (var rule=0; rule<this.rules.length; rule++) {
                var ruleObj = this.rules[rule];
                if (ruleObj.visibilityRule) {
                    for (var rule2=0; rule2<this.rules.length; rule2++) {
                        if (this.rules[rule2].ruleName == this.rules.visibilityRule) ruleObj.visibilityRule = this.rules[rule2];
                        break;
                    }
                }
                if (ruleObj.visibilityElseRule) {
                    for (var rule2=0; rule2< this.rules.length; rule2++) {
                        if (this.rules[rule2].ruleName == this.rules.visibilityElseRule) ruleObj.visibilityElseRule = this.rules[rule2];
                        break;
                    }
                }
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + "readRulesFromXML: " + err);
        }
    };

    attributeCategory.prototype.readConditionFromXML = function(ruleElem,ruleObj) {
        "use strict;"
        try {
            if (!ruleElem || !ruleObj) return;    		

            ruleObj.condition = WCXML.GetElementText(ruleElem,"condition");		
            ruleObj.compareValue = WCXML.GetElementText(ruleElem,"compareValue");
            ruleObj.operator = WCXML.GetElementText(ruleElem,"operator");		
            var attributeName =  WCXML.GetElementText(ruleElem,"attributeName");
            var attributeId =  WCXML.GetElementText(ruleElem,"attributeId");
            ruleObj.attObj = this.findAttribute(attributeId,attributeName);
            if (ruleObj.attObj) ruleObj.attObj.isInRule = true;
            if (ruleObj.condition == 'AND' || ruleObj.condition == 'OR') {
                var subRules = WCXML.GetFirstLevelChilds(ruleElem,"rule"); // DT120572: var added. (omitted by mistake before use strict was there... consequence: bug caused by recursive call)
                if (subRules && subRules.length>1) {
                    ruleObj.ruleObj1 = new Object();
                    ruleObj.ruleObj1.parentObj = ruleObj;
                    this.readConditionFromXML(subRules[0],ruleObj.ruleObj1);

                    ruleObj.ruleObj2 = new Object();
                    ruleObj.ruleObj2.parentObj = ruleObj;
                    this.readConditionFromXML(subRules[1],ruleObj.ruleObj2);
                } else {
                    // this rule is broken  
                    alert(ACatStrings.t('ACatBrokenRuleProblem'));
                }

            }


        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.readConditionFromXML: " + err);
        }
    };

    /**
    * Loads lists into this.lists and then calls for each an ajax function to fill them
    * @param {String} listsXML Content of the <lists> element. (without the <lists> and </lists> tage
    * @param {String} baseURL. Ajax URL to be called
    * @param {Boolean} fromCache. Says whether the XML is loaded from cache AND it has or will load the lists.  This indicates that there is another attribute category object with the same attribute category and therefore using the same lists
    */
    attributeCategory.prototype.readListsFromXML = function(listsXML,baseURL,aCatCacheObj,fromCache) {
        try {
            if (!listsXML) return;
            if (!this.lists) this.lists = new Array();
            var listsArr = WCXML.GetFirstLevelChilds(listsXML,"list");

            for (var list=0; list<listsArr.length; list++) {
                var listElem = listsArr[list];
                var listObj = new Object();
                this.lists.push(listObj);

                listObj.name =  WCXML.GetElementText(listElem,"name");	
                listObj.type =  WCXML.GetElementText(listElem,"type");	
                listObj.counter = WCXML.GetElementText(listElem,"counter");
                if (!listObj.counter) {
                    listObj.counter = null; // normalize to null, not empty string
                } else {
                    listObj.counter = parseInt(listObj.counter);
                }



                if (listObj.type == 'database') {
                    listObj.loadingType = 'OnTheFly'; 
                    // if there are other image or checkbox/radio renderings, should load immediate

                } else { 
                    listObj.loadingType = 'Immediate';
                }
                if (listObj.type != "Excel" && listObj.type != 'database') {

                    alert(ACatStrings.t('ACatListProblem1') + listObj.name + ACatStrings.t('ACatListProblem2') );
                    continue;
                }

                if (listObj.type == 'Excel') {
                    listObj.file = WCXML.GetElementText(listElem,"file");
                    if (!listObj.file) {

                        alert(ACatStrings.t('ACatListProblem1') + listObj.name + ACatStrings.t('ACatListProblem3'));
                        continue;
                    }

                    listObj.sheet = WCXML.GetElementText(listElem,"sheet");	
                    if (!listObj.sheet) listObj.sheet = "Sheet1";  // Excel defaults the first (and often only) sheet to the name Sheet1
                }
                listObj.hierarchical = WCXML.GetElementText(listElem,"hierarchical");
                if (listObj.hierarchical) listObj.hierarchical = parseInt(listObj.hierarchical,10);

                // handle listObj.invalidateAutomatically: the setting to determine whether selecting a not allowed value will force the previously selected values to be invalidated or not
                // the default (as a function) is true (what most people expect). However when not having it in the XML, it defaults to false (such XML's are not expected to exist - the setting is expected to be there always).
                // for Excel (front-end) lists: default true and becomes false by changing lists.xml (explicitly set the attribute InvalidateAutomatically = false --> works well
                // for database lists:
                // in softwareVersion 2 (WCR 14.0.x) by mistake invalidateAutomatically was set always to false, while the working is identical to true 
                // (the option did not exist and I (HADW) was sloppy, so there are plenty of XML's in databases which have this one wrong. 
                // Solution is to overrule what sits in the XML with the known behavior until somebody sets it differently (but that is only possible in version 3 and later). 
                listObj.invalidateAutomatically = WCXML.GetElementBoolean(listElem,"invalidateAutomatically");
				
                // set default value
				if(WCXML.GetElementText(listElem,"invalidateAutomatically") === ""){
					listObj.invalidateAutomatically = true;  
				}
                if (listObj.type == 'database' && parseInt(this.version,10) < 3){
                	// this.version is the version of the XML while softwareVersion is the version of the software.
                	listObj.invalidateAutomatically = true;  
                }

				listObj.allowOtherValues = WCXML.GetElementBoolean(listElem,"allowOtherValues");
				listObj.autofillExternalValue = WCXML.GetElementBoolean(listElem,"autofillExternalValue");

                listObj.uniqueIdentifyerColumn	= WCXML.GetElementText(listElem,"uniqueIdentifyerColumn");  // experimental: 1 column is assigned as unique identifyer for the rest
                if (listObj.uniqueIdentifyerColumn === "") listObj.uniqueIdentifyerColumn=-1;

                listObj.updateValuesNeeded = true;
                
                listObj.columns = new Array();
                var columnArr = WCXML.GetFirstLevelChilds(listElem,"column");
                for (var col=0; col< columnArr.length; col++) {
                    var columnElem = columnArr[col];
                    if (!columnElem) continue;
                    var columnObj = new Object();


                    
                    columnObj.columnName = WCXML.GetElementText(columnElem,"columnName");				
                    columnObj.attributeId = WCXML.GetElementText(columnElem,"attributeId");	
                    columnObj.attributeName = WCXML.GetElementText(columnElem,"attributeName");
                    columnObj.uniqueIdentifyer = WCXML.GetElementBoolean(columnElem,"uniqueIdentifyer");
                    columnObj.maxValuesInDropdown = WCXML.GetElementBoolean(columnElem,"maxValuesInDropdown");

                    /*
                    columnObj.restrictToCombinations = WCXML.GetElementBoolean(columnElem,"restrictToCombinations");
                    columnObj.autoSelect = WCXML.GetElementBoolean(columnElem,"autoSelect");
                    */

                    var attObj = this.findAttribute(columnObj.attributeId,columnObj.attributeName);
                    if (attObj) {
                        attObj.listObj = listObj;
                        attObj.columnObj = columnObj;				
                        columnObj.attObj = attObj;
                        attObj.restrictType = 'List';
                        var cellObjObj = this.findCell(attObj);
                        if (cellObjObj) {
                            var cellObj = cellObjObj.cellObj;
                            if (cellObj && cellObj.renderOption == 'RadioCheckboxes' || cellObj.renderOption == 'Images') listObj.loadingType = 'Immediate';
                        }
                    } else {
                        // this attribute no longer exists whereas it was part of the cascading dropdown
                        //if (this.layoutId) alert("Attribute " + columnObj.attributeName + " was part of a cascading dropdown but no longer exists. Please correct the Attribute Category"); 
                        columnObj.attributeId = null; columnObj.attributeName = null;
                    }

                    columnObj.currentIndex = -1;

                    listObj.columns.push(columnObj);
                }


            } // end for list



            // do back-end call to fill the list with values and combinations
            // However, in case of the admin layout page, we delay reading until having the entire list of lists
            // same logic for reconnecting the buttons to lists
            if (!this.layoutId && this.currentView != "Header") {	
                this.connectAllButtonsToLists();  
                var loadListNeeded = true;
                if (fromCache) {
                    loadListNeeded = false;
                    if  (!aCatCacheObj.aCatObjects[0].lists || !aCatCacheObj.aCatObjects[0].lists[list]) loadListNeeded=true;
                }
                if (loadListNeeded) {
                    for (var list=0; list<listsArr.length; list++) {
                        var listObj = this.lists[list];
                        if (listObj) this.readListAjax(listObj,baseURL);  
                    }
                } else {

                    for (var list=0; list<listsArr.length; list++) {
                        var listObj2 = this.lists[list];
                        var listObj = aCatCacheObj.aCatObjects[0].lists[list];
                        if (listObj && listObj2 && listObj.combinations) {  // only copy when the list is already loaded. In the other case, the copy will be done when the load returns - in onSuccess of readListAjax
                            this.copyListData(listObj,listObj2);	
                        }
                    }
                }
            }


        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ " attributeCategory.readListsFromXML" + ACatStrings.t('ACatAtLine') + err.lineNumber + '  : ' + err.message);
        }
    };

    /**
    * connects all buttons in all views to lists by calling per view connectViewButtonsToLists
    * must be called after this.lists is complete (remark it's loaded in 2 times when on the management page
    */
    attributeCategory.prototype.connectAllButtonsToLists = function() {
        "use strict";
        try {
            var view, row, col, viewObj;

            if (this.views) {
                for (var view=0; view<this.views.length; view++) {
                    var viewObj = this.views[view];
                    if (viewObj) this.connectViewButtonsToLists(viewObj.cells);
                }
            } else {
                this.connectViewButtonsToLists(this.cells);
            }


        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ " attributeCategory.connectAllButtonsToLists" + ACatStrings.t('ACatAtLine') + err.lineNumber + '  : ' + err.message);
        }

    };


    /**
    * re-connects buttons in a view to lists 
    * @param cells can either be ACatObj.views.cells or ACatObj.cells 
    **/
    attributeCategory.prototype.connectViewButtonsToLists = function(cells) {
        "use strict";
        try {
            var row, col, button, cellObj, buttonObj,list, att, attObj2, column, listObj;

            if (cells == null) return;
            for (row=0; row < cells.length; row++) {
                if ( cells[row] ) {
                    for (col=0; col < cells[row].length; col++) {
                        cellObj = cells[row][col];
                        if (cellObj && cellObj.buttons) {
                            for (button=0; button < cellObj.buttons.length; button++) {
                                buttonObj = cellObj.buttons[button];

                                if (buttonObj.listName) {
                                    listObj = this.findList(buttonObj.listName, null, buttonObj.listCounter);  // convert listName to listObj
                                    if (listObj) {								
                                        buttonObj.listObj = listObj;
                                        if (!listObj.buttons) listObj.buttons = [];
                                        listObj.buttons.push(buttonObj);
                                        buttonObj.listName = null; // to not give the impression that this is the correct way of using the list, and to avoid saving to XML

                                        // convert columnName to columnObj for all input attributes
                                        var objArr = [buttonObj.inputAtts,buttonObj.outputAtts];
                                        for (var arrIdx=0; arrIdx< objArr.length; arrIdx++) {
                                            var arr = objArr[arrIdx];  // so this is bottonObj.inputAtts or buttonObj.outputAtts
                                            if (arr) {
                                                for (att=0; att < arr.length; att++) {
                                                    attObj2 = arr[att];
                                                    for (column=0; column < listObj.columns.length; column++)  {
                                                        if (listObj.columns[column].columnName == attObj2.columnName) {
                                                            attObj2.columnObj = listObj.columns[column]; break;
                                                            attObj2.columnName = null; // to not give the impression that this is the correct way of using the list, and to avoid saving to XML
                                                        }
                                                    }
                                                }
                                            }
                                        }

                                    } else {
                                        alert('Cannot find list named ' + buttonObj.listName + ' which is used for a button in cell [' + row + '][' + col + ']'); // TODO decide whether it needs translation - depends on how likely it is that it happens, no translation also better for KnowledgeBase
                                    }
                                }

                            }
                        }
                    }
                }
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ " attributeCategory.connectViewButtonsToLists" + ACatStrings.t('ACatAtLine') + err.lineNumber + '  : ' + err.message);
        }

    };

    /**
    * Reads a single list from the back-end via an Ajax call
    * @param {Object} listObj object containing the spec of the list and receiving the result
    * @description For webserver based lists, uses listObj.file Filename of an Excel file to read, listObj.sheet Name of the worksheet to be read. If omitted, Sheet1 will be read, {Array} listObj.columns array of column names
    * @param {String} baseURL. URL of the JSP or Servlet to be called for webserver based lists. Not used for database lists
    * @return Returns nothing. Results come in listObj.columns (which will be overwritten) and listObj.combinations
    */
    attributeCategory.prototype.readListAjax = function(listObj, baseURL) {
        try {
            var url;


            if (listObj.loadingType == 'OnTheFly') return;	 // OnTheFly lists are never loaded in advance
            if (this.checkIdenticalLists === false ) {
                // check for identical lists, only with the first call to readListAjax. From then onwards it knows
                // it is important doing this at the very beginning since the rest is asynchronous so you don't know in which sequence things arrive
                // algorithm: take the first list and find all lists which are the same.
                // then take the second list and do the same, but only if that list is different from the first
                for (var list1=0; list1< this.lists.length; list1++) {
                    var listObj1 =  this.lists[list1];
                    if ((listObj1.type != 'Excel') || listObj1.identicalList) continue;  
                    for (var list2=list1+1; list2< this.lists.length; list2++) {
                        var listObj2 = this.lists[list2];
                        if (listObj2.type != 'Excel') continue;

                        if (listObj1.file != listObj2.file) continue;
                        if (listObj1.sheet != listObj2.sheet) continue;
                        if (listObj1.type != listObj2.type) continue;
                        if (listObj1.columns.length != listObj2.columns.length) continue;
                        for (var col=0; col<listObj1.columns.length; col++) {
                            if (listObj1.columns[col].columnName != listObj2.columns[col].columnName) break;			
                        }
                        if (col != listObj1.columns.length) continue;

                        // if you get here, you have 2 identical lists
                        listObj2.identicalList = listObj1;					
                    }
                }
                this.checkIdenticalLists = true;
            }

            if (!listObj.identicalList || listObj.identicalList.loadingType == 'OnTheFly') {
                if ( listObj.type == 'Excel') {   // only load the first occurance of identical lists

                // create the Ajax url
                    url = baseURL;
                url += '?fileName=' + WCXML.urlencode(listObj.file);
                url += '&sheetName=' + WCXML.urlencode(listObj.sheet);
                url += '&listName=' + WCXML.urlencode(listObj.name);


                // read columns and append to Ajax url


                for (var col=0; col< listObj.columns.length; col++) {				
                    url += '&column=' + WCXML.urlencode(listObj.columns[col].columnName);
                }

                } else {
                    // database list
                    var nColumns = listObj.columns.length;

                        var url = window.webcenterConfig.jspBaseUrl + 'GetRowsFromList.jsp?listType=WCR_DB_List&outputType=JSON&indexed=1';
                    url += '&list=' + WCXML.urlencode(listObj.name);		
                    url += '&maxrows=10000';  // To do: make this configurable
                    url += '&maxcolrows=1000';


                    for (var column=0; column< nColumns; column++ ) {
                        var columnObj = listObj.columns[column];
                        url += '&output=' +  WCXML.urlencode(columnObj.columnName);
                    }

                }
                var attCatObj = this;  // closure

                var callParams = {
                        url: url,
                        type: "GET",
                        dataType: "text",
                        cache: false, // Note: call will be cached by the AjaxCache, but not by the browser, as list can update 
                        timeout: 120000
                    };
                AjaxCache.ajax(callParams).done(function(jsonContent) {
                    try {	
                    	var jsonObj = attCatObj.readJSONResponse(jsonContent, callParams);

                        if (jsonObj.error) {
                            alert(ACatStrings.t('ACatListProblem4') + listObj.name + ACatStrings.t('ACatListProblem5') + jsonObj.error.message + ACatStrings.t('ACatListProblemDriver64'));
                        } else {
                            for (var col=0; col<listObj.columns.length; col++) {
                                var columnObj = listObj.columns[col];
                                columnObj.values = jsonObj.columns[col].values;

                                columnObj.currentIndex = attCatObj.findColumnIndex(columnObj);	

                                //columnObj.dataProvider = new SDropDataProvider({'dataSource': "ReadExcelServletCombined.jsp"});
                            }
                            listObj.combinations = jsonObj.combinations;

                            // look whether there are identical lists
                            // share the heavy lifting data (which is read-only anyway and therefore identical)
                            for (list2=0; list2 < attCatObj.lists.length ;list2++) {
                                var listObj2 = attCatObj.lists[list2];
                                if (listObj2.identicalList == listObj) {
                                    attCatObj.copyListData(listObj,listObj2);									
                                }
                            }
                            try {
                                // determine list index
                                for (var listIdx=0 ;listIdx< attCatObj.lists.length;listIdx++) {
                                    if (attCatObj.lists[listIdx] == listObj) break;
                                }

                                // look whether there are other attribute categories to update
                                for (var cacheIdx=0; cacheIdx< gACatCache.length; cacheIdx++) {
                                    if (gACatCache[cacheIdx].aCatObjects[0] == attCatObj) {
                                        for (var otherCat= 1; otherCat < gACatCache[cacheIdx].aCatObjects.length; otherCat++) {
                                            var otherACatObj = gACatCache[cacheIdx].aCatObjects[otherCat];
                                            if (otherACatObj.lists && otherACatObj.lists[listIdx]) {  // check needed because header view does not have lists
                                                attCatObj.copyListData(listObj,otherACatObj.lists[listIdx]);	
                                                otherACatObj.displayAttributeCategory();		// repaint this attribute category, otherwise the dropdowns remain on Loading...		
                                            }
                                        }
                                        break;
                                    }

                                }
                            } catch(err) {
                                //alert("readListAjax onSuccess problem: " + err);
                            }


                            attCatObj.displayAttributeCategory();


                        }
                    } catch (err) {

                        alert(ACatStrings.t('ACatJSONProblem1') + listObj.name + ACatStrings.t('ACatJSONProblem2') + jsonContent);
                        //Typically the excel sheet is still open. Try again later.
                        /*
                        if (!listObj.failures) listObj.failures=1; else listObj.failures++;
                        if (listObj.failures && listObj.failures<10) {							
                            setTimeout(function(){attCatObj.readListAjax(listObj,baseURL)},1000);
                        }
                        */
                    }
                }).fail(function(errObj) {
                    if (errObj && errObj.status !== 0) {
                    	alert(ACatStrings.t('ACatAjaxProblem1') + errObj.responseText + '  status ' + errObj.status + '\n' + ACatStrings.t('ACatAjaxProblem2') + url);
                    }
                });
            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.readListAjax:\n"+ err);
        }
    };

    /** 
    * copies column possible values and combinations from listObj1 to listObj2 (by reference)
    */
    attributeCategory.prototype.copyListData = function(listObj1, listObj2) {
        if (listObj1 && listObj2 && listObj1.columns) {  
            for (var col=0; col<listObj1.columns.length; col++) {
                var columnObj2 = listObj2.columns[col];
                if (columnObj2) {
                    columnObj2.values = listObj1.columns[col].values;	
                    columnObj2.currentIndex = this.findColumnIndex(columnObj2);  // don't copy the currentIndex!!!	
                }
            }
            listObj2.combinations = listObj1.combinations;
        }
    };

    /**
    * Picks the rigth document source from the Gallery and links to cellObj, then creates a docTableX for this cell
    */
    attributeCategory.prototype.initializeDocTableWithDocSources = function(cellObj, viewName) {
        "use strict";
        try {
            var attObj = cellObj.attObj;

            // skip the document table initialization if this is not a document reference attribute!
            if (!attObj.isDocReference()) return;


            // pick up the right dshObj
            for (var ds=0; ds < this.docSourcesGallery.length; ds++) {
                if (this.docSourcesGallery[ds].name == cellObj.docSourceName) {
                    cellObj.dshObj = this.docSourcesGallery[ds];
                    var dshObj = cellObj.dshObj;

                    // potentially update the name after a name change of the attribute
                    // for Default view, you can do this blindly
                    // Bur for other views, you can do this only if you have a dedicated configuration for this view				
                    if ( viewName == 'Default' || dshObj.name.substring(dshObj.name.length - viewName.length) == viewName ) {
                        var docSourceName = attObj.name + '_' + viewName;
                        if (cellObj.dshObj.name != docSourceName) {
                            cellObj.dshObj.name = docSourceName;
                        }
                        cellObj.docSourceName = docSourceName; 
                    }
                }
            }
            if (!cellObj.dshObj) {  // if nothing exists, create one
                cellObj.dshObj = this.createDocumentSource(this.dshConfigTablePHID, cellObj, null);
                logger += '\ninitializeDocTableWithDocSources doc source ' + cellObj.docSourceName + ' not found in gallery ';
            }

            this.createDocTableX(cellObj);

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.initializeDocTableWithDocSources:\n' + err);
		}
    };
		
    /** 
	* Picks up a render configuration from the docRenderGallery by name and links it to the cellObj of the cell we are currently reading
	* @author HADW 19 March 2015 - Personal Review 20 Mar 15
	* @param {Object} cellObj - cell to which the gallery's entry will be linked (in cellObj.docLinkTableOptions)
	* @param {String} renderName - name of the document reference render configuration
	* @param {String} viewName - name of the view in which cellObj sits
	* @return {String} errMsg - returns empty string if a corresponding configuration is found, else an error message
	*/
	attributeCategory.prototype.linkCellToRenderGallery = function(cellObj, renderName, viewName) {
		"use strict";
		try {
			var attObj = cellObj.attObj;
			if (!attObj || !attObj.isDocReference()) throw new Error('linkCellToRenderGallery called on a cell with no document reference attribute');
			
			for (var dlt = 0; dlt < this.docRenderGallery.length; dlt++) {
				if (this.docRenderGallery[dlt].getName() == renderName) {
					cellObj.docLinkTableOptions = this.docRenderGallery[dlt];
					
					// potentially update the name after a name change of the attribute
					// same logic as for document sources		
					var docLinkTableOptionsName = cellObj.docLinkTableOptions.getName();
					if ( viewName == 'Default' || docLinkTableOptionsName.substring(docLinkTableOptionsName.length - viewName.length) == viewName ) {
						cellObj.docLinkTableOptions.setName(attObj.name + '_' + viewName);
					}
					
					// set the list of available reference properties based on reference category set to this doc. ref. attribute
					if (attObj.referencePropertyACat) {
						cellObj.docLinkTableOptions.setProperties(attObj.referencePropertyACat.attributes);
					}
					break;
				}
			}
			if (!cellObj.docLinkTableOptions && cellObj.renderOption == 'Table') {
				return '\n' + 'Could not find render configuration for attribute ' + attObj.name;
			}
			
		} catch(err) {
			alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.linkCellToRenderGallery:\n' + err);
			throw err;
		}
		return '';
	};
			
	/** 
    * create an empty document table to hold the document references
    * @param cellObj - object of the cell which will contain the document table
    */
    attributeCategory.prototype.createDocTableX = function(cellObj) {
        try {
        	if (this.isProjectInfoACat) {   //JANH TODO consider adding this.isDocRefPropertiesACat here too!!!
        		// Don't create DOcTableX instance at all as "project info" category is there only
        		// to display values of project attributes in read-only mode inline in corresponding
        		// document row!
        		return;
        	}
        	
            var attObj = cellObj.attObj;
            var dshObj = cellObj.dshObj;

            attObj._docTablePromise = new Promise(function(resolveDocTablePromise, rejectDocTablePromise) {
            	
            	// load the JS classes to create documents table related objects
	        	this.docTableXFactoryPromise = Promisify.require("doctablex/DocTableXFactory");
	        	this.documentRowFactoryPromise = Promisify.require("doctablex/DocumentRowFactory");
	
	        	Promise.all([this.docTableXFactoryPromise, this.documentRowFactoryPromise, this.docRenderGalleryPromise]).then(function (promiseResult) {
	        		var docTableXFactory = promiseResult[0];
	        		
		            // ("allow multiple values" is imposed in "One Column Layout")
		            var isAllowMultipleValues = attObj.allowMultipleValues;
		
		            // prepare rendering options for the documents table
		            var docTableRenderOptions = null;
		            if (cellObj.docLinkTableOptions) {
		            	docTableRenderOptions = cellObj.docLinkTableOptions.getConfigObj();
		            }
		            if (!docTableRenderOptions || !docTableRenderOptions.columns || (docTableRenderOptions.columns.length == 0)) {
		                // there is no doc. link table options configuration - this means this is not doc. table 
		            	// layout rendering, but either "one column layout" or "simple layout" (in grid)
		            	docTableRenderOptions = {};
		            	if (this.layout == 'OneColumn') {   // no grid layout
		            		// create the default configuration to show document reference values in a concise way...
		            		isAllowMultipleValues = true;   // allways allow multiple referenced documents
		                    docTableRenderOptions.renderMethod = 'table';
		                    docTableRenderOptions.columns = [];
		                    docTableRenderOptions.columns.push({label : '', type : 'UpDownArrows', typeDetail : '', visibility : 'ReadOnly'});
		                    docTableRenderOptions.columns.push({label : 'DocRefColLabel_Thumbnail', type : 'DocumentProperty', typeDetail : 'Thumbnail', visibility : 'ReadOnly'});
		                    docTableRenderOptions.columns.push({label : 'DocRefColLabel_DocumentSpec', type : 'DocumentName', typeDetail : '', visibility : 'ReadOnly'});
		                    docTableRenderOptions.columns.push({label : '', type : 'DeleteButton', typeDetail : '', visibility : 'Editable'});
		            	} else {
		            		// "simple layout" rendering is implemented as special view mode 'list' in doc. table
		            		docTableRenderOptions.renderMethod = 'list';
		            	}
		            } else {
		            	// the "doc. link options" configuration exists - this is the table layout rendering
		                docTableRenderOptions.renderMethod = 'table';
		            }
		
		            // add system-wide configuration options (for attribute category the document thumbnails are always displayed if added in rendering configuration)
		            docTableRenderOptions.showThumbnails = true;
		    		
		            // add few more implicit (not configurable) render options
		            docTableRenderOptions.showGreenLinksOnTop = true;
		            docTableRenderOptions.showGreenLinksAtBottom = false;
		    		
		            // Propagate style in style object
		            // Note: height and width are not propagated
		            if(cellObj.styleObj && !cellObj.styleObj.standardStyle){
		            	var styleObj = {};
		            	// if labels and values are different, use value style for table rendering
		            	var copyFrom = cellObj.styleObj;
		            	if(cellObj.styleObj.isDifferentStyles){
		            		copyFrom = cellObj.styleObj.valueStyleObj;
		            	}
		            	
		            	// set css styles to be propagated
		            	if(copyFrom.background) styleObj["background-color"] = copyFrom.background;
		            	if(copyFrom.fontColor) styleObj["color"] = copyFrom.fontColor;
		            	if(copyFrom.fontFamily) styleObj["font-family"] = copyFrom.fontFamily;
		            	if(copyFrom.fontSize) styleObj["font-size"] = copyFrom.fontSize + "pt";
		            	if(copyFrom.fontStyle) styleObj["font-style"] = copyFrom.fontStyle;
		            	if(copyFrom.fontWeight) styleObj["font-weight"] = copyFrom.fontWeight;
		            	
		            	// propagate acat style styleObj for deeper level attribute categories
		            	var inlineStyleObject = ObjectHandler.getCloneOfObject(cellObj.styleObj);
		            	inlineStyleObject.height = undefined;
		            	inlineStyleObject.width = undefined;
		            	inlineStyleObject.columnWidth = undefined;
		            	if(cellObj.styleObj.isDifferentStyles){
		            		inlineStyleObject.valueStyleObj.height = undefined;
		            		inlineStyleObject.valueStyleObj.width = undefined;
		            		inlineStyleObject.valueStyleObj.columnWidth = undefined;
		            	}
		            	styleObj.inlineStyleObject = inlineStyleObject;
		            	
		            	docTableRenderOptions.styleObj = styleObj;
		            }
		            
		            // create an instance of DocTableX or one of its variants (there is currently only TextContentTableX mixin)
		            docTableXFactory.createDocTableX({
		                dshObj : dshObj,   // link between DocTableX and DocumentSourceHandler objects
						parentDocVerIDs : this.aCatOptions.docTableParentDocVerIDs,
						parentDocRow : this.parentDocRow,
						prefix : this.prefix + attObj.id,   // main prefix to be used by this DocTableX instance
						phElemID : this.prefix + attObj.id + '_docTable',   // HTML element ID, where this doc table will be put
						aCatView : 'Default',   // don't inherit view from the current (this) attribute category!
						saveXMLRootTag : 'drs',
						onChangeCallback : this.changeDocReference.bind(this, attObj),
						isInConfigPreview : this.layoutId == 'ACatLayout',
						currentUser : this.currentUser,
						isEditable : this.isAttrEditable(cellObj),   // the document reference attribute edit-ability status is inherited by the docs. table
						parentACatNotExpandedYet : this.aCatNotExpandedYet,
						allowMultiple : isAllowMultipleValues,
						docReferenceMode : attObj.docRefMode,
						showGreenLinksOnTop : true,   // specific for document reference attributes. normal upload has the "green links" only at the bottom
						projectID : this.projectID,
						taskID: this.taskID, 
						propsACatID : attObj.referenceCategoryID,
						projectCreationContext : this.aCatOptions.projectCreationContext,
						renderOptions : docTableRenderOptions
		            }).then(function (docTableObj) {
		            	// set special set of click methods for document references
			            // (can be done only now as both dshObj and docTable instances needs to be initialized)
			            // (it's not in the DocTableX constructor as DocTableX can be used in different use cases than doc. refs.)
			            dshObj.setAdvancedClickMethods('DocReference', docTableObj.thisInDOM);
		
			            // save the created document table object to attribute object
			            attObj.docTable = docTableObj;

		            	// resolve the main documents table promise as finally all needed to create
		            	// a DocTableX instance is loaded
		            	resolveDocTablePromise(docTableObj);
		            	
		            	this.displayAttributeCategory();
		            	
		            }.bind(this));
	        	}.bind(this));   // end of Promise.all...
	        	
            }.bind(this));
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.createDocTableX:\n' + err);
        }
    };
    
    
    /*
    * Create a default document source (does not create a docTableX for it, the caller should do that
    * @author HADW - personal review 6-Feb-2015
    * @param {String} dshConfigTablePHID - The id of the element in which the rendering will be done
    * @param {Object} cellObj - cell to which the document source will be assigned and in which it will display, must have an attObj. Is null when called when reading the gallery
    */
    attributeCategory.prototype.createDocumentSource = function(dshConfigTablePHID, cellObj, docSourceXML) {
        var dshObj = null;
        try {

            var attObj;
            if (cellObj) attObj = cellObj.attObj;
            var docSourceName = null;
            var includeBOMSource = false;
            if (attObj) {
            	docSourceName = attObj.name + '_' + this.currentView;	// not known when reading the gallery
            	includeBOMSource = (attObj.BOMInclude == '1') && (attObj.BOMEditable == '1');   // && this.BOMAllowed
            }
            if (!docSourceXML) docSourceXML = '';

            var updateCallBackValue = function() {
            	if (this.setACatChanged) {
            		this.setACatChanged();
            	}
            	if (this.displayAttributeCategoryPreview) {
            		this.displayAttributeCategoryPreview();
            	}
            }.bind(this);

            // --- start - Make SavedSearchContextInfo ---
            // Used to collect the context & "dynamic" values of the SmartNames on the page... (for Saved Search with SmartNames)
            // You need to create a new SavedSearchContextInfo object for every context.
            // A context is different for different Attribute Category instances.
            // (However, inside the same category instance, one could re-use this object)
            // Examples of different contexts: 
            //  - Project Creation has more "intermediate values" then just the attribute category... (also project name, ...)
            //  - A document attribute category inside a project or document attribute category has completelly different values.
            //  - ...
            
            // If we are the "top-level" attribute category on Project Creation page, the context gets passed by the Project Creation script...
            // (Because of these additional intermediate "project name", ... smartnames which also need to be resolved)
            var savedSearchContextInfo = this.aCatOptions["overrideSavedSearchContextInfo"];
            
            // Otherwise (most of the time) - make context for this category
            if(!savedSearchContextInfo) {
            	// -> Make a context for this project/document/docversion which uses intermediate values of the category...
	            savedSearchContextInfo = new SavedSearchContextInfo();
	            savedSearchContextInfo.setProjectId(this.projectID);
	            savedSearchContextInfo.setDocumentVersionId(this.docVersionID);
	            savedSearchContextInfo.setDocumentId(this.documentID);
	            savedSearchContextInfo.pushCollector(new AttributeCategoryDynamicSmartNameCollector(this));
            }
            // --- end - Make SavedSearchContextInfo ---

            var dshObj = new DocumentSourceHandler({
                docSourceXML : docSourceXML,
                docSourceTableId : dshConfigTablePHID,   // configuration table can be only one on a page
                functionPrefix : this.functionPrefix + '.docSourcesGallery[' + this.docSourcesGallery.length +']',			
                prefix : this.prefix,   // prefix used in configuration part			
                currentLanguage : (this.currentUser) ? this.currentUser.lang : null,  // language code of the current userAgent
                textClass : '',
                updateCallBack : updateCallBackValue,
                linksId : null,   // managed by DocTableX internally
                hasBillOfMaterials : includeBOMSource,
                isDocReferenceSection : true,   // makes visible the "By reference" document source
                docSavedSearchNameList : this.docSavedSearchNameList,
                savedSearchContextInfo : savedSearchContextInfo,
                documentSourceTypes : ['Browse','DocumentTemplates','SavedSearch','CurrentProject'],  // to not have ProjectFolder
                name: docSourceName,   // can be null, in that case will be read from the XML
                tableClass: 'table',
                projectId: this.projectID,
                docVersionId : this.docVersionID
            }, DocSrcHdlrStrings);

            if (cellObj) {
                cellObj.dshObj = dshObj; 
                cellObj.docSourceName = docSourceName;		
            }

            this.docSourcesGallery.push(dshObj);

            if (!docSourceXML) {
                // set the WebCenter defaults		
                var section = 'DocReference';
                if (this.isTaskTypePage) section += '~TaskExecutionPage';   // for task type, TEP, task details
                dshObj.setWebCenterDefault(section);
            }


        } catch(err) {
            alert("Problem in attributeCategory.createDocumentSource: " + err);
            return '';
        }

        return dshObj;
    };


    // To do: document and get rid of attCatObj in favor of this
    attributeCategory.prototype.readValuesFromXML = function(valuesXML) {
        var attCatObj = null;
        try {
            attCatObj = this;

            if (!attCatObj) throw(ACatStrings.t('ACatAjaxHandleProblem'));
            if (!valuesXML) throw(ACatStrings.t('ACatAttributeProblem'));
            var errorElem = WCXML.GetFirstFirstLevelChild(valuesXML,"error");
            if (errorElem) {
            	var errorCode = WCXML.GetElementText(errorElem,"code");
                var errorMsg = WCXML.GetElementText(errorElem,"message");
                
                // if we have a form builder view, ignore the fact that there are no attributes
                if(errorCode == "043" && (attCatObj.currentView == 'ProjectCreation' || attCatObj.currentView == 'TaskExecution')){
                	return;
                }
                
                if (attCatObj.errorMsg)  attCatObj.errorMsg += "\n"+errorMsg; else  attCatObj.errorMsg = errorMsg;
                if (attCatObj.currentView != 'Header') {         
                    // don't alert when there are no attributes. This is already shown on the page
                    if(errorCode != "043"){
                    	alert(ACatStrings.t('ACatErrorReadingAttributes') + ":\n" + errorMsg);
                    } else {
                    	attCatObj.loaded = true;
                    }
                }
                attCatObj.displayAttributeCategory();
                return;
            }


            var attributeCategoryElem=WCXML.GetFirstFirstLevelChild(valuesXML,"attribute_category");
            if (!attributeCategoryElem){
            	throw(ACatStrings.t('ACatServerXMLProblem3'));
            }

            // read userunits
            attCatObj.userUnits = WCXML.GetElementText(attributeCategoryElem,"user_units");

            var attributesElem=WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"atts");
            if (attributesElem) {
                // read the restricted sets
                var rsArr = WCXML.GetFirstLevelChilds(attributesElem,"rs");
                if (rsArr) {
                    for (var rs=0; rs<rsArr.length; rs++) {
                        var rsElem = rsArr[rs];
                        if (rsElem) {
                            rsObj = new Object;
                            rsObj.id = WCXML.GetElementText(rsElem,"id");
                            rsObj.name = WCXML.GetElementText(rsElem,"rsn");
                            rsObj.type = WCXML.GetElementText(rsElem,"rsat");
                            var type2 = rsObj.type;
                            if (type2) {
                                type2 = type2.toLowerCase();
                            } else {
                                type2 = 'text';
                            }
                            rsObj.values = new Array();
                            var valArr = rsElem.getElementsByTagName("rsv");
                            if (valArr && valArr.length>0) {
                                for (var rsVal=0; rsVal< valArr.length; rsVal++) {
                                    var id = WCXML.GetElementAttribute(valArr[rsVal],"id");
                                    var value = WCXML.GetElementText(valArr[rsVal],"value");

                                    if (value && type2 != 'integer' && type2 != 'number' && type2 != 'text' && type2 != 'float' && type2 != 'datetime') value = WCHTML.getFloatString(value,6);
                                    if (value && type2 == 'float') value = parseFloat(value).toString();  

                                    rsObj.values.push({
                                        id:id,
                                        value:value
                                    });
                                }

                            }
                            if (!attCatObj.restrictedSets) attCatObj.restrictedSets=new Array();
                            attCatObj.restrictedSets.push(rsObj);
                        }

                    }
                } // end reading restricted sets

                // read the attribute values
                var attArr = WCXML.GetFirstLevelChilds(attributesElem,"att");
                if (attArr && attArr.length>0) {
                    for (var attribute = 0; attribute < attArr.length; attribute++) {
                        var attElem = attArr[attribute];

                        attCatObj.readAttributeValue(attElem);
                    }
                }
            }
            // if there's a second values url, load this one if it's not loaded yet
            if(attCatObj.valuesURLMerge!=null && $.trim(attCatObj.valuesURLMerge)!='' && !(attCatObj.secondrun==true)){
                attCatObj.secondrun=true;
                attCatObj.getAttributeValues(true);
            }else if(attCatObj.autoUpdateProjAttrsCheckbox){
                attCatObj.getProjectAttributeValues();
            }else{			
                attCatObj.loaded = true;
                attCatObj._displayAttributeCategory();
                attCatObj.onValuesLoaded();
            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.readValuesFromXML:\n' + err);
        }
    };

    /**
 * Function to read attribute value from given element. It supports reading of both: text-based attribute 
 * values of "standard" attributes and document reference values.
 * 
 * @param attElem - Parsed content of <att> element from the input XML.
 */
    attributeCategory.prototype.readAttributeValue = function(attElem) {
        try {
            var attId = WCXML.GetElementText(attElem, "id");
            if (attId) {
                var attObj = this.findAttribute(attId);
                if (attObj) {
                    attObj.type = WCXML.GetElementText(attElem, "at");
                    attObj.typeLocalized = WCXML.GetElementText(attElem, "atl");

                    if (this.copyValues === true) {
                        if (attObj.isDocReference()) {
                            // read document reference attribute value (fill prepared documents table)
                            this.readDocumentReferenceValue(attElem, attObj);
                        } else {   
                            // read "standard" attribute (not document reference) value
                            this.readStandardAttributeValue(attElem, attObj);

                            // Set the index of the column of list restriction if there is any. Only for immediately loading list
                            var columnObj = attObj.columnObj;
                            if (columnObj && columnObj.values) {
                                columnObj.currentIndex = this.findColumnIndex(columnObj,attObj.value);		
                                attObj.listObj.updateValuesNeeded = true;
                            }
                            
                            if(attObj.listObj && attObj.listObj.autofillExternalValue){ 
                            	if(attObj.listObj.loadingType == 'OnTheFly'){
		                        	// look whether autoselect of other items is needed
		                         	this.checkAutoSelectLazyLoading(attObj,false);
		                        } else {
		                        	attObj.listObj.forceValueUpdate = true;
		                        }
                            }
                        }
                    } else if (attObj.isDocReference() && attObj._docTablePromise) {
                    	// don't forget to resolve the Promise saying when all the document rows of this 
                    	// documents table were added to the documents table also for case when 
                    	// attribute values are not copied!
                    	attObj._docTablePromise.then(function (promiseResult) {
                    		var docTable = promiseResult;
                    	   	if (docTable) {
                    	   		docTable.dataLoadingFinished();
                    	   	}
                    	}.bind(this));
                    }
                    

                    attObj.unit = WCXML.GetElementText(attElem, "au");  
                    attObj.unitLocalized = WCXML.GetElementText(attElem, "aul");  
                    if (!attObj.unit) attObj.unit = attObj.type;  // typical for Integer, Text, Float

                    // check whether restricted set
                    var rsId = WCXML.GetElementText(attElem, "rs");
                    if (rsId && this.restrictedSets) {
                        for (var rs=0; rs< this.restrictedSets.length; rs++) {
                            if (rsId == this.restrictedSets[rs].id) {
                                attObj.rs = this.restrictedSets[rs];	
                                attObj.restrictType = 'RestrictedSet';
                                if (attObj.columnObj) { // a list type attribute was changed into a restricted set DT 120805
                                    attObj.columnObj.attObj = null;
                                    attObj.columnObj = null;
                                }
                                attObj.listObj = null; // never a listObj when type is RestrictedSet
                                var valArr = attObj.rs.values;
                                if (valArr && valArr.length ==2 && valArr[0].value == 'yes' && valArr[1].value == 'no') attObj.isCheckBox = true;
                                break;
                            }
                        }
                    }
                    if (attObj.isDateTimeNotInRS()) {
	                    if (attObj.value) attObj.value = this.correctForDaylightSaving(new Date(parseInt(attObj.value)));
	                    attObj.processDateAttrValue();
	                }
                } else {
                    // so you get a value for an attribute not in the category --> do nothing
                }
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.readAttributeValue:\n' + err);
        }
    };


    /**
	 * Read text attribute value. Doesn't support document references.
	 * 
	 * @param attElem - Parsed content of <att> element from the input XML.
	 * @param attObj - Attribute object where to store read value.
	 */
    attributeCategory.prototype.readStandardAttributeValue = function(attElem, attObj) {
		try {
			var av = WCXML.GetElementText(attElem,"av");
			if ((av != null) && (av != '')) { //overwrite only when there's a value
				attObj.value = av;
				// DT112473 Fix empty value at beginning
				if (attObj.value.indexOf('|') == 0 && attObj.value.length > 1) {
					attObj.value = " " + attObj.value;
	            }
	        }
	    } catch(err) {
	        alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.readStandardAttributeValue:\n' + err);
	    }
    };

    /**
	 * Read document references values
	 * 
	 * @param {DOM object} attElem - Parsed content of <att> element from the input XML.
	 * @param {Attribute} attObj - Attribute object where to store read value.
	 * @param {Object} result - Object containing:
	 * 		- {Array} docRows: Array of document rows to be filled by this method. 
	 * 		- {String} value: String representation of the document reference attribute value.
	 * 		- {String} valueHyperlink: HTML code representing hyperlinks to referenced documents in WebCenter.
	 * 		If the 'result' parameter is specified (and not null), all loaded document rows are stored 
	 * 		in the 'docRows' array instead of appended to the corresponding documents table.
	 * 		'value' and 'valueHyperlink' are updated the same way as Attribute's members of the same name.
	 * 		If the 'result' parameter is not specified (or null) all the doc. rows are appended to the 
	 * 		corresponding documents table and the 'attObj' object's 'value' and 'valueHyperlink' members
	 * 		are updated directly. 
	 * 		In both cases the generated document rows are the same!
	 * @returns {Promise} - Promise object that is resolved when all data of actual document reference attribute
	 * 		are loaded and stored in its documents table.
	 */
	attributeCategory.prototype.readDocumentReferenceValue = function(attElem, attObj, targetObject) {
        try {
        	var docRefValuePromises = [];
        	
    		return Promise.all([attObj._docTablePromise, this.documentRowFactoryPromise, this.docRenderGalleryPromise])
    				.then(function (promiseResult) {
            	var docTable = promiseResult[0];
            	var documentRowFactory = promiseResult[1];
            	
            	if (!docTable) {
            		return;
            	}

                // a string containing names of all referenced documents as it'd be a multivalue 
                // text attribute - usefull for read-only cases and back-compatibility
                var docNames = '';
                var docHyperlinks = '';

	        	// read the document reference XML value
		        var drsElem = WCXML.GetFirstFirstLevelChild(attElem, "drs");
		        if (drsElem) {
		            var drArr = WCXML.GetFirstLevelChilds(drsElem, "dr");
		            if (drArr) {
	                    // array of referenced documents with their properties (= value of doc. ref. attribute)
	                    for (var dr = 0; dr < drArr.length; dr++) {
	                        var drElem = drArr[dr];
	                        var drvObj = {};
							drvObj.drID = WCXML.GetElementText(drElem, "id");
							drvObj.docVerID = WCXML.GetElementText(drElem, "dv");
							drvObj.docID = WCXML.GetElementText(drElem, "doc_id");   // not in XML!
							
							drvObj.docName = WCXML.GetElementText(drElem, "dn");
							docNames += drvObj.docName + ' | ';
							docHyperlinks += '<a class="'+window.webcenterConfig.jspBaseUrl+'docrefdoclink darktext_darklink_11" href="docdetails.jsp?menu_file=myprojects&amp;docVerID=' + drvObj.docVerID + '" target="_blank">' + WCXML.xmlencode(drvObj.docName) + '</a>' + ' | ';
	
	                        // (BOM count for document references was despeced)
//		                        var count = WCXML.GetElementText(drElem, "count");
//      		                if (count) drvObj.bomCount = parseFloat(count).toString();

	                        drvObj.apprStat = WCXML.GetElementText(drElem, "astat");   // not in XML!
	                        var verIdx = WCXML.GetElementText(drElem, "dvidx");
	                        if (verIdx) drvObj.docVerIdx = parseInt(verIdx).toString();
	                        drvObj.thumbnailURL = WCXML.GetElementText(drElem, "thumb_url");
	
	                        if (this.isProjectInfoACat) {   //JANH TODO consider adding this.isDocRefPropertiesACat here too!!!
	                        	// if we are loading this attribute value as part of "project info" in DocTableX,
	                        	// skip loading of ref. properties as their are not used in this scenario!
	                        	drvObj.properties = [];
	                        } else {
		        				var refPropsElem = WCXML.GetFirstFirstLevelChild(drElem, "refatts");
		        				if (refPropsElem) {
				                    var refPropsArr = WCXML.GetFirstLevelChilds(refPropsElem, "att");
		                            if (refPropsArr && refPropsArr.length > 0) {
		                                drvObj.properties = [];
		                                for (var refPropIdx = 0; refPropIdx < refPropsArr.length; refPropIdx++) {
				                            var propObj = new Attribute({
				                            	prefix : this.prefix + 'refProp_'
				                            });
		                                    // load the current document reference property value
				                            this.readDocRefPropertyValue(refPropsArr[refPropIdx], propObj);
				                            
				                            if (propObj.isDateTimeNotInRS()) {
				                            	// special handling of date&time (if its not in a restricted set)
				                                if (propObj.value) propObj.value = this.correctForDaylightSaving(new Date(parseInt(propObj.value)));
				                                propObj.processDateAttrValue();
				                                
				                                //JANH TODO special handling of rich text attribute/property might be needed here too!!!
				                            }
				                            drvObj.properties.push(propObj);
				                        }
		                            }
		                        }
	                        }
                            if (targetObject) {
                                // if 'targetObject' exists (is passed as a parameter), don't append the document
                                // row to the 'docTable', but store it in the provided targetObject.docRows array.
                                // this is for cases when we are loading just content of the documents table,
                                // not initializing the whole table (like in case of reading project attribute
                                // values in order to replace a task specification values).
                                if (!targetObject.docRows) targetObject.docRows = [];
                                drvObj.parentDocTable = docTable;
                                // (don't forget to set the correct parent doc. table for this doc. row!)
                                docRefValuePromises.push(documentRowFactory.createStaticDocumentRow(drvObj)
                                		.then(function (docRowObj) {
                        			targetObject.docRows.push(docRowObj);
		                        }.bind(this)));
                            } else {
                                // otherwise create the row instance and append it to the documents table 
                                // of the actual attribute.
                            	var lastItem = (dr == drArr.length-1);
                            	docRefValuePromises.push(docTable.appendStaticDocRow(drvObj, lastItem));
                            }
	                    }   // end of for-cycle
		            }
		        }
		        
	        	// wait for all data to be loaded...
	            return Promise.all(docRefValuePromises).then(function () {
	                // set standard attribute value to contain referenced document names as 
	        		// multivalue text value once the document reference value is fully loaded
	                var value = "";
	                var valueHyperlink = "";
	                if (docNames.length > 0) {
	                    value = docNames.slice(0, docNames.length-3);   // (to remove the last pipe)
	                    valueHyperlink = docHyperlinks.slice(0, docHyperlinks.length-3); 
	                }
	                // the same difference in behaviour controlled by existance of 'targetObject' parameter as above!
	                if (targetObject) {
	                    targetObject.value = value;
	                    targetObject.valueHyperlink = valueHyperlink;
	                } else {
	                    attObj.setValue(value, valueHyperlink);
	                }
	                
	            	// repaint attribute category this time with fully initialized documents table in it
	                this.displayAttributeCategory();

	                // resolve the Promise saying that all the document rows of this documents table
	                // were added to the documents table. Take care the document rows don't have to be
	                // (and often are not) fully initialized yet!
	                return attObj.docTable.dataLoadingFinished();
	            }.bind(this))
	            .catch(function (reason) {
	            	// handle any potential errors and make sure the DocTableX.docRowsInsertedPromise is
	            	// resolved/failed in this case as well!
	            	if (window.console && window.console.error) {
	            		// we want to keep track of what went wrong in the JS console...
	            		window.console.error(reason+"");
	            	}
	            	return attObj.docTable.dataLoadingFailed();
	            }.bind(this));
    		}.bind(this));   // end of "outer" Promise.all...
    		
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.readDocumentReferenceValue:\n' + err);
        }
    };    


/**
 * Function to read all info provided about a single attribute (property) of actually processed document reference.
 * 
 * Take care the attribute objects created by this function - they are just partially initialized!
 * 
 * @param attElem - Parsed content of <att> element within <refatts> from the input XML.
 * @param attObj - Attribute object where to store read value.
 */
    attributeCategory.prototype.readDocRefPropertyValue = function(attElem, attObj) {
        try {
            if (!attElem) return;
            attObj.id = WCXML.GetElementText(attElem, "id");
            attObj.name = WCXML.GetElementText(attElem, "an");
            attObj.rsId = WCXML.GetElementText(attElem, "rs");
            attObj.type = WCXML.GetElementText(attElem, "at");
            attObj.typeLocalized = WCXML.GetElementText(attElem, "atl");
            
            if (this.copyValues === true) {
                // read "standard" attribute (not document reference) value
                this.readStandardAttributeValue(attElem, attObj);					
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.readDocRefPropertyValue:\n' + err);
        }
    };

    /**
    * Sets defaults for task specifications with project attribute values
    * @param valuesXML: XML returning from the SDK call GetAttributes.jsp?projectid=..., see private SDK documentation for details
    */
    attributeCategory.prototype.readProjectAttributeValuesFromXML= function(valuesXML) {
        var attCatObj = null;
        try {
            attCatObj = this;

            if (!attCatObj) throw(ACatStrings.t('ACatAjaxHandleProblem'));
            if (!valuesXML) throw(ACatStrings.t('ACatAttributeProblem'));
            var errorElem = WCXML.GetFirstFirstLevelChild(valuesXML,"error");
            if (errorElem) {
                var errorCode = WCXML.GetElementText(errorElem,"code");
                // if error code is 043 (no attributes), just do nothing
                if(errorCode != "043"){
                    var errorMsg = WCXML.GetElementText(errorElem,"message");
                    if (attCatObj.errorMsg)  attCatObj.errorMsg += "\n"+errorMsg; else  attCatObj.errorMsg = errorMsg;
                    if (attCatObj.currentView != 'Header') {
                        alert(ACatStrings.t('ACatErrorReadingAttributes') + ":\n" + errorMsg);
                    }
                }else{
                    //If no project attributes found, everything is loaded
                    attCatObj.loaded = true;
                }

                attCatObj._displayAttributeCategory();
                if (attCatObj.loaded) {
                	attCatObj.onValuesLoaded();
                }
                return;
            }

            var attributeCategoryElem=WCXML.GetFirstFirstLevelChild(valuesXML,"attribute_category");
            if (!attributeCategoryElem){
            	throw(ACatStrings.t('ACatServerXMLProblem3'));
            }

            // something we need to remember over all project attributes
            var didUpdateValue = false;
            var docTablePromises = [];
            
            var attributesElem=WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"atts");
            if (attributesElem) {
                // read the attribute values
                var attArr = WCXML.GetFirstLevelChilds(attributesElem,"att");
                if (attArr && attArr.length>0) {
                    for (var attribute = 0; attribute < attArr.length; attribute++) {
                        var attElem = attArr[attribute];
                        var attId = WCXML.GetElementText(attElem,"id");
                        if (attId) {
                            attObj = attCatObj.findAttribute(attId);
                            if (attObj) {
                                // for document references the value is just "|" separated list of referenced document names
                                attObj.projAttValue = WCXML.GetElementText(attElem,"av");
                                // check whether restricted set
                                var rsId = WCXML.GetElementText(attElem,"rs");

                                var projDocRefs = {};
                                var docTablePromise = null;   //used only for document reference attributes
                                if (attObj.isDocReference()) {
                                    // in case of document reference attribute read and create the rows in
                                    // special datastructure defined within the readDocumentReferenceValue()
                                    // method, which directly stores just document rows and related info
                                    // (no documents table, which is the same as in the 'attObj' here)
                                    // read document reference attribute value (fill 'projDocRefs' object)
                                    docTablePromise = attCatObj.readDocumentReferenceValue(attElem, attObj, projDocRefs);
                                }
                                
                                docTablePromises.push(Promise.all([docTablePromise]).then(function (attObj, projDocRefs, rsId) {
                                    if (attObj.isDocReference()) {
                                        attObj.projDocRefs = projDocRefs;
                                        attObj.projAttValue = projDocRefs.value;
                                    }

                                	// handle daylight saving in date attributes with restricted set configured
                                    if (attObj.type && attObj.type.toLowerCase() == 'datetime' && !rsId && attObj.projAttValue && attObj.projAttValue!="") {		
                                        attObj.projAttValue = attCatObj.correctForDaylightSaving(new Date(parseInt(attObj.projAttValue)));
                                    }

                                    if (attObj.projAttValue) {
                                        // replace attribute values for attributes that
                                        //     -- are empty, or have value "***"
                                        //     -- are read-only, and have no restriction
                                        var cellObjRes = attCatObj.findCell(attObj);
                                        var visibility = "";
                                        if(cellObjRes){
                                            visibility = attCatObj.checkVisibility(cellObjRes.cellObj);
                                        }
                                        if((!attObj.value || attObj.value == "" || attObj.value == "***" 
                                            || (visibility ===  "ReadOnly" && (!attObj.restrictType || attObj.restrictType === "None" || attObj.restrictType === "DocumentReference"))
                                            || (attObj.value == attObj.projAttValue && attObj.columnObj && attObj.columnObj.autoSelected))){
                                        	// last condition: it could be that while reading the values, an attribute gets automatically filled out
                                        	// however. If the value is read from the XML, we want to make sure that cascading happens correctly, so 
                                        	// we handle it as if the attribute was not yet filled out
                                        		
                                            if (attObj.isDocReference() && attObj.docTable) {
                                                // set the document reference specific members
                                                attObj.docTable.reset();
                                                attObj.docTable.initWithDocRows(projDocRefs.docRows);
                                                // set the flag that the value was injected by application logic. 
                                                // this is needed, e.g. for read-only doc. reference values that can be
                                                // changed by this action and we need to make sure the value is saved!
                                                attObj.docTable.setInjectedValue(true);
                                                attObj.setValue(projDocRefs.value, projDocRefs.valueHyperlink);
                                            } else {
                                                // not a document reference attribute
                                            	attObj.value = attObj.projAttValue;
                                            }
                                            didUpdateValue = true;

                                            if (rsId && attCatObj.restrictedSets) {
                                                for (var rs=0; rs< attCatObj.restrictedSets.length; rs++) {
                                                    if (rsId == attCatObj.restrictedSets[rs].id) {
                                                        attObj.rs = attCatObj.restrictedSets[rs];
                                                        break;
                                                    }
                                                }
                                            }
                                            if (attObj.isDateTimeNotInRS()) {
                                            	attObj.processDateAttrValue();
                                            }
                                            if(attObj.columnObj){
                                            	attObj.columnObj.autoSelected = false;
                                                if(attObj.listObj.loadingType == 'OnTheFly'){
                                                        attCatObj.checkAutoSelectLazyLoading(attObj,false);
                                                } else {								
                                                    attCatObj.changeListSelection(attObj,attObj.value);
                                                }
                                            }
                                        }
                                    }
                                }.bind(this, attObj, projDocRefs, rsId)));
                            }
                        }   //end of if (attId)...
                    }
                }
            }
            Promise.all(docTablePromises).then(function () {
                if (didUpdateValue) {
                    attCatObj.setAttributesChanged();
                }
	            attCatObj.loaded = true;
	            attCatObj._displayAttributeCategory();
	            attCatObj.onValuesLoaded();
            }.bind(this));
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.readProjectAttributeValuesFromXML:\n' + err);
        }
    };

    /**
    * called when clicking on the [!] sign when the task specification is different from the project attribute value
    * @param row, col - row and col of the cell on which the operation is done 
    */
    attributeCategory.prototype.recoverProjectAttributeValue = function(row,col) {
        try {
            if(!isNaN(row) && this.cells[row]){
                cellObj = this.cells[row][col];
                if(cellObj && cellObj.attObj){
                    attObj = cellObj.attObj;
                    attObj.value = attObj.projAttValue;	

                    if (attObj.isDocReference() && attObj.projDocRefs && attObj.docTable) {
                        // set the document reference specific members
                        attObj.docTable.reset();
                        var projDocRefs = attObj.projDocRefs;
                        attObj.docTable.initWithDocRows(projDocRefs.docRows);
                        attObj.setValue(projDocRefs.value, projDocRefs.valueHyperlink);

                    } else if (attObj.type.toLowerCase() == 'richtext'){
                        $("#"+attObj.id).summernote('code', attObj.value);
                    } else if (attObj.isDateTimeNotInRS()) {					
                    	attObj.processDateAttrValue();					
                        halfhours = attObj.value.getHours() * 2 + Math.floor(attObj.value.getMinutes()/30);
                        document.getElementById("longFormat_adate" + attObj.id).value = attObj.hiddenDate;
                        document.getElementById("date_period_of_day_" + attObj.id).value = halfhours;
                    } else {
                        document.getElementById(attObj.id).value = attObj.value;
                    }
                    try{	
                        if(attObj.type.toLowerCase() == 'datetime' && !attObj.rs){
                            this.changeDateTime(row,col);
                        } else if (attObj.columnObj) {  // attribute is part of cascading dropdown

                            // redo auto-selection
                            attObj.columnObj.autoSelected = false;
                            if(attObj.listObj.loadingType == 'OnTheFly'){
                                this.checkAutoSelectLazyLoading(attObj,true);
                            } else {								
                                this.changeListSelection(attObj,attObj.value);
                            }

                        }else{
                            this.changeText(document.getElementById(attObj.id),row,col);
                        }
                    }catch(err){}
                }
                delayhidethumb();
                this.displayAttributeCategory();
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.recoverProjectAttributeValue:\n' + err);
        }
    };

    
	
    /**
    * convert a value (potentially representing multiple values) to an index (potentially representing multiple indices
    * example: convert Belgium | France to 10~23 where Belgium is country with index 10 and France country with index 23
    * @param columnObj object representing the column containing the allowed values
    * @param value Optional value to be found. If not provided, uses columnObj.attObj.value (meaning it converts value to index for the column)
    */
    attributeCategory.prototype.findColumnIndex = function(columnObj,value) {
        try {
            var foundIndex = -1;
            var valueArr;
            var newIndex = '';

            if (value==null && columnObj.attObj) value = columnObj.attObj.value;

            if (columnObj && value != '' && value != null) {
                var valArr = columnObj.values;  // the set of possible values

                if (value.constructor == String) {
                    valueArr = value.split(this.multiSep);  // the set of actual values
                } else {
                    valueArr = [];
                    valueArr.push(value);
                }


                for (var idx=0; idx< valueArr.length; idx++) {

                    if ( valArr && valArr.length>0) {									
                        var columnLength = valArr.length;				
                        for (var idx2=0; idx2<columnLength; idx2++) {
                            if (valArr[idx2] == valueArr[idx]) break;   // == and not === on purpose! sometimes this matches "3" to the integer 3
                        }
                        if (idx2<columnLength) foundIndex = idx2; else foundIndex=-1;
                    }
                    if (newIndex) newIndex+= '~';
                    newIndex +=  foundIndex;
                }


            }

            if (newIndex == '') newIndex = -1;
            return newIndex;
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.findColumnIndex:\n' + err);
        }
    };

    /**
    * finds the cell where an attribute is placed
    * @return null if attribute is not found in the grid
    * @return {Object} {cellObj,row,col}: the found cellObj and the row and col where it's positioned
    */
    attributeCategory.prototype.findCell = function(attObj) {
        try {
        	if (!this.cells) return null;

            var Obj = {};
            for (var row=0; row < this.nRows; row++) {
            	if (!this.cells[row]) return null;
                for (var col=0;col < this.nCols; col++) {
                    var cellObj = this.cells[row][col];
                    if (cellObj) {
                        var attObj2 = cellObj.attObj;
                        if (attObj2 && attObj2 == attObj) {  
                            Obj.cellObj = cellObj;
                            Obj.row = row;
                            Obj.col = col;
                            return Obj;
                        }
                    }
                }
            }

            return null;
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.findCell:\n' + err);
        }
    };


    attributeCategory.prototype.readAttributeList = function(attributeListElem) {
        try {

            if (attributeListElem) {
                var attributeArr = WCXML.GetFirstLevelChilds(attributeListElem,"attribute");
                if (attributeArr) {
                    if (!this.attributes) this.attributes = new Array();
                    for (var att = 0; att<attributeArr.length; att++) {

                        var deleted = WCXML.GetElementText(attributeArr[att],"deleted");
                        if (!deleted || deleted == "0") {

                            var attObj = new Attribute({
                                prefix : this.prefix,
                                name : WCXML.GetElementText(attributeArr[att],"name"),
                                id : WCXML.GetElementAttribute(attributeArr[att],"id"),
                                description : WCXML.GetElementAttribute(attributeArr[att],"description"),
                                unitId : WCXML.GetElementText(attributeArr[att],"attribute_unit_id"),
                                rsId : WCXML.GetElementText(attributeArr[att],"restricted_set_id"),
                                value : "",
                                placedOriginally : true
                            });

                            //load is_required flag from xml if task type
                            if (this.isTaskTypePage) {
                                var isRequired = WCXML.GetElementText(attributeArr[att],"is_required");
                                attObj.setIsRequired(isRequired == '1');
                            }

                            // if document reference, get more info
                            if (attObj.isDocReference()) {
                                if (this.docRefAttrInReferenceCategory) {
                                    // a document reference attribute is placed in an attribute category used 
                                    // as a Reference Category (of at least one document reference attribute)
                                    // this if basically filters out all document referenced from Reference 
                                    // Categories - the only suitable solution as it's very hard to make sure
                                    // such a wrong configuration is never created.
                                } else {
                                    attObj.restrictType = 'DocumentReference';							
                                    attObj.setDocRefMode(WCXML.GetElementText(attributeArr[att],"doc_ref_mode"));
                                    attObj.BOMInclude = WCXML.GetElementText(attributeArr[att],"include_in_bom");
                                    attObj.BOMEditable = WCXML.GetElementText(attributeArr[att],"include_in_bom_editable");
                                    attObj.BOMCount = WCXML.GetElementText(attributeArr[att],"show_count");	
                                    attObj.referenceCategoryID = WCXML.GetElementText(attributeArr[att],"ref_category_id");
                                    attObj.referenceCategoryName = WCXML.GetElementText(attributeArr[att],"ref_category_name");
                                    
                                    //JANH TODO - Here we create a race condition! The reference properties must first be loaded, 
                                    // but you risk that the values are coming in  first so you cannot be sure that the reference 
                                    // properties are already known at that moment. 
                                    if (attObj.referenceCategoryID) this.getDocumentReferenceProperties(attObj);  
                                }
                            }

                            this.attributes.push(attObj);

                        }
                    } // end for
                }
            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.readAttributeList:\n' + err);
        }
    };

    /**
    * @author HADW
    * goes fetching via Ajax the reference property category and fills it into attObj.referencePropertyACat
    * WARNING: THIS IS AN ASYNCHRONOUS CALL. Whereas referencePropertyACat is immediately there, it only launches the fetching of the XML with the real info
    */
    attributeCategory.prototype.getDocumentReferenceProperties = function(attObj) {
        'use strict';
        try {
        	if (this.layoutId) {
        		// we want to load reference category only when configuring attribute category!
        		// in execution part this is handled by DocTableX and individual document rows...
	            var aCatOptions = {};
	            // first make a new object under the Div containing the mother attribute category
	            var refCatId = this.prefix + 'referenceCategory_' + attObj.id;
	            $('#' + this.id).append('<div id="'+ refCatId +'">');
	            aCatOptions.ACatObj = refCatId;
                aCatOptions.readURL = window.webcenterConfig.jspBaseUrl + 'GetAttributeCategoryXML.jsp?categoryid=' + attObj.referenceCategoryID + '&ajax=1';
	            aCatOptions.currentUser = this.currentUser;
	            // to detect endless loops when doc. ref. attribute is placed in an attribute category used 
	            // as a Reference Category
	            aCatOptions.docRefAttrInReferenceCategory = true;
	            aCatOptions.isDocRefPropertiesACat = true;

/*//JANH TODO!!! This part doesn't work yet as both callbackOnLoad, callbackOnRender are not called in case of reference category!?!? A new callback will be probably needed...
	            // create a callback to update list of doc. reference properties available in the doc. table rendering configuration config.
	            // of this attribute (based on the loaded referencePropertyACat category)
        		var updateRenderConfig = null;
        		var cellObj = this.findCell(attObj);
        		if (cellObj) cellObj = cellObj.cellObj;
        		if (cellObj && cellObj.docLinkTableOptions) {
        			updateRenderConfig = function(renderConfigObj) {
        				var refACat = this.referencePropertyACat;
        				if (refACat && refACat.loaded && !refACat.errorMsg) {
        					renderConfigObj.setProperties(refACat.attributes);
        				}
        			}.bind(attObj, cellObj.docLinkTableOptions.getConfigObj());
        		}
	            aCatOptions.callbackOnLoad = updateRenderConfig;
*/
	            // load and initialize the reference attribute category of this doc. ref. attribute
	            attObj.referencePropertyACat = new attributeCategory(aCatOptions);                        	
        	}
        } catch (err) {
            alert("Problem in getDocumentReferenceProperties: " + err);
        }
    };

    /**
    * Finds an attribute in the list aCatObj.attributes based on id and name. if both are given, name is given more importance than id.
    * @param returnIdx {Boolean} if true, the index in aCatObj.attributes will be returned, else the attribute object.
    * @return {Object} or {int} depending on setting of returnIdx. Returns null if not found in case of returnIdx false and -1 if true
    */
    attributeCategory.prototype.findAttribute = function(attId,attName,returnIdx) {
        try {
            var attObj = null, attIdx = -1;
            if (attName) {
                for (var attribute=0; attribute < this.attributes.length; attribute++) {
                    if (this.attributes[attribute].name.toLowerCase() == attName.toLowerCase()) {
                        attObj = this.attributes[attribute];
                        attIdx = attribute;
                        break; // jump out of the for loop
                    }
                }
            }
            if (attId && !attObj) {  // not found, try with the Id
                for (var attribute=0; attribute < this.attributes.length; attribute++) {
                    if (this.attributes[attribute].id == attId) {
                        attObj = this.attributes[attribute];
                        attIdx = attribute;
                        break; // jump out of the for loop
                    }
                }
            }

            return returnIdx ? attIdx : attObj;

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.findAttribute:\n' + err);
            return null;
        }
    };
    
    
    /**
     * Get a property object by name
     * @returns {Property} - property with given name (or undefined)
     */
    attributeCategory.prototype.getPropertyByName = function(name) {
    	return _.find(this.otherProperties, function(property){
    		return property.name === name;
    	});
    };

    /**
    * converts listName (and optionally listCounter and listType) to listObj
    * @return {Object} - returns the found listObj or null if not found
    * @param {String} listName - name of the list to be found. Remark that listName is not unique. Only the combination of listName, listType and listCounter is unique
    * @param {String} listType - database or Excel.  If null, no checking is done on list type
    * @param {String} listCounter - counter of the list if the same list is copied. Remark that listCounter can be null, also when the same list is used multiple times (the first one is null). 
    */
    attributeCategory.prototype.findList = function(listName, listType, listCounter) {
        "use strict";
        var listObj = null;
        try {

            for (var list=0; list < this.lists.length; list++) {
                listObj = this.lists[list];
                var nameMatches = (listObj.name == listName);						
                var typeMatches = ( !listType ) || (listType == listObj.type); 						
                var counterMatches = (!listCounter && !listObj.counter) || (listCounter == listObj.counter);

                if (nameMatches && typeMatches &&  counterMatches) break;

            }
            if (list == this.lists.length) listObj = null; // not found

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.findList:\n' + err);
        }
        return listObj;
        };

    /**
    * Error handling for problems with ajax call
    */
    attributeCategory.prototype.readError = function(status,ObjHdl,description) {

        var attCatObj = this;
        var displayElement = document.getElementById(attCatObj.displayId);

        var loginForm = document.getElementById('sessionexpiredform');

        if ( (description == 'SESSION_EXP' || description == '998') && loginForm!= null ) {
            // redirect to the login.jsp for login again. 
            loginForm.submit();  		
        } else {
            if (attCatObj.currentView != 'Header') {
                if (displayElement) {
                    var htmlcode = ACatStrings.t('ACatErrorLoadingACat') + ACatStrings.t('ACatServerStatus') + status;
                    displayElement.innerHTML = htmlcode;
                } else {
                    // alert only when status != 0. Status 0  means typically that the user tries leaving the page
                    if (status && status != '0') alert(ACatStrings.t('ACatServerXMLProblem1') + status);
                }
            }

            attCatObj.errorStatus = ACatStrings.t('ACatServerXMLProblem2');

            // check other objects which hoped getting the results from this call
            for (var cacheIdx = 0 ; cacheIdx< gACatCache.length; cacheIdx++) {
                if (gACatCache[cacheIdx].aCatObjects[0] == attCatObj) {
                    for (var otherCat= 1; otherCat < gACatCache[cacheIdx].aCatObjects.length; otherCat++) {
                        var otherACatObj = gACatCache[cacheIdx].aCatObjects[otherCat];
                        otherACatObj.errorStatus = ACatStrings.t('ACatServerXMLProblem2');
                        if (otherACatObj.currentView != 'Header') {
                            var displayElement2 = document.getElementById(otherACatObj.displayId);
                            if (displayElement2) displayElement2.innerHTML = htmlcode;						
                        }
                    }

                    break;
                }
            }
        }
    };
    
    /**
     * Error handling for problems with ajax call
     */
     attributeCategory.prototype.readErrorObj = function(errObj) {
    	 this.readError(errObj.status+'',null,errObj.responseText);
     };

    /**
    * Remembers whether there have been done changes to the attribute values without saving
    */
    attributeCategory.prototype.setAttributesChanged = function() {
        try {        	
            if (this.attributesChanged != true) {
                this.attributesChanged = true;
                var attributesChangedField = document.getElementById("attributesChangedField");
                if ( attributesChangedField) attributesChangedField.innerHTML = '<font color="#ff0000">' + ACatStrings.t('ACatAttributesChanged') + '</font>';
                var attributesChangedField2 = document.getElementById("attributesChangedField2");
                if ( attributesChangedField2) attributesChangedField2.innerHTML = '<font color="#ff0000">' + ACatStrings.t('ACatAttributesChanged') + '</font>';

                this.notifyChangeListeners();
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.setAttributesChanged: \n' + err);
        }
    };


    
    /*
    attributeCategory.prototype.loadList = function(restrictedObj) 
    Obsolete function. Code can be found in AttributeCategory_lastgood_20111010.js
    Code was never checked in.
    */
	


    ////////////////////////////////////////////////////////////////////////////////////////////
    // VALIDATION FUNCTIONS
    ////////////////////////////////////////////////////////////////////////////////////////////

    
    /**
     * This method MUST be called whenever an attribute category is about to be submitted.
     * It runs the validation and if it's OK (or user says it's OK) the save XML is generated for
     * all document reference attributes. Without this step document reference changes are not saved!
     * 
     * @param {Boolean} forceValidationOK - Flag determining how strict the validation is. Check
     * 		description of "force" parameter of attributeCategory.validateAttributes() for more details.
     * @param {Boolean} doDocumentNameCheck - Do a backend check on document names if true
	 *
	 * @returns It returns the result of the validation.
     */
    attributeCategory.prototype.prepareForSubmit = function(forceValidationOK, doDocumentNameCheck) {
        if (!this.loaded) {
            alert(ACatStrings.t('ACatCannotSave') + '\n' + ACatStrings.t('ACatNotLoaded'));		
            return false;
        }
        
        var validationOK = true;
        
        if (this.isValidationNeeded()) {
        	// validate the attribute category also if it's not explicitely set that the validation is not needed
        	// if there is a document reference 
        	validationOK = this.validateAttributes(forceValidationOK);
        }

        if (validationOK && doDocumentNameCheck) {
        	var evInst = EventBasedValidator.getInstance();
        	var uniqueNamesObj = evInst.createUniqueNamesObj();
        	evInst.validateNamesGlobally(uniqueNamesObj);
        	validationOK = uniqueNamesObj.isOk();
        }
        
        if (validationOK) {
	        // if the validation is OK or not required (or user said it's OK to save with issues),
        	// prepare document references in the attribute category to be submitted
			this.convertDocReferencesToXML();
			
			// set the state signalizing the attribute category is ready to submit (the submit is actually
			// in progress already at this point). it is used e.g. to control the "unsaved changes" warning
			// logic
			this.submitInProgress = true;
        }
        return validationOK;
    };
    
    
    /**
     * This function is called to check if any Approval is done and to validate the Password if required
	 * It's currently called when performing save on - Document Attributes, Project Attributes, 
	 * Full width view of TexContents and Attributes Widget.
	 * 
     * @param callback - Callback function to be called once password validation is completed
     * @param options - options for PasswordValidationPopup
     */
    attributeCategory.prototype.checkPasswordAndSave = function(callback, options) {
		if (this.attributes) {
			var docRowsInsertedPromises = [];
			
			// go over all attributes in this attr. cat. view and gather all documents tables promises
			this.executeOnAllDocReferences(function(docTableObj) {
				docRowsInsertedPromises.push(docTableObj.docRowsInsertedPromise);
			}, this);

			// check the passwords only if all documents tables are fully loaded
			Promise.all(docRowsInsertedPromises).then(function () {
				var isPasswordRequired= false;
				for (var i=0; i<this.attributes.length; i++) {	
					var attribute = this.attributes[i];
					if (attribute.docTable) {
						isPasswordRequired = attribute.docTable.checkIfPasswordRequired(callback);
					}
					if (isPasswordRequired) {
						break;
					}
				}
    			if (isPasswordRequired) {
        			// If password is required - open popup to get password from user, save otherwise
    				this.passwordValidationPopup = new PasswordValidationPopup(options);
    				this.passwordValidationPopup.onSubmit(callback);
    				this.passwordValidationPopup.open();
    			} else if (callback) {
    				callback();
    			}
			}.bind(this));
		} else {
			// there are no attributes, but the callback should be executed anyway...
			if (callback) {
				callback();
			}
		}
    };
    
    /***************************************************************
    * Validates the attributes versus required constraints.
    * @param {Boolean} force - Flag whether to allow (value false) or disallow (value true) the attribute
    * 		category to be saved if one or more required attributes are not filled.
    * @param {Object} validObj - Validation object holding all necessary information about found validation
    * 		problems together with the related data needed to notify user. 
    * 		It's structure is defined in this method.
    * @param {Array.<String>} ignoreList - Array of attribute IDs to skip during this validation.
    * @param {StaticDocumentRow or a subclass} curDocRowObj - Object representing the currently processed
    * 		document row when validating document reference attribute value. 
    * 		It's expected to be null if the currently validated attribute is not a document reference.
    * @param {int} depth - Depth of the validation recursion. It's > 1 only for document reference
    * 		attributes referencing documents with their own attribute categories...
	*
    * @return Returns true if all ok, false if some required fields are not filled out. It also 
    * 		highlights attribute values, which failed the validation.
    */
    attributeCategory.prototype.validateAttributes = function(force, validObj, ignoreList, curDocRowObj, depth) {
    	
    	var MAX_ERROR_MESSAGES = 20;
    	
        try {
            if (!this.loaded) {
                alert(ACatStrings.t('ACatCannotSave') + '\n' + ACatStrings.t('ACatNotLoaded'));		
                return false;
            }
        	
            if (!this.isValidationNeeded()) return true;

            if (!depth) depth = 1;
            
        	if (!validObj) {   // (at the root level)
        		// initialize the validation object ('validObj') in case of the root of the doc. ref. tree structure.
        		// object holding all found validation errors together with the related data needed to notify user.
        		validObj = {
        			isTemplate : this.isTemplate,   // support for special behavior in a template (context)
       	    		errors : [],   // array of error objects containing: 
       	    					   // 'level', 'element', docRowObj and 'msg' or 'attrLabel' members
    	    		isOK : function() {   // function returning true if no problem found during validation
    	    			return (this.errors.length == 0);
    	    		}, 
    	    		newDocs: [],
    	    		newDocsDOMElems: {}
    	    	};
        		
        		// clear all validation highlights potentially left there from the previous validation
        		$('#' + this.displayId + ' .validationHighlight').removeClass('validationHighlight');
        		_.each(this.attributes, function(attr){
        			attr.highlighted = false;
        		});
        		
        	}
        	
        	// this is for validation objects created outside the attribute category. Check for example
        	// EventBasedValidator.createUniqueNamesObj()... The EventBasedValidator doesn't know whether
        	// it's used in the template context or not, because it doesn't need that info. As soon as
        	// the validation object is passed to an attribute category, it needs to know the context
        	// in order to ignore missing values of required attributes (for templates) or report them.
        	if (validObj.isTemplate === undefined) {
        		validObj.isTemplate = this.isTemplate;
        	}
        	
        	// buttons validation (except "check unique" button) is handled differently, because they might
        	// required repaint of the whole attribute category
            var buttonErrorMsg = '';

            for (var row=0; row < this.nRows; row++) {
                for (var col=0;col < this.nCols; col++) {
                    var cellObj = this.cells[row][col];
                    if (cellObj) {
                    	// If the label is empty/null, show name, so no null check (or error message will look weird)
                    	var label = cellObj.label ? cellObj.label : cellObj.name; 
                    	
                        var attObj = cellObj.attObj;
                    	if (attObj) {  // if it's not an attribute, no need to check on being required
                        	if (attObj.id && ignoreList && (ignoreList.indexOf(attObj.id) > -1)) {
                        		// skip attributes from the "ignore list"
                        		continue;
                        	}
                        	
                        	var isRequired = (this.checkVisibility(cellObj) == 'Required');
                            if (isRequired && !validObj.isTemplate && !this.checkOnlyDocReferences()) {
                                // this cell's attribute is required and not in a template (context)
                            	
                                var requiredValMissing = false;
                                if (attObj.isDocReference()) {
                                	if (!attObj.docTable || (attObj.docTable.getNumberOfRows() == 0)) {
                                		requiredValMissing = true;
                                	}
                                } else if (!attObj.isCheckBox && cellObj.type != "Property" && (attObj.value == null || attObj.value == "")) { 
                                	requiredValMissing = true;
                                }
                                if (requiredValMissing) {
                                    // create an error object holding required info about the problem
                                	var errorObj = {
       		            				level : depth,
       		            				attribute: attObj,
                            			attrLabel : label,
                            			element : $(this.getAttributeInDOM(attObj)).add($(this.getAttributeInDOM(attObj)).closest(".validation-markup-element")),
                            			docRowObj : curDocRowObj
                                	};
    		            			validObj.errors.push(errorObj);
                                }
                            }
                            
                        	// special validation of document reference attribute values
                            if (attObj.isDocReference() && attObj.docTable) {
                            	// execute the recursive validation of document reference attribute values 
                            	// (documents tables content)
                            	attObj.docTable.validateDocTable(validObj, depth);
                            }
                        }

                        if (cellObj.buttons && (cellObj.buttons.length > 0) && !validObj.isTemplate 
                        		&& !this.checkOnlyDocReferences()) {
                            // if there are buttons in the current cell, check whether none of them 
                            // has an implicit validation action (by nature/defined)
                            for (var button = 0; button < cellObj.buttons.length; button++) {
                                var buttonObj = cellObj.buttons[button];
                                if (buttonObj.func.toLowerCase() == "checkuniquevalue") {
                                    // the uniqueness of this attribute value always has to be checked
                                    if (attObj && !this.validateAttributeUniqueValue(attObj, false)) {
                                    	// create an error object holding required info about the problem
                                    	var errMsg = ACatStrings.t('ACatUniqueValueCheckResult1') + label;
                                    	// alter error message based on if it's task specification or atribute
                                    	if (this.type && (this.type == this.typeOptions[2])) {
                                            // task specification case
                                        	errMsg += ACatStrings.t('ACatUniqueValueCheckResult2_spec');
                                        } else {
                                        	errMsg += ACatStrings.t('ACatUniqueValueCheckResult2');
                                        }
                                        // create an error object holding required info about the problem
                                    	var errorObj = {
                                    		isBlocking : true,
                                    		level : depth,
                                			msg : errMsg,
                                			attribute: attObj,
                                			element : $(this.getAttributeInDOM(attObj)).add($(this.getAttributeInDOM(attObj)).closest(".validation-markup-element")),
                                			docRowObj : curDocRowObj
                                    	};
                                        validObj.errors.push(errorObj);
                                    }

                                } else if (buttonObj.execOnSave) {
                                	// these button(s) errors are handled differently...
                                    var buttonErrObj = null;
                                    buttonErrObj = this.clickACatButton(buttonObj, null, 'onSave');	
                                    if (buttonErrObj && buttonErrObj.errorMsg ) buttonErrorMsg += '\nValidation Error on ' + buttonObj.label + ' : ' + buttonErrObj.errorMsg;
                                }
                            }
                        }
                        
                    }   // end of: if (cellObj) {...
                }
            }
            

            // handle buttons related errors with priority
            if (buttonErrorMsg) {
				alert(buttonErrorMsg);
				return false;
            }

            // main processing of the reported validation problems
        	if ((depth == 1)) {
        		var errorMsg = '';
        		var firstReportedIssueIdx = -1;

        		if(!validObj.isOK()) {
        		// if the validation is not OK and we are at the root level, run over 
        		// all the found problems and add them to the 'errorMsg' displayed
        		// as the result of the whole validation process. the blocking issues have priority
        		for (var passNum = 1; passNum < 3; passNum++) {
	        		for (var errIdx = 0; errIdx < validObj.errors.length; errIdx++) {
	        			var errObj = validObj.errors[errIdx];
	        			if (passNum == 1) {
	        				// in first pass skip all non-blocking issues
	        				if (!errObj.isBlocking) continue;
	            			// if a blocing problem found, make sure the user is not allowed to save the changes
	            			force = true;
	        			} else {
	        				// in second pass skip already processed blocking issues
	        				if (errObj.isBlocking) continue; 
	        			}
	        			// remember the index of the first reported problem
	        			if (firstReportedIssueIdx < 0) firstReportedIssueIdx = errIdx;
	        			
				    	// protection against too many validation error messages in the alert/confirm dialog
					    if (errIdx < MAX_ERROR_MESSAGES) {
		        			// add the current error message
		        			if (errObj.msg) {
		        				errorMsg += '\n' + errObj.msg;
		        			} else if (errObj.attrLabel) {
		        				errorMsg += '\n' + ACatStrings.t('ACatRequiredAttribute') + ' ' + errObj.attrLabel + ' ' + ACatStrings.t('ACatRequiredButNull');
		        			}
					    } else if (errIdx == MAX_ERROR_MESSAGES) {
		        			errorMsg += '\n...';
					    }
	        			if (errObj.element && errObj.element.length > 0 && errObj.element[0] !== undefined) {
						    // highlight the (jQuery) element in yellow						    
						    if(errObj.attribute){
						    	errObj.attribute.highlighted = true;
						    } else {
						    	errObj.element.addClass('validationHighlight');
						    }
	        			}
					    if (errObj.docRowObj) {
						    // make sure all attribute categories, where a validation problem was found are expanded
					    	// (applicable only for document reference attributes)
					    	errObj.docRowObj.expandBranchTillThis(errObj.level);
					    }
	        		}
        		}
        		
        		this.highlightAttributes();

                // focus on the first reported problem
	            if (firstReportedIssueIdx >= 0 && validObj.errors[firstReportedIssueIdx] && validObj.errors[firstReportedIssueIdx].element) {
                	validObj.errors[firstReportedIssueIdx].element.focus();
                }

                if (this.requestedView == 'Creation' || this.aCatOptions.projectCreationContext || force == true) {
               		alert(ACatStrings.t('ACatCannotSave') + '\n' + errorMsg);
                    return false;
                } else {
               		return confirm(ACatStrings.t('ACatConfirmSave') + '\n' + errorMsg);
                }
        	}
        	}
        	
        	// validation OK
        	return true;

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ " attributeCategory.validateAttributes: " + err.toString());
            return '';
        }
    };

    
    /**
     * Method returning attribute's label defined in this attribute category. 
     * If the label cannnot be found (is not defined etc.), the name of the attribute is returned. 
     * If the attribute doesn't exist empty string is returned.
     * @returns {String}
     */
    attributeCategory.prototype.getAttributeLabel = function(attObj) {
    	var label = '';
    	if (!attObj) return label;
    	
    	var foundCell = this.findCell(attObj);
    	if (foundCell) {
    		var cellObj = foundCell.cellObj;
    		label = (cellObj.label) ? cellObj.label : cellObj.name;
    	}
    	if (!label) label = attObj.name;
    	return label;
    };

    
    /**
     * Method returning attribute's HTML DOM object representing the rendered attribute value input/selector.
     * @param   {Attribute}  attObj        The attribute object, which DOM object should be found and returned.
     * @param   {boolean}   disableBackupStrategyLookup [Optional] disables looking up attribute element by id "prefix + attribute-id"
     * @returns {DOM object} The found DOM object of specified attribute or null if no attribute specified.
     */
    attributeCategory.prototype.getAttributeInDOM = function (attObj, disableBackupStrategyLookup) {
        var elem = null;
        if (attObj && attObj.id) {
            return this.getAttributeInDOMById(attObj.id, disableBackupStrategyLookup);
        }
        return null;
    };
    
    /**
     * Method returning attribute's HTML DOM object representing the rendered attribute value input/selector.
     * @param   {Attribute} attObjId                    Id of attribute
     * @param   {boolean} disableBackupStrategyLookup [Optional] If true, disables looking up attribute element by id "prefix + attribute-id"
     * @returns {DOM object} The found DOM object of specified attribute or null if no attribute specified.
     */
    attributeCategory.prototype.getAttributeInDOMById = function (attObjId, disableBackupStrategyLookup) {
        // Look up attribute element in map
        var mapByPrefix = this._attributeElementMap[this.prefix];
        if (mapByPrefix) {
            var mappedElement = mapByPrefix[attObjId];
            if (mappedElement) {
                return mappedElement;
            }
        }

        if (!disableBackupStrategyLookup) {
            // Map didn't contain attribute element, fallback to old method of fetching
            return document.getElementById(this.prefix + attObjId);
        }

        return null;
    };

    
    /**
     * Check if validation is needed
     * @return Returns true if validation is needed
     */
    attributeCategory.prototype.isValidationNeeded = function(doAdvancedChecks) {
        if (this.attributesChanged) {
            // if attributes have changed, return true immediately
            return true;		
        }
    	
    	if (this.aCatNotExpandedYet) {
    		// do not validate this attribute category if it has not been expanded (even once) yet
    		return false;
    	}
    	
        if (this.currentUser && !this.currentUser.canEditAttributes && !this.hasActiveViewDocReference()) {
            // no validation needed if not editable unless there is a document reference attribute
        	// within the attribute category!
            return false;		
        }
        
        if (doAdvancedChecks) {
    		// if no attributes have changed, check if there are JS button which should be called on save
    		return this.hasButtonActionOnSave();
        }
        
        // in all other cases the validation is needed!
        return true;
    };

    
    /**
     * Check if the validation needs to be propagated to the descendants == attribute categories of referenced
     * documents, even if this attribute category is read-only and/or not modified.
     * 
     * @return Returns true if validation neededs to be just propagated, not executed on the current attribute
     * 		category. Otherwise false is returned.
     */
    attributeCategory.prototype.checkOnlyDocReferences = function() {
    	if (this.currentUser && !this.currentUser.canEditAttributes && this.hasActiveViewDocReference()) {
            // the attribute category is readonly, but there is at least one document reference
    		// attribute within (the active view) so the validation needs to be propagated to
    		// descendants (if any) as the referenced documents can have an editable attribute category!
             return true;
        }
    	return false;
    };

    
    /**
     * Check if there are buttons to be executed on save
     * @return returns   true if there are buttons that need execution when saving
     */
    attributeCategory.prototype.hasButtonActionOnSave = function() {
         try {
            for (var row=0; row < this.nRows; row++) {
            	for (var col=0;col < this.nCols; col++) {
                	if (this.cells && this.cells[row]) {
                		var cellObj = this.cells[row][col];
                        if (cellObj && cellObj.buttons && cellObj.buttons.length > 0) {
                            for (var button = 0; button < cellObj.buttons.length; button++) {
                                var buttonObj = cellObj.buttons[button];
                                if (buttonObj.func.toLowerCase() == "checkuniquevalue" || buttonObj.execOnSave) {
                                    return true;
                                }
                            }
                        }
                	}
                 }
             }
             return false;
         } catch(err) {
             alert(ACatStrings.t('ACatProblemIn')+ " attributeCategory.hasButtonActionOnSave: " + err.toString());
             return false;
         }
    };


    /**
    * called as a part of attributes validation process
    * @param {attObj} attObj: The attribute to be checked.
    * @param {boolean} buttonClicked: TRUE if this method is called when user explicitly click 
    * 	"check unique value" button, FALSE otherwise.
    */
    attributeCategory.prototype.validateAttributeUniqueValue = function (attObj, buttonClicked) {
        "use strict";

        var retValue = false;
        try {
            var attName = attObj.name;
            var attValue = attObj.value;
            if (!attValue || attValue.length == 0) {
                if (buttonClicked) {
                    // report the problem here because it's a direct action of user (not like in validation)
                    alert(ACatStrings.t('ACatUniqueValueCheckNoValue'));
                    return false;
                } 
                else return true;   // empty values are ignored
            }
            if (!this.type || this.type.length == 0) {
                if (buttonClicked) {
                    // report the problem here because it's a direct action of user (not like in validation)
                    alert(ACatStrings.t('ACatUniqueValueCheckNoType'));
                    return false;
                }
                else return true;   // when we don't know what to check, don't block the whole save process
            }

                var url = window.webcenterConfig.jspBaseUrl + 'GetObjectsHavingAttValue.jsp';
            url += '?type=' + this.type;
            url += '&attribute=' + WCXML.urlencode(attName) + ':' + WCXML.urlencode(attValue);
            url += '&returntype=count';
            // find out what to exlude (if anything)
            var excludeID = "";
            if (this.type == this.typeOptions[0]) excludeID = this.projectID;  // project
            else if (this.type == this.typeOptions[1]) excludeID = this.documentID;  // document
            else if (this.type == this.typeOptions[2]) excludeID = this.taskID;  // task
            if (excludeID.length > 0) {
                url += '&excludeid=' + WCXML.urlencode(excludeID);
            }

            var aCatObj = this;

            AjaxCache.ajax({
                url: url,				
                type: "GET",
                dataType: "xml",
                cache: false,
                async: false,
                timeout: 120000
            }).done(function(returnXML) {	
                try {
                	/* Ajax call returns an XML like following:
                    <result>
                        <count>10</count>
                    </result>
                	 */
                    var mainElem = WCXML.GetFirstFirstLevelChild(returnXML, "result");
                    if (!mainElem) {
                        mainElem = WCXML.GetFirstFirstLevelChild(returnXML, "error");
                        if (mainElem) {
                            // SDK error returned
                            var sdkErrorCode = WCXML.GetElementSelfTextContent(WCXML.GetFirstFirstLevelChild(mainElem, "code"));
                            var sdkErrorMsg = WCXML.GetElementSelfTextContent(WCXML.GetFirstFirstLevelChild(mainElem, "message"));
                            alert("Server returned error\n\n" + sdkErrorCode + ": " + sdkErrorMsg);
                            return;
                        }
                    }
                    var countElem =  WCXML.GetFirstFirstLevelChild(mainElem, "count");
                    var count = parseInt(WCXML.GetElementSelfTextContent(countElem));
                    if (isNaN(count)) {
                        alert("Error processing return of ajax call to GetObjectsHavingAttValue.jsp: integer value expected as count");
                    } else {
                        if (buttonClicked) {
                            // explicit action of the user clicking the button - needs immediate feedback
                            if (count > 0) {
                                if (aCatObj.type == aCatObj.typeOptions[2]) {
                                    // alter error message for task specifications
                                    alert(ACatStrings.t('ACatUniqueValueCheckResult1') + attName + ACatStrings.t('ACatUniqueValueCheckResult2_spec'));
                                } else {
                                    // standard error for document and project attributes
                                    alert(ACatStrings.t('ACatUniqueValueCheckResult1') + attName + ACatStrings.t('ACatUniqueValueCheckResult2'));
                                }
                            } else {
                                if (aCatObj.type == aCatObj.typeOptions[2]) {
                                    // alter error message for task specifications
                                    alert(ACatStrings.t('ACatUniqueValueCheckResultOK_spec'));
                                } else {
                                    // standard error for document and project attributes
                                    alert(ACatStrings.t('ACatUniqueValueCheckResultOK'));
                                }
                            }
                        } else {
                            // just an implicit validation check - set the return value based on AJAX call result
                            retValue = (count == 0);
                        }
                    }
                } catch (err) {
                    alert("Error processing return of ajax call to GetObjectsHavingAttValue.jsp: " + err);
                }
            }).fail(function(errObj){
            	if(errObj.status !== 0){
            		alert("Error in Ajax call to GetObjectsHavingAttValue.jsp " + errObj.responseText);
            	}
            });
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.validateAttributeUniqueValue' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);
            return false;
        }
        return retValue;
    };


    ////////////////////////////////////////////////////////////////////////////////////////////
    // ONCHANGE FUNCTIONS
    ////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * A helper method to ease emiting of the "attribute value changed" event.
     * 
     * @param {Attribute object} attObj - Object representing the attribute, which value was changed.
     */
    attributeCategory.prototype.emitAttributevalueChanged = function(attObj) {
		AttributeEventBus.getInstance().emitAttrValueChanged(this.getUniqueAttrValID(attObj), attObj);
    };
    
    
    /**
    * Called when somebody changes the date of a date or time or a DateTime attribute
    * Updates the attObj.value field with the Date
    */
    attributeCategory.prototype.changeDateTime = function(row,col) {
        var mustRepaint = false;
        var attObj = null;
        try {  
            attObj = this.cells[row][col].attObj;

            if (attObj) {
                var attId = attObj.id;
                var dateField = document.getElementById(this.prefix + "longFormat_adate" + attId );
                var hourField= document.getElementById(this.prefix + "date_period_of_day_" + attId);

                if (dateField && hourField) {
                    var dateString = ConvertUtils.getEnglishFormattedDate(ACatStrings, dateField.value);

                    if (dateString) {
                        // update the attObj.value field. this field must be in JavaScript Date format
                        newDate = new Date(dateString);  // normalized date string is used to create a Date object.

                        if ( (newDate.constructor != Date) || isNaN(newDate.getTime()) ) {
                            // entered something which can't be interpreted as a date
                            // put back to old value
                            var attValue = attObj.value;
                            if (attValue.constructor == Date) {
                                dateField.value = WCHTML.getDateString(attValue);
                            } else {
                                dateField.value = "";
                                hourField.value = -1;
                            }

                        } else {    

                            // add hours and minutes
                            var halfhours = this.defaultHourAsHalfHours;
                            if (hourField.value != null && hourField.value!="") halfhours = parseInt(hourField.value);
                            if (halfhours == -1) {
                                halfhours = this.defaultHourAsHalfHours;
                                hourField.value = halfhours;
                            }

                            var hours = Math.floor(halfhours/2);
                            var minutes = (hours * 2 == halfhours)? 0:30;
                            newDate.setHours(hours); newDate.setMinutes(minutes);
                            attObj.value = newDate;

                            // update the hidden adate fields
                            attObj.processDateAttrValue();

                            attObj.changed = true;
                            this.setAttributesChanged();
                        }
                    } else { // this is not a date --> somebody emptied the field --> put it on "No date"
                        hourField.value = -1;
                        attObj.value = ""; 
                        attObj.hiddenDate = "";
                        // don't touch attObj.oldDate here!
                        mustRepaint = true;
                    }
                    var hiddenElem = document.getElementById(this.prefix + "adate" + attId );
                    if (hiddenElem) hiddenElem.value = attObj.hiddenDate;
                    hiddenElem = this.getAttributeInDOM(attObj);
                    if (hiddenElem) hiddenElem.value = attObj.hiddenDate;
                }
            }
            this.checkTaskSyncWarning(attObj);
            this.executeButtonDependencies(attObj);
            
            this.emitAttributevalueChanged(attObj);

        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.changeDateTime: " +err);
        }
        // always repaint
        if (mustRepaint) {
            this.displayAttributeCategory();
        } else {
            this.checkRepaintAttributeCategory(attObj);
        }
    };


    /**
    * Called when somebody changes a restricted set value.
    * Supports 2 cases: pure restricted sets and lists. 
    * Used in the attribute pages, but also in the search panels (when somebody changes a selection in a cascading dropdown in a search panel)
    * @param {DOM Object} elem DOM object of the changed element (mostly a HTML <select>)
    * @param {Integer} row row of the grid in which the element sits
    * @param {Integer} col column of the grid in which the element sits
    * @param {Boolean} radio true if this element is a radio button selector (with or without images), false if it's a dropdown
    */
    attributeCategory.prototype.changeSelect = function(elem,row,col,radio) {
        var attObj = null;
        var restrictedChanged = false;
        try { 
            if (!elem) throw("elem "+ ACatStrings.t('ACatNotDefinedProblem'));
            var cellObj = this.cells[row][col];
            attObj = cellObj.attObj;
            var newValue;
            if (radio) {
                var selected = parseInt($('input[name=' + this.prefix + 'ImgChk_' + attObj.id + ']:checked').attr("idx"),10);
                if (isNaN(selected)) {
                    newValue = '';
                } else {
                    if (attObj.rs) {
                        newValue = attObj.rs.values[selected].value;
                    } else if (attObj.columnObj) {
                        newValue = attObj.columnObj.values[selected];
                    }
                }
            } else {
                newValue = elem.value;	

            }

            if (attObj) {
                if (newValue == '$$ShowAll$$' || newValue == '$$More$$') {
                    if (newValue == '$$ShowAll$$') attObj.showAll = true;
                    if (newValue == '$$More$$') attObj.showMore = true;
                    restrictedChanged = true;
                    this.updateSearchDropdowns(attObj.listObj);
                } else {			
                    attObj.value = newValue;			
                    if (attObj.columnObj) {
                        restrictedChanged = true;
                        this.changeListSelection(attObj,newValue,elem,row,col);
                    } else if (attObj.rs) {
                        $('input[name=' + this.prefix + attObj.id + ']').val(newValue);
                    }
                    
                    this.setAttributesChanged();
                    attObj.changed = true;
                }
                this.checkTaskSyncWarning(attObj);
                this.executeButtonDependencies(attObj);
                
                this.emitAttributevalueChanged(attObj);
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.changeSelect: " +err);
        }

        if (restrictedChanged) {
            this.displayAttributeCategory();  // always repaint if this is a dropdown which is part of a list
        } else {
            this.checkRepaintAttributeCategory(attObj);   // only repaint if a rule or button is dependent on this cell
        }

    };


    /**
    * Called when changing the selection of a cascading dropdown field
    * @param newValue is simply the new value if the field is not multiSelect and is something like abc | def in case of a multiSelect field
    * @param elem The element on which the attribute is changed (normally a HTML <select>)
    * @param row row in which the changed attribute sits
    * @param col col in which the changed attribute sits
    * @return 
    * internally, column object currentIndex is used, which is encoded for multi value with ~, with indices and no spaces, so 3~123
    * Only used for loadingType 'Immediate' 
    * 
    */
    attributeCategory.prototype.changeListSelection = function(attObj,newValue,elem,row,col) {
        try {

            var listObj = attObj.listObj;
            var columnObj = attObj.columnObj;
            var newIndex=-1;
            if (listObj  && columnObj != null) {	

                if (listObj.loadingType != 'Immediate') throw new Error('changeListSelection called with a non-immediate loading list: '  + listObj.name);
                // Step 1: determine the new index. For multi-value this is something like 3~123

                if (newValue != "") newIndex =  this.findColumnIndex(columnObj,newValue);

                // deselect auto-selected dropdowns when needed
                if (!attObj.allowMultipleValues) {
                    if (columnObj.currentIndex != -1 && columnObj.currentIndex!= newIndex) {  // the latter is possible when changeSelect is called programmatorically
                        // We change a dropdown which already had a value --> deselect all auto-selected dropdowns
                        // This is not done for multi-value as there is often no need for it, but also because we don't remember by which ones the auto-selection was done. 
                        for (var col2=0; col2<listObj.columns.length; col2++) {
                            var colObj2 = listObj.columns[col2];
                            if (colObj2.autoSelected && colObj2.attObj != attObj) {  // second condition is to avoid that you deselect again what was just selected. With Show All, you can indeed change an autoselected field
                                colObj2.currentIndex = -1;

                                //colObj2.currentValue = "";
                                if (colObj2.attObj) {
                                    colObj2.attObj.value = "";
                                    this.emitAttributevalueChanged(colObj2.attObj);
                                }
                                colObj2.autoSelected = false;							
                            }
                        }
                    }
                }

                columnObj.currentIndex = newIndex;
                // invalidate all selected values in other dropdowns which are conflicting with the current column
                if (listObj.invalidateAutomatically) this.invalidateConflictingValues(listObj,columnObj);

                var success = this.updateListAttributeValues(listObj,newIndex==-1? false:true);  // do not do auto-fill when putting something back on please select, otherwise you risk filling yourself again
                if (!success && !attObj.allowMultipleValues && elem) {  // !elem can happen !!! see callers
                    // an invalid selection was done. Bring back the other dropdowns to please select
                    // HADW CR: no longer clear when this can happen. 
                    // since the dropdowns are limited to the valid values, this should not happen
                    // however it can happen anyway when first saving and when somebody changed the attribute category (for example adding extra columns or making a current column with already a value listen to a list)
                    this.deselectListAttributeValues(listObj);
                    elem.value = newValue; // this migth call ourselves...
                    this.changeSelect(elem,row,col);  // in case we are not called (for example on Firefox...)
                }


                columnObj.autoSelected = false;  // remember that this selection was done by hand

                this.updateSearchDropdowns(listObj);

                this.emitAttributevalueChanged(attObj);
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.changeListSelection: " +err);
        }

    };

    
    /** 
    * Invalidate all selected values in other dropdowns in listObj which are conflicting with the values in column columnObj
    * Assumes that columnObj already has the new value and the new index
    * Algorithm: run over all combinations. For all ones which are valid given current value of the column, put the other columns' entries to valid
    * then look whether these other column's values are valid
    * Only used for loadingType 'Immediate', not for 'OnTheFly' 
    */
    attributeCategory.prototype.invalidateConflictingValues = function(listObj,columnObj) {
        try {
            var combiArr = listObj.combinations;
            if (!combiArr) return;  // typically means that the ajax call for this list did not yet return (or failed). 
            var col;
            var ncols = listObj.columns.length;

            // determine current column 
            var curCol = -1;
            for (var curCol=0; curCol< ncols; curCol++) {
                if (listObj.columns[curCol] == columnObj) break;
            }
            if (curCol>=ncols) return;
            var curColIndex = listObj.columns[curCol].currentIndex;
            if (curColIndex == -1) return;  // no need to invalidate anything if there is no value
            var curColIndexArr = null;
            if (curColIndex.constructor == String) curColIndexArr = curColIndex.split('~');



            // initialize a local valid array (which is different from the one we remember because we want to see what is valid considering only columnObj
            var lValidArr = new Array();
            for (var col=0; col< ncols; col++) {				
                lValidArr[col] = new Array();

                for (var idx=0; idx<listObj.columns[col].values.length; idx++) {
                    lValidArr[col][idx]=0;				
                }
            }

            // run over all combinations and make values in lValidArr true (=valid) based on the value in the combination
            for (var combi=0; combi < combiArr.length; combi++) {
                var combination = combiArr[combi];
                var indexArr = combination.split("~");
                var combiValid = false;

                // look whether this combination is valid.
                // it is valid as soon as the value of this combination for the set column is one of the values for this column
                var combiIndexForCurCol = indexArr[curCol];
                for (col=0; col< ncols; col++) indexArr[col] = parseInt(indexArr[col],10);

                if (curColIndexArr) {	// current integer is a string with potentially multiple values
                    for (var idx=0; idx<curColIndexArr.length; idx++) {
                        if (curColIndexArr[idx] == combiIndexForCurCol) {combiValid = true; break;}
                    }
                } else {  // current index is just an integer
                    if (curColIndex == combiIndexForCurCol) combiValid = true;  
                }

                // when the combination is valid, validate the according values in lValidArr
                if (combiValid) {
                    for (var col=0; col< ncols; col++) lValidArr[col][indexArr[col]]=true;	
                }

            }	

            //pick up the indices for each of the columns.  
            var currentArr = new Array();
            for (col=0; col<ncols; col++) {
                if (col != curCol) {
                    var columnObj = listObj.columns[col];
                    var colIndex = columnObj.currentIndex;
                    if (colIndex == -1) continue;  // never invalidate what is not there
                    var colIndexArr = null;
                    var newIndex = '', newValue='';
                    if (colIndex.constructor == String) {
                        colIndexArr = colIndex.split('~');

                        for (var idx=0; idx<colIndexArr.length; idx++) {  // build up a new encoded index with only the allowed values
                            if (lValidArr[col][parseInt(colIndexArr[idx],10)]) {
                                if (newIndex) newIndex += '~';
                                newIndex += colIndexArr[idx];
                                if (newValue) newValue += this.multiSep;
                                newValue += columnObj.values[colIndexArr[idx]];
                            }
                            if (newIndex != columnObj.currentIndex) {
                                columnObj.currentIndex = newIndex? newIndex : -1;
                                if (columnObj.attObj) columnObj.attObj.value = newValue;
                            }
                        }
                    } else {
                        if (!lValidArr[col][colIndex]) {
                            columnObj.currentIndex = -1; columnObj.autoSelected = false;
                            if (columnObj.attObj) columnObj.attObj.value = '';

                        };
                    }



                }
            }		
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.invalidateConflictingValues: " +err);
        }

    };


    // update dropdowns for affected columns in case of search results panel 
    attributeCategory.prototype.updateSearchDropdowns = function(listObj) {
        try {
           for (var col2=0; col2<listObj.columns.length; col2++) {
                var colObj2 = listObj.columns[col2];
                var attObj2 = colObj2.attObj;

                if (attObj2) {
                    var parentElem = this.getAttributeInDOM(attObj2, true);
                    if (parentElem) {
                        // you need to update the dropdown for the search panel
                        // for that we need the position in the cell array (renderRestrictedList needs this)
                        var row3,col3;
                        for (row3=0; row3<this.cells.length; row3++) {
                            var row3Arr = this.cells[row3];
                            if (row3Arr) {
                                for (col3=0; col3<row3Arr.length; col3++) {
                                    var cellObj = row3Arr[col3];
                                    if (cellObj && cellObj.attObj == attObj2) {
                                        $(parentElem).html(this.renderRestrictedList(this.cells[row3][col3],row3,col3,true));
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.updateSearchDropdowns: " +err);
        }

    };

    /**
     * Notify listeners that a value was updated from an external source (e.g. ProjectAttributes)
     */
    attributeCategory.prototype.notifyChangeListeners = function() {
        _.each(this.valueChangedListeners, function(listener){
            listener.call();
        }.bind(this));
    };

    /**
    * Called when blurring an input field (text but also  float etc)
    * Introduced to solve JIRA issue WCR-5275: text selected from browser's autocomplete not remembered correctly on some browsers (Safari on Mac)
    * blurText is called AFTER changeText if changeText happens. If not, only blurText is called and then it calls changeText to force the change
    */
    attributeCategory.prototype.blurText = function(elem,row,col) {
        'use strict';

        var attObj = null;
        var valueChanged = false;	

        try {    
            if (this.cells[row] && this.cells[row][col]) attObj = this.cells[row][col].attObj;
            var newValue = elem.value;
            if (attObj && attObj.value != newValue) {
                // this will call changeText multiple times in case changeText changes the value (f.e. normalization of numeric input)
                // this is considered harmless
                this.changeText(elem,row,col);
            }

        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.blurText: " +err);		

        }

    };

    /**
    * Called when somebody changes a single line or multiline text
    */
    attributeCategory.prototype.changeText = function(elem,row,col) {
        var attObj = null;
        var valueChanged = false;	

        try {    
            if (!elem) throw("elem "+ ACatStrings.t('ACatNotDefinedProblem'));
            this.setAttributesChanged();
            var newValue = elem.value;

            if (!attObj && this.cells[row] && this.cells[row][col]) attObj = this.cells[row][col].attObj;

            if(attObj && attObj.type){

                // check floating point and integer types
                switch (attObj.type.toLowerCase()) {
                    case 'richtext':
                        newValue = $(elem).summernote("code");
                        elem.value = newValue;
                        break;
                    case 'text':
                    case 'datetime':
                        break;// do nothing
                    case 'integer':
                    case 'number':
                        WCHTML.checkNumericInput(elem,"Integer",false,attObj.value);
                        break;
                    case 'area':
                    case 'area1':
                    case 'area2':
                    case 'volume':
                    case 'volume1':
                    case 'volume2':
                    case 'weight':
                    case 'weight1':
                    case 'float':
                        WCHTML.checkNumericInput(elem,null,false,attObj.value);
                        break;
                    case 'distance': 
                    case 'distance1':
                        if(attObj.fractionalIfImperial  && this.userUnits && this.userUnits.toLowerCase() == "imperial") {
                            newValue = WCHTML.fractionalToDecimal(newValue);
                            var fracValue = '';
                            if(isNaN(newValue)){
                                newValue = '';
                            }else{
                                fracValue = WCHTML.decimalToFractional(newValue);
                            }
                            elem2 = elem;
                            elem = this.getAttributeInDOM(attObj);
                            elem.value = newValue;
                            elem2.value = fracValue;
                        } else {
                            WCHTML.checkNumericInput(elem,null,false,attObj.value);
                        }
                        break;
                    default:				
                        break;
                } // end switch

                if (attObj.value !== elem.value) valueChanged = true;
                attObj.value = elem.value;
                // CR HADW: this looks contradictory to valueChanged however it is not really. 
                // changeText can be called more than once for 1 change. The second time valueChanged 
                // will be false (since the first call already did the job)
                // Still if you get here, you are actually sure that the value has changed (otherwise 
                // changeText should not be called at all). 
                // valueChanged is needed to avoid calling the checkAutoSelectLazyLoading too many times 
                // (each time is a server call).
                if (attObj.listObj && attObj.listObj.loadingType == 'OnTheFly' && valueChanged) {
                    this.checkAutoSelectLazyLoading(attObj,false);  // auto-selects single remaining				
                }
                attObj.changed = true;  

                this.checkTaskSyncWarning(attObj);     
            }
            this.executeButtonDependencies(attObj);

            this.emitAttributevalueChanged(attObj);
        } catch (err) {
            if (err.name != "NS_ERROR_UNEXPECTED") {  // DT112225: ugly message shown for tiny_mce, also when no good reason
                alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.changeText: " +err);
            }

        }

        if (attObj) this.checkRepaintAttributeCategory(attObj);
    };


    /**
     * Method to generate a unique identifier of an attribute VALUE (not attribute) assigned to a particular
     * object in WebCenter. It's constructed in a way it's unique for different attribute values and 
     * the same for the same attribute value even if it's part of different instances of the same category.
     * It's used to identify the "very same" attribute values spread over a page to keep their values in sync!
     * 
     * @param {Attribute} attObj - The attribute for which the attribute value unique ID should be generated.
     * 		Note the second part of the info is taken from 'this' attribute category object so it's important
     * 		in which context this method is called!
	 * @returns - Unique identifier of given attribute value assigned to given WebCenter object.
     */
    attributeCategory.prototype.getUniqueAttrValID = function(attObj) {
    	var result = "";
    	if (attObj) {
    		// note the order of if-else branches below cannot be changed without breaking the functionality!
            if (this._disableSynchronizingAttributes) {
            	// when the synchronization is disabled make sure the generated ID is never the same
                result = "_" + _.uniqueId();
            } else if (this.aCatOptions.isDocRefPropertiesACat) {
    			// for doc. ref. properties - the ID needs to be extended by reference ID
    			//JANH TODO there is actually nothing right now that can identify the very same reference property value!
    			result = "refAttrUID:" + _.uniqueId();
    		} else if (this.parentDocRow && this.parentDocRow.newDocTmpID) {
	    		result = "newDocUID:" + this.parentDocRow.newDocTmpID;
    		} else if (this.prefix && (this.prefix.indexOf("BOMDoc_") == 0 || this.prefix.indexOf("docBOM") == 0)) {
    			// this is attribute category within BOM section!
    			if (this.aCatOptions.newDocTmpID) {
    				// if it's a newly added document the 'this.aCatOptions.newDocTmpID' should be set
    				result = "newDocUID:" + this.aCatOptions.newDocTmpID
    			} else {
    				// otherwise 'this.docVersionID' of the existing document is set from the BOM section.
    				result = "docVerAttrUID:" + this.docVersionID;
    			}
    		} else if (this.docVersionID) {
	    		result = "docVerAttrUID:" + this.docVersionID;
	    	} else if (this.taskID) {
	    		result = "taskAttrUID:" + this.taskID;
	    	} else if (this.projectID) {
	    		result = "projectAttrUID:" + this.projectID;
	    	} else {
	    		if (!this.layoutId) {
	    			DocTableXUtils.error("A very lonely attribute found " + attObj.name);   //JANH debug - then DEL!!!
		    	}
	    	}
	    	result +=  "_attr:" + attObj.id;
    	}
    	return result;
    };

    
    /**
     * Method to generate a unique identifier of an attribute category instance assigned to a particular
     * object in WebCenter. It's constructed in a way it's unique for different category instances and 
     * the same for the same attribute category assigned to the same object even if it's part of different 
     * instance of the object (reference to document for example).
     * It's used to identify the "very same" attribute category instances spread over a page to keep their 
     * attribute values in sync!
     * 
	 * @returns - Unique identifier of this attribute category assigned to given WebCenter object.
     */
    attributeCategory.prototype.getUniqueCategoryID = function() {
    	var result = "";
		// note the order of if-else branches below cannot be changed without breaking the functionality!
        if (this._disableSynchronizingAttributes) {
        	// when the synchronization is disabled make sure the generated ID is never the same
            result = "_" + _.uniqueId();
        } else if (this.aCatOptions.isDocRefPropertiesACat) {
			// for doc. ref. properties - the ID needs to be extended by reference ID
			//JANH TODO there is actually nothing right now that can identify the very same reference property value!
			result = "refACatUID:" + _.uniqueId();
		} else if (this.parentDocRow && this.parentDocRow.newDocTmpID) {
    		result = "newDocACatUID:" + this.parentDocRow.newDocTmpID;
		} else if (this.prefix && (this.prefix.indexOf("BOMDoc_") == 0 || this.prefix.indexOf("docBOM") == 0)) {
			// this is attribute category within BOM section!
			if (this.aCatOptions.newDocTmpID) {
				// if it's a newly added document the 'this.aCatOptions.newDocTmpID' should be set
				result = "newDocACatUID:" + this.aCatOptions.newDocTmpID
			} else {
				// otherwise 'this.docVersionID' of the existing document is set from the BOM section.
				result = "docVerACatUID:" + this.docVersionID;
			}
		} else if (this.docVersionID) {
			result = "docVerACatUID:" + this.docVersionID;
    	} else if (this.taskID) {
    		result = "taskACatUID:" + this.taskID;
    	} else if (this.projectID) {
    		result = "projACatUID:" + this.projectID;
    	} else {
    		if (!this.layoutId) {
    			DocTableXUtils.error("Not assigned attribute category:" + this.name);   //JANH debug - then DEL!!!
	    	}
    	}
    	return result;
    };

    
    /**
     * Event handler dedicated to synchronize values of multiple instances of a single attribute value
     * when it's changed on one place.
     * It's based on AttributeEventBus, which uses WCEvent.js as event framework.
     * It synchronizes all attributes across any single page even if the attributes are spread over multiple
     * independent sections / instances of attribute category (like on Project Creation page for example).
     * 
     * IMPORTANT: It doesn't support reference property values in the first version!
     * 
     * @param {String} srcID - Unique identifier of attribute value that was changed == sent the event.
     * @param {Attribute} srcAttObj - object representing the changed attribute (initiator) itself.
     */
    attributeCategory.prototype.onChangeAttrValueEventHandler = function(srcID, srcAttObj) {
		if (!srcID || !srcAttObj) {
			return;   // nothing to do if the source of the event is not known
		}
		// go over all attributes in this attribute category and update "shared" attribute values
        for (var row = 0; row < this.nRows; row++) {
            for (var col = 0; col < this.nCols; col++) {
                if (this.cells[row] && this.cells[row][col]) {
                    var cellObj = this.cells[row][col];
                    if (cellObj.attObj) {
                        var attObj = cellObj.attObj;
                        if ((srcID == this.getUniqueAttrValID(attObj)) && !attObj.isSameInstanceAs(srcAttObj)) {
                        	// another "instance" of the same attribute value found in this attribute category - update it
	        				if (!attObj.isSameType(srcAttObj)) {
	        					alert("Attribute types are not the same in 'onChangeAttrValueEventHandler()' event listener!");
	        					continue;
	        				}

	        				// stop emitting "on change" event for the implicitely modified attributes to prevent infinite loops
        					attObj.setProcessingOnChangeEvent(true);

        					if (attObj.isDocReference()) {
	                        	if (attObj.docTable && srcAttObj.docTable) {
	                                (function(attObj, srcAttObj) {   // closure for attObj and srcAttObj objects
	        	                    	// continue only if the particular document row is fully loaded
	                                	attObj.docTable.docRowsInsertedPromise.then(function () {
		        							// set the attObj.docTable documents table to "modified elsewhere" mode
		        	        				// it goes to read only mode and mirror all the changes.
		        							attObj.docTable.modifiedElsewhere(srcAttObj.docTable, false);
		
		        							// perform the synchronization of the attObj.docTable and srcAttObj.docTable:
		        							// empty the documents table before copying document rows from the source attribute
		        	        				attObj.docTable.reset(true);
		        	        				// copy/clone all document rows from srcAttObj.docTable to attObj.docTable
		        	        				srcAttObj.docTable.copyDocumentRowsTo(attObj.docTable, true).then(function() {
		        	        					// (copying of document rows is as well an asynchronous operation)
		        	        					this.onChangeAttValue(attObj);
		        	        					attObj.setProcessingOnChangeEvent(false);
		        	        				}.bind(this));
	        	                        }.bind(this));
	                                }.bind(this))(attObj, srcAttObj);
        						} else {
    	        					// document reference attribute without DocTableX instance initialized
    	        					attObj.setValue(srcAttObj.value, srcAttObj.valueHyperlink);
            						this.onChangeAttValue(attObj);
    	        					attObj.setProcessingOnChangeEvent(false);
        						}
	        				} else {
	        					// other than document reference attribute types
	        					if (attObj.setValue(srcAttObj.value)) {
		        					this.onChangeAttValue(attObj);
		        					// "is changed" flag etc. is set within onChangeAttValue() method or methods it calls
	        					}
	        					attObj.setProcessingOnChangeEvent(false);
	        				}
                        }
                    }
                }
            }
        }

	};
	
	
    /**
     * Event handler dedicated to update values of attributes in a newly initialized attribute category
     * if there is already the same category assigned to the very same project/document/task on the page
     * and such an attribute category was already modified by the user (so what's on the page is not what's
     * stored in DB at that moment).
     * It's based on AttributeEventBus, which uses WCEvent.js to manage events.
     * It copies values of all attributes from 'this' category (which is the already existing one) to
     * the 'srcACatObj' category, which is the just created/initialized category.
     * It works across any single page even if the categories are spread over multiple independent 
     * sections / instances of attribute category (like on Project Creation page for example).
     * 
     * IMPORTANT: It doesn't support reference property category values in the first version!
     * 
     * @param {attributeCategory} srcACatObj - object representing the new (just instantiated and loaded) attribute category.
     */
    attributeCategory.prototype.onCategoryLoadedEventHandler = function(srcACatObj) {
		if (this.attributesChanged && srcACatObj && (this.prefix !== srcACatObj.prefix) && !srcACatObj._onCategoryLoadedEventProcessed 
				&& (srcACatObj.getUniqueCategoryID() == this.getUniqueCategoryID())) {
			srcACatObj._onCategoryLoadedEventProcessed = true;
			
			// copy all the attribute values from 'this' attribute catogory if the new attribute category (source of the event)
			// is the same category assigned to the same WebCenter object (project, document or task)
			srcACatObj.copyAttrValuesFromCategory(this);
			// repaint the attribute category after all attribute values are updated
			srcACatObj.displayAttributeCategory();
		}
	};

	

    /**
     * This method copies all attribute values from specified attribute category 'srcACatObj' to this attribute category.
     * Note it is ment to copy/clone values among different instances of the same attribute category, but 
     * there is no conceptual problem to use it with instances of different categories...
     * 
     * @param srcACatObj {attributeCategory} - Attribute category instance from which the values should be copied/cloned.
     */
    attributeCategory.prototype.copyAttrValuesFromCategory = function(srcACatObj) {
        if (srcACatObj && srcACatObj.attributes && this.attributes) {
        	if (srcACatObj.attributes.length != this.attributes.length) {
        		alert("Attribute values cloning: Source and destination attribute categories have different count of attributes");   //JANH debug - then DEL!!!
        		return;
        	}
        	// go over all attributes in this attribute category and overwrite their values from the source category
        	var isModified = false;
        	// array of promises governing loading and initialization of document rows + their adding 
        	// to documents table
        	var docRowsInsertedPromises = [];
        	// array of promisses governing copying of document rows from source to destination documents able.
        	var docRowsCopiedPromisses = [];

        	for (var i = 0; i < this.attributes.length; i++) {
        		var attObj = this.attributes[i];
        		if (attObj && attObj.id) {
    				var srcAttObj = srcACatObj.findAttribute(attObj.id);
    				if (srcAttObj) {
    					
        				// stop emitting "on change" event for the implicitely modified attributes to prevent infinite loops
    					attObj.setProcessingOnChangeEvent(true);

	        			if (attObj.isDocReference()) {
	        				if (attObj.docTable && srcAttObj.docTable) {
		        				// store aside all the documents tables promises to be able to synchronize
		        				// the final action(s) (based on 'isModified' flag)
		        				docRowsInsertedPromises.push(attObj.docTable.docRowsInsertedPromise);

		        				(function(attObj, srcAttObj) {   // closure for attObj and srcAttObj objects
			                    	// continue only if the particular document row is fully loaded
			                        attObj.docTable.docRowsInsertedPromise.then(function () {
			        					if (srcAttObj.docTable.getIsChanged() && (srcACatObj.getUniqueCategoryID() == this.getUniqueCategoryID())) {
				        					// set the attObj.docTable documents table to "modified elsewhere" mode
			    	        				// it goes to read only mode and mirror all the changes.
				        					attObj.docTable.modifiedElsewhere(srcAttObj.docTable, false);
				        				}
				        				// empty the documents table before copying document rows from the source attribute
				        				attObj.docTable.reset(true);
				        				// copy/clone all document rows from srcAttObj.docTable to attObj.docTable
				        				docRowsCopiedPromisses.push(srcAttObj.docTable.copyDocumentRowsTo(attObj.docTable, false).then(function () {
				        					isModified = true;
				        					attObj.setProcessingOnChangeEvent(false);
				        				}.bind(this)));
			                        }.bind(this));
		        				}.bind(this))(attObj, srcAttObj);
	        				} else {
	        					// document reference attribute without DocTableX instance initialized
	        					if (attObj.setValue(srcAttObj.value, srcAttObj.valueHyperlink)) {
	        						isModified = true;
	        					}
	        					attObj.setProcessingOnChangeEvent(false);

	        				}
	        			} else {
	        				// all standard types of attributes
        					if (attObj.setValue(srcAttObj.value)) {
        						isModified = true;
        					}
        					attObj.setProcessingOnChangeEvent(false);
	        			}
        			} else {
    					alert("Attribute values cloning: Attribute " + attObj.name + " not found in " + srcACatObj.name + " source attribute category");   //JANH debug - then DEL!!!
    					return;
        			}
        		} else {
        			alert("Attribute values cloning: Not initialized attribute found in " + this.name + " destination attribute category");   //JANH debug - then DEL!!!
        			return;
        		}
        	}
        	// perform the last step(s) only after all DocTableX stuff is loaded/inserted and copied
        	var allPromises = docRowsInsertedPromises.concat(docRowsCopiedPromisses);
        	Promise.all(allPromises).then(function () {
	        	if (isModified) {
	            	// set this attribute category to be modified as at least one attribute value was changed...
	        		this.setAttributesChanged();
	        		this.displayAttributeCategory();
	        	}
        	}.bind(this));
        }
    };

    
    
    
    /**
     * Checks if the warning flag for synchronizing project attributes with task specifications should be set
     */
    attributeCategory.prototype.checkTaskSyncWarning = function(attObj){
        var isDateAndNotRS = ((attObj.type && attObj.type.toLowerCase() == 'datetime') && !attObj.rs);

        // may have to place warning flag if specification will alter existing project attribute value
        if(this.autoUpdateProjAttrsCheckbox){
            var warning = document.getElementById("imgwarning"+attObj.id);
            if (warning) {
                if (attObj.isDocReference() && attObj.projDocRefs) {
                    // for document reference attribute compare the content of documents table
                    if (attObj.docTable && !attObj.docTable.hasSameContentAs(attObj.projDocRefs.docRows)) {
                        warning.style.display = "inline";
                        warning.style.visibility = "visible";
                    }
                } else if((isDateAndNotRS && (attObj.value.toString() != attObj.projAttValue.toString())) || 
                        (!isDateAndNotRS && (attObj.value != attObj.projAttValue))){
                    warning.style.display = "inline";
                    warning.style.visibility = "visible";
                } else {
                    warning.style.display = "none";
                    warning.style.visibility = "hidden";
                }
            }
        }
	};


    /**
    * Updates property validArr in all attributes coupled to list represented by listObj
    * Algorithm: Look into all combinations and mark according indices to be valid 
    * validArr[idx]==0 means not valid, 1 means always valid, 2 means valid because what makes it invalid is a selection on either the same column or a column further downstream in a hierarchical set
    * 3 means that the value is currently selected
    * @param doAutoSelect {Boolean} determines whether elements will be autoselected. Autoselection is normally only done after a change of one of the attributes by the user, not when loading from the database 
    * We want to guarantee that what sits in the database is also shown. 
    * @return {Boolean} false if an invalid selection was made, true in all other circumstances (including that you can't do the work because lists are not yet loaded)
    */
    attributeCategory.prototype.updateListAttributeValues = function(listObj,doAutoSelect) {
        try {

            var combiArr = listObj.combinations;
            if (!combiArr) return true;  // typically means that the ajax call for this list did not yet return (or failed). 
            var col;
            var ncols = listObj.columns.length;
            var lastColWithValue = -1;

            //pick up the indices for each of the columns.  
            var currentArr = new Array();
            for (col=0; col<ncols; col++) {
                if (listObj.columns[col].autoSelected) {
                    currentArr.push(-1);	// effect is that autoselected columns don't restrict other columns. 		
                } else {
                    currentArr.push(listObj.columns[col].currentIndex);			
                }
            }

            // reset all valid arrays (colObj.validArr) to 0
            for (var col=0; col< ncols; col++) {				
                var colObj = listObj.columns[col];
                var columnLength = colObj.values.length;
                if (!colObj.validArr) colObj.validArr = new Array(columnLength);
                for (var idx=0; idx<columnLength; idx++) {
                    colObj.validArr[idx]=0;				
                }
                if (colObj.currentIndex.constructor == String) {
                    var selectedArr = colObj.currentIndex.split('~');
                    for (var selected=0; selected<selectedArr.length; selected++) {
                        colObj.validArr[selectedArr[selected]]=3;  // whatever is selected is set to 3
                    }
                }
                if (colObj.currentIndex!=-1) lastColWithValue = col;
            }

            // run over all combinations and make values in validArr true (=valid) based on the value in the combination
            for (var combi=0; combi < combiArr.length; combi++) {
                var combination = combiArr[combi];
                var indexArr = combination.split("~");
                for (col=0; col< ncols; col++) indexArr[col] = parseInt(indexArr[col],10);


                var mismatches = 0;  // Count number of mismatches for this combination.  A mismatch is an index for a column which is not currently selected in a column which has a selection
                var colMismatch = new Array();
                var firstMismatch = -1;
                for (var col=0; col< ncols; col++) {colMismatch[col]=false;}

                for (var col=0; col< ncols; col++) {	
                    if (currentArr[col] == -1) continue;  // column has no value or is not in use --> never invalidate a combination for this

                    if (currentArr[col].constructor != String || currentArr[col].indexOf("~") < 0) {	 // single value selected for this column
                        if (currentArr[col]!=indexArr[col]) {
                            mismatches++;colMismatch[col]=true;
                            if (mismatches ==1) firstMismatch=col;
                        };
                    } else {
                        // multiple values are selected for this column
                        var currentArr2 = currentArr[col].split('~');
                        var found = false;
                        for (var idx2=0; idx2<currentArr2.length; idx2++) {
                            if (currentArr2[idx2]==indexArr[col]) {found=true; break;};
                        }
                        if (!found) {
                            mismatches++;colMismatch[col]=true;
                            if (mismatches ==1) firstMismatch=col;
                        };
                    }
                }



                for (var col=0; col< ncols; col++) {
                    var idx = indexArr[col];
                    var colObj = listObj.columns[col];

                    if (colObj.validArr[idx]==1 || colObj.validArr[idx]==3) continue; // it's already fully valid, no reason to find more proof

                    if (currentArr[col] != -1 && colMismatch[col]) {
                        // This column is already filled out with a different value than the one of this combination
                        // If that is the only mismatch, or in the case of hierarchical lists, the first mismatch, you still need to allow changing this dropdown
                        // So set this combination's validity for this column ONLY to true

                        if (listObj.hierarchical) {
                            if (col == firstMismatch) colObj.validArr[idx]=2;	
                        } else {
                            if (mismatches <2) colObj.validArr[idx]=2;							
                        }

                    } else {  
                        // this column is not filled out or has the value of this combination
                        // If there are no mismatches, allow all column's validity
                        if (mismatches == 0) {
                            colObj.validArr[idx] = 1;
                        } else if (listObj.hierarchical && col<firstMismatch) {
                            colObj.validArr[idx]= 2;
                        }
                    }

                }

            }

            // autoselect single remaining 		
            for (var col=0; col< ncols; col++) {
                var nValid = 0;
                var colObj =  listObj.columns[col];			
                var columnLength = colObj.values.length;
                var validIdx = -1;	
                for (var idx=0; idx<columnLength; idx++) {
                    if (colObj.validArr[idx] ) {nValid++; validIdx = idx;};
                }
                colObj.nValid = nValid;
                if (nValid ==1 && colObj.currentIndex == -1) {  // only auto-fill what is not filled yet
                    if (doAutoSelect || listObj.forceValueUpdate) {
                        colObj.currentIndex = validIdx;				
                        var newValue = colObj.values[validIdx];
                        //colObj.currentValue = newValue;
                        var attObj = colObj.attObj;
                        if (attObj){
                            attObj.value = newValue;
                            this.emitAttributevalueChanged(attObj);
                        }			
                        colObj.autoSelected = true;  // Remember what was automatically selected. 
                    }
                } else if (nValid==0 && colObj.currentIndex != -1){
                    // something was selected which makes nothing in this column still valid 
                    // DT102015. Fix 22 March: also check whether something was there in the first place - check on currentIndex !=-1
                    // If you don't do this, you will fail any selection for which no values exist in another column
                    // this includes "irrelevant columns". 
                    // Worse: you risk an endless loop because return false will force another call to updateListAttributeValues...
                    return false;
                }

            }

            // backwards autoselect for hierarchical	
            if (listObj.hierarchical && (doAutoSelect || listObj.forceValueUpdate)) {
                for (var col=0; col< lastColWithValue; col++) {
                    var colObj = listObj.columns[col];
                    var attObj = colObj.attObj;
                    if (attObj && attObj.allowMultipleValues) {
                        // a further column in a hierarchical set has a value. Select everything which is allowed for that value (this risks some funny effects if the data is not really hierarchial...)
                        var newValue = attObj.value;
                        var newIndex = colObj.currentIndex;
                        if (newIndex == -1)  newIndex = '';
                        var columnLength = colObj.values.length;
                        for (var idx=0; idx<columnLength; idx++) {
                            if (colObj.validArr[idx]==1 ) {
                                if (newValue) newValue += ' | ';
                                newValue += colObj.values[idx];
                                if (newIndex) newIndex += '~';
                                newIndex += idx;					
                            }			
                        }
                        // TO DO check on maximum size
                        colObj.currentIndex = newIndex;
                        if (newIndex == '') colObj.currentIndex = -1;

                        attObj = colObj.attObj;
                        if (attObj){
                             attObj.value = newValue;
                            this.emitAttributevalueChanged(attObj);
                        }
                        // TO DO: check on need to update hidden fields (calling changeText)
                    }
                }
            }



            // autoselect unique identifyer column
            var uniqueIDCol = listObj.uniqueIdentifyerColumn;
            if (uniqueIDCol != -1 && uniqueIDCol !== ""  && listObj.columns[uniqueIDCol]) {
                // check whether all other columns have values
                for (col=0; col<listObj.columns.length; col++) {
                    if (col==uniqueIDCol) continue;
                    if (listObj.columns[col].currentIndex == -1) break;
                }
                if (col==listObj.columns.length) {
                    // all other columns have values: select all valid values for the unique column
                    var columnObj = listObj.columns[uniqueIDCol];
                    var attObj = columnObj.attObj;
                    if (attObj) {
                        var validArr = columnObj.validArr;
                        var valueArr = columnObj.values;
                        var nItems = validArr.length;
                        var newValue = '',newIdx='';
                        for (var idx=0; idx< nItems; idx++) {
                            if (validArr[idx]) {
                                if (newValue) newValue += ' | ';
                                newValue += valueArr[idx];
                                if (newIdx) newIdx += '~';
                                newIdx += idx;
                            }
                        }
                        columnObj.value = newValue;
                        attObj.value = newValue;
                        attObj.combiIdx = this.findCombiIdx(attObj.listObj, columnObj, -1, attObj.value);
                        this.emitAttributevalueChanged(attObj);
                        //this.changeText(document.getElementById(this.prefix  + columnObj.attObj.id),null,null, columnObj.attObj);
                    }

                }

            }

            // full repaint - I think not needed
            //this.displayAttributeCategory();
            listObj.updateValuesNeeded=false;
            listObj.forceValueUpdate = false;
            return true;

        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.updateListAttributeValues" + ACatStrings.t('ACatAtLine') + err.lineNumber + "  : " + err.message);

        }

    };

    /**
    * deselects all dropdowns from a list 
    */
    attributeCategory.prototype.deselectListAttributeValues = function(listObj) {
    	try {
            var ncols = listObj.columns.length;
            for (var col=0; col< ncols; col++) {

                var colObj = listObj.columns[col];			
                colObj.validArr = null;			
                colObj.currentIndex = -1;		
				colObj.otherValueChosen = false;
                if (colObj.attObj) {
                    colObj.attObj.value = "";  // attObj is null if the column is not assigned.
                }
            }
            
            this.displayAttributeCategory();
            this._fireListValuesDeselected();
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.deselectListAttributeValues" + ACatStrings.t('ACatAtLine') + err.lineNumber + "  : " + err.message);

        }
    };

    /**
    * highlights all attributes linked to the same list 
    * @param {String} listName - name of the list
    * @param {String} listType - type of the list (Excel or database)
    * @param {String} listCounter - counter of the list (if duplicated)
    * @param {Boolean} highlight - if true, related items will be  highlighted, else highlight will be taken away
    */
    attributeCategory.prototype.highlightRelatedValues = function(listName, listType,listCounter,highlight) {
        try {
            // find list object
            var listObj = this.findList(listName, listType, listCounter);

            if (listObj) {
                var ncols = listObj.columns.length;
                for (var col=0; col< ncols; col++) {

                    var colObj = listObj.columns[col];			
                    if (colObj.attObj) {
                        var attId = colObj.attObj.id;
                        var elem = this.getAttributeInDOM(colObj.attObj);
                        // single value selector
                        if (elem) {
                            if (highlight) {
                                elem.style.backgroundColor = 'yellow';
                            } else {
                                elem.style.backgroundColor = '';	// not entirely right, should be the highlight of the style, but I won't solve this... (not important enough and with next repaint it's again ok)
                            }
                        }

                        // multi value selector
                        var elem = document.getElementById('mvdiv_' + this.prefix + attId);
                        if (elem) {
                            if (highlight) {
                                elem.style.backgroundColor = 'yellow';
                            } else {
                                elem.style.backgroundColor = '';	// not entirely right, should be the highlight of the style, but I won't solve this... (not important enough and with next repaint it's again ok)
                            }
                        }
                    }
                }

            }

        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.highlightRelatedValues" + ACatStrings.t('ACatAtLine') + err.lineNumber + "  : " + err.message);

        }
    };

    /**
    * Checks whether an attribute category  must be repainted entirely (and does the repaint) when a certain attribute changed value
    * @param attObj: the attribute which changed value
    */
    attributeCategory.prototype.checkRepaintAttributeCategory = function(attObj) {
        if (attObj.isInRule || (attObj.buttonDependency && attObj.buttonDependency.length>0) ) {
            this.displayAttributeCategory();
        }
    };


    /**
	 * Method to call defined function on all document reference attributes in this 
	 * attribute category. The context of the function (this) is the attribute's documents table object.
	 * Any returned value is ignored.
	 * 
	 * @param {Function} func - Function to be called in the context of the attribute's documents table.
	 * @param {Array.<Object>} params - array of parameters to be passed to the called function.
	 */
    attributeCategory.prototype.executeOnAllDocReferences = function(func, ctx, params) {
        if ((!this.loaded && !this.errorMsg) || !func) {
        	// attribte category not initialized correctly and/or no function to be executed defined - bail out
        	return;
        }
        // make sure 'params' is not undefined or null
        if (!params) params = [];
        // go over all attributes in this attr. cat. view and execute passed function 'func' with 
        // parameters 'params' on each document reference attribute with already initialized 
        // document table
        for (var row = 0; row < this.nRows; row++) {
            for (var col = 0; col < this.nCols; col++) {
                if (this.cells[row] && this.cells[row][col]) {
                    var cellObj = this.cells[row][col];
                    if (cellObj.attObj) {
                        var attObj = cellObj.attObj;
                    	var docTableObj = attObj.docTable;
                        if (attObj.isDocReference() && docTableObj) {
                        	// add the current documents table as the first parameter for the called function
                        	// to be able to access it within it and execute the 'func'.
                        	func.apply(ctx, [docTableObj].concat(params));
                        }
                    }
                }
            }
        }
    };


    attributeCategory.prototype.setACatIsExpanded = function() {
        if ((!this.loaded && !this.errorMsg) || !this.aCatNotExpandedYet) {
        	// attribute category not initialized properly or the this.aCatNotExpandedYet flag 
        	// is already set - bail out
        	return;
        }
    	// set the attribute category to be expanded
    	this.aCatNotExpandedYet = false;
    
        // go over all attributes in this attr. cat. and update the documents table
        // parentACatNotExpandedYet status/flag for each reference attribute with already 
        // initialized document table
    	this.executeOnAllDocReferences(function(docTableObj) {
    		// 'docTableObj' is always the "current" documents table object!
    		docTableObj.parentACatNotExpandedYet = false;
    	}, this);
    };


    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // DISPLAY FUNCTIONS
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



    var logInfo = "";
    /**
    * Used to display an attribute category in a document or project or as preview in admin
    * @param {} temp - 
    * @param {Int} repaintDepth: how deep you are in the call stack of recursive calling of displayAttributeCategory
    */
    attributeCategory.prototype._displayAttributeCategory = function(repaintDepth) {
        if (!repaintDepth){
        	repaintDepth = 0;
        }
        if(!this.loaded && !this.errorMsg){
        	return;
        }
        if(this.delayedLoading){
        	return;   
        }
        
        // When DOM operations affected the element, the attached ACatObj may not be in place
        // Make sure we always have the object in position when rendering.
        // (because methods using document.getElementById("x").ACatObj are written here)
        var storeElem = document.getElementById(this.id);
        if (storeElem && !storeElem.ACatObj){
        	storeElem.ACatObj = this;
        }

        this.repaintNeeded = false;
        // check the element to write into
        var displayElement = $('#'+this.displayId);
        if (displayElement.length == 0){
        	return;   // cannot find displayElement for attribute category - rendering is skipped...
        }

        // before html is removed from element, be sure to correctly destroy some elements in the DOM to prevent errors
        this.destroy();

		// we might lose position because of the rerender
        var position= $(window).scrollTop();
		
		// the rerender will also throw away our current focus, store the focused element and focus it again after rendering
		// note: we use the id of the element since the rerender create a new dom element...
		var focusedElementID = displayElement.find(":focus").attr("id");
        this.whenRendered(function(){
            var requestAnimationFrame = window.requestAnimationFrame || window.mozRequestAnimationFrame || window.webkitRequestAnimationFrame || window.msRequestAnimationFrame;
            requestAnimationFrame(function(){                
                // fix scroll position
                $(window).scrollTop(position);
                // focus the previously focused element
                $("#"+focusedElementID).focus();
            });
        });
        displayElement.empty();
        
        if (this.noCurrentViewDisplay) {
        	if (this.requestedView && this.name) {
        		// Show error msg
        		var isDocContext = (this.docVersionID !== null && this.docVersionID.length > 0) || (this.documentID !== null && this.documentID.length > 0);
        		var errMsg;
        		if (isDocContext) {
        			errMsg = ACatStrings.t('ACatNonExistingViewProblemDoc', this.requestedView, this.name);
        		} else {
        			errMsg = ACatStrings.t('ACatNonExistingViewProblemProj', this.requestedView, this.name);
        		}
        		
            	// a non-existing view with no fallback is asked. Display error message...
            	var div = $('<div />').addClass("alert alert-info").text(errMsg);
            	div.prepend($('<i />').addClass("icon icon-normal icon-WCR_Msg-Info"));
            	displayElement.append(div);
        	}
        	// something else could go wrong with setting/loading the view - don't display anything
        	return;
        }    
        
        // clear array of rich text fields, as it will be built up during rendering
        tinyMceMgrObj.tmceEditors[this.id] = [];

        if (this.currentView == 'Header' && this.noHeader == true) {
            // system asks for header view but it does not exist
            // leave htmlcode empty, so display nothing
        } else {
            logInfo += "time: " + new Date() + "  errorMsg "  + this.errorMsg + "\n"; 
            if (this.errorMsg) {
                if (this.currentView != 'Header') {
                	displayElement.append($('<div />').addClass("alert alert-danger").text(this.errorMsg));
                }
            } else if (this.currentView == 'Header') {
            	displayElement.append(this.displayTableHeader());							
            } else {				
            	this.displayTable(displayElement);
                // make the "Save" and "Export" (to JDF) buttons visible on project's and document's edit attribute page
                // it's here to show the buttons only after the attribute category is loaded and (almost) displayed
               	$("#AttributeActionButtons").show();
            }
        }        
        
        if (this.currentView != 'Header') {
        	displayElement.append('<div id="'+this.prefix+'hiddenFieldsContainer"></div>');  // will be filled with hidden fields in case of creation view

        	
        	// First make sure there are placeholders for children attribute categories in the DOM while we are waiting for 
        	// the "updateDocReferencesEditability" by already displaying the document tables now!
        	// Process calendars at the same time as they are completely unrelated (to optimize code).
            for (var row=0;row<this.nRows;row++) {
                for (var col=0; col<this.nCols; col++) {
                    if (this.cells[row] && this.cells[row][col]) {
                        var cellObj = this.cells[row][col];
                        if (cellObj.attObj) {
                            var attObj = cellObj.attObj;
                            if (attObj.type && attObj.type.toLowerCase() == 'datetime') {
                                var dateId = "#" + this.prefix + "longFormat_adate" + attObj.id;
                                var hourId = "#" + this.prefix + "date_period_of_day_" + attObj.id;
                                this.createCalendar(dateId,hourId);
                            } else if (attObj.isDocReference() && attObj.docTable) {
                                // display the documents table of the actual document reference attribute
                                attObj.docTable.displayTable();
                            }
                        }
                    }
                }
            } // end enable calendars / document references

            
            // edit-ability of all documents tables can be change without full reload of the page 
            // due to attribute category rules.
            // This is to make sure the related documents tables follows these changes and are updated 
            // before rendering!
            // Note all document tables are repainted within updateDocReferencesEditability() method
            // (it calls DocTableX.setIsEditable() with second parameter == true in there).
            this.updateDocReferencesEditability();
        }

        if (this.requestedView == 'Creation'){
        	this.addDefaultHiddenFields();
        }

        if (this.repaintNeeded) {
            if (repaintDepth < cMaxRepaints) {
                this._displayAttributeCategory(repaintDepth + 1); // needed when you have chained rules
            } else {				
                throw new Error('Maximum number of repaints reached. Endless loop?  Check your rules to make sure you have no cyclic dependencies. \nTypical mistake is to make a cell hidden based on a value of itself or 2 cells making each other hidden.'); 
            }
        } else {
			if (this.callbackOnRender) {
			    this.callbackOnRender();
			}

            if(this.currentView != 'Header'){
                try {
                    tinyMceMgrObj.addRichTextFields(this.id);
                } catch(err) {
                    // do nothing. Typical case is code being used in 10.2 for cascading dropdowns. 
                }
            }
        }
        
        // highlight all attributes that need highlighting, so highlight is retained when repainting
        this.highlightAttributes();
        
        this._applyWhenRendered();
    };

    /** 
    * Displays table in WebCenter look
    */
    attributeCategory.prototype.displayTable = function(container) {
            if (!this.cells){
            	return '';
            }

            var displayRow=0; // counts the rows you are really displaying
            var hidden = $("<div />").hide();
            var table = $("<table />").addClass("acat wcr-form");
            container.append(table);
            
            var useAlternativeRendering = this.useAlternativeRendering && !this.blockAlternativeRendering;
            // use colgroups to assist layout
            if(useAlternativeRendering){
            	table.css("table-layout", "fixed");
            	
            	/**
            	 * Get width and unit of style object
            	 * For width, it will take the column width, or the value width if that one's bigger
            	 */
            	var getWidth = function(styleObj){
            		var marginCompensation = 30;
            		
            		var w = 0;
            		var unit = "";
            		if(styleObj.columnWidth){
            			var colw = parseInt(styleObj.columnWidth);
            			if(!isNaN(colw)){
            				w = colw;
            				unit = styleObj.columnWidthUnit;
            			}
            		}
            		
					if (!unit || unit == "") {
						unit = "%";
					}
            		return {width: w, unit: unit};
            	};
                
                // The default width of the label values.
                var defLabelPixWidth = Math.min(200, 750/this.nCols);
                if(this.currentView && this.currentView.toLowerCase().indexOf("viewer") == 0){
                	defLabelPixWidth = Math.min(100, 300/this.nCols);
                }

                // Variables to hold the width of the columns of Acat.
                var lwidth = {};
                var vwidth = {};
                
                // Variables to hold the total width of custom widths, a custom width is a width that is specifically defined by the admin who configured the attribute category.
                var totStaticWidth = 0; // The total static amount of width of the table expressed in pixels. (Holds custom widths as well as "defLabelPixWidth").
                var totRelativeSizedWidth = 0; // The total amount of relative width of the table already taken up by custom defined column widths expressed in %. (Hold only custom widths)
                var nUnsizedValueCols = 0; // The number of value columns without any static or relative custom width (Their sizes need later be calculated based on "totStaticWidth" and "totTakenPercents")
                
                // STEP 1: Iterate over all cells of the attribute category and find the most custom "important" widths for each columns
                for(var col=0; col<this.nCols; col++){
                	
                	// The most "important" custom width value for the "L"abel column
                	var lmax = undefined; // Start off with undefined width for the "L"abel column.
                	// The unit in which this most "important" custom width of the "L"abel column is defined.
        			var lunit = "%"; // Assume initially that the user specifies the width "%" as "px" takes precedence.
                	// The most "important" custom width value for the "V"alue column
        			var vmax = undefined; // Start off with undefined width for the "V"alue column.
                	// The unit in which this most "important" custom width of the "V"alue column is defined.
        			var vunit = "%"; // Assume initially that the user specifies the width "%" as "px" takes precedence.
        			// This boolean will indicate wether the width value of the "L"abel column is included in the one from the "V"alue column.
        			var lIncludedInValue = false;
        			
        			// Now let's start finding these values, start iterating over every cell of the column ...
                	for (var row = 0; row < this.nRows; row++) {
                		if (this.cells[row] && this.cells[row][col]) {
                			// The cell object.
                			var cellObj = this.cells[row][col];
                			// Check if this cell object has a custom style and only spans one column 
                			// (we don't take sizes of multi spanning columns into account).
                			if(cellObj.styleObj && (!cellObj.colSpan || cellObj.colSpan == 1)){
                				// Check if the style of this cell has a custom width.
                				var styleObj = cellObj.styleObj;
                				
                				var colWidth = getWidth(styleObj);
                				if(colWidth.width > 0){
            						var w = colWidth.width;
            						var unit = colWidth.unit;
            						
        							// Important! It's possible to have a different width for the value than for the label.
        							// Check if no specific width for the label and value were defined ...
        							if(!styleObj.isDifferentStyles){
        								// A common width was specified that encapsulates both the label and the value.
        								// Check if this width was provided in pixels.
        								if (unit == "px") {
        									// By default use 1/3 for the label.
        									var lw = w * (1 / 3);
            								var vw = w * (2 / 3);
            								// But cap it to a maximum of "defLabelPixWidth".
        									if (lw > defLabelPixWidth) {
        										lw = defLabelPixWidth;
        										vw = w - defLabelPixWidth;
        									}
        									// Check if these should be the new most "important" widths for the value and label columns.
        									if (!lmax || lw > lmax) {
        										lmax = lw;
        										lunit = "px";
        									}
        									if (!vmax || vw > vmax) {
        										vmax = vw;
        										vunit = "px";
        									}
        								} 
        								// Check if the width of the cell was provided in percentages.
        								else if (unit == "%") {
        									// Only continue if the most important "size" for the value column is set to "%" so far.
        									// This makes that sizes set in "px" take precedence over sizes set in "%".
        									if (vunit == "%") {
        										// Set the label to the default width.
            									var lw = defLabelPixWidth;
            									if (lunit == "%") {
            										lunit = "px";
            										lmax = lw;
            										lIncludedInValue = true;
            									} else if (!lmax || lw > lmax) {
            										lmax = lw;
            										lunit = "px";
            									}
            									// Set the percentage width.
            									if (!vmax || w > vmax) {
            										vmax = w;
            									}
        									}
        								}
    								} 
        							// If label and value have different styles (This covers labels)
        							else if (styleObj.isDifferentStyles) {
        								if (unit == lunit) {
        									if(!lmax || w > lmax){
        										lmax = w;
        									}
        								} else if (unit == "px") {
        									lmax = w;
        									lunit = "px";
        								}
    								}
            					}
                				// If label and value have different styles (This covers values)
                				if(styleObj.isDifferentStyles && styleObj.valueStyleObj){
                					var valColWidth = getWidth(styleObj.valueStyleObj);
                					
                					if (valColWidth.width > 0) {
                						var w = valColWidth.width;
        	        					var unit = valColWidth.unit;

    	        						if (unit == vunit) {
        									if(!vmax || w > vmax){
        										vmax = w;
        									}
        								} else if (unit == "px") {
        									vmax = w;
        									vunit = "px";
        								}
                					}
        						}
                			}	
                		}
                	}
                	
                	if(lmax){
                		if(lunit ==  "%") {
                			// Only add the width of the label column to the total of static space of the table if the width 
                			// of the label column is not already included in that of the value column.
                			if (!lIncludedInValue) {
                				totRelativeSizedWidth += lmax;
                			}
                			lwidth[col] = {
                    				num : lmax,
                    				unit : "%",
                    				includedInValue : lIncludedInValue
                    		};
                		} else if(lunit ==  "px") {
                			// Only add the width of the label column to the total of static space of the table if the width 
                			// of the label column is not already included in that of the value column.
                			if (!lIncludedInValue) {
                				totStaticWidth += lmax;
                			}
                			lwidth[col] = {
                    				num : lmax,
                    				unit : "px",
                    				includedInValue : lIncludedInValue
                    		};
                		}
                		
                	} else {
                		// If no custom size for a label was found, take 250px.
                		totStaticWidth += defLabelPixWidth;
                		lwidth[col] = {
                				num : defLabelPixWidth,
                				unit : "px",
                				includedInValue : false
                		};
                	}
                	if(vmax){
                		if(vunit ==  "%"){
                			totRelativeSizedWidth += vmax;
                			vwidth[col] = {
                    				num : vmax,
                    				unit : "%"
                    		};
                		} else if(vunit ==  "px") {
                			totStaticWidth += vmax;
                			vwidth[col] = {
                    				num : vmax,
                    				unit : "px"
                    		};
                		}
                	} else {
                		// If no custom size for a value was found, calculate it later based on "totRelativeSizedWidth" and "totStaticWidth".
                		nUnsizedValueCols++;
                	}
                	
                }
                
                // Check the sanity of the given percentages and make adjustments.
                if (totRelativeSizedWidth + nUnsizedValueCols > 100) {
                	// Adjust the column percentages to a sane one.
                	for(var col=0; col<this.nCols; col++) {
                    	if (vwidth[col] && vwidth[col].unit == "%") {
                    		vwidth[col].num = 100 * (vwidth[col].num / totRelativeSizedWidth);
                    	}
                    	if (lwidth[col] && lwidth[col].unit == "%") {
                    		lwidth[col].num = 100 * (lwidth[col].num / totRelativeSizedWidth);
                    	}
                    }
                	totRelativeSizedWidth = 100;
                }
                
                // Calculate the total css width of every column.
                for(var col=0; col<this.nCols; col++) {
                	if (lwidth[col]) {	
                		lwidth[col].css = lwidth[col].num + lwidth[col].unit;
                	} else {
                		// Should never get here ... 
                	}
                	if (vwidth[col]) {
                		if (vwidth[col].unit == "%" && lwidth[col].unit == "px" && lwidth[col].includedInValue) {
                			vwidth[col].css = "calc("+vwidth[col].num+"% - "+lwidth[col].num+"px)";
                		} else if (vwidth[col].unit == "%" && lwidth[col].unit == "%" && lwidth[col].includedInValue) {
                			vwidth[col].css = max(0, vwidth[col].num - lwidth[col].num) + "%";
                		} else {
                			vwidth[col].css = vwidth[col].num + lwidth[col].unit;
                		} 
                	} else {
                		var remainingPercentage = (100 - totRelativeSizedWidth) / nUnsizedValueCols;
                		var takenPixels = totStaticWidth / nUnsizedValueCols;
                		vwidth[col] = {
                				css : "calc("+remainingPercentage+"% - "+takenPixels+"px)"
                		};
                	}
                }

                // create the colgroups
                var colgroup = $('<colgroup>');
                for(var col=0; col<this.nCols; col++){
                	if(lwidth[col]){
                		colgroup.append($('<col>').css('width', lwidth[col].css));
                	}
                	if(vwidth[col]){
                		colgroup.append($('<col>').css('width', vwidth[col].css));
                	}
                }
                table.append(colgroup);
                
                if(nUnsizedValueCols == 0 && totRelativeSizedWidth == 0){
                	table.css("width", totStaticWidth);
                }
            }
            

            // determine whether to put a header bar or a rounded title
            if (this.layout == "OneColumn" && this.currentView != 'Header') {
            	var tr = $('<tr />');
                for (var col=0; col< this.nCols; col++) {
                    tr.append($('<th />').text(ACatStrings.t('ACatName')));
                    tr.append($('<th />').text(ACatStrings.t('ACatValue')));
                }
                table.append($('<thead />').append(tr));
            }  

            var body = $('<tbody />');

            for (var row = 0; row < this.nRows; row++) {
                var rowEmpty = true;
                var rowElem = $('<tr />');

                if (this.cells[row]) {
                    for (var col=0; col<this.nCols; col++) {
                        var cellObj = this.cells[row][col];
                        if (cellObj) {
                            var visibility = "Editable";
                            visibility = this.checkVisibility(cellObj);  // returns one of  Editable, ReadOnly, Hidden, HiddenKeepValues or Required                    

                            if (visibility !=  "Hidden" && visibility !=  "HiddenKeepValues") {	
                                switch (cellObj.type) {
                                    case 'Title':							
                                    	rowElem.append(this.renderTitle(cellObj, row, col, visibility));
                                        break;
                                    case 'Spacer':
                                    	rowElem.append(this.renderSpacer(cellObj));
                                        break;
                                    case 'Ruler':
                                    	rowElem.append(this.renderRuler(cellObj));
                                        break;
                                    case 'Button':
                                        if (visibility !=  "ReadOnly"){
                                        	rowElem.append(this.renderButton(cellObj,row,col,false));
                                        }
                                        break;
                                    case 'Attribute':
                                    case 'Property':
                                        var attObj = cellObj.attObj;
                                        if (attObj) {
                                            if (cellObj.visibility == 'Hidden' || cellObj.visibility == 'HiddenKeepValues') {
                                                // returns from being hidden
                                                var colObj = attObj.columnObj;
                                                if (colObj) {  // part of a list.  Value can already be determined by select single options, so don't overwrite this.												
                                                    if (attObj.isInRule && (attObj.columnObj.autoSelected )) this.repaintNeeded = true; // for chained rules
                                                } else if (cellObj.visibility == 'Hidden' && attObj.previousVisibleValue!=null ) {
                                                    // Show the last value when it was still visible.
                                                    attObj.value = attObj.previousVisibleValue;
                                                    if (attObj.isInRule) this.repaintNeeded = true; // for chained rules											
                                                }											
                                            }
                                            if (cellObj.visibility == 'Hidden'){
                                            	cellObj.attObj.previousVisibleValue = "";  // no longer remember once shown
                                            }
                                         
                                            if (attObj.columnObj && attObj.columnObj.uniqueIdentifyer=='1') {
                                            	rowElem.append(this.renderItemList(cellObj,row,col,visibility));		// experiment (showing list as a row)
                                            } else {
                                            	rowElem.append(this.renderAttribute(cellObj,row,col,visibility));
                                            }
                                        } // end if attObj
                                        break;                            
                                    case 'HeaderBar':
                                        var colSpan = cellObj.colSpan? cellObj.colSpan : 1;
                                        rowElem.append(this.renderHeaderBar(colSpan, col==0, col+colSpan>=this.nCols));
                                        break;
                                    default:
                                        alert(ACatStrings.t('ACatUnknownCellTypeProblem') + cellObj.type + ACatStrings.t('ACatId') + cellObj.id);
                                        break;
                                }
                                rowEmpty = false;

                                // determine colspan
                                var colSpan = parseInt(cellObj.colSpan);
                                if (!colSpan){
                                	colSpan = 1;
                                }

                                // skip colspanned columns
                                col += colSpan-1;
                            } else {					
                            	rowElem.append(this.renderEmptyCell());
                                var attObj = cellObj.attObj;
                                if (attObj) {		

                                	hidden.append(this.renderHiddenCell(attObj, visibility == "HiddenKeepValues"));							
                                    if (visibility !=  "HiddenKeepValues" && attObj.value != "" && !attObj.columnObj) {  // invisible attributes carry no value except when they are part of a cascading dropdown								
                                        attObj.previousVisibleValue = cellObj.attObj.value;
                                        attObj.value = "";
                                        if (attObj.isInRule){
                                        	this.repaintNeeded = true; // for chained rules
                                        }
                                    }
                                }
                            }
                            cellObj.visibility = visibility;  // remember to see whether it changes
                        } else {  // !cellObj
                        	rowElem.append(this.renderEmptyCell());
                        }
                    }
                }

                // now show the row
                if (rowEmpty === false) {                    
                    if(this.styles[0].background){
                    	rowElem.css("background-color", this.styles[0].background);
                    } else if (displayRow%2 == 1 && this.alternateBackground) {
                    	rowElem.addClass("alt");
                    }

                    displayRow++;
                    body.append(rowElem);
                }
            }
            body.append(hidden);  // hidden fields
            table.append(body);
            return table;
    };


    /** 
    * Displays table in WebCenter header look
    * supposes to get a OneColumn layout so the rows and columns you see in the header are constructed on the fly
    */
    attributeCategory.prototype.displayTableHeader = function() {
        if (!this.cells){
        	return '';
        }            
        var table = $('<table />').addClass("header-attributes-cell project-info");

        // construct a new array with all elements from the header view. This to cater with empty cells. 
        // it is seldomly needed but problems have arrived probably caused by historical bugs of which the resulting view is stored in XML so not easy to clean
        var tempArr = new Array();
        for (var row=0; row< this.nRows; row++) {
            var cellObj = this.cells[row][0];  // only one column is taken into account
            if (cellObj && cellObj.type =='Attribute' ){
            	tempArr.push(cellObj);
            }
        }

        var nCols = Math.floor((tempArr.length-1)/4) +1;
        var nRows = Math.floor((tempArr.length-1)/nCols) + 1;
        for (var row=0; row< nRows; row++) {
            var tr = $('<tr class="headerRow" />');  

            for (var col=0; col< nCols; col++) {  // columns in the actual header display, not in the header view!!!
                var rowIdx = row + nRows*col;
                if (rowIdx < tempArr.length) {
                    var cellObj = tempArr[rowIdx];
                    var attObj = cellObj.attObj;					   
                    if (attObj) {
                        tr.append(this.renderAttribute(cellObj,row,col,"ReadOnly",false));
                    } 
                }
            }
            table.append(tr);
        }
        return table;
    };



    /**
    * Creates htmlcode for an attribute field. Like each renderer, it returns a list of <td>'s adding up to the needed column span (as defined in cellObj.colSpan)
    * @param cellObj cell object containing the attribute
    * @param row col : row and col where the field is
    * @param {Boolean} visibility: any of Editable, Required, Hidden (typically not), HiddenKeepValues, ReadOnly
    * @param {Boolean} forPrint - when true, rendering is optimized for print-outs (no interaction --> no buttons). 
    */
    attributeCategory.prototype.renderAttribute = function(cellObj, row, col, visibility, forPrint) {
        var attObj = null;
        try {
            var htmlcode = '';

            attObj = cellObj.attObj;
            if (!attObj) {
            	throw(ACatStrings.t('ACatRenderAttrProblem') + " attObj");
            }

            //DETERMINE COLSPAN
            ///////////////////////
            var colSpan = 1;
            if (cellObj.colSpan >1){
            	colSpan =  cellObj.colSpan * 2 - 1 ;
            }
            if (cellObj.labelPosition == 'Top'){
            	colSpan += 1;
            	
            	var styleStr = 'style="width: 100%"';
		        if(this._inheritWidth(cellObj)){
		        	var styleStr = 'style="width: inherit"';
		        }
            	htmlcode += '<td class="validation-markup-element" colspan="'+colSpan+'" ' + this.renderStyle(cellObj.styleObj,"Value") + '><table margin="0" '+styleStr+'><tr>';
            }

            // RENDER THE LABEL 		
            //////////////////////////////////////////////////////////////////////////////////////		
            if(attObj instanceof Property && attObj.getIsRequired() && !attObj.hidden){
            	visibility = "Required";
            }
            
            var labelId = undefined;
    		if(cellObj.attObj.labelId){
    			labelId = cellObj.attObj.labelId;
    		}
            
    		var editable = this.isAttrCatEditable() && !(this.showFakedReadOnly || visibility == "ReadOnly");
            htmlcode += this.renderAttributeLabel(cellObj, row, col, visibility, editable,cellObj.renderOption == 'Images' || cellObj.renderOption == 'RadioCheckboxes', labelId);   // always renders a <td>...</td>

            if (cellObj.labelPosition == 'Top'){
            	htmlcode += '</tr><tr>';
            }


            // RENDER THE VALUE     Looks like  <td id="doc1_td_5_5" style="..." colspan="...">...content of a render function...</td>
            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            var extraValueLabel = "";
            if(this.nCols == 1 && cellObj.labelPosition !== 'Top' && cellObj.styleObj && !(cellObj.styleObj.width || cellObj.styleObj.columnWidth)){
            	extraValueLabel = "oneColValue";
            }
            
            htmlcode += '<td ';
            if (this.currentView != 'Header') {
            	htmlcode += ' id="' + this.prefix + 'td_' + row + '_' + col + '"';  
            }
			if (cellObj.labelPosition != 'Top'){
				htmlcode += " class='validation-markup-element'";	
			}

            htmlcode += this.renderStyle(cellObj.styleObj,attObj.restrictType == 'DocumentReference'? "Hyperlink": "Value", null, null, extraValueLabel);



            if (colSpan >1 && cellObj.labelPosition != 'Top') {
                htmlcode += ' colspan="' + colSpan +  '" ';
            }
            htmlcode += '>';

            htmlcode += this.renderAttributeValueInternal(cellObj, attObj, row, col, visibility, undefined, forPrint);      
            htmlcode += '</td>';
            
            if (cellObj.labelPosition == 'Top'){
            	htmlcode += '</tr></table></td>';
            }

            return htmlcode;
        } catch(err) {
            var msg = "";
            if (attObj) msg += "\n attObj.name: " + attObj.name + "\n attObj.type: " + attObj.type;
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderAttribute at line : ' + err.lineNumber + ": " +  err.message + msg);
            return '<td class="darktext_greenlink_11">Missing attribute</td><td></td><td></td>';
        }
    };
    
    /**
     * Render label
     */
     attributeCategory.prototype.renderUnitLabel = function(cellObj, editable) {
         var attObj = cellObj.attObj;
         var htmlcode = '';
         var unitHtml = '';        
         var subtle = false;
         var span = false;
         var hasValue = (attObj.value!="" && attObj.value!=null);

         if (attObj && (this.currentView != 'Header' || hasValue) ) {  // don't show units in header when there is no value
        	 if (cellObj.customUnitLabel) {
              	unitHtml += cellObj.customUnitLabel;
              	span = !editable;
             } else if (attObj.unitLocalized) {
             	unitHtml += attObj.unitLocalized;
             	span = !editable;
             } else if (this.showTypeAsUnit) {
            	 subtle = true;
                 if (attObj.rs) {
                 	unitHtml += attObj.rs.name;
                 } else if (attObj.typeLocalized) {
                 	unitHtml += attObj.typeLocalized;
                 } else if (attObj.unit) {
                 	unitHtml += attObj.unit;				
                 } else if (attObj.type) {
                     // DT125977: you typically get here in the admin page for just placed attributes. For these, typeLocalized is not known yet and unit neither --> just put type (while previously nothing was put)
                 	unitHtml += attObj.type;  
                 }
             }
         }
         
         if (this.currentView != 'Header' && !span) {
        	 var style = "";
        	 var classes = "input-group-addon";
        	 if(subtle){
        		 classes += " subtle"; 
        	 }
        	 if(!editable && !hasValue){
        		 htmlcode += '<span>&nbsp;</span>';
        	 }
        	 htmlcode += '<label'+style+' class="'+classes+'" for="'+this.prefix + attObj.id+'">'; // large padding right creates the effect that maximum space is put here
        	 htmlcode += '<span '+this.renderStyle(cellObj.styleObj,"Unit")+'>';
        	 htmlcode += unitHtml;
        	 htmlcode += '</span>';
        	 htmlcode += '</label>';
         } else if(hasValue){
        	 htmlcode += '<span class="ReadOnlyAttr">&nbsp;';
        	 htmlcode += unitHtml;
        	 htmlcode += '</span>';
         }

         // only return label html if label is not empty
         return unitHtml?htmlcode:'';
     };

    /**
    * renders the label of the attribute and the Select All, Deselect All and Show All actions in case of attribute images
    * @param {Object} cellObj - cellObj for this attribute, remark you cannot reconstruct this from row,col in case of header view (where the cells are onecolumn layout)
    */
    attributeCategory.prototype.renderAttributeLabel = function(cellObj,row, col, visibility,editable, showActions,id) {
        var htmlcode = '';
        try {

            var attObj = cellObj.attObj;
            if (!cellObj || !attObj) throw new Error("cellObj or attObj is null for row " + row + "  col " + col);
            
            var extraClassLabel = "";
            if(this.nCols == 1 && cellObj.labelPosition !== 'Top' && cellObj.styleObj && !(cellObj.styleObj.width || cellObj.styleObj.columnWidth)){
            	extraClassLabel = "oneColLabel";
            }

            var label = cellObj.label != null ? cellObj.label : cellObj.name; // distinguish null from ''; '' should show empty label, null should show name
            htmlcode += '<td ';
            htmlcode += this.renderStyle(cellObj.styleObj,"Label", null, null, extraClassLabel);
            if (cellObj.labelImage)  htmlcode += ' align="center" ';
            if (attObj.name != label) htmlcode += ' title="'+WCXML.xmlencode(cellObj.attObj.name)+'" ';  // tooltip with the real attribute name

            var classes = "attributeLabel";
            if(cellObj.labelPosition == 'Top'){
            	classes += " labelTop";
            }
            
            htmlcode += '><label class="'+classes+'" '+this.renderStyle(cellObj.styleObj,"LabelInternal")+' for="' + this.prefix + attObj.id + '"' + (id? ' id="'+id+'"' : '') + '>';
            
            if (cellObj.labelImage) {
                    htmlcode += '<img src="'+window.webcenterConfig.jspBaseUrl+'custom/AttributeLabelImages/'+cellObj.labelImage+'" alt="'+label+'" title="'+label+'">';
            } else {
                if (this.currentView == 'Header') {
                    var defaultLabel = this.findHeaderLabel(attObj);  // look for the same attribute in the default view
                    if (defaultLabel) label = defaultLabel;
                }
                htmlcode += WCXML.xmlencode(ACatStrings.t(label)); // try a translation as well

                if (this.currentView == 'Header') htmlcode += ':';  //Add a colon after the label
            }
            // Red asterisk if "Required"        
            if (visibility == 'Required'  && editable) htmlcode += '&nbsp;<span class="esko-green">*</span>';  

            // Warning if value will overwrite project attribute value        
            if (attObj.projAttValue && attObj.projAttValue != "" && editable) {
                var clickMethod = this.functionPrefix + '.recoverProjectAttributeValue('+row+','+col+'); return false';

                htmlcode += '<div  id="warning'+attObj.id+'" style="visibility:hidden;display:none"><div class="darktext_darklink_11" style="background-color:#FFFFDD">';
                htmlcode += '<font color="red">'+ACatStrings.t('ACatProjectAttributeOverwriteWarning')+'</font>';
                htmlcode += '<br><a class="darktext_darklink_11" href="#" onclick="' + clickMethod +'">'+ACatStrings.t('ACatLoadCurrentProjectAttributeValue')+'</a>';
                htmlcode += '</div></div>'; // double div to shown the innerHTML of the outer one in a popup

                var hoverMethod = 'try{delayshowpopupbyid(event, \'warning'+attObj.id+'\', 300)}catch(err){}';
                var clickMethod2 = 'try{showpopupbyid(event, \'warning'+attObj.id+'\', 300);}catch(err){}';

                // only display when there's a difference between the values
                    htmlcode += '<img id="imgwarning'+attObj.id+'" src="'+window.webcenterConfig.imageBaseUrl+'warning.png"';

                var hiddenStyle = 'style="visibility:hidden;display:none"';
                if (attObj.isDocReference()) {
                    if (attObj.docTable && attObj.projDocRefs && attObj.docTable.hasSameContentAs(attObj.projDocRefs.docRows)) {
                        htmlcode += hiddenStyle;
                    }
                } else if (attObj.type && attObj.type.toLowerCase() == 'datetime' && !attObj.rs) {
                    if (attObj.projAttValue.toString() == attObj.value.toString()){
                        htmlcode += hiddenStyle;
                    }
                } else {
                    if (attObj.projAttValue == attObj.value) {
                        htmlcode += hiddenStyle;
                    }
                }

                htmlcode += ' onmouseover="' + hoverMethod +'" onclick="' + clickMethod2 +'" onmouseout="delayhidethumb()">';
            }	

            if (cellObj && cellObj.infoPage) {		
                var clickMethod = 'popw=window.open(&quot;./custom/AttributesHelp/'+  WCXML.xmlencode(cellObj.infoPage) + '&quot;,&quot;Info&quot;,' + this.helpOptionsString + ');popw.focus();';
                    htmlcode += '<a class="inline_help btn" href="#" onclick="' + clickMethod +'"><i class="icon icon-WCR_Info-Balloon icon-normal"></i></a>';
            }
            
            htmlcode += '</label>';


            if (showActions && visibility != 'ReadOnly' && editable) {
                htmlcode += '<label class="attributeActionsLabel" '+this.renderStyle(cellObj.styleObj,"LabelInternal")+'>';

                if (attObj.allowMultipleValues) {
                    clickMethod = this.functionPrefix + '.selectAllMultiValue(\''+ this.prefix + 'ImgChk_' + attObj.id +'\', '+row+', '+ col+'); return false; ';
                    htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatSelectAll')+'</a>';
                    clickMethod = this.functionPrefix + '.deselectAllMultiValue(\''+ this.prefix + 'ImgChk_' + attObj.id +'\', '+row+', '+ col+'); return false; ';
                    htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatDeselectAll')+'</a>';
                    if (attObj.columnObj) {
                        if (attObj.showAll) {
                            clickMethod = this.functionPrefix + '.showAllImages('+row+', '+ col+',\'' + visibility + '\',false); return false; ';
                            htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatShowAllowed')+'</a>';								
                        } else {
                            clickMethod = this.functionPrefix + '.showAllImages('+row+', '+ col+',\'' + visibility + '\',true); return false; ';
                            htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatShowUnfiltered')+'</a>';		
                        }
                    }					
                } else {
                    var clickMethod = this.functionPrefix + '.deselectAllSingle(\''+ this.prefix + 'ImgChk_' + attObj.id +'\','+row+','+ col+'); return false; ';
                    htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatDeselect')+'</a>';
                }
                htmlcode += '</label>';
            }
            


            htmlcode += '</td>';
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderAttributeLabel at line : ' + err.lineNumber + ": " +  err.message );
        }
        return htmlcode;
    };

    attributeCategory.prototype.findHeaderLabel = function(attObj) {
        try {
            if (!this.views) return null;
            var cells = this.views[0].cells; 
            for (var row=0; row<cells.length; row++) {
                var rowArr = cells[row];
                if (rowArr) {
                    for (var col=0; col<rowArr.length; col++) {
                        if (rowArr[col] && rowArr[col].attObj== attObj) {		
                            return rowArr[col].label != null ? rowArr[col].label : rowArr[col].name; // distinguish null from ''; '' should show empty label, null should show name
                        }
                    }
                }

            }
            return null;

        } catch(err) {
            // do nothing
            return null;
        }

    };


    /**
    * renders the internals of the <td> of the value cell of an attributes
    * Parameters defined as in renderAttribute
    */
    attributeCategory.prototype.renderAttributeValueInternal = function(cellObj, attObj, row, col, visibility, editable, forPrint) {
        'use strict';
        var htmlcode = '';

        try {
        	editable = editable !== false && (this.isAttrCatEditable() && (visibility == "Required" || visibility == "Editable"));
        	
        	if (!this.isSaveButtonShown && (visibility != 'Hidden') && (visibility != 'HiddenKeepValues') 
        			&& editable && (this.currentView != 'Header') && !this.isInactive && !this.errorMsg) {
           		// if the current attribute (category) is editable, show the save button as well.
           		// this works fine with tree-like structures made by document reference attributes too!
           		// as soon as one of the editable descendant attribute categories or even just a single
        		// editable attribute in document reference column is rendered the Save button appears. 
        		// it's never hidden again unless the whole page is realoaded.
           		// note doc. ref. properties are handled by this too.
           		$("#SaveButtonRowID").show();
           		$("#HeaderActionSaveAttrButton").removeClass("hiddenElement");
           		// remember the Save button was already shown
           		this.isSaveButtonShown = true;
        	}
        	
            var changeMethod;
            var hasButton = false;
            if (cellObj.buttons && cellObj.buttons.length>0) hasButton = true;

            if (attObj) {  // test is probably not necessary, you're thrown out earlier if attObj does not exist
		        var formEntry = $('<div class="form-entry" />');
		        if(cellObj.labelPosition != 'Top' && this._inheritWidth(cellObj)){
		        	formEntry.css("width", "inherit");
		        }
		        
		        var inputGroup = $('<div class="input-group '+attObj.type+'" '+this.renderStyle(cellObj.styleObj,"ValueGroup")+' />');
		        formEntry.append(inputGroup);
		        
		        if(attObj.isDocReference()){
		        	formEntry.css({
		        		"overflow-y": "hidden",
		        		"overflow-x": "auto"
		       		});
		        } else {
		        	formEntry.css({
		        		"overflow": "hidden"
		       		});
		        }
		        
		        var attValue = attObj.value;
                var theName = this.prefix + attObj.id;

                htmlcode += this.renderHiddenAttributeInfo(attObj);

                if (attObj instanceof Property) {
                	htmlcode += this.renderProperty(cellObj, row, col, visibility, forPrint);
                } else if (cellObj.renderOption == 'Images' || cellObj.renderOption == 'RadioCheckboxes') {
                    htmlcode += this.renderAttributeImages(cellObj,row,col,visibility);
                } else if (attObj.rs) { // restricted set

                    if (attObj.isCheckBox) {
                        // render as checkbox 
                        htmlcode += this.renderCheckbox(cellObj,row,col,editable);
                    } else {
                        if (editable) {
                            if (attObj.allowMultipleValues) {
                                htmlcode += this.renderSelectMultiple(cellObj,row,col,true);
                            } else {
                                htmlcode += this.renderSelect(cellObj,row,col);
                            }
                        } else {
                        	htmlcode += '<span class="ReadOnlyAttr">';
                        	var renderHiddenValue = true;
                            if (attObj.type && attObj.type.toLowerCase() == 'datetime') {
                                    var attDate = this.correctForDaylightSaving(new Date(parseInt(attValue)));
                                    if (attValue) {
                                    	htmlcode += WCHTML.getDateHourString(attDate);  // in case of restricted set, the attValue is a time stamp, not a Date object!
                                    }
                            } else if(attObj.fractionalIfImperial && this.userUnits && this.userUnits.toLowerCase() == "imperial"){
                                if (attValue) {
                                	htmlcode += WCHTML.decimalToFractional(attValue);
                                } 
                            } else {
                                if (attObj.allowMultipleValues && this.currentView != 'Header') {
                                	renderHiddenValue = false; // this is already done in the renderselectmultple method
                                    htmlcode += this.renderSelectMultiple(cellObj,row,col,false);
                                } else {
                                    htmlcode += attValue;
                                }
                            }	
                            htmlcode += '</span>';
                            
                            if(renderHiddenValue){
                            	htmlcode += '<input type="hidden" name="'+theName+'" value="'+WCXML.xmlencode(attValue)+'">';	
	                        }
	                    }
                    }

                } else if ( attObj.listObj  && (attObj.listObj.loadingType != 'OnTheFly' || attObj.allowMultipleValues)) {
                    if (attObj.columnObj && attObj.columnObj.uniqueIdentifyer=='2') {
                        htmlcode += this.renderUniqueIdentfyerBox(cellObj,row,col,editable);
                    } else if (attObj.allowMultipleValues) {
                        htmlcode += this.renderSelectMultiple(cellObj,row,col,editable);
                    } else {
                        htmlcode += this.renderRestrictedList(cellObj,row,col,editable);
                    }
                } else if (  attObj.restrictType == 'Attribute' ) {
                    if (attObj.allowMultipleValues) {
                        htmlcode += this.renderSelectMultiple(cellObj,row,col,editable);
                    } else {	
                        if (editable) {
                            htmlcode += this.renderOtherAttributeSelector(cellObj,row,col);
                        } else {
                        	var theName = this.prefix + attObj.id;
                        	htmlcode += '<span class="ReadOnlyAttr">';
                            htmlcode += attValue;
                            htmlcode += '</span>';
                            htmlcode += '<input type="hidden" name="'+theName+'" value="'+WCXML.xmlencode(attValue)+'">';	
                        }
                    }
                } else if (cellObj.ruleObj && cellObj.ruleObj.visibilityRestriction && cellObj.ruleObj.visibilityRestriction.restrictedValues) {
                	// restriction based on rules --> not released feature
                    if (editable) {
                        htmlcode += this.renderRestriction(cellObj,row,col);
                    } else {
                    	var theName = this.prefix + attObj.id;
                    	htmlcode += '<span class="ReadOnlyAttr">';
                        htmlcode += attValue;
                        htmlcode += '</span>';
                        htmlcode += '<input type="hidden" name="'+theName+'" value="'+WCXML.xmlencode(attValue)+'">';	
                    }

                } else if (attObj.restrictType == 'DocumentReference') {

                	htmlcode += this.renderDocumentReferencePH(cellObj, row, col);

                } else {  // not restricted or an on-the-fly list column

                    if (attObj.type && attObj.type.toLowerCase() == 'datetime') {
                        // DATE TIME

                        var dateString = '';
                        if (attValue.constructor == Date) dateString = WCHTML.getDateString(attValue);

                        if (editable) {                   	
                        	// put hidden field for date in format 12/03/2010
                            var longDateName = this.prefix + "longFormat_adate" + attObj.id;
                            var hourName = this.prefix + "date_period_of_day_" + attObj.id;
                            changeMethod = this.functionPrefix + '.changeDateTime(' + row + ',' + col + ')';
                            
                            //  hour, uses same changeMethod
                            var halfhours = this.defaultHourAsHalfHours;

                            if (attValue) {
                                if (attValue.constructor != Date) throw(ACatStrings.t('ACatAttrDateProblem') + attValue);
                                halfhours = attValue.getHours() * 2 + Math.floor(attValue.getMinutes()/30);
                            } else {
                                halfhours = -1; // will display as --:--
                            }
                        	
                            // create HTML
                            htmlcode += '<div class="form-datetime">';
                            htmlcode += '<div class="form-date">';
                        	htmlcode += '<div class="input-group">';
                        	htmlcode += '<input type="text" class="form-control" size="16" onchange="' + changeMethod + '"  value=" ' + WCXML.xmlencode(dateString) + '" ';                        
                            htmlcode += this.renderStyle(cellObj.styleObj,"DateField");
                            htmlcode += ' name="'+ longDateName +'" id="'+ longDateName +'" valign="middle" />';
                            htmlcode += '<label for="'+longDateName+'" class="form-datetime-btn input-group-addon">';
                        	htmlcode += '<i class="icon icon-WCR_Calendar" />';
                    		htmlcode += '</label>';
                        	htmlcode += '</div></div>';
                        	htmlcode += '<div class="form-time">';
                        	htmlcode += '<div class="input-group">';
                        	htmlcode +=  this.displayHourDropdown(halfhours,hourName,hourName,changeMethod,cellObj.styleObj);
                        	htmlcode += '</div></div></div>';        
                        } else {
                        	htmlcode += '<span class="ReadOnlyAttr">';
                            if (!this.showFakedReadOnly) {
                                if (attValue) {
                                	if(!(attValue instanceof Date)){
                                		attValue = this.correctForDaylightSaving(new Date(parseInt(attValue)));
                                	}
                                    htmlcode += WCHTML.getDateHourString(attValue);	
                                    // DT110156: hour was not submitted so you fell back on 0:00. Date is submitted because you called renderHiddenAttributeInfo already
                                    var theName2 = this.prefix + 'date_period_of_day_' + attObj.id;
                                    var attValue2 = attValue.getHours() * 2 + Math.floor(attValue.getMinutes()/30);	
                                    htmlcode += '<input type="hidden" name="'+theName2+'" value="'+attValue2+'">';			
                                }		

                            } else {
                                htmlcode += 'May, 24 2010 18:00';
                            }
                            htmlcode += '</span>';
                        }
                    } else if(attObj.type && attObj.type.toLowerCase() == 'richtext'){
                        var rowCnt = (cellObj.rowSize)? cellObj.rowSize : 120;
                        var theName = this.prefix + attObj.id;
                        //inputGroup.removeClass("input-group");
                        if (editable) {
                            changeMethod = this.functionPrefix + '.changeText(this,' + row + ',' + col+');return false;';
							htmlcode += "<div class='main-content'><div class='RichTextEdit'>";
                            htmlcode += '<textarea class="form-control" style="width: 100%;height: ' + rowCnt + 'px;" type="text" id="' + theName + '" name="' + theName + '" onchange="' + changeMethod + '" ';
                            htmlcode += this.renderStyle(cellObj.styleObj,"InputField",0,hasButton);
                            htmlcode += '>'+ WCXML.xmlencode(attValue)+'</textArea></div>';
                            htmlcode += '<a href="#" '+this.renderStyle(cellObj.styleObj,"Value", undefined, undefined, "hidden-print")+' onclick="'+this.functionPrefix+'.edittmce(\''+theName+'\'); return false;">'+ACatStrings.t('ACatShowAdvancedControls')+'</a>';
   							htmlcode += "</div>";
                            tinyMceMgrObj.tmceEditors[this.id].push(theName);
                            tinyMceMgrObj.rows[attObj.id] = row;
                            tinyMceMgrObj.cols[attObj.id] = col;
                        }else{
                            if (!this.showFakedReadOnly) {
                                var divheight = rowCnt;
                                var value = attValue;

                                htmlcode += '<div class="ReadOnlyRichText"';
                                if (!this.printLayout) {
                                    htmlcode += ' style=" max-height: ' + divheight + 'px; ';
                                    if (this.currentView != 'Header'){
                                    	htmlcode +=  'height: ' + divheight + 'px;';
                                    } else { 
                                    	htmlcode +=  'height: expression( this.scrollHeight > '+(divheight-1)+' ? \''+divheight+'px\' : \'auto\' );';
                                    }
                                    htmlcode += '"';
                                }
                                htmlcode += '>';
                                htmlcode += value;
                                htmlcode += '</div>';
                                
                                // hidden value
                                htmlcode += '<input type="hidden" name="'+theName+'" value="'+WCXML.xmlencode(attValue)+'">';
                            }else{
                                htmlcode += '<div style="width: 100%; max-height: 120px; min-height: 16px; overflow: auto; text-align: left; margin: 0px 0px 0px 0px; ">Ille ego qui <b>fuerim</b>, tenerorum lusor <i>amorum</i>, <br>quem legis, ut noris, <u>accipe posteritas</u>.</div>';
                            }
                        }

                    } else if(attObj.fractionalIfImperial && this.userUnits && this.userUnits.toLowerCase() == "imperial"){
                        attValue = WCXML.xmlencode(attValue);
                        var displayValue = attValue;
                        if (displayValue) {
                            displayValue = WCHTML.getFloatString(attValue,6);
                            displayValue = WCHTML.decimalToFractional(displayValue);
                        }

                        if (editable) {                        
                            var theName = this.prefix + attObj.id;
                            changeMethod = this.functionPrefix + '.changeText(this,' + row + ',' + col+')';

                            // field for fractional value
                            htmlcode += '<input type="text" class="form-control" value="' + displayValue + '" id="' + theName + '_disp" name="' + theName + '_disp" onchange="' + changeMethod + '" ';						
                            htmlcode += this.renderStyle(cellObj.styleObj,"InputField",0,hasButton);
                            htmlcode += '>';

                            // hidden field with real value
                            htmlcode += '<input type="hidden" value="' + attValue + '" id="' + theName + '" name="' + theName + '">';
                        } else { // not editable
                        	htmlcode += '<span class="ReadOnlyAttr">';
                            if (!this.showFakedReadOnly) {
                            	var theName = this.prefix + attObj.id;
                                htmlcode += displayValue;                            
                             // hidden field with real value
                                htmlcode += '<input type="hidden" value="' + attValue + '" id="' + theName + '" name="' + theName + '">';
                            } else {
                                htmlcode += '1 + 1/4';
                            }
                            htmlcode += '</span>';
                        }

                    } else {
                        // ALL THE OTHERS: units, single line text, text area, Float ...

                        attValue = WCXML.xmlencode(attValue);
                        var displayValue = attValue;
                        var type2;  
                        type2 = attObj.type;
                        if (type2) {
                            type2 = type2.toLowerCase();
                        } else {
                            type2 = 'text';
                        }
                        if (attValue && type2 != 'integer' && type2 != 'number' && type2!= 'text' && type2 != 'float') displayValue = WCHTML.getFloatString(attValue,6);
                        if (attValue && type2 == 'float') displayValue = parseFloat(attValue).toString();  // normalized display of floats - DT119585, this will make 25.00000 show as 25 while not changing the actual value, see DevTrack for advanced reasoning.

                        var theName = this.prefix + attObj.id;
                        if (editable) {                        
                            changeMethod = this.functionPrefix + '.changeText(this,' + row + ',' + col+')';

                            if(cellObj.rowSize && parseInt(cellObj.rowSize) > 1) {
                                htmlcode += '<textarea class="form-control" id="' + theName + '" name="' + theName + '" class="' + this.valueClass + '" rows="' + cellObj.rowSize + '" onchange="' + changeMethod + '" ';
                                htmlcode += this.renderStyle(cellObj.styleObj,"InputField",0,hasButton);
                                htmlcode += ' >';
                                htmlcode += attValue;
                                htmlcode += '</textarea>';
                            } else {
                                var sDropStyle = false;
                                if (attObj && attObj.listObj && attObj.listObj.loadingType == 'OnTheFly') sDropStyle = true;
                                var blurMethod = this.functionPrefix + '.blurText(this,' + row + ',' + col+')';

                                
								if (sDropStyle) {
									changeMethod = this.functionPrefix + '.changeDBInputValue(this,' + row + ',' + col+')';
								}
                                htmlcode += '<input type="text" class="form-control" value="' + displayValue + '" id="' + theName + '" name="' + theName + '" onchange="' + changeMethod + '" onblur="' + blurMethod + '" ';
                                htmlcode += this.renderStyle(cellObj.styleObj,"InputField",0,hasButton || sDropStyle);
                                if (sDropStyle) {
                                	var noEnter = ' onkeydown="if((event.keyCode ? event.keyCode : event.which ? event.which : event.charCode) == 13){event.preventDefault();}"';
                                    var keyUpMethod = this.functionPrefix + '.getDBValues(this,event,' + row + ',' + col+ ')';
                                    htmlcode += ' onkeyup="'+keyUpMethod+'" ';
                                    htmlcode += noEnter;
                                    htmlcode += ' autocomplete = "off" '; 
                                }
                                htmlcode += '>';
                                if (sDropStyle) {
                                    var clickMethod = this.functionPrefix + '.getDBValues(this,event,' + row + ',' + col+ ')';
                                    htmlcode += '<label class="input-group-addon dropdown-button" onclick = "'+clickMethod+'"><i class="icon icon-WCR_Down-S"></i></label>';
                                }

                            }
                        } else { // not editable
                        	htmlcode += '<span class="ReadOnlyAttr">';
                            if (!this.showFakedReadOnly) {

                                htmlcode += displayValue;                            
                                htmlcode += '<input type="hidden" value="' + displayValue + '" id="' + theName + '" name="' + theName + '" />';	

                            } else {
                                if (cellObj.rowSize && parseInt(cellObj.rowSize) > 1) {
                                    // this probably never happens since we don't allow rowsize anymore for header view
                                    htmlcode += ' <div align="left">Ille ego qui fuerim, tenerorum lusor amorum, <br>quem legis, ut noris, accipe posteritas.</div>';
                                } else {
                                    htmlcode += 'Esko';
                                }
                            }
                            htmlcode += '</span>';
                        }

                    }
                }
                
                htmlcode += this.renderUnitLabel(cellObj, editable);
                
                if (this.currentView != 'Header') {  // units are in header view stuck immediately after the value
                    if (cellObj.buttons && this.isAttrCatEditable()){
                    	var btnhtmlcode = this.renderButton(cellObj,row,col,true, forPrint);  
                    	
                    	// make sure button is not taking the width of the input field 
                    	var containerInputGroup = $('<div class="input-group" '+this.renderStyle(cellObj.styleObj,"BtnContainer")+' />');
                    	formEntry.append(containerInputGroup.append(inputGroup).append(btnhtmlcode));
                    }
                }
                
                inputGroup.html(htmlcode);
                return formEntry[0].outerHTML;
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderAttributeValueInternal:\n' + err);
        }
        return '';
    };


    /**
    * @param leftCorner determines whether you need to put a left corner cell
    * @param rightCorner determines whether you need to put a right corner cell
    */
    attributeCategory.prototype.renderHeaderBar = function(colSpan) {
        var htmlcode = '';

        if (!colSpan){
        	colSpan=1;
        }

        for (var col=0; col<colSpan; col++) {
            htmlcode += '<td align="left" class="attributesGridLayoutHeader attributesGridLayoutHeader-first">' + ACatStrings.t('ACatName')+ '</td>';					
            htmlcode += '<td align="left" class="attributesGridLayoutHeader attributesGridLayoutHeader-last">' + ACatStrings.t('ACatValue')+ '</td>';
        }

        return htmlcode;
    };

    /**
    * This function creates a calendar object.
    * @param  idName id of the input field
    * @param  idHour id of the selection of period in the day
    */
    attributeCategory.prototype.createCalendar = function(idName, idHour) {
        if ($(idName) !=null )
        {
            $(idName).datepicker(
            {
                dateFormat: "M dd, yy",
                onClose: function(dateText, inst)
                {
                    /*
                    var element = $(idHour)[0];
                    if(element && element.selectedIndex == 0) element.selectedIndex = (36 + 1);
                    */
                }
            });
        }
    };

    /**
    * creates HTML code for a dropdown box for restricted sets
    * @param cellObj: cell containing the attribute with restricted set
    * @param row: row index of the cell
    * @param col: col index of the cell
    */
    attributeCategory.prototype.renderSelect = function(cellObj,row,col) {
    
        var htmlcode = '';
        try {
            var rsObj = cellObj.attObj.rs;
            if (!rsObj || !rsObj.values) {
            	return '<span>&nbsp;</span>'; // should never occur
            }
            var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+')';
            var theName = this.prefix  + cellObj.attObj.id;

            var type2 = cellObj.attObj.type;
            if (type2) {
                type2 = type2.toLowerCase();
            } else {
                type2 = 'text';
            }
            var attValue = cellObj.attObj.value;
            if (attValue && type2 != 'integer' && type2 != 'number' && type2 != 'text' && type2 != 'float' && type2 != 'datetime') attValue = WCHTML.getFloatString(attValue,6);
            if (attValue && type2 == 'float') attValue = parseFloat(attValue).toString();  

            htmlcode += '<select class="form-control" onchange="' + changeMethod + '" id="' + theName + '" name="' + theName + '" ';
            htmlcode += this.renderStyle(cellObj.styleObj,"Dropdown");

            htmlcode += '>';
            htmlcode += '<option value="">---' + ACatStrings.t('ACatPleaseSelect') + '---</option>';
            var found = false; // did we find the selected value in the list of options
            for (var option=0; option < rsObj.values.length; option++) {
                var value = rsObj.values[option].value;
                var valueDisplay = value;
                if (cellObj.attObj.type.toLowerCase() == 'datetime') valueDisplay= WCHTML.getDateHourString(this.correctForDaylightSaving(new Date(parseInt(value))));		
                if(cellObj.fractionalIfImperial && this.userUnits && this.userUnits.toLowerCase() == "imperial"){
                    valueDisplay = WCHTML.decimalToFractional(value);
                }
                htmlcode += '<option value="'+ WCXML.xmlencode(value) +'"';
                if (attValue == value) {htmlcode += ' selected '; found=true;};
                htmlcode += '>' + WCXML.xmlencode(valueDisplay) + '</option>';
            }
            if (cellObj.attObj.value && !found) {
                var value = cellObj.attObj.value;
                var valueDisplay = value;

                if (cellObj.attObj.type.toLowerCase() == 'datetime') valueDisplay= WCHTML.getDateHourString(this.correctForDaylightSaving(new Date(parseInt(value))));		
                htmlcode += '<option value="'+ WCXML.xmlencode(value) +'" selected ';            
                htmlcode += '>' + WCXML.xmlencode(valueDisplay) + ' (' + ACatStrings.t('ACatDeleted') + ')</option>';
            }
            htmlcode += '</select>';
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderSelect:\n' + err);
        }
        return htmlcode;
    };

    attributeCategory.prototype.correctAttributeRestricted = function(attObj) {
        try {
            var currentValueArr = attObj.value.split(this.multiSep);
            var attObj2 = attObj.restrictToAttObj;
            var newValue = '';
            if (attObj2) {
                currentValueArr2 = attObj2.value.split(this.multiSep);
                for (var idx = 0 ; idx<currentValueArr.length; idx++) {
                    for (var idx2 = 0 ; idx2<currentValueArr2.length; idx2++) {
                        if (currentValueArr[idx] == currentValueArr2[idx2]) break;
                    }
                    if (idx2 < currentValueArr2.length) {
                        if (newValue != '') newValue += this.multiSep;
                        newValue += currentValueArr[idx];
                    }
                }

            }
            if (newValue != attObj.value) {
                attObj.value = newValue;
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderSelect:\n' + err);
        }

    };

    /**
    * creates HTML code for a dropdown box for restricted sets
    * @param cellObj: cell containing the attribute with restricted set
    * @param row: row index of the cell
    * @param col: col index of the cell
    */
    attributeCategory.prototype.renderSelectMultiple = function(cellObj,row,col,editable) {
        var htmlcode = '';
        try {
            var attObj = cellObj.attObj;    
            var listObj = attObj.listObj;
            var valid;


            if (attObj.restrictType == 'Attribute') {
                this.correctAttributeRestricted(attObj);    // throw out all values which are no longer allowed. TO DO : move to where the change is actually done instead of doing this each time a repaint is done
                if (!attObj.restrictToAttObj || attObj.restrictToAttObj.value=='') return '<span>&nbsp;</span>';  // show nothing in case of attribute restriction with an empty father attributes					
            }

            if (this.currentView != 'Header') {

                var currentValueArr = attObj.value.split(this.multiSep);	
                var currentIndexArr = null;
                var clickMethod = this.functionPrefix + '.showMultiValueDropdown(this,event,'+row+','+col+'); ';
                if (listObj && listObj.loadingType == 'OnTheFly') clickMethod = this.functionPrefix + '.getDBValues(this,event,'+row+','+col+'); ';
                try {
                    if (attObj.columnObj) {
                        if (!attObj.columnObj.validArr || (attObj.listObj && attObj.listObj.updateValuesNeeded) ) this.updateListAttributeValues(attObj.listObj, false);
                        currentIndexArr = attObj.columnObj.currentIndex.split('~');
                    }
                } catch(err) {
                    // any reason why currentIndexArr cannot be calculated, for example it's not a list or it has no value, or value -1 (an integer) etc
                    // do nothing --> no currentIndexArr --> always valid
                }

                htmlcode += '<div class="form-div main-content" id="mvdiv_'+ this.prefix + attObj.id+'" style="vertical-align:top; padding: 0 0 0 0;';
                if (editable || currentValueArr.length>4){
                    htmlcode += 'border:1px solid #cccccc;background-color:white;';
                }
                if (currentValueArr.length>4 && !this.printLayout) htmlcode+='height:80px; overflow-y:auto;';
                htmlcode+= '"';
                if (editable) htmlcode+= ' onclick="'+clickMethod+'"';
                htmlcode+= '><table padding="0">';

                var fieldType = 'MultiValue';
                if(!editable){
                    fieldType = 'Value';
                }

                if (attObj.value != "") {
                    for (var idx=0; idx< currentValueArr.length; idx++) {
                        //valid = currentIndexArr? (attObj.columnObj.validArr[currentIndexArr[idx]]) : true;

                        if (currentIndexArr && attObj.columnObj.validArr) {
                            valid = attObj.columnObj.validArr[currentIndexArr[idx]];
                        } else {
                            valid=true;

                        }

                        htmlcode += '<tr><td class="darktext_greenlink_11" ' + this.renderStyle(cellObj.styleObj,fieldType) + '><nobr>'+WCXML.xmlencode(currentValueArr[idx])+'&nbsp;';

                        if (!valid) htmlcode += WCXML.xmlencode(ACatStrings.t('ACatNotValid'));
                        htmlcode += '</nobr></td></tr>';  // the &nbsp; is needed for the case of empty (attObj.value = " "). 
                    }
                } else if (editable) {
                    htmlcode += '<tr><td class="darktext_greenlink_11"  ' + this.renderStyle(cellObj.styleObj,fieldType) + '><nobr>---'+ACatStrings.t('ACatPleaseSelect')+'---</nobr></td></tr>';	
                }
                htmlcode += '</table></div>';

                var theName = this.prefix  + attObj.id;
                if (editable) {
                    // drop icon
                	htmlcode += '<div class="input-group-addon dropdown-button" onclick="'+clickMethod+'">';
                	htmlcode += '<i id="'+theName+'_popUpIcon" class="icon icon-WCR_Down-S"></i>';
                    htmlcode += '</div>';
                }

                    // hidden element
                    htmlcode += '<input type="hidden" id="' + theName +'" name="' + theName +'" value="'+WCXML.xmlencode(attObj.value)+'">';
            } else {
                htmlcode += WCXML.xmlencode(attObj.value);
            }



        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderSelectMultiple:\n' + err);
        }
        return htmlcode;
    };


    /**
    * opens a pop-up with the values of a restricted set or a list
    */
    attributeCategory.prototype.showMultiValueDropdown = function(parentObj,event,row,col,showAll) {
        try {
            var htmlcode = '';
            // find the values
            var cellObj = this.cells[row][col];		// should always exist if we get here
            var isMobileMultiValue = WCHTML.isMobile();  // when you are here, you are always multi-value

            if(this.checkVisibility(cellObj) != "ReadOnly"){
                var attObj = cellObj.attObj;
                if (isMobileMultiValue) htmlcode += '<div id="dbValuesDivAround" style = "height:250px; overflow-y:auto;">';
                htmlcode += '<table id="multiValueDropdownTable">';
                var clickMethod = this.functionPrefix + '.selectAllMultiValue(); return false; ';
                htmlcode += '<tr><td class="darktext_greenlink_11" ><a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatSelectAll')+'</a></td></tr>';
                var clickMethod = this.functionPrefix + '.deselectAllMultiValue(); return false; ';
                htmlcode += '<tr><td class="darktext_greenlink_11"><a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatDeselectAll')+'</a></td></tr>';
                var clickMethod = this.functionPrefix + '.showAllMultiValue(\''+attObj.id+'\'); return false; ';
                htmlcode += '<tr><td class="darktext_greenlink_11"><a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatShowUnfiltered')+'</a></td></tr>';

                htmlcode += '<tr><td class="darktext_greenlink_11" id="multiValueDropdownTableInner">';
                htmlcode += this.multiValueDropdownHTML(attObj,cellObj);
                htmlcode += '</td></tr>';

                htmlcode += '</table>';
                if (isMobileMultiValue) { // add an OK button
                    htmlcode += '</div>';
                    htmlcode += '<div style="width: 95%; text-align: center; padding: 6px;"><input type="button" class="buttonsecondary" value="OK" onclick="hidemenu()"></div>';
                }

            var menuItems = [];  // dropdownmenu expects and array of html code snippets, so we give it an array of 1 element
            menuItems.push(htmlcode);	
            var target = $(parentObj).closest(".input-group").find(".form-control, .form-div")[0] || parentObj;
            var menuWidth = '350px', menuHeight = '500px',scrollHor = false, nohide=false, bgcolor = "#ffffff",nohideonclick=true;
            dropdownmenu(target,event,menuItems,menuWidth,menuHeight,scrollHor,nohide,bgcolor,nohideonclick) ;	
            var aCatObj = this;  // closure
            $("#dropmenudiv").css({
            	'min-width': $(target).width()+'px',
            	'margin-top': '-1px'
            });
            updateHoverPosition(target);
            dropmenuobj.onchange = function(event) {
                aCatObj.changeSelectMultiple(row,col); 			
            };

            // prevent the internal of the dropdown menu to trigger the onmouseout/onchange of the enclosing div by giving it its own onmouseout which stops the event propagation
            document.getElementById("multiValueDropdownTable").onchange = function(event){ if (event) event.stopPropagation();};
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.showMultiValueDropdown' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }


    };

    /**
    * Called when selection of a multi-value attribute is changed
    * @param row {int} row of the cell containing the multivalue attribute
    * @param col {int} column of the cell containing the multivalue attribute
    * @param imageList {Boolean} indicates that this function is called from an image list with checkboxes
    */
    attributeCategory.prototype.changeSelectMultiple = function(row,col,imageList) {
        try {
            var cellObj = this.cells[row][col];		// should always exist if we get here
            if (!cellObj) return;
            var attObj = cellObj.attObj;  
            if (!attObj) return;
            
            var restrictedChanged = false;  // remembers whether a part of a cascading dropdown was changed
			var otherValueChosen = false; // whether one of the values 'is not valid'
			var hashArr = [];

            if (attObj.listObj && attObj.listObj.loadingType == 'OnTheFly' && dropmenuobj && dropmenuobj.style.visibility != 'hidden') return;

            // figure out what was selected
            var newValue = '';
            var checkArr;
            if (imageList) {
                checkArr = document.getElementsByName(this.prefix + 'ImgChk_' + attObj.id);	
            } else {
                //JANH CRR - This selects all Multivalue selects in all categories on the page! 
                // Will this work correctly in document references context too?
                checkArr = document.getElementsByName('MultiValueSelect');
            }
            if (checkArr && checkArr.length>0) {				
				// first accept the normal values
                for (var idx=0; idx<checkArr.length; idx++) {
                    if (checkArr[idx].checked && checkArr[idx].id.indexOf('_other_') == -1) {
                        
						if (newValue) newValue += this.multiSep;						
						newValue += WCXML.xmldecode(checkArr[idx].value);// values were encoded for inline html concatenation, so decode again
						hashArr[checkArr[idx].value] = true;						
                    }
                }
				// now accept the 'other' values and check whether people did not enter something already selected
				// hint: 'other values' only exist for 'OnTheFly' lists
				for (var idx=0; idx<checkArr.length; idx++) {
                    if (checkArr[idx].checked && checkArr[idx].id.indexOf('_other_') != -1) {                   
						
						// go reading the input field with the same index
						var idInput = '#mv_other_' + checkArr[idx].id.substr('mvchk_other_'.length);
						otherValue = $(idInput).val();
						if (otherValue != '' && !hashArr[otherValue]) {
							if (newValue) newValue += this.multiSep;						
							newValue += otherValue;	
							otherValueChosen = true;
						}
                    }
                }
            }	
            
            // see whether the value has changed
            if (newValue != attObj.value) {			
                if (attObj.listObj && attObj.listObj.loadingType == 'Immediate' ) {
                    this.changeListSelection(attObj,newValue,document.getElementById('multiValueDropdownTable'),row,col);
                    restrictedChanged = true;
                }
//JANH CRR original code causing troubles when the attObj.columnObj did not exist. 
// Question to HADW: Is it a valid situation here that the attObj.columnObj doesn't exist?
//            	attObj.columnObj.otherValueChosen = otherValueChosen;
                
                attObj.value = newValue;
                if (attObj.columnObj) {
                	attObj.columnObj.autoSelected = false;
                	attObj.columnObj.otherValueChosen = otherValueChosen;   //JANH CRR - the above moved here
    				if (attObj.columnObj.uniqueIdentifyer) {
                		attObj.combiIdx = this.findCombiIdx(attObj.listObj, attObj.columnObj, -1, attObj.value);
                	}
                } 

                if (imageList) {
                    document.getElementById(this.prefix + attObj.id).value = newValue;
                } else {
                    // repaint the element itself
                    $("#"+this.prefix + 'td_'+row+'_'+col).html(this.renderAttributeValueInternal(cellObj, attObj,row,col,cellObj.visibility, true));				
                }


                var elem = document.getElementById(this.prefix  + attObj.id);
                if (elem) {
                	this.changeText(elem,row,col);  // typically not called for a table based document link
                } else {
                	this.executeButtonDependencies(attObj);
                }

                if (attObj.listObj && attObj.listObj.loadingType == 'OnTheFly' ) this.checkAutoSelectLazyLoading(attObj,false);  // auto-selects single remaining

                // check if a global repaint is needed
                if (restrictedChanged) {
                    this.displayAttributeCategory();  // always repaint if this is a dropdown which is part of a list
                } else {
                    this.checkRepaintAttributeCategory(attObj);   // only repaint if a rule is applied to this cell
                }

            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.changeSelectMultiple' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }

    };

    /**
    * should only be called for unique identifyer columns
    * determines the current indices in the combination array of the list based on the value provided for the unique identifyer
    */
    attributeCategory.prototype.findCombiIdx = function(listObj, columnObj, currentIndex, value) {
        var foundIdx = '';
        try {
            if (currentIndex == -1) currentIndex = this.findColumnIndex(columnObj, value);
            if (currentIndex == -1) return '';

            var idxArr = currentIndex.split("~");


            // find which column this is about
            for (var column=0; column<listObj.columns.length;column++) {
                if (columnObj == listObj.columns[column]) break;
            }

            for (var idx=0; idx<idxArr.length; idx++) {	
                for (var combi=0; combi<listObj.combinations.length; combi++) {
                    combiString = listObj.combinations[combi];
                    idxArr2 = combiString.split("~");
                    thisColIdx = idxArr2[column];
                    if (thisColIdx == idxArr[idx]) {
                        if (foundIdx) foundIdx += '~';
                        foundIdx += combi;
                        break;
                    }
                }


            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.findCombiIdx' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }
        return foundIdx;
    }

    /**
    * creates HTML code to be put in the dropdownmenu having all items. 
    * The invalid items are still painted but with display:none, allowing a show all option to just make them visibly by changing the visibility of the rows
    */
    attributeCategory.prototype.multiValueDropdownHTML = function(attObj,cellObj) {
        var htmlcode='';
        try {
            var currentValueArr = attObj.value.split(this.multiSep);
            var valueArr;
            var controlType = 'checkbox';


            htmlcode += '<table class="wcr-form">';				
            if (attObj) {
                var rsObj = attObj.rs;
                var columnObj = attObj.columnObj;			
                var rsType = attObj.restrictType;


                switch (rsType) {
                    case 'RestrictedSet':
                        if (rsObj) valueArr = rsObj.values;
                        break;
                    case 'List':
                        if (attObj.columnObj) valueArr = columnObj.values;
                        break;
                    case 'Attribute':
                        var attObj2 = attObj.restrictToAttObj;
                        if (attObj2) valueArr = attObj2.value.split(this.multiSep);
                        break;
                    case 'DocumentReference':
                        valueArr = cellObj.docList;
                        if (!attObj.allowMultipleValues) controlType = 'radio';
                        break;
                }

                if (valueArr) {
                    for (var idx=0;idx<valueArr.length;idx++) {
                        var curVal = (rsType == 'RestrictedSet') ? valueArr[idx].value : valueArr[idx];
                        if (!curVal) continue; //don't show empty strings
                        // figure out whether this value is currently selected
                        var displayVal =  curVal;
                        if (rsType == 'DocumentReference') {displayVal =  WCXML.xmlencode(valueArr[idx].name); curVal = valueArr[idx].versionId};
                        var selected = false;
                        for (var idx2=0 ; idx2<currentValueArr.length; idx2++) {
                            if (currentValueArr[idx2] == curVal) {selected=true; break;};
                        }

                        var showItalic = false;
                        var showInvalid = false;
                        if (columnObj && columnObj.validArr && !columnObj.validArr[idx]) {
                            if (selected) showInvalid=true;						
                            showItalic = true;
                        }

                        curVal = WCXML.xmlencode(curVal);

                        htmlcode += '<tr class="darktext_greenlink_11" name="MultiValueSelectTr" ';
                        if (showItalic) htmlcode += 'style="display:none;" ';
                        htmlcode += '><td><div class="form-entry"><div class="'+controlType+'"><label for="mvchk_' + idx + '"><input type="'+controlType+'" id="mvchk_' + idx + '" value="'+curVal+'" name="MultiValueSelect"';
                        if (selected) htmlcode += ' checked="checked" ';						
                        htmlcode += '>';
                        if (showItalic) htmlcode += ' <i><font color="#c0c0c0">';
                        htmlcode += displayVal;
                        if (showItalic) htmlcode += ' </font></i>';
                        htmlcode +='</label>';						
                        if (showInvalid) htmlcode += '&nbsp;'+ WCXML.xmlencode(ACatStrings.t('ACatNotValid')) ;
                        htmlcode += '</div></div></td></tr>';
                        if (rsType == 'DocumentReference') {						
                            htmlcode += '<tr><td/><td id="popUpDetails_'+idx+'"></tr>';						
                        }


                    }

                } 
            }
            htmlcode += '</table>';

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.multiValueDropdownHTML' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }	
        return htmlcode;

    };

    attributeCategory.prototype.selectAllMultiValue = function(elemName,row,col) {
        try {
            var checkArr;
            if (elemName) {
                checkArr = document.getElementsByName(elemName);
            } else {
                checkArr = document.getElementsByName('MultiValueSelect');
            }
            if (checkArr && checkArr.length>0) {
                for (var idx=0; idx<checkArr.length; idx++) {
                    var checkElem = checkArr[idx];
                    var trElem = checkElem.parentNode.parentNode;
                    if (trElem.style.display != 'none') {  // only select what is visible
                        checkElem.checked = true;
                    }
                }

            }
            if (elemName) this.changeSelectMultiple (row,col,true);
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.selectAllMultiValue' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }
    };

    attributeCategory.prototype.deselectAllMultiValue = function(elemName,row,col) {
        try {
            var checkArr;
            if (elemName) {
                checkArr = document.getElementsByName(elemName);
            } else {
                checkArr = document.getElementsByName('MultiValueSelect');
            }

            if (checkArr && checkArr.length>0) {
                for (var idx=0; idx<checkArr.length; idx++) {
                    checkArr[idx].checked = false;
                }

            }
            if (elemName) this.changeSelectMultiple (row,col,true);
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.selectAllMultiValue' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }
    };

    /**
     * Generate and render HTML for a row with checkbox and text field for multi-select dropdown
     */
    attributeCategory.prototype.otherMultiValue = function() {
			var idx = 0;
			// find highest index
			idx = $( "input[name='OtherValueInput']" ).length;
			
		var htmlcode = '<tr><td class="checkboxWidth"><input type="checkbox" checked="checked" id="mvchk_other_'+ idx +'" name="MultiValueSelect"></td>';
			htmlcode += '<td><input type="text" id="mv_other_' + idx + '" name="OtherValueInput"></td></tr>';
			// put it before the first value
			$("#mvselect_valuetable > tbody > tr:first").before(htmlcode);               
			$("#mv_other_" + idx).focus();
    };

    attributeCategory.prototype.deselectAllSingle = function(elemName,row,col) {
        try {
            var checkArr;
            if (elemName) {
                radioArr = document.getElementsByName(elemName);
                if (radioArr && radioArr.length >0) {
                    for (var idx=0; idx<radioArr.length; idx++) {

                        radioArr[idx].checked = false;
                    }
                    this.changeSelect (radioArr[0],row,col,true);
                }
            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.deselectAllSingle' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }
    };
    /**
    * executed after clicking on show all 
    */
    attributeCategory.prototype.showAllMultiValue = function(attId) {
        try {


            $('tr[name=MultiValueSelectTr]').each( function(index) {
                $(this).show();            
            });

            // DT125934: by doing the above, the hover will expand potentially very big if it wasn't limited to 350 pixels before (which is the case when only few items were valid)
            $(dropmenuobj).height(350);
            $(dropmenuobj).css('overflow-y', 'scroll');
            var objId= this.prefix + attId + '_popUpIcon';
            var obj = document.getElementById(objId); 
            if (obj) updateHoverPosition( obj, 0, 0);

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.selectAllMultiValue' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }
    };



    /**
    * Sets showAll parameter on the attribute determined by row,col to the value of showAll and repaints the applicable cell
    */
    attributeCategory.prototype.showAllImages = function(row,col,visibility,showAll) {
        try {		
            var cellObj = this.cells[row][col];
            var attObj = cellObj.attObj;
            attObj.showAll = showAll;
            var elem = document.getElementById('td_'+row+'_'+col);
            if (elem) elem.innerHTML = this.renderAttributeImages(cellObj,row,col,visibility);


        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.showAllImages' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }
    };

    /**
    * EXPERIMENTAL
    * renders an item list in the cell determined by cellObj by putting a table in there with all columns of the list. 
    * Might get extra parameters later to determine which ones are really shown
    * TO DO Decide whether to throw this away
    */
    attributeCategory.prototype.renderItemList = function(cellObj,row,col,editable) {
        var htmlcode = '';
        try {
            var attObj = cellObj.attObj;
            var listObj = attObj.listObj;
            if (!attObj || !listObj) return '';
            var col;


            htmlcode += '<td ';
            if (this.currentView != 'Header') htmlcode += ' id="' + this.prefix + 'td_' + row + '_' + col + '"';  
            htmlcode += this.renderStyle(cellObj.styleObj,"Value");

            // determine colspan
            var colSpan = 1;
            if (cellObj.colSpan >1) colSpan =  cellObj.colSpan * 2 - 1 ;
            colSpan ++;  // to cater for the label column        
            htmlcode += ' colspan="' + colSpan +  '" ';

            htmlcode += '>';		

            htmlcode += '<table id="itemList_'+attObj.id+'" style="width:100%;" class="ordtable darktext_greenlink_11" >';
            // Paint the header		
            htmlcode += '<th/>';  // column for the delete icon
            for (col=0; col<listObj.columns.length; col++) {
                htmlcode += '<th>' + listObj.columns[col].columnName + '</th>';		
            }
            // Paint the data
            var idxArr = new Array();
            if (attObj.combiIdx) idxArr= attObj.combiIdx.split("~");  // unique identifyers have a combiIdx field like 123~349 saying which unique combinations it represents
            for (var idx=0; idx< idxArr.length; idx++) {

                var combination = idxArr[idx];
                var colIdxArr = listObj.combinations[combination].split("~");
                htmlcode += '<tr>';

                var clickMethod = '';			
                    htmlcode += '<td><img src="'+window.webcenterConfig.imageBaseUrl+'delete_icon_15.gif" onclick="'+clickMethod+'"/></td>';		

                for (var col=0;  col<listObj.columns.length; col++) {
                    htmlcode += '<td>' + listObj.columns[col].values[colIdxArr[col]] + '</td>';					
                }
                htmlcode += '</tr>';
            }

            htmlcode += '</table>';
            htmlcode += '</td>';

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.renderItemList' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }
        return htmlcode;
    }

    /**
	* EXPERIMENTAL - Author HADW
    * renders an item list in a text box with a select button and a clear button. 
    * Might get extra parameters later to determine which ones are really shown
    * TO DO Decide whether to throw this away
    */
    attributeCategory.prototype.renderUniqueIdentfyerBox = function(cellObj,row,col,editable) {
        var htmlcode = '';
        try {
            var attObj = cellObj.attObj;
            var listObj = attObj.listObj;
            if (!attObj || !listObj) return '';
            var col;



            htmlcode += '<table><tr>';
            htmlcode += '<td><textarea id="textArea_'+attObj.id+'" cols="40" rows="6">';
            // Paint the data
            var idxArr = new Array();
            if (attObj.combiIdx) idxArr= attObj.combiIdx.split("~");  // unique identifyers have a combiIdx field like 123~349 saying which unique combinations it represents
            for (var idx=0; idx< idxArr.length; idx++) {
                var combination = idxArr[idx];
                var colIdxArr = listObj.combinations[combination].split("~");
                if (col>0) htmlcode += '\n';
                for (var col=0;  col<listObj.columns.length; col++) {
                    if (col>0) htmlcode += ' ~ ';
                    htmlcode += listObj.columns[col].values[colIdxArr[col]];					
                }

            }
            htmlcode += '</textarea></td>';
            var clickMethod = this.functionPrefix + '.showCascadingTreeDialog(this,' + row + ',' + col+')';
            htmlcode += '<td><input type="button" value="Select" class="buttonsecondary" onclick="'+clickMethod+'"></td>';

            htmlcode += '</tr></table>';

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.renderUniqueIdentfyerBox' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );
        }
        return htmlcode;
    }



    /**
    * creates HTML code for a checkbox
    * @param cellObj: cell containing the attribute with restricted set
    * @param row: row index of the cell
    * @param col: col index of the cell
    */
    attributeCategory.prototype.renderCheckbox = function(cellObj,row,col,editable) {
    
        var htmlcode = '';
        try {
            var changeMethod = this.functionPrefix + '.changeCheckbox(this,' + row + ',' + col+')';
            var theName = this.prefix  + cellObj.attObj.id;
            var attObj = cellObj.attObj;
            if (!attObj) return '';
            
            var align = 'style="text-align: ';
            if(cellObj.styleObj && cellObj.styleObj.valueStyleObj && cellObj.styleObj.valueStyleObj.alignment){
            	align += cellObj.styleObj.valueStyleObj.alignment;
            } else if (cellObj.styleObj && !cellObj.styleObj.valueStyleObj && cellObj.styleObj.alignment) {
            	align += cellObj.styleObj.alignment;
            } else {
            	align += 'left';
            }
            align += ';"';

            htmlcode += '<label class="main-content" ' + align + '><input type="checkbox" onclick="' + changeMethod + '" id="chkbox_' + cellObj.attObj.id + '" ';
            htmlcode += this.renderStyle(cellObj.styleObj,"Checkbox");
            if (attObj.value == 'yes') {
                htmlcode += ' checked ';
            }
            if (!editable) htmlcode += ' disabled ';
            htmlcode += '/></label>';
            htmlcode += '<input type="hidden" id="' + theName + '" name="' + theName + '" value="'+ WCXML.xmlencode(attObj.value)+'">';

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderCheckbox:\n' + err);
        }
        return htmlcode;
    };

    /**
    * Called when a checkbox is changed
    */
    attributeCategory.prototype.changeCheckbox = function(elem,row,col) {
        var attObj = null;
        try {
            var cellObj = this.cells[row][col];
            if (cellObj) {
                this.setAttributesChanged();
                attObj = cellObj.attObj;
                if (attObj) {
                    // update the attribute value based on the checkbox state
                    if (elem.checked) {
                        attObj.value = 'yes';
                    } else {
                        attObj.value = 'no';
                    }
                    attObj.changed = true;  
                    // update the hidden field so that submit is done correctly
                    var hiddenElem = document.getElementById(this.prefix  + cellObj.attObj.id);
                    if (hiddenElem) hiddenElem.value=attObj.value;

                    this.executeButtonDependencies(attObj);

                    this.emitAttributevalueChanged(attObj);
                }
            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.changeCheckbox:\n' + err);
        }
        if (attObj) this.checkRepaintAttributeCategory(attObj);

    };

    /**
    * creates HTML code for a dropdown box for restricted values created from rules (read from excel etc)
    * @param cellObj: cell containing the attribute with restricted set
    * @param row: row index of the cell
    * @param col: col index of the cell
    */
    attributeCategory.prototype.renderRestriction = function(cellObj,row,col) {
    
        var htmlcode = '';
        try {
            var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+')';
            var theName = this.prefix  + cellObj.attObj.id;

            // find the restricted object
            var rsObj = cellObj.ruleObj.visibilityRestriction; // you are sure it exists otherwise you don't get here
            // what attribute?
            var attObj = null;
            var attIdx = rsObj.attribute0;
            if (attIdx >=0 && attIdx<this.attributes.length) attObj = this.attributes[attIdx];
            // find the array with values
            if (attObj && attObj.value!=null && attObj.value!= "" && rsObj.restrictedValues[attObj.value]) {
            var valArr = null;
                valArr = rsObj.restrictedValues[attObj.value];
            } else {
                valArr = rsObj.restrictedValues["$$$ALL$$$"];
            }

            htmlcode += '<select class="form-control" onchange="' + changeMethod + '" id="' + theName + '" name="' + theName + '" ';
            htmlcode += this.renderStyle(cellObj.styleObj,"Dropdown");

            htmlcode += '>';
            htmlcode += '<option value="">---' + ACatStrings.t('ACatPleaseSelect') + '---</option>';
            var found = false; // did we find the selected value in the list of options
            for (var option=0; option < valArr.length; option++) {
                var value = valArr[option];
                var valueDisplay = value;

                htmlcode += '<option value="'+ WCXML.xmlencode(value) +'"';
                if (cellObj.attObj.value == value) {htmlcode += ' selected '; found=true;};
                htmlcode += '>' + WCXML.xmlencode(valueDisplay) + '</option>';
            }
            if (cellObj.attObj.value && !found) {
                var value = cellObj.attObj.value;
                var valueDisplay = value;

                htmlcode += '<option value="'+ WCXML.xmlencode(value) +'" selected ';            
                htmlcode += '>' + WCXML.xmlencode(valueDisplay) + ' (' + ACatStrings.t('ACatDeleted') + ')</option>';
            }
            htmlcode += '</select>';
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderRestriction'+ ACatStrings.t('ACatAtLine') + err.lineNumber + ": " +  err.message );
        }
        return htmlcode;
    };

    /**
    *  creates HTML code for a dropdown box for restricted values from a List in an Excel sheet (or later in WebCenter)
    * @param cellObj: cell containing the attribute with restricted set
    * @param row: row index of the cell
    * @param col: col index of the cell
    */
    attributeCategory.prototype.renderRestrictedList = function(cellObj,row,col,editable) {
    
        var htmlcode = '';
        try {
            var attObj = cellObj.attObj;
            if (!attObj || !attObj.listObj) {
            	return '<span>&nbsp;</span>'; // should not occur
            }

            var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+')';

            var theName = this.prefix  + attObj.id;

            var listObj = attObj.listObj; 
            
            var columnObj = attObj.columnObj;
            if (!columnObj) throw(ACatStrings.t('ACatColNotSelectedProblem'));

                try {
                    if (attObj.columnObj  ) {
                        if (!attObj.columnObj.validArr || (attObj.listObj && attObj.listObj.updateValuesNeeded) ) this.updateListAttributeValues(attObj.listObj,false);					
                    }
                } catch(err) {
                    // do nothing (cascading dropdown will not be initialized), but this is probably solved by later rendering calls.
                }

            if (editable) {
                // find the array with values
                //var date1 = new Date();
                if (attObj.columnObj  ) {
                    valArr = columnObj.values;
                    if (!valArr) {
                        debugMsg += "\n" + new Date() + ": valArr does not exist for " + attObj.name; 
                        return 'loading...';
                    }
                }

                htmlcode += '<select class="form-control attachleft search-input-lg" onchange="' + changeMethod + '" id="' + theName + '" name="' + theName + '" ';
                htmlcode += this.renderStyle(cellObj.styleObj,"Dropdown");

                htmlcode += '>';
                htmlcode += '<option value="">---' + ACatStrings.t('ACatPleaseSelect') + '---</option>';


                var validArr = columnObj.validArr;  // using an array to allow counting the number of remaining options

                if (attObj.columnObj  ) {
                    // Determine maximum number of entries to show in the dropdown
                    var maxValuesInDropdown = columnObj.maxValuesInDropdown;
                    if (!maxValuesInDropdown) maxValuesInDropdown = 500;

                    // determine how many valid entries there are to display
                    var nValid = columnObj.nValid;
                    if (!nValid && nValid !== 0 || attObj.showAll) nValid =  valArr.length;  // we'll show everything

                    var found = false; // did we find the selected value in the list of options
                    var optionLength = valArr.length;
                    var cutOff = 0;  // number cut off because there would be too many entries in the dropdown

                    if (maxValuesInDropdown && nValid > maxValuesInDropdown && !attObj.showMore && !attObj.showAll) {optionLength = maxValuesInDropdown; cutOff = ( nValid - maxValuesInDropdown);};
                    var htmlcodeArr = new Array();
                    logger += "\n dropdown for " + attObj.name + " length " + optionLength + " maxvaluesinDropdown "+ maxValuesInDropdown + " columnObj.maxvaluesinDropdown " +  columnObj.maxValuesInDropdown + " columnObj.nValid "+columnObj.nValid + "  !showmore " + !attObj.showMore;
                    var somethingHidden = false;  // remembers whether something has been hidden because it is filtered away by previous selections
                    var displayedOptions =0;
                    //for (var option=0;displayedOptions < optionLength && option < valArr.length; option++) {
                    for (var option=0; option < valArr.length; option++) {
                        var value = valArr[option];
                        //var selected = (attObj.value == value);
                        var selected = (option == parseInt(columnObj.currentIndex));
                        var valid = true;
                        if (validArr && validArr[option]==false) valid = false;

                        if (displayedOptions < optionLength || selected) {  // never hide what is selected
                            if (valid || attObj.showAll || selected) {					
                                var hc = '<option value="'+ WCXML.xmlencode(value) +'"';
                                if ( selected) {hc += ' selected '; found=true;};						
                                if (!valid && !selected) hc+= ' style= "font-style:italic;color:#c0c0c0;" '
                                hc += '>' + WCXML.xmlencode(value);
                                if (!valid && selected) hc+= ' <i>' + ACatStrings.t('ACatNotValid') + '</i>';  // say that it is invalid (there is a second case of this below)
                                hc += '</option>';
                                htmlcodeArr.push(hc);
                                displayedOptions++;
                            } else {
                                somethingHidden=true;
                            }
                        }
                    }
                    if (cutOff) {
                        var hc = '<option value="$$More$$"><i>'+ cutOff + ACatStrings.t('ACatMoreEntries') +'</i></option>';
                        htmlcodeArr.push(hc);
                    }
                    htmlcode += htmlcodeArr.join();
                    if (somethingHidden) htmlcode += '<option value="$$ShowAll$$" class="greentext_11">' + ACatStrings.t('ACatShowAll') + '</option>';

                    // If selection not found and there is a value anyway, show it with "(NOT VALID)" 
                        if ($.trim(attObj.value) && !found) {
                        var value = attObj.value;			

                        htmlcode += '<option value="'+ WCXML.xmlencode(value) +'" selected ';            
                        htmlcode += '><i>' + WCXML.xmlencode(value) + ACatStrings.t('ACatNotValid') + '</i></option>';
                    } 

                    //alert("render list " + attObj.name + " took "+ (new Date() - date1));
                    attObj.showAll = false;  // only renders once with all values
                    attObj.showMore = false;  // only renders once with all values		

                } else {
                    // only show the current value  
                    htmlcode += '<option value="'+ WCXML.xmlencode(attObj.value) +'" selected>'+WCXML.xmlencode(attObj.value)+'</option>';
                }
                htmlcode += '</select>';
                htmlcode += '<input type="checkbox" value="ON" style="display:none" checked="" id="' + theName + '_excel" name="' + theName + '_excel" >';  // HADW CR: what is this?
            } else {
                htmlcode += attObj.value;
                htmlcode += '<input type="hidden" name="' + theName + '" value="' + WCXML.xmlencode(attObj.value) + '">';
            }



        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderRestrictedList' + ACatStrings.t('ACatAtLine') + err.lineNumber + ": " +  err );
        }
        return htmlcode;
    };

    attributeCategory.prototype.renderOtherAttributeSelector = function(cellObj,row,col) {
    
        var htmlcode = '';
        try {

            var attObj = cellObj.attObj;
            if (!attObj) return;

            var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+')';
            var theName = this.prefix  + cellObj.attObj.id;
            var attObj2 = attObj.restrictToAttObj;

            if (attObj2 && attObj2.value) {
                var valArr = attObj2.value.split(this.multiSep);

                if (valArr.length == 1) attObj.value = attObj2.value;   // automatically select the single one allowed
                htmlcode += '<select class="form-control" onchange="' + changeMethod + '" id="' + theName + '" name="' + theName + '" ';
                htmlcode += this.renderStyle(cellObj.styleObj,"Dropdown");

                htmlcode += '>';
                htmlcode += '<option value="">---' + ACatStrings.t('ACatPleaseSelect') + '---</option>';
                var found = false; // did we find the selected value in the list of options
                for (var option=0; option < valArr.length; option++) {
                    var value = valArr[option];
                    var valueDisplay = value;

                    htmlcode += '<option value="'+ WCXML.xmlencode(value) +'"';
                    if (attObj.value == valArr[option]) {htmlcode += ' selected '; found=true;};
                    htmlcode += '>' + WCXML.xmlencode(valueDisplay) + '</option>';
                }
                if (cellObj.attObj.value && !found) { 

                    if (attObj.allowMismatchingRestriction) {  // currently never set for lack of a method to do so
                        var valueDisplay = attObj.value;

                        htmlcode += '<option  value="NOT FOUND" selected ';            
                        htmlcode += '>' + WCXML.xmlencode(valueDisplay) + ' (' + ACatStrings.t('ACatDeleted') + ')</option>';
                    } else {
                        attObj.value = ''; 	
                    }
                }
                htmlcode += '</select>';
            } else {
                // do nothing. No dropdown will be shown when there are no base values
            	htmlcode += '<span>&nbsp;</span>';
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderOtherAttributeSelector:\n' + err);
        }
        return htmlcode;
    };


    /**
    * creates HTML code to be put in a <td> or similar element to contain styling of styleObj
    * @param styleObj: object containing all styling information
    * @param fieldType: one of 'Label','DateField','InputField','Dropdown','Value', 'Hyperlink' 
    * @param paddingRight adds padding to the right, typical for the unit column
    * @param roomForButton - leaves room to put a button after an input field
    */
    attributeCategory.prototype.renderStyle = function(styleObj,fieldType,paddingRight,roomForButton, extraClasses) {    	
        try {
            var htmlcode = '';
            if(!extraClasses){
            	extraClasses = '';
            }
            if (styleObj) {  
            	var hasValueStyle = !!styleObj.valueStyleObj;
				var useAlternativeRendering = this.useAlternativeRendering && !this.blockAlternativeRendering;
				
                // apply class if there is one
                var cssClass = null;  // don't call it class because that's a reserved word
                if (fieldType == 'Label' || fieldType == 'Title') {
                    cssClass = styleObj.classLabel; 
                    if (!cssClass) cssClass = this.styles[0].classLabel;            
                } else {
                    cssClass = styleObj.classValue; 
                    if (!cssClass) cssClass = this.styles[0].classValue;               
                }
				
				if ((fieldType == 'Label' || fieldType == 'Value')  && !useAlternativeRendering && !styleObj.columnWidth){
					extraClasses += " mw-for-labelvalue ";
				}
				
                htmlcode += ' class="' + extraClasses + (extraClasses?' ':'') + cssClass +'" ';

                // apply styling 
                htmlcode += ' style="';
                if (fieldType.indexOf('Label') == -1 && fieldType.indexOf('Title') == -1 && styleObj.valueStyleObj) { 
                  // if it's a value and label and value are styled differently, take the value style
                      styleObj = styleObj.valueStyleObj;
                }

                // set background color but not for the input fields and dropdowns		
                if (styleObj.background && fieldType!='InputField' && fieldType != 'Dropdown' && fieldType != 'DateField' && fieldType != 'MultiValue' && fieldType != 'RoundedTitle' && fieldType!='Unit') {
                    htmlcode += 'background-color:' + this.fixStrictModeColor(styleObj.background) + ';';
                }

                // set alignment
                //if (styleObj.alignment != "") htmlcode += 'text-align:' + styleObj.alignment + ';';    // does not check on null or undefined
                if (styleObj.alignment) htmlcode += 'text-align:' + styleObj.alignment + ';'; 

                if (fieldType != 'Button' && fieldType != 'Title'){
                	htmlcode += 'vertical-align: top;';
                }

                // set padding-right (typical for unit/type column)
                if (paddingRight >0) {
                    htmlcode += 'padding-right:' + paddingRight + 'px;';
                }

                // set height
                if(styleObj.height) {
                    htmlcode += 'height:' + styleObj.height + 'px;';       		
                }

                // set width should not be applied to the <td> but should be applied to the cell itself (the input cell).

                if (fieldType == 'ValueGroup') {
                	if(styleObj.width) {
                		var unit = 'px;';
            			if(styleObj.widthunit && styleObj.widthunit == "%") {
            				unit =  '%;';  
                        }
                        htmlcode += 'max-width:' + styleObj.width + unit;
                        htmlcode += 'min-width:' + styleObj.width + unit;
                    }
                }
                
                // Width of the column		
                if (fieldType == 'Label' || fieldType == 'Value') {
                	
                    
                    if(!useAlternativeRendering){
                    	if(styleObj.columnWidth){
                    		var width = styleObj.columnWidth;
                    		if(!hasValueStyle){
                    			if(fieldType == 'Label'){
                    				width *= 1/3;
                    			} else {
                    				width *= 2/3;
                    			}
                    		}
                    		
                    		htmlcode += 'width:' + width;  

                            if(styleObj.columnWidthUnit && styleObj.columnWidthUnit == "%") {
                                htmlcode +=  '%;';  
                            } else {
                                htmlcode +=  'px;';  
                            }	
                    	}
                    }
                }


                // set font properties
                if (styleObj.fontWeight) htmlcode += 'font-weight:' + styleObj.fontWeight + ';';
                if (styleObj.fontStyle) htmlcode += 'font-style:' + styleObj.fontStyle + ';';
                if (styleObj.fontWeight) htmlcode += 'font-weight:' + styleObj.fontWeight + ';';
                if (styleObj.fontFamily) htmlcode += 'font-family:' + styleObj.fontFamily + ';';
                if (styleObj.fontSize) htmlcode += 'font-size:' + styleObj.fontSize + 'pt;';
                if (styleObj.fontColor) htmlcode += 'color:' + styleObj.fontColor + ';';

                if (fieldType == 'MultiValue') {
                    htmlcode += 'padding:1px; background-color:white;';
                }

                htmlcode += '" '; // end style


            } else { // no style
                if (this.currentView != 'Header') {
                    if (fieldType == 'Label') {
                    	htmlcode += ' class="' + extraClasses + 'darktext_darklink_11" ';
                    }
                    else if (fieldType == 'InputField') {
                    htmlcode += ' class="' + extraClasses + 'darktext_darklink_11" style="width:100%" ';
                    } else {
                    htmlcode += ' class="' + extraClasses + 'darktext_darklink_11" ';
                    }
                } else { // Header view
                    if (fieldType == 'Label') {
                        htmlcode += ' class="' + extraClasses + 'Black_Bold_11" align="right" style="padding-left:40px;" ';
                    } else {
                        if (fieldType == 'Hyperlink') {
                            htmlcode += ' class="' + extraClasses + 'darktext_darklink_11" ';
                        } else {
                        htmlcode += ' class="' + extraClasses + 'Black_11" ';
                    }
                }
            }
            }

          } catch (err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderStyle:\n' + err);
            return '';
          }
        return htmlcode;  
    };

    /**
    * creates HTML code to render a ruler (horizontal line) with thickness and color
    * @return returns the HTML code
    */
    attributeCategory.prototype.renderRuler = function(cellObj) {
        var td = $('<td colspan="' + (cellObj.colSpan*2) + '"/>');
    	var hr = $("<div />");
    	hr.css("background-color", this.fixStrictModeColor(cellObj.rulerColor) || "black");
        if (cellObj.rulerThickness) {
            hr.css("height", cellObj.rulerThickness);	
        }
    	td.append(hr);

        return td;
    };

    /**
    * creates HTML code to render a set of buttons defined in cellObj.buttons
    * @param {Boolean} inline: when true, the button will be put inline in an existing cell
    8 @param {Boolean} forPrint - when true, button will not be rendered (intention is to not show buttons on a printed result
    * @return returns the HTML code
    */
    attributeCategory.prototype.renderButton = function(cellObj,row,col,inline,forPrint) {
        var htmlcode = '';
        try {

            if (!inline) {
                htmlcode += '<td '+this.renderStyle(cellObj.styleObj,"Label")+'/><td colspan="' + (cellObj.colSpan*2 - 1) + '" ' + this.renderStyle(cellObj.styleObj,"Button") + '>';  
                htmlcode += '<div class="form-entry">';
            }
            if (!forPrint) {
                var clickMethod =  this.functionPrefix + '.clickACatButton(null, this,\'click\');return false;';

                if (cellObj.buttons) {
                    for (var button=0; button < cellObj.buttons.length; button++) {
                        var buttonObj = cellObj.buttons[button];
                        if (buttonObj && buttonObj.style != 'hidden') {
                            var id= this.prefix + 'ACatButton~' + row + '~' + col + '~' + button; 
                            
                            htmlcode += '<div class="input-group-btn"';
                            if(!inline){
                            	htmlcode += ' style="float: left; width: auto;"';
                            }
                            htmlcode += '>';
                            if (buttonObj.style != 'greenlink') {		
                                htmlcode += '<input type="button" value="'+ WCXML.xmlencode(buttonObj.label) +'" onclick="'+clickMethod+'" id="'+id+'"';
                                if (buttonObj.style && buttonObj.style !='plainbutton'){
                                	htmlcode += ' class="'+buttonObj.style+'" ';
                                }
                                if(button === 0){
                                	htmlcode += ' style="margin-left:5px"';
                                }
                                htmlcode += '/>';
                            } else {
                                htmlcode += '<span class="darktext_greenlink_11"><a  href="#" onclick="'+clickMethod+'"  id="'+id+'">'+WCXML.xmlencode(buttonObj.label)+'</a></span>';
                            }
                            htmlcode += '</div>';
                        }
                    }
                }
            }

            if (!inline) htmlcode += '</div></td>';
  
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderButton:\n' + err);
            return '';
        }
        return htmlcode;
    };

    /**
    * creates HTML code to render space with a background color
    * @return returns the HTML code
    */
    attributeCategory.prototype.renderSpacer = function(cellObj) {
        var htmlcode = '';
        var requestedHeight;
        try {
            htmlcode += '<td colspan="' + (cellObj.colSpan*2) + '" style="';		
            if (cellObj.height) {
                htmlcode += 'height:' + cellObj.height +'px;';	
                requestedHeight = cellObj.height;
            } else {
                htmlcode += 'height:28px;';	//18 typical height + 2 x 5 pix padding	
                requestedHeight = 28;
            }
            // if multiple columns, cellpadding af neighbours will interfere with height, so place a div in here
            htmlcode += 'padding:0;"><div style="height:'+requestedHeight+'px;width:100%;padding:0;';
            if (cellObj.background) {
                htmlcode += 'background-color:' + this.fixStrictModeColor(cellObj.background) + ';';		
            }
            htmlcode += '"></div></td>';

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderSpacer:\n' + err);
            return '';
        }
        return htmlcode;
    };

    /**
    * creates HTML code to render a title with styling
    * @param roundedTitle creates HTML optimized for the typical WCR rounded title. Other styling is disregarded
    * @return returns the HTML code
    */
    attributeCategory.prototype.renderTitle = function(cellObj, row, col, visibility, forPrint) {
        var htmlcode = '';

        var labelstyle = '';
        if(cellObj.styleObj){
        	labelstyle = ' style="'
        	+ (cellObj.styleObj.alignment ? ('text-align:' + cellObj.styleObj.alignment + ';') : '')
        	+ (cellObj.styleObj.fontSize ? ('font-size:' + cellObj.styleObj.fontSize + 'pt;') : '')
        	+ (cellObj.styleObj.fontWeight ? ('font-weight:' + cellObj.styleObj.fontWeight + ';') : '')
        	+ (cellObj.styleObj.fontFamily ? ('font-family:' + cellObj.styleObj.fontFamily + ';') : '')
        	+ (cellObj.styleObj.fontStyle ? ('font-style:' + cellObj.styleObj.fontStyle + ';') : '')
        	+ '"';
        }
        
        htmlcode += '<td colspan="' + (cellObj.colSpan*2) + '" ' + this.renderStyle(cellObj.styleObj,"Title", null, null, "attributesGridLayoutTitle") + '><label'+labelstyle+'>' ;        
        htmlcode += WCXML.xmlencode(cellObj.label);
        if (cellObj && cellObj.infoPage) {		
            var clickMethod = 'popw=window.open(&quot;./custom/AttributesHelp/'+ WCXML.xmlencode(cellObj.infoPage) + '&quot;,&quot;Info&quot;,' + this.helpOptionsString + ');popw.focus();';
            htmlcode += '<a class="inline_help btn" href="#" onclick="' + clickMethod +'"><i class="icon icon-WCR_Info-Balloon icon-normal"></i></a>';
        }

        // buttons if there are any
        var editable = this.isAttrCatEditable();
        if (visibility != 'Editable' && visibility != 'Required') {
        	editable = false;
        }
        if (editable && cellObj.buttons) {
        	htmlcode += this.renderButton(cellObj, row, col, true, forPrint);
        }
        htmlcode += '</label></td>';

        return htmlcode;
    };

    /**
    * creates HTML code to render an attribute as a grid of images
    * Known restriction: doesn't listen to maxValuesInDropdown --> Put this in FS
    * @return returns the HTML code
    */
    attributeCategory.prototype.renderAttributeImages = function(cellObj,row,col,visibility) {
        try {
            var editable = this.isAttrCatEditable();
            var clickMethod;
            var htmlcode = '';
            var columnObj;
            var attObj = cellObj.attObj;  
            var idx,value, rsType = '';
            if (!attObj) return '';
            var imageHeight = cellObj.imageHeight;  // images will be scaled to all have this height
            if (!imageHeight) imageHeight = 100;
            //var onloadFunction = 'WCHTML.scaleImageProportionally(this,100,100);';
            var listName = attObj.listObj? attObj.listObj.name : 'default';
            if (cellObj.renderImageType == 'automatic' || !cellObj.renderImageType) {
                    onerrorFunction = 'WCHTML.alternativeImage(this,\''+window.webcenterConfig.jspBaseUrl+'custom/AttributeValueImages/'+listName+'.jpg\');';
                cellObj.renderImageType = 'automatic';  // in case renderImageType is not set and you get here anyway, set it to automatic.
            } else {
                onerrorFunction = '';			
            }
            if (!cellObj.nImageColumns) cellObj.nImageColumns = 6;



            var valueArr;
            var curValArr = [];
            var colSpan;

            if (attObj.rs) {valueArr = attObj.rs.values; rsType = 'RestrictedSet';};
            if (attObj.columnObj) {
                columnObj =  attObj.columnObj; valueArr = columnObj.values; 
                rsType = 'List';
                if (attObj.listObj && attObj.listObj.updateValuesNeeded) this.updateListAttributeValues(attObj.listObj,false);					
            };	


            if (valueArr && visibility != 'Hidden' && visibility != 'HiddenKeepValues') {
                colSpan = cellObj.colSpan * 2;
                if (!colSpan){
                	colSpan = 2;
                }

                htmlcode += '<input id="'+ this.prefix + attObj.id +'" type="hidden" value="'+ WCXML.xmlencode(attObj.value)+'" name="'+ this.prefix + attObj.id +'">';
                htmlcode += '<div class="MV_CheckBoxes main-content"';
                if (cellObj.renderHeight) htmlcode += 'style="height:' + cellObj.renderHeight + 'px;overflow:auto;"';
                htmlcode += '>';
                var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+',true)';
                if (attObj.allowMultipleValues) {
                    changeMethod = this.functionPrefix + '.changeSelectMultiple(' + row + ',' + col+',true)';
                    if (attObj.value) curValArr = attObj.value.split(this.multiSep);				
                }

                if (cellObj.imageAlignment == 'DistributeOverWidth') {
                    htmlcode += '<table width="100%">';
                } else {
                    htmlcode += '<table>';
                }

                // determine which and how many valid entries there are to display
                var nValid;
                if ( rsType == 'RestrictedSet') {
                    nValid = valueArr.length;
                } else if (rsType == 'List') {
                    nValid = columnObj.nValid;
                    if (!nValid && nValid !== 0) nValid =  valueArr.length;  // we'll show everything
                    if ( nValid > 0) {
                        // construct a new valueArr with only the valid entries
                        var valueArr2 = [];
                        for (idx = 0; idx<valueArr.length; idx++) {
                            if (attObj.showAll || !columnObj.validArr || columnObj.validArr[idx]) { 
                                valueArr2.push({value: valueArr[idx], idx: idx});
                            }
                        }
                        valueArr = valueArr2;
                    }
                }

                // determine whether there are selected values which are no longer in validArr (f.e. restricted set value taken out)
                var extraValueArr = [];
                if (attObj.allowMultipleValues) {
                    for (var idx = 0 ; idx< curValArr.length; idx++) {
                        for (var idx2=0; idx2<valueArr.length; idx2++) {
                            if (valueArr[idx2].value == curValArr[idx]) break;
                        }
                        if (idx2 >= valueArr.length) extraValueArr.push(curValArr[idx]);
                    }
                } else if (attObj.value) {
                    for (var idx2=0; idx2<valueArr.length; idx2++) {
                        if (valueArr[idx2].value == attObj.value) break;
                    }
                    if (idx2 >= valueArr.length) extraValueArr.push(attObj.value);
                }

                // determine number of rows
                var nImageRows = Math.ceil((nValid +extraValueArr.length ) / cellObj.nImageColumns );
                var widthPerc = 100/ cellObj.nImageColumns ;
                var imgExtension = (cellObj.renderImageType == 'automatic') ? '.jpg' : ('.' + cellObj.renderImageType);
                var cellAlign = 'center';
                if (cellObj.restrictedValueLabelPosition != 'LabelBelow' ) cellAlign = 'left';
                var extraClass = (cellAlign == 'center' ? "centerAlign" : "");


                for (var row=0; row < nImageRows; row++) {

                    // Images
                    if (cellObj.renderOption === 'Images') {
                        htmlcode += '<tr>';			
                        for (col=0; col< cellObj.nImageColumns; col ++) {									
                            idx = row*cellObj.nImageColumns + col; 
                            if (idx < valueArr.length ||   (row== nImageRows-1 &&  extraValueArr.length>0  && (idx-valueArr.length <extraValueArr.length) ) ) {
                                if (idx < valueArr.length) {
                                    value = valueArr[idx].value;
                                } else {
                                    value = extraValueArr[idx-valueArr.length];
                                }
                                if (cellObj.imageAlignment == 'DistributeOverWidth') {
                                    htmlcode += '<td align="'+cellAlign+'" width="'+ widthPerc+'%">';
                                } else {
                                    htmlcode += '<td align="'+cellAlign+'"style="padding:0 10 0 10" >';
                                }
                                //htmlcode += '<img src="./custom/AttributeValueImages/'+ WCXML.xmlencode(value) + imgExtension + '" height="' + imageHeight +'" onload="'+onloadFunction+'" onerror="'+onerrorFunction+'">';
                                    htmlcode += '<img src="'+window.webcenterConfig.jspBaseUrl+'custom/AttributeValueImages/'+ WCXML.xmlencode(value) + imgExtension + '" style="height: ' + imageHeight +'px"  onerror="'+onerrorFunction+'">';
                                htmlcode += '</td>';

                            } else {
                                htmlcode += '<td/>';  // needed to avoid that there are less columns which will make them take more space --> ugly alignment
                            }

                        }

                        htmlcode += '</tr>';
                    }

                    // Radio buttons or checkboxes
                    htmlcode += '<tr>';
                    for (col=0; col< cellObj.nImageColumns; col ++) {
                        idx = row*cellObj.nImageColumns + col; 
                        if (idx < valueArr.length ||   (row== nImageRows-1 &&  extraValueArr.length>0  && (idx-valueArr.length <extraValueArr.length) ) ) {
                            var curIdx = idx;					
                            if (idx < valueArr.length) {
                                value = valueArr[idx].value;
                                if (rsType == 'List') curIdx = valueArr[idx].idx;
                            } else {
                                value = extraValueArr[idx-valueArr.length];
                            }
                            if (cellObj.imageAlignment == 'DistributeOverWidth') {
                                htmlcode += '<td align="'+cellAlign+'" width="'+ widthPerc+'%">';
                            } else {
                                htmlcode += '<td align="'+cellAlign+'"style="padding:2 10 2 10" >';
                            }


                            htmlcode +=  '<span '+this.renderStyle(cellObj.styleObj,"SplitValue")+' for="'+this.prefix+'ImgChk_'+attObj.id+'_'+ idx + '">';
                            if (attObj.allowMultipleValues) {   // put checkboxes
                                htmlcode += '<input type="checkbox" style="vertical-align: middle; margin: 0px;" name="'+this.prefix + 'ImgChk_' + attObj.id +'" id="'+this.prefix+'ImgChk_'+attObj.id+'_'+ idx + '"  value="'+ WCXML.xmlencode(value) +'" idx="'+curIdx+'" onchange="'+ changeMethod +'"';		
                                for (var idx2=0; idx2<curValArr.length; idx2++) {
                                    if (curValArr[idx2] === value) {htmlcode += ' checked=checked '; break;};
                                }						
                                if (!editable || visibility == 'ReadOnly') htmlcode += ' disabled=disabled ';
                                htmlcode += '>';

                            } else {   // put radio buttons
                                htmlcode += '<input type="radio" style="vertical-align: middle; margin: 0px;" name="'+this.prefix + 'ImgChk_' + attObj.id+'" value="'+ WCXML.xmlencode(value) +'" idx="'+curIdx+'" id="'+this.prefix+'ImgChk_'+attObj.id+'_'+ idx + '"  onchange="'+ changeMethod +'"'; 
                                if (attObj.value === value) htmlcode += ' checked=checked ';
                                if (!editable || visibility == 'ReadOnly') htmlcode += ' disabled=disabled ';
                                htmlcode += '>';
                            }
                            if (cellObj.restrictedValueLabelPosition == 'LabelRight' ) {
                                htmlcode +=  '&nbsp;&nbsp;' + WCXML.xmlencode(value);
                                if (idx>=valueArr.length){
                                	htmlcode += '&nbsp;' + ACatStrings.t('ACatNotValid');
                                }
                            }
                            htmlcode +=  '</span>';  
                            htmlcode += '</td>';
                        } else {
                            htmlcode += '<td/>';
                        }
                    }
                    htmlcode += '</tr>';

                    // Text
                    if (cellObj.restrictedValueLabelPosition == 'LabelBelow' ) {
                        htmlcode += '<tr>';
                        for (col=0; col< cellObj.nImageColumns; col ++) {
                            idx = row*cellObj.nImageColumns + col; 
                            if (idx < valueArr.length ||   (row== nImageRows-1 &&  extraValueArr.length>0  && (idx-valueArr.length <extraValueArr.length) ) ) {
                                if (idx < valueArr.length) {
                                    value = valueArr[idx].value;
                                } else {
                                    value = extraValueArr[idx-valueArr.length];
                                }
                                htmlcode += '<td align="'+cellAlign+'" style="padding:2 10 10 10" class="darktext_greenlink_11">';
                                htmlcode +=  '<label '+this.renderStyle(cellObj.styleObj,"SplitValue", undefined, undefined, extraClass)+' for="'+this.prefix+'ImgChk_'+attObj.id+'_'+ idx + '">' +WCXML.xmlencode(value) + '</label>';
                                if (idx>=valueArr.length) htmlcode += '&nbsp;' + ACatStrings.t('ACatNotValid');
                                htmlcode += '</td>';
                            } else {
                                htmlcode += '<td/>';
                            }
                        }
                        htmlcode += '</tr>';
                    }
                }

                htmlcode += '</table>';
                htmlcode += '</div></td>';
            } else if (visibility == 'HiddenKeepValues'){
            	var attValue = attObj.value;
                var theName = this.prefix + attObj.id;
            	htmlcode += '<input type="hidden" name="'+theName+'" value="'+WCXML.xmlencode(attValue)+'">';
            }

            return htmlcode;
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderAttributeImages at line' +  err.lineNumber + ':\n' + err);
        }
    };

    /**
    * creates HTML code to render an emty cell
    * @return returns the HTML code
    */
    attributeCategory.prototype.renderEmptyCell = function() {
        try {
            return '<td></td><td></td>';
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderEmptyCell:\n' + err);
        }
    };

    attributeCategory.prototype.renderHiddenAttributeInfo = function(attObj, hiddenCell) {
        var htmlcode = '';
        try {
            var theName = this.prefix + "UN" + attObj.id;
            htmlcode += '<input type="hidden" name="' + theName + '" value="' + attObj.unitId + '">';

            // set the isRS hidden field to indicate whether it is a restricted set
            var rsVal = "0";
            if (attObj.rs) rsVal = "1";
            theName = this.prefix + "isRS" + attObj.id;
            htmlcode += '<input type="hidden" name="' + theName + '" id="' + theName + '" value="' + rsVal + '">';

            if (attObj.type && attObj.type.toLowerCase() == 'datetime' && rsVal != "1") {
                var dateName = this.prefix +  attObj.id;
                var aDateName = this.prefix + "adate" + attObj.id;
                var saveADateName = this.prefix + "save_adate" + attObj.id;                        

                htmlcode += '<input type="hidden" name="' + dateName + '" id="' + dateName + '" value="' + (hiddenCell?"":attObj.hiddenDate) + '">';  
                htmlcode += '<input type="hidden" name="' + aDateName + '" id="' + aDateName + '" value="' + (hiddenCell?"":attObj.hiddenDate) + '">';
                htmlcode += '<input type="hidden" name="' + saveADateName + '" id="' + saveADateName + '" value="' + (hiddenCell?"":attObj.oldDate) + '">';
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderHiddenAttributeInfo:\n' + err);
        }
        return htmlcode;
    };

    /**
    * creates HTML code to render a hidden cell
    * @param values If true, attObj.value is added, if false empty values are used
    * @return returns the HTML code
    */
    attributeCategory.prototype.renderHiddenCell = function(attObj,values) {
        var htmlcode = '';
        try {
            if (attObj) {
                htmlcode += this.renderHiddenAttributeInfo(attObj, !values); 
                var theName = this.prefix  + attObj.id;	
                var	value = values? attObj.value : '';

                htmlcode += '<input type="hidden" value="'+ WCXML.xmlencode(value)+'" name="'+theName+'" id="'+theName+'" >';
                if (attObj.type && attObj.type.toLowerCase() == 'datetime' && !attObj.rs) {
                    var longDateName = this.prefix + "longFormat_adate" + attObj.id;
                    htmlcode += '<input type="hidden" value="" name="'+longDateName+'" id="'+longDateName+'" >';	 // assumes hidden = empty
                    var hourName = this.prefix + "date_period_of_day_" + attObj.id;	
                    var halfhours = 0;
                    if (value) halfhours = value.getHours() * 2 + Math.floor(value.getMinutes()/30);
                    htmlcode += '<input type="hidden" value="'+ halfhours +'" name="'+hourName+'" id="'+hourName+'" >';				
                }				
            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderHiddenCell:\n' + err);
        }
        return htmlcode;
    };

    /**
    * Figures out whether a cell is visible or not
    * This function is recursive
    * @param cellObj: object representing the cell which visibility needs to be investigated
    * @param ruleObj: rule object to check against, if null cellObj.ruleObj is used
    * @return one of Hidden, ReadOnly, Required, Editable
    */
    attributeCategory.prototype.checkVisibility = function(cellObj) {
        try {
            if (!cellObj) return true;  // should not happen
            ruleObj = cellObj.ruleObj;

            // special case: hidden properties because of unavailability. e.g. Documents section when no documents allowed 
            if(cellObj.type == 'Property' && cellObj.attObj && cellObj.attObj.hidden && !this.displayAttributeCategoryPreview){
            	return "Hidden";
            }

            if (!ruleObj){
            	return "Editable";  // happens when called and there is no rule applied (=frequent)
            }
            if (!ruleObj.condition){
            	return ruleObj.name; // this is a standard rule
            }
            var trueOrFalse = this.checkVisibilityRecursive(ruleObj);
            if (trueOrFalse) {
                return ruleObj.visibility;
            } else {
                return ruleObj.visibilityElse;
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.checkVisibility:\n' + err);
            return false;  // whatever cannot be interpreted is so ugly you don't want to show it...
        }
    };

    
    /**
     * Method returning true when the specified cell contains an attribute that is "editable" or "required".
     * Note 'HiddenKeepValues' as well as "Hidden" are considered "readonly", because they are not
     * physically rendered on the page, thus cannot support complex new values as e.g. uploaded document.
     * 
     * @param {Object} cellObj - Cell object containing attribute to be checked.
     */
    attributeCategory.prototype.isAttrEditable = function(cellObj) {
	    var attrVisibility = this.checkVisibility(cellObj); // returns one of true, Editable, ReadOnly, Hidden, HiddenKeepValues or Required
	    return ((attrVisibility != 'ReadOnly') && (attrVisibility != 'Hidden') && (attrVisibility != 'HiddenKeepValues'));
    };

    /** 
     * Returns true if the whole attribute category is editable for currently logged in user.
     * False, if it's not editable and/or if the current user is not known.
     */
    attributeCategory.prototype.isAttrCatEditable = function() {
    	return (this.currentUser && this.currentUser.canEditAttributes);
    };
    
    /**
     * Function works case sensitive and does cut leading and trailing spaces
     */
    attributeCategory.prototype.checkMultipleValues = function(valueString,compareValue) {
        var valueArr = valueString.split(this.multiSep);
            compareValue = $.trim(compareValue).toLowerCase();
        var match = false;
        for (var idx=0; idx<valueArr.length; idx++) {
                if ($.trim(valueArr[idx]).toLowerCase() == compareValue) {match=true; break;};
        }
        return match;

    };
    /**
    * Figures out whether the combined condition of a rule is true or false
    * @param ruleObj: rule object to check against, if null cellObj.ruleObj is used
    * @return true or false
    */
    attributeCategory.prototype.checkVisibilityRecursive = function(ruleObj) {
        try {

            if (!ruleObj) return true;  // happens when called and there is no rule applied (=frequent)

            if (ruleObj.attObj) {  // visibility  depends on attribute value -- leaf of the rule tree
                // find the attribute with this id
                var attObj = ruleObj.attObj;
                switch (ruleObj.operator) {
                    case '=':
                        if (!attObj.allowMultipleValues) {
                                return ($.trim(attObj.value).toLowerCase() == $.trim(ruleObj.compareValue).toLowerCase()) ;
                        } else {
                            return (this.checkMultipleValues(attObj.value,ruleObj.compareValue));
                        }
                        break;
                    case '!=':
                        if (!attObj.allowMultipleValues) {
                                return ($.trim(attObj.value).toLowerCase() != $.trim(ruleObj.compareValue).toLowerCase()) ;
                        } else {
                            return !(this.checkMultipleValues(attObj.value,ruleObj.compareValue));
                        }
                        break;
                    case '>':
                        return ( parseFloat(attObj.value) > parseFloat(ruleObj.compareValue)) ;
                        break;
                    case '>=':
                        return ( parseFloat(attObj.value) >= parseFloat(ruleObj.compareValue)) ;
                        break;
                    case '<':
                        return ( parseFloat(attObj.value) < parseFloat(ruleObj.compareValue)) ;
                        break;
                    case '<=':
                        return ( parseFloat(attObj.value) <= parseFloat(ruleObj.compareValue)) ;
                        break;
                    case 'One Of':
                        // split compareValue to line breaks
                        var compareArr = ruleObj.compareValue.split("\n");
                        var found = false;
                        for (var idx=0; idx<compareArr.length; idx++) {
                            if (!attObj.allowMultipleValues) {
                                    if ($.trim(attObj.value).toLowerCase() == $.trim(compareArr[idx]).toLowerCase()) {
                                    found = true; break;
                                }
                            } else {
                                if (this.checkMultipleValues(attObj.value,compareArr[idx])) {
                                    found = true; break;
                                }
                            }
                        }
                        return (found) ;
                        break;

                    default:

                        throw(ACatStrings.t('ACatOpNotSupportedProblem1') + ruleObj.operator + ACatStrings.t('ACatOpNotSupportedProblem2'));
                }
            } else { // and-or node
                var visible1 = this.checkVisibilityRecursive(ruleObj.ruleObj1);
                var visible2 = this.checkVisibilityRecursive(ruleObj.ruleObj2);
                if (ruleObj.condition == 'AND' && visible1 && visible2) return true;
                if (ruleObj.condition == 'OR' && (visible1 || visible2)) return true;
                return false;
            }
            return true;  // probably we never get here
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.checkVisibilityRecursive:\n' + err);
            return false;  // whatever cannot be interpreted is so ugly you don't want to show it...
        }
    };

    /**
    * Creates html code for the hour selector typical for selecting a due hour
    * Re-used between the plain panel and the due date recalculation pop-up
    */
    attributeCategory.prototype.displayHourDropdown = function(hour,id,name,changeMethod,styleObj)  {
        var htmlcode = '';
        try {
            if (!id) throw(ACatStrings.t('ACatDispHourProblem1'));
            if (!name) throw(ACatStrings.t('ACatDispHourProblem2'));
            
            htmlcode += '<select class="form-control" id="' + id + '" name="' + name + '"';
            if (changeMethod) htmlcode += ' onchange="' + changeMethod + '"';
            if (styleObj) htmlcode += this.renderStyle(styleObj,"Dropdown");
            htmlcode += '>';

            // first the --:--
            if (hour == -1) {
                htmlcode += '<option value="-1" selected>--:--</option>';
            } else {
                htmlcode += '<option value="-1">--:--</option>';
            }
            for (var counter=0; counter < 24; counter++) {
                var value = counter * 2;
                var selected = (value == hour); // true or false
                var displaybase = counter<10 ? '0'+counter : counter;
                var display = displaybase + ":00";
                htmlcode += '<option value="' + value + '"' + (selected ? ' selected ' : '') + '>' + display + '</option>';
                value = counter * 2 + 1;
                display = displaybase + ":30";
                selected = (value == hour); // true or false
                htmlcode += '<option value="' + value + '"' + (selected ? ' selected ' : '') + '>' + display + '</option>';
            }         
            htmlcode += '</select>';
            htmlcode += '<label for="'+name+'" class="form-datetime-btn input-group-addon">';
        	htmlcode += '<i class="icon icon-WCR_Clock"/>';
    		htmlcode += '</label>';
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ " attributeCategory.displayHourDropdown: " +err);
        }
        return htmlcode;
    };

    /**
    * Typically called for a creation view
    * When documents are in creation view, the document itself typically does not yet exist in WebCenter
    * There is only a document template but the document template does not get copied when uploading a file, only the attribute category is taken over
    * This means the default values of the document template don't make it to the new document unless you submit them here
    * Since they are not in the creation view or they are read-only or hidden in the creation view, we need to add them here
    */
    attributeCategory.prototype.addDefaultHiddenFields = function () {

        try {
            var htmlcode = '';	
            var cells = this.cells;
            if (this.views && this.requestedView != 'Default') cells = this.views[0].cells;
            // now cells will always points to the cells in the default view

            // Run over all attributes in the default view
            if (cells) {
                for (var row=0; row<cells.length; row++) {
                    var rowArr = cells[row];
                    if (rowArr) {
                        for (var col=0; col<rowArr.length;col++) {
                            var cellObj = cells[row][col];
                            if (cellObj && cellObj.type=='Attribute') {
                                var attObj = cellObj.attObj;
                                if (attObj && attObj.type ) {
                                    var theName;
                                    var attValue = attObj.value;

                                    if (attObj.type.toLowerCase() == 'datetime' && attValue && !attObj.rs) {  // if it's a date and there is actually a date filled out
                                        theName = this.prefix + 'date_period_of_day_' + attObj.id;
                                        attValue = attValue.getHours() * 2 + Math.floor(attValue.getMinutes()/30);							
                                    } else {									
                                        theName = this.prefix + attObj.id;	
                                        if (attObj.type.toLowerCase() == 'richtext'	) attValue = WCXML.QuoteEncode(attValue);
                                    }
                                    var exists = document.getElementsByName(theName);  // check whether such an element already exists (meaning it is put there mostly editable). If not, put it hidden.
                                    if (!exists || exists.length==0) {
                                        htmlcode += '<input type="hidden" name="'+theName+'" value="'+  WCXML.xmlencode(attValue)+'">';
                                        htmlcode += this.renderHiddenAttributeInfo(attObj);
                                    }

                                }

                            }
                        }
                    }

                }
            }
            var hiddenFieldsContainer = document.getElementById(this.prefix + "hiddenFieldsContainer");
            if (hiddenFieldsContainer) hiddenFieldsContainer.innerHTML = htmlcode;
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.addDefaultHiddenFields' + ACatStrings.t('ACatAtLine') + err.lineNumber + ': ' +  err.message );	
        }
    };



    /**
    * Function not used, could be used later
    */
    attributeCategory.prototype.noenter = function() {
        var flag = true;    
        if(window.event && window.event.keyCode == 13) {
            flag = false;
        }
        return flag;
    };

    /**
    * Corrects for a flaw in the design: 
    * The server knows the time zone offset of the client: at login, a little javascript is run which submits the 
    * clients time zone offset to the server.
    * Then the server uses this time zone offset to interpret the time zone of the entered data
    * However, it falsely assumes that the entered dates are in the same time zone as the time zone of the client.
    * This goes wrong when the client is in a country with daylight saving.  
    * When in August, a user is entering 10 December 10:00, this user means 10:00 winter time.
    * However, the server assumes Summer time since it's August...
    * The misery is that this bug made it to the database and is stored wrongly.
    * So the problem cannot be corrected here. All we can do is show the information consistently.
    * consistency means that at least the date is shown the same when clicking on Change (and the page reloads). 
    * Without the below code, the date can shift an hour each time you click change (if the entered date is in Summer while it's current winter or reversely). 
    */
    attributeCategory.prototype.correctForDaylightSaving = function (date) {

        try {
            var shift = date.getTimezoneOffset() - (new Date()).getTimezoneOffset();
            if (shift !=0) date = new Date(date.getTime() + shift*60000); 
        } catch (err) {
            // do no correction 		
        }
        return date;
    };

    var logger ='';

    /**
    * called when you click on a button defined in the category
    * @param {Obj} buttonObj - Object of the button to be executed. If null, elem will be checked to find the button
    * @param {DOMObj} elem - the button DOM element. Required if buttonObj == null. Its id looks like 000002_0003233423ACatButton~1~0~0. First part is an ID which never contains a ~
    * @param {String} clickType - on of 'onLoad', 'onSave', 'click', 'onChange'
    */

    attributeCategory.prototype.clickACatButton = function (buttonObj, elem, clickType) {
        "use strict";
        var errorObj=null;
        try {
            var row,col, cellObj, buttonObj,  button;

            if (!buttonObj) {
                var arr = elem.id.split("~");
                row = arr[1];
                col = arr[2];
                button = arr[3];

                cellObj = this.cells[row][col];
                buttonObj = cellObj.buttons[button];
            }

            if (buttonObj) {
                var listObj = buttonObj.listObj;
                switch (buttonObj.func) {  
                    case 'LookUpAttributes':
                        errorObj = this.clickACatButtonLookUpAttributes(buttonObj, clickType=='onSave');
                        break;
                    case 'JavaScript':
                        errorObj = this.clickACatButtonJavaScript(buttonObj, clickType);
                        break;				
                    case 'CheckUniqueValue':
                        // find the attObj for which uniqueness needs to be tested
                        var attObj = this.findAttObjToCheck(buttonObj);

                        if (attObj) this.clickACatButtonCheckUniqueValue(attObj);  // currently not foreseen on error handling since treated separately in validation
                        break;
                    default:
                        alert('This function is not implemented:' + buttonObj.func);
                }
                buttonObj.clicked++;
            }

        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.clickACatButton' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
        } 
        return errorObj;

    }

    /**
    * returns the attObj of the cell in which a button resides. If this cell does not have an attibute, returns null.
    * typically used for finding the attribute to check uniqueness on when its button is clicked (the buttonObj does not know itself where it is)
    */
    attributeCategory.prototype.findAttObjToCheck = function(buttonObj) {
        "use strict";
        var attObj=null;
        try {
            var found = false;
            for (var row=0; row < this.nRows  && !found; row++) {
                if (!this.cells[row]) continue;
                for (var col=0;col < this.nCols  && !found; col++) {
                    var cellObj = this.cells[row][col];
                    if (cellObj) {		
                        if (cellObj.buttons && cellObj.buttons.length > 0) {

                            for (var button = 0; button < cellObj.buttons.length; button++) {
                                if (buttonObj == cellObj.buttons[button]) {
                                    found = true;
                                    break;
                                }

                            }
                            attObj = cellObj.attObj;
                        }				
                    }
                }
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.findAttObjToCheck' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
        } 
        return attObj;
    };

    /**
    * called when you click on a button of the type LookUpAttributes
    * @param {Object.<buttonObj>} buttonObj: The buttonObj for the clicked button
    * @param {Boolean} synchronous - Whether the look up call will freeze the UI waiting until the answer comes back, only advised when validating the form
    * @param {Obj} errorObj - validation error object returned only when the call is synchronous
    */

    attributeCategory.prototype.clickACatButtonLookUpAttributes = function (buttonObj, synchronous) {
        "use strict";
        var errorObj=null;
        try {

            var listObj = buttonObj.listObj;			
            var arr, att;
                var url = window.webcenterConfig.jspBaseUrl + 'GetRowsFromList.jsp?';
            var inputsGiven = false;
            if (synchronous !== true) synchronous = false;  // false, undefined, 0 (even 1...) all go to false

            // add list
            if (!listObj) throw new Error ('list is not or no longer defined');
            url += 'list=' + WCXML.urlencode(listObj.name);
            url += '&listType=WCR_DB_List';  
            url += '&maxrows=1';   // no maxcolrows needed when not indexed
            url += '&outputType=XML';

            arr= buttonObj.inputAtts;
            if (arr && arr.length >0) {						
                for (att=0; att< arr.length; att++) {
                    if (arr[att].attObj && arr[att].attObj.value !== '' && arr[att].columnObj) {
                        var attValue = arr[att].attObj.value;
                        url += '&input=' + WCXML.urlencode(arr[att].columnObj.columnName) + ':' + (arr[att].exact? '1' : '0') + ':'  + WCXML.urlencode(attValue); 
                        inputsGiven = true;
                    }
                }
            }
            var arr= buttonObj.outputAtts;

            if (arr && arr.length >0) {								
                for (att=0; att< arr.length; att++) {							
                    // url += '&output=' + WCXML.urlencode(arr[att].columnName);
                    // To do: bring this back when output works correctly on the JSP call
                }
            }

            if (!inputsGiven) {alert(ACatStrings.t('ACatNoInputValues')); return;}

            var aCatObj = this;  // closure and also buttonObj is a closure

            AjaxCache.ajax({
                url: url,				
                type: "GET",
                dataType: "xml",
                cache: false,
                async: !synchronous   // synchronous only when called from validation 
            }).done(function(returnXML) {	
                try {
                	/* Ajax call returns an XML like following:
                    <result>
                        <header>
                            <column>trademark</column>
                            <column>brand</column>
                            <column>flavor</column>
                            <column>bfn1</column>
                            <column>brand_formula_number</column>
                            <column>atc_brand_code</column>
                            <column>brand_nickname</column>
                        </header>
                        <row>
                            <value>Retired Tropicana Juice Drinks</value>
                            <value>Sugar Free Fruit Punch</value>
                            <value>Acai Fruit Punch</value>
                            <value>2723</value>
                            <value/>
                            <value>42</value>
                            <value>Tampico</value>
                        </row>
                    </result>
                    */
                    var column;
                    var mainElem = WCXML.GetFirstFirstLevelChild(returnXML,"result");

                    var headerElem =  WCXML.GetFirstFirstLevelChild(mainElem,"header");
                    var columnNameArr =  WCXML.GetFirstLevelChilds(headerElem,"column");
                    var columnNames = [];  // list of columnnames as they returned from the call

                    var rowElem = WCXML.GetFirstFirstLevelChild(mainElem,"row");	// only first row is read						
                    var columnArr = WCXML.GetFirstLevelChilds(rowElem,"value");

                    // read the column names first
                    if (columnNameArr && columnNameArr.length>0) {
                        for (column=0; column <	columnNameArr.length; column++) {
                            columnNames.push(WCXML.GetElementSelfTextContent(columnNameArr[column]));
                        }
                    }

                    // read the only row we read...
                    if (columnArr && columnArr.length>0) {
                        var arr = buttonObj.outputAtts;
                        for (var column=0; column < columnArr.length; column++) {
                            var columnName = columnNames[column];
                            for (var att=0; att < arr.length; att++) {
                                if (arr[att].columnObj && arr[att].columnObj.columnName == columnName) {
                                    var newValue = WCXML.GetElementSelfTextContent(columnArr[column])
                                    if (newValue != arr[att].attObj.value) {
                                        arr[att].attObj.value = newValue;							
                                        aCatObj.onChangeAttValue( arr[att].attObj );										
                                    }
                                    break;
                                }
                            }
                        }
                    } else {						
                        if (synchronous) {
                            errorObj = {errorMsg : ACatStrings.t('ACatNoFittingRow'), errorCode : 'ACatNoFittingRow'};
                        } else {
                            alert(ACatStrings.t('ACatNoFittingRow'));
                        }
                    }
                    aCatObj.displayAttributeCategory();
                } catch (err) {
                    alert("Error processing return of ajax call to GetRowsFromList.jsp: " + err ); 
                }
            }).fail(function(errObj){
            	if(errObj.status !== 0){
            		alert("Error in Ajax call to GetRowsFromList.jsp " + errObj.responseText);
            		if (synchronous) {
                    	errorObj = {errorMsg : 'Error in Ajax call to GetRowsFromList.jsp', errorCode : 'Error in Ajax call to GetRowsFromList.jsp'};
                    }
            	}
            });

        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.clickACatButtonLookUpAttributes' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
        } 
        return errorObj;

    }

    /**
    * called when you click on a button of the type JavaScript
    * @param {Object} buttonObj: The buttonObj for the clicked button
    * @param {Boolean} clickType - determines whether the call is done at Save time, typically to suppress alerts
    */

    attributeCategory.prototype.clickACatButtonJavaScript = function (buttonObj, clickType) {
        "use strict";
        var errorObj=null;
        var docatt, projatt, bomdocatt;
        try {

            var inProjectCreation = false;
            var projectAttElem = document.getElementById('projectAttributesBlockId');
            if (projectAttElem) inProjectCreation = true;  // To do: evaluate whether a more robust check can be done, f.e. via this.type		

            // construct the att Object
            var att = {};
            var attOld = {};
            for (var attIdx = 0;  attIdx < this.attributes.length; attIdx++) {
                var attObj = this.attributes[attIdx];
                if ( attObj && attObj.placed) {
                    //att[attObj.name] = attObj.value;  // almost all strings, except dates which are of Date type
                    att[attObj.name] = this.getCalculationValue(attObj);
                    attOld[attObj.name] = this.getCalculationValue(attObj);

                }
            }

            // construct projatt
            if (buttonObj.javaScript.indexOf('projatt') != -1 ) {

                if (inProjectCreation) {			
                    // try constructing a projatt object				
                    if (projectAttElem && projectAttElem.ACatObj && projectAttElem.ACatObj.attributes) {
                        projatt = {};
                        var projAttArr = projectAttElem.ACatObj.attributes;
                        for (var attIdx = 0;  attIdx < this.attributes.length; attIdx++) {
                            var attObj = projAttArr[attIdx];
                            if ( attObj && attObj.placed) {
                                projatt[attObj.name] = this.getCalculationValue(attObj);
                            }
                        }
                    }				

                } else {
                    // always go reading it, even if it was loaded already for copy from project att to task specifications. Reason: not the same attributes! You need all project attributes, and really the latest.
                    projatt = this.getProjectAttributeValuesForButton();  // synchronous ajax call			
                }
            }

            // construct docatt
            docatt = [];
            bomdocatt = [];
            if (buttonObj.javaScript.indexOf('docatt') != -1 ) this.createDocAttForButton(docatt, bomdocatt);   // the if will also detect bomdocatt since docatt is part of it


			// utility functions are provided through customFormExtension (a file located in the custom folder of the WebCenter instance)
            errorObj = this.executeJavaScriptButton.call(customFormExtension,buttonObj.javaScript, att, projatt, buttonObj.label, clickType, docatt, bomdocatt);  // 'call' takes 1 extra parameter, which is the definition of this. 

            // Validate and update the values - To Do: think of moving this in a separate function
            var msg = '';
            var anythingChanged = false;
            for (var attIdx = 0;  attIdx < this.attributes.length; attIdx++) {
                var attObj = this.attributes[attIdx];
                var validChange = true;  // will be set to false if the new value is not of the right type 
                var isChanged = true; 
                var newValue =  att[attObj.name];			
                var oldValue = attOld[attObj.name];

                if ( attObj && attObj.placed  && attObj.type ) {  // mind !== instead of != otherwise you cannot set via JavaScript from '' to 0 because it will think it's the same

                    // validate
                    switch ( attObj.type.toLowerCase() ) {
                        case 'datetime':
                            if ( newValue !== '' && newValue.constructor != Date ) {
                                validChange = false;
                                msg += '\nNew Value for attribute ' + attObj.name + ' is not a Date type';							
                            } 
                            if (newValue === oldValue) isChanged = false;   
                            break;	
                        case 'text':
                            // check maximum length and type
                            if ( newValue.constructor == Number) newValue = newValue.toString();
                            if ( newValue.constructor != String ) {
                                validChange = false;
                                msg += '\nNew Value for attribute ' + attObj.name + ' is not a string';
                            } 
                            if (newValue === oldValue) isChanged = false;
                            break;
                        case 'richtext':
                            isChanged = false;
                            if(newValue !== oldValue){
                                validChange = false;
                                msg += '\nYou cannot set a new value for rich text attributes. Attribute: ' + attObj.name ;
                            }
                            break;
                        case 'documentreference':
                        	isChanged = false;
                        	break;
                        case 'float':  
                        case 'integer':					
                        case 'area':
                        case 'area1':
                        case 'area2':
                        case 'volume':
                        case 'volume1':
                        case 'volume2':
                        case 'weight':
                        case 'weight1':
                        case 'distance': 
                        case 'distance1':


                            if (newValue === oldValue) isChanged = false;
                            if ( newValue !== '' && newValue.constructor != Number || isNaN(newValue) )  {
                                validChange = false;
                                msg += '\nNew Value for attribute ' + attObj.name + ' is not a number';
                            } else {
                                if (newValue !== '' && attObj.type.toLowerCase() == 'integer') {
                                    newValue = parseInt(newValue);  // cut off to integer								
                                }
                            }
                            break;
                    }  // end switch

                    if (validChange && isChanged) {
                    	anythingChanged = true;
                        attObj.value = newValue;  
                        this.onChangeAttValue(attObj);	

                        // list values need to be propagated
                        if(attObj.listObj){
                            if(attObj.listObj.loadingType == 'OnTheFly'){
                                this.checkAutoSelectLazyLoading(attObj,false);
                            } else {
                                this.changeListSelection(attObj,newValue);
                            }
                        }
                    } 
                }
            }
            if (msg) alert('Problem executing button ' + buttonObj.label + msg);

            if(anythingChanged){
            	this.displayAttributeCategory();  // anything can have changed so full repaint.
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.clickACatButtonJavaScript' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
        } 
        return errorObj;
    };

    attributeCategory.prototype.createDocAttForButton = function(docatt, bomdocatt) {
        "use strict";
        try {
            // try constructing a docatt Object - will only work in project creation
            var docIdx = 0, docIdxBOM = 0;
            var aCatObj= this;
            var prefix = "documentRowId";
            
            // run over all document rows
            // get doctableX for normal project documents
            if($("#documentsTableContainerDiv")[0] && $("#documentsTableContainerDiv")[0].docTable){
            	var documentsTable = $("#documentsTableContainerDiv")[0].docTable;
            	
            	if(documentsTable && documentsTable.docRows){
            		_.each(documentsTable.docRows, _.bind(function(docRow){
            			// create associative array for document, even if no attributes exist (empty object)
                        var assoArray;
                        assoArray = docatt[docIdx] = {};
                        docIdx++;

                        if(docRow.aCatObj){
                        	var docAttArr = docRow.aCatObj.attributes;
	                        for (var attIdx = 0;  attIdx < docAttArr.length; attIdx++) {
	                            var attObj = docAttArr[attIdx];
	                            if ( attObj && attObj.placed) {
	                                assoArray[attObj.name] = aCatObj.getCalculationValue(attObj);
	                            }
	                        }
                        }
                        
            		}, this));
            	}   	
            }
            
            // for BOM documents, run over DOM
            $('[id^="'+prefix+'"]').each( function(index) {
                var curId = $(this).attr('id');  
                
                // can be documentRowId1/documentRowId1BOM or documentRowId1Attributes/documentRowId1BOMAttributes
                // --> Skip attribute rows
                if (curId.indexOf('Attributes') === -1){
                	var isBOM = false; 
                    curId = curId.substring(prefix.length);
                    if (curId.indexOf('BOM') != -1){
                    	isBOM = true;
                    } 

                    // create associative array for document, even if no attributes exist (empty object)
                    var assoArray;
                    if (isBOM) {
                        assoArray = bomdocatt[docIdxBOM] = {};
                        docIdxBOM++;
                        
                        // run over attributes to fill associative array
                        var attCatElem = $("#documentDivId"+curId+"Attributes"); // Note that "BOM" was not stripped from the ID
                        if (attCatElem.length === 1) { 
                            var docACatObj = attCatElem[0].ACatObj;  // get this document's attribute category object
                            if (docACatObj) {                            
    	                        var docAttArr = docACatObj.attributes;
    	                        for (var attIdx = 0;  attIdx < docAttArr.length; attIdx++) {
    	                            var attObj = docAttArr[attIdx];
    	                            if ( attObj && attObj.placed) {
    	                                assoArray[attObj.name] = aCatObj.getCalculationValue(attObj);
    	                            }
    	                        }
    	                    }
    	                }
                    }                               
                }       
            });
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.createDocAttForButton' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
        } 
        return docatt;
    };

    attributeCategory.prototype.getProjectAttributeValuesForButton = function() {
        "use strict";
        var projatt = [];
        try {
            var url = window.webcenterConfig.jspBaseUrl + 'GetAttributes.jsp?projectid=' + this.projectID;
            
            var attCatObj = this;
            AjaxCache.ajax({
                url: url,
                type: "GET",
                dataType: "xml",
                cache: false,
                async: false,
                timeout: 120000
            }).done(function(valuesXML) {
                try {
                    if (!valuesXML) throw(ACatStrings.t('ACatAttributeProblem'));
                    var errorElem = WCXML.GetFirstFirstLevelChild(valuesXML,"error");
                    if (errorElem) {
                        var errorCode = WCXML.GetElementText(errorElem,"code");
                        // if error code is 043 (no attributes), just do nothing
                        if (errorCode != "043") {
                            var errorMsg = WCXML.GetElementText(errorElem,"message");
                            alert(ACatStrings.t('ACatErrorReadingAttributes') + ":\n" + errorMsg);							
                        }
                        return;
                    }

                    var attributeCategoryElem=WCXML.GetFirstFirstLevelChild(valuesXML,"attribute_category");
                    if (!attributeCategoryElem){
                    	throw(ACatStrings.t('ACatServerXMLProblem3'));
                    }

                    var attributesElem=WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"atts");
                    if (attributesElem) {

                        // read the attribute values
                        var attArr = WCXML.GetFirstLevelChilds(attributesElem,"att");
                        if (attArr && attArr.length>0) {
                            for (var attribute = 0; attribute < attArr.length; attribute++) {
                                var attElem = attArr[attribute];
                                var attName = WCXML.GetElementText(attElem,"an");
                                if (attName) {			
                                    var value =  WCXML.GetElementText(attElem,"av");									
                                    var type = WCXML.GetElementText(attElem,"at");
                                    var rsId = WCXML.GetElementText(attElem,"rs");	

                                    value = attCatObj.getCalculationValue({type: type, value:value});  // convert strings to float/integer for numbers. Does nothing with date

                                    // convert date to Date format
                                    if (type && type.toLowerCase() == 'datetime' && !rsId && value ) {		
                                        value = attCatObj.correctForDaylightSaving(new Date(parseInt(value)));
                                    }
                                    projatt[attName] = value;

                                }
                            }
                        }

                    }
                } catch (err) {
                    alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.getProjectAttributeValuesForButton read Ajax return' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
                }		 
            }).fail(_.bind(this.readErrorObj, this));
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.getProjectAttributeValuesForButton' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
        } 
        return projatt;
    };
		
    /**
    * Converts the typical string format of attribute values (strings even for numeric types!) into numbers and leaves the rest untouched
    * @param attObj - attribute object containing the value in attObj.value, must have attObj.type filled out correctly
    * @return The value as will be used in JavaScript calculations
    */
    attributeCategory.prototype.getCalculationValue = function (attObj) {
        "use strict";
        var returnValue = attObj.value;
        try {		

            if (attObj.type && attObj.value) { 
                switch ( attObj.type.toLowerCase() ) {
                    case 'integer': 				
                        returnValue = parseInt( returnValue );  // also converts "0"  to 0. 
                        break;

                    case 'float': case 'area': case 'area1': case 'area2':
                    case 'volume': case 'volume1': case 'volume2': case 'weight': case 'weight1':
                    case 'distance': case 'distance1':
                        returnValue = parseFloat( returnValue ); 

                        break;
                    // default: do nothing
                }
            }
            // else it remains what it was, which might be 0 or ''. So for one of the number types '' is NOT converted to 0 (otherwise there is no way to see the difference anymore which has business value)
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.getCalculationValue' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
        }
        return returnValue;
    }

    /**
    * Executes the code of a JavaScript Button
    * @param {String} code - the code
    * @param {Array} att - associative array of current attribute values (att['AttName'] = attValue)
    * @param {Array} projatt - associative array of current project attribute values, only available for document attributes and task specifications, null when editing project attributes
    * @param {String} buttonLabel - label of the button (used to give feedback)
    * @param {String} clickType - event which triggered the button (same as in clickACatButton)
    * @param {Array.Array} docatt - array of associative arrays of document attributes. docatt[n]['AttName']  is the value of document attribute with name AttName in document n in project creation's documents section
    * @param {Array.Array} bomdocatt - same as docatt but for the documents in the BOM section of project creation
    */
    attributeCategory.prototype.executeJavaScriptButton = function(code,att, projatt, buttonLabel, clickType, docatt, bomdocatt) {
	
        'use strict';
        var errorObj = null;

        try	 {

            // first make a couple of globals unavailable by masking them with a local variable 
            var codePrefix = 'var document = null, storedObj = null, aCatStoredObj=null, jQuery=null, $ = null, window = null';
            codePrefix += ', keywordsearchform=null, detailslinksform=null, docactionform= null, actionform = null, actionform2 = null, inputform = null';
            codePrefix += ', attributeCategory=null, tinyMceMgr = null, ajax = null, setTimeout = null, setInterval = null;';

            var args = 'att, projatt, event, docatt, bomdocatt'; 

            var myFunc = new Function(args,codePrefix + code);   // no longer uses eval to allow returning an errorObj
            errorObj = myFunc.call(this,att,projatt, clickType, docatt, bomdocatt);  // clickType is given through as 'event'
            if (errorObj && clickType == 'click') alert(errorObj.errorMsg);  // only chat when people have clicked the button
        } catch (err) {
            alert('Problem executing button ' + buttonLabel + '\nCheck your script!\n' + err);
        }
        return errorObj;
    };

    /**
     * Called after all values (also from project when in TEP) are loaded and after attribute category is displayed
     */
    attributeCategory.prototype.onValuesLoaded = function() {
        try	 {
        	// execute all buttons which are marked to run 'on load'
            this.executeButtonsOnLoad();

            // notify other same attribute categories assigned to the same object (project, document, task) that the new attribute 
            // category (this one) is initialized and rendered
    		AttributeEventBus.getInstance().emitCategoryLoaded(this);

        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.onValuesLoaded' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);
        }
    };

    /**
    * execute all buttons which are marked to run 'on load'
    */
    attributeCategory.prototype.executeButtonsOnLoad = function() {
        'use strict';
        try	 {
            for (var row=0; row < this.nRows; row++) {
                for (var col=0;col < this.nCols; col++) {
                    var cellObj = this.cells[row][col];
                    if (cellObj) {		
                        if (cellObj.buttons && cellObj.buttons.length > 0) {

                            for (var button = 0; button < cellObj.buttons.length; button++) {
                                var buttonObj = cellObj.buttons[button];
                                if (buttonObj.execOnLoad) {

                                    var errorObj = null;
                                    errorObj = this.clickACatButton(buttonObj, null,'onLoad');	
                                    if (errorObj && errorObj.errorMsg ) alert(buttonObj.label + ' : ' + errorObj.errorMsg);
                                }
                            }
                        }				
                    }
                }
            }


        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.executeButtonsOnLoad' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);
        }

    };

    /**
    * Called when you click on a button of the type CheckUniqueValue.
    * @param {Attribute} attObj: The attribute to be checked.
    */
    attributeCategory.prototype.clickACatButtonCheckUniqueValue = function (attObj) {
	   return this.validateAttributeUniqueValue(attObj, true);
    };


    /**
    * to be called when an attribute value is changed programatorically
    * Guarantees that all needed additional operations (f.e. for cascading dropdowns) are done
    * Depending on type of attribute value repaint can be done in this method or left for the caller.
    * 
    * @param {Attribute} attObj: The attribute that was changed.
    */
    attributeCategory.prototype.onChangeAttValue = function (attObj) {
        // To do: this function could be used more than it is today (f.e. after loading project attributes)
        try {
            var obj = this.findCell(attObj);  
            if (obj) {
                var cellObj = obj.cellObj;
                var row = obj.row;
                var col = obj.col;

                // this attribute category has to be repainted whenever there is a change imposed 
                // by the attribute values synchronization logic even/when it's read-only! 
                // (either the whole category or particular attribute)             	
                var isRepaintNeeded = !this.isAttrCatEditable() || !this.isAttrEditable(cellObj);
                
                var elem = this.getAttributeInDOM(attObj);
                if (elem) {                	
                    switch (elem.tagName.toLowerCase()) {   // To do : test with all types of controls including checkbox etc
                        case 'input':
                            elem.value = attObj.value;
                            if (attObj.isDocReference()) {
                            	this.changeDocReference(attObj);
                            	// (repaint is handled by documents table itself)
                            	isRepaintNeeded = false;
                            } else if (attObj.isDateTimeNotInRS()) {
                                attObj.processDateAttrValue();
                                this.executeButtonDependencies(attObj);
                                isRepaintNeeded = true;
                            } else if (attObj.columnObj) {
                            	// attribute is part of cascading dropdown - redo auto-selection
                                if (attObj.listObj.loadingType == 'OnTheFly'){
                                    this.checkAutoSelectLazyLoading(attObj, true);
                                } else {								
                                    this.changeListSelection(attObj, attObj.value);
                                }
                                this.executeButtonDependencies(attObj);
                                isRepaintNeeded = true;
                            } else {
                           		this.changeText(elem,row,col);
                           		if (cellObj.renderOption == 'Images' || cellObj.renderOption == 'RadioCheckboxes'
                           			|| attObj.rs || attObj.allowMultipleValues || attObj.listObj) {
                           			isRepaintNeeded = true;
                           		}
                            }
                            break;
                            
                        case 'select':
                            elem.value = attObj.value;
                            this.changeSelect(elem,row,col);
                   			isRepaintNeeded = true;
                            break;
                            
                        case 'textarea':   // for rich-text attributes
							if(attObj.isRichText()){ 
								$("#"+elem.id).summernote('code', attObj.value); 
							} else { 
								elem.value = attObj.value; 
							} 
							this.changeText(elem,row,col);
                        	break;
                    }
                } else {
                    // also execute button dependencies for read-only values
                    this.executeButtonDependencies(attObj);	
                }
                this.setAttributesChanged();
                
                if (isRepaintNeeded) {
                	// repaint the attribute value itself as its value is not directly rendered (checkboxes, radiobuttons, etc.)
	                var forPrint = false;   // print layout doesn't support changes - no "on change" handler should be called in that context
	                $("#"+this.prefix+'td_'+row+'_'+col).html(this.renderAttributeValueInternal(cellObj, attObj, row, col, this.checkVisibility(cellObj), undefined, forPrint));
                } 
                
                this.checkRepaintAttributeCategory(attObj);               
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.onChangeAttValue' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
        }
    };

    /**
    * execute buttons which are dependent on the attribute represented by attObj
    */
    attributeCategory.prototype.executeButtonDependencies = function (attObj) {
        'use strict';
        try {

            // to avoid endless loop, we keep track of which attObj are being executed. Only checks synchronous loops. For asynchronous loops, there is a counter, see cMaxButtonClick.		
            if (!this.pendingExecuteButtonDependencies) this.pendingExecuteButtonDependencies = [];
            var found=false;
            for (var att=0; att< this.pendingExecuteButtonDependencies.length; att++) {
                if (this.pendingExecuteButtonDependencies[att] == attObj) {
                    found=true; break;
                }
            }
            if (!found) {
                this.pendingExecuteButtonDependencies.push(attObj);
                if (attObj.buttonDependency && attObj.buttonDependency.length>0) {
                    for (var dep = 0; dep < attObj.buttonDependency.length; dep++) {
                        var buttonObj = attObj.buttonDependency[dep];
                        //logger += '\nButton ' + buttonObj.label + ' clicked at ' + new Date() + '  count ' + buttonObj.clicked + ' attribute: ' + attObj.name ;
                        if (buttonObj && buttonObj.viewName == this.currentView) {
                            if (buttonObj.clicked > cMaxButtonClick) {   // if button too many times clicked, this is probably an endless loop. Stop clicking it automatically at least (they can still click by hand)
                                var errMsg = '';
                                if (buttonObj.style == 'hidden') errMsg += 'Hidden ';
                                errMsg += 'Button ' + buttonObj.label + ' clicked more than '+ cMaxButtonClick + ' times. \n Please check for endless loops.';
                                throw new Error(errMsg);
                            } else {
                                var errorObj = null;
                                errorObj = this.clickACatButton(buttonObj, null,'onChange');	
                                if (errorObj && errorObj.errorMsg ) alert(buttonObj.label + ' : ' + errorObj.errorMsg);				
                            }							
                        }
                    }
                }
                // pull it off again
                for (var att=0; att< this.pendingExecuteButtonDependencies.length; att++) {
                    if (this.pendingExecuteButtonDependencies[att] == attObj) {
                        this.pendingExecuteButtonDependencies.splice(att,1); break;
                    }
                }			
            } 
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + ' attributeCategory.executeButtonDependencies' + ACatStrings.t('ACatAtLine') + err.lineNumber + ' : ' + err.message);	
        }
    };

    attributeCategory.prototype.alertSessionLost = function alertSessionLost() {
	   alert(sessionLostMsg);
    }


    ////////////////////////////////////////////////////////////////////////////////////////////////
    // DOCUMENT REFERENCES
    ////////////////////////////////////////////////////////////////////////////////////////////////



    /**
     * Called by documents table of a changed document reference attribute.
     */
    attributeCategory.prototype.changeDocReference = function(attObj) {
        try {
        	if (!attObj) {
        		return;
        	}
            // updating attribute value (attObj.value) based on value composed by docTable
        	// the attObj.value consists of referenced document names delimited as a multivalue attribute
        	var docTable = attObj.docTable;
        	if (docTable) {
            	attObj.setDocReferenceValue(docTable);
        	}

            this.setAttributesChanged();
            attObj.changed = true;
            this.checkTaskSyncWarning(attObj);
            this.executeButtonDependencies(attObj);
            
            this.emitAttributevalueChanged(attObj);
            
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.changeDocReference: " + err);
        }
    };



    /**
     * Renders a documents table placeholder for given document reference attribute.
     * In case of header view just hyperlinks of referenced documents are rendered instead of the full
     * documents table.
     */
    attributeCategory.prototype.renderDocumentReferencePH = function(cellObj, row, col) {
        var htmlcode = '';
        try {
        	if (!cellObj || !cellObj.attObj) return htmlcode;
        	
        	var docTable = cellObj.attObj.docTable;

            if (this.currentView != 'Header') {
	            // skip rendering of documents table if there is nothing to render
	            if (!docTable) return htmlcode;

                // add placeholder HTML element at the place where the whole document table should be rendered
                htmlcode += '<div class="docrefattribute main-content" id="' + docTable.phElemID + '"></div>';

            } else {
                //htmlcode += WCXML.xmlencode(cellObj.attObj.value);
                if (cellObj.attObj.valueHyperlink) {
                    htmlcode += cellObj.attObj.valueHyperlink;  // show hyperlinks. Encoding is already handled
                }
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.renderDocumentReferencePH:\n' + err);
        }
        return htmlcode;
    };



    /**
	 * Method to prepare document reference attributes to be saved.
	 * It MUST be called before submit of any attribute category!
	 * It should be called only if validation is OK or not needed.
	 * This is for document reference attributes only!
	 * 
	 * The method internally calls onSubmit() for each initialized documents table within the 
	 * attribute category recursively generating the XML, which is send to server on submit.
	 * Without the XML server won't process any changes in document references attributes.
	 */
    attributeCategory.prototype.convertDocReferencesToXML = function() {
        if (!this.loaded && !this.errorMsg) return;
        // go over all attributes in this attr. cat. and execute onSubmit() method on each document 
        // reference attribute with already initialized documents table
    	this.executeOnAllDocReferences(function(docTableObj) {
    		// 'docTableObj' is always the "current" documents table object!
    		docTableObj.onSubmit();
    	}, this);
    };

    
    /**
     * Method taking care of updating documents tables edit-ability based on doc. ref. attribute visibility
     * type. It's called whenever the attribute category is about to be displayed to make sure all
     * visibility type changes in attribute category (done via a rule for example) are catched.
	 */
    attributeCategory.prototype.updateDocReferencesEditability = function() {
        if (!this.loaded && !this.errorMsg) return;
        
        // precalculate whether the whole attribute category is editable (logged in user has to have appropriate rights)
        var canUserEditAttributes = this.isAttrCatEditable();

        // go over all document reference attributes in this attr. cat. and update their edit-ability 
        // status based on passed value ('isEditable' parameter) or actual visibility within this 
        // attribute category
        var promises = [];
        for (var row = 0; row < this.nRows; row++) {
            for (var col = 0; col < this.nCols; col++) {
                if (this.cells[row] && this.cells[row][col]) {
                    var cellObj = this.cells[row][col];
                    if (cellObj.attObj) {
                        var attObj = cellObj.attObj;
                        if (attObj.isDocReference() && attObj.docTable) {
                        	(function(cellObj, attObj) {   // closure for cellObj and attObj
		                    	// continue only if the particular document row is fully loaded
                        		promises.push(attObj._docTablePromise.then(function () {
		                        	var attVisibility = this.checkVisibility(cellObj);
		                        	if (this.isInactive) {
		                        		// propagate the hidden flag - all descendants of a hidden document reference
		                        		// attribute are hidden as well...
		                        		attObj.docTable.setIsInactive(true);
		                        	} else {
		                            	// calculate if the current document reference attribute is hidden or not and 
		                            	// update the corresponding documents table accordingly.
		                                attObj.docTable.setIsInactive((attVisibility == 'Hidden') || (attVisibility == 'HiddenKeepValues'));
		                        	}
		
		                            // special handling of hidden attributes that do not keep their values...
		                        	// explicitely set this doc. ref. attribute to stop referencing all documents.
		                        	// it effectively wipe out all the document rows from the corresponding 
		                        	// documents table when (but not sooner than) submitted! 
		                        	// Note this is done only in this documents table, NOT in its children!
		                        	attObj.docTable.clearValues = (attVisibility == 'Hidden');
		
		                    		// calculate the editability based on the current document reference attribute 
		                            // visibility type and set it for this documents table and all its children!
		                            attObj.docTable.setIsEditable(this.isAttrEditable(cellObj) && canUserEditAttributes, true);
		                        }.bind(this)));
                        	}.bind(this))(cellObj, attObj);
                        }
                    }
                }
            }
        } 
        return Promise.all(promises);
    };


    ////////////////////////////////////////////////////////////////////////////////////////////////
    // CASCADING DROPDOWNS + LISTS FUNCTIONS
    ////////////////////////////////////////////////////////////////////////////////////////////////


    /**
    * Gets values for a dropdown from the WebCenter database
    * @param {DOMObject} elem - the input field OR the dropdown arrow OR a link in the pop-up, whichever has triggered this function to be called
    * @param {DOMEvent} event - the DOM event (click, keyup, ...)
    * @param {int} row - row of the current cell 
    * @param {int} col - column of the current cell 
    * @param {Boolean} showAll - if true, values are shown irrespective of whether they are allowed or not (meaning no filtering). Don't confuse with showing really all values in the DB, which is not supported, but see maxResults
    * @param {Boolean} forSearch - if true, this is called from a search params panel
    * @param {Number} maxResults - maximum number of results shown. If omitted, 200 results are shown. maxResults cannot be larger than 2000 (to avoid overload). 
    */
    attributeCategory.prototype.getDBValues = function(elem,event,row,col,showAll,forSearch, maxResults) {

        "use strict";
        try {

            var cellObj = this.cells[row][col];
            var attObj = cellObj.attObj;
            var columnObj = attObj.columnObj;
            var columnName = columnObj.columnName;
            var listObj = attObj.listObj;
            var listName = listObj.name;
            var patternUsed = elem.tagName.toLowerCase() === "input"; // remembers whether you got here through typing or by clicking on the dropdown icon
            var pattern = elem.value;
            var url,url2;
            var multiValue = false;  // whether multivalue checkboxes will be shown

            if (!forSearch) forSearch = false;  // normalize forSearch

            if (forSearch) {
                multiValue =  columnObj.searchMultiple;  // both single as multivalue attributes can be searched for with OR clauses
            } else {
                multiValue =  attObj.allowMultipleValues;
            }

            // replace elem with the input field if the user clicked on the dropdown icon or on 'show all' or in the filter of the multi-value pop-up
            // If you don't do that, the pop-up appears under the image instead of under the input field, which is a worse user experience. In case of the show all anchor it even goes top left since the thing is gone by then
            if (elem.tagName.toLowerCase() == 'img' || elem.tagName.toLowerCase() == 'a' || elem.id == 'MultiValueFilterInput') {
                var elem2;
                if (attObj.allowMultipleValues && !forSearch) {
                    elem2 = document.getElementById('mvdiv_' + this.prefix + attObj.id);
                } else {
                    elem2 = document.getElementById(this.prefix + attObj.id);
                }

                if (elem.id != 'MultiValueFilterInput') {
                    patternUsed = false;
                }
                if (elem2) elem = elem2;  // so if we fail finding the right input field, continue using the image, resulting in the pop-up to come under the image, which still works
            }

            var menuItems = [];  // dropdownmenu expects and array of html code snippets, so we give it an array of 1 element
                menuItems.push('<img src="'+window.webcenterConfig.imageBaseUrl+'ajax_loader.gif">');			
            var menuWidth = '350px', menuHeight = '500px',scrollHor = false, nohide=false, bgcolor = "#ffffff",nohideonclick=false;
            var target = this.getAttributeInDOM(attObj, true) || $(elem).closest(".input-group").find(".form-control, .form-div")[0] || elem;
            // don't rerender the whole thing if there is a content container (e.g. multivalue; keep the filter)
            if(multiValue && ($("#dropmenudiv").css("display") != "none" && $("#dropmenudiv").find("#dropmenucontainer").length > 0)){
            	$("#dropmenucontainer").html(menuItems);	
            } else {
            	dropdownmenu(target,event,menuItems,menuWidth,menuHeight,scrollHor,nohide,bgcolor,nohideonclick) ;	
            	$("#dropmenudiv").css({
                	'min-width': $(target).width()+'px',
                	'margin-top': '-1px'
                });
            }
            dropmenuobj.onchange = function(event) {};
            updateHoverPosition(target);

            if (forSearch) document.getElementById('dropmenudiv').style.zIndex = 1200;   // to put it above a jQuery dialog which sits on 1002.

            // construct the url
                url = window.webcenterConfig.jspBaseUrl + 'GetRowsFromList.jsp?listType=WCR_DB_List&outputType=JSON&indexed=1';
            url += '&list=' + WCXML.urlencode(listName);
            url += '&output=' + WCXML.urlencode(columnName);
            if (!maxResults) maxResults = 200;
            if (maxResults > cMaxResultsDBValues)  maxResults = cMaxResultsDBValues;
            url += '&maxrows=' + maxResults;  
            url += '&maxcolrows=200';


            if (pattern && patternUsed) url += '&input='+ WCXML.urlencode(columnName) + ':0:' + WCXML.urlencode(pattern);  // TO DO: encoding of columnName should also escape the : ?

            url2 = url; // the part needed to load all 

            // check for additional input values by checking other columns of the same list
            if (listObj && !showAll) {
                for (var column=0; column < listObj.columns.length; column++) {
                    var columnObj2 = listObj.columns[column];
                    if (columnObj2 && columnObj2.attObj && (columnObj2.attObj != attObj) && columnObj2.attObj.value && !columnObj2.autoSelected && !columnObj2.otherValueChosen) {
                        url += '&input='+ WCXML.urlencode(columnObj2.columnName) + ':1:' + WCXML.urlencode(columnObj2.attObj.value);
                    }
                }			

            }

            var aCatObj = this;  // closure

            // ajax call
            var callParams = {
                url: url,				
                type: "GET",
                dataType: "text",
                cache: false // Note: call will be cached by the AjaxCache, but not by the browser, as list can update 
            };
            AjaxCache.ajax(callParams).done(function(jsonContent) {  
                aCatObj.displayDBValues(jsonContent, event, elem, row, col, multiValue, attObj, forSearch, dropmenuobj,url2,showAll,pattern,callParams);				
            }).fail(function(errObj){
                // When an error occurs, check whether the URL length is too big (whatever the exact limit is, we do not know)
                // In case the URL is quite long, suggest a solution to the user
                if (url.length > 2000){
                    errorMsg = ACatStrings.t('ACatListLongURLErr') + ' (' + ACatStrings.t('ACatListLongURLDetailsErr') + ' ' + url.length + ')';
                    aCatObj.handleAjaxError(errObj);
                } 
                else{
                    aCatObj.handleAjaxError(errObj); 	
                }
                dropmenuobj.style.visibility = 'hidden';
            });
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.getDBValues:\n' + err);
        }	
    };

    /**
    * Fills dropmenudiv with values provided via jsonContent
    * @param {String} jsonContent - jsonContent returned by the Ajax call
    * @param {DOMEvent} event - event which triggers the dropdown to appear (click, keyup)
    * @param {DOMObj} elem - DOM object of the element under which the dropdown will appear
    * @param {int} row - row of the cell
    * @param {int} col - col of the cell
    * @param {Boolean} multiValue - indicates whether the dialog allows multivalue selection
    * @param {Object} attObj - attribute object of the current attribute
    * @param {Boolean} forSearch - true if this is called from the search panels
    * @param {DOMObj} dropmenuobj - DOM object to be filled with the created information
    * @param {String} urlOtherValues - url to get other values (values not fitting to current selection). If not provided, other values will not be loaded
    * @param {Boolean} showAll - if true, all values are shown irrespective of whether they are allowed or not
    * @param {String} pattern - text typed in the filter. null is allowed and is seen as the same as ''
    */
    attributeCategory.prototype.displayDBValues = function(jsonContent, event, elem, row, col, multiValue, attObj, forSearch, dropmenuobj,urlOtherValues,showAll, pattern) {
        "use strict";	
        try {           
            var jsonObj;					
            var listObj = attObj.listObj;
            var listName = listObj.name;
            var columnObj = attObj.columnObj;
            var isMobileMultiValue = WCHTML.isMobile() && multiValue;	// when mobile and multi-value, we need to add an OK button because there is no other intuitive way to close the pop-up
            if (!pattern) pattern = '';

            var jsonObj = this.readJSONResponse(jsonContent, callParams);

            if (!jsonObj.success || jsonObj.success == "0") {
                alert('Cannot get values ' + jsonObj.error.code + ' : '+  jsonObj.error.message);
                dropmenuobj.style.visibility = 'hidden';
            } else {

                var column;
                var rows_returned = parseInt(jsonObj.rows_returned,10);
                var rows_total = parseInt(jsonObj.rows_total,10);
                var curVal = attObj.value;
                var curValArr;
                if (multiValue) {
                    curValArr = curVal.split(this.multiSep);  // the set of actual values				
                }

                var htmlcode = '';
                var clickMethod;
                var selected;
                var hashArr = [];

                // read the column names first
                var rowArr = jsonObj.columns[0].values;
                if (isMobileMultiValue) htmlcode += '<div id="dbValuesDivAround" style = "height:250px; overflow-y:auto;">';

                htmlcode += '<table id="dbValues" style="margin-right:20px; margin-left: 10px;">';


                // top links for advanced functions
                clickMethod = this.functionPrefix + '.showAdvancedDBFunctions(event); return false;';
                htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctionsHide" ><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBShowAdvancedFunctions"  onclick = "'+clickMethod+'">' + ACatStrings.t('ACatShowAdvancedDBFunctions')+'</a></td></tr>';

                clickMethod = this.functionPrefix + '.clearListValues(\''+WCXML.xmlencode(listName)+'\',\''+ listObj.type +'\','+ listObj.counter +',true); return false';
                htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctions" style="display:none;"><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBResetValues" onclick = "'+clickMethod+'">' + ACatStrings.t('ACatResetListValues')+'</a></td></tr>';


                clickMethod = this.functionPrefix + '.highlightRelatedValues(\''+WCXML.xmlencode(listName)+'\',\''+ listObj.type +'\','+ listObj.counter +',1); return false';
                htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctions" style="display:none;"><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBHighlightRelatedValues" onclick = "'+clickMethod+'">' + ACatStrings.t('ACatHighlightRelatedValues')+'</a>' ;


                clickMethod = this.functionPrefix + '.highlightRelatedValues(\''+WCXML.xmlencode(listName)+'\',\''+ listObj.type +'\','+ listObj.counter +',0); return false';
                htmlcode += '<a href="#" id="getDBResetHighlight" onclick = "'+clickMethod+'">' + ACatStrings.t('ACatResetHighlight')+'</a></td></tr>';


                clickMethod = this.functionPrefix + '.getDBValues(this,event,'+row+','+ col + ',true, '+forSearch+'); return false';
                htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctions" style="display:none;"><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBShowAll"  onclick = "'+clickMethod+'">' + ACatStrings.t('ACatShowUnfiltered')+'</a></td></tr>';


                if (forSearch) {
                    htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctions" style="display:none;"><td style="cursor:default;" class="nobr" colspan="2">';
                    if (columnObj.searchMultiple) {
                        clickMethod = this.functionPrefix + '.allowSearchMultiple(event,'+row+','+ col + ',true,\''+attObj.id+'\',false); return false';
                        htmlcode += '<a href="#" id="getDBSearchMultiple"  onclick = "'+clickMethod+'">' + ACatStrings.t('ACatSearchSingle')+'</a>';
                    } else {
                        clickMethod = this.functionPrefix + '.allowSearchMultiple(event,'+row+','+ col + ',true,\''+attObj.id+'\',true); return false';
                        htmlcode += '<a href="#" id="getDBSearchMultiple"  onclick = "'+clickMethod+'">' + ACatStrings.t('ACatSearchMultiple')+'</a>';
                    }
                    htmlcode += '</td></tr>';
                }

                // select all, only for multivalue
                if (multiValue) {						
                    clickMethod = this.functionPrefix + '.selectAllMultiValue(); return false; ';
                    htmlcode += '<tr class="darktext_greenlink_11"><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBSelectAll"  onclick = "'+clickMethod+'">' + ACatStrings.t('ACatSelectAll')+'</a>';
                    clickMethod = this.functionPrefix + '.deselectAllMultiValue(); return false; ';
                    htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatDeselectAll')+'</a>';
					if (listObj.allowOtherValues) {
						clickMethod = this.functionPrefix + '.otherMultiValue(); return false; ';
						htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+ACatStrings.t('ACatOtherValue')+'</a>';
                    }
					htmlcode += '</td></tr>';

                }


                if (multiValue) {							
                    var keyUpMethod = this.functionPrefix + '.getDBValues(this,event,'+row+','+ col + ',true, '+forSearch+');';
                    htmlcode += '<tr class="darktext_greenlink_11"><td colspan="2">';
                    htmlcode += '<input class="form-control" type="text" id="MultiValueFilterInput" autocomplete = "off" onkeyup="'+keyUpMethod+'" style="margin: 5px" />';  // value is set later to have the cursor at the end
                    htmlcode += '</td></tr>';
                }

                var htmlcode2 = '';
                if(multiValue){
            		htmlcode += '<tr><td colspan="2"><div id="dropmenucontainer" style="width: 100%; display: block;">';
            		htmlcode2 = '<table style="width: 100%" id="mvselect_valuetable">';
            	}                

                // add entry for "other value" 
                if(pattern && attObj.listObj.allowOtherValues){
                	if (!multiValue) {
                        clickMethod = this.functionPrefix + '.selectDBValue(\''+attObj.id+'\', \''+WCXML.xmlencode(WCHTML.encodeForJScript(pattern))+'\','+showAll+',\''+elem.id+'\'); return false';
                        htmlcode2 += '<tr class="darktext_greenlink_11"><td onclick="'+clickMethod+'" style="padding-left: 10px; cursor:default;" class="nobr">' + WCXML.xmlencode(pattern)+' ('+ACatStrings.t('ACatOtherValue')+')</td></tr>';
                    }
                }
                
                // first put the valid set
                if (rowArr && rowArr.length >0) {	


                    for (var row2=0; row2<rowArr.length; row2++) {		
                        var value =  rowArr[row2];


                        if (multiValue) {									
                            // figure out whether this one is selected
                            selected = false;
                            for (var idx=0; idx < curValArr.length; idx++) {
                                if (curValArr[idx] == value && value !='') {
                                    selected=true;								
                                }

                            }
                            
                            htmlcode2 += '<tr class="darktext_greenlink_11">';
                            htmlcode2 += '<td class="checkboxWidth"><input  type="checkbox" id="mvchk_' + row2 + '" value="'+WCXML.xmlencode(value)+ '" name="MultiValueSelect" ' ;
                            if (selected) htmlcode2 += 'checked';
                            htmlcode2 += '/></td>';
                            htmlcode2 += '<td style="cursor:default;" class="nobr">' + WCXML.xmlencode(value)+'</td>';
                            htmlcode2 += '</tr>';
                        } else {
                            clickMethod = this.functionPrefix + '.selectDBValue(\''+attObj.id+'\', \''+WCXML.xmlencode(WCHTML.encodeForJScript(value))+'\','+showAll+',\''+elem.id+'\'); return false';
                            htmlcode2 += '<tr class="darktext_greenlink_11"><td onclick="'+clickMethod+'" style="cursor:default;" class="nobr">' + WCXML.xmlencode(value)+'</td></tr>';

                        }
                        hashArr[value]=true;

                    }
                } else {
                	htmlcode2 += '<tr class="darktext_greenlink_11"><td colspan="2">'+ ACatStrings.t('ACatNothingValid')+'</td></tr>';  
					// nothing is valid because a previously saved value for at least one of the fields was either using 'other' or disappeared from the list. 
					// we however don't know which one... see UDD for more info					
					
                }


                // the remaining selected one(s) must always be displayed too
                if (multiValue) {	
                    for (var idx=0; idx < curValArr.length; idx++) {
                        if (!hashArr[curValArr[idx]] && (curValArr[idx] != '' || curValArr.length > 1)) {  // show not yet shown values but not the empty value unless the empty value is clearly wanted - seen from the fact that there are multiple things selected	
                            value = WCXML.xmlencode(curValArr[idx]);
                            htmlcode2 += '<tr class="darktext_greenlink_11">';
                            htmlcode2 += '<td class="checkboxWidth"><input  type="checkbox" id="mvchk_' + row2 + '" value="'+WCXML.xmlencode(value)+ '" name="MultiValueSelect" checked /></td>' ;					
                            htmlcode2 += '<td style="cursor:default;" class="nobr">' + value + '</td>';
                            htmlcode2 += '</tr>';
                            row2++;
                            hashArr[curValArr[idx]] = true;
                        }
                    }
                }
				attObj.validValuesHash = hashArr;

                // indicate that something is missing
                if (rows_returned < rows_total) {
                	htmlcode2 += '<tr class="darktext_greenlink_11 orange_medium_bold"><td class="nobr" colspan="2">' + ACatStrings.t('ACatOnly') + rows_returned  + ACatStrings.t('ACatValuesShownOf') + rows_total +'</td></tr>';
                    htmlcode2 += '<tr class="darktext_greenlink_11 orange_medium_bold"><td class="nobr" colspan="2">' + ACatStrings.t('ACatTypePartInInputField') +'</td></tr>';
                    var rows_to_get = rows_total;

                    var labelShowAll = ACatStrings.t('ACatShowAll');
                    if ( rows_total > cMaxResultsDBValues ) {
                        labelShowAll += ' (max. ' + cMaxResultsDBValues + ')';  // no translation needed
                        rows_to_get = cMaxResultsDBValues;
                    }
                    clickMethod = this.functionPrefix + '.getDBValues(this,event,'+row+','+ col + ',true, '+forSearch+','+ rows_to_get + '); return false';
                    htmlcode2 += '<tr class="darktext_greenlink_11 orange_medium_bold"><td class="nobr" colspan="2"><a href="#" id="getDBShowAllBottom"  onclick = "'+clickMethod+'">' +  labelShowAll +'</a></td></tr>';

                }

                var aCatObj = this;  // closure
                // now put the rest, but only when there no or only 1 valid one
                if (rowArr.length <= 1 && urlOtherValues) {
                    // try immediately load the rest?
                	var callParams = {
                    	url: urlOtherValues, 
                    	type: "GET", 
                    	dataType: "text",	
                    	cache: false, // Note: call will be cached by the AjaxCache, but not by the browser, as list can update 
                    	async: false // TODO: CR comment: synchronous ajax is not a good idea 
                    };
                    AjaxCache.ajax(callParams).done(function(jsonContent2) {
                    	var jsonObj2 = aCatObj.readJSONResponse(jsonContent2, callParams);

                        if (jsonObj2.success) {
                            var rowArr2 = jsonObj2.columns[0].values;
                            if (rowArr2 && rowArr2.length >0) {	
                            	htmlcode2 += '<tr class="darktext_greenlink_11"><td colspan="2"><hr></td></tr>';

                            	htmlcode2 += '<tr class="darktext_greenlink_11"><td colspan="2">'+ ACatStrings.t('ACatValuesRequiringChanges') + '</td></tr>';

                                for (var row3=0; row3<rowArr2.length; row3++) {		
                                    var value =  rowArr2[row3];
                                    if (hashArr[value]) continue;  // don't display values which are already displayed in the first set

                                    if (multiValue) {									
                                        // figure out whether this one is selected
                                        selected = false;
                                        for (var idx=0; idx < curValArr.length; idx++) {
                                            if (curValArr[idx] == value && value !='') selected=true;
                                        }

                                        htmlcode2 += '<tr class="darktext_greenlink_11">';
                                        htmlcode2 += '<td class="checkboxWidth"><input type="checkbox" id="mvchk_' + (rowArr.length + row2) + '" value="'+WCXML.xmlencode(value)+ '" name="MultiValueSelect" ' ;  // row2 is correct, not row3!
                                        if (selected) htmlcode2 += 'checked';
                                        htmlcode2 += '/></td>';
                                        htmlcode2 += '<td style="cursor:default;" class="nobr">' + WCXML.xmlencode(value)+'</td>';
                                        htmlcode2 += '</tr>';
                                    } else {											
                                        clickMethod = aCatObj.functionPrefix + '.selectDBValue(\''+attObj.id+'\', \''+WCXML.xmlencode(WCHTML.encodeForJScript(value))+'\',true,\''+elem.id+'\'); return false';
                                        htmlcode2 += '<tr class="darktext_greenlink_11"><td onclick="'+clickMethod+'" style="cursor:default;" class="nobr"><i>' + WCXML.xmlencode(value)+'</i></td></tr>';
                                    }
                                    row2++;
                                }

								attObj.validValuesHash = hashArr;
                            }

                        }
                    });
                }
                                
                if(multiValue){
                	htmlcode2 += '</table>'
                	htmlcode += htmlcode2;
                	htmlcode += '</div></td></tr>';
                } else {
                	htmlcode += htmlcode2;
                }

                htmlcode += '</table>';
                if (isMobileMultiValue) { // add an OK button
                    htmlcode += '</div>';
                    htmlcode += '<div style="width: 95%; text-align: center; padding: 6px;"><input type="button" class="buttonsecondary" value="OK" onclick="hidemenu()"></div>';
                }

                var menuItems = [];  // dropdownmenu expects and array of html code snippets, so we give it an array of 1 element
                menuItems.push(htmlcode);	

                var menuWidth = '350px', menuHeight = '500px',scrollHor = false, nohide=false, bgcolor = "#ffffff",nohideonclick=false;
                if (multiValue) nohideonclick = true;
                if(multiValue && ($("#dropmenudiv").css("display") != "none" && $("#dropmenudiv").find("#dropmenucontainer").length > 0) && htmlcode2){
                	$("#dropmenucontainer").html(htmlcode2);	
                } else {
                	var target = this.getAttributeInDOM(attObj, true) || $(elem).closest(".input-group").find(".form-control, .form-div")[0] || elem;
                	dropmenuobj.style.visibility = 'hidden';
                	dropdownmenu(target,event,menuItems,menuWidth,menuHeight,scrollHor,nohide,bgcolor,nohideonclick) ;	
                	dropmenuobj.style.visibility = 'visible';   // gets hidden in dropdownmenu because it was visible before 
                	$("#dropmenudiv").css({
                    	'min-width': $(target).width()+'px',
                    	'margin-top': '-1px'
                    });
                	updateHoverPosition(target);
                	
                	 // set focus on the input field if it exists
                    var filterElem = document.getElementById('MultiValueFilterInput');
                    if (multiValue && filterElem ) {
                        filterElem.focus();
                        // now set (again) the value
                        filterElem.value = pattern;
                    }

                }
                
                // decide what to do when the content of the dropdownmenu changes (only interesting for checkbox selection in multivalue)

                if (attObj.allowMultipleValues && !forSearch) {
                    dropmenuobj.onchange = function(event) {aCatObj.changeSelectMultiple(row,col); }
                } else if (forSearch && columnObj.searchMultiple) {
                    dropmenuobj.onchange = function(event) {aCatObj.searchSelectMultiple(elem,attObj.id); }
                } else {
                    dropmenuobj.onchange = function(event) {}
                }

            }					


        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn')+ " attributeCategory.displayDBValues" + ACatStrings.t('ACatAtLine') + err.lineNumber + '  : ' + err.message);
            dropmenuobj.style.visibility = 'hidden';
        }
	
    };

    /**
    * show the advanced functions for the cascading dropdown pop-up and hide the Advanced... text
    * @param {Event} event - the click event
    */
    attributeCategory.prototype.showAdvancedDBFunctions = function(event) {
        "use strict";
        try {
            $(".getDBAdvancedFunctions").show(); 
            $(".getDBAdvancedFunctionsHide").hide();

             // avoid the click to close the hover
            if (event.stopPropagation) {
                event.stopPropagation();   // W3C model
            } else {
                event.cancelBubble = true; // IE model
            }


        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn')+ " attributeCategory.showAdvancedDBFunctions" + ACatStrings.t('ACatAtLine') + err.lineNumber + '  : ' + err.message);
        }
    };

    /**
    * Called when any action is done on the multiple select in search dropdowns
    * @param {DOMObject} searchInputField - input field for this search criterion
    * @param {String} attId - attribute id for the attribute linked to this search criterion
    * @description uses global dropmenuobj: the div used for the pop-up
    */
    attributeCategory.prototype.searchSelectMultiple = function(searchInputField,attId) {
        "use strict";
        try {

            if (dropmenuobj.style.visibility == 'hidden') {  // only do something when the menu gets Hidden
                // find what is selected and construct a search string from it
                var checkArr = document.getElementsByName('MultiValueSelect');
                var searchString = '';
                var attValue = '';

                if (checkArr && checkArr.length>0) {				
                    for (var idx=0; idx < checkArr.length; idx++) {
                        if (checkArr[idx].checked) {
                            if (searchString) searchString += ' || ';
                            searchString += '\"' + WCXML.QuoteEncode(checkArr[idx].value, true) + '\"';  // encode " but not '
                            if (attValue) attValue += this.multiSep;
                            attValue += checkArr[idx].value;
                        }
                    }
					
					var selectedValues = _.map(_.filter(checkArr, function(obj){
						return obj.checked;
					}), function(checked){
						return checked.value;
					});
                    
                    var attObj = this.findAttribute(attId);
                    attObj.value = attValue;
                    this.onChangeAttValue(attObj);

                    searchInputField.value = searchString;

                    if (searchInputField.onblur && typeof searchInputField.onblur == 'function') searchInputField.onblur();
                    this._fireSearchSelectMultiple(attObj, selectedValues);
                }
            }
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.searchSelectMultiple:\n' + err);
        }	
    };

    /**
    * resets all the values of a list
    * @param {Boolean} closeDropMenuDiv - if true an attempt will be made to close dropmenudiv and avoid its closure onchange to be executed
    */
    attributeCategory.prototype.clearListValues = function(listName, listType, listCounter, closeDropMenuDiv) {
        try {
            var listObj = this.findList(listName, listType, listCounter);
            if (listObj) this.deselectListAttributeValues(listObj);
            if(dropmenuobj){                
                dropmenuobj.onchange = '';
                dropmenuobj.style.visibility = 'hidden';
            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.clearListValues:\n' + err);
        }	
    };

    /**
    * Called typically from the search panels to allow multiple select in a cascading dropdown. Only supported for back-end lists
    * first set of parameters (elem --> showAll): see definition in getDBvalues
    * @param {String} attId - attribute Id of the attribute for which you allow multiple search values.
    * @param {Boolean} searchMultiple - true if you want to enable mutliple search, false if you want to disable it
    */
    attributeCategory.prototype.allowSearchMultiple = function(event,row,col,showAll, attId, searchMultiple) {
        "use strict";
        try {
            var attObj = this.findAttribute(attId);
            if (attObj && attObj.columnObj) attObj.columnObj.searchMultiple = searchMultiple;
            dropmenuobj.style.visibility = 'hidden';
            var elem = this.getAttributeInDOMById(attId);
            this.getDBValues(elem,event,row,col,showAll,true);
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.allowSearchMultiple:\n' + err);
        }	
    };


    /**
     * Select a value from a database list cascading dropdown
     * @param {String} attid - id of the current attribute (for which a value was chosen)
     * @param {String} value - the chosen value
     * @param {Boolean} showAll - whether the current dropdown is in showAll mode
     * @return {Boolean} whether or not the selection was applied
     */
	attributeCategory.prototype.selectDBValue = function(attid,value,showAll,elemId,ignoreIfNoMatch) {
		var attObj = this.findAttribute(attid);
		if (attObj) {
			var skipHandle = false;
			var handle = function(){
				attObj.value = value;
				attObj.columnObj.autoSelected = false;
				this.onChangeAttValue(attObj);
				// look whether autoselect of other items is needed
				this.checkAutoSelectLazyLoading(attObj,showAll);
				this.executeButtonDependencies(attObj);
				this.emitAttributevalueChanged(attObj);
			}.bind(this);

			var foundMatch = false;			
			for (var prop in attObj.validValuesHash) {
				if (!attObj.validValuesHash.hasOwnProperty(prop)) continue;
				if (prop.toLowerCase() == value.toLowerCase() && attObj.validValuesHash[prop] === true) {
					value = prop;   // normalize the value to the right case
					foundMatch = true;
					break;
				}
			}

			if(!foundMatch){
				if(ignoreIfNoMatch || !attObj.listObj.allowOtherValues){
					return;
				} else {
					var elem = $("#"+elemId);
					var oldVal = elem.val();
					elem.val(value);
					skipHandle = true;
					// very unclear when confirm is shown while the input doesn't have the right value
					$(document).ready(_.defer(function(){ // dom ready + deferred needed to have DOM updated before the confirm is shown...
						var changeAllowed = confirm(ACatStrings.t('ACatListConfirmOtherValue'));
						if (!changeAllowed) {
							elem.val(oldVal);
						} else {
							handle();
						}
					}));
				}
			}

			if(!skipHandle){
				handle();
			}
		}

		var elem = document.getElementById(elemId);
		if (elem && elem.onblur && typeof elem.onblur == 'function') {
			elem.onblur();
		}
	};
     
	/**
	* Called when changing an INPUT field used for a cascading dropdown
	* Checks case-insensitive if what is in the field is a valid choice and if it is, normalizes it and accepts it, else resets the field to the old value
	* @author HADW - PR 8 Apr 2015
	*/
	attributeCategory.prototype.changeDBInputValue = function(elem,row,col) {
		'use strict';
		var elemId = elem.id;
		var attObj = this.cells[row][col].attObj;
		var prevValue = attObj.value;
		var inputValue = elem.value;
		
		elem = document.getElementById(elemId);
		var valueChanged = false;

		if (!elem) throw("elem "+ ACatStrings.t('ACatNotDefinedProblem'));	       

		logger += '\nchangeDBInputValue on ' + attObj.name + '  @ ' + new Date();


		// first figure out whether this is a wanted value		
		if (elem.value && attObj.validValuesHash && attObj.validValuesHash[elem.value] === true) {		
			this.selectDBValue(attObj.id,elem.value,true,elem.id);   // HADW PR: showall on true is correct
		} else {
			var value = elem.value;
			// clear value in case the value is invalid
			if(!inputValue) {
				attObj.value = "";
				this.onChangeAttValue(attObj);
				attObj.columnObj.validArr = null;
				attObj.columnObj.currentIndex = -1;
				attObj.columnObj.otherValueChosen = false;
			} else {
				elem.value = prevValue;  // input field contains invalid entry --> reset to the old value
			}

			this.selectDBValue(attObj.id,value,true,elem.id,true);
		}	
	};
	
    /**
    * Called when an attribute's value has changed when the attribute is part of a cascading dropdown of list type 'database'
    * Checks whether the other attributes of the same cascading dropdown only have 1 possible valid value, in that case, selects that value automatically
    * When invalidateAutomatically is set on true, it will erase invalid values
    * @param {Obj} attObj - attribute object of the cell in which a value was just selected
    * @param {Boolean} showAll - indicates whether the current dropdown is in show all mode (allowing to select invalid values, this way forcing more repaint than otherwise)
    */
    attributeCategory.prototype.checkAutoSelectLazyLoading = function(attObj,showAll) {

        try {
            var listObj = attObj.listObj;

            // construct a URL with the known input values and ask for all distinct column values.
            var url = window.webcenterConfig.jspBaseUrl + 'GetRowsFromList.jsp?listType = WCR_DB_List&outputType=JSON&indexed=1&list=' + WCXML.urlencode(listObj.name); 
            if(listObj && !listObj.invalidateAutomatically){
            	// maxrows/maxcolrows put on 2, as soon as you have 2 lines , you know what you need to know (only completion of single value is needed).
            	url += "&maxrows=2&maxcolrows=2";
            }     

            for (var column=0;column <listObj.columns.length; column++) {
                var columnObj = listObj.columns[column];
                var attObj2 = columnObj.attObj;
                var colName = columnObj.columnName;
                if (attObj2) {
                    if ( attObj2.value && !columnObj.autoSelected && !columnObj.otherValueChosen) {			// autoselected values are never used to exclude valid values for other columns (since the user hasn't made this choice explicitly) 		
                        url += '&input=' + WCXML.urlencode(colName) + ':1:' + WCXML.urlencode(attObj2.value);
                    } else if (!columnObj.otherValueChosen) {
                        url += '&output=' + WCXML.urlencode(colName);	// whatever does not have a value explicitly set is ready to be replaced automatically. We want to know how many different values there are for this

                    }
                }
            }
            var aCatObj = this;


            // ajax call to get the valid values of the other columns
            var callParams = {
                url: url,				
                type: "GET",
                dataType: "text",
                cache: false // Note: call will be cached by the AjaxCache, but not by the browser, as list can update 
            };
            AjaxCache.ajax(callParams)
            .done(function(jsonContent) {
                try {	
                    var somethingChanged = false;
                    var jsonObj = aCatObj.readJSONResponse(jsonContent, callParams);

                    if (!jsonObj.success || jsonObj.success == "0") {
                        alert('Cannot check on autocomplete: checkAutoSelectLazyLoading code ' + jsonObj.error.code + ' : '+  jsonObj.error.message);
                    } else {
                        var rowsReturned = parseInt(jsonObj.rows_returned);
                        if (rowsReturned > 0) {
                            for (var column=0;column <listObj.columns.length; column++) {	

                                var columnObj2 = listObj.columns[column];
                                var colName = columnObj2.columnName;
                                var attObj2 = columnObj2.attObj;
                                if (attObj2 && (!attObj2.value || columnObj2.autoSelected || listObj.invalidateAutomatically)) {
                                    // this should appear in the output columns
                                    for (outcol=0; outcol< jsonObj.columns.length; outcol++) {
                                        if (jsonObj.columns[outcol].name == colName) {
                                            if (jsonObj.columns[outcol].values) {
                                            	if(jsonObj.columns[outcol].values.length == 1){
                                            		// then we can autoselect												
                                                    var newValue = WCXML.QuoteDecode(jsonObj.columns[outcol].values[0]);  // TODO: Test this with values having quotes
                                                    if (newValue != attObj2.value) {
                                                        attObj2.value = newValue;
                                                        columnObj2.autoSelected = true;
                                                        aCatObj.onChangeAttValue(attObj2);												
                                                        somethingChanged = true;
                                                        aCatObj.emitAttributevalueChanged(attObj2);
                                                    }
                                            	}else if(listObj.invalidateAutomatically === true) {
                                                	// if value current value is not in possible results, invalidate it
                                            		if(attObj2.value){
                                            			var curvalues = attObj2.value.split(' | ');
                                            			var intersect = _.intersection(curvalues, jsonObj.columns[outcol].values);
                                            			var newValue = intersect.join(' | ');
                                            			if (newValue != attObj2.value) {
                                            				attObj2.value = newValue;
                                            				aCatObj.onChangeAttValue(attObj2);												
                                                            somethingChanged = true;
                                                            aCatObj.emitAttributevalueChanged(attObj2);
                                            			}
                                            		}
                                                }
                                            } 
                                        }
                                    }
                                }
                            }
                        } else {
                            // an invalid selection was made/typed: no row in the list has this set of input values
                            if (listObj.invalidateAutomatically === true) {
                                if (showAll) {
                                    // this was done on purpose --> de-assign the rest and try again auto-selecting
                                    var column2;
                                    for (var column2=0; column2<listObj.columns.length; column2++) {
                                        var columnObj2 = listObj.columns[column2];
                                        var attObj2 = columnObj2.attObj;
                                        if (attObj2 && attObj2.value && attObj2 !== attObj) {  
                                            attObj2.value = ""; 
                                            columnObj2.autoSelected = false;
                                            somethingChanged = true;
                                            aCatObj.emitAttributevalueChanged(attObj2);
                                        }
                                    }
									
									if(somethingChanged){										
                                    	aCatObj.checkAutoSelectLazyLoading(attObj, true);
									}
                                } else if(!listObj.allowOtherValues) {
                                    alert('Invalid selection made. \nThe current selection for list ' + listObj.name + ' \ndoes not correspond to one of the allowed combinations. \nSee Knowledge Base article 160826276 ' );	
                                    // You typically get this message several times when loading the list... Maybe try remembering that you already sent a warning for this list?									
                                }
                            } // and otherwise allow it...  --> new in WebCenter 14.1, making the behavior compatible with the Excel lists
                        }
                    }	
                    if (somethingChanged) aCatObj.displayAttributeCategory();  // TODO: do a less expensive call - if you do so, you probably need again onChangeAttValue above
                    
                } catch (err) {
                    alert("Error processing return of ajax call to GetRowsFromList.jsp: " + err ); 
                }
            })
            .fail(_.bind(function(errObj){
                this.handleAjaxError(errObj); 
            }, this));
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.selectDBValue:\n' + err);
        }	
    };





    attributeCategory.prototype.handleAjaxError = function(errObj) {

        var loginForm = document.getElementById('sessionexpiredform');

        if ( errObj.responseText == 'SESSION_EXP' && loginForm!= null ) {
            // redirect to the login.jsp for login in again. 
            loginForm.submit();  		
        } else {
            alert("Ajax call unsuccessful. Status: " + errObj.status + "\n" + errObj.responseText);
        }
    };

	  
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Experiment day EXTENDED Forms
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /**
    * Creates htmlcode for a property field. Like each renderer, it returns a list of <td>'s adding up to the needed column span (as defined in cellObj.colSpan)
    * @param cellObj cell object containing the attribute
    * @param row col : row and col where the field is
    * @param visibility: any of Editable, Required, Hidden (typically not),  HiddenKeepValues, ReadOnly
    * @param forPrint: if true, will be optimized for printing
    */
    attributeCategory.prototype.renderProperty = function(cellObj,row,col, visibility, forPrint) {
	    var htmlcode = '';		

	    var attObj = cellObj.attObj;
	    if (!attObj) {
	    	throw(ACatStrings.t('ACatRenderAttrProblem') + " attObj");
	    }

        // determine colspan
        var colSpan = 1;
        if (cellObj.colSpan >1){
        	colSpan =  cellObj.colSpan * 2 - 1 ;
        }
	
	    htmlcode += '<td colspan="'+colSpan+'"';
	    var tdId = this.prefix + 'td_' + row + '_' + col;
	    if (this.currentView != 'Header') htmlcode += ' id="' + tdId + '"';  
	    htmlcode += this.renderStyle(cellObj.styleObj,"Value");
	
	    htmlcode += '>';
	
	    var elem =  attObj.getPropertyElement();
		if(elem.length > 0){
			this.whenRendered(function(){
	    		$('#'+tdId +' > .form-entry > .input-group').prepend(elem);
	    	});
		}  	
	
	    htmlcode += '</td>';
        return htmlcode;
    }

    attributeCategory.prototype.addOtherProperties = function(elem) {
        try {
            this.otherProperties = [];
            if (this.isTaskTypePage) {
            	this.otherProperties.push(new Property({name:'Description', getPropElemFn: function() {
            		return $('#descriptionRow .form-entry').removeClass("form-entry").addClass("main-content");
            	},
                isVisibleFn: function(prop){
                	return $('#descriptionRow .form-entry').length > 0;
                }}));
            	this.otherProperties.push(new Property({name:'Status', getPropElemFn: function() {
            		return $('#tep_status').removeClass("form-entry").addClass("main-content");
            	}}));
            	this.otherProperties.push(new Property({name:'AssignedTo', getPropElemFn: function() {
            		return $('#tep_assigned_to').removeClass("form-entry").addClass("main-content");
            	}}));
            	this.otherProperties.push(new Property({name:'PreviousAssignee', getPropElemFn: function() {
            		return $('#tep_previous_assignee').removeClass("form-entry").addClass("main-content");
            	},
                isVisibleFn: function(prop){
                	return $('#tep_previous_assignee').length > 0;
                }}));
            	this.otherProperties.push(new Property({name:'Due' , getPropElemFn: function() {
            		return $('#tep_due > .form-entry').removeClass("form-entry").addClass("main-content");
            	}}));
            	this.otherProperties.push(new Property({name:'Estimated', getPropElemFn: function() {
            		return $('#tep_estimated > .form-entry').removeClass("form-entry").addClass("main-content");
            	}}));
            	this.otherProperties.push(new Property({name:'CheckList', 
            		getPropElemFn: function() {
	            		return $('#checklistDetailsBlockId').addClass('propertyContainer').addClass("main-content");
	            	},
	                isVisibleFn: function(){
	                	return window.taskExPage && window.taskExPage.pageSections && window.taskExPage.pageSections["checklist"] && window.taskExPage.pageSections["checklist"].state === "SHOW";
	                }}));
            	this.otherProperties.push(new Property({name:'Discussions', 
            		getPropElemFn: function() {
            			// need to force display on element because it may be hidden after initialization by the 
            			// slide() method in DiscussionSection
	            		$('#discussionDetailsBlockId').addClass('displayBlock');
	            		return $('#discussionDetailsBlockId').addClass('propertyContainer').addClass("main-content");
	            	},
	                isVisibleFn: function(){
	                	return window.taskExPage && window.taskExPage.pageSections && window.taskExPage.pageSections["discussion"] && window.taskExPage.pageSections["discussion"].state === "SHOW";
	                }}));
            	this.otherProperties.push(new Property({name:'Documents', 
            		getPropElemFn: function() {
	            		return $('#documentsDetailsBlockId').addClass('propertyContainer').addClass("main-content");
	            	},
	                isVisibleFn: function(){
	                	return window.taskExPage && window.taskExPage.pageSections && window.taskExPage.pageSections["documents"] && window.taskExPage.pageSections["documents"].state === "SHOW";
	                }, labelId: 'taskDocumentsProp'}));
            	this.otherProperties.push(new Property({name:'ButtonRow', getPropElemFn: function() {
            		return $('#lowerButtonRow .form-entry').removeClass("form-entry").addClass("main-content");
            	}}));
            } else {
            	this.otherProperties.push(new Property({name:'TemplateSelection', 
            		getPropElemFn: function(prop){ 
            			prop._wasVisible = $("#projectSelectionRowId").is(':visible');
            			$("#projectSelectionRowId").before('<div id="projectSelectionRowId_PH" />');
	                	return $('<table />').addClass('propertyContainer').addClass("main-content").append($("#projectSelectionRowId").show());           	
	                }, 
	                restorePropElemFn: function(prop){ 
	                	if(!prop._wasVisible){
	                		$("#projectSelectionRowId", prop.getPropertyElement()).hide();
	                	}
	                	$("#projectSelectionRowId_PH").replaceWith($("#projectSelectionRowId", prop.getPropertyElement()));
	                }}));
            	this.otherProperties.push(new Property({name:'ProjectName', 
            		getPropElemFn: _.bind(function(prop){ 
            			$("#PROJ_CREATION_NAME,#PROJ_EDITFIELD_NAME").addClass("displayNone");
            			$("#projectNameRowId").before('<div id="projectNameRowId_PH" />');
            			$("#projectNameEditFieldRowId").before('<div id="projectNameEditFieldRowId_PH" />');
            			if($("#projectNameEditFieldRowId").css("display") !== "none"){
            				var cell = this.findCell(prop);
            				if(cell && cell.cellObj){
            					// overwrite label, but not while editing
            					if((!cell.cellObj.label || cell.cellObj.label==="ProjectName") && !this.layoutId){
            						cell.cellObj.label = "ProjectNameSuffix";
            					}
            				}
            			}
            			$("#projectAutoNameMsg").before('<div id="projectAutoNameMsg_PH" />');           	
            			return $('<table />').addClass('propertyContainer').addClass("main-content").append($("#projectNameRowId")).append($("#projectNameEditFieldRowId")).append($("#projectAutoNameMsg"));           	
	                }, this), 
	                restorePropElemFn: function(prop){ 
	                	$("#projectNameRowId_PH").replaceWith($("#projectNameRowId", prop.getPropertyElement()));
            			$("#projectNameEditFieldRowId_PH").replaceWith($("#projectNameEditFieldRowId", prop.getPropertyElement()));
            			$("#projectAutoNameMsg_PH").replaceWith($("#projectAutoNameMsg", prop.getPropertyElement()));   
            			$("#PROJ_CREATION_NAME,#PROJ_EDITFIELD_NAME").removeClass("displayNone");
	                }, 
	                isRequiredFn: function(){
	                	var fieldDisplayed = $("#projectNameRowId").css("display") !== "none";
	                	return fieldDisplayed && window.gProjectCreationForm && window.gProjectCreationForm.name !== "DONT_SHOW";
	                }
                }));
                this.otherProperties.push(new Property({name:'Requestor', 
                	getPropElemFn: function(){ 
                		$("#selectRequestorId").before('<div id="selectRequestorId_PH" />');
	                	return $('#selectRequestorId');
	                }, 
	                restorePropElemFn: function(prop){ 
	                	$("#selectRequestorId_PH").replaceWith(prop.getPropertyElement());  
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.requestor !== "DONT_SHOW";
	                }
                }));
                this.otherProperties.push(new Property({name:'ProjectManager', 
                	getPropElemFn: function(){ 
                		$("#selectProjectManagerId").before('<div id="selectProjectManagerId_PH" />');
	                	return $('#selectProjectManagerId');
	                }, 
	                restorePropElemFn: function(prop){ 
	                	$("#selectProjectManagerId_PH").replaceWith(prop.getPropertyElement());    
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.manager !== "DONT_SHOW";
	                }
                }));
                this.otherProperties.push(new Property({name:'Status', 
                	getPropElemFn: function(){ 
                		$("#selectProjectStatusId").before('<div id="selectProjectStatusId_PH" />');
	                	return $('#selectProjectStatusId');
	                }, 
	                restorePropElemFn: function(prop){ 
	                	$("#selectProjectStatusId_PH").replaceWith(prop.getPropertyElement());   
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.setStatus !== "DONT_SHOW";
	                }
                }));
                this.otherProperties.push(new Property({name:'Customer', 
                	getPropElemFn: function(){ 
                		$('#selectProjectCustomerId').parent().before('<div id="selectProjectCustomerId_PH" />');
                		return $('#selectProjectCustomerId').parent();
                	},
	                restorePropElemFn: function(prop){ 
	                	$("#selectProjectCustomerId_PH").replaceWith(prop.getPropertyElement());   
	                },
	                isRequiredFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.customer === "REQUIRED";
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.customer !== "DONT_SHOW";
	                }
                }));
                this.otherProperties.push(new Property({name:'Location', 
                	getPropElemFn: function(){ 
                		$('#selectProjectLocationId').before('<div id="selectProjectLocationId_PH" />');
                		return $('#selectProjectLocationId');
	                },
	                restorePropElemFn: function(prop){ 
	                	$("#selectProjectLocationId_PH").replaceWith(prop.getPropertyElement());   
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.customer !== "DONT_SHOW";
	                }
                }));
                this.otherProperties.push(new Property({name:'Thumbnail', 
                	getPropElemFn: function(){ 
                		$("#projectThumbnailRowId > .form-entry").before('<div id="projectThumbnailRowId_PH" />');
	                	return $("#projectThumbnailRowId > .form-entry").removeClass("form-entry");
	                },
	                restorePropElemFn: function(prop){ 
	                	$("#projectThumbnailRowId_PH").replaceWith(prop.getPropertyElement().addClass("form-entry"));   
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.setThumbnails !== "DONT_SHOW";
	                }
	            }));
                this.otherProperties.push(new Property({name:'Description', 
                	getPropElemFn: function(){ 
                		$("#projectDescriptionId").before('<div id="projectDescriptionId_PH" />');
                		return $('#projectDescriptionId');
                	},
	                restorePropElemFn: function(prop){ 
	                	$("#projectDescriptionId_PH").replaceWith(prop.getPropertyElement());   
	                },
	                isRequiredFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.description === "REQUIRED";
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.description !== "DONT_SHOW";
	                }             
                }));
                this.otherProperties.push(new Property({name:'DueDate', 
                	getPropElemFn: function(){
                		var elem = $("#projectDueDateRowId > .form-entry");
                		elem.before('<div id="projectDueDateRowId_PH" />');
                		elem.removeClass("form-entry").addClass("main-content");
	                	return elem;	
	                },
	                restorePropElemFn: function(prop){ 
	                	prop.getPropertyElement().addClass("form-entry").removeClass("main-content");
	                	$("#projectDueDateRowId_PH").replaceWith(prop.getPropertyElement());   
	                },
	                isRequiredFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.dueDate === "REQUIRED";
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.dueDate !== "DONT_SHOW";
	                }      
	            }));
                this.otherProperties.push(new Property({name:'Planning', 
                	getPropElemFn: function(){ 
                		$("#projectPlanningId").before('<div id="projectPlanningId_PH" />').addClass("main-content");
	                	return $('#projectPlanningId');
	                },
	                restorePropElemFn: function(prop){ 
	                	$("#projectPlanningId_PH").replaceWith(prop.getPropertyElement().removeClass("main-content"));   
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.planning !== "DONT_SHOW";
	                }
	            }));
                this.otherProperties.push(new Property({name:'LastTaskEnds', 
                	getPropElemFn: function(prop){ 
                		prop.planningLastTask = $("#planningLastTask");
                		$("#planningLastTask").before('<div id="planningLastTask_PH" />');
                		$("#planningLastTask").detach();
                		$("#projectPlanningRowId .form-group:nth-child(2)").after('<div id="projectPlanningRowId_PH" />').addClass("main-content")
	                	return $("#projectPlanningRowId .form-group:nth-child(2)");  // also used for FirstTaskStarts
	                },
	                restorePropElemFn: function(prop){ 
	                	$("#planningLastTask_PH").replaceWith(prop.planningLastTask); 
	                	$("#projectPlanningRowId_PH").replaceWith(prop.getPropertyElement().removeClass("main-content")); 
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.planning !== "DONT_SHOW";
	                },
	                isRequiredFn: function(){
	                	return $('#projectPlanningId').val() == "backward";
	                }, labelId: 'planningLastTask'
	            }));
                this.otherProperties.push(new Property({name:'Members',
                	getPropElemFn: function(){ 
                		$("#mainMembersRowId .panel-body").before('<div id="mainMembersRowId_PH" />');
	                	return $('#mainMembersRowId .panel-body').addClass("inlineSection").addClass('propertyContainer').addClass('main-content');
	                },
	                restorePropElemFn: function(prop){ 
	                	$("#mainMembersRowId_PH").replaceWith(prop.getPropertyElement().removeClass("inlineSection")); 
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && (window.gProjectCreationForm.inviteMembers !== "DONT_SHOW" || window.gProjectCreationForm.replaceGroups !== "DONT_SHOW");
	                }}));
                this.otherProperties.push(new Property({name:'BillOfMaterials',
                	getPropElemFn: function(){ 
                		$("#mainBomDetailsRowId .panel-body").before('<div id="mainBomDetailsRowId_PH" />');
	                	return $('#mainBomDetailsRowId .panel-body').addClass("inlineSection");
	                },
	                restorePropElemFn: function(prop){ 
	                	$("#mainBomDetailsRowId_PH").replaceWith(prop.getPropertyElement().removeClass("inlineSection")); 
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.bom !== "DONT_SHOW";
	                }}));
                this.otherProperties.push(new Property({name:'Documents',
                	getPropElemFn: function(){ 
                		$("#mainDocumentDetailsRowId .panel-body").before('<div id="mainDocumentDetailsRowId_PH" />');
	                	return $('#mainDocumentDetailsRowId .panel-body').addClass("inlineSection");
	                },
	                restorePropElemFn: function(prop){ 
	                	$("#mainDocumentDetailsRowId_PH").replaceWith(prop.getPropertyElement().removeClass("inlineSection")); 
	                },
	                isRequiredFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.uploadDoc === "REQUIRED";
	                },
	                isVisibleFn: function(){
	                	return window.gProjectCreationForm && window.gProjectCreationForm.uploadDoc !== "DONT_SHOW";
	                }}));
            }

            if (!this.galleryType) this.galleryType = 'Attributes';

        } catch(err) {
            alert("Problem in attributeCategory.addOtherProperties: " + err);
            return '';
        }
    };

    attributeCategory.prototype.detectIfIE = function() {
        return /msie/i.test(navigator.userAgent) && !/opera/i.test(navigator.userAgent);
    }

    attributeCategory.prototype.fixStrictModeColor = function(color) {
        try {
            if(color && color.match(/^[0-9A-Fa-f]{6}$/g)){
                color = "#" + color;
            }
            return color;
        } catch(err) {
            alert("Problem in attributeCategory.fixStrictModeColor: " + err);
            return '';
        }
    };

    /**
     * Make the passed view object the current view.
     * Note: This will not repaint the attribute category.
     * @param viewObj    -       view object
     */
    attributeCategory.prototype.selectView = function(viewObj) {
    	var view = viewObj;
    	
    	// these properties should always be loaded from the original view
		this.currentView = view.name;
        this.currentViewFound = true;

        // load mobile view if available
        if(WCHTML.isMobile()){
        	var mobileView = this.getViewByName(viewObj.name + '_mobile')
            if(mobileView){
            	view = mobileView;
            }
        }

        // these options should be loaded from the mobile view if available
    	this.cells = view.cells;
    	this.layout = view.layout;
        this.nRows = view.nRows;
        this.nCols = view.nCols;
        this.showTypeAsUnit = view.showTypeAsUnit;
        this.alternateBackground = view.alternateBackground;
        this.addToGeneralSection = view.addToGeneralSection;
        this.keepSessionAlive = view.keepSessionAlive;
        this.useAlternativeRendering = view.useAlternativeRendering;
        
    };

    /**
     * Activate a view, based on the passed name.
     * If No view was loaded before (delayed loading with delayedLoading option), it will read the values and lists from the server.
     * After the view is loaded, the attribute Category is repainted
     * @param viewName   -        name of the view
     */
    attributeCategory.prototype.activateViewByName = function(viewName) {
        var viewObj = null;
        this.currentView = viewName;
        this.requestedView = viewName;

		if (!this.views){
			if(viewName === "Default"){
				this.loadView();
				return;
			} else {
				throw new Error("No views are defined");
			}
		}
		
        for(var i=0 ; i<this.views.length ; i++){
            if(this.views[i] && this.views[i].name.toLowerCase() == viewName.toLowerCase()){
                viewObj = this.views[i];
                break;
            }
        }

        if(viewObj){
        	if(viewName === "ProjectCreation"){
        		this.initializeProjectCreationForm();
        	} else if(viewName === "TaskExecution"){
        		this.initializeTEPForm();
        	}
        	
        	// select the requested view
            this.selectView(viewObj);
            // re-initialize all document reference attribute values (documents tables) in order to
            // propagate the attribute category view change to descendant categories
            this.recreateDocTableXForCurrentView();
            
            this.loadView();

        }else{
            this.currentViewFound = false;
        }
    };
    
    /**
     * Load View
     */
    attributeCategory.prototype.loadView = function() {
    	if(this.delayedLoading){
            this.loadValues();
            if(this.delayedListLoading){
                this.delayedListLoading();
            }
        } else {
            this.displayAttributeCategory()
        }
    };

    /**
     * Get a list of the available views in the category 
     * @returns {Array.String} - Array withj names of available views
     */
    attributeCategory.prototype.getViewNames = function() {
        var names = [];
        for(var i=0 ;this.views && i<this.views.length ; i++){
            if(this.views[i] && this.views[i].name){
                names.push(this.views[i].name);
            }
        }

        return names;
    };
    
    /**
     * Get a list of the available views in the category 
     * @returns {Array.String} - Array withj names of available views
     */
    attributeCategory.prototype.getViewIndex = function(viewName) {
        for(var i=0 ;this.views && i<this.views.length ; i++){
            if(this.views[i] && this.views[i].name === viewName){
                return i;
            }
        }

        return -1;
    };
    
    /**
     * Get a view object by name
     * @returns {ViewObject} - view with given name (or null)
     */
    attributeCategory.prototype.getViewByName = function(name) {
    	return _.find(this.views, function(view){
    		return view && view.name === name;
    	});
    };
    
    /**
     * Hide all elements of the task execution page to initialize form builder
     */
    attributeCategory.prototype.initializeTEPForm = function(name) {
    	// hide sections
    	$("#mainWcTaskInfoRowId").hide();
    	$("#descriptionRow").hide();
		$('#mainChecklistDetailsRowId').hide();
		$('#mainDiscussionDetailsRowId').hide();
		$('#mainDocumentsDetailsRowId').hide();
		$("#lowerButtonRow").hide();
		$("#upperButtonRow").hide();

		// put form in new element outside of the panel
		$("#mainAttributesDetailsRowId").before($('<div id="mainAttributesDetailsRowId_content" />'))
		$("#mainAttributesDetailsRowId_content").append($("#taskAttributesDetailsBlockId"));
		$("#mainAttributesDetailsRowId").hide();
    };
    
    /**
     * Hide all elements of the project creation page to initialize form builder
     */
    attributeCategory.prototype.initializeProjectCreationForm = function(name) {
    	// hide sections
    	$("#projectSelectionRowId").hide();
    	$("#mainMembersRowId").hide();
    	$("#mainBomDetailsRowId").hide();
    	$("#mainDocumentDetailsRowId").hide();
    	$("#mainProjectDetailsRowId").hide();

    	// put form in new element outside of the panel
    	$("#mainAttributesRowId").before($('<div id="mainAttributesRowId_content" />'));
    	$("#mainAttributesRowId .panel-body").before($('<div id="mainAttributesRowId_PH" />'));
    	$("#mainAttributesRowId_content").append($("#mainAttributesRowId .panel-body"));
		$("#mainAttributesRowId").hide();
    	
    };
    
    /**
     * Restore attribute section for project creation page
     * Note: this should not be in the destroy method, as it should not be done whenever a repaint is done
     */
    attributeCategory.prototype.restoreAttributesSection = function(name) {
		$('#mainAttributesRowId_PH').replaceWith($('#mainAttributesRowId_content').children());
		$('#mainAttributesRowId_content').remove();
    };
    
 

    /**
     * Checks if the active view has a document reference present
     * @returns {Boolean}
     */
    attributeCategory.prototype.hasActiveViewDocReference = function(){
        for (var row=0;row<this.nRows;row++) {
            for (var col=0; col<this.nCols; col++) {
                if (this.cells[row] && this.cells[row][col]) {
                    var cellObj = this.cells[row][col];

                    if (cellObj.attObj) {
                        var attObj = cellObj.attObj;

                        if (attObj.isDocReference()) {
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    };

    /**
     * Load attribute values from server
     */
    attributeCategory.prototype.loadValues = function() {
        this.delayedLoading = false;
        if (this.valuesURL && !this.noCurrentViewDisplay && !(this.currentView == 'Header' && this.noHeader)) {
            this.getAttributeValues();	
        }
        this.displayAttributeCategory();
    };
    
    /**
     * Call this method before moving the DOM element of the attribute category
     * this is to make sure tinyMCE still works correctly afterwards
     */
    attributeCategory.prototype.beforeMove = function() {
    	this.destroy();
    };
    
    /**
     * Destroy the attribute category: destroy rich text editors and remove data from DOM.
     * Optionally it can also remove already initialized attributeCategory object(s) from memory.
     * 
	 * @param {boolean} destroyACatObj - 'true' if the attributeCategory object from all descendant 
	 * 		documents tables should be removed from the memory (all changes user might have already 
	 * 		done will be lost);
	 * 		'false'/<no value> when all attributeCategory objects of descendant doc. tables should be kept.
     */
    attributeCategory.prototype.destroy = function(destroyACatObj) {
        if (destroyACatObj) {
        	// not only visual/HTML aspect (and related objects) of the attribute category, but also
        	// the data model of the attribute category should be destroyed...
        	// remove event listeners on attribute category level too!
        	AttributeEventBus.getInstance().removeOnChangeAttrValueListener(this._attrValueChangeListenerCallback);
        	AttributeEventBus.getInstance().removeOnCategoryLoadedListener(this._categoryLoadedListenerCallback);
        }
        
        // propagate through to doc references
    	this.executeOnAllDocReferences(function(docTableObj) {
    		// 'docTableObj' is always the "current" documents table object!
    		docTableObj.destroy(destroyACatObj);
    	}, this);
    	
    	_.each(this.otherProperties, function(prop){
    		prop.restoreElement();
    	});
    };
    
    /**
     * Call this method after moving the DOM element of the attribute category
     * this is to make sure tinyMCE still works correctly 
     */
    attributeCategory.prototype.afterMove = function(){
        try {
        	 this.reconstruct();
        } catch(err) {
            // do nothing. Typical case is code being used in 10.2 for cascading dropdowns. 
        }
    }
    
    /**
     * Reconstruct the attribute category: rebuild rich text editors
     **/
    attributeCategory.prototype.reconstruct = function(){
        // propagate through to doc references
    	this.executeOnAllDocReferences(function(docTableObj) {
    		// 'docTableObj' is always the "current" documents table object!
    		docTableObj.reconstruct();
    	}, this);
    	
    	this.displayAttributeCategory();
    };
    
    //////////////////////////
    // BELOW ARE FUNCTIONS FOR TinyMCE
    //////////////////////////
    /**
    *
    * OBJECT CLASS FOR TINYMCE (RICH TEXT ATTRIBUTE FIELD) MANAGER
    *
    */
    function tinyMceMgr() {
    	// for initialization, have a debounced method for each category
    	var debouncedInits = {};
    	this.addRichTextFields = function(id){
    		if(!debouncedInits[id]){
    			debouncedInits[id] = _.debounce(_.bind(this._addRichTextFields,this),150);
    		}
    		debouncedInits[id](id);
    	};
    	
        this.tmceEditors = [];
        this.initialized = false;
        this.rows = [];
        this.cols = [];
    }

    /**
    * Initialize Rich text editor pane
    */
    tinyMceMgr.prototype.initTinyMCEInstances = function(callback){
        if(!this.initialized){
            this.initialized = true;
            try{
            	var thisTinyMCEMgr = this;
                tinyMCE.init({
                	skin: "lightgrayflat",
                    selector : "#tinymceeditarea",
                    plugins: "textcolor code pagebreak fullscreen print searchreplace charmap insertdatetime table hr link image",

                    toolbar1 : "bold italic underline strikethrough | subscript superscript | forecolor backcolor | formatselect fontselect fontsizeselect | code pagebreak print",
                    toolbar2 : "undo redo | bullist numlist | alignleft aligncenter alignright alignjustify | outdent indent blockquote | charmap insertdatetime | table | hr removeformat | link unlink image | searchreplace",
                    toolbar_items_size: 'small',

                    paste_retain_style_properties : "margin, padding, width, height, font-size, font-weight, font-family, color, text-align, ul, ol, li, text-decoration, border, background, float, display",

                    height: 300,
                    resize: false,
                    menubar: false,
                    statusbar: false,
                    
                    setup : function(ed) {
                        ed.on("init", _.bind(function() {                        	
                        	callback();
                        }, thisTinyMCEMgr));
                    }
                });

            }catch(err){
                this.initialized = false;
            }
        } else {
        	callback();
        }
    };

    tinyMceMgr.prototype._addRichTextFields = function(catId){
        if(this.tmceEditors[catId] && this.tmceEditors[catId].length>0){
			var queue = new ExecutionQueue();
			queue.forEach(this.tmceEditors[catId], function (val) {
			 	var elem = $("#" + val);
                var height = elem.outerHeight();
                elem.summernote({
                	airMode: true,
					popover: {
						air: [['style', ['bold', 'italic', 'underline', 'clear']],
						      ['font', ['strikethrough', 'superscript', 'subscript']]]
					},
					callbacks: {
						onBlur: function() {
							elem.change();
						}
					}         
                }).addClass("has-summernote");
                var editor = elem.parent().find(".note-editor");
                editor.height(height);
                editor.find(".note-dropzone, .note-editing-area, .note-editable").height(height-5);
			}, this);
			queue.run(function(){},this);
        }
    };

    if (!window.tinyMceMgrObj){
    	window.tinyMceMgrObj = new tinyMceMgr();
    }
    var tinyMceMgrObj = window.tinyMceMgrObj;
    
    window.editing = "";

    function committmce(){    	
    	// set editor content
    	var content = tinyMCE.get("tinymceeditarea").getContent();
    	$("#"+editing).summernote('code', content);
    	// hide modal
        Popup.hide('modaltmce');
        // trigger onchange
        try {
            var attCatObj = tinyMceMgrObj.attCatObj;
            if(attCatObj){
                attId = editing;
                attObj = attCatObj.findAttribute(attId);
                attObj.value = tinyMCE.get("tinymceeditarea").getContent();
                var elem = document.getElementById(attId);
                elem.value = attObj.value;
                attCatObj.changeText(elem,tinyMceMgrObj.rows[attId],tinyMceMgrObj.cols[attId]);
            }
        } catch (err) {}
        // focus attribute
        $("#"+editing).summernote('focus')
    }

    attributeCategory.prototype.edittmce = function(areaId){
        if ($("#modaltmce").length === 0){
            var modaltmce = $("<div>").attr("id", "modaltmce").css({
                width: "800px",
                height: "430px",
                border: "2px solid black",
                'background-color': "#fff",
                padding: "15px",
                'text-align': "center",
                display: "none"
            })
            
        	modaltmce.append($("<textarea>").attr("id", "tinymceeditarea").attr("name", "tinymceeditarea").attr("cols", 30).css("width", "100%"));

			var modalFooter = $("<div />").addClass("clearfix").css({
				'text-align': "right",
				'padding-top': "15px",
			});
			var cancelButton = new Button(null, null, false, "Cancel", ACatStrings.t("ACatCancel"), true, function () {
				tinyMCE.get('tinymceeditarea').setContent("");
		        Popup.hide('modaltmce');
		        $("#" + editing).focus();
			});
			cancelButton.addClass("btn btn-default btn-md");

			var saveButton = new Button(null, null, false, "OK", ACatStrings.t("ACatSave"), true, function () {
				committmce('tinymceeditarea');
			});
			saveButton.addClass("btn btn-primary btn-md");

			var buttonGroup = $("<div>").addClass("btn-group pull-right");
			buttonGroup.append(cancelButton.element).append(saveButton.element);
			modalFooter.append(buttonGroup);

			modaltmce.append(modalFooter);
            $(document.body).append(modaltmce);
        }
        
        tinyMceMgrObj.initTinyMCEInstances(function(){
        	// set editor content
        	var content = $("#"+areaId).summernote('code');
        	tinyMCE.get("tinymceeditarea").setContent(content);
        	// show editor
            Popup.showModal('modaltmce');
            editing = areaId;
            
            try{
                tinyMCE.get("tinymceeditarea").focus();
            } catch (err) {
            }

        });
    }
    
    attributeCategory.prototype.highlightAttributes = function(){
	    _.each(this.attributes, _.bind(function(attr){
	    	if(attr.highlighted){
	    		var element = $(this.getAttributeInDOM(attr)).add($(this.getAttributeInDOM(attr)).closest(".validation-markup-element"));
	    		element.addClass('validationHighlight');
	    	}
	    	
		}, this));
    };
    
    /*
     * Defer a function until rendering is done
     */
    attributeCategory.prototype.whenRendered = function(deferredFn){
        if(!this.afterRender){
        	this.afterRender = [];
        }
        this.afterRender.push(deferredFn);
    }
    
    /*
     * Defer a function until rendering is done
     */
    attributeCategory.prototype._applyWhenRendered = function(deferredFn){
    	if(this.afterRender){
    		// execute fake deferred
            _.each(this.afterRender, function(deferredFn){
            	deferredFn();
            });
            this.afterRender = [];
    	}
    }
    
    attributeCategory.prototype.registerAttributeElement = function(attObjId, elem){
        if(!this._attributeElementMap[this.prefix]) {
            this._attributeElementMap[this.prefix] = {};
        }
        this._attributeElementMap[this.prefix][attObjId] = elem;
    };
    
    attributeCategory.prototype.onListValuesDeselected = function(callback){
        this._listValuesDeselectedCallbacks.push(callback);
    };
    
    attributeCategory.prototype._fireListValuesDeselected = function(){
        _.each(this._listValuesDeselectedCallbacks, function(callback){
            callback();
        });
    };
	
	attributeCategory.prototype.onSearchSelectMultiple = function(callback){
        this._searchSelectMultipleCallbacks.push(callback);
    };
    
    attributeCategory.prototype._fireSearchSelectMultiple = function(attObj, selectedValues){
        _.each(this._searchSelectMultipleCallbacks, function(callback){
            callback(attObj, selectedValues);
        });
    };
    attributeCategory.prototype._inheritWidth = function(cellObj){
    	return (!(this.useAlternativeRendering && !this.blockAlternativeRendering) && cellObj.styleObj 
				&& ((!cellObj.styleObj.valueStyleObj && cellObj.styleObj.columnWidth && cellObj.styleObj.columnWidthUnit === 'px')
        		|| (cellObj.styleObj.valueStyleObj && cellObj.styleObj.valueStyleObj.columnWidth && cellObj.styleObj.valueStyleObj.columnWidthUnit === 'px')));
    };
    
    attributeCategory.prototype.readJSONResponse = function (jsonContent, callParams) {
	    var jsonObj = null;
    	try {						
	        jsonObj = jsonParse(jsonContent);
	    } catch(e1) {   
	    	// if jsonParse fails, fall back on eval (which is slower)
	    	try{
	    		jsonObj = eval('(' + jsonContent + ')');
	    	} catch(e2) {
	    		// if that also fails, check for an SDK error
	    		try {
	    			var error = $($.parseXML(jsonContent)).find("error");
	    			if(error && error.length > 0){
	    				jsonObj = {
    			    		error: {
    			    			code: error.find("code").text(),
    			    			message: error.find("message").text()
    			    		}
    			    	}
	    			}
	    		} catch(e3){/*do nothing, it's not an XML*/}
	    	}
	    }
	    
	    if(!jsonObj){
	    	jsonObj = {
	    		error: {
	    			code: -1,
	    			message: "Server response could not be read"
	    		}
	    	}
	    }
	    
	    if(jsonObj.error && callParams){
	    	// remove cached entry
	    	delete AjaxCache.ajax.cache[JSON.stringify(callParams)];
	    }
	    
	    return jsonObj;
    }

    return attributeCategory;
});

