define(["wcr/webcenter_html",
        "wcr/webcenter_ajax",
        "wcr/webcenter_xml",
        "wcr/documentSourceHandler",
        "doctablex/DocTableXRenderConfig",
        "attributecategory/AttributeCategory",
        "attributecategory/Attribute", 
        "attributecategory/Property", 
		"component/helpIcon/HelpIcon",
		"utils/webi18n!ACatStrings",
		"utils/webi18n!ACLayoutStrings",
        "utils/Promisify",
        "jquery",  
        "underscore",
        "jqueryUI",
        "lib/PromisePolyfill",
        "wcr/json-minified",
        "wcr/hover",
        "utils/webcss!./AttributeCategoryLayout.css"], 
function(WCHTML,
          ajax,
          WCXML,
          DocumentSourceHandler,
          DocTableXRenderConfig,
          attributeCategory,
          Attribute,
          Property,
		  HelpIcon,
		  ACatStrings,
		  ACLayoutStrings,
		  Promisify,
          $,
          _){

    
    // AttributeCategoryLayout.js
    // Author: Hans Dewitte, Esko 2010 - 2015
    // Contributing authors: Nandakumar Purohit, Bram Van Ongevalle, Jan Hornys
	// Module converted to Require.js by Pieter-Jan Piro, March 2015


    var swapDrag = false;  // Used for drag-and-drop functionality in the cells table

    /**
    * initialization of the layout page
    * @param {String} layoutId Id of the element (typically a div or td) where to put the entire attribute category definition user interface
    */
    attributeCategory.prototype.startLayout = function (layoutId) {
        try {		

            // initializations
            this.currentRowIndex = null;
            this.currentColIndex = null;
            this.operation = null;
            this.attObj = null;
            if (layoutId) {
                this.layoutId = layoutId;
            }
            this.layoutChanged = false;
            //this.setDefaultLayoutLanguageStrings();
            if (this.allAttributesURL) {
                this.loadAllAttributes(this.allAttributesURL);
                this.setDefaultUnits();
            }
            if (this.allListsURL) this.readAllLists(this.allListsURL);
            this.titleDialogCreated = false;
            this.spacerDialogCreated = false;
            this.terminologyDialogCreated = false;
            this.rulerDialogCreated = false;
            this.styleDialogCreated = false;
            this.listDialogCreated = false;
            this.docRenderDialogCreated = false;
            this.docSourceDialogCreated = false;
            this.attributeFilter = "";
            this.lastListUsed = null;  // remembers last list selected in cell details - just to improve the ease of use

            // initializations for drag and drop
            this.currentRow = -1;
            this.currentCol = -1;
            this.source_currentRow = -1;
            this.source_currentCol = -1;
            this.insertAttribute = false;
            this.dragCurrentAttribute = null;
            this.startTime = 0;
            this.endTime = 0;	



            window.onbeforeunload = this.confirmExit;  // confirmExit is the code called when somebody tries to get away from the page
            this.confirmNeeded = true;  // by default, attempts to leave the page will throw a confirm box

            this.displayLayoutInit();

            // change view to last saved view if time is < 1 minute ago
            if (this.currentViewAtLastSave && this.currentViewAtLastSave!= 'Default' && this.modificationTimeStamp) {
                var currentTime = new Date().getTime();
                if (currentTime - this.modificationTimeStamp < 60000) { // less than 60 seconds			
                    // figure out whether this view exists (if it was empty, it wasn't saved)
                    var viewIdx = this.findView(this.currentViewAtLastSave);  // returns -1 if not found
                    if (viewIdx > 0) this.changeView(null,this.currentViewAtLastSave);
                }
            }
            this.aCatChanged = false;  // remember whether changes were done
            storedObj = this; 	// for debug purposes only. Quickest way to get in Firebug to the attribute category object. 

            // to avoid that you lose your session after a number of minutes while editing
            // remark that for the Attribute Category Edit page, we always do this, so it does NOT listen to the setting in the attribute category itself
            window.setInterval(function(){
                if(keepSessionAlive){
                    keepSessionAlive(attributeCategory.prototype.alertSessionLost);
                }
            }, 300000);

        } catch (err) {
            alert("Problem in attributeCategory.startLayout: " + err);
        }

    };
    /** 
    * Is called when navigating away from the page
    * You don't always have to ask confirmation.  It's not needed when
    * 1. The attribute category has not changed
    * 2. The user exits the page using one of the action buttons (Save, Start Approval Cycle, etc)
    */
    attributeCategory.prototype.confirmExit = function() {
      if (storedObj.confirmNeeded && storedObj.aCatChanged) {
          return ACLayoutStrings.t('ACLayoutConfirmPageExit');
      } 
    };


    attributeCategory.prototype.setACatChanged = function() {
      try {
        if (this.aCatChanged != true) {
          this.aCatChanged = true; 
          var aCatChangedField = document.getElementById("aCatChangedField");
          if ( aCatChangedField) aCatChangedField.innerHTML = '<div class="alert alert-warning" role="alert">' + (this.isTaskTypePage?ACLayoutStrings.t('ACLayoutLayoutChanged'):ACLayoutStrings.t('ACLayoutCategoryChanged')) + '</div>'; 
        }
      } catch (err) {
        alert("Problem in ComplexApproval.setApprovalRuleChanged: " + err);
      }
    };

    /**
    * Sets units for each of the attribute types
    * Only needed for the layout management page because the units of the attribute are only given by the server in case of real editing of document or project attributes. 
    * A little bit an ugly construct but it works and it is limited to the admin page.
    * Needs to be extended each time a type is added. If forgotten, only the preview will not display the type, so only a very minor problem.
    */
    attributeCategory.prototype.setDefaultUnits = function() {
        try {
            if (!this.attributes) return;
            for (var attribute = 0 ; attribute<this.attributes.length; attribute++) {
                var attObj = this.attributes[attribute];
                if (attObj) {
                    switch (attObj.unitId) {
                        case "00001_0000000001":	
                            attObj.typeLocalized = 'Text';
                            break;
                        case "00001_0000000021":	
                            attObj.typeLocalized = 'Rich Text';
                            break;
                        case "00001_0000000022":	
                            attObj.typeLocalized = 'Document Reference';
                            break;
                        case "00001_0000000005":	
                            attObj.unitLocalized = 'sq in/sq cm';
                            attObj.typeLocalized = 'Area(sq in./sq cm)';
                            break;
                        case "00001_0000000009":	
                            attObj.unitLocalized = 'sq in/sq mm';
                            attObj.typeLocalized = 'Area(sq in/sq mm)';
                            break;
                        case "00001_0000000010":	
                            attObj.unitLocalized = 'sq ft/sq m';
                            attObj.typeLocalized = 'Area(sq ft/sq m)';
                            break;
                        case "00001_0000000020":	
                            attObj.unitLocalized = '';
                            attObj.typeLocalized = 'Date and Time';
                            break;
                        case "00001_0000000004":	
                            attObj.unitLocalized = 'in,mm';
                            attObj.typeLocalized = 'Distance(in,mm)';
                            break;
                        case "00001_0000000008":	
                            attObj.unitLocalized = 'in,m';
                            attObj.typeLocalized = 'Distance(in,m)';
                            break;
                        case "00001_0000000003":	
                            attObj.unitLocalized = '';
                            attObj.typeLocalized = 'Float';
                            break;
                        case "00001_0000000002":	
                            attObj.unitLocalized = '';
                            attObj.typeLocalized = 'Integer';
                            break;
                        case "00001_0000000006":	
                            attObj.unitLocalized = 'cu in,cu cm';
                            attObj.typeLocalized = 'Volume(cu in,cu cm)';
                            break;
                        case "00001_0000000011":	
                            attObj.unitLocalized = 'cu in,cu dm';
                            attObj.typeLocalized = 'Volume(cu in,cu dm)';
                            break;
                        case "00001_0000000012":	
                            attObj.unitLocalized = 'cu ft,cu m';
                            attObj.typeLocalized = 'Volume(cu ft,cu dm)';
                            break;
                        case "00001_0000000007":	
                            attObj.unitLocalized = 'lbs,gr';
                            attObj.typeLocalized = 'Weight(lbs,gr)';
                            break;
                        case "00001_0000000013":	
                            attObj.unitLocalized = 'lbs,kg';
                            attObj.typeLocalized = 'Weight(lbs,kg)';
                            break;
                        default:
                            break;
                    }

                }
            }

        } catch(err) {
            // do nothing, say nothing. Not important enough to bother the user
        }

    };

    /**
    * Load all attributes (those used and those not used by this category) from the server
    * @param {String} readURL URL of the JSP or servlet to be called via Ajax. Typically the SDK call. 
    */
    attributeCategory.prototype.loadAllAttributes = function(readURL) {
        try {
            if (!readURL) return;

            var aCatObj = this; // Remember the current approval object
            ajax({			
                url: readURL,
                type: "GET",
                data: "xml",
                debug: this.debug,
                synchronous: false,            
                onSuccess: function(returnXML) {
                    aCatObj.readAllAttributesXML(returnXML);
                },
                onError: _.bind(this.readError, this),
                appObj: this  // transfer the attribute category object to the onSuccess function
            });
        } catch (err) {
            alert("Problem in loadAllAttributes: " + err);
        }
    };

    /**
    * Interprets XML coming from the loadAllAttributes call (when success)
    * Consequently the attributes gallery will be filled up with the system attributes
    */
    attributeCategory.prototype.readAllAttributesXML = function(xml) {
        try {
            var allAttributesElem = WCXML.GetFirstFirstLevelChild(xml,"attributes");
            if (allAttributesElem) {

                // read units
                var attributeUnitListElem = WCXML.GetFirstFirstLevelChild(allAttributesElem,"attribute_unit_list");
                if (attributeUnitListElem) {
                    var attributeUnitListArr = WCXML.GetFirstLevelChilds(attributeUnitListElem,"attribute_unit");
                    if (attributeUnitListArr) {
                        for (var attUnit = 0; attUnit<attributeUnitListArr.length; attUnit++) {
                            var attUnitObj = new Object();
                            attUnitObj.name =  WCXML.GetElementText(attributeUnitListArr[attUnit],"name");
                            attUnitObj.id = WCXML.GetElementAttribute(attributeUnitListArr[attUnit],"id");
                            attUnitObj.type =  WCXML.GetElementText(attributeUnitListArr[attUnit],"type");
                            var metricConversionElement = WCXML.GetFirstFirstLevelChild(attributeUnitListArr[attUnit],"metric_conversion");
                            if (attUnitObj.type.toLowerCase() == 'units' &&  metricConversionElement) {
                                attUnitObj.unit =  WCXML.GetElementText(metricConversionElement,"from");
                            } else {
                                attUnitObj.unit = "";
                            }
                            if (!this.attributeUnits) this.attributeUnits = new Array();
                            this.attributeUnits.push(attUnitObj);
                        } // end for
                    }
                }

                // read restricted sets
                var attributeRSListElem = WCXML.GetFirstFirstLevelChild(allAttributesElem,"restricted_set_list");
                if (attributeRSListElem) {
                    var attributeRSArr = WCXML.GetFirstLevelChilds(attributeRSListElem,"restricted_set");
                    if (attributeRSArr) {
                        for (var rs = 0; rs<attributeRSArr.length; rs++) {
                            var rsObj = new Object();
                            rsObj.name =  WCXML.GetElementText(attributeRSArr[rs],"name");
                            rsObj.id =  WCXML.GetElementAttribute(attributeRSArr[rs],"id");

                            rsObj.values = new Array();
                            var valArr = attributeRSArr[rs].getElementsByTagName("restricted_set_value");
                            if (valArr && valArr.length>0) {
                                for (var rsVal=0; rsVal< valArr.length; rsVal++) {
                                    var id = WCXML.GetElementAttribute(valArr[rsVal],"id");
                                    var value = WCXML.GetElementText(valArr[rsVal],"value");
                                    rsObj.values.push({
                                        id:id,
                                        value:value
                                    });
                                }
                            }
                            if (!this.restrictedSets) this.restrictedSets=new Array();
                            this.restrictedSets.push(rsObj);

                        } // end for
                    }
                }
                // read attributes themselves
                var attributeListElem = WCXML.GetFirstFirstLevelChild(allAttributesElem,"attribute_list");
                if (attributeListElem) {
                    var attributeArr = WCXML.GetFirstLevelChilds(attributeListElem,"attribute");
                    if (attributeArr) {
                        for (var att = 0; att<attributeArr.length; att++) {

                            var attObj = new Attribute({
                                name : WCXML.GetElementText(attributeArr[att],"name"),
                                id : WCXML.GetElementAttribute(attributeArr[att],"id"),
                                description : WCXML.GetElementAttribute(attributeArr[att],"description"),
                                unitId : WCXML.GetElementText(attributeArr[att],"attribute_unit_id"),
                                placedArr : new Array(),
                                rsId : WCXML.GetElementText(attributeArr[att],"restricted_set_id"),
                                value : ""
                            });

                            this.setAttributeUnit(attObj);

                            if (attObj.rsId && this.restrictedSets) {
                                for (var rs=0; rs< this.restrictedSets.length; rs++) {
                                    if (attObj.rsId == this.restrictedSets[rs].id) {
                                        attObj.rs = this.restrictedSets[rs];
                                        attObj.restrictType = 'RestrictedSet';

                                        var valArr = attObj.rs.values;
                                        if (valArr && valArr.length ==2 && valArr[0].value == 'yes' && valArr[1].value == 'no') {
                                            attObj.isCheckBox = true;
                                        }
                                        break;
                                    }
                                }

                            }

                            // if document reference, get more info  - DEKE
                            if (attObj.isDocReference()) {
                                if (!attObj.type || attObj.type.length == 0) {
                                    alert("Not initialized 'attObj.type' for Document Reference! (!attObj.type || attObj.type.length == 0)");
                                }
                                attObj.typeLocalized = 'Document Reference'; 
                                attObj.restrictType = 'DocumentReference';							
                                attObj.setDocRefMode(WCXML.GetElementText(attributeArr[att],"doc_ref_mode"));
                                attObj.BOMInclude = WCXML.GetElementText(attributeArr[att],"include_in_bom");
                                attObj.BOMEditable = WCXML.GetElementText(attributeArr[att],"include_in_bom_editable");
                                attObj.BOMCount = WCXML.GetElementText(attributeArr[att],"show_count");	
                                attObj.referenceCategoryID = WCXML.GetElementText(attributeArr[att],"ref_category_id");
                                attObj.referenceCategoryName = WCXML.GetElementText(attributeArr[att],"ref_category_name");
                            }

                            this.mergeAttribute(attObj);  // goes looking for the same attribute in the already existing list.
                        } // end for
                    }
                }

            } // end if ( allAttributesElem)
            this.displayLayout();

        } catch (err) {
            alert("Problem in readAllAttributesXML: " + err);
        }

    };



    attributeCategory.prototype.setAttributeUnit = function(attObj) {
        try {
            var unitId = attObj.unitId;
            if (!unitId || !this.attributeUnits) return;

            for (var unit=0; unit<this.attributeUnits.length; unit++) {
                var unitObj = this.attributeUnits[unit];
                if (unitObj.id == unitId) {
                    attObj.unit = unitObj.unit;
                    //attObj.type = unitObj.type;  // UNITS or TEXT or NUMBER or FLOAT
                    attObj.type = unitObj.name;  // UNITS or TEXT or NUMBER or FLOAT
                    break;
                }
            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.setAttributeUnit:\n' + err);
        }
    };

    /*
    * Takes a new attribute and puts it in the attributes array after checking whether
    * this attribute is already there. If so, update it with the information in attObj.
    */
    attributeCategory.prototype.mergeAttribute = function(attObj) {
        try {
            if (!this.attributes) this.attributes = new Array();
            var matchName=-1, matchId=-1;
            var matchAttObj = null;
            var matchIdx = -1;
            for (var att=0; att<this.attributes.length; att++) {
                if (this.attributes[att].name == attObj.name) matchName = att;
                if (this.attributes[att].id == attObj.id) matchId= att;
            }
            if (matchName != -1) {  // there is a name match --> always pick this up
                matchAttObj = this.attributes[matchName];
                matchIdx = matchName;
            } else if (matchId != -1  && !this.imported) {  // on an imported set, you cannot match on ID (if a match is found, it would be wrong anyway).
                matchAttObj = this.attributes[matchId];
                matchIdx = matchId;
            }

            // now attObj is the one coming from loadAllAttributes and matchAttObj is from the attribute category XMLDocument
            // Both have part of the needed information
            // 
            if (matchAttObj) {
                // update the attObj read from Category XML
                matchAttObj.id = attObj.id;
                matchAttObj.type = attObj.type;
                matchAttObj.rs = attObj.rs;
                if ( attObj.restrictType == 'RestrictedSet') {
                    matchAttObj.restrictType = attObj.restrictType;  // the attributes loaded from the system know whether it's a restricted set, the category XML knows whether list/attributerestriction, ...
                    if (matchAttObj.columnObj) { // a list type attribute was changed into a restricted set DT 120805
                        matchAttObj.columnObj.attObj = null;
                        matchAttObj.columnObj = null;
                    }
                    matchAttObj.listObj = null; 
                }
                if ( matchAttObj.restrictType == 'RestrictedSet' )  {
                    matchAttObj.restrictType = attObj.restrictType;  // covers the situation where the XML says it's a restricted set while the system knows it's changed to prompt				
                }
                if (attObj.restrictType == 'DocumentReference') matchAttObj.restrictType = attObj.restrictType;
                matchAttObj.isCheckBox = attObj.isCheckBox;
                matchAttObj.unit = attObj.unit;
                matchAttObj.description = attObj.description;			
                this.attributes.splice(matchIdx,1);	 // take attObj away from its original position (where it was not in sequence)
                this.attributes.push(matchAttObj); // put the attribute at the end to get it in alphabetical sequence

                matchAttObj.docRefMode = attObj.docRefMode;
                matchAttObj.BOMInclude = attObj.BOMInclude;
                matchAttObj.BOMEditable = attObj.BOMEditable;
                matchAttObj.BOMCount = attObj.BOMCount;
                
				if (matchAttObj.referenceCategoryID != attObj.referenceCategoryID) {
					// the attribute has got a new reference category!!!
					matchAttObj.referenceCategoryID = attObj.referenceCategoryID;
					// you need to load it!
					this.getDocumentReferenceProperties(matchAttObj);					
				}
				
                matchAttObj.referenceCategoryName = attObj.referenceCategoryName;  // name can have changed for the same category, just take over the new name
				
                
            } else {
                // no match is found.  Append this attribute at the end
                this.attributes.push(attObj);
            }

        } catch (err) {
            alert("Problem in mergeAttribute: " + err);
        }

    };

    /**
    * Reads lists from the server via Ajax
    * readAllLists must be done AFTER reading the lists from the AttributeCategory xml, but before filling the lists out of Excel
    * Therefore this filling is done from this code if and only if you are in the layout page
    */
    attributeCategory.prototype.readAllLists = function(url) {

        try{
            var attCatObj = this;  // make available to Ajax onSuccess via closure
            var rndString = Math.floor(Math.random() * 1000000).toString();
            url += '&rndparam='+rndString;  // to avoid caching, each time another bogus parameter is added. this guarantees that we always get the latest set of lists.
            ajax({
                url: url,
                type: "GET",
                data: "xml",
                debug: this.debug,
                synchronous: false,				
                onSuccess: function(xml) {
                    try {						
                        var tempLists = new Array();
                        var list;
                        var col;
                        var attObj;

                        var oldListArr = attCatObj.lists;

                        // Error handling. When the file is simply not there and there are no lists configured in the category, we stay silent (this is the default installation).
                        var readError = WCXML.GetElementText(xml,"error");
                        if (readError) {
                            if (readError != "File Not Found") alert("Error reading lists.xml: " + readError);
                            // Clean out all lists
                            var msg='';
                            if (attCatObj.lists) {
                                for ( list = 0; list<oldListArr.length; list++) {
                                    var listObj = oldListArr[list];
                                    for (col = 0; col < listObj.columns.length; col++) {
                                        attObj = listObj.columns[col].attObj;
                                        if (attObj) {
                                            msg += '\n'+attObj.name;
                                            attObj.listObj = null;
                                            attObj.columnObj = null;
                                        }
                                    }
                                }
                            }
                            attCatObj.lists = null;
                            if (msg) alert("lists.xml was not found on \nWebInstance\\custom\\excel. \n\nFollowing attributes will no longer be linked to a list:" + msg );
                            return; // don't read lists if there are none configured.
                        }
                        var allListsElem=WCXML.GetFirstFirstLevelChild(xml,"lists");
                        if (allListsElem) {           

                            var listArr = WCXML.GetFirstLevelChilds(allListsElem,"list");
                            if (listArr) {
                                // Read all lists into a temporary array
                                for ( list = 0; list<listArr.length; list++) {
                                    var listElem = listArr[list];
                                    var listObj = new Object();
                                    listObj.uniqueIdentifyerColumn = -1;
                                    tempLists.push(listObj);

                                    listObj.name =  WCXML.GetElementText(listElem,"name");								
                                    listObj.type =  WCXML.GetElementText(listElem,"type");
                                    if (listObj.type == 'database') {
                                        listObj.loadingType = 'OnTheFly'; 
                                        listObj.invalidateAutomatically = true;
                                    } else { 
                                        listObj.loadingType = 'Immediate';
                                        listObj.sheet =  WCXML.GetElementText(listElem,"sheet");
                                        listObj.file =  WCXML.GetElementText(listElem,"file");
                                        listObj.hierarchical =  WCXML.GetElementAttribute(listElem,"hierarchical");
                                        if (listObj.hierarchical) {
                                        	listObj.hierarchical = parseInt(listObj.hierarchical,10);
                                        }
                                        listObj.invalidateAutomatically =  WCXML.GetElementAttribute(listElem,"invalidateAutomatically");
                                        if (listObj.invalidateAutomatically === "" || listObj.invalidateAutomatically === "1"){
                                        	listObj.invalidateAutomatically = true;
                                        } else {
                                        	listObj.invalidateAutomatically=false;
                                        }
                                    }


                                    var columnArr = WCXML.GetFirstLevelChilds(listElem,"column");
                                    if (columnArr) {
                                        listObj.columns = new Array();
                                        for (col = 0; col < columnArr.length; col++) {
                                            var colObj = new Object();
                                            listObj.columns.push(colObj);
                                            colObj.columnName = WCXML.GetElementText(columnArr[col],"columnName");
                                            colObj.uniqueIdentifyer = WCXML.GetElementAttribute(columnArr[col],"uniqueIdentifyer");
                                            if (listObj.type != 'database') {
                                                colObj.maxValuesInDropdown = parseInt(WCXML.GetElementAttribute(columnArr[col],"maxValuesInDropdown"),10);
                                                if (!colObj.maxValuesInDropdown || colObj.maxValuesInDropdown<0) colObj.maxValuesInDropdown=null;
                                            }
                                            if (colObj.uniqueIdentifyer) listObj.uniqueIdentifyerColumn = col;
                                        }
                                    }
                                }

                                // merge lists from the attribute category with the lists just read from the custom folder.
                                // ALGORITHM
                                // The merge algorithm is very different between database lists and web server lists
                                // In both cases, the new list array will be the one we just read, because this is the only one in alphabetical sequence 
                                // See further commetns with CASE 1 and CASE 2 for the specific algorithms
                                // there is some code duplication in the below code but trying to merge this will lead to hard to read code


                                if (oldListArr) {
                                    // Run over all old lists
                                    for ( list = 0; list< oldListArr.length; list++) {
                                        // go looking for the same list in the new lists
                                        var listObj = oldListArr[list];
                                        if (listObj.alreadyMerged) {  // a copied list will immediately be handled at first occurence of the same db list. No second operation needed									
                                            continue;
                                        }									

                                        for (var list2=0; list2 < tempLists.length; list2++) {	
                                            var tmpListObj = tempLists[list2];
                                            if (listObj.name == tmpListObj.name && listObj.type == tmpListObj.type) {
                                                var errorMsg = "";
                                                // check whether the same file,type,sheet --> only shows a warning. If the same colums exist, then it's ok...											


                                                if (listObj.type == 'Excel') {
                                                    // CASE 1: Web Server lists
                                                    // From the lists in the attribute category, we must pick up the column to attObj links
                                                    // the listObj used will be the one from lists.xml and only the attribute links will be got from the list array created from the category XML
                                                    // We try doing this as well as possible, meaning as long as the name of the list and the name of the column somewhere matches, it will work
                                                    // The new column sequence will be the sequence of lists.xml  
                                                    // when a linked column or even an entire list are no longer configured, the linked attributes are brought down to simple text fields.

                                                    if (listObj.file.toLowerCase() != tempLists[list2].file.toLowerCase()) errorMsg += "\nList " + listObj.name + " has changed file from " + listObj.file + " to " + tempLists[list2].file;											
                                                    if (listObj.sheet.toLowerCase() != tempLists[list2].sheet.toLowerCase()) errorMsg += "\nList " + listObj.name + " has changed sheet from " + listObj.sheet + " to " + tempLists[list2].sheet;

                                                    // Find back all linked columns (can be in different order!). We don't care about not linked columns 
                                                    for (col = 0; col < listObj.columns.length; col++) {
                                                        attObj = listObj.columns[col].attObj;
                                                        if (attObj) {
                                                            // find its corresponding column in the new list and copy the attribute links 
                                                            var col2=0;
                                                            for (col2=0; col2 < tempLists[list2].columns.length; col2++) {
                                                                if (listObj.columns[col].columnName.toLowerCase() == tempLists[list2].columns[col2].columnName.toLowerCase()) {

                                                                    if (attObj) {
                                                                        tempLists[list2].columns[col2].attObj = attObj;
                                                                        attObj.columnObj = tempLists[list2].columns[col2];
                                                                        attObj.listObj = tempLists[list2];
                                                                    }
                                                                    break;
                                                                }													
                                                            }
                                                            if (col2 >= tempLists[list2].columns.length) {
                                                                // this column is used in the old config but not in the new one. Remove attribute links and warn
                                                                attObj.columnObj = null;
                                                                attObj.listObj = null;
                                                                errorMsg += '\n' + ACLayoutStrings.t('ACLayoutColumnLost') + ': ' + attObj.name;
                                                            }
                                                        }

                                                    }

                                                } else {
                                                    // CASE 2: database lists
                                                    // The database does not know the sequence of the columns
                                                    // In addition, there can be several lists in the category XML pointing to the same database list
                                                    // In the below, listObj is the one from the category XML and tmpListObj is the one just read from the list of lists on the server
                                                    // contrary to CASE1, we will now keep listObj since it contains way more information. We'll only use tmpListObj for its position in the final Array
                                                    // and to check whether all is still there


                                                    // put this listObj after the tmpListObj, we can't remove tmpListObj yet because it can be matched multiple times
                                                    tempLists.splice(list2+1,0,listObj);
                                                    tmpListObj.markedForDeletion = true;
                                                    // check whether the columns are still valid
                                                    for (col = 0; col < listObj.columns.length; col++) {
                                                        attObj = listObj.columns[col].attObj;  // can be null

                                                        // find its corresponding column in the new list
                                                            var col2=0;
                                                            for (col2=0; col2 < tempLists[list2].columns.length; col2++) {
                                                                if (listObj.columns[col].columnName.toLowerCase() == tempLists[list2].columns[col2].columnName.toLowerCase()) {																
                                                                tempLists[list2].columns[col2].used = true;
                                                                    break;
                                                                }													
                                                            }

                                                            if (col2 >= tempLists[list2].columns.length) {
                                                            if (attObj) {
                                                                // this column is used in the old config but not in the new one. Remove attribute links and warn
                                                                attObj.columnObj = null;
                                                                attObj.listObj = null;
                                                                errorMsg += '\n' + ACLayoutStrings.t('ACLayoutColumnLost') + ': ' + attObj.name;
                                                                // take out this column as it does not exist anymore in the database
                                                            }
                                                            listObj.columns.splice(col,1);   // whether used or not, this column must be taken out of the list since it's not in the database
                                                            col--;
                                                        }

                                                    }
                                                    // figure out which columns are new in the new list and put these columns at the end - they will have no attributes linked yet
                                                    // this will only happen when the list in the database was changed (delete and create a new one with the same name)
                                                    for (col2=0; col2 < tempLists[list2].columns.length; col2++) {
                                                        if (!tempLists[list2].columns[col2].used) {
                                                            listObj.columns.push(tempLists[list2].columns[col2]);
                                                        }
                                                    }

                                                }


                                                if (errorMsg) alert(errorMsg);

                                                break;
                                            }
                                        }
                                        if (list2 == tempLists.length) {  
                                            alert("List with name " + oldListArr[list].name + " is no longer configured");
                                            // The attribute category contains a list which is no longer in the custom/excel folder or in the database 
                                            // Since you can't populate the dropdown anymore, bring all linked attributes down to normal text fields
                                            // Unlink all its linked attributes
                                            try {
                                                if (listObj.columns) {
                                                    for (col = 0; col < listObj.columns.length; col++) {
                                                        var colObj = listObj.columns[col];
                                                        if (colObj) {
                                                            var attObj = colObj.attObj;
                                                            if (attObj) {
                                                                colObj.attObj = null;
                                                                attObj.columnObj = null;
                                                                attObj.listObj = null;
                                                            }
                                                        }
                                                    }
                                                }
                                            } catch (err) {
                                                alert("Problem cleaning out no longer configured list: " + err);
                                            }

                                        }
                                    }
                                    // remove objects marked for deletion
                                    for (var list2=tempLists.length-1; list2 >=0 ; list2--) {	// back to front because we take elements out
                                        var tmpListObj = tempLists[list2];
                                        if (tmpListObj.markedForDeletion) tempLists.splice(list2,1);
                                    }

                                }
                                attCatObj.lists = tempLists;  // the old lists have no longer interesting information left and are now dumped to garbage collection

                                attCatObj.connectAllButtonsToLists();  // only now we can reconnect the lists
                                // Now read all lists. This is a potential problem if many lists are configured. 
                                // We assume that this is not the case (so that nobody will configure 30 different lists). 
                                // Possible improvement is to only read the list when it's actually used (meaning at least one column is linked)							
                                for ( list = 0; list<tempLists.length; list++) {								
                                    attCatObj.readListAjax(tempLists[list],attCatObj.readListsURL); 								
                                }									

                            } // end if ListArr

                        } else {
                            alert('Cannot read cascading dropdowns configuration file. \nThe file lists.xml on  WebInstance\\custom\\excel is probably corrupt. \nPlease check for mistakes in the XML or strange characters.');
                        } // end if allListsElem
                        //alert("list length at end of readAllLists " + attCatObj.lists.length);
                    } catch (err) {
                        alert('Problem in readAllLists XML interpretation of excel lists at line ' + err.lineNumber + ': ' +  err.message );
                    }
                }, // end onSuccess
                onError: function(status, objHdl, errMsg) {
                	if(status != '0'){
                		alert("ajax call reading all lists returned with error. \nError Message: " + errMsg + "\ncalled URL: " + url);
                	}
            	}
            }); // end ajax	

        } catch (err) {
            alert("Problem in readAllLists: " + err);
        } 

    };

    /**
    * Saves the layout as XML to the back-end
    */
    attributeCategory.prototype.saveLayout = function(saveAs) {
        try {
            // Create xml for the layout and put it in field xmlcontent        
            var xmlcontent = '';
            var cellObj;
            if (this.views || this.layout != 'OneColumn') { 
				xmlcontent += '<attributeCategory>';
				xmlcontent = xmlcontent + '<name>' + WCXML.xmlencode(this.name) + '</name>';
				xmlcontent += '<version>'+this.softwareVersion+'</version>';  // make sure to stay with integer numbers here because you don't want to compare non-integer numbers
				var currentDateTime = new Date();
				xmlcontent += '<modificationDate>'+ currentDateTime.toString() +'</modificationDate>';
				xmlcontent += '<modificationTimeStamp>'+ currentDateTime.getTime() +'</modificationTimeStamp>';
				if (this.currentView) {
					xmlcontent += '<currentViewAtLastSave>'+ WCXML.xmlencode(this.currentView) +'</currentViewAtLastSave>';
				}
              
				xmlcontent += this.convertStylesToXML();
				xmlcontent += this.convertRulesToXML();
				xmlcontent += this.convertTerminologyToXML();
				xmlcontent += this.convertListsToXML();
				xmlcontent += this.convertDocumentSourcesToXML();
				xmlcontent += this.convertDocRenderGalleryToXML();

              if (this.views) {
                  for (var view=0; view < this.views.length; view++) {
                      if (this.views[view]) xmlcontent +=  this.convertViewToXML(view);
                  }
              } else {
                  xmlcontent +=  this.convertViewToXML();
              }

              xmlcontent = xmlcontent + '</attributeCategory>';
              if (!xmlcontent) {
                  alert(ACLayoutStrings.t('ACLayoutCannotSaveLayout'));
                  return false; // something went wrong with conversion to xml.  So save nothing.
              }
            }
            if (document.getElementById("xmlcontent")) {
                document.getElementById("xmlcontent").value = xmlcontent;
                if (this.saveFieldId) {
                    if (document.getElementById(this.saveFieldId)) {
                        document.getElementById(this.saveFieldId).value = xmlcontent;
                    }
                }          
            }

        // fill newCatName field if the value has changed
        if (document.actionform && document.actionform.categoryName) {
            if (document.getElementById("newCatName") && document.actionform.categoryName.value != this.name) {
                document.getElementById("newCatName").value = document.actionform.categoryName.value;
            }
            else if (saveAs) {
                    alert(ACLayoutStrings.t('ACLayoutSupplyNewName'));
                (document.actionform.categoryName).focus();
                return false;
            }
            }

            // Create hidden fields for removed and added attributes
            var removedAttributesElement = document.getElementById("removedAttributesDiv");
            if (removedAttributesElement) {
                var htmlcode = '';
                for (var att=0; att < this.attributes.length; att++) {
                    var attObj = this.attributes[att];
                    if (attObj && attObj.placedOriginally) {
                        htmlcode += '<input type="hidden" name="removedAttributes" value="' + attObj.id + '">';
                    }
                }
                removedAttributesElement.innerHTML = htmlcode;
            }

            var addedAttributesElement = document.getElementById("addedAttributesDiv");
            for (var att=0; att<this.attributes.length; att++) this.attributes[att].alreadyWritten = false;
            if (addedAttributesElement) {
                var htmlcode = '';
                if (this.views) {
                    for (var view = 0; view < this.views.length; view++) {
                        var viewObj = this.views[view];
                        if (!viewObj) continue;
                        for (var row=0; row < viewObj.cells.length; row++) {
                            if (viewObj.cells[row]) {

                                for (var col=0; col < viewObj.cells[row].length; col ++) {
                                    cellObj = viewObj.cells[row][col];
                                    if (cellObj) {
                                        var attObj = cellObj.attObj;
                                        if (cellObj.type == 'Attribute' && attObj && !attObj.alreadyWritten) {
                                            htmlcode += '<input type="hidden" name="addedAttributes" value="' + attObj.id + '">';
                                            attObj.alreadyWritten = true;  // avoid writing the same attribute twice if it's placed multiple times
                                        }
                                    }
                                }
                            }
                        }
                    }
                } else {  // no views in use, just use the current
                    for (var row=0; row < this.cells.length; row++) {
                        if (this.cells[row]) {
                            for (var col=0; col < this.cells[row].length; col ++) {
                                if (this.cells[row][col]) {
                                    cellObj = this.cells[row][col];
                                    var attObj = cellObj.attObj;
                                    if (cellObj.type == 'Attribute' && attObj && !attObj.alreadyWritten) {
                                        htmlcode += '<input type="hidden" name="addedAttributes" value="' + attObj.id + '">';
                                        // only check for required specifications here (no views available)
                                        if(this.isTaskTypePage && this.cells[row][col].ruleObj == this.isRequiredRule){
                                            htmlcode += '<input type="hidden" name="'+attObj.id+'_required" value="required">';
                                        }
                                        attObj.alreadyWritten = true;  // avoid writing the same attribute twice if it's placed multiple times  -- UI doesn't allow this, so this is just for safety
                                    }
                                }
                            }
                        }
                    }

                }
                addedAttributesElement.innerHTML = htmlcode;
            } // end if addedAttributesEleemnt


            if ($("#actionVal").length && (this.submitAllowed!= false)) {
                $("#actionVal").val(this.isTaskTypePage ? "SaveTaskTypeLayout" : saveAs ? "SaveNewCategoryWithLayout" : "SaveCategoryWithLayout");
              this.confirmNeeded = false;
              var submit = $('<input type="submit" class="displayNone" />');
              $(document.actionform).append(submit);
              submit.click();
          }
          this.aCatChanged = false;
          return true;


        } catch (err) {
            alert("Problem in saveLayout: " + err);
        }
        return false;
    };

    /**
    * Converts attributes in this.attributes to an xml string
    * @return returns the xml string
    */
    attributeCategory.prototype.convertViewToXML = function(view) {

        try {
            var xmlcode= '';

            var viewObj = this;
            if (view!=null) viewObj = this.views[view];

            if (!viewObj || viewObj.nRows<1) return ''; // no view or empty view

            xmlcode += '<view>';

            if (view!=null) {
                xmlcode += WCXML.ElementEncode('name',viewObj.name);
            } else {
                xmlcode += WCXML.ElementEncode('name','Default'); // No views used --> only Default
            }
            xmlcode += WCXML.ElementEncode('nRows',viewObj.nRows+"");
            xmlcode += WCXML.ElementEncode('nCols',viewObj.nCols+"");
            xmlcode += WCXML.ElementEncode('layout',viewObj.layout);
            xmlcode += WCXML.ElementEncodeBoolean('alternateBackground',viewObj.alternateBackground);
            xmlcode += WCXML.ElementEncodeBoolean('showTypeAsUnit',viewObj.showTypeAsUnit);
            xmlcode += WCXML.ElementEncodeBoolean('addToGeneralSection',viewObj.addToGeneralSection);
            xmlcode += WCXML.ElementEncodeBoolean('keepSessionAlive',viewObj.keepSessionAlive);
            xmlcode += WCXML.ElementEncodeBoolean('useAlternativeRendering',viewObj.useAlternativeRendering);

            for (var row=0; row < viewObj.nRows;row++) {
                try {
                    if (viewObj.cells[row]) {
                        for(var col=0; col< viewObj.nCols;col++) {				
                            var cellObj = viewObj.cells[row][col];

                            if (cellObj) {
                                xmlcode += '<cell>';
                                xmlcode += WCXML.ElementEncode('row',row+"");
                                xmlcode += WCXML.ElementEncode('col',col+"");
                                for (var prop in cellObj) {
                                	if (!cellObj.hasOwnProperty(prop)) continue;
                                    switch (prop) {
                                        case 'attObj': 
                                        case 'docList':
                                            break;
                                        case 'styleObj':
                                            if (cellObj.styleObj) xmlcode += WCXML.ElementEncode('style',cellObj.styleObj.name);
                                            break;
                                        case 'ruleObj':
                                            if (cellObj.ruleObj) xmlcode += WCXML.ElementEncode('rule',cellObj.ruleObj.name);
                                            break;
                                        case 'dshObj':										
                                            if (cellObj.dshObj) xmlcode += WCXML.ElementEncode('docSourceName',cellObj.dshObj.name);
                                            break;
                                        case 'docSourceName':
                                            break; // disregard this docSourceName (only used when reading to reconnect from gallery), you already saved the one from dshObj which is the one you really use.
                                        case 'docLinkTableOptions':
                                            if (cellObj.docLinkTableOptions && cellObj.renderOption == 'Table') xmlcode += WCXML.ElementEncode('docRenderName',cellObj.docLinkTableOptions.getName());
                                            break;
                                        case 'buttons':
                                            if (cellObj.buttons) xmlcode += this.convertButtonsToXML(cellObj.buttons);
                                            break;
                                        default: // all the other properties							
                                            // make difference between null and empty string for labels; empty string writes tags, null doesn't
                                            if (cellObj[prop]  || (prop == 'label' && cellObj[prop] != null)) xmlcode += WCXML.ElementEncode(prop,cellObj[prop]);
                                    } // end switch (prop)
                                }
                                if (cellObj && cellObj.attObj ) {
                                    var attObj = cellObj.attObj;
                                    if (attObj.restrictType && attObj.restrictType!='None'  && (attObj.restrictType != 'Attribute' || attObj.restrictToAttObj)) xmlcode += WCXML.ElementEncode('restrictType',attObj.restrictType);
                                    if (attObj.restrictType == 'Attribute') {
                                        if (attObj.restrictToAttObj) {
                                            var attObj2 = attObj.restrictToAttObj;
                                            if (attObj2) {											
                                                xmlcode += WCXML.ElementEncode('restrictAttributeId',attObj2.id);	
                                                xmlcode += WCXML.ElementEncode('restrictAttributeName',attObj2.name);																						
                                            }
                                        }

                                    }
                                }


                                xmlcode += '</cell>';
                            } // end if cellObj
                        } // end for col
                    }
                } catch(err) {
                    alert("Problem in view "+ view + "  row "+ row + " : " + err);
                }
            } // end for row

            xmlcode += '</view>';
            return xmlcode;
        } catch(err) {
            alert("Problem in attributeCategory.convertViewToXML: " + err);
            return '';
        }
    };



    /**
    * converts cellObj.buttons to XML
    * @param {Array} buttons  typically cellobj.buttons
    * @return {String} xml code to append to the XML or empty string if it goes wrong.
    */
    attributeCategory.prototype.convertButtonsToXML = function(buttons) {

        "use strict";
        var xmlcode= '', i;
        try {

            if (buttons && buttons.length>0) {
                xmlcode += '<buttons>';
                for (var button=0; button< buttons.length; button++) {
                    var Obj = buttons[button];
                    if (Obj) {
                        xmlcode += '<button>';
                        for (var prop in Obj) {
                        	if (!Obj.hasOwnProperty(prop)) continue;
                            switch (prop) {
                                case 'inputAtts':
                                    if (Obj.inputAtts && Obj.inputAtts.length>0) {
                                        for (var i=0; i< Obj.inputAtts.length; i++) {
                                            var Obj2 = Obj.inputAtts[i];								

                                            if (Obj2 && Obj2.attObj && Obj2.columnObj)  {
                                                xmlcode += '<inputAtt>';									
                                                xmlcode += WCXML.ElementEncode('id',Obj2.attObj.id);
                                                xmlcode += WCXML.ElementEncode('name',Obj2.attObj.name);
                                                xmlcode += WCXML.ElementEncode('columnName',Obj2.columnObj.columnName); 
                                                xmlcode += WCXML.ElementEncodeBoolean('exact',Obj2.exact); 
                                                xmlcode += '</inputAtt>';
                                            }
                                        }
                                    }
                                    break;
                                case 'outputAtts':
                                    if (Obj.outputAtts && Obj.outputAtts.length>0) {
                                        for (var i=0; i< Obj.outputAtts.length; i++) {
                                            var Obj2 = Obj.outputAtts[i];

                                            if (Obj2 && Obj2.attObj && Obj2.columnObj)  {
                                                xmlcode += '<outputAtt>';	
                                                xmlcode += WCXML.ElementEncode('id',Obj2.attObj.id);
                                                xmlcode += WCXML.ElementEncode('name',Obj2.attObj.name);									
                                                xmlcode += WCXML.ElementEncode('columnName',Obj2.columnObj.columnName); 									
                                                xmlcode += '</outputAtt>';
                                            }
                                        }
                                    }
                                    break;
                                case 'listObj':
                                    if (Obj.listObj && this.lists && this.lists.length>0) {
                                        // find the list object in the array of lists
                                        for (var i=0; i< this.lists.length; i++) {
                                            if (this.lists[i] == Obj.listObj) {										
                                                xmlcode += WCXML.ElementEncode('listName',Obj.listObj.name); 
                                                if (Obj.listObj.counter) xmlcode += WCXML.ElementEncode('listCounter',Obj.listObj.counter); 
                                                break;
                                            }
                                        }
                                    }
                                    break;
                                case 'changeAtts':
                                    if (Obj.execOnChange && Obj.changeAtts && Obj.changeAtts.length>0) {
                                        for (var i=0; i< Obj.changeAtts.length; i++) {
                                            var Obj2 = Obj.changeAtts[i];

                                            if (Obj2 && Obj2.attObj)  {
                                                xmlcode += '<changeAtt>';	
                                                xmlcode += WCXML.ElementEncode('id',Obj2.attObj.id);
                                                xmlcode += WCXML.ElementEncode('name',Obj2.attObj.name);									
                                                xmlcode += '</changeAtt>';
                                            }
                                        }
                                    }
                                default:
                                    if (Obj[prop]) xmlcode += WCXML.ElementEncode(prop,Obj[prop]);
                                    break;
                            }
                        }
                        xmlcode += '</button>';
                    }
                }
                xmlcode += '</buttons>';	
            }



        } catch(err) {
            alert("Problem in attributeCategory.convertButtonsToXML: " + err);        
            xmlcode= '';  // better dump the button than writing a corrupt XML string into the database
        }
        return xmlcode;

    };

    /**
    * Saves the styles
    */
    attributeCategory.prototype.convertStylesToXML = function() {
        var xmlcode ='';
        try {
            xmlcode += '<styles>';
            for (var style=0; style<this.styles.length; style++) {
                var styleObj = this.styles[style];
                if (styleObj) {
                    xmlcode += '<style>';
                    xmlcode +=  WCXML.ElementEncodeBoolean('standardStyle',styleObj.standardStyle);
                    xmlcode += WCXML.ElementEncode('name',styleObj.name);
                    if (styleObj.classLabel) xmlcode += WCXML.ElementEncode('classLabel',styleObj.classLabel);
                    if (styleObj.classValue) xmlcode += WCXML.ElementEncode('classValue',styleObj.classValue);
                    if (styleObj.background) xmlcode += WCXML.ElementEncode('background',styleObj.background);
                    if (styleObj.alignment) xmlcode += WCXML.ElementEncode('alignment',styleObj.alignment);
                    if (styleObj.fontColor) xmlcode += WCXML.ElementEncode('fontColor',styleObj.fontColor);
                    if (styleObj.fontStyle) xmlcode += WCXML.ElementEncode('fontStyle',styleObj.fontStyle);
                    if (styleObj.fontWeight) xmlcode += WCXML.ElementEncode('fontWeight',styleObj.fontWeight);
                    if (styleObj.fontFamily) xmlcode += WCXML.ElementEncode('fontFamily',styleObj.fontFamily);
                    if (styleObj.fontSize) xmlcode += WCXML.ElementEncode('fontSize',styleObj.fontSize);
                    if (styleObj.width) xmlcode += WCXML.ElementEncode('width',styleObj.width);
                    if (styleObj.widthunit) xmlcode += WCXML.ElementEncode('widthunit',styleObj.widthunit);
                    if (styleObj.height) xmlcode += WCXML.ElementEncode('height',styleObj.height);
                    if (styleObj.columnWidth) xmlcode += WCXML.ElementEncode('columnWidth',styleObj.columnWidth);
                    if (styleObj.columnWidthUnit) xmlcode += WCXML.ElementEncode('columnWidthUnit',styleObj.columnWidthUnit);

                    if (styleObj.valueStyleObj) {

                        xmlcode += WCXML.ElementEncodeBoolean('valueDifferent',true);
                        var styleObj2 = styleObj.valueStyleObj;					
                        if (styleObj2.background) xmlcode += WCXML.ElementEncode('valueBackground',styleObj2.background);
                        if (styleObj2.alignment) xmlcode += WCXML.ElementEncode('valueAlignment',styleObj2.alignment);
                        if (styleObj2.fontColor) xmlcode += WCXML.ElementEncode('valueFontColor',styleObj2.fontColor);
                        if (styleObj2.fontStyle) xmlcode += WCXML.ElementEncode('valueFontStyle',styleObj2.fontStyle);
                        if (styleObj2.fontWeight) xmlcode += WCXML.ElementEncode('valueFontWeight',styleObj2.fontWeight);
                        if (styleObj2.fontFamily) xmlcode += WCXML.ElementEncode('valueFontFamily',styleObj2.fontFamily);
                        if (styleObj2.fontSize) xmlcode += WCXML.ElementEncode('valueFontSize',styleObj2.fontSize);
                        if (styleObj2.width) xmlcode += WCXML.ElementEncode('valueWidth',styleObj2.width);
                        if (styleObj2.widthunit) xmlcode += WCXML.ElementEncode('valueWidthunit',styleObj2.widthunit);
                        if (styleObj2.height) xmlcode += WCXML.ElementEncode('valueHeight',styleObj2.height);
                        if (styleObj2.columnWidth) xmlcode += WCXML.ElementEncode('valueColumnWidth',styleObj2.columnWidth);	
                        if (styleObj2.columnWidthUnit) xmlcode += WCXML.ElementEncode('valueColumnWidthUnit',styleObj2.columnWidthUnit);						
                    }

                    if (styleObj.valueStyleObj != null) {

                    }
                    xmlcode += '</style>';
                } else {
                    alert(ACLayoutStrings.t('ACLayoutStyle') + " " + style + " " + ACLayoutStrings.t('ACLayoutDoesNotExist'));
                    continue;
                }
            }

            xmlcode += '</styles>';
        } catch (err) {
            alert("Problem in convertStylesToXML: " + err);
            xmlcode = "";	// Better drop the styles than generating invalid XML which will destroy the entire layout
        }
        return xmlcode;
    };

    /**
    * Saves the document source gallery
    * @author HADW - personal review 5-Feb-2015
    */
    attributeCategory.prototype.convertDocumentSourcesToXML = function() {
        'use strict';
        var xmlcode ='';
        try {
            this.cleanGallery('docSourcesGallery');  // remove unused items
            if ( this.docSourcesGallery && this.docSourcesGallery.length>0 ) {
                xmlcode += '<docSourcesGallery>';
                for (var ds=0; ds < this.docSourcesGallery.length; ds++) {
                    var dshObj = this.docSourcesGallery[ds];
                    if (dshObj) {            
                        xmlcode += dshObj.saveToXML();                
                    } else {
                        alert("Document source " + ds + " does not exist." );  // No translation: not supposed to happen
                    }
                }

                xmlcode += '</docSourcesGallery>';
            }
        } catch (err) {
            alert("Problem in convertDocumentSourcesToXML: " + err);
            xmlcode = "";	// Better drop the styles than generating invalid XML which will destroy the entire layout
        }
        return xmlcode;
    };


	/**
	* Saves the document render gallery to XML
	* @author HADW - pers review 20 Mar 15
	*/
	attributeCategory.prototype.convertDocRenderGalleryToXML = function() {
		'use strict';
		var xmlcode ='';
		try {
			this.cleanGallery('docRenderGallery');  // remove unused items
			if ( this.docRenderGallery && this.docRenderGallery.length>0 ) {
				xmlcode += '<docRenderGallery>';
				for (var ds=0; ds < this.docRenderGallery.length; ds++) {
					var docLinkTableOptions = this.docRenderGallery[ds];
					if (docLinkTableOptions) {
						xmlcode += docLinkTableOptions.convertToXML();
					}
				}
				xmlcode += '</docRenderGallery>';
			}
		} catch (err) {
			alert("Problem in convertDocRenderGalleryToXML: " + err);
			xmlcode = "";	
		}
		return xmlcode;
	};

	/**
	* Clean document source gallery by removing all dshObj or docLinkTableOptions which are not used
	* @param {String} type - the gallery you want to clean: one of 'docSourcesGallery' or 'docRenderGallery'
	* @author HADW - pers review 20 Mar 15
	*/
	attributeCategory.prototype.cleanGallery = function(type) {
		'use strict'; 
		try {
			var row, col,view, ds;
			var gallery, prop; 
			
			switch (type) {
			case 'docSourcesGallery':
				gallery = this.docSourcesGallery;	
				prop = 'dshObj';
				break;
			case 'docRenderGallery':
				gallery = this.docRenderGallery;
				prop = 'docLinkTableOptions';
				break;		
			default: 
				throw new Error('cleanGallery called with unknown type ' + type);
			}
		
			if (gallery && gallery.length>0) {			
				for (var ds=0; ds < gallery.length; ds++) {
					gallery[ds].usedByAttribute = false;   
				}
				
				if (this.views) {
					for (view=0; view < this.views.length; view++) {	
						var viewObj = this.views[view];
						if (viewObj) {
							for (row=0; row < viewObj.nRows; row++) {
								if(viewObj.cells[row]) {
									for (col=0; col<viewObj.cells[row].length; col++) {
										var cellObj = this.views[view].cells[row][col];
										if (cellObj && cellObj[prop]) cellObj[prop].usedByAttribute = true;
									}
								}
							}
						}
					}
				} else {  // no views, only a default view or a task type
					for (row=0; row < this.cells.length; row++) {
						if (this.cells[row]) {
							for (col=0; col< this.cells[row].length; col++) {
								var cellObj = this.cells[row][col];
								if (cellObj && cellObj[prop]) cellObj[prop].usedByAttribute = true;
							}
						}
					}
				}
				for (ds=gallery.length-1; ds >=0 ; ds--) {
					if ( gallery[ds].usedByAttribute === false) {

						gallery.splice(ds,1);
					}
				}
			
			}
				
		} catch (err) {
			alert("Problem in cleanGallery: " + err);
		}

	};
    /**
    * Saves the lists to XML 
    * @return {String} xmlcode to be concatenated to the general xml (<lists> element with all its content). 
    */
    attributeCategory.prototype.convertListsToXML = function() {
        var xmlcode ='';
        try {

            if(this.lists){
                xmlcode += '<lists>';

                for (var list=0; list<this.lists.length; list++) {
                    var listObj = this.lists[list];
                    var listUsed = false;	

                    if (listObj) {
                        var xmlcode2 = '';     
                        if (listObj.buttons && listObj.buttons.length>0) listUsed = true;
                        xmlcode2 += '<list>';
                        xmlcode2 += WCXML.ElementEncode('name',listObj.name);
                        xmlcode2 += WCXML.ElementEncode('type',listObj.type);
                        if (listObj.file) xmlcode2 += WCXML.ElementEncode('file',listObj.file);
                        if (listObj.sheet) xmlcode2 += WCXML.ElementEncode('sheet',listObj.sheet);
                        if (listObj.counter) xmlcode2 += WCXML.ElementEncode('counter',listObj.counter);
                        if (listObj.uniqueIdentifyerColumn >=0) xmlcode2 += WCXML.ElementEncode('uniqueIdentifyerColumn',listObj.uniqueIdentifyerColumn);
						if (listObj.hierarchical) xmlcode2 += WCXML.ElementEncode('hierarchical',listObj.hierarchical);
                        xmlcode2 += WCXML.ElementEncode('invalidateAutomatically',listObj.invalidateAutomatically);
						if (listObj.allowOtherValues) xmlcode2 += WCXML.ElementEncode('allowOtherValues',listObj.allowOtherValues);
						if (listObj.autofillExternalValue) xmlcode2 += WCXML.ElementEncode('autofillExternalValue',listObj.autofillExternalValue);

                        for (var col=0; col<listObj.columns.length; col++) {
                            var columnObj = listObj.columns[col];
                            if (columnObj) {
                                xmlcode2 += '<column>';
                                xmlcode2 += WCXML.ElementEncode('columnName',columnObj.columnName);
                                if (columnObj.uniqueIdentifyer) xmlcode2 += WCXML.ElementEncode('uniqueIdentifyer',columnObj.uniqueIdentifyer);
                                if (columnObj.maxValuesInDropdown) xmlcode2 += WCXML.ElementEncode('maxValuesInDropdown',columnObj.maxValuesInDropdown);
                                var attObj = columnObj.attObj;
                                if (attObj && attObj.placed>0) {
                                    xmlcode2 += WCXML.ElementEncode('attributeId',attObj.id);
                                    xmlcode2 += WCXML.ElementEncode('attributeName',attObj.name);
                                    listUsed = true;
                                }					
                                xmlcode2 += '</column>';
                            }
                        }
                        xmlcode2 += '</list>';
                        if (listUsed) xmlcode += xmlcode2;  // only add lists which are actually in use. if not, just forget xmlcode2...
                    } else {
                        alert(ACLayoutStrings.t('ACLayoutList') + " " + list + " " + ACLayoutStrings.t('ACLayoutDoesNotExist'));
                        continue;
                    }
                }

                xmlcode += '</lists>';
            }
        } catch (err) {
            alert("Problem in convertListsToXML: " + err);
            xmlcode = "";  // Better drop the lists than generating invalid XML which will destroy the entire layout
        }
        return xmlcode;
    };




    /**
    * Saves the visibility rules
    */
    attributeCategory.prototype.convertRulesToXML = function() {
        var xmlcode ='';
        try {
            xmlcode += '<rules>';
            for (var rule=0; rule<this.rules.length; rule++) {
                var ruleObj = this.rules[rule];
                if (ruleObj) {
                    xmlcode += this.convertConditionsToXML(ruleObj);                
                } else {
                    alert(ACLayoutStrings.t('ACLayoutRule') + " " + rule + " " +ACLayoutStrings.t('ACLayoutDoesNotExist'));
                    continue;
                }
            }

            xmlcode += '</rules>';
        } catch (err) {
            alert("Problem in convertRulesToXML: " + err);
            xmlcode = "";
        }
        return xmlcode;
    };

    /**
    * Reads terminology from xml
    * The terminology looks as follows
    * <terminology>
    * 	<term>
    *		<id>PROJ_CREATION_PROJECT_DUE_DATE</id>
    *		<name>Due Date</name>
    *		<default>Samples to be delivered by</default>
    *		<fr>Échantillons à délivrer vers</fr>
    *	</term>
    *   ... more terms
    * </terminology>
    */
    attributeCategory.prototype.convertTerminologyToXML = function() {
        var xmlcode ='';
        try {
            xmlcode += '<terminology>';
            for (var term=0; term<this.terms.length; term++) {
                var termObj = this.terms[term];
                if (termObj) {

                    if (termObj.defaultValue && $.trim(termObj.defaultValue) != "") {
                        xmlcode += '<term>';

                        xmlcode += WCXML.ElementEncode('id',termObj.id);
                        xmlcode += WCXML.ElementEncode('name',termObj.name);
                        xmlcode += WCXML.ElementEncode('default',termObj.defaultValue);


                        if (termObj.languages) {
                            for (var language in termObj.languages) {
                            	if (termObj.languages.hasOwnProperty(language)) {
                                xmlcode += WCXML.ElementEncode(termObj.languages[language].code,termObj.languages[language].value);
                            }
                        }
                        }
                        xmlcode += '</term>';
                    }
                } 
            }
            if (this.projectCreationSequence) xmlcode += WCXML.ElementEncode('sequence',this.projectCreationSequence);

            xmlcode += '</terminology>';
        } catch (err) {
            alert("Problem in convertTerminologyToXML: " + err);
            xmlcode = "";
        }
        return xmlcode;
    };


    /**
    * Saves the visibility rules
    */
    attributeCategory.prototype.convertConditionsToXML = function(ruleObj) {
        var xmlcode ='';
        try {
            xmlcode += '<rule>';
            if (ruleObj.condition) xmlcode += WCXML.ElementEncode('condition',ruleObj.condition);
            if (ruleObj.attObj) {
                xmlcode += WCXML.ElementEncode('attributeName',ruleObj.attObj.name);
                xmlcode += WCXML.ElementEncode('attributeId',ruleObj.attObj.id);
            }

            if (ruleObj.compareValue) xmlcode += WCXML.ElementEncode('compareValue',ruleObj.compareValue);
            if (ruleObj.operator) xmlcode += WCXML.ElementEncode('operator',ruleObj.operator);
            xmlcode += WCXML.ElementEncode('name',ruleObj.name);
            xmlcode +=  WCXML.ElementEncodeBoolean('standardRule',ruleObj.standardRule);
            if (ruleObj.visibility) xmlcode += WCXML.ElementEncode('visibility',ruleObj.visibility);
            if (ruleObj.visibilityRule) xmlcode += WCXML.ElementEncode('visibilityRule',ruleObj.visibilityRule.name);
            if (ruleObj.visibilityElse) xmlcode += WCXML.ElementEncode('visibilityElse',ruleObj.visibilityElse);
            if (ruleObj.visibilityElseRule) xmlcode += WCXML.ElementEncode('visibilityElseRule',ruleObj.visibilityElseRule.name);
            if (ruleObj.depth) xmlcode += WCXML.ElementEncode('depth',ruleObj.depth);  // at least 1, but only for the root

            if (ruleObj.ruleObj1) xmlcode += this.convertConditionsToXML(ruleObj.ruleObj1);
            if (ruleObj.ruleObj2) xmlcode += this.convertConditionsToXML(ruleObj.ruleObj2);                

            xmlcode += '</rule>';


        } catch (err) {
            alert("Problem in convertConditionsToXML: " + err);
            xmlcode = "";
        }
        return xmlcode;
    };

    /**
    * Move the selected attribute in the layout one up or down
    * @param up when true (or 1), the movement is up, otherwise down
    */
    attributeCategory.prototype.moveUpDown = function(up) {
        var selectedRow = null;
        try {
            this.setACatChanged();
            selectedRow = parseInt(this.currentRowIndex);
            var selectedCol = parseInt(this.currentColIndex);
            if (selectedRow==null || selectedCol==null) {
                alert(ACLayoutStrings.t('ACLayoutNothingSelected')); return;
            }

            var otherCellObj = null;	//Object of the destination cell to be swapped
            var cellObj = null;			//Object of the source cell to be swapped
            var cellColSpan = 1;

            if(this.cells[selectedRow]) {
                cellObj = this.cells[selectedRow][selectedCol];
            }
            if (!cellObj) return;

            if(cellObj.colSpan) cellColSpan = parseInt(cellObj.colSpan);

            var destinationRow = null;


            if (up && selectedRow>0) {
                destinationRow = selectedRow-1;
                while (destinationRow>0) {
                    if (this.cells[destinationRow] && this.cells[destinationRow][0] && this.cells[destinationRow][0].colSpan && this.cells[destinationRow][0].colSpan == this.nCols) {
                        // fully spanning row
                        var cellObj2 = this.cells[destinationRow][0];
                        if (cellObj2.type == 'Title' || cellObj2.type == 'Ruler' || cellObj2.type == 'HeaderBar' || cellObj2.type == 'Spacer') {
                            destinationRow--;
                        } else {
                            break;
                        }
                    } else {
                        break;
                    }
                }
            } else if (!up && selectedRow<this.cells.length-1) {
                destinationRow = selectedRow+1;
                while (destinationRow<this.nRows) {
                    if (this.cells[destinationRow] && this.cells[destinationRow][0] && this.cells[destinationRow][0].colSpan && this.cells[destinationRow][0].colSpan == this.nCols) {
                        // fully spanning row
                        var cellObj2 = this.cells[destinationRow][0];
                        if (cellObj2.type == 'Title' || cellObj2.type == 'Ruler' || cellObj2.type == 'HeaderBar' || cellObj2.type == 'Spacer') {
                            destinationRow++;
                        } else {
                            break;
                        }
                    } else {
                        break;
                    }
                }
            } else {
                return;
            }

            otherCellObj = this.cells[destinationRow][selectedCol];

            if (!otherCellObj) {
                // figure out wheter colspanned to the left. If it is, don't allow moving
                for (var col2=selectedCol-1; col2>=0; col2--) {
                    if (this.cells[destinationRow][col2]) {
                        var colSpan2=1;
                        if (this.cells[destinationRow][col2].colSpan) colSpan2=this.cells[destinationRow][col2].colSpan;
                        if (col2 + colSpan2 > selectedCol) {
                            alert(ACLayoutStrings.t('ACLayoutCannotMoveCell'));
                            return;
                        }
                        break;
                    }
                }
            }


            // do the swap		
            this.cells[destinationRow][selectedCol] = cellObj;
            this.cells[selectedRow][selectedCol] = otherCellObj; // may be null

            // correct colSpan
            for (var col2=selectedCol+1;col2<selectedCol + cellColSpan  ;col2++) {
                if (this.cells[destinationRow][col2]) {
                    this.cells[destinationRow][selectedCol].colSpan = col2 - selectedCol;
                    break;
                }
            }
            if (otherCellObj) {
                var cellColSpan2 = 1;
                if (otherCellObj.colSpan) cellColSpan2 = parseInt(otherCellObj.colSpan);
                for (var col2=selectedCol+1;col2<selectedCol + cellColSpan2  ;col2++) {
                    if (this.cells[selectedRow][col2]) {
                        this.cells[selectedRow][selectedCol].colSpan = col2 - selectedCol;
                        break;
                    }
                }
            }


            this.currentRowIndex = destinationRow;
            this.displayLayout();


        } catch (err) {
            alert("Problem in attributeCategory.moveUpDown: " + err + "  selectedRow: " + selectedRow);
        }

    };





    attributeCategory.prototype.moveRightLeft = function(right) {
        try {
            this.setACatChanged();
            var selectedRow = parseInt(this.currentRowIndex);
            var selectedCol = parseInt(this.currentColIndex);        
            var otherCellObj = null;
            if (selectedRow==null || selectedCol==null) {
                alert(ACLayoutStrings.t('ACLayoutNothingSelected')); return;
            }
            var cellObj = this.cells[selectedRow][selectedCol];
            if (!cellObj) return;
            // determine colspan of selected cell
            var colSpan = 1;
            if (cellObj.colSpan) colSpan = parseInt(cellObj.colSpan);
            var colSpan2 = 1;

            if (right && selectedCol + colSpan < this.nCols) { // if to the right and there is room 

                otherCellObj = this.cells[selectedRow][selectedCol + colSpan];

                if(otherCellObj && otherCellObj.colSpan) colSpan2 = parseInt(otherCellObj.colSpan);				

                this.cells[selectedRow][selectedCol] = otherCellObj;
                for (var col=selectedCol+1; col< selectedCol+colSpan2; col++) {
                    this.cells[selectedRow][col] = null;
                }
                this.cells[selectedRow][selectedCol+colSpan2] = cellObj;
                for (var col=selectedCol+colSpan2+1; col< selectedCol+colSpan2+colSpan; col++) {
                    this.cells[selectedRow][col] = null;
                }

                this.currentColIndex = selectedCol+colSpan2;  // set the selection where the cell you moved arrived

            } else if (!right && selectedCol > 0) {  // move left
                // find col to what you need to swap. 
                var destinationCol = selectedCol-1;                  

                for (var col = selectedCol-1;col >= 0;col--) {
                    otherCellObj = this.cells[selectedRow][col];
                    if (otherCellObj) {
                        // look whether it spans the immediate left		
                        var colSpan3 = 1;					
                        if (otherCellObj.colSpan) colSpan3 = parseInt(otherCellObj.colSpan);
                        if (col + colSpan3 >= selectedCol) {
                            destinationCol = col; colSpan2= colSpan3;
                        } else {
                            otherCellObj=null;
                        }
                        break;
                    }
                }
                // now put the element on selectedCol on destinationCol with own colSpan and put the one previously on destinationCol to the right from it

                this.cells[selectedRow][destinationCol] = cellObj;		
                for (col=destinationCol+1; col< destinationCol+colSpan; col++) this.cells[selectedRow][col] = null;			
                this.cells[selectedRow][destinationCol+ colSpan] =  otherCellObj;
                for (col=destinationCol+colSpan+1; col< destinationCol+colSpan+colSpan2; col++) this.cells[selectedRow][col] = null;			
                this.currentColIndex = destinationCol;



            }
            this.displayLayout();

        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.moveRightLeft line ' + err.lineNumber + ": " +  err.message );
        }

    };

    attributeCategory.prototype.changeSelectAttribute = function(selectElem) {
        try {

            if (!this.insertAttribute) {  // don't update the attribute index while dragging
                this.selectedAttributeIdx = selectElem.value;  // index in the attributes table
            }
        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn') + " attributeCategory.changeSelectAttribute: " + err.lineNumber + ": " +  err.message );
        }

    };



    /**
    * Removes an attribute from all views except Default
    * @param attObj attribute to be removed
    */
    attributeCategory.prototype.deleteAttributeFromOtherViews = function(attObj) {
        try{
            this.setACatChanged();
            if (!this.views) return;
            for (var view=1; view<this.cViewOptions.length; view++) {	
                var viewObj = this.views[view];
                if (viewObj) {
                    for(var row=0;row<viewObj.nRows;row++) {
                        try {
                            if(viewObj.cells[row]) {
                                for(var col=0; col<viewObj.cells[row].length;col++) {
                                    var cellObj = this.views[view].cells[row][col];
                                    if (cellObj && cellObj.type == "Attribute" && cellObj.attObj == attObj) {
                                        // then remove this one
                                        if(viewObj.layout == 'OneColumn') {
                                            viewObj.cells.splice(row, 1); // Splice the single dimension array for OneColumn.
                                            viewObj.nRows = viewObj.cells.length;
                                        } else {
                                            viewObj.cells[row][col] = null; // Remove the object from cell
                                        }
                                        attObj.placedArr[view]--;
                                        if (attObj.placed) attObj.placed--;	
                                    }
                                }
                            }
                        } catch (err) {
                            // do nothing
                        }

                    } // end for row
                }

            } // end for view
        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+" attributeCategory.deleteAttributeFromOtherViews: " + err + " line " + err.lineNumber);
            return '';
        }
    };



    /*
    * Place current attribute in the layout
    * @param row: the row to put it to
    * @param col: the column to put it to
    * If row or col is null, it is put to the currently selected cell.
    */
    attributeCategory.prototype.placeCurrentAttribute = function(row, col) {
        var selectedCol = null;
        var selectedRow = null;
        try {
            this.setACatChanged();
            if (!this.selectedAttributeIdx) {
                alert(ACLayoutStrings.t('ACLayoutNoAttributeSelected'));
                return;
            }
            var attObj;
            if (this.galleryType == 'Properties') {
                attObj = this.otherProperties[this.selectedAttributeIdx];
            } else {
                attObj = this.attributes[this.selectedAttributeIdx];
            }
            if(attObj.placedArr && attObj.placedArr[this.currentViewIdx] >= 1) {
                // Confirm whether user wants to place an attribute which is already placed

                alert(ACLayoutStrings.t('ACLayoutAlertMultiplePlacement'));
                return;
            }

            selectedRow = row; 
            selectedCol = col; 
            if (!selectedRow && selectedRow!=0) selectedRow = parseInt(this.currentRowIndex);  // if there is no selectedRow, it can be null, undefined,"" ...
            if (!selectedCol && selectedCol!=0) selectedCol = parseInt(this.currentColIndex);

            if ((!selectedRow && selectedRow!=0) || (!selectedCol && selectedCol!=0) ) {
                if (this.layout == 'OneColumn') {
                    selectedRow = this.cells.length-1;
                    selectedCol = 0;
                } else {  
                    alert(ACLayoutStrings.t('ACLayoutNoCellSelected'));return;
                }
            };

            // CONDITION TO CHECK THE CURRENT VIEW AND DEFAULT VIEW ATTRIBUTES		
            // you can place an attribute to the default view or to another view if it's already in the default view.
            var isAttributeInsertAllowed = (this.currentView == "Default");  
            if (attObj.placedArr && attObj.placedArr[0]) isAttributeInsertAllowed=true;
            if (this.galleryType == 'Properties')  isAttributeInsertAllowed=true;

            if(!isAttributeInsertAllowed) {  // should never happen, prevented by the UI --> alert not translated
                alert("This attribute cannot be added to this view as it is not added in the Default View.");  
            } else {
            	var cellObj = this.createAttributeCell(attObj);
                
                // header view is disabled when there are no attributes in it.
                // enable header view when adding attribute (for preview purpose)
                if (this.currentView == 'Header') {
                	this.noHeader = false;
                }

                if(this.layout == 'OneColumn') {
                    if (!this.cells) this.cells = new Array();
                    // Create a row with one column containing the cell
                    var rowObj = new Array();
                    rowObj.push(cellObj);
                    this.cells.splice(parseInt(selectedRow+1), 0, rowObj);
                } else {
                    if (this.currentView != 'Header') {
                        cellObj.styleObj = this.styles[0];
                    } else {
                        cellObj.styleObj = this.styles[2];
                    }
                    // look whether there is something there
                    // no need to check for a spanning attribute since these cannot be selected... (at least let's hope)

                    var conf= true;
                    if (this.cells[selectedRow][selectedCol]) {
                        conf =confirm(ACLayoutStrings.t('ACLayoutConfirmReplace1'));
                    }
                    if (conf) {
                        if (this.cells[selectedRow][selectedCol]) this.removeCurrentAttribute();
                        this.cells[selectedRow][selectedCol] = cellObj;
                    } else {
                        return; 
                    }
                }
                if(!attObj.placedArr[this.currentViewIdx]) {
                    attObj.placedArr[this.currentViewIdx] = 1;
                } else {
                    attObj.placedArr[this.currentViewIdx]++;
                }
                if (!attObj.placed) attObj.placed=1; else attObj.placed++;


                if (attObj.type && attObj.type.toLowerCase()  == 'documentreference' ) {
                    cellObj.restrictType = 'DocumentReference'; 
                    cellObj.renderOption = 'List';
                    // create a new document source and according table
                    if (this.currentView != 'Header') {
                        if (this.currentView == 'Default') {					
                            this.createDocumentSource(this.dshConfigTablePHID, cellObj, null);
                        } else  {
                            // find the one from the default view
                            var cellObjDefault = this.findDefaultCellObj(cellObj);  // find in the default view the cell with this attribute. 	
                            if (cellObjDefault) {
                                cellObj.dshObj = cellObjDefault.dshObj;							
                            }		
                        }
                        this.createDocTableX(cellObj);
                    }
					
					// check if you need to load the document reference property category
					if (attObj.referenceCategoryID && !attObj.referencePropertyACat) {
						this.getDocumentReferenceProperties(attObj);
					}


                };

                this.currentRowIndex = selectedRow;
                this.currentColIndex = selectedCol+1;
                if (this.currentColIndex >= this.nCols) {
                    this.currentRowIndex++;this.currentColIndex = 0;
                }
                this.nRows = this.cells.length;
                if (this.currentViewIdx != null && this.views) { // update also nRows in the view (this is NOT shared!)
                    var viewObj = this.views[this.currentViewIdx];
                    if (viewObj) viewObj.nRows = this.nRows;
                }

                // check if you need to add another row.  The last row must always be empty in grid layout
                if (this.layout == 'Grid' && selectedRow == this.nRows-1) this.addRow();



                this.displayLayout();
                // select current row if in one column layout


                if (this.layout == 'OneColumn') {
                    //nothing was selected so it's put at the end
                    var elem = document.getElementById("attributesListBox");
                    if (elem) {
                        if (selectedRow+1<elem.length) {
                            elem.selectedIndex = parseInt(selectedRow)+1; 
                            this.currentRowIndex = selectedRow+1;
                        } else {
                            elem.selectedIndex = elem.length -1; 
                            this.currentRowIndex = elem.length -1; 
                        }
                    }
                    if(this.isTaskTypePage) 
                        document.getElementById("cellisrequired").innerHTML = this.displayCellDetails();
                }

            }
        } catch (err) {
            alert("Problem in attributeCategory.placeCurrentAttribute: " + err + ": " + selectedCol);
        }

    };
    
    /*
     * Create a cell with given attribute in it
     * @returns {CellObject} cell
     */
     attributeCategory.prototype.createAttributeCell = function(attObj){
    	 var cellObj = new Object();
         cellObj.attObj = attObj;
         cellObj.type = 'Attribute';
         if (this.galleryType == 'Properties' || attObj instanceof Property) cellObj.type = 'Property';
         cellObj.name = attObj.name;
         // note: label should be null (or undefined) as long as it's not changed, so don't give it a value here
         cellObj.id = attObj.id;
         
         return cellObj;
     };

    /*
    * Remove current attribute from the layout
    */
    attributeCategory.prototype.removeCurrentAttribute = function()
    {
        this.setACatChanged();
        this.removeCellAttribute(this.currentRowIndex, this.currentColIndex);
    };

    /*
    * @param row, col: row and col where you want to remove. If row==null, checks the entire column col, if col==null, checks the entire row row, if both are null, returns false
    * @return true if you can remove, false if you cannot remove
    */
    attributeCategory.prototype.getConfirmationAttributeRemove = function(row, col) {
        try {
            var elementFound = false;
            var confMsg = '';  
            var checkOtherViews = false;
            if (this.views && this.currentView === "Default") checkOtherViews = true;		

            if (col == null) { // checks an entire row
                for (var col2 = 0; col2 < this.nCols; col2++) {
                    if (this.cells[row] && this.cells[row][col2]) {
                    	elementFound = true;
                    	if(this.cells[row][col2].attObj){
                    		var attObj = this.cells[row][col2].attObj;
                            if (checkOtherViews) {
                                for (var view =1; view<this.cViewOptions.length; view++) {
                                    if (attObj.placedArr && attObj.placedArr[view]>0) {
                                        confMsg += '\n' + attObj.name + ' ' + ACLayoutStrings.t('ACLayoutInView') + ' ' + this.cViewOptions[view];
                                    }
                                }
                            }
                    	}
                    }
                }
            } else if (row == null) { // checks an entire column
                for (var row2 = 0; row2 < this.nRows; row2++) {
                    if (this.cells[row2] && this.cells[row2][col]) {
                    	elementFound = true;
                    	if(this.cells[row2][col].attObj){
                    		var attObj = this.cells[row2][col].attObj;
                            if (checkOtherViews) {
                                for (var view =1; view<this.cViewOptions.length; view++) {
                                    if (attObj.placedArr && attObj.placedArr[view]>0) {
                                        confMsg += '\n' + attObj.name + ' ' + ACLayoutStrings.t('ACLayoutInView') + ' ' + this.cViewOptions[view];
                                    }
                                }
                            }
                    	}
                    }
                }
            } else if (row!=null && col != null) {
                if (this.cells[row] && this.cells[row][col] && this.cells[row][col].attObj) {
                    var attObj = this.cells[row][col].attObj;				
                    if (checkOtherViews) {
                        for (var view =1; view<this.cViewOptions.length; view++) {
                            if (attObj.placedArr && attObj.placedArr[view]>0) {
                                confMsg += '\n' + attObj.name + ' ' + ACLayoutStrings.t('ACLayoutInView') + ' ' + this.cViewOptions[view];
                            }
                        }
                    }
                }
            }

            var confirmDeletion = true;
            if (row ==  null && elementFound) {  // delete column with at least one element (of any type)			
                if (confMsg) {
                    confMsg += '\n\n' + ACLayoutStrings.t('ACLayoutConfirmDeleteColumn');
                    confirmDeletion = confirm(ACLayoutStrings.t('ACLayoutConfirmAttributesDeletionOtherViews') + confMsg);
                } else {
                    confirmDeletion = confirm(ACLayoutStrings.t('ACLayoutConfirmDeleteColumn'));
                }

            } else if (col==null && elementFound) { // delete row with at least one element (of any type)		
                if (confMsg) {
                    confMsg += '\n\n' + ACLayoutStrings.t('ACLayoutConfirmDeleteRow');
                    confirmDeletion=confirm(ACLayoutStrings.t('ACLayoutConfirmAttributesDeletionOtherViews') + confMsg);
                } else {
                    confirmDeletion = confirm(ACLayoutStrings.t('ACLayoutConfirmDeleteRow'));
                }
            } else {
                if (confMsg) confirmDeletion=confirm(ACLayoutStrings.t('ACLayoutConfirmAttributeDeletionOtherViews') + confMsg);
            }
            return confirmDeletion;


        } catch (err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.getConfirmationAttributeRemove line ' + err.lineNumber + ": " +  err.message );
            return true; // just do it, asking confirmation is a luxury
        }

    };

    /*
    * Remove attribute (or another cell type) from the layout
    *@param row:  row position to remove from
    *@param col: column position to remove from
    *@param noconfirm: do not ask confirmation to also delete the attribute from the other views when deleting from the default view
    * noconfirm is typically set to true by other callers which have already asked confirmation (deleteRown and deleteColumn)
    */
    attributeCategory.prototype.removeCellAttribute = function(row, col,noConfirm,noRepaint) {
        var selectedRow = null;
        var selectedCol = null;
        try {
            this.setACatChanged();
            //NAPU corrected/commented by HADW
            var attObj = null;
            var cellObj = null;
            //var confirmDeletion = true;
            selectedRow = row; //this.currentRowIndex;
            selectedCol = col; //this.currentColIndex;
            this.operation = 'Remove'; // why needed???
            if(!selectedCol) selectedCol = 0;

            if (selectedRow!=null && selectedCol!=null) {
                // get attObj
                //var attObj = null;			
                if (this.cells[selectedRow] && this.cells[selectedRow][selectedCol]) {
                    cellObj =  this.cells[selectedRow][selectedCol]
                    if (cellObj) attObj = cellObj.attObj; // can be null!
                }
                if (cellObj && cellObj.buttons) {
                    while (cellObj.buttons && cellObj.buttons.length > 0) {
                        this.removeButton(0, cellObj, true);
                    }
                }
                if (!attObj && this.cells && this.cells[selectedRow]) {
                    // this is a layout element such as a Ruler, HeaderBar, Title.  Just remove it
                    this.cells[selectedRow][selectedCol] = null; 

                } else if(attObj) {
                    if (noConfirm || this.getConfirmationAttributeRemove(row,col)===true) {
                        if (this.currentView == 'Default') this.deleteAttributeFromOtherViews(attObj);

                        if(this.layout == 'OneColumn') {
                            this.cells.splice(selectedRow, 1); // Splice the single dimension array for OneColumn.						

                        } else if(this.layout == 'Grid') {
                            if(this.cells[selectedRow] && this.cells[selectedRow][selectedCol]) {
                                attObj = this.cells[selectedRow][selectedCol].attObj;
                            }
                            this.cells[selectedRow][selectedCol] = null; // Remove the object from cell					
                        }

                        // decrement the placed counters
                        if (attObj.placedArr[this.currentViewIdx]) attObj.placedArr[this.currentViewIdx]--;
                        if (attObj.placed) attObj.placed--;
                        if (attObj.isInRule) {
                            // To Do: Find the rule and remove it. I think this is wrong from the beginning but after 3 years nobody complained.

                            // isInRule is also set when this attribute is used as a restriction to another.  FS says to then remove the restriction and fall back on plain text. Here we go...
                            for (var att=0; att<this.attributes.length; att++) {
                                var attObj2= this.attributes[att];
                                if (attObj2 && attObj2.restrictToAttObj == attObj) {
                                    attObj2.restrictToAttObj = null;
                                    attObj2.restrictType = 'None';
                                }
                            }
                        }

                        // remove this attribute from button change lists
                        if (attObj.buttonDependency) {  
                            for (var dep=0; dep < attObj.buttonDependency.length; dep++) {
                                var buttonObj = attObj.buttonDependency[dep];
                                if (this.currentView == 'Default' || buttonObj.viewName == this.currentView) {
                                    var changeAtts = buttonObj.changeAtts;
                                    if (changeAtts) {
                                        for (var changeAtt = changeAtts.length-1; changeAtt >=0 ; changeAtt--) {  // backwards because we splice the array
                                            if (changeAtts[changeAtt].attObj == attObj) {
                                                changeAtts.splice(changeAtt,1);
                                            }
                                        }
                                    }
                                    if (changeAtts.length == 0) attObj.buttonDependency[dep].execOnChange = false;
                                }
                            }
                        }
                        
                        // remove link with list. Note that this should only happen when we do this in default view, as 
                        // the list will be linked to the attribute in every view
                        if (attObj.columnObj && attObj.listObj && this.currentView == "Default") {  
                        	attObj.columnObj.attObj = null;
                        }
                    }
                }
            } else {
                alert(ACLayoutStrings.t('ACLayoutNoCellSelected'));
            }
            this.nRows = this.cells.length;
            if (this.currentViewIdx != null && this.views) { // update also nRows in the view (this is NOT shared!)
                var viewObj = this.views[this.currentViewIdx];
                if (viewObj) viewObj.nRows = this.nRows;
            }
            if (!noRepaint) this.displayLayout();

        } catch (err) {
            alert("Problem in attributeCategory.removeCellAttribute: " + err + ": " + selectedCol);
        }

    };

    /**
    * This part can only be painted once
    * When starting a dialog, jquery moves the <div> and gets extremely confused if you
    * repaint these divs afterwards (so you have 2, with the same id).
    */
    attributeCategory.prototype.displayLayoutInit = function() {
        var htmlcode='';
        htmlcode += '<input type="hidden" id="xmlcontent" name="xmlcontent"  value="">';
        //htmlcode += '<input type="hidden" id="actionVal" name="actionVal"  value="">';

        htmlcode += '<input type="hidden" id="newCatName" name="newCatName"  value="">';

        htmlcode += '<div id="addedAttributesDiv"></div>';
        htmlcode += '<div id="removedAttributesDiv"></div>';

        // initialize the add title dialog.  Initially, it contains nothing at all
        htmlcode += '<div class="ui-dialog ui-widget ui-widget-content ui-corner-all ui-draggable ui-resizable" ';
        htmlcode += 'style="display:none;" id="titleDialog" title="' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutAddTitle')) + '"></div>';

        // initialize the add spacer dialog. 
        htmlcode += '<div class="ui-dialog ui-widget ui-widget-content ui-corner-all ui-draggable ui-resizable" ';
        htmlcode += 'style="display:none;" id="spacerDialog" title="' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutAddSpacer')) + '"></div>';

        // initialize the rules dialog.  
        htmlcode += '<div class="ui-dialog ui-widget ui-widget-content ui-corner-all ui-draggable ui-resizable" ';
        htmlcode += 'style="display:none;" id="rulesDialog" title="' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditRule')) + '"></div>';

        // initialize the style dialog.  
        htmlcode += '<div class="ui-dialog ui-widget ui-widget-content ui-corner-all ui-draggable ui-resizable" ';
        htmlcode += 'style="display:none;" id="styleDialog" title="' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditStyle')) + '"></div>';

        // initialize the document sources dialog.  
        htmlcode += '<div class="ui-dialog ui-widget ui-widget-content ui-corner-all ui-draggable ui-resizable" ';
        htmlcode += 'style="display:none;" id="docSourceDialog" title="' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditDocumentSources')) + '"></div>';

		// initialize the document sources dialog.  
		htmlcode += '<div class="ui-dialog ui-widget ui-widget-content ui-corner-all ui-draggable ui-resizable" ';
		htmlcode += 'style="display:none;" id="docRenderDialog" title="' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditRenderConfiguration')) + '"></div>';
        // initialize the add ruler dialog. 
        htmlcode += '<div class="ui-dialog ui-widget ui-widget-content ui-corner-all ui-draggable ui-resizable" ';
        htmlcode += 'style="display:none;" id="rulerDialog" title="' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutAddRuler')) + '"></div>';

        // initialize the edit terminology dialog.  
        htmlcode += '<div class="ui-dialog ui-widget ui-widget-content ui-corner-all ui-draggable ui-resizable" ';
        htmlcode += 'style="display:none;" id="terminologyDialog" title="' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditTerminology')) + '"></div>';

         // initialize the list dialog. 
        htmlcode += '<div class="ui-dialog ui-widget ui-widget-content ui-corner-all ui-draggable ui-resizable" ';
        htmlcode += 'style="display:none;" id="listDialog" title="' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutConfigureList')) + '"></div>';

        var mouseUpMethod = this.functionPrefix + '.body_onmouseup(event)';
        var mouseMoveMethod =  this.functionPrefix + '.dragAttribute(event)';
        htmlcode += '<div id="mainDisplayArea" onmouseup="'+ mouseUpMethod +'" onmousemove="'+ mouseMoveMethod +'" ></div>';  // this is where displayLayout paints into
        if (document.getElementById(this.layoutId)) {
            document.getElementById(this.layoutId).innerHTML = htmlcode;
        }

        this.displayLayout();

    };

    attributeCategory.prototype.toggleScroll = function() {
        var elem = document.getElementById('ACatPanel');
        if (elem && elem.style.overflow=="auto") {
            elem.style.overflow="";
            elem.style.height = null;

        } else {
            elem.style.overflow="auto";
            elem.style.height = "300px";
        }
        this.displayLayout();
    };
    /**
    * Displays the entire layout user interface (left panel with attributes, right panels with layout and bottom panel with preview
    */
    attributeCategory.prototype.displayLayout = function() {

        try {



            //if (this.layout == 'Grid') this.checkLastRow(); // guarantees that the last row is free
            if (document.getElementById('GridDiv')) this.gridScrollPos = document.getElementById('GridDiv').scrollTop;  // remember scrollbar position
            var htmlcode = '';
            var changeMethod;

            // Main table
            htmlcode += '<table>';

            // Show the View selector

            htmlcode += '<tr class="darktext_greenlink_11"><td/>';

           // htmlcode += this.displayViewsHeader() + '</tr>' ;		

            htmlcode += this.displayViewsHeader() ;   // HADW 22March 2012:  moved <tr> into displayViewsHeader to make this function simply write a row

            // Export XML hidden UI

            var clickMethod =  this.functionPrefix + '.hideTextArea(\'exportXML\')';
            htmlcode += '<tr><td/><td colspan="5" style="display:none" id="exportXMLtd"><textarea rows="10" cols="60" id="exportXML"  ></textarea>';
            htmlcode += '<input type="button" value="Hide" onclick="'+clickMethod+'"></td></tr>'; 

            // Import XML hidden UI
            var clickMethod =  this.functionPrefix + '.importXML()';
            htmlcode += '<tr><td/><td colspan="5" style="display:none" id="importXMLtd"><textarea rows="10" cols="60" id="importXML"  ></textarea>';
            htmlcode += '<input type="button" value="Start Import" onclick="'+clickMethod+'"></td></tr>'; 

            htmlcode += '<tr><td><img src="images/clearpixel.gif" height="10"/></td></tr>'; // spacer

            // row with the filter, titles and operations:
            htmlcode += '<tr class="darktext_greenlink_11">';
            // filter icon
            clickMethod = this.functionPrefix + '.setAttributeFilter(document.getElementById(\'attributeFilterField\'), true, true)';

            htmlcode += '<td valign="bottom"></td>';  

            // filter input field
            if ((this.easterEgg && this.layout == 'Grid' && (this.currentView == 'Creation' || this.isTaskTypePage )
            		|| ((!this.isTaskTypePage && this.currentView == 'ProjectCreation') || (this.isTaskTypePage && this.currentView == 'TaskExecution')))) {  // allow changing the gallery from a list of attributes to a list of other properties
                changeMethod =  this.functionPrefix + '.changeGalleryType(this)';
                htmlcode += '<td valign="bottom" class="ACatLayoutGalleryFilterCol"><div class="ACatLayoutGalleryFilter"><select id="galleryTypeSelect" onchange="' + changeMethod+'">';
                htmlcode += '<option value="Attributes" ' +  ( this.galleryType=='Attributes' ? 'selected' : '') + '>' + (this.isTaskTypePage?ACLayoutStrings.t('ACLayoutSpecifications'):ACLayoutStrings.t('ACLayoutAttributes'))+ '</option>';   
                htmlcode += '<option value="Properties" ' + ( this.galleryType=='Properties'? 'selected': '' ) +'>Other Properties</option>';   
                htmlcode += '</select><br>';
            } else {
                htmlcode += '<td valign="bottom" class="ACatLayoutGalleryFilterCol"><div class="ACatLayoutGalleryFilter">'+ (this.isTaskTypePage?ACLayoutStrings.t('ACLayoutSpecifications'):ACLayoutStrings.t('ACLayoutAttributes'))+':<br>';        
            }
            var noEnter = ' onkeydown="if((event.keyCode ? event.keyCode : event.which ? event.which : event.charCode) == 13){event.preventDefault();}"';
            changeMethod =  this.functionPrefix + '.setAttributeFilter(this, false, ((event.keyCode ? event.keyCode : event.which ? event.which : event.charCode) == 13))';
            var blurMethod =  this.functionPrefix + '.setAttributeFilter(this, true)';
          	htmlcode += '<div class="input-group" style="width: 100%">';
            htmlcode += '<input type="text" id="attributeFilterField" class="form-control" onblur="'+blurMethod+'" onkeyup="'+changeMethod+'"' + noEnter + ' value="'+ WCXML.xmlencode(this.attributeFilter) +'">';
            htmlcode += '<span class="input-group-addon"><i class="icon icon-WCR_Search"></i></span>';
            htmlcode += '</div>';
            htmlcode += '</div>';
            htmlcode += '</td>';


            htmlcode += '<td></td>';   // on top of the << >> buttons        
            htmlcode += '<td valign="bottom" class="ACatLayoutToolbarCol">' + this.displayTopOperations() + '</td>';
            htmlcode += '<td></td>';   // on top of the Move buttons  
            htmlcode += '<td></td>';   // on top of the cell details
            htmlcode += '</tr>';

            // row with the tables and buttons
            htmlcode += '<tr><td><img src="images/clearpixel.gif" width="10"/></td>';		
            htmlcode += '<td class="darktext_darklink_11"  valign="top" id="tdAttributeSelector">' + this.displayAttributeSelector() + '</td>';

            htmlcode += '<td class="ACatLayoutOperationCol">' + this.displayLayoutOperations() + '</td>';

            if(this.layout == "Grid") {
            	
            	// A heuristic method to calculate the needed width of the grid.
            	var width = 450;
            	var neededWidth = this.nCols * 200;
            	if (neededWidth > width) {
            		width = neededWidth;
            	}
            	
            	htmlcode += '<td valign="top" style="width: '+width+'px">' + this.displayLayoutGrid() + '</td>';
                htmlcode += '<td class="ACatLayoutMoveCol">' + this.displayUpDownButtons() + '</td>';             
                // column with cell details
                htmlcode += '<td><div id="celldetails">' + this.displayCellDetails() + '</div></td>';
            } else {			
                htmlcode += '<td class="darktext_darklink_11" style="width: 450px">' + this.displayLayoutOneColumn() + '</td>';
                htmlcode += '<td class="ACatLayoutMoveCol">' + this.displayUpDownButtons()+ '</td>';
                if(this.isTaskTypePage){
                    htmlcode += '<td><div id="cellisrequired"></div></td>';
                }else{
                    htmlcode += '<td><img src="images/clearpixel.gif" width="100"/></td>';	// trick to avoid that the onecolumn selector is too wide caused by "even distribution" of columns
                }
            }
            htmlcode += '</tr>';

            // message in red that the category has changed
            htmlcode += '<tr><td/><td colspan="5" id="aCatChangedField">';
            if (this.aCatChanged) htmlcode += '<div class="alert alert-warning" role="alert">' + (this.isTaskTypePage?ACLayoutStrings.t('ACLayoutLayoutChanged'):ACLayoutStrings.t('ACLayoutCategoryChanged')) + '</div>';
            htmlcode += '</td></tr>';
            htmlcode += '</table>';
            if (document.getElementById("mainDisplayArea")) {
                document.getElementById("mainDisplayArea").innerHTML = htmlcode;
                // disable text selection
                var tableElem = document.getElementById("grid");
                if (tableElem) {
                    if (typeof tableElem.onselectstart!="undefined") tableElem.onselectstart=function(){return false;}; // IE
                    else if (typeof tableElem.style.MozUserSelect!="undefined") tableElem.style.MozUserSelect="none"; //Firefox 
                    else tableElem.onmousedown=function(){return false;};//All other route (ie: Opera)
                    tableElem.style.cursor = "default";
                }
            }



           // show preview.  No preview if OneColumn layout and no views used

            // header --> this is probably not used becuase nobody has a 'PreviewHeader' section...
            htmlcode = '';
            htmlcode += '<tr><td><img src="images/clearpixel.gif" height="20"/></td></tr>';
            htmlcode += '<tr class="detailsHeader"><td class="admin_title">';
            htmlcode += '<img src="images/clearpixel.gif" width="10"/>' + ACLayoutStrings.t('ACLayoutPreview') + ':';
            htmlcode += '</td></tr>';
            htmlcode += '<tr class="white_background"><td><img src="images/clearpixel.gif" width="1" height="1" /></td></tr>';
            // skinnyEskoGrayRow
            htmlcode += '<td class="esko_grey_background" height="1"><img height="1" width="1" src="images/clearpixel.gif"/></td>';
            if (document.getElementById('PreviewHeader')) {
                document.getElementById('PreviewHeader').innerHTML = htmlcode;
            }

            this.displayAttributeCategoryPreview();		


            if(document.getElementById("cellisrequired")) 
                document.getElementById("cellisrequired").innerHTML = this.displayCellDetails();


           if (document.getElementById('GridDiv'))  document.getElementById('GridDiv').scrollTop = this.gridScrollPos;  // put scrollbar where it was




        } catch (err) {
            alert("Problem in attributeCategory.displayLayout: " + err);
        }

    };

    /**
    * displays the attribute category preview at the bottom
    */
    attributeCategory.prototype.displayAttributeCategoryPreview = function() {
        try {
            this.loaded = true;	
            this.displayAttributeCategory();		
        } catch (err) {
            alert("Problem in attributeCategory.displayAttributeCategoryPreview: " + err);
        }
    };

    /**
     displays the row above the layout creator. 
    * This has:
    *	2 checkboxes only for task specification layout
    * 	edit terminology and export link
    * 	views management
    * 	checkboxes for alternating background and attribute types
    *
    * In case there is no task management license, displays nothing at all
    *
    */
    attributeCategory.prototype.displayViewsHeader = function() {
        var htmlcode = '';
        var clickMethod;

        try {
            htmlcode += '<tr class="darktext_greenlink_11"><td/>';
            if (this.allowGridLayout) {
            	var colsTaken = 1;
                if (this.easterEgg) {
                    htmlcode += '<td>';
                    clickMethod = this.functionPrefix + '.exportXML(this); return false;';
                    htmlcode += '<a href="#" onclick="'+clickMethod+'">Export to XML</a>';

                    clickMethod = this.functionPrefix + '.showImportXML(this); return false;';
                    htmlcode += '<br><a href="#" onclick="'+clickMethod+'">Import from XML</a>';
                    htmlcode += '</td>';
                    htmlcode += '<td/>';
                    colsTaken = 3;
                } else {
                    if(!this.isTaskTypePage) {
                        colsTaken = 1;
                    } else {
                    	colsTaken = 3;
                    };
                }
                htmlcode += '<td colspan="'+(6-colsTaken)+'"><table>';

				                   
				htmlcode += '<tr><td class="darktext_greenlink_11">';
				
				htmlcode += '<div class="ACatLayoutViewsBar">';
				htmlcode += '<div style="float: left; padding-right: 5px">';
				htmlcode += '<div class="input-group" role="group">';
				htmlcode += '<span class="input-group-addon">' + ACLayoutStrings.t('ACLayoutCurrentView') + ':</span>';
				if (this.views) {  // show view selector dropdown			


					var changeMethod = this.functionPrefix + '.changeView(this)';
					htmlcode += '<select id="currentViewSelect" class="form-control" onChange="'+ changeMethod +'">';

					for (var option=0; option < this.cViewOptions.length; option++) {
						if (this.views[option] == null) continue;
						var viewName = ACLayoutStrings.tFallback('ACLayout'+ this.cViewOptions[option], this.cViewOptions[option]);
						if (!viewName) viewName = this.cViewOptions[option];

						htmlcode += '<option value="' + WCXML.xmlencode(this.cViewOptions[option]) + '" ';
						if (this.cViewOptions[option] == this.currentView) htmlcode += ' selected ';
						htmlcode += '>' + WCXML.xmlencode(viewName) + '</option>';
					}
					htmlcode += '</select>';
				} else {
					htmlcode += '<input type="text" class="form-control" value="Default" disabled="disabled"\>';
				}
				htmlcode += '</div>';
				htmlcode += '</div>';
				
				if (this.views && this.currentViewIdx > 0) {
					htmlcode += '<div style="float: left; padding-right: 5px">';
					htmlcode += '<div class="btn-group" role="group">';
					// delete view link
					var clickMethod = this.functionPrefix + '.deleteView(); return false;';
					htmlcode += '<button class="btn btn-default" onclick="'+ clickMethod+'">'+ ACLayoutStrings.t('ACLayoutDeleteView')+'</button>';
					htmlcode += '</div>';
					htmlcode += '</div>';
				}
				
                if(this.isTaskTypePage){
					
					if (!this.views || !this.views[4]) {
						htmlcode += '<div style="float: left; padding-right: 5px">';
						htmlcode += '<div class="btn-group" role="group">';
                        clickMethod = this.functionPrefix + '.startViews(\'TaskExecution\'); return false;';
                        htmlcode += '<button class="btn btn-default" onclick="'+ clickMethod +'">'+ ACLayoutStrings.t('ACLayoutAddTaskExecution') +'</button>';
                        htmlcode += '</div>';
    					htmlcode += '</div>';
                    }
					
					if (this.showEditTerminology && this.layout == 'Grid') {
						htmlcode += '<div style="float: left; padding-right: 5px">';
						htmlcode += '<div class="btn-group" role="group">';
                        clickMethod = this.functionPrefix + '.editTerminology(); return false;';
                        htmlcode += '<button class="btn btn-default" onclick="'+ clickMethod +'">'+ ACLayoutStrings.t('ACLayoutEditTerminology') +'</button>';
                        htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/concept/co_wc_Edit_Terminology");
                        htmlcode += '</div>';
    					htmlcode += '</div>';
                    }
					
					htmlcode+= '<div style="clear: both"></div></div></td></tr><tr><td class="ACatLayoutViewDetails">';
					
					// Automatically update project attributes from task specifications
                    var clickMethod = this.functionPrefix + '.changeViewOptions(this);';
                    //htmlcode += '<tr class="darktext_greenlink_11"><td style="padding-bottom:5px">';
                    htmlcode += '<input type="checkbox" id="autoUpdateProjAttrsCheckbox" name="autoUpdateProjAttrsCheckbox" value="" onclick="'+clickMethod+'" '; 
                    if (this.autoUpdateProjAttrsCheckbox) htmlcode += ' checked ';
                    if (!this.canEditTaskType) htmlcode += ' disabled ';
                    htmlcode += '>';  
                    htmlcode += '<label for="autoUpdateProjAttrsCheckbox">' + ACLayoutStrings.t('ACLayoutAutoUpdateProjectAttrs') + '</label>';
					htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/task/ta_wc_SpecificationsTab");
					htmlcode += "</td></tr>";

					
                    // Clean Project Attributes via Empty Specifications 
                    clickMethod = this.functionPrefix + '.changeViewOptions(this);';
                    htmlcode += '<tr class="darktext_greenlink_11 cleanProjAttrsViaEmptySpecsRow"';
                    if (!this.autoUpdateProjAttrsCheckbox) {
                    	htmlcode += ' style="display : none;" ';
                    }
                    htmlcode +='><td class="ACatLayoutViewDetails" style="padding-left:30px; padding-bottom:5px">';
                    
                    htmlcode += '<input type="checkbox" id="cleanProjAttrsViaEmptySpecs" name="cleanProjAttrsViaEmptySpecs" onclick="'+clickMethod+'" ';
                    if (this.cleanProjAttrsViaEmptySpecs) {
                    	htmlcode += ' checked ';
                    }
                    if (!this.canEditTaskType) {
                    	htmlcode += ' disabled ';
                    }
                    
                    htmlcode += '>';
                    htmlcode += '<label for="cleanProjAttrsViaEmptySpecs">' + ACLayoutStrings.t('ACLayoutCleanProjAttrsViaEmptySpecs') + '</label>';
               		//htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/task/ta_wc_SpecificationsTab");
                    htmlcode += '</td></tr>';

                    // Replace placeholder groups from task specifications
                    clickMethod = this.functionPrefix + '.changeViewOptions(this);';
                    htmlcode += '<tr class="darktext_greenlink_11"><td class="ACatLayoutViewDetails">';
                    htmlcode += '<input type="checkbox" id="assignRolesByTaskSpecsCheckbox" name="assignRolesByTaskSpecsCheckbox" value="" onclick="'+clickMethod+'" '; 
                    if (this.assignRolesByTaskSpecsCheckbox) htmlcode += ' checked ';
                    if (!this.canEditTaskType) htmlcode += ' disabled ';
                    htmlcode += '>';
                    htmlcode += '<label for="assignRolesByTaskSpecsCheckbox">' + ACLayoutStrings.t('ACLayoutAssignRolesByTaskSpecs') + '</label></td></tr>';
                } else { // attributes layout page

                	htmlcode += '<div style="float: left; padding-right: 5px">';
					htmlcode += '<div class="btn-group" role="group">';

                    if (!this.views || !this.views[1]) {
                        clickMethod = this.functionPrefix + '.startViews(\'Creation\'); return false;';
                        htmlcode += '<button type="button" class="btn btn-default" onclick="'+ clickMethod +'">'+ ACLayoutStrings.t('ACLayoutAddCreation') +'</button>';
                    }
					
					if (!this.views || !this.views[2]) {
                        clickMethod = this.functionPrefix + '.startViews(\'ProjectCreation\'); return false;';
                        htmlcode += '<button type="button" class="btn btn-default" onclick="'+ clickMethod +'">'+ ACLayoutStrings.t('ACLayoutAddProjectCreation') +'</button>';
                    }
                    if (!this.views || !this.views[3]) {
                        clickMethod = this.functionPrefix + '.startViews(\'Header\'); return false;';
                        htmlcode += '<button type="button" class="btn btn-default" onclick="'+ clickMethod +'">'+ ACLayoutStrings.t('ACLayoutAddHeader') +'</button>';
                    }

                    clickMethod = this.functionPrefix + '.addView(); return false;';
                    htmlcode += '<button type="button" class="btn btn-default" onclick="'+ clickMethod +'">'+ ACLayoutStrings.t('ACLayoutAddView') +'</button>';
                	htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/concept/co_wc_AttributesView");
    				
                	htmlcode += '</div>';
					htmlcode += '</div>';
					
					htmlcode += '<div style="float: left; padding-right: 5px">';
					htmlcode += '<div class="btn-group" role="group">';
                	
                    if (this.showEditTerminology && this.layout == 'Grid' && (!this.currentView || this.currentView == 'Default' || this.currentView == 'Creation' || this.currentView == 'ProjectCreation')) {
                        clickMethod = this.functionPrefix + '.editTerminology(); return false;';
                        htmlcode += '<button type="button" class="btn btn-default" onclick="'+ clickMethod +'">'+ ACLayoutStrings.t('ACLayoutEditTerminology') +'</button>';
                        htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/task/ta_wc_EditTerminologyforProjectCreation");
                    }
                    
                    htmlcode += '</div>';
					htmlcode += '</div>';
					htmlcode += '<div style="clear: both"></div>';
					htmlcode += '</div>';
                    htmlcode += '</td></tr>';
                }

                if (this.layout == 'Grid') {
                    // alternating background:
                    var clickMethod = this.functionPrefix + '.changeViewOptions(this);';
                    htmlcode += '<tr class="darktext_greenlink_11"><td class="ACatLayoutViewDetails">';
                    htmlcode += '<input type="checkbox" id="viewDetailsAlternating" value="" onclick="'+clickMethod+'" '; 
                    if (this.alternateBackground) htmlcode += ' checked ';
                    htmlcode += '/>';  
                    htmlcode += '<label for="viewDetailsAlternating">' + ACLayoutStrings.t('ACLayoutAlternatingBackground') + '</label></td></tr>';

                    // show also attribute types and restricted set types 
                    htmlcode += '<tr class="darktext_greenlink_11"><td class="ACatLayoutViewDetails">';
                    htmlcode += '<input type="checkbox" id="viewDetailsTypes" value="" onclick="'+clickMethod+'" '; 
                    if (this.showTypeAsUnit) htmlcode += ' checked ';
                    htmlcode += '/>';
                    htmlcode += '<label for="viewDetailsTypes">' + ACLayoutStrings.t('ACLayoutShowTypes') + '</label></td></tr>';	

                    //keep session alive
                    htmlcode += '<tr class="darktext_greenlink_11"><td class="ACatLayoutViewDetails">';
                    htmlcode += '<input type="checkbox" id="keepSessionAliveChkbox" value="" onclick="'+clickMethod+'" '; 
                    if (this.keepSessionAlive) htmlcode += ' checked ';
                    htmlcode += '/>';
                    htmlcode += '<label for="keepSessionAliveChkbox">' + ACLayoutStrings.t('ACLayoutKeepSessionAlive') + '</label></td></tr>';	

                    
                    // use legacy rendering
                	htmlcode += '<tr class="darktext_greenlink_11"><td class="ACatLayoutViewDetails">';
                    htmlcode += '<input type="checkbox" id="alternativeRenderingChkbox" value="" onclick="'+clickMethod+'" '; 
                    if (this.useAlternativeRendering) htmlcode += ' checked ';
                    htmlcode += '/>';
                    htmlcode += '<label for="alternativeRenderingChkbox">' + ACLayoutStrings.t('ACLayoutAlternativeRendering') + '</label></td></tr>';	
                
                    if (this.currentView == 'Creation' && (this.easterEgg || this.addToGeneralSection) ) {  // show only if easter egg enabled OR somebody already set up the addToGeneralSection in a previous session
                        // add attribute/specification section to general section
                        htmlcode += '<tr class="darktext_greenlink_11"><td class="ACatLayoutViewDetails">';
                        htmlcode += '<input type="checkbox" id="addToGeneralChkBox" value="" onclick="'+clickMethod+'" '; 
                        if (this.addToGeneralSection) htmlcode += ' checked ';
                        htmlcode += '/>';
                        htmlcode += '<label for="viewDetailsTypes">' + ACLayoutStrings.t('ACLayoutAddToGeneralSection') + '</label></td></tr>';	
                    }
                }

                htmlcode += '</table></td>';
                if(this.isTaskTypePage) htmlcode += '<td/><td/>';  // move checkboxes left in task type page


            }
            htmlcode += '</tr>';

        } catch (err) {
            alert("Problem in attributeCategory.displayViewsHeader: " + err);
        }
        return htmlcode;
    };

    attributeCategory.prototype.changeViewOptions = function(elem) {
        try {
            this.setACatChanged();
            if (elem) {
                if (elem.id == "viewDetailsAlternating") {
                    this.alternateBackground = elem.checked;
                    if (this.views) this.views[this.currentViewIdx].alternateBackground = this.alternateBackground;		

                } else if (elem.id == "viewDetailsTypes") {
                    this.showTypeAsUnit = elem.checked;
                    if (this.views) this.views[this.currentViewIdx].showTypeAsUnit = this.showTypeAsUnit;									
                } else if (elem.id == "autoUpdateProjAttrsCheckbox") {
                    this.autoUpdateProjAttrsCheckbox = elem.checked;
                    //also show/hide the "Clear Project Attributes via Empty Specifications" row
                    if (elem.checked) {
                    	$(".cleanProjAttrsViaEmptySpecsRow").show();
                    } else {
                    	$(".cleanProjAttrsViaEmptySpecsRow").hide();
                    }
                } else if (elem.id == "cleanProjAttrsViaEmptySpecs") {
                    this.cleanProjAttrsViaEmptySpecs = elem.checked;								
                } else if (elem.id == "assignRolesByTaskSpecsCheckbox") {
                    this.assignRolesByTaskSpecsCheckbox = elem.checked;								
                } else if (elem.id == "addToGeneralChkBox") {
                    this.addToGeneralSection = elem.checked;
                    if (this.views) this.views[this.currentViewIdx].addToGeneralSection = this.addToGeneralSection;				
                } else if (elem.id == "keepSessionAliveChkbox"  ) {
                    this.keepSessionAlive = elem.checked;
                    if (this.views) this.views[this.currentViewIdx].keepSessionAlive = this.keepSessionAlive;	
                } else if (elem.id == "alternativeRenderingChkbox"  ) {
                    this.useAlternativeRendering = elem.checked;
                    if (this.views) this.views[this.currentViewIdx].useAlternativeRendering = this.useAlternativeRendering;	
                }

                this.displayAttributeCategoryPreview();	
            }
        } catch (err) {
            alert("Problem in attributeCategory.changeViewOptions: " + err);
        }

    };
    /**
    * Start using views.  Will show a views dropdown
    */
    attributeCategory.prototype.startViews = function(viewName) {
        try {
            this.setACatChanged();
            if (!this.views) {  // copy the default view to view 0
                this.views = new Array(this.cViewOptions.length);
                this.views[0] = new Object();
                this.views[0].nRows = this.nRows;
                this.views[0].nCols = this.nCols;
                this.views[0].alternateBackground = this.alternateBackground;
                this.views[0].showTypeAsUnit = this.showTypeAsUnit;
                this.views[0].layout = this.layout;
                this.views[0].cells = this.cells;
                this.views[0].name = "Default";

                this.currentView = "Default";
                this.currentViewIdx = 0;
            // the other 2 will be null
            }
            this.changeView(null,viewName);
            this.displayLayout();
        } catch (err) {
            alert("Problem in attributeCategory.startViews: " + err);
        }

    };

    /**
    * Delete the current view. Impossible for the default view
    */
    attributeCategory.prototype.deleteView = function() {
        try {
            this.setACatChanged();
            this.galleryType = 'Attributes';
            if (this.currentView == "Default") {
                alert(ACLayoutStrings.t('ACLayoutCannotDeleteDefault'));
                return;
            }
            var conf = confirm(ACLayoutStrings.t('ACLayoutConfirmDeleteView'));
            if (!conf) return;
            var viewIdx = this.currentViewIdx;
            if (this.currentViewIdx < this.views.length) {
                this.views[this.currentViewIdx]=null;  //throws away the view
                // now switch to default view
                this.currentViewIdx = 0;
                this.currentView = this.views[0].name;
                this.nCols = this.views[0].nCols;
                this.nRows = this.views[0].nRows;
                this.layout = this.views[0].layout;
                this.alternateBackground = this.views[0].alternateBackground;
                this.showTypeAsUnit = this.views[0].showTypeAsUnit;
                this.cells = this.views[0].cells;

                // diminish the placed counters
                for (var att=0; att<this.attributes.length; att++) {
                	if(this.attributes[att].placedArr[viewIdx]){
                		this.attributes[att].placed--;
                	}
                    this.attributes[att].placedArr[viewIdx]=0;
                }
                // check if it's the last one
                for (var i=1;i<this.cViewOptions.length; i++) {
                    if (this.views[i]!=null) break;
                }
                if (i == this.cViewOptions.length) {
                    this.views = null; // last non-default view is deleted, go back to non-views mode
                }
            }
            this.displayLayout();
        } catch (err) {
            alert("Problem in attributeCategory.deleteView: " + err);
        }

    };

    /**
    * converts view name to view index in this.views
    * @param {String} viewName - name of the view
    * @return {Int} - view index in this.views, 0 for Default view (also if this.views does not exist) -1 if not found
    */
    attributeCategory.prototype.findView = function(viewName) {
        var viewIdx = -1;
        try {
            if (this.views) {
                for (var view=0; view< this.views.length; view++) {
                    if (this.views[view] && this.views[view].name == viewName) {
                        viewIdx = view;
                        break;
                    }
                }
            } else {
                if (viewName == 'Default') viewIdx = 0;
            }		

        } catch (err) {
            alert("Problem in attributeCategory.findView: " + err);
        }
        return viewIdx;
    };

    /**
    * Change the current view to the one selected in the view select box
    * @param selectElem: name of the selectbox containing the views selector.
    * @param viewName: direct choice of view without using the selector
    * One of both parameters must be given. If both are given (not advised), the selection takes precedence
    */
    attributeCategory.prototype.changeView = function(selectElem,viewName) {
        try {

            var newView = viewName;
            if (selectElem) newView = selectElem.value;
            this.galleryType = 'Attributes';

            // find the index
            for (var view=0; view< this.cViewOptions.length; view++) {
                if (this.cViewOptions[view] == newView) break;
            }
            if (view == this.cViewOptions.length) throw("Unknown view"); // should not happen
            this.currentViewIdx = view;

            if (!this.views || this.views[view] == null) {
              this.setACatChanged();   
              // then this is a new view not yet defined. 
              if (!this.views) this.views = new Array(this.cViewOptions.length);
              var viewObj = new Object();           
              this.views[view] = viewObj;
              if (this.cViewOptions[view] != 'Header') { // treat header differently

                // first make a copy
                viewObj.layout = this.layout;
                viewObj.name = newView;
                viewObj.nRows = this.nRows;
                viewObj.nCols = this.nCols;		
                viewObj.alternateBackground = this.alternateBackground;	
                viewObj.showTypeAsUnit = this.showTypeAsUnit;				
                viewObj.addToGeneralSection = this.addToGeneralSection;	
                viewObj.keepSessionAlive = this.keepSessionAlive;	
                viewObj.useAlternativeRendering = this.useAlternativeRendering;
                
                var allowProperties = viewObj.name.indexOf("Default") === 0 || viewObj.name.indexOf("ProjectCreation") === 0 || viewObj.name.indexOf("TaskExecution") === 0;

                // make a deep copy of this.cells but without copying the attribute objects
                viewObj.cells = new Array(this.cells.length);
                for (var row=0; row < this.cells.length; row++) {
                    if (!this.cells[row]) continue;
                    viewObj.cells[row] = new Array(this.cells[row].length);
                    for (var col=0; col < this.cells[row].length; col++) {
                        var sourceObj = this.cells[row][col];
                        if (!sourceObj) continue;
                        
                        var attObj = sourceObj.attObj;
                        if(!allowProperties && attObj && attObj instanceof Property){
                        	continue;
                        }

                        var cellObj = viewObj.cells[row][col] = new Object();
                        for (prop in sourceObj) { // copy all properties.  Objects, f.e. attObj are shallow copied
                        	if (sourceObj.hasOwnProperty(prop)) {
                            cellObj[prop] = sourceObj[prop];
                        }
                        }

                        // update "placed" counters
                        if (attObj) {
                            attObj.placed++;  // overall placement
                            // placement in the new view
                            if (!attObj.placedArr[this.currentViewIdx]) {
                                attObj.placedArr[this.currentViewIdx] = 1;
                            } else {
                                attObj.placedArr[this.currentViewIdx]++;
                            }
                        }
                        // copy the buttons
                        if (sourceObj.buttons) {
                            this.copyButtons(sourceObj, cellObj, viewName);						
                        }
                    }
                }



              } else {  // header view: default to OneColumn design 
                viewObj.layout = 'OneColumn';
                viewObj.name = newView;
                viewObj.nRows = 0;
                viewObj.nCols = 1;		
                viewObj.cells = new Array();
                viewObj.alternateBackground = false;
                viewObj.showTypeAsUnit = false;		
                viewObj.addToGeneralSection = false;	
                viewObj.keepSessionAlive = false;	
                viewObj.useAlternativeRendering = false;

              }
              
              this.addFormData(viewName);            
            }
            // switch to selected view
            this.cells = this.views[view].cells;
            this.layout = this.views[view].layout;
            this.nRows = this.views[view].nRows;
            this.nCols = this.views[view].nCols; 
            this.alternateBackground = this.views[view].alternateBackground;
            this.showTypeAsUnit = this.views[view].showTypeAsUnit;
            this.addToGeneralSection = this.views[view].addToGeneralSection;
            this.keepSessionAlive = this.views[view].keepSessionAlive;
            this.useAlternativeRendering = this.views[view].useAlternativeRendering;

            this.currentView = newView;
            this.currentRowIndex = null; this.currentColIndex = null;
            
            // show header as readonly in preview		
            if (this.currentView == 'Header'){
          	  this.showFakedReadOnly = true;  
            } else {
          	  this.showFakedReadOnly = false;	
            }

            this.recreateDocTableXForCurrentView();

            this.displayLayout();

        } catch (err) {
            alert("Problem in attributeCategory.changeView: " + err);
        }
    };
    
    /**
     * Add form specific fields to the category layout.
     * @param viewName: view to which the element should be attached
     */
     attributeCategory.prototype.addFormData = function(viewName) {
    	 var view = null, before = null, after = null;
    	 // Add form for ProjectCreation and TaskExecution
         if (viewName === 'TaskExecution'){
        	 view = this.getViewByName('TaskExecution');
        	 
        	 // rows to prepend
        	 var description = this.createAttributeCell(this.getPropertyByName('Description'));
        	 var status = this.createAttributeCell(this.getPropertyByName('Status'));
        	 var assignedTo = this.createAttributeCell(this.getPropertyByName('AssignedTo'));
        	 var previousAssignee = this.createAttributeCell(this.getPropertyByName('PreviousAssignee'));
        	 var due = this.createAttributeCell(this.getPropertyByName('Due'));
        	 var estimated = this.createAttributeCell(this.getPropertyByName('Estimated'));
        	 
        	 before = [[description],[status],[assignedTo],[previousAssignee],[due],[estimated]];
        	 _.each(before, _.bind(function(row){
        		 row[0].colSpan = view.nCols;
        	 }, this));

        	 // rows to append
        	 var checkList = this.createAttributeCell(this.getPropertyByName('CheckList'));
        	 var discussions = this.createAttributeCell(this.getPropertyByName('Discussions'));
        	 var documents = this.createAttributeCell(this.getPropertyByName('Documents'));
        	 var buttonRow = this.createAttributeCell(this.getPropertyByName('ButtonRow'));
        	 buttonRow.label = '';
        	 
        	 after = [[checkList],[discussions],[documents],[buttonRow]];
        	 _.each(after, _.bind(function(row){
        		 row[0].colSpan = view.nCols;
        	 }, this));
        	 
        	 var viewIdx = this.getViewIndex(viewName);
        	 _.each(this.otherProperties, _.bind(function(prop){
        		 prop.placedArr[viewIdx] = 1;
        	 }, this));
        	 
        	// repaint the attribute gallery
        	 this.galleryType = 'Properties';
            $("#tdAttributeSelector").html(this.displayAttributeSelector());
         } else if (viewName === 'ProjectCreation') {
        	 view = this.getViewByName('ProjectCreation');

        	 // rows to prepend
        	 var templateSelector = this.createAttributeCell(this.getPropertyByName('TemplateSelection'));
        	 templateSelector.label = '';
        	 var projectName = this.createAttributeCell(this.getPropertyByName('ProjectName'));
        	 var requestor = this.createAttributeCell(this.getPropertyByName('Requestor'));
        	 var projectManager = this.createAttributeCell(this.getPropertyByName('ProjectManager'));
        	 var status = this.createAttributeCell(this.getPropertyByName('Status'));
        	 var customer = this.createAttributeCell(this.getPropertyByName('Customer'));
        	 var location = this.createAttributeCell(this.getPropertyByName('Location'));
        	 var thumbnail = this.createAttributeCell(this.getPropertyByName('Thumbnail'));
        	 var description = this.createAttributeCell(this.getPropertyByName('Description'));
        	 var dueDate = this.createAttributeCell(this.getPropertyByName('DueDate'));
        	 var planning = this.createAttributeCell(this.getPropertyByName('Planning'));
        	 var lastTaskEnds = this.createAttributeCell(this.getPropertyByName('LastTaskEnds'));

        	 // rows to append
        	 var members = this.createAttributeCell(this.getPropertyByName('Members'));
        	 var billOfMaterials = this.createAttributeCell(this.getPropertyByName('BillOfMaterials'));
        	 var documents = this.createAttributeCell(this.getPropertyByName('Documents'));

        	 before = [[templateSelector], [projectName], [requestor], [projectManager], [status], [customer], [location], [description], [dueDate], [planning], [lastTaskEnds], [thumbnail]];
        	 after = [[members],[billOfMaterials],[documents]];
        	 _.each(before, _.bind(function(row){
        		 row[0].colSpan = view.nCols;
        	 }, this));
        	 _.each(after, _.bind(function(row){
        		 row[0].colSpan = view.nCols;
        	 }, this));
        	 
        	 var viewIdx = this.getViewIndex(viewName);
        	 _.each(this.otherProperties, _.bind(function(prop){
        		 prop.placedArr[viewIdx] = 1;
        	 }, this));

             // repaint the attribute gallery
        	 this.galleryType = 'Properties';
             $("#tdAttributeSelector").html(this.displayAttributeSelector());
         }
         
         // merge everything
         if(view){
        	 this.trimRows(view.cells);
        	 view.cells = before.concat(view.cells);
        	 view.cells = view.cells.concat(after);
        	 view.nRows = view.cells.length;
         }
     };

    /**
    * Copies buttons from one cell to another, typically when copying a view
    * @param sourceObj - cellObj from where to copy the buttons
    * @param destObj - cellObj to which to copy the buttons 
    */
    attributeCategory.prototype.copyButtons = function(sourceObj,destObj, newViewName) {
        'use strict';
        try {

            destObj.buttons = [];
            if (sourceObj.buttons) {
                for (var button=0; button< sourceObj.buttons.length; button++) {
                    var sourceButtonObj = sourceObj.buttons[button];
                    var destButtonObj = {};
                    for (prop in sourceButtonObj) { // copy all properties.  
                    	if (!sourceButtonObj.hasOwnProperty(prop)) continue;
                        switch (prop) {
                            case 'inputAtts':
                            case 'outputAtts':
                            case 'changeAtts':
                                this.copyReferencedAtts(sourceButtonObj,destButtonObj,prop);
                                break;
                            default:
                                destButtonObj[prop] = sourceButtonObj[prop];
                        }
                    }


                    // change the view name if requested
                    if (newViewName) destButtonObj.viewName = newViewName;
                    destObj.buttons.push(destButtonObj);
                }
            }

        } catch (err) {
            alert("Problem in attributeCategory.copyButtons: " + err);
        }
    };

    /**
    * Copies buttons from one cell to another, typically when copying a view
    * @param {string} type - on of inputAtts, outputAtts and changeAtts
    */
    attributeCategory.prototype.copyReferencedAtts = function(sourceButtonObj,destButtonObj,type) {
        'use strict';
        try {
            var sourceAtts = sourceButtonObj[type];
            var destAtts = [];

            if (sourceAtts && sourceAtts.constructor == Array) {
                destButtonObj[type] = destAtts;
                for (var att=0; att< sourceAtts.length; att++) {
                    destAtts[att] = {};
                    // Tthis does not check if this attribute is placed in this view. When copying the view, this will not be a problem --> skipped here
                    for (var prop in sourceAtts[att]) {
                    	if (sourceAtts[att].hasOwnProperty(prop)) {
                        destAtts[att][prop] =  sourceAtts[att][prop];
                    }
                }
            }
            }



        } catch (err) {
            alert("Problem in attributeCategory.copyReferencedAtts: " + err);
        }
    };

    /**
    * Adds a view with a fixed name
    * First use is task type views, which need to have the name of the task type
    * The attribute category code however doesn't care whether this is for a task type or any other use
    * The name is all what counts...
    * @param {String} name Name of the view. If null, a name will be prompted (normal case) 
    * @author HADW
    */
    attributeCategory.prototype.addView = function(name) {
        try {
            var viewOptions = this.cViewOptions;
            if (!name) {
                name = prompt(ACLayoutStrings.t('ACLayoutViewName'));
                if(name === null){
                	// cancel was clicked
                	return;
                }
            }
            if (!name) {alert(ACLayoutStrings.t('ACLayoutNameCannotBeEmpty')); return;};

            // check if name already exists
            for (var idx=0; idx<viewOptions; idx++) {
                if (viewOptions[idx] == name) {alert(ACLayoutStrings.t('ACLayoutViewNameExists')); return;};				
            }

            this.cViewOptions.push(name);
            //ACLayoutStrings.t('ACLayout'+ name) = name;
            this.startViews(name);

        } catch (err) {
            alert("Problem in attributeCategory.changeView: " + err);
        }
    };
    attributeCategory.prototype.displayTopOperations = function() {
        try {
            var htmlcode = '';

            if (this.layout == 'OneColumn' ) {
                if (this.allowGridLayout===true && this.currentView != 'Header') {			
                    var clickMethod = this.functionPrefix + '.useGridLayout(); return false;';
                    htmlcode += '<table class="ACatLayoutToolbar"><tr>' ;
                    htmlcode += '<td><a href="#" onclick="' + clickMethod  + '">' + ACLayoutStrings.t('ACLayoutUseGridLayout') + '</a></td>';
                    htmlcode += '</tr></table>' ;
                } else {
                    htmlcode += '<img src="images/clearpixel.gif" height="1" width="200px"/>';
                }
            } else {
                htmlcode += '<table class="ACatLayoutToolbar">' ;
                var clickMethod = this.functionPrefix + '.removeGridLayout(); return false;';
                htmlcode += '<tr><td colspan="3"><a href="#" onclick="' + clickMethod  + '">' + ACLayoutStrings.t('ACLayoutRemoveGridLayout') + '</a></td></tr>';


                // row operations
                htmlcode += '<tr><td>';
                clickMethod = this.functionPrefix + '.addRow();';

                htmlcode += '<img onclick="' + clickMethod  + '" src="./images/grid_add_row.png" class="ACatLayoutToolbarButton"/>';

                clickMethod = this.functionPrefix + '.addRow(-1);';
                htmlcode += '&nbsp;<img onclick="' + clickMethod  + '" src="./images/grid_insert_row.png" class="ACatLayoutToolbarButton"/>';

                clickMethod = this.functionPrefix + '.deleteRow(-1);';
                htmlcode += '&nbsp;<img onclick="' + clickMethod  + '" src="./images/grid_remove_row.png" class="ACatLayoutToolbarButton"/>';

                // column operations
                clickMethod = this.functionPrefix + '.addColumn();';
                htmlcode += '&nbsp;&nbsp;&nbsp;<img onclick="' + clickMethod  + '" src="./images/grid_add_column.png" class="ACatLayoutToolbarButton"/>';

                clickMethod = this.functionPrefix + '.addColumn(-1); ';
                htmlcode += '&nbsp;<img onclick="' + clickMethod  + '" src="./images/grid_insert_column.png" class="ACatLayoutToolbarButton"/>';

                clickMethod = this.functionPrefix + '.deleteColumn(-1); ';
                htmlcode += '&nbsp;<img onclick="' + clickMethod  + '" src="./images/grid_remove_column.png" class="ACatLayoutToolbarButton"/>';

                htmlcode += '</td>';


                // "Insert"  buttons
                htmlcode += '<td class="darktext_greenlink_11">&nbsp;&nbsp;&nbsp;&nbsp;' + ACLayoutStrings.t('ACLayoutInsert')+ ':</td>';
                // title operations
                clickMethod = this.functionPrefix + '.addTitle(); ';
                htmlcode += '<td><img onclick="' + clickMethod  + '" src="./images/grid_insert_title.png" class="ACatLayoutToolbarButton"/>';

                // For white spaces
                clickMethod = this.functionPrefix + '.addSpacer(); ';
                htmlcode += '&nbsp;<img onclick="' + clickMethod  + '" src="./images/grid_insert_space.png" class="ACatLayoutToolbarButton"/>';

                // For Ruler
                clickMethod = this.functionPrefix + '.addRuler(); ';
                htmlcode += '&nbsp;<img onclick="' + clickMethod  + '" src="./images/grid_insert_line.png" class="ACatLayoutToolbarButton"/>';

                // For header bar
                clickMethod = this.functionPrefix + '.addHeaderBar(); ';
                htmlcode += '&nbsp;<img onclick="' + clickMethod  + '" src="./images/grid_insert_table_header.png" class="ACatLayoutToolbarButton"/>';

                // For button
                clickMethod = this.functionPrefix + '.addButton(); ';
                htmlcode += '&nbsp;<img id="insertButton" onclick="' + clickMethod  + '" src="./images/grid_insert_button.png" class="ACatLayoutToolbarButton"/></td>';
                htmlcode += '</tr></table>';
            }

            return htmlcode;
        } catch (err) {
            alert("Problem in attributeCategory.displayTopOperations: " + err);
        }
    };


    /**
    * Creates html code for the cell details pane to the right of the cell table in the grid layout
    * @return {string} HTML code for the cell details pane
    */
    attributeCategory.prototype.displayCellDetails = function() {
        try {


            var changeMethod = this.functionPrefix + '.applyCellProperties(' + this.currentRowIndex + ',' + this.currentColIndex +')'; 
                    // change HADW 30 July 2012: added row and column since you cannot assume that they are still the same when the onchange is executed when blurring through clicking on another cell in the grid. DT105599 
            var cellObj = null;
            var htmlcode = '';
            var row = this.currentRowIndex, col=this.currentColIndex;

            htmlcode += '<table id="cellDetailsTable">';

            // In case nothing selected or empty cell, show a message that no cell details can be shown
            if (this.currentRowIndex == null || this.currentColIndex== null || !this.cells[this.currentRowIndex] || !this.cells[this.currentRowIndex][this.currentColIndex]) {
                htmlcode += '<tr><td class="darktext_greenlink_11"><em>'  + ACLayoutStrings.t('ACLayoutNoCellDetails') + '</em></td></tr></table>';
                return htmlcode;
            }

            if(this.cells[this.currentRowIndex]) {

                cellObj = this.cells[this.currentRowIndex][this.currentColIndex];  
                var attObj = cellObj.attObj; // can be null!

                // header
                htmlcode += '<tr class="darktext_greenlink_11 CellDetailsTableHeader">';
                htmlcode += '<td><b>' + ACLayoutStrings.t('ACLayout' + ((this.isTaskTypePage && cellObj.type=="Attribute")?"Specification":cellObj.type))+'</b></td>';
                htmlcode += '<td><b>';
                switch (cellObj.type) {
                    case "Title":
                        htmlcode += cellObj.label; break;
                    case "Attribute":
                        if (attObj) {
                            htmlcode += WCXML.xmlencode(attObj.name); 
                        } 
                        break;
                    case "Property":
                        if (attObj) {
                            htmlcode += WCXML.xmlencode(ACatStrings.t(attObj.name)); 
                        } 
                        break;
                    case "Button":
                        if (cellObj.buttons && cellObj.buttons.length>0) {
                            htmlcode += WCXML.xmlencode(cellObj.buttons[0].label); 
                        }
                    default:
                        break;
                }
                htmlcode += '</b></td></tr>';



                // attribute type
                if (cellObj.type == 'Attribute') {
                    htmlcode += '<tr class="darktext_greenlink_11">';
                    htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsAttributeType') + '</td>';            
                    htmlcode += '<td>' + WCXML.xmlencode(attObj.type) + '</td>';
                    htmlcode += '</tr>';
                }

                if (this.layout == 'OneColumn' ) {
                    // is required flag
                    if(cellObj) {
                        var ruleObj = cellObj.ruleObj;

                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutRequired') + '</td>';            
                        htmlcode += '<td><input id="isrequiredcheckbox" type="checkbox" onclick="' + changeMethod + '"';
                        if (ruleObj == this.isRequiredRule)
                            htmlcode += 'checked';
                        htmlcode += '/></td></tr>';
                    }
                } else {		
                    // label
                    if(cellObj && cellObj.type && cellObj.type == 'Title') {
                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsTitle') + '</td>';
                        var value = "";
                        if (cellObj.label) value = WCXML.xmlencode(cellObj.label);                
                        htmlcode += '<td> <input type=text id="label" style="width:100%"  value="'+ value +'" onchange="' + changeMethod + '"/> </td>';				
                        htmlcode += '</tr>';                

                    }
                    else if(cellObj && cellObj.type && (cellObj.type == 'Attribute' || cellObj.type == 'Property')) {  // easter Egg
                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsTitle') + '</td>';
                        // if label is null, we should use the name of the attribute, but if it's an empty string, we use that
                        var label = '';
                        if (cellObj.label != null) {
                            label = WCXML.xmlencode(cellObj.label);    
                        } else {
                            label = WCXML.xmlencode(cellObj.name); 
                        }
                        htmlcode += '<td><table>';

                        htmlcode += '<tr class="darktext_greenlink_11"><td>' + ACLayoutStrings.t('ACLayoutLabelText') + '</td><td> <input type=text id="label" size="40" value="'+label+'" onchange="' + changeMethod + '"/></td></tr>';



                        // Label image
                        htmlcode += '<tr class="darktext_greenlink_11"><td>' + ACLayoutStrings.t('ACLayoutLabelImage') + '</td>';
                        htmlcode += '<td> <input type=text id="labelImage" size="40" value="';
                        if(cellObj && cellObj.labelImage) htmlcode += WCXML.xmlencode(cellObj.labelImage);					
                        htmlcode += '" onchange="' + changeMethod + '"/>';
                        //htmlcode += '<input type="button" class="buttonsecondary" value = "'+ACLayoutStrings.t('ACLayoutSelectIcon')+'" name="iconGalleryOpenBtn"> ';
                        htmlcode += '</td></tr>';

                        // Label Position

                        htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutLabelPosition') + ': </td><td><select id="labelPosition" onchange="'+changeMethod+'" >';
                        var positionOptions = ['Left','Top'];
                        for (var option=0; option<positionOptions.length;option++) {
                            htmlcode += '<option value="'+positionOptions[option]+ '"' ;								
                            if (positionOptions[option] == cellObj.labelPosition) htmlcode +=' selected ';
                            htmlcode += '>'+ ACLayoutStrings.t('ACLayout' +positionOptions[option]) + '</option>';
                        }
                        htmlcode += '</select></td></tr>';
                        
                        // custom unit label
                        if(cellObj.type == 'Attribute'){
                        	htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutCustomUnitLabel') + '</td>';
                            htmlcode += '<td> <input type=text id="customUnitLabel" size="10" value="';
                            if(cellObj && cellObj.customUnitLabel){
                            	htmlcode += WCXML.xmlencode(cellObj.customUnitLabel);					
                            }
                            htmlcode += '" onchange="' + changeMethod + '"/>';
                            htmlcode += '</td></tr>';
                        }
                        

                        htmlcode += ' </table></td></tr>';	      // close the label table          
                    }

                    // colspan
                    // check maximum colspan

                    for (var col2=col+1; col2<this.nCols; col2++) {
                        if (this.cells[row][col2]) break;
                    }
                    var maxColspan = col2-col;
                    if (maxColspan <1) maxColspan =1;        			

                    htmlcode += '<tr class="darktext_greenlink_11">';
                    htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsColSpan') + '</td>';

                    htmlcode += '<td><select id="colspan" onchange="'+changeMethod+'">';
                    for (var span=1; span<= maxColspan; span++) {
                        if (cellObj && cellObj.colSpan == span) {
                            htmlcode += '<option value="' + span + '" selected>' + span + '</option>';
                        } else {
                            htmlcode += '<option value="' + span + '">' + span + '</option>';
                        }
                    }
                    htmlcode += '</select></td>';


                    // Row size field for TextArea (Attributes of type TEXT only and not for restricted sets or lists)
                    if(cellObj && cellObj.attObj && cellObj.attObj.type && cellObj.attObj.type.toLowerCase() == 'text' && !cellObj.attObj.rs && !attObj.listObj) {


                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsRowSize') + ':</td>';
                        if(cellObj && cellObj.rowSize) {
                            htmlcode += '<td> <input type=text id="rowsize" size=2 maxlength="2" value="'+cellObj.rowSize+'" onchange="' + changeMethod + '" onkeypress="'+this.functionPrefix + '.noenter();"/> </td>';
                        }
                        else {
                            htmlcode += '<td> <input type=text id="rowsize" size=2 maxlength="2" value="" onchange="' + changeMethod + '" onkeypress="'+this.functionPrefix + '.noenter();"/> </td>';
                        }
                        htmlcode += '</tr>';

                    }

                    // Allow setting the height of rich text box
                    if(cellObj && cellObj.attObj && cellObj.attObj.type && cellObj.attObj.type.toLowerCase() == 'richtext') {


                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutHeight') + '</td>';
                        if(cellObj && cellObj.rowSize) {
                            htmlcode += '<td> <input type=text id="rowsize" size=4 maxlength="4" value="'+cellObj.rowSize+'" onchange="' + changeMethod + '" onkeypress="'+this.functionPrefix + '.noenter();"/>';
                        }
                        else {
                            htmlcode += '<td> <input type=text id="rowsize" size=4 maxlength="4" value="" onchange="' + changeMethod + '" onkeypress="'+this.functionPrefix + '.noenter();"/>';
                        }
                        htmlcode += ACLayoutStrings.t('ACLayoutWidthHeightMeasure')+'</td></tr>';

                    }

                    if (cellObj.type == 'Ruler') {
                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>Thickness:</td>';
                        var changeMethod2 = this.functionPrefix + '.changeRuler(\'Thickness\')';
                        htmlcode += '<td><input id="rulerThickness" size="3" maxlength="3" type="text" value="' + cellObj.rulerThickness + '" onchange="' + changeMethod2 + '" >&nbsp;px</td>';
                        htmlcode += '</tr>';

                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>Color:</td>';
                        var changeMethod2 = this.functionPrefix + '.changeRuler(\'Color\')';
                        htmlcode += '<td><input id="rulerColor" type="text" value="' + WCXML.xmlencode(cellObj.rulerColor) + '" onchange="' + changeMethod2 + '" ></td>';
                        htmlcode += '</tr>';
                    }
                    if (cellObj.type == 'Spacer') {
                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>Height:</td>';
                        var changeMethod2 = this.functionPrefix + '.changeSpacer(\'Height\')';
                        htmlcode += '<td><input id="spacerHeight" size="3" maxlength="3" type="text" value="' + cellObj.height + '" onchange="' + changeMethod2 + '" >&nbsp;px</td>';
                        htmlcode += '</tr>';

                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>Color:</td>';
                        var changeMethod2 = this.functionPrefix + '.changeSpacer(\'Color\')';
                        htmlcode += '<td><input id="spacerColor" type="text" value="' + WCXML.xmlencode(cellObj.background) + '" onchange="' + changeMethod2 + '" ></td>';
                        htmlcode += '</tr>';
                    }
                    // style
                    if (cellObj && cellObj.type != 'Ruler' && cellObj.type != 'HeaderBar' && cellObj.type != 'Spacer') {
                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>'+ ACLayoutStrings.t('ACLayoutStyle') +':</td>';
                        var changeMethod2 = this.functionPrefix + '.changeStyle(this)';
                        htmlcode += '<td><select id="attStyle" onchange="'+ changeMethod2 +'">';
                        var styleIdx=0;
                        for (var style=0; style<this.styles.length; style++) {                    
                            var name = this.styles[style].name;
                            htmlcode += '<option value="'+ style +'"';
                            if (cellObj.styleObj == this.styles[style]) {
                                htmlcode += ' selected ';
                                styleIdx = style;
                            }
                            htmlcode += '>' + WCXML.xmlencode(name) + '</option>';
                        }
                        htmlcode += '</select>';
                        clickMethod = this.functionPrefix + '.showStyle(null); return false;';
                        htmlcode += '<br><a href="#" onclick="' + clickMethod +'">'+ ACLayoutStrings.t('ACLayoutAddStyle') +'</a>';
                        if (cellObj.styleObj!=null && !cellObj.styleObj.standardStyle) {
                            clickMethod = this.functionPrefix + '.showStyle(' + styleIdx + '); return false;';
                            htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="' + clickMethod +'">'+ ACLayoutStrings.t('ACLayoutEditStyle') +'</a>';
                            clickMethod = this.functionPrefix + '.deleteStyle(' + styleIdx + '); return false;';
                            htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="' + clickMethod +'">'+ ACLayoutStrings.t('ACLayoutDeleteStyle') +'</a>';
                        }
                        htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/task/ta_wc_DefineAttributeStyles");
                        htmlcode += '</td>';
                        htmlcode += '</tr>';
                    }

                    if (cellObj) {
                        // visibility rules
                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsVisibility') + '</td>';
                        htmlcode += '<td>';
                        var changeMethod2 = this.functionPrefix + '.changeRule(this)';
                        var ruleIdx = -1;
                        htmlcode += '<select id="attVisibility" onchange="'+ changeMethod2 +'">';
                        for (var rule=0; rule<this.rules.length; rule++) {
                            var name = this.rules[rule].name;
                            if(cellObj.type == 'Property' && (name == "ReadOnly" || name == "Required")){
                            	continue;
                            }
                            if (this.rules[rule].standardRule) {
                                name = ACLayoutStrings.t('ACLayout' + name);
                            }

                            htmlcode += '<option value="'+ rule +'"';						
                            if (cellObj.ruleObj == this.rules[rule]) {
                                htmlcode += ' selected ';

                                ruleIdx = rule;
                            }
                            htmlcode += '>' + WCXML.xmlencode(name) + '</option>';
                        }					

                        htmlcode += '</select>';					

                        var changeMethod2 = this.functionPrefix + '.showRule(); return false;';
                        htmlcode += '<br><a href="#" class="darktext_greenlink_11" onclick="' + changeMethod2 + '">' + ACLayoutStrings.t('ACLayoutAddRule')+ '</a>';

                        if (cellObj && cellObj.ruleObj && cellObj.ruleObj.standardRule == false) {
                            var changeMethod3 = this.functionPrefix + '.showRule(' + ruleIdx + '); return false;';
                            htmlcode += '&nbsp;&nbsp;&nbsp;&nbsp;<a href="#" class="darktext_greenlink_11" onclick="' + changeMethod3 + '">' + ACLayoutStrings.t('ACLayoutEditRule')+ '</a>';
                            var changeMethod3 = this.functionPrefix + '.deleteRule('+ruleIdx+'); return false;';
                            htmlcode += '&nbsp;&nbsp;&nbsp;&nbsp;<a href="#" class="darktext_greenlink_11" onclick="' + changeMethod3 + '">' + ACLayoutStrings.t('ACLayoutDeleteRule')+ '</a>';
                        }
                        htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/task/ta_wc_AddAttributeRules");
                        htmlcode += '</td>';
                        htmlcode += '</tr>';
                    }  // end if cellObj


                    // Restrictions							
                    if (cellObj && attObj && cellObj.attObj.type ) {	
                        var restrictType = attObj.restrictType;

                        if (attObj.rs && attObj.type.toLowerCase() != 'text') {
                            // only give feedback on the name of the restricted set
                            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutRestrictedSet') + ':</td><td>'+ WCXML.xmlencode(attObj.rs.name)+'</td></tr>';
                        } else if (attObj.type.toLowerCase() == 'text') {
                            var curListObj = attObj.listObj;  // can be null
                            htmlcode += '<tr class="darktext_greenlink_11" >';
                            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsRestrictedValues') + ":"; 
                            htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/concept/co_wc_CascadingDropdowns");
                            htmlcode += '</td>';
                            var changeMethod2 = this.functionPrefix + '.changeRestrict(this)';
                            htmlcode += '<td><table class="darktext_greenlink_11"  >';

                            htmlcode += '<tr><td>';
                            if (restrictType == 'RestrictedSet') {  // show just a label for a normal restricted set
                                htmlcode += ACLayoutStrings.t('ACLayout' + restrictType) +':';
                            } else {  // allow selecting
                                htmlcode += '<select id="restrictTypeSelector" onchange="'+changeMethod2+'" >';
                                var restrictTypeOptions = ['None','List','Attribute'];
                                for (var option=0; option<restrictTypeOptions.length; option++) {
                                    if (restrictTypeOptions[option] != 'List' || this.lists && this.lists.length>0) {
                                        htmlcode += '<option value="'+restrictTypeOptions[option]+'"';
                                        if (restrictType == restrictTypeOptions[option]) htmlcode += ' selected ';								
                                        htmlcode += '>'+ACLayoutStrings.t('ACLayout' + restrictTypeOptions[option]) + '</option>';
                                    }
                                }
                            }
                            htmlcode += '</td>';

                            // List selector						
                            if (restrictType == 'List') {	
                                htmlcode += '<td class="nobr">';

                                // list selector itself
                                var selectedList = -1;
                                htmlcode += '<select id="ListSelector" onchange="'+changeMethod2+'" >';  // no width: 100% since it does not work well in IE
                                htmlcode += '<option value="-1">-- '+ACatStrings.t('ACatPleaseSelect') + ' --</option>';

                                // first the ones on the database (these are typically the first but we don't count on this)
                                for (var list=0; list<this.lists.length; list++) {
                                    var listObj = this.lists[list]; 
                                    if (listObj.type != 'database') continue;
                                    var displayString = WCXML.xmlencode(listObj.name);
                                    if (listObj.counter) displayString += '<i> (' +  listObj.counter + ') </i>';  // shows copied lists (can only happen for DB lists)

                                    htmlcode += '<option value="'+list+'" ';
                                    if (listObj == curListObj) {htmlcode += ' selected '; selectedList = list};							
                                    htmlcode += '>'+ displayString + '</option>';
                                }

                                // then the ones on the web instance - mark as obsolete, but only when there are any
                                var htmlcode2 = '';
                                for (var list=0; list<this.lists.length; list++) {
                                    var listObj = this.lists[list]; 
                                    if (listObj.type == 'database') continue;							

                                    htmlcode2 += '<option value="'+list+'" ';
                                    if (listObj == curListObj) {htmlcode2 += ' selected '; selectedList = list};							
                                    htmlcode2 += '>'+  WCXML.xmlencode(listObj.name) + '</option>';
                                }
                                if (htmlcode2) {
                                    htmlcode += '<option disabled="disabled">&nbsp;</option>';
                                    htmlcode += '<option disabled="disabled"><font color="#d0d0d0">'+ ACLayoutStrings.t('ACLayoutExcelWebServer') +'</font></option>';

                                    htmlcode += htmlcode2;

                                }
                                htmlcode += '</select>';

                                // Link 'Copy List' only when DB lists
                                if (selectedList >=0  && this.lists[selectedList].type != 'Excel') {
                                    var copyListMethod = this.functionPrefix + '.copyList(' + selectedList + ');return false;';
                                    htmlcode += '&nbsp;<a href="#" onclick="'+copyListMethod+'" class="darktext_greenlink_11" >'+  ACLayoutStrings.t('ACLayoutCopyList')+'</a>';
									htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/task/ta_wc_CopyingLists");
                                }
                                htmlcode += '</td></tr>';

                            } else if (restrictType == 'Attribute') {					
                                // New in WCR12 Dec 2012 - first for Avery - author HADW												
                                htmlcode += '<td><select id="AttributeRestrictionSelector" onchange="'+changeMethod2+'" >';
                                htmlcode += '<option value="-1">-- '+ACatStrings.t('ACatPleaseSelect') + ' --</option>';
                                for (var att=0; att<this.attributes.length; att++) {
                                    var attObj2 = this.attributes[att];
                                    if (attObj2.placed < 1 || !attObj2.type || attObj2.type.toLowerCase() != 'text' || !attObj2.allowMultipleValues) continue;   // only allow multi-value text fields as restriction
                                    htmlcode += '<option value="' + att + '"';
                                    if (this.attributes[att] == attObj.restrictToAttObj) htmlcode += ' selected ';
                                    htmlcode += '>' + WCXML.xmlencode(this.attributes[att].name) + '</option>';
                                }
                                htmlcode += '</select></td>';

                            } else if (restrictType == 'RestrictedSet') {
                                htmlcode += '<td>'+ WCXML.xmlencode(attObj.rs.name)+'</td>';
                            }

                            // Column selector, only when a list is selected
                            if (restrictType == 'List' && curListObj && curListObj.columns) {
                                htmlcode += '<tr><td >'+ ACLayoutStrings.t('ACLayoutColumn') + ': </td>';
                                htmlcode += '<td class="nobr"><select id="ListColumnSelector"  onchange="'+changeMethod2+'" >';
                                for (var column=0; column<curListObj.columns.length; column++) {
                                    var colObj = curListObj.columns[column];
                                    htmlcode += '<option value="'+column+'" ';
                                    if (attObj.columnObj == colObj) htmlcode += ' selected ';
                                    htmlcode += '>'+ colObj.columnName + '</option>';
                                }	
                                htmlcode += '</select>';	
                                if (curListObj.type == 'database') {
                                    var reorderColumnMethod = this.functionPrefix + '.showListDialog(' + selectedList + '); return false;';
                                    htmlcode += '&nbsp;<a href="#" onclick="'+reorderColumnMethod+'" class="darktext_greenlink_11" >'+  ACLayoutStrings.t('ACLayoutReorderColumns')+'</a>';
                                	htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/task/ta_wc_ConfigureListsinAttributeCategory");   
                                }
                                htmlcode += '</td></tr>';	


                            }

                            // ability to use images to display
                            if ( restrictType && restrictType != 'None'  && restrictType != 'Attribute' ) {
                                htmlcode += '<tr><td>'+ ACLayoutStrings.t('ACLayoutRendering') + ': </td><td><select id="RenderingSelector" onchange="'+changeMethod2+'" >';
                                var renderOptions = ['Dropdown','RadioCheckboxes','Images'];
                                for (var opt=0; opt<renderOptions.length; opt++) {
                                    htmlcode += '<option value="'+renderOptions[opt]+'" ';
                                    if (cellObj.renderOption == renderOptions[opt]) htmlcode += ' selected ';
                                    htmlcode += '>'+  ACLayoutStrings.t('ACLayout' + renderOptions[opt]) + '</option>';
                                }
                                htmlcode += '</select>';
                                htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/concept/co_wc_Radiobuttons");
                                htmlcode += '</td>';
                                htmlcode += '</tr>';	
                                if (cellObj.renderOption && (cellObj.renderOption == 'RadioCheckboxes' || cellObj.renderOption == 'Images' )) {
                                    htmlcode += '<tr><td>'+ ACLayoutStrings.t('ACLayoutRenderColumns') + ': </td><td><input type="text" id="RenderColumns" onchange="'+changeMethod2+'" size="6" value="'+ cellObj.nImageColumns + '"></td></tr>';
                                    // image alignment options
                                    htmlcode += '<tr><td>'+ ACLayoutStrings.t('ACLayoutImageAlignment') + ': </td><td><select id="ImageAlignment" onchange="'+changeMethod2+'" >';
                                    var alignmentOptions = ['DistributeOverWidth','AlignLeft'];
                                    for (var option=0; option<alignmentOptions.length;option++) {
                                        htmlcode += '<option value="'+alignmentOptions[option]+ '"' ;								
                                        if (alignmentOptions[option] == cellObj.imageAlignment) htmlcode +=' selected ';
                                        htmlcode += '">'+ ACLayoutStrings.t('ACLayout' +alignmentOptions[option]) + '</option>';
                                    }
                                    htmlcode += '</select></td></tr>';
                                    // label location
                                    htmlcode += '<tr><td>'+ ACLayoutStrings.t('ACLayoutRestrictedValueLabelPosition') + ': </td><td><select id="RestrictedValueLabelPosition" onchange="'+changeMethod2+'" >';
                                    var labelOptions = ['LabelBelow','LabelRight'];
                                    for (var option=0; option<labelOptions.length;option++) {
                                        htmlcode += '<option value="'+labelOptions[option]+ '"' ;								
                                        if (labelOptions[option] == cellObj.restrictedValueLabelPosition) htmlcode +=' selected ';
                                        htmlcode += '">'+ ACLayoutStrings.t('ACLayout' +labelOptions[option]) + '</option>';
                                    }
                                    htmlcode += '</select></td></tr>';


                                    if (cellObj.renderOption == 'Images' ) {
                                        // Image height and cell height
                                        htmlcode += '<tr><td>'+ ACLayoutStrings.t('ACLayoutImageHeight') + ': </td><td><input id="ImageHeight" onchange="'+changeMethod2+'" size="6" value="'+ cellObj.imageHeight + '" >&nbsp; pix</td></tr>';	
                                        htmlcode += '<tr><td>'+ ACLayoutStrings.t('ACLayoutRenderHeight') + ': </td><td><input id="RenderHeight" onchange="'+changeMethod2+'" size="6" value="'+ cellObj.renderHeight + '" >&nbsp; pix</td></tr>';	
                                        htmlcode += '<tr><td>'+ ACLayoutStrings.t('ACLayoutImageType') + ': </td><td>'; 

                                        // Image type
                                        htmlcode += '<select id="RenderImageType" onchange="'+changeMethod2+'">';
                                        var typeOptions = ['jpg','JPG','jpeg','JPEG','gif','GIF','png','PNG','automatic'];
                                        for (var opt=0; opt<typeOptions.length; opt++) {
                                            htmlcode += '<option value="'+typeOptions[opt]+'" ';
                                            if (cellObj.renderImageType == typeOptions[opt]) htmlcode += ' selected ';
                                            htmlcode += '>'+ typeOptions[opt] + '</option>';
                                        }
                                        htmlcode += '</select></td></tr>';	
                                    }
                                }
                            }
                            htmlcode += '</table></td>';
                            htmlcode += '</tr>';				
                        }
                    }
                


                    // allow multiple values
                    var allowMultipleValues = false;
                    if (attObj && attObj.type && attObj.type.toLowerCase() == 'text' && (attObj.rs || attObj.listObj || attObj.restrictType == 'Attribute') && !attObj.isCheckBox ) allowMultipleValues = true;
                    // allow multiple values for Document References
                    if (attObj && attObj.isDocReference()) allowMultipleValues = true;
                    if (allowMultipleValues) {
                        htmlcode += '<tr class="darktext_greenlink_11" >';
                        htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsAllowMultiple') + ':</td>';
    //					changeMethod = this.functionPrefix + '.applyCellProperties(' + this.currentRowIndex + ',' + this.currentColIndex +')'; 

                        htmlcode += '<td><input type="checkbox" id="ChkBx_AllowMultipleValues" onclick = "'+changeMethod+'"';
                        if (attObj.allowMultipleValues === true) htmlcode += ' checked="checked" ';
                        if (this.currentView != 'Default') htmlcode += ' disabled="disabled" ';
                        htmlcode += '/></td></tr>';
                    }

					// Document reference rendering (for document reference attributes only)
					if (attObj && attObj.isDocReference()) {					
						htmlcode += this.displayDocRefDetails( row, col);					
					}
                    // imperial values
                    if (attObj && attObj.type && (attObj.type.toLowerCase() == 'distance' || attObj.type.toLowerCase() == 'distance1')) {
                        htmlcode += '<tr class="darktext_greenlink_11" >';
                        htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsImperial') + ':</td>';
    //					changeMethod = this.functionPrefix + '.applyCellProperties(' + this.currentRowIndex + ',' + this.currentColIndex +')'; 

                        htmlcode += '<td><input type="checkbox" id="ChkBx_Imperial" onclick = "'+changeMethod+'"';
                        if (attObj.fractionalIfImperial === true) htmlcode += ' checked="checked" ';
                        if (this.currentView != 'Default') htmlcode += ' disabled="disabled" ';
                        htmlcode += '/></td></tr>';
                    }

                    // Buttons
                    if (cellObj.buttons && cellObj.buttons.length>0) {

                        for (var button=0; button<cellObj.buttons.length; button++) {
                            var buttonObj = cellObj.buttons[button];
                            htmlcode += '<tr class="darktext_greenlink_11">';

                            // label column and remove link 
                            htmlcode += '<td valign="top" >' + ACLayoutStrings.t('ACLayoutButton') + ':' ;
                            var changeMethod2 =  this.functionPrefix + '.removeButton(' + button +'); return false;'; 
                            htmlcode += '<br><a href="#" onclick="' + changeMethod2 + '">' +  ACLayoutStrings.t('ACLayoutRemove')+ '</a>' ;
                            htmlcode +=  '</td>';

                            // values table
                            htmlcode += '<td valign="top" id="buttonDetails_'+button+'">' +  this.displayButtonDetails(cellObj,button) + '</td>';

                            htmlcode += '</tr>';
                        }
                    }


                    if(this.showInfoPage && cellObj && (cellObj.type == 'Title'|| cellObj.type == 'Attribute')) {
                        // info page
                        htmlcode += '<tr class="darktext_greenlink_11">';
                        htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsInfoPage') + ':</td>';

                        var changeMethod2 = this.functionPrefix + '.changeInfoPage(this)';
                        var value = "";
                        if (cellObj.infoPage) value = WCXML.xmlencode(cellObj.infoPage);                
                        htmlcode += '<td>';
                        htmlcode += ACLayoutStrings.t('ACLayoutInfoHtmlPage') + '<br>';
                        htmlcode += '<i>&lt;WebInstanceRoot&gt;</i>/custom/AttributesHelp/<br> <input type=text id="infoPage" style="width:99%; padding: 0px; margin: 0px" value="'+ value +'" onchange="' + changeMethod2 + '" placeholder=""/> ';					

                        htmlcode += '</td>';
                        htmlcode += '</tr>';
                    }
                }
            }
            htmlcode += '</table>';
            return htmlcode;

        } catch(err) {
            alert("Problem in attributeCategory.displayCellDetails: " + err + " Line: " + err.lineNumber + ': ' +  err.message );
            return '';
        }

    };

    /**
    * displays the row 'Document Source(s)' in cell details with links to edit/create/return to default
    * @author HADW - personal review 5 Feb 2015
    * 
    */
    attributeCategory.prototype.displayDocSourceOptions = function(cellObj, row, col) {
        "use strict";
        var htmlcode = '';	

        try {
            var attObj = cellObj.attObj;

            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDocumentSource') + ':</td>';


            htmlcode += '<td id="cellDetailsDocSources">';
            var dshObj = cellObj.dshObj;
            if (!dshObj) dshObj = this.createDocumentSource(this.dshConfigTablePHID, cellObj, null);  // safety. When placing or reading, you should always get a dshObj		
            htmlcode += WCXML.xmlencode(dshObj.name);

            var selectedIdx;  // index in docSourceGallery
            for (selectedIdx =0; selectedIdx< this.docSourcesGallery.length; selectedIdx++) {
                if (this.docSourcesGallery[selectedIdx] == dshObj) {
                    break;
                }
            }
            if (selectedIdx >= this.docSourcesGallery.length) {
                throw new Error('dshObj of cell on row ' + row + ' col ' + col + ' not found in the docSourcesGallery');
            }

            clickMethod = this.functionPrefix + '.showDocumentSource(' + selectedIdx + ', '+ row +', '+ col +'); return false;';
            htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/concept/co_wc_DocumentSourcesConfig");
            
            htmlcode += '<br><a href="#" onclick="' + clickMethod +'"><i class="icon icon-WCR_Actions-Edit12"></i> '+ ACLayoutStrings.t('ACLayoutEditDocumentSource') +'</a>';

            if (this.currentView != 'Default') {
                // Figure out whether this setup is for the current view
                if ( dshObj.name.substring(dshObj.name.length - this.currentView.length) == this.currentView ) {
                    // if you have a view specific configuration, allow returning to the default configuration
                    clickMethod = this.functionPrefix + '.deleteDocumentSource(' + selectedIdx + ', '+ row +', '+ col +'); return false;';
                    htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="' + clickMethod +'">'+ ACLayoutStrings.t('ACLayoutDeleteDocumentSource') +'</a>';
                } else {  
                    // if the current setup is the default setup, allow making a view specific one
                    clickMethod = this.functionPrefix + '.showDocumentSource(null, '+ row +', '+ col +'); return false;';
                    htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="' + clickMethod +'">'+ ACLayoutStrings.t('ACLayoutAddDocumentSource') +'</a>';	
                }				

            }

            
            htmlcode += '</td>';

            htmlcode += '</tr>'


        } catch(err) {
            alert("Problem in attributeCategory.displayDocSourceOptions: " + " Line: " + err.lineNumber + ':\n ' +  err.message );
            htmlcode = '';
        }
        return htmlcode;

    };

    /**
    * Removes a document source from the docSourcesGallery and reconnects the view to the default view
    * Cannot be done in the default view (the default view must always have a document source)
    * @author HADW - personal review 5 Feb 2015
    * @param {Integer} idx - index of the configuration to be removed from the gallery (index in the gallery)
    * @param {Integer} row,col - row and col of the current cell
    */
    attributeCategory.prototype.deleteDocumentSource = function(idx, row, col) {
        "use strict";

        try {
            if (this.currentView == 'Default' || !this.views) return;
            var cellObj = this.cells[row][col];
            var attObj = cellObj.attObj;
            var cellObjDefault;  // cell in the default view pointing to the same attribute as the current one in the current view

            // first check whether the input is right
            if (this.docSourcesGallery[idx] != this.cells[row][col].dshObj) throw new Error('document source on row ' + row + ' col ' + col + ' does not fit with index in docSourcesGallery ' + idx   );

            // first link current view back to default view's configuration for this attribute
            if (cellObj && attObj) {

                cellObjDefault = this.findDefaultCellObj(cellObj);  // find in the default view the cell with this attribute. 

                if (cellObjDefault) {
                    cellObj.dshObj = cellObjDefault.dshObj;
                    this.createDocTableX(cellObj);  // when changing the dshObj you must create a new docTableX. Existing one will be wiped but you are in the admin page so you don't lose anything
                }

                this.docSourcesGallery.splice(idx,1); 

                // repaint the cell details because you need other links
                var cellDetailsTable = document.getElementById("celldetails");		
                if (cellDetailsTable) cellDetailsTable.innerHTML = this.displayCellDetails();    

                // repaint the preview to show the new docTableX
                this.displayAttributeCategoryPreview();	
            }

        } catch(err) {
            alert("Problem in attributeCategory.deleteDocumentSource: " + err + " Line: " + err.lineNumber + ': ' +  err.message );

        }


    };

    /**
    * @return cellObjDefault - cell in the default view pointing to the same attribute as the current one in cellObj (in current non-default view)
    */

    attributeCategory.prototype.findDefaultCellObj = function(cellObj) {

        "use strict";
        var cellObjDefault;  // cell in the default view pointing to the same attribute as the current one in the current view
        try {
            if (this.currentView == 'Default' || !this.views) return null;
            var attObj = cellObj.attObj;

            if (cellObj && attObj) {
                // find in the default view the cell with this attribute. 
                var defViewObj = this.views[0];
                for (var row=0; row< defViewObj.cells.length; row++) {
                    if (!defViewObj.cells[row]) continue;
                    for (var col=0; col < defViewObj.cells[row].length; col++) {
                        if (defViewObj.cells[row][col] && defViewObj.cells[row][col].attObj == attObj) {
                            cellObjDefault = defViewObj.cells[row][col];
                            break;
                        }
                    }
                }
            }

        } catch(err) {
            alert("Problem in attributeCategory.findDefaultCellObj: " + err + " Line: " + err.lineNumber + ': ' +  err.message );

        }
        return cellObjDefault;


    }

	/**
	* creates HTML to display specific rows for document references in the cell details pane
	* @author HADW - personal review 23 March 2015
	* @param {int} row, col - row and col of the selected cell
	* @return {String} htmlcode
	*/
    attributeCategory.prototype.displayDocRefDetails = function( row, col) {
        "use strict";
        var htmlcode = '';
		

        try {
			var cellObj = this.cells[row][col];
			var attObj = cellObj.attObj;
            var changeMethod = this.functionPrefix + '.changeRestrict(this)';

            // We have de-speced the BOM at a late stage in the development of 14.1,
            // mainly because of lack of time. We think it will be put back in soon, so 
            // we do not take out the code, but only disable it.
            // The code is reviewed and tested.
            // When put back, test for this.BOMAllowed
            if (false) {
                // Bill of Materials read-only feedback
                htmlcode += '<tr class="darktext_greenlink_11">';
                htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutBOM') + ':</td>';
                htmlcode += '<td id="cellDetailsBOMFeedback">';
                // include or exclude from BOM
                if (attObj.BOMInclude == '1') {
                    htmlcode += WCXML.xmlencode(ACLayoutStrings.t('ACLayoutIncludeInBOMDefault'));
                } else {
                    htmlcode += WCXML.xmlencode(ACLayoutStrings.t('ACLayoutExcludeBOMDefault'));
                }
                htmlcode += '<br/>';
                // editable by user or not
                if (attObj.BOMEditable == '1') {
                    htmlcode += WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditableByUser'));
                } else {
                    htmlcode += WCXML.xmlencode(ACLayoutStrings.t('ACLayoutNotEditableByUser'));
                }
                htmlcode += '<br/>';
                // show count or not
                if (attObj.BOMCount == '1') {
                    htmlcode += WCXML.xmlencode(ACLayoutStrings.t('ACLayoutShowCount'));
                } else {
                    htmlcode += WCXML.xmlencode(ACLayoutStrings.t('ACLayoutNotShowCount'));
                }
                htmlcode += '</td></tr>';
            }

            // REFERENCE CATEGORY
            // Reference Category - label
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutReferenceCategory')) + ':</td>';

            // Reference Category - display value read only
            htmlcode += '<td id="cellDetailsReferenceCategory">';
            if (attObj.referenceCategoryName!= null && attObj.referenceCategoryName.length >0){
                htmlcode += WCXML.xmlencode(attObj.referenceCategoryName);            	
            } else{
            	htmlcode += WCXML.xmlencode(ACLayoutStrings.t('ACLayoutNoReferenceCategory'))  ; 
            }
            
            htmlcode += '</td></tr>';

            // DOC. REFERENCE MODE
            // label
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutReferenceMode')) + ':</td>';

            // reference mode - read-only
            var docRefModeString;
			switch (attObj.docRefMode) {
			case 0:
                docRefModeString = ACLayoutStrings.t('ATT_LATEST_VERSION'); break;
            case 1:            
                docRefModeString = ACLayoutStrings.t('ATT_SELECTED_VERSION'); break;            
            case 2:
                docRefModeString = ACLayoutStrings.t('ATT_SELECTED_REVISION'); break;
			default:
				docRefModeString = 'Unknown Document Reference Mode';
            }
            htmlcode += '<td id="cellDetailsDocRefMode">' + WCXML.xmlencode(docRefModeString) + '</td>';
            htmlcode += '</tr>';

			htmlcode += this.displayDocSourceOptions(cellObj, row, col);
			
			// Rendering
			htmlcode += '<tr class="darktext_greenlink_11">';
			htmlcode += '<td>' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutRendering')) + ':</td>';
				
			// Rendering - dropdown with rendering options
			htmlcode += '<td><table><tr class="darktext_greenlink_11">';
			htmlcode += '<td><select id="RenderingSelector" onchange="'+changeMethod+'" >';
			var docRefRenderOptions = ['List','Table'];
			for (var opt = 0; opt < docRefRenderOptions.length; opt++) {
				htmlcode += '<option value="'+docRefRenderOptions[opt]+'" ';
				if (cellObj.renderOption == docRefRenderOptions[opt]) htmlcode += ' selected ';
				htmlcode += '>'+  WCXML.xmlencode(ACLayoutStrings.t('ACLayoutRender' + docRefRenderOptions[opt])) + '</option>';
			}
			htmlcode += '</select>';
			htmlcode += HelpIcon.getInlineHelpTopicHTML("common/wc/task/ta_wc_CreateanAdvancedTableRendering");
			
			htmlcode += '</td></tr>';
			

			if (cellObj.renderOption == 'Table') {						
				if (!cellObj.docLinkTableOptions) {  // fall-back only - should not happen
					this.createDefaultDocReferenceTable(cellObj, true);
				}
				
				var docLinkTableOptionsName = cellObj.docLinkTableOptions.getName();
				var clickMethod = this.functionPrefix + '.showDocumentRenderDialog('  + row +', '+ col +'); return false;';
				htmlcode += '<tr><td>' + WCXML.xmlencode(docLinkTableOptionsName) + '</td></tr>';
				htmlcode += '<tr><td id="cellDetailsDocRenderConfig"><a href="#" onclick="' + clickMethod +'" class="nobr" ><i class="icon icon-WCR_Actions-Edit12"></i> '+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditRenderConfiguration')) +'</a>';
				
				if (this.currentView != 'Default') {
					
					// Figure out whether this setup is for the current view
					if (docLinkTableOptionsName.substring(docLinkTableOptionsName.length - this.currentView.length) == this.currentView ) {
						// if you have a view specific configuration, allow returning to the default configuration
						clickMethod = this.functionPrefix + '.deleteDocumentRenderConfig(' + row +', '+ col +'); return false;';
						htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="' + clickMethod +'" class="nobr">'+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutDeleteRenderConfiguration')) +'</a>';
					} else {  
						// if the current setup is the default setup, allow making a view specific one
						clickMethod = this.functionPrefix + '.showDocumentRenderDialog('+ row +', '+ col +',true); return false;';
						htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="' + clickMethod +'" class="nobr">'+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutAddRenderConfiguration')) +'</a>';	
					}		
				}
				htmlcode += '</td></tr>';
			}
				
			htmlcode +=	'</table>';
			htmlcode += '</td></tr>';
        } catch(err) {
            alert('Problem in attributeCategory.displayDocRefDetails ' +  (err.lineNumber? ('at Line: ' + err.lineNumber) : '') + ':\n ' +  err.message );

        }
        return htmlcode;
    }

    /**
    * Displays the details for a button in a cell defined by cellObj
    * @param cellObj {Object} cell in which the button resides
    * @param button {Integer} index of the button in the array of buttons for this cell
    * @return {String} htmlcode for the button details
    */
    attributeCategory.prototype.displayButtonDetails = function(cellObj,button) {
        "use strict";
        var htmlcode = '';

        try {

            var changeMethod = this.functionPrefix + '.changeButton(this,' + this.currentRowIndex + ',' + this.currentColIndex +'); return false;'; 
            var buttonObj = cellObj.buttons[button];
            var list,att;

            htmlcode += '<table>';

            //label
            htmlcode += '<tr class="darktext_greenlink_11"><td>' + ACLayoutStrings.t('ACLayoutLabel') + '</td>';
            htmlcode += '<td><input type="text" value="'+buttonObj.label+'" onchange="'+changeMethod+'" id="ButtonLabel_'+button+'"></td></tr>';

            //style
            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutStyle') + '</td>';
            var options = ['buttonprimary','buttonsecondary','plainbutton','greenlink','hidden'];
            htmlcode += '<td>'+this.createSelectBox('ButtonStyle_'+button, changeMethod, buttonObj.style, options, null, null)+'</td></tr>';

            //when to execute		
            htmlcode += this.displayButtonExecuteDetails(buttonObj, button, changeMethod);			

            // function
            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutFunction') + '</td>';
            var options = this.buttonFuncOptions;
            htmlcode += '<td>'+this.createSelectBox('ButtonFunction_'+button, changeMethod, buttonObj.func, options, null, null)+'</td></tr>';

            // list
            if ( buttonObj.func == 'LookUpAttributes' ) {
                htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutList') + '</td>';
                htmlcode += '<td><select id="ButtonList_'+button+ '" onchange="'+changeMethod+'" >';  
                htmlcode += '<option value="-1">-- '+ACatStrings.t('ACatPleaseSelect') + ' --</option>';
                if(this.lists){
                    for (list=0; list<this.lists.length; list++) {
                        var listObj = this.lists[list]; 
                        if (listObj.type != 'database') continue;
                        htmlcode += '<option value="'+list+'" ';
                        if (listObj == buttonObj.listObj) htmlcode += ' selected ';
                        htmlcode += '>'+listObj.name + '</option>';
                    }
                }
                htmlcode += '</select></td></tr>'; 


                if (buttonObj.listObj) {
                    var listObj = buttonObj.listObj;

                    // input attributes
                    htmlcode += '<tr class="darktext_greenlink_11"><td colspan="3" style="padding:10 3 3 3"><table>';

                    if (buttonObj.inputAtts && buttonObj.inputAtts.length >0) {
                        // header row
                        htmlcode += '<tr class="darktext_greenlink_11"><td/>';
                        htmlcode += '<td>'+ ACLayoutStrings.t('ACLayoutInputAttribute') + '</td>';
                        htmlcode += '<td>'+ ACLayoutStrings.t('ACLayoutColumn') + '</td>';
                        htmlcode += '<td style="text-align:center">'+ ACLayoutStrings.t('ACLayoutExact') + '</td>';
                        htmlcode += '</tr>';

                        for (att=0; att<buttonObj.inputAtts.length; att++) {										
                            htmlcode += '<tr><td><img id="ButtonRemoveInputAttribute_'+button+'_' + att +'" src="./images/delete_icon_15.png" onclick="'+changeMethod+'"></td>';

                            // attribute selector
                            htmlcode += '<td><select onchange="'+changeMethod+'" id="ButtonInput_'+button+'_'+att + '">';
                            htmlcode += '<option value="-1">'+ ACatStrings.t('ACatPleaseSelect')+'</option>';
                            for (var att2=0; att2<this.attributes.length; att2++) {
                                if (this.attributes[att2].placed < 1) continue;
                                htmlcode += '<option value="' + att2 + '"';
                                if (buttonObj.inputAtts[att].attObj == this.attributes[att2]) htmlcode += ' selected ';
                                htmlcode += '>' + WCXML.xmlencode(this.attributes[att2].name) + '</option>';
                            }
                            htmlcode += '</select></td>';

                            // column selector
                            htmlcode += '<td><select onchange="'+changeMethod+'" id="ButtonInputColumn_'+button+'_'+att + '">';
                            htmlcode += '<option value="-1">'+ ACatStrings.t('ACatPleaseSelect')+'</option>';
                            for (var column=0; column < listObj.columns.length; column++) {
                                htmlcode += '<option value="' + column + '"';
                                if (buttonObj.inputAtts[att].columnObj == listObj.columns[column]) htmlcode += ' selected ';
                                htmlcode += '>' + WCXML.xmlencode(listObj.columns[column].columnName) + '</option>';
                            }
                            htmlcode += '</select></td>';

                            // exact match
                            htmlcode += '<td style="text-align:center"><input type="checkbox" onchange="'+changeMethod+'" id="ButtonInputExact_'+button+'_'+att + '"';
                            if (buttonObj.inputAtts[att].exact) htmlcode += ' checked ';
                            htmlcode += '>';
                            htmlcode += '</td>';

                            htmlcode += '</tr>';
                        }
                    }
                    // link Add Input attribute
                    htmlcode += '<tr class="darktext_greenlink_11"><td colspan="2"><a id="ButtonAddInputAttribute_'+button+'" href="#" onclick="'+changeMethod+'">'+ ACLayoutStrings.t('ACLayoutAddInputAttribute') + '</a></td></tr>';
                    // end of table
                    htmlcode += '</table></td></tr>';


                    //output attributes
                    htmlcode += '<tr class="darktext_greenlink_11"><td colspan="3" style="padding:10 3 3 3"><table>';

                    if (buttonObj.outputAtts && buttonObj.outputAtts.length >0) {
                        // header row
                        htmlcode += '<tr class="darktext_greenlink_11"><td/>';
                        htmlcode += '<td>'+ ACLayoutStrings.t('ACLayoutOutputAttribute') + '</td>';
                        htmlcode += '<td>'+ ACLayoutStrings.t('ACLayoutColumn') + '</td>';				
                        htmlcode += '</tr>';

                        for (att=0; att<buttonObj.outputAtts.length; att++) {							
                            htmlcode += '<tr><td><img id="ButtonRemoveOutputAttribute_'+button+'_' + att +'" src="./images/delete_icon_15.png" onclick="'+changeMethod+'"></td>';

                            htmlcode += '<td><select onchange="'+changeMethod+'" id="ButtonOutput_'+button+'_'+att + '">';
                            htmlcode += '<option value="-1">'+ ACatStrings.t('ACatPleaseSelect')+'</option>';
                            for (var att2=0; att2<this.attributes.length; att2++) {
                                if (this.attributes[att2].placed < 1) continue;
                                htmlcode += '<option value="' + att2 + '"';
                                if (buttonObj.outputAtts[att].attObj == this.attributes[att2]) htmlcode += ' selected ';
                                htmlcode += '>' + WCXML.xmlencode(this.attributes[att2].name) + '</option>';
                            }
                            htmlcode += '</select></td>';

                            // column selector
                            htmlcode += '<td><select onchange="'+changeMethod+'" id="ButtonOutputColumn_'+button+'_'+att + '">';
                            htmlcode += '<option value="-1">'+ ACatStrings.t('ACatPleaseSelect')+'</option>';
                            for (var column=0; column < listObj.columns.length; column++) {
                                htmlcode += '<option value="' + column + '"';
                                if (buttonObj.outputAtts[att].columnObj == listObj.columns[column]) htmlcode += ' selected ';
                                htmlcode += '>' + WCXML.xmlencode(listObj.columns[column].columnName) + '</option>';
                            }
                            htmlcode += '</select></td>';
                            htmlcode += '</tr>';

                        }

                    }
                    htmlcode += '<tr class="darktext_greenlink_11"><td colspan="2"><a id="ButtonAddOutputAttribute_'+button+'" href="#" onclick="'+changeMethod+'">'+ ACLayoutStrings.t('ACLayoutAddOutputAttribute') + '</a></td></tr>';
                    // end of table
                    htmlcode += '</table></td></tr>';
                }
            }
            if (buttonObj.func == 'JavaScript') {
                htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutJSCode') + '</td>';
                htmlcode += '<td><textarea rows="6" cols="60" onchange="'+changeMethod+'" id="ButtonJavaScript_'+button+ '">' + WCXML.xmlencode(buttonObj.javaScript) + '</textarea></td>';  
            }


            htmlcode += '</table>';		


        } catch(err) {
            alert("Problem in attributeCategory.displayButtonDetails: " + err + " Line: " + err.lineNumber + ': ' +  err.message );

        }
        return htmlcode;
    }

    /**
    * returns HTML code for the button execution details (onLoad,onSave, onChange checkboxes and attribute selector)
    * @param {Obj} buttonObj - attObj.buttons[button]
    * @param {Number} button - index of the button in attObj.buttons
    * @param {String} changeMethod - changmethod string to be put in the generated HTML code
    * @return {String} htmlcode
    */
    attributeCategory.prototype.displayButtonExecuteDetails = function(buttonObj, button, changeMethod) {
        "use strict";
        var htmlcode = '';
        try {

            htmlcode += '<tr class="darktext_greenlink_11"><td style="vertical-align:top;">'+ ACLayoutStrings.t('ACLayoutExecute') + '</td><td>';		

            htmlcode += '<table id="buttonExecuteTable">';
                // on Load 
            htmlcode += '<tr><td><input type="checkbox" id="ButtonExecOnLoad_'+button+'" onchange="'+changeMethod+'"';
            if (buttonObj.execOnLoad) htmlcode += ' checked ';
            htmlcode += '></td><td>'+ ACLayoutStrings.t('ACLayoutOnLoad') +'</td></tr>';

                // on Save
            htmlcode += '<tr><td><input type="checkbox" id="ButtonExecOnSave_'+button+'" onchange="'+changeMethod+'"';
            if (buttonObj.execOnSave) htmlcode += ' checked ';
            htmlcode += '></td><td>'+ ACLayoutStrings.t('ACLayoutOnSave') +'</td></tr>';

                // on Change of Attribute		
            htmlcode += '<tr><td><input type="checkbox" id="ButtonExecOnChange_'+button+'" onchange="'+changeMethod+'"';
            if (buttonObj.execOnChange) htmlcode += ' checked ';
            htmlcode += '></td><td>'+ ACLayoutStrings.t('ACLayoutOnChangeOfAttribute') +'&nbsp;&nbsp;</td>';

            if (buttonObj.execOnChange) {
                for (var att=0; att<buttonObj.changeAtts.length; att++) {	
                    if (att > 0) htmlcode += '</tr><tr><td/><td/>';
                    htmlcode += '<td><img id="ButtonRemoveChangeAttribute_'+button+'_' + att +'" src="./images/delete_icon_15.png" onclick="'+changeMethod+'"></td>';

                    htmlcode += '<td><select onchange="'+changeMethod+'" id="ButtonChangeAttribute_'+button+'_'+att + '">';
                    htmlcode += '<option value="-1">'+ ACatStrings.t('ACatPleaseSelect')+'</option>';
                    for (var att2=0; att2<this.attributes.length; att2++) {
                        if (this.attributes[att2].placed < 1) continue;
                        htmlcode += '<option value="' + att2 + '"';
                        if (buttonObj.changeAtts[att].attObj == this.attributes[att2]) htmlcode += ' selected ';
                        htmlcode += '>' + WCXML.xmlencode(this.attributes[att2].name) + '</option>';
                    }
                    htmlcode += '</select></td>';

                }
                if (buttonObj.changeAtts.length > 0) htmlcode += '</tr><tr><td/><td/>';
                htmlcode += '<td colspan="2"><a id="ButtonAddChangeAttribute_'+button+'" href="#" onclick="'+changeMethod+'">'+ ACLayoutStrings.t('ACLayoutAddChangeAttribute') + '</a></td>';  // green link 'Add Attribute'
            }

            htmlcode += '</tr></table>';
            htmlcode += '</td></tr>';


        } catch(err) {
            alert("Problem in attributeCategory.displayButtonDetails: " + err + " Line: " + err.lineNumber + ': ' +  err.message );

        }
        return htmlcode;
    }

    /**
    * Makes a copy of a list to use the list multiple times. To be done only for back-end lists
    * a listObj can now have an extra property 'counter'. Only 1 listObj with the same name can have no counter, all others must have unique counters
    * @param {Int} listIdx - index of the source list to be copied in the lists array (this.lists)
    */
    attributeCategory.prototype.copyList = function(listIdx) {
        "use strict";
        try {
            var listObj = this.lists[listIdx];
            if (!listObj) throw new Error("no list has index " + listIdx);
            if (listObj.type == 'Excel') throw new Error("cannot copy a web-server based list. Please copy the list in lists.xml instead");
            var listObj2 = {};

            this.setACatChanged(); 

            listObj2.hierarchical = listObj.hierarchical;
            listObj2.invalidateAutomatically = listObj.invalidateAutomatically;
            listObj2.autofillExternalValue = listObj.autofillExternalValue;
            listObj2.loadingType = listObj.loadingType;
            listObj2.name = listObj.name;
            listObj2.type = listObj.type;
            listObj2.uniqueIdentifyerColumn = listObj.uniqueIdentifyerColumn;


            /* find copy index */
            var counter = 2;
            var insertPosition = -1; // where to put the list in the list of lists
            for (var list=0; list<this.lists.length; list++) {
                var listObj3 = this.lists[list];
                if (listObj3.name == listObj2.name) {
                    if (insertPosition == -1) insertPosition = list;
                    if (listObj3.counter && listObj3.counter >= counter) {
                        counter = listObj3.counter + 1;
                        insertPosition = list;
                    }
                }
            }
            listObj2.counter = counter;

            // copy columns
            listObj2.columns = [];
            for (var column=0; column < listObj.columns.length; column++) {
                var columnObj = listObj.columns[column];
                var columnObj2 = {};
                columnObj2.columnName = columnObj.columnName;
                columnObj2.currentIndex = -1; 
                columnObj2.maxValuesInDropdown = columnObj.maxValuesInDropdown; 
                columnObj2.uniqueIdentifyer = columnObj.uniqueIdentifyer; 
                columnObj2.values = columnObj.values; //share the values but not validArr
                columnObj2.validArr = null;			
                listObj2.columns.push(columnObj2);
            }

            listObj2.combinations = listObj.combinations;  // share combinations (note HADW: combinations are still to be developed for back-end lists

            // Buttons are NOT copied
            listObj2.buttons = [];

            // Now insert the listObj in the list of lists
            this.lists.splice(insertPosition+1,0,listObj2);

            // and assign this list to the currently selected item
            var listSelectorElem = document.getElementById('ListSelector');
            
			// insert option in the select, otherwise, we might have an invalid selection
            var current = $(listSelectorElem).find("option")[insertPosition];
            if(current){
            	$(current).after('<option value="'+(insertPosition+1)+'">'+listObj2.name+'</option>');
            	listSelectorElem.value = insertPosition+1;
            }
            
            this.changeRestrict(listSelectorElem);
        } catch(err) {
            alert("Problem in attributeCategory.copyList: " + err + " Line: " + err.lineNumber + ': ' +  err.message );
        }
    };

	/**
	* Finds an entry in docRenderGallery based on its name
	* @param {String} name - name of the config to be found
	* @return idx of the entry in the docRenderGallery or -1 if not found
	*/
	attributeCategory.prototype.findInDocRenderGallery = function(name) {
		'use strict';
		
		try {
			for (var idx=0; idx < this.docRenderGallery.length; idx++) {
				if (this.docRenderGallery[idx].getName() == name) break;
			}
			if (idx < this.docRenderGallery.length) {
				return idx;
			} else {
				return -1;
			}
		} catch(err) {
			alert('Problem in findInDocRenderGallery at line ' + err.lineNumber + ' : ' + err.message);
			return -1;
		}
	};

	/**
	* Creates a default table configuration for a document reference or picks up an existing one if it already exists in the gallery
	* To be called each time a new document reference is set to table rendering
	*
	* @param {Boolean} reUse - if true, a gallery item with the same name will be picked up, otherwise that same item will be reset
	*/
	attributeCategory.prototype.createDefaultDocReferenceTable = function(cellObj, reUse) {
		'use strict';
		try {
			var attObj = cellObj.attObj;
			var name = attObj.name + '_' + this.currentView;
			var idx = this.findInDocRenderGallery(name);
			
			var docLinkTableOptions = null;
			if (idx >= 0) {
				if (reUse) {
					// already exists in the gallery so you don't need to create an entry
					docLinkTableOptions = this.docRenderGallery[idx];
					cellObj.docLinkTableOptions = docLinkTableOptions;
				} else {
					this.docRenderGallery.splice(idx, 1);  // throw it away
				}
			} 
		
			if (idx < 0 || !reUse) {
				var docRefProperties = null;
				if (attObj.referencePropertyACat) {
					docRefProperties = attObj.referencePropertyACat.attributes;
				}

				// create the default rendering configuration for document reference attributes
				docLinkTableOptions = new DocTableXRenderConfig({
					defaultName : name,
					section : 'AttributeCategory',
					phElemID : 'DocRenderConfigDivID',
					configObj : null,   // default configuration will be generated
					attributes : this.attributes,   // this is all attributes in the system
					properties : docRefProperties,   // this is just attributes in reference attribute category
					changeCallback : this.setACatChanged.bind(this)
				});
				
				cellObj.docLinkTableOptions = docLinkTableOptions;
				this.docRenderGallery.push(docLinkTableOptions);
			}
			this.setACatChanged();	
		} catch(err) {
			alert('Problem in createDefaultDocReferenceTable at line ' + err.lineNumber + ' : ' + err.message);
		}
	};

	/**
	* For document references only
	* Move current view (not the default view) back to the render configuration of the default view
	* @author HADW - personal review 23 March 2015
	* @param {int} row,col - row and column of the currently selected cell
	*/
	attributeCategory.prototype.deleteDocumentRenderConfig = function( row, col) {
		'use strict';
		try {
			if (this.currentView == 'Default') throw new Error('deleteDocumentRenderConfig called from default view');  
			var cellObj = this.cells[row][col];		
			var attObj = cellObj.attObj;

			// Link current view back to default view's configuration for this attribute
			if (cellObj && attObj) {
			
				var cellObjDefault = this.findDefaultCellObj(cellObj);  // find in the default view the cell with the same attribute. 
				
				if (cellObjDefault) {
					cellObj.docLinkTableOptions = cellObjDefault.docLinkTableOptions;	
					cellObj.renderOption = cellObjDefault.renderOption;  // if you take over the config of default view, you must also take over the render option
				}
				
				// repaint the cell details because you need other links
				var cellDetailsTable = document.getElementById("celldetails");		
				if (cellDetailsTable) cellDetailsTable.innerHTML = this.displayCellDetails();    

				// repaint the preview to show the new docTableX
				this.displayAttributeCategoryPreview();	
			}
			
			
		} catch(err) {
			alert('Problem in deleteDocumentRenderConfig at line ' + err.lineNumber + ' : ' + err.message);
		}
	};
	
    /**
    * Handles row operations on the document sources table 
    * @param elem the element on which a change was done (all elements in the button detail call this function)
    * @param idx: index of the row on which the operation is done (null for operation Add)
    * @param operation: one of 'Add','Remove','Up','Down'
    */
    attributeCategory.prototype.changeButton = function(elem, row, col) {
        "use strict";
        try {		
            // find current cell
            var cellObj = this.cells[row][col];	
            if (!cellObj || !cellObj.buttons || cellObj.buttons.length<1) throw new Error ("changeButton called on an element without a button");

            var 	id = elem.id,
                    idArr = id.split('_'),
                    detail = idArr[0],  // which field
                    button = parseInt(idArr[1],10),  // which button
                    buttonObj = cellObj.buttons[button],
                    listObj = buttonObj.listObj,   // list assigned to this button. Can be null
                    idx = parseInt(elem.value,10),  // idx in a select box if the element is a select box, else mostly NaN
                    att,   // attribute index in a table of input or output attributes				
                    repaintNeeded = true;  // whether the button detail block must be repainted

            if (idArr.length > 2) att = parseInt(idArr[2],10);

            switch (detail) {

                case 'ButtonLabel':
                    buttonObj.label = elem.value;	
                    repaintNeeded = false;				
                    break;

                case 'ButtonStyle':
                    buttonObj.style = elem.value;
                    repaintNeeded = false;	
                    break;

                case 'ButtonFunction':
                    // first check whether this button choice is allowed
                    var valid = true;
                    if (elem.value == 'CheckUniqueValue') {
                        if (!cellObj.attObj) {alert(ACLayoutStrings.t('ACLayoutButtonOnlyAllowedOnAttributes')); valid= false;};
                        if (cellObj.attObj && cellObj.attObj.type && cellObj.attObj.type.toLowerCase() == 'datetime') { alert(ACLayoutStrings.t('ACLayoutCheckUniqueNoDates')); ; valid= false;};
                        if (cellObj.attObj && cellObj.attObj.type && cellObj.attObj.type.toLowerCase() == 'richtext') { alert(ACLayoutStrings.t('ACLayoutCheckUniqueNoRichText')); ; valid= false;};
                        if (cellObj.attObj && cellObj.attObj.allowMultipleValues) { alert(ACLayoutStrings.t('ACLayoutCheckUniqueNoMultipleValues')); ; valid= false;};
                        // value will set back automatically through the repaint. 
                    }
                    if (valid) {
                    buttonObj.func = elem.value;				
                    // when changing function, the datails must always be cleared
                    buttonObj.JavaScript = '';
                    buttonObj.listObj = null;
                    buttonObj.inputAtts = [];
                    buttonObj.outputAtts = [];
                    }
                    break;

                case 'ButtonList':
                    var newListIdx = parseInt(elem.value,10);
                    var newListObj;
                    if (newListIdx >=0) newListObj = this.lists[ parseInt(elem.value,10) ];  // else you have deselected the list and newListObj is null
                    var oldListObj = buttonObj.listObj;   // oldListObj may be null if nothing was selected before you did this selection
                    if (oldListObj && oldListObj != newListObj) {
                        // take buttonObj away from old listObj
                        for (var b=0; b< oldListObj.buttons; b++) {
                            if (oldListObj.buttons[b] == buttonObj) oldListObj.buttons.splice(b,1); break;
                        }
                    }
                    buttonObj.listObj = newListObj; 
                    // if input and output attributes already existed, get rid of them (links are wrong anyway)
                    buttonObj.inputAtts = [];
                    buttonObj.outputAtts = [];

                    // update listObj.buttons
                    if (newListObj) {
                        if (!newListObj.buttons) newListObj.buttons = [];
                        newListObj.buttons.push(buttonObj);		
                    }

                    break;

                case 'ButtonInput':
                case 'ButtonOutput':										
                    var attObj = this.attributes[idx];
                    var attObj2 = (detail == 'ButtonInput')? buttonObj.inputAtts[att] : buttonObj.outputAtts[att];
                    if (attObj) {					
                        attObj2.attObj = attObj; 

                        // try finding same name in the columns
                        for (var column=0; column < listObj.columns.length; column++) {
                            if (listObj.columns[column].columnName == attObj.name) {
                                attObj2.columnObj = listObj.columns[column]; break;
                            }
                        }
                    } else {
                        throw new Error('attribute index out of bounds:' + idx);
                    }
                    break;

                case 'ButtonInputColumn':
                case 'ButtonOutputColumn':
                    var attObj2 = (detail == 'ButtonInputColumn')? buttonObj.inputAtts[att] : buttonObj.outputAtts[att];
                    if (listObj && idx < listObj.columns.length) {
                        attObj2.columnObj = listObj.columns[idx];
                    } else {
                        throw new Error('column index out of bounds:' + idx);
                    }
                    repaintNeeded = false;	
                    break;

                case 'ButtonInputExact':
                    buttonObj.inputAtts[att].exact = elem.checked;
                    repaintNeeded = false;	
                    break;

                case 'ButtonAddInputAttribute':
                    buttonObj.inputAtts.push({attObj:null, columnObj: null, exact:true});
                    break;

                case 'ButtonRemoveInputAttribute':
                    buttonObj.inputAtts.splice(att,1);
                    break;

                case 'ButtonAddOutputAttribute':
                    buttonObj.outputAtts.push({attObj:null, columnObj: null});
                    break;

                case 'ButtonRemoveOutputAttribute':
                    buttonObj.outputAtts.splice(att,1);
                    break;

                case 'ButtonJavaScript':
                    buttonObj.javaScript = elem.value;
                    break;


                case 'ButtonExecOnLoad':
                    buttonObj.execOnLoad = elem.checked;
                    repaintNeeded = false;	
                    break;

                case 'ButtonExecOnSave':
                    buttonObj.execOnSave =  elem.checked;
                    repaintNeeded = false;	
                    break;

                case 'ButtonExecOnChange':
                    buttonObj.execOnChange =  elem.checked;
                    break;

                case 'ButtonAddChangeAttribute':
                    buttonObj.changeAtts.push({attObj:null});
                    break;

                case 'ButtonChangeAttribute':										
                    var attObj = this.attributes[idx];  // the referenced attribute
                    if (attObj) {					
                        buttonObj.changeAtts[att].attObj = attObj; 	
                        if (!attObj.buttonDependency) attObj.buttonDependency = [];
                        attObj.buttonDependency.push(buttonObj);					
                    } else {
                        throw new Error('attribute index out of bounds:' + idx);
                    }
                    repaintNeeded = false;	
                    break;

                case 'ButtonRemoveChangeAttribute':
                    // first take out button dependency
                    this.removeAttDependency(buttonObj, att);

                    break;

                default:
                    throw new Error ('unknown detail ' + detail);
                    break;

            }

            this.setACatChanged();	
            // repaint the button's details		
            if (repaintNeeded) {
                var repaintElem = document.getElementById("buttonDetails_" + button);	
                var htmlcode = this.displayButtonDetails(cellObj, button);    			
                if (repaintElem) repaintElem.innerHTML = htmlcode;   			
            }

            // repaint the preview
            this.displayAttributeCategoryPreview();	

        } catch(err) {
            alert('Problem in changeButton at line ' + err.lineNumber + ' : ' + err.message);
        }
    };

    attributeCategory.prototype.removeAttDependency = function(buttonObj, att) {
        try {
            var attObj = buttonObj.changeAtts[att].attObj;
            if (attObj && attObj.buttonDependency) {
                for (var dep=0; dep< attObj.buttonDependency.length; dep++) {
                    if (attObj.buttonDependency[dep] == buttonObj) {
                        attObj.buttonDependency.splice(dep,1);
                        break;
                    }
                }
            }
            buttonObj.changeAtts.splice(att,1);		
        } catch(err) {
            alert('Problem in removeAttDependency at line ' + err.lineNumber + ' : ' + err.message);
        }
    };

    
    /**
    * Called when changing the list or column for an attribute. The difference is seen from the id of elem
    * @param {id} elem id of the changed dropdown
    */
    attributeCategory.prototype.changeRestrict = function(elem) {
        try {
            if (this.currentRowIndex == null || this.currentColIndex==null) return;
            this.setACatChanged();
            var cellObj = this.cells[this.currentRowIndex][this.currentColIndex];
            if (!cellObj) return;
            var attObj = cellObj.attObj;
            if (!attObj) return;
            var columnAlreadyAssigned = false;
            var removeColumnAssignment = false;
            var repaintCellDetails = true;  // some changes need the cell details to be repainted to show/hide the right fields. Some don't and for input fields it is often even bothering since clicking Enter creates problems 

            var oldColObj = attObj.columnObj;



            switch (elem.id) {
            case 'restrictTypeSelector':
                attObj.restrictType = elem.value;
                if (attObj.restrictType != 'List') {
                    attObj.listObj = null; 
                    attObj.columnObj = null;
                    cellObj.renderOption='Dropdown';
                }
                if (attObj.restrictType != 'Attribute') {
                    attObj.restrictToAttObj = null;
                }
                if (attObj.restrictType === 'None') {
                    attObj.allowMultipleValues = false;
                }
                removeColumnAssignment = true;

                if (attObj.restrictType == 'List' && this.lastListUsed != null && this.lastListUsed >=0) {
                    this.selectList(attObj,this.lastListUsed,false);
                    this.selectDefaultListLoadingType(attObj.listObj);
                }

                break;
            case 'ListSelector':
                this.selectList(attObj, parseInt(elem.value,10), true);	
                removeColumnAssignment = true;			
                break;

            case 'ListColumnSelector':
                var restrictedColumn = parseInt(elem.value);

                var listObj = attObj.listObj;

                // link attribute to the new column
                if (restrictedColumn != -1) {
                    var colObj = listObj.columns[restrictedColumn]; 
                    if (colObj) {
                        // first look whether this column is already assigned
                        if (colObj.attObj) {
                            alert(ACLayoutStrings.t('ACLayoutColumnAlreadyAssigned'));
                            columnAlreadyAssigned = true;
                        } else {						

                            attObj.columnObj = colObj;
                            colObj.attObj = attObj;
                            colObj.currentIndex = -1;

                        }

                    } else {
                        throw("restrictedColumn out of bounds: " + restrictedColumn);
                    }
                } else {
                    attObj.columnObj = null;

                }		
                removeColumnAssignment = true;
                 break;	

            case 'AttributeRestrictionSelector':	 
                var idx = parseInt(elem.value);
                if (idx >=0 && idx < this.attributes.length) {
                    attObj.restrictToAttObj = this.attributes[idx];
                    this.attributes[idx].isInRule = true;
                } else {
                    attObj.restrictToAttObj = null;
                    //this.attributes[idx].isInRule = false;  // TO DO: only put it on false if no longer in any rule at all.  Not done yet in 12.0 since this is more risky and only contributes to speed.
                }
                removeColumnAssignment = true;
                break;

            case 'RenderingSelector':
				cellObj.renderOption = elem.value;

				if ( attObj.isDocReference() ) {
					
					if (cellObj.renderOption == 'Table') {
						if (this.currentView == 'Default') {						
							this.createDefaultDocReferenceTable( cellObj, true);
						} else {

							// first check whether the default view has table rendering
							var cellObjDefault = this.findDefaultCellObj(cellObj);
							if (cellObjDefault.renderOption == 'Table') {
								cellObj.docLinkTableOptions = cellObjDefault.docLinkTableOptions;
							} else {
								this.createDefaultDocReferenceTable(cellObj, true);
							}
						}
					} else {
						cellObj.docLinkTableOptions = null;  // configuration can be dangling in the docSourceGallery
					}

                } else {
                    if (cellObj.renderOption != 'Dropdown') {
                        // images or checkboxes
                        cellObj.nImageColumns = 6;
                        if (!cellObj.restrictedValueLabelPosition) cellObj.restrictedValueLabelPosition = 'LabelBelow';
                        if(attObj.listObj){
                            // for lists to be displayed this way, their values have to be loaded
                            attObj.listObj.loadingType = 'Immediate';
                            this.readListAjax(attObj.listObj, this.readListsURL);
                        }
                    } else {
                        this.selectDefaultListLoadingType(attObj.listObj);
                        cellObj.nImageColumns = 0;
                    }
                    cellObj.imageAlignment = 'DistributeOverWidth';
                    if (cellObj.renderOption == 'Images') {
                        cellObj.renderHeight = '';
                        cellObj.imageHeight = 100;
                        cellObj.renderImageType = 'jpg';
                    }
                }
                break;

            case 'ImageAlignment':
                cellObj.imageAlignment = elem.value;			
                repaintCellDetails = false;	
                break;
            case 'RestrictedValueLabelPosition':
                cellObj.restrictedValueLabelPosition = elem.value;	
                repaintCellDetails = false;				
                break;

            case 'RenderColumns':
                cellObj.nImageColumns = parseInt(elem.value,10);		
                repaintCellDetails = false;			
                break;

            case 'RenderHeight':
                cellObj.renderHeight = parseInt(elem.value,10);	
                repaintCellDetails = false;			
                break;	
            case 'ImageHeight':
                cellObj.imageHeight = parseInt(elem.value,10);	
                repaintCellDetails = false;					
                break;	
            case 'RenderImageType':
                cellObj.renderImageType = elem.value;		
                repaintCellDetails = false;		
                break;


            default: 
                throw(new Error("unknown elem.id")); 
            }

            this.setACatChanged();	

            // take away old column assignment
            if (removeColumnAssignment) {
                if (!columnAlreadyAssigned && oldColObj) {										
                    oldColObj.currentIndex = -1;
                    oldColObj.attObj = null;						
                }
            }

            if (repaintCellDetails) {
                var cellDetailsTable = document.getElementById("celldetails");		
                if (cellDetailsTable) cellDetailsTable.innerHTML = this.displayCellDetails();    			
            } 



            this.displayAttributeCategoryPreview();	
        } catch(err) {
            alert('Problem in attributeCategory.changeRestrict line ' + err.lineNumber + ': ' +  err.message );
            return '';
        }
    };

    /**
    * Set default loading type for a list.
    * @param {Obj} listObj
    */
    attributeCategory.prototype.selectDefaultListLoadingType = function(listObj) {
        'use strict';
        try {
            if(listObj){
                if (listObj.type == 'database') {
                    listObj.loadingType = 'OnTheFly'; 
                } else { 
                    listObj.loadingType = 'Immediate';
                }
            }
        } catch(err) {
            alert('Problem in attributeCategory.selectDefaultListLoadingType line ' + err.lineNumber + ': ' +  err.message );
        }
    }

    /**
    * Selects a list either after selecting the list from the list dropdown or automatically when changing the restrict type
    * Also tries doing an automatic selection of the column
    * @param {Obj} attObj - attribute object, one of this.attributes
    * @param {Number} listIdx  - typically the to Integer parsed value of the dropdown including -1 for please select
    */
    attributeCategory.prototype.selectList = function(attObj,listIdx,alertIfNoColumnsLeft) {
        'use strict';
        try {
            if (listIdx != -1) {

                var listObj = this.lists[listIdx];
                // default to a column with the same name (if still free) or to first unused column
                var firstFree = -1;
                var matchCol = -1;
                var restrictedColumn = -1;
                for (var col=0; col<listObj.columns.length; col++) {
                    var colObj = listObj.columns[col];
                    if (!colObj.attObj) {
                        if (firstFree == -1) firstFree = col;
                        if (attObj.name == colObj.columnName) matchCol = col;
                    }
                }
                if (matchCol != -1) {
                    restrictedColumn = matchCol;
                } else if (firstFree != -1) {
                    restrictedColumn = firstFree;
                }

                if (restrictedColumn >= 0 ) {					
                    var colObj = listObj.columns[restrictedColumn]; 										
                    attObj.columnObj = colObj;
                    colObj.attObj = attObj;
                    colObj.currentIndex = -1;
                    attObj.listObj = listObj; 
                    this.lastListUsed = listIdx;
                } else {
                    if (alertIfNoColumnsLeft) alert(ACLayoutStrings.t('ACLayoutAllColumnsInUse'));
                }


            } else {
                attObj.listObj = null; 				
                attObj.columnObj = null;
                this.lastListUsed = null;			
            }

        } catch(err) {
            alert('Problem in attributeCategory.selectList line ' + err.lineNumber + ': ' +  err.message );
        }
    };

    attributeCategory.prototype.changeInfoPage = function(elem) {
        try {		
            if (this.currentRowIndex == null || this.currentColIndex==null) return;
            this.setACatChanged();
            this.cells[this.currentRowIndex][this.currentColIndex].infoPage = elem.value;
            this.displayLayout();
        } catch(err) {
            alert("Problem in attributeCategory.changeRuler: " + err);
            return '';
        }
    };


    attributeCategory.prototype.changeRuler = function(feature) {
        try {		
            if (this.currentRowIndex == null || this.currentColIndex==null) return;
            this.setACatChanged();
            if (feature == 'Thickness') {
                var inputElement = document.getElementById("rulerThickness");
                WCHTML.checkNumericInput(inputElement,"IntegerPositiveStrict");
                if (inputElement) this.cells[this.currentRowIndex][this.currentColIndex].rulerThickness = inputElement.value;

            } else if (feature == 'Color') {
                var inputElement = document.getElementById("rulerColor");			
                if (inputElement) this.cells[this.currentRowIndex][this.currentColIndex].rulerColor = inputElement.value;
            } else {
                throw("unknown feature");
            }
            this.displayAttributeCategory();
        } catch(err) {
            alert("Problem in attributeCategory.changeRuler: " + err);
            return '';
        }
    };

    attributeCategory.prototype.changeSpacer = function(feature) {
        try {		
            if (this.currentRowIndex == null || this.currentColIndex==null) return;
            this.setACatChanged();
            if (feature == 'Height') {
                var inputElement = document.getElementById("spacerHeight");
                WCHTML.checkNumericInput(inputElement,"IntegerPositiveStrict");
                if (inputElement) this.cells[this.currentRowIndex][this.currentColIndex].height = inputElement.value;

            } else if (feature == 'Color') {
                var inputElement = document.getElementById("spacerColor");			
                if (inputElement) this.cells[this.currentRowIndex][this.currentColIndex].background = inputElement.value;
            } else {
                throw("unknown feature");
            }
            this.displayAttributeCategory();
        } catch(err) {
            alert("Problem in attributeCategory.changeSpacer: " + err);
            return '';
        }
    };

    attributeCategory.prototype.changeRule = function(selectElem) {
        try {
            this.setACatChanged();
            if (this.currentRowIndex>=0 && this.currentColIndex>=0) {
                var idx = parseInt(selectElem.value);
                if (idx>=0 && idx < this.rules.length) {
                    this.cells[this.currentRowIndex][this.currentColIndex].ruleObj = this.rules[idx];
                }
            }
            this.displayLayout();
        } catch(err) {
            alert("Problem in attributeCategory.changeStyle: " + err);
        }
    };

    attributeCategory.prototype.changeStyle = function(selectElem) {
        try {
            this.setACatChanged();
            if (this.currentRowIndex>=0 && this.currentColIndex>=0) {
                var idx = parseInt(selectElem.value);
                if (idx>=0 && idx < this.styles.length) {
                    this.cells[this.currentRowIndex][this.currentColIndex].styleObj = this.styles[idx];
                }
            }
            this.displayLayout();
        } catch(err) {
            alert("Problem in attributeCategory.changeStyle: " + err);
        }
    };

    /**
    * @param noRepaint: avoids repainting the entire attribute category page. Set on true if you'll repaint anyway or to avoid that the repaint changes the element which generates the applyCellProperites call
    */
    attributeCategory.prototype.applyCellProperties = function(row,col,noRepaint) {
        try{
            this.setACatChanged();
            var nextCellObj = null;
            var repaintNeeded = false;

            if(this.cells[row]) {
                var cellObj = this.cells[row][col];

                if(this.layout == 'OneColumn' && cellObj){
                    if(document.getElementById("isrequiredcheckbox").checked)
                        cellObj.ruleObj = this.isRequiredRule;
                    else
                        cellObj.ruleObj = "";
                } else {
                    if(cellObj && cellObj.colSpan && cellObj.colSpan > 1) {
                        nextCellObj = this.cells[row][col+parseInt(cellObj.colSpan)];
                    } else {
                        nextCellObj = this.cells[row][col+1];
                    }
                    var colSpanValue = document.getElementById("colspan").value;
                    if (!colSpanValue) colSpanValue = 1;
                    if(cellObj) {
                        if (!cellObj.colSpan) cellObj.colSpan = 1; // normalize old colspan
                        if (cellObj.colSpan != colSpanValue) repaintNeeded = true;  // means colspan will be changed
                        if(cellObj.colSpan && colSpanValue <= cellObj.colSpan && colSpanValue >= 1) {
                            cellObj.colSpan = document.getElementById("colspan").value;
                        } else if (nextCellObj == null && colSpanValue != "" && colSpanValue > 1) {// Check if colSpan value is greater than 1 and the next cell is empty.
                            cellObj.colSpan = document.getElementById("colspan").value;
                        } else if(nextCellObj != null && colSpanValue != "" && colSpanValue > 1) { // Check if colSpan value is greater than 1 and the next cell is NOT empty.
                            alert(ACLayoutStrings.t('ACLayoutColSpanError'));
                        }

                        // Assign other attributes selected in GridLayout to properties of attribute object.
                        if (cellObj.type == 'Title' && document.getElementById("label")) {
                            var originalLabel = cellObj.label;
                            cellObj.label = document.getElementById("label").value;
                            // also repaint here if changed, because title is shown in grid
                            if (originalLabel == null || originalLabel != cellObj.label ) repaintNeeded = true;
                        }
                        if ((cellObj.type == 'Attribute'  || cellObj.type == 'Property') && document.getElementById("label")) {
                            // if label is the same as the attribute name, don't store it
                            // make sure empty label is '' and not null
                            var labelValue = document.getElementById("label").value;
                            if(!labelValue){
                                labelValue = '';
                            } else if (cellObj.attObj && (labelValue == cellObj.attObj.name)){
                                labelValue = null;
                            }
                            cellObj.label = labelValue;						
                        }
                        if ((cellObj.type == 'Attribute'  || cellObj.type == 'Property') && document.getElementById("labelImage")) {
                            cellObj.labelImage = document.getElementById("labelImage").value;						
                        }
                        if ((cellObj.type == 'Attribute'  || cellObj.type == 'Property') && document.getElementById("labelPosition")) {
                            cellObj.labelPosition = document.getElementById("labelPosition").value;						
                        }
                        if (cellObj.type == 'Attribute' && document.getElementById("customUnitLabel")) {
                            cellObj.customUnitLabel = document.getElementById("customUnitLabel").value;						
                        }

                        if (document.getElementById("rowsize")) {
                            WCHTML.checkNumericInput(document.getElementById("rowsize"),"IntegerPositiveStrict",null,cellObj.rowSize);
                            cellObj.rowSize = document.getElementById("rowsize").value;
                        }
                        if (this.currentView == 'Default') {
                            if (document.getElementById("ChkBx_AllowMultipleValues")) {
                                cellObj.allowMultipleValues = document.getElementById("ChkBx_AllowMultipleValues").checked;
                                if (cellObj.attObj) cellObj.attObj.allowMultipleValues = cellObj.allowMultipleValues; // unfortunately double storage is needed for visibility rules. 
                            }
                        }
                        if (this.currentView == 'Default') {
                            if (document.getElementById("ChkBx_Imperial")) {
                                cellObj.fractionalIfImperial = document.getElementById("ChkBx_Imperial").checked;
                                // TODO: check if this is needed for imperial attributes
                                if (cellObj.attObj) cellObj.attObj.fractionalIfImperial = cellObj.fractionalIfImperial; // unfortunately double storage is needed for visibility rules. 
                            }
                        }
                    } else { // should not happen, prevented by the UI --> alert not translated
                        alert('Please assign an attribute to change the properties');
                    }
                }
            }
            if (!noRepaint && repaintNeeded) {
                this.displayLayout();  // should only be done for dropdowns, not for input fields. Currently only done for colspan since this needs to update the grid
            } else {  // only repaint the preview section
                this.displayAttributeCategoryPreview();	
            }



        } catch(err) {
            alert("Problem in attributeCategory.applyCellProperties: " + err);
            return '';
        }
    };




    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // DRAG AND DROP OPERATIONS
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    var logString ='';

    attributeCategory.prototype.mouseDown_attributeToInsert = function(e, obj) {
        var attributeSelector = document.getElementById("attributeSelector");
        if (!attributeSelector) return;
        this.insertAttribute = true;
        logString += '\n setting insertAttribute to true' ;
        startTime = +new Date();
        var attObj = this.attributes[obj.value];
        this.dragCurrentAttribute = this.attributes[obj.value];
        this.selectedAttributeIdx = obj.value;
        e?evt=e:evt=window.event;
        var elem = document.getElementById("dragme");

        if (elem) {
            elem.style.left = evt.clientX + 10 + "px";
            elem.style.top = evt.clientY + 10 + "px";
            elem.style.display = 'block';
            elem.innerHTML = WCXML.xmlencode(attObj.name);
        }
        document.body.style.cursor = 'move';
        attributeSelector.style.cursor = 'move';  // also a move cursor in the selector to show you are dragging
        //attributeSelector.disabled = true;
    };


    attributeCategory.prototype.dragAttribute = function(e) {
        if(this.insertAttribute) {
            e?evt=e:evt=window.event;
            var elem = document.getElementById("dragme");
            if (elem) {
                elem.style.left = evt.clientX + 10 + "px";
                elem.style.top = evt.clientY + 10 + "px";
                elem.style.display = 'block';
            }
        }
    };


    attributeCategory.prototype.mouseUp_attributeToInsert = function() {
        var attributeSelector = document.getElementById("attributeSelector");
        if (!attributeSelector) return;
        this.insertAttribute = false;
        var elem = document.getElementById("dragme");
        if (elem) elem.style.display = 'none';
        document.body.style.cursor = 'default';
        attributeSelector.style.cursor = 'default';
        //attributeSelector.disabled = false;

    };

    attributeCategory.prototype.displayAttributeSelector = function() {
        try {
            //var selectedRow = this.currentRowIndex;
            //var selectedCol = this.currentColIndex;
        	var addFormElements = this.galleryType == 'Properties' && (this.currentView == 'ProjectCreation' || this.currentView == 'TaskExecution');
        	
            var htmlcode = '';
            var changeMethod = this.functionPrefix + '.changeSelectAttribute(this)';
            var attArr = this.attributes;
            
            if (this.galleryType == 'Properties') attArr = this.otherProperties;   // fake the list of attributes to another array. The rest remains the same.
            //var onDownMethod = this.functionPrefix + '.mouseDown_attributeToInsert(event, this)';
            //var onUpMethod = this.functionPrefix + '.mouseUp_attributeToInsert()';
            if (!this.attributes) return '';
            var selectSize = '22';
            var htmlArr = [];

            if (this.layout == 'OneColumn') {
                selectSize = '15'; // same size as the OneColumn select box at the right
            } else {
                //if (this.nRows > 5) selectSize = (this.nRows) * 1.7;  // empirical rule to make it look good		
            }

            htmlArr.push('<select id="attributeSelector" onchange="' + changeMethod + '" size="'+ selectSize+'">');

            var htmlcode2 = '';
            for (var attribute=0; attribute< attArr.length; attribute++) {
                if (attribute %20 == 0) {
                    htmlArr.push(htmlcode2);
                    htmlcode2 = '';
                }
                var attObj =attArr[attribute];
                //show if filter allows.
                if (attObj && (this.passesFilter(attObj.name))) {
                    if (this.currentView == 'Default' || addFormElements || attObj.placedArr[0] )  {
                        htmlcode2 += '<option value="'+ attribute +'"';

                        if(!attObj.placed){
                        	attObj.placed = 0; // normalization only, no real function
                        }

                        var name = addFormElements? ACatStrings.t(attObj.name) : attObj.name;
                        htmlcode2 += '>' + WCXML.xmlencode(name);
                        if(attObj.placedArr[this.currentViewIdx] == 1) { // The default appearance of attributes (for single selection).
                        	 htmlcode2 += "(*)";
                        } else if(attObj.placedArr[this.currentViewIdx] > 1) { // Indicate multiple selections using * with the particular count.
                        	htmlcode2 += "(*" + attObj.placedArr[this.currentViewIdx] + ")";
                        }

                        htmlcode2 += '</option>';
                    }
                }
            }
            htmlArr.push(htmlcode2);
            htmlArr.push('</select>');
            htmlcode = htmlArr.join('');
            return htmlcode;

        } catch(err) {
            alert("Problem in attributeCategory.displayAttributeSelector: " + err);
            return '';
        }

    };

    attributeCategory.prototype.passesFilter = function(name) {

        try {
            name = name.toLowerCase();
            if (!this.attributeFilter) {  // covers attributeFilter == "" but also null, undefined, ... 
                return true;
            } else if (this.attributeFilter.indexOf("*") == -1) {
                if (name.indexOf(this.attributeFilter) != -1) {
                    return true;
                } else {
                    return false;
                }
            } else {
                // Filter contains a * --> split it into the * portions
                var splitFilter =  this.attributeFilter.split("*");
                // Remark that multiple * is not allowed.
                // also remark that attribute filter does not contain leading or trailing spaces
                if (splitFilter.length >2) return true;
                if (this.attributeFilter.indexOf("*") == (this.attributeFilter.length-1) ) {  // filter looks like abc*
                    var filter2 = this.attributeFilter.substr(0,this.attributeFilter.length-1).toLowerCase(); 	
                    if (name.length < filter2.length) return false;
                    if (name.substr(0,filter2.length) == filter2) return true; else return false;
                } else if (this.attributeFilter.indexOf("*") == 0) {  // filter looks like *abc
                    var filter2 = this.attributeFilter.substr(1,this.attributeFilter.length-1).toLowerCase(); 				
                    if (name.length < filter2.length) return false;
                    if (name.substr(name.length-filter2.length,filter2.length) == filter2) return true; else return false;
                } else {
                    // * in the middle 
                    if (name.length < splitFilter[0].length + splitFilter[1].length) return false;
                    if (name.substr(0,splitFilter[0].length) != splitFilter[0]) return false;
                    if (name.substr(name.length-splitFilter[1].length, splitFilter[1].length) != splitFilter[1]) return false;
                    return true;
                }


            }

        } catch(err) {  
            alert("line " + err.lineNumber + ": " +  err.message );
            return true;
        }
    };

    attributeCategory.prototype.displayLayoutOperations = function() {
        try {
            var htmlcode = '<table>';
            // button >>
            htmlcode += '<tr><td align="center">';
            var changeMethod = this.functionPrefix + '.placeCurrentAttribute()';
            htmlcode += '<button class="btn btn-default" type="button" onclick="' + changeMethod + '"><i class="icon icon-WCR_Chevron-Right12"></i></button>';
            htmlcode += '</td></tr>';

            // button <<
            htmlcode += '<tr><td align="center">';
            var changeMethod = this.functionPrefix + '.removeCurrentAttribute()';
            htmlcode += '<button class="btn btn-default" type="button" onclick="' + changeMethod + '"><i class="icon icon-WCR_Chevron-Left12"></i></button>';
            htmlcode += '</td></tr>';

            htmlcode += '</table>';
            return htmlcode;

        } catch(err) {
            alert("Problem in attributeCategory.displayLayoutOperations: " + err);
            return '';
        }

    };

    /**
    * Go from OneColumn layout to Grid layout
    */
    attributeCategory.prototype.useGridLayout = function() {
        try{
            this.setACatChanged();
            this.layout = "Grid"; // Toggle the global variable used to recognize OneColumn/Grid.
            if (this.cells.length < 12) {
                for (var row = this.cells.length; row< 12; row++) {
                    this.cells.push(new Array());
                }
            }
            this.nRows = this.cells.length;
            // also update the view
            if (this.views) {
                this.views[this.currentViewIdx].layout = this.layout;
                this.views[this.currentViewIdx].nRows = this.nRows;
            }
            // add a header bar at the top
            this.currentRowIndex = 0; this.currentColIndex=0;
            this.addHeaderBar();
            this.currentRowIndex = this.nRows-1; this.currentColIndex=0;
            this.showEditTerminology = true;
            this.displayLayout();
        } catch(err) {
            alert("Problem in attributeCategory.useGridLayout: " + err);
            return '';
        }
    };

    /**
    * update the attribute filter when changed and update the display based on this
    * @param elem DOM object of the filter input field 
    */
    attributeCategory.prototype.setAttributeFilter = function(elem, checkEaster, canCauseBlur) {
        try {        	
            var repaintLayout = false; 
            if (!elem) return;
            
            // if the method can cause a blur, we should call the blur right away, 
            // otherwise, it might repaint and the blur will throw an exception
            // the blur will call this method again with easter egg check
            if(canCauseBlur){
            	$(elem).blur();
            	return;
            }
            
            var newFilter ="";
            if (elem.value) newFilter = $.trim(elem.value.toLowerCase());
            var splitFilter = newFilter.split("*");
            if (splitFilter.length >2) {
                if (splitFilter.length == 3 && splitFilter[0]=="" && splitFilter[2]=="") {  // *abc* --> replace with abc
                    newFilter = splitFilter[1]; 
                } else {
                    alert(ACLayoutStrings.t('ACLayoutFilterWithMultipleAsterisk'));
                    newFilter = "";
                }
            }
            this.attributeFilter = newFilter;			

            // Easter eggs to switch on advanced functions
            if (this.attributeFilter && this.attributeFilter.toLowerCase() == 'easter' && checkEaster) {
                this.showEditTerminology=true;  // is default false for Task Specifications
                this.easterEgg = true; 
                repaintLayout=true;
            };


            if (repaintLayout) {
                this.displayLayout();  //repaint everything
            } else {
                // only repaint the attribute gallery
                var attSelectorElem = document.getElementById('tdAttributeSelector');
                if (attSelectorElem) attSelectorElem.innerHTML = this.displayAttributeSelector();
            }
        } catch(err) {
            alert("Problem in attributeCategory.setAttributeFilter: " + err);
            return '';
        }
    };

    /**
    * Moves from grid layout to one column layout
    * All layout information (titles, colors, whitespace) will be lost
    * items will appear top left to bottom right
    */
    attributeCategory.prototype.removeGridLayout = function() {
        try{
            this.setACatChanged();
            var conf = confirm(ACLayoutStrings.t('ACLayoutRemoveGridConfirm'));
            if (!conf) return;
            var cellArr = new Array();
            this.layout = "OneColumn";

            for (var row=0; row< this.cells.length; row++) {
                if (!this.cells[row]) continue;
                for (var col=0; col<this.cells[row].length; col++) {
                    var cellObj = this.cells[row][col];

                    if (cellObj && (cellObj.type == 'Attribute' || cellObj.type == 'Property')) {
                        var attObj = cellObj.attObj;
                        // remove layout features
                        cellObj.colSpan = null;
                        cellObj.rowSize = null;
                        cellObj.label = null;
                        cellObj.background = null;
                        cellObj.styleObj = null;
                        cellObj.customUnitLabel = null;
                        cellObj.labelPosition = 'Left';
                        cellObj.buttons = null;
                        cellObj.infoPage = null;
                        
                        if(!this.isTaskTypePage) cellObj.ruleObj = null; // BVON: Specification will keep the rule to know if it's required

                        if (attObj.restrictType == 'Attribute' || attObj.restrictType == 'List') attObj.restrictType = 'None';
                        attObj.restrictToAttObj = null;
                        if (attObj && attObj.columnObj) {
                            var columnObj = attObj.columnObj;
                            attObj.listObj = null;
                            attObj.columnObj = null;						
                            columnObj.attObj = null;
                            columnObj.attributeId = null;
                            columnObj.attributeName = null;
                            columnObj.currentIndex = -1;
                        }
                        attObj.isInRule = false;

                        // append it to the new cells array
                        var rowArr = new Array(1);
                        rowArr[0]=cellObj;
                        cellArr.push(rowArr);
                    }
                }
            }
            this.cells = cellArr;

            this.nRows = this.cells.length;
            this.nCols = 1;
            // also update the view
            if (this.views != null) {
                this.views[this.currentViewIdx].layout = this.layout;
                this.views[this.currentViewIdx].nRows = this.nRows;
                this.views[this.currentViewIdx].nCols = this.nCols;
                this.views[this.currentViewIdx].cells = cellArr;
            }
            this.displayLayout();
        } catch(err) {
            alert("Problem in attributeCategory.removeGridLayout: " + err);
            return '';
        }
    };
    /**
    * Adds a column before column colIndexInsert
    * If colIndexInsert == -1, adds a column at the current selection
    * If colIndexInsert == null, adds a column at the end
    */
    attributeCategory.prototype.addColumn = function(colIndexInsert) {
        try{
            this.setACatChanged();
            var cellObj = null;
            if(colIndexInsert == -1) {
                colIndexInsert = this.currentColIndex;
            }
            if(colIndexInsert == null) {
                colIndexInsert = this.nCols;
            }
            this.nCols++;
            for (var row=0; row < this.cells.length; row++) {
                this.cells[row].splice(colIndexInsert, 0, null); //Add column to each row.
                cellObj = this.cells[row][0];
                if(cellObj && cellObj.colSpan == this.nCols-1 && cellObj.type != 'Attribute'  ) { // Check if a row has a title with colspan throughout. If so, merge the new column with the colspan.
                    cellObj.colSpan++;
                }
            }
            // also update the view
            if (this.views) {
                this.views[this.currentViewIdx].nCols = this.nCols;
            }

            this.displayLayout();


        } catch(err) {
            alert("Problem in attributeCategory.addColumn: " + err);
            return '';
        }
    };

    attributeCategory.prototype.deleteColumn = function(colIndexDelete) {
        try{
            this.setACatChanged();

            if(colIndexDelete == null) {
                colIndexDelete = this.nCols-1;
            }
            if(colIndexDelete == -1) {
                colIndexDelete = this.currentColIndex;
                if (colIndexDelete == null || colIndexDelete >= this.nCols) {
                    alert(ACLayoutStrings.t('ACLayoutSelectCellToDeleteColumn'));
                    return;
                }
            }

            if (this.getConfirmationAttributeRemove(null,colIndexDelete) === true) {

                var noConfirm = true;
                for (var row=0;row<this.cells.length;row++) {      			
                    this.removeCellAttribute(row,colIndexDelete,noConfirm,true);
                }

                // update colspan of surrounding elements
                for (var row=0; row < this.cells.length; row++) {
                    if(this.cells[row]) {
                        // first look whether you are in a colspan area
                        if (this.cells[row][colIndexDelete] == null && colIndexDelete>0)
                        {
                            for (var col2=colIndexDelete-1;col2>=0 ;col2-- )
                            {
                                if (this.cells[row][col2] && this.cells[row][col2].colSpan > colIndexDelete-col2)
                                { // then this cell spans into the column we are going to delete
                                    this.cells[row][col2].colSpan--;
                                }
                            }
                        }

                        this.cells[row].splice(colIndexDelete, 1); //Splice and delete a column for each row.
                    }
                }

                this.nCols--;
                if (this.nCols ==0) this.nCols = 1;
                // also update the view
                if (this.views) {
                    this.views[this.currentViewIdx].nCols = this.nCols;
                }

                if (this.currentColIndex >= this.nCols) {this.currentRowIndex = null;this.currentColIndex = null;}; // no selection left

                this.displayLayout();
            }

        } catch(err) {
            alert("Problem in attributeCategory.deleteColumn: " + err);
            return '';
        }
    };



    /**
    * Adds or inserts a row
    * @param rowIndexInsert place where to insert
    * if null, row is added at the end
    * if -1, row is inserted before the currently selected cell
    */
    attributeCategory.prototype.addRow = function(rowIndexInsert) {
        try {
            this.setACatChanged();
            if (rowIndexInsert == -1) rowIndexInsert = this.currentRowIndex;
            if(rowIndexInsert==null) { // Adds new row to the end.
                this.cells.push(new Array());
                rowIndexInsert = this.cells.length-1;
            }
            else { // Inserts in the middle based on the user selection.
                this.cells.splice(rowIndexInsert, 0, new Array(this.nCols));
            }
            this.nRows = this.cells.length;
            // also update the view
            if (this.views != null) {
                this.views[this.currentViewIdx].nRows = this.nRows;
            }
            // To select the left most cell after adding / inserting new row.
            //if (rowIndexInsert === null) {
            if (rowIndexInsert == null) {
                this.setRowColIndex(this.nRows-1, 0); // Select the left most cell from the last row.
            }
            else {
                this.setRowColIndex(rowIndexInsert, 0); // Select the left most cell from the new row inserted.
            }
            this.currentRowIndex = rowIndexInsert;  // select the row just added, leftmost cell
            this.currentColIndex = 0;
            this.displayLayout();


        } catch(err) {
            alert("Problem in attributeCategory.addRow: " + err);
            return '';
        }
    };

    /**
    * Deletes a row from the grid layout
    * @param {int} rowIndexDelete index of the row to be deleted. When null deletes the last row, when -1 the current row will be deleted
    */
    attributeCategory.prototype.deleteRow = function(rowIndexDelete) {
        try {


            if (rowIndexDelete==null) rowIndexDelete = this.cells.length-1; // delete last row
            if (rowIndexDelete == -1) rowIndexDelete = this.currentRowIndex;  // delete row of the current selection
            if (rowIndexDelete === null || rowIndexDelete < 0) {
                if (rowIndexDelete === null) alert(ACLayoutStrings.t('ACLayoutSelectCellToDeleteRow'));
                return;
            }
            this.setACatChanged();

            if (this.getConfirmationAttributeRemove(rowIndexDelete,null) === true) {
                var noConfirm = true;
                for(var col=0;col<this.nCols;col++) {
                    this.removeCellAttribute(rowIndexDelete,col,noConfirm,true);
                }

                if (this.nRows > 1) this.cells.splice(rowIndexDelete, 1); // Splice and delete a row from two dimensional array.

                this.nRows = this.cells.length;
                // also update the view
                if (this.views != null) {
                    this.views[this.currentViewIdx].nRows = this.nRows;
                }

                if (this.currentRowIndex !== null && this.currentRowIndex >= this.nRows) {this.currentRowIndex = null;this.currentColIndex = null;}; // no selection left

                this.displayLayout();
            }



        } catch(err) {
            alert("Problem in attributeCategory.deleteRow: " + err);
            return '';
        }
    };

    attributeCategory.prototype.displayLayoutOneColumn = function () {
        try {
            if (this.layout == 'Grid') {
                return this.displayLayoutGrid();
            }
            // simple layout
            var htmlcode = '';
            var changeMethod = this.functionPrefix + '.selectLayoutAttribute(this)';
            if (this.detectIfIE()) {
                if (this.cells.length >0) {
                    htmlcode += '<select onchange="' + changeMethod + '" size="15" id="attributesListBox">';  // let it take the automatic width, works fine in IE
                } else {
                    htmlcode += '<select onchange="' + changeMethod + '" size="15" id="attributesListBox" style="width:200px">'; // fix width to 200 px, otherwise it's a very narrow box
                }
            } else {
                htmlcode += '<select onchange="' + changeMethod + '" size="15" id="attributesListBox" style="width:100%">';  
                // works fine in all browsers except IE which shows nothing unless there is something forcing a width and then it's fixed to that
            }		
            //htmlcode += '<select onchange="' + changeMethod + '" size="15" id="attributesListBox" >';

            if (this.cells) {
                // Read all the attributes from layout and put it into a single column to populate it in a list box.
                for (var row = 0; row < this.cells.length; row++) {
                    if (this.cells[row]) {
                        var cellObj = this.cells[row][0];
                        if (cellObj) {
                            htmlcode += '<option value="' + row + '"';
                            if (row == this.currentRowIndex) {
                                htmlcode += ' selected ';
                            }
                            htmlcode += '>' + WCXML.xmlencode(cellObj.name);
                            htmlcode += '</option>';						
                        }
                    }
                }
            }
            htmlcode += '</select>';
            return htmlcode;

        } catch (err) {
            alert("Problem in attributeCategory.displayLayoutOneColumn: " + err);
            return '';
        }

    };



    /**
    * sets this.currentRowIndex and this.currentColIndex
    */
    attributeCategory.prototype.setRowColIndex = function (selectRowIndex, selectColIndex) {
        try {
            var oldSelectedElement = null;  
            if (this.currentRowIndex!=null && this.currentColIndex!=null) {
                oldSelectedElement = document.getElementById("id_" + this.currentRowIndex + "_" + this.currentColIndex);

                // the old element can be an attribute for which a property was changed without blurring the element (clicking on the new grid cell is the blurring operation,
                // which brings you directly here without/before doing the onchange of the element. This is browser dependent. Safari on Mac reacts clearly different from Firefox on PC.
                // Therefore, we do an explicit applyCellProperties here for the OLD element. 
                if (this.cells[this.currentRowIndex] && this.cells[this.currentRowIndex][this.currentColIndex]) this.applyCellProperties(this.currentRowIndex,this.currentColIndex,true);
            }

            this.currentRowIndex = selectRowIndex;
            this.currentColIndex = selectColIndex;

            var cellDetailsTable = document.getElementById("celldetails");

            if (cellDetailsTable) cellDetailsTable.innerHTML = this.displayCellDetails();        

            if (oldSelectedElement) {
            	$(oldSelectedElement).removeClass("Selected");
            }
            var element = document.getElementById("id_" + selectRowIndex + "_" + selectColIndex);
            if (element) {
            	$(element).addClass("Selected");
            }
        } catch (err) {
            alert("Problem in attributeCategory.setRowColIndex: " + err);
            return '';
        }
    };



    attributeCategory.prototype.displayLayoutGrid = function() {
        try {
            var cellObj = null;
            var htmlcode = '';
            var htmlArr = [];
            //htmlcode +='<table><tr>';

            var mouseOverMethod = "";
            var mouseMoveMethod = "";
            var mouseDownMethod = "";
            var mouseUpMethod = "";

            //htmlcode += '<td valign="top">';

            htmlArr.push('<div id="GridDiv" style="height:350px; overflow:auto; ">');
            htmlArr.push('<table id="grid" width="100%"  >');


            // rest of the table
            for (var row=0; row < this.cells.length; row++) {
                if(this.cells[row]) {
                    var htmlcode2 = '<tr>';


                    for(var col=0;col<this.nCols;col++) {
                        cellObj = this.cells[row][col];
                        var attObj = null;
                        if(this.cells[row][col]) {
                            attObj = this.cells[row][col].attObj;
                        }

                        //this.currentRowIndex = row;

                        mouseOverMethod = this.functionPrefix + '.handle_onmouseover('+row+','+col+')';
                        mouseMoveMethod = this.functionPrefix + '.handle_onmousemove(event,'+row+','+col+')';
                        mouseDownMethod = this.functionPrefix + '.handle_onmousedown('+row+','+col+')';
                        mouseUpMethod = this.functionPrefix + '.handle_onmouseup('+row+','+col+')';
                        clickMethod = this.functionPrefix + '.setRowColIndex('+row+','+col+')';


                        htmlcode2 += '<td id="id_'+row+'_'+col+'" class="darktext_darklink_11';
                        
                        if(this.currentRowIndex == row && this.currentColIndex == col) {
                            htmlcode2 += ' Selected';
                        }
                        htmlcode2 += '" onclick="' + clickMethod +  '" onmouseover="' + mouseOverMethod +  '" onmousemove="' + mouseMoveMethod +  '" onmousedown="' + mouseDownMethod + '; return false;" onmouseup="' + mouseUpMethod + '" style="cursor:default;padding:3pt;"';

                        if (cellObj && cellObj.colSpan)
                        {
                            htmlcode2 += ' colspan="'+cellObj.colSpan+'" ';
                        }
                        htmlcode2 += '>';

                        if(cellObj) {
                            switch(cellObj.type) {
                                case 'Title': {
                                    htmlcode2 += '<B>[Title]';
                                    if(cellObj.label) htmlcode2 += WCXML.xmlencode(cellObj.label);
                                    htmlcode2 += '</B>';
                                    if (cellObj.buttons && cellObj.buttons.length>0) {
                                        for (var b=0; b< cellObj.buttons.length; b++) htmlcode2 += '&nbsp;<B>[Button ' + WCXML.xmlencode(cellObj.buttons[b].label) + ']</B>';
                                    }
                                    break;
                                }
                                case 'Spacer': {
                                    htmlcode2 += '<B>'+'[Spacer]'+'</B>';
                                    break;
                                }

                                case 'Ruler': {
                                    htmlcode2 += '<B>'+'[Ruler]'+'</B>';
                                    break;
                                }
                                case 'HeaderBar': {
                                    htmlcode2 += '<B>'+'[HeaderBar]'+'</B>';
                                    break;
                                }
                                case 'Attribute': 
                                case 'Property':  // easter Egg
                                    if (attObj) {
                                    	if(cellObj.type === "Property"){
                                    		htmlcode2 += '<B>['+ACatStrings.t(attObj.name)+']</B>';
                                    	} else {
                                    		htmlcode2 += WCXML.xmlencode(attObj.name);
                                    	} 
                                    }
                                    else {
                                        htmlcode2 += '&nbsp;';
                                    }
                                    if (cellObj.buttons && cellObj.buttons.length>0) {
                                        for (var b=0; b< cellObj.buttons.length; b++) htmlcode2 += '&nbsp;<B>[Button ' + WCXML.xmlencode(cellObj.buttons[b].label) + ']</B>';
                                    }
                                    break;
                                case 'Button': {
                                    htmlcode2 += '<B>';
                                    if (cellObj.buttons && cellObj.buttons.length>0) {
                                        for (var b=0; b< cellObj.buttons.length; b++) htmlcode2 += '[Button ' + WCXML.xmlencode(cellObj.buttons[b].label) + ']';
                                    }
                                    htmlcode2 += '</B>';
                                    break;
                                }
                                default:
                                    htmlcode2 += '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
                            } // End of switch
                            if(cellObj.colSpan) {
                                col += cellObj.colSpan-1;
                            }
                        } //End of If
                        else {
                            htmlcode2 += '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
                        }
                        htmlcode2 += '</td>';

                    }
                    htmlcode2 += '</tr>';
                    htmlArr.push(htmlcode2);
                }

            }
            htmlArr.push('</table>');
            htmlArr.push('</div>');

            htmlcode = htmlArr.join('');

            //htmlcode += '</td>';
            //htmlcode += '</tr>';
            //htmlcode += '</table>';

            return htmlcode;
        } catch(err) {
            alert("Problem in attributeCategory.displayLayoutGrid: " + err);
            return '';
        }
    };

    attributeCategory.prototype.selectLayoutAttribute = function(selectElem) {
        try {
            if (selectElem) {
                this.currentRowIndex = selectElem.value;
                this.currentColIndex = 0;
                if(this.isTaskTypePage) 
                    document.getElementById("cellisrequired").innerHTML = this.displayCellDetails();
            }
        } catch(err) {
            alert("Problem in attributeCategory.selectLayoutAttribute: " + err);
            return '';
        }
    };

    attributeCategory.prototype.displayUpDownButtons = function() {
        try {
            var htmlcode ='<table>';
            // up and down
            var clickMethod = this.functionPrefix + '.moveUpDown(1); ';
            htmlcode += '<tr><td><button type="button" class="btn btn-default" onclick="'+clickMethod+'"><i class="icon icon-WCR_Up"></i></button></td></tr>';
            var clickMethod = this.functionPrefix + '.moveUpDown(0);';
            htmlcode += '<tr><td><button type="button" class="btn btn-default"  onclick="'+clickMethod+'"><i class="icon icon-WCR_Down"></i></button></td></tr>';

            if (this.nCols > 1 ) {  // left      
                clickMethod = this.functionPrefix + '.moveRightLeft(0); ';
                htmlcode += '<tr><td><button type="button" class="btn btn-default"  onclick="'+clickMethod+'"><i class="icon icon-WCR_Left"></i></button></td></tr>';

             // right
                clickMethod = this.functionPrefix + '.moveRightLeft(1); ';
                htmlcode += '<tr><td><button type="button" class="btn btn-default"  onclick="'+clickMethod+'"><i class="icon icon-WCR_Right"></i></button></td></tr>';       
            }
            htmlcode += '</table>';
            return htmlcode;

        } catch(err) {
            alert("Problem in attributeCategory.displayUpDownButtons: " + err);
            return '';
        }
    };

    attributeCategory.prototype.validateLayout = function() {
        return true;
    };

    /**
    * mouse clicked to indicate the source cell
    */
    attributeCategory.prototype.handle_onmousedown = function(row, col) {
        this.source_currentRow = row;
        this.source_currentCol = col;
        var elem = document.getElementById("dragme");
        if (elem) {
            var displayValue = "";
            var cellObj = this.cells[row][col];
            if (cellObj) {
                switch (cellObj.type) {
                case 'Title':
                    displayValue = '[Title] ' + cellObj.label;
                    break;			
                case 'Attribute':
                    displayValue = cellObj.name;
                    break;
                default:
                    displayValue = cellObj.type;
                }
            }
            elem.innerHTML = WCXML.xmlencode(displayValue);
        }
        swapDrag = true;
        startTime = +new Date();  // we can do this smarter, so this is not needed
        //this.setRowColIndex(row,col);
        return false;
    };




    /**
    * any onmouseup wherever on the page stops dragging
    */
    attributeCategory.prototype.body_onmouseup = function(event) {
        var elem = document.getElementById("dragme");
        if (elem) elem.style.display = 'none';
        swapDrag = false; // stop dragging
        /*
        if (this.insertAttribute) {
            if (this.currentRow!=null && this.currentCol !=null) {
                this.placeCurrentAttribute(this.currentRow, this.currentCol);
            }
        }
        */
        this.insertAttribute = false;
        document.body.style.cursor = "";
    };



    attributeCategory.prototype.handle_onmouseover = function(row, col) {
        this.currentRow = row;
        this.currentCol = col;
        //logString += "\n handle_onmouseover row: "+row + "   col:" + col;
    };

    attributeCategory.prototype.handle_onmousemove = function(e, row, col) {
        //logString += "\n onmousemove row:" + row + "  col: " + col;
        this.currentRow = row;
        this.currentCol = col;
        var elem = document.getElementById("id_" + row + "_" + col);
        if (swapDrag || this.insertAttribute) {		
            if (elem) elem.style.cursor = "move";
            e?evt=e:evt=window.event;
            var elem2 = document.getElementById("dragme");
            if (elem2) {
                elem2.style.left = evt.clientX + 5 + "px";
                // calculate effect of the scroll bar of mainContent
                var offsetTop =0,scrollTop=0;
                var mainContentElem = null;
                try {
                    if (document.getElementsByClassName("mainContent") && document.getElementsByClassName("mainContent")[0]) {
                        mainContentElem =  document.getElementsByClassName("mainContent")[0];
                    }

                } catch(err) {
                    // stupid Internet Explorer...
                    var divArr = document.getElementsByTagName("div");

                    for (var i=0;i<divArr.length;i++) {					
                        if (divArr[i].getAttribute('classname') == "mainContent"  ) break;
                    }
                    mainContentElem = divArr[i];				
                }
                if (mainContentElem) {

                    try {
                        offsetTop = mainContentElem.offsetTop;
                    } catch(err) {
                        alert(err);
                    }
                    try {
                        scrollTop = mainContentElem.scrollTop;
                    } catch(err) {
                        alert(err);
                    }


                }


                elem2.style.top = (evt.clientY -offsetTop + scrollTop + 5) + "px";
                elem2.style.display = 'block';
            }
        }

    };

    attributeCategory.prototype.handle_onmouseup = function(row,col) {

        logString += "\n handle_onmouseup row: "+row + "   col:" + col + "   this.currentRow: " + this.currentRow + "  this.currentCol:" + this.currentCol;
        var elem = document.getElementById("dragme");
        if (elem) elem.style.display = 'none';
        if(this.currentRow == this.source_currentRow && this.currentCol == this.source_currentCol) {
            swapDrag = false;		

        }
        if(this.insertAttribute) {
            // first re-select the attribute you dragged (selection might have moved while dragging)
            // var elem = document.getElementById("attributeSelector");
            // TODO??? if (elem) elem.selectedIndex = this.
            this.placeCurrentAttribute(this.currentRow, this.currentCol);
            this.insertAttribute = false;
            var elem = document.getElementById("dragme");
            if (elem) elem.style.display = 'none';	
        } else if(swapDrag) {

            var tempCellObj = this.cells[this.source_currentRow][this.source_currentCol];
            this.cells[this.source_currentRow][this.source_currentCol] = this.cells[this.currentRow][this.currentCol];
            this.cells[this.currentRow][this.currentCol] = tempCellObj;
        // adapt colspan...: give colspan the max of existing colspan and allowed colspan (don't overwrite other things)
            this.correctColspan(this.source_currentRow);
            if (this.currentRow != this.source_currentRow) this.correctColspan(this.currentRow);



            swapDrag = false;
            this.currentRowIndex = this.currentRow; this.currentColIndex = this.currentCol;
            this.aCatChanged = true;
            this.displayLayout();

        }

    };

    /**
    * runs over a row and corrects the colspan to never create overruns
    */
    attributeCategory.prototype.correctColspan = function(row) {
        if (!this.cells[row]) return; // an empty row
        for (var col=0; col<this.nCols; col++) {
            if (this.cells[row][col]) {
                curColspan = this.cells[row][col].colSpan? parseInt(this.cells[row][col].colSpan): 1;
                // find maximum colspan
                for (var col2=col+1; col2<this.nCols;col2++) {
                    if (this.cells[row][col2]) break;
                }
                var maxColSpan = col2 - col;
                if (maxColSpan < curColspan ) this.cells[row][col].colSpan =  maxColSpan;
            }
        }
    };

    /**
    * Add Title in the Grid and Preview section
    */
    attributeCategory.prototype.addTitle = function() {
        try{	
            var row = this.currentRowIndex;
            var col = this.currentColIndex;
            if (row == null) {
                this.addRow(); // add a row at the end
                row = this.nRows-1; 
            }
            if (col == null) col = 0;
            // check if cell occupied
            if (this.cells[row][col]) {
                this.addRow(row);			
            }
            var htmlcode = '';
            htmlcode += '<table class="table table-no-borders">';

            // Title: <input field>
            // Display only for the actual titles with text.
            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutTitle') + ':</td>';
            htmlcode += '<td><input type="text" value="" size="40" id="attTitleDialogTitle"></td></tr>';

            // Title: <colspan field>
            htmlcode += '<tr class="darktext_greenlink_11"><td><nobr>'+ ACLayoutStrings.t('ACLayoutColSpan') + ':</nobr></td>';
            htmlcode += '<td id="addTitleDialogColspanTd">';

            // look for maximum colspan        
            for (var col2=col; col2<this.nCols; col2++) {
                if (this.cells[row][col2]) break;
            }
            var maxColspan = col2-col;
            if (maxColspan <1) throw("this cell is occupied");
            htmlcode += '<select id="attTitleDialogColSpan">';
            for (var span=1; span<= maxColspan; span++) {
                htmlcode += '<option value="' + span + '">' + span + '</option>';
            }
            htmlcode += '</select>';
            htmlcode += '</td></tr>';

            // Style
            if (this.styles) {
                htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutStyle') + ':</td>';
                htmlcode += '<td><select id="attTitleDialogStyle">';
                for (var style=0; style< this.styles.length; style++) {
                    htmlcode += '<option value="' + style + '"';
                    if (style == 1) htmlcode += ' selected '; // default title style
                    htmlcode += '>' + WCXML.xmlencode(this.styles[style].name) + '</option>';
                }
                htmlcode += '</select></td></tr>';
            }


            htmlcode += '</table>';

            if (document.getElementById("titleDialog")) document.getElementById("titleDialog").innerHTML = htmlcode;

            var ACatObj = this;  // organize a closure... ACatObj will be available in the OK function

            var buttonObj = new Object();
            buttonObj[ACLayoutStrings.t('ACLayoutCancel')]= function(){
                $(this).dialog("close");
            };
            buttonObj[ACLayoutStrings.t('ACLayoutOK')] = function(){
                cellObj = new Object();
                ACatObj.cells[ACatObj.currentRowIndex][ACatObj.currentColIndex] = cellObj;
                cellObj.type = 'Title';
                cellObj.label = $("#attTitleDialogTitle").val();
                cellObj.colSpan =  $("#attTitleDialogColSpan").val();
                cellObj.styleObj = ACatObj.styles[$("#attTitleDialogStyle").val()];
                $(this).dialog("close");
                ACatObj.setACatChanged();
                ACatObj.displayLayout();
            };

            // create the title dialog but don't open it
            if (this.titleDialogCreated == false) {  // create it only once
                $("#titleDialog").dialog({
                    autoOpen: false,
                    modal: false,
                    closeOnEscape: true,
                    draggable: true,
                    height: 'auto',
                    resizable: true,
                    width: '500',               
                    buttons: buttonObj
                });
                this.titleDialogCreated = true;
            }

            $("#titleDialog").dialog("open");

        } catch(err) {
            alert("Problem in attributeCategory.addTitle: " + err);
            return '';
        }
    };

    /**
    * Add Spacer in the preview section
    */
    attributeCategory.prototype.addSpacer = function() {
        try{
            var row = this.currentRowIndex;
            var col = this.currentColIndex;
            if (row == null) {
                this.addRow(); // add a row at the end
                row = this.nRows-1; 
            }
            if (col == null) col = 0;
            // check if cell occupied
            if (this.cells[row][col]) {
                this.addRow(row);			
            }
            var htmlcode = '';
            htmlcode += '<table class="table table-no-borders">';

            // Height: <input field>
            // Display only for the actual titles with text.
            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutHeight') + ':</td>';
            htmlcode += '<td><input type="text" value="" size="10" id="attSpacerDialogHeight">&nbsp;px</td></tr>';

            // Color: <input field>
            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutColor') + ':</td>';
            htmlcode += '<td><input type="text" value="" size="10" id="addSpacerDialogColor"></td></tr>';

            htmlcode += '</table>';

            document.getElementById("spacerDialog").innerHTML = htmlcode;

            var ACatObj = this;  // organize a closure... ACatObj will be available in the OK function

            var buttonObj = new Object();
            buttonObj[ACLayoutStrings.t('ACLayoutCancel')]= function(){
                $(this).dialog("close");
            };
            buttonObj[ACLayoutStrings.t('ACLayoutOK')] = function(){
                cellObj = new Object();
                ACatObj.cells[ACatObj.currentRowIndex][ACatObj.currentColIndex] = cellObj;
                cellObj.type = 'Spacer';
                cellObj.label = '';

                WCHTML.checkNumericInput($("#attSpacerDialogHeight")[0],"IntegerPositiveStrict");
                cellObj.height =  $("#attSpacerDialogHeight").val();
                if(cellObj.height === ""){
                	cellObj.height =  3;
                }
                cellObj.background = $("#addSpacerDialogColor").val() || "#FFFFFF";
                cellObj.colSpan = 1;
                $(this).dialog("close");
                ACatObj.setACatChanged();
                ACatObj.displayLayout();
            };

            // create the spacer dialog but don't open it
            if (this.spacerDialogCreated == false) {  // create it only once
                $("#spacerDialog").dialog({
                    autoOpen: false,
                    modal: false,
                    closeOnEscape: true,
                    draggable: true,
                    height: 'auto',
                    resizable: true,
                    width: '250',                
                    buttons: buttonObj
                });
                this.spacerDialogCreated = true;
            }

            $("#spacerDialog").dialog("open");

        } catch(err) {
            alert("Problem in attributeCategory.addSpacer: " + err);
            return '';
        }
    };

    attributeCategory.prototype.addHeaderBar = function() {

        try {
            var row = this.currentRowIndex;
            var col = this.currentColIndex;
            if (row == null) {
                this.addRow(); // add a row at the end
                row = this.nRows-1; 
            }
            if (col == null) col = 0;
            // check if cell occupied
            if (this.cells[row][col]) {
                this.addRow(row);			
            }
            this.cells[row][col]=new Object();
            this.cells[row][col].type = "HeaderBar";

            // set colSpan to 1 except when row completely empty, then span entire row
            this.cells[row][col].colSpan = 1;		

            if (col==0 && this.nCols>1) {
                var rowEmpty=true;
                for (var col2=1; col2<this.nCols; col2++) {
                    if (this.cells[row][col2]) rowEmpty=false;
                }
                if (rowEmpty === true) this.cells[row][col].colSpan = this.nCols;	
            }
            this.setACatChanged();

            this.displayLayout();


        } catch(err) {
            alert("Problem in attributeCategory.addHeaderBar: " + err);
            return '';
        }
    };

    attributeCategory.prototype.addButton = function() {

        try {
            var row = this.currentRowIndex;
            var col = this.currentColIndex;
            if (row == null) {
                this.addRow(); // add a row at the end
                row = this.nRows-1; 
            }
            if (col == null) col = 0;


            var cellObj = this.cells[row][col]
            if (cellObj) {  // occupied cell --> add button if allowed
                // check if allowed type		
                if (cellObj.type == 'HeaderBar' || cellObj.type == 'Ruler' || cellObj.type == 'Spacer') {
                    alert(ACLayoutStrings.t('ACLayoutNoButtonAllowed'))
                    return;
                }

            } else {
                // new cell with only a button
                cellObj = this.cells[row][col]=new Object();
                cellObj.type = 'Button';			
                cellObj.colSpan = 1;	
            }

            // add the button
            if (!cellObj.buttons) cellObj.buttons = []; // Actually allow a set of buttons
            var buttonObj = new this.buttonObj('LookUpAttributes','Click','buttonprimary', null, this.currentView);		
            cellObj.buttons.push(buttonObj);


            this.setACatChanged();		
            this.displayLayout();


        } catch(err) {
            alert("Problem in attributeCategory.addButton: " + err);
            return '';
        }
    };
    /**
    * Removes button with index 'button' from cellObj
    * @param {Int} button
    * @param {Obj} cellObj - if null, current cell is used
    * @param {Boolean} noRemoveCell - flag to say that the cell cannot be deleted, typically because you are already deleting it
    */
    attributeCategory.prototype.removeButton = function(button, cellObj, noRemoveCell) {

        try {
            var row = this.currentRowIndex;
            var col = this.currentColIndex;


            if (!cellObj) cellObj = this.cells[row][col];
            if (cellObj && cellObj.buttons) {  
                var buttonObj = cellObj.buttons[button];
                if (buttonObj.changeAtts) {
                    for (var att=0; att < buttonObj.changeAtts.length; att++) {
                        this.removeAttDependency(buttonObj, att);
                    }
                }

                cellObj.buttons.splice(button,1);

                // check whether we have emptied the cell
                if (cellObj.buttons.length == 0) {
                    cellObj.buttons = null;
                    if (cellObj.type == 'Button'  && !noRemoveCell) this.removeCellAttribute(row, col, true, true);
                }
            }



            this.setACatChanged();		
            this.displayLayout();


        } catch(err) {
            alert("Problem in attributeCategory.addButton: " + err);
            return '';
        }
    };

    /**
    * Add Spacer in the preview section
    */
    attributeCategory.prototype.editTerminology = function() {

        try{

            var htmlcode = '';
            
            htmlcode += '<div name="acat_edit_terminology" class="add-padding">';

            if (this.isTaskTypePage) {
                htmlcode += '<p class="add-padding-bottom">'+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditTerminologyTEP')) + '</p>';
            } else {
                htmlcode += '<p class="add-padding-bottom">'+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditTerminologyProjectCreation')) + '</p>';
            }
            
            if(this.currentView == 'ProjectCreation'){
            	htmlcode += '<p class="help-block">'+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutEditTerminologyWarning')) + '</p>';
            }
            
            htmlcode += '<table class="table-full-width">';
            
            htmlcode += '<thead><tr><th class="rightalign add-padding-right">'+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutDefaultTerm')) + '</th><th>'+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutNewTerm')) + '</th></tr></thead>';
            
            htmlcode += '<tbody>';
            
            if (!this.terms) return;
            for (var term = 0; term < this.terms.length; term++) {
                if ((this.isTaskTypePage && this.terms[term].TEP) || (!this.isTaskTypePage && !this.terms[term].TEP)) {
                    htmlcode += '<tr><td class="td-nowrap rightalign"><label for="term_' + this.terms[term].id + '">'+ WCXML.xmlencode(ACLayoutStrings.t(this.terms[term].id)) + '</label></td>';
                    htmlcode += '<td><input type="text" size="40" value="'+ WCXML.xmlencode(this.terms[term].defaultValue) + '" id="term_' + this.terms[term].id + '" class="form-control"></td></tr>';

                }
            }        

            // Sequence
            if (!this.isTaskTypePage) {
                var changeMethod = this.functionPrefix + '.checkValidSequence()';
                htmlcode += '<tr><td class="td-nowrap rightalign"><label for="terminologySequence">'+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutSequence')) + '</label></td>';
                htmlcode += '<td><input type="text" size="10" value="'+ WCXML.xmlencode(this.projectCreationSequence) + '" id="terminologySequence" onchange="'+changeMethod+'" class="form-control"></td></tr>';
                htmlcode += '<tr><td></td><td><p class="help-block">Example: GABDM = Gen. Details | Attrs | BOM | Docs | Members</p></td></tr>'
            }

            htmlcode += '</tbody>';
            
            htmlcode += '</table>';
            
            htmlcode += '</div>';

            document.getElementById("terminologyDialog").innerHTML = htmlcode;

            var ACatObj = this;  // organize a closure... ACatObj will be available in the OK function

            var buttonObj = new Object();
            buttonObj[ACLayoutStrings.t('ACLayoutCancel')]= function(){
                $(this).dialog("close");
            };
            buttonObj[ACLayoutStrings.t('ACLayoutOK')] = function(){
                if (!this.isTaskTypePage) {
                    for (var term = 0; term < ACatObj.terms.length; term++) {
                        ACatObj.terms[term].defaultValue = $("#term_"+ACatObj.terms[term].id).val();
                    }
                    ACatObj.projectCreationSequence = $("#terminologySequence").val();	
                }
                
                $(this).dialog("close");
                ACatObj.setACatChanged();
            };

            // create the  dialog but don't open it
            if (this.terminologyDialogCreated == false) {  // create it only once
                $("#terminologyDialog").dialog({
                    autoOpen: false,
                    modal: false,
                    closeOnEscape: true,
                    draggable: true,
                    height: 'auto',
                    resizable: true,
                    width: '600',                
                    buttons: buttonObj
                });
                this.terminologyDialogCreated = true;
            }

            $("#terminologyDialog").dialog("open");

        } catch(err) {
            alert("Problem in attributeCategory.editTerminology: " + err);
            return '';
        }
    };

    attributeCategory.prototype.checkValidSequence = function() {
        try {		
            var newSequence = $("#terminologySequence").val().toUpperCase();
            // check validity of newSequence: must have no other characters than GABDM and each character can be there only once
            var allowedChars = ['G','A','B','D','M'];
            var valid = true;
            for (var idx=0; idx< newSequence.length; idx++) {
                var charVal = newSequence.substr(idx,1);				
                for (var idx2=0; idx2 < allowedChars.length; idx2++) {
                    if (charVal == allowedChars[idx2]) break;
                }
                if (idx2 >= allowedChars.length) valid=false;
                // look whether this character appears twice
                if ( (newSequence.split(charVal).length ) > 2 ) valid = false;
            }
            if (valid) {
                $("#terminologySequence").val(newSequence);  // so now it's with certainty uppercase and valid
            } else {
                $("#terminologySequence").val(this.projectCreationSequence);
            }
            return valid;
        } catch(err) {
            alert("Problem in attributeCategory.checkValidSequence: " + err);
            return true;
        }

    };

    /**
    * Add Ruler in the preview section
    */
    attributeCategory.prototype.addRuler = function() {

        try{
            var htmlcode = '';


            // look for maximum colspan
            var row= this.currentRowIndex;
            var col= this.currentColIndex;
            if (row == null) {	// this probably doesn't happen anymore.  It got hard to not select anything...
                this.addRow(); // add a row at the end
                row = this.nRows-1;  // should be a free row... 
            }
            if (col == null) col = 0;
            // check if cell occupied
            if (this.cells[row][col]) {
                this.addRow(row);			
            }
            for (var col2=col; col2<this.nCols; col2++) {
                if (this.cells[row][col2]) break;
            }
            var maxColspan = col2-col;
            if (maxColspan <1) throw("this cell is occupied");
            htmlcode += '<table class="table table-no-borders">';
            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutColSpan') + ':</td>';
            htmlcode += '<td><select value="1" id="attRulerDialogColSpan">';
            for (var span=1; span<= maxColspan-1; span++) {
                htmlcode += '<option value="' + span + '">' + span + '</option>';
            }
            htmlcode += '<option value="' + span + '" selected>' + span + '</option>'; // always select maximum colspan
            htmlcode += '</select></td></tr>';

            // Height: <input field>
            // Display only for the actual titles with text.
            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutHeight') + ':</td>';
            htmlcode += '<td><input type="text" value="" size="10" id="attRulerDialogHeight" onchange="WCHTML.checkNumericInput(this,\'IntegerPositiveStrict\')">&nbsp;px</td></tr>';

            // Color: <input field>
            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutColor') + ':</td>';
            htmlcode += '<td><input type="text" value="" size="10" id="attRulerDialogColor"></td></tr>';
            //htmlcode += '<table><tr><td>test</td></tr><tr><td>test12321</td></tr>';
            htmlcode += '</table>';

            document.getElementById("rulerDialog").innerHTML = htmlcode;

            var ACatObj = this;  // organize a closure... ACatObj will be available in the OK function

            var buttonObj = new Object();
            buttonObj[ACLayoutStrings.t('ACLayoutCancel')]= function(){
                $(this).dialog("close");
            };
            buttonObj[ACLayoutStrings.t('ACLayoutOK')] = function(){
                cellObj = new Object();
                ACatObj.cells[ACatObj.currentRowIndex][ACatObj.currentColIndex] = cellObj;
                cellObj.type = 'Ruler';
                cellObj.label = '';			
                cellObj.rulerThickness =  $("#attRulerDialogHeight").val();
                if(cellObj.rulerThickness === ""){
                	cellObj.rulerThickness =  3;
                }
                cellObj.rulerColor = $("#attRulerDialogColor").val() || "#000000";
                cellObj.colSpan = $("#attRulerDialogColSpan").val();

                $(this).dialog("close");
                ACatObj.setACatChanged();
                ACatObj.displayLayout();

            };

            // create the ruler dialog but don't open it
            if (this.rulerDialogCreated == false) {  // create it only once
                $("#rulerDialog").dialog({
                    autoOpen: false,
                    modal: false,
                    closeOnEscape: true,
                    draggable: true,
                    height: 'auto',
                    resizable: true,
                    width: '250',                
                    buttons: buttonObj
                });
                this.rulerDialogCreated = true;
            }

            $("#rulerDialog").dialog("open");

        } catch(err) {
            alert("Problem in attributeCategory.addRuler: " + err);
            return '';
        }
    };

    attributeCategory.prototype.checkLastRow = function() {
        try {
        // check whether you need to add a row at the end.  We guarantee that there is always an empty row at the end
            if (this.cells && this.cells[this.nRows-1]) {
                var addRowFlg = false;
                for (var col=0; col< this.nCols; col++) {
                    if (this.cells[this.nRows-1][col] != null) addRowFlg = true; 
                }
                if (addRowFlg) this.addRow();
            }	
            if (this.nCols<1) this.nCols=1;
        } catch (err) {
            // do nothing, not important enough to send warnings
        }
    };



    /*
    * Set different styling for both Label and Value
    *@param int: style index of the current cell
    */
    attributeCategory.prototype.deleteStyle = function(styleIdx) {
        try {
            // do not allow deleting standard styles
            if (this.styles[styleIdx].standardStyle == true) return;
            if (styleIdx<0 || styleIdx>this.styles.length) throw("styleIdx out of bounds");

            var styleObj = this.styles[styleIdx];

            // reset the style for cells having the style you are going to delete
            // handling is different depending whether there are views or not
            if (this.views) {
                for (var view=0; view<this.views.length; view++) {
                    var viewObj = this.views[view];
                    if (viewObj == null) continue;
                    for (var row=0; row < viewObj.cells.length; row++) {
                        if (viewObj.cells!=null && viewObj.cells[row]!=null) {
                            for (var col=0; col < viewObj.cells[row].length; col++) {
                                if (viewObj.cells[row][col]!=null) {
                                    if (viewObj.cells[row][col].styleObj == styleObj) {
                                        // return to standard styles, which is 0 for attributes and 1 for titles
                                        if (viewObj.cells[row][col].type == "Attribute") {
                                            viewObj.cells[row][col].styleObj = this.styles[0];
                                        } else {
                                            viewObj.cells[row][col].styleObj = this.styles[1];
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            } else {
                for (var row=0; row < this.cells.length; row++) {
                    if (this.cells[row]!=null) {
                        for (var col=0; col < this.cells[row].length; col++) {
                            if (this.cells[row][col]!=null) {
                                if (this.cells[row][col].styleObj == styleObj) {
                                    // return to standard styles, which is 0 for attributes and 1 for titles
                                    if (this.cells[row][col].type == "Attribute") {
                                        this.cells[row][col].styleObj = this.styles[0];
                                    } else {
                                        this.cells[row][col].styleObj = this.styles[1];
                                    }
                                }
                            }
                        }
                    }
                }
            }
            this.styles.splice(styleIdx,1);
            this.setACatChanged();
            this.displayLayout();
        } catch(err) {
            alert("Problem in attributeCategory.deleteStyle: " + err);
            return '';
        }
    };


    /*
    * Set defferent styling for both Label and Value
    *@param Style: style object of the current cell
    */
    attributeCategory.prototype.setDifferentStyle = function(currentCellStyleObj) {
        try{

            var element = document.getElementById("differentstyle"); // not a very interesting name...
            var valueStyleBody = document.getElementById("valuestylebody");
            var valuestyle = document.getElementById("valuestyle");
            var labelstyle = document.getElementById("labelstyle");



            var valueStyleObj = null;

            if(currentCellStyleObj && currentCellStyleObj != null)
                valueStyleObj = currentCellStyleObj.valueStyleObj;  // never null???
            if(element && element.checked == true) {  // if the checkbox is checked
            	$("#styleDialog").dialog({
                    width: '800px'             
                });

                var copyStyle = document.getElementById("copystyle");  // the left panel, should not be called copystyle ...
                if (valuestyle) valuestyle.style.display = 'block';
                if (labelstyle) labelstyle.style.display = 'block';

                if(copyStyle) {
                    var cloneStyle = copyStyle.cloneNode(true);
                    if(valueStyleBody.style.display == 'none') {
                        valueStyleBody.style.display = 'block';
                    }
                    else {
                        var row = valueStyleBody.appendChild(cloneStyle);
                        // Change the IDs of Style properties for the 'Value'
                        row.rows[0].cells[1].childNodes[0].id = "backgroundColorValue";
                        row.rows[1].cells[1].childNodes[0].id = "alignmentValue";
                        row.rows[2].cells[1].childNodes[0].id = "fontFamilyValue";
                        row.rows[3].cells[1].childNodes[0].id = "fontStyleValue";
                        row.rows[4].cells[1].childNodes[0].id = "fontWeightValue";
                        row.rows[5].cells[1].childNodes[0].id = "fontSizeValue";
                        row.rows[6].cells[1].childNodes[0].id = "fontColorValue";
                        row.rows[7].cells[1].childNodes[0].id = "cellWidthValue";
                        row.rows[7].cells[1].childNodes[1].id = "valuewidthunit";
                        row.rows[8].cells[1].childNodes[0].id = "cellHeightValue";
                        row.rows[9].cells[1].childNodes[0].id = "columnWidthValue";
                        row.rows[9].cells[1].childNodes[1].id = "columnWidthUnitValue";


                        if(valueStyleObj && valueStyleObj != null) {
                            // Populate the styling of 'value' to display current values.
                            row.rows[0].cells[1].childNodes[0].value = valueStyleObj.background;
                            row.rows[1].cells[1].childNodes[0].value = valueStyleObj.alignment;
                            row.rows[2].cells[1].childNodes[0].value = valueStyleObj.fontFamily;
                            row.rows[3].cells[1].childNodes[0].value = valueStyleObj.fontStyle;
                            row.rows[4].cells[1].childNodes[0].value = valueStyleObj.fontWeight;
                            row.rows[5].cells[1].childNodes[0].value = valueStyleObj.fontSize;
                            row.rows[6].cells[1].childNodes[0].value = valueStyleObj.fontColor;
                            row.rows[7].cells[1].childNodes[0].value = valueStyleObj.width;
                            row.rows[7].cells[1].childNodes[1].value = valueStyleObj.widthunit;
                            row.rows[8].cells[1].childNodes[0].value = valueStyleObj.height;
                            row.rows[9].cells[1].childNodes[0].value = valueStyleObj.columnWidth;
                            row.rows[9].cells[1].childNodes[1].value = valueStyleObj.columnWidthUnit;

                        }
                    }
                }
            }
            else {
            	$("#styleDialog").dialog({
                    width: '500px'             
                });
                valueStyleBody.style.display = 'none';

                if (valuestyle) valuestyle.style.display = 'none';
                if (labelstyle) labelstyle.style.display = 'none';



            }
        } catch(err) {
            alert("Problem in attributeCategory.setDifferentStyle: " + err);
            return '';
        }
    };

    /*
    * The Style dialog box of a current cell
    * @param int: The style index of current cell
    */
    attributeCategory.prototype.showStyle = function(styleIdx) {
        try{
            var styleDialogElem = document.getElementById("styleDialog");
            if (!styleDialogElem) throw("Rules dialog is not initialized");
            var value;

            var styleObj = null;
            if (styleIdx != null) styleObj = this.styles[styleIdx];

            var htmlcode = '';
            htmlcode += '<table class="table table-no-borders" width="100%">';
            htmlcode += '<tr class="darktext_greenlink_11"><td><table>';
            // style Name: <input field>
            value = "";
            if (styleObj) value = styleObj.name;
            htmlcode += '<tr class="darktext_greenlink_11"><td>'+ ACLayoutStrings.t('ACLayoutStyleName') + ':&nbsp;</td>';
            htmlcode += '<td><input type="text" value="'+ WCXML.xmlencode(value) +'" id="styleName"></td></tr>';
            htmlcode += '</table></td></tr>';

            htmlcode += '<tr><td colspan="3"><hr/></td></tr>';

            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td style="text-align:center"><B><span id="labelstyle" style="display:none">'+ ACLayoutStrings.t('ACLayoutLabelStyle')+'</span></B></td>';
            htmlcode += '<td></td>';
            htmlcode += '<td style="text-align:center"><B><span id="valuestyle" style="display:none">'+ ACLayoutStrings.t('ACLayoutValueStyle')+'</span></B></td>';
            htmlcode += '</tr>';
            htmlcode += '<tr><td><table class="table table-no-borders">';

            // remember style index for when clicking the OK button
            if (styleIdx == null) styleIdx=-1;
            htmlcode += '<input type="hidden" id="styleIdx" value="' + styleIdx + '">';

            htmlcode += '<tbody id="copystyle">';

            // background color
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsBgColor') + '</td>';
            htmlcode += '<td>';
            htmlcode += this.createTextField(styleObj, "backgroundColor", null);
            htmlcode += '</td>';
            htmlcode += '</tr>';

            // label alignment
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsalignment') + '</td>';
            htmlcode += '<td>';
            htmlcode += this.createComboBox(styleObj, "alignment", this.alignmentOptions);
            htmlcode += '</td>';
            htmlcode += '</tr>';

            // font family
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsFontFamily') + '</td>';
            htmlcode += '<td>';
            htmlcode += this.createComboBox(styleObj, "fontFamily", this.fontOptions);
            htmlcode += '</td>';
            htmlcode += '</tr>';

            // font style
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsFontStyle') + '</td>';
            htmlcode += '<td>';
            htmlcode += this.createComboBox(styleObj, "fontStyle", this.fontStyleOptions);
            htmlcode += '</td>';
            htmlcode += '</tr>';

            // font weight
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsFontWeight') + '</td>';
            htmlcode += '<td>';
            htmlcode += this.createComboBox(styleObj, "fontWeight", this.fontWeightOptions);
            htmlcode += '</td>';
            htmlcode += '</tr>';

            // font size
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsFontSize') + '</td>';
            htmlcode += '<td>';
            htmlcode += this.createTextField(styleObj, "fontSize", null);
            htmlcode += '&nbsp;pt</td>';
            htmlcode += '</tr>';

            // font color
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsFontColor') + '</td>';
            htmlcode += '<td>';
            htmlcode += this.createTextField(styleObj, "fontColor", null);
            htmlcode += '</td>';
            htmlcode += '</tr>';


            // width
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsWidth') + '</td>';

            htmlcode += '<td>';
            htmlcode += this.createTextField(styleObj, "cellWidth", null);
            htmlcode += this.createComboBox(styleObj, "widthunit", this.widthHeightUnitOptions);
            htmlcode += '</td>';
            htmlcode += '</tr>';




            // height
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsHeight') + '</td>';

            htmlcode += '<td>';
            htmlcode += this.createTextField(styleObj, "cellHeight", null);

            htmlcode += '&nbsp;px</td>';
            htmlcode += '</tr>';

            // column width
            htmlcode += '<tr class="darktext_greenlink_11">';
            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutDetailsColumnWidth') + '</td>';

            htmlcode += '<td>';
            htmlcode += this.createTextField(styleObj, "columnWidth", null);
            htmlcode += this.createComboBox(styleObj, "columnWidthUnit", this.widthHeightUnitOptions);
            htmlcode += '</td>';
            htmlcode += '</tr>';

            htmlcode += '</tbody>';

            htmlcode += '</table></td>';
            htmlcode += '<td>&nbsp;&nbsp;&nbsp;</td>';
            htmlcode += '<td><table class="table table-no-borders" id="valuestylebody">';

            htmlcode += '</table></td></tr>';

            htmlcode += '<tr><td colspan="3"><hr/></td></tr>';

            // Different style : <input field>
            value = "false";
            var changeMethod = this.functionPrefix + '.setDifferentStyle()';
            htmlcode += '<tr class="darktext_greenlink_11"><td colspan="3"><label for="differentstyle"><input type="checkbox" id="differentstyle" onclick="'+changeMethod+'">&nbsp;'+ ACLayoutStrings.t('ACLayoutStyleDifferent') + '</label></td>';
            htmlcode += '</tr>';

            htmlcode += '</table>';

            var ACatObj = this;  // organize a closure... ACatObj will be available in the OK function

            var buttonObj = new Object();
            buttonObj[ACLayoutStrings.t('ACLayoutCancel')]= function(){
                $(this).dialog("close");
            };
            buttonObj[ACLayoutStrings.t('ACLayoutOK')] = function(){
                
                var isValid = true;
                try {
                    var newName = $("#styleName").val();

                    if (newName) {
                        var styleIdx = document.getElementById("styleIdx").value;
                        // check whether name already exists
                        var goodName = true;
                        for (var style=0; style<ACatObj.styles.length; style++) {
                            if (ACatObj.styles[style].name == newName && style!= styleIdx) {
                                alert(ACLayoutStrings.t('ACLayoutStyleExists'));
                                goodName = false;
                            }
                        }
                        if (goodName) {
                            var styleObj = null;
                            var valueStyleObj = null;

                            if (styleIdx >= 0 ) {
                                styleObj = ACatObj.styles[styleIdx];
                                if(styleObj.valueStyleObj)
                                    valueStyleObj = styleObj.valueStyleObj;
                                else
                                    valueStyleObj = new Object();
                            } else {
                                styleObj = new Object();
                                valueStyleObj = new Object();
                                ACatObj.styles.push(styleObj);
                                styleIdx = ACatObj.styles.length -1;
                            }

                            //var widthValue = $("#cellWidth").val();

                            styleObj.name = newName;
                            styleObj.background = $("#backgroundColor").val();
                            if (WCHTML.isHex(styleObj.background)) styleObj.background = "#" + styleObj.background;
                            styleObj.alignment = $("#alignment").val();
                            styleObj.fontStyle = $("#fontStyle").val();
                            styleObj.fontWeight = $("#fontWeight").val();
                            styleObj.fontFamily = $("#fontFamily").val();
                            styleObj.fontSize = $("#fontSize").val();
                            styleObj.fontColor = $("#fontColor").val();
                            if (WCHTML.isHex(styleObj.fontColor)) styleObj.fontColor = "#" + styleObj.fontColor;
                            styleObj.standardStyle = false;						
                            styleObj.width = $("#cellWidth").val();	
                            styleObj.widthunit = $("#widthunit").val();

                            styleObj.height = $("#cellHeight").val();

                            styleObj.columnWidth = $("#columnWidth").val();							
                            styleObj.columnWidthUnit = $("#columnWidthUnit").val();


                            var element = document.getElementById("differentstyle");
                            if(element && element.checked) {
                                valueStyleObj.background = $("#backgroundColorValue").val();
                                if (WCHTML.isHex(valueStyleObj.background)) valueStyleObj.background = "#" + valueStyleObj.background;
                                valueStyleObj.alignment = $("#alignmentValue").val();
                                valueStyleObj.fontStyle = $("#fontStyleValue").val();
                                valueStyleObj.fontWeight = $("#fontWeightValue").val();
                                valueStyleObj.fontFamily = $("#fontFamilyValue").val();
                                valueStyleObj.fontSize = $("#fontSizeValue").val();
                                valueStyleObj.fontColor = $("#fontColorValue").val();
                                if (WCHTML.isHex(valueStyleObj.fontColor)) valueStyleObj.fontColor = "#" + valueStyleObj.fontColor;
                                valueStyleObj.standardStyle = false;
                                valueStyleObj.width = $("#cellWidthValue").val();
                                valueStyleObj.widthunit = $("#valuewidthunit").val();

                                valueStyleObj.height = $("#cellHeightValue").val();														

                                valueStyleObj.columnWidth = $("#columnWidthValue").val();							
                                valueStyleObj.columnWidthUnit = $("#columnWidthUnitValue").val();

                                styleObj.valueStyleObj = valueStyleObj;
                                styleObj.isDifferentStyles = true;
                            }
                            else {
                                styleObj.valueStyleObj = null;
                                styleObj.isDifferentStyles = false;
                            }

                            if(isValid) {
                                // set to the current cell
                                if (ACatObj.currentRowIndex>=0 && ACatObj.currentColIndex>=0) {
                                    ACatObj.cells[ACatObj.currentRowIndex][ACatObj.currentColIndex].styleObj = styleObj;
                                }
                                $(this).dialog("close");
                                ACatObj.setACatChanged();
                                ACatObj.displayLayout();
                            }
                            else {
                                alert("% cannot be greater than 100");
                            }
                        }
                    } else {
                        alert(ACLayoutStrings.t('ACLayoutNoStyleName'));
                    }

                } catch(err) {
                    alert("Problem in saving style: " +err);
                }
            };

            if (this.styleDialogCreated == false) {  // dialog will only be called the first time
                $("#styleDialog").dialog({
                    autoOpen: false,
                    modal: false,
                    closeOnEscape: true,
                    draggable: true,
                    height: 'auto',
                    resizable: true,
                    width: (styleObj && styleObj.isDifferentStyles)?'800px':'500px',                
                    buttons: buttonObj
                });
                this.styleDialogCreated = true;
            }
            styleDialogElem.innerHTML = htmlcode;

            var element = document.getElementById("differentstyle");
            var valuestyle = document.getElementById("valuestyle");
            var labelstyle = document.getElementById("labelstyle");

            //var widthheightinperc = document.getElementById("widthheightinperc");
            var currentCellStyleObj = this.cells[this.currentRowIndex][this.currentColIndex].styleObj;
            if(currentCellStyleObj && currentCellStyleObj != null) {
                if(currentCellStyleObj.isDifferentStyles == true) {
                    element.checked = true;
                    this.setDifferentStyle(currentCellStyleObj);
                    if (valuestyle) valuestyle.style.display = 'block';
                    if (labelstyle) labelstyle.style.display = 'block';

                }
                else {
                    if (valuestyle)valuestyle.style.display = 'none';
                    if (labelstyle)labelstyle.style.display = 'none';

                }

            }

            $("#styleDialog").dialog("open");

        } catch(err) {
            alert("Problem in attributeCategory.showStyle: " + err);
            return '';
        }
    };

    /*
    * Creates combo box as part of Style dialog box
    * @param Object: The style object of the current cell
    * @param String: The ID of the form field
    * @param Array: The ID of the form field
    */
    attributeCategory.prototype.createComboBox = function(styleObj, fieldId, dataArray) {
        try{
            // label alignment
            var htmlcode = "";
            htmlcode += '<select style="padding-bottom:2px; padding-top: 1px;" id="'+fieldId+'">';
            if(dataArray && dataArray != null) {
                for (var index=0; index<dataArray.length; index++) {
                    htmlcode += '<option value="' + dataArray[index] + '" ';

                    if (styleObj && styleObj[fieldId] === dataArray[index]) htmlcode += ' selected ';

                    htmlcode += '>';

                    // the display value
                    switch (fieldId) {
                    case "alignment": 
                    case "fontStyle":
                    case "fontWeight":
                        // localized
                        htmlcode += ACLayoutStrings.t('ACLayout' + dataArray[index]);
                        break;
                    default:
                        htmlcode += dataArray[index] ;
                    }

                    htmlcode += '</option>';
                }
            }
            htmlcode += '</select>';
        } catch(err) {
            alert("Problem in attributeCategory.createComboBox: " + err);
            return '';
        }

        return htmlcode;
    };
    /*
    * Creates HTML re-usable combo box (select)
    * @param id {String} The ID of the form field
    * @param changeMethod {String} method to call onchange
    * @param selectedValue {String} currently selected value, 
    * @param options {Array} array of strings with available options
    * @param pleaseSelect {String} what to put as first element. if null, no first element will be placed
    * @param localized {Boolean} if set, the display value will be taken from the i18n by appending the option to ACLayout
    * @return {String} html code with the select box
    */
    attributeCategory.prototype.createSelectBox = function(id, changeMethod, selectedValue, options, pleaseSelect, localized) {
        var htmlcode = '';

        try{		
            htmlcode += '<select id="'+id+'" ';
            if (changeMethod) htmlcode += ' onchange="'+changeMethod+'"';		
            htmlcode += '>';

            if(options && options.length>0) {
                if (pleaseSelect) htmlcode += '<option value="-1">' + pleaseSelect + '</option>';
                for (var index=0; index<options.length; index++) {
                    htmlcode += '<option value="' + WCXML.xmlencode(options[index]) + '" ';				

                    if (selectedValue === options[index]) htmlcode += ' selected ';

                    htmlcode += '>';
                    if (localized) {
                        htmlcode += WCXML.xmlencode(ACLayoutStrings.t('ACLayout' + options[index]));
                    } else {
                        htmlcode += WCXML.xmlencode(options[index]);
                    }

                    htmlcode += '</option>';
                }
            }
            htmlcode += '</select>';
        } catch(err) {
            alert("Problem in attributeCategory.createSelectBox: " + err);       
        }

        return htmlcode;
    };

    /*
    * Creates text field as part of Style dialog box
    * @param Object: The style object of the current cell
    * @param String: The ID of the form field
    * @param Array: The ID of the form field
    * @return {String} htmlcode generated or empty string if it goes wrong
    */

    attributeCategory.prototype.createTextField = function(styleObj, fieldId, dataArray) {
        try{
        // label alignment
        var htmlcode = "";
        var value = "";
        htmlcode += '<input type=text id="'+fieldId+'" ';
        switch(fieldId) {

            case "fontSize": {
                if(styleObj && styleObj.fontSize) {
                    value = styleObj.fontSize; 
                }
                else 
                    value="";
                htmlcode += 'size="3" maxlength="7" value="'+ value+'" onchange="WCHTML.checkNumericInput(this,\'Positive\')"/>';
                break;
            }

            case "backgroundColor": {
                if(styleObj && styleObj.background) {
                    value = WCXML.xmlencode(styleObj.background); 
                }
                else 
                    value="";
                htmlcode += 'size="10" maxlength="7" value="'+value+'" />';
                break;
            }

            case "fontColor": {
                if(styleObj && styleObj.fontColor) {
                    value = WCXML.xmlencode(styleObj.fontColor); 
                }
                else 
                    value="";
                htmlcode += 'size="10" maxlength="7" value="'+value+'" />';
                break;	
            }

            case "cellWidth": {
                if(styleObj && styleObj.width) {
                    value = styleObj.width; 
                }
                else 
                    value="";
                htmlcode += 'size="10" maxlength="4" value="'+value+'" onchange="WCHTML.checkNumericInput(this,\'IntegerPositiveStrict\')" />';
                break;
            }

            case "cellHeight": { 
                if(styleObj && styleObj.height) {
                    value = styleObj.height; 
                }
                else 
                    value="";
                htmlcode += 'size="10" maxlength="3" value="'+value+'" onchange="WCHTML.checkNumericInput(this,\'IntegerPositiveStrict\')" />';
                break;
            }
            case "columnWidth": { 
                if(styleObj && styleObj.columnWidth) {
                    value = styleObj.columnWidth; 
                }
                else 
                    value="";
                htmlcode += 'size="10" maxlength="3" value="'+value+'" onchange="WCHTML.checkNumericInput(this,\'IntegerPositiveStrict\')" />';
                break;
            }
        }
        } catch(err) {
            alert("Problem in attributeCategory.createTextField: " + err);
            return '';
        }

        return htmlcode;
    };

    /**
    * Dialog to configure a list
    * @param {Int} listIdx: index in this.lists. listIdx only needed and relevant if listObj is null
    *
    */
    attributeCategory.prototype.showListDialog = function(listIdx) {
        "use strict";
        try{
            var dialogElem = document.getElementById("listDialog");
            if (!dialogElem) throw("List dialog is not initialized");

            var listObj = this.lists[listIdx];
            if (!listObj) throw new Error('list with index ' + listIdx + ' is not configured' );

            var htmlcode = '';
            htmlcode += '<div class="add-padding" width="100%" style="padding-bottom:10px">' + ACLayoutStrings.t('ACLayoutExplainListConfig') + "<br/>" + ACLayoutStrings.t('ACLayoutExplainListConfig2') +'</div>';
            htmlcode += '<div class="add-padding" width="100%" id="listConfigDiv">';
            htmlcode += this.displayListConfigTable(listIdx);
            htmlcode += '</div></div>';

            htmlcode += '<div class="wcr-form" width="100%"><div class="form-entry">'; 
            // Invalidate Automatically
            htmlcode += '<div class="checkbox"><label>'; 
            htmlcode += '<input type="checkbox" id="Chk_InvalidateAutomatically" ';
            if (listObj.invalidateAutomatically) htmlcode += ' checked="checked" ';
            htmlcode += '>';
            htmlcode += ACLayoutStrings.t('ACLayoutInvalidateAutomatically') +'</label></div>';
			
			// Allow Other Values
            htmlcode += '<div class="checkbox"><label>'; 
            htmlcode += '<input type="checkbox" id="Chk_AllowOtherValues" ';
            if (listObj.allowOtherValues) htmlcode += ' checked="checked" ';
            htmlcode += '>';
            htmlcode += ACLayoutStrings.t('ACLayoutAllowOtherValues') +'</label></div>';
            
            // Autofill when filled outside of attribute category (e.g. 1 value is filled out in the WF)
            htmlcode += '<div class="checkbox"><label>'; 
            htmlcode += '<input type="checkbox" id="Chk_AutofillExternalValue" ';
            if (listObj.autofillExternalValue) htmlcode += ' checked="checked" ';
            htmlcode += '>';
            htmlcode += ACLayoutStrings.t('ACLayoutAutofillExternalValue') +'</label>';
            htmlcode += '</div></div>';

            var ACatObj = this;  // organize a closure... ACatObj will be available in the OK function
            var buttonObj = new Object();
            
            // since the buttons are only created once, we cannot use closure on the listObj (as this will change)
            // instead, we will use closure on "this" and store the listObj as a field
            var that = this;
            that.selectedListObj = listObj; 

            buttonObj[ACLayoutStrings.t('ACLayoutOK')] = function(){ 
            	var listObj = that.selectedListObj;
				var repaintNeeded = false;
                var newInvalidateAutomatically = $("#Chk_InvalidateAutomatically").is(':checked');
				var newAllowOtherValues = $("#Chk_AllowOtherValues").is(':checked');
				var newAutofillExternalValue = $("#Chk_AutofillExternalValue").is(':checked');
                $(this).dialog("close");
                if (newInvalidateAutomatically != listObj.invalidateAutomatically) {
                    listObj.invalidateAutomatically = newInvalidateAutomatically;
					repaintNeeded=true;
				}
                if (newAllowOtherValues != listObj.allowOtherValues) {
                    listObj.allowOtherValues = newAllowOtherValues;
					repaintNeeded=true;
				}	
                if (newAutofillExternalValue != listObj.autofillExternalValue) {
                    listObj.autofillExternalValue = newAutofillExternalValue;
					repaintNeeded=true;
				}	
				if (repaintNeeded === true){
                    ACatObj.displayAttributeCategoryPreview();
                }
            };

            if (this.listDialogCreated == false) {  // dialog will only be called the first time
                $("#listDialog").dialog({
                    autoOpen: false,
                    modal: true,
                    closeOnEscape: false,
                    draggable: true,
                    height: 'auto',
                    resizable: true,
                    width: 'auto',                
                    buttons: buttonObj,
                    maxHeight: $(window).height()-20
                });
                this.listDialogCreated = true;
            }
            dialogElem.innerHTML = htmlcode;

            $("#listDialog").dialog("open");

        } catch(err) {
            alert("Problem in attributeCategory.showListDialog: " + err);
            return '';
        }
    };

    /**
    * Displays the table with columns for the list configuration dialog
    * @param {Int} listIdx - index in ACatObj.lists, should be a database list, but the display function doesn't care
    */
    attributeCategory.prototype.displayListConfigTable = function(listIdx) {
        "use strict";
        var htmlcode = '';


        try {
            var listObj = this.lists[listIdx];
            if (!listObj) throw Error('Cannot configure list with listIdx: ' + listIdx + '. Index out of bounds?');

            htmlcode += '<table class="table" cellpadding = "3" border="1" width="100%">';  // don't move out the table to the caller since then you need to update the content of the table instead of the content of the td, which IE can't do
            // header
            htmlcode += '<thead style="background-color: #ECE8DA"><tr class="darktext_greenlink_11">';  // ECE8DA is the color of the jQuery dialog header. Most esthetic to reuse this
            htmlcode += '<th/>';  // up down arrow column
            htmlcode += '<th>' + ACLayoutStrings.t('ACLayoutColumn') + '</th>';
            htmlcode += '<th>' + ACLayoutStrings.t('ACLayoutAttribute') + '</th>';				
            htmlcode += '</tr></thead>';

            // Display the rows
            if (listObj.columns) {
                for (var row=0; row< listObj.columns.length; row++) {
                    htmlcode += this.displayListTableRow(listIdx,row, (row == listObj.columns.length -1 ) );
                }
            }
            htmlcode += '</table>';

        } catch(err) {
            alert("Problem in attributeCategory.displayListConfigTable: " + err + " Line: " + err.lineNumber + ': ' +  err.message );
        }
        return htmlcode;

    }

    /** 
    * adds a line to the column list in list configuration dialog
    * @param {Int} listIdx: index of the list in ACadObj.lists
    * @param {Int} row: index in the column table of the list (listObj.columns)
    * @param {Boolean} last: indicates that it's the last row in the table : then don't put the arrow down icon
    */
    attributeCategory.prototype.displayListTableRow = function(listIdx, row, last) {
        "use strict";
        var htmlcode = '';

        try {	
            var listObj = this.lists[listIdx];  
            var columnObj = listObj.columns[row];
            if (!columnObj) throw new Error('Cannot display row' + row + ' because columnObj does not exist');

            // construct a row
            htmlcode += '<tr class="darktext_greenlink_11" valign="top" >';

            // up/down icons
            htmlcode += '<td>';
            if (row >0) {
                clickMethod =  this.functionPrefix + '.changeListTableRow('+ listIdx + ',' +row+',\'Up\'); return false;';
                htmlcode += '<img src="./images/ascending.png" onclick="'+clickMethod+'"/>';
            }	
            htmlcode += '<br>';
            if (!last) {
                clickMethod =  this.functionPrefix + '.changeListTableRow('+ listIdx + ',' +row+',\'Down\'); return false;';
                htmlcode += '<img src="./images/descending.png" onclick="'+clickMethod+'"/>';
            } else {
                htmlcode += '&nbsp;';
            }
            htmlcode += '</td>';

            // Column Name
            htmlcode += '<td>' + WCXML.xmlencode(columnObj.columnName) + '</td>';				

            // Assigned Attribute
            htmlcode += '<td>';
            if (columnObj.attObj) {
                htmlcode += WCXML.xmlencode(columnObj.attObj.name);
            } 		
            htmlcode += '</td>';

            htmlcode += '</tr>';

        } catch(err) {
            alert('Problem in displayListTableRow at line ' + err.lineNumber + ' : ' + err.message);
            htlcode = '';  // better nothing than wrong
        }
        return htmlcode;

    }


    /**
    * Allows moving columns up/down in the list configuration 
    * @param column: index of the column on which the operation is done 
    * @param operation: one of 'Add','Remove','Up','Down'
    */
    attributeCategory.prototype.changeListTableRow = function(listIdx,column,operation) {
        "use strict";
        try {		
            var listObj = this.lists[listIdx];
            var columnArr = listObj.columns;
            var columnObj;

            switch (operation) {

                case 'Up':
                    if ( column > 0 ) {
                        columnObj = columnArr[column];  // remember the object
                        columnArr.splice(column,1); // take it out of the Array
                        columnArr.splice(column-1,0,columnObj);  // put it back 1 up
                    }
                    break;

                case 'Down':
                    if ( column < columnArr.length -1) {
                        columnObj = columnArr[column];  // remember the object
                        columnArr.splice(column,1); // take it out of the Array
                        columnArr.splice(column+1,0,columnObj);  // put it back 1 up
                }
                    break;

                default:
                    throw new Error ('Unknown operation : ' + operation);
            }

            this.setACatChanged();	

            // repaint the table	

            var listConfigDiv = document.getElementById("listConfigDiv");			
            var htmlcode = this.displayListConfigTable(listIdx);	
            if (listConfigDiv) listConfigDiv.innerHTML = htmlcode;   			

            // repaint the preview
            this.displayAttributeCategoryPreview();	

        } catch(err) {
            alert('Problem in changeListTableRow at line ' + err.lineNumber + ' : ' + err.message);
        }

    }

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //
    //                   RULES
    //
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
    * Author: Hans Dewitte, 2010 - 2011
    * This module extends the attributeCategory object
    * This code manages rules and its user interface
    * The base object handled is attributeCategory.rules which is an array of ruleObj
    * this.curRuleObj is a temporary ruleObj maintained during the editing of the rule in its dedicated dialog.
    * When opening the dialog, the current rule is copied to this.curRuleObj (the "correct" deep copy - not obvious). 
    * when clicking Cancel, this curRuleObj is just thrown away, when clicking OK, it is put in the array of rules and linked to the right cells
    * 
    */

    /**
    * Rule concept: See UDD 
    */
    /**
    * Show the Rule dialog box
    * Called both when clicking on New Rule as when clicking on Edit Rule. In addition, it can be called to repaint itself from within one of the handling routines
    * @param ruleIdx: index in the this.rules table of the rule to show. if null or -1, create a new rule. ruleIdx is of not needed when useCurRuleObj is true
    * @param useCurRuleObj: boolean indicating to use this.curRuleObj as container of the rule information (typically when repainting)
    */
    attributeCategory.prototype.showRule = function(ruleIdx,useCurRuleObj) {

        try{
            var value;
            var ruleObj;

            var rulesDialogElem = document.getElementById("rulesDialog");
            if (!rulesDialogElem) throw("Rules dialog is not initialized");

            // determine curRuleObj and ruleIdx. curRuleObj is a temporary ruleObj keeping the changes during the editing of the ruleObj in the edit rule dialog
            if (!useCurRuleObj) {
                if (ruleIdx != null) {
                    // shallow copy ruleObj				
                    this.curRuleObj= this.copyRuleObj(this.rules[ruleIdx]);

                } else {  // start with a new rule. This is a ruleObj only linked to this.curRuleObj but not in this.rules yet.	
                    this.curRuleObj = new Object();
                    this.curRuleObj.condition = "attributeIf";
                    this.curRuleObj.name = "";
                    this.curRuleObj.depth=1;  // how deep the current rule has and/or conditions
                    this.curRuleObj.visibility = "Editable";
                    this.curRuleObj.visibilityElse = "Editable";

                    ruleIdx=-1;
                }
                this.curRuleObj.ruleIdx = ruleIdx;
            } else {
                ruleIdx = this.curRuleObj.ruleIdx; // use the current rule object
            }
            ruleObj = this.curRuleObj;  		

            // Build up the dialog's html from the ruleObj information.
            var htmlcode = '';
            htmlcode += '<table class="table table-no-borders">';

            // remember rule index in a hidden field, for when clicking the OK button        
            htmlcode += '<input type="hidden" id="ruleIdx" value="' + ruleIdx + '">';


            // rule Name: <input field>
            value = "";
            value = ruleObj.name;	
            htmlcode += '<tr class="darktext_greenlink_11"><td colspan="' + this.curRuleObj.depth + '">'+ ACLayoutStrings.t('ACLayoutRuleName') + ':</td>';
            var changeMethod = this.functionPrefix + '.changeRuleName()';
            htmlcode += '<td colspan="6"><input type="text" size="60" value="'+ WCXML.xmlencode(value)+'" id="ruleName" onchange="'+changeMethod+'"></td></tr>';
            htmlcode += '<tr><td><img src="images/clearpixel.gif" height="10"/></td></tr>'; // spacer

            // Block with conditions
            htmlcode += this.showCondition(ruleObj,null,ruleIdx,0);		
            htmlcode += '<tr><td><img src="images/clearpixel.gif" height="10"/></td></tr>'; // spacer


            // The cell is
            htmlcode += '<tr><td colspan="8"><table class="table table-no-borders table-autowidth">';
            //htmlcode += '<tr class="darktext_greenlink_11" valign="top"><td colspan="'+this.curRuleObj.depth+'">'+ ACLayoutStrings.t('ACLayoutTheCellIs') + ':</td>';
            htmlcode += '<tr class="darktext_greenlink_11" valign="top"><td>'+ ACLayoutStrings.t('ACLayoutTheCellIs') + ':</td>';
            value = "";
            if (ruleObj) value = ruleObj.visibility;
            htmlcode += '<td>' + this.addRuleDropdownHTML(value,"cellVisibility") + '&nbsp;&nbsp;</td>';

            /*
            if (value == "Editable" || value == "Required") {			
                htmlcode += '<td>'  +this.addRestrictionDropdownHTML(ruleObj.visibilityRestriction,"cellVisibilityRestriction")+ '</td>';
            }
            */
            htmlcode += '</tr>';


            // Else the cell is
            //htmlcode += '<tr class="darktext_greenlink_11" valign="top"><td colspan="'+this.curRuleObj.depth+'">'+ ACLayoutStrings.t('ACLayoutElseTheCellIs')+'</td>';
            htmlcode += '<tr class="darktext_greenlink_11" valign="top"><td>'+ ACLayoutStrings.t('ACLayoutElseTheCellIs')+'</td>';
            value = "";
            if (ruleObj) value = ruleObj.visibilityElse;
            htmlcode += '<td>' + this.addRuleDropdownHTML(value,"cellVisibilityElse") + '&nbsp;&nbsp;</td>';

            /*
            if (value == "Editable" || value == "Required") {
                htmlcode += '<td>'  + this.addRestrictionDropdownHTML(ruleObj.visibilityElseRestriction,"cellVisibilityElseRestriction")+ '</td>'; 			
            }
            */		
            htmlcode += '</tr>';


            htmlcode += '</table></td></tr>'; // end table for "The cell is" 


            htmlcode += '<tr><td><img src="images/clearpixel.gif" height="10"/></td></tr>'; // spacer        
            htmlcode += '</table>';

            rulesDialogElem.innerHTML = htmlcode;
            var ACatObj = this;  // organize a closure... ACatObj will be available in the OK function

            var buttonObj = new Object();
            buttonObj[ACLayoutStrings.t('ACLayoutCancel')]= function(){
                $(this).dialog("close");
                this.curRuleObj = null;
            };
            buttonObj[ACLayoutStrings.t('ACLayoutOK')] = function(){

                try {
                    var newName = $("#ruleName").val();
                    //var oldRuleObj = null;

                    if (newName) {
                        var ruleIdx = parseInt(document.getElementById("ruleIdx").value);

                        var goodInput = true;
                        // check whether name already exists
                        for (var rule=0; rule<ACatObj.rules.length; rule++) {
                            if (ACatObj.rules[rule].name == newName && rule!= ruleIdx) {
                                alert(ACLayoutStrings.t('ACLayoutRuleExists'));
                                goodInput = false;
                            }
                        }
                        // check whether all filled out
                        if (goodInput) {
                            var errorMsg = '';
                            errorMsg = ACatObj.checkRuleInput();

                            if (errorMsg) {
                                alert(errorMsg);
                                goodInput=false;
                            }
                        }
                        if (goodInput) {
                            var ruleObj = ACatObj.curRuleObj;
                            if (ruleIdx >= 0 ) {
                                // update all occurances of the old role object, not just the one you are editing
                                ACatObj.updateRuleObj(ACatObj.rules[ruleIdx],ACatObj.curRuleObj); 
                                ACatObj.rules[ruleIdx] = ACatObj.curRuleObj;  // update the array too														


                            } else {                            
                                ACatObj.rules.push(ruleObj);
                                ruleIdx = ACatObj.rules.length -1;
                            }


                            ruleObj.standardRule = false;                        


                            // set to the current cell
                            if (ACatObj.currentRowIndex>=0 && ACatObj.currentColIndex>=0) {
                                ACatObj.cells[ACatObj.currentRowIndex][ACatObj.currentColIndex].ruleObj = ruleObj;
                            }
                            $(this).dialog("close");
                            ACatObj.setACatChanged();
                            ACatObj.curRuleObj = null;
                            if (ruleObj.visibilityRestriction && ruleObj.visibilityRestriction.type=="ExcelSearch") {
                                ACatObj.loadList(ruleObj.visibilityRestriction);
                                if (ruleObj.visibilityRestriction.attribute0 >=0) ACatObj.attributes[ruleObj.visibilityRestriction.attribute0].isInRule = true; 
                            }
                            if (ruleObj.visibilityElseRestriction && ruleObj.visibilityElseRestriction.type=="ExcelSearch") {
                                ACatObj.loadList(ruleObj.visibilityElseRestriction);
                                if (ruleObj.visibilityElseRestriction.attribute0 >=0) ACatObj.attributes[ruleObj.visibilityElseRestriction.attribute0].isInRule = true; 
                            }

                            ACatObj.displayLayout();

                        }

                        //

                    } else {
                        alert(ACLayoutStrings.t('ACLayoutNoRuleName'));
                    }

                } catch(err) {
                    alert("Problem in saving rule: " +err);
                }
            };

            $("#rulesDialog").dialog({
                autoOpen: false,
                modal: false,
                closeOnEscape: true,
                draggable: true,
                height: 'auto',
                resizable: true,
                width: '750px',            
                buttons: buttonObj
            });
            $("#rulesDialog").dialog("open");


        } catch(err) {
            alert('Problem in attributeCategory.showRules: line ' + err.lineNumber + ": " +  err.message );	
            return '';
        }
    };

    attributeCategory.prototype.copyRuleObj = function(ruleObj) {
        try {
            var cloneObj = new Object();
            for (var prop in ruleObj) {
            	if (!ruleObj.hasOwnProperty(prop)) continue;
                switch (prop) {
                case "ruleObj1":
                case "ruleObj2":
                    cloneObj[prop] = this.copyRuleObj(ruleObj[prop]);
                    cloneObj[prop].parentObj = cloneObj;  // don't just copy the parent object since you'll have the original back...
                    break;
                case "parentObj":
                    break;  // handled above
                case "visibilityRestriction":
                case "visibilityElseRestriction":
                    cloneObj[prop] = new Object();
                    for (var prop2 in ruleObj[prop]) {
                    	if (ruleObj[prop].hasOwnProperty(prop2)) {
                        cloneObj[prop][prop2] = ruleObj[prop][prop2];
                    }
                    }
                default:
                    cloneObj[prop] = ruleObj[prop];
                    break;
                }

            }
            return cloneObj;
        } catch(err) {
            throw('Problem in attributeCategory.copyRuleObj: line ' + err.lineNumber + ": " +  err.message );	
            return '';
        }
    };
    attributeCategory.prototype.updateRuleObj = function(oldRuleObj,newRuleObj) {
        try {	
            if (this.views) {
                for (var view = 0; view < this.views.length; view++) {
                    var viewObj = this.views[view];
                    if (!viewObj) continue;
                    for (var row=0; row < viewObj.cells.length; row++) {
                        if (viewObj.cells[row]) {
                            for (var col=0; col < viewObj.cells[row].length; col ++) {
                                if (viewObj.cells[row][col]) {
                                    var ruleObj = viewObj.cells[row][col].ruleObj;
                                    if (ruleObj && ruleObj == oldRuleObj) {
                                        viewObj.cells[row][col].ruleObj = newRuleObj;
                                    }
                                }
                            }
                        }
                    }
                }
            } else {  // no views in use, just use the current
                for (var row=0; row < this.cells.length; row++) {
                    if (this.cells[row]) {
                        for (var col=0; col < this.cells[row].length; col ++) {
                            if (this.cells[row][col]) {
                                var ruleObj = this.cells[row][col].ruleObj;
                                if (ruleObj && ruleObj == oldRuleObj) {
                                    this.cells[row][col].ruleObj = newRuleObj;										
                                }
                            }
                        }
                    }
                }

            }

        } catch(err) {
            alert("Problem in attributeCategory.updateRuleObj: " + err);
        }

    };


    /**
    * Paints the dropdown with Editable, ReadOnly, Hidden and Required
    * @param {String} value The current visibility of the selected cell (one of Editable, Hidden, Required, ReadOnly)
    * @param {String} id id to give to the dropdown
    */
    attributeCategory.prototype.addRuleDropdownHTML = function(value,id)  {
        var htmlcode = '';
        try {		
            var changeMethod = this.functionPrefix + '.changeRuleVisibility(\'' + id + '\')'; 
            htmlcode += '<select id="' + id +'" onChange = "'+changeMethod+'">';		
            var options = new Array("Editable","ReadOnly","Hidden","HiddenKeepValues","Required");

            for (var option=0; option<options.length;option++) {
                htmlcode += '<option value="' + options[option] + '"';
                if (value == options[option]) htmlcode += ' selected ';
                htmlcode += '>' + WCXML.xmlencode(ACLayoutStrings.t('ACLayout' + options[option])) + '</option>';
            }
            htmlcode += '</select>';

        } catch(err) {
            alert("Problem in attributeCategory.addRuleDropdownHTML: " + err);
            return '';
        }
        return htmlcode;
    };



    /*
    attributeCategory.prototype.addRestrictionDropdownHTML = function(visibilityObj,id)  {
    Function is obsolete. Last implementation can be found in AttributeCategoryRule_lastgood_20111010.js	
    This code was never checked in.	
    */


    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Change functions
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
    * Called when changing the rule name.  Remembers the current rule name in this.curRuleObj.name
    */
    attributeCategory.prototype.changeRuleName = function()  {
        if (this.curRuleObj && document.getElementById("ruleName")) {
            this.curRuleObj.name = document.getElementById("ruleName").value;
        }

    };

    attributeCategory.prototype.changeRuleVisibility = function(id)  {
        if (id == "cellVisibility") {
            if (this.curRuleObj) this.curRuleObj.visibility = $("#cellVisibility").val();
        } else if (id == "cellVisibilityElse") {
            if (this.curRuleObj) this.curRuleObj.visibilityElse = $("#cellVisibilityElse").val();
        }
        // repaint to allow showing/hiding the extra dialogs 
        this.showRule(null,true);

    };



    /**
    * Called when somebody changes a condition
    */
    attributeCategory.prototype.changeCondition = function(rowIdx) {
        try{
            var curRowIdxObj = new Object(); curRowIdxObj.idx = 0;
            var ruleObj = this.findCondition(this.curRuleObj,rowIdx,curRowIdxObj,0);			
            var repaintNeeded = false;
            if (ruleObj) {
                var att = $("#attributeRuleSelector_"+rowIdx).val();
                if (att >= 0) {
                    ruleObj.attObj = this.attributes[att];
                    ruleObj.attObj.isInRule = true;
                }
                var operator = this.cOperators[ $("#attributeOperatorSelector_"+rowIdx).val() ];
                if (operator == 'One Of' && ruleObj.operator != 'One Of') repaintNeeded = true;
                if (operator != 'One Of' && ruleObj.operator == 'One Of') repaintNeeded = true;
                ruleObj.operator = operator;
                ruleObj.compareValue = $("#compareValue_"+rowIdx).val();
                if (repaintNeeded) this.showRule(null,true);
            } else {
                throw("ruleObj not found");
            }


        } catch (err) {
            alert("Problem in changeCondition: " +err);
        }
    };
    /**
    * Called when somebody changes AND to OR or reversely
    */
    attributeCategory.prototype.changeAndOr = function(rowIdx) {
        try{
            var curRowIdxObj = new Object(); curRowIdxObj.idx = 0;
            var ruleObj = this.findCondition(this.curRuleObj,rowIdx,curRowIdxObj,0);			

            if (ruleObj) {
                ruleObj.condition = $("#conditionDropdown_"+rowIdx).val();	
                if (!ruleObj.condition) {
                    throw("no condition found" );
                }
            } else {
                throw("ruleObj not found");
            }


        } catch (err) {
            alert("Problem in changeAndOr: " +err);
        }
    };



    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Display functions
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


    /**
    * recursive function returning htmlcode to be put in the edit rule dialog
    */
    attributeCategory.prototype.showCondition = function(ruleObj,rowNrObj,ruleIdx,depth) {
        var htmlcode ='';
        try {
            if (!rowNrObj) {
                rowNrObj = new Object();
                rowNrObj.depth = 0;
                rowNrObj.idx = 0; 
            } else {

            }

            if (ruleObj) {
                switch (ruleObj.condition) {
                    case 'AND':
                    case 'OR':
                        var changeMethod = this.functionPrefix + '.changeAndOr('+rowNrObj.idx+')';
                        var selectId = 'conditionDropdown_'+rowNrObj.idx;
                        rowNrObj.idx++;
                        htmlcode += this.showCondition(ruleObj.ruleObj1,rowNrObj,ruleIdx,depth+1);	
                        htmlcode += '<tr>';
                        for (var d=0;d<=depth;d++) htmlcode += "<td></td>";

                        htmlcode += '<td class="darktext_greenlink_11">';

                        if (ruleObj.ruleObj1.condition != 'attributeIf' || ruleObj.ruleObj2.condition != 'attributeIf') {

                            if (ruleObj.condition == 'AND') {
                                htmlcode += '<select id="'+selectId+'" onchange="'+changeMethod+'"><option value="AND" selected>'+ ACLayoutStrings.t('ACLayoutAND') +'</option><option value="OR">'+ ACLayoutStrings.t('ACLayoutOR') +'</option>';
                            } else {
                                htmlcode += '<select id="'+selectId+'" onchange="'+changeMethod+'"><option value="AND" >'+ ACLayoutStrings.t('ACLayoutAND') +'</option><option value="OR" selected>'+ ACLayoutStrings.t('ACLayoutOR') +'</option>';
                            }

                        }
                        htmlcode += '</td>';

                        htmlcode += this.showCondition(ruleObj.ruleObj2,rowNrObj,ruleIdx,depth+1);

                        break;				
                    case 'attributeIf':					
                        htmlcode += this.addAttributeConditionHTML(ruleObj,ruleIdx,rowNrObj.idx,depth);
                        rowNrObj.idx++;
                        break;
                    default:
                        throw({lineNumber:"default condition",message:"unknown condition"});
                        break;
                }
            }

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.showCondition line ' + err.lineNumber + ": " +  err.message );	
        }
        return htmlcode;
    };
    /**
    * recursive function to check whether all is filled out in a rule
    * @param ruleObj object from where to start the search
    * @param msg error message we are building up
    */
    attributeCategory.prototype.checkRuleInput = function(ruleObj,msg) {
        try {		

            if (!ruleObj) ruleObj = this.curRuleObj;

            if (!msg) msg="";

            if (ruleObj.ruleObj1) {
                msg = this.checkRuleInput(ruleObj.ruleObj1,msg);
                msg = this.checkRuleInput(ruleObj.ruleObj2,msg);
            } else {
                if (!ruleObj.attObj) msg += "\n" + ACLayoutStrings.t('ACLayoutNoAttributeSelected');
                //if (!ruleObj.compareValue) msg += "\n" + ACLayoutStrings.t('ACLayoutNoCompareValue');  // took out this check because it has a real use case : DT 118133
            }


            return msg;

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.checkRuleInput line ' + err.lineNumber + ": " +  err.message );	
            return false;		
        }
    };

    /**
    * recursive function finding a condition in a rule
    * @param ruleObj object from where to start the search
    * @param rowIdx row index for which we search
    * @param curRowIdxObj pointer to  row index where we are
    */
    attributeCategory.prototype.findCondition = function(ruleObj,rowIdx,curRowIdxObj,depth) {
        try {
            if (!depth) depth=1;
            if (!curRowIdxObj) {
                curRowIdxObj = new Object();
                curRowIdxObj.idx = 0;			
            }
            var foundRuleObj = null;
            if (curRowIdxObj.idx == rowIdx) {
                foundRuleObj=ruleObj;
                curRowIdxObj.depth = depth; // feed back the depth			
            } else {
                if (ruleObj.ruleObj1) {
                    curRowIdxObj.idx++;
                    foundRuleObj = this.findCondition(ruleObj.ruleObj1,rowIdx,curRowIdxObj,depth+1);
                    if (!foundRuleObj) {
                        curRowIdxObj.idx++;
                        foundRuleObj = this.findCondition(ruleObj.ruleObj2,rowIdx,curRowIdxObj,depth+1);
                    }


                } else {
                    // this is a leaf
                    // do nothing, will keep foundRuleObj on null
                }

            }
            return foundRuleObj;

        } catch(err) {
            alert(ACatStrings.t('ACatProblemIn')+ ' attributeCategory.findCondition line ' + err.lineNumber + ": " +  err.message );	
            return false;		
        }
    };
    /**
    * Delete the condition 
    * @param int: row index
    */
    attributeCategory.prototype.removeConditionHTML = function(rowIdx) {
        try{		
            if (!this.curRuleObj) return;
            var curRowIdxObj = new Object(); curRowIdxObj.idx = 0;
            var ruleObj = this.findCondition(this.curRuleObj,rowIdx,curRowIdxObj,0);		
            var parentObj = ruleObj.parentObj;
            if (ruleObj && parentObj) {
                var newParent = parentObj.ruleObj1;
                if (parentObj.ruleObj1 == ruleObj) newParent = parentObj.ruleObj2;
                    // when you take out rule1, rule2 is promoted to parent rule and vice versa

                if (parentObj.parentObj) {
                    newParent.parentObj = parentObj.parentObj; 
                    if (parentObj.parentObj.ruleObj1 == parentObj) {
                        parentObj.parentObj.ruleObj1 = newParent;
                    } else  {
                        parentObj.parentObj.ruleObj2 = newParent;					
                    } 

                } else { // the other becomes the root 
                    newParent.parentObj = null;
                    // copy the root features to the new root
                    newParent.name = this.curRuleObj.name;
                    newParent.ruleIdx = this.curRuleObj.ruleIdx;
                    newParent.visibility = this.curRuleObj.visibility;
                    newParent.visibilityElse = this.curRuleObj.visibilityElse;
                    newParent.visibilityRestriction = this.curRuleObj.visibilityRestriction;
                    newParent.visibilityElseRestriction = this.curRuleObj.visibilityElseRestriction;
                    this.curRuleObj = newParent;
                    this.curRuleObj.depth = 1;
                }


            }

            this.showRule(null,true);

        } catch (err) {
            alert("Problem in removeConditionHTML: " +err);
        }
    };

    /**
    * Produces HTML for ( one condition line in the Edit Rule dialog
    * @param rowIdx Index of the line in the set (so a number)
    * @parem depth How deep you are - for indentation
    */
    attributeCategory.prototype.addAttributeConditionHTML = function(ruleObj,ruleIdx,rowIdx,depth) {
        // select box on  rule type
        var htmlcode ='';
        try{
            var prefix = '';
            htmlcode += '<tr class="darktext_greenlink_11" valign="top" >';
            var moveIfRight = 0;

            for (var d=0; d<depth; d++) htmlcode += "<td></td>";
            if (!this.curRuleObj.depth) this.curRuleObj.depth=1;
            //if (ruleObj.parentObj && ruleObj.parentObj.ruleObj1 == ruleObj) {
                //prefix = 'IF';
            if (ruleObj.parentObj && ruleObj.parentObj.ruleObj2 == ruleObj && ruleObj.parentObj.ruleObj1.condition == "attributeIf") {
                // put the parent's AND/OR here
                prefix = ruleObj.parentObj.condition;
            } else {
                prefix = 'IF';
                // move the if to the right if its brother is a complex condition
                if (ruleObj.parentObj) {
                    if (ruleObj.parentObj.ruleObj1 == ruleObj && ruleObj.parentObj.ruleObj2.condition != 'attributeIf') moveIfRight = 1;
                    if (ruleObj.parentObj.ruleObj2 == ruleObj && ruleObj.parentObj.ruleObj1.condition != 'attributeIf') moveIfRight = 1;
                }
            }
            if (moveIfRight) htmlcode += '<td></td>';
            htmlcode += '<td>';
            var dropdownIdx = rowIdx-2;
            var changeMethod = this.functionPrefix + '.changeAndOr('+ dropdownIdx +')';
            switch (prefix) {
                case 'IF':
                    htmlcode += ACLayoutStrings.t('ACLayoutIf');
                    break;
                case 'AND':
                    htmlcode += '<select id="conditionDropdown_'+ dropdownIdx +'" onchange="'+changeMethod+'"><option value="AND" selected>'+ ACLayoutStrings.t('ACLayoutAND') +'</option><option value="OR">'+ ACLayoutStrings.t('ACLayoutOR') +'</option>';
                    break;
                case 'OR':
                    htmlcode += '<select id="conditionDropdown_'+ dropdownIdx +'" onchange="'+changeMethod+'"><option value="AND" >'+ ACLayoutStrings.t('ACLayoutAND') +'</option><option value="OR" selected>'+ ACLayoutStrings.t('ACLayoutOR') +'</option>';
                    break;
                default:
                    break;
            }
            htmlcode += '</td>' ;
            for (d=depth+1+moveIfRight; d<this.curRuleObj.depth; d++) htmlcode += "<td>&nbsp;&nbsp;&nbsp;&nbsp;</td>";

            htmlcode += '<td>' + ACLayoutStrings.t('ACLayoutValueOfAttribute')+'</td>';

            // select box with list of placed attributes
        var changeMethod = this.functionPrefix + '.changeCondition('+rowIdx+')';
            htmlcode += '<td><select id="attributeRuleSelector_' + rowIdx +'" onchange="'+changeMethod+'">';
            htmlcode += '<option value="-1">---' + ACLayoutStrings.t('ACLayoutSelectAttribute') +'---</option>';
            for (var att=0; att<this.attributes.length; att++) {
                if (this.attributes[att].placed < 1) continue;
                htmlcode += '<option value="' + att + '"';
                if (ruleObj && ruleObj.attObj == this.attributes[att]) htmlcode += ' selected ';
                htmlcode += '>' + WCXML.xmlencode(this.attributes[att].name) + '</option>';
            }
            htmlcode += '</select></td>';

            // select box with type of logical operator		
            htmlcode += '<td><select id="attributeOperatorSelector_' + rowIdx +'" onchange="'+changeMethod+'">';
            var options = this.cOperators;
            for (var option=0; option<options.length; option++) {
                htmlcode += '<option value="' + option + '"';
                if (ruleObj && ruleObj.operator == this.cOperators[option]) htmlcode += ' selected ';
                htmlcode += '>' + options[option] + '</option>';
            }
            htmlcode += '</select></td>';
            // compare value
            value = '';
            if (ruleObj && ruleObj.compareValue) value = WCXML.xmlencode(ruleObj.compareValue);
            if (ruleObj && (ruleObj.operator === "One Of")) {
                htmlcode += '<td id="compareValueField_' + rowIdx +'"><textarea rows="4" cols="30" id="compareValue_' + rowIdx +'" onchange="'+changeMethod+'">' + value + '</textarea></td>';
            } else {
                htmlcode += '<td id="compareValueField_' + rowIdx +'"><input type="text" id="compareValue_' + rowIdx +'" value="' + WCXML.xmlencode(value) + '" onchange="'+changeMethod+'"></td>';
            }
            // For AND condition
            clickMethod = this.functionPrefix + '.addRule('+rowIdx+',\'AND\'); return false;';
            htmlcode += '<td>&nbsp;&nbsp;<a href="#" onclick="' + clickMethod  + '">' + ACLayoutStrings.t('ACLayoutAnd') + '</a></td>';

            if (rowIdx >0) {
                clickMethod = this.functionPrefix + '.removeConditionHTML('+rowIdx+'); return false;';
                htmlcode += '<td><i class="icon icon-WCR_Remove icon-normal" onclick="' + clickMethod +'"></i></td>';
            }
            htmlcode += '</tr>';

        } catch (err) {
            alert("Problem in addAttributeConditionHTML: " +err);
        }
        return htmlcode;
    };
    
    /**
    * Called when somebody clicks on the and link to add a condition
    */
    attributeCategory.prototype.addRule = function(rowIdx, condition) {
        var curRowIdxObj = new Object(); curRowIdxObj.idx = 0;
        var ruleObj = this.findCondition(this.curRuleObj,rowIdx,curRowIdxObj,0);  // the object on which the and takes place
        var depth = curRowIdxObj.depth;
        if (!this.curRuleObj.depth) this.curRuleObj.depth = 1;
        if (depth >= this.curRuleObj.depth) this.curRuleObj.depth = depth+1;
        var parentObj = ruleObj.parentObj; 	

        var conditionObj = new Object();  // object carrying the condition which will become between the parent and the existing rule

        conditionObj.condition = condition;
        conditionObj.parentObj = parentObj; 

        // move existing rule under conditionObj
        conditionObj.ruleObj1 = ruleObj; 
        ruleObj.parentObj = conditionObj;

        //now add a new ruleObj under conditionObj
        var ruleObj2 = new Object();
        conditionObj.ruleObj2 = ruleObj2;	
        ruleObj2.condition = 'attributeIf';
        ruleObj2.operator = '=';
        ruleObj2.parentObj = conditionObj;	


        if (parentObj) {
            if (parentObj.ruleObj1 == ruleObj)	{
                parentObj.ruleObj1 = conditionObj;
            } else {
                parentObj.ruleObj2 = conditionObj;
            }
        } else {  // no parentObj, this is the root. Copy the root features to conditionObj because this will become the new root		
            conditionObj.depth = this.curRuleObj.depth; this.curRuleObj.depth = null;
            conditionObj.name = this.curRuleObj.name; this.curRuleObj.name = null; 
            conditionObj.ruleIdx = this.curRuleObj.ruleIdx; this.curRuleObj.ruleIdx = null; 
            conditionObj.visibility = this.curRuleObj.visibility; this.curRuleObj.visibility = null; 
            conditionObj.visibilityElse = this.curRuleObj.visibilityElse; this.curRuleObj.visibilityElse = null;
            conditionObj.visibilityRestriction = this.curRuleObj.visibilityRestriction; this.curRuleObj.visibilityRestriction = null; 
            conditionObj.visibilityElseRestriction = this.curRuleObj.visibilityElseRestriction; this.curRuleObj.visibilityElseRestriction = null; 

            this.curRuleObj = conditionObj;		// remember the new root
        }

        this.showRule(null,true);
    };






    /*
    * Delete the current Rule
    * @param Rule: The current rule index
    */
    attributeCategory.prototype.deleteRule = function(ruleIdx) {
        this.cells[this.currentRowIndex][this.currentColIndex].ruleObj = null; //Remove the rule from grid cell
        // You also need to remove the rule from all other cells having the same rule (HADW fix)
        for (var row=0; row<this.nRows;row++) {
            if (this.cells[row]) {
                for (var col=0; col<this.nCols;col++) {
                    var cellObj = this.cells[row][col];
                    if (cellObj && cellObj.ruleObj == this.rules[ruleIdx]) cellObj.ruleObj = null;
                }
            }
        }
        this.rules.splice(ruleIdx, 1); //Delete the rule from rules map.
        this.displayLayout();
    };

    /*
    * The document source dialog box for a current cell
    * @author HADW - personal review done on 6-Feb-2015
    * @param {Integer} selectedIdx: The index of the document source in the document source gallery. When null a new document source configuration will be created
    * @param {Integer} row - row index of the current cell
    * @param {Integer} col - column index of the current cell
    */
    attributeCategory.prototype.showDocumentSource = function(selectedIdx, row, col) {
        'use strict';
        try {
			var dialogElem = document.getElementById("docSourceDialog");
			if (!dialogElem) throw("Document Sources dialog is not initialized");

            var dshConfigTablePHID = this.dshConfigTablePHID;
            var dshObj = null;

            if (selectedIdx != null) dshObj = this.docSourcesGallery[selectedIdx];

            // If none exists: add a new one. typically happens when clicking on Add Configuration in which case selectedIdx = null
            if (!dshObj) {
                var cellObj = this.cells[row][col];
                if (cellObj) {
                    dshObj = this.createDocumentSource(dshConfigTablePHID, cellObj, null);
                    this.createDocTableX(cellObj);  
                }
            } 		

            var htmlcode = '';

            htmlcode += '<div class="docsource_dialogbody">';

            htmlcode += '<div class="add-padding"><span id="configurationName"><strong>'+ WCXML.xmlencode( dshObj.name ) +'</strong></span></div>';
            htmlcode += '<div id="' + dshConfigTablePHID + '" class="docsourceconfig popup">';	
            htmlcode += dshObj.displayDocumentSourceTable();
            
            htmlcode += '</div>';
            htmlcode += '</div>';


            var ACatObj = this;  // organize a closure... ACatObj will be available in the OK function

            var buttonObj = new Object();

            buttonObj[ACLayoutStrings.t('ACLayoutOK')] = function(){

            	// Validate the dialog before we close it. If not valid, do not close the dialog!  
            	if (!dshObj.validateAll()){
            		return ;
            	}
            	
                $(this).dialog("close");
                // repaint the cell details
                var cellDetailsTable = document.getElementById("celldetails");		
                if (cellDetailsTable) cellDetailsTable.innerHTML = ACatObj.displayCellDetails();    	

            };

            if (this.docSourceDialogCreated == false) {  // dialog will only be called the first time
    			// get the height of the browser's window and adapt the max. dialog height according to it.
            	var viewportHeight = $(window).height();
            	var maxHeight = viewportHeight - viewportHeight / 10;
            	if (maxHeight < 400) maxHeight = 400;

            	$("#docSourceDialog").dialog({
                    autoOpen: false,
                    modal: true,
                    closeOnEscape: false,
                    draggable: true,
                    resizable: true,
                    width: 'auto',
                    height: 'auto',
                    maxWidth: 1200,
                    maxHeight: maxHeight,
                    dialogClass: 'no-close',				
                    buttons: buttonObj,
                    drag: function() {
                    	$(this).attr("moved", "1");
                    }
                });
            	
            	this.docSourceDialogCreated = true;
            }
            else {
            	// The dialaog is OK to be initialized only once with exception of button(s), because 
            	// the 'dshObj' used in the button(s) action handler might changed from the last time!
            	// (if user opens document source configuration of another (e.g.) doc. reference...)
            	$("#docSourceDialog").dialog('option', 'buttons', buttonObj);
            }
            
            dialogElem.innerHTML = htmlcode;
            
            $("#docSourceDialog").dialog("open");
            $(".no-close .ui-dialog-titlebar-close").hide();  // should be tackled by the CSS but did not see this working yet, so I force it

        } catch(err) {
            alert("Problem in attributeCategory.showDocumentSource: " + err);
        }
    };
	
    
	/**
	 * The document source dialog box for a current cell
	 * @author HADW, JANH
	 * @param {Integer} row - row index of the current cell
	 * @param {Integer} col - column index of the current cell
	 * @param {Boolean} createNew - if true, a new configuration will be created for the current view.
	 */
	attributeCategory.prototype.showDocumentRenderDialog = function(row, col, createNew) {
		'use strict';
		try {
			var dialogElem = document.getElementById("docRenderDialog");
			if (!dialogElem) throw new Error("Document Sources dialog is not initialized");	
			var	cellObj = this.cells[row][col];

			if (createNew) this.createDefaultDocReferenceTable(cellObj, true);	
			var docLinkTableOptions = cellObj.docLinkTableOptions;
			if (!docLinkTableOptions) throw new Error('Current cell has no docLinkTableOptions');
			
			// start HTML generation
			var htmlcode = '';
			
			htmlcode += '<div class="docrender_dialogbody">';
			
			// name 
			htmlcode += '<div class="add-padding-left add-padding-top"><strong>' + ACLayoutStrings.t('ACLayoutName') + ':&nbsp;';
			htmlcode += '<span id="renderConfigurationName" >'+ WCXML.xmlencode(docLinkTableOptions.getName()) +'<strong></span></div>';

			// main placeholder for the doc. reference documents table rendering configuration
			htmlcode += '<div id="'+ docLinkTableOptions.phElemID +'" class="docrenderingconfig popup"></div>';
			
			htmlcode += '</div>';
			
			// render the HTML skeleton including the placeholder for the rendering configuration
			dialogElem.innerHTML = htmlcode;
			
			// set the list of available reference properties based on reference category set to this doc. ref. attribute
			var attObj = cellObj.attObj;
			if (attObj && attObj.referencePropertyACat) {
				cellObj.docLinkTableOptions.setProperties(attObj.referencePropertyACat.attributes);
			}
			// if in task type, make sure the attribute are all the attributes in the system instead of just the task type specifications
			var setAttributesDone = _.bind(function () {
				if (this.isTaskTypePage) {
					return Promisify.require("doctablex/AJAXDataCache").then(function (AJAXDataCache) {
						return Promise.resolve(AJAXDataCache.getInstance().getAttributes()).then(function (attributes) {
							docLinkTableOptions.setAttributes(attributes);
						});
					});
				}
				return Promise.resolve();
			}, this);

			setAttributesDone().then(_.bind(function () {
				// render the document link table (== doc table rendering configuration options) to the prepared placeholder
				docLinkTableOptions.render();

				// prepare the dialog to be opened
				var ACatObj = this; // closure

				var buttonObj = new Object();

				// define buttons (just OK button in this case)
				buttonObj[ACLayoutStrings.t('ACLayoutOK')] = function () {
					$(this).dialog("close");
					// repaint the cell details
					var cellDetailsTable = document.getElementById("celldetails");
					if (cellDetailsTable) cellDetailsTable.innerHTML = ACatObj.displayCellDetails();

					ACatObj.recreateDocTableXForCurrentView();
					// repaint the preview to show the new column ordering
					ACatObj.displayAttributeCategoryPreview();
				};

				if (this.docRenderDialogCreated == false) { // dialog will only be called the first time
					// get the height of the browser's window and adapt the max. dialog height according to it.
					var viewportHeight = $(window).height();
					var maxHeight = viewportHeight - viewportHeight / 10;
					if (maxHeight < 400) maxHeight = 400;

					$("#docRenderDialog").dialog({
						autoOpen: false,
						modal: true,
						closeOnEscape: false,
						draggable: true,
						resizable: true,
						width: 'auto',
						height: 'auto',
						maxWidth: 1200,
						maxHeight: maxHeight,
						dialogClass: 'no-close',
						buttons: buttonObj,
						drag: function () {
							$(this).attr("moved", "1");
						}
					});

					this.docRenderDialogCreated = true;
				} else {
					// update whatever is not constant for different document rendering configurations
					// (see update of buttons object in showDocumentSource() above)
					// currently there is nothing that requires an update/reinitialization.
				}

				// open the initialized dialog
				$("#docRenderDialog").dialog("open");
				$(".no-close .ui-dialog-titlebar-close").hide(); // should be tackled by the CSS but did not see this working yet, so I force it
			}, this)).catch(function(err) {
				window.alert("Problem in attributeCategory.showDocumentRenderDialog: " + err);
			});
		} catch (err) {
			window.alert("Problem in attributeCategory.showDocumentRenderDialog: " + err);
		}
	};

    /**
    * Imports an entire attribute category by taking an existing XML file 
    * non-existing attributes are dropped (until a create function is written)
    */
    attributeCategory.prototype.exportXML = function() {
        try{				
            this.saveFieldId = "exportXML";
            this.submitAllowed = false;
            this.saveLayout();
            this.submitAllowed = true;

            var layoutXML = document.getElementById("exportXML").value;

            var xmlcode = '';
            xmlcode += '<attribute_category>';
            xmlcode += '<deleted>0</deleted>';
            xmlcode += '<name>My category</name>';
            xmlcode += '<category_XML>';
            xmlcode += layoutXML;
            xmlcode += '</category_XML>';
            var attList = this.attributes;
            if (attList) {
                xmlcode += '<attribute_list>';
                for (var att=0; att<attList.length; att++) {
                  var attObj = attList[att];
                  if (attObj && attObj.placed>0) {
                    xmlcode += '<attribute id="';
                    xmlcode += attObj.id;
                    xmlcode += '">';
                    xmlcode += WCXML.ElementEncode('deleted',0);
                    xmlcode += WCXML.ElementEncode('name',attObj.name);
                    if (attObj.description) xmlcode += WCXML.ElementEncode('name',attObj.description);
                    if (attObj.unitId) xmlcode += WCXML.ElementEncode('attribute_unit_id',attObj.unitId);
                    if (attObj.rs) xmlcode += WCXML.ElementEncode('restricted_set_id',attObj.rs.id);
                    xmlcode += '</attribute>';

                  }
                }
                xmlcode += '</attribute_list>';
            }
            xmlcode += '</attribute_category>';

            document.getElementById("exportXML").value = xmlcode;

            document.getElementById("exportXMLtd").style.display = "";  // make text area visible

        } catch (err) {
            alert("Problem in exportXML: " +err);
        }
    };


    /**
    * Imports an entire attribute category by taking an existing XML file 
    * non-existing attributes are dropped (until a create function is written)
    */

    attributeCategory.prototype.showImportXML = function() {
        try{	
            document.getElementById("importXMLtd").style.display = "";  // make text area visible			

        } catch (err) {
            alert("Problem in showImportXML: " +err);
        }
    };

    attributeCategory.prototype.importXML = function() {
        try{			
            var xmlString = document.getElementById("importXML").value;  
            // convert string to xml DOM
            var doc;
            if (window.ActiveXObject){
              doc=new ActiveXObject('Microsoft.XMLDOM');
              doc.async='false';
              doc.loadXML(xmlString);
            } else {
              var parser=new DOMParser();
              doc=parser.parseFromString(xmlString,'text/xml');
            }
            // remove all other views
            this.views = null;
            this.currentView = 'Default';
            // remove all rows
            while (this.cells.length>1) {
                this.deleteRow(this.cells.length-1);
            }
            if (this.nRows>0) this.deleteRow(0); //delete last row


            var attributeCategoryElem = WCXML.GetFirstFirstLevelChild(doc,'attribute_category');
            if (!attributeCategoryElem) throw new Error('top element of XML must be <attribute_category>');

            var attributeListElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,'attribute_list');
            if (attributeListElem) {
                var attributeList = WCXML.GetFirstLevelChilds(attributeListElem,'attribute');
                for (var att=0; att<attributeList.length; att++) {
                    var attElem = attributeList[att];
                    if (attElem) {
                        var attName = WCXML.GetElementText(attElem,'name');
                        // see whether you find it
                        if (!this.findAttribute(null,attName)) {
                            if (this.isTaskTypePage) {
                                alert("Import uses specification " + attName + " but it does not exist on this system. Please first create this specification");
                            } else {
                                alert("Import uses attribute " + attName + " but it does not exist on this system. Please first create this attribute");
                            }

                        }
                    }
                }
            }

            this.imported = true; this.importEaster = true;
            if (xmlString) this.readReturnXML(doc,null,true);

        } catch (err) {
            alert("Problem in importXML: " +err);
        }
    };



    /*
    * Cancelling the Layout
    */
    attributeCategory.prototype.cancelLayout = function () {
        var doIt = confirm(ACLayoutStrings.t('ACLayoutConfirmCancelLayout'));
        if (doIt == true) {
            new attributeCategory(this.aCatOptions);
        }
    };
    
    /**
     * Remove empty lines at the end and beginning of the category
     * @param {Array.Array.Object} - grid representation of cells
     */
     attributeCategory.prototype.trimRows = function (cells) {
         // remove rows at the start
    	 while(cells[0] && this.isEmptyRow(cells[0])){
    		 cells.splice(0, 1);
    	 }
    	 
    	 // remove rows at the end
    	 while(cells[cells.length-1] && this.isEmptyRow(cells[cells.length-1])){
    		 cells.splice(cells.length-1, 1);
    	 }
     };
     
     /**
      * Check if row is empty, meaning it contains only null and undefined
      * @param {Array.Object} - row representation of cells
      * @returns
      */
      attributeCategory.prototype.isEmptyRow = function (row) {
          for(var i=0 ; i<row.length ; i++){
        	  if(row[i]){
        		  return false;
        	  }
          }
          return true;
      };

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Experiment day EXTENDED Forms
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /**
    * called when you change the gallery between attribute-specifications and other properties
    * @param elem DOM object of the filter input field 
    */
    attributeCategory.prototype.changeGalleryType = function(elem) {
        try {
            this.galleryType = elem.value;

            this.displayLayout();  //repaint everything

        } catch(err) {
            alert("Problem in attributeCategory.changeGalleryType: " + err);
            return '';
        }
    };
    
    
    return attributeCategory;
});
